(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 13.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[   2091161,      50523]
NotebookOptionsPosition[   1748190,      43467]
NotebookOutlinePosition[   1945306,      47376]
CellTagsIndexPosition[   1944366,      47341]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["QCPITT",
 FontSize->24]], "Title",
 TaggingRules->{},
 CellChangeTimes->{3.857571280631002*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1427136295,ExpressionUUID->"646b22a5-a59f-4e8c-8c02-c28304c26aac"],

Cell[CellGroupData[{

Cell["\<\
Frank Tabakin
Department of Physics & Astronomy
University of Pittsburgh\
\>", "Subsubsection",
 TaggingRules->{},
 CellChangeTimes->{{3.857563406003869*^9, 3.857563438317791*^9}, {
  3.857730812164626*^9, 3.8577308230772448`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[1]:=",
 CellID->1444391315,ExpressionUUID->"7ea05f2a-081f-4e49-aa3a-fedd3a4205e0"],

Cell["\<\
The quantum computer simulation codes QDENSITY, QCWAVE, BTSYSTEM, and  \
CIRCUITS are presented as a combined version: QCPITT.  These  Mathematica \
codes provide a pedagogic tool for understanding the basic ideas of a quantum \
computer and in addition provide a tool for studying the advantages and \
vulnerabilities of a quantum computer. In QDENSITY, a density matrix \
description is used to describe quantum systems; whereas, QCWAVE uses a \
quantum state description. BTSYSTEM is used not only for qubits (Binary), but \
also for qutrits (Triplet) states and can be extended to qudrits (more than 3 \
levels).  CIRCUITS provides a graphical display of quantum circuits. These \
mathematical codes were developed at the University of Pittsburgh during the \
1990s and have been published and posted,  accompanied by many detailed \
tutorials.

These codes are designed to be simple, open and approachable.  Overly \
sophisticated definitions are avoided, in favor of  readily understood, \
revisable and expandable versions.  A modular, instructive approach is used \
to generate the basic elements that make up a quantum circuit. The QCPITT \
package includes: Qdensity.m,  QCWave.m, Circuits.m, and BTSystem.m, which \
contain the tools needed in quantum circuits, e.g. multi-qubit kets, \
projectors, gates, etc.  Selected examples of the basic commands are \
presented here and in tutorial notebooks that serve as a full guide and \
templates. Application is made to a variety of relevant cases, including \
Teleportation, Quantum Fourier Transform, Grover\[CloseCurlyQuote]s search \
and Shor\[CloseCurlyQuote]s algorithm, in separate notebooks: QFT.nb, \
Teleportation.nb, Grover.nb and Shor.nb, where each algorithm is explained in \
detail. A Mathematica palette containing some commands is also included: \
QCPITTpalette.nb. Recent developments include: (1) application to qutrits and \
qudits; (2) extension to closed and open system dynamics using a \
Lindblad/Beretta formulation, (3) studies of one and two qubit gate dynamics. \
and (4)  multi-dimensional  tunnelling (in preparation).  Future goals are \
delineated.\
\>", "Abstract",
 TaggingRules->{},
 CellChangeTimes->{
  3.857571350492466*^9, {3.858087500537691*^9, 3.858087507343938*^9}, {
   3.8581736657175007`*^9, 3.858173729358176*^9}, {3.858848466787771*^9, 
   3.858848536000142*^9}, {3.858848581247189*^9, 3.858848594827466*^9}, {
   3.858848625843875*^9, 3.858848649942279*^9}, {3.858848706349235*^9, 
   3.85884879162628*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 FontSlant->"Plain",
 CellID->634856722,ExpressionUUID->"0e4513ec-b784-4e44-b9be-a5d64db7d3b8"]
}, Open  ]]
}, Open  ]],

Cell["Introduction", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575718136293507`*^9, {3.858176905300309*^9, 3.858176909755438*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2146841522,ExpressionUUID->"462e26e4-0666-4afe-bbb0-339dd94daf7f"],

Cell["\<\
QCPITT is a collection of Mathematica packages that simulate a quantum \
computer on a classical one. This simulation closely follows the procedures \
that would occur in an actual quantum computer, wherein quantum interference, \
superposition and entanglement play key roles.  Although emphasis is on using \
a density matrix, QCPITT also handles state vectors. The package comes with \
numerous tutorials that enable the user to build any quantum circuit out of \
conventional quantum gates. These notebooks illustrate the most relevant \
quantum algorithms:  teleportation, searching and factoring algorithms in a \
self contained way.  Extensions to both binary (qubit ) and triplet (qutrit) \
systems are included.\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.856097309002644*^9, 3.856097317451788*^9}, {
   3.856363578562482*^9, 3.856363674921479*^9}, {3.856363728756091*^9, 
   3.8563637440668793`*^9}, {3.856365624834725*^9, 3.8563656256220627`*^9}, {
   3.856365662353849*^9, 3.8563658614050083`*^9}, 3.8563774792469797`*^9, {
   3.8572301804045*^9, 3.857230362600912*^9}, {3.857230395597664*^9, 
   3.857230426614558*^9}, {3.857237014702279*^9, 3.857237068163226*^9}, {
   3.857237099731213*^9, 3.857237104174683*^9}, {3.8572379723253593`*^9, 
   3.857237990899757*^9}, {3.8588488445238323`*^9, 3.858848871096383*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellID->1158596779,ExpressionUUID->"5b410322-fc74-464c-8761-54bddc764685"],

Cell[TextData[StyleBox["Installation",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.8575699422557487`*^9, 3.858967105145105*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->263940212,ExpressionUUID->"ffc28544-b529-4787-acca-cdd8ac95f2ce"],

Cell[TextData[{
 "Version: 6.0-April 2022--For ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 13"
}], "Abstract",
 TaggingRules->{},
 CellChangeTimes->{{3.395090887981901*^9, 3.395090925377634*^9}, {
  3.5122043095457993`*^9, 3.512204334243861*^9}, {3.512204716872068*^9, 
  3.512204717487535*^9}, {3.61332114153409*^9, 3.61332117425388*^9}, {
  3.624192719233993*^9, 3.624192746841957*^9}, {3.655397982735033*^9, 
  3.6553979855009193`*^9}, {3.8046750450687523`*^9, 3.804675051459118*^9}, {
  3.8560964673663054`*^9, 3.856096490136567*^9}, {3.8572284646518106`*^9, 
  3.857228483394965*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 FontSlant->"Plain",
 CellID->422814197,ExpressionUUID->"2b461a9b-e87e-43b8-b13b-d6c82e44156f"],

Cell[TextData[{
 StyleBox["Click  ",
  FontSize->18],
 StyleBox[ButtonBox["QCPITT",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.pitt.edu/~tabakin/QDENSITY/QCPITT.zip"], None},
  ButtonNote->"http://www.pitt.edu/~tabakin/QDENSITY/QCPITT.zip"],
  FontSize->18],
 StyleBox["  to download and then unzip QCPITT.zip. All the quantum computing \
simulation codes will start by loading the Qdensity, QCWave , BTSystem and  \
QCircuits  packages. To make the packages fully available put the QDENSITY \
folder directory for example to the location:  \
\[OpenCurlyDoubleQuote]/Users/youraccount/Library/Mathematica/Applications/\
\[CloseCurlyDoubleQuote] on your user account, as deduced for a MacOS below: ",
  FontSize->18]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.8579215703044662`*^9, 3.8581765055485764`*^9, {3.858849030062064*^9, 
   3.858849087381216*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->953630619,ExpressionUUID->"c79b1b97-590b-402f-b11a-b86feb2f48ff"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FileNameJoin", "[", " ", 
  RowBox[{"{", 
   RowBox[{
   "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
    "\"\<QDENSITY\>\""}], "}"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857416883067651*^9, 3.85741689109011*^9}, 
   3.857416933479959*^9, 3.857416964049262*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[1]:=",
 CellID->725022207,ExpressionUUID->"1b1f7252-9285-4261-9875-d8e605f93702"],

Cell[BoxData[
 TagBox["\<\"/Users/franktabakin/Library/Mathematica/Applications/QDENSITY\"\>\
",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260727215242`*^9},
 CellLabel->"Out[1]=",
 CellID->931308411,ExpressionUUID->"beebce15-ba03-4a4d-b2ee-ca9350df68d9"]
}, Open  ]],

Cell["\<\
To check proper installation type in:\[LineSeparator]SystemOpen[\
\[OpenCurlyDoubleQuote]/Users/franktabakin/Library/Mathematica/Applications/\
QDENSITY\[CloseCurlyDoubleQuote]] \
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.8574180105382957`*^9, 3.857418038234988*^9}, {
  3.857730852823078*^9, 3.857730856793789*^9}, {3.858849134839322*^9, 
  3.858849156391693*^9}, {3.8588492060929127`*^9, 3.858849212111782*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->666673628,ExpressionUUID->"a4703e14-a299-4714-89ee-29d516877b6b"],

Cell[TextData[{
 StyleBox["To access the tutorials on a range of examples click here Click  ",
  
  FontSize->18],
 Cell[BoxData[
  ButtonBox["TUTORIALS",
   BaseStyle->"Hyperlink",
   ButtonData->{
     URL["http://www.pitt.edu/~tabakin/QDENSITY/Tutorials.zip"], None},
   ButtonNote->"http://www.pitt.edu/~tabakin/QDENSITY/Tutorials.zip"]],
  CellChangeTimes->{{3.858174055576445*^9, 3.858174055578929*^9}},
  ExpressionUUID->"c552602b-9eff-488f-baff-cd6b5a95a34d"],
 StyleBox["  . These and a lots more are  alsoincluded directly in the full \
download.",
  FontSize->18]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.8579215703044662`*^9, 3.8581765055485764`*^9, {3.8581765468931417`*^9, 
   3.8581766184210052`*^9}, {3.858849277779378*^9, 3.858849328405106*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1262785300,ExpressionUUID->"e00e0296-14f8-4e20-a7ce-4512f364f2ee"],

Cell[TextData[StyleBox["Once the Context is set with Mathematica running , \
load Qdensity, QCWave and, if required, Circuits and BTSystem using:",
 FontFamily->"Helvetica",
 FontSize->14]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321198283746*^9, 3.6133212155955057`*^9}, {
   3.8046748973996763`*^9, 3.804674906579303*^9}, {3.856097091458552*^9, 
   3.856097110088636*^9}, 3.856192059790223*^9, {3.856364675196232*^9, 
   3.856364685939908*^9}, {3.8563647272171507`*^9, 3.856364743102263*^9}, {
   3.857237445705262*^9, 3.857237563112193*^9}, {3.857417891330803*^9, 
   3.857417944633361*^9}, {3.8574179857762403`*^9, 3.857417990095211*^9}, 
   3.857418057360512*^9, {3.858849354044359*^9, 3.858849451605303*^9}},
 TextJustification->0,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellID->500701437,ExpressionUUID->"72c04f85-fa84-45c7-a506-90fc65a88a9d"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "shdw"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<QDENSITY`QCWave`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<QDENSITY`BTSystem`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<QDENSITY`Circuits`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613237576903551*^9, 3.613237577366025*^9}, {
   3.80467486180441*^9, 3.804674862267816*^9}, 3.804674892452276*^9, {
   3.856097063817255*^9, 3.856097076518053*^9}, {3.856363455628429*^9, 
   3.8563634622936277`*^9}, {3.856364701296528*^9, 3.8563647080123873`*^9}, {
   3.856376414243559*^9, 3.856376426311267*^9}, {3.8565521913586884`*^9, 
   3.8565521948123703`*^9}, {3.856621037071043*^9, 3.856621038824173*^9}, {
   3.856621109981332*^9, 3.8566211178583612`*^9}, {3.856621426055084*^9, 
   3.85662145564252*^9}, 3.856726881905161*^9, {3.856727723910207*^9, 
   3.856727751723989*^9}, {3.8567277931109943`*^9, 3.856727834747912*^9}, 
   3.8567295013092012`*^9, {3.858938532223632*^9, 3.858938532836831*^9}, {
   3.858939879858122*^9, 3.858939880568348*^9}, {3.858941085044058*^9, 
   3.858941086561026*^9}, {3.858967013804846*^9, 3.858967014972423*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellLabel->"In[2]:=",
 CellID->1448122713,ExpressionUUID->"936059f3-43bf-4a5d-97c9-8572aa33f93f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "introBT"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613237576903551*^9, 3.613237577366025*^9}, {
   3.80467486180441*^9, 3.804674862267816*^9}, 3.804674892452276*^9, {
   3.856097063817255*^9, 3.856097076518053*^9}, {3.856363455628429*^9, 
   3.8563634622936277`*^9}, {3.856364701296528*^9, 3.8563647080123873`*^9}, {
   3.856376414243559*^9, 3.856376426311267*^9}, {3.8565521913586884`*^9, 
   3.8565521948123703`*^9}, {3.856621037071043*^9, 3.856621038824173*^9}, {
   3.856621109981332*^9, 3.8566211178583612`*^9}, {3.856621426055084*^9, 
   3.85662145564252*^9}, 3.856726881905161*^9, {3.856727723910207*^9, 
   3.856727751723989*^9}, {3.8567277931109943`*^9, 3.856727834747912*^9}, 
   3.8567295013092012`*^9, {3.8572377341033907`*^9, 3.857237743001957*^9}, {
   3.8572377799257936`*^9, 3.8572378068590927`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellLabel->"In[7]:=",
 CellID->304033761,ExpressionUUID->"8c88cedf-4903-4600-b36e-e724e4a12286"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"The quantum states and density matix of quantum \
systems which consist of\\nmixtures of qubits and qutrits are provided in \
this notebook.\\nPartial trace, and othe commands are also provided \\nand \
are illustrated.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`BTSystem`introBT\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"The quantum states and density matix of quantum \
systems which consist of\\nmixtures of qubits and qutrits are provided in \
this notebook.\\nPartial trace, and othe commands are also provided \\nand \
are illustrated.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "The quantum states and density matix of quantum systems which consist \
of\nmixtures of qubits and qutrits are provided in this notebook.\nPartial \
trace, and othe commands are also provided \nand are illustrated.", 
     "Documentation" -> None, "OwnValues" -> None, "UpValues" -> None, 
     "DownValues" -> None, "SubValues" -> None, "DefaultValues" -> None, 
     "NValues" -> None, "FormatValues" -> None, "Options" -> None, 
     "Attributes" -> {}, "FullName" -> "QDENSITY`BTSystem`introBT"|>, False]],
  
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260728124237`*^9},
 CellTags->"b:5.0",
 CellLabel->"Out[7]=",
 CellID->1639644341,ExpressionUUID->"1425609d-48c3-4418-9542-957084c5356a"]
}, Open  ]],

Cell["\<\
To list all commands type in: ?QDENSITY`Qdensity`* , ?QDENSITY`QCWave`*, \
?QDENSITY`BTSysten`*, ?QDENSITY`Circuits`*.  To list commands starting with a \
particular letter such as \"C\" in each package type:\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.8561919985433807`*^9, 3.856192025487042*^9}, {
   3.8563435815694103`*^9, 3.8563436699618273`*^9}, {3.8563445026192217`*^9, 
   3.856344535504429*^9}, {3.85636437627493*^9, 3.8563646424992123`*^9}, {
   3.856364791129423*^9, 3.8563648722434597`*^9}, {3.8563650140657797`*^9, 
   3.856365047566119*^9}, {3.8563650790670633`*^9, 3.8563651136494827`*^9}, 
   3.856365604314191*^9, {3.856376430428441*^9, 3.85637644070473*^9}, {
   3.8567313445022573`*^9, 3.856731376359942*^9}, {3.856731417433854*^9, 
   3.856731527007228*^9}, {3.8567315611181707`*^9, 3.8567315711868362`*^9}, {
   3.857237584116355*^9, 3.857237621050187*^9}, 3.857416678280321*^9, {
   3.858849522639475*^9, 3.858849595843959*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellID->1166265940,ExpressionUUID->"64289182-a051-4239-8c47-5ce1fc80eb14"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`C*"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613321230783722*^9, 3.6133212398385773`*^9}, 
   3.6133971194048977`*^9, 3.613397155369432*^9, {3.85609705471546*^9, 
   3.856097057564794*^9}, {3.856097132232477*^9, 3.856097162968091*^9}, {
   3.856097247644438*^9, 3.856097253134474*^9}, 3.8560972870030727`*^9, {
   3.856098055344186*^9, 3.85609806490592*^9}, {3.856098111366088*^9, 
   3.856098133134186*^9}, 3.85609816326691*^9, {3.8560982287494307`*^9, 
   3.856098262141293*^9}, {3.856343457683543*^9, 3.856343530673567*^9}, {
   3.856365337516493*^9, 3.8563653412292137`*^9}, 3.856376440711306*^9, {
   3.856730282786313*^9, 3.856730286891732*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellLabel->"In[8]:=",
 CellID->822164855,ExpressionUUID->"803a85c8-a63a-4861-9475-de20a3f3bfe1"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
              
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["CNOT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "CNOT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Comm", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "Comm", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CompPTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "CompPTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ControlledY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "ControlledY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CPHASE", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "CPHASE", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Coeff", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "Coeff", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CompMeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "CompMeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ControlledX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "ControlledX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ControlledZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "ControlledZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CPHASEV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-c596842e-3db5-4f51-8e06-02babd5cc73d", {
                    "CPHASEV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.19]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326072861929*^9},
 CellTags->"b:5.0",
 CellLabel->"Out[8]=",
 CellID->992280883,ExpressionUUID->"5c6b1d06-880d-4560-b19d-cb30dc826d1e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`QCWave`D*"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613321230783722*^9, 3.6133212398385773`*^9}, 
   3.6133971194048977`*^9, 3.613397155369432*^9, {3.85609705471546*^9, 
   3.856097057564794*^9}, {3.856097132232477*^9, 3.856097162968091*^9}, {
   3.856097247644438*^9, 3.856097253134474*^9}, 3.8560972870030727`*^9, {
   3.856098055344186*^9, 3.85609806490592*^9}, {3.856098111366088*^9, 
   3.856098133134186*^9}, 3.85609816326691*^9, {3.8560982287494307`*^9, 
   3.856098262141293*^9}, {3.856343457683543*^9, 3.856343530673567*^9}, {
   3.8563655421461906`*^9, 3.856365574738142*^9}, 3.856376440731142*^9, 
   3.8567302966730223`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellLabel->"In[9]:=",
 CellID->1788765850,ExpressionUUID->"6cf172ce-a5cf-4e94-bb77-fc49ffffa74e"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`QCWave`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`QCWave`", "InformationGridGroupHeader"]}],
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["DForm", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8c3e7335-d3e7-4bc4-83fc-3ded715ebc5c", {
                    "DForm", "QDENSITY`QCWave`"}, False},
                  ButtonNote->"QDENSITY`QCWave`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["DFormA", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8c3e7335-d3e7-4bc4-83fc-3ded715ebc5c", {
                    "DFormA", "QDENSITY`QCWave`"}, False},
                  ButtonNote->"QDENSITY`QCWave`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.475]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260729054193`*^9},
 CellTags->"b:5.0",
 CellLabel->"Out[9]=",
 CellID->2054281626,ExpressionUUID->"efcb9b81-4676-4cb9-b100-2ebdfcab90a0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Circuits`C*"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613321230783722*^9, 3.6133212398385773`*^9}, 
   3.6133971194048977`*^9, 3.613397155369432*^9, {3.85609705471546*^9, 
   3.856097057564794*^9}, {3.856097132232477*^9, 3.856097162968091*^9}, {
   3.856097247644438*^9, 3.856097253134474*^9}, 3.8560972870030727`*^9, {
   3.856098055344186*^9, 3.85609806490592*^9}, {3.856098111366088*^9, 
   3.856098133134186*^9}, 3.85609816326691*^9, {3.8560982287494307`*^9, 
   3.856098262141293*^9}, {3.856343457683543*^9, 3.856343530673567*^9}, 
   3.8563764407398157`*^9, 3.856730301329558*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.0",
 CellLabel->"In[10]:=",
 CellID->513716746,ExpressionUUID->"cfa4bb5b-c11e-49c6-818e-ca1913a5fd20"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Circuits`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Circuits`", "InformationGridGroupHeader"]}],
              
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["C2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "C2", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["C3X", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "C3X", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CircuitW", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CircuitW", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNPHI", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNPHI", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRR3", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRR3", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRRnm2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRRnm2", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRX", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CX", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["C2X", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "C2X", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Circuit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "Circuit", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNOTC", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNOTC", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRG", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRRn", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRRn", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRSM", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRSM", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRY", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CY", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["C3", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "C3", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CircuitG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CircuitG", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNOTS", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNOTS", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRR2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRR2", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRRnm1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRRnm1", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRSP", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRSP", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CNTRZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CNTRZ", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-436c95ce-67da-4167-b44b-98ee9c8b9830", {
                    "CZ", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.11875]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326072949781*^9},
 CellTags->"b:5.0",
 CellLabel->"Out[10]=",
 CellID->1216695363,ExpressionUUID->"99e833b5-c05d-4972-a18a-4f232eba6c8f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`BTSystem`B*"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8560980922516108`*^9, 3.856098094771553*^9}, {
   3.856098176808053*^9, 3.8560981929859667`*^9}, {3.856343713885194*^9, 
   3.856343715749875*^9}, 3.856376440748836*^9, 3.8567303061903*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[11]:=",
 CellID->1599448578,ExpressionUUID->"f705a0da-dc2c-4806-a810-ad34a1ce0f7a"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`BTSystem`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`BTSystem`", "InformationGridGroupHeader"]}],
              
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["B", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "B", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BL", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BL", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BraT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BraT", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BTbit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BTbit", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BTop", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BTop", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BTOp1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BTOp1", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BTop2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BTop2", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["BTOp2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-45dd7f37-960b-4f9c-984d-7cb864a62dd3", {
                    "BTOp2", "QDENSITY`BTSystem`"}, False},
                  ButtonNote->"QDENSITY`BTSystem`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.11875]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326072994306*^9},
 CellLabel->"Out[11]=",
 CellID->659679207,ExpressionUUID->"c695688e-fbec-4fb1-b88f-a237ed667529"]
}, Open  ]],

Cell[TextData[StyleBox["Single-Qubits ",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858177450966955*^9, 3.8581774807914467`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->194863427,ExpressionUUID->"bac64e0c-c513-4938-859f-7d8e0d5c31b0"],

Cell["Single-qubit states", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575718136293507`*^9, {3.858176905300309*^9, 3.858176909755438*^9}, {
   3.85817703330509*^9, 3.858177143275281*^9}, {3.858177507825489*^9, 
   3.858177516116932*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1923252123,ExpressionUUID->"895a0a57-4151-47b6-b987-e39754ad7e7d"],

Cell[TextData[{
 StyleBox["As an introduction to using the QCPITT package, let us start by \
setting up the single qubit states, first in the form of Dirac state vectors \
and then as a density matrix.  A qubit is a simple quantum system with two \
discrete quantum states. For example, this can be a spin 1/2 system that \
could point in the \[PlusMinus] z-directions.  Or it could be the two \
circular polarization states of a photon.  We denote these two possibilities \
as a |0\[RightAngleBracket] or a |1\[RightAngleBracket] state.  In a \
classical computer these are distinct, separate and measurable states,  which \
are physical realizations for binary counting and are called bits.   For a \
quantum system,  the act of measurement disturbs the system and therefore the \
best description is that the quantum system is simultaneously in the |0\
\[RightAngleBracket]  and |1\[RightAngleBracket] state with associated \
probability amplitudes.  This is called a qubit and is described as a linear \
combination or superposition.  There are two complex probability amplitudes, ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox["a", TraditionalForm]],ExpressionUUID->
  "6753e0dd-57fb-4096-a715-550900ac9a15"],
 StyleBox[" and ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox["b", TraditionalForm]],ExpressionUUID->
  "d3da38a4-d719-4a78-9d33-14a4fb7f5e2a"],
 StyleBox[" ,  which yield the probability for the state  |0\
\[RightAngleBracket] as ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["a", "*"]}], TraditionalForm]],
  FontSize->24,ExpressionUUID->"f207ffc1-6b6c-4d73-86af-14ca1b47e434"],
 StyleBox[" ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox["a", TraditionalForm]],
  FontSize->24,ExpressionUUID->"9cee2df7-14fe-4b51-9d37-af7e36c92d21"],
 "  ",
 StyleBox["and for  |1\[RightAngleBracket] as ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "*"], TraditionalForm]],ExpressionUUID->
  "b44134cd-406f-4d63-855c-a32a82398157"],
 StyleBox[" ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox["b", TraditionalForm]],ExpressionUUID->
  "feed64c6-d943-4482-b076-c1bf3ef7818b"],
 ".",
 StyleBox["  Using various forms along with the DForm to display a Dirac ket \
we have  ",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQzbaX93DK7jeOSbyc10F0SFO+cT6QzgnSMgXRDnlR
liDa7pKBJ4hOapHwA9EByQfDQfTHMM98EJ237FE1iO7RTmSS3PvGcdvJ+wog
Om2XmCqIvpdqaQaiy1uPWYHos/MNfEE0y//2FBDtte9MK4ieodj3FER/91r6
BkSfYoiNkQbSYgcfgOmmfMVsEO233R1MX/hw+28BkNZ7+gJMh8i4sBQC6Y6F
zdwg+kffcn4Qvc/+I5hO69+mAqKzDt7WAtFiN7JCwfLyzeEg2sqh+5P2AaC4
8EwGHSB90zeMFUSfZQoXBtEMebv1QHSA1G99EM2n+NgURC9ayWMHogEAc6fO

  "],
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->766262050,ExpressionUUID->"8705f09c-8d1a-48b2-8e2b-21c265ca1fad"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{"Clear", "[", 
   RowBox[{
    FormBox["a",
     TraditionalForm], ",", "b"}], "]"}],
  FontColor->GrayLevel[0]], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   TemplateBox[{"\[Psi]"},
    "Ket"], "=", 
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], " ", 
     TemplateBox[{"0"},
      "Ket"]}], "+", 
    RowBox[{
     FormBox["b",
      TraditionalForm], 
     TemplateBox[{"1"},
      "Ket"], " "}]}]}],
  FontColor->GrayLevel[0]], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]", "=", 
  RowBox[{
   RowBox[{
    FormBox["a",
     TraditionalForm], " ", 
    RowBox[{"Ket", "[", "0", "]"}]}], "+", 
   RowBox[{
    FormBox["b",
     TraditionalForm], " ", 
    RowBox[{"Ket", "[", "1", "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%%", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Ket", "[", "0", "]"}], "==", " ", 
   TemplateBox[{"0"},
    "Ket"]}], "&&", 
  RowBox[{
   RowBox[{"Ket", "[", "1", "]"}], "==", " ", 
   TemplateBox[{"1"},
    "Ket"]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6133250358604097`*^9, 3.613325039695675*^9}, 
   3.856346523240962*^9, 3.8572387659386587`*^9, {3.8572396123529253`*^9, 
   3.857239683838079*^9}, {3.857241255889233*^9, 3.857241262042248*^9}, {
   3.857594044954316*^9, 3.85759406929985*^9}, {3.858850009715805*^9, 
   3.8588500456522303`*^9}, {3.8588500771499033`*^9, 3.858850106065596*^9}, {
   3.858850200248843*^9, 3.858850212443606*^9}, {3.858850690979898*^9, 
   3.858850704513856*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[12]:=",
 CellID->1731517089,ExpressionUUID->"8bb4b87e-9597-4f2c-8d22-b0067ef28360"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a"},
     {"b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073013863*^9},
 CellLabel->"Out[13]=",
 CellID->1917846235,ExpressionUUID->"f408f889-65a7-471f-8d71-a9682d9536cf"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a"},
     {"b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607302029*^9},
 CellLabel->"Out[14]=",
 CellID->1659613822,ExpressionUUID->"f392737c-a6a6-4317-8433-57214b2b1057"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", "b", "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607302372*^9},
 CellLabel->"Out[15]=",
 CellID->1611899118,ExpressionUUID->"7fe08351-014c-466f-8f56-f60ec8916391"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", "b", "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073028413*^9},
 CellLabel->"Out[16]=",
 CellID->682269321,ExpressionUUID->"1770de92-0498-4e88-a10a-2471010586ec"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073041265*^9},
 CellLabel->"Out[17]=",
 CellID->1485286714,ExpressionUUID->"25193d62-8687-42fb-ab29-4478ba3ad567"]
}, Open  ]],

Cell[TextData[StyleBox["A single qubit state is also written above using our \
equivalent Ket and Bra notation. Set the cell option \
\[OpenCurlyDoubleQuote]convert to\[CloseCurlyDoubleQuote] ->  standard.  ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240507146802*^9, 3.857240546284153*^9}, {
   3.857418427603351*^9, 3.857418437795698*^9}, {3.858850391117999*^9, 
   3.858850427973339*^9}, {3.8588507319076233`*^9, 3.8588507667942963`*^9}, {
   3.8588508289853563`*^9, 3.858850876798512*^9}, 3.858851512503398*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->721432069,ExpressionUUID->"791483d1-21fe-4d85-aa2b-fde78d025c98"],

Cell[TextData[{
 StyleBox["We normalize the state and set ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      SuperscriptBox["a", "*"]}], "+", 
     RowBox[{"b", " ", 
      SuperscriptBox["b", "*"]}]}], "}"}], "\[Rule]", "1"}]],
  CellChangeTimes->{
   3.8574160875852957`*^9, {3.857418459745562*^9, 3.8574184694439077`*^9}},
  FontFamily->"Times New Roman",
  FontWeight->"Regular",ExpressionUUID->
  "97b091c4-46bc-439c-865b-6bf5d0913e14"],
 StyleBox[",   which is a statement that we have the system and all of the \
system in hand.  The equation below asserts that the two states are distinct",
  
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, 3.857240953078332*^9, {
   3.85724223498763*^9, 3.8572423687802963`*^9}, {3.8572424424139223`*^9, 
   3.857242517260233*^9}, {3.857243176763379*^9, 3.85724321119724*^9}, {
   3.85741849278531*^9, 3.8574185137939053`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1553898169,ExpressionUUID->"62dfc3d2-c862-43b9-8249-79d613148b49"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox[
  RowBox[{" ", 
   RowBox[{
    RowBox[{
     RowBox[{"Bra", "[", "1", "]"}], " ", ".", " ", 
     RowBox[{"Ket", "[", "0", "]"}]}], "==", 
    RowBox[{"{", 
     RowBox[{"{", "0", "}"}], "}"}]}]}],
  FontWeight->"Plain"]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
   3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
   3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
   3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
   3.857240222212597*^9, 3.857240347246189*^9}, {3.857717426170455*^9, 
   3.8577174545663557`*^9}, 3.857900790754176*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[18]:=",
 CellID->350600306,ExpressionUUID->"b44e2eae-c3c5-4fe8-90f2-bda784ad6fa2"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073049354*^9},
 CellLabel->"Out[18]=",
 CellID->863889470,ExpressionUUID->"fb553eaa-4f90-4b6f-9dde-317f15ddafa5"]
}, Open  ]],

Cell[TextData[StyleBox[" The above Ket and Bra state vectors can also be \
displayed in Dirac notation using our DForm and DFormA  \
\[OpenCurlyDoubleQuote]Dirac Form\[CloseCurlyDoubleQuote] commands.  DForm is \
contained in QCWAVE.  It also prints out results in Dirac Form: ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321370591607*^9, 3.6133214350049677`*^9}, {
   3.6133249786670017`*^9, 3.6133249902641*^9}, {3.613722470598411*^9, 
   3.61372247383969*^9}, {3.624128295391075*^9, 3.624128295660448*^9}, {
   3.624192932739489*^9, 3.624192957758863*^9}, {3.6241931737731333`*^9, 
   3.624193182480035*^9}, {3.856344966914513*^9, 3.856344967927186*^9}, {
   3.856366794976356*^9, 3.856366796877421*^9}, {3.857242930703719*^9, 
   3.857242952217639*^9}, 3.85741977600453*^9, {3.857421591027165*^9, 
   3.857421592330755*^9}, {3.857460951823299*^9, 3.857460953971871*^9}, 
   3.8575939337392282`*^9, {3.857716255534788*^9, 3.857716281187456*^9}, {
   3.858851056937998*^9, 3.858851062999692*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->797502331,ExpressionUUID->"c4048d76-ef73-45f4-9f7a-d70e3d200e85"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   FormBox["a",
    TraditionalForm], " ", 
   RowBox[{"Ket", "[", "0", "]"}]}], " ", "+", " ", 
  RowBox[{"b", " ", 
   RowBox[{"Ket", "[", "1", "]"}]}]}], "\n", 
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], " ", 
     TemplateBox[{"0"},
      "Ket"]}], "+", 
    RowBox[{"b", 
     TemplateBox[{"1"},
      "Ket"]}]}], " ", "==", "%"}],
  FontColor->GrayLevel[0]], "\n", 
 RowBox[{"DForm", "[", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Ket", "[", "0", "]"}]}], " ", "+", " ", 
   RowBox[{"b", " ", 
    RowBox[{"Ket", "[", "1", "]"}]}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857593937475128*^9, 3.8575939374774323`*^9}, {
  3.857716292324112*^9, 3.857716294978183*^9}, {3.8588510810013857`*^9, 
  3.8588511307034082`*^9}, {3.8588511645327387`*^9, 3.85885124457764*^9}, {
  3.858851281595509*^9, 3.858851306038558*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[19]:=",
 CellID->2110286995,ExpressionUUID->"421a7c60-625f-4d61-9bed-b3607455d1e5"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a"},
     {"b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073061449*^9},
 CellLabel->"Out[19]=",
 CellID->253384923,ExpressionUUID->"fd62b351-0873-40f1-be7c-339f89bd9ee4"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260730752087`*^9},
 CellLabel->"Out[20]=",
 CellID->180328860,ExpressionUUID->"be0ac67a-fe09-45ba-bfe1-d907ec0c953f"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", "b", "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073079009*^9},
 CellLabel->"Out[21]=",
 CellID->687724103,ExpressionUUID->"2198bad7-f7f9-4f04-8f97-ef84011f1203"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", " ", 
   RowBox[{"Bra", "[", "0", "]"}]}], " ", "+", " ", 
  RowBox[{"b", " ", 
   RowBox[{"Bra", "[", "1", "]"}]}]}], "\n", 
 RowBox[{
  RowBox[{"a", " ", 
   TemplateBox[{"0"},
    "Bra"]}], "+", 
  RowBox[{"b", " ", 
   TemplateBox[{"1"},
    "Bra"]}]}], "\n", 
 RowBox[{"DFormA", "[", 
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"Bra", "[", "0", "]"}]}], " ", "+", " ", 
   RowBox[{"b", " ", 
    RowBox[{"Bra", "[", "1", "]"}]}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857593937475128*^9, 3.8575939374774323`*^9}, {
  3.857716292324112*^9, 3.857716294978183*^9}, {3.8588510810013857`*^9, 
  3.8588511307034082`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[22]:=",
 CellID->698403514,ExpressionUUID->"6b27ead0-f23e-4eef-844f-2caca4de0718"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a", "b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073087962*^9},
 CellLabel->"Out[22]=",
 CellID->822010967,ExpressionUUID->"66c09235-f671-42a8-bc16-5ca137f84483"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a", "b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073102398*^9},
 CellLabel->"Out[23]=",
 CellID->776021218,ExpressionUUID->"fbbe3cc6-25ed-46ba-b4dd-085eaa107cbd"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\" ( \"", "a", "\" ) \"", "\" < \"", "\"0\"", "\" |\"", 
    "\" + \"", "\" ( \"", "b", "\" ) \"", "\" < \"", "\"1\"", "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073106408*^9},
 CellLabel->"Out[24]=",
 CellID->920422695,ExpressionUUID->"2f3bb96f-d964-4398-ba4f-89fc69654a40"]
}, Open  ]],

Cell[TextData[StyleBox[" Here we introduce the row (bra form or adjoint/dual \
space) and the column (ket form) to\ncomplete Dirac's humorous label of the \
overlap  \[LeftAngleBracket]0 | 1\[RightAngleBracket] as a \"bracket.\"  The \
advantage of this brief notation is that it handles  superposition, \
normalization, orthogonality and completeness with grace,   with every step \
having a carefully designed physical quantum purpose",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321370591607*^9, 3.6133214350049677`*^9}, {
   3.6133249786670017`*^9, 3.6133249902641*^9}, {3.613722470598411*^9, 
   3.61372247383969*^9}, {3.624128295391075*^9, 3.624128295660448*^9}, {
   3.624192932739489*^9, 3.624192957758863*^9}, {3.6241931737731333`*^9, 
   3.624193182480035*^9}, {3.856344966914513*^9, 3.856344967927186*^9}, {
   3.856366794976356*^9, 3.856366796877421*^9}, {3.857242930703719*^9, 
   3.857242952217639*^9}, 3.85741977600453*^9, {3.857421591027165*^9, 
   3.857421592330755*^9}, {3.857460951823299*^9, 3.857460953971871*^9}, 
   3.8575939337392282`*^9, {3.857716255534788*^9, 3.857716281187456*^9}, {
   3.858851056937998*^9, 3.858851062999692*^9}, {3.858851541176095*^9, 
   3.858851943425713*^9}, {3.858854755057848*^9, 3.8588547575892153`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1671184905,ExpressionUUID->"a9dfffb4-7bca-4bef-989b-ddf73abe43c3"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Ket", "[", "0", "]"}], "==", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1"},
       {"0"}
      },
      GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}], " ", "&&", 
  RowBox[{
   RowBox[{"Ket", "[", "1", "]"}], "==", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0"},
       {"1"}
      },
      GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Bra", "[", "0", "]"}], "==", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0"}
      },
      GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}], "&&", " ", 
  RowBox[{
   RowBox[{"Bra", "[", "1", "]"}], "==", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "1"}
      },
      GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613233477447679*^9, {3.6133212815718603`*^9, 3.6133213205090647`*^9}, {
   3.624192961528791*^9, 3.6241929923917713`*^9}, {3.858851338449651*^9, 
   3.858851467049621*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"i:4", "b:5.1.0"},
 CellLabel->"In[25]:=",
 CellID->1176236190,ExpressionUUID->"1bab1f71-26b8-44fd-bea0-4f808dccef1d"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073115999*^9},
 CellTags->{"i:4", "b:5.1.0"},
 CellLabel->"Out[25]=",
 CellID->1219525524,ExpressionUUID->"f35f807e-03ec-48f0-a4e8-f2d8de1b6c7f"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073120626*^9},
 CellTags->{"i:4", "b:5.1.0"},
 CellLabel->"Out[26]=",
 CellID->981554900,ExpressionUUID->"58d87256-090b-4173-b33b-b541daa036bf"]
}, Open  ]],

Cell["\<\
For a qubit pointing in the direction specified by the Euler angles \[Alpha] \
\[Rule] (\[Theta],\[Phi]), we have:\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857461414115203*^9, 3.857461527256467*^9}, {
  3.857461583841734*^9, 3.85746158465473*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1203505892,ExpressionUUID->"e13aa294-4bfe-4442-846b-f2dda9b651d7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Psi]", "\[Alpha]"], "=", " ", 
    RowBox[{"Ket1", "[", 
     RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}]}], "  ", 
   "\[IndentingNewLine]", 
   RowBox[{"DForm", "[", 
    SubscriptBox["\[Psi]", "\[Alpha]"], "]"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857461328760482*^9, 3.857461364310032*^9}, {
  3.8574616917736187`*^9, 3.8574616933319197`*^9}, {3.857462139043955*^9, 
  3.8574621650877657`*^9}, {3.857488483979597*^9, 3.857488523796309*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[27]:=",
 CellID->1095091655,ExpressionUUID->"dce7f39e-ae04-4a54-804b-9a4bfb85c746"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073132502*^9},
 CellLabel->"Out[27]=",
 CellID->23828046,ExpressionUUID->"6ac1a507-8e4a-495e-afc8-7734b35f4e7b"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
      RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"0\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
      RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"1\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073137148*^9},
 CellLabel->"Out[28]=",
 CellID->1046010780,ExpressionUUID->"2c875efc-d510-4930-a005-1e1cc2a96d3c"]
}, Open  ]],

Cell[TextData[StyleBox["This state can also be obtained by a rotation.",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857900820195475*^9, 3.857900831108283*^9}, {3.8579008676821632`*^9, 
   3.857900876729253*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1709645044,ExpressionUUID->"d9456fa3-fece-4cb3-9b05-7d544938cf6a"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"RotZ", "[", "\[Phi]", "]"}], ".", 
  RowBox[{"RotY", "[", "\[Theta]", "]"}], ".", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ket1", "[", 
   RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], "]"}], "==", "%"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[29]:=",
 CellID->1676876806,ExpressionUUID->"72a37a5b-f28b-43dc-a165-fd1b5f82a611"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260731471786`*^9},
 CellLabel->"Out[29]=",
 CellID->1002045118,ExpressionUUID->"b378155b-aa5c-4cb5-9d19-9ac421d7ecf5"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260731520348`*^9},
 CellLabel->"Out[30]=",
 CellID->805538452,ExpressionUUID->"a7c80db6-098e-423f-a1f2-12e29bb2cb7c"]
}, Open  ]],

Cell[TextData[StyleBox["A general Wigner rotation with Euler angles  \
\[Theta], \[Phi], and \[Gamma] is",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857900820195475*^9, 3.857900831108283*^9}, {3.8579008676821632`*^9, 
   3.857900876729253*^9}, {3.857901007555524*^9, 3.857901087060597*^9}, {
   3.858677302654284*^9, 3.8586773046536427`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1862003300,ExpressionUUID->"90e0606d-6b4e-449a-b633-544475cfc220"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"WR", "[", 
    RowBox[{"\[Phi]v_", ",", " ", "\[Theta]v_", " ", ",", "\[Gamma]v_"}], " ",
     "]"}], ":=", 
   RowBox[{
    RowBox[{"RotZ", "[", "\[Phi]v", "]"}], ".", 
    RowBox[{"RotY", "[", "\[Theta]v", "]"}], ".", 
    RowBox[{"RotZ", "[", "\[Gamma]v", "]"}]}]}],
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular"], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"WRA", "[", 
    RowBox[{"\[Phi]v_", ",", " ", "\[Theta]v_", " ", ",", "\[Gamma]v_"}], " ",
     "]"}], ":=", 
   RowBox[{
    RowBox[{"RotZ", "[", 
     RowBox[{"-", "\[Gamma]v"}], "]"}], ".", 
    RowBox[{"RotY", "[", 
     RowBox[{"-", "\[Theta]v"}], "]"}], ".", 
    RowBox[{"RotZ", "[", 
     RowBox[{"-", "\[Phi]v"}], "]"}]}]}],
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular"], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"WR", "[", 
     RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "\[Gamma]"}], " ", 
     "]"}], ".", 
    RowBox[{"WRA", "[", 
     RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "\[Gamma]"}], "]"}]}], 
   "  ", "//", "FullSimplify"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "==", 
  RowBox[{"s", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"WR", "[", 
  RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "\[Gamma]"}], " ", 
  "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.8579010524821177`*^9, {3.857901097448627*^9, 3.857901154756669*^9}, {
   3.857901215808301*^9, 3.857901260762957*^9}, {3.8583360793097267`*^9, 
   3.858336098017478*^9}, {3.8583374122511873`*^9, 3.858337439270443*^9}, {
   3.858852232781826*^9, 3.858852269695154*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[31]:=",
 CellID->887258699,ExpressionUUID->"04c02445-dc8d-44bd-96ea-df27115c85b4"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260731645193`*^9},
 CellLabel->"Out[34]=",
 CellID->947852076,ExpressionUUID->"6bfd3f18-1f78-4e90-8939-fb9ce85e15b8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"]}], "-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"], "-", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]]}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"]}], "+", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"], "+", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073169299*^9},
 CellLabel->"Out[35]=",
 CellID->516147886,ExpressionUUID->"80fe2521-b1ac-4a4f-bb64-5a8da1bb12a9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"WR", "[", 
  RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "\[Gamma]"}], " ", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"WR", "[", 
    RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "\[Gamma]"}], "]"}], 
   ".", 
   RowBox[{"Ket", "[", "0", "]"}]}], " ", "/.", " ", 
  RowBox[{"\[Gamma]", "->", "0"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ket1", "[", 
   RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], "]"}], "==", 
  "%"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", 
  RowBox[{"Ket1", "[", 
   RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], "]"}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8583362264694843`*^9, 3.858336294279016*^9}, 
   3.858337457305481*^9, 3.858837317419351*^9, {3.858852286968158*^9, 
   3.858852288090342*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[36]:=",
 CellID->1263581457,ExpressionUUID->"75d6416d-0638-443e-8264-49c3a73f2e35"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"]}], "-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"], "-", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]]}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"]}], "+", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Gamma]"}], "2"], "+", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073179771*^9},
 CellLabel->"Out[36]=",
 CellID->1211069769,ExpressionUUID->"3d5127a9-af18-4064-8729-cee09a4b2991"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073185501*^9},
 CellLabel->"Out[37]=",
 CellID->599601317,ExpressionUUID->"baba1eb5-5385-4ba5-82df-53d847f3c155"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073189468*^9},
 CellLabel->"Out[38]=",
 CellID->1506063095,ExpressionUUID->"22d9fd68-cd96-4200-926f-69dd4fce485b"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
      RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"0\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
      RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"1\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607319305*^9},
 CellLabel->"Out[39]=",
 CellID->12690476,ExpressionUUID->"0f571668-205d-40d0-90a7-f0acc19c805c"]
}, Open  ]],

Cell[TextData[StyleBox["So we recover Ket1.  This is called a spinor \
rotation--a spin 1/2 rotation, where the Pauli operators are the generators \
of the unitary Wigner rotations.",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857900820195475*^9, 3.857900831108283*^9}, {3.8579008676821632`*^9, 
   3.857900876729253*^9}, {3.857901007555524*^9, 3.857901087060597*^9}, {
   3.858677302654284*^9, 3.8586773046536427`*^9}, {3.858852331974106*^9, 
   3.858852342470641*^9}, {3.8588770794998217`*^9, 3.8588771942534246`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->2954107,ExpressionUUID->"38bb8e14-24e9-499a-bdc1-1933384fd1e6"],

Cell[TextData[{
 StyleBox["This can be used to rotate states, operators and/or density \
matrices to expose any useful symmetries.\n\n",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[" If we need to find  the probability amplitude of the |0\
\[RightAngleBracket] or |1\[RightAngleBracket] state in Ket1, we form the \
\"bracket\" or overlap or projection as",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857900820195475*^9, 3.857900831108283*^9}, {3.8579008676821632`*^9, 
   3.857900876729253*^9}, {3.858852035700109*^9, 3.8588520992183437`*^9}, {
   3.858852142069963*^9, 3.85885217541533*^9}, {3.8588545119868937`*^9, 
   3.858854627177475*^9}, 3.858854664888089*^9, {3.8588547317929792`*^9, 
   3.858854735775167*^9}, {3.858854774824748*^9, 3.858854799924273*^9}, 
   3.858854836041634*^9, {3.858854887090468*^9, 3.858854919468144*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->2015713543,ExpressionUUID->"061df848-1281-4ebf-89fe-eb1c11b2fa8d"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  StyleBox["a",
   FontFamily->"Helvetica",
   FontWeight->"Plain"], 
  StyleBox["=",
   FontFamily->"Helvetica",
   FontWeight->"Plain"], 
  RowBox[{
   StyleBox[
    RowBox[{"Bra", "[", "0", "]"}],
    FontFamily->"Helvetica",
    FontWeight->"Plain"], 
   StyleBox[".",
    FontFamily->"Helvetica",
    FontWeight->"Plain"], 
   RowBox[{"Ket1", "[", 
    RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], 
    "]"}]}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"as", "=", 
   RowBox[{
    RowBox[{"Bra1", "[", 
     RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], "]"}], ".", 
    RowBox[{"Ket", "[", "0", "]"}]}]}],
  FontFamily->"Helvetica",
  FontWeight->"Plain"], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prob0", "=", 
   RowBox[{"as", " ", "a"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox["b",
   FontFamily->"Helvetica",
   FontWeight->"Plain"], 
  StyleBox["=",
   FontFamily->"Helvetica",
   FontWeight->"Plain"], 
  RowBox[{
   StyleBox[
    RowBox[{"Bra", "[", "1", "]"}],
    FontFamily->"Helvetica",
    FontWeight->"Plain"], 
   StyleBox[".",
    FontFamily->"Helvetica",
    FontWeight->"Plain"], 
   RowBox[{"Ket1", "[", 
    RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], 
    "]"}]}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"bs", "=", 
   RowBox[{
    RowBox[{"Bra1", "[", 
     RowBox[{"\[Theta]", ",", " ", "\[Phi]"}], "]"}], ".", 
    RowBox[{"Ket", "[", "1", "]"}]}]}],
  FontFamily->"Helvetica",
  FontWeight->"Plain"], "\[IndentingNewLine]", 
 RowBox[{"prob1", "=", 
  RowBox[{"bs", " ", "b"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prob0", "+", "prob1"}], "==", 
  "1"}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8583362264694843`*^9, 3.858336294279016*^9}, 
   3.858337457305481*^9, 3.858837317419351*^9, {3.858852286968158*^9, 
   3.858852288090342*^9}, {3.85885484031223*^9, 3.858854871599255*^9}, {
   3.858854928030163*^9, 3.8588549627477617`*^9}, {3.858855012951014*^9, 
   3.858855198664125*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[40]:=",
 CellID->1758193617,ExpressionUUID->"dcaf459b-730f-4071-8d34-b6e176d3c55a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260732085457`*^9},
 CellLabel->"Out[40]=",
 CellID->687949079,ExpressionUUID->"a193ae6e-ea9b-4cef-aafa-f2a565d2d8a9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073213788*^9},
 CellLabel->"Out[41]=",
 CellID->308215305,ExpressionUUID->"93e7be1d-da10-4d57-bdc4-94075639f657"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073217628*^9},
 CellLabel->"Out[42]=",
 CellID->444074571,ExpressionUUID->"3ab183d8-28a5-4dd7-ae08-2f519f820ab4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073221559*^9},
 CellLabel->"Out[43]=",
 CellID->1055641472,ExpressionUUID->"fa7beac7-eba3-4aa6-9902-0d13867ab589"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073225555*^9},
 CellLabel->"Out[44]=",
 CellID->1803812037,ExpressionUUID->"d1aae179-8ae4-45a3-aa25-6b442284aaa1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073229456*^9},
 CellLabel->"Out[45]=",
 CellID->642261160,ExpressionUUID->"fd9874a2-0910-4128-b52a-3a610cf7b16a"],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"], "+", 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"]}], "}"}], "}"}], "\[Equal]",
    "1"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073233078*^9},
 CellLabel->"Out[46]=",
 CellID->1230506298,ExpressionUUID->"908a5307-3368-4df8-b3c6-dc5d97e94882"]
}, Open  ]],

Cell[TextData[StyleBox["This example shows how to pick out the desired \
probability amplitude, calculate the probability and after repeated \
\"measurements\"  confirm that the full system has yielded a net probability \
of 1.  Soon we will deal with an extension of this idea where we get that \
selected part of a state vector in hand with its associated amplitude.  This \
is called a projection operator.\n",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857900820195475*^9, 3.857900831108283*^9}, {3.8579008676821632`*^9, 
   3.857900876729253*^9}, {3.858852035700109*^9, 3.8588520992183437`*^9}, {
   3.858852142069963*^9, 3.85885217541533*^9}, {3.8588545119868937`*^9, 
   3.858854627177475*^9}, 3.858854664888089*^9, {3.8588547317929792`*^9, 
   3.858854735775167*^9}, {3.858854774824748*^9, 3.858854799924273*^9}, 
   3.858854836041634*^9, {3.858854887090468*^9, 3.858854919468144*^9}, 
   3.85885523926534*^9, {3.8588552889347677`*^9, 3.858855588763783*^9}, {
   3.858855660551119*^9, 3.858855739493043*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1863427519,ExpressionUUID->"40db12d6-90af-4aa6-bbd0-d25685c24a7f"],

Cell[TextData[StyleBox["Pauli Spin Operators",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967105145105*^9, 3.858967133394075*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1489412029,ExpressionUUID->"07ce06ce-f55c-4908-a99a-f06074a56ead"],

Cell[TextData[{
 StyleBox["\nAny quantum system with two distinct addressable levels involves \
Pauli spin operators,  which take on the form of a set of 2 x 2 matrices.   \
This is the quantum SU(2) group. The Pauli spin matrices",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[" ",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox["are defined in this package as either s[i] or ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[Sigma]", "i"]],
  CellChangeTimes->{{3.613723767874393*^9, 3.613723884860752*^9}},
  ExpressionUUID->"89b991d0-5df0-479b-9cb9-bee64c08d4f4"],
 StyleBox["  or by using the built in PauliMatrix[i], where i denotes the \
{x,y, z}  or (1,2,3) vector components and i = 0 denotes the unit matrix.",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.85756603878119*^9, 3.857566060336678*^9}, {
   3.8588523791238947`*^9, 3.858852383646709*^9}, {3.8588557869014807`*^9, 
   3.8588558079189787`*^9}, {3.858855856233264*^9, 3.858855947668297*^9}, 
   3.8589671460029297`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1091194281,ExpressionUUID->"512b636f-9a2a-4e6a-b7f7-c3ab4f67472e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"s", "[", "0", "]"}], "==", 
   SubscriptBox["\[Sigma]", "0"], "==", 
   RowBox[{"PauliMatrix", "[", "0", "]"}]}], "&&", 
  RowBox[{
   RowBox[{"s", "[", "1", "]"}], "==", 
   SubscriptBox["\[Sigma]", "1"], "==", 
   RowBox[{"PauliMatrix", "[", "1", "]"}]}], "&&", 
  RowBox[{
   RowBox[{"s", "[", "2", "]"}], "==", 
   SubscriptBox["\[Sigma]", "2"], "==", 
   RowBox[{"PauliMatrix", "[", "2", "]"}]}], "&&", 
  RowBox[{
   RowBox[{"s", "[", "3", "]"}], "==", 
   SubscriptBox["\[Sigma]", "3"], "==", 
   RowBox[{"PauliMatrix", "[", "3", "]"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613723767874393*^9, 3.613723884860752*^9}, {
  3.857563100755495*^9, 3.857563123894925*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[47]:=",
 CellID->155418018,ExpressionUUID->"7d20adc1-cb2c-4eb9-849d-b6d98b160d0b"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260732423983`*^9},
 CellLabel->"Out[47]=",
 CellID->851829876,ExpressionUUID->"ad859d9c-8447-404d-8514-4556076dc6bb"]
}, Open  ]],

Cell[TextData[{
 StyleBox["These three form a spin vector {",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[Sigma]", "1"]],
  TaggingRules->{},
  CellChangeTimes->{{3.613723767874393*^9, 3.613723884860752*^9}, {
   3.857563100755495*^9, 3.857563123894925*^9}},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "e8abf211-b967-4e1a-beb7-9bf8942ecd7d"],
 " ",
 StyleBox[",",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[Sigma]", "2"]],
  TaggingRules->{},
  CellChangeTimes->{{3.613723767874393*^9, 3.613723884860752*^9}, {
   3.857563100755495*^9, 3.857563123894925*^9}},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "be3e151b-a079-4f11-a4c0-31dc6686d7df"],
 StyleBox[", ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[Sigma]", "3"]],
  TaggingRules->{},
  CellChangeTimes->{{3.613723767874393*^9, 3.613723884860752*^9}, {
   3.857563100755495*^9, 3.857563123894925*^9}},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "10a883ff-c1a2-4485-9896-1ba4fc31450b"],
 StyleBox["}  built from three matrices with zero trace and ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[Sigma]", "0"]],
  TaggingRules->{},
  CellChangeTimes->{{3.613723767874393*^9, 3.613723884860752*^9}, {
   3.857563100755495*^9, 3.857563123894925*^9}},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "115c00ec-982d-42f7-889e-bf935a670a27"],
 StyleBox["  is a 2X2 identity matrix with trace two.  They are all Hermitian \
matrices and therefore qualify as vector spin observables.",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.857564452776771*^9, 3.8575646573834248`*^9}, {
   3.8575658495741453`*^9, 3.857565876882691*^9}, {3.857566109432453*^9, 
   3.857566169890395*^9}, {3.8588559716635523`*^9, 3.858856002938509*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->717347682,ExpressionUUID->"8cc86ccb-2c28-4875-afe4-1f49df8e8af5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Row", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\"\<\!\(\*SubscriptBox[\(\[Sigma]\), \(0\)]\) =\>\"", "  ", 
     RowBox[{"MatrixForm", "[", 
      RowBox[{"s", "[", "0", "]"}], "]"}]}], ",", 
    RowBox[{"\"\<  \!\(\*SubscriptBox[\(\[Sigma]\), \(1\)]\) =\>\"", "   ", 
     RowBox[{"MatrixForm", "[", 
      RowBox[{"s", "[", "1", "]"}], "]"}]}], ",", 
    RowBox[{"\"\<  \!\(\*SubscriptBox[\(\[Sigma]\), \(2\)]\) =\>\"", "   ", 
     RowBox[{"MatrixForm", "[", 
      RowBox[{"s", "[", "2", "]"}], "]"}]}], ",", 
    RowBox[{"\"\<  \!\(\*SubscriptBox[\(\[Sigma]\), \(3\)]\) =\>\"", "   ", 
     RowBox[{"MatrixForm", "[", 
      RowBox[{"s", "[", "3", "]"}], "]"}]}]}], "}"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8575650843975353`*^9, 3.8575651801678467`*^9}, {
  3.857565211758026*^9, 3.857565214705389*^9}, {3.8575652582997713`*^9, 
  3.8575658269530783`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[48]:=",
 CellID->237211231,ExpressionUUID->"d5d524c0-f4c8-476a-bbd4-1cbae9fb0b3a"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
    RowBox[{"\"\\!\\(\\*SubscriptBox[\\(\[Sigma]\\), \\(0\\)]\\) =\"", " ", 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
         GridBox[{{"1", "0"}, {"0", "1"}}, RowSpacings -> 1, ColumnSpacings -> 
          1, RowAlignments -> Baseline, ColumnAlignments -> Center], 
         "\[NoBreak]", ")"}], 
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}], 
    RowBox[{
     "\"  \\!\\(\\*SubscriptBox[\\(\[Sigma]\\), \\(1\\)]\\) =\"", " ", 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
         GridBox[{{"0", "1"}, {"1", "0"}}, RowSpacings -> 1, ColumnSpacings -> 
          1, RowAlignments -> Baseline, ColumnAlignments -> Center], 
         "\[NoBreak]", ")"}], 
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}], 
    RowBox[{
     "\"  \\!\\(\\*SubscriptBox[\\(\[Sigma]\\), \\(2\\)]\\) =\"", " ", 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
         GridBox[{{"0", 
            RowBox[{"-", "\[ImaginaryI]"}]}, {"\[ImaginaryI]", "0"}}, 
          RowSpacings -> 1, ColumnSpacings -> 1, RowAlignments -> Baseline, 
          ColumnAlignments -> Center], "\[NoBreak]", ")"}], 
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}], 
    RowBox[{
     "\"  \\!\\(\\*SubscriptBox[\\(\[Sigma]\\), \\(3\\)]\\) =\"", " ", 
      TagBox[
       RowBox[{"(", "\[NoBreak]", 
         GridBox[{{"1", "0"}, {"0", 
            RowBox[{"-", "1"}]}}, RowSpacings -> 1, ColumnSpacings -> 1, 
          RowAlignments -> Baseline, ColumnAlignments -> Center], 
         "\[NoBreak]", ")"}], 
       Function[BoxForm`e$, 
        MatrixForm[BoxForm`e$]]]}]},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260732552853`*^9},
 CellLabel->"Out[48]=",
 CellID->1578735030,ExpressionUUID->"7f6bac79-b304-4310-8c73-2ef7f19a0264"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Note the Ket  and Bra states are eigenstates of the ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[Sigma]", "3"]],
  CellChangeTimes->{{3.857565645202107*^9, 3.8575656553722467`*^9}, 
    3.857565724325838*^9, {3.8575657879775*^9, 3.857565828152111*^9}},
  FontSize->18,ExpressionUUID->"42565fd7-b9e7-4194-a44f-99eb03cddd12"],
 StyleBox[" operator.",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.857564452776771*^9, 3.8575646573834248`*^9}, {
   3.8575658495741453`*^9, 3.857565876882691*^9}, {3.857566109432453*^9, 
   3.857566169890395*^9}, {3.857566213552692*^9, 3.857566221551381*^9}, 
   3.8575662584859467`*^9, {3.857569114637803*^9, 3.857569158035862*^9}, 
   3.857716319582336*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->731571697,ExpressionUUID->"68180cf7-ec6e-4c60-aae1-5f7267a7ac5b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Sigma]", "3"], ".", 
    RowBox[{"Ket", "[", "0", "]"}]}], "==", 
   RowBox[{
    RowBox[{"+", "1"}], " ", 
    RowBox[{"Ket", "[", "0", "]"}]}]}], "&&", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Sigma]", "3"], ".", 
    RowBox[{"Ket", "[", "1", "]"}]}], "==", 
   RowBox[{
    RowBox[{"-", "1"}], " ", 
    RowBox[{"Ket", "[", "1", "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Bra", "[", "0", "]"}], ".", 
    SubscriptBox["\[Sigma]", "3"]}], "==", 
   RowBox[{
    RowBox[{"+", "1"}], " ", 
    RowBox[{"Bra", "[", "0", "]"}]}]}], "&&", 
  RowBox[{
   RowBox[{
    RowBox[{"Bra", "[", "1", "]"}], ".", 
    SubscriptBox["\[Sigma]", "3"]}], "==", 
   RowBox[{
    RowBox[{"-", "1"}], 
    RowBox[{"Bra", "[", "1", "]"}], " "}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857568157045841*^9, 3.8575682434480886`*^9}, {
   3.857569175228182*^9, 3.857569268279292*^9}, 3.857584332136272*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[49]:=",
 CellID->1736156787,ExpressionUUID->"7828b306-78b8-4ed1-8be8-77cd668e870b"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073267943*^9},
 CellLabel->"Out[49]=",
 CellID->1639190525,ExpressionUUID->"490b9194-7a83-463c-8fdf-991985f47de1"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073272627*^9},
 CellLabel->"Out[50]=",
 CellID->1954929959,ExpressionUUID->"0a881632-ecd7-4116-99c6-2179839799c1"]
}, Open  ]],

Cell[BoxData[
 StyleBox[
  RowBox[{" ", 
   RowBox[{
    RowBox[{
     FormBox[
      OverscriptBox["\[Sigma]", "\[RightVector]"],
      TraditionalForm], " ", "=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"s", "[", "1", "]"}], ",", 
       RowBox[{"s", "[", "2", "]"}], ",", 
       RowBox[{"s", "[", "3", "]"}]}], "}"}]}], ";"}]}],
  FontSize->18]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8577309086275797`*^9, 3.857730943871333*^9}, {
   3.857730978457727*^9, 3.857731014210886*^9}, {3.8588524533090687`*^9, 
   3.858852454929915*^9}, 3.8588540179251947`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[51]:=",
 CellID->1020640493,ExpressionUUID->"b0718b17-c0f0-4edd-91ac-539b3832f0a6"],

Cell[TextData[{
 StyleBox["and the Ket1  and Bra1 vectors are eigenstates of the ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  StyleBox[
   RowBox[{
    StyleBox[" ",
     FontFamily->"Helvetica",
     FontWeight->"Plain"], 
    RowBox[{
     FormBox[
      OverscriptBox["\[Sigma]", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", " ", 
     OverscriptBox["n", "^"], 
     StyleBox[" ",
      FontFamily->"Helvetica",
      FontWeight->"Plain"]}]}],
   FontSize->18]],
  CellChangeTimes->{{3.857716553934258*^9, 3.857716572689931*^9}, 
    3.8577166645682707`*^9, {3.857716694853833*^9, 3.857716704913774*^9}, {
    3.8577168017415323`*^9, 3.8577168338720083`*^9}},ExpressionUUID->
  "b95c3ef4-4cb2-487b-ab85-979015cc65bd"],
 StyleBox[" operator.",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.857564452776771*^9, 3.8575646573834248`*^9}, {
   3.8575658495741453`*^9, 3.857565876882691*^9}, {3.857566109432453*^9, 
   3.857566169890395*^9}, {3.857566213552692*^9, 3.857566221551381*^9}, 
   3.8575662584859467`*^9, {3.857568943013465*^9, 3.857569031080187*^9}, {
   3.857569274016356*^9, 3.857569275625057*^9}, {3.857716583747754*^9, 
   3.857716598771413*^9}, 3.8577168586914673`*^9, {3.857716926882218*^9, 
   3.857716967604279*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1596268905,ExpressionUUID->"f07e6c48-11a2-41e5-9677-9303d6aa7dd6"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"nx", " ", "=", 
   RowBox[{
    RowBox[{"Sin", "[", "\[Theta]", "]"}], " ", 
    RowBox[{"Cos", "[", "\[Phi]", "]"}]}]}], ";", 
  RowBox[{"ny", " ", "=", 
   RowBox[{
    RowBox[{"Sin", "[", "\[Theta]", "]"}], " ", 
    RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], ";", 
  RowBox[{"nz", " ", "=", 
   RowBox[{"Cos", "[", "\[Theta]", "]"}]}], " ", ";"}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{" ", 
   RowBox[{
    OverscriptBox["n", "^"], " ", "=", 
    RowBox[{"{", 
     RowBox[{"nx", ",", "ny", ",", "nz"}], "}"}]}]}],
  FontSize->18], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   StyleBox[
    FormBox[
     OverscriptBox["\[Sigma]", "\[RightVector]"],
     TraditionalForm],
    FontSize->18], 
   StyleBox[" ",
    FontSize->18], 
   StyleBox["=",
    FontSize->18], 
   RowBox[{
    RowBox[{
     RowBox[{"s", "[", "#", "]"}], "&"}], "/@", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ndot\[Sigma]", "=", 
  RowBox[{
   StyleBox[
    RowBox[{"Dot", "[", " ", 
     RowBox[{
      OverscriptBox["n", "^"], 
      StyleBox[" ",
       FontFamily->"Helvetica",
       FontWeight->"Plain"], 
      StyleBox[",",
       FontFamily->"Helvetica",
       FontWeight->"Plain"], 
      StyleBox[" ",
       FontFamily->"Helvetica",
       FontWeight->"Plain"], 
      FormBox[
       OverscriptBox["\[Sigma]", "\[RightVector]"],
       TraditionalForm]}], 
     StyleBox["]",
      FontFamily->"Helvetica",
      FontWeight->"Plain"]}],
    FontSize->18], "//", "FullSimplify"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ndot\[Sigma]", ".", 
    RowBox[{"Ket1", "[", 
     RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}]}], "==", " ", 
   RowBox[{"Ket1", "[", 
    RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}]}], "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Bra1", "[", 
     RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}], " ", ".", "ndot\[Sigma]"}], "==",
    " ", 
   RowBox[{"Bra1", "[", 
    RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}]}], "//", 
  "FullSimplify"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8575683214415483`*^9, 3.8575683639217176`*^9}, {
   3.857568408518716*^9, 3.8575687307834063`*^9}, {3.857568763214602*^9, 
   3.857568864120208*^9}, 3.8575689023571377`*^9, {3.8575690721222897`*^9, 
   3.85756910134902*^9}, {3.857717091363689*^9, 3.8577170997919817`*^9}, 
   3.8577175698971653`*^9, 3.85771767772747*^9, {3.8577309569437723`*^9, 
   3.8577309677343483`*^9}, {3.857731852141973*^9, 3.857731872114231*^9}, {
   3.85773193683738*^9, 3.857731961527049*^9}, {3.8577320733137503`*^9, 
   3.8577321278537893`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[52]:=",
 CellID->1526048797,ExpressionUUID->"573a4e68-3bbb-44c6-bcf2-fa9932aee81a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{
        RowBox[{"Cos", "[", "\[Phi]", "]"}], " ", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
      {
       RowBox[{
        RowBox[{"Sin", "[", "\[Theta]", "]"}], " ", 
        RowBox[{"Sin", "[", "\[Phi]", "]"}]}]},
      {
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607328948*^9},
 CellLabel->"Out[53]=",
 CellID->8021813,ExpressionUUID->"6ebd6cff-d005-4a1b-93cb-5498a3074938"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", "\[Theta]", "]"}], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      RowBox[{"-", 
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260732951603`*^9},
 CellLabel->"Out[55]=",
 CellID->1678624977,ExpressionUUID->"ec09cae3-34c6-434e-a8d7-0a1e07fcb71d"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073299027*^9},
 CellLabel->"Out[56]=",
 CellID->1157317800,ExpressionUUID->"b16018af-0958-45cc-97a5-81ab941173ba"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073302618*^9},
 CellLabel->"Out[57]=",
 CellID->1280873721,ExpressionUUID->"f4c32f87-2789-4d82-8696-8987ac66758b"]
}, Open  ]],

Cell[TextData[StyleBox["Other properties follow:",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.857564452776771*^9, 3.8575646573834248`*^9}, {
   3.8575658495741453`*^9, 3.857565876882691*^9}, {3.857566109432453*^9, 
   3.857566169890395*^9}, {3.857566213552692*^9, 3.857566221551381*^9}, 
   3.8575662584859467`*^9, {3.857568943013465*^9, 3.857569031080187*^9}, {
   3.857569274016356*^9, 3.857569275625057*^9}, {3.857716583747754*^9, 
   3.857716598771413*^9}, 3.8577168586914673`*^9, {3.857716926882218*^9, 
   3.857716967604279*^9}, {3.858852494213049*^9, 3.858852512201985*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->46958099,ExpressionUUID->"10bd2765-324d-417f-9f15-04abf6b77beb"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{
    StyleBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[Sigma]", "1"], " ", ".", " ", 
       SubscriptBox["\[Sigma]", 
        RowBox[{"2", "  "}]]}], "==", " ", 
      RowBox[{"I", "  ", 
       SubscriptBox["\[Sigma]", "3"]}]}],
     FontWeight->"Bold"], 
    StyleBox[" ",
     FontWeight->"Bold"], 
    StyleBox["&&",
     FontWeight->"Plain"], 
    StyleBox[" ",
     FontWeight->"Bold"], 
    StyleBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[Sigma]", "2"], "  ", ".", " ", 
       SubscriptBox["\[Sigma]", 
        RowBox[{"3", "  "}]]}], "==", " ", 
      RowBox[{"I", " ", 
       SubscriptBox["\[Sigma]", "1"]}]}],
     FontWeight->"Regular"], 
    StyleBox["&&",
     FontWeight->"Plain"], 
    StyleBox[
     RowBox[{" ", 
      StyleBox[" ",
       FontWeight->"Bold"]}]], 
    StyleBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[Sigma]", "3"], "  ", ".", " ", 
       SubscriptBox["\[Sigma]", "1"]}], " ", "==", " ", 
      RowBox[{"I", " ", 
       SubscriptBox["\[Sigma]", "2"]}]}],
     FontWeight->"Regular"]}], "\[IndentingNewLine]"}],
  FontSize->18], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    StyleBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[Sigma]", "1"], "  ", ".", " ", 
       SubscriptBox["\[Sigma]", "1"]}], "  ", "==", 
      SubscriptBox["\[Sigma]", "0"]}],
     FontWeight->"Regular"], 
    StyleBox["  ",
     FontWeight->"Regular"], 
    StyleBox["&&",
     FontWeight->"Plain"], 
    StyleBox[
     RowBox[{" ", 
      StyleBox[" ",
       FontWeight->"Bold"]}]], 
    StyleBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[Sigma]", "2"], "  ", ".", " ", 
       SubscriptBox["\[Sigma]", "2"]}], "  ", "==", 
      SubscriptBox["\[Sigma]", "0"]}],
     FontWeight->"Regular"], 
    StyleBox["    ",
     FontWeight->"Regular"], 
    StyleBox["&&",
     FontWeight->"Plain"], 
    StyleBox[" ",
     FontWeight->"Bold"], 
    StyleBox[
     RowBox[{
      RowBox[{
       SubscriptBox["\[Sigma]", "3"], "  ", ".", " ", 
       SubscriptBox["\[Sigma]", "3"]}], "  ", "==", 
      SubscriptBox["\[Sigma]", "0"]}],
     FontWeight->"Regular"]}], 
   StyleBox["\[IndentingNewLine]",
    FontWeight->"Regular"]}],
  FontSize->18], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[Sigma]", "1"], "  ", ".", " ", 
     SubscriptBox["\[Sigma]", "2"]}],
    FontWeight->"Regular"], 
   StyleBox["  ",
    FontWeight->"Regular"], 
   StyleBox["==",
    FontWeight->"Regular"], 
   StyleBox[" ",
    FontWeight->"Regular"], 
   RowBox[{"I", 
    StyleBox[" ",
     FontWeight->"Regular"], 
    SubscriptBox["\[Sigma]", "3"]}]}], 
  StyleBox["  ",
   FontWeight->"Regular"], 
  StyleBox["&&",
   FontWeight->"Plain"], 
  StyleBox[
   RowBox[{" ", 
    StyleBox[" ",
     FontWeight->"Bold"]}]], 
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[Sigma]", "2"], "  ", ".", " ", 
     SubscriptBox["\[Sigma]", "3"]}],
    FontWeight->"Regular"], 
   StyleBox["  ",
    FontWeight->"Regular"], 
   StyleBox["==",
    FontWeight->"Regular"], 
   RowBox[{"I", 
    StyleBox[" ",
     FontWeight->"Regular"], 
    SubscriptBox["\[Sigma]", "1"]}]}], 
  StyleBox["    ",
   FontWeight->"Regular"], 
  StyleBox["&&",
   FontWeight->"Plain"], 
  StyleBox[" ",
   FontWeight->"Bold"], 
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[Sigma]", "3"], "  ", ".", " ", 
     SubscriptBox["\[Sigma]", "1"]}],
    FontWeight->"Regular"], 
   StyleBox["  ",
    FontWeight->"Regular"], 
   StyleBox["==",
    FontWeight->"Regular"], 
   RowBox[{"I", 
    StyleBox[" ",
     FontWeight->"Regular"], 
    SubscriptBox["\[Sigma]", "2"]}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857566270805284*^9, 3.857566353324642*^9}, {
   3.857566387672019*^9, 3.857566623171165*^9}, {3.857566691168898*^9, 
   3.857566702328617*^9}, {3.857717112357342*^9, 3.857717133097571*^9}, {
   3.85771756993666*^9, 3.857717569948415*^9}, 3.857717651253695*^9, {
   3.8577177196801653`*^9, 3.8577177222530107`*^9}, {3.857851640411729*^9, 
   3.8578516404168*^9}, {3.85790131036651*^9, 3.857901336815672*^9}, {
   3.8579018060713367`*^9, 3.857901924963587*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[58]:=",
 CellID->2038932978,ExpressionUUID->"1bd8d563-4afa-4edf-8242-1b82aa4aa711"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260733169327`*^9},
 CellLabel->"Out[58]=",
 CellID->1268551319,ExpressionUUID->"87343f3a-c53f-4a5e-9232-148e43bf03a1"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073322151*^9},
 CellLabel->"Out[59]=",
 CellID->324371017,ExpressionUUID->"1669dd63-ab6b-46f3-ba32-68c6a43c1986"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073325561*^9},
 CellLabel->"Out[60]=",
 CellID->1390186633,ExpressionUUID->"ac7ee7c5-7a9b-4d08-83fa-c5c67033428d"]
}, Open  ]],

Cell["\<\
We define raising and lowering operators that jump up from an assumed lower \
level
|0>  to  |1>  (sp) or fall down from |1>  to |0> (sm).   Indeed, one could \
start from this operator, define all of its desired attributes and then find \
you have again developed the Pauli operators\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.857564452776771*^9, 3.8575646573834248`*^9}, {
   3.8575658495741453`*^9, 3.857565876882691*^9}, {3.857566109432453*^9, 
   3.857566169890395*^9}, {3.857566213552692*^9, 3.857566221551381*^9}, 
   3.8575662584859467`*^9, {3.857568943013465*^9, 3.857569031080187*^9}, {
   3.857569274016356*^9, 3.857569275625057*^9}, {3.857716583747754*^9, 
   3.857716598771413*^9}, 3.8577168586914673`*^9, {3.857716926882218*^9, 
   3.857716967604279*^9}, {3.858852494213049*^9, 3.858852512201985*^9}, {
   3.8588766431954613`*^9, 3.858876949211446*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1165871760,ExpressionUUID->"b5d8fbde-d918-44d8-861c-fb4aec3e03c0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"sm", "=", 
  RowBox[{
   StyleBox[
    RowBox[{
     StyleBox["(",
      FontWeight->"Bold"], 
     RowBox[{
      StyleBox[
       SubscriptBox["\[Sigma]", "1"],
       FontWeight->"Bold"], 
      StyleBox["+",
       FontWeight->"Bold"], 
      StyleBox["  ",
       FontWeight->"Regular"], 
      StyleBox[
       RowBox[{"I", " ", 
        SubscriptBox["\[Sigma]", "2"]}],
       FontWeight->"Regular"]}], 
     StyleBox[")",
      FontWeight->"Regular"]}],
    FontSize->18], 
   StyleBox["/",
    FontWeight->"Regular"], 
   StyleBox["2",
    FontWeight->"Regular"]}]}], "\[IndentingNewLine]", 
 RowBox[{"sp", "=", 
  RowBox[{
   StyleBox[
    RowBox[{
     StyleBox["(",
      FontWeight->"Bold"], 
     RowBox[{
      StyleBox[
       SubscriptBox["\[Sigma]", "1"],
       FontWeight->"Bold"], 
      StyleBox["-",
       FontWeight->"Bold"], 
      StyleBox["  ",
       FontWeight->"Regular"], 
      StyleBox[
       RowBox[{"I", " ", 
        SubscriptBox["\[Sigma]", "2"]}],
       FontWeight->"Regular"]}], 
     StyleBox[")",
      FontWeight->"Regular"]}],
    FontSize->18], 
   StyleBox["/",
    FontWeight->"Regular"], 
   StyleBox["2",
    FontWeight->"Regular"]}]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{"sp", ".", 
    RowBox[{"Ket", "[", "0", "]"}]}],
   FontWeight->"Regular"], 
  StyleBox["==",
   FontWeight->"Regular"], 
  RowBox[{"Ket", "[", "1", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{"sm", ".", 
    RowBox[{"Ket", "[", "1", "]"}]}],
   FontWeight->"Regular"], 
  StyleBox["==",
   FontWeight->"Regular"], 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Bra", "[", "0", "]"}], ".", "sm"}], "==", 
  RowBox[{"Bra", "[", "1", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Bra", "[", "1", "]"}], ".", "sp"}], "==", 
  RowBox[{"Bra", "[", "0", "]"}]}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857566270805284*^9, 3.857566353324642*^9}, {
   3.857566387672019*^9, 3.857566623171165*^9}, {3.857566691168898*^9, 
   3.857566702328617*^9}, {3.857717112357342*^9, 3.857717133097571*^9}, {
   3.85771756993666*^9, 3.857717569948415*^9}, 3.857717651253695*^9, {
   3.8577177196801653`*^9, 3.8577177222530107`*^9}, {3.857851640411729*^9, 
   3.8578516404168*^9}, {3.85790131036651*^9, 3.857901336815672*^9}, {
   3.8579018060713367`*^9, 3.857901924963587*^9}, {3.858856191616756*^9, 
   3.858856425933378*^9}, {3.858856457098689*^9, 3.858856499802094*^9}, {
   3.858856578565374*^9, 3.858856620786405*^9}, {3.85885713268537*^9, 
   3.858857155868243*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[61]:=",
 CellID->1128177368,ExpressionUUID->"109d637f-a49c-41e4-be33-5a9a263e7db3"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "1"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073336196*^9},
 CellLabel->"Out[61]=",
 CellID->244661471,ExpressionUUID->"4db31d71-50e9-4d72-8a46-24894681c555"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"1", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073341372*^9},
 CellLabel->"Out[62]=",
 CellID->1741642739,ExpressionUUID->"2e282278-aebc-4be6-b346-fe692014059c"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073345128*^9},
 CellLabel->"Out[63]=",
 CellID->1587405763,ExpressionUUID->"ef349b95-2205-45db-940b-e89214f66a7e"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260733487663`*^9},
 CellLabel->"Out[64]=",
 CellID->1339312060,ExpressionUUID->"67cff5ca-b6e6-4b96-8516-15dcea4eb53a"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073352358*^9},
 CellLabel->"Out[65]=",
 CellID->2041688213,ExpressionUUID->"a0df63f4-ea44-4f9b-984a-48320fc68dcf"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073355888*^9},
 CellLabel->"Out[66]=",
 CellID->12600478,ExpressionUUID->"5e71b0cb-ed52-4546-9007-fca6145dc493"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"sp", ".", "sm"}], "-", 
   RowBox[{"sm", ".", "sp"}]}], "==", 
  RowBox[{"-", 
   RowBox[{"s", "[", "3", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"sp", ".", "sm"}], "+", 
   RowBox[{"sm", ".", "sp"}]}], "==", 
  RowBox[{"s", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"sp", ".", 
    RowBox[{"s", "[", "3", "]"}]}], "-", 
   RowBox[{
    RowBox[{"s", "[", "3", "]"}], ".", "sp"}]}], "==", " ", 
  RowBox[{"2", " ", "sp"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sp", ".", 
   RowBox[{"s", "[", "3", "]"}]}], "==", 
  RowBox[{"-", 
   RowBox[{
    RowBox[{"s", "[", "3", "]"}], ".", "sp"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"sm", ".", 
    RowBox[{"s", "[", "3", "]"}]}], "-", 
   RowBox[{
    RowBox[{"s", "[", "3", "]"}], ".", "sm"}]}], "==", 
  RowBox[{
   RowBox[{"-", "2"}], " ", "sm"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"sp", ".", 
    RowBox[{"s", "[", "3", "]"}]}], "==", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{"s", "[", "3", "]"}], ".", "sp"}]}]}], "&&", 
  RowBox[{
   RowBox[{"sm", ".", 
    RowBox[{"s", "[", "3", "]"}]}], "==", 
   RowBox[{"-", 
    RowBox[{
     RowBox[{"s", "[", "3", "]"}], ".", "sm"}]}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.858857213322007*^9, 3.8588574891978693`*^9}, {
  3.858857609002651*^9, 3.858857624469961*^9}, {3.8588576810603247`*^9, 
  3.858857713503807*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[67]:=",
 CellID->613794198,ExpressionUUID->"8514d453-a12e-4f3e-9abc-805b447dfcc0"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260733697987`*^9},
 CellLabel->"Out[67]=",
 CellID->1322836340,ExpressionUUID->"2e339475-27ff-4e34-bfcc-4dfbb1c93442"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073374708*^9},
 CellLabel->"Out[68]=",
 CellID->12168989,ExpressionUUID->"96160e05-1b3a-47c9-9ff9-d7009a8dff85"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073378759*^9},
 CellLabel->"Out[69]=",
 CellID->513251658,ExpressionUUID->"cea2252a-0874-4c4a-b634-184cf5939815"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073382586*^9},
 CellLabel->"Out[70]=",
 CellID->1926289654,ExpressionUUID->"1d098063-eee3-4ae8-88d1-5bd53d09fd03"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073386207*^9},
 CellLabel->"Out[71]=",
 CellID->1834236253,ExpressionUUID->"749e7d88-be3e-4190-b70e-5deb8cf679da"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260733897867`*^9},
 CellLabel->"Out[72]=",
 CellID->880951486,ExpressionUUID->"338d5da5-abdf-4cc4-b47c-acbdf79af97e"]
}, Open  ]],

Cell["\<\
The Pauli matrices form a basis for any trace 1 Hermitian matrix.  Later we \
will see that a density matrix for one qubit is a  trace-1, Hermitian matrix \
and can be expressed in terms of a real polarization vector {Px, Py, Pz}  of \
magnitude P. An Hermitian matrix has real eigenvalues.
\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.857461713773139*^9, 3.857461747343512*^9}, {
   3.857563655408656*^9, 3.8575637895216103`*^9}, {3.857563986637396*^9, 
   3.857564062070312*^9}, {3.857564452776771*^9, 3.8575646573834248`*^9}, {
   3.8575658495741453`*^9, 3.857565876882691*^9}, {3.857566109432453*^9, 
   3.857566169890395*^9}, {3.857566213552692*^9, 3.857566221551381*^9}, 
   3.8575662584859467`*^9, {3.857566744346588*^9, 3.857566865578514*^9}, {
   3.857567086880521*^9, 3.857567203839726*^9}, {3.8579019459436398`*^9, 
   3.857901947114162*^9}, {3.8588578890626583`*^9, 3.858857995860737*^9}, {
   3.858858079797678*^9, 3.8588580862104273`*^9}, {3.85887698796842*^9, 
   3.8588770094221487`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1111559247,ExpressionUUID->"1b43ce23-81ec-42ff-aa98-8e3b6bfc0fc0"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{"\[Rho]q1", "=", " ", 
   RowBox[{
    RowBox[{"1", "/", "2"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"s", "[", "0", "]"}], " ", "+", " ", 
      RowBox[{"Px", " ", 
       RowBox[{"s", "[", "1", "]"}]}], " ", "+", " ", 
      RowBox[{"Py", " ", 
       RowBox[{"s", "[", "2", "]"}]}], " ", "+", " ", 
      RowBox[{"Pz", " ", 
       RowBox[{"s", "[", "3", "]"}]}]}], ")"}]}]}],
  FontFamily->"Times New Roman",
  FontSize->18], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Eigenvalues", "[", "\[Rho]q1", "]"}], " ", "/.", " ", 
  RowBox[{
   SqrtBox[
    RowBox[{
     SuperscriptBox["Px", "2"], "+", 
     SuperscriptBox["Py", "2"], "+", 
     SuperscriptBox["Pz", "2"]}]], "->", "P"}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857566270805284*^9, 3.857566353324642*^9}, {
  3.857566387672019*^9, 3.857566623171165*^9}, {3.857566691168898*^9, 
  3.857566702328617*^9}, {3.857566903840571*^9, 3.857566940987771*^9}, {
  3.857567017333497*^9, 3.857567069634392*^9}, {3.857595154424938*^9, 
  3.857595159002*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[73]:=",
 CellID->624365819,ExpressionUUID->"05398b34-07d6-48bb-ad1e-084f141bfba7"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"1", "+", "Pz"}], "2"], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"Px", "-", 
         RowBox[{"\[ImaginaryI]", " ", "Py"}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"Px", "+", 
         RowBox[{"\[ImaginaryI]", " ", "Py"}]}], ")"}]}], 
      FractionBox[
       RowBox[{"1", "-", "Pz"}], "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260733990307`*^9},
 CellLabel->"Out[73]=",
 CellID->1399362784,ExpressionUUID->"08c3fcd6-55db-45cf-b435-f0c3db413edf"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{"1", "-", "P"}], "2"]},
      {
       FractionBox[
        RowBox[{"1", "+", "P"}], "2"]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607340475*^9},
 CellLabel->"Out[74]=",
 CellID->1996892391,ExpressionUUID->"1153870d-32df-462e-bba0-db7147011d7f"]
}, Open  ]],

Cell["\<\
The Wigner rotation WR[\[Phi], \[Theta] ,\[Gamma]] can be used to rotate a \
spin operator to a new direction:\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.858336390642211*^9, 3.8583364699782133`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->2039372162,ExpressionUUID->"0f34bf7c-5eb9-45df-9c63-bdcc36b3dab0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"nv", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Sin", "[", "\[Theta]", "]"}], "  ", 
      RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Sin", "[", "\[Theta]", "]"}], "  ", 
      RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
     RowBox[{"Cos", "[", "\[Theta]", "]"}]}], " ", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sv", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"s", "[", "1", "]"}], ",", 
     RowBox[{"s", "[", "2", "]"}], ",", 
     RowBox[{"s", "[", "3", "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nv", ".", "sv"}], " ", "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"WR", "[", 
     RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "0"}], "]"}], ".", 
    RowBox[{"s", "[", "3", "]"}], ".", 
    RowBox[{"WRA", "[", 
     RowBox[{"\[Phi]", ",", " ", "\[Theta]", " ", ",", "0"}], "]"}]}], " ", "==", 
   RowBox[{"nv", ".", "sv"}]}], "  ", "//", "FullSimplify"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.858336153887414*^9, 3.858336179469755*^9}, {
  3.858858177754573*^9, 3.858858184674429*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[75]:=",
 CellID->450621595,ExpressionUUID->"f06fa297-ebe1-44dd-8567-fdac3051a8cf"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", "\[Theta]", "]"}], 
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      RowBox[{"-", 
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260734179497`*^9},
 CellLabel->"Out[77]=",
 CellID->895980221,ExpressionUUID->"445a95b6-23a4-4683-af28-d62173c9a048"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073423149*^9},
 CellLabel->"Out[78]=",
 CellID->584241583,ExpressionUUID->"d3684238-27e6-4328-82c9-465d68f5e13d"]
}, Open  ]],

Cell[TextData[StyleBox["Single Qubit Projection Operators",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858177450966955*^9, 3.8581774807914467`*^9}, 
   3.8589672138786173`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->115326685,ExpressionUUID->"20d42eee-0b5e-4072-98d3-d588004b8520"],

Cell["", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575718136293507`*^9, {3.858176905300309*^9, 3.858176909755438*^9}, {
   3.85817703330509*^9, 3.8581771778016977`*^9}, 3.858967219027647*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1933253184,ExpressionUUID->"b5d4e574-5fa8-4935-adfc-3d628c9911bd"],

Cell[TextData[{
 "To pick out parts of the qubit,  we introduce the projection operators  ",
 Cell[BoxData[
  SubscriptBox["\[ScriptCapitalP]", "0"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}},
  FontFamily->"Times New Roman",ExpressionUUID->
  "5996f55a-365e-4088-9266-dd8b2d49590a"],
 " and ",
 Cell[BoxData[
  SubscriptBox["\[ScriptCapitalP]", "1"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}},
  FontFamily->"Times New Roman",ExpressionUUID->
  "895e5ed7-82c1-42d9-8dd7-957e2a8bc772"],
 ":"
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.858858301515491*^9, 3.858858307692813*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1757173766,ExpressionUUID->"266b47db-0102-4aab-bcbd-2d5f7f52f5d1"],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox["\[ScriptCapitalP]", "0"], "\[IndentingNewLine]", 
 SubscriptBox["\[ScriptCapitalP]", "1"], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[ScriptCapitalP]", "0"], "==", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"s", "[", "0", "]"}], "+", 
      RowBox[{"s", "[", "3", "]"}]}], ")"}], "/", "2"}]}], 
  StyleBox["&&",
   FontWeight->"Plain"], " ", 
  RowBox[{
   SubscriptBox["\[ScriptCapitalP]", "1"], "==", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"s", "[", "0", "]"}], "-", 
      RowBox[{"s", "[", "3", "]"}]}], ")"}], "/", "2"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613236934577767*^9, 3.6132369352505417`*^9}, {
   3.857241093065118*^9, 3.857241093551936*^9}, {3.8575672205437193`*^9, 
   3.857567294918477*^9}, 3.85785164041903*^9, 3.85817239810462*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[79]:=",
 CellID->1388185552,ExpressionUUID->"d3e219f5-2e01-4e70-b9d9-d8b85396cec0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073432446*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[79]=",
 CellID->1053051397,ExpressionUUID->"16f64f7d-33f4-4bbc-80f7-fa33c5816d74"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073437777*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[80]=",
 CellID->1222094452,ExpressionUUID->"fc0dda16-2515-43d1-96d0-9e97e0207348"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260734417753`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[81]=",
 CellID->1207854856,ExpressionUUID->"132d7cee-3cab-4aab-854c-e438748cae1b"]
}, Open  ]],

Cell[TextData[{
 StyleBox["The projection operators ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[ScriptCapitalP]", "0"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Plain",ExpressionUUID->
  "2436e35c-7aa5-4887-9f4a-ef5c7489b09c"],
 StyleBox[" and ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 Cell[BoxData[
  SubscriptBox["\[ScriptCapitalP]", "1"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Plain",ExpressionUUID->
  "24e1299b-bfce-4baa-ada3-05666e956426"],
 " do indeed pick out the associated part of the one qubit state \[Psi]  and \
of the state ",
 Cell[BoxData[
  SubscriptBox["\[Psi]", "\[Alpha]"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}, {3.857243234131968*^9, 
    3.85724325826435*^9}, {3.8574205428432827`*^9, 3.857420603224278*^9}, {
    3.857461953474538*^9, 3.85746203279425*^9}, {3.857462063893753*^9, 
    3.8574621083513308`*^9}, {3.8574621942816067`*^9, 3.857462212023601*^9}, {
    3.857462287636854*^9, 3.857462310323709*^9}, {3.857462342230369*^9, 
    3.8574624632577953`*^9}, {3.8574626982377453`*^9, 3.8574627488682003`*^9},
     3.8574796380412283`*^9, 3.857479835237175*^9, {3.857480406153956*^9, 
    3.857480457758625*^9}, 3.857480504705387*^9, {3.8574885634301558`*^9, 
    3.857488583785948*^9}, 3.857502840742144*^9, {3.857502979326013*^9, 
    3.857502980936702*^9}, {3.85750302521621*^9, 3.857503025815857*^9}},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "1d76eda3-c010-4fbd-b56f-bbed0b3f8d38"],
 "."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.8575673529806833`*^9, 3.857567490367691*^9}, {
   3.858858329144812*^9, 3.858858375295452*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->256705393,ExpressionUUID->"1e0e917b-98c8-494a-82c8-612f219c9fac"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Clear", "[", " ", 
  RowBox[{"\[Psi]", " ", ",", "a", ",", "b"}], "  ", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], "  ", 
     RowBox[{"Ket", "[", "0", "]"}]}], "+", 
    RowBox[{
     FormBox["b",
      TraditionalForm], "  ", 
     RowBox[{"Ket", "[", "1", "]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DForm", "[", "\[Psi]", "]"}], 
  StyleBox[
   RowBox[{" ", 
    StyleBox[" ",
     FontWeight->"Bold"]}]]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], " ", ".", " ", "\[Psi]"}],
    FontWeight->"Bold"], 
   StyleBox["==",
    FontWeight->"Bold"], 
   StyleBox[" ",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"a", "  ", 
     RowBox[{"Ket", "[", "0", "]"}]}],
    FontWeight->"Plain"]}], 
  StyleBox[" ",
   FontWeight->"Plain"]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "1"], " ", ".", " ", "\[Psi]"}],
    FontWeight->"Bold"], 
   StyleBox["==",
    FontWeight->"Bold"], 
   StyleBox[" ",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"b", "    ", 
     RowBox[{"Ket", "[", "1", "]"}]}],
    FontWeight->"Plain"]}], 
  StyleBox[" ",
   FontWeight->"Plain"]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     StyleBox["Adj",
      FontWeight->"Plain"], 
     StyleBox["[",
      FontWeight->"Bold"], "\[Psi]", "]"}], 
    StyleBox["   ",
     FontWeight->"Bold"], 
    StyleBox[".",
     FontWeight->"Bold"], 
    StyleBox["   ",
     FontWeight->"Bold"], 
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "0"],
     FontWeight->"Bold"]}], 
   StyleBox["==",
    FontWeight->"Bold"], 
   StyleBox[" ",
    FontWeight->"Bold"], 
   RowBox[{
    RowBox[{"Conjugate", "[", "a", "]"}], 
    StyleBox["  ",
     FontWeight->"Bold"], 
    StyleBox[
     RowBox[{"Bra", "[", "0", "]"}],
     FontWeight->"Plain"]}]}], 
  StyleBox[" ",
   FontWeight->"Plain"]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["Adj",
     FontWeight->"Plain"], 
    StyleBox["[",
     FontWeight->"Bold"], 
    StyleBox["\[Psi]",
     FontWeight->"Bold"], 
    StyleBox["]",
     FontWeight->"Bold"]}], 
   StyleBox["    ",
    FontWeight->"Bold"], 
   StyleBox[".",
    FontWeight->"Bold"], 
   StyleBox["   ",
    FontWeight->"Bold"], 
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", "1"],
    FontWeight->"Bold"]}], 
  StyleBox["==",
   FontWeight->"Bold"], 
  RowBox[{
   RowBox[{"Conjugate", "[", "b", "]"}], 
   StyleBox["  ",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"Bra", "[", "1", "]"}],
    FontWeight->"Plain"]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
   3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
   3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
   3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
   3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
   3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
   3.857240897390497*^9, 3.85724092056569*^9}, {3.857243234131968*^9, 
   3.85724325826435*^9}, {3.8574205428432827`*^9, 3.857420603224278*^9}, {
   3.857567495983399*^9, 3.85756760790276*^9}, 3.8575941381710443`*^9, {
   3.857594346142507*^9, 3.8575944441804113`*^9}, {3.857717153364809*^9, 
   3.857717160440463*^9}, {3.8577175699740267`*^9, 3.857717569981118*^9}, {
   3.858859116810885*^9, 3.858859118212556*^9}, {3.858859248969925*^9, 
   3.858859350645709*^9}, {3.858859409433717*^9, 3.858859448428706*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[82]:=",
 CellID->900017173,ExpressionUUID->"6bbe0890-90bb-402f-9039-ff64459d416a"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", "b", "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607345548*^9},
 CellLabel->"Out[84]=",
 CellID->253649701,ExpressionUUID->"6e44daed-8e92-46a5-bcfc-be23cf132cf7"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073462121*^9},
 CellLabel->"Out[85]=",
 CellID->335485412,ExpressionUUID->"317aa23c-3bbe-4be3-8ae3-771b3f13fe97"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073465955*^9},
 CellLabel->"Out[86]=",
 CellID->1358746109,ExpressionUUID->"00dbe06f-4243-45b4-b53e-57b8ef9ccd28"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073469562*^9},
 CellLabel->"Out[87]=",
 CellID->104751765,ExpressionUUID->"b65a9f07-a339-4b31-a993-1763c66fab10"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073472954*^9},
 CellLabel->"Out[88]=",
 CellID->1773807462,ExpressionUUID->"6285c803-0e42-4ee7-b13f-baa9e5bde015"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"DForm", "[", 
   SubscriptBox["\[Psi]", "\[Alpha]"], "]"}], 
  StyleBox[
   RowBox[{" ", 
    StyleBox[" ",
     FontWeight->"Bold"]}]]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], " ", ".", " ", 
     SubscriptBox["\[Psi]", "\[Alpha]"]}],
    FontWeight->"Plain"], 
   StyleBox["==",
    FontWeight->"Plain"], 
   StyleBox[" ",
    FontWeight->"Plain"], 
   RowBox[{
    StyleBox[
     TagBox[
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{
        StyleBox["Cos",
         FontWeight->"Plain"], "[", 
        FractionBox["\[Theta]", 
         StyleBox["2",
          FontWeight->"Plain"]], "]"}]}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]],
     FontWeight->"Plain"], 
    StyleBox["  ",
     FontWeight->"Bold"], 
    StyleBox[
     RowBox[{"Ket", "[", "0", "]"}],
     FontWeight->"Plain"]}]}], 
  StyleBox[" ",
   FontWeight->"Plain"]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "1"],
     FontWeight->"Plain"], 
    StyleBox[" ",
     FontWeight->"Bold"], 
    StyleBox[".",
     FontWeight->"Bold"], 
    StyleBox[" ",
     FontWeight->"Bold"], 
    StyleBox[
     SubscriptBox["\[Psi]", "\[Alpha]"],
     FontWeight->"Plain"]}], 
   StyleBox["==",
    FontWeight->"Plain"], 
   RowBox[{
    StyleBox[
     TagBox[
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"+", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], 
       RowBox[{
        StyleBox["Sin",
         FontWeight->"Plain"], "[", 
        FractionBox["\[Theta]", 
         StyleBox["2",
          FontWeight->"Plain"]], "]"}]}],
      Function[BoxForm`e$, 
       MatrixForm[BoxForm`e$]]],
     FontWeight->"Plain"], 
    StyleBox["   ",
     FontWeight->"Bold"], 
    StyleBox[
     RowBox[{"Ket", "[", "1", "]"}],
     FontWeight->"Plain"]}]}], 
  StyleBox[" ",
   FontWeight->"Plain"]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Assuming", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Theta]", "\[Element]", " ", "Reals"}], 
      StyleBox["&&",
       FontWeight->"Plain"], " ", 
      RowBox[{"\[Phi]", "\[Element]", " ", "Reals"}]}], ",", 
     RowBox[{"Refine", "[", 
      RowBox[{"Adj", "[", 
       SubscriptBox["\[Psi]", "\[Alpha]"], "]"}], "]"}]}], "]"}], " ", ".", 
   "   ", 
   SubscriptBox["\[ScriptCapitalP]", "0"]}], "==", 
  RowBox[{
   TagBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"+", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
     RowBox[{"Cos", "[", 
      FractionBox["\[Theta]", "2"], "]"}]}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], 
   StyleBox[
    RowBox[{" ", 
     StyleBox[" ",
      FontWeight->"Plain"]}]], 
   StyleBox[
    RowBox[{"Bra", "[", "0", "]"}],
    FontWeight->"Plain"]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Assuming", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Theta]", "\[Element]", " ", "Reals"}], 
      StyleBox["&&",
       FontWeight->"Plain"], " ", 
      RowBox[{"\[Phi]", "\[Element]", " ", "Reals"}]}], ",", 
     RowBox[{"Refine", "[", 
      RowBox[{"Adj", "[", 
       SubscriptBox["\[Psi]", "\[Alpha]"], "]"}], "]"}]}], "]"}], " ", ".", 
   "   ", 
   SubscriptBox["\[ScriptCapitalP]", "1"]}], "==", 
  RowBox[{
   TagBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], 
     RowBox[{"Sin", "[", 
      FractionBox["\[Theta]", "2"], "]"}]}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], " ", 
   RowBox[{"Bra", "[", "1", "]"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857594461088387*^9, 3.8575944812821207`*^9}, {
  3.8577171706298122`*^9, 3.857717188441008*^9}, {3.8577175700115347`*^9, 
  3.8577175700179987`*^9}, {3.857851640421884*^9, 3.857851640423917*^9}, {
  3.8581724315179358`*^9, 3.8581724433654118`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[89]:=",
 CellID->920608321,ExpressionUUID->"f372be1e-667d-4cac-8f03-7d63088f44ee"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
      RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"0\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
      RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"1\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073483469*^9},
 CellLabel->"Out[89]=",
 CellID->1014026180,ExpressionUUID->"1c2ccd6c-0787-4ce0-a031-ff5e7191a290"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260734909897`*^9},
 CellLabel->"Out[90]=",
 CellID->307784668,ExpressionUUID->"7348ef03-a71a-4c35-9c20-14ac5c46d768"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073495184*^9},
 CellLabel->"Out[91]=",
 CellID->1796293100,ExpressionUUID->"b231c5ce-4c7a-41e1-afcb-3e3627ce8c92"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073499021*^9},
 CellLabel->"Out[92]=",
 CellID->973022174,ExpressionUUID->"06b39800-c7bc-488a-8ca6-54aa308b71f7"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607350283*^9},
 CellLabel->"Out[93]=",
 CellID->495706513,ExpressionUUID->"957f5bc1-042b-4e0b-bf44-9bc34b4d3cef"]
}, Open  ]],

Cell[TextData[{
 StyleBox["And for the conjugate state",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 " ",
 Cell[BoxData[
  SubscriptBox["\[Psi]", "\[Alpha]"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}, {3.857243234131968*^9, 
    3.85724325826435*^9}, {3.8574205428432827`*^9, 3.857420603224278*^9}, {
    3.857461953474538*^9, 3.85746203279425*^9}, {3.857462063893753*^9, 
    3.8574621083513308`*^9}, {3.8574621942816067`*^9, 3.857462212023601*^9}, {
    3.857462287636854*^9, 3.857462310323709*^9}, {3.857462342230369*^9, 
    3.8574624632577953`*^9}, {3.8574626982377453`*^9, 3.8574627488682003`*^9},
     3.8574796380412283`*^9, 3.857479835237175*^9, {3.857480406153956*^9, 
    3.857480457758625*^9}, 3.857480504705387*^9, {3.8574885634301558`*^9, 
    3.857488583785948*^9}, 3.857502840742144*^9, {3.857502979326013*^9, 
    3.857502980936702*^9}, {3.85750302521621*^9, 3.857503025815857*^9}},
  FontFamily->"Times New Roman",
  FontWeight->"Regular",ExpressionUUID->
  "b542c3a3-67f6-4f90-9571-932486c01cee"],
 "  :"
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325059999547*^9}, 
   3.856346276781164*^9, {3.8563669757877493`*^9, 3.856366979523031*^9}, {
   3.857239436658148*^9, 3.857239467911229*^9}, {3.8572397108926086`*^9, 
   3.857239713985901*^9}, {3.857240092081153*^9, 3.857240214969061*^9}, {
   3.8572403614891148`*^9, 3.85724047345047*^9}, {3.857240953078332*^9, 
   3.857241050505108*^9}, {3.8575673529806833`*^9, 3.857567490367691*^9}, {
   3.858858329144812*^9, 3.858858375295452*^9}, {3.858859549306325*^9, 
   3.8588595778591623`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1852374601,ExpressionUUID->"8370951a-626f-4804-8fe4-b67424a26141"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"DForm", "[", 
  SubscriptBox["\[Psi]", "\[Alpha]"], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]c", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"Adj", "[", 
      SubscriptBox["\[Psi]", "\[Alpha]"], "]"}], ",", 
     RowBox[{
      RowBox[{"\[Theta]", "\[Element]", "Reals"}], " ", 
      StyleBox["&&",
       FontWeight->"Plain"], " ", 
      RowBox[{"\[Phi]", "\[Element]", " ", "Reals"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DFormA", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]c", " ", ".", 
   SubscriptBox["\[ScriptCapitalP]", "0"]}], "==", 
  RowBox[{
   TagBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"+", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
     RowBox[{"Cos", "[", 
      FractionBox["\[Theta]", "2"], "]"}]}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], "  ", 
   RowBox[{"Bra", "[", "0", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]c", " ", ".", " ", 
   SubscriptBox["\[ScriptCapitalP]", "1"]}], "==", 
  RowBox[{
   TagBox[
    RowBox[{
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
     RowBox[{"Sin", "[", 
      FractionBox["\[Theta]", "2"], "]"}]}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], "  ", 
   RowBox[{"Bra", "[", "1", "]"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8575034338877697`*^9, 3.857503454070775*^9}, {
   3.857503486719912*^9, 3.857503487247291*^9}, {3.857503847198186*^9, 
   3.85750390536007*^9}, {3.85750398207762*^9, 3.8575040068614798`*^9}, {
   3.857504078948884*^9, 3.857504130093967*^9}, {3.85750419979014*^9, 
   3.8575042112730093`*^9}, {3.857717201457088*^9, 3.857717202837597*^9}, {
   3.857717570049677*^9, 3.857717570051175*^9}, {3.857728500991283*^9, 
   3.857728531927356*^9}, 3.8578516404260817`*^9, 3.8581724587086143`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[94]:=",
 CellID->267973840,ExpressionUUID->"776f2bf9-aa6f-4b34-9df9-68a75408e796"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
      RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"0\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
      RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\")\"", "\" |\"", "\"1\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073516156*^9},
 CellLabel->"Out[94]=",
 CellID->1250297278,ExpressionUUID->"98170634-9860-48ac-9420-8a0d4875dd57"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\" ( \"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
      RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\" ) \"", "\" < \"", "\"0\"", 
    "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
      RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}], "\" ) \"", "\" < \"", "\"1\"", 
    "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073523858*^9},
 CellLabel->"Out[96]=",
 CellID->618865451,ExpressionUUID->"219b2454-cef7-4e52-8fec-9de7abca5e22"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073529891*^9},
 CellLabel->"Out[97]=",
 CellID->1008332536,ExpressionUUID->"1279782f-6d15-43c7-9bc1-20a951128edb"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607353379*^9},
 CellLabel->"Out[98]=",
 CellID->1251474075,ExpressionUUID->"2bd9f5df-c508-4627-9a67-79de9402bb43"]
}, Open  ]],

Cell[TextData[{
 "Note ",
 Cell[BoxData[
  SubscriptBox["\[ScriptCapitalP]", "0"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}},
  FontFamily->"Times New Roman",ExpressionUUID->
  "7c9818fd-c2d8-4b37-8fdb-d1523ed7437b"],
 " is also the single qubit  projection operator  |0\[RightAngleBracket]\
\[LeftAngleBracket]0|  and  ",
 Cell[BoxData[
  SubscriptBox["\[ScriptCapitalP]", "1"]],
  TaggingRules->{},
  CellChangeTimes->{{3.6133250358604097`*^9, 3.613325051817881*^9}, {
    3.8563464110863113`*^9, 3.856346417421904*^9}, 3.856346487463047*^9, {
    3.857239422904276*^9, 3.857239427366303*^9}, {3.857239723099821*^9, 
    3.857239839505124*^9}, {3.8572399205963907`*^9, 3.8572400075717297`*^9}, {
    3.857240222212597*^9, 3.857240347246189*^9}, {3.857240674693865*^9, 
    3.8572406929788017`*^9}, {3.857240740645938*^9, 3.857240862312213*^9}, {
    3.857240897390497*^9, 3.85724092056569*^9}},
  FontFamily->"Times New Roman",ExpressionUUID->
  "acd9193e-a6f9-4544-b736-8c46554ac94e"],
 " the single qubit  projection operator  |1\[RightAngleBracket]\
\[LeftAngleBracket]1| ."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.8563666032844954`*^9, 3.856366605883383*^9}, {
   3.857242563505273*^9, 3.857242619175333*^9}, 3.857242797603217*^9, {
   3.8574194029599113`*^9, 3.85741940915853*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->904569019,ExpressionUUID->"76142ac1-212c-44d8-8824-5f1a350fa1dc"],

Cell[TextData[{
 StyleBox[" For the |1\[RightAngleBracket] ",
  FontFamily->"Helvetica"],
 StyleBox["state, we have",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624192863641254*^9, 3.6241928761844397`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1054765220,ExpressionUUID->"e3fef630-c316-443f-b674-b6195389c722"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], " ", ".", 
     RowBox[{"Ket", "[", "0", "]"}]}], "==", 
    RowBox[{"Ket", "[", "0", "]"}]}], "&&", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "1"], " ", ".", 
     RowBox[{"Ket", "[", "1", "]"}]}], "==", 
    RowBox[{"Ket", "[", "1", "]"}]}]}],
  FontWeight->"Plain"], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    StyleBox["Flatten",
     FontWeight->"Plain"], 
    StyleBox["[",
     FontWeight->"Bold"], 
    RowBox[{
     StyleBox[
      SubscriptBox[
       StyleBox["\[ScriptCapitalP]",
        FontWeight->"Bold"], "0"],
      FontWeight->"Plain"], 
     StyleBox[" ",
      FontWeight->"Bold"], 
     StyleBox[".",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"Ket", "[", "1", "]"}],
      FontWeight->"Plain"]}], 
    StyleBox["]",
     FontWeight->"Plain"]}], 
   StyleBox["==",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"ZeroT", "[", "2", "]"}],
    FontWeight->"Plain"]}], 
  StyleBox["&&",
   FontWeight->"Plain"], 
  StyleBox[" ",
   FontWeight->"Bold"], 
  StyleBox[
   RowBox[{
    RowBox[{"Flatten", "[", 
     RowBox[{
      SubscriptBox[
       StyleBox["\[ScriptCapitalP]",
        FontWeight->"Bold"], "1"], "  ", ".", " ", 
      RowBox[{"Ket", "[", "0", "]"}]}], "]"}], "==", 
    RowBox[{"ZeroT", "[", "2", "]"}]}],
   FontWeight->"Plain"]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.85724263535316*^9, 3.857242768780056*^9}, {
   3.857717221050963*^9, 3.857717229589019*^9}, {3.857717570068501*^9, 
   3.857717570073451*^9}, {3.857728557093013*^9, 3.857728706708516*^9}, {
   3.857851640428733*^9, 3.857851640430374*^9}, 3.857851800308651*^9, 
   3.858172467022275*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[99]:=",
 CellID->2109030202,ExpressionUUID->"87d3bcc7-7a17-4e6d-8fd9-d088220a7b82"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073542253*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[99]=",
 CellID->1069939807,ExpressionUUID->"e8a37290-a701-4057-8619-769f721640e5"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073547386*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[100]=",
 CellID->1597759918,ExpressionUUID->"e0710e2b-9a81-4155-8a7d-50add735e77d"]
}, Open  ]],

Cell[TextData[StyleBox[" These Ket and Bra state vectors are directly \
displayed in Dirac notation (set the cell option \
\[OpenCurlyDoubleQuote]convert to\[CloseCurlyDoubleQuote] ->  standard)",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321370591607*^9, 3.6133214350049677`*^9}, {
  3.6133249786670017`*^9, 3.6133249902641*^9}, {3.613722470598411*^9, 
  3.61372247383969*^9}, {3.624128295391075*^9, 3.624128295660448*^9}, {
  3.856366695614904*^9, 3.856366758818201*^9}, {3.85741995295835*^9, 
  3.8574199564458*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->811402856,ExpressionUUID->"de65a51e-ecbc-4710-912a-cffd1b489ae8"],

Cell[CellGroupData[{

Cell[BoxData[{
 TemplateBox[{"0"},
  "Ket"], "\n", 
 TemplateBox[{"1"},
  "Bra"]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613233477447679*^9, {3.6133212815718603`*^9, 3.613321343767681*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"i:4", "b:5.1.0"},
 CellLabel->"In[101]:=",
 CellID->726642342,ExpressionUUID->"933d7000-d66e-43ad-afc6-168921c12a96"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260735606833`*^9},
 CellTags->{"i:4", "b:5.1.0"},
 CellLabel->"Out[101]=",
 CellID->1415419738,ExpressionUUID->"13b4580b-2d05-4c74-9d97-8a60cb49771e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260735710707`*^9},
 CellTags->{"i:4", "b:5.1.0"},
 CellLabel->"Out[102]=",
 CellID->1283178747,ExpressionUUID->"26090d83-bfc7-4447-88f5-42c1cd7b6ff3"]
}, Open  ]],

Cell[TextData[{
 StyleBox[" One can also build ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[ScriptCapitalP]", "0"], TraditionalForm]],ExpressionUUID->
  "f8899d3c-dbe3-44d1-aa2a-e302da7b3d88"],
 " ",
 StyleBox["using the KroneckerProduct symbol ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"ecacb4c3-add7-46b6-9121-755dd03e42c6"],
 StyleBox["  (see later comment)",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321453679144*^9, 3.613321461013953*^9}, {
   3.613722492007531*^9, 3.613722561270419*^9}, {3.613722636529367*^9, 
   3.6137226680053263`*^9}, {3.624186872347973*^9, 3.624186917876183*^9}, {
   3.624186955474079*^9, 3.6241869562063017`*^9}, {3.624193024191987*^9, 
   3.6241930283870687`*^9}, 3.856344986647437*^9, {3.856345527554949*^9, 
   3.856345539806026*^9}, 3.8574199783950443`*^9, {3.857421695113956*^9, 
   3.857421732261784*^9}, 3.859055240856876*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->2034662426,ExpressionUUID->"ab50856d-5bb3-4032-9655-8eeb47840a9f"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
   RowBox[{"Bra", "[", "0", "]"}]}],
  FontWeight->"Plain"], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "0"], "==", "%"}],
   FontWeight->"Bold"], 
  StyleBox["&&",
   FontWeight->"Plain"], 
  StyleBox[" ",
   FontWeight->"Plain"], 
  RowBox[{
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", "0"],
    FontWeight->"Bold"], 
   StyleBox["==",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{
     TemplateBox[{"0"},
      "Ket"], "\[CircleTimes]", " ", 
     TemplateBox[{"0"},
      "Bra"]}],
    FontWeight->"Plain"]}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"Ket", "[", "1", "]"}], " ", "\[CircleTimes]", " ", 
   RowBox[{"Bra", "[", "1", "]"}]}],
  FontWeight->"Plain"], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "==", "%"}],
   FontWeight->"Bold"], 
  StyleBox["&&",
   FontWeight->"Plain"], 
  StyleBox[" ",
   FontWeight->"Plain"], 
  RowBox[{
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", "1"],
    FontWeight->"Bold"], 
   StyleBox["==",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{
     TemplateBox[{"1"},
      "Ket"], "\[CircleTimes]", " ", 
     TemplateBox[{"1"},
      "Bra"]}],
    FontWeight->"Plain"]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8563450829496527`*^9, 3.856345131391944*^9}, {
  3.85756770811539*^9, 3.857567851070966*^9}, {3.857851640434087*^9, 
  3.857851640435924*^9}, {3.857851827612803*^9, 3.8578518373885717`*^9}, {
  3.859055240865094*^9, 3.859055240869384*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[103]:=",
 CellID->1009141171,ExpressionUUID->"f18c5bab-64c3-43ea-b6f4-e010a573543f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073584868*^9},
 CellLabel->"Out[103]=",
 CellID->1354417655,ExpressionUUID->"2e6d3ee0-5e6a-48e7-afaa-0b5a752b9b59"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073599864*^9},
 CellLabel->"Out[104]=",
 CellID->370869385,ExpressionUUID->"d7200518-dc13-41e8-8ff6-56192120203c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073604792*^9},
 CellLabel->"Out[105]=",
 CellID->1204210101,ExpressionUUID->"cec567fa-7401-42d2-a9c3-28c2134c495f"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073616433*^9},
 CellLabel->"Out[106]=",
 CellID->1273278179,ExpressionUUID->"ae20154d-8a86-48a1-9aa4-5ab7514829e7"]
}, Open  ]],

Cell[TextData[{
 StyleBox["A  general |\[Psi]> ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"b1dfc749-343a-4ba2-85ca-4a8f41c3b2a7"],
 StyleBox[" < \[Psi] |  Kronecker product is an operator (matrix)",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
   3.624188910429326*^9, 3.624188985287815*^9}, {3.6241934459235764`*^9, 
   3.624193452302559*^9}, {3.8560384211709967`*^9, 3.856038421207473*^9}, 
   3.856101375625527*^9, {3.856377926922001*^9, 3.856377958607066*^9}, {
   3.857572105175497*^9, 3.8575727311340446`*^9}, {3.857572790951664*^9, 
   3.857572910619753*^9}, {3.857573049669677*^9, 3.857573088844695*^9}, {
   3.8576266204393377`*^9, 3.8576266898658657`*^9}, {3.8578521404906063`*^9, 
   3.857852158356164*^9}, {3.857852197949232*^9, 3.8578522046998653`*^9}, {
   3.85785227461545*^9, 3.8578524135542927`*^9}, {3.858877389015086*^9, 
   3.858877422899418*^9}, {3.8588775675767403`*^9, 3.858877617324967*^9}, {
   3.858879993361844*^9, 3.858880048116064*^9}, {3.858935231098172*^9, 
   3.858935249705537*^9}, 3.858935690119213*^9, {3.8589369092735357`*^9, 
   3.858937077013294*^9}, 3.85905524090898*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->415603317,ExpressionUUID->"5b4ae767-c154-49a6-bbc2-60e7c504a48b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Psi]", " ", "=", 
  RowBox[{
   RowBox[{
    FormBox["a1",
     TraditionalForm], " ", 
    RowBox[{"Ket", "[", "0", "]"}]}], "+", 
   RowBox[{"a2", " ", 
    RowBox[{"Ket", "[", "1", "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]c", "=", 
  RowBox[{
   RowBox[{"a1c", " ", 
    RowBox[{"Bra", "[", "0", "]"}]}], "+", 
   RowBox[{"a2c", " ", 
    RowBox[{"Bra", "[", "1", "]"}]}]}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"\[Psi]", "  ", "\[CircleTimes]", "  ", "\[Psi]c", "   "}],
  FontFamily->"Helvetica"]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8563779897677794`*^9, 3.856377995021587*^9}, {
   3.856378063107504*^9, 3.856378119662374*^9}, {3.8588776436061707`*^9, 
   3.858877662776703*^9}, 3.858877707788372*^9, {3.858877746605549*^9, 
   3.858877832159272*^9}, {3.858877940828677*^9, 3.858877946824053*^9}, {
   3.858878882958482*^9, 3.858878945508842*^9}, {3.858878981993787*^9, 
   3.858879020536867*^9}, {3.8588790509799643`*^9, 3.858879053184376*^9}, {
   3.8588790860463877`*^9, 3.858879157327306*^9}, {3.858879207245596*^9, 
   3.858879331029359*^9}, {3.858879413657714*^9, 3.858879463558489*^9}, {
   3.858879511721764*^9, 3.858879521790598*^9}, {3.858879568038204*^9, 
   3.858879800302876*^9}, {3.858879910168659*^9, 3.858879911269844*^9}, 
   3.8588839687358093`*^9, 3.858884123577643*^9, {3.858884203989806*^9, 
   3.8588842416650887`*^9}, {3.8588843144225903`*^9, 3.858884315353805*^9}, {
   3.858884496010447*^9, 3.858884552260962*^9}, {3.8588845937851458`*^9, 
   3.858884600241918*^9}, {3.858935356887651*^9, 3.85893536251406*^9}, {
   3.858935405995116*^9, 3.858935532997342*^9}, {3.858935736367825*^9, 
   3.8589357912219477`*^9}, {3.8589358383272533`*^9, 3.858935848890414*^9}, {
   3.858935894511743*^9, 3.858935959760765*^9}, {3.858936099302146*^9, 
   3.858936107351988*^9}, {3.8589363586238623`*^9, 3.8589364424289494`*^9}, {
   3.8589365487410393`*^9, 3.85893672887752*^9}, {3.8589371194275723`*^9, 
   3.858937135504376*^9}, 3.859055240915703*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[107]:=",
 CellID->1903618416,ExpressionUUID->"12435e57-3a67-4786-8ba1-9871ec31d610"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a1"},
     {"a2"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073626786*^9},
 CellLabel->"Out[107]=",
 CellID->311812098,ExpressionUUID->"9651d4dd-2e16-4ebc-a23f-3eed3b60846e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a1c", "a2c"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073632564*^9},
 CellLabel->"Out[108]=",
 CellID->765832406,ExpressionUUID->"2cdf4bff-7ccd-4254-9ef7-466aac5148dc"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"a1", " ", "a1c"}], 
      RowBox[{"a1", " ", "a2c"}]},
     {
      RowBox[{"a1c", " ", "a2"}], 
      RowBox[{"a2", " ", "a2c"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073637038*^9},
 CellLabel->"Out[109]=",
 CellID->1475751171,ExpressionUUID->"bbe63fcf-d744-414b-ba6c-60447147d5a6"]
}, Open  ]],

Cell[TextData[StyleBox["These projection operators satisfy",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321453679144*^9, 3.613321461013953*^9}, {
   3.613722492007531*^9, 3.613722561270419*^9}, {3.613722636529367*^9, 
   3.6137226680053263`*^9}, {3.624186872347973*^9, 3.624186917876183*^9}, {
   3.624186955474079*^9, 3.6241869562063017`*^9}, {3.624193024191987*^9, 
   3.6241930283870687`*^9}, 3.856344986647437*^9, {3.856345204693364*^9, 
   3.856345209861588*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1616577872,ExpressionUUID->"14299fed-db8f-4335-9418-dd68c718829a"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], " ", "+", " ", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}], "==", 
    RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "&&", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], "  ", ".", " ", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}], "==", 
    RowBox[{"ZeroMatrix", "[", "2", "]"}]}]}],
  FontWeight->"Plain"], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], "  ", ".", " ", " ", 
     SubscriptBox["\[ScriptCapitalP]", "0"]}], "==", 
    SubscriptBox["\[ScriptCapitalP]", "0"]}], "&&", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "1"], "  ", ".", " ", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}], "==", 
    SubscriptBox["\[ScriptCapitalP]", "1"]}]}],
  FontWeight->"Plain"]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.6132372804999647`*^9, 3.624126754972478*^9, {3.624278410513*^9, 
   3.624278510252934*^9}, 3.8560384211324472`*^9, 3.8560385962273397`*^9, {
   3.856345158112487*^9, 3.856345280062546*^9}, 3.856613226046926*^9, {
   3.857567878860434*^9, 3.857567963844142*^9}, {3.857717237479706*^9, 
   3.857717246071471*^9}, {3.857717570101712*^9, 3.857717570104917*^9}, {
   3.857728771768228*^9, 3.857728782422256*^9}, {3.85785164043817*^9, 
   3.857851640439443*^9}, 3.857851848211315*^9, 3.858172476359827*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[110]:=",
 CellID->41798954,ExpressionUUID->"70174e60-e1de-45d0-a2a3-24e56054f41e"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073650853*^9},
 CellLabel->"Out[110]=",
 CellID->1661291794,ExpressionUUID->"ed44b9ae-dc6f-4b23-ac51-09f315276114"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260736566257`*^9},
 CellLabel->"Out[111]=",
 CellID->1896287111,ExpressionUUID->"706448a0-974e-4547-8dc1-522baf1a397b"]
}, Open  ]],

Cell[TextData[StyleBox["Additional projection operators are also available in \
the package:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321453679144*^9, 3.613321461013953*^9}, {
   3.613722492007531*^9, 3.613722561270419*^9}, {3.613722636529367*^9, 
   3.6137226680053263`*^9}, {3.624186872347973*^9, 3.624186917876183*^9}, {
   3.624186955474079*^9, 3.6241869562063017`*^9}, {3.624193024191987*^9, 
   3.6241930283870687`*^9}, 3.856344986647437*^9, {3.856345204693364*^9, 
   3.856345209861588*^9}, {3.8575696796182003`*^9, 3.857569710320051*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->402056739,ExpressionUUID->"67c30f82-85f7-421b-9c83-a9cbd6c65bd3"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"?", "PX"}], "\[IndentingNewLine]", 
 RowBox[{"?", "PY"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857668129110345*^9, 3.857668152793129*^9}, {
  3.857671255263533*^9, 3.857671272386043*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[112]:=",
 CellID->1110883851,ExpressionUUID->"53fc7c8e-9193-4aa1-897e-3115185ac60c"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"PX[a], \\n\\n\\t Projection operators onto the \
eigenstates \\nof SigmaX. PX[0] and PX[1] are equivalent to |+\
\[RightAngleBracket]\[LeftAngleBracket]+| and \\n|-\[RightAngleBracket]\
\[LeftAngleBracket]-| respectively.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"PX", "[", "0", "]"}], ":=", 
                    RowBox[{
                    RowBox[{"KetX", "[", "0", "]"}], "\[CircleTimes]", 
                    RowBox[{"BraX", "[", "0", "]"}]}]}]}, {" "}, {
                    RowBox[{
                    RowBox[{"PX", "[", "1", "]"}], ":=", 
                    RowBox[{
                    RowBox[{"KetX", "[", "1", "]"}], "\[CircleTimes]", 
                    RowBox[{"BraX", "[", "1", "]"}]}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`PX\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"PX[a], \\n\\n\\t Projection operators onto the \
eigenstates \\nof SigmaX. PX[0] and PX[1] are equivalent to |+\
\[RightAngleBracket]\[LeftAngleBracket]+| and \\n|-\[RightAngleBracket]\
\[LeftAngleBracket]-| respectively.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "PX[a], \n\n\t Projection operators onto the eigenstates \nof SigmaX. \
PX[0] and PX[1] are equivalent to |+\[RightAngleBracket]\[LeftAngleBracket]+| \
and \n|-\[RightAngleBracket]\[LeftAngleBracket]-| respectively.", 
     "Documentation" -> None, "OwnValues" -> None, "UpValues" -> None, 
     "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`PX, {
       QDENSITY`Qdensity`PX[0] :> CircleTimes[
          QDENSITY`Qdensity`KetX[0], 
          QDENSITY`Qdensity`BraX[0]], QDENSITY`Qdensity`PX[1] :> CircleTimes[
          QDENSITY`Qdensity`KetX[1], 
          QDENSITY`Qdensity`BraX[1]]}], "SubValues" -> None, "DefaultValues" -> 
     None, "NValues" -> None, "FormatValues" -> None, "Options" -> None, 
     "Attributes" -> {}, "FullName" -> "QDENSITY`Qdensity`PX"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073738819*^9},
 CellLabel->"Out[112]=",
 CellID->92669450,ExpressionUUID->"f0bac21d-ab6b-4bf7-8a88-986cceb5a597"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"PY[a], \\n\\n\\t Projection operators onto the \
eigenstates \\nof SigmaY.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"PY", "[", "0", "]"}], ":=", 
                    RowBox[{
                    RowBox[{"KetY", "[", "0", "]"}], "\[CircleTimes]", 
                    RowBox[{"BraY", "[", "0", "]"}]}]}]}, {" "}, {
                    RowBox[{
                    RowBox[{"PY", "[", "1", "]"}], ":=", 
                    RowBox[{
                    RowBox[{"KetY", "[", "1", "]"}], "\[CircleTimes]", 
                    RowBox[{"BraY", "[", "1", "]"}]}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`PY\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"PY[a], \\n\\n\\t Projection operators onto the \
eigenstates \\nof SigmaY.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "PY[a], \n\n\t Projection operators onto the eigenstates \nof SigmaY.", 
     "Documentation" -> None, "OwnValues" -> None, "UpValues" -> None, 
     "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`PY, {
       QDENSITY`Qdensity`PY[0] :> CircleTimes[
          QDENSITY`Qdensity`KetY[0], 
          QDENSITY`Qdensity`BraY[0]], QDENSITY`Qdensity`PY[1] :> CircleTimes[
          QDENSITY`Qdensity`KetY[1], 
          QDENSITY`Qdensity`BraY[1]]}], "SubValues" -> None, "DefaultValues" -> 
     None, "NValues" -> None, "FormatValues" -> None, "Options" -> None, 
     "Attributes" -> {}, "FullName" -> "QDENSITY`Qdensity`PY"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073813676*^9},
 CellLabel->"Out[113]=",
 CellID->569802075,ExpressionUUID->"02b96406-fba4-4ddc-9763-515ede207d89"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Special Note:\nIn Mathematica the  symbol \[OpenCurlyDoubleQuote]",
  
  FontSize->18],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"2d442b83-953c-430c-98da-8d86b99b0a33"],
 StyleBox["\[CloseCurlyDoubleQuote] appears in two versions, one is as \
CircleTimes and the other is as TensorProduct. In earlier versions of \
QDensity, whenever the TensorProduct form appeared, the AF (array flatten) \
operator was needed to obtain proper matrix form.  Now we use the CircleTimes \
form and set  CircleTimes = KroneckerProduct, which does not require AF.   \
The AF step has nevertheless been included in the code so that legacy \
versions can still be used.  All of this is included in the *.m files. (The  \
KroneckerProduct is equivalent to a tensor product.)  We have omitted all \
cases of the  tensorproduct version of \[OpenCurlyDoubleQuote]",
  FontSize->18],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"cd594ad3-d3be-4cda-9c50-4de597a8b27e"],
 StyleBox["\[CloseCurlyDoubleQuote] .",
  FontSize->18]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321453679144*^9, 3.613321461013953*^9}, {
   3.613722492007531*^9, 3.613722561270419*^9}, {3.613722636529367*^9, 
   3.6137226680053263`*^9}, {3.624186872347973*^9, 3.624186917876183*^9}, 
   3.624186950322571*^9, 3.624193052518622*^9, {3.8560976049386597`*^9, 
   3.856097808870267*^9}, {3.856345555252537*^9, 3.856345888786195*^9}, {
   3.856345920084217*^9, 3.8563460861928043`*^9}, 3.8563461476512117`*^9, {
   3.856346190402931*^9, 3.8563461960614862`*^9}, {3.856347135701276*^9, 
   3.8563471682705593`*^9}, 3.8563669206509237`*^9, 3.856376440798267*^9, {
   3.857420101652182*^9, 3.857420189493581*^9}, {3.857717285318667*^9, 
   3.8577172858808804`*^9}, {3.859055240957493*^9, 3.859055241007037*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->224595378,ExpressionUUID->"65b7138e-2c0b-4209-b047-79430580396f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "AF"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624126794410203*^9, 3.624126795974943*^9}, 
   3.85634621479405*^9, 3.8563771261584167`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[114]:=",
 CellID->535364109,ExpressionUUID->"5b97cfcb-7e71-48aa-9e8b-ae363c55ce9f"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"AF[state]\\n\\n   AF denotes ArrayFlatten, \
produces a correct matrix when needed for \\nthe older Tensorproduct setup.  \
Acts only on matrix. Defined by\\nAF=ArrayFlatten\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"AF", "[", 
                    RowBox[{"Private`st_", "?", "MatrixQ"}], "]"}], ":=", 
                    RowBox[{"ArrayFlatten", "[", "Private`st", "]"}]}]}, {
                    " "}}, GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`AF\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"AF[state]\\n\\n   AF denotes ArrayFlatten, \
produces a correct matrix when needed for \\nthe older Tensorproduct setup.  \
Acts only on matrix. Defined by\\nAF=ArrayFlatten\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "AF[state]\n\n   AF denotes ArrayFlatten, produces a correct matrix when \
needed for \nthe older Tensorproduct setup.  Acts only on matrix. Defined by\n\
AF=ArrayFlatten", "Documentation" -> None, "OwnValues" -> None, "UpValues" -> 
     None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`AF, {QDENSITY`Qdensity`AF[
          PatternTest[
           Pattern[Private`st, 
            Blank[]], MatrixQ]] :> ArrayFlatten[Private`st]}], "SubValues" -> 
     None, "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None,
      "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Qdensity`AF"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073929188*^9},
 CellLabel->"Out[114]=",
 CellID->1033892552,ExpressionUUID->"b2444fd4-e723-4628-9789-24d76ee8c870"]
}, Open  ]],

Cell[TextData[StyleBox["Density Matrix",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858177450966955*^9, 3.8581774807914467`*^9}, 
   3.8589672138786173`*^9, 3.858967246627688*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2098327973,ExpressionUUID->"11064670-1345-4b12-ab84-f3b1f527d215"],

Cell["", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575718136293507`*^9, {3.858176905300309*^9, 3.858176909755438*^9}, {
   3.85817703330509*^9, 3.8581771778016977`*^9}, 3.858967219027647*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->621791177,ExpressionUUID->"fe051219-0d0c-4175-9359-a26ff82e8c34"],

Cell[TextData[{
 StyleBox[" ",
  FontFamily->"Helvetica"],
 StyleBox["The density matrix is an alternate description of a quantum \
system.  The basic point of a density matrix is that it d",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox["escribes a classical ensemble average of an underlying quantum \
system.  In quantum mechanics the expected probability for any observable \
\[CapitalOmega]  is given by the expectation value < ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "fac700a7-9f2a-450e-b220-192db64da711"],
 StyleBox[" | \[CapitalOmega] | ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "9db92f6d-15f4-4b45-b8d0-f274c9e82f3d"],
 StyleBox["  >,   where  in  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "113916ac-eab8-4939-a130-285ee8ba019b"],
 StyleBox[" for example, \[Alpha] labels the direction that a spin is \
pointing.  (  In general a particle with spin is describe by the product of a \
spatial times a spin part such as \[Psi](x,t ) |",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "a10166ba-6112-482a-8a7f-9bd93fa49d2e"],
 StyleBox[">.  Here we  consider just  systems of spins,  without reference \
to the spatial wave functions.   The basic idea can be applied of course to \
the full spatial & spin system .)\n\nThus < ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "1f5287aa-489e-4763-abed-c8000e1c6615"],
 StyleBox[" | \[CapitalOmega] | ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "f5b7c33d-864b-4f50-9565-8e09a40fd838"],
 StyleBox["  > is a quantum probability for any specific spin direction \
\[Alpha].   If the spin direction occurs with a classical probability ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[DoubleStruckCapitalP]", "\[Alpha]"], TraditionalForm]],
  ExpressionUUID->"c195d8ba-4c86-46e6-aedc-e78ed4fd75ba"],
 StyleBox[" ,   we can take the classical average over an ensemble of quantum \
systems:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
   3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
   3.857420803944667*^9}, {3.857422515532448*^9, 3.857422677051435*^9}, {
   3.857422720443028*^9, 3.857422991080081*^9}, {3.857423023119973*^9, 
   3.8574233538046017`*^9}, {3.857423387683235*^9, 3.8574237376037493`*^9}, {
   3.857423773998373*^9, 3.857423922173999*^9}, {3.8574245500680943`*^9, 
   3.857424585519006*^9}, {3.857488661621876*^9, 3.857488741867947*^9}, {
   3.857488901277999*^9, 3.857488924363018*^9}, 3.8576717257202473`*^9, {
   3.857671834854065*^9, 3.857671932154661*^9}, {3.8577173241105137`*^9, 
   3.8577173241119347`*^9}, {3.85771757012768*^9, 3.857717570129148*^9}, {
   3.857728828393509*^9, 3.857728830039247*^9}, {3.857728876775701*^9, 
   3.8577288767771387`*^9}, {3.858840949968072*^9, 3.858840970576837*^9}, {
   3.85895804980337*^9, 3.8589580646646767`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->283093329,ExpressionUUID->"a1c0bc78-1e48-43fc-8295-db6eabc1c8e6"],

Cell[TextData[{
 StyleBox[" ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   StyleBox[
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         UnderscriptBox["\[Sum]", "\[Alpha]",
          LimitsPositioning->False], " ", 
         RowBox[{
          SubscriptBox[
           StyleBox["P", "TB"], "\[Alpha]"], " ", 
          RowBox[{"\[LeftAngleBracket]", 
           RowBox[{
            SubscriptBox["\[Psi]", "\[Alpha]"], "|", "\[CapitalOmega]", "|", 
            " ", 
            SubscriptBox["\[Psi]", "\[Alpha]"]}], 
           "\[RightAngleBracket]"}]}]}], " ", "\[LongEqual]", 
        RowBox[{
         UnderscriptBox["\[Sum]", 
          RowBox[{
           StyleBox["m", "TI"], ",", " ", 
           SuperscriptBox[
            StyleBox["m", "TI"], "\[Prime]"]}],
          LimitsPositioning->False], " ", 
         RowBox[{
          UnderscriptBox["\[Sum]", "\[Alpha]",
           LimitsPositioning->False], " ", 
          RowBox[{
           SubscriptBox[
            StyleBox["P", "TB"], "\[Alpha]"], " ", 
           RowBox[{"\[LeftAngleBracket]", 
            RowBox[{
             SubscriptBox["\[Psi]", "\[Alpha]"], " ", "|", 
             StyleBox["m", "TI"]}], "\[RightAngleBracket]"}], " ", 
           RowBox[{"\[LeftAngleBracket]", 
            RowBox[{
             SuperscriptBox[
              StyleBox["m", "TI"], "\[Prime]"], "|", 
             SubscriptBox["\[Psi]", "\[Alpha]"]}], "\[RightAngleBracket]"}], 
           " ", 
           RowBox[{"\[LeftAngleBracket]", 
            RowBox[{
             StyleBox["m", "TI"], "|", "\[CapitalOmega]", "|", 
             SuperscriptBox[
              StyleBox["m", "TI"], "\[Prime]"]}], 
            "\[RightAngleBracket]"}]}]}]}], "\[LongEqual]", 
        RowBox[{"Tr", "[", " ", 
         RowBox[{"\[Rho]", "\[CenterDot]", "\[CapitalOmega]"}], " ", "]"}]}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"\[Rho]", "\[LongEqual]", 
         RowBox[{
          UnderscriptBox["\[Sum]", "\[Alpha]",
           LimitsPositioning->True], 
          StyleBox[" ", "TB"], 
          SubscriptBox[
           StyleBox["P", "TB"], "\[Alpha]"]}]}], 
        StyleBox[" ", "TB"], "|", 
        SubscriptBox["\[Psi]", "\[Alpha]"]}]}], "\[RightAngleBracket]"}], 
     StyleBox[" ", "TB"], 
     RowBox[{"\[LeftAngleBracket]", 
      RowBox[{
       SubscriptBox["\[Psi]", "\[Alpha]"], "|", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Tr", "[", "\[Rho]", "]"}], "\[LongEqual]", 
        RowBox[{
         UnderscriptBox["\[Sum]", "\[Alpha]",
          LimitsPositioning->False], 
         StyleBox[" ", "TB"], 
         RowBox[{
          SubscriptBox[
           StyleBox["P", "TB"], "\[Alpha]"], 
          StyleBox[" ", "TB"], 
          RowBox[{"\[LeftAngleBracket]", 
           StyleBox[" ", "TB"], 
           RowBox[{
            SubscriptBox["\[Psi]", "\[Alpha]"], "|", 
            SubscriptBox["\[Psi]", "\[Alpha]"]}], 
           "\[RightAngleBracket]"}]}]}], 
        StyleBox[" ", "TB"], "\[LongEqual]", 
        RowBox[{
         UnderscriptBox["\[Sum]", "\[Alpha]",
          LimitsPositioning->False], 
         StyleBox[" ", "TB"], 
         SubscriptBox[
          StyleBox["P", "TB"], "\[Alpha]"]}], 
        StyleBox[" ", "TB"], "\[LongEqual]", 
        StyleBox["1", "TB"]}]}]}]}],
    FontWeight->"Plain"], TraditionalForm]],
  CellChangeTimes->{
   3.8589573744660063`*^9, {3.8589580414991426`*^9, 3.858958042535307*^9}, {
    3.858958583151518*^9, 3.858958584109562*^9}},
  EmphasizeSyntaxErrors->True,
  FontSize->24,ExpressionUUID->"e2882a80-732d-4537-83da-c943bf0db092"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
   3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
   3.857420803944667*^9}, {3.857424614631863*^9, 3.8574247647321987`*^9}, {
   3.857488782549274*^9, 3.8574888962742367`*^9}, {3.857488937694089*^9, 
   3.8574890613834667`*^9}, 3.857504334344377*^9, {3.85750437040906*^9, 
   3.8575044265095387`*^9}, 3.857504545863558*^9, {3.859053855493763*^9, 
   3.859053864939105*^9}, {3.860308694332636*^9, 3.86030870537444*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1372227264,ExpressionUUID->"2004a039-55f4-425d-911d-96f92cf3571d"],

Cell[TextData[{
 StyleBox[" Here the \[Alpha] sum (or integral) is over various possible spin \
directions \[Alpha], with probability  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  StyleBox[
   FormBox[
    SubscriptBox[
     StyleBox["P", "TB"], "\[Alpha]"],
    TraditionalForm],
   FontSize->24]],
  CellChangeTimes->{{3.857420361710814*^9, 3.8574203834008417`*^9}, {
    3.85742396986348*^9, 3.857423991217189*^9}, 3.8574240243239117`*^9, {
    3.857424105304082*^9, 3.857424274137732*^9}, {3.857424319871764*^9, 
    3.857424455511717*^9}},ExpressionUUID->
  "7c4ec8ee-da71-46cb-870b-ea254c668816"],
 StyleBox[".  The spin states, called the computational states, are the basic \
|0> and |1>.  From the above definition,  it follows that the density \
operator ",
  FontFamily->"Helvetica"],
 StyleBox["\[Rho]",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox[" is Hermitian and has a trace of 1,  ",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
   3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
   3.857420803944667*^9}, {3.857424614631863*^9, 3.8574247647321987`*^9}, {
   3.857488782549274*^9, 3.8574888962742367`*^9}, {3.857488937694089*^9, 
   3.8574890613834667`*^9}, 3.857504334344377*^9, {3.85750437040906*^9, 
   3.8575044265095387`*^9}, 3.857504545863558*^9, {3.859053855493763*^9, 
   3.859053864939105*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->509110344,ExpressionUUID->"64a68ffc-4eb0-4b19-a155-5fa55b7ee947"],

Cell[TextData[{
 StyleBox["  From the above definition,  it follows that overall value of an \
observable ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox["\[CapitalOmega]",
  FontFamily->"Helvetica",
  FontSize->18,
  FontWeight->"Plain"],
 StyleBox[" is \n",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[" ",
  FontFamily->"Helvetica",
  FontSize->24,
  FontWeight->"Plain"],
 StyleBox["<  \[CapitalOmega]   >   = ",
  FontFamily->"Helvetica",
  FontSize->24,
  FontWeight->"Plain"],
 Cell[BoxData[
  RowBox[{
   StyleBox["  ",
    FontFamily->"Times",
    FontSize->18], 
   StyleBox[
    RowBox[{"Tr", " ", 
     StyleBox["[",
      FontFamily->"Times"], 
     StyleBox[" ",
      FontFamily->"Times"], 
     StyleBox[
      RowBox[{"\[Rho]", " ", ".", " ", "\[CapitalOmega]"}],
      FontFamily->"Times"], 
     StyleBox[" ",
      FontFamily->"Times",
      FontColor->GrayLevel[0]], 
     StyleBox["]",
      FontFamily->"Times",
      FontColor->GrayLevel[0]]}],
    FontSize->24], 
   StyleBox[" ",
    FontFamily->"Times",
    FontSize->24,
    FontColor->GrayLevel[0]]}]],
  CellChangeTimes->{{3.85748907727853*^9, 3.857489217331621*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Plain",ExpressionUUID->
  "28637c14-9502-464a-9720-6726591e96e6"],
 StyleBox[",\nso for example the ensemble average polarization is",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
   3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
   3.857420803944667*^9}, {3.857424614631863*^9, 3.8574247647321987`*^9}, {
   3.857488782549274*^9, 3.8574888962742367`*^9}, {3.857488937694089*^9, 
   3.8574890613834667`*^9}, 3.857504334344377*^9, {3.85750437040906*^9, 
   3.8575044265095387`*^9}, 3.857504545863558*^9, {3.85754327554312*^9, 
   3.85754333908671*^9}, {3.857543633834303*^9, 3.8575436776735477`*^9}, {
   3.857543981674858*^9, 3.857544079179964*^9}, {3.857851887237876*^9, 
   3.8578518957877817`*^9}, {3.8590928230121326`*^9, 3.859092863677952*^9}, {
   3.8591086121000843`*^9, 3.859108616994401*^9}, {3.85912747146942*^9, 
   3.859127471469643*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1067632665,ExpressionUUID->"dc617e94-8ac5-42b4-b8fa-05b4ea0436ca"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   OverscriptBox["P", "\[RightVector]"], TraditionalForm]],
  FontSize->24,
  FontWeight->"Plain",ExpressionUUID->
  "3ce76294-84e5-475c-b69b-1b7c9248c6d0"],
 StyleBox[" = <  ",
  FontSize->24,
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongRightArrow]"], " "], " "}], 
   TraditionalForm]],
  FontSize->24,ExpressionUUID->"ff47ba1d-1f06-432d-9924-a9f5b7dd0edd"],
 StyleBox[" >   = ",
  FontSize->24,
  FontWeight->"Plain"],
 Cell[BoxData[
  RowBox[{
   StyleBox[
    StyleBox[
     RowBox[{
      StyleBox[" ",
       FontSize->18], 
      StyleBox[" ",
       FontSize->24]}]],
    FontFamily->"Times"], 
   RowBox[{
    StyleBox["Tr",
     FontSize->24], 
    StyleBox[" ",
     FontSize->24], 
    StyleBox["[",
     FontFamily->"Times",
     FontSize->18], 
    StyleBox[" ",
     FontFamily->"Times",
     FontSize->18], 
    StyleBox[
     RowBox[{"\[Rho]", " ", ".", 
      StyleBox[" ",
       FontWeight->"Bold"], 
      FormBox[
       SubscriptBox[
        OverscriptBox["\[Sigma]", "\[LongRightArrow]"], " "],
       TraditionalForm]}],
     FontFamily->"Times"], 
    StyleBox[" ",
     FontFamily->"Times",
     FontSize->18,
     FontColor->GrayLevel[0]], 
    StyleBox["]",
     FontFamily->"Times",
     FontSize->18,
     FontColor->GrayLevel[0]]}], 
   StyleBox[" ",
    FontFamily->"Times",
    FontSize->18,
    FontColor->GrayLevel[0]]}]],
  CellChangeTimes->{{3.85748907727853*^9, 3.857489217331621*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Plain",ExpressionUUID->
  "9e822635-f3e5-445d-9bae-b333b30f4155"],
 StyleBox[" .",
  FontSize->24,
  FontWeight->"Bold"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.857489502695981*^9, {3.857489974445554*^9, 3.8574899744459343`*^9}, {
   3.857490049049232*^9, 3.8574900529417067`*^9}, {3.857490106126162*^9, 
   3.857490106924589*^9}, {3.857504582949891*^9, 3.857504582950307*^9}, {
   3.85754337737356*^9, 3.857543462071681*^9}, {3.857543563163644*^9, 
   3.857543567408516*^9}, {3.8575436936799927`*^9, 3.857543713255692*^9}, {
   3.857543789650116*^9, 3.857543789651276*^9}, {3.857543824717209*^9, 
   3.857543865318452*^9}, {3.857543919489586*^9, 3.857543950173843*^9}, {
   3.8591083885411797`*^9, 3.859108451828986*^9}, {3.8591085895634604`*^9, 
   3.8591085958769608`*^9}, {3.859127474295092*^9, 3.8591274742953444`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1476329970,ExpressionUUID->"83251749-2971-4122-8fbf-b5ceb3077c5c"],

Cell[TextData[StyleBox[" And the ensemble average energy  is",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
   3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
   3.857420803944667*^9}, {3.857424614631863*^9, 3.8574247647321987`*^9}, {
   3.857488782549274*^9, 3.8574888962742367`*^9}, {3.857488937694089*^9, 
   3.8574890613834667`*^9}, 3.857504334344377*^9, {3.85750437040906*^9, 
   3.8575044265095387`*^9}, 3.857504545863558*^9, {3.85754327554312*^9, 
   3.85754333908671*^9}, {3.857543633834303*^9, 3.8575436776735477`*^9}, {
   3.857543981674858*^9, 3.8575441200289087`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->582374062,ExpressionUUID->"e081223b-7099-4505-8d4e-be90b0429133"],

Cell[TextData[{
 StyleBox["E = <  H   >   ",
  FontSize->24,
  FontWeight->"Plain"],
 StyleBox["= ",
  FontSize->24,
  FontWeight->"Plain"],
 Cell[BoxData[
  RowBox[{
   StyleBox["  ",
    FontFamily->"Times",
    FontSize->24], 
   RowBox[{
    StyleBox[
     RowBox[{
      StyleBox["T",
       FontSize->24], 
      StyleBox["r",
       FontSize->18]}]], 
    StyleBox[" ",
     FontSize->18], 
    StyleBox["[",
     FontFamily->"Times",
     FontSize->18], 
    StyleBox[
     StyleBox[
      RowBox[{
       StyleBox[" ",
        FontSize->18], " "}]],
     FontFamily->"Times"], 
    StyleBox[
     RowBox[{"\[Rho]", " ", ".", 
      StyleBox[" ",
       FontWeight->"Bold"], 
      FormBox["H",
       TraditionalForm]}],
     FontFamily->"Times"], 
    StyleBox["  ",
     FontFamily->"Times",
     FontSize->18,
     FontColor->GrayLevel[0]], 
    StyleBox["]",
     FontFamily->"Times",
     FontSize->18,
     FontColor->GrayLevel[0]]}], 
   StyleBox[" ",
    FontFamily->"Times",
    FontSize->18,
    FontColor->GrayLevel[0]]}]],
  CellChangeTimes->{{3.85748907727853*^9, 3.857489217331621*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Plain",ExpressionUUID->
  "fbc8ffae-e499-4d38-9f8a-845d1cccf358"],
 StyleBox[" .",
  FontSize->24,
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.857489502695981*^9, {3.857489974445554*^9, 3.8574899744459343`*^9}, {
   3.857490049049232*^9, 3.8574900529417067`*^9}, {3.857490106126162*^9, 
   3.857490106924589*^9}, {3.857504582949891*^9, 3.857504582950307*^9}, {
   3.85754337737356*^9, 3.857543462071681*^9}, {3.857543563163644*^9, 
   3.857543567408516*^9}, {3.8575436936799927`*^9, 3.857543713255692*^9}, {
   3.857543789650116*^9, 3.857543789651276*^9}, {3.857543824717209*^9, 
   3.857543865318452*^9}, {3.857543919489586*^9, 3.857543950173843*^9}, {
   3.857544136915738*^9, 3.857544152432996*^9}, {3.857626121324247*^9, 
   3.8576261312017527`*^9}, {3.8591084578586397`*^9, 3.859108462842718*^9}, {
   3.859108527452154*^9, 3.859108570361848*^9}, 3.8591086017706413`*^9, {
   3.859127477117148*^9, 3.859127477117366*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->809286249,ExpressionUUID->"d172148a-6af9-4aa3-9e7b-4c0ce51762e8"],

Cell[TextData[{
 StyleBox[" There are two different meanings to the density matrix.   One is \
an ensemble average over a set of quantum spin systems, which is dilute \
enough so that there are no quantum effects between each spin system.  (This \
is for either a spin lattice or a beam of particles.)  Another application is \
to a single spin system that is prepared in a variety of spin directions with \
each direction having an assigned probability.  The same equations apply to \
both situations.\n\nThe density matrix, in contrast to a state vector ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  StyleBox[
   RowBox[{"|", 
    RowBox[{
     FormBox[
      SubscriptBox["\[Psi]", "\[Alpha]"],
      TraditionalForm], ">"}]}],
   FontSize->24]],
  CellChangeTimes->{{3.85748907727853*^9, 3.857489217331621*^9}},
  FontFamily->"Times New Roman",
  FontSize->36,
  FontWeight->"Bold",ExpressionUUID->"21e89778-1dfb-4a89-834b-9d9895cce7ec"],
 ", which describes mixed and not just pure states, offers a larger range of \
resources.   It does however require not just ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    SubscriptBox["n", "q"]], TraditionalForm]],ExpressionUUID->
  "761fa30f-1aa9-4057-9237-43c09ab9d849"],
 " dimensional space,  but  a  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    SubscriptBox["n", "q"]], TraditionalForm]],ExpressionUUID->
  "37d2dc88-e51a-46b3-a2f3-93d2438e8f6f"],
 " x  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    SubscriptBox["n", "q"]], TraditionalForm]],ExpressionUUID->
  "d0ed205a-c7c6-4a00-9cf0-4e5e1812a672"],
 " demand ( ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "q"], TraditionalForm]],ExpressionUUID->
  "d59efbae-ba22-444a-a97c-e7fa50ee19a7"],
 " denotes the number of qubits).\n\nThe idea of using a density matrix \
arises largely from Dirac and von Neumann;  several scholars have advocated \
formulating quantum theory founded on the density matrix, especially Steven \
Weinberg just before his recent death. It is fundamental to the measurement \
process and is a realistic presentation of nature which seems always to come \
impure and adorned with its environmental influences. "
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
  3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
  3.857420803944667*^9}, {3.857424614631863*^9, 3.8574247647321987`*^9}, {
  3.857488782549274*^9, 3.8574888962742367`*^9}, {3.857488937694089*^9, 
  3.8574890613834667`*^9}, {3.857490288783155*^9, 3.857490406324204*^9}, {
  3.857504610382648*^9, 3.8575048009941263`*^9}, {3.8588441994411373`*^9, 
  3.858844301210278*^9}, {3.858844335253044*^9, 3.858844345255674*^9}, {
  3.8588443862503157`*^9, 3.85884477025769*^9}, {3.858844989868384*^9, 
  3.8588450027707443`*^9}, {3.858845056906911*^9, 3.858845086900284*^9}, {
  3.858845120301568*^9, 3.85884515605471*^9}, {3.858859718509519*^9, 
  3.8588597810259438`*^9}, {3.8588598143939466`*^9, 3.858859950183578*^9}, {
  3.858860004297031*^9, 3.85886024980932*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1134805856,ExpressionUUID->"44604929-f489-467e-b2ba-ef9aebb2b6bc"],

Cell[CellGroupData[{

Cell["Density Matrix Examples", "Subsubsection",
 TaggingRules->{},
 CellChangeTimes->{3.857504930918144*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[115]:=",
 CellID->815224539,ExpressionUUID->"549a8bb5-d180-4d21-859b-f7471e232801"],

Cell[TextData[{
 StyleBox[" A single qubit density matrix can be constructed from a general \
qubit  with all ensemble members having the same spin direction as  \[Rho]1= ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  StyleBox[
   RowBox[{"|", 
    RowBox[{
     FormBox[
      SubscriptBox["\[Psi]", "\[Alpha]"],
      TraditionalForm], 
     StyleBox[" ",
      FontFamily->"Helvetica"], ">", 
     StyleBox[" ",
      FontFamily->"Helvetica"], "<", 
     StyleBox[" ",
      FontFamily->"Helvetica"], 
     FormBox[
      SubscriptBox["\[Psi]", "\[Alpha]"],
      TraditionalForm]}], "|", 
    StyleBox[" ",
     FontFamily->"Helvetica"]}],
   FontColor->GrayLevel[0]]],
  CellChangeTimes->{{3.857420361710814*^9, 3.8574203834008417`*^9}, {
    3.85742396986348*^9, 3.857423991217189*^9}, 3.8574240243239117`*^9, {
    3.857424105304082*^9, 3.857424274137732*^9}, {3.857424319871764*^9, 
    3.857424455511717*^9}},ExpressionUUID->
  "e298cd73-45be-4dd7-a31d-30576cfc3a30"],
 ", using Ket1[\[Theta], \[Phi]]"
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613325055860667*^9, 3.613325105154909*^9}, {
  3.6241931137315693`*^9, 3.624193132248291*^9}, {3.857420786832594*^9, 
  3.857420803944667*^9}, {3.857504948640861*^9, 3.857504953196747*^9}, {
  3.857504991683755*^9, 3.8575050018646507`*^9}, {3.857505323394362*^9, 
  3.857505411816964*^9}, {3.8575054423222303`*^9, 3.857505448630896*^9}, {
  3.857544432074259*^9, 3.857544455373514*^9}, {3.8575616324637213`*^9, 
  3.857561656988893*^9}, {3.857851912890729*^9, 3.857851971303746*^9}, {
  3.8588602969488497`*^9, 3.8588603275858183`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->776438691,ExpressionUUID->"2b576ef2-4c46-401d-a02b-b0f9243b7292"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Ket1", "[", 
  RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox["\[Rho]1",
   FontFamily->"Helvetica"], 
  StyleBox["=",
   FontFamily->"Helvetica"], 
  StyleBox[" ",
   FontFamily->"Helvetica"], 
  RowBox[{
   RowBox[{
    RowBox[{"Ket1", "[", 
     RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}], "\[CircleTimes]", 
    RowBox[{"Bra1", "[", 
     RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}]}], " ", "//", 
   "FullSimplify"}]}], "\n", 
 RowBox[{
  RowBox[{"SpinObs1", "[", 
   StyleBox["\[CapitalOmega]_",
    FontFamily->"Helvetica"], "  ", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tr", "[", 
      RowBox[{
       RowBox[{"s", "[", "#", "]"}], " ", ".", " ", 
       StyleBox["\[CapitalOmega]",
        FontFamily->"Helvetica"]}], "]"}], "&"}], "/@", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], " ", "//", 
   "FullSimplify"}]}], "\[IndentingNewLine]", 
 RowBox[{"SpinObs1", "[", " ", 
  StyleBox["\[Rho]1",
   FontFamily->"Helvetica"], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857544257024581*^9, 3.8575444029523497`*^9}, 
   3.85756147024831*^9, {3.857561584437951*^9, 3.857561587440508*^9}, {
   3.857561685024439*^9, 3.8575617927114697`*^9}, 3.857562034277808*^9, 
   3.857717355032427*^9, 3.857717570131362*^9, {3.857815805434104*^9, 
   3.8578158073210783`*^9}, {3.857851985846198*^9, 3.857851989636691*^9}, {
   3.858860753502553*^9, 3.8588607886473713`*^9}, {3.858860853224312*^9, 
   3.858860854539825*^9}, 3.85886091895144*^9, 3.859055241015006*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[1]:=",
 CellID->194542161,ExpressionUUID->"5fa62ff9-44bf-409a-beb3-08ad06687b12"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260739542093`*^9},
 CellLabel->"Out[1]=",
 CellID->1341933587,ExpressionUUID->"41552067-1e0a-4720-8b96-c0329151cb69"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607396129*^9},
 CellLabel->"Out[2]=",
 CellID->1004535915,ExpressionUUID->"d70adea0-2251-4467-a77e-f35d563e7c2e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{
        RowBox[{"Cos", "[", "\[Phi]", "]"}], " ", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
      {
       RowBox[{
        RowBox[{"Sin", "[", "\[Theta]", "]"}], " ", 
        RowBox[{"Sin", "[", "\[Phi]", "]"}]}]},
      {
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073966587*^9},
 CellLabel->"Out[4]=",
 CellID->1866174338,ExpressionUUID->"132c5460-1569-4eac-8320-c2c14576efbe"]
}, Open  ]],

Cell[TextData[{
 StyleBox["or by using the  ",
  FontFamily->"Helvetica"],
 "\[Rho]",
 StyleBox["[\[Psi]] command (seen below to be Hermitian)",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613321453679144*^9, 3.613321461013953*^9}, {
   3.613722492007531*^9, 3.613722561270419*^9}, {3.613722636529367*^9, 
   3.6137226680053263`*^9}, {3.624186872347973*^9, 3.624186917876183*^9}, 
   3.624186950322571*^9, {3.624188131831559*^9, 3.624188187203614*^9}, {
   3.624193147447171*^9, 3.624193152115807*^9}, 3.857421993092958*^9, {
   3.857505821505418*^9, 3.8575058377274942`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->92159943,ExpressionUUID->"99414cb7-98f5-4b81-a1c0-bb87486fcf32"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Psi]", "=", 
  StyleBox[" ",
   FontFamily->"Helvetica"], 
  RowBox[{"Ket1", "[", 
   RowBox[{"\[Theta]", ",", "\[Phi]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Rho]", "[", "\[Psi]", "]"}], "   ", "/.", " ", 
    RowBox[{
     RowBox[{"Conjugate", "[", "\[Theta]", "]"}], "->", "\[Theta]"}]}], " ", "/.",
    " ", 
   RowBox[{
    RowBox[{"Conjugate", "[", "\[Phi]", "]"}], "->", "\[Phi]"}]}], "  ", "//",
   "FullSimplify"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8574223462061234`*^9, 3.8574223557465477`*^9}, {
   3.857505201480942*^9, 3.857505291728856*^9}, {3.857561600626274*^9, 
   3.857561606021845*^9}, {3.8575621587344427`*^9, 3.857562208718657*^9}, {
   3.857594615856123*^9, 3.857594630706719*^9}, {3.857595088643976*^9, 
   3.857595110730748*^9}, {3.857852037225026*^9, 3.857852038205755*^9}, 
   3.8600695412393217`*^9, {3.860069716136917*^9, 3.860069740840274*^9}, {
   3.860071630991794*^9, 3.860071649400099*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[5]:=",
 CellID->166283053,ExpressionUUID->"726dcc08-65de-4a30-a169-388c2b64198a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]}]], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326073977181*^9},
 CellLabel->"Out[5]=",
 CellID->793434331,ExpressionUUID->"370f92c5-a392-4dad-b1c0-0b75d2b8ca08"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260739835243`*^9},
 CellLabel->"Out[6]=",
 CellID->1524836894,ExpressionUUID->"e0c8284a-a18c-44f5-8059-bd79ecc0c65f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "\[Rho]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{3.624187095336755*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[7]:=",
 CellID->596452514,ExpressionUUID->"1a03f5a1-d501-4596-9896-0d47caaaa945"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"\[Rho][statein], \\n\\n\\t returns a density \
matrix from an input state\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"\[Rho]", "[", "Private`statin_", "]"}], ":=", 
                    RowBox[{"Private`statin", "\[CircleTimes]", 
                    RowBox[{"Adj", "[", "Private`statin", "]"}]}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`\[Rho]\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"\[Rho][statein], \\n\\n\\t returns a density \
matrix from an input state\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "\[Rho][statein], \n\n\t returns a density matrix from an input state", 
     "Documentation" -> None, "OwnValues" -> None, "UpValues" -> None, 
     "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`\[Rho], {QDENSITY`Qdensity`\[Rho][
          Pattern[Private`statin, 
           Blank[]]] :> CircleTimes[Private`statin, 
          QDENSITY`Qdensity`Adj[Private`statin]]}], "SubValues" -> None, 
     "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Qdensity`\[Rho]"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074073324*^9},
 CellLabel->"Out[7]=",
 CellID->2011317807,ExpressionUUID->"0b4c55ad-70e7-4e32-88f3-2d7f4a2a483a"]
}, Open  ]],

Cell[TextData[{
 StyleBox["And we can evaluate the ",
  FontFamily->"Helvetica"],
 StyleBox[" trace and purity",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox["  ",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox["of the above density matrix.\nFor a normalized state, these are \
both equal to 1 . The entropy is zero in line with this being a highly \
ordered ensemble with all spins aligned.  ( We use  EnTropy to avoid the MM \
Entropy command.)",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624193238563397*^9, 3.624193315253763*^9}, {
   3.8563466206487703`*^9, 3.856346629647828*^9}, 3.8575058546265993`*^9, {
   3.857562781867791*^9, 3.8575629336887712`*^9}, {3.858861041865858*^9, 
   3.858861093717875*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1312746903,ExpressionUUID->"b25e9833-ca2c-4a81-9bc7-c9fe257a6b66"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Tr", "[", "\[Rho]1", "]"}], "  ", "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "==", "1"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Purity", "[", "\[Rho]1", "]"}], "   ", "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "==", "1"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"EnTropy", "[", "\[Rho]1", "]"}], "   ", "//", "Simplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "==", "0"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856346554964061*^9, 3.856346558963612*^9}, {
  3.856346654160756*^9, 3.8563466682016077`*^9}, {3.856346699904043*^9, 
  3.8563467509026527`*^9}, {3.8574901852867126`*^9, 3.8574902046323137`*^9}, {
  3.857505494195004*^9, 3.857505672995573*^9}, {3.857505711528884*^9, 
  3.857505797533229*^9}, {3.857505869262772*^9, 3.857505899662714*^9}, {
  3.857562311949772*^9, 3.8575623292084637`*^9}, {3.857562389902771*^9, 
  3.857562438907712*^9}, {3.8575625255926437`*^9, 3.85756256028551*^9}, {
  3.857562734190283*^9, 3.85756274728712*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[8]:=",
 CellID->90209443,ExpressionUUID->"be08d6a9-3be6-4b72-b4b0-234f8ba73d14"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260740895977`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[9]=",
 CellID->678294822,ExpressionUUID->"b0b420f0-f43c-4866-956c-0a4e159e6679"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074095595*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[11]=",
 CellID->1257875078,ExpressionUUID->"b998e457-8460-4fa1-8b26-eb171a0d7aa0"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074100457*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[13]=",
 CellID->1277414878,ExpressionUUID->"345ce0fb-cd42-41d0-a1d7-476536215fc3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Rho]1", ".", "\[Rho]1"}], " ", "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"%", "-", "\[Rho]1"}], " ", "//", "FullSimplify"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8575059168729753`*^9, 3.857505978269503*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[14]:=",
 CellID->1177281290,ExpressionUUID->"20c9ba24-3ee0-4619-8d27-6acfdf7b8b49"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SuperscriptBox[
       RowBox[{"Cos", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "\[ImaginaryI]"}], " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      SuperscriptBox[
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}], "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074114422*^9},
 CellLabel->"Out[14]=",
 CellID->1117835487,ExpressionUUID->"5b8c745b-3d64-45a7-9bc0-820050dc3f78"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074120713*^9},
 CellLabel->"Out[15]=",
 CellID->1274086053,ExpressionUUID->"dd80ac80-7113-4b58-be51-7a6e296742a1"]
}, Open  ]],

Cell["\<\
The above is an example of a pure state;  one that satisfies  \[Rho] . \[Rho] \
= \[Rho]\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857505992655663*^9, 3.857506028901989*^9}, {
  3.857562243638607*^9, 3.857562296516511*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1322053176,ExpressionUUID->"02ad9a19-7e44-471b-8834-b73fc97c1953"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"?", "Purity"}], "\[IndentingNewLine]", 
 RowBox[{"?", "EnTropy"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613237712460162*^9, {3.8575625998868647`*^9, 3.857562606973176*^9}, {
   3.857562647181879*^9, 3.8575626788840847`*^9}, {3.8575627551840563`*^9, 
   3.8575627558372297`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[16]:=",
 CellID->26653719,ExpressionUUID->"b6d8da8a-b89c-4faa-8ec1-2cb7950240ea"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"Purity[\[Rho]]\\n\\n\\t returns the purity of the \
state represented by the \\n\\n density matrix \[Rho]. It is simply \
Tr[\[Rho].\[Rho]].\\n\\n In general the Purity is a real number between 0 and \
1, \\n\\n being equal to 1 only if the state is a pure state.\"\>", 
              "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"Purity", "[", 
                    RowBox[{"Private`A_", "?", "MatrixQ"}], "]"}], ":=", 
                    RowBox[{"Chop", "[", 
                    RowBox[{"Tr", "[", 
                    RowBox[{"Private`A", ".", "Private`A"}], "]"}], 
                    "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`Purity\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"Purity[\[Rho]]\\n\\n\\t returns the purity of the \
state represented by the \\n\\n density matrix \[Rho]. It is simply \
Tr[\[Rho].\[Rho]].\\n\\n In general the Purity is a real number between 0 and \
1, \\n\\n being equal to 1 only if the state is a pure state.\"\>", 
              "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "Purity[\[Rho]]\n\n\t returns the purity of the state represented by the \
\n\n density matrix \[Rho]. It is simply Tr[\[Rho].\[Rho]].\n\n In general \
the Purity is a real number between 0 and 1, \n\n being equal to 1 only if \
the state is a pure state.", "Documentation" -> None, "OwnValues" -> None, 
     "UpValues" -> None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`Purity, {QDENSITY`Qdensity`Purity[
          PatternTest[
           Pattern[Private`A, 
            Blank[]], MatrixQ]] :> Chop[
          Tr[
           Dot[Private`A, Private`A]]]}], "SubValues" -> None, 
     "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Qdensity`Purity"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074209111*^9},
 CellLabel->"Out[16]=",
 CellID->1004848534,ExpressionUUID->"5638ce95-49c1-4476-a188-cdf5cd320f08"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"\\nEnTropy[\[Rho]],\\n\\n\\t gives the entropy of \
the system described by the \\n\\n density matrix \[Rho] . Defined = -Tr[ \
\[Rho] . Log[\[Rho],2]] . \"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"EnTropy", "[", 
                    RowBox[{"O_", "?", "MatrixQ"}], "]"}], ":=", 
                    RowBox[{"Module", "[", 
                    RowBox[{
                    RowBox[{"{", "}"}], ",", 
                    RowBox[{
                    RowBox[{"Private`M", "=", 
                    RowBox[{"Chop", "[", 
                    RowBox[{"Eigenvalues", "[", "O", "]"}], "]"}]}], ";", 
                    RowBox[{"Private`MOPO", "=", 
                    RowBox[{"DeleteCases", "[", 
                    RowBox[{"Private`M", ",", "0"}], "]"}]}], ";", 
                    RowBox[{"Private`OLOGO", "=", 
                    RowBox[{"-", 
                    RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                    RowBox[{"Private`i", "=", "1"}], 
                    RowBox[{"Length", "[", "Private`MOPO", "]"}]], 
                    RowBox[{
                    
                    RowBox[{"Private`MOPO", "\[LeftDoubleBracket]", 
                    "Private`i", "\[RightDoubleBracket]"}], " ", 
                    RowBox[{"Log", "[", 
                    RowBox[{"2", ",", 
                    
                    RowBox[{"Private`MOPO", "\[LeftDoubleBracket]", 
                    "Private`i", "\[RightDoubleBracket]"}]}], "]"}]}]}]}]}], 
                    ";", "Private`OLOGO"}]}], "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`EnTropy\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"\\nEnTropy[\[Rho]],\\n\\n\\t gives the entropy of \
the system described by the \\n\\n density matrix \[Rho] . Defined = -Tr[ \
\[Rho] . Log[\[Rho],2]] . \"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "\nEnTropy[\[Rho]],\n\n\t gives the entropy of the system described by \
the \n\n density matrix \[Rho] . Defined = -Tr[ \[Rho] . Log[\[Rho],2]] . ", 
     "Documentation" -> None, "OwnValues" -> None, "UpValues" -> None, 
     "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`EnTropy, {QDENSITY`Qdensity`EnTropy[
          PatternTest[
           Pattern[O, 
            Blank[]], MatrixQ]] :> Module[{}, Private`M = Chop[
             Eigenvalues[O]]; Private`MOPO = DeleteCases[Private`M, 0]; 
          Private`OLOGO = -Sum[Part[Private`MOPO, Private`i] Log[2, 
                Part[Private`MOPO, Private`i]], {Private`i, 1, 
               Length[Private`MOPO]}]; Private`OLOGO]}], "SubValues" -> None, 
     "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Qdensity`EnTropy"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607428456*^9},
 CellLabel->"Out[17]=",
 CellID->677910930,ExpressionUUID->"1b6b8d34-8cbe-4608-8a9d-098fdda59c60"]
}, Open  ]],

Cell[TextData[StyleBox["In papers associated with these codes,  the general \
definition of a density matrix is presented, along with a discussion of a \
pure state,  a mixed state, and for two or more qubits an entangled state.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624193238563397*^9, 3.624193315253763*^9}, {
  3.8563466206487703`*^9, 3.856346629647828*^9}, {3.8563469252897387`*^9, 
  3.8563470465667543`*^9}, {3.857563009796298*^9, 3.857563017772379*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1492245405,ExpressionUUID->"42d52334-ebd8-42b2-8101-32588eb65786"],

Cell[TextData[{
 StyleBox["The package also contains  definitions for | + \
\[RightAngleBracket] and | - \[RightAngleBracket] states, ",
  FontFamily->"Helvetica"],
 StyleBox["i.e.  for states in the plus or minus x-direction",
  FontFamily->"Helvetica",
  FontVariations->{"CompatibilityType"->0}],
 StyleBox[" :",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6133253196828814`*^9, 3.6133253305715427`*^9}, {
  3.624193329980069*^9, 3.624193335649941*^9}, {3.8576262255807447`*^9, 
  3.857626227346924*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1292456657,ExpressionUUID->"ca61ec89-fda4-4d23-bf24-1cfa5bf6831e"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"KetX", "[", "0", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\n", 
 RowBox[{"KetX", "[", "1", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613237809734796*^9, {3.6133253478006697`*^9, 3.613325389789362*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[18]:=",
 CellID->521944794,ExpressionUUID->"72145571-a2e5-4c72-8ab8-a9b4f6e60298"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074300714*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[18]=",
 CellID->1248253175,ExpressionUUID->"1ef457bb-7798-4985-80af-d1e2651c9d90"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", "\"(\"", 
    
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074306961*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[19]=",
 CellID->623364682,ExpressionUUID->"8fea3d15-22cb-4c10-b27c-0605993d8af1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260743131647`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[20]=",
 CellID->1464980308,ExpressionUUID->"e098b170-9bf4-4b46-b9da-bb9e1e5ebea7"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", "\"(\"", 
    
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260743186197`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[21]=",
 CellID->1024023701,ExpressionUUID->"7eca5b89-614c-4c9c-8c80-39a9a1ae5271"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Sigma]", "1"], ".", 
   RowBox[{"KetX", "[", "0", "]"}]}], " ", "==", " ", 
  RowBox[{"KetX", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Sigma]", "1"], ".", 
   RowBox[{"KetX", "[", "1", "]"}]}], " ", "==", 
  RowBox[{"-", 
   RowBox[{"KetX", "[", "1", "]"}]}]}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[22]:=",
 CellID->2002938410,ExpressionUUID->"846f7911-b786-43b9-8805-61d75eb1c43d"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260743342133`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[22]=",
 CellID->832395649,ExpressionUUID->"82ef3be0-edf6-4203-8b32-035047b1925a"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260743401012`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[23]=",
 CellID->1365196185,ExpressionUUID->"991ce154-3a68-48a8-9759-a60280aacbad"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Scalar products",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[" are simple to compute; for instance, if we have a general state \
in the computational basis, we can compute its scalar product with the | + \
\[RightAngleBracket] or | - \[RightAngleBracket] operators (KetX[0] and \
KetX[1]). We can also get the Dirac form for  the adjoint or Bra space using \
DFormA :",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624193405569557*^9, 3.624193420093543*^9}, {
  3.857626458009408*^9, 3.85762647320818*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1579690145,ExpressionUUID->"603f6fcc-c0ce-4ed3-ba9e-d265642f0371"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"State1", "=", 
  RowBox[{
   RowBox[{"a", " ", 
    TemplateBox[{"0"},
     "Bra"]}], "+", 
   RowBox[{"b", " ", 
    TemplateBox[{"1"},
     "Bra"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"DFormA", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "*", 
   RowBox[{"Bra", "[", "0", "]"}]}], "+", 
  RowBox[{"b", "*", 
   RowBox[{"Bra", "[", "1", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"DFormA", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613237916793249*^9, {3.613394938526773*^9, 3.613394956964025*^9}, {
   3.857626306100552*^9, 3.857626306889571*^9}, {3.857626352281454*^9, 
   3.8576264134026546`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[24]:=",
 CellID->1341871396,ExpressionUUID->"db686234-811d-466c-abc7-dd32106ff803"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a", "b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074356106*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[24]=",
 CellID->1136996747,ExpressionUUID->"b704a28d-c954-4f84-81f8-13099517917e"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\" ( \"", "a", "\" ) \"", "\" < \"", "\"0\"", "\" |\"", 
    "\" + \"", "\" ( \"", "b", "\" ) \"", "\" < \"", "\"1\"", "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074363328*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[25]=",
 CellID->1194835823,ExpressionUUID->"b66c26b2-4f1e-4b93-9b01-54888bd75db8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"a", "b"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074370117*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[26]=",
 CellID->486094064,ExpressionUUID->"c5539882-b2a6-49e2-b585-d902a2fdf678"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\" ( \"", "a", "\" ) \"", "\" < \"", "\"0\"", "\" |\"", 
    "\" + \"", "\" ( \"", "b", "\" ) \"", "\" < \"", "\"1\"", "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074375849*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[27]=",
 CellID->755140460,ExpressionUUID->"c2b4cd6e-3ca4-462c-bdcc-c06fc8afd577"]
}, Open  ]],

Cell[TextData[StyleBox["Now compute  scalar product <state1 | + \
\[RightAngleBracket]  and  <state1 | - \[RightAngleBracket] ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241884349658527`*^9, 3.624188538792109*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1114284212,ExpressionUUID->"0ac79b5a-fe60-404f-8003-29642a9d8a38"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"State1", ".", 
  RowBox[{"KetX", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"State1", ".", 
  RowBox[{"KetX", "[", "1", "]"}]}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellLabel->"In[28]:=",
 CellID->1717349244,ExpressionUUID->"eca7385c-53b2-4aa6-978f-5714cee4f78e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["a", 
        SqrtBox["2"]], "+", 
       FractionBox["b", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260744152822`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[28]=",
 CellID->17357450,ExpressionUUID->"54c9c7ae-00ad-4532-8ab6-59a6f0f13599"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["a", 
        SqrtBox["2"]], "-", 
       FractionBox["b", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260744218197`*^9},
 CellTags->"b:5.1.0",
 CellLabel->"Out[29]=",
 CellID->1816369805,ExpressionUUID->"28ac27d1-7873-4f1f-be5c-f034f35a000e"]
}, Open  ]],

Cell["The \[ScriptCapitalS] and \[ScriptCapitalT]  unitary operators  (* as \
used in qiskit *)", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575718136293507`*^9, {3.858176905300309*^9, 3.858176909755438*^9}, {
   3.85817703330509*^9, 3.858177143275281*^9}, {3.858177507825489*^9, 
   3.858177516116932*^9}, {3.860067673297924*^9, 3.860067718568983*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->21438158,ExpressionUUID->"a01cb002-95eb-40ec-b759-0ea4659b60a6"],

Cell[TextData[StyleBox["The following unitary operators \[ScriptCapitalS] and \
\[ScriptCapitalT]  are used extensively in Qiskit. They are basically \
rotations about the z-axis,  with a phase change on just the Ket[1] part.   \
Note that \[ScriptCapitalS] is the Matrix square root of s[3].  The \
properties are:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241884349658527`*^9, 3.624188538792109*^9}, {
  3.860072770093857*^9, 3.86007294682303*^9}, {3.8600729984401197`*^9, 
  3.860073007247382*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1140252415,ExpressionUUID->"6ecce6d3-fd32-495c-aeb8-a60ec79d9c7c"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"Rot", "[", "\[Phi]_", "]"}], ":=", "  ", 
   RowBox[{
    SuperscriptBox[
     StyleBox["\[ExponentialE]",
      FontSize->18], 
     FractionBox[
      RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}], "2"]], " ", 
    RowBox[{"RotZ", "[", "\[Phi]", "]"}]}]}],
  FontFamily->"Source Code Pro",
  FontSize->18,
  FontWeight->"SemiBold"], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{"Rot", "[", "\[Phi]", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox["\[ScriptCapitalS]",
   FontSize->24], 
  StyleBox["=",
   FontSize->24], 
  StyleBox[
   RowBox[{
    StyleBox[" ",
     FontSize->24], " "}]], 
  RowBox[{"Rot", "[", 
   RowBox[{"Pi", "/", "2"}], "]"}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"\[ScriptCapitalS]", ".", "\[ScriptCapitalS]"}], " ", "==", 
   RowBox[{"s", "[", "3", "]"}]}],
  FontSize->24], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox["\[ScriptCapitalT]",
   FontSize->24], 
  StyleBox[" ",
   FontSize->24], 
  StyleBox["=",
   FontSize->24], 
  StyleBox[
   RowBox[{
    StyleBox[" ",
     FontSize->24], " "}]], 
  RowBox[{"Rot", "[", 
   RowBox[{"Pi", "/", "4"}], "]"}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"\[ScriptCapitalT]", ".", "\[ScriptCapitalT]"}], "==", " ", 
   "\[ScriptCapitalS]"}],
  FontSize->24], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox["\[ScriptCapitalS]a",
   FontSize->24], 
  StyleBox["=",
   FontSize->24], 
  StyleBox[" ",
   FontSize->24], 
  RowBox[{"Adj", "[", 
   StyleBox["\[ScriptCapitalS]",
    FontSize->24], 
   StyleBox["]",
    FontSize->24]}]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox["\[ScriptCapitalT]a",
   FontSize->24], 
  StyleBox["=",
   FontSize->24], 
  StyleBox[" ",
   FontSize->24], 
  RowBox[{"Adj", "[", 
   StyleBox["\[ScriptCapitalT]",
    FontSize->24], 
   StyleBox["]",
    FontSize->24]}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"\[ScriptCapitalS]a", ".", "\[ScriptCapitalS]"}], "==", 
   RowBox[{"\[ScriptCapitalT]a", ".", "\[ScriptCapitalT]"}], "==", 
   RowBox[{"s", "[", "0", "]"}]}],
  FontSize->24], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"\[ScriptCapitalS]", ".", "\[ScriptCapitalS]"}], " ", "==", 
   RowBox[{"s", "[", "3", "]"}]}],
  FontSize->24]}], "Input",
 CellChangeTimes->{{3.8600615147830973`*^9, 3.8600615253018103`*^9}, {
  3.860061558674101*^9, 3.860061744568447*^9}, {3.8600618599502077`*^9, 
  3.860062338850816*^9}, {3.860067399504128*^9, 3.860067458524167*^9}, {
  3.860067514413653*^9, 3.8600675156637774`*^9}},
 CellLabel->"In[30]:=",
 CellID->1681352457,ExpressionUUID->"54e9d456-d14f-4116-b881-359bc92d2582"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"\[ImaginaryI]", " ", "\[Phi]"}]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260744352083`*^9},
 CellLabel->"Out[31]=",
 CellID->1247270530,ExpressionUUID->"54d73158-7a3d-4f42-93e5-2bc80f3c9aad"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "\[ImaginaryI]"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074442852*^9},
 CellLabel->"Out[32]=",
 CellID->1287919343,ExpressionUUID->"3cb4cbf8-e600-42f2-b91f-8b2e635c8b4d"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260744481773`*^9},
 CellLabel->"Out[33]=",
 CellID->1572719688,ExpressionUUID->"94001d28-43e7-4f4b-b705-8ff898381451"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", 
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "4"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074453384*^9},
 CellLabel->"Out[34]=",
 CellID->1471068940,ExpressionUUID->"e087f93a-1081-4ee5-b824-3cf94d833c75"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074458582*^9},
 CellLabel->"Out[35]=",
 CellID->276446669,ExpressionUUID->"9d3d1d10-0ff4-4387-8709-9142f66c5e55"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", 
      RowBox[{"-", "\[ImaginaryI]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260744636097`*^9},
 CellLabel->"Out[36]=",
 CellID->1232037422,ExpressionUUID->"a1bcada3-f886-4209-9a33-3d1947c48a61"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "4"]}]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074468698*^9},
 CellLabel->"Out[37]=",
 CellID->815627180,ExpressionUUID->"2b9473f7-a5e2-45d3-a21e-00ea68a26bef"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074473687*^9},
 CellLabel->"Out[38]=",
 CellID->857895459,ExpressionUUID->"018a295f-78e5-457c-9518-64fcb9fdf821"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074478586*^9},
 CellLabel->"Out[39]=",
 CellID->443773455,ExpressionUUID->"c246a987-796c-4fd9-ac34-9b759dbcc466"]
}, Open  ]],

Cell[TextData[StyleBox["The phase changes produced are as follows:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241884349658527`*^9, 3.624188538792109*^9}, {
  3.860072770093857*^9, 3.86007294682303*^9}, {3.8600729984401197`*^9, 
  3.860073007247382*^9}, {3.860073037757756*^9, 3.860073059414276*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1704654754,ExpressionUUID->"5bece0b1-a9db-4a26-8c9b-ed3ec189b3b0"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{
    RowBox[{"a", " ", 
     RowBox[{"Ket", "[", "0", "]"}]}], "+", " ", 
    RowBox[{"b", " ", 
     RowBox[{"Ket", "[", "1", "]"}]}]}], "]"}],
  FontSize->24], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"\[ScriptCapitalS]", ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", " ", 
       RowBox[{"Ket", "[", "0", "]"}]}], "+", " ", 
      RowBox[{"b", " ", 
       RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}],
  FontSize->24], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"\[ScriptCapitalT]", ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", " ", 
       RowBox[{"Ket", "[", "0", "]"}]}], "+", " ", 
      RowBox[{"b", " ", 
       RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}],
  FontSize->24]}], "Input",
 CellChangeTimes->{{3.8600671141441107`*^9, 3.860067154558991*^9}, 
   3.8600673949080267`*^9, {3.8600675271932096`*^9, 3.8600675378043756`*^9}},
 CellLabel->"In[40]:=",
 CellID->607432992,ExpressionUUID->"b20f5c26-3e45-4b3c-a764-e4164a48a573"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", "b", "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074495488*^9},
 CellLabel->"Out[40]=",
 CellID->289752154,ExpressionUUID->"50723bbc-434f-401f-8e9e-e61df3d534bf"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"\[ImaginaryI]", " ", "b"}], "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074502715*^9},
 CellLabel->"Out[41]=",
 CellID->1519024529,ExpressionUUID->"2a076d91-94c9-453b-b038-d9a69db0e4af"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "a", "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"b", " ", 
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Pi]"}], "4"]]}], "\")\"", "\" |\"", 
    "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607450869*^9},
 CellLabel->"Out[42]=",
 CellID->959663367,ExpressionUUID->"44bb7560-c9fe-440b-83dc-ecf484e4011c"]
}, Open  ]],

Cell[TextData[StyleBox["The density matrix changes by rotating the  x-y plane \
polarizations :",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241884349658527`*^9, 3.624188538792109*^9}, {
  3.860072770093857*^9, 3.86007294682303*^9}, {3.8600729984401197`*^9, 
  3.860073007247382*^9}, {3.860073037757756*^9, 3.860073059414276*^9}, {
  3.860073154287545*^9, 3.860073292134972*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.0",
 CellID->1200399909,ExpressionUUID->"dc2ff4fd-4d6e-445b-b23d-2f6465d9e6ae"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"rhoz", "=", "  ", 
    RowBox[{
     RowBox[{"1", "/", "2"}], " ", 
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"s", "[", "0", "]"}], " ", "+", " ", 
       RowBox[{"Px", " ", 
        RowBox[{"s", "[", "1", "]"}]}], "+", " ", 
       RowBox[{"Py", " ", 
        RowBox[{"s", "[", "2", "]"}]}], "+", " ", 
       RowBox[{"Pz", " ", 
        RowBox[{"s", "[", "3", "]"}]}]}], ")"}]}]}], ";"}],
  FontSize->24], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "1", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "2", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "3", "]"}]}], "]"}]}], "}"}], " ", "//", 
  "Simplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[ScriptCapitalS]", ".", "rhoz", ".", 
   RowBox[{"Adj", "[", "\[ScriptCapitalS]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "1", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "2", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "3", "]"}]}], "]"}]}], "}"}], " ", "//", 
  "Simplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[ScriptCapitalT]", ".", "rhoz", ".", 
   RowBox[{"Adj", "[", "\[ScriptCapitalT]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "1", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "2", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "3", "]"}]}], "]"}]}], "}"}], " ", "//", 
  "Simplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Rot", "[", "\[Phi]", "]"}], ".", "rhoz", ".", " ", 
   RowBox[{"Rot", "[", 
    RowBox[{"-", "\[Phi]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "1", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "2", "]"}]}], "]"}], ",", 
    RowBox[{"Tr", "[", 
     RowBox[{"%", ".", 
      RowBox[{"s", "[", "3", "]"}]}], "]"}]}], "}"}], " ", "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rhoz", "=."}], ";", "  ", 
  RowBox[{"Px", "=."}], ";", " ", 
  RowBox[{"Py", "=."}], ";", " ", 
  RowBox[{"Pz", "=."}], ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.860067166444675*^9, 3.860067323691472*^9}, {
   3.8600675529558163`*^9, 3.8600676491685963`*^9}, 3.860072076503255*^9, {
   3.8600722008582773`*^9, 3.8600722013011847`*^9}, {3.86007226130547*^9, 
   3.86007230127735*^9}, 3.860073090848875*^9},
 CellLabel->"In[43]:=",
 CellID->1965925749,ExpressionUUID->"940322fb-dd84-494a-8e11-44349113ea5f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"Px"},
      {"Py"},
      {"Pz"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074522769*^9},
 CellLabel->"Out[44]=",
 CellID->1460020199,ExpressionUUID->"03c00dad-e05d-4c4c-a79b-7aa62057a8c9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"-", "Py"}]},
      {"Px"},
      {"Pz"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074529828*^9},
 CellLabel->"Out[46]=",
 CellID->1411333971,ExpressionUUID->"2bf7ebd5-05e3-4e44-a13f-dff68c9fb9b9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{"Px", "-", "Py"}], 
        SqrtBox["2"]]},
      {
       FractionBox[
        RowBox[{"Px", "+", "Py"}], 
        SqrtBox["2"]]},
      {"Pz"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074535383*^9},
 CellLabel->"Out[48]=",
 CellID->1333169482,ExpressionUUID->"4c9cc9be-c072-4a6d-8415-962501a179c8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{
        RowBox[{"Px", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "-", 
        RowBox[{"Py", " ", 
         RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}]},
      {
       RowBox[{
        RowBox[{"Py", " ", 
         RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "+", 
        RowBox[{"Px", " ", 
         RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}]},
      {"Pz"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260746118803`*^9},
 CellLabel->"Out[50]=",
 CellID->1882594823,ExpressionUUID->"c5aa7ebe-94ba-492d-b063-5ba9beec917c"]
}, Open  ]],

Cell["General 0ne-qubit Density Matrix ", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575718136293507`*^9, {3.858177293903816*^9, 3.8581773475556297`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2146841522,ExpressionUUID->"d8a3e705-1ae1-4618-be39-1b045b781d93"],

Cell[TextData[StyleBox[" From the above discussion, we see that a one-qubit \
density matrix is a Hermitian  operator with unit trace and as mentioned \
earlier can be described in terms of the Pauli matrices.  It has positive \
definite eigenvalues and its observables stay within Bloch spheres.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
   3.624188910429326*^9, 3.624188985287815*^9}, {3.6241934459235764`*^9, 
   3.624193452302559*^9}, {3.8560384211709967`*^9, 3.856038421207473*^9}, 
   3.856101375625527*^9, {3.856377926922001*^9, 3.856377958607066*^9}, {
   3.857506196078925*^9, 3.857506301008903*^9}, {3.8575063385391493`*^9, 
   3.857506356235615*^9}, {3.857569766221891*^9, 3.8575698809511127`*^9}, {
   3.857626529931571*^9, 3.857626544601797*^9}, {3.8588611646839943`*^9, 
   3.8588612234774723`*^9}, {3.8588773229873533`*^9, 3.858877339614984*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1128508301,ExpressionUUID->"79d1d607-9794-4f8d-a426-8f73873942ce"],

Cell[TextData[StyleBox["Multi-Qubit states",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.8575699422557487`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->428411084,ExpressionUUID->"b2279946-c0d8-473c-90b3-b5e1bff5f050"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1480602742,ExpressionUUID->"59e6e2cd-efa0-41a6-ab04-c5f8538a831d"],

Cell[TextData[{
 StyleBox[" Multi-qubit states and density matrices are built as tensor \
products of single qubit ones.  For example, the ",
  FontFamily->"Helvetica"],
 StyleBox[" ",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox["two qubit density matrices \n  | 00 > ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"a7eab344-97af-4aac-aafa-66be157a6bdc"],
 " ",
 StyleBox["< 00 | , | 01 > ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"56d10569-4dea-46cc-a945-7b823199bb50"],
 " ",
 StyleBox["< 01 | , | 10 > ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"d1141514-adae-4043-9308-a9e3724382ef"],
 " ",
 StyleBox["< 10 | , and  | 11 > ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
 "\[CircleTimes]"],ExpressionUUID->"f7cd12b7-9a53-4cd5-b800-8961ef11c2f6"],
 " ",
 StyleBox["< 11 |  are obtained by:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
   3.624188910429326*^9, 3.624188985287815*^9}, {3.6241934459235764`*^9, 
   3.624193452302559*^9}, {3.8560384211709967`*^9, 3.856038421207473*^9}, 
   3.856101375625527*^9, {3.856377926922001*^9, 3.856377958607066*^9}, {
   3.8578520901683292`*^9, 3.857852103432166*^9}, {3.8590552410214233`*^9, 
   3.859055241027671*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->53899069,ExpressionUUID->"1f087290-0716-406e-9674-68b89ecd0124"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[ScriptCapitalP]", "0"], "\[CircleTimes]", 
  SubscriptBox["\[ScriptCapitalP]", "0"]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[ScriptCapitalP]", "0"], "\[CircleTimes]", 
  SubscriptBox["\[ScriptCapitalP]", "1"]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
  SubscriptBox["\[ScriptCapitalP]", "0"]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
  SubscriptBox["\[ScriptCapitalP]", "1"]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8563779897677794`*^9, 3.856377995021587*^9}, {
  3.856378063107504*^9, 3.856378119662374*^9}, {3.859055241028763*^9, 
  3.859055241031259*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[52]:=",
 CellID->1709130074,ExpressionUUID->"c10252cd-53e7-4e25-8108-2ddd27e391ba"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074623404*^9},
 CellLabel->"Out[52]=",
 CellID->1894618910,ExpressionUUID->"0b38c440-f57c-4920-91c4-d89f4ca3e4ae"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260746303062`*^9},
 CellLabel->"Out[53]=",
 CellID->2119986301,ExpressionUUID->"c4a5bbb9-dc08-4d63-9fff-bca34cc9fded"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "1", "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074635771*^9},
 CellLabel->"Out[54]=",
 CellID->1708573530,ExpressionUUID->"54760f59-e900-45dd-a429-13f723aa7242"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074641172*^9},
 CellLabel->"Out[55]=",
 CellID->1991474673,ExpressionUUID->"9cff0d88-3b78-4bc5-b2d6-b400ba4ff7c0"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 StyleBox[" ",
  FontFamily->"Helvetica"],
 "One example should clarify the meaning of the Kronecker product for two \
square matrices: "
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
   3.624188910429326*^9, 3.624188985287815*^9}, {3.6241934459235764`*^9, 
   3.624193452302559*^9}, {3.8560384211709967`*^9, 3.856038421207473*^9}, 
   3.856101375625527*^9, {3.856377926922001*^9, 3.856377958607066*^9}, {
   3.857572105175497*^9, 3.8575727311340446`*^9}, {3.857572790951664*^9, 
   3.857572910619753*^9}, {3.857573049669677*^9, 3.857573088844695*^9}, {
   3.8576266204393377`*^9, 3.8576266898658657`*^9}, {3.8578521404906063`*^9, 
   3.857852158356164*^9}, {3.857852197949232*^9, 3.8578522046998653`*^9}, {
   3.85785227461545*^9, 3.8578524135542927`*^9}, {3.858877389015086*^9, 
   3.858877422899418*^9}, {3.8588775675767403`*^9, 3.858877617324967*^9}, {
   3.858879993361844*^9, 3.858880048116064*^9}, 3.8590552410773983`*^9, {
   3.8590562002147493`*^9, 3.8590562600996304`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1821072405,ExpressionUUID->"d0b16d95-259b-4e8b-ae94-8f5abaea07e3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[" ",
   FontColor->GrayLevel[0]], 
  RowBox[{
   StyleBox[
    RowBox[{"Clear", "[", 
     RowBox[{"a", ",", "b", ",", "A", ",", "B"}], "]"}],
    FontColor->GrayLevel[0]], "\[IndentingNewLine]", 
   RowBox[{
    StyleBox["A",
     FontColor->RGBColor[1, 0, 0]], 
    StyleBox["=",
     FontColor->GrayLevel[0]], 
    RowBox[{
     StyleBox["Array",
      FontColor->GrayLevel[0]], 
     StyleBox["[",
      FontColor->GrayLevel[0]], 
     RowBox[{
      StyleBox["a",
       FontWeight->"Plain",
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox[",",
       FontColor->GrayLevel[0]], 
      StyleBox[
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}],
       FontColor->GrayLevel[0]]}], 
     StyleBox["]",
      FontColor->GrayLevel[0]]}]}], "\[IndentingNewLine]", 
   RowBox[{
    StyleBox["B",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["=",
     FontColor->GrayLevel[0]], 
    StyleBox[
     RowBox[{"Array", "[", 
      RowBox[{"b", ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}]}], "]"}],
     FontColor->GrayLevel[0]]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Grid", "[", 
     RowBox[{
      RowBox[{
       StyleBox["A",
        FontColor->RGBColor[1, 0, 0]], "\[CircleTimes]", 
       StyleBox["B",
        FontColor->RGBColor[0, 0, 1]]}], 
      StyleBox[" ",
       FontColor->RGBColor[0, 0, 1]], 
      StyleBox[",",
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{"Dividers", "\[Rule]", "All"}]}], "]"}], 
    StyleBox["/.",
     FontColor->RGBColor[0, 0, 1]], 
    RowBox[{
     StyleBox["{",
      FontColor->RGBColor[0, 0, 1]], 
     StyleBox[" ",
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      StyleBox[
       RowBox[{"a", "->", " ", 
        RowBox[{"Style", "[", 
         RowBox[{
          StyleBox["a",
           FontWeight->"Plain"], ",", "Red"}], "]"}]}],
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox[",",
       FontColor->RGBColor[1, 0, 0]], 
      StyleBox[
       RowBox[{"b", "->", " ", 
        RowBox[{"Style", "[", 
         RowBox[{"b", ",", "Blue"}], "]"}]}],
       FontColor->RGBColor[0, 0, 1]]}], 
     StyleBox[" ",
      FontColor->RGBColor[0, 0, 1]], "}"}], " "}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8563779897677794`*^9, 3.856377995021587*^9}, {
   3.856378063107504*^9, 3.856378119662374*^9}, {3.8588776436061707`*^9, 
   3.858877662776703*^9}, 3.858877707788372*^9, {3.858877746605549*^9, 
   3.858877832159272*^9}, {3.858877940828677*^9, 3.858877946824053*^9}, {
   3.858878882958482*^9, 3.858878945508842*^9}, {3.858878981993787*^9, 
   3.858879020536867*^9}, {3.8588790509799643`*^9, 3.858879053184376*^9}, {
   3.8588790860463877`*^9, 3.858879157327306*^9}, {3.858879207245596*^9, 
   3.858879331029359*^9}, {3.858879413657714*^9, 3.858879463558489*^9}, {
   3.858879511721764*^9, 3.858879521790598*^9}, {3.858879568038204*^9, 
   3.858879800302876*^9}, {3.858879910168659*^9, 3.858879911269844*^9}, 
   3.8588839687358093`*^9, 3.858884123577643*^9, {3.858884203989806*^9, 
   3.8588842416650887`*^9}, {3.8588843144225903`*^9, 3.858884315353805*^9}, {
   3.858884496010447*^9, 3.858884552260962*^9}, {3.8588845937851458`*^9, 
   3.858884600241918*^9}, 3.858935644748756*^9, 3.859055241085747*^9, {
   3.859090753841963*^9, 3.85909076146653*^9}, 3.8590912084816647`*^9, {
   3.8600736816855383`*^9, 3.860073686400264*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[56]:=",
 CellID->1721137469,ExpressionUUID->"e4597cca-b1f5-481e-b4f1-48696b072f6a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "1"}], "]"}], 
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "2"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"2", ",", "1"}], "]"}], 
      RowBox[{"a", "[", 
       RowBox[{"2", ",", "2"}], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074657011*^9},
 CellLabel->"Out[57]=",
 CellID->34706969,ExpressionUUID->"ad6c3307-a6a9-4036-9596-b1522a1613b2"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"b", "[", 
       RowBox[{"1", ",", "1"}], "]"}], 
      RowBox[{"b", "[", 
       RowBox[{"1", ",", "2"}], "]"}]},
     {
      RowBox[{"b", "[", 
       RowBox[{"2", ",", "1"}], "]"}], 
      RowBox[{"b", "[", 
       RowBox[{"2", ",", "2"}], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260746642933`*^9},
 CellLabel->"Out[58]=",
 CellID->1351959646,ExpressionUUID->"1e86a161-460a-4ca6-b87f-1cc8b4436d75"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "2"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "2"}], "]"}]}]},
     {
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "2"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"1", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "2"}], "]"}]}]},
     {
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "2"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"1", ",", "2"}], "]"}]}]},
     {
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "1"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "2"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "1"}], "]"}]}], 
      RowBox[{
       RowBox[{
        StyleBox["a",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]], "[", 
        RowBox[{"2", ",", "2"}], "]"}], " ", 
       RowBox[{
        StyleBox["b",
         StripOnInput->False,
         LineColor->RGBColor[0, 0, 1],
         FrontFaceColor->RGBColor[0, 0, 1],
         BackFaceColor->RGBColor[0, 0, 1],
         GraphicsColor->RGBColor[0, 0, 1],
         FontColor->RGBColor[0, 0, 1]], "[", 
        RowBox[{"2", ",", "2"}], "]"}]}]}
    },
    AutoDelete->False,
    GridBoxDividers->{"Columns" -> {{True}}, "Rows" -> {{True}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260746710787`*^9},
 CellLabel->"Out[59]=",
 CellID->2008785062,ExpressionUUID->"141334c4-ddfb-4ce1-9da4-458ffd4dd41a"]
}, Open  ]],

Cell[TextData[{
 StyleBox[" ",
  FontFamily->"Helvetica"],
 "The SpinObs command determines the polarizations and tensor correlations \
for a 2 qubit density matrix.  The first examples are for the above density \
matrices generated by 2 qubit projection operators.  The definitions used in \
SpinObs are as follows\n   PA= {Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "1"], 
   StyleBox["\[CircleTimes]",
    FontSize->16], 
   SubscriptBox["\[Sigma]", "0"]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"a43e23bb-70ad-4ffd-855d-8abbc5ffd15c"],
 ")], Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "2"], 
   StyleBox["\[CircleTimes]",
    FontSize->16], 
   SubscriptBox["\[Sigma]", "0"], 
   StyleBox[" ",
    FontSize->16]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"3be1c806-6e04-4368-8366-3a535f1a25ac"],
 ")], Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "3"], 
   StyleBox["\[CircleTimes]",
    FontSize->16], 
   StyleBox[" ",
    FontSize->16], 
   SubscriptBox["\[Sigma]", "0"]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"0eed6d45-11fc-47f3-8c5a-d297f03d1c13"],
 ")]}, \n   PB= {Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "0"], 
   StyleBox["\[CircleTimes]",
    FontSize->16], 
   StyleBox[" ",
    FontSize->16], 
   SubscriptBox["\[Sigma]", "1"]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"752c191e-e44c-4972-a6ad-56f6457c1368"],
 ")], Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "0"], 
   StyleBox["\[CircleTimes]",
    FontSize->16], 
   StyleBox[" ",
    FontSize->16], 
   SubscriptBox["\[Sigma]", "2"]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"a1fb150c-58e2-4c05-a179-3abc3a9c4e5b"],
 ")], Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "0"], 
   StyleBox["\[CircleTimes]",
    FontSize->16], 
   StyleBox[" ",
    FontSize->16], 
   SubscriptBox["\[Sigma]", "3"]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"1db145d1-599d-44a6-bab2-34f6e39e7a78"],
 ")]}, \n   and for the 3x3 tensor correlations\n    T[i , j ]= Tr[\[Rho] . (",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "i"], 
   StyleBox["\[CircleTimes]",
    FontSize->18], 
   StyleBox[" ",
    FontSize->18], 
   SubscriptBox["\[Sigma]", "j"]}]],
  TaggingRules->{},
  CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}},
  FontFamily->"Times New Roman",
  FontSize->24,ExpressionUUID->"0e1649a1-a321-4863-a61c-6779af1e89f9"],
 ")]}, for i and j=1..3.  ",
 "\n    These are product states and therefore the tensor correlations are \
simply the product \n     T[ i , j ]= ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["PA", 
    RowBox[{"i", " "}]], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "a1965676-0bad-4b77-b779-23dcb98dab8f"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["PB", "j"], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "1cab9d0f-355a-4120-b8ba-3167354b78ef"],
 " ."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
   3.624188910429326*^9, 3.624188985287815*^9}, {3.6241934459235764`*^9, 
   3.624193452302559*^9}, {3.8560384211709967`*^9, 3.856038421207473*^9}, 
   3.856101375625527*^9, {3.856377926922001*^9, 3.856377958607066*^9}, {
   3.857572105175497*^9, 3.8575727311340446`*^9}, {3.857572790951664*^9, 
   3.857572910619753*^9}, {3.857573049669677*^9, 3.857573088844695*^9}, {
   3.8576266204393377`*^9, 3.8576266898658657`*^9}, {3.8578521404906063`*^9, 
   3.857852158356164*^9}, {3.857852197949232*^9, 3.8578522046998653`*^9}, {
   3.85785227461545*^9, 3.8578524135542927`*^9}, {3.85905524110557*^9, 
   3.8590552411059*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->852049279,ExpressionUUID->"669fcedb-6a9f-4c29-9912-3dd3e75bc1f2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SpinObs"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{3.857632388914548*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[60]:=",
 CellID->1067550071,ExpressionUUID->"599cc6ab-2736-4d6b-8f81-d78dc46406c0"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"SpinObs[\[CapitalOmega]] Module to look at Spin \
Observables (polarizations PA,PB and\\nspin correlations 3 x 3 matrix T)\\n \
for 2 qubit density matrix \[CapitalOmega]   \"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"SpinObs", "[", 
                    RowBox[{"Private`\[CapitalOmega]_", "?", "MatrixQ"}], 
                    "]"}], ":=", 
                    RowBox[{"Module", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    
                    RowBox[{"Private`TOUT", ",", "Private`PWA", ",", 
                    "Private`PWB", ",", "Private`TWAB", ",", "Private`PA", 
                    ",", "Private`PB", ",", "Private`TAB", ",", "Private`i", 
                    ",", "Private`j"}], "}"}], ",", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"Private`i_", ",", "Private`j_"}], "]"}], ":=", 
                    RowBox[{"Tr", "[", 
                    RowBox[{
                    RowBox[{"ArrayFlatten", "[", 
                    RowBox[{
                    RowBox[{"s", "[", "Private`i", "]"}], "\[CircleTimes]", 
                    RowBox[{"s", "[", "Private`j", "]"}]}], "]"}], ".", 
                    "Private`\[CapitalOmega]"}], "]"}]}], ";", 
                    RowBox[{"Private`PWA", "=", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"1", ",", "0"}], "]"}], ",", 
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"2", ",", "0"}], "]"}], ",", 
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"3", ",", "0"}], "]"}]}], "}"}]}], ";", 
                    RowBox[{"Private`PWB", "=", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"0", ",", "1"}], "]"}], ",", 
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"0", ",", "2"}], "]"}], ",", 
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"0", ",", "3"}], "]"}]}], "}"}]}], ";", 
                    RowBox[{"Private`TWAB", "=", 
                    RowBox[{"{", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"Private`TOUT", "[", 
                    RowBox[{"Private`i", ",", "Private`j"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"Private`i", ",", "1", ",", "3"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"Private`j", ",", "1", ",", "3"}], "}"}]}], "]"}],
                     "}"}]}], ";", 
                    RowBox[{"Private`PA", "=", 
                    RowBox[{"Simplify", "[", "Private`PWA", "]"}]}], ";", 
                    RowBox[{"Private`PB", "=", 
                    RowBox[{"Simplify", "[", "Private`PWB", "]"}]}], ";", 
                    RowBox[{"Private`TAB", "=", 
                    RowBox[{"Simplify", "[", 
                    RowBox[{"Flatten", "[", 
                    RowBox[{"Private`TWAB", ",", "1"}], "]"}], "]"}]}], ";", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"\"PA=\"", " ", 
                    RowBox[{"MatrixForm", "[", "Private`PA", "]"}]}], ",", 
                    RowBox[{"\"PB=\"", " ", 
                    RowBox[{"MatrixForm", "[", "Private`PB", "]"}]}], ",", 
                    RowBox[{"\"TAB=\"", " ", 
                    RowBox[{"MatrixForm", "[", "Private`TAB", "]"}]}]}], 
                    "}"}]}]}], "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`SpinObs\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"SpinObs[\[CapitalOmega]] Module to look at Spin \
Observables (polarizations PA,PB and\\nspin correlations 3 x 3 matrix T)\\n \
for 2 qubit density matrix \[CapitalOmega]   \"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "SpinObs[\[CapitalOmega]] Module to look at Spin Observables \
(polarizations PA,PB and\nspin correlations 3 x 3 matrix T)\n for 2 qubit \
density matrix \[CapitalOmega]   ", "Documentation" -> None, "OwnValues" -> 
     None, "UpValues" -> None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`SpinObs, {QDENSITY`Qdensity`SpinObs[
          PatternTest[
           Pattern[Private`\[CapitalOmega], 
            Blank[]], MatrixQ]] :> 
        Module[{Private`TOUT, Private`PWA, Private`PWB, Private`TWAB, 
           Private`PA, Private`PB, Private`TAB, Private`i, Private`j}, 
          Private`TOUT[
             Pattern[Private`i, 
              Blank[]], 
             Pattern[Private`j, 
              Blank[]]] := Tr[
             Dot[
              ArrayFlatten[
               CircleTimes[
                QDENSITY`Qdensity`s[Private`i], 
                QDENSITY`Qdensity`s[Private`j]]], Private`\[CapitalOmega]]]; 
          Private`PWA = {
             Private`TOUT[1, 0], 
             Private`TOUT[2, 0], 
             Private`TOUT[3, 0]}; Private`PWB = {
             Private`TOUT[0, 1], 
             Private`TOUT[0, 2], 
             Private`TOUT[0, 3]}; Private`TWAB = {
             Table[
              Private`TOUT[Private`i, Private`j], {Private`i, 1, 3}, {
              Private`j, 1, 3}]}; Private`PA = Simplify[Private`PWA]; 
          Private`PB = Simplify[Private`PWB]; Private`TAB = Simplify[
             Flatten[Private`TWAB, 1]]; {
           "PA=" MatrixForm[Private`PA], "PB=" MatrixForm[Private`PB], "TAB=" 
            MatrixForm[Private`TAB]}]}], "SubValues" -> None, "DefaultValues" -> 
     None, "NValues" -> None, "FormatValues" -> None, "Options" -> None, 
     "Attributes" -> {}, "FullName" -> "QDENSITY`Qdensity`SpinObs"|>, False]],
  
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260747604856`*^9},
 CellLabel->"Out[60]=",
 CellID->1060122547,ExpressionUUID->"3720952f-5069-45bc-880d-338144e97559"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  StyleBox[
   RowBox[{
    StyleBox["S",
     FontSize->18], 
    StyleBox["pinObs",
     FontSize->16]}]], 
  StyleBox["[",
   FontSize->18], 
  StyleBox[
   RowBox[{
    StyleBox[
     SubscriptBox[
      StyleBox["\[ScriptCapitalP]",
       FontSize->16], "0"],
     FontSize->18], 
    StyleBox["\[CircleTimes]",
     FontSize->16], 
    StyleBox["  ",
     FontSize->16], 
    StyleBox[
     SubscriptBox[
      StyleBox["\[ScriptCapitalP]",
       FontSize->16], "0"],
     FontSize->18]}],
   FontSize->18], 
  StyleBox["]",
   FontSize->18]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    StyleBox["S",
     FontSize->18], 
    StyleBox["pinObs",
     FontSize->16]}]], 
  StyleBox["[",
   FontSize->16], 
  StyleBox[
   RowBox[{
    SubscriptBox[
     StyleBox["\[ScriptCapitalP]",
      FontSize->16], "0"], 
    StyleBox["\[CircleTimes]",
     FontSize->16], 
    StyleBox["  ",
     FontSize->16], 
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "1"],
     FontSize->16]}],
   FontSize->18], 
  StyleBox["]",
   FontSize->16]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   StyleBox[
    RowBox[{"S", 
     StyleBox["pinObs",
      FontSize->16]}]],
   FontSize->18], 
  StyleBox["[",
   FontSize->16], 
  StyleBox[
   RowBox[{
    SubscriptBox[
     StyleBox["\[ScriptCapitalP]",
      FontSize->16], "1"], 
    StyleBox["\[CircleTimes]",
     FontSize->16], 
    StyleBox["  ",
     FontSize->16], 
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "0"],
     FontSize->16]}],
   FontSize->18], 
  StyleBox["]",
   FontSize->16]}], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    StyleBox["Sp",
     FontSize->18], 
    StyleBox["inObs",
     FontSize->16]}]], 
  StyleBox["[",
   FontSize->16], 
  StyleBox[
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", "   ", 
    SubscriptBox["\[ScriptCapitalP]", "1"]}],
   FontSize->16], 
  StyleBox["]",
   FontSize->16]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857570003351318*^9, 3.857570066206106*^9}, {
   3.85763302007199*^9, 3.8576330299172907`*^9}, {3.857815842138473*^9, 
   3.8578158520962143`*^9}, 3.85785243085046*^9, {3.8578524919235277`*^9, 
   3.857852623227923*^9}, 3.859055241155034*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[61]:=",
 CellID->1625879544,ExpressionUUID->"136dbe5b-d9d0-495d-86a6-0fcb54e654e1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"1"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"1"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", "1"}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074778947*^9},
 CellLabel->"Out[61]=",
 CellID->1955311917,ExpressionUUID->"20b64de6-a573-4276-b26a-13306b434c68"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"1"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", 
             RowBox[{"-", "1"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260747872667`*^9},
 CellLabel->"Out[62]=",
 CellID->2132622625,ExpressionUUID->"1b751396-73df-4289-96bd-580c77e65e02"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"1"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", 
             RowBox[{"-", "1"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326074794135*^9},
 CellLabel->"Out[63]=",
 CellID->1486389945,ExpressionUUID->"e22b04a5-50cd-4447-b8dc-21acba5d7e09"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", "1"}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260748011503`*^9},
 CellLabel->"Out[64]=",
 CellID->78540064,ExpressionUUID->"b02eaa3d-a88d-4076-a41a-74551fe60378"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"plt", "[", 
   RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
  RowBox[{"MatrixPlot", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "i"], "\[CircleTimes]", 
     SubscriptBox["\[ScriptCapitalP]", "j"]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Row", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"plt", "[", 
     RowBox[{"0", ",", "0"}], "]"}], ",", 
    RowBox[{"plt", "[", 
     RowBox[{"0", ",", "1"}], "]"}], ",", 
    RowBox[{"plt", "[", 
     RowBox[{"1", ",", "0"}], "]"}], ",", 
    RowBox[{"plt", "[", 
     RowBox[{"1", ",", "1"}], "]"}]}], "}"}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856038737104965*^9, 3.856038826307837*^9}, {
   3.856038857762939*^9, 3.8560390721685877`*^9}, {3.856039103309476*^9, 
   3.85603920775646*^9}, {3.8560392442179613`*^9, 3.8560393748791428`*^9}, 
   3.859055241157166*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[65]:=",
 CellID->1791037296,ExpressionUUID->"86b0f2dd-2e8c-4583-a631-59c68e83dfcc"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
    GraphicsBox[
     RasterBox[CompressedData["
1:eJxTTMoPSmVmYGBggWIQGwI+2I/S+OmHVSLr3B/egvJhgHJzAY4PPIg=
      "], {{0, 
      0}, {4, 4}}, {0, 1}], Frame -> True, FrameLabel -> {None, None}, 
     FrameTicks -> {{{{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}, {{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}}, {{{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}, {{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], ImageSize -> Small, 
     Method -> {
      "AxisPadding" -> Scaled[0.02], "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultPlotStyle" -> Automatic, "DomainPadding" -> Scaled[0.02], 
       "RangePadding" -> Scaled[0.05]}], 
    GraphicsBox[
     RasterBox[CompressedData["
1:eJxTTMoPSmVmYGBggWIQGwI+2A9X+mGVyDr3h7fsGVDAwLsLRgMAHh48iA==

      "], {{0, 0}, {4, 4}}, {0, 1}], Frame -> True, 
     FrameLabel -> {None, None}, FrameTicks -> {{{{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}, {{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}}, {{{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}, {{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], ImageSize -> Small, 
     Method -> {
      "AxisPadding" -> Scaled[0.02], "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultPlotStyle" -> Automatic, "DomainPadding" -> Scaled[0.02], 
       "RangePadding" -> Scaled[0.05]}], 
    GraphicsBox[
     RasterBox[CompressedData["
1:eJxTTMoPSmVmYGBggWIQGwI+2A82+mGVyDr3h7fsGVDAwLuL2jQArh48iA==

      "], {{0, 0}, {4, 4}}, {0, 1}], Frame -> True, 
     FrameLabel -> {None, None}, FrameTicks -> {{{{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}, {{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}}, {{{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}, {{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], ImageSize -> Small, 
     Method -> {
      "AxisPadding" -> Scaled[0.02], "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultPlotStyle" -> Automatic, "DomainPadding" -> Scaled[0.02], 
       "RangePadding" -> Scaled[0.05]}], 
    GraphicsBox[
     RasterBox[CompressedData["
1:eJxTTMoPSmVmYGBggWIQGwI+2FOLflglss794S17BhRAPfOHGw0APi08iA==

      "], {{0, 0}, {4, 4}}, {0, 1}], Frame -> True, 
     FrameLabel -> {None, None}, FrameTicks -> {{{{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}, {{3.5, 
          FormBox["1", TraditionalForm]}, {2.5, 
          FormBox["2", TraditionalForm]}, {1.5, 
          FormBox["3", TraditionalForm]}, {0.5, 
          FormBox["4", TraditionalForm]}}}, {{{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}, {{0.5, 
          FormBox["1", TraditionalForm]}, {1.5, 
          FormBox["2", TraditionalForm]}, {2.5, 
          FormBox["3", TraditionalForm]}, {3.5, 
          FormBox["4", TraditionalForm]}}}}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], ImageSize -> Small, 
     Method -> {
      "AxisPadding" -> Scaled[0.02], "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultPlotStyle" -> Automatic, "DomainPadding" -> Scaled[0.02], 
       "RangePadding" -> Scaled[0.05]}]},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075054686*^9},
 CellLabel->"Out[66]=",
 CellID->1467896390,ExpressionUUID->"0b7ceb8b-df8d-42b3-af4f-6c11a9a93ff6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[ScriptCapitalP]", "0"], " ", "==", " ", 
   RowBox[{"AF", "[", 
    RowBox[{
     RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
     RowBox[{"Bra", "[", "0", "]"}]}], "]"}]}], 
  StyleBox["&&",
   FontWeight->"Plain"], " ", 
  RowBox[{
   SubscriptBox["\[ScriptCapitalP]", "1"], " ", "==", " ", 
   RowBox[{"AF", "[", 
    RowBox[{
     RowBox[{"Ket", "[", "1", "]"}], " ", "\[CircleTimes]", " ", 
     RowBox[{"Bra", "[", "1", "]"}]}], "]"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613238445217594*^9, 3.613238529594364*^9}, {
   3.6132386068173*^9, 3.613238708987796*^9}, 3.61323874676079*^9, {
   3.613238853053121*^9, 3.6132388633013983`*^9}, {3.613239254165518*^9, 
   3.613239261939003*^9}, {3.6241934802629757`*^9, 3.624193497349824*^9}, {
   3.856038421220458*^9, 3.856038421221904*^9}, 3.856378176555017*^9, {
   3.856378233986102*^9, 3.856378254191545*^9}, 3.857851640442698*^9, 
   3.858172530350636*^9, 3.85817762117138*^9, {3.85905524115871*^9, 
   3.859055241159693*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[67]:=",
 CellID->854177356,ExpressionUUID->"266b7de7-e43c-45c2-86d4-b607efd4d607"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260751942387`*^9},
 CellLabel->"Out[67]=",
 CellID->642489994,ExpressionUUID->"c2940d72-2b35-4b27-be3c-8bfdd8b698ac"]
}, Open  ]],

Cell[TextData[{
 StyleBox["For example, the ",
  FontFamily->"Helvetica"],
 StyleBox[" ",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox["two qubit",
  FontFamily->"Helvetica"],
 StyleBox[" state basis \n  | 00 > ",
  FontFamily->"Helvetica"],
 " ",
 StyleBox[" , | 01 > ",
  FontFamily->"Helvetica"],
 " ",
 StyleBox[" , | 10 >  , and  | 11 > ",
  FontFamily->"Helvetica"],
 " ",
 StyleBox[" are obtained by:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
   3.6241888840140457`*^9, 3.6241888982815933`*^9}, {3.624188995470202*^9, 
   3.624189017653082*^9}, {3.6241891828843603`*^9, 3.624189185054461*^9}, 
   3.857852645680065*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->230642097,ExpressionUUID->"bb86afa1-e55f-40ec-9795-f7e7a4655594"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
  RowBox[{"Ket", "[", "1", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ket", "[", "1", "]"}], " ", "\[CircleTimes]", " ", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ket", "[", "1", "]"}], " ", "\[CircleTimes]", " ", 
  RowBox[{"Ket", "[", "1", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613238910424962*^9, 3.613239078086611*^9}, {
  3.624193545050322*^9, 3.624193545062729*^9}, {3.8560384212252703`*^9, 
  3.856038421229526*^9}, {3.85637839446257*^9, 3.856378430560211*^9}, {
  3.859055241161207*^9, 3.8590552411639442`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[68]:=",
 CellID->193741444,ExpressionUUID->"14e4918e-dd3b-40bb-a169-5ee2c053727f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1"},
     {"0"},
     {"0"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607520658*^9},
 CellLabel->"Out[68]=",
 CellID->781552042,ExpressionUUID->"6294d5fe-2750-4942-82bc-ed4e6a3a7f11"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"00\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075213324*^9},
 CellLabel->"Out[69]=",
 CellID->1535184842,ExpressionUUID->"bfe21686-46ca-46d8-b45f-4735fe51b5eb"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"1"},
     {"0"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075219509*^9},
 CellLabel->"Out[70]=",
 CellID->1983104436,ExpressionUUID->"662eefe7-5a34-4337-8c10-1702604c9c10"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"01\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075225247*^9},
 CellLabel->"Out[71]=",
 CellID->1090054412,ExpressionUUID->"0d09c2e7-011e-45e0-b8e8-7651352e9e87"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {"1"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260752315807`*^9},
 CellLabel->"Out[72]=",
 CellID->361509927,ExpressionUUID->"6ceebc45-3031-47df-b0de-861d3bf3c889"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"10\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075237822*^9},
 CellLabel->"Out[73]=",
 CellID->666105726,ExpressionUUID->"b965179b-bd66-491f-8261-1274aea2820b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {"0"},
     {"1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075243842*^9},
 CellLabel->"Out[74]=",
 CellID->406685071,ExpressionUUID->"7bf5dc2b-8d18-4a60-b8be-7c994d6539d3"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"11\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075249407*^9},
 CellLabel->"Out[75]=",
 CellID->1863023543,ExpressionUUID->"d31dd1f8-97e6-43e8-b66d-3c56cddab543"]
}, Open  ]],

Cell[TextData[{
 StyleBox["A general ",
  FontFamily->"Helvetica"],
 StyleBox[" two qubit",
  FontFamily->"Helvetica"],
 StyleBox[" state    a | 00 > ",
  FontFamily->"Helvetica"],
 " ",
 StyleBox[" + b | 01 > ",
  FontFamily->"Helvetica"],
 " ",
 StyleBox[" +c  | 10 >  + d  | 11 > ",
  FontFamily->"Helvetica"],
 " ",
 StyleBox[" is obtained by:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241885987223053`*^9, 3.624188790952106*^9}, {
  3.6241888840140457`*^9, 3.6241888982815933`*^9}, {3.624188995470202*^9, 
  3.624189017653082*^9}, {3.6241891828843603`*^9, 3.624189237255275*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->627240456,ExpressionUUID->"072576ad-af65-44a4-b78f-f3a8bbb650b0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Psi]2", "=", 
  RowBox[{"Sum", "[", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"i", ",", "j"}], "]"}], 
     RowBox[{
      RowBox[{"Ket", "[", "i", "]"}], " ", "\[CircleTimes]", " ", 
      RowBox[{"Ket", "[", "j", "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", "1"}], "}"}]}], "]"}]}], "\n", 
 RowBox[{"DForm", "[", "\[Psi]2", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613239092907892*^9, 3.613239181291891*^9}, 
   3.613239223359084*^9, {3.61347281954606*^9, 3.613472839707197*^9}, {
   3.624189080455742*^9, 3.624189081070806*^9}, {3.6241891450310307`*^9, 
   3.62418914557896*^9}, 3.624193545071043*^9, {3.856038421233617*^9, 
   3.856038421237598*^9}, {3.8560394359092073`*^9, 3.856039470200428*^9}, {
   3.856378461004395*^9, 3.856378519009467*^9}, {3.8563785819999638`*^9, 
   3.856378653741102*^9}, {3.8563786910502367`*^9, 3.8563787238974*^9}, 
   3.856378818183332*^9, {3.8563789834430323`*^9, 3.856378995265852*^9}, 
   3.859055241165604*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[76]:=",
 CellID->33120869,ExpressionUUID->"f7c116ca-3a57-44b8-86b1-105b57299532"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"a", "[", 
       RowBox[{"0", ",", "0"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"0", ",", "1"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "0"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "1"}], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075264762*^9},
 CellLabel->"Out[76]=",
 CellID->1525113941,ExpressionUUID->"9933d0c7-3920-47f8-8917-d774bda21099"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"0", ",", "0"}], "]"}], "\")\"", "\" |\"", "\"00\"", "\" >\"", 
    "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"0", ",", "1"}], "]"}], "\")\"", "\" |\"", "\"01\"", "\" >\"", 
    "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"1", ",", "0"}], "]"}], "\")\"", "\" |\"", "\"10\"", "\" >\"", 
    "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"1", ",", "1"}], "]"}], "\")\"", "\" |\"", "\"11\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607527155*^9},
 CellLabel->"Out[77]=",
 CellID->225195795,ExpressionUUID->"2945626f-45f8-4d82-aff2-9e5f0423baab"]
}, Open  ]],

Cell[TextData[StyleBox["A general three qubit state is obtained by:",
 FontWeight->"Regular"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6132392900624943`*^9, 3.613239304427401*^9}, {
  3.613239456915832*^9, 3.613239511193788*^9}, {3.624189312530551*^9, 
  3.624189326738677*^9}, {3.624193570474683*^9, 3.6241935958540173`*^9}, {
  3.856378367323345*^9, 3.856378373938188*^9}, {3.856378865216947*^9, 
  3.856378940047023*^9}, {3.8563789777829037`*^9, 3.8563789777829723`*^9}, {
  3.857659632682713*^9, 3.857659638789566*^9}, {3.857852659426774*^9, 
  3.857852664387608*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->464180710,ExpressionUUID->"18f0fdb2-f656-4b47-8a8a-0e6ca39e7aed"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Psi]3", "=", 
  RowBox[{"Sum", "[", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", 
      RowBox[{"i", ",", "j", ",", "k"}], "]"}], 
     RowBox[{
      RowBox[{"Ket", "[", "i", "]"}], " ", "\[CircleTimes]", " ", 
      RowBox[{"Ket", "[", "j", "]"}], "\[CircleTimes]", " ", 
      RowBox[{"Ket", "[", "k", "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "0", ",", "1"}], "}"}]}], "]"}]}], "\n", 
 RowBox[{"DForm", "[", "\[Psi]3", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613239352640203*^9, 3.613239402614545*^9}, 
   3.613239435909606*^9, {3.613240267593163*^9, 3.613240347043003*^9}, {
   3.61324037856991*^9, 3.613240379232555*^9}, {3.613472902555773*^9, 
   3.6134730389793787`*^9}, {3.61347307309839*^9, 3.6134730958600273`*^9}, {
   3.624189356701209*^9, 3.6241893629108887`*^9}, {3.856038421247961*^9, 
   3.856038421249209*^9}, {3.8560395243368607`*^9, 3.856039527735613*^9}, {
   3.856379012464896*^9, 3.856379033110383*^9}, 3.857659592668071*^9, {
   3.857659623224309*^9, 3.857659659220435*^9}, {3.859055241167642*^9, 
   3.859055241168785*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[78]:=",
 CellID->1346479071,ExpressionUUID->"c9e631a1-ab16-406f-9cdd-e270b9657a67"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"a", "[", 
       RowBox[{"0", ",", "0", ",", "0"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"0", ",", "0", ",", "1"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"0", ",", "1", ",", "0"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"0", ",", "1", ",", "1"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "0", ",", "0"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "0", ",", "1"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "1", ",", "0"}], "]"}]},
     {
      RowBox[{"a", "[", 
       RowBox[{"1", ",", "1", ",", "1"}], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260752852592`*^9},
 CellLabel->"Out[78]=",
 CellID->999540551,ExpressionUUID->"acf4e793-e3bf-4097-8a5d-79f6912b3e95"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"0", ",", "0", ",", "0"}], "]"}], "\")\"", "\" |\"", "\"000\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"0", ",", "0", ",", "1"}], "]"}], "\")\"", "\" |\"", "\"001\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"0", ",", "1", ",", "0"}], "]"}], "\")\"", "\" |\"", "\"010\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"0", ",", "1", ",", "1"}], "]"}], "\")\"", "\" |\"", "\"011\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"1", ",", "0", ",", "0"}], "]"}], "\")\"", "\" |\"", "\"100\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"1", ",", "0", ",", "1"}], "]"}], "\")\"", "\" |\"", "\"101\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"1", ",", "1", ",", "0"}], "]"}], "\")\"", "\" |\"", "\"110\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"a", "[", 
      RowBox[{"1", ",", "1", ",", "1"}], "]"}], "\")\"", "\" |\"", "\"111\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075292824*^9},
 CellLabel->"Out[79]=",
 CellID->1660953823,ExpressionUUID->"2545e056-7364-4a88-a144-df0bc1e5571d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"state", "=", 
  RowBox[{
   TemplateBox[{"0"},
    "Ket"], "\[CircleTimes]", 
   TemplateBox[{"1"},
    "Ket"], "\[CircleTimes]", 
   TemplateBox[{"1"},
    "Ket"]}]}], "\n", 
 RowBox[{"DForm", "[", "state", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613239352640203*^9, 3.613239402614545*^9}, 
   3.613239435909606*^9, {3.613240267593163*^9, 3.613240347043003*^9}, {
   3.61324037856991*^9, 3.613240379232555*^9}, {3.613472902555773*^9, 
   3.6134730389793787`*^9}, {3.61347307309839*^9, 3.6134730958600273`*^9}, {
   3.624189356701209*^9, 3.6241893629108887`*^9}, {3.856038421247961*^9, 
   3.856038421249209*^9}, {3.8560395243368607`*^9, 3.856039527735613*^9}, {
   3.856379012464896*^9, 3.856379033110383*^9}, {3.859055241171165*^9, 
   3.859055241171961*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[80]:=",
 CellID->1824587066,ExpressionUUID->"bd5d3b82-cccb-4503-b2da-2d4d6082d913"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {"0"},
     {"1"},
     {"0"},
     {"0"},
     {"0"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075313403*^9},
 CellLabel->"Out[80]=",
 CellID->510975808,ExpressionUUID->"1c62f79d-ab70-4100-8b37-48a15650f869"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"011\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260753209352`*^9},
 CellLabel->"Out[81]=",
 CellID->183300703,ExpressionUUID->"7b3f4321-e561-4df2-81b7-d541ec6e0511"]
}, Open  ]],

Cell[TextData[StyleBox["The associated projection operator is now simply:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6132392900624943`*^9, 3.613239304427401*^9}, {
  3.613239456915832*^9, 3.613239511193788*^9}, {3.613239549432013*^9, 
  3.6132395776562147`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1915822749,ExpressionUUID->"1d49f15a-8550-4882-b664-1f3059be1691"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[" ",
   FontWeight->"Plain",
   FontColor->RGBColor[
    0.15294117647058825`, 0.32941176470588235`, 0.6431372549019608]], 
  RowBox[{
   SubscriptBox["\[ScriptCapitalP]", 
    StyleBox["0",
     ScriptLevel->0]], 
   StyleBox["\[CircleTimes]",
    FontSize->14], 
   StyleBox[" ",
    FontSize->14], 
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", 
     StyleBox["1",
      ScriptLevel->0]],
    ScriptLevel->0], 
   StyleBox[" ",
    ScriptLevel->0], 
   StyleBox["\[CircleTimes]",
    ScriptLevel->0,
    FontSize->14], 
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", 
     StyleBox[
      RowBox[{"1", "  "}],
      ScriptLevel->0]],
    ScriptLevel->0]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613239565259982*^9, 3.613239640136797*^9}, {
  3.6132396857107553`*^9, 3.613239705204183*^9}, {3.6134729658011312`*^9, 
  3.6134729660578613`*^9}, {3.624189400542938*^9, 3.624189402102648*^9}, {
  3.8560384212519913`*^9, 3.856038421253474*^9}, {3.856039547107053*^9, 
  3.856039550281012*^9}, {3.857815861916353*^9, 3.857815887566271*^9}, {
  3.859055241173781*^9, 3.859055241175044*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[82]:=",
 CellID->2010622972,ExpressionUUID->"0f089c63-fb46-4199-ae96-f6598c8f9b56"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075332445*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[82]=",
 CellID->1243365523,ExpressionUUID->"4bfe338b-774f-4e05-b793-754b2b81d531"]
}, Open  ]],

Cell[TextData[StyleBox["As an example, we build a three qubit state as \
follows: \n[Note: KetX[0] and KetX[1] are the eigenstates of SigmaX]",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613239749155994*^9, 3.613239770136057*^9}, {
   3.613396391147394*^9, 3.613396456460988*^9}, {3.613396497982882*^9, 
   3.6133965124835777`*^9}, 3.6241894304769907`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1885104859,ExpressionUUID->"41edd0f4-a913-4fe5-aa0e-06ee74a8aeaa"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ket123", " ", "=", 
   RowBox[{
    RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
    RowBox[{"Ket", "[", "1", "]"}], "\[CircleTimes]", 
    RowBox[{"KetX", "[", "1", "]"}]}]}], " ", ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6133958457522097`*^9, 3.613395848519493*^9}, 
   3.613396026250587*^9, {3.613396091979624*^9, 3.613396166495511*^9}, {
   3.613396220584749*^9, 3.613396225466898*^9}, {3.613396266763863*^9, 
   3.613396309882001*^9}, {3.6241894404356937`*^9, 3.624189451335413*^9}, {
   3.8560384212556458`*^9, 3.856038421256901*^9}, {3.856039560978751*^9, 
   3.8560395707560987`*^9}, {3.859055241176663*^9, 3.859055241177582*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[83]:=",
 CellID->1131121022,ExpressionUUID->"d36ec6b6-bb31-48b7-9d11-688ad68ba4cb"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"010\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"011\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075347948*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[84]=",
 CellID->80976104,ExpressionUUID->"54879a71-f955-493a-814d-081ab1d2821d"]
}, Open  ]],

Cell[TextData[StyleBox["... and build the density matrix for the above state:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613396539605742*^9, 3.613396554117638*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1225773510,ExpressionUUID->"66750b66-a0ea-42a0-a942-3da790a6d238"],

Cell[CellGroupData[{

Cell[BoxData["ket123"], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857573392454029*^9, 3.857573394207159*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[85]:=",
 CellID->2145598433,ExpressionUUID->"eb16a410-7bc3-4a0d-a30d-825c43c0e93e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]},
     {"0"},
     {"0"},
     {"0"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075361603*^9},
 CellLabel->"Out[85]=",
 CellID->1608231377,ExpressionUUID->"28112485-ff4f-476c-a538-1558fcc3a956"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Rho]123", "=", 
  RowBox[{"ket123", "\[CircleTimes]", 
   RowBox[{"Adj", "[", "ket123", "]"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613395852254318*^9, {3.613396342404159*^9, 3.6133963574001083`*^9}, {
   3.61339659458442*^9, 3.6133966209857817`*^9}, {3.6241894787518*^9, 
   3.624189509846406*^9}, {3.624189632417712*^9, 3.624189702415988*^9}, {
   3.856039591961628*^9, 3.856039613268571*^9}, {3.8575734088787727`*^9, 
   3.85757346572484*^9}, {3.8575735426919317`*^9, 3.857573572414362*^9}, 
   3.8590552411790133`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[86]:=",
 CellID->764756899,ExpressionUUID->"6bc042b9-4294-4e99-9d85-cfe50a73743e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", 
      FractionBox["1", "2"], 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0", "0", "0"},
     {"0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], 
      FractionBox["1", "2"], "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607537857*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[86]=",
 CellID->1315572097,ExpressionUUID->"3c7d3527-7238-4216-a2e5-e7167e15219e"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Then check its ",
  FontFamily->"Helvetica"],
 StyleBox["Trace and Purity",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->579574198,ExpressionUUID->"5140c94c-a5e4-4b80-98c5-5545d90b2af3"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Tr", "[", "\[Rho]123", "]"}], " ", "//", 
  "FullSimplify"}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", "\[Rho]123", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8575733131855583`*^9, 3.857573338733014*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[87]:=",
 CellID->2065206641,ExpressionUUID->"95ee239b-56e3-4ff0-8dc3-d9e7b7f0ff9f"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607539093*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[87]=",
 CellID->328604767,ExpressionUUID->"206067c9-9433-44f3-8595-f681738bab9c"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075397625*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[88]=",
 CellID->1679862532,ExpressionUUID->"ccef0447-e553-49a1-8819-e7eb8f971166"]
}, Open  ]],

Cell[TextData[StyleBox["For computational base states, we can alternatively  \
use the function KetV[List] and BraV[List]:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613396641842534*^9, 3.613396687570217*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->149828826,ExpressionUUID->"dc027092-4b3f-4f7f-abcf-b8212448ff1e"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"KetV", "[", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1"}], "}"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613397362719654*^9, 3.613397368242886*^9}, {
  3.61339917034266*^9, 3.6133992736777477`*^9}, {3.613399310812378*^9, 
  3.61339933698915*^9}, {3.613470980719081*^9, 3.613470981764605*^9}, {
  3.6134712188142033`*^9, 3.613471296107991*^9}, {3.6134715448010883`*^9, 
  3.613471552348054*^9}, {3.613471986218726*^9, 3.613472037065545*^9}, {
  3.624189760478997*^9, 3.624189761540278*^9}, {3.856039628322063*^9, 
  3.8560396288626842`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[89]:=",
 CellID->1322244298,ExpressionUUID->"e62624c5-daf6-4ba5-a42f-996bef153155"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"011\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075411491*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[90]=",
 CellID->482068215,ExpressionUUID->"efbde6d1-d46c-48b5-91cf-bccd6b7b3f8a"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "A", "]"}]], "Input",
 CellChangeTimes->{{3.858967306306622*^9, 3.858967316863701*^9}},
 CellLabel->"In[91]:=",
 CellID->235183781,ExpressionUUID->"57bb4ade-4e26-4889-a880-0ab6b2156919"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"4", "*", 
    RowBox[{"BraV", "[", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], "]"}]}], 
   " ", "+", " ", 
   RowBox[{"2", " ", 
    RowBox[{"BraV", "[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], "]"}]}], 
   "+", 
   RowBox[{"A", "*", 
    RowBox[{"BraV", "[", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}]}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{"DFormA", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613397382777728*^9, 3.61339738834566*^9}, 
   3.613397582815563*^9, 3.613397666567774*^9, 3.6133977911790943`*^9, {
   3.613399507634158*^9, 3.613399525092966*^9}, {3.624189780357535*^9, 
   3.624189781407083*^9}, {3.6241936582826223`*^9, 3.6241936586652937`*^9}, {
   3.856039648470985*^9, 3.8560396506438*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[92]:=",
 CellID->799137330,ExpressionUUID->"c18fab12-935e-4306-a5d3-e86042464a76"],

Cell[BoxData[
 TagBox[
  TemplateBox[{
   "\" + \"", "\" ( \"", "A", "\" ) \"", "\" < \"", "\"00011\"", "\" |\"", 
    "\" + \"", "\" ( \"", "4", "\" ) \"", "\" < \"", "\"01101\"", "\" |\"", 
    "\" + \"", "\" ( \"", "2", "\" ) \"", "\" < \"", "\"11101\"", "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260754330397`*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[93]=",
 CellID->1812173021,ExpressionUUID->"c5024fca-7fe2-4b2d-abae-f58e47c05374"]
}, Open  ]],

Cell[TextData[StyleBox["For example, the following expressions give the same \
output: ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241270179963427`*^9, 3.624127048685725*^9}, {
   3.6241272143100863`*^9, 3.62412729249839*^9}, 3.856038421268099*^9, {
   3.857570219253069*^9, 3.857570220154873*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->738376972,ExpressionUUID->"f83f7190-02f0-451d-9153-b4bbf527b910"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"KetV", "[", 
   RowBox[{"{", "0", "}"}], "]"}], "\[Equal]", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"KetV", "[", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1"}], "}"}], "]"}], "\[Equal]", " ", 
  RowBox[{
   RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
   RowBox[{"Ket", "[", "1", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
   RowBox[{"Ket", "[", "1", "]"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613398016719359*^9, 3.61339803341225*^9}, 
   3.6133980694684553`*^9, {3.6133981079293003`*^9, 3.613398160238776*^9}, {
   3.613398973935718*^9, 3.6133990010821333`*^9}, {3.6133990453034077`*^9, 
   3.613399059004469*^9}, {3.624126993062326*^9, 3.6241270105728607`*^9}, {
   3.624127058500506*^9, 3.624127209936392*^9}, 3.856038421277508*^9, {
   3.856039689781824*^9, 3.856039749440516*^9}, {3.8560397865216017`*^9, 
   3.856039795746303*^9}, {3.856645019090279*^9, 3.856645046831152*^9}, {
   3.85757025180149*^9, 3.857570252465304*^9}, {3.8590552411806297`*^9, 
   3.859055241181508*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellLabel->"In[94]:=",
 CellID->1741256038,ExpressionUUID->"8badbd6e-ef53-4816-8a40-46afa00fbb4c"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075447111*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[94]=",
 CellID->1578882193,ExpressionUUID->"c5f09a22-6c7d-47d4-949e-7ad6acad5b85"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260754539623`*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[95]=",
 CellID->1965075419,ExpressionUUID->"115996dd-f693-4b54-8514-abcb61d99248"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"1"},
     {"0"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260754596453`*^9},
 CellTags->"b:5.1.1",
 CellLabel->"Out[96]=",
 CellID->871559920,ExpressionUUID->"ed8e1f7b-ae2e-404f-a561-5fc44ae3ff68"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ket", "[", "a", "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624127496489051*^9, 3.624127515518977*^9}, 
   3.624127563900861*^9, {3.624128403799021*^9, 3.6241284176457233`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[97]:=",
 CellID->1530572997,ExpressionUUID->"ebef3a97-96c7-4138-bde4-e07af7be0886"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"a"},
   "Ket"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075475276*^9},
 CellLabel->"Out[97]=",
 CellID->321619253,ExpressionUUID->"38606a2a-b950-43c6-a9b7-c7d8906ee185"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Ket", "[", "a", "]"}], "\[CircleTimes]", 
  RowBox[{"Ket", "[", "b", "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624128435097604*^9, 3.624128440618766*^9}, 
   3.8560384212800007`*^9, 3.859055241183058*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[98]:=",
 CellID->1781236813,ExpressionUUID->"19bdd8fc-7fed-4c74-b6d0-d11a74ee468b"],

Cell[BoxData[
 TagBox[
  RowBox[{"KroneckerProduct", "[", 
   RowBox[{
    TemplateBox[{"a"},
     "Ket"], ",", 
    TemplateBox[{"b"},
     "Ket"]}], "]"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607548775*^9},
 CellLabel->"Out[98]=",
 CellID->238445672,ExpressionUUID->"1d0f2f96-268d-4ba7-a8e7-2029c18f471b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 TagBox[
  RowBox[{
   TemplateBox[{"a"},
    "Ket"], "\[CircleTimes]", 
   TemplateBox[{"b"},
    "Ket"]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856038421281619*^9, 3.8560384212832108`*^9}, 
   3.859055241184791*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[99]:=",
 CellID->1325076911,ExpressionUUID->"2ea3781d-1a83-434b-9793-181911f11833"],

Cell[BoxData[
 TagBox[
  RowBox[{"KroneckerProduct", "[", 
   RowBox[{
    TemplateBox[{"a"},
     "Ket"], ",", 
    TemplateBox[{"b"},
     "Ket"]}], "]"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075505744*^9},
 CellLabel->"Out[99]=",
 CellID->351924002,ExpressionUUID->"07c231b5-2af7-488b-ac14-9f0526d1066a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Ket", "[", "a", "]"}], "\[CircleTimes]", 
   RowBox[{"Ket", "[", "b", "]"}]}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"KetV", "[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], "]"}], " "}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624128435097604*^9, 3.6241284911551437`*^9}, {
   3.624128540644599*^9, 3.624128541510832*^9}, {3.624128649377005*^9, 
   3.624128669412656*^9}, 3.8560384212878304`*^9, {3.856039840388419*^9, 
   3.856039840962967*^9}, 3.859055241186502*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[100]:=",
 CellID->1162223787,ExpressionUUID->"eead8c48-ee59-4d0f-92e3-9dda3a98edba"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {"0"},
     {"1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075519689*^9},
 CellLabel->"Out[100]=",
 CellID->616737776,ExpressionUUID->"c770b6ff-87a3-4720-8360-e6717545c0cd"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"11\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607552682*^9},
 CellLabel->"Out[101]=",
 CellID->1755720810,ExpressionUUID->"1667494a-2dba-436e-9ecf-090e65beb109"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {"0"},
     {"1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075533702*^9},
 CellLabel->"Out[102]=",
 CellID->1974343890,ExpressionUUID->"00dcbd7f-a680-46f8-8d6b-3889f33f2581"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"11\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075540092*^9},
 CellLabel->"Out[103]=",
 CellID->2027929433,ExpressionUUID->"f663377a-4b11-4e3f-8ce9-8c68b5bf35a8"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
  RowBox[{"Ket", "[", "1", "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.638523867292844*^9, 3.638523871174526*^9}, 
   3.856038421290923*^9, {3.856039864406516*^9, 3.856039867636898*^9}, 
   3.85905524118793*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[104]:=",
 CellID->2131418644,ExpressionUUID->"daf3705f-f8bf-4d39-8e25-82cb8b17722b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"1"},
     {"0"},
     {"0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075556773*^9},
 CellLabel->"Out[104]=",
 CellID->697018894,ExpressionUUID->"063dacf4-f23e-4cd2-9cbe-07b26b9f12a8"]
}, Open  ]],

Cell[TextData[{
 StyleBox["CNOT[2,1,2]  is the usual CNOT operator ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{
    "for", " ", "two", " ", "qubits", " ", "as", " ", "defined", " ", "by", "  ", 
     StyleBox[
      RowBox[{
       SubscriptBox["\[ScriptCapitalP]", 
        StyleBox["0",
         ScriptLevel->0]], "\[CircleTimes]", 
       RowBox[{"s", "[", "0", "]"}]}],
      ScriptLevel->0]}], 
    StyleBox["+",
     ScriptLevel->0], 
    StyleBox[
     RowBox[{
      SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
      RowBox[{"s", "[", "1", "]"}]}],
     ScriptLevel->0]}], 
   StyleBox[",",
    ScriptLevel->0]}]],
  CellChangeTimes->{3.85609916897044*^9},ExpressionUUID->
  "8091f7ff-562a-41aa-bc46-06a4cfbed154"],
 "with control on qubit 1.\n",
 StyleBox["CNOT[2,2,2]  is the usual CNOT operator ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{
    "for", " ", "two", " ", "qubits", " ", "as", " ", "defined", " ", "by", "  ", 
     StyleBox[
      RowBox[{
       RowBox[{"s", "[", "0", "]"}], "\[CircleTimes]", 
       SubscriptBox["\[ScriptCapitalP]", 
        StyleBox["0",
         ScriptLevel->0]]}],
      ScriptLevel->0]}], 
    StyleBox[" ",
     ScriptLevel->0], 
    StyleBox["+",
     ScriptLevel->0], 
    StyleBox[
     RowBox[{
      RowBox[{"s", "[", "1", "]"}], "\[CircleTimes]", 
      SubscriptBox["\[ScriptCapitalP]", "1"]}],
     ScriptLevel->0]}], 
   StyleBox[",",
    ScriptLevel->0], 
   StyleBox[" ",
    ScriptLevel->0]}]],
  CellChangeTimes->{3.85609916897044*^9},ExpressionUUID->
  "ae79ffc5-7915-4d4b-bd40-e502750ac8e5"],
 "with control on qubit 1."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613396641842534*^9, 3.613396687570217*^9}, {
  3.858336885852271*^9, 3.858337029136138*^9}, {3.859055241190545*^9, 
  3.8590552411957617`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1312550803,ExpressionUUID->"350b9b6b-8dd3-4693-a7cc-3016ffa4315e"],

Cell[TextData[StyleBox["Note s[1] is the not gate:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613396641842534*^9, 3.613396687570217*^9}, {
  3.858336885852271*^9, 3.858337029136138*^9}, {3.8583371187306633`*^9, 
  3.858337133535851*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->422389411,ExpressionUUID->"0924ebea-1d92-47ec-ae10-f63872bd02e1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"s", "[", "1", " ", "]"}], ".", 
    RowBox[{"Ket", "[", "0", "]"}]}], "==", 
   RowBox[{"Ket", "[", "1", "]"}]}], 
  StyleBox["&&",
   FontWeight->"Plain"], 
  RowBox[{
   RowBox[{
    RowBox[{"s", "[", "1", " ", "]"}], ".", 
    RowBox[{"Ket", "[", "1", "]"}]}], "==", 
   RowBox[{"Ket", "[", "0", "]"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8576608518318377`*^9, 3.857660936690525*^9}, 
   3.8578516404462423`*^9, {3.858172551533207*^9, 3.858172552535631*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[105]:=",
 CellID->1819302349,ExpressionUUID->"610f056c-36f7-4e27-9c6d-4168a136fcf8"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326075569178*^9},
 CellLabel->"Out[105]=",
 CellID->273257073,ExpressionUUID->"4658c6c7-c97f-4a0b-9a2c-e2049348fa86"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Circuit*"}]], "Input",
 CellChangeTimes->{{3.8600734608330507`*^9, 3.8600734831412067`*^9}},
 CellLabel->"In[106]:=",
 CellID->858753316,ExpressionUUID->"41999070-83f8-4fc3-a4e2-d997de3a064b"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["Notebook$$16$318441`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               
               StyleBox["Notebook$$16$318441`", 
                "InformationGridGroupHeader"]}],
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["CircuitToOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a6a7d17d-c8a1-4a43-a4a5-18ce862dd71a", {
                    "CircuitToOp", "Notebook$$16$318441`"}, False},
                  ButtonNote->"Notebook$$16$318441`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CircuitToOp2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a6a7d17d-c8a1-4a43-a4a5-18ce862dd71a", {
                    "CircuitToOp2", "Notebook$$16$318441`"}, False},
                  ButtonNote->"Notebook$$16$318441`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.475]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]},
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Circuits`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Circuits`", "InformationGridGroupHeader"]}],
              
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["Circuit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-a6a7d17d-c8a1-4a43-a4a5-18ce862dd71a", {
                    "Circuit", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CircuitG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a6a7d17d-c8a1-4a43-a4a5-18ce862dd71a", {
                    "CircuitG", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["CircuitW", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a6a7d17d-c8a1-4a43-a4a5-18ce862dd71a", {
                    "CircuitW", "QDENSITY`Circuits`"}, False},
                  ButtonNote->"QDENSITY`Circuits`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.31666666666666665`]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{
      "ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {2 -> GrayLevel[0.85]}},
     
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260756125593`*^9},
 CellLabel->"Out[106]=",
 CellID->1311588625,ExpressionUUID->"87d12fa4-de21-4d9a-bbce-ade7106842bb"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Circuit", "[", 
     RowBox[{
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"C"},
         {"CX"}
        },
        GridBoxDividers->{"Columns" -> {{True}}, "Rows" -> {{True}}}], 
       "\[NoBreak]", ")"}], ",", 
      RowBox[{"{", "}"}]}], "]"}], ",", 
    RowBox[{"Circuit", "[", 
     RowBox[{
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"CX"},
         {"C"}
        },
        GridBoxDividers->{"Columns" -> {{True}}, "Rows" -> {{True}}}], 
       "\[NoBreak]", ")"}], ",", 
      RowBox[{"{", "}"}]}], "]"}]}], "}"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.85766038121061*^9, 3.857660447477482*^9}, {
   3.857660687331286*^9, 3.857660795614683*^9}, 3.8583371502320147`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[107]:=",
 CellID->731125707,ExpressionUUID->"2557d480-d927-43ea-9f8e-60f73b425e65"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {InsetBox[
      FormBox[
       GraphicsBox[{
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {-0.1, 0.}]}, 
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
         {GrayLevel[0], Thickness[0.007], LineBox[{{0, 0.}, {0.4, 0.}}]}, 
         {GrayLevel[0], Thickness[0.007], 
          LineBox[{{0, -0.30000000000000004`}, {
            0.4, -0.30000000000000004`}}]}, 
         {PointSize[0.04000000000000001], PointBox[{0.2, 0.}], 
          {RGBColor[0, 0, 1], InsetBox[
            StyleBox["\<\"\[CirclePlus]\"\>",
             StripOnInput->False,
             FontFamily->"Symbol",
             FontSize->34,
             FontWeight->"Bold"], {0.2, -0.30000000000000004}]}, 
          {GrayLevel[0], Thickness[Large], 
           LineBox[{{0.2, 0.}, {0.2, -0.30000000000000004`}}]}}, 
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {0.6000000000000001, 0.}]}, 
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {0.6000000000000001, -0.30000000000000004}]}},
        Background->GrayLevel[1],
        ImageSize->500],
       TraditionalForm], {266.6666666666667, -124.425}, {
      Center, Center}, {500.00000000000006, 236.99999999999997}], InsetBox[
      FormBox[
       GraphicsBox[{
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {-0.1, 0.}]}, 
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
         {GrayLevel[0], Thickness[0.007], LineBox[{{0, 0.}, {0.4, 0.}}]}, 
         {GrayLevel[0], Thickness[0.007], 
          LineBox[{{0, -0.30000000000000004`}, {
            0.4, -0.30000000000000004`}}]}, 
         {PointSize[0.04000000000000001], 
          PointBox[{0.2, -0.30000000000000004`}], 
          {RGBColor[0, 0, 1], InsetBox[
            StyleBox["\<\"\[CirclePlus]\"\>",
             StripOnInput->False,
             FontFamily->"Symbol",
             FontSize->34,
             FontWeight->"Bold"], {0.2, 0.}]}, 
          {GrayLevel[0], Thickness[Large], 
           LineBox[{{0.2, -0.30000000000000004`}, {0.2, 0.}}]}}, 
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {0.6000000000000001, 0.}]}, 
         {GrayLevel[0], InsetBox[
           StyleBox[
            TemplateBox[{"\"\"", "\"\"", "\"\""},
             "RowDefault"],
            StripOnInput->False,
            FontFamily->"Times",
            FontSize->Scaled[0.05],
            FontWeight->"Bold"], {0.6000000000000001, -0.30000000000000004}]}},
        Background->GrayLevel[1],
        ImageSize->500],
       TraditionalForm], {800., -124.425}, {
      Center, Center}, {500., 236.99999999999997}]}, {}},
   ImageSize->{
     UpTo[600], Automatic},
   PlotRange->{{0, 1066.6666666666667`}, {-248.85, 0}},
   PlotRangePadding->{6, 5}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076568775*^9},
 CellLabel->"Out[107]=",
 CellID->2097867567,ExpressionUUID->"70c26705-27fd-49dc-a0e9-77ee198ecbe5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"CN", 
  StyleBox["=",
   ScriptLevel->0,
   FontSize->16], 
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", 
      StyleBox["0",
       ScriptLevel->0]], "\[CircleTimes]", 
     SubscriptBox["\[Sigma]", "0"]}],
    ScriptLevel->0], 
   StyleBox["+",
    ScriptLevel->0], 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
    SubscriptBox["\[Sigma]", "1"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CNOT", "[", 
   RowBox[{"2", ",", "1", ",", "2"}], "]"}], "  ", 
  RowBox[{"(*", "  ", 
   RowBox[{
   "Control", " ", "qubit", " ", "1", " ", "NOT", " ", "on", " ", "qubit", 
    " ", "2"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ControlledX", "[", 
   RowBox[{"2", ",", "1", ",", "2"}], "]"}], "==", 
  "%"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CNOT", "[", 
   RowBox[{"2", ",", "2", ",", "1"}], "]"}], "  ", 
  RowBox[{"(*", "  ", 
   RowBox[{
   "Control", " ", "qubit", " ", "2", " ", "Not", " ", "on", " ", "qubit", 
    " ", "1"}], "*)"}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sigma]", "0"], "\[CircleTimes]", 
     SubscriptBox["\[ScriptCapitalP]", 
      StyleBox["0",
       ScriptLevel->0]]}], "+", 
    RowBox[{
     SubscriptBox["\[Sigma]", "1"], "\[CircleTimes]", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}]}], "==", "%"}],
  ScriptLevel->0]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.638523425969421*^9, 3.63852343337566*^9}, {
   3.6385234695574102`*^9, 3.638523504229761*^9}, {3.638523545024722*^9, 
   3.638523570289719*^9}, {3.638523650994432*^9, 3.638523683330885*^9}, {
   3.856038421300522*^9, 3.856038421302664*^9}, {3.856039907518118*^9, 
   3.85603991860042*^9}, {3.8566006731670237`*^9, 3.856600703484694*^9}, {
   3.856600876163528*^9, 3.8566009433143177`*^9}, {3.85660246775849*^9, 
   3.8566024851646967`*^9}, 3.856613138895651*^9, {3.8566450957261763`*^9, 
   3.856645096257036*^9}, 3.8566452090069227`*^9, 3.856701636502595*^9, {
   3.8567329905011578`*^9, 3.8567329919937687`*^9}, {3.857570293303594*^9, 
   3.857570293967866*^9}, {3.857570374619504*^9, 3.8575704082334747`*^9}, {
   3.857570476928566*^9, 3.857570506503986*^9}, {3.857570549034992*^9, 
   3.857570723238604*^9}, 3.8576331568850946`*^9, 3.857660348983961*^9, {
   3.857852724300295*^9, 3.857852814198262*^9}, {3.8590552411979237`*^9, 
   3.859055241202037*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[108]:=",
 CellID->83465538,ExpressionUUID->"3336553c-4cb8-44b4-8e55-74e40473aa4f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "0", "1"},
     {"0", "0", "1", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260766404533`*^9},
 CellLabel->"Out[108]=",
 CellID->1473682123,ExpressionUUID->"50fe3584-d06d-495d-9824-c31e237eeebc"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "0", "1"},
     {"0", "0", "1", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260766485043`*^9},
 CellLabel->"Out[109]=",
 CellID->1246540410,ExpressionUUID->"35e232e4-9872-432d-888a-23262d9acb2f"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076654153*^9},
 CellLabel->"Out[110]=",
 CellID->1901694385,ExpressionUUID->"daa1ba19-9a7b-49b6-9cb5-8f24c481ef3e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "0", "0", "1"},
     {"0", "0", "1", "0"},
     {"0", "1", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076659791*^9},
 CellLabel->"Out[111]=",
 CellID->778410394,ExpressionUUID->"abeaba3d-da8e-456a-9bbd-81bdf90d16fe"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076665208*^9},
 CellLabel->"Out[112]=",
 CellID->273114434,ExpressionUUID->"ffd94a7b-469f-457c-8a6b-2fa52a9bc3ae"]
}, Open  ]],

Cell[TextData[StyleBox["The proper action of the CNOT  gate is verified \
below:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613396641842534*^9, 3.613396687570217*^9}, {
  3.857571113483108*^9, 3.8575711406414013`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->147551722,ExpressionUUID->"8247f088-0ba6-48d1-bbbf-138ef69f01af"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CN", ".", 
   RowBox[{
    StyleBox["(",
     FontSize->16], 
    RowBox[{
     RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
     RowBox[{"Ket", "[", "0", "]"}]}], ")"}]}], "==", 
  RowBox[{
   RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
   RowBox[{"Ket", "[", "0", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   RowBox[{"CN", ".", 
    RowBox[{
     StyleBox["(",
      FontSize->16], 
     RowBox[{
      RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
      RowBox[{"Ket", "[", "1", "]"}]}], ")"}]}], "==", 
   RowBox[{
    RowBox[{"Ket", "[", "0", "]"}], "\[CircleTimes]", 
    RowBox[{"Ket", "[", "1", "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CN", ".", 
   RowBox[{
    StyleBox["(",
     FontSize->16], 
    RowBox[{
     RowBox[{"Ket", "[", "1", "]"}], "\[CircleTimes]", 
     RowBox[{"Ket", "[", "0", "]"}]}], ")"}]}], "==", 
  RowBox[{
   RowBox[{"Ket", "[", "1", "]"}], "\[CircleTimes]", 
   RowBox[{"Ket", "[", "1", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CN", ".", 
   RowBox[{
    StyleBox["(",
     FontSize->16], 
    RowBox[{
     RowBox[{"Ket", "[", "1", "]"}], "\[CircleTimes]", 
     RowBox[{"Ket", "[", "1", "]"}]}], ")"}]}], "==", 
  RowBox[{
   RowBox[{"Ket", "[", "1", "]"}], "\[CircleTimes]", 
   RowBox[{"Ket", "[", "0", "]"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.638523425969421*^9, 3.63852343337566*^9}, {
  3.6385234695574102`*^9, 3.638523504229761*^9}, {3.638523545024722*^9, 
  3.638523570289719*^9}, {3.638523650994432*^9, 3.638523683330885*^9}, {
  3.856038421300522*^9, 3.856038421302664*^9}, {3.856039907518118*^9, 
  3.85603991860042*^9}, {3.856099387246867*^9, 3.856099512024186*^9}, {
  3.856099646957522*^9, 3.856099647725861*^9}, {3.856602439173789*^9, 
  3.8566024607822933`*^9}, {3.857851640449194*^9, 3.857851640450943*^9}, {
  3.857852849636511*^9, 3.857852916234231*^9}, {3.85905524120411*^9, 
  3.8590552412121363`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[113]:=",
 CellID->832742574,ExpressionUUID->"c2f895d7-e4aa-4a58-a374-92b287633f60"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076676119*^9},
 CellLabel->"Out[113]=",
 CellID->1061738599,ExpressionUUID->"685abdc4-28d9-43a6-bd09-42a0b076c286"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076682311*^9},
 CellLabel->"Out[114]=",
 CellID->1199010470,ExpressionUUID->"185ea40d-28f0-4d90-8647-3558fc74c95c"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076687703*^9},
 CellLabel->"Out[115]=",
 CellID->1929909489,ExpressionUUID->"33c924bb-b264-418b-bdb0-0d5d4a1c9f22"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076693668*^9},
 CellLabel->"Out[116]=",
 CellID->395777335,ExpressionUUID->"040f4bfb-c1b7-443e-b70f-03648c77d96a"]
}, Open  ]],

Cell[TextData[StyleBox["The construction of other control gates follows the \
same procedures.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613396641842534*^9, 3.613396687570217*^9}, {
  3.857571113483108*^9, 3.8575711406414013`*^9}, {3.858884920814979*^9, 
  3.8588849622715273`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.1",
 CellID->1203011483,ExpressionUUID->"0a0691fe-ad9b-4811-9815-d39febd8453d"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ControlledZ", "[", 
   RowBox[{"2", ",", "1", ",", "2"}], "]"}], "==", 
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", 
      StyleBox["0",
       ScriptLevel->0]], "\[CircleTimes]", 
     SubscriptBox["\[Sigma]", "0"]}],
    ScriptLevel->0], 
   StyleBox["+",
    ScriptLevel->0], 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
    SubscriptBox["\[Sigma]", "3"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CPHASE", "[", 
   RowBox[{"2", ",", "1", ",", "2"}], "]"}], "==", 
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", 
      StyleBox["0",
       ScriptLevel->0]], "\[CircleTimes]", 
     SubscriptBox["\[Sigma]", "0"]}],
    ScriptLevel->0], 
   StyleBox["+",
    ScriptLevel->0], 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
    SubscriptBox["\[Sigma]", "3"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ControlledY", "[", 
   RowBox[{"2", ",", "1", ",", "2"}], "]"}], "==", 
  RowBox[{
   StyleBox[
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", 
      StyleBox["0",
       ScriptLevel->0]], "\[CircleTimes]", 
     SubscriptBox["\[Sigma]", "0"]}],
    ScriptLevel->0], 
   StyleBox["+",
    ScriptLevel->0], 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
    SubscriptBox["\[Sigma]", "2"], " "}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.638523425969421*^9, 3.63852343337566*^9}, {
  3.6385234695574102`*^9, 3.638523504229761*^9}, {3.638523545024722*^9, 
  3.638523570289719*^9}, {3.638523650994432*^9, 3.638523683330885*^9}, {
  3.856038421300522*^9, 3.856038421302664*^9}, {3.856039907518118*^9, 
  3.85603991860042*^9}, {3.856099387246867*^9, 3.856099512024186*^9}, {
  3.856099646957522*^9, 3.856099647725861*^9}, {3.856602439173789*^9, 
  3.8566024607822933`*^9}, {3.857851640449194*^9, 3.857851640450943*^9}, {
  3.857852849636511*^9, 3.857852916234231*^9}, {3.8588849917588387`*^9, 
  3.858885002558666*^9}, {3.858885044702182*^9, 3.85888519016473*^9}, {
  3.858885251152712*^9, 3.85888530658183*^9}, {3.8590552412142973`*^9, 
  3.859055241219698*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[117]:=",
 CellID->1809609647,ExpressionUUID->"48c8bf72-0fdb-40c3-be68-2af8e916d230"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076711874*^9},
 CellLabel->"Out[117]=",
 CellID->1072814586,ExpressionUUID->"f1a45c3d-4e0b-4e89-a08b-d77a56653d6e"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076720035*^9},
 CellLabel->"Out[118]=",
 CellID->522046690,ExpressionUUID->"72c0d098-d476-40bc-a734-f5f749d3e6da"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076727216*^9},
 CellLabel->"Out[119]=",
 CellID->1568020835,ExpressionUUID->"035b1736-a6bc-43c4-806e-431ad66a3dcc"]
}, Open  ]],

Cell[TextData[StyleBox["Bell states",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.8589675141996593`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->459418141,ExpressionUUID->"6e5e7dbf-3a1e-4e19-b04e-32513867125c"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1236984822,ExpressionUUID->"fb7263ff-4ba6-4c08-82c9-567a1e4e5750"],

Cell[TextData[StyleBox[" The Bell (two qubit) and the GHZ (three qubit) \
states are important for understanding entanglement. ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624193746911874*^9, 3.6241938344127073`*^9}, 
   3.856099714400394*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->1573400686,ExpressionUUID->"4ceb7ec0-d33c-4955-b037-3d31a97f8fe0"],

Cell[TextData[{
 StyleBox["The ",
  FontFamily->"Helvetica"],
 StyleBox["Bell states",
  FontFamily->"Helvetica"],
 StyleBox[" are the following:\n",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "00"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "45f73f46-49b7-46a8-b0dd-36001942a403"],
 "( |00\[RightAngleBracket] + |11\[RightAngleBracket] )\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "01"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "27796982-d2f6-4613-b7b9-1b07c9421ede"],
 "( |01\[RightAngleBracket] + |10\[RightAngleBracket] )\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "10"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "0647741f-0c2e-49a4-abd4-d63be4fc6d9d"],
 "( |00\[RightAngleBracket] - |11\[RightAngleBracket] )\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "11"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "b13bd9f4-ccd6-4f9b-9b72-647261f08aae"],
 "( |01\[RightAngleBracket] - |10\[RightAngleBracket] )"
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->31700444,ExpressionUUID->"b2025513-d84e-43f0-9ba2-c0a0f51efb2b"],

Cell[TextData[StyleBox["They can all be constructed by building  a Bell \
operator from a Hadamard and then a CNOT gate.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624193911679122*^9, 3.624193943403536*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->213417852,ExpressionUUID->"a20296b5-b24f-4643-a6e0-bc4518f6d0c8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"had", "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419409666529*^9, 3.624194109940762*^9}, {
  3.856602514502843*^9, 3.85660254339229*^9}, {3.8566025821646423`*^9, 
  3.856602592521537*^9}, {3.856602625723961*^9, 3.8566026341360617`*^9}, {
  3.85664527153104*^9, 3.8566452768761473`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[120]:=",
 CellID->616148074,ExpressionUUID->"d01f4d51-3fff-4419-ad3b-a13929241b5a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076739336*^9},
 CellLabel->"Out[120]=",
 CellID->1954341508,ExpressionUUID->"8167432b-5872-454e-a76e-e27453bc8c23"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"CN", ".", 
  RowBox[{"had", "[", 
   RowBox[{"2", ",", "1"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Bellop", "=", " ", 
  RowBox[{"CN", ".", 
   RowBox[{"(", 
    RowBox[{"\[ScriptCapitalH]", "\[CircleTimes]", 
     RowBox[{"s", "[", "0", "]"}]}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"%", "==", "%%"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419409666529*^9, 3.624194109940762*^9}, {
   3.856602514502843*^9, 3.85660254339229*^9}, {3.8566025821646423`*^9, 
   3.856602592521537*^9}, {3.856602625723961*^9, 3.8566026341360617`*^9}, {
   3.856645297113361*^9, 3.856645317589127*^9}, 3.859055241221314*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[121]:=",
 CellID->929336186,ExpressionUUID->"98107538-1556-4605-b74e-358ca6648d9d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076758778*^9},
 CellLabel->"Out[121]=",
 CellID->855344091,ExpressionUUID->"914169aa-7849-42d8-a0dd-f750da4c2973"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076767498*^9},
 CellLabel->"Out[122]=",
 CellID->512604419,ExpressionUUID->"7bc29fc6-dae2-4301-9b5b-463033a50ea7"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076774497*^9},
 CellLabel->"Out[123]=",
 CellID->1785562016,ExpressionUUID->"3e840b35-a2e7-4ed3-9510-01b913caaa35"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"?", "Circuit"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"BellZ", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"H", ",", "1"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"C", ",", "CX"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"BellC", "=", 
  RowBox[{"Circuit", "[", 
   RowBox[{"BellZ", ",", 
    RowBox[{"{", "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", "\"\<With measurement of qubit 2\>\"", "\
\[IndentingNewLine]", 
 RowBox[{"Circuit", "[", 
  RowBox[{
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"H", "C"},
      {"1", "CX"}
     },
     GridBoxDividers->{"Columns" -> {{True}}, "Rows" -> {{True}}}], 
    "\[NoBreak]", ")"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.477070157806766*^9, 3.477070281685054*^9}, {
   3.4770703792574263`*^9, 3.47707042927422*^9}, {3.477070573445764*^9, 
   3.477070938257218*^9}, {3.477070975153408*^9, 3.477071126070225*^9}, {
   3.477071282223071*^9, 3.477071315285598*^9}, {3.477071499087633*^9, 
   3.477071530300399*^9}, {3.477073069188739*^9, 3.477073070010247*^9}, {
   3.4770736958391542`*^9, 3.477073708823803*^9}, {3.477079242977557*^9, 
   3.477079290506935*^9}, {3.477093808836267*^9, 3.477093810415637*^9}, {
   3.477094412705249*^9, 3.4770944142135677`*^9}, {3.47709491182619*^9, 
   3.4770949249268312`*^9}, {3.477095234320334*^9, 3.477095262988146*^9}, 
   3.477095496445863*^9, {3.477137200006351*^9, 3.477137286916142*^9}, {
   3.4771373253048077`*^9, 3.4771373490266247`*^9}, {3.4771377125308332`*^9, 
   3.4771377132904453`*^9}, {3.477159534022073*^9, 3.477159542105319*^9}, {
   3.4771638303782177`*^9, 3.477163832137855*^9}, {3.477178560731306*^9, 
   3.477178561329048*^9}, {3.477489410831441*^9, 3.477489435613223*^9}, {
   3.4774895034301653`*^9, 3.477489605463584*^9}, {3.477489806780609*^9, 
   3.477489835926475*^9}, {3.4774898779796352`*^9, 3.4774899129767733`*^9}, {
   3.4774899671857557`*^9, 3.4774899950314302`*^9}, {3.4774902316484013`*^9, 
   3.477490295239564*^9}, {3.477490386324018*^9, 3.477490408520185*^9}, {
   3.477529679907599*^9, 3.4775296853639717`*^9}, {3.47752989748345*^9, 
   3.477530003182823*^9}, {3.477530083023734*^9, 3.4775300948826027`*^9}, 
   3.477530136050435*^9, 3.47753016860983*^9, {3.477530205476824*^9, 
   3.477530275902094*^9}, 3.477530308739037*^9, {3.477530559384218*^9, 
   3.477530586364442*^9}, {3.4775306304486637`*^9, 3.477530632724867*^9}, 
   3.4775307294213247`*^9, {3.477530764966612*^9, 3.477530773215063*^9}, 
   3.4775308327835093`*^9, 3.477530871937608*^9, {3.477530932945829*^9, 
   3.477530937337886*^9}, {3.477576684921529*^9, 3.477576715496839*^9}, {
   3.477576818175784*^9, 3.477577004957158*^9}, {3.477577083222196*^9, 
   3.477577106433175*^9}, {3.477577148038652*^9, 3.47757725510058*^9}, 
   3.477577294927318*^9, {3.477577362162107*^9, 3.477577480058915*^9}, 
   3.4775778618903313`*^9, 3.477577898411117*^9, {3.477578375099497*^9, 
   3.477578434673678*^9}, 3.477588996180668*^9, {3.477675563947165*^9, 
   3.4776755712487926`*^9}, {3.477676856219151*^9, 3.477676870229965*^9}, 
   3.477678710588191*^9, {3.4776790921758127`*^9, 3.477679109401775*^9}, {
   3.477679160537922*^9, 3.477679182451352*^9}, {3.856100131176408*^9, 
   3.856100176344674*^9}, {3.856100233889306*^9, 3.8561002769669743`*^9}, {
   3.856100316781047*^9, 3.856100325546768*^9}, {3.856100360252043*^9, 
   3.85610040045059*^9}, {3.8561051175778008`*^9, 3.856105138019251*^9}, {
   3.8561051769832993`*^9, 3.8561052011004953`*^9}, {3.8561061021081*^9, 
   3.856106163449855*^9}, 3.85660267306327*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[124]:=",
 CellID->1389156325,ExpressionUUID->"17396c8c-87f6-4568-8821-3bbe1c84a279"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"Circuit[mat,meters] generates a quantum\\n circuit \
diagram with operators stipulated by the\\ncircuit  matrix 'mat' and the \
selection of measurements \\nat the end.  For example. Circuit[mat,{1,6}] \
places\\nmeters at end of qubit 1 and 6 lines.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"Circuit", "[", 
                    RowBox[{"matr_", ",", "pickmeters_"}], "]"}], ":=", 
                    RowBox[{"Module", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"cl", ",", "mtrs", ",", "nqc", ",", "fwo"}], 
                    "}"}], ",", 
                    RowBox[{
                    RowBox[{"mtrs", "=", "pickmeters"}], ";", 
                    RowBox[{"nqc", "=", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"Length", "/@", "matr"}], ")"}], 
                    "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}], 
                    ";", 
                    RowBox[{"cl", "=", 
                    RowBox[{"3", " ", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"Length", "/@", "matr"}], ")"}], 
                    "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                    " ", "boxsize"}]}], ";", 
                    RowBox[{
                    RowBox[{"fwo", "[", 
                    RowBox[{"ir_", ",", "ic_"}], "]"}], ":=", 
                    RowBox[{"WiresWO", "[", 
                    RowBox[{"ir", ",", "ic", ",", "matr"}], "]"}]}], ";", 
                    RowBox[{"Style", "[", 
                    RowBox[{"Show", "[", 
                    RowBox[{
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"Initstate", "[", 
                    RowBox[{"icc", ",", "matr"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"icc", ",", "1", ",", 
                    RowBox[{"Length", "[", "matr", "]"}]}], "}"}]}], "]"}], 
                    ",", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"Wires", "[", 
                    RowBox[{"icw", ",", "matr"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"icw", ",", "1", ",", 
                    RowBox[{"Length", "[", "matr", "]"}]}], "}"}]}], "]"}], 
                    ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"fwo", ",", 
                    RowBox[{"Position", "[", 
                    RowBox[{"matr", ",", "WO"}], "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"OQO", ",", 
                    RowBox[{"get1", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"OQO", ",", 
                    RowBox[{"get1R", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"CNTRG", ",", 
                    RowBox[{"get2", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"CNOTC", ",", 
                    RowBox[{"get2X", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"CNOTC", ",", 
                    RowBox[{"get2X2", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"CNOTC", ",", 
                    RowBox[{"get2X3", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"WBOX", ",", 
                    RowBox[{"box", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"Toff", ",", 
                    RowBox[{"get3", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"Toff", ",", 
                    RowBox[{"get32", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"Toff", ",", 
                    RowBox[{"get33", "[", "matr", "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"Finalstate", "[", 
                    RowBox[{"ifs", ",", "matr"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"ifs", ",", "1", ",", 
                    RowBox[{"Length", "[", "matr", "]"}]}], "}"}]}], "]"}], 
                    ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"meter2", ",", 
                    RowBox[{"Position", "[", 
                    RowBox[{"matr", ",", "M"}], "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"DBAR", ",", 
                    RowBox[{"Position", "[", 
                    RowBox[{"matr", ",", "DB"}], "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"DBAR2", ",", 
                    RowBox[{"Position", "[", 
                    RowBox[{"matr", ",", "DBU"}], "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"BMR", ",", 
                    RowBox[{"Position", "[", 
                    RowBox[{"matr", ",", "BM"}], "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Apply", "[", 
                    RowBox[{"BML", ",", 
                    RowBox[{"Position", "[", 
                    RowBox[{"matr", ",", "BML"}], "]"}], ",", 
                    RowBox[{"{", "1", "}"}]}], "]"}], ",", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"meter", "[", 
                    RowBox[{
                    
                    RowBox[{"mtrs", "\[LeftDoubleBracket]", "ime", 
                    "\[RightDoubleBracket]"}], ",", "nqc"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"ime", ",", "1", ",", 
                    RowBox[{"Length", "[", "mtrs", "]"}]}], "}"}]}], "]"}], 
                    ",", 
                    RowBox[{"Background", "\[Rule]", "White"}], ",", 
                    RowBox[{"ImageSize", "\[Rule]", "picsize"}]}], "]"}], 
                    "]"}]}]}], "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Circuits`Circuit\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"Circuit[mat,meters] generates a quantum\\n circuit \
diagram with operators stipulated by the\\ncircuit  matrix 'mat' and the \
selection of measurements \\nat the end.  For example. Circuit[mat,{1,6}] \
places\\nmeters at end of qubit 1 and 6 lines.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "Circuit[mat,meters] generates a quantum\n circuit diagram with \
operators stipulated by the\ncircuit  matrix 'mat' and the selection of \
measurements \nat the end.  For example. Circuit[mat,{1,6}] places\nmeters at \
end of qubit 1 and 6 lines.", "Documentation" -> None, "OwnValues" -> None, 
     "UpValues" -> None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Circuits`Circuit, {QDENSITY`Circuits`Circuit[
          Pattern[QDENSITY`Circuits`matr, 
           Blank[]], 
          Pattern[QDENSITY`Circuits`pickmeters, 
           Blank[]]] :> 
        Module[{QDENSITY`Circuits`cl, QDENSITY`Circuits`mtrs, 
           QDENSITY`Circuits`nqc, QDENSITY`Circuits`fwo}, 
          QDENSITY`Circuits`mtrs = QDENSITY`Circuits`pickmeters; 
          QDENSITY`Circuits`nqc = Part[
             Map[Length, QDENSITY`Circuits`matr], 1]; 
          QDENSITY`Circuits`cl = 3 Part[
              Map[Length, QDENSITY`Circuits`matr], 1] 
            QDENSITY`Circuits`boxsize; QDENSITY`Circuits`fwo[
             Pattern[QDENSITY`Circuits`ir, 
              Blank[]], 
             Pattern[QDENSITY`Circuits`ic, 
              Blank[]]] := 
           QDENSITY`Circuits`WiresWO[
            QDENSITY`Circuits`ir, QDENSITY`Circuits`ic, 
             QDENSITY`Circuits`matr]; Style[
            Show[
             Table[
              QDENSITY`Circuits`Initstate[
              QDENSITY`Circuits`icc, QDENSITY`Circuits`matr], {
              QDENSITY`Circuits`icc, 1, 
               Length[QDENSITY`Circuits`matr]}], 
             Table[
              QDENSITY`Circuits`Wires[
              QDENSITY`Circuits`icw, QDENSITY`Circuits`matr], {
              QDENSITY`Circuits`icw, 1, 
               Length[QDENSITY`Circuits`matr]}], 
             Apply[QDENSITY`Circuits`fwo, 
              Position[QDENSITY`Circuits`matr, QDENSITY`Circuits`WO], {1}], 
             Apply[QDENSITY`Circuits`OQO, 
              QDENSITY`Circuits`get1[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`OQO, 
              QDENSITY`Circuits`get1R[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`CNTRG, 
              QDENSITY`Circuits`get2[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`CNOTC, 
              QDENSITY`Circuits`get2X[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`CNOTC, 
              QDENSITY`Circuits`get2X2[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`CNOTC, 
              QDENSITY`Circuits`get2X3[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`WBOX, 
              QDENSITY`Circuits`box[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`Toff, 
              QDENSITY`Circuits`get3[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`Toff, 
              QDENSITY`Circuits`get32[QDENSITY`Circuits`matr], {1}], 
             Apply[QDENSITY`Circuits`Toff, 
              QDENSITY`Circuits`get33[QDENSITY`Circuits`matr], {1}], 
             Table[
              QDENSITY`Circuits`Finalstate[
              QDENSITY`Circuits`ifs, QDENSITY`Circuits`matr], {
              QDENSITY`Circuits`ifs, 1, 
               Length[QDENSITY`Circuits`matr]}], 
             Apply[QDENSITY`Circuits`meter2, 
              Position[QDENSITY`Circuits`matr, QDENSITY`Circuits`M], {1}], 
             Apply[QDENSITY`Circuits`DBAR, 
              Position[QDENSITY`Circuits`matr, QDENSITY`Circuits`DB], {1}], 
             Apply[QDENSITY`Circuits`DBAR2, 
              Position[QDENSITY`Circuits`matr, QDENSITY`Circuits`DBU], {1}], 
             Apply[QDENSITY`Circuits`BMR, 
              Position[QDENSITY`Circuits`matr, QDENSITY`Circuits`BM], {1}], 
             Apply[QDENSITY`Circuits`BML, 
              Position[QDENSITY`Circuits`matr, QDENSITY`Circuits`BML], {1}], 
             Table[
              QDENSITY`Circuits`meter[
               Part[QDENSITY`Circuits`mtrs, QDENSITY`Circuits`ime], 
               QDENSITY`Circuits`nqc], {QDENSITY`Circuits`ime, 1, 
               Length[QDENSITY`Circuits`mtrs]}], Background -> White, 
             ImageSize -> QDENSITY`Circuits`picsize]]]}], "SubValues" -> None,
      "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Circuits`Circuit"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260768659277`*^9},
 CellLabel->"Out[124]=",
 CellID->1004319734,ExpressionUUID->"5dd900c3-e566-4ef8-91a4-e2df6b3aff9f"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, 0.}, {0.7000000000000001, 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {
       0.7000000000000001, -0.30000000000000004`}}]}, 
    {EdgeForm[GrayLevel[0]], FaceForm[RGBColor[0.87, 0.94, 1]], 
     RectangleBox[{0.1, -0.1}, {0.30000000000000004, 0.1}], InsetBox[
      StyleBox["H",
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.2, 0.}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.30000000000000004`}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, -0.30000000000000004}]}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076882897*^9},
 CellLabel->"Out[126]=",
 CellID->380303918,ExpressionUUID->"c29d0914-89ab-4d5d-ab1d-254032fea5f9"],

Cell[BoxData[
 TagBox["\<\"With measurement of qubit 2\"\>",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076894404*^9},
 CellLabel->"Out[127]=",
 CellID->856616354,ExpressionUUID->"1e50ea41-e01e-420f-889b-277cf00a886e"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, 0.}, {0.7000000000000001, 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {
       0.7000000000000001, -0.30000000000000004`}}]}, 
    {EdgeForm[GrayLevel[0]], FaceForm[RGBColor[0.87, 0.94, 1]], 
     RectangleBox[{0.1, -0.1}, {0.30000000000000004, 0.1}], InsetBox[
      StyleBox["H",
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.2, 0.}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.30000000000000004`}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, -0.30000000000000004}]}, {
     {RGBColor[1, 0, 0], Thickness[Large], 
      CircleBox[{0.7000000000000001, -0.30000000000000004}, 0.1, 
       NCache[{(-90) Degree, 90 Degree}, {-1.5707963267948966`, 
        1.5707963267948966`}]]}, 
     {RGBColor[1, 0, 0], Thickness[Large], 
      LineBox[{{0.7000000000000001, -0.4}, {
       0.7000000000000001, -0.20000000000000004`}}]}, 
     {RGBColor[1, 0, 0], Thickness[Large], 
      ArrowBox[{{0.7000000000000001, -0.30000000000000004`}, {
       0.8, -0.20000000000000004`}}], 
      {RGBColor[1, 0, 0], Dashing[{0, Small}], 
       LineBox[{{0.6000000000000001, -0.30000000000000004`}, {
        0.7000000000000001, -0.30000000000000004`}}]}}}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076902053*^9},
 CellLabel->"Out[128]=",
 CellID->420692190,ExpressionUUID->"62a95af5-cb99-48b1-9925-28e38d202f80"]
}, Open  ]],

Cell[BoxData[""], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856100282355442*^9, 3.856100285549995*^9}, 
   3.856105227537155*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[129]:=",
 CellID->1623344346,ExpressionUUID->"034bec5f-0037-4e00-b809-9b0e12b05515"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{
    RowBox[{"Adj", "[", "Bellop", "]"}], ".", "Bellop"}],
   ScriptLevel->0], 
  StyleBox["==",
   ScriptLevel->0], 
  RowBox[{"IdentityMatrix", "[", "4", "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613474725914527*^9, {3.613475476151991*^9, 3.61347551145238*^9}, {
   3.613475725777679*^9, 3.613475746502296*^9}, {3.856602696968548*^9, 
   3.856602717938486*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[130]:=",
 CellID->434533884,ExpressionUUID->"9e8e09e3-5c6a-44b1-ad8b-24b264350687"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076931281*^9},
 CellLabel->"Out[130]=",
 CellID->1057471406,ExpressionUUID->"e055cca6-c039-4a5f-b9ba-d021b4969ec2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Bellop", ".", 
  RowBox[{"AF", "[", " ", 
   RowBox[{
    RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
    RowBox[{"Ket", "[", "0", "]"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Bellop", ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
     RowBox[{"Ket", "[", "0", "]"}]}], ")"}]}], "==", "%"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419424716914*^9, 3.6241942483612432`*^9}, 
   3.6241943270803843`*^9, 3.85603842130717*^9, {3.856039962913104*^9, 
   3.856040067944312*^9}, {3.8566041134343357`*^9, 3.856604118621447*^9}, {
   3.859055241222908*^9, 3.859055241223752*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[131]:=",
 CellID->1766588588,ExpressionUUID->"31d6e3ff-9f0d-48b3-b1bc-35507991ac59"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       SqrtBox["2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607694416*^9},
 CellLabel->"Out[131]=",
 CellID->347258070,ExpressionUUID->"fcd80fa0-9f02-433a-a434-de26fe33977a"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607695255*^9},
 CellLabel->"Out[132]=",
 CellID->1653313591,ExpressionUUID->"25407c94-66e7-4f85-98fb-66618acd8335"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Then full 4 Bell basis states are then",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[": ",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195080892084*^9, 3.6241951065190496`*^9}, {
  3.6241957741917477`*^9, 3.624195798637121*^9}, {3.6242730400993147`*^9, 
  3.62427304550955*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->2092978741,ExpressionUUID->"586bbec2-4eea-4339-b199-179e89f9bd21"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["B", 
    RowBox[{"i_", ",", "  ", "j_"}]], ":=", " ", 
   RowBox[{"Bellop", ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Ket", "[", "i", "]"}], " ", "\[CircleTimes]", " ", 
      RowBox[{"Ket", "[", "j", "]"}]}], ")"}]}]}], ";"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419512417188*^9, 3.624195167554059*^9}, {
   3.624195230688199*^9, 3.624195232477228*^9}, {3.6241954736571827`*^9, 
   3.624195479416016*^9}, {3.624195529965721*^9, 3.62419571270053*^9}, {
   3.624272992075905*^9, 3.624273032662613*^9}, 3.856038421309806*^9, {
   3.8560400871939583`*^9, 3.856040091826727*^9}, {3.858337225631466*^9, 
   3.8583372275895557`*^9}, 3.859055241225212*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[133]:=",
 CellID->911562127,ExpressionUUID->"699ba197-e76c-4207-b34c-959c9a9e941a"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"DForm", "[", 
  SubscriptBox["B", 
   RowBox[{"0", ",", "0"}]], " ", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", 
  SubscriptBox["B", 
   RowBox[{"0", ",", "1"}]], " ", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", 
  SubscriptBox["B", 
   RowBox[{"1", ",", "0"}]], " ", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", 
  SubscriptBox["B", 
   RowBox[{"1", ",", "1"}]], " ", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613473349125198*^9, 3.6134733534557247`*^9}, {
   3.624194357915715*^9, 3.624194537703003*^9}, 3.6241951514884357`*^9, {
   3.6242730542533693`*^9, 3.62427305761736*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[134]:=",
 CellID->476330791,ExpressionUUID->"13a1b7b8-8083-4495-8cc6-69a656a8fb6d"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"00\"", "\" >\"", "\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"11\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260769745817`*^9},
 CellLabel->"Out[134]=",
 CellID->950615404,ExpressionUUID->"9dfd7331-97f9-4c07-9196-09e6245c7827"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"01\"", "\" >\"", "\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"10\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260769838543`*^9},
 CellLabel->"Out[135]=",
 CellID->1862244995,ExpressionUUID->"c3b6817c-942b-42b9-bdaa-27d4f7d2e93f"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"00\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"11\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076991253*^9},
 CellLabel->"Out[136]=",
 CellID->1324959587,ExpressionUUID->"7414f41b-ff15-490b-b8ef-148c72229b76"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"01\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"10\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326076998932*^9},
 CellLabel->"Out[137]=",
 CellID->1337842966,ExpressionUUID->"70a69116-dfba-4a87-84c3-bb99a1e9ad78"]
}, Open  ]],

Cell[TextData[StyleBox[" The following steps illustrate how to convert MM to \
python code, as part of a future plan to build qiskit code for\nsubmission to \
a real quantum computer.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624193746911874*^9, 3.6241938344127073`*^9}, 
   3.856099714400394*^9, {3.860073886020409*^9, 3.8600739877221127`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->1912508314,ExpressionUUID->"90f4dc09-7246-4ae1-93c2-2c7a86e9d0b6"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Needs", "[", "\"\<ToPython`\>\"", "]"}], "  ", 
   RowBox[{"(*", "   ", 
    RowBox[{
     RowBox[{"[", "MIT", "]"}], 
     RowBox[{"(", 
      RowBox[{"LICENSE", ".", "md"}], ")"}], " ", 
     RowBox[{"\[Copyright]", "[", 
      RowBox[{"Gustavo", " ", "Wiederhecker"}], "]"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"https", ":"}], "//", 
       RowBox[{
        RowBox[{"github", ".", "com"}], "/", "gwiederhecker"}]}], ")"}]}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{"?", "ToPython"}]}]}]], "Input",
 CellChangeTimes->{{3.8597995938556747`*^9, 3.8597996819728394`*^9}, {
   3.859799853315681*^9, 3.8597998977442493`*^9}, {3.859800080790205*^9, 
   3.8598001047688293`*^9}, {3.859800261790942*^9, 3.859800329761032*^9}, {
   3.8598003700705633`*^9, 3.859800454140676*^9}, 3.8598005008388433`*^9, 
   3.859800563487658*^9, {3.860073789938233*^9, 3.86007379418896*^9}},
 CellLabel->"In[138]:=",
 CellID->573361425,ExpressionUUID->"77bf8c40-8dc8-4560-98c8-fc8331b16287"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"ToPython[expression,numpystring] converts \
Mathematica expression to a Numpy compatible expression.\\n because Numpy can \
be imported in several ways, numpystring is a string that will be added to \
appended to function names, e.g., Cos->numpy.cos\"\>", "InformationUsageText",
              
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"ToPython", "[", 
                    RowBox[{"Private`x_", ",", 
                    RowBox[{"Private`numpyprefix_", ":", "\"numpy\""}]}], 
                    "]"}], ":=", 
                    RowBox[{"Module", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Private`expression", "=", "Private`x"}], ",", 
                    "Private`greekrule", ",", "Private`PythonForm", ",", 
                    RowBox[{"Private`numpypre", "=", "Private`numpyprefix"}], 
                    ",", "Private`lp", ",", "Private`rp", ",", "Private`a", 
                    ",", "Private`b"}], "}"}], ",", 
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"Private`numpyprefix", "\[Equal]", "\"\""}], ",", 
                    
                    RowBox[{"Private`sep", "=", "\"\""}], ",", 
                    RowBox[{"Private`sep", "=", "\".\""}]}], "]"}], ";", 
                    RowBox[{"Private`lp", "=", "\"( \""}], ";", 
                    RowBox[{"Private`rp", "=", "\" )\""}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{"Rational", "[", 
                    RowBox[{"Private`a_", ",", "Private`b_"}], "]"}], "]"}], 
                    ":=", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Private`a", "]"}], 
                    "<>", "\"/\"", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`b", "]"}]}]}],
                     ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{"Complex", "[", 
                    RowBox[{"Private`a_", ",", "Private`b_"}], "]"}], "]"}], 
                    ":=", 
                    RowBox[{"\"complex\"", "<>", "Private`lp", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`a", "]"}], 
                    "<>", "\",\"", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`b", "]"}], 
                    "<>", "Private`rp"}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{"Private`a_", " ", "Private`b_"}], "]"}], ":=", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Private`a", "]"}], 
                    "<>", "\" * \"", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`b", "]"}]}]}],
                     ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{"Private`a_", "+", "Private`b_"}], "]"}], ":=", 
                    RowBox[{"Private`lp", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`a", "]"}], 
                    "<>", "\" + \"", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`b", "]"}], 
                    "<>", "Private`rp"}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{"Private`h_", "[", "Private`args__", "]"}], "]"}],
                     ":=", 
                    RowBox[{"Private`numpypre", "<>", "Private`sep", "<>", 
                    RowBox[{"ToLowerCase", "[", 
                    RowBox[{"Private`PythonForm", "[", "Private`h", "]"}], 
                    "]"}], "<>", "Private`lp", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`args", "]"}], 
                    "<>", "Private`rp"}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    SuperscriptBox["Private`a_", "Private`b_"], "]"}], ":=", 
                    RowBox[{"Private`lp", "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`a", "]"}], 
                    "<>", "Private`rp", "<>", "\"**\"", "<>", "Private`lp", 
                    "<>", 
                    RowBox[{"Private`PythonForm", "[", "Private`b", "]"}], 
                    "<>", "Private`rp"}]}], ";", 
                    RowBox[{
                    
                    RowBox[{"Private`PythonForm", "[", "Private`a_ListQ", 
                    "]"}], ":=", 
                    
                    RowBox[{"Private`numpypre", "<>", "Private`sep", "<>", 
                    "\"array\"", "<>", 
                    RowBox[{"StringReplace", "[", 
                    RowBox[{
                    RowBox[{"ToString", "[", "Private`a", "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"\"{\"", "\[Rule]", "\"[\""}], ",", 
                    RowBox[{"\"}\"", "\[Rule]", "\"]\""}]}], "}"}]}], 
                    "]"}]}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Arg", "]"}], "=", 
                    
                    RowBox[{"Private`numpypre", "<>", "Private`sep", "<>", 
                    "\"angle\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Csc", "]"}], ":=", 
                    
                    RowBox[{"\"1/\"", "<>", "Private`numpypre", "<>", 
                    "Private`sep", "<>", "\"sin\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Sec", "]"}], ":=", 
                    
                    RowBox[{"\"1/\"", "<>", "Private`numpypre", "<>", 
                    "Private`sep", "<>", "\"cos\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Cot", "]"}], ":=", 
                    
                    RowBox[{"\"1/\"", "<>", "Private`numpypre", "<>", 
                    "Private`sep", "<>", "\"tan\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Csch", "]"}], ":=", 
                    
                    RowBox[{"\"1/\"", "<>", "Private`numpypre", "<>", 
                    "Private`sep", "<>", "\"sinh\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Sech", "]"}], ":=", 
                    RowBox[{"\"1/\"", "<>", "Private`numpypre", "<>", 
                    "Private`sep", "<>", "\"cosh\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "Coth", "]"}], ":=", 
                    
                    RowBox[{"\"1/\"", "<>", "Private`numpypre", "<>", 
                    "Private`sep", "<>", "\"tanh\""}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{"{", "Private`args__", "}"}], "]"}], ":=", 
                    
                    RowBox[{"Private`numpypre", "<>", "Private`sep", "<>", 
                    "\"array\"", "<>", "Private`lp", "<>", "\"[\"", "<>", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", 
                    RowBox[{
                    RowBox[{"{", "Private`args", "}"}], 
                    "\[LeftDoubleBracket]", "Private`ii", 
                    "\[RightDoubleBracket]"}], "]"}], "<>", "\",\""}], ",", 
                    RowBox[{"{", 
                    RowBox[{"Private`ii", ",", "1", ",", 
                    RowBox[{"Length", "[", 
                    RowBox[{"{", "Private`args", "}"}], "]"}]}], "}"}]}], 
                    "]"}], "<>", "\"]\"", "<>", "Private`rp"}]}], ";", 
                    RowBox[{
                    RowBox[{"Private`PythonForm", "[", "\[Pi]", "]"}], "=", 
                    
                    RowBox[{"Private`numpypre", "<>", "Private`sep", "<>", 
                    "\"pi\""}]}], ";", 
                    RowBox[{
                    
                    RowBox[{"Private`PythonForm", "[", "\[ExponentialE]", 
                    "]"}], "=", 
                    
                    RowBox[{"Private`numpypre", "<>", "Private`sep", "<>", 
                    "\"e\""}]}], ";", 
                    RowBox[{
                    
                    RowBox[{"Private`PythonForm", "[", "Private`r_Real", 
                    "]"}], ":=", 
                    RowBox[{"Block", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"Private`a", "=", 
                    RowBox[{"MantissaExponent", "[", "Private`r", "]"}]}], 
                    "}"}], ",", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"Private`r", "\[GreaterEqual]", "0"}], ",", 
                    RowBox[{
                    RowBox[{"ToString", "[", 
                    RowBox[{"N", "[", 
                    RowBox[{
                    
                    RowBox[{"Private`a", "\[LeftDoubleBracket]", "1", 
                    "\[RightDoubleBracket]"}], ",", "6"}], "]"}], "]"}], "<>",
                     "\"e\"", "<>", 
                    RowBox[{"ToString", "[", 
                    
                    RowBox[{"Private`a", "\[LeftDoubleBracket]", "2", 
                    "\[RightDoubleBracket]"}], "]"}]}], ",", 
                    RowBox[{"\"(\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"N", "[", 
                    RowBox[{
                    
                    RowBox[{"Private`a", "\[LeftDoubleBracket]", "1", 
                    "\[RightDoubleBracket]"}], ",", "6"}], "]"}], "]"}], "<>",
                     "\"e\"", "<>", 
                    RowBox[{"ToString", "[", 
                    
                    RowBox[{"Private`a", "\[LeftDoubleBracket]", "2", 
                    "\[RightDoubleBracket]"}], "]"}], "<>", "\")\""}]}], 
                    "]"}]}], "]"}]}], ";", 
                    RowBox[{"Private`greekrule", "=", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"\"\[Alpha]\"", "\[Rule]", "\"alpha\""}], ",", 
                    RowBox[{"\"\[Beta]\"", "\[Rule]", "\"beta\""}], ",", 
                    RowBox[{"\"\[Gamma]\"", "\[Rule]", "\"gamma\""}], ",", 
                    RowBox[{"\"\[Delta]\"", "\[Rule]", "\"delta\""}], ",", 
                    
                    RowBox[{"\"\[CurlyEpsilon]\"", "\[Rule]", 
                    "\"curlyepsilon\""}], ",", 
                    RowBox[{"\"\[Zeta]\"", "\[Rule]", "\"zeta\""}], ",", 
                    RowBox[{"\"\[Eta]\"", "\[Rule]", "\"eta\""}], ",", 
                    RowBox[{"\"\[Theta]\"", "\[Rule]", "\"theta\""}], ",", 
                    RowBox[{"\"\[Iota]\"", "\[Rule]", "\"iota\""}], ",", 
                    RowBox[{"\"\[Kappa]\"", "\[Rule]", "\"kappa\""}], ",", 
                    RowBox[{"\"\[Lambda]\"", "\[Rule]", "\"lambda\""}], ",", 
                    RowBox[{"\"\[Mu]\"", "\[Rule]", "\"mu\""}], ",", 
                    RowBox[{"\"\[Nu]\"", "\[Rule]", "\"nu\""}], ",", 
                    RowBox[{"\"\[Xi]\"", "\[Rule]", "\"xi\""}], ",", 
                    RowBox[{"\"\[Omicron]\"", "\[Rule]", "\"omicron\""}], ",", 
                    RowBox[{"\"\[Pi]\"", "\[Rule]", "\"pi\""}], ",", 
                    RowBox[{"\"\[Rho]\"", "\[Rule]", "\"rho\""}], ",", 
                    
                    RowBox[{"\"\[FinalSigma]\"", "\[Rule]", 
                    "\"finalsigma\""}], ",", 
                    RowBox[{"\"\[Sigma]\"", "\[Rule]", "\"sigma\""}], ",", 
                    RowBox[{"\"\[Tau]\"", "\[Rule]", "\"tau\""}], ",", 
                    RowBox[{"\"\[Upsilon]\"", "\[Rule]", "\"upsilon\""}], ",", 
                    RowBox[{"\"\[CurlyPhi]\"", "\[Rule]", "\"curlyphi\""}], 
                    ",", 
                    RowBox[{"\"\[Chi]\"", "\[Rule]", "\"chi\""}], ",", 
                    RowBox[{"\"\[Psi]\"", "\[Rule]", "\"psi\""}], ",", 
                    RowBox[{"\"\[Omega]\"", "\[Rule]", "\"omega\""}], ",", 
                    RowBox[{"\"\[CapitalAlpha]\"", "\[Rule]", "\"Alpha\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalBeta]\"", "\[Rule]", "\"Beta\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalGamma]\"", "\[Rule]", "\"Gamma\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalDelta]\"", "\[Rule]", "\"Delta\""}], 
                    ",", 
                    
                    RowBox[{"\"\[CapitalEpsilon]\"", "\[Rule]", 
                    "\"CurlyEpsilon\""}], ",", 
                    RowBox[{"\"\[CapitalZeta]\"", "\[Rule]", "\"Zeta\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalEta]\"", "\[Rule]", "\"Eta\""}], ",", 
                    
                    RowBox[{"\"\[CapitalTheta]\"", "\[Rule]", "\"Theta\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalIota]\"", "\[Rule]", "\"Iota\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalKappa]\"", "\[Rule]", "\"Kappa\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalLambda]\"", "\[Rule]", "\"Lambda\""}],
                     ",", 
                    RowBox[{"\"\[CapitalMu]\"", "\[Rule]", "\"Mu\""}], ",", 
                    RowBox[{"\"\[CapitalNu]\"", "\[Rule]", "\"Nu\""}], ",", 
                    RowBox[{"\"\[CapitalXi]\"", "\[Rule]", "\"Xi\""}], ",", 
                    
                    RowBox[{"\"\[CapitalOmicron]\"", "\[Rule]", 
                    "\"Omicron\""}], ",", 
                    RowBox[{"\"\[CapitalPi]\"", "\[Rule]", "\"Pi\""}], ",", 
                    RowBox[{"\"\[CapitalRho]\"", "\[Rule]", "\"Rho\""}], ",", 
                    
                    RowBox[{"\"\[CapitalSigma]\"", "\[Rule]", "\"Sigma\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalTau]\"", "\[Rule]", "\"Tau\""}], ",", 
                    
                    
                    RowBox[{"\"\[CapitalUpsilon]\"", "\[Rule]", 
                    "\"Upsilon\""}], ",", 
                    RowBox[{"\"\[CapitalPhi]\"", "\[Rule]", "\"CurlyPhi\""}], 
                    ",", 
                    RowBox[{"\"\[CapitalChi]\"", "\[Rule]", "\"Chi\""}], ",", 
                    
                    RowBox[{"\"\[CapitalPsi]\"", "\[Rule]", "\"Psi\""}], ",", 
                    
                    RowBox[{"\"\[CapitalOmega]\"", "\[Rule]", "\"Omega\""}]}],
                     "}"}]}], ";", 
                    RowBox[{
                    
                    RowBox[{"Private`PythonForm", "[", "Private`allOther_", 
                    "]"}], ":=", 
                    RowBox[{"StringReplace", "[", 
                    RowBox[{
                    RowBox[{"ToString", "[", 
                    RowBox[{"Private`allOther", ",", "FortranForm"}], "]"}], 
                    ",", "Private`greekrule"}], "]"}]}], ";", 
                    RowBox[{"CopyToClipboard", "[", 
                    RowBox[{"Private`PythonForm", "[", "Private`expression", 
                    "]"}], "]"}], ";", 
                    
                    RowBox[{"Private`PythonForm", "[", "Private`expression", 
                    "]"}]}]}], "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"ToPython`ToPython\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"ToPython[expression,numpystring] converts \
Mathematica expression to a Numpy compatible expression.\\n because Numpy can \
be imported in several ways, numpystring is a string that will be added to \
appended to function names, e.g., Cos->numpy.cos\"\>", "InformationUsageText",
              
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "ToPython[expression,numpystring] converts Mathematica expression to a \
Numpy compatible expression.\n because Numpy can be imported in several ways, \
numpystring is a string that will be added to appended to function names, \
e.g., Cos->numpy.cos", "Documentation" -> None, "OwnValues" -> None, 
     "UpValues" -> None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, ToPython`ToPython, {ToPython`ToPython[
          Pattern[Private`x, 
           Blank[]], 
          Optional[
           Pattern[Private`numpyprefix, 
            Blank[]], "numpy"]] :> 
        Module[{Private`expression = Private`x, Private`greekrule, 
           Private`PythonForm, Private`numpypre = Private`numpyprefix, 
           Private`lp, Private`rp, Private`a, Private`b}, 
          If[Private`numpyprefix == "", Private`sep = "", Private`sep = "."]; 
          Private`lp = "( "; Private`rp = " )"; Private`PythonForm[
             Rational[
              Pattern[Private`a, 
               Blank[]], 
              Pattern[Private`b, 
               Blank[]]]] := 
           Private`PythonForm[Private`a] <> "/" <> 
            Private`PythonForm[Private`b]; Private`PythonForm[
             Complex[
              Pattern[Private`a, 
               Blank[]], 
              Pattern[Private`b, 
               Blank[]]]] := 
           "complex" <> Private`lp <> Private`PythonForm[Private`a] <> "," <> 
            Private`PythonForm[Private`b] <> Private`rp; 
          Private`PythonForm[Pattern[Private`a, 
               Blank[]] Pattern[Private`b, 
               Blank[]]] := 
           Private`PythonForm[Private`a] <> " * " <> 
            Private`PythonForm[Private`b]; 
          Private`PythonForm[Pattern[Private`a, 
               Blank[]] + Pattern[Private`b, 
               Blank[]]] := 
           Private`lp <> Private`PythonForm[Private`a] <> " + " <> 
            Private`PythonForm[Private`b] <> Private`rp; Private`PythonForm[
             Pattern[Private`h, 
              Blank[]][
              Pattern[Private`args, 
               BlankSequence[]]]] := 
           Private`numpypre <> Private`sep <> ToLowerCase[
              Private`PythonForm[Private`h]] <> Private`lp <> 
            Private`PythonForm[Private`args] <> Private`rp; 
          Private`PythonForm[Pattern[Private`a, 
               Blank[]]^Pattern[Private`b, 
               Blank[]]] := 
           Private`lp <> Private`PythonForm[Private`a] <> Private`rp <> "**" <> 
            Private`lp <> Private`PythonForm[Private`b] <> Private`rp; 
          Private`PythonForm[
             Pattern[Private`a, 
              Blank[ListQ]]] := 
           Private`numpypre <> Private`sep <> "array" <> StringReplace[
              ToString[Private`a], {"{" -> "[", "}" -> "]"}]; 
          Private`PythonForm[Arg] = 
           Private`numpypre <> Private`sep <> "angle"; 
          Private`PythonForm[Csc] := 
           "1/" <> Private`numpypre <> Private`sep <> "sin"; 
          Private`PythonForm[Sec] := 
           "1/" <> Private`numpypre <> Private`sep <> "cos"; 
          Private`PythonForm[Cot] := 
           "1/" <> Private`numpypre <> Private`sep <> "tan"; 
          Private`PythonForm[Csch] := 
           "1/" <> Private`numpypre <> Private`sep <> "sinh"; 
          Private`PythonForm[Sech] := 
           "1/" <> Private`numpypre <> Private`sep <> "cosh"; 
          Private`PythonForm[Coth] := 
           "1/" <> Private`numpypre <> Private`sep <> "tanh"; 
          Private`PythonForm[{
              Pattern[Private`args, 
               BlankSequence[]]}] := 
           Private`numpypre <> Private`sep <> "array" <> Private`lp <> "[" <> 
            Table[Private`PythonForm[
                Part[{Private`args}, Private`ii]] <> ",", {Private`ii, 1, 
               Length[{Private`args}]}] <> "]" <> Private`rp; 
          Private`PythonForm[Pi] = Private`numpypre <> Private`sep <> "pi"; 
          Private`PythonForm[E] = Private`numpypre <> Private`sep <> "e"; 
          Private`PythonForm[
             Pattern[Private`r, 
              Blank[Real]]] := 
           Block[{Private`a = MantissaExponent[Private`r]}, 
             If[Private`r >= 0, ToString[
                N[
                 Part[Private`a, 1], 6]] <> "e" <> ToString[
                Part[Private`a, 2]], "(" <> ToString[
                N[
                 Part[Private`a, 1], 6]] <> "e" <> ToString[
                Part[Private`a, 2]] <> ")"]]; 
          Private`greekrule = {
            "\[Alpha]" -> "alpha", "\[Beta]" -> "beta", "\[Gamma]" -> "gamma",
              "\[Delta]" -> "delta", "\[CurlyEpsilon]" -> "curlyepsilon", 
             "\[Zeta]" -> "zeta", "\[Eta]" -> "eta", "\[Theta]" -> "theta", 
             "\[Iota]" -> "iota", "\[Kappa]" -> "kappa", "\[Lambda]" -> 
             "lambda", "\[Mu]" -> "mu", "\[Nu]" -> "nu", "\[Xi]" -> "xi", 
             "\[Omicron]" -> "omicron", "\[Pi]" -> "pi", "\[Rho]" -> "rho", 
             "\[FinalSigma]" -> "finalsigma", "\[Sigma]" -> "sigma", "\[Tau]" -> 
             "tau", "\[Upsilon]" -> "upsilon", "\[CurlyPhi]" -> "curlyphi", 
             "\[Chi]" -> "chi", "\[Psi]" -> "psi", "\[Omega]" -> "omega", 
             "\[CapitalAlpha]" -> "Alpha", "\[CapitalBeta]" -> "Beta", 
             "\[CapitalGamma]" -> "Gamma", "\[CapitalDelta]" -> "Delta", 
             "\[CapitalEpsilon]" -> "CurlyEpsilon", "\[CapitalZeta]" -> 
             "Zeta", "\[CapitalEta]" -> "Eta", "\[CapitalTheta]" -> "Theta", 
             "\[CapitalIota]" -> "Iota", "\[CapitalKappa]" -> "Kappa", 
             "\[CapitalLambda]" -> "Lambda", "\[CapitalMu]" -> "Mu", 
             "\[CapitalNu]" -> "Nu", "\[CapitalXi]" -> "Xi", 
             "\[CapitalOmicron]" -> "Omicron", "\[CapitalPi]" -> "Pi", 
             "\[CapitalRho]" -> "Rho", "\[CapitalSigma]" -> "Sigma", 
             "\[CapitalTau]" -> "Tau", "\[CapitalUpsilon]" -> "Upsilon", 
             "\[CapitalPhi]" -> "CurlyPhi", "\[CapitalChi]" -> "Chi", 
             "\[CapitalPsi]" -> "Psi", "\[CapitalOmega]" -> "Omega"}; 
          Private`PythonForm[
             Pattern[Private`allOther, 
              Blank[]]] := StringReplace[
             ToString[Private`allOther, FortranForm], Private`greekrule]; 
          CopyToClipboard[
            Private`PythonForm[Private`expression]]; 
          Private`PythonForm[Private`expression]]}], "SubValues" -> None, 
     "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "ToPython`ToPython"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077088749*^9},
 CellLabel->"Out[139]=",
 CellID->972945472,ExpressionUUID->"5eb5a8ac-91b7-44c8-8458-486fc09c128e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"expression", "=", 
  RowBox[{"Bellop", ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
     RowBox[{"Ket", "[", "0", "]"}]}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bell00pyt", "=", 
   RowBox[{"ToPython", "[", "expression", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"bell00pyt", "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"Copy", " ", "results", " ", "to", " ", "clipboard"}], "*)"}], 
  "\n", 
  RowBox[{"(*", 
   RowBox[{" ", 
    RowBox[{
     RowBox[{"CopyToClipboard", "[", 
      RowBox[{"PythonForm", "[", "expression", "]"}], "]"}], ";", "\n", 
     RowBox[{"PythonForm", "[", "expression", "]"}]}], "]"}], " ", 
   "*)"}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.859800532196652*^9, 3.859800532670512*^9}, {
  3.859800582496286*^9, 3.859800647207059*^9}, {3.8598190633528433`*^9, 
  3.859819064622966*^9}, {3.8600738356108027`*^9, 3.860073836195674*^9}},
 CellLabel->"In[140]:=",
 CellID->1252668281,ExpressionUUID->"dce8d334-433e-4622-9675-a9c3784e700b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       SqrtBox["2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260771067753`*^9},
 CellLabel->"Out[140]=",
 CellID->1354059646,ExpressionUUID->"11056209-9f8f-4138-b8b3-1a8ab1fc81a1"],

Cell[BoxData[
 TagBox["\<\"numpy.array( [numpy.array( [( 2 )**( -1/2 ),] ),numpy.array( \
[0,] ),numpy.array( [0,] ),numpy.array( [( 2 )**( -1/2 ),] ),] )\"\>",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260772241573`*^9},
 CellLabel->"Out[142]=",
 CellID->796263063,ExpressionUUID->"bd20f09e-b6a0-479b-b2d0-99cce772fb06"]
}, Open  ]],

Cell[TextData[StyleBox["These  Bell states are of great importance in quantum \
circuits. Check, for instance, the Teleportation circuit in the tutorials.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624281767463629*^9, 3.624281781018852*^9}, {
   3.8575712022923326`*^9, 3.857571222632255*^9}, 3.859800543682806*^9, {
   3.860074016657551*^9, 3.860074018851355*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->2020051968,ExpressionUUID->"c2163940-5b4a-416a-b7bd-e1d941569499"],

Cell["Then the density matrices \[Rho]Bell for these Bell states are :", \
"Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241948152094717`*^9, 3.624194829282537*^9}, {
  3.6241949955937233`*^9, 3.624194996684126*^9}, {3.624273433301545*^9, 
  3.6242734347190933`*^9}, {3.7946579116090593`*^9, 3.794657955366804*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->1444450753,ExpressionUUID->"0f36b87d-6a31-4caa-9dfe-915a9b6f1daa"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Rho]Bell", 
   RowBox[{"i_", ",", "j_"}]], ":=", 
  RowBox[{
   SubscriptBox["B", 
    RowBox[{"i", ",", "j"}]], " ", "\[CircleTimes]", " ", 
   RowBox[{"Adj", "[", 
    SubscriptBox["B", 
     RowBox[{"i", ",", "j"}]], "  ", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[Rho]Bell", 
   RowBox[{"0", ",", "0"}]], " "}], "\[IndentingNewLine]", 
 SubscriptBox["\[Rho]Bell", 
  RowBox[{"0", ",", "1"}]], "\[IndentingNewLine]", 
 SubscriptBox["\[Rho]Bell", 
  RowBox[{"1", ",", "0"}]], "\[IndentingNewLine]", 
 SubscriptBox["\[Rho]Bell", 
  RowBox[{"1", ",", "1"}]]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6241949551296053`*^9, 3.624194968930561*^9}, {
   3.62419500210884*^9, 3.624195040706745*^9}, {3.624273148057992*^9, 
   3.6242732355718517`*^9}, {3.624273340973035*^9, 3.6242734027126007`*^9}, {
   3.8560401106962643`*^9, 3.8560401135662403`*^9}, {3.85757365031911*^9, 
   3.857573701262649*^9}, 3.857852995099064*^9, 3.85905524122745*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[143]:=",
 CellID->1728659983,ExpressionUUID->"89e68763-1797-42a4-a602-e5558020733b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {
      FractionBox["1", "2"], "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607724304*^9},
 CellLabel->"Out[144]=",
 CellID->1822695384,ExpressionUUID->"044538dd-93e4-4ce8-b9da-ee5d6bf374bd"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], 
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"], 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077251869*^9},
 CellLabel->"Out[145]=",
 CellID->187094433,ExpressionUUID->"715e23f8-0665-49ca-be89-5da1f539f997"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}]},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077259571*^9},
 CellLabel->"Out[146]=",
 CellID->580076223,ExpressionUUID->"a3237d78-a465-4d81-ad59-dca7a93ccc24"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0"},
     {"0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077266944*^9},
 CellLabel->"Out[147]=",
 CellID->1912806228,ExpressionUUID->"357c5321-e76e-470c-8238-00195e793eb3"]
}, Open  ]],

Cell["\<\
 Bell states have zero  vector polarizations and diagonal tensor polarization.\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241948152094717`*^9, 3.624194829282537*^9}, {
  3.6241949955937233`*^9, 3.624194996684126*^9}, {3.624273433301545*^9, 
  3.6242734347190933`*^9}, {3.7946579116090593`*^9, 3.794657955366804*^9}, {
  3.857633269037065*^9, 3.857633336088196*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->2031050894,ExpressionUUID->"3d7e8883-1d90-4a2e-9c6d-a2f75115bdd9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"SpinObs", "[", 
    SubscriptBox["\[Rho]Bell", 
     RowBox[{"0", ",", "0"}]], "]"}], " ", "\[IndentingNewLine]", " ", 
   RowBox[{"SpinObs", "[", 
    SubscriptBox["\[Rho]Bell", 
     RowBox[{"0", ",", "1"}]], "]"}], "\[IndentingNewLine]", " ", 
   RowBox[{"SpinObs", "[", 
    SubscriptBox["\[Rho]Bell", 
     RowBox[{"1", ",", "0"}]], "]"}], "\[IndentingNewLine]", " ", 
   RowBox[{"SpinObs", "[", 
    SubscriptBox["\[Rho]Bell", 
     RowBox[{"1", ",", "1"}]], "]"}], " ", "\[IndentingNewLine]"}]}]], "Input",\

 TaggingRules->{},
 CellChangeTimes->{{3.624273682662374*^9, 3.624273711156938*^9}, 
   3.624278771795455*^9, 3.624278958189372*^9, {3.62427914402831*^9, 
   3.624279169542389*^9}, {3.8560401225649433`*^9, 3.856040153166127*^9}, {
   3.857853109185813*^9, 3.857853151424828*^9}, 3.858172590785638*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[148]:=",
 CellID->282074018,ExpressionUUID->"fd7c5acb-e4e5-42db-a281-b3144d2afbc1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"1", "0", "0"},
            {"0", 
             RowBox[{"-", "1"}], "0"},
            {"0", "0", "1"}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607728093*^9},
 CellLabel->"Out[148]=",
 CellID->704078406,ExpressionUUID->"dc1ad17e-7f98-4adc-9c54-fc628bed2c14"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"1", "0", "0"},
            {"0", "1", "0"},
            {"0", "0", 
             RowBox[{"-", "1"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077290823*^9},
 CellLabel->"Out[149]=",
 CellID->810874672,ExpressionUUID->"28fa9ef4-4172-49d9-ad87-3d74627bc332"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {
             RowBox[{"-", "1"}], "0", "0"},
            {"0", "1", "0"},
            {"0", "0", "1"}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607729911*^9},
 CellLabel->"Out[150]=",
 CellID->439388442,ExpressionUUID->"493b10bf-db0a-40e8-8040-bc415fb9ca10"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {
             RowBox[{"-", "1"}], "0", "0"},
            {"0", 
             RowBox[{"-", "1"}], "0"},
            {"0", "0", 
             RowBox[{"-", "1"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077307159*^9},
 CellLabel->"Out[151]=",
 CellID->1752766308,ExpressionUUID->"a0df5d01-5585-4a07-a33b-9a89d208fd2b"]
}, Open  ]],

Cell[TextData[StyleBox["Bell states are the prime example of an entangled \
state and are often used to gauge a states entanglement.  Entanglement is a \
correlation between spins that can not be produced by a product, a mixed or \
any classical procedure.  It is a correlation that arises only in the quantum \
theory.  It is a key resource for quantum computing.  See the tutorial on \
Bell states.",
 FontFamily->"Times New Roman",
 FontWeight->"Regular"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.857853187198538*^9, {3.857853218537816*^9, 3.857853403096965*^9}, {
   3.8588854226947393`*^9, 3.858885438501422*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1681331124,ExpressionUUID->"78c087b7-43b4-4de9-83af-9d870c1193ba"],

Cell[TextData[{
 StyleBox["GHZ",
  FontSize->36,
  FontColor->RGBColor[0, 0, 1]],
 StyleBox[" states",
  FontSize->36,
  FontColor->RGBColor[0, 0, 1]]
}], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->725256518,ExpressionUUID->"994af28d-ec69-4ba1-9f7d-a6b62f61e797"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->269117654,ExpressionUUID->"40d98d31-4e65-47a8-9cbe-fd0149e7b59e"],

Cell[TextData[StyleBox["Another type of entangled states are the GHZ states \
which can be prepared by having the  following operator act on a state with \
three qubits :",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241946940899677`*^9, 3.624194777711978*^9}, {
  3.857853423606035*^9, 3.85785348889071*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->902749858,ExpressionUUID->"033c8045-1fa5-4c46-a41c-f1fdbaa0d5d0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"CNOT", "[", 
  RowBox[{"3", ",", "1", ",", "3"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"CNOT", "[", 
  RowBox[{"3", ",", "1", ",", "2"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"had", "[", 
  RowBox[{"3", ",", "1"}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613723188502368*^9, 3.6137231992058477`*^9}, {
  3.624194606284298*^9, 3.6241946204709253`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[152]:=",
 CellID->544897351,ExpressionUUID->"a7e1553a-8d69-4fa2-ab2a-83cb15a79cb7"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "1", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "1", "0", "0"},
     {"0", "0", "0", "0", "1", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "1"},
     {"0", "0", "0", "0", "0", "0", "1", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077325993*^9},
 CellLabel->"Out[152]=",
 CellID->1710152657,ExpressionUUID->"f9da1787-1aa5-4fbe-8f4f-b90598468682"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "1", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "1", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "1"},
     {"0", "0", "0", "0", "1", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "1", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260773345547`*^9},
 CellLabel->"Out[153]=",
 CellID->614574751,ExpressionUUID->"b63fa631-731c-4c72-b50a-42db6c061c6b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0"},
     {"0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0", "0", "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0", "0"},
     {"0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"},
     {"0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077342433*^9},
 CellLabel->"Out[154]=",
 CellID->1908981634,ExpressionUUID->"44daae2e-6cdb-4c7f-8650-42dc03187e1b"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GHZop", "=", 
    RowBox[{
     RowBox[{"CNOT", "[", 
      RowBox[{"3", ",", "1", ",", "3"}], "]"}], ".", 
     RowBox[{"CNOT", "[", 
      RowBox[{"3", ",", "1", ",", "2"}], "]"}], ".", 
     RowBox[{"had", "[", 
      RowBox[{"3", ",", "1"}], "]"}]}]}], ";"}], 
  "\[IndentingNewLine]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.61347991741864*^9, 3.613479923268484*^9}, {
  3.624195195925902*^9, 3.624195227999786*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellLabel->"In[155]:=",
 CellID->1219507449,ExpressionUUID->"6e3c251c-2ba8-4b44-a379-a3e5e8101f7b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"matGHZ", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"H", ",", "1", ",", "1"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"C", ",", "1", ",", "CX"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"C", ",", "CX", ",", "1"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"GHZC", "=", 
  RowBox[{"Circuit", "[", 
   RowBox[{"matGHZ", ",", 
    RowBox[{"{", "}"}]}], "]"}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856105324224328*^9, 3.8561053466005287`*^9}, {
  3.856105377716186*^9, 3.856105378907476*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[156]:=",
 CellID->1327906532,ExpressionUUID->"83985a01-d559-437b-ba0f-cff5894cb5e4"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.6000000000000001}]}, 
    {GrayLevel[0], Thickness[0.007], LineBox[{{0, 0.}, {1., 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {1., -0.30000000000000004`}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.6000000000000001}, {1., -0.6000000000000001}}]}, 
    {EdgeForm[GrayLevel[0]], FaceForm[RGBColor[0.87, 0.94, 1]], 
     RectangleBox[{0.1, -0.1}, {0.30000000000000004, 0.1}], InsetBox[
      StyleBox["H",
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.2, 0.}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.8, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.8, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.8, 0.}, {0.8, -0.30000000000000004`}}]}}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.6000000000000001}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.6000000000000001}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, -0.6000000000000001}]}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077370947*^9},
 CellLabel->"Out[157]=",
 CellID->848636458,ExpressionUUID->"6b58454a-4864-4179-a452-4c555a8c01df"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{"\"\<With measurement of qubit 2\>\"", "\[IndentingNewLine]", 
 RowBox[{"Circuit", "[", 
  RowBox[{
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"H", "C", "C"},
      {"1", "1", "CX"},
      {"1", "CX", "1"}
     },
     GridBoxDividers->{"Columns" -> {{True}}, "Rows" -> {{True}}}], 
    "\[NoBreak]", ")"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856105416888014*^9, 3.856105427452381*^9}, {
  3.856105478907914*^9, 3.8561055380413103`*^9}, {3.856105950068969*^9, 
  3.8561059508782463`*^9}, {3.856106192570197*^9, 3.856106194062354*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[158]:=",
 CellID->232906771,ExpressionUUID->"156ed9f6-f736-4e16-9da4-a5f5b3b6f9b2"],

Cell[BoxData[
 TagBox["\<\"With measurement of qubit 2\"\>",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260773995037`*^9},
 CellLabel->"Out[158]=",
 CellID->1594049879,ExpressionUUID->"ddc663fb-5548-4e38-a0fa-0f91150b4057"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.6000000000000001}]}, 
    {GrayLevel[0], Thickness[0.007], LineBox[{{0, 0.}, {1., 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {1., -0.30000000000000004`}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.6000000000000001}, {1., -0.6000000000000001}}]}, 
    {EdgeForm[GrayLevel[0]], FaceForm[RGBColor[0.87, 0.94, 1]], 
     RectangleBox[{0.1, -0.1}, {0.30000000000000004, 0.1}], InsetBox[
      StyleBox["H",
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.2, 0.}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.8, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.8, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.8, 0.}, {0.8, -0.30000000000000004`}}]}}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.6000000000000001}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.6000000000000001}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, -0.6000000000000001}]}, {
     {RGBColor[1, 0, 0], Thickness[Large], 
      CircleBox[{1., -0.30000000000000004}, 0.1, 
       NCache[{(-90) Degree, 90 Degree}, {-1.5707963267948966`, 
        1.5707963267948966`}]]}, 
     {RGBColor[1, 0, 0], Thickness[Large], 
      LineBox[{{1., -0.4}, {1., -0.20000000000000004`}}]}, 
     {RGBColor[1, 0, 0], Thickness[Large], 
      ArrowBox[{{1., -0.30000000000000004`}, {1.1, -0.20000000000000004`}}], 
      {RGBColor[1, 0, 0], Dashing[{0, Small}], 
       LineBox[{{0.9000000000000001, -0.30000000000000004`}, {
        1., -0.30000000000000004`}}]}}}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260774094267`*^9},
 CellLabel->"Out[159]=",
 CellID->1337820028,ExpressionUUID->"ea964b46-b008-4840-8951-0bb7b119d13f"]
}, Open  ]],

Cell["Then the  basic GHZ state  is:", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195080892084*^9, 3.6241951065190496`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->677768375,ExpressionUUID->"721cafa1-d38e-4a90-bf2a-75998990bd06"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"GHZ", "=", 
    RowBox[{"GHZop", ".", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
       RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", " ", 
       RowBox[{"Ket", "[", "0", "]"}]}], ")"}]}]}], ";"}],
  FontSize->18], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", "%", "]"}],
  FontSize->18]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419512417188*^9, 3.624195167554059*^9}, {
  3.624195230688199*^9, 3.624195232477228*^9}, {3.8560384213135567`*^9, 
  3.856038421315383*^9}, {3.856040172582754*^9, 3.856040181188549*^9}, {
  3.859055241229604*^9, 3.859055241230906*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[160]:=",
 CellID->999814937,ExpressionUUID->"4077cc5d-0b4e-4793-8ba6-379c308ce269"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"000\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"111\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077430913*^9},
 CellLabel->"Out[161]=",
 CellID->587426506,ExpressionUUID->"69fa777b-5954-4f9f-886a-9e4a29682a7d"]
}, Open  ]],

Cell[TextData[StyleBox["The density matrix for this GHZ state  is:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.6241948152094717`*^9, 3.624194829282537*^9}, {
   3.6241949955937233`*^9, 3.624194996684126*^9}, {3.624195315479396*^9, 
   3.624195332230014*^9}, {3.624195418856732*^9, 3.62419542143207*^9}, 
   3.856040264057281*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->1977592031,ExpressionUUID->"d1350ba3-91b3-4902-944c-5083a05378cf"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Rho]GHZ", "=", 
  RowBox[{"GHZ", " ", "\[CircleTimes]", " ", 
   RowBox[{"Adj", "[", "GHZ", "  ", "]"}]}]}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"Tr", "[", "\[Rho]GHZ", "]"}], "==", 
   RowBox[{"Tr", "[", 
    RowBox[{"\[Rho]GHZ", ".", "\[Rho]GHZ"}], "]"}], "\[Equal]", "1"}],
  FontSize->18]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6241949551296053`*^9, 3.624194968930561*^9}, {
   3.62419500210884*^9, 3.624195040706745*^9}, {3.624195338949274*^9, 
   3.6241954092154627`*^9}, {3.8560402880547333`*^9, 3.856040291689973*^9}, {
   3.857573804550894*^9, 3.857573825520142*^9}, 3.8581777284659557`*^9, 
   3.859055241232731*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[162]:=",
 CellID->784860398,ExpressionUUID->"bfb58204-be27-4674-a23c-a0b24be20f62"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {
      FractionBox["1", "2"], "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077452053*^9},
 CellLabel->"Out[162]=",
 CellID->894961083,ExpressionUUID->"39e3bd89-98b7-439e-bbdd-c9bd5fa04208"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260774607563`*^9},
 CellLabel->"Out[163]=",
 CellID->635458393,ExpressionUUID->"8199e7a2-3c98-4089-a0f1-a377b8521bf1"]
}, Open  ]],

Cell["\<\
Which is mathematically equivalent to building the state and then its density \
matrix  as:\
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->1304149312,ExpressionUUID->"f8bab412-229a-4874-9747-da15022d46be"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"GHZvec", "=", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       TemplateBox[{"0"},
        "Ket"], "\[CircleTimes]", 
       TemplateBox[{"0"},
        "Ket"]}], ")"}], "\[CircleTimes]", 
     TemplateBox[{"0"},
      "Ket"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       TemplateBox[{"1"},
        "Ket"], "\[CircleTimes]", 
       TemplateBox[{"1"},
        "Ket"]}], ")"}], "\[CircleTimes]", 
     TemplateBox[{"1"},
      "Ket"]}]}], 
   SqrtBox["2"]]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\n", 
 RowBox[{
  RowBox[{"\[Rho]GHZvec", "=", 
   RowBox[{"GHZvec", "\[CircleTimes]", 
    RowBox[{"Adj", "[", "GHZvec", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"%", "-", "\[Rho]GHZ"}], " ", "//", "FullSimplify"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"%", "==", 
  RowBox[{"ZeroMatrix", "[", 
   RowBox[{"2", "^", "3"}], "]"}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613479684446885*^9, {3.613480129298129*^9, 3.6134801392256603`*^9}, 
   3.613504219535428*^9, {3.613504257019457*^9, 3.613504300835391*^9}, {
   3.624189857781704*^9, 3.6241898588583593`*^9}, {3.856038421320249*^9, 
   3.8560384213319883`*^9}, {3.856040308079253*^9, 3.856040363356257*^9}, {
   3.857573909061116*^9, 3.85757391614738*^9}, {3.8575739479417152`*^9, 
   3.857574017580042*^9}, {3.859055241237576*^9, 3.859055241248374*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellLabel->"In[164]:=",
 CellID->1619385502,ExpressionUUID->"b07206fb-b895-4bbd-8453-214c35bab236"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0"},
     {"0"},
     {"0"},
     {"0"},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       SqrtBox["2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260774849253`*^9},
 CellTags->"b:5.1.2",
 CellLabel->"Out[164]=",
 CellID->713034620,ExpressionUUID->"0b0de7c8-edc7-4bce-92bb-bc026e7a75b9"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"000\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"111\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077495514*^9},
 CellTags->"b:5.1.2",
 CellLabel->"Out[165]=",
 CellID->1014108080,ExpressionUUID->"bfb5d2f2-01ea-439b-b9da-f0714fa262ff"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077504408*^9},
 CellTags->"b:5.1.2",
 CellLabel->"Out[168]=",
 CellID->1684277968,ExpressionUUID->"b5284a9b-acb3-4679-bea5-5509770ca6b4"]
}, Open  ]],

Cell["Then GHZ observables for this case are then: ", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195080892084*^9, 3.6241951065190496`*^9}, {
  3.6241957741917477`*^9, 3.624195798637121*^9}, {3.8576335352385483`*^9, 
  3.85763355183296*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->533165535,ExpressionUUID->"3dd8387d-d3b5-4294-bb31-fca365b6fb0b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SpinObs3", "[", 
   RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"Tr", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", 
      SubscriptBox["\[Sigma]", "j"], "\[CircleTimes]", 
      SubscriptBox["\[Sigma]", "k"]}], ")"}], ".", "\[Rho]GHZ"}], " ", 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"PA", "=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SpinObs3", "[", 
      RowBox[{"1", ",", "0", ",", "0"}], "]"}], ",", 
     RowBox[{"SpinObs3", "[", 
      RowBox[{"2", ",", "0", ",", "0"}], "]"}], ",", 
     RowBox[{"SpinObs3", "[", 
      RowBox[{"3", ",", "0", ",", "0"}], "]"}]}], "}"}], "==", 
   RowBox[{"ZeroT", "[", "3", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"PB", "=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SpinObs3", "[", 
      RowBox[{"0", ",", "1", ",", "0"}], "]"}], ",", 
     RowBox[{"SpinObs3", "[", 
      RowBox[{"0", ",", "2", ",", "0"}], "]"}], ",", 
     RowBox[{"SpinObs3", "[", 
      RowBox[{"0", ",", "3", ",", "0"}], "]"}]}], "}"}], "==", 
   RowBox[{"ZeroT", "[", "3", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"PC", "=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"SpinObs3", "[", 
      RowBox[{"0", ",", "1", ",", "0"}], "]"}], ",", 
     RowBox[{"SpinObs3", "[", 
      RowBox[{"0", ",", "2", ",", "0"}], "]"}], ",", 
     RowBox[{"SpinObs3", "[", 
      RowBox[{"0", ",", "3", ",", "0"}], "]"}]}], "}"}], "==", 
   RowBox[{"ZeroT", "[", "3", "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857633568214488*^9, 3.8576335723648767`*^9}, {
   3.85763367522901*^9, 3.857634143491157*^9}, 3.8576343326397448`*^9, {
   3.8590552412508583`*^9, 3.8590552412522573`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[169]:=",
 CellID->1406836767,ExpressionUUID->"108bfd4c-f0d3-431b-a8b4-71f5699e72e9"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077517788*^9},
 CellLabel->"Out[170]=",
 CellID->1677857069,ExpressionUUID->"31450e9b-1167-43f9-b317-6d07e68cb7d9"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077526101*^9},
 CellLabel->"Out[171]=",
 CellID->1135989926,ExpressionUUID->"c85a2181-020c-428a-9616-da3d8fc3ad53"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077533105*^9},
 CellLabel->"Out[172]=",
 CellID->1453064452,ExpressionUUID->"3207b22a-60e0-4bd2-a7c0-b49dd55a628a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox[
  RowBox[{"Dimensions", "[", "\[Rho]GHZ", "]"}],
  FontFamily->"Helvetica",
  FontSize->18]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857854156376102*^9, 3.857854156377933*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[173]:=",
 CellID->77259091,ExpressionUUID->"5426402a-73a9-4c61-a6a2-9cb021098c6f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"8"},
      {"8"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077552422*^9},
 CellLabel->"Out[173]=",
 CellID->470211080,ExpressionUUID->"a34977fd-67aa-498d-9733-a938e39b298e"]
}, Open  ]],

Cell[TextData[{
 "For three qubits, we have nine polarization observables  and 27 double spin \
correlations and then  27  triple spin correlations. For a total of 63 \
observables.   (For ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "q"], TraditionalForm]],ExpressionUUID->
  "16150ddb-2f3e-4ef8-bc4d-69fbec0c67ec"],
 "=4, we get     ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    SubscriptBox["n", "q"]], TraditionalForm]],ExpressionUUID->
  "1ca3adac-2dad-4420-aa51-253eb7f6560c"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    SubscriptBox["n", "q"]], TraditionalForm]],ExpressionUUID->
  "0ea507ed-ce56-4cd5-8a59-ba8fe50b4be1"],
 " - 1= 255 observables.)"
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195080892084*^9, 3.6241951065190496`*^9}, {
   3.6241957741917477`*^9, 3.624195798637121*^9}, {3.8576335352385483`*^9, 
   3.85763355183296*^9}, {3.857634357962928*^9, 3.857634485731337*^9}, 
   3.8578536183802032`*^9, {3.857853654222438*^9, 3.8578536826638727`*^9}, {
   3.857854077116995*^9, 3.857854100663566*^9}, {3.857854151700807*^9, 
   3.857854190226219*^9}, {3.858177739781546*^9, 3.858177740680985*^9}, {
   3.858177842832862*^9, 3.858177853126814*^9}, {3.8588854974072857`*^9, 
   3.858885649713522*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->776787928,ExpressionUUID->"20803f82-634d-4746-aced-32444db20b1b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"T12", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"SpinObs3", "[", 
     RowBox[{"i", ",", "j", ",", "0"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"T23", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"SpinObs3", "[", 
     RowBox[{"0", ",", "i", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"T13", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"SpinObs3", "[", 
     RowBox[{"i", ",", "0", ",", "j"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], "]"}]}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[174]:=",
 CellID->1719388873,ExpressionUUID->"91d7c117-cef8-4e72-8e49-f77d3cf003b4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077567288*^9},
 CellLabel->"Out[174]=",
 CellID->97800600,ExpressionUUID->"aa2733de-6743-4601-9868-eb06d8d6fc7b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077576765*^9},
 CellLabel->"Out[175]=",
 CellID->909694245,ExpressionUUID->"aac8a4cc-0168-451e-aefe-53c37bc9485f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077584565*^9},
 CellLabel->"Out[176]=",
 CellID->1295092722,ExpressionUUID->"0f401fa5-b213-469d-a733-47f1c09338bc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"SpinObs3", "[", 
    RowBox[{"i", ",", "j", ",", "1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"SpinObs3", "[", 
    RowBox[{"i", ",", "j", ",", "2"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"SpinObs3", "[", 
    RowBox[{"i", ",", "j", ",", "3"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857634199145673*^9, 3.8576342478244*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[177]:=",
 CellID->193715260,ExpressionUUID->"e6a2fd97-cabd-49a7-b59f-3d34a73f7a39"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607760452*^9},
 CellLabel->"Out[177]=",
 CellID->1876616309,ExpressionUUID->"a143ede5-30a1-4038-8b6f-73aa59ce4282"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", 
      RowBox[{"-", "1"}], "0"},
     {
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077613864*^9},
 CellLabel->"Out[178]=",
 CellID->197981874,ExpressionUUID->"b6bd21aa-b9f8-4035-b1c4-46a9d1fc3dfa"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260776217117`*^9},
 CellLabel->"Out[179]=",
 CellID->1029187115,ExpressionUUID->"13da467b-1f9f-4fdd-ba39-6f79713a30f0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"SpinObs3", "[", 
    RowBox[{"i", ",", "j", ",", "k"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "1", ",", "3"}], "}"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8576342826212587`*^9, 3.8576342994065228`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[180]:=",
 CellID->901706553,ExpressionUUID->"d85bbf77-501e-4891-9a18-7e2b94c901ba"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"1"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {
           RowBox[{"-", "1"}]},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {
           RowBox[{"-", "1"}]},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {
           RowBox[{"-", "1"}]},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}]},
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077675727*^9},
 CellLabel->"Out[180]=",
 CellID->1714548867,ExpressionUUID->"b706d8ec-7a1d-4e75-bf98-40c52033c651"]
}, Open  ]],

Cell["The 8  distinct GHZ basis states are then: ", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195080892084*^9, 3.6241951065190496`*^9}, {
  3.6241957741917477`*^9, 3.624195798637121*^9}, {3.858177868955668*^9, 
  3.858177893278414*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.2",
 CellID->1951873683,ExpressionUUID->"7afe57c2-2263-4464-be38-77a4fbf81fab"],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"GHZ2", "[", 
     RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], ":=", 
    RowBox[{"GHZop", ".", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Ket", "[", "i", "]"}], " ", "\[CircleTimes]", " ", 
       RowBox[{"Ket", "[", "j", "]"}], " ", "\[CircleTimes]", " ", 
       RowBox[{"Ket", "[", "k", "]"}]}], ")"}]}]}], ";"}],
  FontSize->16]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419512417188*^9, 3.624195167554059*^9}, {
  3.624195230688199*^9, 3.624195232477228*^9}, {3.6241954736571827`*^9, 
  3.624195479416016*^9}, {3.624195529965721*^9, 3.62419571270053*^9}, {
  3.8560384213344793`*^9, 3.8560384213360367`*^9}, {3.856040381959568*^9, 
  3.856040407541718*^9}, {3.8590552412540092`*^9, 3.859055241255355*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[181]:=",
 CellID->180798609,ExpressionUUID->"731c3364-363a-41bc-924d-47bd8321f2bd"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   StyleBox["DForm",
    FontSize->16], 
   StyleBox["[",
    FontSize->16], 
   RowBox[{"GHZ2", "[", 
    RowBox[{"0", ",", "0", ",", "0"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   StyleBox["DForm",
    FontSize->16], 
   StyleBox["[",
    FontSize->16], 
   RowBox[{"GHZ2", "[", 
    RowBox[{"0", ",", "0", ",", "1"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   StyleBox["DForm",
    FontSize->16], 
   StyleBox["[",
    FontSize->16], 
   RowBox[{"GHZ2", "[", 
    RowBox[{"0", ",", "1", ",", "0"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"GHZ2", "[", 
    RowBox[{"0", ",", "1", ",", "1"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"GHZ2", "[", 
    RowBox[{"1", ",", "0", ",", "0"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"GHZ2", "[", 
    RowBox[{"1", ",", "0", ",", "1"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"GHZ2", "[", 
    RowBox[{"1", ",", "1", ",", "0"}], "]"}], "]"}],
  FontSize->16], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"DForm", "[", 
   RowBox[{"GHZ2", "[", 
    RowBox[{"1", ",", "1", ",", "1"}], "]"}], "]"}],
  FontSize->16]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624195729281499*^9, 3.62419574397359*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[182]:=",
 CellID->2140344605,ExpressionUUID->"c3d52d2e-d26e-433e-a9e6-290cf9d9f7b6"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"000\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"111\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077703452*^9},
 CellLabel->"Out[182]=",
 CellID->1487232223,ExpressionUUID->"2014c594-055a-497c-8819-4c4e655228c5"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"001\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"110\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077713369*^9},
 CellLabel->"Out[183]=",
 CellID->1349804528,ExpressionUUID->"a67e8792-0c32-4172-8090-5648012e9a19"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"010\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"101\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077722069*^9},
 CellLabel->"Out[184]=",
 CellID->147455188,ExpressionUUID->"7d1c1e2a-85a6-4139-887e-5c827f430d09"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"011\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"100\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077730568*^9},
 CellLabel->"Out[185]=",
 CellID->83216078,ExpressionUUID->"8c7016c5-a50e-4081-bc89-1f4cc226ba1f"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"000\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"111\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077739418*^9},
 CellLabel->"Out[186]=",
 CellID->600768749,ExpressionUUID->"aabe6975-967b-4299-8d44-f6b945280e89"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"001\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"110\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077748053*^9},
 CellLabel->"Out[187]=",
 CellID->1799739896,ExpressionUUID->"145e1ea6-863e-4b76-b7c2-6395c978b988"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"010\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"101\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260777566423`*^9},
 CellLabel->"Out[188]=",
 CellID->921264077,ExpressionUUID->"40a5361f-592f-46ea-b49b-6eee9e5d5959"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"011\"", "\" >\"", "\" + \"", 
    "\"(\"", 
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"100\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260777650137`*^9},
 CellLabel->"Out[189]=",
 CellID->328247269,ExpressionUUID->"d8ce2b70-fb92-4caa-8eb0-37eea8065406"]
}, Open  ]],

Cell[TextData[StyleBox["Subsystems, Partial Traces",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, {3.859053944287949*^9, 3.859053951823889*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->459850196,ExpressionUUID->"11c5b36d-574f-4717-a142-39d08637e107"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2056082266,ExpressionUUID->"6c89412c-4f3c-42a9-ba55-6fbee5f2adae"],

Cell[TextData[{
 StyleBox[" ",
  FontFamily->"Helvetica"],
 "The density matrix of a specific subsystem can be computed easily by taking \
partial traces of the density matrix of the full system, e.g. we will build \
the 4 qubit state |0111\[RightAngleBracket] and then recover the state of \
either the subsystem (1) or the subsystems (24).  For example, starting with \
a four qubit state |0111\[RightAngleBracket], its density matrix is \
constructed and then partial traces taken:"
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellID->442078480,ExpressionUUID->"8da6c954-81d1-4f7c-8a2d-bb93e4a1b939"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Rho]", "1234"], "=", 
  RowBox[{
   SubscriptBox["\[ScriptCapitalP]", 
    StyleBox["0",
     ScriptLevel->0]], "\[CircleTimes]", 
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", 
     StyleBox["1",
      ScriptLevel->0]],
    ScriptLevel->0], 
   StyleBox["\[CircleTimes]",
    ScriptLevel->0], 
   StyleBox[" ",
    ScriptLevel->0], 
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", 
     StyleBox[
      RowBox[{"1", " "}],
      ScriptLevel->0]],
    ScriptLevel->0], 
   StyleBox["\[CircleTimes]",
    ScriptLevel->0], 
   StyleBox[" ",
    ScriptLevel->0], 
   StyleBox[
    SubscriptBox["\[ScriptCapitalP]", 
     StyleBox["1",
      ScriptLevel->0]],
    ScriptLevel->0]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox[
    StyleBox["\[Rho]",
     ScriptLevel->0], "1"], "=", 
   RowBox[{"PTr", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "3", ",", "4"}], "}"}], ",", 
     SubscriptBox["\[Rho]", "1234"]}], "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[Rho]", "12"], "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"3", ",", "4"}], "}"}], ",", 
    SubscriptBox["\[Rho]", "1234"]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SpinObs", "[", 
   SubscriptBox["\[Rho]", "12"], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["\[Rho]", "24"], "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "3"}], "}"}], ",", 
    SubscriptBox["\[Rho]", "1234"]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SpinObs", "[", 
  SubscriptBox["\[Rho]", "24"], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613504320353313*^9, 3.613504340252046*^9}, {
  3.613504372549904*^9, 3.613504404430119*^9}, {3.6241898726993313`*^9, 
  3.624189875722471*^9}, {3.6242792238477697`*^9, 3.624279240667081*^9}, {
  3.62427966534685*^9, 3.624279685247588*^9}, {3.856038421341392*^9, 
  3.856038421345612*^9}, {3.8560404313716516`*^9, 3.856040436840063*^9}, {
  3.857574047971779*^9, 3.857574050929929*^9}, {3.8578543884101553`*^9, 
  3.857854401734236*^9}, {3.858177962862535*^9, 3.85817796993782*^9}, {
  3.8590552412578583`*^9, 3.8590552412609587`*^9}, {3.859127657197921*^9, 
  3.859127667468857*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[190]:=",
 CellID->1867078387,ExpressionUUID->"d4d8d20a-1e3f-413b-b85f-aa6e7dff1774"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "1", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077780054*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[190]=",
 CellID->2113844587,ExpressionUUID->"52624d07-f318-4354-82f1-6c2ef6cdc781"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077790266*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[191]=",
 CellID->1564001388,ExpressionUUID->"e6ebfea7-49a0-4f9d-90e2-fa6b1cd53162"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077798525*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[192]=",
 CellID->971488836,ExpressionUUID->"7421b217-191c-4bdb-a5f0-803cc6ee4c23"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"1"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", 
             RowBox[{"-", "1"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260778069487`*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[193]=",
 CellID->494698511,ExpressionUUID->"e8280fcc-fc7b-45e8-a4f0-e1eea23fb5db"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077816103*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[194]=",
 CellID->1488676840,ExpressionUUID->"0231a761-c77f-4f65-9a03-681557d99e56"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", "1"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", "1"}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077826519*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[195]=",
 CellID->2007162330,ExpressionUUID->"0f0e7297-493e-41fb-810d-40dba7d73559"]
}, Open  ]],

Cell["\<\
 We can check basic properties of these density matrices, such as their \
length, trace and Purity:\
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellID->955400345,ExpressionUUID->"b29baa0e-fe1c-48bd-88e0-374783fa36ab"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Length", "[", 
  SubscriptBox["\[Rho]", "1234"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Tr", "[", 
  SubscriptBox["\[Rho]", "1234"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", 
  SubscriptBox["\[Rho]", "1234"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Length", "[", 
  SubscriptBox["\[Rho]", "24"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Tr", "[", 
  SubscriptBox["\[Rho]", "24"], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[196]:=",
 CellID->678369796,ExpressionUUID->"77c15bce-2531-4060-9197-2399d7284025"],

Cell[BoxData[
 TagBox["16",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260778491993`*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[196]=",
 CellID->1704153234,ExpressionUUID->"d709897f-1571-436d-acf3-810a4d442fbd"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260778585157`*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[197]=",
 CellID->714003799,ExpressionUUID->"ad6925ed-128c-47a3-8994-7714ff181a17"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077866274*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[198]=",
 CellID->123230357,ExpressionUUID->"8f1b77b0-072d-47f6-b870-4c42917c1a98"],

Cell[BoxData[
 TagBox["4",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077874338*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[199]=",
 CellID->958374391,ExpressionUUID->"4eda8a4a-3c8c-46f4-b912-a57caff2ab02"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607788197*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[200]=",
 CellID->1271037037,ExpressionUUID->"ffef8353-cebd-4bc5-931c-9efb271e5c90"]
}, Open  ]],

Cell["\<\
 Similarly we can produce a more complicated density matrix that is not pure,\
\
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellID->370479449,ExpressionUUID->"70d28756-e093-4be9-b8f1-b392682f057c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a123", "=", 
  RowBox[{
   RowBox[{"1", "/", "3"}], "*", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      StyleBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[ScriptCapitalP]", 
           StyleBox["0",
            ScriptLevel->0]], "\[CircleTimes]", 
          SubscriptBox["\[ScriptCapitalP]", 
           StyleBox["1",
            ScriptLevel->0]]}], ")"}], "\[CircleTimes]", 
        SubscriptBox["\[ScriptCapitalP]", 
         StyleBox["1",
          ScriptLevel->0]]}],
       ScriptLevel->0], 
      StyleBox[")",
       ScriptLevel->0]}], 
     StyleBox["+",
      ScriptLevel->0], 
     RowBox[{"(", 
      StyleBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
          SubscriptBox["\[ScriptCapitalP]", 
           StyleBox["1",
            ScriptLevel->0]]}], ")"}], "\[CircleTimes]", 
        SubscriptBox["\[ScriptCapitalP]", 
         StyleBox["1",
          ScriptLevel->0]]}],
       ScriptLevel->0], 
      StyleBox[")",
       ScriptLevel->0]}], 
     StyleBox["+",
      ScriptLevel->0], 
     RowBox[{"(", 
      StyleBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["\[ScriptCapitalP]", 
           StyleBox["0",
            ScriptLevel->0]], "\[CircleTimes]", 
          SubscriptBox["\[ScriptCapitalP]", "0"]}], ")"}], "\[CircleTimes]", 
        SubscriptBox["\[ScriptCapitalP]", 
         StyleBox["1",
          ScriptLevel->0]]}],
       ScriptLevel->0], 
      StyleBox[")",
       ScriptLevel->0]}]}], 
    StyleBox[")",
     ScriptLevel->0]}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613504484584209*^9, 3.613504533034775*^9}, {
  3.6241898976295567`*^9, 3.6241898994948378`*^9}, {3.856038421348343*^9, 
  3.8560384213582497`*^9}, {3.8560404739191723`*^9, 3.8560404953500137`*^9}, {
  3.8590552412633533`*^9, 3.8590552412699003`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[201]:=",
 CellID->1093006690,ExpressionUUID->"460546ec-f266-49dc-9e00-7e552c36a036"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "3"], "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "3"], "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "3"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077895732*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[201]=",
 CellID->1299644150,ExpressionUUID->"b2f838b1-8cb1-45fd-8c02-46127f757a1a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Tr", "[", "a123", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", "a123", "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[202]:=",
 CellID->588322392,ExpressionUUID->"4e5ef63c-59d4-454f-af2e-982bd035eea4"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077918832*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[202]=",
 CellID->1063055413,ExpressionUUID->"4ebcc279-60ab-4540-974a-6d945f01c779"],

Cell[BoxData[
 TagBox[
  FractionBox["1", "3"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077928185*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[203]=",
 CellID->1855866898,ExpressionUUID->"ea572a49-e594-47a2-b8da-50234f88eb30"]
}, Open  ]],

Cell["\<\
It is interesting to show the density matrix for each qubit taking partial \
traces. These subspace density matrices can be thought of  as the density \
matrices of one qubit in the environment of the remaining 2 qubits:\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857854448946891*^9, 3.857854477400393*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellID->919731556,ExpressionUUID->"6fc7e343-f33d-4c7a-9824-bcb0fb7d441d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PTr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", "a123"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[204]:=",
 CellID->340970586,ExpressionUUID->"c6d7e4cf-3956-4b95-9d40-31cdf055eb8b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["2", "3"], "0"},
     {"0", 
      FractionBox["1", "3"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077942815*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[204]=",
 CellID->1502465658,ExpressionUUID->"b0e49e6d-4a47-4e54-a7f3-abc7fbc186ef"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PTr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", "a123"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[205]:=",
 CellID->1206268706,ExpressionUUID->"ca9d29d8-68be-49a9-9834-9d318c4a5059"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "3"], "0"},
     {"0", 
      FractionBox["2", "3"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607796565*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[205]=",
 CellID->1827592580,ExpressionUUID->"c3acaadc-b9c5-4088-b0f1-f2785bcafc6e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PTr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", "a123"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.3",
 CellLabel->"In[206]:=",
 CellID->1512514062,ExpressionUUID->"712643d0-bc1d-482b-a119-3420db6664e8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326077982172*^9},
 CellTags->"b:5.1.3",
 CellLabel->"Out[206]=",
 CellID->889889019,ExpressionUUID->"252ba9f4-5bd9-4f1b-877b-8b07ded432a5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinObs", "[", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", "3", "}"}], ",", "a123"}], "]"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624279781630013*^9, 3.6242797953634453`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[207]:=",
 CellID->1059910495,ExpressionUUID->"d4f7e0e4-6550-4199-9175-6f1f1672256d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              FractionBox["1", "3"]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {
              RowBox[{"-", 
               FractionBox["1", "3"]}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", 
             FractionBox["1", "3"]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260780054893`*^9},
 CellLabel->"Out[207]=",
 CellID->1288624625,ExpressionUUID->"cfa9f9bd-7f4a-463b-9765-58cddf65579a"]
}, Open  ]],

Cell[TextData[{
 " Pauli spin matrices can be used to build the more general 2 and n-qubit \
density matrices, e.g. for a general 2 qubit state",
 StyleBox[" the density matrix",
  FontVariations->{"CompatibilityType"->0}],
 " is:"
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195926113078*^9, 3.624195927192295*^9}, {
  3.8581780075921803`*^9, 3.8581780082776546`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->985654226,ExpressionUUID->"5c768566-a768-4ae1-a789-8924ed79d262"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Clear", "[", " ", 
    RowBox[{
    "\[Rho]AB", ",", "\[Tau]", ",", " ", "\[Rho]ABP", ",", "\[Rho]ABP2"}], 
    " ", "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     StyleBox[
      RowBox[{"\[Rho]AB", "[", "t_", "]"}],
      FontFamily->"Helvetica"], 
     StyleBox[":=",
      FontFamily->"Helvetica"], 
     StyleBox[" ",
      FontFamily->"Helvetica"], 
     RowBox[{
      StyleBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "/", "4"}], ")"}], "*", 
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Tau]", "[", 
            RowBox[{"i", ",", "j"}], "]"}], " ", "*", 
           RowBox[{"(", " ", 
            RowBox[{
             SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
             SubscriptBox["\[Sigma]", "j"]}], ")"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "0", ",", "3"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "0", ",", "3"}], "}"}]}], "]"}]}],
       FontFamily->"Helvetica"], 
      StyleBox[" ",
       FontFamily->"Helvetica"], 
      StyleBox["/.",
       FontFamily->"Helvetica"], 
      RowBox[{
       StyleBox["{",
        FontFamily->"Helvetica"], 
       RowBox[{
        StyleBox[
         RowBox[{
          RowBox[{"\[Tau]", "[", 
           RowBox[{"0", ",", "0"}], "]"}], "\[Rule]", "1"}],
         FontFamily->"Helvetica"], 
        StyleBox[",",
         FontFamily->"Helvetica"], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        StyleBox[
         RowBox[{
          RowBox[{"\[Tau]", "[", " ", 
           RowBox[{"1", ",", "0"}], "]"}], "\[Rule]", 
          RowBox[{"PAx", "[", "t", "]"}]}],
         FontFamily->"Helvetica"], 
        StyleBox[",",
         FontFamily->"Helvetica"], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        RowBox[{
         StyleBox[
          RowBox[{"\[Tau]", "[", " ", 
           RowBox[{"2", ",", "0"}], "]"}],
          FontFamily->"Helvetica"], 
         StyleBox["\[Rule]",
          FontFamily->"Helvetica"], 
         RowBox[{
          StyleBox["PAy",
           FontFamily->"Helvetica"], "[", "t", "]"}]}], 
        StyleBox[",",
         FontFamily->"Helvetica"], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        RowBox[{
         StyleBox[
          RowBox[{"\[Tau]", "[", " ", 
           RowBox[{"3", ",", "0"}], "]"}],
          FontFamily->"Helvetica"], 
         StyleBox["\[Rule]",
          FontFamily->"Helvetica"], 
         RowBox[{
          StyleBox["PAz",
           FontFamily->"Helvetica"], "[", "t", "]"}]}], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        StyleBox[",",
         FontFamily->"Helvetica"], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        RowBox[{
         StyleBox[
          RowBox[{"\[Tau]", "[", " ", 
           RowBox[{"0", ",", " ", "1"}], "]"}],
          FontFamily->"Helvetica"], 
         StyleBox["\[Rule]",
          FontFamily->"Helvetica"], 
         RowBox[{"PBx", "[", "t", "]"}]}], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        StyleBox[",",
         FontFamily->"Helvetica"], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        RowBox[{
         StyleBox[
          RowBox[{"\[Tau]", "[", " ", 
           RowBox[{"0", ",", " ", "2"}], "]"}],
          FontFamily->"Helvetica"], 
         StyleBox["\[Rule]",
          FontFamily->"Helvetica"], 
         RowBox[{"PBy", "[", "t", "]"}]}], 
        StyleBox[",",
         FontFamily->"Helvetica"], 
        StyleBox[" ",
         FontFamily->"Helvetica"], 
        RowBox[{
         StyleBox[
          RowBox[{"\[Tau]", "[", " ", 
           RowBox[{"0", ",", " ", "3"}], "]"}],
          FontFamily->"Helvetica"], 
         StyleBox["\[Rule]",
          FontFamily->"Helvetica"], 
         RowBox[{"PBz", "[", "t", "]"}]}]}], 
       StyleBox["}",
        FontFamily->"Helvetica"]}]}]}], 
    StyleBox[";",
     FontFamily->"Helvetica"]}], 
   StyleBox["\[IndentingNewLine]",
    FontFamily->"Helvetica"], 
   StyleBox[
    RowBox[{"FullSimplify", "[", "  ", 
     RowBox[{"\[Rho]AB", "[", "t", "]"}], "]"}],
    FontFamily->"Helvetica"], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"Tr", "[", "%", "]"}], "]"}], "==", "1"}], "\[IndentingNewLine]", 
   RowBox[{"SpinObs", "[", " ", 
    RowBox[{"\[Rho]AB", "[", "t", "]"}], "]"}], " "}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
   3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
   3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
   3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
   3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
   3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
   3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
   3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
   3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
   3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
   3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
   3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9, 
   3.859117278944582*^9, {3.859117431210958*^9, 3.8591174575271463`*^9}, {
   3.859117554545706*^9, 3.859117579286273*^9}, {3.859117619284017*^9, 
   3.859117667373569*^9}, {3.859118217744068*^9, 3.8591182589858913`*^9}, {
   3.859118596348218*^9, 3.859118713334632*^9}, 3.859125611851819*^9, {
   3.859125650031088*^9, 3.859125688946774*^9}, {3.859574037739699*^9, 
   3.859574058888464*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[208]:=",
 CellID->201723424,ExpressionUUID->"7a100f1a-ce09-4175-a1b4-98d88c234dc6"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"PAz", "[", "t", "]"}], "+", 
         RowBox[{"PBz", "[", "t", "]"}], "+", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"3", ",", "3"}], "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PBx", "[", "t", "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"PBy", "[", "t", "]"}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"3", ",", "1"}], "]"}]}], "+", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"3", ",", "2"}], "]"}]}], ")"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PAx", "[", "t", "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"PAy", "[", "t", "]"}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"1", ",", "3"}], "]"}]}], "+", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"2", ",", "3"}], "]"}]}], ")"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Tau]", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Tau]", "[", 
             RowBox[{"1", ",", "2"}], "]"}], "+", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"2", ",", "1"}], "]"}]}], ")"}]}], "-", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"2", ",", "2"}], "]"}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PBx", "[", "t", "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PBy", "[", "t", "]"}]}], "+", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"3", ",", "1"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"\[Tau]", "[", 
           RowBox[{"3", ",", "2"}], "]"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"PAz", "[", "t", "]"}], "-", 
         RowBox[{"PBz", "[", "t", "]"}], "-", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"3", ",", "3"}], "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Tau]", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Tau]", "[", 
             RowBox[{"1", ",", "2"}], "]"}], "-", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"2", ",", "1"}], "]"}]}], ")"}]}], "+", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"2", ",", "2"}], "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PAx", "[", "t", "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PAy", "[", "t", "]"}]}], "-", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"1", ",", "3"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"\[Tau]", "[", 
           RowBox[{"2", ",", "3"}], "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PAx", "[", "t", "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PAy", "[", "t", "]"}]}], "+", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"1", ",", "3"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"\[Tau]", "[", 
           RowBox[{"2", ",", "3"}], "]"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Tau]", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"\[Tau]", "[", 
           RowBox[{"1", ",", "2"}], "]"}]}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"\[Tau]", "[", 
           RowBox[{"2", ",", "1"}], "]"}]}], "+", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"2", ",", "2"}], "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"PAz", "[", "t", "]"}], "+", 
         RowBox[{"PBz", "[", "t", "]"}], "-", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"3", ",", "3"}], "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PBx", "[", "t", "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PBy", "[", "t", "]"}]}], "-", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"3", ",", "1"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"\[Tau]", "[", 
           RowBox[{"3", ",", "2"}], "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[Tau]", "[", 
          RowBox[{"1", ",", "1"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Tau]", "[", 
             RowBox[{"1", ",", "2"}], "]"}], "+", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"2", ",", "1"}], "]"}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"2", ",", "2"}], "]"}]}]}], ")"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PAx", "[", "t", "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"PAy", "[", "t", "]"}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"1", ",", "3"}], "]"}]}], "-", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"2", ",", "3"}], "]"}]}], ")"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PBx", "[", "t", "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"PBy", "[", "t", "]"}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"3", ",", "1"}], "]"}]}], "-", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"3", ",", "2"}], "]"}]}], ")"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"PAz", "[", "t", "]"}], "-", 
         RowBox[{"PBz", "[", "t", "]"}], "+", 
         RowBox[{"\[Tau]", "[", 
          RowBox[{"3", ",", "3"}], "]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260781013107`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[210]=",
 CellID->541492380,ExpressionUUID->"81d3bf94-4238-4b4c-a5f5-d5b8277049eb"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078112863*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[211]=",
 CellID->1974218321,ExpressionUUID->"b846603e-6e92-4680-8ec9-361a9bb04abf"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {
              RowBox[{"PAx", "[", "t", "]"}]},
             {
              RowBox[{"PAy", "[", "t", "]"}]},
             {
              RowBox[{"PAz", "[", "t", "]"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {
              RowBox[{"PBx", "[", "t", "]"}]},
             {
              RowBox[{"PBy", "[", "t", "]"}]},
             {
              RowBox[{"PBz", "[", "t", "]"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {
             RowBox[{"\[Tau]", "[", 
              RowBox[{"1", ",", "1"}], "]"}], 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"1", ",", "2"}], "]"}], 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"1", ",", "3"}], "]"}]},
            {
             RowBox[{"\[Tau]", "[", 
              RowBox[{"2", ",", "1"}], "]"}], 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"2", ",", "2"}], "]"}], 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"2", ",", "3"}], "]"}]},
            {
             RowBox[{"\[Tau]", "[", 
              RowBox[{"3", ",", "1"}], "]"}], 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"3", ",", "2"}], "]"}], 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"3", ",", "3"}], "]"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078121187*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[212]=",
 CellID->2064974897,ExpressionUUID->"0f3d3d2e-3118-4e2e-96f9-41fe349c1bd2"]
}, Open  ]],

Cell[TextData[{
 "For a product \[Rho]A[t]\[CircleTimes] \[Rho]B[t] of two uncorrelated \
spins,   we get \[Rho]ABP[t], with  tensor polarization  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", 
    RowBox[{"i", ",", 
     RowBox[{"j", " ", "="}]}]], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "e15d7d67-7c66-424f-9ba0-86d75b030635"],
 " ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["PA", "i"], TraditionalForm]],
  FormatType->TraditionalForm,ExpressionUUID->
  "4c3bb61d-a171-44c9-9827-5b56ffe2cf73"],
 "  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["PB", "j"], TraditionalForm]],ExpressionUUID->
  "4030e507-c7aa-4093-a3ae-58a988364dac"],
 "  . "
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624195926113078*^9, 3.624195927192295*^9}, {
  3.8581780075921803`*^9, 3.8581780082776546`*^9}, {3.8591262687334137`*^9, 
  3.859126372572126*^9}, {3.859127739811837*^9, 3.859127762723453*^9}, {
  3.859127793391247*^9, 3.859127924342876*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->1313411647,ExpressionUUID->"2b21b997-7386-46cc-b10f-5cd709e13a8f"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]ABP", "[", "t_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[Rho]AB", "[", "t", "]"}], 
    StyleBox["     ",
     FontFamily->"Helvetica"], 
    StyleBox["/.",
     FontFamily->"Helvetica"], 
    StyleBox[" ",
     FontFamily->"Helvetica"], 
    RowBox[{
     StyleBox["{",
      FontFamily->"Helvetica"], 
     StyleBox[" ",
      FontFamily->"Helvetica"], 
     RowBox[{
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"1", ",", "1"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAx", "[", "t", "]"}], "  ", 
        RowBox[{"PBx", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"1", ",", "2"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAx", "[", "t", "]"}], " ", 
        RowBox[{"PBy", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"1", ",", "3"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAx", "[", "t", "]"}], " ", 
        RowBox[{"PBz", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"2", ",", "1"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAy", "[", "t", "]"}], "   ", 
        RowBox[{"PBx", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"2", ",", "2"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAy", "[", "t", "]"}], "  ", 
        RowBox[{"PBy", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"2", ",", "3"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAy", "[", "t", "]"}], "  ", 
        RowBox[{"PBz", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"3", ",", "1"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAz", "[", "t", "]"}], "  ", 
        RowBox[{"PBx", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"3", ",", "2"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAz", "[", "t", "]"}], " ", 
        RowBox[{"PBy", "[", "t", "]"}]}]}], ",", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"3", ",", "3"}], "]"}], "->", 
       RowBox[{
        RowBox[{"PAz", "[", "t", "]"}], " ", 
        RowBox[{"PBz", "[", "t", "]"}]}]}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]ABP2", "[", "t_", "]"}], ":=", "  ", 
   RowBox[{
    RowBox[{"\[Rho]A", "[", "t", "]"}], "\[CircleTimes]", " ", 
    RowBox[{"\[Rho]B", "[", "t", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SpinObs", "[", " ", 
   RowBox[{"\[Rho]ABP", "[", "t", "]"}], "]"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SpinObs", "[", " ", 
   RowBox[{"\[Rho]ABP2", "[", "t", "]"}], "]"}], " ", "==", "%"}]}], "Input",
 CellChangeTimes->{{3.859117470442093*^9, 3.85911753146098*^9}, {
  3.859118721639988*^9, 3.859118758570739*^9}, {3.8591257131909857`*^9, 
  3.859125932028266*^9}, {3.859126390383129*^9, 3.859126578520093*^9}, {
  3.859126672141114*^9, 3.8591266742875357`*^9}, {3.859126712299818*^9, 
  3.859126782339417*^9}, {3.859127950536193*^9, 3.8591279545614853`*^9}, {
  3.860074294719344*^9, 3.8600742996112623`*^9}, {3.860075943656448*^9, 
  3.860075970554044*^9}, {3.8603088947762403`*^9, 3.8603088958011217`*^9}},
 CellLabel->"In[213]:=",
 CellID->141263213,ExpressionUUID->"161abee2-218d-4f07-9ad8-8423db4772e1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {
              RowBox[{"PAx", "[", "t", "]"}]},
             {
              RowBox[{"PAy", "[", "t", "]"}]},
             {
              RowBox[{"PAz", "[", "t", "]"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {
              RowBox[{"PBx", "[", "t", "]"}]},
             {
              RowBox[{"PBy", "[", "t", "]"}]},
             {
              RowBox[{"PBz", "[", "t", "]"}]}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {
             RowBox[{
              RowBox[{"PAx", "[", "t", "]"}], " ", 
              RowBox[{"PBx", "[", "t", "]"}]}], 
             RowBox[{
              RowBox[{"PAx", "[", "t", "]"}], " ", 
              RowBox[{"PBy", "[", "t", "]"}]}], 
             RowBox[{
              RowBox[{"PAx", "[", "t", "]"}], " ", 
              RowBox[{"PBz", "[", "t", "]"}]}]},
            {
             RowBox[{
              RowBox[{"PAy", "[", "t", "]"}], " ", 
              RowBox[{"PBx", "[", "t", "]"}]}], 
             RowBox[{
              RowBox[{"PAy", "[", "t", "]"}], " ", 
              RowBox[{"PBy", "[", "t", "]"}]}], 
             RowBox[{
              RowBox[{"PAy", "[", "t", "]"}], " ", 
              RowBox[{"PBz", "[", "t", "]"}]}]},
            {
             RowBox[{
              RowBox[{"PAz", "[", "t", "]"}], " ", 
              RowBox[{"PBx", "[", "t", "]"}]}], 
             RowBox[{
              RowBox[{"PAz", "[", "t", "]"}], " ", 
              RowBox[{"PBy", "[", "t", "]"}]}], 
             RowBox[{
              RowBox[{"PAz", "[", "t", "]"}], " ", 
              RowBox[{"PBz", "[", "t", "]"}]}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078144546*^9},
 CellLabel->"Out[215]=",
 CellID->549874712,ExpressionUUID->"954d6cf3-7615-42fd-9e2c-f304dc803490"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078155734*^9},
 CellLabel->"Out[216]=",
 CellID->1122151543,ExpressionUUID->"820734f5-cd72-4b21-a1e4-d44536b09a48"]
}, Open  ]],

Cell[TextData[{
 StyleBox[" The general two-qubit",
  FontFamily->"Helvetica"],
 StyleBox[" density matrix",
  FontFamily->"Helvetica",
  FontVariations->{"CompatibilityType"->0}],
 StyleBox[" is of the form  \[Rho]AB= ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", "4"], "  ", "[", 
    RowBox[{"1", " ", "+", " ", 
     SubscriptBox[
      OverscriptBox["P", "\[RightVector]"], 
      RowBox[{"A", " "}]]}]}], TraditionalForm]],ExpressionUUID->
  "0a80706d-ae1f-491f-b422-f0ff1bb6a747"],
 StyleBox[".",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[Sigma]", "\[RightVector]"], 
      RowBox[{"A", " "}]], "  ", "+", " ", 
     SubscriptBox[
      OverscriptBox["P", "\[RightVector]"], 
      RowBox[{"B", " "}]]}]}], TraditionalForm]],ExpressionUUID->
  "7f8de537-0c5f-46bb-8945-09cb75f58686"],
 StyleBox[". ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[RightVector]"], "B"], " "}], 
   TraditionalForm]],ExpressionUUID->"83c4ffd0-514b-4853-91ed-5baa6f13208b"],
 StyleBox[" + ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongRightArrow]"], 
     RowBox[{"A", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "a88573b9-033a-4d28-8577-2d45c92d4af8"],
 StyleBox[". ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   OverscriptBox["T", "\[LongLeftRightArrow]"], TraditionalForm]],
  ExpressionUUID->"3939a457-71eb-4eaf-84e1-6bfcf206d236"],
 StyleBox[" . ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongLeftArrow]"], 
     RowBox[{"A", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "1eb8104c-46ab-4255-a8ca-f28f57d589d0"],
 StyleBox["] .  The last tensor correlation term  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongRightArrow]"], 
     RowBox[{"A", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "0f957490-50b4-4104-8e4d-422788ba0637"],
 StyleBox[".  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   OverscriptBox["T", "\[LongLeftRightArrow]"], TraditionalForm]],
  ExpressionUUID->"8cd25187-917e-4918-b3db-57d4eede8176"],
 " .  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongLeftArrow]"], 
     RowBox[{"A", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "ca6938bf-7031-418e-b8c2-2e39e704da33"],
 "has nine terms. (They can be reconstituted as a scalar  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongRightArrow]"], 
     RowBox[{"A", " "}]], " ", 
    StyleBox[".",
     FontFamily->"Helvetica"]}], TraditionalForm]],ExpressionUUID->
  "74cc0702-66e6-4483-9309-d02e2751b8af"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongRightArrow]"], 
     RowBox[{"B", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "5cd27fbb-23a2-4569-9de3-8ec7df3437bf"],
 ", an axial vector  ",
 StyleBox[" ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongRightArrow]"], 
     RowBox[{"A", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "8765c366-393b-4cb4-9073-da7d05eb08f4"],
 StyleBox["x  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[Sigma]", "\[LongLeftArrow]"], 
     RowBox[{"B", " "}]], " "}], TraditionalForm]],ExpressionUUID->
  "f6af361b-25cf-49c0-8eaf-6cdc1978e7f3"],
 " and a rank  2 traceless tensor {",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    SubscriptBox["\[Sigma]", 
     RowBox[{"1", " "}]], " "], TraditionalForm]],ExpressionUUID->
  "1b0c0acc-2cf1-4c37-b650-1c729db5f653"],
 StyleBox["x ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     RowBox[{
      SubscriptBox["\[Sigma]", "1"], "}"}], 
     RowBox[{" ", 
      RowBox[{"2", 
       SubscriptBox["\[Mu]", " "]}]}]], " "}], TraditionalForm]],
  ExpressionUUID->"8ff99006-f7b6-4af4-838f-27839fe279f9"],
 ".This is the process of adding two vectors to net 0, 1 and 2 rank tensors. \
Thus we again have 1+3+5=9 terms.) ",
 StyleBox["The tensor term ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"<", 
    RowBox[{
     SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
     SubscriptBox["\[Sigma]", "j"]}]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "1e40e304-37d4-4587-9e16-60423ac3fe2a"],
 StyleBox[" > = Tr [( ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{
   SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
   SubscriptBox["\[Sigma]", "j"]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "ae42567c-6798-416d-92d3-1f324b29af8d"],
 StyleBox[" )\[Rho]AB] ,  for i, j 1 ...3 , is often called a \
spin-correlation.\nSince product states yield ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"<", 
    RowBox[{
     SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
     SubscriptBox["\[Sigma]", "j"]}]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "c535f906-fa6e-4928-a61e-6af41cd5730a"],
 StyleBox[" > \[LongRightArrow] ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"<", 
    SubscriptBox["\[Sigma]", "i"]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "dd170cf7-6e78-48fb-bae5-5069c3f36b2c"],
 StyleBox[" > ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"<", " ", 
    SubscriptBox["\[Sigma]", "j"]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "030475bb-2f9c-41f1-9f88-ea39b54f072b"],
 StyleBox[" >,  a more appropriate designation as a spin-correlation is\nthe \
difference :  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  SubscriptBox["C", 
   RowBox[{"ij", " "}]]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "17b3db89-7f75-4458-ae71-8e59407f66c7"],
 Cell[BoxData[
  RowBox[{"=", " ", 
   RowBox[{"<", 
    RowBox[{
     SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
     SubscriptBox["\[Sigma]", "j"]}]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "a16ef6af-a5fd-45bb-a49e-0604308783fd"],
 StyleBox[" >  - ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"<", 
    SubscriptBox["\[Sigma]", "i"]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "c78447ee-e4ef-4ef3-8f1d-21d9f3c276c7"],
 StyleBox[" > ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{"<", " ", 
    SubscriptBox["\[Sigma]", "j"]}]}]],
  TaggingRules->{},
  CellChangeTimes->{
   3.613504696948608*^9, {3.613571376224772*^9, 3.6135713888075*^9}, {
    3.613571445229944*^9, 3.613571470202325*^9}, {3.613571577342877*^9, 
    3.613571594710926*^9}, {3.613571716892363*^9, 3.613571720261849*^9}, {
    3.6135718094980907`*^9, 3.61357182741768*^9}, 3.624193545107458*^9, 
    3.8560384213608828`*^9, {3.856040576238782*^9, 3.856040616142495*^9}, {
    3.857574154631443*^9, 3.857574159782631*^9}, {3.857574190458086*^9, 
    3.857574239447118*^9}, {3.857845378152843*^9, 3.857845410752781*^9}, 
    3.857854602727146*^9, {3.85785467972618*^9, 3.857854733520033*^9}, 
    3.85905524127393*^9, {3.859055515957287*^9, 3.859055517341097*^9}, {
    3.85905632026763*^9, 3.859056330315859*^9}, 3.859089203970686*^9, 
    3.859089242953815*^9, {3.859089533500646*^9, 3.859089550486223*^9}, {
    3.8590904538886023`*^9, 3.859090470508258*^9}, 3.859090512928125*^9},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",ExpressionUUID->
  "6aadbdeb-0eb3-4dd4-a893-661421821cca"],
 StyleBox[" >.",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.613504742769607*^9, 3.857672009828741*^9, {3.858178122714569*^9, 
   3.858178645153555*^9}, {3.858178677442281*^9, 3.858178813858109*^9}, {
   3.858178852122693*^9, 3.85817887914305*^9}, {3.859126845752914*^9, 
   3.8591270170188427`*^9}, {3.8595741039119864`*^9, 3.859574111020317*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->205059545,ExpressionUUID->"696c8d91-be15-4eb7-9b07-f4e082068e79"],

Cell[TextData[StyleBox["Sometimes, a random one-qubit density matrix or a \
random 2 qubit density matrix is needed.  Being Hermitian, it has real \
eigenvalues, which sum to one and are positive definite.  For a density \
matrix, the system described is always pure in contrast to random state \
vectors. (The procedure used to build the random 2 qubit density matrix can \
be easily generalized to the n-qubit case.) \n\n Consider a random 2 qubit \
density matrix.  ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.613504742769607*^9, 3.857672009828741*^9, {3.858178907774623*^9, 
   3.8581789096649647`*^9}, {3.858178948275576*^9, 3.858178986559813*^9}, {
   3.858179427420116*^9, 3.858179586205213*^9}, {3.859092067803482*^9, 
   3.859092157960506*^9}, {3.8590921995794992`*^9, 3.859092373164941*^9}, {
   3.859092408085411*^9, 3.859092459400436*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->20671885,ExpressionUUID->"58533d77-6a16-48c6-9074-c73db65dd1c9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rho", "=", "RandomQubit2"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[217]:=",
 CellID->255844472,ExpressionUUID->"8178b701-bb6a-4ff2-8337-abdb3305ac24"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.2636799135826376`", 
      RowBox[{
       RowBox[{"-", "0.051710843767116016`"}], "-", 
       RowBox[{"0.11337086422187627`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.14114072110062995`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.14080388688545054`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.006075526782523987`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.01410568261381895`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.051710843767116016`"}], "+", 
       RowBox[{"0.11337086422187627`", " ", "\[ImaginaryI]"}]}], 
      "0.28337508175809734`", 
      RowBox[{
       RowBox[{"-", "0.23520180669710639`"}], "+", 
       RowBox[{"0.06556180294247989`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.06886043097654912`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.101385562941282`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.14114072110062995`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.14080388688545054`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.23520180669710639`"}], "-", 
       RowBox[{"0.06556180294247989`", " ", "\[ImaginaryI]"}]}], 
      "0.35199471532786036`", 
      RowBox[{
       RowBox[{"-", "0.015413591859501152`"}], "-", 
       RowBox[{"0.026755034123577232`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.006075526782523987`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.01410568261381895`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.06886043097654912`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.101385562941282`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.015413591859501152`"}], "+", 
       RowBox[{"0.026755034123577232`", " ", "\[ImaginaryI]"}]}], 
      "0.1009502893314047`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078169958*^9},
 CellLabel->"Out[217]=",
 CellID->799595247,ExpressionUUID->"f5935a57-7cd7-4bb7-8d1a-539dfe47ddc9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"rho", "\[Equal]", 
  RowBox[{"Adj", "[", "rho", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tr", "[", "rho", "]"}], "==", "1."}], "\[IndentingNewLine]", 
 RowBox[{"Chop", "[", 
  RowBox[{"SpinObs", "[", "rho", "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tr", "[", 
   RowBox[{"Chop", "[", 
    RowBox[{"rho", ".", "rho"}], "]"}], "]"}], "==", 
  RowBox[{"Purity", "[", "rho", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<Purity=\>\"", ",", 
   RowBox[{"Purity", "[", "rho", "]"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tr", "[", 
   RowBox[{"Eigenvalues", "[", "rho", "]"}], "]"}], "==", 
  "1"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tr", "[", 
   RowBox[{
    RowBox[{"Eigenvalues", "[", "rho", "]"}], "^", "2"}], "]"}], "==", 
  RowBox[{"Purity", "[", "rho", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Eigenvalues", "[", "rho", "]"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"n", ",", "Ev"}], "}"}]}], ",", 
   RowBox[{"PlotLabel", "\[Rule]", "Eigenvalues"}], ",", 
   RowBox[{"AxesOrigin", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0"}], "}"}]}], ",", 
   RowBox[{"Joined", "\[Rule]", "False"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", "Thick", ",", 
      RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6138139128927517`*^9, 3.61381396165273*^9}, {
   3.613814477492661*^9, 3.61381451307972*^9}, {3.613814543301866*^9, 
   3.613814543735861*^9}, {3.624279905817997*^9, 3.624279932786491*^9}, {
   3.6242799648402987`*^9, 3.624280054117082*^9}, 3.856732596711876*^9, {
   3.858179001188006*^9, 3.858179112089613*^9}, {3.858179157194436*^9, 
   3.858179252932151*^9}, {3.858179308922929*^9, 3.858179313850189*^9}, {
   3.858179356128364*^9, 3.858179408188472*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[218]:=",
 CellID->1687504028,ExpressionUUID->"c369f066-2a26-4ce0-9dd2-14889626582a"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607819232*^9},
 CellLabel->"Out[218]=",
 CellID->1992137050,ExpressionUUID->"9fbc1f35-4f24-4842-b1d1-4d5e7cdd1375"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078202099*^9},
 CellLabel->"Out[219]=",
 CellID->429858973,ExpressionUUID->"d52e4b79-8c26-4177-a6cc-c86e418387ef"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0.42000230415435813`"},
             {
              RowBox[{"-", "0.48437889965346503`"}]},
             {"0.09410999068146982`"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {
              RowBox[{"-", "0.13424887125323434`"}]},
             {"0.280251796690907`"},
             {"0.2313492578209959`"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {
             RowBox[{"-", "0.4582525598291648`"}], "0.15933497111259767`", 
             "0.14456058024816165`"},
            {
             RowBox[{"-", "0.10291224065732188`"}], 
             RowBox[{"-", "0.48255466695926075`"}], 
             RowBox[{"-", "0.0788366478883371`"}]},
            {
             RowBox[{"-", "0.07259450381522972`"}], "0.17323166019659805`", 
             RowBox[{"-", "0.2707395941719154`"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078211253*^9},
 CellLabel->"Out[220]=",
 CellID->504770703,ExpressionUUID->"6254dfe1-f52e-40fc-89ab-de495d6c8f3b"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078221094*^9},
 CellLabel->"Out[221]=",
 CellID->1120799671,ExpressionUUID->"c4ef37ad-eb66-455d-8aef-81cf90a871f2"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Purity=\"\>", "\[InvisibleSpace]", "0.5461232265449495`"}],
  SequenceForm["Purity=", 0.5461232265449495],
  Editable->False]], "Print",
 CellChangeTimes->{3.860326078230277*^9},
 CellLabel->"During evaluation of In[218]:=",
 CellID->1701002660,ExpressionUUID->"57226440-ad11-42f7-a21b-d71b559682e8"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260782335787`*^9},
 CellLabel->"Out[223]=",
 CellID->1449397756,ExpressionUUID->"56d2e21c-61eb-4956-bdeb-f29f3bc306c9"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078241424*^9},
 CellLabel->"Out[224]=",
 CellID->1121077171,ExpressionUUID->"27be9219-9773-4b76-a86d-0a0cdb0530a6"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, 
    {RGBColor[1, 0, 0], PointSize[Large], Thickness[Large], 
     PointBox[{{1., 0.7064291981434487}, {2., 0.19447964747453664`}, {3., 
      0.09617793095506985}, {4., 0.0029132234269446853`}}]}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["n", TraditionalForm], 
     FormBox["Ev", TraditionalForm]},
   AxesOrigin->{0., 0.},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotLabel->FormBox["Eigenvalues", TraditionalForm],
   PlotRange->{{0., 4}, {0., 0.7064291981434487}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078294284*^9},
 CellLabel->"Out[225]=",
 CellID->2069977866,ExpressionUUID->"ad74f0e7-2e15-4e82-b893-643b57f47e15"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"?", "Purity"}], "\[IndentingNewLine]", 
 RowBox[{"?", "RandomQubitN"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613724635990411*^9, 3.613724639549246*^9}, {
  3.613814558690959*^9, 3.613814591628696*^9}, {3.6138148675905323`*^9, 
  3.613814868253376*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[226]:=",
 CellID->63672667,ExpressionUUID->"58a03225-4843-445a-a9b2-f5246e135aea"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"Purity[\[Rho]]\\n\\n\\t returns the purity of the \
state represented by the \\n\\n density matrix \[Rho]. It is simply \
Tr[\[Rho].\[Rho]].\\n\\n In general the Purity is a real number between 0 and \
1, \\n\\n being equal to 1 only if the state is a pure state.\"\>", 
              "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"Purity", "[", 
                    RowBox[{"Private`A_", "?", "MatrixQ"}], "]"}], ":=", 
                    RowBox[{"Chop", "[", 
                    RowBox[{"Tr", "[", 
                    RowBox[{"Private`A", ".", "Private`A"}], "]"}], 
                    "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`Purity\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"Purity[\[Rho]]\\n\\n\\t returns the purity of the \
state represented by the \\n\\n density matrix \[Rho]. It is simply \
Tr[\[Rho].\[Rho]].\\n\\n In general the Purity is a real number between 0 and \
1, \\n\\n being equal to 1 only if the state is a pure state.\"\>", 
              "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "Purity[\[Rho]]\n\n\t returns the purity of the state represented by the \
\n\n density matrix \[Rho]. It is simply Tr[\[Rho].\[Rho]].\n\n In general \
the Purity is a real number between 0 and 1, \n\n being equal to 1 only if \
the state is a pure state.", "Documentation" -> None, "OwnValues" -> None, 
     "UpValues" -> None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`Purity, {QDENSITY`Qdensity`Purity[
          PatternTest[
           Pattern[Private`A, 
            Blank[]], MatrixQ]] :> Chop[
          Tr[
           Dot[Private`A, Private`A]]]}], "SubValues" -> None, 
     "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Qdensity`Purity"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607840414*^9},
 CellLabel->"Out[226]=",
 CellID->882912462,ExpressionUUID->"8f0a613f-323e-4c0b-b58b-3d7e1dd03bb0"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"RandomQubitN[nqv], \\n\\n\\t returns a random \
density matrix for a nqv qubit\\nstate.\\nFor compiled version of \
RandomQubitN--use cRQN[nqv], gives PackedArray\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"RandomQubitN", "[", "Private`nqv_", "]"}], ":=", 
                    
                    RowBox[{"Module", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    
                    RowBox[{"Private`tmp", ",", "Private`tmp2", ",", 
                    "Private`tmp3", ",", "Private`nqs", ",", "Private`dim", 
                    ",", "Private`normit"}], "}"}], ",", 
                    RowBox[{
                    RowBox[{"Private`nqs", "=", "Private`nqv"}], ";", 
                    RowBox[{"Private`dim", "=", 
                    SuperscriptBox["2", "Private`nqs"]}], ";", 
                    RowBox[{"Private`tmp", "=", 
                    RowBox[{"RandomComplex", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"-", "1"}], "-", "\[ImaginaryI]"}], ",", 
                    RowBox[{"1", "+", "\[ImaginaryI]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"Private`dim", ",", "Private`dim"}], "}"}]}], 
                    "]"}]}], ";", 
                    RowBox[{"Private`tmp2", "=", 
                    RowBox[{"Chop", "[", 
                    RowBox[{
                    FractionBox["1", "2"], " ", 
                    RowBox[{"(", 
                    RowBox[{"Private`tmp", "+", 
                    RowBox[{"Adj", "[", "Private`tmp", "]"}]}], ")"}]}], 
                    "]"}]}], ";", 
                    RowBox[{"Private`tmp3", "=", 
                    RowBox[{"Chop", "[", 
                    RowBox[{"Private`tmp2", ".", "Private`tmp2"}], "]"}]}], 
                    ";", 
                    RowBox[{"Private`normit", "=", 
                    RowBox[{"Tr", "[", "Private`tmp3", "]"}]}], ";", 
                    RowBox[{"Private`\[Rho]N", "=", 
                    FractionBox["Private`tmp3", "Private`normit"]}], ";", 
                    RowBox[{"Chop", "[", "Private`\[Rho]N", "]"}]}]}], 
                    "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`RandomQubitN\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"RandomQubitN[nqv], \\n\\n\\t returns a random \
density matrix for a nqv qubit\\nstate.\\nFor compiled version of \
RandomQubitN--use cRQN[nqv], gives PackedArray\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "RandomQubitN[nqv], \n\n\t returns a random density matrix for a nqv \
qubit\nstate.\nFor compiled version of RandomQubitN--use cRQN[nqv], gives \
PackedArray", "Documentation" -> None, "OwnValues" -> None, "UpValues" -> 
     None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`RandomQubitN, {
       QDENSITY`Qdensity`RandomQubitN[
          Pattern[Private`nqv, 
           Blank[]]] :> 
        Module[{Private`tmp, Private`tmp2, Private`tmp3, Private`nqs, 
           Private`dim, Private`normit}, Private`nqs = Private`nqv; 
          Private`dim = 2^Private`nqs; 
          Private`tmp = 
           RandomComplex[{-1 - I, 1 + I}, {Private`dim, Private`dim}]; 
          Private`tmp2 = 
           Chop[(Private`tmp + QDENSITY`Qdensity`Adj[Private`tmp])/2]; 
          Private`tmp3 = Chop[
             Dot[Private`tmp2, Private`tmp2]]; 
          Private`normit = Tr[Private`tmp3]; 
          Private`\[Rho]N = Private`tmp3/Private`normit; 
          Chop[Private`\[Rho]N]]}], "SubValues" -> None, "DefaultValues" -> 
     None, "NValues" -> None, "FormatValues" -> None, "Options" -> None, 
     "Attributes" -> {}, "FullName" -> "QDENSITY`Qdensity`RandomQubitN"|>, 
    False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078483182*^9},
 CellLabel->"Out[227]=",
 CellID->2145231711,ExpressionUUID->"c7e59ddb-8aa8-4a3a-95a8-74ef7033ef17"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "one"], "=", "RandomQubit1"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Tr", "[", 
    SubscriptBox["\[Rho]", "one"], "]"}], "\[IndentingNewLine]", 
   RowBox[{"Chop", "[", 
    RowBox[{"Purity", "[", 
     SubscriptBox["\[Rho]", "one"], "]"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{"Eigenvalues", "[", 
    SubscriptBox["\[Rho]", "one"], "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Tr", "[", "%", "]"}], "==", "1."}], "\[IndentingNewLine]", 
   RowBox[{"Tr", "[", 
    RowBox[{
     RowBox[{"Eigenvalues", "[", 
      SubscriptBox["\[Rho]", "one"], "]"}], "^", "2"}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Purity", "[", 
     SubscriptBox["\[Rho]", "one"], "]"}], "==", "%"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8581796519228897`*^9, 3.85817967511931*^9}, {
  3.858179723087044*^9, 3.8581798155173187`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[228]:=",
 CellID->699658063,ExpressionUUID->"c36cbf0e-7130-42d2-b7b5-f81a66a99bfd"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.19563724555376252`", 
      RowBox[{
       RowBox[{"-", "0.1010090941744139`"}], "+", 
       RowBox[{"0.36874440929612556`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.1010090941744139`"}], "-", 
       RowBox[{"0.36874440929612556`", " ", "\[ImaginaryI]"}]}], 
      "0.8043627544462374`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078503457*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[228]=",
 CellID->806417294,ExpressionUUID->"f4cd31e4-5da3-4abb-baaa-4eb8e7636719"],

Cell[BoxData[
 TagBox["0.9999999999999999`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078513075*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[229]=",
 CellID->686164041,ExpressionUUID->"628ca715-b15b-455a-9145-42096ba4f71e"],

Cell[BoxData[
 TagBox["0.9776239255743695`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078522065*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[230]=",
 CellID->371852697,ExpressionUUID->"c564d13e-13ce-42e1-98cb-191780b12298"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0.9886839088686927`"},
      {"0.011316091131307215`"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078530602*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[231]=",
 CellID->550865709,ExpressionUUID->"5cd90fb3-b7bc-4478-b117-6247d5e5a079"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078539174*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[232]=",
 CellID->597968946,ExpressionUUID->"40bd79a2-cf69-4a76-86fd-9de6d8235c85"],

Cell[BoxData[
 TagBox["0.9776239255743695`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078547461*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[233]=",
 CellID->1988091109,ExpressionUUID->"e16e7d3d-c708-429b-b56f-9229ed7d035b"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078556355*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[234]=",
 CellID->1149205290,ExpressionUUID->"d0d3f249-6929-4eaa-b891-acdf731b57be"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Rho]", "two"], "=", "RandomQubit2"}], "\[IndentingNewLine]", 
 RowBox[{"Tr", "[", 
  SubscriptBox["\[Rho]", "two"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Chop", "[", 
  RowBox[{"Purity", "[", 
   SubscriptBox["\[Rho]", "two"], "]"}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[235]:=",
 CellID->1680097516,ExpressionUUID->"bbf31f51-3ff9-4af1-bda7-8d82c4ebd0c9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.32943302414724696`", 
      RowBox[{
       RowBox[{"-", "0.05344507641161705`"}], "-", 
       RowBox[{"0.14021547188936245`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.007010679855940242`"}], "-", 
       RowBox[{"0.03505810067495139`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"-", "0.09856776783541396`"}], "-", 
       RowBox[{"0.04644205988807285`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.05344507641161705`"}], "+", 
       RowBox[{"0.14021547188936245`", " ", "\[ImaginaryI]"}]}], 
      "0.0947967698716734`", 
      RowBox[{"0.06219127793767326`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.008459505937702628`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.07377478082845491`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.0966036513911625`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.007010679855940242`"}], "+", 
       RowBox[{"0.03505810067495139`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.06219127793767326`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.008459505937702628`", " ", "\[ImaginaryI]"}]}], 
      "0.1681987887233606`", 
      RowBox[{"0.09842045722022286`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.2181987950793757`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{
       RowBox[{"-", "0.09856776783541396`"}], "+", 
       RowBox[{"0.04644205988807285`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.07377478082845491`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.0966036513911625`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{"0.09842045722022286`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.2181987950793757`", " ", "\[ImaginaryI]"}]}], 
      "0.40757141725771895`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260785787067`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[235]=",
 CellID->225387912,ExpressionUUID->"7377ff9c-faf8-45db-b8c0-d79c92914f80"],

Cell[BoxData[
 TagBox["0.9999999999999999`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078588903*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[236]=",
 CellID->2080967323,ExpressionUUID->"3027bfb5-3b5d-45d4-b0db-22c0e078fc5c"],

Cell[BoxData[
 TagBox["0.5352759320736409`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078597314*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[237]=",
 CellID->899122619,ExpressionUUID->"42623be4-585c-4545-93f0-e39c9b9efff0"]
}, Open  ]],

Cell[TextData[StyleBox["Now the subsystems are also not pure states, for \
instance, the subsystem corresponding to qubit number 2 of the two qubits \
density matrix is obtained by performing the partial trace of qubit 1,",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->136477009,ExpressionUUID->"d73cbb6c-6597-4f9a-90e5-df1d14978916"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["\[Rho]", 
   RowBox[{"2", ",", "two"}]], "=", 
  RowBox[{"Chop", "[", 
   RowBox[{"PTr", "[", 
    RowBox[{
     RowBox[{"{", "1", "}"}], ",", 
     SubscriptBox["\[Rho]", "two"]}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", 
  SubscriptBox["\[Rho]", "two"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", 
  SubscriptBox["\[Rho]", 
   RowBox[{"2", ",", "two"}]], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8581798614748297`*^9, 3.858179899400064*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[238]:=",
 CellID->256555267,ExpressionUUID->"440f67bd-3b21-4580-a6a6-f81976db1660"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.49763181287060754`", 
      RowBox[{"0.04497538080860581`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.3584142669687381`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.04497538080860581`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.3584142669687381`", " ", "\[ImaginaryI]"}]}], 
      "0.5023681871293924`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260786115294`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[238]=",
 CellID->1858917241,ExpressionUUID->"7d95a954-460e-41ed-baf9-a0226e5ac0be"],

Cell[BoxData[
 TagBox["0.5352759320736409`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078621338*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[239]=",
 CellID->1229972931,ExpressionUUID->"6b4a7cb9-a9f7-43aa-b01f-88a4c4215267"],

Cell[BoxData[
 TagBox["0.7609783599117935`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078629182*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[240]=",
 CellID->477465295,ExpressionUUID->"f82a5d3b-df80-4e19-b534-eaa4db650605"]
}, Open  ]],

Cell[TextData[StyleBox["Use RandomQubitN[n] for n qubits",
 FontColor->GrayLevel[0]]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.614029969288464*^9, 3.61402999526291*^9}, {
  3.6140300269645433`*^9, 3.6140300291688643`*^9}, {3.614030077766451*^9, 
  3.614030087081059*^9}, {3.856100978774246*^9, 3.856101013762191*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->1924692940,ExpressionUUID->"0042349a-67a5-4b61-a324-847d5b09cd5a"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "4"], "=", 
   RowBox[{"RandomQubitN", "[", "4", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Adj", "[", 
   SubscriptBox["\[Rho]", "4"], "]"}], "==", 
  SubscriptBox["\[Rho]", "4"]}], "\[IndentingNewLine]", 
 RowBox[{"Tr", "[", 
  SubscriptBox["\[Rho]", "4"], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Chop", "[", 
  RowBox[{"Purity", "[", 
   SubscriptBox["\[Rho]", "4"], "]"}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.61402969089848*^9, 3.6140297707245197`*^9}, {
   3.614029866255438*^9, 3.6140298843911037`*^9}, 3.614029937746583*^9, 
   3.614030122867091*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[241]:=",
 CellID->768067819,ExpressionUUID->"314dd437-16d4-4f41-a6d5-3867ef618bb7"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078649967*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[242]=",
 CellID->1760307873,ExpressionUUID->"eb8035e4-1e4a-4ade-aa21-5a3a374190f5"],

Cell[BoxData[
 TagBox["0.9999999999999999`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260786600943`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[243]=",
 CellID->1759700439,ExpressionUUID->"cbf5a05e-b01d-4e48-991f-1b6483db2a24"],

Cell[BoxData[
 TagBox["0.14577102118666396`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260786686687`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[244]=",
 CellID->140491789,ExpressionUUID->"4f22ba43-33b0-476e-9c8b-25370c285aeb"]
}, Open  ]],

Cell[TextData[StyleBox["Analogous functions for one-qubit Ket and Bra exist.  \
These are however pure states.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.857672009894503*^9, {3.858861689302279*^9, 3.858861702520803*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->391631816,ExpressionUUID->"23d544a2-af77-4240-b011-fd4d2ec93727"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"RandomKet1", ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{"RandomBra1", ";"}], "\[IndentingNewLine]", 
 RowBox[{"DFormA", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613504796013287*^9, 3.613504834610401*^9}, {
  3.858179931271047*^9, 3.858179935747957*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[245]:=",
 CellID->118553457,ExpressionUUID->"2ec33de3-e5b6-4e89-b603-d0be6efcce9e"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{"0.5300458261673421`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.5931803318861034`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"0\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"0.21160664642887797`", "\[VeryThinSpace]", "+", 
      RowBox[{"0.5678125951519531`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260787133493`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[246]=",
 CellID->1160969670,ExpressionUUID->"d5de0961-3667-47e0-b50f-f3c3745cb9c1"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\" ( \"", 
    RowBox[{
      RowBox[{"-", "0.46573933406742946`"}], "-", 
      RowBox[{"0.6882345070689713`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"0\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{
      RowBox[{"-", "0.5514044666298752`"}], "+", 
      RowBox[{"0.07330245672950032`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"1\"", "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078725037*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[248]=",
 CellID->380827048,ExpressionUUID->"8acff93d-3429-4ba5-971e-61c926a6e099"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"A3", "=", 
   RowBox[{"RandomKetN", "[", "3", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"B3", "=", 
   RowBox[{"RandomBraN", "[", "3", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"DFormA", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQPembr+G5N68c/xyOBdOLn78vB9HX3JOqQPSnzp1T
QPTzK7bTQbS+KMNsEJ1nz7wKRKfcebkWRNfMOboJRHus7NgNoo/ptR0H0Ycc
np8G0U3B/ZdB9G3lC7dAtAFD/R0QfUtLUvg8kF57T0wERH8+uf3GTSC9rej8
TRAt/LHwHojeVd4Ept3edr0E0SJyR1+D6JY62x8gek/ldDD9N3rqPxB9iWcF
mH72+uXSW0A69GfhMhBt0Gyv/B2kXzcMTL/Y85zZ4O0rx35nLhYQrSC+nRNE
337dww2i08QOCoPoPIM5YiA65+HXoKMMrx0v9d4OBtF2nRpvn+x74yh06DGY
BgB1aNGd
  "],
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[249]:=",
 CellID->1810611030,ExpressionUUID->"b659fad7-38d3-4c25-90ac-98f17bfebf85"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{
      RowBox[{"-", "0.008944729038572997`"}], "-", 
      RowBox[{"0.24818418534738154`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"000\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"0.2857864593392232`", "\[VeryThinSpace]", "+", 
      RowBox[{"0.1372251034267232`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"001\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"0.24196240710918815`", "\[VeryThinSpace]", "+", 
      RowBox[{"0.018090909372046243`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"010\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"0.2884260544864963`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.0204893009082653`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"011\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{
      RowBox[{"-", "0.3726303562657387`"}], "+", 
      RowBox[{"0.22485107084161302`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"100\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"0.18389699093087702`", "\[VeryThinSpace]", "+", 
      RowBox[{"0.221382011550819`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"101\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{
      RowBox[{"-", "0.07738126595335018`"}], "+", 
      RowBox[{"0.3420514346723698`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"110\"", "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"0.4258869597831053`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.34457401704581103`", " ", "\[ImaginaryI]"}]}], "\")\"", 
    "\" |\"", "\"111\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032607874774*^9},
 CellLabel->"Out[250]=",
 CellID->1058927676,ExpressionUUID->"cbb1569a-dd57-4ec9-9bd7-7f047c1caec1"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\" ( \"", 
    RowBox[{"0.27391929441164437`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.02103001723033502`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"000\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{"0.20563735611695755`", "\[VeryThinSpace]", "+", 
      RowBox[{"0.42334195353022996`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"001\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{
      RowBox[{"-", "0.24436334435024892`"}], "-", 
      RowBox[{"0.3564344257960624`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"010\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{"0.0014767256069217125`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.20279218568786594`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"011\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{"0.040141135565399146`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.21213943268331095`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"100\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{"0.332840192511504`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.36402140717900716`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"101\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{"0.06846337873738308`", "\[VeryThinSpace]", "-", 
      RowBox[{"0.01819071372261197`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"110\"", "\" |\"", "\" + \"", "\" ( \"", 
    RowBox[{
      RowBox[{"-", "0.2656821761810295`"}], "+", 
      RowBox[{"0.3310909966943344`", " ", "\[ImaginaryI]"}]}], "\" ) \"", 
    "\" < \"", "\"111\"", "\" |\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078761862*^9},
 CellLabel->"Out[252]=",
 CellID->1200467843,ExpressionUUID->"5c4e5926-1aac-4020-a1aa-02f3993bbd3b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Chop", "[", 
   RowBox[{
    RowBox[{"Adj", "[", "A3", "]"}], ".", "A3"}], "]"}], "==", 
  RowBox[{"Chop", "[", 
   RowBox[{"B3", ".", 
    RowBox[{"Adj", "[", "B3", "]"}]}], "]"}], "==", 
  RowBox[{"{", 
   RowBox[{"{", "1.", "}"}], "}"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140156770880623`*^9, 3.614015754106941*^9}, {
   3.624282786708405*^9, 3.624282793682947*^9}, 3.62428479533409*^9, {
   3.857816600903432*^9, 3.857816631895269*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[253]:=",
 CellID->1096303220,ExpressionUUID->"513526c1-3397-44f3-afa8-03c9023c9ba6"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078780177*^9},
 CellLabel->"Out[253]=",
 CellID->654173736,ExpressionUUID->"2ecbdf2a-af21-484d-9935-c11820b83c95"]
}, Open  ]],

Cell[TextData[StyleBox["Qubit Dynamics",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1415280785,ExpressionUUID->"763cb688-6e74-4e57-a511-cc15e8535b7e"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1095279323,ExpressionUUID->"3cee1896-c370-4e38-8023-9520513d906b"],

Cell[TextData[{
 "An example of qubit dynamics is presented first as a magnetic spin-1/2 \
particle in a uniform magnetic field.  Assuming the particle has a magnet \
moment \[Mu] and is in a uniform magnetic field B,  the interaction is - \
\[Mu].B.    Assume that the B field points in the z-direction, and that the \
magnetic moment is  ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[Mu]", "\[RightVector]"], TraditionalForm]],ExpressionUUID->
  "cdd56e0a-7faa-452c-bbdf-6e3ade0bbe4f"],
 " =",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SubscriptBox["\[Mu]", "0"]}], TraditionalForm]],ExpressionUUID->
  "6837093c-f11c-4862-8900-f5e77f3686a5"],
 " ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[Sigma]", "\[RightVector]"], TraditionalForm]],
  ExpressionUUID->"229379bf-ef3b-4dd2-a21a-07b96a79ec1f"],
 ", we are led to consider a Hamiltonian  ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["H", "0", "A"], TraditionalForm]],ExpressionUUID->
  "105fe1f5-f616-4592-b604-fb5ce873ea66"],
 " for qubit A and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["H", "0", "B"], TraditionalForm]],ExpressionUUID->
  "0ee13026-70e5-49f1-ad99-2607a8f488fb"],
 " for qubit B.   Each qubit is in a uniform B-field in the same direct with \
possibly different strengths.   The time-dependent term VR[t] is the famous \
Rabi driving term that involves a B-field that rotates in the x-y plane with \
a driving frequency \[Omega].  Here \[Omega]2 specifies the strength of that \
rotating B-field.   A Rabi resonance occurs for qubit A when \[Omega] = \
\[Omega]AL.  It provides a dynamic rendition of a NOT gate.  We consider two \
separate qubits here for later cases."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
  3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
  3.8576755155558033`*^9}, {3.857843592321775*^9, 3.857843627500601*^9}, {
  3.857843669880412*^9, 3.857843849680228*^9}, {3.8578438797246933`*^9, 
  3.857844007373714*^9}, {3.857844215051877*^9, 3.8578442502424603`*^9}, {
  3.857846075210211*^9, 3.8578461115258636`*^9}, {3.858182217462998*^9, 
  3.858182274694037*^9}, {3.858182311618331*^9, 3.8581824841492777`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->897207608,ExpressionUUID->"2dea4c61-501c-4112-adb3-35484335a7de"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Clear", "[", 
  RowBox[{"HA", ",", "HB", ",", "HA0", ",", "HB0", ",", "VR"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Omega]AL", "=", " ", "."}], " ", ";", "  ", 
  RowBox[{"\[Omega]BL", " ", "=", " ", "."}], "  ", ";", " ", 
  RowBox[{"\[Omega]2", "=", " ", "."}], "  ", ";", " ", 
  RowBox[{"\[Omega]", "=", " ", "."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HA0", "=", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "/", "2"}], " ", "\[Omega]AL", " ", 
    RowBox[{"s", "[", "3", "]"}]}]}], " ", ";", 
  RowBox[{
   FormBox["HB0",
    TraditionalForm], "=", "  ", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "/", "2"}], " ", "\[Omega]BL", " ", 
    RowBox[{"s", "[", "3", "]"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"VR", "[", "t_", "]"}], ":=", "  ", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "\[Omega]2"}], "/", "2"}], 
   RowBox[{"(", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"s", "[", "1", "]"}], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"\[Omega]", " ", "t"}], "]"}]}], "-", 
     RowBox[{
      RowBox[{"s", "[", "2", "]"}], " ", 
      RowBox[{"Sin", "[", " ", 
       RowBox[{"\[Omega]", " ", "t"}], " ", "]"}]}]}], " ", 
    ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HA", "[", "t_", "]"}], ":=", " ", 
  RowBox[{"HA0", " ", "+", 
   RowBox[{"VR", "[", "t", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HB", "[", "t_", "]"}], ":=", 
  RowBox[{"HB0", " ", "+", 
   RowBox[{"VR", "[", "t", "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"HA", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{"HB", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HA0", " ", "/.", "   ", 
   RowBox[{"\[Omega]AL", "->", "100"}]}], " "}], "\[IndentingNewLine]", 
 RowBox[{"HB0", " ", "/.", "   ", 
  RowBox[{"\[Omega]BL", "->", "80", " "}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613815178436349*^9, 3.613815178930174*^9}, {
   3.613815743497026*^9, 3.613815763071269*^9}, {3.613815973798089*^9, 
   3.613815993932239*^9}, {3.613816024002586*^9, 3.613816144101493*^9}, {
   3.613816175308153*^9, 3.613816214175352*^9}, {3.613816284270664*^9, 
   3.6138163801932096`*^9}, {3.613816415226655*^9, 3.613816476478944*^9}, {
   3.613816534504377*^9, 3.613816547968773*^9}, {3.613816984786481*^9, 
   3.613816992714682*^9}, {3.613839045743402*^9, 3.613839054482753*^9}, {
   3.613839091560678*^9, 3.613839092077115*^9}, {3.613839180341464*^9, 
   3.6138391981599216`*^9}, {3.613839297921694*^9, 3.61383930073361*^9}, {
   3.613839348667479*^9, 3.613839349251565*^9}, {3.613840687310046*^9, 
   3.613840691561686*^9}, {3.613899033984764*^9, 3.6138990346930027`*^9}, {
   3.61401551924181*^9, 3.614015520320747*^9}, {3.614015565721573*^9, 
   3.614015580403791*^9}, {3.614015646033984*^9, 3.614015668880912*^9}, {
   3.6242827756837673`*^9, 3.624282782861062*^9}, {3.857675543298749*^9, 
   3.857675576903265*^9}, {3.857675622321055*^9, 3.8576756244457893`*^9}, 
   3.857675728971648*^9, {3.8576757961802588`*^9, 3.857675809569374*^9}, {
   3.857817674100165*^9, 3.857817713710783*^9}, {3.8578185979871197`*^9, 
   3.8578186222305*^9}, {3.857818653000173*^9, 3.857818888895993*^9}, {
   3.8578431154733257`*^9, 3.857843337257534*^9}, {3.857843390583345*^9, 
   3.8578433913823347`*^9}, {3.85784386774117*^9, 3.85784387125119*^9}, {
   3.857844030145879*^9, 3.85784403709896*^9}, {3.857845898920947*^9, 
   3.857846034421413*^9}, {3.8578461454801064`*^9, 3.8578461652001953`*^9}, {
   3.858182499009842*^9, 3.858182630467759*^9}, {3.8581826667196608`*^9, 
   3.8581828112017727`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[254]:=",
 CellID->675654531,ExpressionUUID->"72a325a0-337e-40d0-b28f-e989726cd2a4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", 
       FractionBox["\[Omega]BL", "2"]}], "0"},
     {"0", 
      FractionBox["\[Omega]BL", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078802964*^9},
 CellLabel->"Out[256]=",
 CellID->1729191959,ExpressionUUID->"67a133be-b9d5-495b-af3c-ffe8e81c421f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", 
       FractionBox["\[Omega]AL", "2"]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "2"]}], " ", "\[Omega]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"t", " ", "\[Omega]"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"t", " ", "\[Omega]"}], "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "2"]}], " ", "\[Omega]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"t", " ", "\[Omega]"}], "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"t", " ", "\[Omega]"}], "]"}]}]}], ")"}]}], 
      FractionBox["\[Omega]AL", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260788135233`*^9},
 CellLabel->"Out[260]=",
 CellID->633060062,ExpressionUUID->"390b93ee-edf9-45e1-b3ad-145a1f2f9b41"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", 
       FractionBox["\[Omega]BL", "2"]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "2"]}], " ", "\[Omega]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"t", " ", "\[Omega]"}], "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"t", " ", "\[Omega]"}], "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "2"]}], " ", "\[Omega]2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"t", " ", "\[Omega]"}], "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"Sin", "[", 
           RowBox[{"t", " ", "\[Omega]"}], "]"}]}]}], ")"}]}], 
      FractionBox["\[Omega]BL", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078823482*^9},
 CellLabel->"Out[261]=",
 CellID->1206474796,ExpressionUUID->"5a937b39-3266-4981-98fc-148ce7a0bd80"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "50"}], "0"},
     {"0", "50"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078833496*^9},
 CellLabel->"Out[262]=",
 CellID->681301398,ExpressionUUID->"859cefd0-b306-4015-a837-1dfcee8683db"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "40"}], "0"},
     {"0", "40"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078842248*^9},
 CellLabel->"Out[263]=",
 CellID->249790382,ExpressionUUID->"9684a667-975a-4a4c-bb65-48aad74c2b59"]
}, Open  ]],

Cell["\<\
where  \[Omega]AL and \[Omega]BL   are the Larmor frequencies for qubit A and \
B;    Planck's constant  is set to 1.  For example,  for \[Omega]AL=  100  \
there is a lower level at - 50 and an upper level at + 50.   These are the \
two level of  qubit A.   How does a qubit move in this case?   Clearly the \
polarization vector and the associated density matric are time - dependent  \
and the density matrices are of the general form below.  Each density matrix \
has unit trace and is  Hermitian for a real polarization vector.\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, 3.857818586870779*^9, {3.85784336770089*^9, 
   3.8578435635434*^9}, 3.857843653758728*^9, {3.857844056041666*^9, 
   3.8578441173894997`*^9}, {3.857844274419078*^9, 3.857844373060218*^9}, {
   3.857846198376946*^9, 3.8578462610962*^9}, 3.857920896122405*^9, 
   3.858186763520447*^9, 3.858885931764453*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->582757263,ExpressionUUID->"985101a8-7c32-4fdc-9112-529d67b1e756"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Clear", "[", 
  RowBox[{"\[Rho]A", ",", "\[Rho]B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   RowBox[{"\[Rho]A", "[", "t_", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"1", "/", "2"}], " ", 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{"s", "[", "0", "]"}], " ", "+", " ", 
      RowBox[{
       RowBox[{"PAx", "[", "t", "]"}], " ", 
       RowBox[{"s", "[", "1", "]"}]}], " ", "+", 
      RowBox[{
       RowBox[{"PAy", "[", "t", "]"}], " ", 
       RowBox[{"s", "[", "2", "]"}]}], " ", "+", 
      RowBox[{
       RowBox[{"PAz", "[", "t", "]"}], 
       RowBox[{"s", "[", "3", "]"}]}]}], ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   RowBox[{"\[Rho]B", "[", "t_", "]"}], ":=", " ", 
   RowBox[{
    RowBox[{"1", "/", "2"}], " ", 
    RowBox[{"(", " ", 
     RowBox[{
      RowBox[{"s", "[", "0", "]"}], " ", "+", " ", 
      RowBox[{
       RowBox[{"PBx", "[", "t", "]"}], " ", 
       RowBox[{"s", "[", "1", "]"}]}], " ", "+", 
      RowBox[{
       RowBox[{"PBy", "[", "t", "]"}], " ", 
       RowBox[{"s", "[", "2", "]"}]}], " ", "+", 
      RowBox[{
       RowBox[{"PBz", "[", "t", "]"}], 
       RowBox[{"s", "[", "3", "]"}]}]}], ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"\[Rho]A", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{"\[Rho]B", "[", "t", "]"}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857676033146562*^9, 3.85767624505081*^9}, {
  3.857818524173695*^9, 3.857818574743631*^9}, {3.8578441264575567`*^9, 
  3.8578441851601543`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[264]:=",
 CellID->1024088374,ExpressionUUID->"cba99342-a767-476f-87dc-8e46b3772d5f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"PAz", "[", "t", "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PAx", "[", "t", "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PAy", "[", "t", "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PAx", "[", "t", "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PAy", "[", "t", "]"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"PAz", "[", "t", "]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260788563766`*^9},
 CellLabel->"Out[267]=",
 CellID->2043904036,ExpressionUUID->"3a081f3b-c2ad-45c3-bda8-6758acbcd75d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"PBz", "[", "t", "]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PBx", "[", "t", "]"}], "-", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PBy", "[", "t", "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PBx", "[", "t", "]"}], "+", 
         RowBox[{"\[ImaginaryI]", " ", 
          RowBox[{"PBy", "[", "t", "]"}]}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"PBz", "[", "t", "]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260788671827`*^9},
 CellLabel->"Out[268]=",
 CellID->746715640,ExpressionUUID->"4baa5ed8-77e0-47e9-a646-27b9c6c53ca3"]
}, Open  ]],

Cell[TextData[{
 "The Hamiltonian  HA[t]  is a generator of infinitesimal time displacements, \
hence Dirac and von Neumann deduced  that the equation of motion for \
\[Rho]A[t] is:",
 Cell[BoxData[
  FormBox["    ", TraditionalForm]],ExpressionUUID->
  "529b788f-5579-41cb-8e08-5e87ef161688"],
 "\n",
 Cell[BoxData[
  StyleBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]],
      TraditionalForm], " ", "\[Rho]A"}]}],
   FontSize->24]],
  FontFamily->"Times New Roman",ExpressionUUID->
  "99204ab9-d75b-47d3-b76b-70e92dcf7dcc"],
 "[t]  =  - i   [ HA[t] . \[Rho]A[t] - \[Rho]A[t] . HA[t] ]."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.858186810615019*^9, 3.858186824340279*^9}, {3.858187131244975*^9, 
   3.858187134340464*^9}, {3.8587046831336937`*^9, 3.858704689039668*^9}, {
   3.858885972595112*^9, 3.858886017569853*^9}, {3.8589372777197447`*^9, 
   3.8589372777198973`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1885079621,ExpressionUUID->"d2783001-50ca-46d7-8e26-ff9b455d91bb"],

Cell["\<\
The above  coupled matrix equation can be solved using NDSolve with an \
initial condition of
\[Rho]B[0] or \[Rho]B[0].  Or one can deduce specific dynamic equations for \
the polarization vectors using \
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.858186810615019*^9, 3.858186824340279*^9}, {3.8581868697008123`*^9, 
   3.858187066769903*^9}, {3.85818715897621*^9, 3.8581871783377*^9}, {
   3.858187247842086*^9, 3.858187399628632*^9}, 3.859118039753581*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1330108578,ExpressionUUID->"fcbcad92-bfae-40ef-b6bd-f1e62c52729d"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"Pvec", "[", "t_", "]"}], ":=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Tr", "[", 
       RowBox[{
        RowBox[{"s", "[", "1", "]"}], ".", 
        RowBox[{"\[Rho]A", "[", "t", "]"}]}], "]"}], ",", 
      RowBox[{"Tr", "[", 
       RowBox[{
        RowBox[{"s", "[", "2", "]"}], ".", 
        RowBox[{"\[Rho]A", "[", "t", "]"}]}], "]"}], ",", 
      RowBox[{"Tr", "[", 
       RowBox[{
        RowBox[{"s", "[", "3", "]"}], ".", 
        RowBox[{"\[Rho]A", "[", "t", "]"}]}], "]"}]}], "  ", "}"}], "  ", "//",
     "FullSimplify"}]}],
  FontSize->18], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"Pvec", "[", "t", "]"}],
  FontSize->18]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8581874308151503`*^9, 3.858187461010167*^9}, {
  3.858187493122724*^9, 3.858187592705042*^9}, {3.8581880413587837`*^9, 
  3.858188057013575*^9}, {3.85876321506572*^9, 3.858763231399272*^9}, {
  3.858937326883061*^9, 3.8589373501233873`*^9}, {3.858937387631444*^9, 
  3.858937403065833*^9}, {3.859118046708828*^9, 3.859118069095934*^9}, {
  3.8591184843190727`*^9, 3.8591184966897583`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[269]:=",
 CellID->583441831,ExpressionUUID->"19224dfb-e542-413e-82d3-fcf1bd964a40"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"PAx", "[", "t", "]"}]},
      {
       RowBox[{"PAy", "[", "t", "]"}]},
      {
       RowBox[{"PAz", "[", "t", "]"}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078887904*^9},
 CellLabel->"Out[270]=",
 CellID->155328412,ExpressionUUID->"3211d361-cf25-48ee-88d0-4f561decb56d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"eqAx", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"PAx", "[", "t", "]"}], ",", "t"}], "]"}], "==", 
    StyleBox[
     RowBox[{
      RowBox[{"-", " ", "I"}], "  ", 
      RowBox[{"Tr", "[", 
       RowBox[{
        FormBox[
         RowBox[{" ", 
          SubscriptBox["\[Sigma]", "1"]}],
         TraditionalForm], "  ", ".", " ", 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"HA", "[", "t", "]"}], " ", ".", " ", 
           RowBox[{"\[Rho]A", "[", "t", "]"}]}], " ", "-", " ", 
          RowBox[{
           RowBox[{"\[Rho]A", "[", "t", "]"}], " ", ".", " ", 
           RowBox[{"HA", "[", "t", "]"}]}]}], " ", ")"}]}], " ", "]"}]}],
     FontSize->24]}], 
   StyleBox[" ",
    FontSize->24], 
   StyleBox["//",
    FontSize->24], 
   StyleBox["FullSimplify",
    FontSize->24]}]}], "\[IndentingNewLine]", 
 RowBox[{"eqAy", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"PAy", "[", "t", "]"}], ",", "t"}], "]"}], "==", 
    StyleBox[
     RowBox[{
      RowBox[{"-", " ", "I"}], "  ", 
      RowBox[{"Tr", "[", 
       RowBox[{
        FormBox[
         RowBox[{" ", 
          SubscriptBox["\[Sigma]", "2"]}],
         TraditionalForm], "  ", ".", " ", 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"HA", "[", "t", "]"}], " ", ".", " ", 
           RowBox[{"\[Rho]A", "[", "t", "]"}]}], " ", "-", " ", 
          RowBox[{
           RowBox[{"\[Rho]A", "[", "t", "]"}], " ", ".", " ", 
           RowBox[{"HA", "[", "t", "]"}]}]}], " ", ")"}]}], " ", "]"}]}],
     FontSize->24]}], 
   StyleBox[" ",
    FontSize->24], 
   StyleBox["//",
    FontSize->24], 
   StyleBox["FullSimplify",
    FontSize->24]}]}], "\[IndentingNewLine]", 
 RowBox[{"eqAz", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"PAz", "[", "t", "]"}], ",", "t"}], "]"}], "==", 
    StyleBox[" ",
     FontSize->24], 
    StyleBox[
     RowBox[{
      RowBox[{"-", " ", "I"}], "  ", 
      RowBox[{"Tr", "[", 
       RowBox[{
        FormBox[
         RowBox[{" ", 
          SubscriptBox["\[Sigma]", "3"]}],
         TraditionalForm], "  ", ".", " ", 
        RowBox[{"(", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"HA", "[", "t", "]"}], " ", ".", " ", 
           RowBox[{"\[Rho]A", "[", "t", "]"}]}], " ", "-", " ", 
          RowBox[{
           RowBox[{"\[Rho]A", "[", "t", "]"}], " ", ".", " ", 
           RowBox[{"HA", "[", "t", "]"}]}]}], " ", ")"}]}], " ", "]"}]}],
     FontSize->24]}], 
   StyleBox[" ",
    FontSize->24], 
   StyleBox["//",
    FontSize->24], 
   StyleBox["FullSimplify",
    FontSize->24]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.85784583735957*^9, 3.857845850713438*^9}, {
   3.857846689394622*^9, 3.85784704875557*^9}, {3.857847085437963*^9, 
   3.8578471010804787`*^9}, {3.857847138993623*^9, 3.857847157154825*^9}, {
   3.85784720057734*^9, 3.857847343488904*^9}, {3.8578476745479517`*^9, 
   3.857847797443273*^9}, {3.858188081402144*^9, 3.858188088344376*^9}, {
   3.8581881200782967`*^9, 3.858188330842496*^9}, 3.858937430551426*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[271]:=",
 CellID->1273091116,ExpressionUUID->"7fa7607f-559e-409c-bd34-45a9812e3a85"],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Omega]AL", " ", 
     RowBox[{"PAy", "[", "t", "]"}]}], "+", 
    RowBox[{"\[Omega]2", " ", 
     RowBox[{"PAz", "[", "t", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"t", " ", "\[Omega]"}], "]"}]}]}], "\[Equal]", 
   RowBox[{
    SuperscriptBox["PAx", "\[Prime]",
     MultilineFunction->None], "[", "t", "]"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326078972094*^9},
 CellLabel->"Out[271]=",
 CellID->1411989379,ExpressionUUID->"5df4b52e-af2c-4ed4-bd10-f1196f0b10d1"],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Omega]AL", " ", 
     RowBox[{"PAx", "[", "t", "]"}]}], "+", 
    RowBox[{
     SuperscriptBox["PAy", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", 
   RowBox[{"\[Omega]2", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"t", " ", "\[Omega]"}], "]"}], " ", 
    RowBox[{"PAz", "[", "t", "]"}]}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260790305433`*^9},
 CellLabel->"Out[272]=",
 CellID->1611193503,ExpressionUUID->"5ac56d9f-8473-41eb-922b-f77f5497dcb5"],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Omega]2", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"t", " ", "\[Omega]"}], "]"}], " ", 
     RowBox[{"PAy", "[", "t", "]"}]}], "+", 
    RowBox[{"\[Omega]2", " ", 
     RowBox[{"PAx", "[", "t", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"t", " ", "\[Omega]"}], "]"}]}], "+", 
    RowBox[{
     SuperscriptBox["PAz", "\[Prime]",
      MultilineFunction->None], "[", "t", "]"}]}], "\[Equal]", "0"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326079114355*^9},
 CellLabel->"Out[273]=",
 CellID->1707899898,ExpressionUUID->"4e26d33b-36f2-4270-b046-adb14c2b05d1"]
}, Open  ]],

Cell[TextData[{
 "One approach is to solve the above three coupled equations for the \
polarization vector \n{PAx[t], PAy[t], PAz[t]} assuming initial values at t = \
0.  The second approach is to solve for the density matrix \[Rho]A[t],  for \
an initial value of \[Rho]A[0]  using\n",
 Cell[BoxData[
  StyleBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]],
      TraditionalForm], " ", "\[Rho]A"}]}],
   FontSize->24]],
  FontFamily->"Times New Roman",ExpressionUUID->
  "fb3f0bdc-2d17-4950-97f2-24e1963025b8"],
 "[t]  =  - i   [ HA[t] . \[Rho]A[t] - \[Rho]A[t] . HA[t]]  to deduce the \
evolving density matrix. Then use the following to obtain {PAx[t], PAy[t], \
PAz[t]}.  The magnitude of the polarization vector stays inside the Bloch \
sphere P[t] \[LessSlantEqual] 1 at all times, since the density matrix has \
unit trace.  For  P[t] = 1, we have a pure state P[t] = 1 and for  P[t] < 1 \
we have a mixed state."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848532215272*^9}, {3.857848578484812*^9, 
   3.857848611215054*^9}, {3.8578486459296103`*^9, 3.857848646533626*^9}, {
   3.8587634154399633`*^9, 3.8587634237312927`*^9}, {3.85876401002242*^9, 
   3.858764014111137*^9}, {3.85893745040862*^9, 3.858937455875023*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1257041351,ExpressionUUID->"aba21136-242b-4724-b036-6686a8afdf58"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{"P", "[", "t_", "]"}], " ", ":=", " ", 
   RowBox[{
    RowBox[{"Pvec", "[", "t", "]"}], ".", 
    RowBox[{"Pvec", "[", "t", "]"}]}]}],
  FontSize->18], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"Pvec", "[", "t", "]"}],
  FontSize->18], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{"P", "[", "t", "]"}],
  FontSize->18]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.85876310390069*^9, 3.8587631840685787`*^9}, {
   3.858763278405045*^9, 3.8587633491007223`*^9}, 3.859118468028887*^9, 
   3.859118509121786*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[274]:=",
 CellID->1736955736,ExpressionUUID->"8823fb43-9bc2-4171-940d-cfa7804f178c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"PAx", "[", "t", "]"}]},
      {
       RowBox[{"PAy", "[", "t", "]"}]},
      {
       RowBox[{"PAz", "[", "t", "]"}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260791301947`*^9},
 CellLabel->"Out[275]=",
 CellID->1407159884,ExpressionUUID->"2b7161ea-949f-4cb7-bed4-07cf42fb0a1c"],

Cell[BoxData[
 TagBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"PAx", "[", "t", "]"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{"PAy", "[", "t", "]"}], "2"], "+", 
   SuperscriptBox[
    RowBox[{"PAz", "[", "t", "]"}], "2"]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260791411858`*^9},
 CellLabel->"Out[276]=",
 CellID->2046249624,ExpressionUUID->"d5f2d2b9-b1bd-4b12-8e6b-390bdf33b926"]
}, Open  ]],

Cell["\<\
We adopt the second approach because many other attributes, such as energy, \
power and entropy, can be evaluated once \[Rho]A[t] is known.  A sample case \
is shown for a pure Larmor precession case (\[Omega]2=0), followed by a Rabi \
case at resonance:\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.8587640424115543`*^9, 3.858764064518141*^9}, {3.8588862219967937`*^9, 
   3.858886231451289*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->437104054,ExpressionUUID->"868d063b-5624-4b57-aefe-0eec9d1664fe"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"fpop1", "=", 
   RowBox[{"FileNameJoin", "[", " ", 
    RowBox[{"{", 
     RowBox[{
     "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
      "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", 
      "\"\<LarmorOneQsol.nb\>\""}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"NotebookEvaluate", "[", "fpop1", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.858088170112619*^9, 3.8580881710501823`*^9}, {
   3.85808830034595*^9, 3.8580883090186653`*^9}, {3.85808943924435*^9, 
   3.858089497597969*^9}, {3.858089690888028*^9, 3.858089695848138*^9}, {
   3.8580926323937473`*^9, 3.85809264027004*^9}, {3.858092715014454*^9, 
   3.858092723037389*^9}, {3.8581433920854673`*^9, 3.858143435987483*^9}, {
   3.858143520256908*^9, 3.858143578571362*^9}, {3.8581436508721952`*^9, 
   3.858143660276867*^9}, 3.858143739884026*^9, {3.8581438123348227`*^9, 
   3.858143819546928*^9}, {3.858143886551488*^9, 3.858143920695712*^9}, 
   3.858144094460387*^9, {3.85814433491558*^9, 3.858144343183497*^9}, 
   3.858144410247892*^9, {3.858144446573944*^9, 3.8581444500166397`*^9}, 
   3.858145270222972*^9, 3.858145778113679*^9, {3.858146031242115*^9, 
   3.858146038407642*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[277]:=",
 CellID->488748264,ExpressionUUID->"6e5fafac-6efb-4600-adbc-8a1185655e53"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       TemplateBox[{
         GraphicsBox[{{{{}, {}, 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.368417, 0.506779, 0.709798], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwV2Xc81d8fB3BKCN9QVpNKdpRuab8lomHLKhGRZBcJITPZ7jAzsu6wSfYR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                "]]}, 
              Annotation[#, "Charting`Private`Tag$58505#1"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.880722, 0.611041, 0.142051], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwVmXk8Fe8Xx5FIliKkSIWElESlwpEQkbIvbbJlC9e+y56yJTvhznWjxZo9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                "]]},
               Annotation[#, "Charting`Private`Tag$58505#2"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.560181, 0.691569, 0.194885], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQrZxa97eTXczOZM/TN5MM3tvPVuvecsfExR7GP9wm
FHLbJATO/yDwnimTLxXO3znnwPrJHvlwftRZ6aiNy0vg/OSdj7kfiFfB+V0t
C9PVtOvh/Lh7pT9nhjbB+SzhLFdPcrTC+Zb7e1tbe9vh/M+xd88u8eiC823e
qL/2v9oD5wtdj5ZTCe2H883/rPqylnUinL+mZemRfs9JcH6KRnPC4hWT4fyk
qtB2b8apcL6Gy9IZO52mwfmGmjdKI+dPh/MZ2zps9z+ZAeeL/JO+GOY6C87P
VVnyvmn+bDi/79DKHyWP5sD5n6onnZ3hMA/Ol137o6epfT6c735sZTZDygI4
f+HhE2U3BBfC+fdq1MqFtyD4ew+Xxj5PXgTn70+072wVWAznL93tvqptI4Kv
s9hxo0LCEjh/ulvVWlnmpXC+qqPu+yeLEPyQkxfFXAOWwfnpvFuNOT4h+DL3
O81SFi6H87fkzNVTdl0B5+cHaueseYrgnzm8qv/c5JVwPs+njBXBZqvg/Hdr
JZYXPkPwPx9weh05eTWcf6d/tcpSkzVwftobVpfOxwj+FZb5ERfa1sL5qSWb
Nj1UW4fwb+H2T91XEPy1Wr/VZ5atR8TPzUkKPBcR/NkuU3v2i26A8wFRBr1t

                "]]}, 
              Annotation[#, "Charting`Private`Tag$58505#3"]& ]}}, {}}, {
          DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
           AxesOrigin -> {0, 0}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, DisplayFunction -> Identity, 
           PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
           All, DisplayFunction -> Identity, AspectRatio -> 
           NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True},
            AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
           Identity, Frame -> {{False, False}, {False, False}}, 
           FrameLabel -> {{None, None}, {None, None}}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, GridLinesStyle -> Directive[
             GrayLevel[0.5, 0.4]], 
           Method -> {
            "DefaultBoundaryStyle" -> Automatic, 
             "DefaultGraphicsInteraction" -> {
              "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
               "Effects" -> {
                "Highlight" -> {"ratio" -> 2}, 
                 "HighlightPoint" -> {"ratio" -> 2}, 
                 "Droplines" -> {
                  "freeformCursorMode" -> True, 
                   "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
             "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
             None, "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& )}}, 
           PlotRange -> {{0., 0.06283185178951316}, {-1., 1.}}, 
           PlotRangeClipping -> True, PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {Automatic, Automatic}}, 
           Ticks -> {Automatic, Automatic}}], 
         FormBox[
          FormBox[
           
           TemplateBox[{"\"Px\"", "\"Py\"", "\"Pz\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
               False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.368417, 0.506779, 0.709798]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.880722, 0.611041, 0.142051]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.560181, 0.691569, 0.194885]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"{", "}"}]}], ",", 
                 RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], 
               "]"}]& ), Editable -> True], TraditionalForm], 
          TraditionalForm]},
        "Legended",
        DisplayFunction->(GridBox[{{
            TagBox[
             ItemBox[
              PaneBox[
               TagBox[#, "SkipImageSizeLevel"], 
               Alignment -> {Center, Baseline}, BaselinePosition -> Baseline],
               DefaultBaseStyle -> "Labeled"], "SkipImageSizeLevel"], 
            ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}},
           AutoDelete -> False, GridBoxItemSize -> Automatic, 
          BaselinePosition -> {1, 1}]& ),
        Editable->True,
        InterpretationFunction->(RowBox[{"Legended", "[", 
           RowBox[{#, ",", 
             RowBox[{"Placed", "[", 
               RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]},
      {
       TagBox[
        StyleBox[
         DynamicModuleBox[{Notebook$$104$326079`t$$ = 0.039950702026183456`, 
          Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
          Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, 
          Typeset`animvar$$ = 1, Typeset`name$$ = "\"untitled\"", 
          Typeset`specs$$ = {{
            Hold[Notebook$$104$326079`t$$], 0, Rational[1, 50] Pi}}, 
          Typeset`size$$ = {360., {197., 202.}}, Typeset`update$$ = 0, 
          Typeset`initDone$$, Typeset`skipInitDone$$ = True}, 
          
          DynamicBox[Manipulate`ManipulateBoxes[
           1, StandardForm, "Variables" :> {Notebook$$104$326079`t$$ = 0}, 
            "ControllerVariables" :> {}, 
            "OtherVariables" :> {
             Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
              Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
              Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
              Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> Show[{
               Notebook$$104$326079`plt1[
                Notebook$$104$326079`vecA[Notebook$$104$326079`t$$], 
                Notebook$$104$326079`t$$], Notebook$$104$326079`plpathA}], 
            "Specifications" :> {{
              Notebook$$104$326079`t$$, 0, Rational[1, 50] Pi, 
               AnimationRunning -> True, AnimationRate -> 1, 
               AnimationRepetitions -> 10, 
               AppearanceElements -> {
                "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
                 "DirectionButton"}}}, 
            "Options" :> {
             ControlType -> Animator, AppearanceElements -> None, 
              DefaultBaseStyle -> "Animate", DefaultLabelStyle -> 
              "AnimateLabel", SynchronousUpdating -> True, ShrinkingDelay -> 
              10.}, "DefaultOptions" :> {}],
           ImageSizeCache->{437., {234., 239.}},
           SingleEvaluation->True],
          Deinitialization:>None,
          DynamicModuleValues:>{},
          SynchronousInitialization->True,
          UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
          UnsavedVariables:>{Typeset`initDone$$},
          UntrackedVariables:>{Typeset`size$$}], "Animate",
         Deployed->True,
         StripOnInput->False],
        Manipulate`InterpretManipulate[1]]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326080203228*^9},
 CellLabel->"Out[278]=",
 CellID->1514307510,ExpressionUUID->"33defc2a-8adb-4235-993f-6365d00d312e"]
}, Open  ]],

Cell["\<\
Top plots show the time-dependence of the polarization. Next as above but  \
with rotated uniform field:\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.8588862812282877`*^9, 
   3.8588863125490723`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2014739072,ExpressionUUID->"451c8a0e-04b0-4b06-97ff-552dc0a725e6"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"fpop2", "=", 
     RowBox[{"FileNameJoin", "[", " ", 
      RowBox[{"{", 
       RowBox[{
       "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
        "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", 
        "\"\<LarmorOneQrot.nb\>\""}], "}"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"NotebookEvaluate", "[", "fpop2", "]"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.858144567072925*^9, 3.858144578716819*^9}, {
  3.8581448300907173`*^9, 3.8581448387306013`*^9}, {3.858145495180764*^9, 
  3.858145571739979*^9}, {3.858146025325881*^9, 3.8581460450084476`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[279]:=",
 CellID->1104787296,ExpressionUUID->"efe8f920-845c-429f-be8d-3c4e3aab5ec9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       TemplateBox[{
         GraphicsBox[{{{{}, {}, 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.368417, 0.506779, 0.709798], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwV2Xk4VF8YB3BkqyRK/UpalCTRSqXkixYqJNnatKBUQpISUZZkKWU3M5YQ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                "]]}, 
              Annotation[#, "Charting`Private`Tag$60013#1"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.880722, 0.611041, 0.142051], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwVmXc8le8bx0U0iEjS8k1KJW2VJJ9oKEVkJGkahYqklMxj75JNxjnPOfZI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                "]]}, 
              Annotation[#, "Charting`Private`Tag$60013#2"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.560181, 0.691569, 0.194885], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwV2Xk4lG8XB3DJUqiQrUIbEaVNO74SkZIlS/ZSlqSUKJX6FUrKPjP2irLM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                "]]}, 
              Annotation[#, "Charting`Private`Tag$60013#3"]& ]}}, {}}, {
          DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
           AxesOrigin -> {0, 0}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, DisplayFunction -> Identity, 
           PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
           All, DisplayFunction -> Identity, AspectRatio -> 
           NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True},
            AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
           Identity, Frame -> {{False, False}, {False, False}}, 
           FrameLabel -> {{None, None}, {None, None}}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, GridLinesStyle -> Directive[
             GrayLevel[0.5, 0.4]], 
           Method -> {
            "DefaultBoundaryStyle" -> Automatic, 
             "DefaultGraphicsInteraction" -> {
              "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
               "Effects" -> {
                "Highlight" -> {"ratio" -> 2}, 
                 "HighlightPoint" -> {"ratio" -> 2}, 
                 "Droplines" -> {
                  "freeformCursorMode" -> True, 
                   "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
             "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
             None, "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& )}}, 
           PlotRange -> {{0., 0.06283185178951316}, {-1., 1.}}, 
           PlotRangeClipping -> True, PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {Automatic, Automatic}}, 
           Ticks -> {Automatic, Automatic}}], 
         FormBox[
          FormBox[
           
           TemplateBox[{"\"Px\"", "\"Py\"", "\"Pz\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
               False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.368417, 0.506779, 0.709798]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.880722, 0.611041, 0.142051]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.560181, 0.691569, 0.194885]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"{", "}"}]}], ",", 
                 RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], 
               "]"}]& ), Editable -> True], TraditionalForm], 
          TraditionalForm]},
        "Legended",
        DisplayFunction->(GridBox[{{
            TagBox[
             ItemBox[
              PaneBox[
               TagBox[#, "SkipImageSizeLevel"], 
               Alignment -> {Center, Baseline}, BaselinePosition -> Baseline],
               DefaultBaseStyle -> "Labeled"], "SkipImageSizeLevel"], 
            ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}},
           AutoDelete -> False, GridBoxItemSize -> Automatic, 
          BaselinePosition -> {1, 1}]& ),
        Editable->True,
        InterpretationFunction->(RowBox[{"Legended", "[", 
           RowBox[{#, ",", 
             RowBox[{"Placed", "[", 
               RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]},
      {
       TagBox[
        StyleBox[
         DynamicModuleBox[{Notebook$$105$326080`t$$ = 0.020202586897578423`, 
          Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
          Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, 
          Typeset`animvar$$ = 1, Typeset`name$$ = "\"untitled\"", 
          Typeset`specs$$ = {{
            Hold[Notebook$$105$326080`t$$], 0, Rational[1, 50] Pi}}, 
          Typeset`size$$ = {360., {197., 202.}}, Typeset`update$$ = 0, 
          Typeset`initDone$$, Typeset`skipInitDone$$ = True}, 
          
          DynamicBox[Manipulate`ManipulateBoxes[
           1, StandardForm, "Variables" :> {Notebook$$105$326080`t$$ = 0}, 
            "ControllerVariables" :> {}, 
            "OtherVariables" :> {
             Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
              Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
              Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
              Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> Show[{
               Notebook$$105$326080`plt1[
                Notebook$$105$326080`vecA[Notebook$$105$326080`t$$], 
                Notebook$$105$326080`t$$], Notebook$$105$326080`plpathA}], 
            "Specifications" :> {{
              Notebook$$105$326080`t$$, 0, Rational[1, 50] Pi, 
               AnimationRunning -> True, AnimationRate -> 0.5, 
               AppearanceElements -> {
                "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
                 "DirectionButton"}}}, 
            "Options" :> {
             ControlType -> Animator, AppearanceElements -> None, 
              DefaultBaseStyle -> "Animate", DefaultLabelStyle -> 
              "AnimateLabel", SynchronousUpdating -> True, ShrinkingDelay -> 
              10.}, "DefaultOptions" :> {}],
           ImageSizeCache->{438., {234., 239.}},
           SingleEvaluation->True],
          Deinitialization:>None,
          DynamicModuleValues:>{},
          SynchronousInitialization->True,
          UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
          UnsavedVariables:>{Typeset`initDone$$},
          UntrackedVariables:>{Typeset`size$$}], "Animate",
         Deployed->True,
         StripOnInput->False],
        Manipulate`InterpretManipulate[1]]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326081996957*^9},
 CellLabel->"Out[280]=",
 CellID->1137619973,ExpressionUUID->"e876c999-d475-4445-aee5-7e01be9c0f76"]
}, Open  ]],

Cell[TextData[StyleBox["Next a Rabi driving field at resonance w=wL. ",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188517604759*^9, 
   3.858188556156542*^9}, {3.8588863379516983`*^9, 3.858886339023411*^9}, 
   3.859290039361102*^9, {3.8592901440983*^9, 3.859290144858568*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1748979971,ExpressionUUID->"77a7d123-074a-4420-8673-7ee271b3434e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"fpop4", "=", 
     RowBox[{"FileNameJoin", "[", " ", 
      RowBox[{"{", 
       RowBox[{
       "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
        "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", 
        "\"\<RabiOneQsol.nb\>\""}], "}"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"NotebookEvaluate", "[", "fpop4", "]"}], " ", 
   "\[IndentingNewLine]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8580883614559593`*^9, 3.858088393497546*^9}, {
   3.858088426314584*^9, 3.85808842678581*^9}, {3.858088561451343*^9, 
   3.858088561832678*^9}, {3.8580886833239393`*^9, 3.858088689026577*^9}, {
   3.858088913625505*^9, 3.858088921949749*^9}, {3.85808938414286*^9, 
   3.858089390912674*^9}, {3.858089533959215*^9, 3.858089554877647*^9}, {
   3.8580895859914713`*^9, 3.8580895935511227`*^9}, {3.858089632217955*^9, 
   3.858089681661626*^9}, {3.85814607411379*^9, 3.858146099617082*^9}, {
   3.858146164610263*^9, 3.858146172801646*^9}, {3.85817275030464*^9, 
   3.858172759426033*^9}, 3.858886346526204*^9, 3.859282740997452*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[281]:=",
 CellID->1126512153,ExpressionUUID->"4d0e5bb3-c316-4d7b-be16-5edeb148bdfb"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       TemplateBox[{
         GraphicsBox[{{{{}, {}, 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.368417, 0.506779, 0.709798], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUV3c81e8Xl6SSlhQiFb4NUUhFqndJkiSpJC3SMBsqGQ3ZSraykgYhZGZ7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                "]]}, Annotation[#, "Charting`Private`Tag$60419#1"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.880722, 0.611041, 0.142051], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUmHk8lF8XwO2kSKUQSRIqSZKQOhKRLalEiYiyRcmu7KTFEpIoIdn3fb/2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                "]]}, 
              Annotation[#, "Charting`Private`Tag$60419#2"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.560181, 0.691569, 0.194885], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwd13k0VP//B3BljUhky1K0ELJElhYvxKcICVFUkiTZ6pvImiKZFmv23TBk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                "]]}, 
              Annotation[#, "Charting`Private`Tag$60419#3"]& ]}}, {}}, {
          DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
           AxesOrigin -> {0, 0}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, DisplayFunction -> Identity, 
           PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
           All, DisplayFunction -> Identity, AspectRatio -> 
           NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True},
            AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
           Identity, Frame -> {{False, False}, {False, False}}, 
           FrameLabel -> {{None, None}, {None, None}}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, GridLinesStyle -> Directive[
             GrayLevel[0.5, 0.4]], 
           Method -> {
            "DefaultBoundaryStyle" -> Automatic, 
             "DefaultGraphicsInteraction" -> {
              "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
               "Effects" -> {
                "Highlight" -> {"ratio" -> 2}, 
                 "HighlightPoint" -> {"ratio" -> 2}, 
                 "Droplines" -> {
                  "freeformCursorMode" -> True, 
                   "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
             "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
             None, "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& )}}, 
           PlotRange -> {{0., 0.6283185178951315}, {-1., 1.}}, 
           PlotRangeClipping -> True, PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {Automatic, Automatic}}, 
           Ticks -> {Automatic, Automatic}}], 
         FormBox[
          FormBox[
           
           TemplateBox[{"\"Px\"", "\"Py\"", "\"Pz\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
               False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.368417, 0.506779, 0.709798]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.880722, 0.611041, 0.142051]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.560181, 0.691569, 0.194885]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"{", "}"}]}], ",", 
                 RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], 
               "]"}]& ), Editable -> True], TraditionalForm], 
          TraditionalForm]},
        "Legended",
        DisplayFunction->(GridBox[{{
            TagBox[
             ItemBox[
              PaneBox[
               TagBox[#, "SkipImageSizeLevel"], 
               Alignment -> {Center, Baseline}, BaselinePosition -> Baseline],
               DefaultBaseStyle -> "Labeled"], "SkipImageSizeLevel"], 
            ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}},
           AutoDelete -> False, GridBoxItemSize -> Automatic, 
          BaselinePosition -> {1, 1}]& ),
        Editable->True,
        InterpretationFunction->(RowBox[{"Legended", "[", 
           RowBox[{#, ",", 
             RowBox[{"Placed", "[", 
               RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]},
      {
       TagBox[
        StyleBox[
         DynamicModuleBox[{Notebook$$106$326082`t$$ = 0.017278391079828023`, 
          Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
          Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, 
          Typeset`animvar$$ = 1, Typeset`name$$ = "\"untitled\"", 
          Typeset`specs$$ = {{
            Hold[Notebook$$106$326082`t$$], 0, Rational[1, 5] Pi}}, 
          Typeset`size$$ = {360., {197., 202.}}, Typeset`update$$ = 0, 
          Typeset`initDone$$, Typeset`skipInitDone$$ = True}, 
          
          DynamicBox[Manipulate`ManipulateBoxes[
           1, StandardForm, "Variables" :> {Notebook$$106$326082`t$$ = 0}, 
            "ControllerVariables" :> {}, 
            "OtherVariables" :> {
             Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
              Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
              Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
              Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> Show[{
               Notebook$$106$326082`plt1[
                Notebook$$106$326082`vecB[Notebook$$106$326082`t$$], 
                Notebook$$106$326082`t$$], Notebook$$106$326082`plpathB}], 
            "Specifications" :> {{
              Notebook$$106$326082`t$$, 0, Rational[1, 5] Pi, 
               AnimationRunning -> True, AnimationRate -> 0.05, 
               AnimationRepetitions -> 10, 
               AppearanceElements -> {
                "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
                 "DirectionButton"}}}, 
            "Options" :> {
             ControlType -> Animator, AppearanceElements -> None, 
              DefaultBaseStyle -> "Animate", DefaultLabelStyle -> 
              "AnimateLabel", SynchronousUpdating -> True, ShrinkingDelay -> 
              10.}, "DefaultOptions" :> {}],
           ImageSizeCache->{438., {234., 239.}},
           SingleEvaluation->True],
          Deinitialization:>None,
          DynamicModuleValues:>{},
          SynchronousInitialization->True,
          UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
          UnsavedVariables:>{Typeset`initDone$$},
          UntrackedVariables:>{Typeset`size$$}], "Animate",
         Deployed->True,
         StripOnInput->False],
        Manipulate`InterpretManipulate[1]]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260872156363`*^9},
 CellLabel->"Out[282]=",
 CellID->931976970,ExpressionUUID->"9bfd5b18-428b-4f05-8812-486b5af138e9"]
}, Open  ]],

Cell[TextData[StyleBox["Next a Rabi at resonance w=wL with  a rotated  \
Hamiltonian and a rotated initial qubit:",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188517604759*^9, 
   3.858188556156542*^9}, {3.8588863379516983`*^9, 3.858886339023411*^9}, 
   3.859290039361102*^9, {3.859290091628003*^9, 3.8592901360767393`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->805007800,ExpressionUUID->"85eedebf-dc13-4950-bfb5-d7829f038189"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"fpop3", "=", 
   RowBox[{"FileNameJoin", "[", " ", 
    RowBox[{"{", 
     RowBox[{
     "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
      "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", 
      "\"\<RabiOneQrot.nb\>\""}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"NotebookEvaluate", "[", "fpop3", 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[283]:=",
 CellID->1028383359,ExpressionUUID->"fc315e2e-60c5-42eb-84df-6a8eca98444c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       TemplateBox[{
         GraphicsBox[{{{{}, {}, 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.368417, 0.506779, 0.709798], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwcV3k8lO/XlrQpJSlEkpQifSUpkisiRSQJSVKkjbRItiQkZE+FrGUpyZZ9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                "]]}, 
              Annotation[#, "Charting`Private`Tag$61925#1"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.880722, 0.611041, 0.142051], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUV3k4lV8XlSRJpVSGJMlPKkmSJrUkIRqQRAlJNGiWUIQilaFEiQaVKck8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                "]]}, 
              Annotation[#, "Charting`Private`Tag$61925#2"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.560181, 0.691569, 0.194885], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUWHk4lG8XliRpoxSiFS2SVJIWbkklCUmS9kgllSKhTXZC1lJJKkQka3Yv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                "]]}, 
              Annotation[#, "Charting`Private`Tag$61925#3"]& ]}}, {}}, {
          DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
           AxesOrigin -> {0, 0}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, DisplayFunction -> Identity, 
           PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
           All, DisplayFunction -> Identity, AspectRatio -> 
           NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True},
            AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
           Identity, Frame -> {{False, False}, {False, False}}, 
           FrameLabel -> {{None, None}, {None, None}}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, GridLinesStyle -> Directive[
             GrayLevel[0.5, 0.4]], 
           Method -> {
            "DefaultBoundaryStyle" -> Automatic, 
             "DefaultGraphicsInteraction" -> {
              "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
               "Effects" -> {
                "Highlight" -> {"ratio" -> 2}, 
                 "HighlightPoint" -> {"ratio" -> 2}, 
                 "Droplines" -> {
                  "freeformCursorMode" -> True, 
                   "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
             "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
             None, "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& )}}, 
           PlotRange -> {{0., 0.6283185178951315}, {-1., 1.}}, 
           PlotRangeClipping -> True, PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {Automatic, Automatic}}, 
           Ticks -> {Automatic, Automatic}}], 
         FormBox[
          FormBox[
           
           TemplateBox[{"\"Px\"", "\"Py\"", "\"Pz\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
               False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.368417, 0.506779, 0.709798]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.880722, 0.611041, 0.142051]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.560181, 0.691569, 0.194885]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"{", "}"}]}], ",", 
                 RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], 
               "]"}]& ), Editable -> True], TraditionalForm], 
          TraditionalForm]},
        "Legended",
        DisplayFunction->(GridBox[{{
            TagBox[
             ItemBox[
              PaneBox[
               TagBox[#, "SkipImageSizeLevel"], 
               Alignment -> {Center, Baseline}, BaselinePosition -> Baseline],
               DefaultBaseStyle -> "Labeled"], "SkipImageSizeLevel"], 
            ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}},
           AutoDelete -> False, GridBoxItemSize -> Automatic, 
          BaselinePosition -> {1, 1}]& ),
        Editable->True,
        InterpretationFunction->(RowBox[{"Legended", "[", 
           RowBox[{#, ",", 
             RowBox[{"Placed", "[", 
               RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]},
      {
       TagBox[
        StyleBox[
         DynamicModuleBox[{Notebook$$107$326087`t$$ = 0.020967300288789223`, 
          Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
          Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, 
          Typeset`animvar$$ = 1, Typeset`name$$ = "\"untitled\"", 
          Typeset`specs$$ = {{
            Hold[Notebook$$107$326087`t$$], 0, Rational[1, 5] Pi}}, 
          Typeset`size$$ = {360., {197., 202.}}, Typeset`update$$ = 0, 
          Typeset`initDone$$, Typeset`skipInitDone$$ = True}, 
          
          DynamicBox[Manipulate`ManipulateBoxes[
           1, StandardForm, "Variables" :> {Notebook$$107$326087`t$$ = 0}, 
            "ControllerVariables" :> {}, 
            "OtherVariables" :> {
             Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
              Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
              Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
              Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> Show[{
               Notebook$$107$326087`plt1[
                Notebook$$107$326087`vecA[Notebook$$107$326087`t$$], 
                Notebook$$107$326087`t$$], Notebook$$107$326087`plpathA}], 
            "Specifications" :> {{
              Notebook$$107$326087`t$$, 0, Rational[1, 5] Pi, 
               AnimationRunning -> True, AnimationRate -> 0.05, 
               AppearanceElements -> {
                "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
                 "DirectionButton"}}}, 
            "Options" :> {
             ControlType -> Animator, AppearanceElements -> None, 
              DefaultBaseStyle -> "Animate", DefaultLabelStyle -> 
              "AnimateLabel", SynchronousUpdating -> True, ShrinkingDelay -> 
              10.}, "DefaultOptions" :> {}],
           ImageSizeCache->{437., {234., 239.}},
           SingleEvaluation->True],
          Deinitialization:>None,
          DynamicModuleValues:>{},
          SynchronousInitialization->True,
          UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
          UnsavedVariables:>{Typeset`initDone$$},
          UntrackedVariables:>{Typeset`size$$}], "Animate",
         Deployed->True,
         StripOnInput->False],
        Manipulate`InterpretManipulate[1]]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326089366287*^9},
 CellLabel->"Out[284]=",
 CellID->2138697219,ExpressionUUID->"1eb7b1c9-ef4a-414f-be58-b16c6988e68c"]
}, Open  ]],

Cell[TextData[StyleBox["The above Rabi resonance case simulates the action of \
a NOT gate and flips the spin.",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188517604759*^9, 
   3.858188556156542*^9}, {3.8588863379516983`*^9, 3.858886339023411*^9}, {
   3.858886517253488*^9, 3.8588865744756927`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1791024081,ExpressionUUID->"42db371b-9291-4877-8ba7-0d7892c3676d"],

Cell[TextData[StyleBox["Now an off-resonance  w=1.4 wL Rabi oscillation case:",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188737557898*^9, 
   3.8581887857944527`*^9}, {3.858190068270484*^9, 3.8581900700905647`*^9}, {
   3.8587022784115562`*^9, 3.858702279490685*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->358333546,ExpressionUUID->"13a71bd0-39b1-4e90-91b6-1af41b3f51a4"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"fpop5", "=", 
   RowBox[{"FileNameJoin", "[", " ", 
    RowBox[{"{", 
     RowBox[{
     "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
      "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", 
      "\"\<RabiOneQsoloff.nb\>\""}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NotebookEvaluate", "[", "fpop5", "]"}], 
  " "}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.85818862877635*^9, 3.8581887323735228`*^9}, 
   3.8588863769793777`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[285]:=",
 CellID->190738675,ExpressionUUID->"9fe08296-0e67-401e-b3ef-bfe41b15c356"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       TemplateBox[{
         GraphicsBox[{{{{}, {}, 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.368417, 0.506779, 0.709798], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUW3c81e8XR1IJKQ2yS6hQSUbhiKSIVDJKZKSSUZKQ7P01igaZyd5785j3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                "]]}, 
              Annotation[#, "Charting`Private`Tag$62331#1"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.880722, 0.611041, 0.142051], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUWXc8FP4bR1LfoqSBlDQpKSRJ5ZGs7GRWVpSSFZKVvQplFCIj2Xtm+xwO
d8dxe6iUEZJUqGy/+/11r+c+r/ezP5/n/dwdvO1mfIeHi4vLlY+L6/+fzmuf
Apyf6qoE0rHH0l7jQdxEPjlQ7TiMjMsPtDfg4fYNyzAttfPwZLX8VDAdDze6
+j4V/FaD2jMj/z78xEP4p+lFabWrEEMVfPpxEwHK//DvqUw0BC5drOvT/QSY
jtwjmvbbGIRCfmtMyRKAK+mY5EdRM2jeMLhtkxoBdmcpnhVXswSxl3tmJowI
cKJE84qt0y1Q3a2845UVAVTqza7lJNrAtFbz7z33CFBsV3Paw8oOFt4WHfR2
J0CtY7Ni0u/bgFRPqzd7E6DNpfNSbYQDNIvsv/TTjwB4z151uuhdwIS+Xtn7
hAA0X6ru3zJH6O+2C9PiyENBH4yF1e7D27LkDYH+BJiMGLVUYjhBwGfp9z2P
CTAb893W0skZqtJXRmU8CLCSMOfot+YCXcO8c41OBNiUuuyanugGL3Zt0Xtk
R4D/BrgvWCe5w60YDcf7ZgTYRoloc7V6CNebFPFZVwmwk/6fWpCkBwidz58/
eIEAIqy4rhe/PcAhNdVz8QQB9n/YcTW72RM+WhxnnhYlwKGhV32VEV7wKSKj
uH8jAaSGRY3aDR/Bd4Ei38+/8CAzlkEli3rDV+HLPQ8G8SA/cdB8ZNQbGg/H
b3nSgQelqbzB2bLHMGap2bSvCA9qv8qGd6r5gmhLwpeTHnjQmpO7c4TfD9TR
wHSFCR70/tZNKjD8IEb5T8b4WTyYrbT+NHUKgDpkfiBpHgcPNvevpiQGwZXK
5Olhexw0bCxSNTkVDP7R5o7LF3FwYdOD0FtJwbDJAGy27caB6pafG12sQqBu
HpN4uaMHdAT/8T//HQqdOdNcDTt6oG9Ho0GyWRgEmTVt8RvtBsOd/vGZzWGw
J6iuyKO2G0z2rO+siAiHpS+vwg6ZdIPNvk1iJNFIOIbHucnGdIGXlPBxIbWn
IOftVTn/sRMCDsUvva54Ctw7g1qPZXdC2L7/+g7ufwbejWtKpNudkCi47Cq3
8AyuXJb2TBvvgMqFoVqj8liQ7pqd9ZxsB2+H69laAvFAmWFfWCxHUKMgdjLS
NwnULzvp7SS+h/dVzF06lUlQTNDH6J17D02nXq4KTCTBRuOIkNDsOuiQEhh4
ZfISPpH/e0xwqwXKvvWHeadfwUOxQu/d3NXwi3e0Hvs1GQyOccmeIxfBSXrx
lQ3GaSAzkx7k9sMXJqa3iB6LSIOL2xnFPSddIIf3wYx2QxoUvlXqiYKrIKIg
/fq5eDokPz/zQ2rZE/EmlcyIfk8HfdMijYuMePTBqPS1bHgG8Hd9T/yKz0Ua
44x/p25mg0Pt3KGo3Hp0aDbyNsYrG04NHnPwD2pAXGuKxGvPs8EtSktt6kYj
atqdkuPVng0nj74/9UuoGclqmOs3Sb4FIdczL19EtKF9ucwcjfm38E+JrGtX
1YGWKqIEmAI5YP3G7t3c/k7EbD7nc08yB8acrB/tftaJEqkp+jE3cqBhv10V
xRaLNm+wWCBhcuDwVS71Rr5uNG/H0r8V9w6Ist8vIEk8couxv+6S/w4aw7T/
DYfj0VTtjEUgegdkuS5G0xc8+rJpo0P273cQvJonYPKSgPrK5PzGTHPBRt3n
WcS3XqTFbAn645oLxgwWtkm5D3VwaUfwRedCrLW2yulnfajhunW8VFMuGBXv
2Z92mIjeLT7LdxbPgw/BT+8wVPqR+OE9pU/O5YHYqoePQ3A/StV7W/XcKA9E
5ZKkmlE/ep5V31IZmgceUh7+0ecG0H94tY729DyoObPJccRtAIXPEnsotXkw
KrYluDdvAPlpjFHmx/Ngxy6b9v82ktAfVzfWRq58cPHospTeR0LuqUuf9ojm
Q4y33eqCHAl9b48YlZTPBxGL19zOmiR097vgNyXdfCjuPNX+xJKEhne9mbnq
kA/XGyz6JB+Q0C0VyfkbT/LBoa+abO9PQkzH6sUHyfkQL5USIv2MhIwTLq0H
VOSDhoBLk08KCRGbcLzPcflg8t4iQeMdCWmPXd+SNZwP2q/ei8eWkVCnwOft
lUv54Drw+LXmexJSOee0u31nATQMREe6tZJQo+2fvZSTBSD8fcNj7k4SUngW
LDGqUQD75Ja2/OsmoYqarcfmrQsgZnVRWRdPQic+JUtv9CkA8aPMA4scOY/v
kNyehAJgbo1jLuNISEK2TFGyuAA23xAlGXDw6ZZKF5U6C2DVs+39j3YS2hPW
efnqxwKo36BygdZMQgmlBlo3/hTAStFerbUaEuJnsPUebCuEgOEfh+8Vk1Dk
uoNxgGQhOLXaHtmWRULcx3+Zx6kWwmfxwqbpBBIKMPa3yrQshL/3gvu5wkjo
rz+ffYVHIZgR/tzW8iChh3kJ9zAxhWA11B/VakNC0/37XMm5hRBjoYxsdUnI
caHAc6S1EIgCpJ5TZ0lo5OAZ3zlGIVxN+2B/dD8Jsbyuhu/+rwgmxs3NtgwM
oOuZtKfHDhWB/ZfJRkbSAOrvsXlx7kIRbFG6wjtpMoCwe73TLF2KwCulP5La
249AnTvbKZKDT5C+WhLSj5pcYvP8s4pA6G/LUueZflSJyanM4LwburMWJY2x
RKRXKJbWP10Emw2C0oakiWjyxcuw9U3FoLMxUy67uw8dtIk0s1MphjJEf7Dx
Zy9q01yHRMtiWOLTZ1YF9KIbp3yOd3oVg2D4uttWvl6UuHp/5UhJMaj7X63/
vJWATn0d+WraXQzP1fWn7kXjEaHv5kDkcDEI0/INWrjwiOeNfs6kcAkE2PR9
nRjrQZ7Kctql4SXAbVk5ExLQhQQPFct9yioBUsoo+ykLi0r/Oyy2rbkEzswk
LDXJYtEYa9eM2+8SSJHI/MGgdyCTxwtJZ6xL4djPrdK53xBSrEWfm86Vgebi
IFb2VA2ipCvhvxuXAe70Q8piQBVyDauq3udaBrQ18yWLuApUYPwuIjC3DFa0
/fA6KYVI5HektNqOcihjMkU+Rb1EizIGj/FT5bBmOVZrnPIOhq67bpo2qARH
H1polAQWbpRfP/fmZiXMHvG4uX0NC4zN5x317lVCvvu+dIXBLuhv3dBTGlIJ
m7inrnyJ64FWydeRLrWVoBBIcwn8TACl0MD3+9sr4dPG/pCa8l6o/Wg/TiRW
gnNE4fBr/z4oTTileWqiEq5ILB/atK0f3ix3bvwpWgVPGnr3HXpKAhGzorNZ
x6rATqT16OxXErysfH7H8EwVKIXs5D9xmQyxdyy7KvSqwM/2llr6LBn8B2bC
3YOq4Mv57z07Vanw9zitViK2CmQvSF+rek4Fj/DGMVJqFYRnhJkbfqCC0/lw
dbnqKhBNKvK0cqXBjXeivLNjVVC7edY1yI8OjNW1Mzm/q2C/rK32UD0djC3G
7I3XqiBpMt91bJYOOgIVndXC1aDHYPq22TGg2/Hl3O0j1bBLZdEs6BUD1Dp8
D++Uq4byM4VS7d0MOP9YPcxTpxqezp62yz3IhFry8ZrD5tXA23ZY46wuE2RP
bh+l2lfDd/tXUUc9mFAaOS8U7l4Ntgd9MS7JTA7fYaspPKkG/6JdJL4GJuRe
QB5jT6vhlZ979jcGEySSc3NeJldD177amu1zTHjz6ylF/V01eP/dy+fJzwIR
XTeePxXVMC6m/VrgMAte5pnI57VUwwmRW5/ZiiwQ5FK+bYqvBmxF1B+KFgti
bxxI3Miohkbtm5vmTVmwuY63o26kGlKtn4Yr2bEgfPvU7zs/q+F+i0zRm/ss
4HIaOLhnpRqktscx97mxIABbe617cw0IB6wk1j1kwT/xtBDv3TXQNPlaxoEj
e/gGVR07VANROppXj7myYIbqMMw4VQPur26+XHRkgdMpnR1RF2pADv/N9aM1
C8ajT18+p10Dj1NS24jGLLg9uuvhhEkNnF7jduu7woKhS0vZKXY10K5b94Mp
x4IbqZ9JWq41sLv8tt2vfSxgzGK5Fvxq4En6ytrujRz8ybeHGqJrQMjxcP3k
BBN+3nmi7pNcA+MOBhpaPUwIyLK8q5RbAylHz6SceceEzeyz0QtVNaBt37Ra
4c+El0JCxQ2oBm4dkB5GRkw4qDfT60OsAX7njsG7h5igjPK3L07WgOBPvf0x
zZz+WAiVa/xbA4wb21uFwhhwXd7mui9vLQge2uOuocUA5zzhlMUDtcBFrTre
00WHhaG5hkaZWpAyxDgeCaZDhAhp0PdCLYSe2LJJ5xwdMmOixZfMamGCa/eM
4BsanOhyUG1yqIVPPyJ2d12lQf2a6m0/j1q4lGVx2HGeCqSHi3lLcbUwoImb
zeLcDx4LJ5llbC2YjvrulXtPhucJmobNlFrY89Se740eGcR6Dz30/1ILJtGX
nll8IYGCyofa5eVa4B2T/szFRYK7R/QursjXgSk/syT6Sh/MWklZt6jWwcmM
896D2r0QlMIbHGBQB7W9H6XJHL6fuqW1c+V+HUBKzboK4IDwS0Z3lcMTV5dY
dYornXCqdfuNtW3vIYR0P1Sxsw7+mlIfc31/D5jGzYPPJivRNfH9jsEL72Ff
lHvoSe0aVDp+14yLrx7uvMDP1efVIfvHSwpcB+shk5qXJm3ZhEipB2fXzepB
TJX36HJvO5K2ezAc5FAPS4KlOsUzHSjyeB1p/WE92JTXnp7bhkWXmrQr1mPr
oeZZ9+sirW5U/MHNeb2jHpIv6V458oqANuY23gwi1cNut/YF0exeZOu8QXf9
Uz3sWnc2sC/oQ8KrycfXF+rhyStTPovifhQu3ja+dqoBHn5zxQ2qkNHn8U2M
wIsNoPLyQcnrGjJSrrjWtXa1AdIbXCd+HaWg3/D13ZpDA9xV1ok9uoGKrO34
7dbSGuCSuXnQjwwaajxuZhRY2AARAZIDlas0tGs2C9bqGgAsj35NuEFHvaFn
xNdIDaC4HGjqs5WBjuk+EQgcaoAw8S9bHGwYKHRnz8rq9wboGflar1zJQEq5
Nz6s8jWC01tnHz9tJkpyziU82dUIu7eFC/e/YKIZhZnG1YON4C3F3jpMZaK8
rpDU1YuN0OJao3vAmIW4nvdGPdFphK3Hf35UimWhW2a7H6+aN0IAqfvmTAcL
CU0Umq56NELZEunb9mNs5FIxq/4kuBEw7+6dcDVmI9zjiwqrcY1wSjg1VN+f
jY6oRh5+kt4I9RPH9Quz2ShoM0lotbARVrW3J/p2sNEgSZTnyftGWNA8A61f
2Ejxtf3vlc5GKBK+Yuq2zEYJdmVfAsiN8DZK3TtWaBBNH/83sDLUCO8fnFUS
OjaItGZVUcB0I0xWnNJaOzuIcpqela8sNoKITcyogdogWg2lZQRsagLZFCP9
DbqDyEJXPG5lVxOkdI4M7TUaRDU77wUEHGqCXXdnfV5dG0TbPlY9WDndBKXu
w7xehoPIKXf5RsClJtjKFR9Tf3UQdTlr6KzoNIFDWFucneog6hdrmE8yb4L3
JhuQm8IgYhFOZEk7NMEZ5mis7dFBNOybcbXTvQmG322+o7hzEE1JCXL4bBPU
jE68nl9lozlmaObs0yb4azRXm/mVjVYi/2g/S24CXWecmRKBjTYq3ps7+K4J
Tmu5/OgqYaNtXwczGiuaYPLLZWO9Z2wk/FJf+1pLEyz8879DusNGElcws5O4
Jng0WZtlrsJGx2flM4LpTVBAaReb3MlG8m/ztERGmsDo2cBK1DgLXTASma2Y
aQJPEY/2S+9ZSH392Rut5SbolvtZIRjGQmZW7r+9dzVDqdbpeuldLGTDP5q+
7WAzeGbIVYaxmOhes6lmvkwz3C/ely+SxkR+ohfS6ZrN8CtkumbfbiYKw5Vp
uFxvhkZVC1xmPwPFPpb4xWvbDILDcafDIhgok75RQ8G3GcyvK597852OCsJ9
fvZGNIPBBNtmPI2OKs98f22f2Aw+wbuTW7XoqCOBNJNY0gzHlGZ41NJoaFw/
PfX3p2YI7yJs/3uYin6uCFx5OtUM7+/uilnGUNBCSfAPiX/NQOXROzt7k4K2
bLmrZiTYAgnuPDXfYsjoVLfsdLlaC3Cn3Yk7UDKAvFW6wbmgBb79Iu4vPI9D
fKd/j4+7twK5omJ0waYCSbTeCHse2Aon+GNsvxCL0Xld7AHF2FbwfvZeyfJC
PnJ2TLGIKGiF4LMrafan0xA16xLh8FArXEi0fytsFQ7Zgs/KbHXbIHhNL7lB
tRaaMueubrZsgwtbft5qv/MeqCetxivutkGE4f2xjxENwHdV9sB6SBvUHvWp
FH/fAi4hjPiMhjbwmVw/6+rYAcqzh7wGjyIoTf7x/AcFDybBsYKhZxAsK43+
bi0jgMu2v6XHLyMov7KguzW8F7JP4L/63EIw+i7rXN0hIjQ1yIdKOCEYX7a6
93mKCDTNN+K4xwg+zS08OVTeD5vs3cyFkxBQhjxD+oRIIPGbNdeWjaCr+G8c
yZkEykFq8XfLEeRU/zqoiCWBa/pufB0ewSE7dGndiQxT36z2n+DCgLaC2Uxa
CgX+VSZ+X+XDgJy52u36DxTg9elpJAtggL24M+HPPirs55Mz8xXDgMGf/kZM
MhVOEO8c0T+EgUxivvOVASoovkyblTiOgWfremQqLw2MDvG+wCliQHKm58w2
RxpYfVOyenMJA6xvgleqk2jgVOki7a6OAT/QdzJvpcHjxzmLV3QxMHRWbXB5
lAbhKsweYWMM2IGXXeYmOiRs5E/+boGBUYuKoUtSnHncp+qAbDDQZzIzz9ag
Q3HSI/mkuxi4EfVi3MOWDvU3irkcXTCwPBO78T8fOux95Vna7YmBiaqzx81j
6BA4cNHimB8GCl5y9296Q4eR//h4I4MxEK6TYbmvmA6a6gMVXyMx0KjzKC+5
jqM/MPWmRhwHf3jkhG8bHQQa7TblJWEgtWaPWzeWDu5zJ2p40zAwHOpxKQRH
B5rMvLVDNgbOhjP3lOHpoHSvdQs2HwN75UVf63PO3+REvj9choG00qXwuxw8
1yfD22E1GHjkFKa/0koHe2HRbaONGDid45i5k2O/+9pIoxoGA10/QuWqiuhw
IrbkTk43px6KYeaD6XR43u21g4eIAQUDnZaXnPhmuVRa7agYsFJeExznxG92
YdP9djYGlh6+fUSzp0PTI9Kug18wwHc7KPyuHh3EK19jgscx8BtbNJQjT4eQ
qdvOX6Yx8IC73il+Dx3GjpwUUZ3DgEmD5hvFBRpo2/zpzFrEAPM478IbJg0E
aVFiNnztEPTH1Ar/ggZe2671tPG3QzG7cD7rHg1Y2ns9xXe2g/1J1nVzoEFW
aynh04F22JtAOxw7ToVTBWTfW+fbQdb5ysKO/VRIHE472gLtnH4umf79lQJ/
xRzIYprtIP+UKrVeSoHW+L9SH4zbIeHmumTvWQro+YmxLJ3boS+Mkmp+hgyV
tWNhjR7t0HzrP+v+GRLs/Fl2WtS3Hb5L2e4mFJJg0F41ihXBke+zPwaKkOC+
/h1F86x28EhN9TAiEaEv6tTI+7x2+KXdUS0hTwTZjn9xe0rbAT/krdmY0AcL
is++0hvawdq0SDJPpxciJSpemlDaOXtUaphIPg5y5xZmr23sANNPDo/5RDvB
7Y9PRMLWDrjFX6Thf5fzPvxbECbv6IBt5lEBWjXtQFpauGgk3gFh7Lfeqpx7
sMK9GGlwrgM+p/Jma3PmgKng4l7d+x0wl6R366JmBUgI+ZY9c+sA0uahj61+
pTC9cxEIjzogTg1yx6uKIFR40eFqaAdcfK+94+bxXCgXXyzXSu8Ao5FP+/Kf
JgKfzKKaOrED/PrSHhyWSUHkU770MGoHFNkP1H+TeYPeyC46drI7YD7F/G9y
fDaSV1iMVRvvgBjaP1OCax6yubDIUF3vAM3orI/PyeXoxCXf+8EbO+Foz80O
9ZhK9FdlcRlt7YQuZslqflEVilVblACRTuA59sunl12DGq4uOl2S64SdDXoX
CjY2oDBd39WAc53Qh+HTN93YiAz0F1+0XOqE651Hhey5mtCY0WLtBZ1OuP1+
8r7zZDMStFhcO2/fCRKhB0Y2BSH0wdI3wfd+J/wOf3RB9SQG5d9cPNzo1gnq
L9+FRiRj0EWbRW2lgE74+aLXNMu+Hd2/u5io+KoTIoZp92ifO1AL00t8Lb0T
5oZePdms2okEr/4q6srphMKzX601MzrRe+kJjEllJxTMdqV8McCiLRm39fbV
d4LJ99MrKu+wyGrbEHO0tRPEzvJii+exiPc3bcajtxMUkpSv1D7vQha3jfyU
KZ1QJpB2z5zZhUqovRt52J2ApRyO3bm/Gxm/79iXMN4J5wyzCsazulG+pEqh
xY9O8Evuo/J96kaLqY1nJOY7YfO/FUVj4R6U7V+pU8GNhepEk7/BYT1oflqa
4b0ZC0J/C5rd63qQlnWBncp2LMznsHhyx3pQ2sChHxv3YCG+9vH8wR04NKOa
6UPchwV50bjB38o4dLlalPfVYSzkNF+UFruNQy8Pv3px6wQWAuwX5/MicegC
X2z+93NYGK60oXDhcOj5403yNSpYUDyryj8yhkMjk6GtfhpYUIn4tklrHYfO
3ljTVtPDQpAGF1VJGI+ie31p/13HQv+KxCTmJB59vPjHhmyJhd522qdJwKPT
5e7fU22xgL3abdVkhEehB6a9bR2x0Ka8+x7Y4BEj3pFHyhUL2brtu0If4NEJ
ntG4n15Y+Gm8+WnsIzx64mktWu+PhcUQ6RP3n+ARaYydGxjKiW/fBvdDYXh0
xMxUVvMpFi46MwY7IvHocQ+pWSCeEy+Ra7fp/39fUtLToidj4e+Jb7TRKDwS
L+6hvMng6Jt9TveNwKOHYlesHXKxILiTa+JgCB5hY9u+SZdgIY0c92ncD49E
1s4/mqvCAuH40RdUDzx64FbH1dyABfcebursPTxq+yIbG4o48T+u8De0wiMh
41JhnW4sDLTuuDlviEd3OiXf7SBi4fHsaflpVTxqUHh3ik3Fgle6bc1lWTzi
zxdvyh7EgvWP3Ofr+/HIVjhN494wFiJViMayW/CoJno3+fQkFjReCT0amseh
Gw/4J9v+YOHW+LEHH7twqOxjlGfkChaW0w9lqpbiEJfBhnX9DV2wW+utiG4C
DhXILu3+JNgF6sZ5x/zMcWjp7aO3ucJdcDn7Ln+1Eg4Z7Px90lm8Czxlv+Hf
i+DQ/J+JK8vSXXAwKs1El96D1JrpD0W1uuAfftfr5Ms9aDrr66Czfhd8eYBP
YezrQcnhf65grneBqFzHkMa/bvRdf/ceR9suOBce6kEo6kYvv5g01fp2wdLZ
t7//4+tG4xtpPEalXaAUnrehPAuL4qdGH7yr7oIua2krA3csUh6Yo/1t6AJS
reglR1UsepG6syCjqwv+k31bKPC5EylJX9f9PtQFRxe/5Tjv6UQxRpSkyB3d
EB3k3NPh2o4Uz46ssIW74dOOnB/t59rRF9HZOzLi3XDV9ARBj6sdKYzuUKKd
6IZbX3uYG+IxaOjRtY8H1bvhZCh9uuU6QrLppCOt3t0Q9lnqKF9sM6J97a+b
+9ANBant92sP1CA+5UHhgZFuINeVBauGVKPzz7/6Fn/rhpaJz/KCw1UoU3Hl
kt2/bkjacQF3K70S3Y863t0v1AN/RjQUiXMlaE0qnFF0tQe4a+x+D1ZnI0nn
8/9s6nvA3lVC6m1fAtzAqFteaOuBl/a1XjPqryB2l1Hznq4e0MZeOjCOTYXZ
lrtBREoP8A5eb9P4kAUtAi83XZjpAcv4zazcUwVgVDEjvOcoDn6dTsn9crAG
/GbfKfUl4iBYNb5p8n477Np2NyTqNQ4iJIa0xb+2Q/lxKYJaNg5miQLaSzYd
MGJberOxDAdsIWWHOyadoDNQG5iPw8EN76l5RcUuGJvy7rEbwEFifMVZMqeO
gXznBfczcOC/Sg17dqgbqi+1vk0axcHQ8L9rQlt7QLS0Cxu8hgOhoeq3Hv04
qO6JEri4EQ9D3DvdN1zEg97oVbN/W/EAH3cGixfiIXhv/4SLKB5qvm6Tzv//
/6fRjC03FfDQXHRq+0eBPgjNTb2+5wIenNzcC7Nc+mAf5sYb8mU8lLI3YK/1
9oHRvyEZbUM8LB0zVz8TTISGOxNGZ53wEAmfcgnEfjAOKXr9yx0PJe7RLsN7
B2D6zYORksd4ELjeyad3ZwAkaD89D0XgIcY73ov8ewCafla1fIrBg7mPQPSo
KAlMtnptfJ3I8V/761778ySIVltI3p6Nh+nQvht9niQ4ZN30mZCPh46Nkba5
z0nQ4hsgFVmGh/jy4NJPBSQwe6Xy8HItHr7EdrS5IRL8quRqWmnCwx7NFU1j
Ogme9XXwNLTjQVgiezHgGwmOTIbreuLwEDRA7J9YJkHbBq2XpwbwcOeI3vIz
fjJYHPjv0zc6HnAqcw3WYmSYVe49mvcRDzL3BZ7dlCJDrFmcq+0oHgplCi/4
cfjTMQ/DerEpPPinx2TXXCQDJm4HF/MXHi4HEBlc6mS4UUTVTvyHh/mGc2Y3
r5JhHvsqQX8NDwTnwIwWPTI8/2I+uHkjAcQ/pDwWNyCD1IroYexWArTffi3i
r0+GDuGPD4KECKCqdLu1T4cMt85k1iqLcup4RnFGQJMMfw1sV/8cIEDguT8l
54EM8U6HNKuOEcDz9iqPriIZTkSOPXeWIcAG46fal6TJgH2bz5RUIIB0fIwl
vzgZrFvvSYwqE0CPe/+h2m1k+Mc6cT/zMse+iBJVdpUEk44hL75qEUDS+13i
2BQJWH+ZdScNCGCQ9Ui6kkECfMSpj54mBGh55quQgCFB064InuYbBPjxfrwv
gsMfS959kOKxI4CxnJxCLKd+b+TlDa86EmDntHFDjgcJAo0+pzM9CTCv4Jax
rEACt89nO8T9CPBXUELm4k4S2LrGTt4JJgD1wZMKoc8DcDn2/Nn5WAIIGjQ8
SVQfAHmx+JsXkgjQlymn5fO7Hw4Vj4eEviZAf77Vuk9aP/DikvoF8zn+cYlq
Ub4Q4Y/Z1Lx5KUcWcvt2OpAI419VxbKqOfHtjuytFiYCbsOMowwiQNhA7sA5
6IMY0ObWYRHg0O03QjRbAgT0Z0omDBEAQ6bmf/uMB2erP/qsMQJImeV/uH4T
Dwb+OWl3fxOg44ymiI4W557Wr5wJ29ILr7MfDQ7Md8EGzes3CIK9EKyxspBu
3gXztKLgHcK9wC8hXryjHguMWTNi1uFeiHZzmpdx7oTXp6rutlzshdvqTTKt
lRiQyL/z+o9rLxyt/RQuZF8HO862oIuPeiFZVlswuK8GeLBC42H+vRA2hNvl
IFcNo8NIXii6F3YS068++lgG+fv39p162wt9aSKYTR7ZIPOqf82R2gun7BUU
vvGnoPwkCfFzg73wvi3IxZGYiSQSPS7xDffC3VhrFxedd0joxZ6A3BnO+Zqb
0+G9JehvtPXil//6QCZ76l/MnTqECZiZs1TtAz3aG5NvAR3ovL/qzuNaffAg
6uBWHo9OVO2bKL+g3weuseovK+9gUa634sPkm32gucd9XEm7Gz11D5yhePeB
9RrbdOgPHhk7bPumW8bBd6XKJd8nod7btpvFavsgqu3UmcMkElK3q5acauqD
I0JqMUwFMlK0NrsbjesDl8fXn3gskJGYReYodrQPtqjzOr27S0UvzX7xvJzi
2I+uGk9voaJtpmqH7H/3wQcN6hNlQRriMh635V7vg1JL2SOnKmloTPfU0KW9
RDBa/o9iR6YjK53gVf6DRNhTgiyWhBiIoU3Z91GSCBdl7FMI1xgIr+F90+8s
ETaY8AeNdDOQmjrO7+pFIpCTflKFVhmoRW1vmsgVjtxg8kVPlonKoY313ogI
t069XqiJYyJJFcGFCHMiTJ08fI9Yz0TZF28Lm1oTgWV59g9uiIlEL9QqHrlD
BO1ldUwGDwslnuczm3vAwVu3tF05zEJblSwedXgQ4e+y7Gq1KguFKxa/TPAl
QuPDg+ujN1hoTWGlxpbzjsdtgBOkhyz0+IwB9XQUEbK3Oqy5R7CQdp4C6Vkc
EU6wCVjsSxYSERYjfk3i2Nv/333Ztyw0Gc1NUE0jAmWwbrCimIUalia607OJ
YGn47DJUsVC0c3/n33witO4Ie/OhloUshmox18qI8K1FVjy0joWkjNJbS2uI
sLv7i5BsDQsttIc0bWoiwsKT4JejZSyEO3Ov/jaGY18v+kB6Hgul5hnUtnYT
4enFrvOmaSx0T/hslQiRCCLTgzt2xLKQ0lOxck8qEc653HHr92OhzcvcJf1s
IowwBQ7G3WUhlvNkwXHOvY9/kv5A35CFCof6c8PHifCq+/ItwbMs5GNU9/bz
NBEUpwp2MoU58XekZyrPEcHpk+nfnH9MJKIQmv5qkQjFfAM63jQmmsy7l/pr
nQiw/jzyejkTNQgbvtLl64fvHRUqlyKYKPrp2cR8/n5w/2owrGjJRBbLYi+4
d/bD8ta8c+onmGhhaDK6/kA/ZDAtNhZ3MhDOaCBC6Fg/qHUN7+WPZaDUjrpQ
l5P9UFnN7/WS009K+aEBh8/3g2q+m+hpGh35uuxzf2HcD5NGRF2NBhrS+czj
MmXBsdd8GV440dDea9/ua9j0g8j94r1Ke2moSeG9/fKDfuhwLZYnPKSipWVD
c8eIfgjMadm2ykNBBBdFk46YfhCfvmt+9S0ZpX3ed21/Yj8Y1p+R0bhERsqd
33Somf1QujHxnJ8rCfk9C1OBhn5I+bjg/MCViHRW7l9Ia+sHLpmUsLLoPrTX
1UjpD7Yf5kVl+MSyelHTtf3yJeR+CHlbbzWFwaMlkfqjwt/7IUDpkGrihy7k
XzDF/3P/ALy1b17/KMjhh9F9PJQjA3DaWiVZpLEBnXQqX6iVHoBU/SLr/6ze
o48nPcb8zg/Ao3Nc6w9fVyGV6sVmPtMBUHRXzr7WmIW4Wjc/EI8dgN+ONuVx
egVgkTlly500AMFZwv7bT5dCVVCf2djrAXhTvXRHLaoSbqvFqxUXDMDyXYGa
Ets66OwR3qvYOQDV/wUFb//dBpGUY3j9pQHozYn/xU7n8EDLcJtULhJo7d96
5101Ht7oPBr8j5cE228dCqN3ceaYtDlpegsJ5Kh8BOKXPsid2dtSLUKC1jqJ
pmuceWf0eeu5I/tI8ClZ/eahHyRYGVipenWABKe3Jbg8vUMG06qhAp9jJFgW
lxKjGFCAzysnSYUzFxWV+h8+4KZBtUPStspzJPhBfL+62ZEG1qbhTw9eIEHE
8wMK9Xga1CveDeJV4+i/eDNgayQdHCTNlx5pkODvYbdHrz7TQVBE+9GENodn
sd7cXDjLgHuLxx8QDElQf1l/cDebAbum9o5fuE6CW4znAw1HmYAZ3GpXZkaC
4QzpTF43Joi0/DCLtyJBTNjgTZd/TMCWDpG57Ujwzzk62luRBe4ZA3qeDiTA
iWx+zO3BAnxglZrZAw7+VUeu/xALpKirlfWunPPEZ1uCt7EhWlLngCgnL/YB
48//u8CGSf/kOL9HJCiMF9wj5MAGbdLI8gcfEpzy/+T/8ikbCo+cdroUQILL
64tysSVs2Ozrz8oMIgEa9UXzeDbcI/ZocoVx8sO3J4MwxgbcwV11dpEkEIib
m9iywgYpb9vDnU9JsMlHglS9fRCiCaUJR+JIsK922BVzYBAmxRfXI+JJwG1a
9UL+5CBoe2q4TiSRgCx7YPvms4NQ2JPwUTuFBITz1wpVlQdh874hneI0EmRO
adA/XhiEe+4nGrdmkqCnav7Ap/OD0IP1lnR5y+GhiY5jagqDcEy081V/Locn
PS7QFpQehEiX7byyHJ60FnM9VEt8EMbbb3oklJAgIXhbzHeBQc5cLPwyW04C
i3nSaa4lNuQ7zRuYVHP6LeqJ/tMRNvAh1da6OhIE9TQ/iulhw92dcdLCjSQg
7enexV/Ehm5H9mufFhJkC2EKBKPYcKzl6OZBDk+m3w01yrJjQ6Sgh/eFThIw
bG4IIiU2jDu0jb3p5vBqaeFlb342aDZuub6G5/QHs7Gh/yML8gXM222IJLjU
8R+1v4gFfLffnW4nkWCH0/3JIE8WdG25yB/OJMGobIbL3lUmHLGJ9vs6yMnf
gPKycCsTwmtok5pDJLgtYnJtxpcJ6rdcuv77SgJtXzGJ61MMyK1sPPNgkgRi
0j6bFjMYwLuRL6fvOwlUBbmvFxgwAFuWGfhilgSWQ3tGjArocITn+/SvPyTY
TzK0dDSkQ7jZuZvGi5x+ipm82zJPA/X1gXO7uchg3/Uv79Y5GnQacf9OEyBD
vnemxrFMChzK07dZESSD2pzCl09yFAhdfE202kWGmXIXmnEn5/sc+WIJzh4Q
0zNn2PSZBB1z9vb5nD1AQd7uygxXP7Qnd9OrrpDB62VnssNYF/Qd+STP5vDy
b8QLS5ITWGBVz73g4vD2A1V1XtsmO2GmX0LH0IgMX7VeiBHG2mHfJv+26Vtk
aPSl65ulN4P349MFkt5kMOrdc3dyKg9CN2ryGvqSYalV8fF661uIS7pl5x1A
hlXlqC2SR9Iht+KpWFcoGfbNbLh+cDUcyBOjL26/IINqn87uu1HJ6KRFqk9G
IRkGPOZkwgvq0Lnxcjq2hAyR99do7Sn1SM2rS366nAw21nle8WGNyDJ+dlq5
jgxv+laOLuu1okicnh2rneNvk/Yxf6MOlGhm37aOJUNbR5Yl5UwnyhjzFZPE
kcHcU+GnyC4squUqoD/qJ8Omyu+qO3q70LASt87OD2ToI2DzhAXxaLpbuEB5
iLMn/dqvbsjCowWTU7y3h8lw8vaPpLY3BLTt4c22ygkyOF4ghM+J9qGLRXXy
BvNkKNFo8ssZ7Uda5/pePPrH0T9UeT44ZgAZd41Mv1kiQ7snnA0VJqH7w9sL
vnNRQBb3M9OuloSSRZ3EngpQQL3yrVE/hYzu2qx5zQtSIOmchxfPXgo6m5fY
b7OLAr87n29/YE1BNNnmEEUxClgp9pDahygcfmr44a04BYySlidz91KRZ8uo
gsAhCoRHnJjKv05FQtr8k6NSFJi7sin1L4dPjsRlXzY8SQHvAuWISz+oqIqq
kN50mgLbGa8kX3HmcbAofv7oGQpsK5q9vKxOQ0Y2VgYJihRoGAw57+pMQxJ5
vwtWzlPgqo1s2HQ8Df2ciuC+d4kCmC3D1x5V01Cb7N6bVFUKXA7R8N9MpqHn
3uW1KuoUsCm+NZY3TUPWLWrbirUoYNZBPGGwkY5keJiOu3UpIFd37zmPGB2t
aD1oD+bMFU3xfiusDB31xa3vnb5GgWnVkdevLtHRG2qSl7kpBep8p0Z9dOjo
gahUf4cFBX7We353N6EjZZsWyVO3KEC1+LYl5CYdbckzCnltQ4HuYRnlMhs6
GpwaG+S1pwB3BiNw0ZaOimR9FdzvUsD4+X8hTpxzH2+B5x/uU0BSzTJ0Iwev
3fJ2QtOFAutHH5AHrtORMI/i5Wp3CpwMtTAbuEpH41qEtP1eFNDfHxy3hePf
+zjr+ejHFPj4X9F6FMf/COqs/rwfBQy9LnYZcOIzEY0qsAmkQK1rafhdPjo6
YiPG3RtCga9q4r8YMzQ0l1txQzGCAjfKf6gU0mioY+pK7dtoCkymK54eraeh
RFmWgEAsBf7kbH4Wn0pDdt7Ojj4vKOBVjhIbvTn7Ac+rvYbJFFg5ex3lSdMQ
Seu4V9NrCvA+k5oK4KGhrLhW4tEMCtQUTFxYYFCRiuh48Mo7CtS/68zg8+Xs
GzZ+g44FFMDzzxpXalLRp9xtCtRiClzbfU5cWoiKAmTPTRRVUaBq//6FzhwK
0vPuVd1dRwFkcixryZGCxFps0oIbKOBrUe9zRpqCmrSi9c0RBe7MpsR8LiGj
BWt2DS+RAhvOnDV+mUJCuFwXAXcSBZzsQ38ZGJBQyhS34wcqJ55h/ZjHG0hI
0fvE3upBCnSEn/yQTOlHXnH+wTaTFA4/n9I9eLUP/eQfxMV/p8BTBb3P7JJe
5BSrtKNjhgID3MziDP5eZBvz5+2RPxQwYdQP5+PwSO+pW+c3bio8qXK0tpPp
QYcj7Pm8xKigddg017WtHVECdGOjDagQWP/k3WxoMdJfKaI1XqNCccvT7YAr
QDj/zfu/m1AhfgvN5ZtgHmrz6yrTv0mFwonILys3slCRDwwI3aOCyMHnfz1c
wlGw1xmhjBAqMPjeH1VdzoDluYQb/eFU8FiKkVKxfwvenr9y1qM4+vxHHLLa
34GzR9mZ28+pwJ8nzz3jWAgW7pKmkulUmNjZKbpcXAmnncVSq2qpkHnFUAHT
2wDWhnQqpZ4KPJv1p3rfN0Ks/Ivt801UcOMpPGCe0QRTCzxRiu1UqAm2Po+/
1QK5EVOPmolUuD5z3uR6LgLKvdyqjyQqGAd+JmqexQC3nvWPVSon/rK/B7cm
Y8BaiOKgOkiFrdvrVsKM2kEkq/F61zgV/hAVfI4XdYBmqOeL8W9UMHXc3+y6
2AGP7sj0bvpBBckUU88bWp1AkX6rpjNHBc1P9vPSHzuBe/vNwAd/qfCMcvze
7sNYkJ3d1RS7SAWN59oaZY5YiGuIlhtYp8L5SZvTc9+w0JKu5vKLhwYH7T2C
CyS7YCpwpXAHHw3ylIvYe+y7QFvDXcKEnwbpO7mpZ6ld4C114taj7TSwKv6w
a2ZTN+RtHUtJFqIBA7Mm46HcDTwU8+1sERpsGij3/J3aDbJ1O3SXxGjgpqIh
/A/bDTapvZFiB2iw/eE7uU8z3dBiDWtWR2lAc66Qun2xB75fXjwfJEWDF2dU
Rvlte2Dv0ZpH2dI0qDFL31EZ0gOPvx/7MSJHgxTl2qrFth7I7/8ixXuWBuVv
tl0vG+wBWlWaw1ElGkiFVKx5zvfAhlcm2ZoXOPZHunJN+HEg77Pto6MKDQQk
DI3MDuHA9iZO+OllGlRMRGgGK+LghUro9WJ1GiwUbKGxtXHQdvDii14tGsTt
rTVwssTBNO9fwrQODUw2858/fw8Heycr+LYZ0MCQ6nXI4BEOrvbeVzt9jQZF
4U/eVAbjwKf8cKCRCQ1mPfyu3XuGg/yET40PzTn5LJldDEjEAd0r5U/iDRpg
+VsM/qbigNfimlytFSf//fU7PmZw/L2w1YVuSwOtpK38im9xYCfeVfjXngb/
8fjw8b/DQTx30JgwZw94zs/Ve4cjt40pSZx3osH5wBviujk4+NEze/OGCw38
lKG9MwsH+0pKU/zdaZBm0u/J5OwzOs/vUt940iDKbvJrXDIOfB9KbG/z5tRH
/0bjzAscFJoM6nz2pYGI180X3NE4YJx7Gcn1hAZPa2P+EANxgKf0tt0JpsGK
ZNrf+544aHHZ8I8QRoOExWzu1Ls4KN984bRsFA0kDdu97Cxw8Padh+OrZzTY
LL/bc4CT35cqxVlLcTR4c0hO5Ps5HESyh5k2CTR4V7tu1XoUB87br2mfSKVB
psavJ4WrPZw5Fx38Ip0GPFESAZSJHjDWwDTMZ9KgZGPUTjqpB875nz6O8mgw
qTauGZ7VAzwTAltMamgwOl4uZSPbA39CNNQa39Pg+4j4ySNCPTC574mfeBMN
dvGmWK/MdgPReHpqEsPJ7+rB92tV3ZDaRuh90k8DJVsRru9Hu+FUclRcwTca
3FIhhvhw7s9BOdTF/4MGpOYNpwhtWNjV93f14S8axL7aN+yRhIVF7ruuF/9x
8kd/0ERSxgLWRd2QzEuHOI2o3uGQTrihwb1j+QAdNHxytif/bgf9L0pXbQ/T
oabVTD6vpR1U/d1Duo7RYS7tpMiWqHY4WvX59wsZOhy2RRN9ou3wa18b5cgF
OlBPpqb8VMRA1LzfSwMzDl7BsO/j6xY4hg9o9rWkA1aW0d8i0QLYjMCR3Ft0
QLobRffmNwOXVqjs8m06yLxyfyZX2gQ+r5/2FbjR4am4CnFXXgM4qaTycj+l
Q4Os7fWWK7WweWea9MlYOjBSp6o0K2ogfyLd2PwFHX6qyvwYE6mB0fis7LJX
dDhaEPRIc7wKbo3mX7yRQ4cnhK8MMUo5GETXedU008FUlsl3ak8uTN+qTx9q
owPmifWflJIceCbX2PFfBx2++TYkNl58Cz2DLYK2ODrMi/wYuZz7BlRlsKX8
dDoUJUsWmibFgxyFMnZnhg7d+xotd0Io2rXvl4nIQQaMPDM2PR9aibwL3Xyi
jjDAa9mfPzayCrEVfqb/lWRAWUqj2onYapSpPzNCO8WAd+t9J7+n1SLJoGn3
hIsMIJknPajubEDKI5OxWywYYChz72v+Swx643qv0vcmA0q1CqVa97aj9aUJ
6qQ1Az7btL2IyW5H2J0Te3vuMOCKVfXcppIOpK/xtTDMkwHD9PkBj0Yssi0c
xq48ZwBG41whrhiHOhTsJh8kMmCTBgajfxiPjrZ/2frhFQPc7IrpOml4NMX+
bNz4hgF6F896m4URkOfWoS+PihkwdWj9iDH0IUaKFe/XMgawLemPFYv6kNKR
T5ImVQxIb2ixHhMkopWLH13lGxhgwCOeqsEmokjXwZWZLgY88CmeaNUdQJNL
lhLWeAbQnxqLv3g7gHSi2FeIfQwQ26v06sn8ANqezXpWQmWAqOnuyxXaJORx
0qJ8L5MBNJ7mH0kuJERrYJKfDjLgxK2Lfz7Fk1AqmSFyb5gB0c8krP0oJLRk
ZXaROcaA1bO/DjT+IiGrKbqN5iQDpOW8jyoKkBHG2zSs7jsDlBce75mUJKND
G+j5R34yoOlzRm6XKhmFvzAhJM0yYB8+hd1lTkbjYrQfPH8ZcI30euazMxlp
F17f4bHIABUJP6mtwWRUokBVGF5hwAc/NzlIIKOOI+W6qlxMOL0g5SeXTUas
XU9vZ21gQof3kN7pMjKa4XXwXeNjQhWXiZtcAxnx/lGJt9rChCUvnxHZdjLa
+1W0oEWACWp69VXHcGQkS59vFdvBOc8ItuQnkpFm1wDNbxcTUmIh88MAGd2q
K/7OFmaCc+oGg+ckMvLIi+A5L8YEcW+3UnHOefQrW9FUcSYIn/5sHNVLRpkR
F2T/HWSCza/K0Y4uMqp9tEfL7CgThpcMfPtayYhw57dVnRQTSO81E7NqyOiL
aZ/XrpNMkOkerjtXQEZ/NQpiPE8zIVp7cDQ+lYz4FUNzKPJM2H+N9rwgipPP
Y1aNcopMWJ27J+/jRUZKe5RI8eeZcJu1oX3dmowM+HZO/LzIhAPMrvILWmTk
8PfHqoEqZ09veO17XIaM/MZxu8qvcPQ9bxPqESSjeMY7aQEtJrjJpy4LzZJQ
y3tLy159JpRM3reqLCMhSr6C+4lrTGjJnbnPFU1Ck8nbo56aMEHkQ3Pssg0J
7XrcVat1kyNnf/o7t5mETjhm9+ZbMwGfkhH4tWYAqZr7j2y8zQRv4rkSlWsD
yPmcnCD2HhMcA0p/CQX2o1BJfsnDzkxY1JYITRboR6nCE5dC3ZhQ+uvPXFQq
EWH/vXmg6s2EnbqfD/Rn96F9DZt7WsKYMFo6+qHPn4DkC0c/iUUxIUNVFzIm
8Eg7tW3e7xkTtsQy5Y8Y4pGXj9eh8wlMUE1xC3q2A4eISl+e1GUy4Yqir7LO
nS40JtX8clcOE+4OCRraVGLRkkhyiWceE27+U7uZ/68THVvUZcuVMoGwOfra
bv8OFNhYr1DeyIQf2S/771e0oVPKL77n05jwJr7BTHehEv2yV/jdy2SC6NO1
3bYJ5agqjv331yATGKsyDWInSpDC8GGeC8NMCHQoa5tUz0XK0Q0iAz+YsLb9
vlV6cTTSZAxr/ONjwdHdyQNHZ/JhM1eU7r4tLEiIC79kkVkM+OMnr10WYAFr
7dj9ZcNy0HvifStmJwuqngz8GRmsBuMjWz0PSLBgm/FPcX1oAmsPhWyt8yww
7mtN0eDwRYk37DzniywI6Vh1v6jbBcNdgSUJwIJTz4L21Mh0g4Mo/v0HDRaE
9srGFfzoASeMFdHNmAX3LHY/vG/WCyeneKgvTVmQ22ZhsVe6D37sLGQ1WrDg
zH8PQ6xW+8DdcXZ0gw0LJEckZCXT+sF7W9RSygMWOLjOldNYJFBSOsnV6sqx
16Y5On6ODIt25I0jD1nQ0fq56dorMgTUie046cOC2p79Ucs6FAi9VSmJCefY
e9EbeSabCuqRpjJfo/6PZ/W/+UWFjZVL8ltiOP75WD9e4PDA6A0aKiYJnHzI
GF6xoHN4VDHbZDKDBUZZb8OfTNBhV+yK2bW3LPAT6We4SDPgtcsBy6ZcFsTm
GF2JdWZAruwdq9gSFnz8fn304DjnHRSKtvlTzgL5rPSyEM49r5wrtrOuZsFM
oInrl+tMaH7/645sIwsOp1QYutdx+ix1573XLRx/2jKfxHHq3O2r6MSDYQFd
7ZtlCKeOejctnR90suBqc6XqFVkWUC4GuNK6WYDfm1lE4OTZQjzL/RKBBfu1
Uny3e7Dg03q7Rz6RBdcizwdtec6C28NjXtvJLCi+31ZQm8+CyY5Nj31oLAiP
SdHfxLHrmnvCd5jJAozSXcwKBzcfoe+v84EFEhWvHr74yInb0f1JzRALvp1i
qjRNsGBdOylo3wgLxn0ENvr+5Og58T4k4isLpt0Pd/fMsWALPztsZpIFDav+
MznzLIj/sRxhPs2CP+J7ZjfNsmDPgHg0hoPrOaDiMv+dBW8qLz87zsEVGxx0
uMvRezDRITbxLwv83eDbLToL8j2jni8vsuB0c74yE8uCk6bF8Q6rLFAaXVqi
VbKgWpGYSORiwws1sS9mrznfi/x6qcjLhqik/H6rQBa0LQqlZG1iQ17k+Nbv
nD678uHs681b2eC3694clwoLcC0W6Q+3sYHa90woR4QFBpn+GYM72DC0fdyY
/JMJtKDMrCu72TC/6FSX2MmEG3btb0tF2LBfBes2lcSEL2pj73bvY4O67s9t
n2yZcOfIpvzAA2xwPGqt6H6CCe4TesVGx9jw6vajDy9qGPAX51baeJwN8mpB
3cc5vCCgOLH8kAwbLvddXgk8zYAoF1b1/Bk2XHyNeeifSYfMOfvm1Mts0P1R
WYTMaHCEHtnKrcEG7lTrq3KrVCh6X4SctNngNPTxdymn32t9f3ZeNOTINgqb
5oYo0Lvu1/flFhviXcwEpJXJYDSc0X/Vlg2a99YplgQSMDowpGp7NhwxKdbj
siDBSAQfPdyJDawp/+pJZj8sbE38JOXDhiJSvfrLXwQ4LhnUb+7PhtvLLayQ
VTzcUHNGkYFskFXUDZ3lw0OLj+bbsXA2pIzjaxjbeyDk65JDdgIbcr5Mv5RY
7YAtyH5auIQNOlujzkR31YKYx9llviE2qCpNvcqsKUd6cYemzw6zYZgy59SQ
VoUCC7d/chhjw/0xz6J2dg36MvStrWOKDXeW0qT/V3F1x2P9fXENKamsrITM
MsooleIQSZERkr5SGUUpJTIqJKtCSiiFiMzsva792DyeB8+eQpKQzOT3/P68
r/O+57zPfb/v/ZzPdqsqlGGaEhK4SATzRKdA7ZhGpKrAw7vAT4KeHLAr+d2B
rujPr8gLk+DRswgr5flOFPMf84e1KAkiZzLHA2a70Mzr2t7ivSS4qFMrJMfq
QeX/PF/fUSZBQpaa8Q6zATQmfiXkoxoJHkhhF6frB5DYkXMPutVJsPB2C7Hx
/9/H2/K2B46SQFrENuXvlkGkRyCIfTMkwcuN9sO2KTgkZ7dvbdMZElQ+0fRy
+ItDW4fdmbLnSCA08s574yU8wuJWcq5ZkiDe2VGXa+sQqrhgGBNkTYKxROX2
Uw5DKAn70ivlIglqyU90V74OIZd+yRMUBxI82aP19tPZYXTW3FV69SoJ9vTW
KT2JG0YHe79uknAmgT2fp0cOaRgtdun12LmT4F5g5Xj/tRFEORtR9NCDBDm0
pZia1BHU2NH/Nt6TBKGmKe5D5BH0ov3aFZwPCR4/r1J4akpAnqdzDOb8SEAK
7BDkDiIgm9ZZBYHHJDAhEX+lFhLQMUMdXvUgDp+hj75mVALa2xwybR7C4f9Y
w4N7KxFtNOgevBNGgrV3bw+2HyKiMSRUGRVJAt44s/oXNkTUrefwIe8lCaot
9khZPCSiovqMoK4YDv97jRW74oko/uRP5++vSZBvl486i4gooPaIydZ4Eiyt
mkQHdBLRVZ1AVaV3JDjxWS9Fmk5ERtXt/MYfSKALTIvKWSI6cGzXH5cUEhRs
TzXR30BCOyvtiM/SSOBtjj9RvYOE5o58qk/PIIGEV2D8PlESGimbSGvK4tSb
yTL130tCdVoa4Yxcjj6Pb9k3ypDQpxL/W+tfSZC3O1BzkbMO02g2lyomgS22
aJe0FAndKuLV0i3j9FOx8eRxMRKyOGQt6lBJggMOP0+e2UVChws+rAbUkEBp
ufSh6SYSElcbpb+vJ0FhSGaIyTwR/ctTba1qJEGG7t2dBiwiYiv7ZI+0cPxk
WBB1rJeIOnLqoxbaSaDW70HTLCei/P1b7u/uIoH88o/vWklE9DrL3PZwLwlU
HtpI6z8mooeKicetB0gwZCbyzuE/IrqcSd/rhSOBYZC4X8xRIpL/fG+skEgC
c+q1XybjBLRNtrqrj8LRr7t7D62GgKY/bSj8SSdBsG+IctpLAqpMeeOrMkaC
S0Uusn3yBPRxL/m/c99J8OXme/kzP0dQ8Ec5ffcpErQ6ZwbwlY6gc0llW7Pm
SBC21awqRXsEHRJfm2r7w+nf5T/jJ/PDSPjdaezoEgnIbyrEegqHETV++L3s
Ogn0pLYnMqWG0b3XS8op28kQ6bL3KD8Dj8Q7GrZv2EXm/K9wOWk8w6Pmf6FT
zoJksDxlebhcDo+EPQQKDoiTYcUNdstdwaEqY2WNckUyTHbVOjty5uf11f+O
9hiQYZqe0im03I2yNWXFDp0mwyfGp3Mxq13I0n1i6bUJGXjeLktwr3ai9GHv
GjsLMnzplXQMn8Ug4+JoXbYDGYYlQ3N0mlpQzA1kuOJLhn7WZU8/qQp0NDlM
/sojMtxbKeDd/J3zXuFMuRsDyXC4kLj4paQYaRoQ2sPCyOAkILy6GpmLhvfM
nhWII0OSE0mx/utrtBcra7m/gAwLP550HxjLgXae7+ovi8nQUzO7UVW2AO7q
FQpMl5Gh/ZXt9m0qxdCYdwJXWkuG1q8ZGqlW5eASbnMROsnwrPnW3c7r9ZB/
Itzh4igZatDfIuqBduDKF1ENGifDycYa+/zldrCVzFrNmuTwo8gfWmzDwL+/
mKTlGTJcr/2hlW/dCRYN24gf18jAOOFf7WjQAxkHk7LbuChwrcHt8vBcDyyl
KPtNb6KAr6mq0qFPvfApyFRUn5cCMfKO3qem+2BOP9qWLUKBXY/faMnnDYBx
8V4FPgkKmPerDMXMDcCHfQXzh/dSQHQ8fK2IM98ZbeyPC5ejQPvPxQzuWiy8
87rqXKhIgZJbY0oHl7EwxfqlSThAge5z2OLlw4MQ38I/eECdAiE2/vPumYMw
qZWWdkGLAnUSNXt4SIOgl6Fx/5E2Ba7w5yQf48PBWOgF/t6TnHzTaQvn3XGg
84dF/wMUsCyVWFF9i4NXrg8KpQwpIJG0qJdYh4Njxm/N752lwG+JiiyeLXiI
qpCXem9GgTLmFDefEh6YiuU/mywoYCobyk48jQftxNP1kxco8MP1+2LudTy8
4BmOErpIAZsC3fLTj/BA873hcNKeAlsEBNrd3uBBc2JBxdWBArBymS2chYfw
SxGr0VcpQKjucTWtxgO5Q7S7wokC2ulp23Z24kH9eHYS3ZUC5F9zrxyH8RCa
c+zWVncKzD99oQpMPBDEO49reFBgx1RIasl3PKi+sN922ZMCOwXLi2t/4SF4
5TshxIsCjA4pzyu/8TB0KyA7z4cCXK+SIj/M40GZzOuH96PA+8phjD8nHmj6
4czaIwp83Wq2+y9n/2CtiqhiEAVS7pxbVZzEg5Jq3Zh5CAWcLwseX+fUf/zR
rMI3jAIvH1e+ez6ChwE+atinSArok2zcurs488uTO7adLykQ8MDSvKsWD/4/
1+TnYiiw9Hl2/mUuHvquxMxLvKFA8pWv10QS8SDbJ9VqGE8BqyxhvntP8fBQ
rzDO4x0F/sp0+qe646G7AJzjP1BgaN3cMc8CDzLSA5oNKRRo82PMJmnhwfvV
tY3jaRR4IKoU4rMbDx3rM9hdmRQ4vu+y16k/OJC89zTtWDYFJGnk1G04HLRZ
puu/KKBAojqT9CUSBzS/Ir7FYgrY7yVYiF3BwcKnBoJzOccvG5LVrQ7hYP8s
6Z5uHQWakx9xGfcOgr7495O5iAIOqo9A/P0g2BssbhVtoYDiM29VltMgvHgt
lDbTSYEbdDup8BkspFfvu3OllwKO7/X3BpRhoZZ56HjXAOe8Vvya4h9y/K5h
hv08QgGTfDcFvz8DYD4YusHuGwUwz27p1zX0w42VN70tExTIxOWHblHthyDZ
tPfqUxR4tnEK65vQB0Ve9Rq8vykQGmb2HZx7QVBo4Xo9FxX0RGFrML0Lhq1v
NstJUmF6s+7unkdtMP3IJyZWmgon/+zbR+9oBZ6MZ5fXZKlwjWZqwBRuhWPz
qXPDB6jwaZ2vSja3GZLeEmRfHKXCO4XA7afeI3AcOvts5gIV/myjPx4WKIPx
i6pG9c+pMOBkvs1jVxDKrdq8/iuKCs6ZN9//lX6O7kpQq2VjqRBdJa2cYvEa
LVCjD0UmUGFOIuxgZ1YS2uL6S8L6MxUuj13TGbv0BXW1Y4bCvlCh4sZCJzki
G8Xs/xRblcOpJx85oFmYi0SmLLdIFVHBGndw2XesACk+KJmdqKPCgbNCR2b8
S9Ek/kX+nkYq8Bi/z1C9X4YKtJ1vmrdQwXCxUErOpRxprwhRSzup8LIsdl1Q
txIZP/XpCBymgobxIf3NNTWIl33+WTGRChGO9aE7Y2pRn5Gi3iiFCk2SGP64
K3Xo4taR0rNsKozozMyvTNejG6+OpQrPUOHII1Zn4J1GpDzLf/nMbyqkaj0d
M25rRNMXvgsHLFAB58zV0ybRhB6KJL2g/6WC0hGv+z8ampCOn9dpQS4aXH0x
vy9+RzP6RzzHdXoTDZaXUiZW7JtRePKqT+42GhBf8dhunmpG59Zx6lQ+GjB3
4JW+arSgndfzf+zip8Efg952SZ8WlCB/5bqPCA38+RLuOf5pQV/yGsz05Wgw
GCbTSCK3ols7EnkeKNIgSIVn537+NnTQ07M58wANptl30h1PtaEKTZlj29Vp
8NjhqWDypzYU8HZpTleLBk07w1yLetqQ3sLA13vanLjk+eiWhTbUXv1Ubvgk
DWKLBow3nm5HL/Zcpm3Vp8FrgnuEjns7Mn+i+f6EIQ3Ko1PFo162I0E6r81d
YxoIXm1rXctrR8P67J1pZ2ng+0DjQkJXO0pKr+3EmdHAYBO2xGG8HTlufhu6
xZIGc/TMzZc2YtAYxmjl9kUOf30XAV4tDHKM9suOsaeB/btMgxsmGDR8If9i
sQMNjF3k1uQdMMhcjLEZf5UGRynSJ87fxaA2qlDpghMNCMY3t40FYtDJz2eu
i9+gwZGDTf4L0RhU5vZo10l3Gsj8MEsN+4BBqgcL6x09aNC3lPo2KQuDMn6z
bj/1pMF5ayXvkyUYJFktIpHhRQNb2uFLXrUYFBd4rqPdhwYzFk2XDFswiNco
8OF3PxpUrgeGVHRiUMi2Enm+xzRw33ieMdiHQSt93wYPBtFgxyD4pQ5ikNdb
8adWITR4YonRlhvCoEn784e8w2jQwzPG5TqMQdeln1ITIjn1O1yrbnPWhNGy
l9UvOee3a2WPHgdvmTtxnBJDg0+WX4dYnHwdnpIT/17T4KGGQcy1fgzSP2KZ
sC+eBpmNWd31XRhUtfLMyOgdDX7sZU5ytWHQocbKuRsfaHBh/K2xegMGfQn7
8el5Cg3qXWnSFyowSMpU2iI/jQZxPY7+Hl8xKIHfeq0vgwZiWC+zsM8YtHM4
PG82iwZOGtOh6e8wKPxDjb1wHg34aPSWzigMWrs2zXO0gAab80diuYIxyFtR
tsK+mAYruZfumnlh0NQPW5fHZTS4QvJ6V+KMQZSH9Y1NNTTYvmlylmmIQTYn
Z++O1tNAmltwuUwTg7o3KOzlaaJB/kPVgnIZDKqJivI3w9Bg9Pn4T6uVdqR5
oVHJs4sGGQcke/98a0e5ovNDr3tpUKZSwUcYaEfv0//THMHRQDEIl3Ejox3x
u71iLA/TwJEVWi4e3Y4i1VpiJEk08BYMXhDzaUd+Vco/rjNo8Nt16SmPYTu6
2Lf8eWqKBgy+a/mTw22oL07twq4Zjt9/lDsZ17QhY/vrXJq/afAq0vQ4V3Ib
OjLa4eC7TIO9OPajuGttSHglcffGLXT4723AhBWjFWEVtCNEpekgy7yuatPc
gvg+j2sEytLhUNJelUfvWpCJzHvKqAId3EdCeph3WlCjxF+NElU6bP334air
SAsq2tVMMdOhQ6RHvuXalWY0FfUgolSXDt9paosktWa0n1dBU8KADjzhBDLz
bxNK3RwZMX6GDjYebRUO75pQzPJ5zRBbOvyQhfB3rY2o6+E6ZeISHYxaLvJa
RzeiLfNFERYOdDi111hFxLYRBU4LUSWd6fBoe7OuSQtCd9jEiMp7dOg2PlBa
SalHZr2u1KmXdHCwWo1UyqhGkedEI61f0WHbyvExM8Fq1Irp0Kx5Q4d9o1Ib
fz2pQieaVSIj3tOBIMrLHXm+EilXzmrKZtGB/OHuLa3eMnTj8GdqZC5nv8MV
AWfFMpRebBP56ysd0s7y1OMflyKJ/EpqXRkdekLuW93cV4K2pT2JtGumQ+vl
ycSY1wXISOqQVkMbHRZ/6cxvEP2Kgj8wqPKddGgademqSshDS/GGWnP9dDjS
ZFwzHpqNJl5so0VR6ZBxSfSogWQ6aveO12peosMwb1w+/89gVKAi0Du9SgeR
5EPRcMgbxbOibuxZpwNFUQ/jFmiPNPc3m9VsZEDgl/dztDwf4EqoDhLaxgC9
49OsZEIcpNzP/ta2mwHNASqrZsI5QNgfXqx8iAHaGSPK5K91YLbZ6F+jOgPK
jMy95crrAdE3mNppMkBBLrTsZl0DfEl4MvrsCAP2bdHZfMy1EXw2+4hSTzDg
xfUD3in3mkGQ4fzklQkDtli2hvF7tkN47b4uhXMMwIXvT7D+0Q7LCXSROlMG
CAiXz8bdwADj/H9FE+YMWHLESrbbd0Bh7QW2gS0DnnU+OLGu3gVmiQZn568z
4GBcr/GGuF5IN9d7m+/MADNr/pr5f72wxH2C7uLKALafWamLWx989j7sM+TG
AK+8q7pXtfphSUWjMfoWAz7e4gmoe9sP5my17cYeDIjfEjFy4nc/rFgppVV4
MuAImcCTYT0Altvkp+7eZ4CnuOUrm8AB+NIoc1TpAQOqg0PNb2UNgNUhid7E
hwz4MsPFTprnxMdExCz9GPD3aQLlkxgW/iYLOW8NYIDajv6jLB0sZPPtWPZ7
wsFXCJn88sfCv5ZtRhpBDHiw3cA6KwELNo+2vPoezMk/39DwoBgLuZqbSGkh
DHg8vueIaRcW1r+vy18OZUCvWXKcPJODT/vrKRjOADkyl+ziHw7+0nJNVwQD
Lu1pO1KzdRC4+Be4nz1nQNM9jKOr+CBcxMxZnnjJAPLd1s8zSoOQH/jrw+8o
BhjRr0nYcP53NmhPjeXFMOD1TR2JSD0O/ueEhkssAz4UP3sSZszBZ3x7LPmG
AckMh4OnzQZhowMLg49jwOAW+2uNFoNwSYguGB3PgHftL/AzloNQ0EW+cjqR
AYoheIcBTnxTCCF77R0DarRxe+w5+y8dH/pdnsQAO+Xc7v/nL5jB6t39yIBV
he3HrDj1N2f3PVdMYcAuIdfQCq1BuHy1G09LZUCnc7V3seIgFIl0SCemMSBR
TIuuKzoI3H2ttyw+MyB0yGXJjpuDD2sq58nk1N9wP/kvZx4uOtnA1fiFATv+
SjTsJ2Fhy3yNqV82x49BvQXERiw45FUmqOcyYPa344MdmVgocSpjTuQxwGEs
SbY5HAs8EsWqaV8ZoJpYobvgysFjv/raFzLgdvnaSvYpDj4yt1mgmAFFmazP
REksOC5+vhRSxoDGh0YXyzoHoKzg02edCgYsHNxz6erHAeC9kTw9V8mA/3h/
zTz3GIByfEKocy0D6O8Vhly4OfGouP499QzAVh25pPmxH64ZxkrgGxhQmr2t
Om9/P/CVPC8yamaAZVaF3KRmHzi5h6/+bWGA5A/Zbw1FvVAl88y4vI0BU0/j
FpsP9ILzq8cUhU4GnDSr8n26owdqPDy38gxw7oMYoUw+uRMmh8TPV2IZIHpK
WQO71AHi0Pr6Jo4BKUfl9XUtO8BPQHxPxzADjg7z+sfNtYN2RbPacyoDlNti
85R4WqF0fbf19h8cf53XqKrYVAsst8bE2ikG/Dq2+v7o6WoQHLxFuT3NgLs+
3nITwZXglYFce2YZIGs/LVU2XgrqZ939opcY0G+aox4/lw35b+qSd21hQuqG
SPfDPyIRZfUGC/EwgSzCN5Xm9RZtdxVQureNCUPiBltkfD+iW0dvFA3wMWF7
YpEt99kvaD9lV8trISacX0+RXD9RijIUnCeE9jEhmPTyo3ROE8LH7FBrlWWC
TsVqyuMvzWjTUuV9b3kmfPg8u+KT3oKcuvhW8UpMKPyohsuMb0MynhU7Eg4y
webJB7thl06UXLVNS+wkE8rGTIR/nRpA6Xrinnm6TNAsIKenfBlA2a378/SA
CQ+Vy6PKtmJRycAZOddTTBAjiCIyBos6JsKESkyYsL5yd1+yOg713o23OH2O
CX1RwQGbwnFocD7jJcGUCVn3NJmuRByicrVuWjdnQkfnIvmnPx7NiW2cN7Vl
gkZudwP6NIQWUwTU6Rc557F28E3N6BD6K7/Pw+sSE861ncq1VRxGPBr6o+/+
Y0I+yb7QMH0Y8VVayKheYUJIS+xgCGUYCepedUCOTLDvMWXrCI8gybOB+G/X
mdDIc0579tEI2tcfze/vzITY7Gl3Wt4IUrRNNuNzZcIe5LK4jziCVMn5kak3
mCDXkjtbuomANK7XtWq6MeGr7ZBEtAoBaY93c7W7M+HqfhnTL5YEdPIO+aT9
bSaY7PMOW/IiIIPfk35THkzgexAc++QNARn7r5QF3WVC1JHNtw8VEpDp+rZZ
wXtMSHp9U35rJwFZhomrfbnPhKcFxkEbGQR0cfsB9+MPmDAYa5EnPk9A/70+
ltnjzYTF5hKqCTcRXRM1YV59yIQM0XDcSyEiupFst/e3LxOOTnvM0KSI6Lbc
TftwfyZ06nqk6ykR0b2ch/Hij5jAi8xzs1SJyOdQODb/MccP++WSRA8RUUB5
/A79QCY8uh0hH3WQiIJPZJ7FBTFBdcdP2U0qRBTWVBZ24ykTIkQyzzySJ6IX
Z1qblkOYkLfDMPa3BBHF9uLWokKZcOrtLM/tnUQUb80+LhPO8eeuUi/mPwJK
Is75lEYwwdfHQ8JuioBSr24sMX7OhOd50eY9wwSU8U1gmviCCUH91QyDBgLK
vb1P+U4Uh7/Z8WvV6QRUOKt+gyuG03+Y3e3DoQRU5qufHveKCTg7ke9lTgRU
s2ZBU3zNhOZHmBRdPQJq3eZ58fxbJlR1s189mBpBXa8C3zDimSB54DBSQiOo
f3dM34NEJmy+X24yFzOCSPu+GiclMWE8vU+bojCC6Fl1IWofmVDAnyq05ecw
GlXraWhMZsJU5HjC5eJh9Ov4D+3xT0xAZ7AnP2oMo+EPx+9/TGfCpzedEzXj
Q6hhLSLPKoMJd1RMWjKShlBUk7xMXRZHfxNuv42LeKR81nHb6wImGIfG1ou8
xCGBvHxD4yImjGy6XNmwH4eW+FafrBZz1oMxz2NaBhFmIHHOtZwJk4YqDZQZ
LHK5hCXr1HH0oXpsGVQZQGY10iIz9UxIGNialuHWj7Qk71pmIib8ujBZ8927
D21k8LbtamECy5dq3x3Ug1JvGhWMdnL47J8FmlMHIj2sDI4ZYYLR6e2TwlwI
NRO4aw2JTLghFWs0+bUO5ejY/FkiMQFfKt6hZVeD/P7NuDvTOPwiCzeEJJej
3eHK1se+MUF/+cyRNO5sZBmfrMCaY4ITbjBHiysF2kufdR3ZyYKA1WDr6x/a
4Znm0b+9u1iQc8C6LzodA/rFk2o3BFjAGxMg1f+lA2oKrF4nCLPgkp9dAu+X
LijIkb60KMECviuLZNyjPri9H/f8lSQLDKjhBsJu/bA/K7xWSYoFiaWsqcP7
ByA946fUpX0sYGhLr9zlxUJiau23KiUW2F9Olp0oGwQbKU9RqwMsaByt8egV
x4FgsuzZ78os6JmabDN/jIOopOf54gdZ8GCPdfn8cTycFdelFR/i5Hv+03ZD
PB62vJvZdU6DBZN/ygoCfuIhON7uQcBhFhS2XfCJSBgCXeHtmYLaLODe9lVs
y/gQrLxpGM49ygLtx9JZmMPD4B2roEPWYcGdAz5ePzHDoLGLePvBSRbweDyQ
0d8xAtPRUcnb9VgwqyZJLbYYAbeXv7lOGrCg8zCjCdc9Agq8WZr4UyxoS7U8
7MNNAFbkZRcPIxYscAs8EztJgFSenQmbjVnwx2AopdKTAA7hTZiPZ1gQrX4n
2OwTAcS5fZYPn2VBblllPLaXAMPP9qv0nmOBx+F/e08tEeDtRoqDqxkLFG+f
uftemghWT1/FrJ1nQVNHuwzOkAg7uQwb4y1Y8Kjlee0PFyL0BC7Mqlmx4GrK
0AI9hAjP13Lk2i+wYM2kOj8/mQjGj6/YOtqwIOxz+keLciJsWuWPWLBlgaiK
/bf2TiI0+rdWxdixID9/soSfTIQnS76TivYsIES4/lb5TgQdXxVJdJkFlRsP
K4vNE2HxD+28nQMLnMXKmfhVIpR5vwn6dYUF5PlRTScuEtz/fbo44ioLLOHN
WuMGEhz0WmZJX2fBMH+A+tw6EX7M5AtXOXH8NSAgtLhChGzPa8aWLixQeaCF
w84RwXVayG/ClQUXM9NbQseJIHsHkxN8kwVf1zJdBIlEoP8IIIu5s8Atykoq
EEPkzOEHdxTf4vSnYZPeXUIE++9MvbMeHL28vMv/JRFBxC3+HvMOCz4fkRrb
E0wE3JhJur8nC94+mDqx34kIsa5/cQL3OfpkY9r2GxDBfLSQO9eLBU+2JarI
7SXCdmfno6e8WaDRcaxu7wIB0uNanQZ9OHoVbnq43EOAY60KMU6+nP4VC/LX
OfreUJgYDQlgwdM0419Z+gRYsz3LL/SYBXIn3Hwu8XH0Dc898fkJC9Tr1ESs
hkagedzjTUswC+IBxk5cHQF7sb566xAW0Hy7nEBmBGZMDn1nP2PB4V7V319p
wyCVOwubI1hwubVCf856GAI8Hv40imbBK4vyUu19QyCQPCI2FMMCL4PAvQV9
eMjuPWbkGssC5uYa8W/+eBg+uJoUFseC128kvzV14kBjNsik/T0nX4dZ1RvO
vD3h8/zzmS8seBeXb+vG1wdBXyb7RrJYIPFSWt9HpRdERkxXbuawQPLRw3Ol
Z3rA6NhOq8h8jh9i1vpP+HXBp5U3/zpKWLB3g+Iipa4d7II+2p9DLNgTXXOF
RK+H9sjCXecJnPsr3ub5cl8IgniL4+tEDj9pxrv6nNeoKu2XUzGZBdbeVhlj
R5NQbs2hit10FqRf8ewPScxEMVOFDrRvLDhp7t+0Kl6KeJYtwmPHWTDx/oPG
fdlyFMw9U3jqOwv4197OTChUIi8p9U1ZUyxQoExu7ZesRRcti3I8f3Peo5jG
5eTrTajfwRK37w8L+qQ2PTQ0b0Ym7jN/cQssSO453uV2tAXphKhbHlvh+PmZ
8yb0rxVJlRUtcG1gw2vh7pUExw6U2GgpU7qRDbftj9trqXYi/t6Zs66b2bA9
5q/Ooz+daOOYenInDxvuS+3RKA7oRmOixYZvdrLh5Noo0fV8H7oqb3XHiJ+z
X8h1hHuhDxHUZxMWBNhwYqT0V+H7ftR1VmPy8m422Aj5k2fdBpDhRazQDlE2
yGiYRSmVDqA6p/u6SIwNy/7VDa0rA6jgUXGsnCQb7NNWXSRDsEgp0qpmaC8b
RA/fT+5uwqJPb2fZEdJsjq6CBd3/sOjNV42jU7Js6PO65Jx/bxBtr8FeS5Fn
Q8/OkNdhmYMotP3+C0tFNkTowY5PI4PoIb2YWnaADfnmT01eaeHQrx9WPDdV
2IAtyaq7egWH3JZm1cXV2GBQfz/NPRSHmJvfXO4+yIadgqV8+dk4dFlAM/SJ
OhtqfK0tFLtwCLd38OshTTbcbZ9OIUzgkJmy1whTiw2dLt5uLZvxqE1bcMPb
I2yYFdx4fnwvHukZligbH2XDLR+l3tOH8ajS4oLN0jE2bEg8mck4g0fqDnNP
cnXYwHyijaov4VGO25ssh5NsuGIhTcXdwCNZH03sTj02PL03zqXphUcfng6u
NAIbCPZz1iMBeCQc4yX/wIANqcH3+jDBeBSdJGiuYMiG2M+783lC8WhLVonv
iBEb6ltsz33grG0HKt95GLNh37tx98YQPMpYrqveYMLRc9wmKSmQM1/LNpMS
zrKhVfiN2UE/PDIww6yqmLIh4yDLL9gTj2J9eiSbzNggubvCPcEFj2gpWN2L
5mwo5Cu542OHR2odw44/LNhQIS6tJWeCR49nyUHBVmxI7wknZGrjUbcE89Nu
azYIvN/1bJMcHkkYjTXl2rAhLvnlq1M78Mjtzg/OY8oGlq0i2f0PDlUmzGwa
smNDVTKPbTAZh7Y0/pG/Zc8GRZF57miEQzbfV06vX2YD7BH3SkzDobkT3JEH
HNlQsvzvQOtVHDJw5c1puMoGwZ9OugsncCg2ZleX9XU2PD8r0ndaBIfUmOJ8
gS5sWNX8teLYNoge80qrCd1gQ1vEB0GdpEHUpSVvnn2TDfiKl9ds7wwit/CD
sYO32PC75J/y/Z2DqKJQq/imBxv+Hu5+GErBIm7iscG/dzj8LvhYr2VjUbqK
obDSfTaoy/AtGetiEXXA7t0jX8598trgbGUzgFRXHKr5/dnwwPPlf9kiAyhA
zomUGcAGI0sd3O3CfiT20ENy4Akb+JrdWLr5fejCnqef5EPZsPIv0zH7RQ/q
cM3N7o1lg1g+z8s3PzGobOVvlXQuG85N63B9ba9FMrMji+w8zn1BN/19LtWg
qPES7eyvbFCLzxfynKxCrni3MvViNpRZaY+376pAIgX4QoMqNpje1SO8ci1C
vtfzvzi3s0F6QqbZMykejdpFfFPqYMNNKSkvWZNYZGnuJD/VyQYVxsw1/KZw
tP+EWLp3Lxsc45tqvQINEUE4NDkMz4aJUvHEkIg40On4720Wiw1/FK7LyLjk
w5cGbdztUTY8k+n10YktAMFyAUH1MTZYcHGL+NsXwY80zKuq72wYU7M6S71T
Ah8fab3snGFDTlDfEv2/CtjqtbMreo4NW5Uea7DOVYK32/etF+bZIBt3xxV3
uArO26aGkxY59Rd/Ct9eroZ/B7eH/PjHeR9yGjtXjOrhlsJYYyHXKCQ9u3V8
jbcBhvc0rT/YOAoPnTqnCd0NULjV98lf7lF4vnXpnqB0I0iuW9U18ozCf0of
ufp8GyHyj+pq6LZR4P5cOfq3txGusVh+O3aMwtvb4rhGryboIdRXYneOwo4y
7RyHpiY41v9uIZ5/FGKmitj+O5qBv87cW0p4FHIWwq+eS2mGxyUHSlm7R6Fb
2YbKw26GiezNc19ER0H+aUz5ZYUWsEmlq98WHwW1ZplJE9cWaIqv8Ty0ZxSW
GktrCOkt8D/xXdJC
                "]]}, 
              Annotation[#, "Charting`Private`Tag$62331#2"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.560181, 0.691569, 0.194885], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwVm3c8lV8YwPETMiJlRSQqFEmipJ4yyiqrkGRHwyozZUQhqxApo0h2tsyQ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                "]]}, 
              Annotation[#, "Charting`Private`Tag$62331#3"]& ]}}, {}}, {
          DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
           AxesOrigin -> {0, 0}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, DisplayFunction -> Identity, 
           PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
           All, DisplayFunction -> Identity, AspectRatio -> 
           NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True},
            AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
           Identity, Frame -> {{False, False}, {False, False}}, 
           FrameLabel -> {{None, None}, {None, None}}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, GridLinesStyle -> Directive[
             GrayLevel[0.5, 0.4]], 
           Method -> {
            "DefaultBoundaryStyle" -> Automatic, 
             "DefaultGraphicsInteraction" -> {
              "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
               "Effects" -> {
                "Highlight" -> {"ratio" -> 2}, 
                 "HighlightPoint" -> {"ratio" -> 2}, 
                 "Droplines" -> {
                  "freeformCursorMode" -> True, 
                   "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
             "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
             None, "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& )}}, 
           PlotRange -> {{0., 0.6283185178951315}, {-1., 1.}}, 
           PlotRangeClipping -> True, PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {Automatic, Automatic}}, 
           Ticks -> {Automatic, Automatic}}], 
         FormBox[
          FormBox[
           
           TemplateBox[{"\"Px\"", "\"Py\"", "\"Pz\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
               False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.368417, 0.506779, 0.709798]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.880722, 0.611041, 0.142051]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.560181, 0.691569, 0.194885]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"{", "}"}]}], ",", 
                 RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], 
               "]"}]& ), Editable -> True], TraditionalForm], 
          TraditionalForm]},
        "Legended",
        DisplayFunction->(GridBox[{{
            TagBox[
             ItemBox[
              PaneBox[
               TagBox[#, "SkipImageSizeLevel"], 
               Alignment -> {Center, Baseline}, BaselinePosition -> Baseline],
               DefaultBaseStyle -> "Labeled"], "SkipImageSizeLevel"], 
            ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}},
           AutoDelete -> False, GridBoxItemSize -> Automatic, 
          BaselinePosition -> {1, 1}]& ),
        Editable->True,
        InterpretationFunction->(RowBox[{"Legended", "[", 
           RowBox[{#, ",", 
             RowBox[{"Placed", "[", 
               RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]},
      {
       TagBox[
        StyleBox[
         DynamicModuleBox[{Notebook$$108$326089`t$$ = 0.019818402607674954`, 
          Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
          Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, 
          Typeset`animvar$$ = 1, Typeset`name$$ = "\"untitled\"", 
          Typeset`specs$$ = {{
            Hold[Notebook$$108$326089`t$$], 0, Rational[1, 5] Pi}}, 
          Typeset`size$$ = {360., {197., 202.}}, Typeset`update$$ = 0, 
          Typeset`initDone$$, Typeset`skipInitDone$$ = True}, 
          
          DynamicBox[Manipulate`ManipulateBoxes[
           1, StandardForm, "Variables" :> {Notebook$$108$326089`t$$ = 0}, 
            "ControllerVariables" :> {}, 
            "OtherVariables" :> {
             Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
              Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
              Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
              Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> Show[{
               Notebook$$108$326089`plt1[
                Notebook$$108$326089`vecB[Notebook$$108$326089`t$$], 
                Notebook$$108$326089`t$$], Notebook$$108$326089`plpathB}], 
            "Specifications" :> {{
              Notebook$$108$326089`t$$, 0, Rational[1, 5] Pi, 
               AnimationRunning -> True, AnimationRate -> 0.05, 
               AnimationRepetitions -> 10, 
               AppearanceElements -> {
                "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
                 "DirectionButton"}}}, 
            "Options" :> {
             ControlType -> Animator, AppearanceElements -> None, 
              DefaultBaseStyle -> "Animate", DefaultLabelStyle -> 
              "AnimateLabel", SynchronousUpdating -> True, ShrinkingDelay -> 
              10.}, "DefaultOptions" :> {}],
           ImageSizeCache->{438., {234., 239.}},
           SingleEvaluation->True],
          Deinitialization:>None,
          DynamicModuleValues:>{},
          SynchronousInitialization->True,
          UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
          UnsavedVariables:>{Typeset`initDone$$},
          UntrackedVariables:>{Typeset`size$$}], "Animate",
         Deployed->True,
         StripOnInput->False],
        Manipulate`InterpretManipulate[1]]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326095523168*^9},
 CellLabel->"Out[286]=",
 CellID->502876247,ExpressionUUID->"4e528103-629a-4cbb-a669-55897b39c48d"]
}, Open  ]],

Cell[TextData[StyleBox["That off-resonance Rabi oscillation shows the result \
is much closer to the non Rabi result, i,e,  it is off-resonance and does not \
spin flip.",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188737557898*^9, 
   3.8581887857944527`*^9}, {3.858190068270484*^9, 3.8581900700905647`*^9}, {
   3.8587022784115562`*^9, 3.858702279490685*^9}, {3.858886409671021*^9, 
   3.858886487950635*^9}, {3.85888658670438*^9, 3.8588865979994164`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->558168206,ExpressionUUID->"f553844c-2ad1-43f5-a262-3a33e45e3e20"],

Cell[TextData[StyleBox["Once we have a dynamic density matrix, it is simple \
to examine other dynamic aspects,  such as the energy and power evolution and \
changes in entropy",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188737557898*^9, 
   3.8581887857944527`*^9}, {3.858190068270484*^9, 3.8581900700905647`*^9}, {
   3.8581901571780157`*^9, 3.85819063594188*^9}, 3.8581907265958138`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->831021091,ExpressionUUID->"b27eac72-7104-414d-88e7-db7267c77709"],

Cell[TextData[{
 StyleBox["The on-resonance  single qubit Rabi oscillation case is a simple \
dynamic example of a spin flip\nor NOT gate.   This can be simple extended to \
a phase change, or an Hadamard or any other spin rotation gate.  In addition \
a Lindblad/Beretta term can be added to study thermal bath, entropy,\n\
dissipation, noise and measurement processes for a single qubit.  An example \
of such a study is in",
  FontSize->24],
 StyleBox[" ",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox["Annals of Physics Volume 383,  2017,  33  ",
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox[ButtonBox["link",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL[" \
https://www.sciencedirect.com/science/article/abs/pii/S0003491617301252"], 
    None},
  ButtonNote->
   " https://www.sciencedirect.com/science/article/abs/pii/S0003491617301252"],
  FontFamily->"Times New Roman",
  FontSize->24,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0]],
 StyleBox[".  ",
  FontSize->24],
 StyleBox["   Two qubit gates can also be implemented following the same \
procedures with ",
  FontSize->24],
 "QCPITT",
 StyleBox["; a publication is in preparation.",
  FontSize->24]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188737557898*^9, 
   3.8581887857944527`*^9}, {3.858190068270484*^9, 3.8581900700905647`*^9}, {
   3.8581901571780157`*^9, 3.8581905622513323`*^9}, {3.858190695939124*^9, 
   3.858190710080923*^9}, {3.858886787906042*^9, 3.858886793403472*^9}, {
   3.859470459354841*^9, 3.859470494521346*^9}, {3.859470528911572*^9, 
   3.8594705295317793`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->837984814,ExpressionUUID->"2a53a9ac-81e4-4a5c-823d-0059c9bf74fc"],

Cell[TextData[{
 "Here a simple Lindblad term is added to the time evolution of a one-qubit \
density matrix,  A simple example of a Lindblad term is:\n",
 Cell[BoxData[
  StyleBox[
   RowBox[{"  ", 
    RowBox[{
     FormBox[
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", "t"}]],
      TraditionalForm], " ", "\[Rho]A"}]}],
   FontSize->24]],
  FontFamily->"Times New Roman",ExpressionUUID->
  "71f03e53-5fc1-400b-a607-88e882763253"],
 "[t]  =  - i   [ HA[t] . \[Rho]A[t] - \[Rho]A[t] . HA[t]]  + \
\[ScriptCapitalL]1[t]\nwhere the basic Lindblad \[ScriptCapitalL]1[t] \
introduces external non-unitary evolution.   This form has zero trace and is \
Hermitian,  thus assuring that the density matrix maintains its essential \
properties of unit trace and Hermiticity.\n\[ScriptCapitalL]1[t]=\
\[CapitalGamma]v (L.",
 Cell[BoxData[
  StyleBox[
   RowBox[{" ", "\[Rho]A"}],
   FontSize->24]],
  FontFamily->"Times New Roman",ExpressionUUID->
  "7b24e0f0-3d33-4a20-9b3e-9d6d208e49c0"],
 "[t] .Adj[L]-(Adj[L].L.",
 Cell[BoxData[
  StyleBox[
   RowBox[{" ", "\[Rho]A"}],
   FontSize->24]],
  FontFamily->"Times New Roman",ExpressionUUID->
  "695e679d-0d0f-4117-8722-b1d18f17e899"],
 "[t] +",
 Cell[BoxData[
  StyleBox[
   RowBox[{" ", "\[Rho]A"}],
   FontSize->24]],
  FontFamily->"Times New Roman",ExpressionUUID->
  "259a75aa-dc7f-413c-b010-622ca24b54af"],
 "[t] .Adj[L].L)/2) .",
 StyleBox["\n\n",
  FontSize->24],
 " In the simple example below, we take L= ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Sigma]", "z"], TraditionalForm]],ExpressionUUID->
  "509c7f36-6e91-474b-98c5-2bca37a63936"],
 "   and consider a Rabi resonance HA[t].   There is an attenuation of the \
polarization vector and as it goes to zero length.   That corresponds to \
equal eigenvalues of  \[Rho]A at large times and thus an entropy of close to \
1.  The two levels are occupied with equal probability as an equilibrium \
state.\n \n In addition to \[ScriptCapitalL]1[t],  there are additional \
Lindblad/Beretta  non-unitary evolution terms that represent an outside bath \
and/or entropy constraints.   A detailed study was made in Reference xx\n",
 StyleBox["\nHere is a plot of a simple Lindblad term  added to the time \
evolution of a one-qubit density matrix",
  FontSize->24]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857674985236885*^9, 3.8576750184012747`*^9}, {
   3.857675070397909*^9, 3.857675212858832*^9}, {3.857675250481113*^9, 
   3.8576755155558033`*^9}, {3.85767564350385*^9, 3.857675704850971*^9}, {
   3.8576757380844793`*^9, 3.857675789115095*^9}, {3.8576758334631453`*^9, 
   3.857676028520625*^9}, {3.857676297487235*^9, 3.8576763431515703`*^9}, {
   3.857676387758781*^9, 3.85767643504352*^9}, {3.85767656995545*^9, 
   3.857676659743393*^9}, {3.8576767055873337`*^9, 3.857676724982937*^9}, {
   3.857816652235878*^9, 3.8578166665905733`*^9}, 3.857816715378708*^9, {
   3.857816746230492*^9, 3.8578167532391148`*^9}, {3.8578168435994377`*^9, 
   3.85781686589825*^9}, {3.8578177236781387`*^9, 3.857817729528343*^9}, {
   3.857844409972376*^9, 3.85784441100854*^9}, {3.857844451490745*^9, 
   3.857844567601145*^9}, {3.857844619452445*^9, 3.8578446282717123`*^9}, {
   3.857844669151145*^9, 3.85784486286979*^9}, {3.85784500925694*^9, 
   3.857845035481889*^9}, {3.857846283745173*^9, 3.857846287163142*^9}, {
   3.8578478587105303`*^9, 3.857848242577366*^9}, {3.857848305998749*^9, 
   3.857848328427655*^9}, {3.8578483716683283`*^9, 3.857848376413144*^9}, {
   3.857848406795918*^9, 3.857848492961157*^9}, {3.857848546029956*^9, 
   3.8578485575369663`*^9}, {3.857848673042571*^9, 3.8578487595677357`*^9}, {
   3.858188438891322*^9, 3.8581884872171*^9}, {3.858188737557898*^9, 
   3.8581887857944527`*^9}, {3.858190068270484*^9, 3.8581900700905647`*^9}, {
   3.8581901571780157`*^9, 3.85819063594188*^9}, {3.8581907265958138`*^9, 
   3.858190807800126*^9}, {3.858190848985265*^9, 3.858190868550311*^9}, {
   3.858191025197296*^9, 3.8581910442887573`*^9}, {3.858702227253181*^9, 
   3.85870222849507*^9}, {3.858704533993004*^9, 3.858704645895279*^9}, {
   3.858704963138905*^9, 3.858704982545186*^9}, {3.8587050270731*^9, 
   3.858705221577395*^9}, {3.858705256363223*^9, 3.858705840563892*^9}, {
   3.8587058781443653`*^9, 3.858706092365405*^9}, {3.8587061344858713`*^9, 
   3.858706166899403*^9}, {3.8588866669182577`*^9, 3.8588866677448378`*^9}, {
   3.858886700162373*^9, 3.8588867493145313`*^9}, {3.8589379510403147`*^9, 
   3.858937955517905*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1388692359,ExpressionUUID->"43958927-e025-4603-8f2f-e995a2cfa831"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"fpop6", "=", 
   RowBox[{"FileNameJoin", "[", " ", 
    RowBox[{"{", 
     RowBox[{
     "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
      "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", 
      "\"\<Lindblad1-sz.nb\>\""}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NotebookEvaluate", "[", "fpop6", "]"}], 
  " "}], "\[IndentingNewLine]"}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.85818862877635*^9, 3.8581887323735228`*^9}, {
   3.858702297132227*^9, 3.8587023552839108`*^9}, 3.858886805059524*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[287]:=",
 CellID->1537566662,ExpressionUUID->"c1147c5c-9cc5-4384-81e8-2a37b89a2d90"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       TemplateBox[{
         GraphicsBox[{{{{}, {}, 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.368417, 0.506779, 0.709798], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUV3c81t8XR0hSSb6ZKaNBslJZOYiUIllJGWW0aFCyQjZZGYWshOwt27X3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                "]]}, 
              Annotation[#, "Charting`Private`Tag$64407#1"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.880722, 0.611041, 0.142051], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwUl3c8lf8XwBEpSZKyJUmUVJIiOeWbEhlJKCu07JSsZBUhKyuVVcnee39k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                "]]}, 
              Annotation[#, "Charting`Private`Tag$64407#2"]& ], 
             TagBox[{
               Directive[
                Opacity[1.], 
                RGBColor[0.560181, 0.691569, 0.194885], 
                AbsoluteThickness[1.6]], 
               LineBox[CompressedData["
1:eJwV1nk4lOsbB/CpZElKJXJIUVkroWwnvpEOibKlXaGSbC06pD1LtGixlawz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                "]]}, 
              Annotation[#, "Charting`Private`Tag$64407#3"]& ]}}, {}}, {
          DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
           AxesOrigin -> {0, 0}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, DisplayFunction -> Identity, 
           PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
           All, DisplayFunction -> Identity, AspectRatio -> 
           NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True},
            AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
           Identity, Frame -> {{False, False}, {False, False}}, 
           FrameLabel -> {{None, None}, {None, None}}, 
           FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
           GridLines -> {None, None}, GridLinesStyle -> Directive[
             GrayLevel[0.5, 0.4]], 
           Method -> {
            "DefaultBoundaryStyle" -> Automatic, 
             "DefaultGraphicsInteraction" -> {
              "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
               "Effects" -> {
                "Highlight" -> {"ratio" -> 2}, 
                 "HighlightPoint" -> {"ratio" -> 2}, 
                 "Droplines" -> {
                  "freeformCursorMode" -> True, 
                   "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
             "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
             None, "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& ), "CopiedValueFunction" -> ({
                 (Identity[#]& )[
                  Part[#, 1]], 
                 (Identity[#]& )[
                  Part[#, 2]]}& )}}, 
           PlotRange -> {{0., 0.6283185178951315}, {-1., 1.}}, 
           PlotRangeClipping -> True, PlotRangePadding -> {{
              Scaled[0.02], 
              Scaled[0.02]}, {Automatic, Automatic}}, 
           Ticks -> {Automatic, Automatic}}], 
         FormBox[
          FormBox[
           
           TemplateBox[{"\"Px\"", "\"Py\"", "\"Pz\""}, "LineLegend", 
            DisplayFunction -> (FormBox[
              StyleBox[
               StyleBox[
                PaneBox[
                 TagBox[
                  GridBox[{{
                    TagBox[
                    GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                   GridBoxAlignment -> {
                    "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                   False, GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                   GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
                  "Grid"], Alignment -> Left, AppearanceElements -> None, 
                 ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
                 "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
               FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
               False], TraditionalForm]& ), 
            InterpretationFunction :> (RowBox[{"LineLegend", "[", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.368417, 0.506779, 0.709798]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.880722, 0.611041, 0.142051]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                    ",", 
                    RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    
                    TemplateBox[<|
                    "color" -> RGBColor[0.560181, 0.691569, 0.194885]|>, 
                    "RGBColorSwatchTemplate"], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
                 RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
                 RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"{", "}"}]}], ",", 
                 RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], 
               "]"}]& ), Editable -> True], TraditionalForm], 
          TraditionalForm]},
        "Legended",
        DisplayFunction->(GridBox[{{
            TagBox[
             ItemBox[
              PaneBox[
               TagBox[#, "SkipImageSizeLevel"], 
               Alignment -> {Center, Baseline}, BaselinePosition -> Baseline],
               DefaultBaseStyle -> "Labeled"], "SkipImageSizeLevel"], 
            ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}},
           AutoDelete -> False, GridBoxItemSize -> Automatic, 
          BaselinePosition -> {1, 1}]& ),
        Editable->True,
        InterpretationFunction->(RowBox[{"Legended", "[", 
           RowBox[{#, ",", 
             RowBox[{"Placed", "[", 
               RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]},
      {
       TagBox[
        StyleBox[
         DynamicModuleBox[{QDENSITY`Circuits`t$$ = 0.020147654206668492`, 
          Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
          Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, 
          Typeset`animvar$$ = 1, Typeset`name$$ = "\"untitled\"", 
          Typeset`specs$$ = {{
            Hold[QDENSITY`Circuits`t$$], 0, Rational[1, 5] Pi}}, 
          Typeset`size$$ = {869., {3., 10.}}, Typeset`update$$ = 0, 
          Typeset`initDone$$, Typeset`skipInitDone$$ = True}, 
          
          DynamicBox[Manipulate`ManipulateBoxes[
           1, StandardForm, "Variables" :> {QDENSITY`Circuits`t$$ = 0}, 
            "ControllerVariables" :> {}, 
            "OtherVariables" :> {
             Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
              Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
              Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
              Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> Show[{
               Notebook$$109$326095`plt1[
                Notebook$$109$326095`vecB[QDENSITY`Circuits`t$$], 
                QDENSITY`Circuits`t$$], Notebook$$109$326095`plpathB}], 
            "Specifications" :> {{
              QDENSITY`Circuits`t$$, 0, Rational[1, 5] Pi, AnimationRunning -> 
               True, AnimationRate -> 0.05, AnimationRepetitions -> 10, 
               AppearanceElements -> {
                "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
                 "DirectionButton"}}}, 
            "Options" :> {
             ControlType -> Animator, AppearanceElements -> None, 
              DefaultBaseStyle -> "Animate", DefaultLabelStyle -> 
              "AnimateLabel", SynchronousUpdating -> True, ShrinkingDelay -> 
              10.}, "DefaultOptions" :> {}],
           ImageSizeCache->{913., {43., 48.}},
           SingleEvaluation->True],
          Deinitialization:>None,
          DynamicModuleValues:>{},
          SynchronousInitialization->True,
          UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
          UnsavedVariables:>{Typeset`initDone$$},
          UntrackedVariables:>{Typeset`size$$}], "Animate",
         Deployed->True,
         StripOnInput->False],
        Manipulate`InterpretManipulate[1]]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260977989197`*^9},
 CellLabel->"Out[288]=",
 CellID->1492932384,ExpressionUUID->"ffb3e82a-dee0-47b0-8337-8e54dcb54054"]
}, Open  ]],

Cell[TextData[StyleBox["Rotations",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->129759577,ExpressionUUID->"94035194-a202-48d5-afa6-b9ab9f372e60"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1173308934,ExpressionUUID->"d9be0725-e062-4d88-8b75-4faef815e1e6"],

Cell["", "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.857675052445759*^9, {3.8581901381090403`*^9, 3.858190140854268*^9}, 
   3.858967701124072*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->93467406,ExpressionUUID->"8adb5c2f-de23-4e66-bbe2-ae15e9058ee0"],

Cell[TextData[{
 StyleBox["Mathematica contains convenient functions for dealing with \
matrices such as MatrixExp and MatrixPower. They allow one to easily \
construct Rotations and other operations:  \nThe equality ",
  FontFamily->"Helvetica",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", 
    RowBox[{"i", " ", "\[Alpha]", "  ", 
     SubscriptBox["\[Sigma]", "i"]}]], TraditionalForm]],ExpressionUUID->
  "6f11f078-317d-40aa-9989-6bda1399199e"],
 StyleBox["= Cos \[Alpha]  ",
  FontFamily->"Helvetica",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Sigma]", "0"], TraditionalForm]],ExpressionUUID->
  "936bc2c3-6db6-487a-8780-c16e5f9f56ed"],
 StyleBox[" + i Sin \[Alpha] ",
  FontFamily->"Helvetica",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Sigma]", "i"], TraditionalForm]],ExpressionUUID->
  "359fc40a-dd66-4d5f-a328-06797d23db79"],
 StyleBox["  can be demonstrated by comparing",
  FontFamily->"Helvetica",
  FontSlant->"Italic"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.857574443579329*^9, 3.857574446840363*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->158527280,ExpressionUUID->"400315b1-b177-4c62-b26e-0edd8c82338e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"ComplexExpand", "[", 
   RowBox[{"MatrixExp", "[", 
    RowBox[{"i", " ", "\[Alpha]", " ", 
     SubscriptBox["\[Sigma]", "1"]}], "]"}], "]"}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[289]:=",
 CellID->1488272680,ExpressionUUID->"914f3bf7-e6ee-42a7-aa69-a7a0c6f7e655"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "i"}], " ", "\[Alpha]"}]], "2"], "+", 
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"i", " ", "\[Alpha]"}]], "2"]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "i"}], " ", "\[Alpha]"}]]}], "+", 
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"i", " ", "\[Alpha]"}]], "2"]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], " ", 
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "i"}], " ", "\[Alpha]"}]]}], "+", 
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"i", " ", "\[Alpha]"}]], "2"]}], 
      RowBox[{
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{
          RowBox[{"-", "i"}], " ", "\[Alpha]"}]], "2"], "+", 
       FractionBox[
        SuperscriptBox["\[ExponentialE]", 
         RowBox[{"i", " ", "\[Alpha]"}]], "2"]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098044489*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[289]=",
 CellID->515646340,ExpressionUUID->"fbe5180e-e382-4ea6-af96-9a78640748ba"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrigExpand", "[", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"Cos", "[", "\[Alpha]", "]"}], " ", 
     SubscriptBox["\[Sigma]", "0"]}], "+", " ", 
    RowBox[{"i", " ", 
     RowBox[{"Sin", "[", "\[Alpha]", "]"}], " ", 
     SubscriptBox["\[Sigma]", "1"]}]}], ")"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[290]:=",
 CellID->1033159208,ExpressionUUID->"ec24f419-a90d-4ae0-b121-b916f7acde80"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", "\[Alpha]", "]"}], 
      RowBox[{"i", " ", 
       RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]},
     {
      RowBox[{"i", " ", 
       RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], 
      RowBox[{"Cos", "[", "\[Alpha]", "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260980608263`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[290]=",
 CellID->1371520735,ExpressionUUID->"8c5162fb-76c0-4e44-8180-d6eca29b34a7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixExp", "[", 
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], "*", "\[Theta]", "*", 
   RowBox[{
    SubscriptBox["\[Sigma]", "3"], "/", "2"}]}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[291]:=",
 CellID->1901522861,ExpressionUUID->"7de82e02-6ec9-4a49-b194-0002be469499"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
        FractionBox[
         RowBox[{"\[ImaginaryI]", " ", "\[Theta]"}], "2"]}]], "0"},
     {"0", 
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"\[ImaginaryI]", " ", "\[Theta]"}], "2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098084011*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[291]=",
 CellID->59526247,ExpressionUUID->"26c8269a-1b14-4a5a-88bb-416399b581d9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixPower", "[", 
  RowBox[{
   SubscriptBox["\[Sigma]", "3"], ",", "3"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[292]:=",
 CellID->43090173,ExpressionUUID->"4f1582d5-efa6-47c6-8f17-c3294a8b58b3"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260980999603`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[292]=",
 CellID->1877737474,ExpressionUUID->"99005412-e9e2-4eda-9b44-136798e44979"]
}, Open  ]],

Cell[TextData[{
 StyleBox["It allows us to check the formula ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"\[Sigma]", ".", "P"}], ")"}], "2"],
     TraditionalForm], "=", 
    SuperscriptBox["P", "2"]}], TraditionalForm]],ExpressionUUID->
  "00c72063-db6b-436b-83fc-f0122803e333"]
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->514548572,ExpressionUUID->"7030108b-77b6-43cd-b44b-e7f73753c319"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"MatrixPower", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "*", 
      SubscriptBox["\[Sigma]", "1"]}], "+", 
     RowBox[{"b", "*", 
      SubscriptBox["\[Sigma]", "2"]}], "+", 
     RowBox[{"c", "*", 
      SubscriptBox["\[Sigma]", "3"]}]}], ",", "2"}], "]"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellLabel->"In[293]:=",
 CellID->101203135,ExpressionUUID->"fbbaabab-118d-46cf-88fb-b084a3f16ffe"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.8577841292949419`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ")"}], "+", 
       SuperscriptBox["a", "2"], "+", 
       SuperscriptBox["c", "2"]}], 
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"0.8577841292949419`", "\[VeryThinSpace]", "+", 
         RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ")"}], "+", 
       SuperscriptBox["a", "2"], "+", 
       SuperscriptBox["c", "2"]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260981227827`*^9},
 CellTags->"b:5.1.4",
 CellLabel->"Out[293]=",
 CellID->1861532853,ExpressionUUID->"4a3c0989-6519-4668-9746-2c2aca44931a"]
}, Open  ]],

Cell[TextData[StyleBox["The general rotation of a qubit state is given by the \
Wigner rotation WR[\[Phi], \[Theta] ,\[Gamma] ].",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.858337341562845*^9, 3.858337374762082*^9}, {
  3.8583374847263203`*^9, 3.858337486228672*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.1.4",
 CellID->114041521,ExpressionUUID->"cc17e065-37e8-4de6-82ed-c3e71bd47061"],

Cell[TextData[StyleBox["Quantum Gates",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.8589677897478933`*^9, 3.85896780052565*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1073997505,ExpressionUUID->"86a3909e-d805-4a69-a62e-d5f45d4f7317"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1076013500,ExpressionUUID->"dbd7505c-250b-4f3b-8f5d-f9905c5c8089"],

Cell[TextData[StyleBox["In the density matrix language any operator \
\[CapitalOmega] acting on the density matrix of the system \[Rho] , produces \
a new density matrix  \[Rho]\[CloseCurlyQuote]",
 FontFamily->"Helvetica",
 FontWeight->"Plain"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.8583375878627768`*^9, 3.858337638806918*^9}, 
   3.8583377230708313`*^9},
 TextAlignment->Left,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->920889186,ExpressionUUID->"2386e5b5-d9f0-41cd-b978-b135e70278a2"],

Cell[TextData[{
 StyleBox["\[Rho]' ",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox["= ",
  FontFamily->"Helvetica",
  FontSize->14,
  FontWeight->"Bold"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox["\[CapitalOmega]",
     FontFamily->"Helvetica"], 
    StyleBox[" ",
     FontFamily->"Helvetica"], ".", " ", "\[Rho]", " ", ".", " ", 
    SuperscriptBox[
     StyleBox["\[CapitalOmega]",
      FontFamily->"Helvetica"], "\[Dagger]"]}], TraditionalForm]],
  FontWeight->"Bold",ExpressionUUID->"d23fc1b7-90de-4fcd-9f1d-c9210e45403c"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.8583376499087048`*^9, 3.8583377130885887`*^9}, 
   3.858337844682168*^9},
 TextAlignment->-0.5,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->469905173,ExpressionUUID->"652ba143-01be-4936-be81-ec43574aaa12"],

Cell[TextData[{
 StyleBox["If \[CapitalOmega] is unitary  Tr[",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[" ",
  FontFamily->"Helvetica",
  FontSize->14,
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["\[CapitalOmega]",
      FontFamily->"Helvetica"], 
     StyleBox[" ",
      FontFamily->"Helvetica"], ".", " ", "\[Rho]", " ", ".", " ", 
     SuperscriptBox[
      StyleBox["\[CapitalOmega]",
       FontFamily->"Helvetica"], "\[Dagger]"]}], "]"}], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "cd872a2d-59fb-4475-aefb-f7484c8f02e6"],
 StyleBox[" = 1 ,  \[Rho]\[CloseCurlyQuote] remains Hermitian, with trace 1 \
and with the same eigenvalues.  One example of such a unitary operator is one \
that produces a diagonal ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox["\[Rho]\[CloseCurlyQuote]",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox[", with its eigenvalues along the diagonal.   Another example is \
the generator of time evolution   \[CapitalOmega] -> U[t_] :=MatrixExp[- i \
H[t] t], see later.",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.8583375878627768`*^9, 3.858337638806918*^9}, {
  3.8583377230708313`*^9, 3.858337830326652*^9}, {3.858337877431488*^9, 
  3.85833817768703*^9}, {3.858886874730524*^9, 3.858886913724596*^9}},
 TextAlignment->Left,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->866060983,ExpressionUUID->"31c1a603-724d-4381-969e-e6e0d9c3c9c6"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["a) Single qubit operations: Hadamard and rotations",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 CellChangeTimes->{{3.804677447328125*^9, 3.804677457644628*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:10", "b:5.2.0"},
 CellLabel->"In[294]:=",
 CellID->1398872592,ExpressionUUID->"96078100-68c0-4ba8-82c8-170ae445d16b"],

Cell[TextData[{
 StyleBox["The Hadamard operato",
  FontFamily->"Helvetica"],
 StyleBox["r",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox[" for one-qubit is defined as  Had |0\[RightAngleBracket] = (|0\
\[RightAngleBracket] + |1\[RightAngleBracket])/ ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SqrtBox["2"], TraditionalForm]],ExpressionUUID->
  "a38dbdfa-bb50-4d97-8a43-bf4f432f0b08"],
 "  and Had |1\[RightAngleBracket] = ",
 StyleBox[" (|0\[RightAngleBracket] - |1\[RightAngleBracket])/ ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SqrtBox["2"], TraditionalForm]],ExpressionUUID->
  "0b4c678a-4f54-4dfd-89a4-2382d7430c98"],
 StyleBox[", this is equivalent to Had = (",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    StyleBox["\[Sigma]",
     FontFamily->"Helvetica"], "1"], TraditionalForm]],ExpressionUUID->
  "52a0d038-2763-499d-a6ee-9054f23e758e"],
 StyleBox[" + ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    StyleBox["\[Sigma]",
     FontFamily->"Helvetica"], "3"], TraditionalForm]],ExpressionUUID->
  "8ed8e0c7-49ab-46b4-9b9b-f13ed6243d0c"],
 StyleBox["\[RightAngleBracket]/ ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SqrtBox["2"], "."}], TraditionalForm]],ExpressionUUID->
  "9b0aab61-c79e-4eff-8c04-fc802bd7654e"],
 StyleBox["To Hadamard qubit-one in a one-qubit system use:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.613395051437973*^9, 3.613395051953808*^9}, {
   3.857672010090047*^9, 3.8576720101518373`*^9}, 3.857672135666934*^9, {
   3.8583381865855827`*^9, 3.858338205093811*^9}, {3.858886927629425*^9, 
   3.858886942223178*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->200686388,ExpressionUUID->"423adf7a-b4c9-477f-96c2-6e5353d0f829"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HA", " ", "=", " ", 
  RowBox[{"had", "[", 
   RowBox[{"1", ",", "1"}], "]"}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[1]:=",
 CellID->1386797326,ExpressionUUID->"ab430eea-e3bf-4008-8e86-2e56c6f806f8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], 
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]], 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098167849*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[1]=",
 CellID->93407228,ExpressionUUID->"36b27891-1c9e-49d7-a45b-dc6f1732391f"]
}, Open  ]],

Cell[TextData[StyleBox["or the script form:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->1262637191,ExpressionUUID->"32d9dae5-a881-446c-bc7b-aed2b411e7fd"],

Cell[CellGroupData[{

Cell[BoxData[{"\[ScriptCapitalH]", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"s", "[", "1", "]"}], "+", 
     RowBox[{"s", "[", "3", "]"}]}], ")"}], "/", 
   RowBox[{"Sqrt", "[", "2", "]"}]}], "==", "%"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"-", " ", "I"}], " ", 
   RowBox[{"MatrixExp", "[", 
    RowBox[{"I", " ", 
     RowBox[{"Pi", "/", "2"}], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"s", "[", "1", "]"}], "+", 
        RowBox[{"s", "[", "3", "]"}]}], ")"}], "/", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], "]"}]}], "-", "%%"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.860074627070229*^9, 3.860074790140462*^9}, {
  3.860074986457492*^9, 3.860075069954876*^9}, {3.86007514120352*^9, 
  3.860075154534299*^9}, {3.8600751969927273`*^9, 3.8600752286049967`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[2]:=",
 CellID->1857353130,ExpressionUUID->"85fdbe19-9955-432e-9b49-5a26c90133f4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], 
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]], 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098192296*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[2]=",
 CellID->84735039,ExpressionUUID->"384b85fc-54a7-4fc9-953d-4a07427de1ff"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260982033863`*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[3]=",
 CellID->360142512,ExpressionUUID->"e15ba8f9-3220-461d-b735-f370081330f3"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098212831*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[4]=",
 CellID->1306281938,ExpressionUUID->"209d2aaf-0c6e-4dc2-9978-cfde9e8ddfe1"]
}, Open  ]],

Cell[TextData[StyleBox["If we have 3 qubits and want to Hadamard the second \
one,",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->711763118,ExpressionUUID->"ec39b56f-56c0-464c-9557-3b22e44f200b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HAD32", "=", 
  RowBox[{"AF", "[", 
   RowBox[{"had", "[", 
    RowBox[{"3", ",", "2"}], "]"}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.613395079179223*^9, 3.613395086367363*^9}, {
  3.613395131040553*^9, 3.613395145640243*^9}, {3.61339538195849*^9, 
  3.613395393218055*^9}, {3.613395619570511*^9, 3.613395623461936*^9}, {
  3.6135049161353807`*^9, 3.6135049204543056`*^9}, {3.8566372501255627`*^9, 
  3.856637276092434*^9}, {3.856637547152788*^9, 3.856637610187126*^9}, {
  3.856637864531618*^9, 3.85663786864853*^9}, {3.856730720722266*^9, 
  3.8567307540543013`*^9}, {3.856730797029842*^9, 3.856730814895028*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[5]:=",
 CellID->1118810436,ExpressionUUID->"e781abb5-21c3-4c79-b840-288f0f9ea0e6"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", "0", "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", "0", "0", "0"},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0", "0", "0", "0", "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0", "0", "0", "0"},
     {"0", "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", "0", "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0", "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"},
     {"0", "0", "0", "0", "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098227901*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[5]=",
 CellID->955208685,ExpressionUUID->"4763f15c-1741-4cd7-b27a-1dc6e4727dbb"]
}, Open  ]],

Cell[TextData[StyleBox["which  is equivalent to",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->480184720,ExpressionUUID->"bdea370b-d82f-4b45-a5f6-91699749c01f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
  "\[DoubleStruckCapitalI]", "  ", "\[CircleTimes]", " ", "\[ScriptCapitalH]",
    " ", "\[CircleTimes]", "\[DoubleStruckCapitalI]"}], "==", 
  RowBox[{"AF", "[", 
   RowBox[{"had", "[", 
    RowBox[{"3", ",", "2"}], "]"}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6133959477066393`*^9, 3.613395950338436*^9}, {
   3.614015919554443*^9, 3.614015932928617*^9}, {3.624189985515459*^9, 
   3.6241899866155157`*^9}, {3.85603842136961*^9, 3.856038421371151*^9}, {
   3.8566376228554363`*^9, 3.856637624081024*^9}, {3.8567307621678057`*^9, 
   3.8567307713584833`*^9}, 3.858886977865597*^9, {3.859055241275831*^9, 
   3.859055241276613*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[6]:=",
 CellID->221043614,ExpressionUUID->"a7d7ff51-a767-453b-b856-fed2718efbb6"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098252819*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[6]=",
 CellID->1752107470,ExpressionUUID->"22fa7134-b3f8-4d27-aab2-a1444e81ca96"]
}, Open  ]],

Cell[TextData[StyleBox["If we want to apply Hadamards as above to an initial \
system of |000\[RightAngleBracket],  we construct the initial density matrix \
and transform to the new density matrix, also we check the subsystems 2 and 1 \
to verify that it is only qubit 2 that has been subjected to a Hadamard:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.614015951445463*^9, 3.614015952625804*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->2080794835,ExpressionUUID->"2a6d3308-807b-411c-a2ac-5b192fe52903"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Rho]000", "=", 
  RowBox[{"AF", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "0"], "\[CircleTimes]", 
    SubscriptBox["\[ScriptCapitalP]", "0"], "\[CircleTimes]", 
    SubscriptBox["\[ScriptCapitalP]", "0"]}], "]"}]}], "\[IndentingNewLine]", 

 RowBox[{
  RowBox[{"\[Rho]FINAL", "=", 
   RowBox[{"HAD32", " ", ".", " ", "\[Rho]000", " ", ".", " ", 
    RowBox[{"Adj", "[", "HAD32", "]"}]}]}], " "}], "\[IndentingNewLine]", 
 RowBox[{"PTr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", "\[Rho]FINAL"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"PTr", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", "\[Rho]FINAL"}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140159800992002`*^9, 3.61401603406218*^9}, {
  3.624189994042535*^9, 3.6241899950418377`*^9}, {3.856038421373501*^9, 
  3.8560384213749332`*^9}, {3.859055241278591*^9, 3.859055241279751*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[7]:=",
 CellID->147600450,ExpressionUUID->"970e4ff0-f2b9-4508-8764-82077d64850f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260982693863`*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[7]=",
 CellID->1374480626,ExpressionUUID->"7e21fb3e-27f2-48df-9f84-c15e48301af7"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", 
      FractionBox["1", "2"], "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {
      FractionBox["1", "2"], "0", 
      FractionBox["1", "2"], "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098281002*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[8]=",
 CellID->414290403,ExpressionUUID->"1f590e1e-0b85-4d51-ab05-ab4683f7fa1e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], 
      FractionBox["1", "2"]},
     {
      FractionBox["1", "2"], 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098291505*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[9]=",
 CellID->836819647,ExpressionUUID->"ca800559-5be5-4647-92fb-95f71f514d3d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260983012333`*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[10]=",
 CellID->557402926,ExpressionUUID->"813d7053-d557-496e-b9ff-1da22402e551"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"had", "[", 
  RowBox[{"2", ",", "1"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"HALL", "[", "3", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.856637173984448*^9, 3.856637187870625*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[11]:=",
 CellID->1761374433,ExpressionUUID->"e80eb3c5-804f-402f-a192-e34de8b00ef9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098324484*^9},
 CellLabel->"Out[11]=",
 CellID->947309068,ExpressionUUID->"3af14d91-d854-4fb2-ad4f-d5a9f60f9b4d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]], 
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"2", " ", 
         SqrtBox["2"]}]]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260983372383`*^9},
 CellLabel->"Out[12]=",
 CellID->2127230665,ExpressionUUID->"a25f421d-2e79-4ed0-b05d-f3a6477880f0"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Sometimes we need a ",
  FontFamily->"Helvetica"],
 StyleBox["Hadamard on every one of the qubits",
  FontFamily->"Helvetica"],
 StyleBox[";",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox[" in that case you may use the command ",
  FontFamily->"Helvetica"],
 StyleBox["HALL",
  FontFamily->"Helvetica"],
 StyleBox[":",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->377150865,ExpressionUUID->"0e6c5841-2222-4e05-9291-f422ae0119a8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HALL", "[", "3", "]"}], ".", "\[Rho]000", " ", ".", 
  RowBox[{"Adj", "[", 
   RowBox[{"HALL", "[", "3", "]"}], "]"}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"i:2", "b:5.2.0"},
 CellLabel->"In[13]:=",
 CellID->1550345758,ExpressionUUID->"9775c230-9ffb-415f-8280-7fe1bceda115"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], 
      FractionBox["1", "8"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098358287*^9},
 CellTags->{"i:2", "b:5.2.0"},
 CellLabel->"Out[13]=",
 CellID->1058031399,ExpressionUUID->"e4d35865-ef5d-45dc-82e4-fb4987273476"]
}, Open  ]],

Cell[TextData[StyleBox["The action of a Hadamard on state vectors is  also \
simple to evaluate, ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.856101072872884*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->1242065655,ExpressionUUID->"6ca62813-b7ce-40d0-a9af-d31b495f3e96"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"had", "[", 
    RowBox[{"1", ",", "1"}], "]"}], ".", 
   RowBox[{"Ket", "[", "0", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"had", "[", 
    RowBox[{"1", ",", "1"}], "]"}], ".", 
   RowBox[{"KetX", "[", "0", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"had", "[", 
    RowBox[{"1", ",", "1"}], "]"}], ".", 
   RowBox[{"Ket", "[", "1", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62419607026132*^9, 3.624196100784041*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[14]:=",
 CellID->1856414745,ExpressionUUID->"34c1b253-fa12-430c-9c82-a55aaf429426"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", "\"(\"", 
    
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098384067*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[15]=",
 CellID->1169125833,ExpressionUUID->"c36b3f1f-effc-4705-b6bf-f8f84f730d73"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", "1", "\")\"", "\" |\"", "\"0\"", "\" >\""},
   
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032609839703*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[17]=",
 CellID->22505181,ExpressionUUID->"b0f93177-bf9c-4ec4-b389-75eee9a662c7"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", "\"(\"", 
    
    RowBox[{"-", 
      FractionBox["1", 
       SqrtBox["2"]]}], "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098406807*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[19]=",
 CellID->287455195,ExpressionUUID->"126e8411-ccfc-45fd-8723-0a69fffd0f90"]
}, Open  ]],

Cell[TextData[StyleBox["A rotation around the y axis by an angle \[Theta] \
applied to a state |0\[RightAngleBracket] can be generated by the user as:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.62419611301157*^9, 3.624196113494347*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->2120195599,ExpressionUUID->"737e77b0-1523-4e11-9f75-a3c030205c57"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Roty", " ", "=", 
  RowBox[{"MatrixExp", "[", 
   RowBox[{
    RowBox[{"-", " ", "\[ImaginaryI]"}], "*", " ", 
    RowBox[{"s", "[", "2", "]"}], "*", 
    RowBox[{"\[Theta]", "/", "2"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"TrigReduce", "[", 
  RowBox[{"ComplexExpand", "[", 
   RowBox[{"Roty", "  ", ".", " ", 
    SubscriptBox["\[ScriptCapitalP]", "0"], ".", " ", 
    RowBox[{"Adj", "[", "Roty", " ", "]"}]}], "]"}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[20]:=",
 CellID->931277722,ExpressionUUID->"23cebcfc-b49e-46cc-97d0-78b06dfd9b62"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", 
       FractionBox["\[Theta]", "2"], "]"}], 
      RowBox[{"-", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{"Sin", "[", 
       FractionBox["\[Theta]", "2"], "]"}], 
      RowBox[{"Cos", "[", 
       FractionBox["\[Theta]", "2"], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098423169*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[20]=",
 CellID->1568740262,ExpressionUUID->"458e0be8-43f7-4a36-b28f-34bfb7e5c9cd"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"Cos", "[", "\[Theta]", "]"}]}], ")"}]}], 
      FractionBox[
       RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"]},
     {
      FractionBox[
       RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"], 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"Cos", "[", "\[Theta]", "]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098435397*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[21]=",
 CellID->1160849215,ExpressionUUID->"98eca4f6-7e69-4ddf-b095-331923f10502"]
}, Open  ]],

Cell[TextData[{
 StyleBox["and we see that Roty on state | 0 \[RightAngleBracket] gives us \
Cos ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FractionBox["\[Theta]", "2"]],ExpressionUUID->
  "be9335b2-ec9c-4a38-a650-4931bed27459"],
 StyleBox["  | 0\[RightAngleBracket] + Sin ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FractionBox["\[Theta]", "2"]],ExpressionUUID->
  "f8c1c237-7158-4a58-8ea8-70cce3ae15f2"],
 StyleBox["  | 1\[RightAngleBracket];   for  ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  RowBox[{"\[Theta]", " ", "=", " ", 
   RowBox[{"Pi", "/", "2"}]}]],ExpressionUUID->
  "94e70f21-2fdd-48c9-bea0-d243909d6e9d"],
 StyleBox["  gives ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "d4cfdb99-6b74-4aaf-a537-bece5f2f3589"],
 StyleBox[") ( | 0\[VeryThinSpace]\[RightAngleBracket] + | 1 \
\[RightAngleBracket])= |+\[RightAngleBracket].  That is the Hadamard.",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.858338308165799*^9, {3.85833840084489*^9, 3.8583384139178467`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->1123854318,ExpressionUUID->"9af40ab8-176b-4182-b3b5-71a266d5e7a0"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Roty", ".", " ", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614016099736486*^9, 3.614016104551073*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[22]:=",
 CellID->1500847261,ExpressionUUID->"a71de71b-d04b-4e4a-b136-f84d059ac3c1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", 
       FractionBox["\[Theta]", "2"], "]"}]},
     {
      RowBox[{"Sin", "[", 
       FractionBox["\[Theta]", "2"], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098456813*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[22]=",
 CellID->503823510,ExpressionUUID->"e93e6498-eaf8-4e9d-9d64-19319e6a29e1"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{"Cos", "[", 
      FractionBox["\[Theta]", "2"], "]"}], "\")\"", "\" |\"", "\"0\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"Sin", "[", 
      FractionBox["\[Theta]", "2"], "]"}], "\")\"", "\" |\"", "\"1\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098467811*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[23]=",
 CellID->233712449,ExpressionUUID->"66c5a31a-6bba-4278-b068-0f875ec221f2"]
}, Open  ]],

Cell[TextData[{
 StyleBox["or the user can use the QCPITT command RotY[ ",
  FontFamily->"Helvetica"],
 "\[Theta]",
 StyleBox[" ]",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.856376440853305*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellID->396997965,ExpressionUUID->"9ff36dab-3e11-41db-b1fb-16801bdd4f56"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"RotY", "[", "\[Theta]", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"RotY", "[", "\[Theta]", "]"}], ".", " ", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"RotY", "[", 
   RowBox[{"Pi", "/", "2"}], "]"}], ".", " ", 
  RowBox[{"Ket", "[", "0", "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DForm", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614016113854549*^9, 3.614016117780463*^9}, {
  3.624196202821477*^9, 3.624196222015512*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.0",
 CellLabel->"In[24]:=",
 CellID->113106526,ExpressionUUID->"85100578-7d82-42bd-8ae0-e6626e613121"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", 
       FractionBox["\[Theta]", "2"], "]"}], 
      RowBox[{"-", 
       RowBox[{"Sin", "[", 
        FractionBox["\[Theta]", "2"], "]"}]}]},
     {
      RowBox[{"Sin", "[", 
       FractionBox["\[Theta]", "2"], "]"}], 
      RowBox[{"Cos", "[", 
       FractionBox["\[Theta]", "2"], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098484488*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[24]=",
 CellID->257539980,ExpressionUUID->"e59b3a1f-9485-4ecf-9cd5-8991691694df"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"Cos", "[", 
       FractionBox["\[Theta]", "2"], "]"}]},
     {
      RowBox[{"Sin", "[", 
       FractionBox["\[Theta]", "2"], "]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098495941*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[25]=",
 CellID->1294895155,ExpressionUUID->"438b993d-03f4-4581-8228-95e2afb84c42"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    RowBox[{"Cos", "[", 
      FractionBox["\[Theta]", "2"], "]"}], "\")\"", "\" |\"", "\"0\"", 
    "\" >\"", "\" + \"", "\"(\"", 
    RowBox[{"Sin", "[", 
      FractionBox["\[Theta]", "2"], "]"}], "\")\"", "\" |\"", "\"1\"", 
    "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032609850581*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[26]=",
 CellID->703591387,ExpressionUUID->"ce55b428-b606-4a92-8961-d22c8e848b6e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {
      FractionBox["1", 
       SqrtBox["2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098516509*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[27]=",
 CellID->1215480927,ExpressionUUID->"719273fd-c55e-45db-bbe4-b43f3ba3c826"],

Cell[BoxData[
 TagBox[
  TemplateBox[{"\" + \"", "\"(\"", 
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"0\"", "\" >\"", "\" + \"", "\"(\"", 
    
    FractionBox["1", 
     SqrtBox["2"]], "\")\"", "\" |\"", "\"1\"", "\" >\""},
   "RowDefault"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098526032*^9},
 CellTags->"b:5.2.0",
 CellLabel->"Out[28]=",
 CellID->1723489825,ExpressionUUID->"d2b6c0d6-19ed-4f71-9993-81371b97df76"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["b) Two Qubit operations: More on the  CNOT gate",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 CellChangeTimes->{{3.85888709499809*^9, 3.858887099993331*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"i:1", "c:11", "b:5.2.1"},
 CellLabel->"In[29]:=",
 CellID->458354887,ExpressionUUID->"77c1fd1c-ffe7-47d3-9784-83feb0f9ec2d"],

Cell[TextData[{
 StyleBox["Two and in general n-qubit gates can be constructed. Among them \
the CNOT gate plays a very important role. \nA CNOT gate  |a\
\[RightAngleBracket] |b\[RightAngleBracket]  --> |a\[RightAngleBracket]  |(a ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CirclePlus]", " "}], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "10914e4b-c66a-4c34-b3eb-00525b58f36c"],
 StyleBox["b) mod 2\[RightAngleBracket]  operating with qubit 1 as control \
and qubit 2 as target in a circuit with only two qubits can be constructed \
as: ",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.6241962514436703`*^9, {3.858338444743038*^9, 3.858338509984469*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellID->546744020,ExpressionUUID->"43041a90-1cb0-4a74-87e8-e4e8602d5436"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CNOT", "[", 
  RowBox[{"2", ",", "1", ",", "2"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellLabel->"In[1]:=",
 CellID->44218627,ExpressionUUID->"bf4a536c-cffd-4294-8939-45a4120a5a0c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "0", "1"},
     {"0", "0", "1", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098566617*^9},
 CellTags->"b:5.2.1",
 CellLabel->"Out[1]=",
 CellID->1434058039,ExpressionUUID->"21b94af7-8f88-4e48-ab1a-a7cba2d3d955"]
}, Open  ]],

Cell[TextData[{
 StyleBox["If we apply the ",
  FontFamily->"Helvetica"],
 StyleBox["CNOT ",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox["to an initial density matrix  |10\[RightAngleBracket]\
\[LeftAngleBracket]10|, we write, ",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellID->153122127,ExpressionUUID->"54c8dd22-ebd3-4b4a-823e-431c32cb6d2a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CNOT", "[", 
   RowBox[{"2", ",", "1", ",", "2"}], "]"}], " ", ".", " ", 
  RowBox[{"AF", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
    SubscriptBox["\[ScriptCapitalP]", "0"]}], "]"}], ".", 
  RowBox[{"Adj", "[", 
   RowBox[{"CNOT", "[", 
    RowBox[{"2", ",", "1", ",", "2"}], "]"}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614016158482085*^9, 3.614016162766512*^9}, 
   3.614016218093173*^9, 3.6241900374256907`*^9, 3.856038421379703*^9, 
   3.859055241281749*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellLabel->"In[2]:=",
 CellID->838179383,ExpressionUUID->"5eb244d2-f3e0-459b-9565-a5e824f42ff8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098589588*^9},
 CellTags->"b:5.2.1",
 CellLabel->"Out[2]=",
 CellID->1004491192,ExpressionUUID->"735da3b1-d631-4576-be3f-5a37c5a299e1"]
}, Open  ]],

Cell[TextData[StyleBox["which trivially  is seen to be equal to the density \
matrix for |11\[RightAngleBracket]\[LeftAngleBracket]11|",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellID->1935649408,ExpressionUUID->"1d2a3481-0ad2-4caa-a32c-d983326035bf"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"AF", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], "\[CircleTimes]", 
    SubscriptBox["\[ScriptCapitalP]", "1"]}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614016236722087*^9, 3.6140162795248423`*^9}, 
   3.6241900374336863`*^9, 3.8560384213817043`*^9, 3.8590552412834682`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellLabel->"In[3]:=",
 CellID->229831853,ExpressionUUID->"f5550a7b-12fc-44a5-b276-721084790a8d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098607891*^9},
 CellTags->"b:5.2.1",
 CellLabel->"Out[3]=",
 CellID->1114514877,ExpressionUUID->"da946fad-026b-4736-b336-22fcaa8a4717"]
}, Open  ]],

Cell[TextData[{
 StyleBox["A",
  FontFamily->"Helvetica"],
 StyleBox[" more general case",
  FontFamily->"Helvetica"],
 StyleBox[" would be to have a CNOT gate with control in qubit 3 and target \
in qubit 5 in a circuit with 6 qubits:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellID->1608356065,ExpressionUUID->"13d5adce-f13f-4538-a04c-4180b12fc59e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CNOT", "[", 
   RowBox[{"6", ",", "3", ",", "5"}], "]"}], ";"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.1",
 CellLabel->"In[4]:=",
 CellID->1263719666,ExpressionUUID->"8a662743-5658-4b86-ab58-90cfab17ff8e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["c) Two Qubit operations (CPHASE)",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"i:1", "c:12", "b:5.2.2"},
 CellLabel->"In[5]:=",
 CellID->723157632,ExpressionUUID->"f14f877b-8ff0-4d4e-b194-a27251469e91"],

Cell[TextData[{
 StyleBox["The ",
  FontWeight->"Regular"],
 StyleBox["CPHASE",
  FontWeight->"Regular",
  FontColor->GrayLevel[0]],
 StyleBox[" operation, used mainly in the cluster model approach, is defined \
by:",
  FontWeight->"Regular"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.8575864719248*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->1500362651,ExpressionUUID->"ecb0a9ef-6c94-48e3-a22b-5f78348fb1cc"],

Cell[TextData[StyleBox["CPHASE | a b \[RightAngleBracket] = (-1)^(ab)  | a b \
\[RightAngleBracket]",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 TextAlignment->-0.5,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->842092283,ExpressionUUID->"bf737c52-08be-41be-9416-b8e1837e81e7"],

Cell[TextData[StyleBox[" Its syntax is is similar to a CNOT gate, but in this \
case the order of the control and target qubits is not relevant. \nTherefore, \
CPHASE[L,q1,q2], performs a CPHASE gate on qubits q1 and q2 on a system of L \
qubits.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->1127766651,ExpressionUUID->"94636e20-b904-4fae-a254-03f72278173b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"CPHASE", "[", 
  RowBox[{"2", ",", "1", ",", "2"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"CPHASE", "[", 
  RowBox[{"2", ",", "2", ",", "1"}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellLabel->"In[1]:=",
 CellID->683434935,ExpressionUUID->"fdb8f1d6-a0c2-438b-a51e-2b7abf8b10ac"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "1", "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098664875*^9},
 CellTags->"b:5.2.2",
 CellLabel->"Out[1]=",
 CellID->1299168605,ExpressionUUID->"aeaa360d-6027-4483-bbae-cb368aa39e48"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "1", "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260986775627`*^9},
 CellTags->"b:5.2.2",
 CellLabel->"Out[2]=",
 CellID->1660505186,ExpressionUUID->"a59f1c24-5588-4dac-ac18-4722fee6c1b6"]
}, Open  ]],

Cell[TextData[{
 StyleBox["In a multi-qubit environment: of say three qubits, a ",
  FontFamily->"Helvetica"],
 StyleBox["CPHASE",
  FontFamily->"Helvetica",
  FontWeight->"Plain"],
 StyleBox[" between qubits 1 and 2 is:",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.856101113089987*^9, 3.856101121359082*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->1748408389,ExpressionUUID->"49271300-b953-46df-8d14-7417c58c2c95"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CPHASE", "[", 
  RowBox[{"3", ",", "2", ",", "1"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellLabel->"In[3]:=",
 CellID->1481052022,ExpressionUUID->"750862b4-8b1d-4af8-ae3f-5b521996bf51"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "1", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "1", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "1", "0", "0"},
     {"0", "0", "0", "0", "0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", "0", "0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260986939163`*^9},
 CellTags->"b:5.2.2",
 CellLabel->"Out[3]=",
 CellID->1560668337,ExpressionUUID->"6e22b8ed-b7be-42b1-b4c2-b65d66d6127c"]
}, Open  ]],

Cell[TextData[StyleBox["A different version of the same gate  is defined by \
CPHASEV[ i,  j,  a ]  and is used to get the CPHASE action on qubits i and j \
of a  state vector a.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->558381377,ExpressionUUID->"6a744824-9850-4aef-a43d-d3fed472eab3"],

Cell[TextData[StyleBox["For instance, if we apply CPHASE to the two qubit \
vector:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->212265849,ExpressionUUID->"07d12111-05a7-4ced-8283-a63e3c8c0a4c"],

Cell[TextData[StyleBox["CPHASE[ 2, 1, 2]   [  a | 0 0 \[RightAngleBracket]+  \
b | 0 1 \[RightAngleBracket] +  c | 1 0 \[RightAngleBracket] + d | 1 1 \
\[RightAngleBracket] ] \n =   a | 0 0\[RightAngleBracket]+ b | 0 1\
\[RightAngleBracket]+ c | 1 0\[RightAngleBracket]  -  d | 1 1\
\[RightAngleBracket]",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.858887224037364*^9},
 TextAlignment->-0.5,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->581786476,ExpressionUUID->"dd355a86-94c0-4e60-ad12-4911bb130386"],

Cell[TextData[StyleBox["Which corresponds to the following command:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.8567332802222633`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->592257341,ExpressionUUID->"47a4413c-726c-410a-bfe6-49502bcbd5db"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Clear", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"CPHASEV", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "}"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.858887199524274*^9, 3.858887215611477*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellLabel->"In[4]:=",
 CellID->482426342,ExpressionUUID->"87910459-7312-4023-9b21-1fa5f5bda4c2"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"a"},
      {"b"},
      {"c"},
      {
       RowBox[{"-", "d"}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260987191563`*^9},
 CellTags->"b:5.2.2",
 CellLabel->"Out[5]=",
 CellID->787475501,ExpressionUUID->"07db9489-d66f-44da-bfc1-3139f658d400"]
}, Open  ]],

Cell[TextData[StyleBox["Thus CPHASEV[  i  , j , a]  gives the result of  a \
CPHASE acting on qubits i and j for a state vector with expansion \
coefficients a: {a,b,c.....}.  The length of the array gives the total number \
of qubits.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.85833860357299*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellID->1388762569,ExpressionUUID->"0efad72b-15e4-4a54-af92-59c418c7d0a7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CPHASEV", "[", 
   RowBox[{"1", ",", "3", ",", 
    RowBox[{"{", 
     RowBox[{
     "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", "g", ",", 
      "h"}], "}"}]}], "]"}], "\[IndentingNewLine]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{3.858887263809703*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.2",
 CellLabel->"In[6]:=",
 CellID->2080221587,ExpressionUUID->"ef37a271-9694-4c04-888b-2ca6dea493d2"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"a"},
      {"b"},
      {"c"},
      {"d"},
      {"e"},
      {
       RowBox[{"-", "f"}]},
      {"g"},
      {
       RowBox[{"-", "h"}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260987378273`*^9},
 CellTags->"b:5.2.2",
 CellLabel->"Out[6]=",
 CellID->159028846,ExpressionUUID->"4c7367cd-9071-4539-8e8d-2967ae26a9b5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["d) Two Qubit operations, TwoOp",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"i:1", "c:13", "b:5.2.3"},
 CellLabel->"In[7]:=",
 CellID->1250439549,ExpressionUUID->"66fa64c3-4d88-45c6-bfae-e72b46411a05"],

Cell[TextData[StyleBox["General Two Qubit operations are build using TwoOp. \
Its syntax is very simple and allows one to build any two qubit operation \
embedded in an L qubit system. ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.856638137660818*^9, 3.856638140587247*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.3",
 CellID->862530501,ExpressionUUID->"e849669a-46c9-4ccf-94cb-45943aeda80e"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
  "The", "TwoOp", "function", "should", "be", "called", "with", "the", 
   "following", 
   RowBox[{"arguments", ":", "L", ":", 
    RowBox[{"total", "number", "of", "qubits"}]}]}], ",", 
  RowBox[{
   FormBox[
    SubscriptBox["q", "1"],
    TraditionalForm], "and", 
   FormBox[
    SubscriptBox["q", "2"],
    TraditionalForm]}], ",", 
  RowBox[{"qubits", "involved", "in", "the", "operation"}], ",", 
  RowBox[{
   FormBox[
    RowBox[{
     SubscriptBox["op", "1"], "and", 
     SubscriptBox["op", "2"]}],
    TraditionalForm], "are", "the", "Pauli"}]}], "\n", 
 RowBox[{
  RowBox[{"operators", 
   FormBox[
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Sigma]", 
       SubscriptBox["op", "1"]], ",", 
      SubscriptBox["\[Sigma]", 
       SubscriptBox["op", "2"]]}], ")"}],
    TraditionalForm], "performed", "on", 
   FormBox[
    SubscriptBox["q", "1"],
    TraditionalForm], "and", 
   RowBox[{
    FormBox[
     SubscriptBox["q", "2"],
     TraditionalForm], ".", "Thus"}], "for", "a", "three", "qubit", "system", 
   "with", 
   SubscriptBox["\[Sigma]", "0"], "acting", "on", "qubit", "1", "and", 
   SubscriptBox["\[Sigma]", "1"], "acting", "on", "qubit", "2"}], ",", 
  RowBox[{"the", "operator", "is"}]}]}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.62419648465779*^9, 3.6241964902938013`*^9}, {
  3.7946580594267597`*^9, 3.794658074093981*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.3",
 CellID->1035513057,ExpressionUUID->"58ec87eb-ad71-4cab-90bc-edac90886601"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TwoOp", "[", 
  RowBox[{"3", ",", "1", ",", "2", ",", "0", ",", "1"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.3",
 CellLabel->"In[1]:=",
 CellID->660473711,ExpressionUUID->"f15862e1-8c3b-487d-a7f3-12aaf8ae5449"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "1", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "0", "0", "0", "0"},
     {"1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "1", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "1"},
     {"0", "0", "0", "0", "1", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "1", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098785943*^9},
 CellTags->"b:5.2.3",
 CellLabel->"Out[1]=",
 CellID->1328157854,ExpressionUUID->"f74de61e-9abe-4a70-8a4e-41737350f955"]
}, Open  ]],

Cell[TextData[{
 StyleBox["For example, the CNOT and CPHASE gates between any two qubits in a \
total of L qubits have been coded using this function. A ThreeOp[ L,  i,  j, \
k, ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["op", "1"], TraditionalForm]],
  FontFamily->"Helvetica",ExpressionUUID->
  "4b79be38-6d41-4785-8f5a-f4a1c9a58106"],
 StyleBox[" ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["op", "2"], TraditionalForm]],
  FontFamily->"Helvetica",ExpressionUUID->
  "a2b4703e-4d73-4718-9543-093f5812d4b7"],
 StyleBox[" ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   SubscriptBox["op", "3"], TraditionalForm]],
  FontFamily->"Helvetica",ExpressionUUID->
  "d026d8cc-6b53-4f1a-b899-892d51ab5384"],
 StyleBox["] is also provided by QCPITT, which also serves as a building \
block for higher order operators.",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.85637644091549*^9, {3.8588873174478617`*^9, 3.858887318224134*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.3",
 CellID->238689434,ExpressionUUID->"244d9e5a-8a4d-4ff4-966d-09819b4774a7"]
}, Open  ]],

Cell[TextData[StyleBox["Circuits to Operators",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}, {3.8589681608234797`*^9, 
   3.858968166943308*^9}, {3.859543450496705*^9, 3.8595434594294777`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1928629920,ExpressionUUID->"edf09c97-aeae-4cfe-b9a0-31bfa6089694"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->364897622,ExpressionUUID->"6cd292e0-fe28-4e0f-8250-8d3c7d3ecbf2"],

Cell[TextData[StyleBox["Another feature to be included in a future QCPITT \
package is the ability to specify a circuit and then automatically generate \
the explicit operator or matrix corresponding to that circuit.  This is done \
by a few additional commands.   For example, we start by specifying a circuit \
  \"mat.\"  Then use command CircuitG[matr_] to produce a cleaned-up circuit. \
Follow that by opB=CircuitToOp[mat,  to obtain the associated operator.   To \
look at the operator at each stage use the command CircuitToOp2[Bell][[1]] \
and CircuitToOp2[Bell][[2]],etc.  See  tutorial CircuitToOp.nb for details.  \
A few examples are given below:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.85637644091549*^9, {3.8588873174478617`*^9, 3.858887318224134*^9}, {
   3.859543472501923*^9, 3.8595435650117607`*^9}, {3.8595693322733517`*^9, 
   3.859569468714789*^9}, {3.859569516157303*^9, 3.859569530827634*^9}, {
   3.859569568740905*^9, 3.859569603413514*^9}, {3.859569652138357*^9, 
   3.85956976943226*^9}, {3.859574189524024*^9, 3.859574251211586*^9}, {
   3.8595742862192497`*^9, 3.8595743343156548`*^9}, {3.8596174058534107`*^9, 
   3.859617615702159*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.3",
 CellID->1277398455,ExpressionUUID->"bd35af58-9d9a-4d23-a15c-80c29ba3945a"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"S0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mBell", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"H", ",", "S0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"C", ",", "CX"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mGHZ", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"H", ",", "S0", ",", "S0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"C", ",", "CX", ",", "S0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"C", ",", "S0", ",", "CX"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mToff", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"T1", ",", "T2", ",", "TX"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"T1", ",", "TX", ",", "T2"}], "}"}]}], "}"}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{
  3.860310749118313*^9, {3.860311029135301*^9, 3.860311035276671*^9}, 
   3.8603110901669283`*^9, {3.8603175410594273`*^9, 3.860317552815027*^9}, {
   3.860318029901596*^9, 3.8603180419741707`*^9}, 3.860318334974238*^9, {
   3.860324148879163*^9, 3.860324156978793*^9}, 3.860324511554227*^9, {
   3.860325660968679*^9, 3.860325666503224*^9}, 3.860325736671775*^9, {
   3.86032602045749*^9, 3.860326028320135*^9}},
 CellLabel->"In[2]:=",
 CellID->907264004,ExpressionUUID->"9f67f090-9d75-47d1-9641-442ca08b02dd"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"CircuitG", "[", "mBell", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FileNameJoin", "[", " ", 
   RowBox[{"{", 
    RowBox[{
    "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
     "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", "\"\<CToOpBell2.nb\>\""}],
     "}"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"NotebookEvaluate", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CNOT", "[", 
    RowBox[{"2", ",", "1", ",", "2"}], "]"}], ".", 
   RowBox[{"had", "[", 
    RowBox[{"2", ",", "1"}], "]"}]}], " ", "==", "%"}]}], "Input",
 CellChangeTimes->{{3.860324521134921*^9, 3.8603245359623127`*^9}, {
   3.860324668029551*^9, 3.860324694818315*^9}, {3.860324760324904*^9, 
   3.860324773793075*^9}, {3.860324864359558*^9, 3.860324896811282*^9}, 
   3.86032496294559*^9, {3.860325932425675*^9, 3.860325938881056*^9}},
 CellLabel->"In[6]:=",
 CellID->1964003762,ExpressionUUID->"82bc2856-328c-4c4c-8119-447e30ff13e5"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, 0.}, {0.7000000000000001, 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {
       0.7000000000000001, -0.30000000000000004`}}]}, 
    {EdgeForm[GrayLevel[0]], FaceForm[RGBColor[0.87, 0.94, 1]], 
     RectangleBox[{0.1, -0.1}, {0.30000000000000004, 0.1}], InsetBox[
      StyleBox["H",
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.2, 0.}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.30000000000000004`}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, -0.30000000000000004}]}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326098819684*^9},
 CellLabel->"Out[6]=",
 CellID->1404025260,ExpressionUUID->"8f18a51a-ed2f-4f81-a6f5-83748bd89551"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   StyleBox["\<\"Number of qubits =\"\>",
    StripOnInput->False,
    LineColor->RGBColor[1, 0, 0],
    FrontFaceColor->RGBColor[1, 0, 0],
    BackFaceColor->RGBColor[1, 0, 0],
    GraphicsColor->RGBColor[1, 0, 0],
    FontSize->18,
    FontColor->RGBColor[1, 0, 0]], "\[InvisibleSpace]", 
   InterpretationBox[Cell[BoxData[
     FormBox[
      StyleBox["2",
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontSize->18,
       FontColor->RGBColor[1, 0, 0]], TextForm]], "InlineText",ExpressionUUID->
     "e4e8d56b-327d-4496-b894-b7906bd6aa3c"],
    Text[
     Style[2, 
      RGBColor[1, 0, 0], 18]]]}],
  SequenceForm[
   Style["Number of qubits =", 18, 
    RGBColor[1, 0, 0]], 
   Text[
    Style[2, 
     RGBColor[1, 0, 0], 18]]],
  Editable->False]], "Print",
 CellChangeTimes->{3.860326099197194*^9},
 CellLabel->"During evaluation of In[15]:=",
 CellID->902231584,ExpressionUUID->"c30a653f-cb8f-4ef2-ac5d-566d0cd96c5f"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   StyleBox["\<\"Number of Steps = \"\>",
    StripOnInput->False,
    LineColor->RGBColor[1, 0, 0],
    FrontFaceColor->RGBColor[1, 0, 0],
    BackFaceColor->RGBColor[1, 0, 0],
    GraphicsColor->RGBColor[1, 0, 0],
    FontSize->18,
    FontColor->RGBColor[1, 0, 0]], "\[InvisibleSpace]", 
   InterpretationBox[Cell[BoxData[
     FormBox[
      StyleBox["2",
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontSize->18,
       FontColor->RGBColor[1, 0, 0]], TextForm]], "InlineText",ExpressionUUID->
     "ec317ba5-1164-4fe7-be60-1348d89ddc17"],
    Text[
     Style[2, 
      RGBColor[1, 0, 0], 18]]]}],
  SequenceForm[
   Style["Number of Steps = ", 18, 
    RGBColor[1, 0, 0]], 
   Text[
    Style[2, 
     RGBColor[1, 0, 0], 18]]],
  Editable->False]], "Print",
 CellChangeTimes->{3.860326099203382*^9},
 CellLabel->"During evaluation of In[15]:=",
 CellID->424200821,ExpressionUUID->"721fe36d-4f0c-4bca-bcdb-4c36b65bd47f"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   StyleBox["\<\"Number of Gates = \"\>",
    StripOnInput->False,
    LineColor->RGBColor[1, 0, 0],
    FrontFaceColor->RGBColor[1, 0, 0],
    BackFaceColor->RGBColor[1, 0, 0],
    GraphicsColor->RGBColor[1, 0, 0],
    FontSize->18,
    FontColor->RGBColor[1, 0, 0]], "\[InvisibleSpace]", 
   InterpretationBox[Cell[BoxData[
     FormBox[
      StyleBox["2",
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontSize->18,
       FontColor->RGBColor[1, 0, 0]], TextForm]], "InlineText",ExpressionUUID->
     "3df42d9a-6977-4cc6-9f2c-51b052936348"],
    Text[
     Style[2, 
      RGBColor[1, 0, 0], 18]]]}],
  SequenceForm[
   Style["Number of Gates = ", 18, 
    RGBColor[1, 0, 0]], 
   Text[
    Style[2, 
     RGBColor[1, 0, 0], 18]]],
  Editable->False]], "Print",
 CellChangeTimes->{3.8603260992067842`*^9},
 CellLabel->"During evaluation of In[15]:=",
 CellID->2044974154,ExpressionUUID->"46201f85-7647-43ff-b8d2-b96ad69fb173"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326099262176*^9},
 CellLabel->"Out[8]=",
 CellID->1552319690,ExpressionUUID->"b4788a50-7dcb-478c-915b-8681c802d5fa"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260992742777`*^9},
 CellLabel->"Out[9]=",
 CellID->58202427,ExpressionUUID->"f3dabc06-d625-45b4-9264-8cf2f9b58f9e"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{"CircuitG", "[", "mGHZ", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FileNameJoin", "[", " ", 
   RowBox[{"{", 
    RowBox[{
    "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
     "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", "\"\<CToOpGHZ2.nb\>\""}], 
    "}"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"NotebookEvaluate", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CNOT", "[", 
    RowBox[{"3", ",", "1", ",", "3"}], "]"}], ".", 
   RowBox[{"CNOT", "[", 
    RowBox[{"3", ",", "1", ",", "2"}], "]"}], ".", 
   RowBox[{"had", "[", 
    RowBox[{"3", ",", "1"}], "]"}]}], " ", "==", "%"}]}], "Input",
 CellChangeTimes->{{3.860324715200327*^9, 3.860324723559367*^9}, {
  3.8603250041497*^9, 3.86032505592284*^9}, {3.8603259036585913`*^9, 
  3.860325911250936*^9}},
 CellLabel->"In[10]:=",
 CellID->1726970606,ExpressionUUID->"90cc0339-b80d-4271-909f-094afb388905"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"CircuitG", "[", "mToff", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FileNameJoin", "[", " ", 
   RowBox[{"{", 
    RowBox[{
    "$UserBaseDirectory", ",", " ", "\"\<Applications\>\"", ",", 
     "\"\<QDENSITY\>\"", ",", "\"\<POPUP\>\"", ",", "\"\<CToOpToff2.nb\>\""}],
     "}"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"NotebookEvaluate", "[", "%", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Toffoli", "[", 
    RowBox[{"3", ",", "1", ",", "3", ",", "2"}], "]"}], ".", 
   RowBox[{"Toffoli", "[", 
    RowBox[{"3", ",", "1", ",", "2", ",", "3"}], "]"}]}], " ", "==", 
  "%"}]}], "Input",
 CellChangeTimes->{{3.860324715200327*^9, 3.860324723559367*^9}, {
  3.8603250041497*^9, 3.86032505592284*^9}, {3.8603259036585913`*^9, 
  3.860325911250936*^9}, {3.8603262359080753`*^9, 3.8603262986613083`*^9}},
 CellLabel->"In[3]:=",
 CellID->79360055,ExpressionUUID->"6948eaa6-dc37-4c3d-8e99-4f82c1bdeda8"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.6000000000000001}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, 0.}, {0.7000000000000001, 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {
       0.7000000000000001, -0.30000000000000004`}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.6000000000000001}, {
       0.7000000000000001, -0.6000000000000001}}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     PointBox[{0.5, -0.6000000000000001}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.30000000000000004`}}], 
      LineBox[{{0.5, -0.6000000000000001}, {0.5, -0.30000000000000004`}}]}}, 
    {PointSize[0.04000000000000001], PointBox[{0.2, 0.}], 
     PointBox[{0.2, -0.30000000000000004`}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.2, -0.6000000000000001}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.2, 0.}, {0.2, -0.6000000000000001}}], 
      LineBox[{{0.2, -0.30000000000000004`}, {0.2, -0.6000000000000001}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.9000000000000001, -0.6000000000000001}]}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326099367269*^9, 3.860326302038471*^9},
 CellLabel->"Out[3]=",
 CellID->1803152923,ExpressionUUID->"875b2402-d134-435f-9875-a6a0168f75fe"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   StyleBox["\<\"Number of qubits =\"\>",
    StripOnInput->False,
    LineColor->RGBColor[1, 0, 0],
    FrontFaceColor->RGBColor[1, 0, 0],
    BackFaceColor->RGBColor[1, 0, 0],
    GraphicsColor->RGBColor[1, 0, 0],
    FontSize->18,
    FontColor->RGBColor[1, 0, 0]], "\[InvisibleSpace]", 
   InterpretationBox[Cell[BoxData[
     FormBox[
      StyleBox["3",
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontSize->18,
       FontColor->RGBColor[1, 0, 0]], TextForm]], "InlineText",ExpressionUUID->
     "84c16af9-ddd3-4bdb-beed-6f32704d42bc"],
    Text[
     Style[3, 
      RGBColor[1, 0, 0], 18]]]}],
  SequenceForm[
   Style["Number of qubits =", 18, 
    RGBColor[1, 0, 0]], 
   Text[
    Style[3, 
     RGBColor[1, 0, 0], 18]]],
  Editable->False]], "Print",
 CellChangeTimes->{3.860326302427709*^9},
 CellLabel->"During evaluation of In[15]:=",
 CellID->301950397,ExpressionUUID->"2cf9d4aa-47fc-47d5-9a80-7a2d9ec90096"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   StyleBox["\<\"Number of Steps = \"\>",
    StripOnInput->False,
    LineColor->RGBColor[1, 0, 0],
    FrontFaceColor->RGBColor[1, 0, 0],
    BackFaceColor->RGBColor[1, 0, 0],
    GraphicsColor->RGBColor[1, 0, 0],
    FontSize->18,
    FontColor->RGBColor[1, 0, 0]], "\[InvisibleSpace]", 
   InterpretationBox[Cell[BoxData[
     FormBox[
      StyleBox["2",
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontSize->18,
       FontColor->RGBColor[1, 0, 0]], TextForm]], "InlineText",ExpressionUUID->
     "3af221e7-fe96-4dc6-955f-337777dc2a8f"],
    Text[
     Style[2, 
      RGBColor[1, 0, 0], 18]]]}],
  SequenceForm[
   Style["Number of Steps = ", 18, 
    RGBColor[1, 0, 0]], 
   Text[
    Style[2, 
     RGBColor[1, 0, 0], 18]]],
  Editable->False]], "Print",
 CellChangeTimes->{3.860326302434021*^9},
 CellLabel->"During evaluation of In[15]:=",
 CellID->376464518,ExpressionUUID->"ec77b679-095c-45cf-800a-dad04fe42700"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   StyleBox["\<\"Number of Gates = \"\>",
    StripOnInput->False,
    LineColor->RGBColor[1, 0, 0],
    FrontFaceColor->RGBColor[1, 0, 0],
    BackFaceColor->RGBColor[1, 0, 0],
    GraphicsColor->RGBColor[1, 0, 0],
    FontSize->18,
    FontColor->RGBColor[1, 0, 0]], "\[InvisibleSpace]", 
   InterpretationBox[Cell[BoxData[
     FormBox[
      StyleBox["2",
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontSize->18,
       FontColor->RGBColor[1, 0, 0]], TextForm]], "InlineText",ExpressionUUID->
     "57bae9b8-6edc-4076-bafb-1369ebdedc49"],
    Text[
     Style[2, 
      RGBColor[1, 0, 0], 18]]]}],
  SequenceForm[
   Style["Number of Gates = ", 18, 
    RGBColor[1, 0, 0]], 
   Text[
    Style[2, 
     RGBColor[1, 0, 0], 18]]],
  Editable->False]], "Print",
 CellChangeTimes->{3.860326302438122*^9},
 CellLabel->"During evaluation of In[15]:=",
 CellID->752329964,ExpressionUUID->"b8e33a9d-bfa6-494e-bdb6-f14baf7342e1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "1", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "1", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "1", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "1"},
     {"0", "0", "0", "0", "0", "1", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326099367269*^9, 3.860326302486582*^9},
 CellLabel->"Out[5]=",
 CellID->25991412,ExpressionUUID->"b0d57a29-2881-45cc-9a26-2473b5e3ea9d"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326099367269*^9, 3.8603263024996243`*^9},
 CellLabel->"Out[6]=",
 CellID->898481143,ExpressionUUID->"8eb7d353-b029-4425-a15b-f255aa597360"]
}, Open  ]],

Cell[TextData[StyleBox["Hopefully,  this will be extended to a drag and drop \
method for circuit assembly and also for direct input to Qiskit codes.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.85637644091549*^9, {3.8588873174478617`*^9, 3.858887318224134*^9}, {
   3.859543472501923*^9, 3.8595435650117607`*^9}, {3.8595693322733517`*^9, 
   3.859569468714789*^9}, {3.859569516157303*^9, 3.859569530827634*^9}, {
   3.859569568740905*^9, 3.859569603413514*^9}, {3.859569652138357*^9, 
   3.85956976943226*^9}, {3.859574189524024*^9, 3.859574251211586*^9}, {
   3.8595742862192497`*^9, 3.8595743343156548`*^9}, {3.8596174058534107`*^9, 
   3.859617615702159*^9}, {3.859617649834444*^9, 3.859617753471262*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.2.3",
 CellID->1803534785,ExpressionUUID->"2697c18c-ff70-40dc-b3a2-0ff25d39b02a"],

Cell[TextData[StyleBox["Measurements",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.8589677897478933`*^9, 3.85896780052565*^9}, {3.858967852254155*^9, 
   3.858967857946474*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->805520140,ExpressionUUID->"34995d9a-e47a-4781-95f1-09334f1e7618"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->640082292,ExpressionUUID->"14f0b4a7-f3eb-4e80-95a6-85c3a89a765a"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["a) Measuring  in the computational bases with density \
matrix",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"Measurements", "c:15", "b:5.3.0"},
 CellLabel->"In[14]:=",
 CellID->432727106,ExpressionUUID->"bfef1c7b-67ac-46fa-9f73-a960decca00a"],

Cell[TextData[StyleBox["A projective measurement on our system corresponds to \
a projection of the density matrix of the full system into the unmeasured \
subsystem.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.856730876961479*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellID->1464841800,ExpressionUUID->"7fd703ca-332a-4150-a237-cb8d245733c6"],

Cell[TextData[{
 StyleBox["Let us assume we have a four qubit system initially in the | 0 0 0 \
0 \[RightAngleBracket] state, we build  its density matrix ",
  FontFamily->"Helvetica"],
 "\[Rho]4",
 StyleBox["  after a chain of Hadamard and CNOT gates act as,",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellID->843603392,ExpressionUUID->"2087b84c-12f6-436b-a18e-601bb67f5daf"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Ope", "=", 
   RowBox[{
    RowBox[{"CNOT", "[", 
     RowBox[{"4", ",", "1", ",", "2"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"4", ",", "2"}], "]"}], ".", 
    RowBox[{"CNOT", "[", 
     RowBox[{"4", ",", "3", ",", "4"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"4", ",", "1"}], "]"}], ".", 
    RowBox[{"CNOT", "[", 
     RowBox[{"4", ",", "1", ",", "4"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"4", ",", "3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Rho]4", "=", 
  RowBox[{"Ope", ".", " ", 
   RowBox[{"AF", "[", 
    RowBox[{
     SubscriptBox["\[ScriptCapitalP]", "0"], "  ", "\[CircleTimes]", "  ", 
     SubscriptBox["\[ScriptCapitalP]", "0"], " ", "\[CircleTimes]", 
     SubscriptBox["\[ScriptCapitalP]", "0"], " ", "\[CircleTimes]", "  ", 
     SubscriptBox["\[ScriptCapitalP]", "0"]}], "]"}], ".", 
   RowBox[{"Adj", "[", "Ope", "]"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614016439032955*^9, 3.614016484861516*^9}, 
   3.624190037442855*^9, {3.856038421386258*^9, 3.856038421389282*^9}, {
   3.859055241285366*^9, 3.859055241287361*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellLabel->"In[1]:=",
 CellID->755171850,ExpressionUUID->"df3d8886-d538-4602-8838-bcfa86ff7263"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", 
      "0", "0"},
     {
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"], 
      FractionBox["1", "8"], "0", "0", 
      FractionBox["1", "8"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326099903573*^9},
 CellTags->"b:5.3.0",
 CellLabel->"Out[2]=",
 CellID->1805962118,ExpressionUUID->"07d4dc40-be9d-4f8b-b050-b934bbb048a8"]
}, Open  ]],

Cell[TextData[StyleBox["We can then measure for instance qubits 1 and 3 \
simultaneously, the projection operator would be:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellID->544929180,ExpressionUUID->"75a818a4-b3a1-46a0-b34f-3822be32edc5"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["P", 
   RowBox[{"a_", ",", "b_"}]], "  ", ":=", "  ", 
  RowBox[{"AF", "[", " ", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "a"], " ", "\[CircleTimes]", "  ", 
    "\[DoubleStruckCapitalI]", "\[CircleTimes]", 
    SubscriptBox["\[ScriptCapitalP]", "b"], "\[CircleTimes]", " ", 
    "\[DoubleStruckCapitalI]"}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614016522492498*^9, 3.614016554451228*^9}, 
   3.624190037451095*^9, {3.8560384213908463`*^9, 3.856038421393407*^9}, {
   3.859055241288877*^9, 3.85905524129084*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellLabel->"In[3]:=",
 CellID->1754457452,ExpressionUUID->"7567bfe0-3030-4320-9b8c-97fb10fa41a9"],

Cell[TextData[StyleBox["And therefore the state of the remaining subsystem, \
which depends on the values measured, after the measurement reads, ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellID->68438807,ExpressionUUID->"cbc42fde-6cd6-4047-a25f-d2fdacdb9ec1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]24", "[", 
    RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
   RowBox[{"Chop", "[", " ", 
    RowBox[{"PTr", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "3"}], "}"}], "  ", ",", 
      RowBox[{
       RowBox[{"2", "^", "2"}], "*", 
       RowBox[{
        SubscriptBox["P", 
         RowBox[{"a", ",", "b"}]], ".", "\[Rho]4", ".", 
        SubscriptBox["P", 
         RowBox[{"a", ",", "b"}]]}]}]}], " ", "]"}], "]"}]}], ";"}]], "Input",\

 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellLabel->"In[4]:=",
 CellID->1347278104,ExpressionUUID->"4e717616-586f-4547-8026-609143538737"],

Cell[TextData[StyleBox["If the result of the measurement was (01) then we \
would have the following density matrix for the remaining subsystem:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellID->635519960,ExpressionUUID->"211c0012-3e14-46fa-bba3-2dfdc1717141"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Rho]24", "[", 
  RowBox[{"0", ",", "1"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Tr", "[", 
  RowBox[{"\[Rho]24", "[", 
   RowBox[{"0", ",", "1"}], "]"}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellLabel->"In[5]:=",
 CellID->1747791189,ExpressionUUID->"0acf171e-9753-4e36-8db2-89181325c802"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603260999502296`*^9},
 CellTags->"b:5.3.0",
 CellLabel->"Out[5]=",
 CellID->271450528,ExpressionUUID->"e3f9ad02-a399-4fa9-b1a4-22dd98b09a81"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326099963005*^9},
 CellTags->"b:5.3.0",
 CellLabel->"Out[6]=",
 CellID->1013757868,ExpressionUUID->"08113570-68aa-41f7-8c44-77bd9b40b8ee"]
}, Open  ]],

Cell[TextData[StyleBox["The computation could be left depending on the values \
of the measurement:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellID->1845189857,ExpressionUUID->"7c90bc29-7053-48a1-8f31-37bd6cef62f1"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Rho]24", "[", 
  RowBox[{"a", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tr", "[", 
   RowBox[{"\[Rho]24", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614027714445733*^9, 3.614027774043529*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.0",
 CellLabel->"In[7]:=",
 CellID->294580272,ExpressionUUID->"9d7f315d-d755-43ce-a9d8-59bbab8942e8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}], "0", 
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}], "0"},
     {"0", 
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}], "0", 
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}]},
     {
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}], "0", 
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}], "0"},
     {"0", 
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}], "0", 
      RowBox[{
       FractionBox["1", "16"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], "2"]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326100023034*^9},
 CellTags->"b:5.3.0",
 CellLabel->"Out[7]=",
 CellID->93232178,ExpressionUUID->"0434d8fd-bbdc-4194-b14f-afe2eff599a1"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"2", " ", "a"}]]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"2", " ", "b"}]]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326100064123*^9},
 CellTags->"b:5.3.0",
 CellLabel->"Out[9]=",
 CellID->1437636983,ExpressionUUID->"4b6f6c52-9904-4e4c-92e5-f74b513dad81"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["b) Measuring  in the computational bases using \
MeasureKet",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"Measurements", "MeasureKet", "c:16", "b:5.3.1"},
 CellLabel->"In[10]:=",
 CellID->1060825169,ExpressionUUID->"3dd7fa66-0334-4084-bce6-902e9f0997d0"],

Cell[TextData[StyleBox["Consider a similar example to the previous one, using \
kets:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellID->879377989,ExpressionUUID->"4ac0b746-8587-448b-8344-9cc05179fceb"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Ope", "=", 
   RowBox[{
    RowBox[{"CNOT", "[", 
     RowBox[{"4", ",", "1", ",", "2"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"4", ",", "2"}], "]"}], ".", 
    RowBox[{"CNOT", "[", 
     RowBox[{"4", ",", "3", ",", "4"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"4", ",", "1"}], "]"}], ".", 
    RowBox[{"CNOT", "[", 
     RowBox[{"4", ",", "1", ",", "4"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"4", ",", "3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"ket1234", "=", 
  RowBox[{"Ope", ".", " ", 
   RowBox[{"AF", "[", 
    RowBox[{
     RowBox[{"Ket", "[", "0", "]"}], "  ", "\[CircleTimes]", "  ", 
     RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", 
     RowBox[{"Ket", "[", "0", "]"}], " ", "\[CircleTimes]", "  ", 
     RowBox[{"Ket", "[", "0", "]"}]}], "]"}]}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140278237220783`*^9, 3.6140278858873873`*^9}, 
   3.614027944609769*^9, 3.624190037459059*^9, {3.8560384213957853`*^9, 
   3.856038421398629*^9}, {3.8590552412925262`*^9, 3.859055241294395*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellLabel->"In[1]:=",
 CellID->723402469,ExpressionUUID->"0f502680-4556-4ebc-8263-83eba6cae38e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]},
     {"0"},
     {"0"},
     {
      FractionBox["1", 
       RowBox[{"2", " ", 
        SqrtBox["2"]}]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326100117443*^9},
 CellTags->"b:5.3.1",
 CellLabel->"Out[2]=",
 CellID->1026260884,ExpressionUUID->"c98686e0-0fb5-416d-aee4-05a122e21360"]
}, Open  ]],

Cell[TextData[StyleBox["We can then measure for instance qubits 1 and 3 \
simultaneously, using MeasureKet.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellID->1320370816,ExpressionUUID->"04572544-b478-4ace-9ff8-cecabda0c0bb"],

Cell[TextData[StyleBox["The state of the remaining two qubit subsystem, which \
depends on the values measured, after the measurement reads, ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.856731684971336*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellID->1523780061,ExpressionUUID->"249ee173-4749-4add-b9ad-cac13bde2a4b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ket24", "[", 
   RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
  RowBox[{"MeasureKet", "[", 
   RowBox[{"ket1234", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellLabel->"In[3]:=",
 CellID->1428994752,ExpressionUUID->"615ceb4e-3289-4eef-a6b0-946923de8f7c"],

Cell[TextData[StyleBox["If the result of the measurement was (01) then we \
would have the following ket for the remaining  qubit 2 and 4 subsystem:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellID->1805172463,ExpressionUUID->"98c93848-cb1a-4eca-83de-631afbe6a01a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ket24", "[", 
  RowBox[{"0", ",", "1"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellLabel->"In[4]:=",
 CellID->2032104472,ExpressionUUID->"894ac134-a2c8-4a90-b4be-d216b85943aa"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0"},
     {
      FractionBox["1", 
       SqrtBox["2"]]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326100150826*^9},
 CellTags->"b:5.3.1",
 CellLabel->"Out[4]=",
 CellID->1925508331,ExpressionUUID->"8bc3aa38-af4f-4e3b-84f3-32b2ef691ac0"]
}, Open  ]],

Cell[TextData[StyleBox["Construct the back to back density matrix for that \
state:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellID->2147422663,ExpressionUUID->"b18e7336-c2a6-43bf-9f19-fc03abbdd4b9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Rho]24k", "=", 
  RowBox[{"AF", "[", 
   RowBox[{
    RowBox[{"ket24", "[", 
     RowBox[{"0", ",", "1"}], "]"}], "\[CircleTimes]", 
    RowBox[{"Adj", "[", 
     RowBox[{"ket24", "[", 
      RowBox[{"0", ",", "1"}], "]"}], "]"}]}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614027995502036*^9, 3.6140280197666693`*^9}, 
   3.624190037466968*^9, 3.856038421400791*^9, 3.859055241295951*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellLabel->"In[5]:=",
 CellID->1122525559,ExpressionUUID->"d7ae5193-39b0-4652-a806-571dc26986e9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326100168901*^9},
 CellTags->"b:5.3.1",
 CellLabel->"Out[5]=",
 CellID->1587442474,ExpressionUUID->"9cca6f84-ade3-4fcf-966d-cfbca20c2e7d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Purity", "[", "\[Rho]24k", "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellLabel->"In[6]:=",
 CellID->643231218,ExpressionUUID->"3ae39e7c-dff6-4014-8b76-a10b70bad607"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261001955643`*^9},
 CellTags->"b:5.3.1",
 CellLabel->"Out[6]=",
 CellID->1956313837,ExpressionUUID->"769b8556-ed08-4951-9ed0-7d165e4be96e"]
}, Open  ]],

Cell[TextData[StyleBox["Which is identical to the density matrix obtained \
earlier",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.1",
 CellID->354401928,ExpressionUUID->"bf5e9b52-7748-4582-b998-150951546de6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["c) Example with measurements in the X and Y bases \
using the density matrix",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:17", "b:5.3.2"},
 CellLabel->"In[7]:=",
 CellID->1461050765,ExpressionUUID->"82f38740-2373-4da1-99f9-b92faa2cf654"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"figure", "=", 
   RowBox[{"Import", "[", 
    RowBox[{"\"\<QDENSITY/drawings/Had5q.png\>\"", ",", "\"\<PNG\>\""}], 
    "]"}]}], ";"}], "\n", 
 RowBox[{"Show", "[", "figure", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.856376440928895*^9, {3.856730920332514*^9, 3.856730932244432*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[1]:=",
 CellID->1552187557,ExpressionUUID->"9e247dfe-3605-458d-b6e5-a01656ea7945"],

Cell[BoxData[
 TagBox[
  GraphicsBox[RasterBox[CompressedData["
1:eJztnQlQFVe6x03eLHFqKsnMmGQmU5Z5mUzFqpgxMzoxmmWSGSdlMiZGJZpM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    "], {{0, 0}, {300., 143.}}, {0, 255},
    ColorFunction->RGBColor],
   PlotRange->{{0, 300.}, {0, 143.}}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261008774233`*^9},
 CellTags->"b:5.3.2",
 CellLabel->"Out[2]=",
 CellID->1792022889,ExpressionUUID->"649672a8-2b3e-4b35-88dc-28f59ef86e15"]
}, Open  ]],

Cell[TextData[StyleBox["Let us  assume we have a five qubit system built as \
in the figure, which corresponds to:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->2132410390,ExpressionUUID->"851a7271-8b08-4c25-a345-a13a7a4c6eea"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Ope2", "=", 
   RowBox[{
    RowBox[{"CPHASE", "[", 
     RowBox[{"5", ",", "1", ",", "2"}], "]"}], ".", 
    RowBox[{"CPHASE", "[", 
     RowBox[{"5", ",", "2", ",", "3"}], "]"}], ".", 
    RowBox[{"CPHASE", "[", 
     RowBox[{"5", ",", "3", ",", "4"}], "]"}], ".", 
    RowBox[{"CPHASE", "[", 
     RowBox[{"5", ",", "4", ",", "5"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]1", "=", "RandomQubit1"}], ";"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614028055934391*^9, 3.6140281077983847`*^9}, {
   3.614028143651721*^9, 3.614028179328395*^9}, 3.624190037474728*^9, {
   3.856038421402866*^9, 3.8560384214070787`*^9}, {3.856731766009605*^9, 
   3.856731791983487*^9}, 3.856731825544059*^9, {3.856731936572872*^9, 
   3.856731938314386*^9}, {3.856731980413677*^9, 3.8567320028934097`*^9}, {
   3.85673334640835*^9, 3.856733346971781*^9}, {3.8575747149353857`*^9, 
   3.8575747306838284`*^9}, {3.8575753437850733`*^9, 3.857575385188073*^9}, {
   3.8578507356379147`*^9, 3.8578507383710012`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[3]:=",
 CellID->66839947,ExpressionUUID->"827d49ca-b51b-489b-ab92-12dc66312362"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Rho]Initial", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{"\[Rho]1", " ", "\[CircleTimes]", 
       RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}], "\[CircleTimes]", 
       RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}], "\[CircleTimes]", " ", 
       RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}], "\[CircleTimes]", " ", 
       RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Rho]12345", "=", 
     RowBox[{"Chop", "[", " ", 
      RowBox[{"Ope2", ".", "\[Rho]Initial", ".", 
       RowBox[{"Adj", "[", "Ope2", "]"}]}], "]"}]}], ";"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857584678362439*^9, 3.857584685393661*^9}, {
   3.857667777949724*^9, 3.857667777954194*^9}, {3.857672409170528*^9, 
   3.857672425639933*^9}, {3.857716179816329*^9, 3.857716183691523*^9}, 
   3.857849609691996*^9, 3.857850782449897*^9, {3.85905524129777*^9, 
   3.8590552413005667`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[5]:=",
 CellID->1282197004,ExpressionUUID->"65957d83-be46-4e61-a011-a749a75c3f80"]
}, Open  ]],

Cell[TextData[StyleBox["We now measure qubit 1 in the X basis, qubits 2, 3 \
and 4 in the Y basis and check what  \nwe have in qubit 5 ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->245041848,ExpressionUUID->"2d1618ac-1449-48ae-a25c-ee91d2af4ca3"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["PP", 
   RowBox[{"a_", ",", "b_", ",", "c_", ",", "d_"}]], "  ", ":=", 
  RowBox[{
   RowBox[{"\[ScriptCapitalP]X", "[", "a", "]"}], " ", "\[CircleTimes]", "  ", 
   RowBox[{"\[ScriptCapitalP]Y", "[", "b", "]"}], "\[CircleTimes]", " ", 
   RowBox[{"\[ScriptCapitalP]Y", "[", "c", "]"}], "\[CircleTimes]", "  ", 
   RowBox[{"\[ScriptCapitalP]Y", "[", "d", "]"}], "\[CircleTimes]", 
   "\[DoubleStruckCapitalI]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614028196552731*^9, 3.614028252664482*^9}, 
   3.624190037483212*^9, {3.856038421408825*^9, 3.856038421412767*^9}, {
   3.8578510701982803`*^9, 3.857851088829143*^9}, {3.8578511250167227`*^9, 
   3.857851128652953*^9}, {3.859055241302536*^9, 3.859055241305591*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[7]:=",
 CellID->1692188037,ExpressionUUID->"288c82a6-0700-4937-8903-798913575614"],

Cell[TextData[{
 StyleBox["(this projector could have been written using ",
  FontFamily->"Helvetica"],
 StyleBox["MultiQProj[{1,2,2,2,0},{a,b,c,d,0}] )",
  FontFamily->"Helvetica",
  FontWeight->"Bold"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.856101741715795*^9, 3.8561017430096903`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->1478400638,ExpressionUUID->"99b76c70-de96-49a0-ae11-e727527f66c6"],

Cell[TextData[StyleBox["And therefore the state of the remaining subsystem, \
which depends on the values measured, after the measurement reads, ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->1093727076,ExpressionUUID->"9817bba0-0d7d-4715-a61f-08d47c5883e4"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]5", "[", 
    RowBox[{"a_", ",", "b_", ",", "c_", ",", "d_"}], "]"}], ":=", 
   RowBox[{"Chop", "[", " ", 
    RowBox[{"PTr", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], "  ", ",", 
      RowBox[{
       RowBox[{"2", "^", "4"}], "*", 
       RowBox[{
        SubscriptBox["PP", 
         RowBox[{"a", ",", "b", ",", "c", ",", "d"}]], ".", "\[Rho]12345", 
        ".", 
        RowBox[{"Adj", "[", 
         SubscriptBox["PP", 
          RowBox[{"a", ",", "b", ",", "c", ",", "d"}]], " ", "]"}]}]}]}], 
     "]"}], "]"}]}], ";"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[8]:=",
 CellID->1958431659,ExpressionUUID->"e522099b-b039-4124-80dd-6da8ad4ff503"],

Cell[TextData[StyleBox["If the result of the measurement was 1 for qubit 2 in \
the Z bases and 0 for qubit 3 in the X bases then the \nstate of qubits 1 and \
4 would be given by the following:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->879035244,ExpressionUUID->"656bd96d-80a3-44bf-ad97-e78e71bbd24c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Rho]5", "[", 
  RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614028290260133*^9, 3.614028304096052*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[9]:=",
 CellID->1405338195,ExpressionUUID->"b9550836-83b4-4867-a79e-efba18c94ec9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.2665659863028883`", 
      RowBox[{"0.2851212314411107`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.1863958745352417`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.2851212314411108`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.1863958745352416`", " ", "\[ImaginaryI]"}]}], 
      "0.7334340136971117`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326100995275*^9},
 CellTags->"b:5.3.2",
 CellLabel->"Out[9]=",
 CellID->70546248,ExpressionUUID->"d9ada68e-e25f-4a1c-8ad2-3c3e70e005c1"]
}, Open  ]],

Cell[TextData[StyleBox["d) Realization of a Hadamard in the Cluster Mode",
 FontWeight->"Bold",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 CellChangeTimes->{{3.858967958777458*^9, 3.85896797078585*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:17", "b:5.3.2"},
 CellLabel->"In[10]:=",
 CellID->1601367499,ExpressionUUID->"42eb8e31-7dca-4919-ba24-79725a069329"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.858967979140945*^9, 3.8589679807086973`*^9}},
 CellLabel->"In[1]:=",
 CellID->1762967566,ExpressionUUID->"36621e38-6ac0-4314-9866-510056697f52"],

Cell[TextData[StyleBox["Now we can relate this problem to the realization of \
a Hadamard in the Cluster Model, in fact, our \nexample corresponds exactly \
to Fig 2(d) of Raussendorf et al., Phys. Rev. A 022312 (2003). Therefore \
according to those authors the output should be, up to a prefactor depending \
on the previous measurements, a Hadamard of the first qubit:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.858887490441699*^9, 3.858887496516623*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->1493893659,ExpressionUUID->"5dcda514-b9c0-4011-a6f5-e80504fbbe14"],

Cell[TextData[StyleBox["The prefactor is the following:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->1639409736,ExpressionUUID->"08c5c505-c284-4a80-af16-2d6d06251bfa"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["U", 
     RowBox[{"\[CapitalSigma]", ",", "H"}]], "="}], TraditionalForm]],
  FontSize->14,ExpressionUUID->"1edc289d-ecbf-4268-9d54-f8e0e7660c76"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SuperscriptBox[
      SubscriptBox["\[Sigma]", "x"], 
      RowBox[{"a", "+", "c", "+", "d"}]],
     TraditionalForm], 
    SuperscriptBox[
     SubscriptBox["\[Sigma]", "z"], 
     RowBox[{"b", "+", "c"}]]}], TraditionalForm]],
  FontSize->14,ExpressionUUID->"c2a83fd7-1602-4387-a828-8326832571e5"]
}], "Text",
 TaggingRules->{},
 TextAlignment->-0.5,
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->1344241942,ExpressionUUID->"36340450-07e6-4dab-871d-c061c637effd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Prefactor", "[", 
   RowBox[{"a_", ",", "b_", ",", "c_", ",", "d_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"MatrixPower", "[", 
    RowBox[{
     SubscriptBox["\[Sigma]", "1"], ",", 
     RowBox[{"a", "+", "c", "+", "d"}]}], "]"}], ".", 
   RowBox[{"MatrixPower", "[", 
    RowBox[{
     SubscriptBox["\[Sigma]", "3"], ",", 
     RowBox[{"b", "+", "c"}]}], "]"}]}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[2]:=",
 CellID->1146352754,ExpressionUUID->"4f68c5f2-e4cf-4f96-b823-8f3425dff902"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]5Final", "[", 
   RowBox[{"a_", ",", "b_", ",", "c_", ",", "d_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Prefactor", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ".", 
   RowBox[{"\[Rho]5", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ".", 
   RowBox[{"Adj", "[", 
    RowBox[{"Prefactor", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[3]:=",
 CellID->1674939241,ExpressionUUID->"ba608194-9395-4f40-9f91-dfdfb0591002"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chop", "[", 
  RowBox[{"\[Rho]5Final", "[", 
   RowBox[{"1", ",", "1", ",", "0", ",", "1"}], "]"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[4]:=",
 CellID->1488178611,ExpressionUUID->"a3ecc4b2-cb74-4b65-a875-4e54249f5e29"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.2665659863028884`", 
      RowBox[{"0.2851212314411108`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.18639587453524165`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.2851212314411108`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.18639587453524165`", " ", "\[ImaginaryI]"}]}], 
      "0.7334340136971116`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101068037*^9},
 CellTags->"b:5.3.2",
 CellLabel->"Out[4]=",
 CellID->259816992,ExpressionUUID->"94269919-99f7-4690-9ac0-71d25cb637a6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chop", "[", 
  RowBox[{
   RowBox[{"had", "[", 
    RowBox[{"1", ",", "1"}], "]"}], ".", "\[Rho]1", ".", 
   RowBox[{"Adj", "[", 
    RowBox[{"had", "[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellLabel->"In[5]:=",
 CellID->75878468,ExpressionUUID->"fd89ad4c-9a40-4523-a227-4caf81d6912f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.26656598630288836`", 
      RowBox[{"0.28512123144111073`", "\[VeryThinSpace]", "-", 
       RowBox[{"0.18639587453524165`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.2851212314411107`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.18639587453524165`", " ", "\[ImaginaryI]"}]}], 
      "0.7334340136971114`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101086214*^9},
 CellTags->"b:5.3.2",
 CellLabel->"Out[5]=",
 CellID->909930193,ExpressionUUID->"707b5a4c-c20a-421b-abe6-80443ab77445"]
}, Open  ]],

Cell[TextData[StyleBox["The equivalence of the above two results confirms \
that the cluster diagram and the prefactor step does indeed represent a \
Hadamard.  This is a nice exercise to illustrate the cluster model way to \
henerate a specific gate.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.858887513270116*^9, 3.8588875581902637`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.3.2",
 CellID->1831609005,ExpressionUUID->"ca2b8319-7a4c-4350-a2c5-7b0ff8e6adfd"],

Cell[TextData[StyleBox["Entropy and Fidelity",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1560841593,ExpressionUUID->"1ec40ec8-2849-466e-8d16-e8ebd0b016ad"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1551905446,ExpressionUUID->"4e723664-6ab4-4d23-a380-d63a27e48b6f"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["a) Von Neumann Entropy",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:19", "b:5.4.0"},
 CellLabel->"In[6]:=",
 CellID->560075586,ExpressionUUID->"2a96dd91-1063-41c4-85d3-c1e7c447c3d2"],

Cell[TextData[{
 StyleBox["The Von Neumann entropy is defined as:  S[\[Rho]] = - Tr [\[Rho] \
Log[2,\[Rho] ] ];\n",
  FontFamily->"Helvetica"],
 StyleBox["We use command EnTropy for Von Neumann entropy",
  FontFamily->"Helvetica",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.614030534945223*^9, 3.61403055396966*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->769549101,ExpressionUUID->"79f7d0f2-136b-49d4-a57e-355196f6f9a2"],

Cell[TextData[{
 StyleBox["Which can be rewritten as  S[\[Rho]] = -\[Sum] ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["\[Rho]", "\[Lambda]"],
     TraditionalForm], 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      SubscriptBox["\[Rho]", "\[Lambda]"]}], "]"}]}], TraditionalForm]],
  ExpressionUUID->"4c9e6903-6301-4dee-aa3b-e58c8951a7ff"],
 StyleBox[" \nwhere ",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Rho]", "\[Lambda]"], " "}], TraditionalForm]],
  FontFamily->"Helvetica",ExpressionUUID->
  "d8771ce7-d1d0-4f3e-96f3-8dc2e4a465cd"],
 StyleBox["are the eigenvalues of \[Rho], and we keep the convention",
  FontFamily->"Helvetica"],
 " 0*Log[2,0]:=0"
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.857851183877388*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->1665218273,ExpressionUUID->"cb329c1c-cbb4-4c25-8eb1-0da9b75b5cc9"],

Cell[TextData[StyleBox["The Function EnTropy[ ] is already defined in QCPITT, \
let's check some examples, Ex 11.11 and 11.12 of Nielsen and Chuang:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.614030525890181*^9, 3.614030526837672*^9}, 
   3.856376440977313*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->1316626607,ExpressionUUID->"0f5bb2f7-1e80-41ec-beff-3bbd6f960b91"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"\[Rho]1", "=", 
  SubscriptBox["\[ScriptCapitalP]", 
   StyleBox["0",
    ScriptLevel->0]]}], "\[IndentingNewLine]", 
 RowBox[{"\[Rho]2", "=", 
  RowBox[{
   RowBox[{"1", "/", "2"}], "*", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"s", "[", "0", "]"}], "+", 
     RowBox[{"s", "[", "1", "]"}]}], ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"\[Rho]3", "=", 
  RowBox[{
   RowBox[{"1", "/", "3"}], "*", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"s", "[", "0", "]"}], "+", 
     RowBox[{"s", "[", "1", "]"}], "+", 
     SubscriptBox["\[ScriptCapitalP]", 
      StyleBox["0",
       ScriptLevel->0]]}], ")"}]}]}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellLabel->"In[1]:=",
 CellID->871885161,ExpressionUUID->"25386b9f-77c4-4cf6-a233-376cdde6e458"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0"},
     {"0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261011303473`*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[1]=",
 CellID->965703464,ExpressionUUID->"c095bab3-fecb-41e3-b107-909c4aa4cd94"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], 
      FractionBox["1", "2"]},
     {
      FractionBox["1", "2"], 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101143777*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[2]=",
 CellID->2044199069,ExpressionUUID->"e001e1cf-393f-4b8e-bcdc-d25fd174e905"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["2", "3"], 
      FractionBox["1", "3"]},
     {
      FractionBox["1", "3"], 
      FractionBox["1", "3"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101155346*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[3]=",
 CellID->1410367977,ExpressionUUID->"5fa16457-a60f-4879-b4a1-c0f667ef7558"]
}, Open  ]],

Cell[TextData[StyleBox["..and their entropies are:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->1492977446,ExpressionUUID->"5a62cfea-adc2-4dd8-8d29-031bfcdb1501"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"EnTropy", "[", "\[Rho]1", "]"}], "\[IndentingNewLine]", 
 RowBox[{"EnTropy", "[", "\[Rho]2", "]"}], "\[IndentingNewLine]", 
 RowBox[{"EnTropy", "[", "\[Rho]3", "]"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614030566535005*^9, 3.614030575275133*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellLabel->"In[4]:=",
 CellID->1273003374,ExpressionUUID->"80baf6e5-f1ab-41a7-a52b-169f2630e3de"],

Cell[BoxData[
 TagBox["0",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101172106*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[4]=",
 CellID->130093853,ExpressionUUID->"6231fc08-3aea-441c-be57-ba2b8f80fa80"],

Cell[BoxData[
 TagBox["0",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101184189*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[5]=",
 CellID->349034725,ExpressionUUID->"5c45bd6c-bca9-448b-82c9-618033c66c04"],

Cell[BoxData[
 TagBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"3", "-", 
        SqrtBox["5"]}], ")"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        FractionBox["1", "6"], " ", 
        RowBox[{"(", 
         RowBox[{"3", "-", 
          SqrtBox["5"]}], ")"}]}], "]"}]}], 
     RowBox[{"6", " ", 
      RowBox[{"Log", "[", "2", "]"}]}]]}], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"3", "+", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       FractionBox["1", "6"], " ", 
       RowBox[{"(", 
        RowBox[{"3", "+", 
         SqrtBox["5"]}], ")"}]}], "]"}]}], 
    RowBox[{"6", " ", 
     RowBox[{"Log", "[", "2", "]"}]}]]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261011949863`*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[6]=",
 CellID->1448783118,ExpressionUUID->"3fe88c77-124c-49d8-93bf-b6ec35837fb3"],

Cell[BoxData[
 TagBox["0.5500477595827574`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101207864*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[7]=",
 CellID->1528274540,ExpressionUUID->"2c33421e-361a-4c29-b258-ad62d6bc6639"]
}, Open  ]],

Cell[TextData[{
 StyleBox[" Now let\[CloseCurlyQuote]s check entropy of ",
  FontFamily->"Helvetica"],
 "\[Rho]",
 StyleBox["=  p | 0 > < 0 |  + (1-p) | + > < + |  case",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.6242802080850677`*^9, {3.624285236785616*^9, 3.624285238967291*^9}, {
   3.624285378446343*^9, 3.6242853789636106`*^9}, {3.624286081826631*^9, 
   3.624286196903373*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->2135615404,ExpressionUUID->"42524f38-3d8c-4854-9cbe-c9f6b6de1db1"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]Entropy", "[", "p_", "]"}], ":=", 
   RowBox[{
    RowBox[{"p", " ", 
     SubscriptBox["\[ScriptCapitalP]", "0"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "p"}], ")"}], "  ", 
     RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}]}]}]}], ";"}], "\n", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"EnTropy", "[", 
    RowBox[{"\[Rho]Entropy", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<p\>\"", ",", "\"\<Entropy\>\""}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"M", "[", "p_", "]"}], ":=", 
   RowBox[{"Eigenvalues", "[", 
    RowBox[{"\[Rho]Entropy", "[", "p", "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t1", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"M", "[", "p", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1", ",", ".1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t2", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"M", "[", "p", "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1", ",", ".1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"t1", ",", "t2"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624284897350209*^9, 3.624284898190442*^9}, {
   3.624285758241394*^9, 3.62428577728953*^9}, {3.6242862615386667`*^9, 
   3.624286270537112*^9}, {3.8046752682398567`*^9, 3.804675275811548*^9}, {
   3.8046755420145197`*^9, 3.804675544677816*^9}, 3.857667777956437*^9, {
   3.857672568182601*^9, 3.857672568902289*^9}, 3.857716199317803*^9, {
   3.8578512065628757`*^9, 3.8578512101355047`*^9}, {3.859053080195219*^9, 
   3.859053081468027*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellLabel->"In[8]:=",
 CellID->1216838074,ExpressionUUID->"77bb2d8c-2b3c-44a8-92db-b8d38f5c2a44"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwd2Hk0Ff8bB3BLRSlbJJEla/umVXlXaCGF7IVKCUklKbIWWSoUSaIspUSW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        "]]},
      Annotation[#, "Charting`Private`Tag$65492#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["\"p\"", TraditionalForm], 
     FormBox["\"Entropy\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 1}, {0., 0.6008760260407864}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261012943373`*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[9]=",
 CellID->1805775660,ExpressionUUID->"feea8ca8-ff01-4ca8-92f3-45c13e645ae4"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.012833333333333334`], AbsoluteThickness[1.6], 
      PointBox[{{1., 1.}, {2., 0.9527692569068709}, {3., 0.912310562561766}, {
       4., 0.8807886552931954}, {5., 0.860555127546399}, {6., 
       0.8535533905932737}, {7., 0.860555127546399}, {8., 
       0.8807886552931955}, {9., 0.912310562561766}, {10., 
       0.9527692569068709}, {11., 1.}}]}, 
     {RGBColor[0.880722, 0.611041, 0.142051], PointSize[
      0.012833333333333334`], AbsoluteThickness[1.6], 
      PointBox[{{1., 0.}, {2., 0.04723074309312919}, {3., 
       0.08768943743823399}, {4., 0.11921134470680464`}, {5., 
       0.13944487245360104`}, {6., 0.14644660940672627`}, {7., 
       0.13944487245360107`}, {8., 0.11921134470680457`}, {9., 
       0.08768943743823394}, {10., 0.04723074309312915}, {11., 
       0.}}]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->300,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 11.}, {0, 1.}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261013676233`*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[13]=",
 CellID->1432618216,ExpressionUUID->"79fa83cd-56dd-4800-8ef6-7e455f6e333d"]
}, Open  ]],

Cell[TextData[{
 StyleBox[" Now let\[CloseCurlyQuote]s check entropy of ",
  FontFamily->"Helvetica"],
 "\[Rho]",
 StyleBox["=  p | 0 > < 0 |  + (1-p) | - > < - |  case",
  FontFamily->"Helvetica"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.6242802080850677`*^9, {3.624285236785616*^9, 3.624285238967291*^9}, {
   3.624285378446343*^9, 3.6242853789636106`*^9}, {3.624286081826631*^9, 
   3.624286196903373*^9}, {3.624286365487219*^9, 3.6242863693788958`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->910739337,ExpressionUUID->"df8dea70-656e-46f3-bb03-c943dd8e682e"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Clear", "[", "\[Rho]Entropy", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]Entropy", "[", "p_", "]"}], ":=", 
   RowBox[{
    RowBox[{"p", " ", 
     SubscriptBox["\[ScriptCapitalP]", "0"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "p"}], ")"}], "  ", 
     RowBox[{"\[ScriptCapitalP]X", "[", "1", "]"}]}]}]}], ";"}], "\n", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"EnTropy", "[", 
    RowBox[{"\[Rho]Entropy", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<p\>\"", ",", "\"\<Entropy\>\""}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"M", "[", "p_", "]"}], ":=", 
   RowBox[{"Eigenvalues", "[", 
    RowBox[{"\[Rho]Entropy", "[", "p", "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t1", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"M", "[", "p", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1", ",", ".1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"t2", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"M", "[", "p", "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1", ",", ".1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"t1", ",", "t2"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624284897350209*^9, 3.624284898190442*^9}, {
   3.624285758241394*^9, 3.62428577728953*^9}, {3.6242862615386667`*^9, 
   3.624286270537112*^9}, {3.6242863073811398`*^9, 3.624286341525017*^9}, {
   3.804675301981517*^9, 3.804675306219284*^9}, {3.804675555322241*^9, 
   3.80467555613268*^9}, 3.8576677922190237`*^9, {3.857672598130665*^9, 
   3.8576725994124413`*^9}, 3.857716202691634*^9, {3.857851227792573*^9, 
   3.857851230701173*^9}, {3.8590530836791687`*^9, 3.859053085303851*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellLabel->"In[14]:=",
 CellID->1680084790,ExpressionUUID->"af3a70ce-64b0-45c5-9ca9-45cbe17841d2"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwd2Hk0Ff8bB3BLRSlbJJEla/umVXlXaCGF7IVKCUklKbIWWSoUSaIspUSW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        "]]},
      Annotation[#, "Charting`Private`Tag$65914#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["\"p\"", TraditionalForm], 
     FormBox["\"Entropy\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 1}, {0., 0.6008760260407864}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101468589*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[16]=",
 CellID->427706148,ExpressionUUID->"9db8fd82-25d8-49fe-b485-436c4c11c959"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.012833333333333334`], AbsoluteThickness[1.6], 
      PointBox[{{1., 1.}, {2., 0.9527692569068709}, {3., 0.912310562561766}, {
       4., 0.8807886552931954}, {5., 0.860555127546399}, {6., 
       0.8535533905932737}, {7., 0.860555127546399}, {8., 
       0.8807886552931955}, {9., 0.912310562561766}, {10., 
       0.9527692569068709}, {11., 1.}}]}, 
     {RGBColor[0.880722, 0.611041, 0.142051], PointSize[
      0.012833333333333334`], AbsoluteThickness[1.6], 
      PointBox[{{1., 0.}, {2., 0.04723074309312919}, {3., 
       0.08768943743823399}, {4., 0.11921134470680464`}, {5., 
       0.13944487245360104`}, {6., 0.14644660940672627`}, {7., 
       0.13944487245360107`}, {8., 0.11921134470680457`}, {9., 
       0.08768943743823394}, {10., 0.04723074309312915}, {11., 
       0.}}]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->300,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 11.}, {0, 1.}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261015406637`*^9},
 CellTags->"b:5.4.0",
 CellLabel->"Out[20]=",
 CellID->829462927,ExpressionUUID->"82788c0f-78e2-4bb3-a650-24d5436a4dfe"]
}, Open  ]],

Cell[TextData[StyleBox["Examples using GHZ and other Multi-qubit states are \
easily generated in QCPITT.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{3.856376441038207*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.0",
 CellID->2070074774,ExpressionUUID->"8be4c9db-7c8d-445a-9a95-308257c4b589"]
}, Open  ]],

Cell[TextData[StyleBox[" Let's see the behavior of the Fidelity for a couple \
of interesting examples:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 CellTags->"b:5.4.1",
 CellID->746807497,ExpressionUUID->"3c636c69-1843-4910-9312-31d43c7b56e6"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Clear", "[", 
  RowBox[{"\[Rho]Fid", ",", "Fide", ",", "pF1"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]Fid", "[", "p_", "]"}], ":=", 
   RowBox[{
    RowBox[{"p", " ", 
     SubscriptBox["\[ScriptCapitalP]", "0"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "p"}], ")"}], "  ", 
     RowBox[{"\[ScriptCapitalP]X", "[", "1", "]"}]}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Fide", "[", "p_", "]"}], ":=", 
  RowBox[{"Fidelity", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "0"], ",", 
    RowBox[{"\[Rho]Fid", "[", "p", "]"}]}], "]"}]}], "\n", 
 RowBox[{"pF1", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", 
      RowBox[{"Abs", "[", 
       RowBox[{"Fide", "[", "p", "]"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", 
      RowBox[{"-", "5"}], ",", "7"}], "}"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<p\>\"", ",", "\"\<Fidelity\>\""}], "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "300"}], ",", 
    RowBox[{"PlotRange", "->", "All"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{{3.859002685514183*^9, 3.859002686186205*^9}, {
   3.859002744856432*^9, 3.8590027681211863`*^9}, {3.859002802222246*^9, 
   3.859002854770295*^9}, {3.8590247238747673`*^9, 3.859024732392229*^9}, {
   3.8590401537040358`*^9, 3.859040155450886*^9}, {3.859040200642583*^9, 
   3.8590402589961452`*^9}, {3.85904029390723*^9, 3.859040311771497*^9}, {
   3.859040664835993*^9, 3.859040714214347*^9}, {3.859040773906068*^9, 
   3.8590407746574163`*^9}, 3.8590535643038692`*^9, {3.859053633975698*^9, 
   3.8590536381269503`*^9}},
 CellLabel->"In[21]:=",
 CellID->1841029403,ExpressionUUID->"bd048c81-aff1-4d2b-9040-00ebc4707bec"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQnbx+wvv//4UPMIDBB/s61VsH0toR/FmzlSed5UPw
59/vyA/kFILzxS4E+2uKCsL5JwKCXU6rCsD5k4K8Xri58MH5ZTE7febH88D5
30XPqqUEccH57957au/140CYpzTJZF8uG5xvz1q/WTuFBc6f4WBu7ljLBOen
z4l4+mo+A5y/1N9WiMnk734Y30kiIbku9yecL+lwZv7l0K9w/p0VXIaBgR/h
/BNt+6fbqryG872r/izuYn4M529ZdLbt/a1rcP4DM/nTW1fshPPV2V/NflZ8
2B5uHm/4i8snbsD5PWdPZmn0PYHzV26wOuem+wbO/9bB7MJ1+SOcHx+1wHTG
g69wPlPVMtlrK37C+TlWyVLWh/7C+V/uqD20fsLgAONfOBxgsOkIE5zP//sq
556zLHB+9s3+e7Jb2OD8wOkPOl5P4IDzZy3t2eszjQvOZ4jOfctayQPnn0pY
uTyohQ/Ov8nWu67aTADOFzor8IVZTRDOr7U+9O+qphCcX/aXlz1ERhjOX5je
fkVaWQTOn+h4pkdSVRTh/vzVTodFxeD8/S479m6TEofz/+XGm4XxScD5TOyX
Zmr/RfDNPYoOeTFIwfm/76x5IfoJwdc/JveGq1IazofkFwQfAJYyAE8=
        "]]},
      Annotation[#, "Charting`Private`Tag$66338#1"]& ], 
     TagBox[
      {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwllnk4lH0fxe1Ksg0qk2RPkRZbSd+bpJIIhSJLSESlogiVR0oqy1NosbQI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        "]]},
      Annotation[#, "Charting`Private`Tag$66338#2"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["\"p\"", TraditionalForm], 
     FormBox["\"Fidelity\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{All, All},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101686331*^9},
 CellLabel->"Out[24]=",
 CellID->1467061412,ExpressionUUID->"c6bcfd58-6c23-4908-9dbf-fd68a3cbe7f0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]Fid", "[", "p_", "]"}], ":=", 
   RowBox[{
    RowBox[{"p", " ", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "p"}], ")"}], "  ", 
     RowBox[{"\[ScriptCapitalP]X", "[", "1", "]"}]}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Fide", "[", "p_", "]"}], ":=", 
  RowBox[{"Fidelity", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "0"], ",", 
    RowBox[{"\[Rho]Fid", "[", "p", "]"}]}], "]"}]}], "\n", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"Abs", "[", 
      RowBox[{"Fide", "[", "p", "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", 
     RowBox[{"-", "5"}], ",", "7"}], "}"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<p\>\"", ",", "\"\<Fidelity\>\""}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}], ",", 
   RowBox[{"PlotRange", "->", "All"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.80467531893475*^9, 3.804675320979628*^9}, 
   3.857667792221098*^9, {3.857672636831163*^9, 3.857672637606798*^9}, 
   3.8577162049254217`*^9, {3.857851254938422*^9, 3.857851258084227*^9}, {
   3.8590020860244837`*^9, 3.859002176756308*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.1",
 CellLabel->"In[25]:=",
 CellID->1699169011,ExpressionUUID->"617bd12c-6b69-4040-9ce9-f8b288633cc5"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQnbx+wvv//4UPMIDBB/s61VsH0toR/FmzlSed5UPw
59/vyA/kFILzxS4E+2uKCsL5JwKCXU6rCsD5k4K8Xri58MH5ZTE7febH88D5
30XPqqUEccH57957au/140CYpzTJZF8uG5xvz1q/WTuFBc6f4WBu7ljLBOen
z4l4+mo+A5y/1N9WiMnk734Y30kiIbku9yecL+lwZv7l0K9w/p0VXIaBgR/h
/BNt+6fbqryG872r/izuYn4M529ZdLbt/a1rcP4DM/nTW1fshPPV2V/NflZ8
2B5uHm/4i8snbsD5PWdPZmn0PYHzV26wOuem+wbO/9bB7MJ1+SOcHx+1wHTG
g69wPlPVMtlrK37C+TlWyVLWh/7C+V/uqD20fsLgAONfOBxgsOkIE5zP//sq
556zLHB+9s3+e7Jb2OD8wOkPOl5P4IDzZy3t2eszjQvOZ4jOfctayQPnn0pY
uTyohQ/Ov8nWu67aTADOFzor8IVZTRDOr7U+9O+qphCcX/aXlz1ERhjOX5je
fkVaWQTOn+h4pkdSVRTh/vzVTodFxeD8/S479m6TEofz/+XGm4XxScD5TOyX
Zmr/RfDNPYoOeTFIwfm/76x5IfoJwdc/JveGq1IazofkFwQfAJYyAE8=
        "]]},
      Annotation[#, "Charting`Private`Tag$66597#1"]& ], 
     TagBox[
      {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJw1lmk8lAsDxZGyZLtZcj2VrZSk7YZCnadNsrSJ6tKGNpGbW8oUlWraiC5J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        "]]},
      Annotation[#, "Charting`Private`Tag$66597#2"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["\"p\"", TraditionalForm], 
     FormBox["\"Fidelity\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{All, All},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326101809174*^9},
 CellTags->"b:5.4.1",
 CellLabel->"Out[27]=",
 CellID->1641400277,ExpressionUUID->"23be53ab-3cf4-4787-b6d3-2b76ba0537c2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]Fid2", "[", "p_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "p", "]"}], "2"], " ", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "p", "]"}], "2"], "  ", 
     RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}]}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Fide", "[", "p_", "]"}], ":=", 
  RowBox[{"Fidelity", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "0"], ",", 
    RowBox[{"\[Rho]Fid2", "[", "p", "]"}]}], "]"}]}], "\n", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"Abs", "[", 
      RowBox[{"Fide", "[", "p", "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "0", ",", 
     RowBox[{"4", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<p\>\"", ",", "\"\<Fidelity\>\""}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}], ",", 
   RowBox[{"PlotRange", "->", "All"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.804675570849379*^9, 3.8046755725446167`*^9}, 
   3.857667777958435*^9, {3.857672647147566*^9, 3.857672647822057*^9}, 
   3.85771620718334*^9, {3.857851267212159*^9, 3.857851273665496*^9}, {
   3.8590021953428793`*^9, 3.8590022337502327`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.1",
 CellLabel->"In[28]:=",
 CellID->696030071,ExpressionUUID->"9b048550-5dd9-4b4f-92e4-f87f9b2f7359"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQzbyf69xj04l2DGDwwT4oR/LWjEn77WH8iZ9iNk+b
dB7Otz7qxeVU/ADOL6jyS9xc8ALOz92R2sek/AHOv/LCk3N50hc4v0v/G8O2
yB9wfqH2iakHlv6B838lZ6+7m8fgAOMn7Q0LmJDGBOcnf7xy4EAHC5zfY7A9
f1kbG5zPdGC7eUsVB5wvqOV97v4MLjjfY77T4g+TeOB88fdn3Oq388H5s47v
YmTOFIDzbWQd/yukCML52g8EntSWCMH5a1cIZWwtEobza+7FXtZuE4HzC20s
vOdNEYXzf813SXo9QQzOv1AwK6F8kTicb+mu9O/uQgk4/6Sr5QqHOZJw/tKN
My0frZaC80WVdbLiVkrD+ZcYOZok98jA+Vs5ipddOCkL5+uICmQlHZWD89/3
fRMxviYP5yu3t+vnZynA+Y+ibt9yeo7gezy9/HNrniKcX1GUmiH2FsH32zbb
S6xMCc5/9yerhvkbgr/weuOWvGJlRHhk2CZU/kbwZf9OjC6sUoHzn7l3n+Jg
UYXzD6udsZfrRvCzOJaxKHKqwfnbF6jYHp6I4J+ztjkaJKgO5//Imh2c0Ifg
nwiYH8orpgHnJ1naLF8+BcH32lVUGXgbwTe0Xun9W1ETzgcAebnwjg==
        "]]},
      Annotation[#, "Charting`Private`Tag$66839#1"]& ], 
     TagBox[
      {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJw1m3c01+/7x+1NNm/bG3knNIxSdF2pZKUQioxUCBEZldVEKtmhjEpIUXYZ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        "]]},
      Annotation[#, "Charting`Private`Tag$66839#2"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["\"p\"", TraditionalForm], 
     FormBox["\"Fidelity\"", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{All, All},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102095314*^9},
 CellTags->"b:5.4.1",
 CellLabel->"Out[30]=",
 CellID->439526177,ExpressionUUID->"d6a6bf32-8d2c-425f-9889-ea89084cafea"]
}, Open  ]],

Cell[BoxData[""], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.804675570849379*^9, 3.8046755725446167`*^9}, 
   3.8590535338939533`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Plain",
 CellTags->"b:5.4.1",
 CellLabel->"In[31]:=",
 CellID->56085277,ExpressionUUID->"72fa4c96-e9bd-48e1-809a-4f6bbab4835c"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Rho]Fid2", "[", "p_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "p", "]"}], "2"], " ", 
     SubscriptBox["\[ScriptCapitalP]", "1"]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Cos", "[", "p", "]"}], "2"], "  ", 
     RowBox[{"\[ScriptCapitalP]X", "[", "0", "]"}]}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Fide", "[", "p_", "]"}], ":=", 
  RowBox[{"Fidelity", "[", 
   RowBox[{
    SubscriptBox["\[ScriptCapitalP]", "1"], ",", 
    RowBox[{"\[Rho]Fid2", "[", "p", "]"}]}], "]"}]}], "\n", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", 
     RowBox[{"Abs", "[", 
      RowBox[{"Fide", "[", "p", "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p", ",", "0", ",", 
     RowBox[{"4", " ", "\[Pi]"}]}], "}"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<p\>\"", ",", "\"\<Fidelity\>\""}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}], ",", 
   RowBox[{"PlotRange", "->", "All"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.859002919087203*^9, 3.8590029231937037`*^9}},
 CellLabel->"In[32]:=",
 CellID->619177287,ExpressionUUID->"44243b68-cfba-40a1-80e4-93a333b7a50f"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQzbyf69xj04l2DGDwwT4oR/LWjEn77WH8iZ9iNk+b
dB7Otz7qxeVU/ADOL6jyS9xc8ALOz92R2sek/AHOv/LCk3N50hc4v0v/G8O2
yB9wfqH2iakHlv6B838lZ6+7m8fgAOMn7Q0LmJDGBOcnf7xy4EAHC5zfY7A9
f1kbG5zPdGC7eUsVB5wvqOV97v4MLjjfY77T4g+TeOB88fdn3Oq388H5s47v
YmTOFIDzbWQd/yukCML52g8EntSWCMH5a1cIZWwtEobza+7FXtZuE4HzC20s
vOdNEYXzf813SXo9QQzOv1AwK6F8kTicb+mu9O/uQgk4/6Sr5QqHOZJw/tKN
My0frZaC80WVdbLiVkrD+ZcYOZok98jA+Vs5ipddOCkL5+uICmQlHZWD89/3
fRMxviYP5yu3t+vnZynA+Y+ibt9yeo7gezy9/HNrniKcX1GUmiH2FsH32zbb
S6xMCc5/9yerhvkbgr/weuOWvGJlRHhk2CZU/kbwZf9OjC6sUoHzn7l3n+Jg
UYXzD6udsZfrRvCzOJaxKHKqwfnbF6jYHp6I4J+ztjkaJKgO5//Imh2c0Ifg
nwiYH8orpgHnJ1naLF8+BcH32lVUGXgbwTe0Xun9W1ETzgcAebnwjg==
        "]]},
      Annotation[#, "Charting`Private`Tag$67743#1"]& ], 
     TagBox[
      {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJw1nHk01N/7wGVfsu87wzBU2qOi54kWSykkFSKlIgmVFlRKpU0lFAlRKCVL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        "]]},
      Annotation[#, "Charting`Private`Tag$67743#2"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox["\"p\"", TraditionalForm], 
     FormBox["\"Fidelity\"", TraditionalForm]},
   AxesOrigin->{0, 0.7071067811865723},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{All, All},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261024894753`*^9},
 CellLabel->"Out[34]=",
 CellID->1920194806,ExpressionUUID->"568eed8f-28ba-4cbb-af38-f2d4633322c1"]
}, Open  ]],

Cell[TextData[StyleBox["Special States",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}, {3.8589681608234797`*^9, 
   3.858968166943308*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->836847359,ExpressionUUID->"1af3606f-3951-45b8-b9fc-ed7733087af3"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1023158510,ExpressionUUID->"d3e96145-fa72-4bdd-b7a0-e7c7be37d967"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["a) GHZ study  a second look",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 CellChangeTimes->{{3.856102598337225*^9, 3.856102598720098*^9}, {
  3.8588876516227016`*^9, 3.858887659295281*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:4", "b:5.4.2"},
 CellLabel->"In[35]:=",
 CellID->833978088,ExpressionUUID->"a5042c33-4e8a-4f68-bb87-27a3dcf5679d"],

Cell["\<\
Start with a pure three qubit state in the computational basis |abc\
\[RightAngleBracket]\[LeftAngleBracket]abc| :\
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->43961047,ExpressionUUID->"da8a42de-319a-4623-8b82-11040cdca14f"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[Rho]", 
   RowBox[{"a_", ",", "b_", ",", "c_"}]], ":=", 
  RowBox[{"AF", "[", " ", 
   RowBox[{
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "a"],
     ScriptLevel->0], "\[CircleTimes]", " ", 
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "b"],
     ScriptLevel->0], 
    StyleBox["\[CircleTimes]",
     ScriptLevel->0], 
    StyleBox[" ",
     ScriptLevel->0], 
    StyleBox[
     SubscriptBox["\[ScriptCapitalP]", "c"],
     ScriptLevel->0]}], 
   StyleBox["]",
    ScriptLevel->0]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614030638408154*^9, 3.614030645066778*^9}, {
   3.614030725108375*^9, 3.614030743657216*^9}, 3.624190037491364*^9, {
   3.8560384214166813`*^9, 3.856038421418412*^9}, 3.856730985507267*^9, 
   3.8575849135946503`*^9, 3.857658971329865*^9, {3.85905524130768*^9, 
   3.859055241309124*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[1]:=",
 CellID->118942361,ExpressionUUID->"4df0a2e5-26bf-4455-9b40-ebec7c494a13"],

Cell[" Generate the GHZ state operator as seen in the figure:", "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.858968191885439*^9, 3.85896819294307*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1904469617,ExpressionUUID->"1878d201-ae56-43da-86fd-3815d5816e49"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"matGHZ", "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", "  ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"H", ",", "1", ",", "1"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"C", ",", "1", ",", "CX"}], "}"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"C", ",", "CX", ",", "1"}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Circuit", "[", 
  RowBox[{"matGHZ", ",", 
   RowBox[{"{", "}"}]}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.481457700958976*^9, 3.481457928194841*^9}, {
   3.4814579643347883`*^9, 3.48145797980757*^9}, {3.481458028336564*^9, 
   3.481458054446849*^9}, {3.481458186798169*^9, 3.4814582120458813`*^9}, 
   3.4814582651618013`*^9, {3.481458304236559*^9, 3.48145830457121*^9}, 
   3.481467073022008*^9, {3.4814675652981157`*^9, 3.481467820358713*^9}, {
   3.481467885124991*^9, 3.4814679042091417`*^9}, 3.481469289025666*^9, {
   3.4858156648600082`*^9, 3.485815665119932*^9}, 3.804411073590955*^9, {
   3.80441111436487*^9, 3.8044111207102737`*^9}, {3.804411198492185*^9, 
   3.8044111994851103`*^9}, {3.8046758576118813`*^9, 3.804675908473631*^9}, {
   3.8046759491554747`*^9, 3.8046759653205547`*^9}, {3.804677191703292*^9, 
   3.804677216181799*^9}, {3.8046772542316933`*^9, 3.804677277070032*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[2]:=",
 CellID->177094384,ExpressionUUID->"81e22272-8376-433f-99e9-9657bb175343"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {-0.1, -0.6000000000000001}]}, 
    {GrayLevel[0], Thickness[0.007], LineBox[{{0, 0.}, {1., 0.}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.30000000000000004`}, {1., -0.30000000000000004`}}]}, 
    {GrayLevel[0], Thickness[0.007], 
     LineBox[{{0, -0.6000000000000001}, {1., -0.6000000000000001}}]}, 
    {EdgeForm[GrayLevel[0]], FaceForm[RGBColor[0.87, 0.94, 1]], 
     RectangleBox[{0.1, -0.1}, {0.30000000000000004, 0.1}], InsetBox[
      StyleBox["H",
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {0.2, 0.}]}, 
    {PointSize[0.04000000000000001], PointBox[{0.8, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.8, -0.30000000000000004}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.8, 0.}, {0.8, -0.30000000000000004`}}]}}, 
    {PointSize[0.04000000000000001], PointBox[{0.5, 0.}], 
     {RGBColor[0, 0, 1], InsetBox[
       StyleBox["\<\"\[CirclePlus]\"\>",
        StripOnInput->False,
        FontFamily->"Symbol",
        FontSize->34,
        FontWeight->"Bold"], {0.5, -0.6000000000000001}]}, 
     {GrayLevel[0], Thickness[Large], 
      LineBox[{{0.5, 0.}, {0.5, -0.6000000000000001}}]}}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, 0.}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, -0.30000000000000004}]}, 
    {GrayLevel[0], InsetBox[
      StyleBox[
       TemplateBox[{"\"\"", "\"\"", "\"\""},
        "RowDefault"],
       StripOnInput->False,
       FontFamily->"Times",
       FontSize->Scaled[0.05],
       FontWeight->"Bold"], {1.2000000000000002, -0.6000000000000001}]}},
   Background->GrayLevel[1],
   ImageSize->500],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102559887*^9},
 CellLabel->"Out[3]=",
 CellID->329857326,ExpressionUUID->"192e8347-a04b-4068-b0bb-d94d3cf0a9f0"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "GHZop", "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[4]:=",
 CellID->12977146,ExpressionUUID->"0ee2aa6d-b77e-4e12-8557-a2d4db919ff6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"GHZop", "[", 
   RowBox[{"qi_", ",", "qj_", ",", "qk_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"CNOT", "[", 
     RowBox[{"3", ",", "qi", ",", "qk"}], "]"}], ".", 
    RowBox[{"CNOT", "[", 
     RowBox[{"3", ",", "qi", ",", "qj"}], "]"}]}], ".", 
   RowBox[{"had", "[", 
    RowBox[{"3", ",", "qi"}], "]"}]}], " "}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[5]:=",
 CellID->1211044475,ExpressionUUID->"738a328c-2255-4956-bde0-759633f7b387"],

Cell[BoxData[
 RowBox[{"  ", 
  RowBox[{
   RowBox[{
   "GHZ\[Rho]", "   ", "is", " ", "the", " ", "density", " ", "matrix", "  ", 
    "of", " ", "any", " ", "of", " ", "the", " ", "GHZ", " ", "states"}], ",",
    " ", 
   RowBox[{"depending", " ", "on", " ", "the", " ", "initial", " ", 
    RowBox[{"state", ":", " "}]}]}]}]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->174485059,ExpressionUUID->"2df9481b-f8a6-43f7-8749-e9d4c09686c0"],

Cell[BoxData[
 StyleBox[
  RowBox[{" ", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      StyleBox[
       RowBox[{
        StyleBox["G",
         FontSize->18], 
        StyleBox["HZ",
         FontSize->12], 
        StyleBox["\[Rho]",
         FontSize->18]}]], 
      RowBox[{"a_", ",", "b_", ",", "c_"}]], "  ", ":=", 
     RowBox[{
      RowBox[{"GHZop", "[", 
       RowBox[{"1", ",", "2", ",", "3"}], "]"}], ".", 
      SubscriptBox["\[Rho]", 
       RowBox[{"a", ",", "b", ",", "c"}]], ".", 
      RowBox[{"Adj", "[", 
       RowBox[{"GHZop", "[", 
        RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}]}]}], ";"}]}],
  FontSize->18]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[6]:=",
 CellID->1255208458,ExpressionUUID->"941befc2-a701-4368-8e45-f60d660060e6"],

Cell[" Checks on some of the GHZ states: ", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1556689125,ExpressionUUID->"e7fd648e-dec6-4364-a986-7e519029b0c5"],

Cell[CellGroupData[{

Cell[BoxData[
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"0", ",", "0", ",", "0"}]]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[7]:=",
 CellID->480852474,ExpressionUUID->"81ac92b3-69fb-48f7-be76-2ce01e75a6b2"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {
      FractionBox["1", "2"], "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102602332*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[7]=",
 CellID->1256785175,ExpressionUUID->"d90617c6-0bc0-449a-8d9a-f2788b560f43"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"0", ",", "0", ",", "1"}]], "\[IndentingNewLine]", 
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"1", ",", "0", ",", "1"}]]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[8]:=",
 CellID->706566857,ExpressionUUID->"c5494493-5782-4ab1-9b4f-1e25d4175d1d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", "0", "0", "0", 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", "0", "0", "0", 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610262154*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[8]=",
 CellID->139284333,ExpressionUUID->"1be31ca7-fe0a-4200-b422-653f17ca62f8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], "0", "0", "0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0", "0", "0", 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102634776*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[9]=",
 CellID->194609501,ExpressionUUID->"d5ede5ab-72c7-4535-9bd5-15cd117748e9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"0", ",", "1", ",", "0"}]], "\[IndentingNewLine]", 
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"1", ",", "1", ",", "0"}]]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[10]:=",
 CellID->1832081336,ExpressionUUID->"7cef13a1-3b67-49b3-a815-d7f9fe63bf72"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", 
      FractionBox["1", "2"], "0", "0", 
      FractionBox["1", "2"], "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", 
      FractionBox["1", "2"], "0", "0", 
      FractionBox["1", "2"], "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102660309*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[10]=",
 CellID->184288926,ExpressionUUID->"31fa6b93-5c2d-4338-8283-1820a6c3b534"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", 
      FractionBox["1", "2"], "0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0", 
      FractionBox["1", "2"], "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261026741943`*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[11]=",
 CellID->672560308,ExpressionUUID->"c401c446-2c8d-4f72-b456-8ed523cd5752"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"0", ",", "1", ",", "1"}]], "\[IndentingNewLine]", 
 SubscriptBox[
  StyleBox[
   RowBox[{"G", 
    StyleBox["HZ",
     FontSize->12], "\[Rho]"}]], 
  RowBox[{"1", ",", "1", ",", "1"}]]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[12]:=",
 CellID->1170483466,ExpressionUUID->"4d247d11-efa0-4a43-8d56-eae24f3433b3"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "2"], 
      FractionBox["1", "2"], "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "2"], 
      FractionBox["1", "2"], "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102691607*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[12]=",
 CellID->1514662395,ExpressionUUID->"ba5be03f-1f6c-49be-b532-b7c935139c05"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "2"], 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0", "0"},
     {"0", "0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], 
      FractionBox["1", "2"], "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102704946*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[13]=",
 CellID->659721227,ExpressionUUID->"827f1198-eff5-4659-b607-6fb4ca53ae2a"]
}, Open  ]],

Cell["\<\
Now we will focus on one of the states, the usual GHZ, and study its vector \
and tensor polarizations :
 Vector polarizations are zero for GHZ states 
 Rank 2 Tensor polarizations for GHZ state 
 Rank 3 Tensor polarizations for GHZ state \
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.624280594189108*^9, 3.858968254009036*^9, {3.858968304512848*^9, 
   3.858968321599819*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1365406542,ExpressionUUID->"3758cfd0-0967-4f6c-8fde-b4f4ccd45ece"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{
    SubscriptBox["\[Rho]", "GHZ"], "=", 
    SubscriptBox[
     StyleBox[
      RowBox[{"G", 
       StyleBox["HZ",
        FontSize->12], "\[Rho]"}]], 
     RowBox[{"0", ",", "0", ",", "0"}]]}],
   FontSize->24], "\[IndentingNewLine]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.6140311479182453`*^9, {3.6140312160120497`*^9, 3.614031234866044*^9}, {
   3.61403180385396*^9, 3.6140318062157497`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[14]:=",
 CellID->377499552,ExpressionUUID->"a9cccde9-0e06-4373-be0b-e2b544788507"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0"},
     {
      FractionBox["1", "2"], "0", "0", "0", "0", "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610273059*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[14]=",
 CellID->476592728,ExpressionUUID->"95182a7c-a2a4-46e5-85ad-8cbe5f15812a"]
}, Open  ]]
}, Open  ]],

Cell[TextData[StyleBox["Rank 3 Tensor polarizations for GHZ state ",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.624280594189108*^9, 3.858968254009036*^9, {3.858968304512848*^9, 
   3.858968321599819*^9}, {3.858968409283328*^9, 3.8589684210737534`*^9}, {
   3.858968470810294*^9, 3.858968474061584*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->244036809,ExpressionUUID->"bd119060-d86e-41e4-8929-b70e47e43d1e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"T", "[", 
   RowBox[{"i_", ",", "j_", ",", "k_"}], "]"}], ":=", 
  RowBox[{"Tr", "[", 
   RowBox[{
    RowBox[{"AF", "[", 
     RowBox[{
      SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", 
      SubscriptBox["\[Sigma]", "j"], "\[CircleTimes]", 
      SubscriptBox["\[Sigma]", "k"]}], "]"}], ".", 
    SubscriptBox[
     StyleBox[
      RowBox[{"G", 
       StyleBox["HZ",
        FontSize->12], "\[Rho]"}]], 
     RowBox[{"0", ",", "0", ",", "0"}]]}], "]"}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.624190037499164*^9, {3.8560384214207478`*^9, 3.856038421422307*^9}, {
   3.859055241311007*^9, 3.859055241312194*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[15]:=",
 CellID->968277226,ExpressionUUID->"e51cde0d-b15b-4ad1-b38f-a60bd7dd5d9a"],

Cell[TextData[StyleBox["Polarizations for GHZ state ",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.624280594189108*^9, 3.858968254009036*^9, {3.858968304512848*^9, 
   3.858968321599819*^9}, {3.858968409283328*^9, 3.8589684210737534`*^9}, {
   3.858968470810294*^9, 3.858968502558064*^9}, 3.8590911419243803`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->463033963,ExpressionUUID->"e6665051-a251-4272-b97c-76fcc830ab4b"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"PGHZA", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"1", ",", "0", ",", "0"}], "]"}], ",", 
     RowBox[{"T", "[", 
      RowBox[{"2", ",", "0", ",", "0"}], "]"}], ",", 
     RowBox[{"T", "[", 
      RowBox[{"3", ",", "0", ",", "0"}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PGHZB", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"0", ",", "1", ",", "0"}], "]"}], ",", 
     RowBox[{"T", "[", 
      RowBox[{"0", ",", "2", ",", "0"}], "]"}], ",", 
     RowBox[{"T", "[", 
      RowBox[{"0", ",", "3", ",", "0"}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PGHZC", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"0", ",", "0", ",", "1"}], "]"}], ",", 
     RowBox[{"T", "[", 
      RowBox[{"0", ",", "0", ",", "2"}], "]"}], ",", 
     RowBox[{"T", "[", 
      RowBox[{"0", ",", "0", ",", "3"}], "]"}]}], "}"}]}], ";"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140308112799273`*^9, 3.614031081906542*^9}, {
   3.6140311246111403`*^9, 3.6140311298066998`*^9}, {3.6140311615443068`*^9, 
   3.614031237499469*^9}, {3.614031279696382*^9, 3.614031297515914*^9}, {
   3.614031339117463*^9, 3.614031357376213*^9}, 3.859091113597446*^9, 
   3.859091148573648*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[16]:=",
 CellID->204260416,ExpressionUUID->"3e6d7750-5483-4924-a97c-43464fe66bfa"],

Cell[TextData[StyleBox["Rank 2 Tensor polarizations for GHZ state ",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.624280594189108*^9, 3.858968254009036*^9, {3.858968304512848*^9, 
   3.858968321599819*^9}, {3.858968409283328*^9, 3.8589684210737534`*^9}, {
   3.858968470810294*^9, 3.858968502558064*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->2108443174,ExpressionUUID->"6ec9e7e1-20e9-4661-974d-c76ffdece25c"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"TGHZAB", 
   StyleBox["=",
    FontSize->12], 
   RowBox[{"{", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"i", ",", "j", ",", "0"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], "]"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TGHZAC", "=", 
   RowBox[{"{", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"i", ",", "0", ",", "j"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], "]"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TGHZBC", "=", 
   RowBox[{"{", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"0", ",", "i", ",", "j"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], "]"}], "}"}]}], 
  ";"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140314037778397`*^9, 3.614031515754469*^9}, 
   3.614031558269273*^9, {3.859091045657571*^9, 3.859091072016177*^9}, 
   3.859091109469974*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Plain",
 CellLabel->"In[19]:=",
 CellID->1325943806,ExpressionUUID->"4f2eb1d4-5739-4340-8724-7b83fa681bb8"],

Cell[TextData[StyleBox["Rank 3 Tensor polarizations for GHZ state ",
 FontSize->24]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.624280594189108*^9, 3.858968254009036*^9, {3.858968304512848*^9, 
   3.858968321599819*^9}, {3.858968409283328*^9, 3.8589684210737534`*^9}, {
   3.858968470810294*^9, 3.858968502558064*^9}, {3.859091076449885*^9, 
   3.859091077584738*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->553054382,ExpressionUUID->"07e77b04-3689-4544-a342-40e40e470fa5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TGHZABC", 
   StyleBox["=",
    FontSize->12], 
   RowBox[{"{", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"i", ",", "j", ",", "k"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "1", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", "1", ",", "3"}], "}"}]}], "]"}], "}"}]}], 
  ";"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140314037778397`*^9, 3.614031515754469*^9}, 
   3.614031558269273*^9, 3.859091045657571*^9, {3.859091083469672*^9, 
   3.859091101559937*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Plain",
 CellLabel->"In[22]:=",
 CellID->875774058,ExpressionUUID->"14ece725-9666-4ca9-a292-607efa8bc7b9"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"ArrayFlatten", "[", 
  RowBox[{"TGHZAB", ",", "1"}], 
  "]"}], "\[IndentingNewLine]", "TGHZAB", "\[IndentingNewLine]", 
 RowBox[{"ArrayFlatten", "[", 
  RowBox[{"TGHZAC", ",", "1"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"ArrayFlatten", "[", 
  RowBox[{"TGHZBC", ",", "1"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"ArrayFlatten", "[", 
  RowBox[{"TGHZABC", ",", "0"}], "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614031564509616*^9, 3.614031586486964*^9}, {
  3.6241935451189623`*^9, 3.624193545127453*^9}, {3.624227477695138*^9, 
  3.624227492793047*^9}, {3.857584955240952*^9, 3.857585062197701*^9}, {
  3.85905650294636*^9, 3.859056503749559*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[23]:=",
 CellID->699742523,ExpressionUUID->"ff67f2ba-2d53-4156-84db-13d09ea73ee8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102882023*^9},
 CellLabel->"Out[23]=",
 CellID->2127459833,ExpressionUUID->"667d2665-80ff-41ef-b847-099e109d6a97"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"0"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", 
       TagBox[GridBox[{
          {"0"},
          {"0"},
          {"1"}
         },
         GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.5599999999999999]}, 
             Offset[0.27999999999999997`]}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}}],
        Column], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102895224*^9},
 CellLabel->"Out[24]=",
 CellID->614690681,ExpressionUUID->"709e551f-ed4f-4176-90fa-093adc8219c1"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102907064*^9},
 CellLabel->"Out[25]=",
 CellID->1709933497,ExpressionUUID->"6b1ee898-776c-4cbc-b8ca-80ff36dcf312"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0"},
     {"0", "0", "0"},
     {"0", "0", "1"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610291826*^9},
 CellLabel->"Out[26]=",
 CellID->1226119925,ExpressionUUID->"485bcaca-eb30-4c24-bed2-4af39d8038ff"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"1", "0", "0"},
         {"0", 
          RowBox[{"-", "1"}], "0"},
         {"0", "0", "0"}
        },
        GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}}], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0", 
          RowBox[{"-", "1"}], "0"},
         {
          RowBox[{"-", "1"}], "0", "0"},
         {"0", "0", "0"}
        },
        GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}}], "\[NoBreak]", ")"}], 
      RowBox[{"(", "\[NoBreak]", GridBox[{
         {"0", "0", "0"},
         {"0", "0", "0"},
         {"0", "0", "0"}
        },
        GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.7]}, 
            Offset[0.27999999999999997`]}, "Rows" -> {
            Offset[0.2], {
             Offset[0.4]}, 
            Offset[0.2]}}], "\[NoBreak]", ")"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610292942*^9},
 CellLabel->"Out[27]=",
 CellID->144502863,ExpressionUUID->"094d1cba-978c-4302-a7a2-01e5d0cc5215"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"T", "[", 
  RowBox[{"1", ",", "1", ",", "1"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"T", "[", 
  RowBox[{"1", ",", "2", ",", "2"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"T", "[", 
  RowBox[{"2", ",", "1", ",", "2"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"T", "[", 
  RowBox[{"2", ",", "2", ",", "1"}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[28]:=",
 CellID->952133089,ExpressionUUID->"522569ab-0bae-4c9c-ad6c-4ce4a6fef04d"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261029534597`*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[28]=",
 CellID->2003097331,ExpressionUUID->"22b87560-7ac8-41e5-89d9-373d5c202adf"],

Cell[BoxData[
 TagBox[
  RowBox[{"-", "1"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102965498*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[29]=",
 CellID->2103191745,ExpressionUUID->"fca743f1-4e7d-4848-a5c5-a302b8e1988b"],

Cell[BoxData[
 TagBox[
  RowBox[{"-", "1"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102976462*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[30]=",
 CellID->762828595,ExpressionUUID->"65d05e81-7873-4e78-b549-60d22c9be11f"],

Cell[BoxData[
 TagBox[
  RowBox[{"-", "1"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326102987479*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[31]=",
 CellID->382823575,ExpressionUUID->"703dd229-0561-4ea1-835b-6cfcc1d2e55c"]
}, Open  ]],

Cell["We can reconstruct the Density Matrix ", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1600173026,ExpressionUUID->"977b8eb5-5c0f-4d4c-a694-41f04f4645ad"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "/", "8"}], ")"}], "  ", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "3"], "   ", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"j", "=", "0"}], "3"], "  ", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"k", "=", "0"}], "3"], "  ", 
      RowBox[{
       RowBox[{"T", "[", 
        RowBox[{"i", ",", "j", ",", "k"}], "]"}], "*", 
       RowBox[{"(", 
        RowBox[{
         SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", 
         SubscriptBox["\[Sigma]", "j"], "\[CircleTimes]", 
         SubscriptBox["\[Sigma]", "k"]}], ")"}]}]}]}]}]}], "==", 
  SubscriptBox[
   StyleBox[
    RowBox[{"G", 
     StyleBox["HZ",
      FontSize->12], "\[Rho]"}]], 
   RowBox[{"0", ",", "0", ",", "0"}]]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140316590285254`*^9, 3.614031678938483*^9}, {
   3.614031719966981*^9, 3.614031760182764*^9}, 3.624190037507101*^9, {
   3.856038421424555*^9, 3.856038421426176*^9}, {3.857585092955997*^9, 
   3.857585098380128*^9}, {3.859055241314563*^9, 3.859055241315955*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[32]:=",
 CellID->1170336601,ExpressionUUID->"0bc25726-f7ea-4a0a-b1e0-08e28ba068bb"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103121567*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[32]=",
 CellID->254818613,ExpressionUUID->"8e3fa3de-c761-4e65-8643-6a5495160947"]
}, Open  ]],

Cell["or, identically making use of the general SP function: ", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1044530134,ExpressionUUID->"930c24ab-5283-409a-b704-a75e9993ba2f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "/", "8"}], ")"}], "   ", 
   RowBox[{"(", "  ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0"}], "}"}]}], "]"}], "  ", "+", " ", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "3", ",", "0"}], "}"}]}], "]"}], "+", " ", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0", ",", "3"}], "}"}]}], "]"}], "+", " ", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "3", ",", "3"}], "}"}]}], "]"}], "+", " ", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1"}], "}"}]}], "]"}], "-", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "2"}], "}"}]}], "]"}], "-", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1", ",", "2"}], "}"}]}], "]"}], "-", 
     RowBox[{"SP", "[", 
      RowBox[{"3", ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2", ",", "1"}], "}"}]}], "]"}]}], ")"}]}], "==", 
  SubscriptBox[
   StyleBox[
    RowBox[{"G", 
     StyleBox["HZ",
      FontSize->12], "\[Rho]"}]], 
   RowBox[{"0", ",", "0", ",", "0"}]]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140317752912693`*^9, 3.6140317760579576`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[33]:=",
 CellID->1886631560,ExpressionUUID->"40e3c148-b675-404d-acfb-a7388342a736"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103146812*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[33]=",
 CellID->654036573,ExpressionUUID->"0d6d2a04-6bb1-4501-a733-dc0ad4f17055"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SP"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.62428064876297*^9, 3.624280650023032*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[34]:=",
 CellID->1020580965,ExpressionUUID->"eaf6241f-c904-4e82-b649-df4b8d5760f5"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"SP[Length[V],V]\\n\\n\\t SP returns the tensor or \
spin product(SP) of \\n\\n Length[V] qubits with the components\\n\\n \
specified by the array V.  The array V={0, 1,2,q,...}has \\n\\n a length \
equal to the number of qubits\\n\\n and the entries in the array V specify \
the\\n\\n components of the qth Pauli operator.  \\n\\n\\t For example:  \
SP[2,{3,2}] returns the same 4X4 matrix as\\n\\n \
AF[Sigma3\[CircleTimes]Sigma2].  The advantage of SP is that it\\n\\n works \
for large dimensional matrices.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[
            TagBox[GridBox[{
               {
                StyleBox[
                 TagBox[
                  TooltipBox[
                   StyleBox["\<\" Definitions\"\>", "InformationRowLabel",
                    StripOnInput->False],
                   "\"Definitions\"",
                   TooltipStyle->"TextStyling"],
                  Annotation[#, "Definitions", "Tooltip"]& ], "DialogStyle",
                 StripOnInput->False]},
               {
                TemplateBox[{"\"\[ThinSpace]\"", 
                  GridBox[{{
                    RowBox[{
                    RowBox[{"SP", "[", 
                    RowBox[{"Private`nqv_", ",", "Private`Q_"}], "]"}], ":=", 
                    
                    RowBox[{"ArrayFlatten", "[", 
                    RowBox[{"Private`Sp", "[", 
                    RowBox[{"Private`nqv", ",", "Private`Q"}], "]"}], 
                    "]"}]}]}, {" "}}, 
                   GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                   DefaultBaseStyle -> "Column", 
                   GridBoxItemSize -> {
                    "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}]},
                 "RowDefault"]}
              },
              DefaultBaseStyle->"Column",
              GridBoxAlignment->{"Columns" -> {{Left}}},
              
              GridBoxItemSize->{
               "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
             "Column"],
            FrameMargins->{{6, 0}, {3, 6}}], "\<\"\"\>"},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`SP\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"SP[Length[V],V]\\n\\n\\t SP returns the tensor or \
spin product(SP) of \\n\\n Length[V] qubits with the components\\n\\n \
specified by the array V.  The array V={0, 1,2,q,...}has \\n\\n a length \
equal to the number of qubits\\n\\n and the entries in the array V specify \
the\\n\\n components of the qth Pauli operator.  \\n\\n\\t For example:  \
SP[2,{3,2}] returns the same 4X4 matrix as\\n\\n \
AF[Sigma3\[CircleTimes]Sigma2].  The advantage of SP is that it\\n\\n works \
for large dimensional matrices.\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "SP[Length[V],V]\n\n\t SP returns the tensor or spin product(SP) of \n\n \
Length[V] qubits with the components\n\n specified by the array V.  The array \
V={0, 1,2,q,...}has \n\n a length equal to the number of qubits\n\n and the \
entries in the array V specify the\n\n components of the qth Pauli operator.  \
\n\n\t For example:  SP[2,{3,2}] returns the same 4X4 matrix as\n\n AF[Sigma3\
\[CircleTimes]Sigma2].  The advantage of SP is that it\n\n works for large \
dimensional matrices.", "Documentation" -> None, "OwnValues" -> None, 
     "UpValues" -> None, "DownValues" -> 
     Information`InformationValueForm[
      DownValues, QDENSITY`Qdensity`SP, {QDENSITY`Qdensity`SP[
          Pattern[Private`nqv, 
           Blank[]], 
          Pattern[Private`Q, 
           Blank[]]] :> ArrayFlatten[
          Private`Sp[Private`nqv, Private`Q]]}], "SubValues" -> None, 
     "DefaultValues" -> None, "NValues" -> None, "FormatValues" -> None, 
     "Options" -> None, "Attributes" -> {}, "FullName" -> 
     "QDENSITY`Qdensity`SP"|>, False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103265429*^9},
 CellLabel->"Out[34]=",
 CellID->1571377065,ExpressionUUID->"61967dd7-8761-4fa9-873c-ce431109323c"]
}, Open  ]],

Cell["Check on the Entropy of the full system and Sub Entropies", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->952390166,ExpressionUUID->"df6bbfc4-3200-4263-b932-13ddd21656fb"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EnTropy", "[", 
  StyleBox[
   SubscriptBox["\[Rho]", "GHZ"],
   FontSize->24], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140317839190683`*^9, 3.6140317864496403`*^9}, {
  3.614031832899411*^9, 3.6140318342238703`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[35]:=",
 CellID->1625898871,ExpressionUUID->"fbd59ee9-7d97-4612-b8ab-1ae549d780fc"],

Cell[BoxData[
 TagBox["0",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103290082*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[35]=",
 CellID->380440649,ExpressionUUID->"a2a088b2-718e-4878-a017-8c620c54a7bf"]
}, Open  ]],

Cell["\<\
The density matrix of the subsystem [a,b] can be obtained readily, and its 
decomposition into products of Pauli matrices is explicitly given\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{3.857585169506351*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1900400469,ExpressionUUID->"c79efdd7-3e4c-47c5-a57f-94b4585fe558"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032030845154*^9, 3.6140320377964287`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[36]:=",
 CellID->506400652,ExpressionUUID->"61faaa55-732b-4c85-8dc5-120448f439ca"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"2"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103313506*^9},
 CellLabel->"Out[36]=",
 CellID->1007170624,ExpressionUUID->"e3b22ae9-8cff-4f03-9b37-6ca9bf3c97d3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "PartialTrace"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140320605305977`*^9, 3.6140320814856873`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[37]:=",
 CellID->598065103,ExpressionUUID->"198f2f50-5945-463a-a847-2b0244ce2b23"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   StyleBox[
    FrameBox[
     DynamicModuleBox[{System`InformationDump`open$$ = False, 
      System`InformationDump`mouseOver$$ = False}, 
      PaneSelectorBox[{True->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False,
              BaseStyle -> None],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"PTrS[q,L,V] and PTr[traceout,\[CapitalOmega]] \
\\n\\n\\n\\n PTr[traceout,\[CapitalOmega]] returns the partial trace \\n\\n \
of general operator \[CapitalOmega] over those qubits \\n\\n specified by the \
traceout array 'traceout'. \\n\\n\\n\\n PTrS[q, L,V] returns the partial \
trace of spin basis \\n\\n operator of length nqv over single qubit q within \
nqv qubits, \\n\\n with the array V specifying the spin basis operator.\\n\\n \
This method reduces the operator down to the requisite smaller \
arrays.\\n\\n\\n\\n  see also PTrGen\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           PaneBox[GridBox[{
              {
               
               DynamicModuleBox[{System`InformationDump`open$$ = {
                False, False, False, False, False, False}}, 
                StyleBox[GridBox[{
                   {
                    TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"QDENSITY`Qdensity`PartialTrace\"\>"}
                  },
                  AutoDelete->False,
                  GridBoxAlignment->{"Columns" -> {Right, Left}},
                  GridBoxDividers->None,
                  GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                  GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                 StripOnInput->False],
                DynamicModuleValues:>{}]}
             },
             DefaultBaseStyle->"Column",
             GridBoxAlignment->{"Columns" -> {{Left}}},
             GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
             
             GridBoxItemSize->{
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
             GridBoxSpacings->{"Columns" -> {
                 Offset[0.27999999999999997`], {
                  Offset[0.5599999999999999]}, 
                 Offset[0.27999999999999997`]}, "Rows" -> {
                 Offset[0.2], {
                  Offset[3.6]}, 
                 Offset[0.2]}}],
            FrameMargins->{{6, 6}, {6, 3}}], ""},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpener"]], True->
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, False],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"], False->
       TagBox[GridBox[{
          {
           ItemBox[
            PaneBox[
             StyleBox["\<\" Symbol\"\>", "InformationTitleText",
              StripOnInput->False],
             FrameMargins->{{4, 0}, {-1, 1}}],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            PaneBox[
             
             StyleBox["\<\"PTrS[q,L,V] and PTr[traceout,\[CapitalOmega]] \
\\n\\n\\n\\n PTr[traceout,\[CapitalOmega]] returns the partial trace \\n\\n \
of general operator \[CapitalOmega] over those qubits \\n\\n specified by the \
traceout array 'traceout'. \\n\\n\\n\\n PTrS[q, L,V] returns the partial \
trace of spin basis \\n\\n operator of length nqv over single qubit q within \
nqv qubits, \\n\\n with the array V specifying the spin basis operator.\\n\\n \
This method reduces the operator down to the requisite smaller \
arrays.\\n\\n\\n\\n  see also PTrGen\"\>", "InformationUsageText",
              StripOnInput->False,
              LineSpacing->{1.5, 1.5, 3.}],
             FrameMargins->{{10, 10}, {8, 10}}],
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False], 
           ItemBox["\<\"\"\>",
            BaseStyle->"InformationUsageSubtitleBackground",
            StripOnInput->False]},
          {
           ItemBox[
            TagBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {2., 8.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpenerHot"],
                ImageSizeCache->{10., {2., 8.}}]}, Dynamic[
               System`InformationDump`mouseOver$$]],
              Alignment->Left,
              Appearance->{"Default" -> None},
              
              ButtonFunction:>FEPrivate`Set[
               System`InformationDump`open$$, True],
              Evaluator->Automatic,
              FrameMargins->{{9, 0}, {0, 0}},
              ImageMargins->0,
              ImageSize->Full,
              Method->"Preemptive"],
             
             EventHandlerTag[{
              "MouseEntered" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
               "MouseExited" :> 
               FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
               Method -> "Preemptive", PassEventsDown -> Automatic, 
               PassEventsUp -> True}]],
            BaseStyle->"InformationTitleBackground",
            StripOnInput->False], "\[SpanFromLeft]"}
         },
         AutoDelete->False,
         FrameStyle->Directive[
           GrayLevel[0.8], 
           Thickness[Tiny]],
         GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
         
         GridBoxDividers->{
          "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
        "Grid"]}, Dynamic[System`InformationDump`open$$],
       BaselinePosition->Baseline,
       FrameMargins->0,
       ImageSize->Automatic],
      DynamicModuleValues:>{}],
     BaseStyle->"InformationGridFrame",
     StripOnInput->False], "InformationGridPlain",
    StripOnInput->False],
   InformationData[<|
    "ObjectType" -> "Symbol", "Usage" -> 
     "PTrS[q,L,V] and PTr[traceout,\[CapitalOmega]] \n\n\n\n PTr[traceout,\
\[CapitalOmega]] returns the partial trace \n\n of general operator \
\[CapitalOmega] over those qubits \n\n specified by the traceout array \
'traceout'. \n\n\n\n PTrS[q, L,V] returns the partial trace of spin basis \n\n\
 operator of length nqv over single qubit q within nqv qubits, \n\n with the \
array V specifying the spin basis operator.\n\n This method reduces the \
operator down to the requisite smaller arrays.\n\n\n\n  see also PTrGen", 
     "Documentation" -> None, "OwnValues" -> None, "UpValues" -> None, 
     "DownValues" -> None, "SubValues" -> None, "DefaultValues" -> None, 
     "NValues" -> None, "FormatValues" -> None, "Options" -> None, 
     "Attributes" -> {}, "FullName" -> "QDENSITY`Qdensity`PartialTrace"|>, 
    False]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103403908*^9},
 CellLabel->"Out[37]=",
 CellID->1050571914,ExpressionUUID->"eee7ff0f-0aa1-4030-99ba-95818a06ee69"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Subsys", "[", 
   RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"Complement", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "b"}], "}"}]}], "]"}], ",", 
    SubscriptBox["\[Rho]", "GHZ"]}], 
   StyleBox["]",
    FontSize->12]}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[38]:=",
 CellID->1456718161,ExpressionUUID->"95c3c0c2-992e-42e4-934e-f3b44628bf0e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsys", "[", 
  RowBox[{"2", ",", "3"}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032130076726*^9, 3.61403215132615*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[39]:=",
 CellID->1748556127,ExpressionUUID->"5f3f5934-304b-419b-9693-c2ca00cd8a30"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103437819*^9},
 CellLabel->"Out[39]=",
 CellID->351612964,ExpressionUUID->"cc859ba3-d4ca-4bba-87c5-27d749054ff9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsys23", "=", 
  RowBox[{"Subsys", "[", 
   RowBox[{"2", ",", "3"}], "]"}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[40]:=",
 CellID->1816600941,ExpressionUUID->"a1a441ab-f990-4bd7-814e-23f73d610cef"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103463462*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[40]=",
 CellID->1831573051,ExpressionUUID->"6e24f2e0-7271-48dc-ab0f-7f7956320d28"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Subsys23", "=", 
  RowBox[{"Subsys", "[", 
   RowBox[{"2", ",", "3"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SpinObs", "[", "%", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.61403187202464*^9, 3.614031878287242*^9}, {
  3.624280697098756*^9, 3.624280708017912*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[41]:=",
 CellID->837005988,ExpressionUUID->"dd1af27d-069c-4786-81fa-eb230de472ef"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103487711*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[41]=",
 CellID->1217672059,ExpressionUUID->"55c9c578-e3c5-4fb9-aa92-da67b44425f9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "0", "0"},
            {"0", "0", "0"},
            {"0", "0", "1"}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103500927*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[42]=",
 CellID->2042623350,ExpressionUUID->"d79c2b96-af24-4422-8ef5-9518fc6ed155"]
}, Open  ]],

Cell["The Entropy of the subsystem 23 is however:", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1541752466,ExpressionUUID->"259014a0-bd07-4247-9f1c-e84000bcf393"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EnTropy", "[", "Subsys23", "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032202377427*^9, 3.614032203253832*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[43]:=",
 CellID->696011360,ExpressionUUID->"164485b6-37ff-40e0-b848-d50804f50944"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610351836*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[43]=",
 CellID->697062508,ExpressionUUID->"5a505f1b-9620-4af7-bae7-ea0326d3e573"]
}, Open  ]],

Cell["\<\
It is simple to do the same game for the other subsystems, which can be \
checked to give the same result as this one.\
\>", "Text",
 TaggingRules->{},
 CellChangeTimes->{3.858887720912751*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->519502311,ExpressionUUID->"3de15fce-e7ca-4142-8ce7-714410eac64d"],

Cell["\<\
Finally we show the Sub Entropy for the single qubit density matrices: \
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellID->1754315922,ExpressionUUID->"da905970-b161-4ff7-bca5-43d093434d63"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Subsys3", "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2"}], "}"}], ",", 
    SubscriptBox["\[Rho]", "GHZ"]}], 
   StyleBox["]",
    FontSize->12]}]}], "\[IndentingNewLine]", 
 RowBox[{"EnTropy", "[", "Subsys3", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032213886055*^9, 3.614032214537125*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[44]:=",
 CellID->1984711391,ExpressionUUID->"6636c4d3-44d9-4d77-b7f7-f0d9e2ceb1b2"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103541696*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[44]=",
 CellID->1006277571,ExpressionUUID->"810fec02-6154-44a4-b30b-580054fd8b0b"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261035541563`*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[45]=",
 CellID->1073415672,ExpressionUUID->"41a6dcc5-d9af-4122-b826-8180203d90ed"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Subsys2", "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "3"}], "}"}], ",", 
    SubscriptBox["\[Rho]", "GHZ"]}], 
   StyleBox["]",
    FontSize->12]}]}], "\[IndentingNewLine]", 
 RowBox[{"EnTropy", "[", "Subsys2", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032233864457*^9, 3.614032241184369*^9}, 
   3.8589688546019487`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[46]:=",
 CellID->180859686,ExpressionUUID->"892fa14c-568c-4f67-8916-885ec52497db"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103570726*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[46]=",
 CellID->2034564504,ExpressionUUID->"b5280a49-784e-49a2-93e0-0cb136d74e8b"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103582872*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[47]=",
 CellID->543507547,ExpressionUUID->"0c73afa4-8417-4be9-872e-b10e0d0c5765"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Subsys1", "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    SubscriptBox["\[Rho]", "GHZ"]}], 
   StyleBox["]",
    FontSize->12]}]}], "\[IndentingNewLine]", 
 RowBox[{"EnTropy", "[", "Subsys1", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032248083827*^9, 3.614032249486787*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.2",
 CellLabel->"In[48]:=",
 CellID->1019382097,ExpressionUUID->"76f24b89-53fa-43d7-98d1-b201a608ff35"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261036047983`*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[48]=",
 CellID->1483391738,ExpressionUUID->"760f1d12-a2f9-4a5c-982b-8050367e7585"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103617523*^9},
 CellTags->"b:5.4.2",
 CellLabel->"Out[49]=",
 CellID->1227282543,ExpressionUUID->"7195585d-466a-4c9d-ac84-e2205237cf70"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["b) Werner state ( Noisy Entangled State Resource ) ",
 FontColor->RGBColor[0, 0, 1]]], "Subsection",
 CellFrame->True,
 TaggingRules->{},
 CellChangeTimes->{{3.856102598337225*^9, 3.856102598720098*^9}, {
  3.856102661102173*^9, 3.85610266986896*^9}, {3.85896895155231*^9, 
  3.858968953054625*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:4", "b:5.4.2"},
 CellLabel->"In[50]:=",
 CellID->426124609,ExpressionUUID->"414c0236-744a-4774-ac1e-3e43612d0c76"],

Cell[BoxData[" "], "Input",
 CellChangeTimes->{{3.858968905292427*^9, 3.8589689200995483`*^9}},
 CellLabel->"In[1]:=",
 CellID->1258694513,ExpressionUUID->"2575b7e7-4e66-46e1-83b4-fdccffb26159"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
    Cell["",ExpressionUUID->"6768f750-6dbc-4096-bb5d-db2ba5ef8d8e"], 
     TraditionalForm]],ExpressionUUID->
    "1b4d3631-d435-4a1e-8833-630c872d1231"]],ExpressionUUID->
    "1e4a07a1-00b1-4abe-86d6-cb7da0e35e0d"], TraditionalForm]],
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "d6ee1639-f2d9-45af-86e1-ebdd2bb0d8f3"],
 StyleBox["Step 1: The Werner state made of qubits 12  is built",
  FontColor->RGBColor[0, 0, 1]]
}], "Subsubsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:4", "b:5.4.3"},
 CellLabel->"In[2]:=",
 CellID->130368513,ExpressionUUID->"53c7bb61-6b78-470f-8814-bcccc172ee80"],

Cell[BoxData[
 RowBox[{"Build", " ", "the", " ", "Two", " ", "Qubit", " ", "Werner", "  ", 
  "density", " ", 
  RowBox[{"matrix", ".", "    ", "The"}], " ", "Werner", " ", "state", " ", 
  "consistes", " ", "of", " ", "a", " ", "Bell", " ", "State", " ", "with", 
  " ", 
  RowBox[{"noise", ".", " ", "\[IndentingNewLine]", "First"}], "  ", "again", 
  " ", "get", " ", "the", " ", "Bell", " ", "density", " ", "matrix", 
  " "}]], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.858887755463788*^9, {3.85896892836579*^9, 3.858968997813013*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->670278051,ExpressionUUID->"3e4c9e2a-34aa-4c03-9ca0-1ebf80df2c75"],

Cell[TextData[{
 StyleBox["The ",
  FontFamily->"Helvetica"],
 StyleBox["Bell states",
  FontFamily->"Helvetica",
  FontWeight->"Bold"],
 StyleBox[" are the following:\n",
  FontFamily->"Helvetica"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "00"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "350a0f54-f233-4229-858d-30f367c48a4b"],
 "( |00\[RightAngleBracket] + |11\[RightAngleBracket] )\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "01"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "808bdf3f-486b-48ab-8ff7-f9c081ab6eac"],
 "( |01\[RightAngleBracket] + |10\[RightAngleBracket] )\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "10"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "c94b9211-96dc-4b22-bc95-a921959408df"],
 "( |00\[RightAngleBracket] - |11\[RightAngleBracket] )\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "11"], "=", " ", 
    RowBox[{"1", "/", 
     SqrtBox["2"]}]}], TraditionalForm]],ExpressionUUID->
  "db26de12-b2a5-413a-b611-dcfaa3afdc2c"],
 "( |01\[RightAngleBracket] - |10\[RightAngleBracket] )"
}], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1811474642,ExpressionUUID->"733da3eb-a9f9-4fcf-98e4-3f623293f0f0"],

Cell[TextData[StyleBox["We regenerate the Bell states here in a manner that \
allows general labels",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.624283907865839*^9, 3.624283940906536*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->578644575,ExpressionUUID->"7506d326-4e33-4f99-af73-1e7d97056dfd"],

Cell[TextData[StyleBox["They can all be constructed in the following way, \
first we build a Bell operator of the form:",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1473418409,ExpressionUUID->"c95cba18-4335-45f4-ac0e-dd1aa47ec390"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Bellop", "=", 
   RowBox[{
    RowBox[{"CNOT", "[", 
     RowBox[{"2", ",", "1", ",", "2"}], "]"}], ".", 
    RowBox[{"had", "[", 
     RowBox[{"2", ",", "1"}], "]"}]}], " "}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[1]:=",
 CellID->1109266506,ExpressionUUID->"62e71298-7a55-4264-844a-b96c43876f42"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]], "0"},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      FractionBox["1", 
       SqrtBox["2"]]},
     {"0", 
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}]},
     {
      FractionBox["1", 
       SqrtBox["2"]], "0", 
      RowBox[{"-", 
       FractionBox["1", 
        SqrtBox["2"]]}], "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103667717*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[1]=",
 CellID->357270766,ExpressionUUID->"2246db1f-bd02-4529-a8d7-97a97767df17"]
}, Open  ]],

Cell[TextData[StyleBox["Then we build the Bell density matrix",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1136626735,ExpressionUUID->"dab60708-b3ab-4253-bc31-d3f745b179cb"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   SubscriptBox["RB", 
    RowBox[{"a_", ",", "b_"}]], ":=", 
   RowBox[{"Bellop", ".", 
    RowBox[{"AF", "[", 
     StyleBox[
      RowBox[{
       SubscriptBox["\[ScriptCapitalP]", 
        StyleBox["a",
         ScriptLevel->0]], "\[CircleTimes]", 
       SubscriptBox["\[ScriptCapitalP]", 
        StyleBox["b",
         ScriptLevel->0]]}],
      ScriptLevel->0], 
     StyleBox["]",
      ScriptLevel->0]}], ".", 
    StyleBox[
     RowBox[{"Adj", "[", "Bellop", "]"}],
     ScriptLevel->0], " "}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140323294519157`*^9, 3.614032341498624*^9}, 
   3.614075965723287*^9, 3.6242841196834803`*^9, 3.856038421429233*^9, 
   3.859055241317966*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[2]:=",
 CellID->1418495080,ExpressionUUID->"4dc9f31b-e280-4e97-bf80-6c70fff92811"],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   RowBox[{
    RowBox[{"Here", "  ", 
     SubscriptBox["RB", 
      RowBox[{"a_", ",", "b_"}]], " ", "denotes", " ", "the", "  ", "Bell", 
     " ", "density", " ", "matrix"}], "-", " ", 
    RowBox[{"not", " ", "the", " ", "Bell", " ", "states"}]}],
   FontFamily->"Helvetica"]],ExpressionUUID->
  "4373e530-5461-4e2e-b5c8-277dd127b187"],
 "."
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{
  3.6242840601391582`*^9, {3.62428412346388*^9, 3.6242841234638968`*^9}, {
   3.858969040609818*^9, 3.858969044017394*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1430844213,ExpressionUUID->"331d6f83-cdda-4eb6-b996-73efe5159ac2"],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox["RB", 
  RowBox[{"0", ",", "0"}]], "\[IndentingNewLine]", 
 SubscriptBox["RB", 
  RowBox[{"0", ",", "1"}]], "\[IndentingNewLine]", 
 SubscriptBox["RB", 
  RowBox[{"1", ",", "0"}]], "\[IndentingNewLine]", 
 SubscriptBox["RB", 
  RowBox[{"1", ",", "1"}]]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.624283983637344*^9, {3.62428412768147*^9, 3.624284137288898*^9}, 
   3.6242842248937902`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[3]:=",
 CellID->676808785,ExpressionUUID->"78495325-4b57-45c4-bd14-ab85a4d10b84"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", 
      FractionBox["1", "2"]},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {
      FractionBox["1", "2"], "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103696759*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[3]=",
 CellID->285487150,ExpressionUUID->"9a0d5cba-a077-46cd-9ff1-5e5929ba9021"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], 
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"], 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103710099*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[4]=",
 CellID->704062652,ExpressionUUID->"2f77fa47-da39-4cdb-a045-8d67cd9433ac"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0", "0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}]},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0", "0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261037223463`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[5]=",
 CellID->546097194,ExpressionUUID->"7786012e-f47f-4fb7-bc0a-b2e6ec3d921a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", 
      FractionBox["1", "2"], 
      RowBox[{"-", 
       FractionBox["1", "2"]}], "0"},
     {"0", 
      RowBox[{"-", 
       FractionBox["1", "2"]}], 
      FractionBox["1", "2"], "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103734254*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[6]=",
 CellID->2057915927,ExpressionUUID->"838c3dde-cfb6-4788-bed0-30e995d60af6"]
}, Open  ]],

Cell[TextData[StyleBox["Then we build the Werner density matrix W which adds \
a noisy density matrix to the Bell State density matrix with a parameter \
\[Lambda] that varies between 0 and 1;  1 being a pure Bell entangled state, \
0 being completely random or noisy.",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.858887793408276*^9, 3.858887794124774*^9}, {
  3.8589690617475357`*^9, 3.858969062309698*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->2051854983,ExpressionUUID->"0326e2b2-044f-409b-9e43-15ec86bbb1b5"],

Cell[BoxData[
 RowBox[{" ", 
  StyleBox[
   RowBox[{
    SubscriptBox["W", 
     RowBox[{"a_", ",", "b_", ",", 
      StyleBox["\[Lambda]_",
       FontFamily->"Helvetica"]}]], ":=", 
    RowBox[{
     RowBox[{
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"], 
      StyleBox["   ",
       FontFamily->"Helvetica"], 
      RowBox[{"Bellop", ".", 
       RowBox[{"AF", "[", 
        StyleBox[
         RowBox[{
          SubscriptBox["\[ScriptCapitalP]", 
           StyleBox["a",
            ScriptLevel->0]], "\[CircleTimes]", 
          SubscriptBox["\[ScriptCapitalP]", 
           StyleBox["b",
            ScriptLevel->0]]}],
         ScriptLevel->0], 
        StyleBox["]",
         ScriptLevel->0]}], ".", 
       StyleBox[
        RowBox[{"Adj", "[", "Bellop", "]"}],
        ScriptLevel->0]}]}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", " ", 
       RowBox[{"1", "-", " ", 
        StyleBox["\[Lambda]",
         FontFamily->"Helvetica"]}], 
       StyleBox[")",
        FontFamily->"Helvetica"]}], 
      StyleBox["  ",
       FontFamily->"Helvetica"], 
      RowBox[{
       RowBox[{"(", " ", 
        RowBox[{"AF", "[", 
         RowBox[{
          SubscriptBox["\[Sigma]", "0"], "\[CircleTimes]", " ", 
          SubscriptBox["\[Sigma]", "0"]}], "]"}], ")"}], "/", "4"}]}]}]}],
   FontSize->24]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.61403238458573*^9, 3.6140324279740753`*^9}, {
  3.614075965740531*^9, 3.6140759657554207`*^9}, {3.856038421431747*^9, 
  3.856038421433638*^9}, {3.859055241320424*^9, 3.859055241322259*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[7]:=",
 CellID->1234295755,ExpressionUUID->"2709db0f-0240-467b-81e1-04bc8f29d496"],

Cell[TextData[StyleBox["Get general Form of Werner density Matrix for 2 \
qubits;  and  now check Trace",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.858969099525629*^9, 3.858969120911974*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->116724265,ExpressionUUID->"ebe28e0d-1698-4d21-a0c1-bda102f17bb0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  SubscriptBox["W", 
   RowBox[{"a", ",", "b", ",", 
    StyleBox["\[Lambda]",
     FontFamily->"Helvetica"]}]], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.624284255764099*^9, 3.6242842566978903`*^9}, {
  3.8589691503466883`*^9, 3.8589691517154837`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[8]:=",
 CellID->1132152144,ExpressionUUID->"f1b0bcf9-449a-4cdf-97c8-df7835258dea"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      "0", "0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}]},
     {"0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "4"]}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}], 
      "0"},
     {"0", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "4"]}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      "0"},
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}], "0", 
      "0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103769937*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[8]=",
 CellID->21366507,ExpressionUUID->"11a505dd-2edf-42aa-a15e-61b2bb6b696c"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"Tr", "[", 
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]], 
     StyleBox["]",
      FontSize->12]}], 
    StyleBox["]",
     FontSize->12]}], 
   StyleBox["\[Equal]",
    FontSize->12], "1"}]}]], "Input",
 CellLabel->"In[9]:=",
 CellID->1632876762,ExpressionUUID->"5d3f3d60-dfeb-4429-9c51-d6988fccbe30"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103789537*^9},
 CellLabel->"Out[9]=",
 CellID->477114100,ExpressionUUID->"e062c410-b2b8-4b47-adeb-d9a2f8437340"]
}, Open  ]],

Cell[TextData[StyleBox["Particular cases of Werner density Matrix for 2 \
qubits with different starting Bell state ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1375971788,ExpressionUUID->"edb1ff14-41fd-4c11-9759-a8d5538c5563"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Simplify", "[", " ", 
  SubscriptBox["W", 
   RowBox[{"0", ",", "0", ",", 
    StyleBox["\[Lambda]",
     FontFamily->"Helvetica"]}]], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", " ", 
  SubscriptBox["W", 
   RowBox[{"0", ",", "1", ",", 
    StyleBox["\[Lambda]",
     FontFamily->"Helvetica"]}]], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", " ", 
  SubscriptBox["W", 
   RowBox[{"1", ",", "0", ",", 
    StyleBox["\[Lambda]",
     FontFamily->"Helvetica"]}]], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", " ", 
  SubscriptBox["W", 
   RowBox[{"1", ",", "1", ",", 
    StyleBox["\[Lambda]",
     FontFamily->"Helvetica"]}]], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[10]:=",
 CellID->1853930020,ExpressionUUID->"26e0210e-ea4e-43f9-aa94-1d2d0a7eae31"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"], "0", "0", 
      FractionBox["\[Lambda]", "2"]},
     {"0", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "0", "0"},
     {"0", "0", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "0"},
     {
      FractionBox["\[Lambda]", "2"], "0", "0", 
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610381357*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[10]=",
 CellID->152740802,ExpressionUUID->"6aa1cd13-b9ac-4aa2-bf7b-216ff9df1e97"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "0", "0", "0"},
     {"0", 
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"], 
      FractionBox["\[Lambda]", "2"], "0"},
     {"0", 
      FractionBox["\[Lambda]", "2"], 
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"], "0"},
     {"0", "0", "0", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261038266373`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[11]=",
 CellID->871701049,ExpressionUUID->"8c7f8ee0-0930-41be-9acf-5f557d0e6327"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"], "0", "0", 
      RowBox[{"-", 
       FractionBox["\[Lambda]", "2"]}]},
     {"0", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "0", "0"},
     {"0", "0", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "0"},
     {
      RowBox[{"-", 
       FractionBox["\[Lambda]", "2"]}], "0", "0", 
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326103839044*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[12]=",
 CellID->546090709,ExpressionUUID->"89f954d2-3d7b-475d-bfe7-954d90500523"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "0", "0", "0"},
     {"0", 
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"], 
      RowBox[{"-", 
       FractionBox["\[Lambda]", "2"]}], "0"},
     {"0", 
      RowBox[{"-", 
       FractionBox["\[Lambda]", "2"]}], 
      FractionBox[
       RowBox[{"1", "+", "\[Lambda]"}], "4"], "0"},
     {"0", "0", "0", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261038508997`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[13]=",
 CellID->1948922532,ExpressionUUID->"f6aa19d9-0d10-43ed-976d-95419374dd23"]
}, Open  ]],

Cell[TextData[StyleBox["Check eigenvalues of Werner density Matrix for 2 \
qubits with different starting Bell state ",
 FontFamily->"Helvetica"]], "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1161611817,ExpressionUUID->"96e7758e-4e27-4f55-8e83-30774e3d9c88"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Eigenvalues", "[", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
    RowBox[{"b", "->", "0"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Eigenvalues", "[", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Eigenvalues", "[", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", " ", 
    RowBox[{"b", "->", "0"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Eigenvalues", "[", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", " ", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[14]:=",
 CellID->435973442,ExpressionUUID->"df10aa0a-fbde-4aec-be20-7ba91ac87bb7"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104181237*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[14]=",
 CellID->2120411280,ExpressionUUID->"9293671e-a1b6-43fe-9ef7-285013f1a3c0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104195774*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[15]=",
 CellID->261260995,ExpressionUUID->"784b0996-2f19-4851-8db3-caef6ca4f321"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032610420785*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[16]=",
 CellID->1556968497,ExpressionUUID->"87dd2014-db60-4d61-bd4e-dbd24ee203f3"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104220249*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[17]=",
 CellID->881290384,ExpressionUUID->"b09f40a2-4de6-4a67-8932-37369727080d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
    Cell["",ExpressionUUID->"6607b50b-be94-428f-9442-e3fb414494f8"], 
     TraditionalForm]],ExpressionUUID->
    "778204e4-2ce0-4d04-9594-3f0d9dc0d242"]],ExpressionUUID->
    "a0ecb7a7-80f4-4557-b2ed-e570e02aa5b1"], TraditionalForm]],
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "b1fee28b-1bdb-4842-be2c-07ad7e2d1cc7"],
 StyleBox["Step 2: Look at the Polarization and Tensor Polarization of the \
Werner state",
  FontColor->RGBColor[0, 0, 1]],
 "  "
}], "Subsubsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:4", "b:5.4.3"},
 CellLabel->"In[18]:=",
 CellID->191436130,ExpressionUUID->"6947aa9b-4ca1-4dd0-8de8-c51dbebff4f5"],

Cell["\<\
Vector and tensor polarizations of Werner state  
Rank 2 Tensor polarizations for Werner state \
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->536533704,ExpressionUUID->"25513e3a-3ad4-4cf8-aad2-984c588fa77e"],

Cell[CellGroupData[{

Cell[BoxData[{
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"T", "[", 
     RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
    RowBox[{"Tr", "[", 
     RowBox[{
      RowBox[{"AF", "[", 
       RowBox[{
        SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
        SubscriptBox["\[Sigma]", "j"]}], "]"}], ".", 
      SubscriptBox["W", 
       RowBox[{"a", ",", "b", ",", 
        StyleBox["\[Lambda]",
         FontFamily->"Helvetica"]}]]}], 
     StyleBox[" ",
      FontSize->12], 
     StyleBox["]",
      FontSize->12]}]}], 
   StyleBox[";",
    FontSize->12]}],
  FontWeight->"Regular"], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"PWA", 
    StyleBox["=",
     FontSize->12], 
    RowBox[{
     StyleBox["{",
      FontSize->12], 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"1", ",", "0"}], "]"}], ",", 
      RowBox[{"T", "[", 
       RowBox[{"2", ",", "0"}], "]"}], ",", 
      RowBox[{"T", "[", 
       RowBox[{"3", ",", "0"}], "]"}]}], "}"}]}], ";"}],
  FontWeight->"Regular"], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"PWB", 
    StyleBox["=",
     FontSize->12], 
    RowBox[{
     StyleBox["{",
      FontSize->12], 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"0", ",", "0"}], "]"}], ",", 
      RowBox[{"T", "[", 
       RowBox[{"0", ",", "2"}], "]"}], ",", 
      RowBox[{"T", "[", 
       RowBox[{"0", ",", "3"}], "]"}]}], "}"}]}], ";"}],
  FontWeight->"Regular"], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    RowBox[{"TWAB", "=", 
     RowBox[{"Table", 
      StyleBox["[",
       FontSize->12], 
      RowBox[{
       RowBox[{"T", "[", 
        RowBox[{"i", ",", "j"}], "]"}], 
       StyleBox[",",
        FontSize->12], 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"j", ",", "1", ",", "3"}], "}"}]}], "]"}]}], ";"}],
   FontWeight->"Regular"], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", "PWA", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", "PWB", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", "TWAB", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032537117626*^9, 3.6140325593447638`*^9}, 
   3.614075965756727*^9, 3.85603842143821*^9, {3.8575867965767612`*^9, 
   3.857586814054616*^9}, {3.857586874944624*^9, 3.8575869217827787`*^9}, {
   3.8575869545623913`*^9, 3.857587002635178*^9}, 3.8575870628486834`*^9, {
   3.8576593682595053`*^9, 3.857659370937337*^9}, 3.859055241325489*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[1]:=",
 CellID->238129375,ExpressionUUID->"84e13ff0-c022-4ed0-ac70-ece8453fc60c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104276485*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[5]=",
 CellID->652770529,ExpressionUUID->"059d97b3-0f09-4150-956d-4506533073f0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"1"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104290666*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[6]=",
 CellID->1767812027,ExpressionUUID->"a2f15c06-716c-4cb5-be38-c86a88126d6d"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", "\[Lambda]"}], "0", "0"},
     {"0", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"a", "+", "b"}]]}], " ", "\[Lambda]"}], "0"},
     {"0", "0", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104302936*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[7]=",
 CellID->1180481544,ExpressionUUID->"3cd2e40f-73a4-4108-84ca-b28b81b118a6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinObs", "[", 
  SubscriptBox["W", 
   RowBox[{"a", ",", "b", ",", 
    StyleBox["\[Lambda]",
     FontFamily->"Helvetica"]}]], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8576730363785563`*^9, 3.857673055213373*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[8]:=",
 CellID->1248054017,ExpressionUUID->"b4c29c3f-9d36-4715-bc0c-4b035c283778"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"\<\"PA=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"PB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", 
          TagBox[GridBox[{
             {"0"},
             {"0"},
             {"0"}
            },
            
            GridBoxAlignment->{
             "Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[0.4]}, 
                Offset[0.2]}}],
           Column], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]},
      {
       RowBox[{"\<\"TAB=\"\>", " ", 
        TagBox[
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "a"], " ", "\[Lambda]"}], "0", "0"},
            {"0", 
             RowBox[{
              RowBox[{"-", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], 
                RowBox[{"a", "+", "b"}]]}], " ", "\[Lambda]"}], "0"},
            {"0", "0", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}
           },
           GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}}], "\[NoBreak]", ")"}],
         Function[BoxForm`e$, 
          MatrixForm[BoxForm`e$]]]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261043280773`*^9},
 CellLabel->"Out[8]=",
 CellID->1899379022,ExpressionUUID->"c8a45c80-bbfb-4b2c-b17a-6390f8434bcd"]
}, Open  ]],

Cell["\<\
Vector polarizations of Werner state are zero 
Rank 2 Tensor polarizations for Werner state are diagonal\
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1478371264,ExpressionUUID->"0b7b1077-c52e-46d0-8691-32716380f856"],

Cell["Vector polarizations of Werner state are zero ", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->600038258,ExpressionUUID->"cfa49947-2cd4-44c7-81d8-d28852dc3ccd"],

Cell["Look at Purity and Entropy", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->259795894,ExpressionUUID->"b44d39ff-9c0c-4689-b209-569ff5d5a465"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Purity", "[", 
   SubscriptBox["W", 
    RowBox[{"a", ",", "b", ",", 
     StyleBox["\[Lambda]",
      FontFamily->"Helvetica"]}]], 
   StyleBox["]",
    FontSize->12]}], 
  StyleBox["]",
   FontSize->12]}], "\[IndentingNewLine]", 
 RowBox[{"FullSimplify", "[", 
  RowBox[{"EnTropy", "[", 
   SubscriptBox["W", 
    RowBox[{"a", ",", "b", ",", 
     StyleBox["\[Lambda]",
      FontFamily->"Helvetica"]}]], 
   StyleBox["]",
    FontSize->12]}], 
  StyleBox["]",
   FontSize->12]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032588998851*^9, 3.6140325897969513`*^9}, {
  3.614032621352804*^9, 3.614032641185923*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[9]:=",
 CellID->555336107,ExpressionUUID->"9431eeb6-cbe4-47c9-ab1d-86cf3444da46"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", " ", "a"}]], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", " ", "b"}]], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"a", "+", "b"}], ")"}]}]]}], ")"}], " ", 
      SuperscriptBox["\[Lambda]", "2"]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326104504098*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[9]=",
 CellID->518810099,ExpressionUUID->"4992e30f-78ea-47f6-9f6e-d07070906ed7"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{"4", " ", 
     RowBox[{"Log", "[", "2", "]"}]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "a"], " ", "\[Lambda]"}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "b"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "1"}], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "a"]}], ")"}], " ", "\[Lambda]"}]}],
          ")"}], " ", 
        RowBox[{"Log", "[", 
         RowBox[{
          FractionBox["1", "4"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "-", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "a"], " ", "\[Lambda]"}], "+", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "b"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"-", "1"}], ")"}], "a"]}], ")"}], " ", 
             "\[Lambda]"}]}], ")"}]}], "]"}]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "a"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "b"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], 
            RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}]}], ")"}], 
      " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "a"], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "b"], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], 
              RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}]}], 
         ")"}]}], "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "a"], "+", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "b"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "1"}], "+", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "a"]}], ")"}]}]}], ")"}], " ", 
         "\[Lambda]"}]}], ")"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "a"], "+", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "b"], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"-", "1"}], "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "a"]}], ")"}]}]}], ")"}], " ", 
           "\[Lambda]"}]}], ")"}]}], "]"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], "a"]}], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "b"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], 
            RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}]}], ")"}], 
      " ", 
      RowBox[{"Log", "[", 
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"-", "1"}], ")"}], "a"], "-", 
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"-", "1"}], ")"}], "b"], " ", 
              RowBox[{"(", 
               RowBox[{"1", "+", 
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{"-", "1"}], ")"}], "a"]}], ")"}]}]}], ")"}], " ", 
           "\[Lambda]"}]}], ")"}]}], "]"}]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115201024*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[10]=",
 CellID->857727073,ExpressionUUID->"6500a847-645a-4f95-a183-c4ced22f4aa2"]
}, Open  ]],

Cell[BoxData[""], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.857587080204084*^9, 3.857587103708267*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellLabel->"In[11]:=",
 CellID->1839307272,ExpressionUUID->"f028965b-cf28-4640-b3c2-65b81fdb7753"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
    Cell["",ExpressionUUID->"a50fbc87-d575-48a0-9a24-faa6ab00a69c"], 
     TraditionalForm]],ExpressionUUID->
    "f8bc7ccd-e173-43f2-9dc7-279fbcb7121f"]],ExpressionUUID->
    "e7ef8fdf-2ff6-4b8a-bed4-cc239bccdf6e"], TraditionalForm]],ExpressionUUID->
  "44aba2e1-e526-4411-b947-bfa158ba2a18"],
 StyleBox["Step 3:  Structure of the Werner Density Matrix",
  FontColor->RGBColor[0, 0, 1]]
}], "Subsubsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:4", "b:5.4.3"},
 CellLabel->"In[12]:=",
 CellID->570020487,ExpressionUUID->"1a8ad92b-cecd-42aa-a3c0-79223df306c8"],

Cell["\<\
Reconstruct Density Matrix 
Note the Werner state has zero polarization for each qubit and a simple \
tensor polarization structure\
\>", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->409775742,ExpressionUUID->"e0391405-ccc3-4cec-951f-aaa3090bd777"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "/", "4"}], ")"}], "  ", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "3"], "   ", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"j", "=", "0"}], "3"], "    ", 
     RowBox[{
      RowBox[{"T", "[", 
       RowBox[{"i", ",", "j"}], "]"}], "*", 
      RowBox[{"AF", "[", 
       RowBox[{
        SubscriptBox["\[Sigma]", "i"], "\[CircleTimes]", " ", 
        SubscriptBox["\[Sigma]", "j"]}], "]"}]}]}]}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Simplify", "[", 
   RowBox[{"%", "-", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]]}], "]"}], "==", 
  RowBox[{"ZeroMatrix", "[", "4", "]"}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.614032670457302*^9, 3.614075965772884*^9, 3.85603842144139*^9, {
   3.8575871825464153`*^9, 3.857587193244636*^9}, 3.859055241328351*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[1]:=",
 CellID->120839865,ExpressionUUID->"c08e96fa-eb3b-4eec-a51a-da09983ba1e5"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      "0", "0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}]},
     {"0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "4"]}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}], 
      "0"},
     {"0", 
      RowBox[{
       RowBox[{"-", 
        FractionBox["1", "4"]}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      "0"},
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "a"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "b"]}], ")"}], " ", "\[Lambda]"}], "0", 
      "0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115314678*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[1]=",
 CellID->1908677247,ExpressionUUID->"0f838e6a-e042-4b7b-856a-d7d887d6dd75"],

Cell[BoxData[
 TagBox["True",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032611532973*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[2]=",
 CellID->27771443,ExpressionUUID->"cb1e0d7a-02ec-45a7-adbc-6cf16c01189d"]
}, Open  ]],

Cell[TextData[{
 " ",
 StyleBox["Werner Density Matrix  has Form\n= (1/4)  [ 1 + \[Lambda] ( ",
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "a"], "  ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["\[Sigma]", "1"], "\[CircleTimes]", 
      SubscriptBox["\[Sigma]", "1"]}], ")"}]}], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "1e67b156-6c7c-47f8-ad44-3b8ce064c0e2"],
 StyleBox["   - ",
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"a", "+", "b", " "}]], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "605b223c-4870-45da-8dc3-0277d95ac358"],
 StyleBox[" ",
  FontWeight->"Plain"],
 Cell[BoxData[
  RowBox[{"  ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["\[Sigma]", "2"], "\[CircleTimes]", 
     SubscriptBox["\[Sigma]", "2"]}], ")"}]}]],
  FontFamily->"Helvetica",
  FontWeight->"Plain",ExpressionUUID->
  "3b92827d-7f11-481b-b90c-2460fd799a9a"],
 StyleBox["  + ",
  FontWeight->"Plain"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "b"], TraditionalForm]],
  FontWeight->"Plain",ExpressionUUID->
  "a6c45351-67c2-4758-aa22-0babd9d7bea8"],
 Cell[BoxData[
  RowBox[{"  ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["\[Sigma]", "3"], "\[CircleTimes]", 
     SubscriptBox["\[Sigma]", "3"]}], ")"}]}]],
  FontFamily->"Helvetica",
  FontWeight->"Plain",ExpressionUUID->
  "f9d8a7ed-2357-4f72-8c8b-b335e508c8cc"],
 StyleBox[" ]",
  FontWeight->"Plain"]
}], "Text",
 TaggingRules->{},
 CellChangeTimes->{{3.859055241331091*^9, 3.8590552413360243`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1583564640,ExpressionUUID->"d3676458-3b45-4bc7-8c58-ce02f3bf2925"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"reconstruct", "=", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "/", "4"}], ")"}], "    ", 
   RowBox[{"(", "  ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"2", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}]}], "]"}], "  ", "+", 
     RowBox[{"\[Lambda]", 
      RowBox[{"(", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "^", "a"}], "  ", 
         RowBox[{"SP", "[", 
          RowBox[{"2", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "1"}], "}"}]}], "]"}]}], "-", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "^", "a"}], "  ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "^", "b"}], "   ", 
         RowBox[{"SP", "[", 
          RowBox[{"2", ",", 
           RowBox[{"{", 
            RowBox[{"2", ",", "2"}], "}"}]}], "]"}]}], "+", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "^", "b"}], " ", 
         RowBox[{"SP", "[", 
          RowBox[{"2", ",", 
           RowBox[{"{", 
            RowBox[{"3", " ", ",", " ", "3"}], "}"}]}], "]"}]}]}], ")"}]}]}], 
    " ", ")"}]}]}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"reconstruct", "-", 
   SubscriptBox["W", 
    RowBox[{"a", ",", "b", ",", "\[Lambda]"}]]}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[3]:=",
 CellID->1206067209,ExpressionUUID->"98f99a8c-dba7-4eed-ac34-06853fcffa2c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      "0", "0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "a"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}]},
     {"0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "a"], "-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}], "0"},
     {"0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "a"], "-", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}], 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}], 
      "0"},
     {
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "a"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"a", "+", "b"}]]}], ")"}], " ", "\[Lambda]"}], "0", "0", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "b"], " ", "\[Lambda]"}]}], ")"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261153473387`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[3]=",
 CellID->750691102,ExpressionUUID->"b56d98af-5fc8-430a-9bf7-157bf5e99d8b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115361154*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[4]=",
 CellID->398925189,ExpressionUUID->"99438918-f05b-4c0c-9875-cfc7cb0ac348"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[Cell[TextData[Cell[BoxData[
    FormBox[
    Cell["",ExpressionUUID->"7b7d03c4-2080-48be-936f-fc09e7fe9e68"], 
     TraditionalForm]],ExpressionUUID->
    "98943ff3-cdf3-48e8-b366-a10badd0ac31"]],ExpressionUUID->
    "c0948deb-fd95-4209-b1f1-4835d6318e5d"], TraditionalForm]],
  FontColor->RGBColor[0, 0, 1],ExpressionUUID->
  "cdeb3416-4838-41ec-95b0-268c37f669b6"],
 StyleBox["Step 3: Look at the Entropy, Purity and Sub-entropy of the Werner \
state  ",
  FontColor->RGBColor[0, 0, 1]]
}], "Subsubsection",
 CellFrame->True,
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->{"c:4", "b:5.4.3"},
 CellLabel->"In[5]:=",
 CellID->273437630,ExpressionUUID->"e67ab82c-2523-4967-9f99-ea6fd626de3b"],

Cell["Check on Purity", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->51242830,ExpressionUUID->"a6a36d51-1715-443f-a01f-593e1213aeb8"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"FullSimplify", "[", 
  RowBox[{"Purity", "[", 
   StyleBox[
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]],
    FontSize->12], "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Purity", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], " ", "/.", "  ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
    RowBox[{"b", "->", "0"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Purity", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], "/.", "  ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Purity", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", " ", 
    RowBox[{"b", "->", "0"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Purity", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", " ", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[1]:=",
 CellID->97301363,ExpressionUUID->"b1a8498e-acc3-4a89-9bf4-91237a84963e"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", " ", "a"}]], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", " ", "b"}]], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"a", "+", "b"}], ")"}]}]]}], ")"}], " ", 
      SuperscriptBox["\[Lambda]", "2"]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115392716*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[1]=",
 CellID->291705589,ExpressionUUID->"fdf20322-e646-4d7f-8fa6-9fd722a73031"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"3", " ", 
      SuperscriptBox["\[Lambda]", "2"]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115406034*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[2]=",
 CellID->2138151869,ExpressionUUID->"bc2897c3-bcb6-401b-8e06-ed3010031d92"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"3", " ", 
      SuperscriptBox["\[Lambda]", "2"]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115417735*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[3]=",
 CellID->2074597032,ExpressionUUID->"2a1d84da-f58f-4610-b135-ce15c846b56c"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"3", " ", 
      SuperscriptBox["\[Lambda]", "2"]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115429282*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[4]=",
 CellID->1207824132,ExpressionUUID->"44cb1c0f-b233-4adb-bbae-74fee5dd4cff"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"3", " ", 
      SuperscriptBox["\[Lambda]", "2"]}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115440927*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[5]=",
 CellID->1884755080,ExpressionUUID->"77314bb1-32f5-4cfe-9785-e8073ccd8fec"]
}, Open  ]],

Cell["Check on Entropy", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1108952574,ExpressionUUID->"cefebd86-28f5-4eab-aa1e-336911a2455c"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"EnTropy", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], " ", "/.", "  ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
    RowBox[{"b", "->", "0"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"EnTropy", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], "/.", "  ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"EnTropy", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", " ", 
    RowBox[{"b", "->", "0"}]}], "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"EnTropy", "[", 
    StyleBox[
     SubscriptBox["W", 
      RowBox[{"a", ",", "b", ",", 
       StyleBox["\[Lambda]",
        FontFamily->"Helvetica"]}]],
     FontSize->12], "]"}], "]"}], " ", "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "1"}], ",", " ", 
    RowBox[{"b", "\[Rule]", "1"}]}], "}"}]}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032710043343*^9, 3.614032723384405*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[6]:=",
 CellID->155338905,ExpressionUUID->"0dd26462-b5d4-49a5-bd3b-3efd653045a4"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], ")"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"3", " ", "\[Lambda]"}]}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}], "]"}]}]}], 
   RowBox[{"4", " ", 
    RowBox[{"Log", "[", "2", "]"}]}]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115489112*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[6]=",
 CellID->1694836310,ExpressionUUID->"25c9f823-d533-4774-8dc2-b9992be0d3ca"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "-", 
       RowBox[{"3", " ", "\[Lambda]"}]}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}], "]"}]}]}], 
   RowBox[{"4", " ", 
    RowBox[{"Log", "[", "2", "]"}]}]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261155173073`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[7]=",
 CellID->890768362,ExpressionUUID->"778386cc-ed4d-4470-89a4-d6d665655bb5"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "-", 
       RowBox[{"3", " ", "\[Lambda]"}]}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}], "]"}]}]}], 
   RowBox[{"4", " ", 
    RowBox[{"Log", "[", "2", "]"}]}]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115534946*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[8]=",
 CellID->1035356897,ExpressionUUID->"24584b6d-4627-4b7e-ab26-58b11e34eb84"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Lambda]"}], ")"}], " ", 
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], ")"}]}], "+", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{"1", "-", "\[Lambda]"}], "4"], "]"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"3", " ", "\[Lambda]"}]}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{"1", "+", 
         RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}], "]"}]}]}], 
   RowBox[{"4", " ", 
    RowBox[{"Log", "[", "2", "]"}]}]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115560165*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[9]=",
 CellID->183376238,ExpressionUUID->"3a0df6d0-c78c-4a9b-9955-616079962c4e"]
}, Open  ]],

Cell["Prepare for Entropy versus \[Lambda] plot", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1588345230,ExpressionUUID->"da296afa-186b-460c-a66e-e6358a9e2da4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Ent", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"EnTropy", "[", 
     StyleBox[
      SubscriptBox["W", 
       RowBox[{"a", ",", "b", ",", 
        StyleBox["\[Lambda]",
         FontFamily->"Helvetica"]}]],
      FontSize->12], "]"}], "]"}], " ", "/.", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", "x"}], ",", 
     RowBox[{"b", "\[Rule]", "y"}], ",", 
     RowBox[{"\[Lambda]", " ", "\[Rule]", "z"}]}], "}"}]}]}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140327367383137`*^9, 3.6140327374686337`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[10]:=",
 CellID->2096548400,ExpressionUUID->"f6a27bca-82b5-4d86-9714-2f3a1d287da8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Ent", "[", 
     RowBox[{"0", ",", "0", ",", "s"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0.0001", ",", "0.99999"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614075215014765*^9, 3.6140752185969553`*^9}, {
   3.804675332401613*^9, 3.80467533443434*^9}, 3.8590530730498343`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[11]:=",
 CellID->777278824,ExpressionUUID->"37e9de03-3d22-4a96-8b16-c9c988b538b5"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwV1Hk41GsbB3CmcE6ppKg3SRROCwqnInxLjkSRLNEitGiRihZZskSRLNUo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        "]]},
      Annotation[#, "Charting`Private`Tag$83065#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0.0001, 0.99999}, {0., 1.9999999783521847`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115724805*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[11]=",
 CellID->365398621,ExpressionUUID->"8741ff0e-3ed6-4189-a4f2-cc9a0f588e22"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Ent", "[", 
     RowBox[{"0", ",", "1", ",", "s"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0.0001", ",", "0.99999"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614075224205271*^9, 3.614075230256526*^9}, {
   3.8046753426045218`*^9, 3.804675343032488*^9}, 3.859053073984269*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[12]:=",
 CellID->582170267,ExpressionUUID->"c2d3bae5-6c6b-48c1-a331-7d853d4ac2b6"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwV1Hk41GsbB3CmcE6ppKg3SRROCwqnInxLjkSRLNEitGiRihZZskSRLNUo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        "]]},
      Annotation[#, "Charting`Private`Tag$83118#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0.0001, 0.99999}, {0., 1.9999999783521847`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261158018312`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[12]=",
 CellID->1598876374,ExpressionUUID->"3e89a63c-8cb4-48da-8aa1-af26d97f52ed"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Ent", "[", 
     RowBox[{"1", ",", "0", ",", "s"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0.0001", ",", "0.99999"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140752434418917`*^9, 3.614075246984686*^9}, {
   3.8046753466036587`*^9, 3.804675347092985*^9}, 3.8590530755686903`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[13]:=",
 CellID->600181737,ExpressionUUID->"ef6cb6ed-a00c-4c27-9cc5-ec26a51b165f"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwV1Hk41GsbB3CmcE6ppKg3SRROCwqnInxLjkSRLNEitGiRihZZskSRLNUo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        "]]},
      Annotation[#, "Charting`Private`Tag$83171#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0.0001, 0.99999}, {0., 1.9999999783521847`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115881324*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[13]=",
 CellID->852342907,ExpressionUUID->"ab5f7a73-f80e-4460-bec2-4ffd77669feb"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Ent", "[", 
     RowBox[{"1", ",", "1", ",", "s"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "0.0001", ",", "0.99999"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614075252733848*^9, 3.614075259561561*^9}, {
   3.804675352064351*^9, 3.804675352572096*^9}, 3.8590530765008097`*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[14]:=",
 CellID->523763800,ExpressionUUID->"0a0181f7-446e-4a60-8f6c-ffb24714b81e"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwV1Hk41GsbB3CmcE6ppKg3SRROCwqnInxLjkSRLNEitGiRihZZskSRLNUo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        "]]},
      Annotation[#, "Charting`Private`Tag$83224#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0.0001, 0.99999}, {0., 1.9999999783521847`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115957464*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[14]=",
 CellID->1611312459,ExpressionUUID->"0f75bffc-8a28-4ac9-a481-a364356055c1"]
}, Open  ]],

Cell["Same plot for all Bell states cases.", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->362717547,ExpressionUUID->"9fd76b76-27ba-4378-899b-dc012d6a4464"],

Cell["Look at Sub-system 2", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1635222243,ExpressionUUID->"341f08e1-f1d6-47d4-9ece-4c66acef8901"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"r2", "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", "1", "}"}], ",", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Tr", "[", 
    RowBox[{"r2", ".", 
     SubscriptBox["\[Sigma]", "1"]}], "]"}], ",", 
   RowBox[{"Tr", "[", 
    RowBox[{"r2", ".", 
     SubscriptBox["\[Sigma]", "2"]}], "]"}], ",", 
   RowBox[{"Tr", "[", 
    RowBox[{"r2", ".", 
     SubscriptBox["\[Sigma]", "3"]}], "]"}]}], "}"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[15]:=",
 CellID->1832007442,ExpressionUUID->"30bc5f39-b9dd-4fa0-a3bd-7d1bb4589e5b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326115987235*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[15]=",
 CellID->1559709743,ExpressionUUID->"7ee00b8f-0656-42b1-b937-d3bc166554c8"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.86032611600029*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[16]=",
 CellID->1856255319,ExpressionUUID->"1ad27cd9-e285-4e1b-b583-2577121083a1"]
}, Open  ]],

Cell["Entropy and Purity of system 2", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->2076030655,ExpressionUUID->"db46e971-d383-4dd1-8ed6-843d10cac042"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"EnTropy", "[", "r2", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", "r2", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.6140327604969683`*^9, 3.614032761137768*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[17]:=",
 CellID->1680405776,ExpressionUUID->"03277937-9bb1-4988-bc4c-4afddb197358"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116025037*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[17]=",
 CellID->1588206464,ExpressionUUID->"79d69b1b-05f0-49b0-85b6-ae2f72cb87e5"],

Cell[BoxData[
 TagBox[
  FractionBox["1", "2"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261160373993`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[18]=",
 CellID->1911791885,ExpressionUUID->"44a01e06-e0f0-428b-9690-013da4988e81"]
}, Open  ]],

Cell["Look at Sub-system 1", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1022435535,ExpressionUUID->"867a4f3e-0dda-4297-b8e0-1ca0f04c975e"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"r1", "=", 
  RowBox[{"PTr", "[", 
   RowBox[{
    RowBox[{"{", "2", "}"}], ",", 
    SubscriptBox["W", 
     RowBox[{"a", ",", "b", ",", 
      StyleBox["\[Lambda]",
       FontFamily->"Helvetica"]}]]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Tr", "[", 
    RowBox[{"r1", ".", 
     SubscriptBox["\[Sigma]", "1"]}], "]"}], ",", 
   RowBox[{"Tr", "[", 
    RowBox[{"r1", ".", 
     SubscriptBox["\[Sigma]", "2"]}], "]"}], ",", 
   RowBox[{"Tr", "[", 
    RowBox[{"r1", ".", 
     SubscriptBox["\[Sigma]", "3"]}], "]"}]}], "}"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.8581729765419493`*^9, 3.858172991459824*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[19]:=",
 CellID->2057377098,ExpressionUUID->"f67fdb61-1dfb-46ae-9a07-840a31d6379e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["1", "2"], "0"},
     {"0", 
      FractionBox["1", "2"]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116054839*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[19]=",
 CellID->835884781,ExpressionUUID->"fd2ef314-c7f2-44b7-bda8-d8b6e2bfcc88"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116067912*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[20]=",
 CellID->2024979240,ExpressionUUID->"ca665d5a-7bf5-4c09-8acc-59f681ed7af9"]
}, Open  ]],

Cell["Entropy and Purity of system 1", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1702685739,ExpressionUUID->"60a190fa-0f41-492a-a4b0-d9b14ff31f86"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"EnTropy", "[", "r1", "]"}], "\[IndentingNewLine]", 
 RowBox[{"Purity", "[", "r1", "]"}]}], "Input",
 TaggingRules->{},
 CellChangeTimes->{{3.614032767302979*^9, 3.6140327678759127`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[21]:=",
 CellID->1819619098,ExpressionUUID->"6d69bb75-b418-4a88-807c-ef64189740d2"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116092943*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[21]=",
 CellID->505249024,ExpressionUUID->"106d5702-30a6-4174-821d-be14be182ea7"],

Cell[BoxData[
 TagBox[
  FractionBox["1", "2"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116105687*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[22]=",
 CellID->570006130,ExpressionUUID->"9c858dd8-9c76-49f5-a131-c4a9b34bd516"]
}, Open  ]],

Cell["Eigenvalues of sub-systems", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->23563159,ExpressionUUID->"837193cc-8743-4550-9e3c-3db6751fb592"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Eigenvalues", "[", "r1", "]"}], "]"}], 
  "  "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"FullSimplify", "[", 
   RowBox[{"Eigenvalues", "[", "r2", "]"}], "]"}], " "}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[23]:=",
 CellID->889022265,ExpressionUUID->"d94cc086-6829-439a-8eb8-cfe17ac3436c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox["1", "2"]},
      {
       FractionBox["1", "2"]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116122497*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[23]=",
 CellID->168477643,ExpressionUUID->"16513776-9138-4a28-8896-c11073c60b07"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox["1", "2"]},
      {
       FractionBox["1", "2"]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116135626*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[24]=",
 CellID->1220299155,ExpressionUUID->"ae96a3f4-e55c-4c22-a04f-e72aeac07731"]
}, Open  ]],

Cell["Double Check on Entropy of the Full Werner case", "Text",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellID->1753475676,ExpressionUUID->"7c7b7e20-0feb-4575-86b1-a2b18eb01871"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"val", "=", 
  RowBox[{
   RowBox[{"FullSimplify", "[", 
    RowBox[{"Eigenvalues", "[", 
     StyleBox[
      SubscriptBox["W", 
       RowBox[{"a", ",", "b", ",", 
        StyleBox["\[Lambda]",
         FontFamily->"Helvetica"]}]],
      FontSize->12], "]"}], "]"}], " ", "/.", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Rule]", "0"}], ",", " ", 
     RowBox[{"b", "->", "0"}]}], "}"}]}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[25]:=",
 CellID->747598751,ExpressionUUID->"17b1bf7f-cbc3-4e01-9628-88d401078e5f"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       FractionBox[
        RowBox[{"1", "-", "\[Lambda]"}], "4"]},
      {
       RowBox[{
        FractionBox["1", "4"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}]}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116161872*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[25]=",
 CellID->1099069235,ExpressionUUID->"f08a5cd2-83ab-427c-b227-0bbbc7fd8f1d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"val", "[", 
  RowBox[{"[", "1", "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"val", "[", 
  RowBox[{"[", "2", "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"val", "[", 
  RowBox[{"[", "3", "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"val", "[", 
  RowBox[{"[", "4", "]"}], "]"}]}], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[26]:=",
 CellID->2112873107,ExpressionUUID->"3cb795ea-91c1-4823-bd2b-d9e192062301"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{"1", "-", "\[Lambda]"}], "4"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261161804733`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[26]=",
 CellID->1549784798,ExpressionUUID->"b53802d1-e421-495a-b1ea-95b2e4a2fdb9"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{"1", "-", "\[Lambda]"}], "4"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116193419*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[27]=",
 CellID->403945523,ExpressionUUID->"1e4fd516-1ab2-4abb-81cd-047e6d4dd910"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{"1", "-", "\[Lambda]"}], "4"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116205106*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[28]=",
 CellID->916596894,ExpressionUUID->"2ffc1540-7700-4c76-bb68-d45039251f7e"],

Cell[BoxData[
 TagBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"3", " ", "\[Lambda]"}]}], ")"}]}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261162176027`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[29]=",
 CellID->1343797527,ExpressionUUID->"5d948752-e5e0-4e09-a783-c7a9383747aa"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"entcheck", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"val", "[", 
        RowBox[{"[", "1", "]"}], "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{"val", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"val", "[", 
        RowBox[{"[", "2", "]"}], "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{"val", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"val", "[", 
        RowBox[{"[", "3", "]"}], "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{"val", "[", 
          RowBox[{"[", "3", "]"}], "]"}]}], "]"}]}], "+", 
      RowBox[{
       RowBox[{"val", "[", 
        RowBox[{"[", "4", "]"}], "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"2", ",", 
         RowBox[{"val", "[", 
          RowBox[{"[", "4", "]"}], "]"}]}], "]"}]}]}], ")"}]}], 
   "]"}]}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[30]:=",
 CellID->268722557,ExpressionUUID->"c13bdabe-e2ad-453f-90b2-641b5341c13c"],

Cell[BoxData[
 TagBox[
  FractionBox[
   RowBox[{
    RowBox[{"Log", "[", "256", "]"}], "+", 
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "\[Lambda]"}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "\[Lambda]"}], "]"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"3", " ", "\[Lambda]"}]}], ")"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", 
       RowBox[{"3", " ", "\[Lambda]"}]}], "]"}]}]}], 
   RowBox[{"Log", "[", "16", "]"}]],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8603261163163443`*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[30]=",
 CellID->119520790,ExpressionUUID->"bfad8327-6fa6-4cc2-ad29-f654e5519f14"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{"entcheck", ",", 
   RowBox[{"{", 
    RowBox[{"\[Lambda]", ",", "0.00001", ",", "1"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "300"}]}], "]"}]], "Input",
 TaggingRules->{},
 CellChangeTimes->{
  3.614075346273509*^9, {3.804675182198617*^9, 3.804675198860366*^9}, 
   3.859053077576233*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[31]:=",
 CellID->261905858,ExpressionUUID->"4922c4a4-3628-4490-8f52-3df4f5f87976"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJwV1nk01VsUB3BDj4f3iDwqMiQNlLlJ+CIpkhD1lESmJoQnIuQaSqTomguF
cCMUDYaXriLSZCh1KZVZ3Z/IkOn3zjtr7XXW55+9zt7nrL2O4mFvGzc+Hh6e
fBL/79RqucMs/l4DP/EfNFkwWjtDpTzWwXfZf5/+72t3fU1jPE3QVm0r97+X
vulnmgZZolqkXmieeGfA7R/hW23RvVTp/CyxsMBgareZPZaOnkmbJn7GVDKE
lSPGLvXoTxHXiZeMWd5yRs7gMf9x4uX2FttD1VwhM6xgOEocnj2YXlzmDqEA
6UwucXd/NLdT5yg47ocuDREbqq8w/uPBcbRcEJLsI8469ThpyxYvTBWuXPuZ
mHYs+Wf9oZNQZzS84xCX8IXKvcj2gfazWam3xKLmMr6zSv9AcquHVSPxv98H
ZZhu/hi9W2POJvZMePBUNf8UCoSHOyqJmzvslhxQOQ3T0D/KWMRnzqyoGz0e
hPl4xZnrxGsVxk5cKA6Gkefu96nEsR6Xax9qhKJSeqNNFLGuyKFj1r5hsLws
/j2I2P+d1IWf9mdhom4qe5JYObhXOy4jHGXv5jfaE0eyGeeq5CLRW+jrvpLY
QfKYw7lHkXh+7GveYmIdd2vNPU5R+OygHiRM/FVIgTN0PRop5amfv83TMLau
UVuqHIMd3l7FRcRLc3L5+57G4Ib2qaQM4tGfsR1l7hdQIdQhcYH4Rup+hllB
LBQfDrxwI+btnmwPVI0HzR0zXUzcofmJZdIcjxRTBoefuDSiPmyh5yVo+/T/
Rs0RX2mStN13Gbq8/8g+IXbamPbvcGoCxGJMbI4SPwrdICEjy0TsGllu1iyN
Bz7jm038mDhhFHKVQVzqWu7s2cQE/95Nr1yJs821yh4FJiHx3O/sVcRnpdSs
XNuTsVRZ6VP+DKnvtnJ8cXwarljfFkyYJv293lPxtjcNA48YC04QazFzumi9
dPhGd0abEisFKa6zGU7HzeCjstO/aCwwXdY8sf0qFtgNxB0gru+SFDbkzQKb
a31FcIq8h9etmkf/zkKAKz3yYZLG/bpE+8SSLBhO2LQXE7fZnrcdVctG5wZt
LRviJ7oCuSZd2fjg99QgaYJGngDf1sHNNyCqGXRnwTgNj8xfZ7XGciE2ceRM
zA8aE3x8j7/K5GHC4sgza+JID2HeJJM8VAoEFC0hztaQCZ9MykOp5ZRewQiN
D2y98OpNNxFuxxNeS9Gw7A8LNw3Nh9uO2uyu7zTWawgw9guxwNJ1SW8aJOdh
irJFtFjImmnkjSa2+SXFV7OfhWC9bj4jYi/2KoZ8EQv3gna1VAzQKLA1Y/RY
3gJDISY/vZ+G7Ok4hhezCIXbnDdY99LgZ0tEMORL0NK0fItvNw3zafYxj+0l
MK6LillOnKjlZ2PhXQL5EcGAlk80FHJaFaUelaBdN1ldk1je2LzbVr0UbZli
i4e6aOgriYQNvCyFr7vIazMOjcDeuGox0TvY+75k1eN2Gtyjlzc6xpVj8edF
tQmNNJL+rgrivVGOARcFR0Vive19Nbn3y+Evhoulz2jErNAzGf5SjvGMnswX
DTRWfOq3Ctxcgc3Kqgl89TT22+JYYl8FQte7Vu9jk/s24F6tN7yPg3sWMUoq
yfxYZMGnPv4QLVai9tIsGi8DYjOchCqR/k1jhFlIY57TpJO4rBIb0vXWSBA7
5pp5jG+rhJ2lVrhIAQ259dubq5IrISQW1/grj0bmXuPkHZuqkM1IqHtynczP
tE0qh4OrEct9mCqZRiNDXtkmifcRTlswU65F01Bzt6+S1mfj9o4bI0X2JF/g
sh1Z354gwm7bcrlVNLS3nEz54dmA0KG/nu8enQfLctmbg8pNkGjmEZ27M4/t
i+xfMG83o6ZQwNjIdx5+iyRMBFRf4Wz8pG/yunnw3klsecZ9jSBljqpl1xw4
d1x6xjRa8FGk4jnfxTn0c5yHVeNbkahW3rlcYw6VNzLVeXvb4HDNJVni1Szk
2cvbq1a8xcJJaZ9d3rO4J9TmHRH2DoG8Bpy7vLP4VNh8XKOuA3mmlZILkmeg
6zvqMKT4AdZz40wvmRk4dOyNNQriYPYZ223RtWksTOGTCa3qhPhaM/cmxWlM
hHCyGqQ/QidSlq2S/AteNTNREw6fEBMk/nKF6C/s21Vs+3lXNzKfS785EjGF
hVZj3S+muxHQLO7a0DcJCbpz2bqrn7G/vz3lseUk1hkfbJfY+QWND/p3BOZM
QLXtjGrDxBcIsUrb436Ow0/CUIub/BVjS/wvae0cR2tP2t0Mkx6c3ZpYnBj3
EyEuKb1VfT0wlFnL97F1DMKaOvdYUb3wPxnguGXxGEoTPXgVtPoQ4jXIM20y
iqxsgznB1j4I+8gUOwb+QL5xkd+p8H6o2D0XmkwYwdTlmvR9KwbwJuDPWKYr
hXVqXk6CTwYQ9+bEL75r3xF8qCJb32cQP18sKWSkD0NCPzuhT2IIbKkrDffz
BsG+rTj3F3sIuX9W1AZ79cPO21hQ0msYBffF66VX9kKrU4AdKfoNct3v3+wZ
/IJVXc6TwdXfsDDetijHvhvuF5y6eh2/Y07ZzWhjJgcx0fPOcvPfof/elWln
/hZPpLSdu1hc+BtsYF/keY17AaOrDhdz8dqasrj56hUKOsq4fSVcHCxMKpTJ
fIW4dI0zI+VctDYmhj3VfQVbObUU/kdchPDXLRP3e4le5TUv1rRy8fVe7g/n
T80QWC+/+dQMF7vwruVSXiOmkj7S03NcqBzTE5fwacTQxLX6MB4K9zUUjwbq
NeLlA1nbmN8oaC/bYD3S8gwpeku9r4pROGIvvzJqvgGrTf7KYytRqIwV+zK/
ux5me0TExSwoXBirLyp+z0ayTUbbR0sKrWFWhn7pbPRYq6betqYgeEDyssAB
Ns5a7ZS33EehNkRw3RXOYzzYFbvu4mEKLUkuu9d+qMXqHSLmIqcplD59+HlB
Wg1Obc/4kxNMYf35BB23RTV4Yqr6hhVK4ffaCDtcrIbTtp1/m0dS2JtdZW8X
VoVU41j3mEsUpgqMlEocHkJQXyRC8CaFjHc5kTVT5bDVyzB9V0BhrnVx6YKj
5bixRVU4/xaFAs/WzeEdd2GguzPBtIxCe5fVcrr8Dk5tjM2Kqqaw8vS201ou
pejTFKnmb6VwqI6rrC6VDx3NjLDWdgoGVifv8LflgaGhujWng0LIsEtZUnwu
5NR3Nhl/pBD402lkeuo69q6N7QgfpFAvaObdrpWGXFWZq9bfSP1j30ISPJIx
qnLrkCJF4fxGlfs28VcQv+Z5X+1PCiOaPndldseic/UB1uVJCiXDi+sDlKKg
snrY02mawsUU70D94yEIXBWsqTFH4Yey/3G3c954ulKEfPMopB1Xt2isv2bw
H1xEWT8=
        "]]},
      Annotation[#, "Charting`Private`Tag$83557#1"]& ]}, {}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImagePadding->All,
   ImageSize->300,
   Method->{
    "DefaultBoundaryStyle" -> Automatic, 
     "DefaultGraphicsInteraction" -> {
      "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
       "Effects" -> {
        "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
         "Droplines" -> {
          "freeformCursorMode" -> True, 
           "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
     "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         (Identity[#]& )[
          Part[#, 1]], 
         (Identity[#]& )[
          Part[#, 2]]}& )}},
   PlotRange->{{0.00001, 1}, {0., 1.9999999997827131`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116377887*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[31]=",
 CellID->888707585,ExpressionUUID->"e4975683-94f7-451b-a862-68c0854edbde"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"Ent", "[", 
     RowBox[{"1", ",", "1", ",", "\[Lambda]"}], "]"}], "-", "entcheck"}], 
   "]"}], "]"}]], "Input",
 TaggingRules->{},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellTags->"b:5.4.3",
 CellLabel->"In[32]:=",
 CellID->1657762703,ExpressionUUID->"157fa8d6-6666-4815-abe4-4537b24f18d9"],

Cell[BoxData[
 TagBox["0.`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.860326116538081*^9},
 CellTags->"b:5.4.3",
 CellLabel->"Out[32]=",
 CellID->759616210,ExpressionUUID->"af262b2c-d126-4a5e-9bf4-c7d31f22ca3b"]
}, Open  ]]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Quit", "[", "]"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.859053688584633*^9, 3.859053690609037*^9}, {
  3.859054327826935*^9, 3.859054334520749*^9}},
 CellLabel->"In[33]:=",
 CellID->528483340,ExpressionUUID->"f6773634-32c5-4b23-a0ec-22e6bce09cee"],

Cell["\<\
This concludes our tour of the QCPITT project.  Hopefully it will  be of some \
help to you.  Any suggestions, requests and criticisms, etc  would be \
gratefully received by emailing tabakin@pitt.edu.\
\>", "Abstract",
 TaggingRules->{},
 CellChangeTimes->{
  3.857571350492466*^9, {3.858087500537691*^9, 3.858087507343938*^9}, {
   3.8581736657175007`*^9, 3.858173729358176*^9}, {3.858848466787771*^9, 
   3.858848536000142*^9}, {3.858848581247189*^9, 3.858848594827466*^9}, {
   3.858848625843875*^9, 3.858848649942279*^9}, {3.858848706349235*^9, 
   3.85884879162628*^9}, {3.858969943264921*^9, 3.858970309549512*^9}, {
   3.858971697562799*^9, 3.85897178842924*^9}, {3.8589719701544943`*^9, 
   3.858972175919855*^9}, {3.858972257392571*^9, 3.8589724092141647`*^9}, {
   3.85897244373011*^9, 3.858972522436461*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 FontSlant->"Plain",
 CellID->1524764736,ExpressionUUID->"353d10cc-4a82-43f8-b397-b7e93b5cf08e"],

Cell[TextData[StyleBox["Conclusion",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}, {3.8589681608234797`*^9, 
   3.858968166943308*^9}, {3.8589698348131437`*^9, 3.858969837940957*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1275267708,ExpressionUUID->"b1943aa5-2ef2-4e61-86db-560a54a28f50"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{3.856098983323403*^9, 3.856099094688118*^9, 
  3.856101310413926*^9, 3.857569952534441*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->802582215,ExpressionUUID->"00c78d83-2b0c-4e94-8651-abe28f170866"],

Cell["\<\
An introduction to the package QCPITT has been presented, followed by samples \
of its capabilities and facilities. It taps into the basic features of \
Mathematica and hopefully provides a viable and practical tool for learning, \
handling, enjoying and understanding quantum computing.  In addition to the \
selected examples presented here there is much more available and much more \
is possible.  Publications of earlier versions,  along with tutorials and \
applications, are available.   Some of the methods used here were adapted \
from another related supercomputer version called QCMPI written in Fortran. \
Links to these resources are provided below.   Recent developments include: \
(1) application to qutrits and qudits; (2) extension to closed and open \
system dynamics using a Lindblad/Beretta formulation, (3) studies of one and \
two qubit gate dynamics. and (4)  multi-dimensional  tunnelling (in \
preparation).  Future goals are: (1) to  link to  the qiskit/Jupyter setup to \
allow direct submission of QC jobs.  (2) To provide a drag and drop method to \
assemble  executable circuits.  (3)  Error correction studies (4) examine \
tunneling, scattering, and bound states in one to three dimensions.\
\>", "Abstract",
 TaggingRules->{},
 CellChangeTimes->{
  3.857571350492466*^9, {3.858087500537691*^9, 3.858087507343938*^9}, {
   3.8581736657175007`*^9, 3.858173729358176*^9}, {3.858848466787771*^9, 
   3.858848536000142*^9}, {3.858848581247189*^9, 3.858848594827466*^9}, {
   3.858848625843875*^9, 3.858848649942279*^9}, {3.858848706349235*^9, 
   3.85884879162628*^9}, {3.858969943264921*^9, 3.858970309549512*^9}, {
   3.858971697562799*^9, 3.85897178842924*^9}, {3.8589719701544943`*^9, 
   3.858972175919855*^9}, {3.859040452711974*^9, 3.8590404730525703`*^9}, {
   3.859054368507749*^9, 3.859054377338747*^9}, {3.859054409514886*^9, 
   3.859054526583568*^9}, {3.859054567613019*^9, 3.859054567736721*^9}, {
   3.859054623301257*^9, 3.8590546440012827`*^9}, {3.859056560163616*^9, 
   3.8590565906977177`*^9}, {3.8590566215361433`*^9, 
   3.8590566511335497`*^9}, {3.8592334436964827`*^9, 3.859233444725706*^9}, {
   3.859233484307404*^9, 3.859233489302737*^9}, {3.859233524121583*^9, 
   3.859233586684321*^9}, 3.859233789841176*^9, {3.859234332018201*^9, 
   3.859234409307373*^9}, 3.8603265237935*^9},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 FontSlant->"Plain",
 CellID->1694717084,ExpressionUUID->"f515fd47-13e4-4c5b-a3bc-ebf91bb5d762"]
}, Open  ]],

Cell[TextData[StyleBox["Resource Links",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}, {3.8589681608234797`*^9, 
   3.858968166943308*^9}, {3.8589698348131437`*^9, 3.858969837940957*^9}, {
   3.858971817196211*^9, 3.858971827568594*^9}, {3.859230987928824*^9, 
   3.859231001530188*^9}, {3.859233419721457*^9, 3.859233420812913*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->224028014,ExpressionUUID->"a6fcca51-d1a0-41b5-ac8e-03dcd2c9b47d"],

Cell["Publications", "Subtitle",
 CellChangeTimes->{{3.859285540906773*^9, 3.859285549539281*^9}},
 CellID->1670161643,ExpressionUUID->"9a660cc0-d636-4f83-b3cf-9c42710a9fd8"],

Cell[TextData[{
 StyleBox["QDENSITY\[LongDash]A Mathematica Quantum Computer simulation. \
Comput.Phys.Commun.174 June 2006,914. Bruno Juli\[AAcute]-D\[IAcute]az, \
Joseph M. Burdis and Frank Tabakin.  ",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox[ButtonBox["link",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://www.sciencedirect.com/science/article/abs/pii/\
S0010465506000543?via%3Dihub"], None},
  ButtonNote->
   "https://www.sciencedirect.com/science/article/abs/pii/S0010465506000543?\
via%3Dihub"],
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0]],
 StyleBox["\n\nQDENSITY-A Mathematica quantum computer \
simulation.Comput.Phys.Commun.180(3):474 (2009)\nBruno Juli\[AAcute]-D\
\[IAcute]az  ,Joseph M.Burdis,Frank Tabakin.   ",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox[ButtonBox["link",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://www.sciencedirect.com/science/article/abs/pii/\
S0010465506000543?via%3Dihub"], None},
  ButtonNote->
   "https://www.sciencedirect.com/science/article/abs/pii/S0010465506000543?\
via%3Dihub"],
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0]],
 StyleBox["\n\nQCMPI:A parallel environment for quantum computing. \
Comput.Phys.Commun.180(6):948-964 (2009)\nFrank Tabakin, Bruno \
Juli\[AAcute]-D\[IAcute]az.  ",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox[ButtonBox["link",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://www.sciencedirect.com/science/article/abs/pii/\
S0010465508004141"], None},
  ButtonNote->
   "https://www.sciencedirect.com/science/article/abs/pii/S0010465508004141"],
  
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0]],
 StyleBox["\n\n QCWAVE-A Mathematica quantum computer simulation update. \
Comput.Phys.Commun.182(8):1693-1707 (2011)\n Frank Tabakin  and Bruno Juli\
\[AAcute]-D\[IAcute]az.   ",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox[ButtonBox["link",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://www.sciencedirect.com/science/article/abs/pii/\
S0010465511001287"], None},
  ButtonNote->
   "https://www.sciencedirect.com/science/article/abs/pii/S0010465511001287"],
  
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0]],
 StyleBox["\n \n QDENSITY/QCWAVE:A Mathematica quantum computer simulation \
update. Comput.Phys.Commun.201:171-172(2016)\nFrank Tabakin. \n\nModel \
dynamics for quantum computing. Annals of Physics Volume 383, August 2017, \
Pages 33-78  ",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 StyleBox[ButtonBox["link",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL[" \
https://www.sciencedirect.com/science/article/abs/pii/S0003491617301252"], 
    None},
  ButtonNote->
   " https://www.sciencedirect.com/science/article/abs/pii/S0003491617301252"],
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0]],
 StyleBox["\n",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]]
}], "Text",
 CellChangeTimes->{{3.859224978228475*^9, 3.859225016206675*^9}, {
   3.859225096082008*^9, 3.85922511537143*^9}, {3.859225151543416*^9, 
   3.85922524415777*^9}, {3.8592252816400757`*^9, 3.859225307166963*^9}, {
   3.8592253476450663`*^9, 3.8592253519424963`*^9}, {3.859226675773314*^9, 
   3.85922667933645*^9}, {3.859230666389608*^9, 3.859230716622995*^9}, 
   3.85923096633217*^9, {3.85928634069944*^9, 3.859286362412051*^9}, {
   3.859286439446884*^9, 3.8592864394489098`*^9}, {3.859286533334865*^9, 
   3.859286533336768*^9}, {3.859286585614615*^9, 3.8592866130429707`*^9}, {
   3.8592866466289377`*^9, 3.859286675313251*^9}, {3.859286723095385*^9, 
   3.859286736078869*^9}, {3.859286932597859*^9, 3.8592869325999117`*^9}, {
   3.859286995051553*^9, 3.859287020686048*^9}, {3.859287112955946*^9, 
   3.859287142706875*^9}, {3.859287207946122*^9, 3.859287207948463*^9}, {
   3.8592872724160013`*^9, 3.859287318873148*^9}, {3.859289812759078*^9, 
   3.859289815301774*^9}},
 CellID->320828263,ExpressionUUID->"b7b99083-bfb3-4041-89c9-cf75413513ee"],

Cell[TextData[{
 "Package Download  ",
 Cell[BoxData[
  StyleBox[
   RowBox[{
    ButtonBox["QCPITT",
     BaseStyle->"Hyperlink",
     ButtonData->{
       URL["http://www.pitt.edu/~tabakin/QDENSITY/QCPITT.zip"], None},
     ButtonNote->"http://www.pitt.edu/~tabakin/QDENSITY/QCPITT.zip"], "  "}],
   FontSize->18]], "Input",
  TaggingRules->{},
  CellChangeTimes->{{3.858174055576445*^9, 3.858174055578929*^9}},
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[1, 0.5, 0],ExpressionUUID->
  "80e32394-420d-4347-8780-c75bbde10d53"]
}], "Subtitle",
 CellChangeTimes->{{3.859285540906773*^9, 3.859285586690551*^9}, {
  3.859286154508703*^9, 3.859286155785933*^9}},
 CellID->809174117,ExpressionUUID->"35700604-5db4-4071-8960-66417a5ddc7e"],

Cell[TextData[{
 "The list of tutorials is:\n",
 Cell[BoxData[
  TagBox[
   TagBox[GridBox[{
      {"BTGates", "BTtutorial", "Bell_Correlations", "Belltheorem"},
      {"CircuitTutorial", "CircuitTutorialBT", "Cluster", 
       "ConcurrenceTutorial"},
      {"ConcurrenceTutorialBT", "DensityMatrixTutorial", 
       RowBox[{"Discord", "-", "Tests"}], "DiscordTutorial"},
      {"Entanglement", "EntropyTutorial", "FunctionsTour", "GatesQudits"},
      {"Grover", "HybridGates", "Measurement", "PartialTransposeTutorial"},
      {"QCwaveTutorial", "QFT", 
       RowBox[{"EntropyTutorial", " ", "Quantum"}], "Qutrit_operators"},
      {"RandomObs", "Rotations", "SchmidtTutorialQutrits", 
       "SchmidtTutorial"},
      {"Shor", "Teleportation", "TeleportationQutrit", "TernaryTutorial"},
      {"Tutorial", "WernerStates", "WorkBookPartnerBT", "XStateTutorial"}
     },
     AutoDelete->False,
     GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
     GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
    "Grid"],
   Function[BoxForm`e$, 
    MatrixForm[BoxForm`e$]]]], "Output",
  CellChangeTimes->{3.859201965141721*^9, 3.8592019965499687`*^9, 
   3.859233175877164*^9},ExpressionUUID->
  "4a4995c5-e26c-4028-9cae-27167eece9fd"]
}], "Text",
 CellChangeTimes->{
  3.859285645255308*^9, {3.8592860944896603`*^9, 3.859286095174748*^9}},
 CellID->863742561,ExpressionUUID->"82238a1a-a31e-4fd1-b73d-46139cc45cf7"],

Cell[TextData[{
 "Tutorials Download  ",
 ButtonBox["Tutorials",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.pitt.edu/~tabakin/QDENSITY/Tutorials.zip"], None},
  ButtonNote->"http://www.pitt.edu/~tabakin/QDENSITY/Tutorials.zip"]
}], "Subtitle",
 CellChangeTimes->{{3.859285540906773*^9, 3.859285586690551*^9}, {
  3.859285831775302*^9, 3.859285836489376*^9}, {3.859286106437427*^9, 
  3.859286116776136*^9}, {3.859286252589241*^9, 3.8592862828139687`*^9}},
 CellID->548490404,ExpressionUUID->"d8da1847-7d4e-4dd5-9c31-606156ecb3f8"],

Cell[TextData[StyleBox["References",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}, {3.8589681608234797`*^9, 
   3.858968166943308*^9}, {3.8589698348131437`*^9, 3.858969837940957*^9}, {
   3.858971831922495*^9, 3.8589718347909937`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->952766083,ExpressionUUID->"e695390e-046f-49f1-871a-005a68ccb899"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.856098983323403*^9, 3.856099094688118*^9, 3.856101310413926*^9, 
   3.857569952534441*^9, {3.858971919865089*^9, 3.858971961816527*^9}, 
   3.859129372488193*^9, {3.859232460609068*^9, 3.85923247061012*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->1501484949,ExpressionUUID->"3b433320-8a06-4fab-b769-2d5df6f394a8"],

Cell["\<\
 Michael A. Nielsen and Isaac I. Chuang,
``Quantum Computation and Quantum Information \[CloseCurlyQuote]\
\[CloseCurlyQuote], Cambridge University Press (2000).

Teleportation C. H. Bennett, G. Brassard, C. Crepeau, R. Jozsa, A. Peres, and \
W. K. Wootters,
Phys. Rev. Lett. 70, 1895-1899 (1993).

 L. K. Grover, Phys. Rev. Lett. 79, 325-328 (1997).
 
 Peter W. Shor, SIAM J. COMPUT 26 (5) 1484 (1997).
 
 J. S. Bell,  Rev. Mod. Phys. 38 447 (1966).
 
 D. M.  Greenberger , M. A. Horne, and A. Zeilinger, in
Bell\[CloseCurlyQuote]s Theorem, Quantum Theory, and Conceptions of the \
Universe,
edited by M. Kafatos (Kluwer Academics, Dordrecht, The Netherlands, 1989), \
pp. 73.

R. F. Werner,Phys. Rev. A 40, 4277 (1989).

 G. Lindblad, Commun. Math. Phys. 4 ,119 (1976).
 
 R. Raussendorf and H. J. Briegel, Phys. Rev. Lett. 86 (2001) 5188;
R. Raussendorf, D. Brownie and H.J. Briegel, Phys. Rev. A 68, 022312 (2003);
 Michael I. Nielsen, quant - ph/0504097

John von Neumann, ``Mathematical Foundations of Quantum Mechanics\
\[CloseCurlyQuote]\[CloseCurlyQuote],
Princeton University Press (1955).
\
\>", "Abstract",
 TaggingRules->{},
 CellChangeTimes->{
  3.857571350492466*^9, {3.858087500537691*^9, 3.858087507343938*^9}, {
   3.8581736657175007`*^9, 3.858173729358176*^9}, {3.858848466787771*^9, 
   3.858848536000142*^9}, {3.858848581247189*^9, 3.858848594827466*^9}, {
   3.858848625843875*^9, 3.858848649942279*^9}, {3.858848706349235*^9, 
   3.85884879162628*^9}, {3.858969943264921*^9, 3.858970309549512*^9}, {
   3.858971697562799*^9, 3.85897178842924*^9}, {3.8589719701544943`*^9, 
   3.858972175919855*^9}, {3.859040452711974*^9, 3.8590404730525703`*^9}, {
   3.859054368507749*^9, 3.859054377338747*^9}, {3.859054409514886*^9, 
   3.859054526583568*^9}, {3.859054567613019*^9, 3.859054567736721*^9}, {
   3.859054623301257*^9, 3.8590546440012827`*^9}, {3.859056560163616*^9, 
   3.8590565906977177`*^9}, {3.8590566215361433`*^9, 
   3.8590566511335497`*^9}, {3.859128492588955*^9, 3.859128780211577*^9}, {
   3.859128852624497*^9, 3.8591288906771812`*^9}, {3.859128972061213*^9, 
   3.859128988058936*^9}, {3.859129239950247*^9, 3.859129338312274*^9}, {
   3.859202315499914*^9, 3.8592023217826242`*^9}, {3.8592311706542387`*^9, 
   3.859231341758322*^9}, {3.859231423435604*^9, 3.859231454675446*^9}, {
   3.859232474065956*^9, 3.859232478260407*^9}, {3.859285685028904*^9, 
   3.8592857199399443`*^9}, {3.860326362224753*^9, 3.860326389213629*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 FontSlant->"Plain",
 CellID->257579753,ExpressionUUID->"f90e3e77-6240-4375-bf66-1c55eb753dc8"],

Cell[TextData[StyleBox["Acknowledgements",
 FontSize->36,
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.8575699422557487`*^9, {3.858967512893215*^9, 3.858967538613242*^9}, 
   3.858967593514532*^9, 3.85896764844897*^9, 3.858967704891561*^9, {
   3.858968037454359*^9, 3.85896805021144*^9}, {3.8589681608234797`*^9, 
   3.858968166943308*^9}, {3.8589698348131437`*^9, 3.858969837940957*^9}, {
   3.858971831922495*^9, 3.85897185333116*^9}, {3.858972184885816*^9, 
   3.8589722095811234`*^9}, {3.859128433795643*^9, 3.859128434629549*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2026474339,ExpressionUUID->"2b2a631f-3117-4a6c-82be-10fb33ebf0a5"],

Cell[TextData[StyleBox["",
 FontColor->RGBColor[0, 0, 1]]], "Section",
 TaggingRules->{},
 CellChangeTimes->{
  3.856098983323403*^9, 3.856099094688118*^9, 3.856101310413926*^9, 
   3.857569952534441*^9, {3.8591284547544727`*^9, 3.85912845877225*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 CellID->2011057800,ExpressionUUID->"27fdc772-b883-4e32-bd3e-f8c7e567b5c5"],

Cell[TextData[{
 "The  QCPITT project originally included Prof. ",
 StyleBox["Bruno Juli\[AAcute]-D\[IAcute]az",
  FontFamily->"Times New Roman",
  FontSize->18,
  FontWeight->"Regular",
  FontColor->RGBColor[0., 0., 0.]],
 ", (Universitat de Barcelona, Barcelona (Spain)), and  Dr. Joseph M. Burdis, \
to whom I am very grateful.  Questions provided by \
Dr.\[NonBreakingSpace]Kapil K. Sharma are also appreciated; he stimulated the \
extensions to hybrid systems, partial transposition, and to quantum discord. \
The author\[CloseCurlyQuote]s interest in QC was enhanced by communications \
with Dr.\[NonBreakingSpace]Victor Volkov. Earlier this project was supported \
by the National Science Foundation."
}], "Abstract",
 TaggingRules->{},
 CellChangeTimes->{
  3.857571350492466*^9, {3.858087500537691*^9, 3.858087507343938*^9}, {
   3.8581736657175007`*^9, 3.858173729358176*^9}, {3.858848466787771*^9, 
   3.858848536000142*^9}, {3.858848581247189*^9, 3.858848594827466*^9}, {
   3.858848625843875*^9, 3.858848649942279*^9}, {3.858848706349235*^9, 
   3.85884879162628*^9}, {3.858969943264921*^9, 3.858970309549512*^9}, {
   3.858971697562799*^9, 3.85897178842924*^9}, {3.8589719701544943`*^9, 
   3.858972175919855*^9}, {3.859040452711974*^9, 3.8590404730525703`*^9}, {
   3.859054368507749*^9, 3.859054377338747*^9}, {3.859054409514886*^9, 
   3.859054526583568*^9}, {3.859054567613019*^9, 3.859054567736721*^9}, {
   3.859054623301257*^9, 3.8590546440012827`*^9}, {3.859056560163616*^9, 
   3.8590565906977177`*^9}, {3.8590566215361433`*^9, 
   3.8590566511335497`*^9}, {3.859128492588955*^9, 3.859128780211577*^9}, {
   3.859128852624497*^9, 3.8591288906771812`*^9}, {3.859128972061213*^9, 
   3.859128988058936*^9}, {3.859129446266954*^9, 3.859129447062537*^9}, {
   3.859231492716095*^9, 3.859231577381282*^9}, {3.8592322548551064`*^9, 
   3.859232360147357*^9}, 3.859285758334901*^9, {3.8592858682937927`*^9, 
   3.859285880398858*^9}, {3.859285922442626*^9, 3.85928601365736*^9}, {
   3.859289536701947*^9, 3.85928957463962*^9}, {3.859289635017166*^9, 
   3.8592896355360518`*^9}},
 FontFamily->"Times New Roman",
 FontSize->18,
 FontWeight->"Regular",
 FontSlant->"Plain",
 CellID->1757444899,ExpressionUUID->"846406fa-1bc6-46b3-8c6e-62ec672045e1"]
},
CellGrouping->Manual,
WindowSize->{1074, 539},
WindowMargins->{{43, Automatic}, {Automatic, 30}},
DockedCells->{
  Cell[
   BoxData[
    TemplateBox[{}, "MainGridTemplate"]], "DockedCell", 
   CellMargins -> {{-10, -10}, {-8, -8}}, CellFrame -> 0, Background -> 
   RGBColor[0.7185389999999999, 0.5009625, 0.266231], 
   CellTags -> {"MainDockedCell"}, CacheGraphics -> False], 
  Cell[
   BoxData[
    TemplateBox[{}, "ToolsGridTemplate"]], "DockedCell", 
   TaggingRules -> {"Tools" -> True}, CellTags -> {"ToolbarDockedCell"}, 
   CellFrameMargins -> {{39, 5}, {2, 2}}, CellFrame -> {{0, 0}, {1, 0}}, 
   CacheGraphics -> False, CellOpen -> Dynamic[
     CurrentValue[
      EvaluationNotebook[], {TaggingRules, "ToolsOpen"}, True]]]},
TaggingRules->{
 "ResourceType" -> "ComputationalEssay", "ResourceCreateNotebook" -> True, 
  "TemplateVersion" -> Inherited, "CreationTimestamp" -> 
  3.8338901164240208`17.3362146505151*^9, "UpdatedTimestamp" -> 
  3.8338901164245208`17.336214650515153*^9, "CompatibilityTest" -> 
  HoldComplete[
    BinaryDeserialize[
     ByteArray[{56, 67, 58, 120, 156, 133, 143, 205, 78, 131, 64, 20, 133, 
      163, 173, 110, 116, 213, 165, 79, 224, 6, 70, 11, 105, 138, 59, 197, 
      198, 184, 193, 70, 92, 184, 100, 24, 238, 164, 83, 7, 46, 153, 123, 41, 
      229, 237, 29, 140, 186, 193, 196, 245, 249, 249, 206, 209, 115, 90, 164,
       88, 183, 216, 53, 213, 230, 216, 58, 32, 50, 216, 232, 19, 58, 203, 0, 
      42, 202, 175, 95, 129, 176, 115, 10, 242, 129, 24, 234, 212, 26, 104, 
      184, 120, 4, 109, 26, 195, 222, 153, 33, 67, 137, 248, 81, 232, 83, 154,
       229, 192, 62, 25, 252, 25, 217, 58, 115, 144, 12, 133, 69, 89, 253, 24,
       222, 134, 22, 124, 238, 242, 222, 50, 184, 70, 178, 57, 0, 229, 87, 
      227, 156, 142, 229, 216, 46, 237, 134, 72, 14, 191, 133, 139, 169, 70, 
      243, 172, 179, 214, 99, 207, 159, 253, 11, 55, 14, 184, 72, 45, 118, 
      213, 75, 185, 7, 197, 249, 251, 142, 185, 165, 59, 33, 250, 190, 15, 
      123, 180, 218, 201, 90, 141, 122, 168, 176, 22, 88, 238, 133, 251, 110, 
      167, 175, 185, 98, 138, 72, 177, 97, 127, 65, 248, 215, 217, 195, 86, 
      178, 218, 133, 253, 81, 123, 208, 236, 9, 38, 188, 205, 191, 188, 106, 
      149, 68, 55, 75, 121, 27, 64, 164, 203, 32, 6, 41, 131, 36, 134, 117, 
      160, 228, 58, 74, 226, 74, 173, 202, 104, 249, 9, 89, 192, 149, 140}]]],
   "ToolsOpen" -> True, "StatusMessage" -> "", "TryRealOnly" -> False, 
  "AttachedHints" -> {}},
CreateCellID->True,
CellContext->Notebook,
FrontEndVersion->"13.0 for Mac OS X x86 (64-bit) (December 2, 2021)",
StyleDefinitions->Notebook[{
   Cell[
    CellGroupData[{
      Cell[
       StyleData[StyleDefinitions -> "Default.nb"]], 
      Cell[
       StyleData[All, "Working"], DockedCells -> {
         Cell[
          BoxData[
           TemplateBox[{}, "MainGridTemplate"]], "DockedCell", 
          CellMargins -> {{-10, -10}, {-8, -8}}, CellFrame -> 0, Background -> 
          RGBColor[0.7185389999999999, 0.5009625, 0.266231], 
          CellTags -> {"MainDockedCell"}, CacheGraphics -> False], 
         Cell[
          BoxData[
           TemplateBox[{}, "ToolsGridTemplate"]], "DockedCell", 
          TaggingRules -> {"Tools" -> True}, 
          CellTags -> {"ToolbarDockedCell"}, 
          CellFrameMargins -> {{39, 5}, {2, 2}}, 
          CellFrame -> {{0, 0}, {1, 0}}, CacheGraphics -> False, CellOpen -> 
          Dynamic[
            CurrentValue[
             EvaluationNotebook[], {TaggingRules, "ToolsOpen"}, True]]]}, 
       AutoQuoteCharacters -> {}]}, Open]], 
   Cell[
    StyleData["Item"], DefaultNewCellStyle -> "Item"], 
   Cell[
    StyleData["MoreInfoText", StyleDefinitions -> StyleData["Text"]], 
    FontColor -> GrayLevel[0.25]], 
   Cell[
    StyleData["ErrorText", StyleDefinitions -> StyleData["Text"]], 
    ShowCellBracket -> False, CellMargins -> {{66, Inherited}, {10, 10}}, 
    CellElementSpacings -> {"CellMinHeight" -> 0, "ClosedCellHeight" -> 0}, 
    FontWeight -> Bold, FontColor -> RGBColor[1, 0, 0]], 
   Cell[
    StyleData["WarningText", StyleDefinitions -> StyleData["Text"]], 
    ShowCellBracket -> False, CellMargins -> {{66, 35}, {0, 0}}, FontSize -> 
    14, GridBoxOptions -> {BaseStyle -> {}}], 
   Cell[
    StyleData["ButtonText"], FontFamily -> "Sans Serif", FontSize -> 11, 
    FontWeight -> Bold, FontColor -> RGBColor[0.458824, 0.458824, 0.458824]], 
   
   Cell[
    StyleData["MoreInfoOpenerIconTemplate"], 
    TemplateBoxOptions -> {
     DisplayFunction -> (
       PaneSelectorBox[{False -> "$Failed", True -> "$Failed"}, 
        Dynamic[
         CurrentValue["MouseOver"]], ImageSize -> Automatic, FrameMargins -> 
        0]& )}], 
   Cell[
    StyleData["MoreInfoOpenerButtonTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (AdjustmentBox[
        ButtonBox[
         TemplateBox[{
           TemplateBox[{}, "MoreInfoOpenerIconTemplate"], 
           "\"Click for more information\""}, "PrettyTooltipTemplate"], 
         ButtonFunction :> (NotebookDelete[
            CurrentValue[
             ParentCell[
              EvaluationCell[]], {TaggingRules, "AttachedMoreInfoCell"}]]; 
          If[
            And[
             MatchQ[
              CurrentValue[
               ParentCell[
                EvaluationCell[]], {TaggingRules, "AttachedMoreInfoCell"}], 
              Blank[CellObject]], CurrentValue[
               ParentCell[
                EvaluationCell[]], {
               TaggingRules, "AttachedMoreInfoTag"}] === #], CurrentValue[
               ParentCell[
                EvaluationCell[]], {TaggingRules, "AttachedMoreInfoCell"}] = 
             Inherited; CurrentValue[
               ParentCell[
                EvaluationCell[]], {TaggingRules, "AttachedMoreInfoTag"}] = 
             Inherited; Null, CurrentValue[
               ParentCell[
                EvaluationCell[]], {TaggingRules, "AttachedMoreInfoTag"}] = #; 
            CurrentValue[
               ParentCell[
                EvaluationCell[]], {TaggingRules, "AttachedMoreInfoCell"}] = 
             MathLink`CallFrontEnd[
               FrontEnd`AttachCell[
                ParentCell[
                 EvaluationCell[]], #2, "Inline", 
                "ClosingActions" -> {"ParentChanged", "EvaluatorQuit"}]]]), 
         Appearance -> None, Evaluator -> Automatic, Method -> "Preemptive"], 
        BoxBaselineShift -> -0.5, BoxMargins -> 0.2]& )}], 
   Cell[
    StyleData["InlineMoreInfoOpenerButtonTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (AdjustmentBox[
        ButtonBox[
         TemplateBox[{
           TemplateBox[{}, "MoreInfoOpenerIconTemplate"], #4}, 
          "PrettyTooltipTemplate"], ButtonFunction :> (NotebookDelete[
            CurrentValue[
             ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoCell"}]]; If[
            And[
             MatchQ[
              CurrentValue[
               ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoCell"}], 
              Blank[CellObject]], CurrentValue[
               ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoTag"}] === #],
             CurrentValue[
               ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoCell"}] = 
             Inherited; CurrentValue[
               ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoTag"}] = 
             Inherited; Null, CurrentValue[
               ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoTag"}] = #; 
            CurrentValue[
               ReleaseHold[#3], {TaggingRules, "AttachedMoreInfoCell"}] = 
             MathLink`CallFrontEnd[
               FrontEnd`AttachCell[
                ReleaseHold[#3], #2, "Inline", 
                "ClosingActions" -> {"ParentChanged", "EvaluatorQuit"}]]]), 
         Appearance -> None, Evaluator -> Automatic, Method -> "Preemptive"], 
        BoxBaselineShift -> -0.5, BoxMargins -> 0.2]& )}], 
   Cell[
    StyleData["ClickToCopyTemplate"], 
    TemplateBoxOptions -> {
     DisplayFunction -> (PaneSelectorBox[{False -> TagBox[
           GridBox[{{#, 
              ButtonBox[
               GraphicsBox[{
                 GrayLevel[0.75], 
                 Thickness[
                  NCache[
                   Rational[2, 45], 0.044444444444444446`]], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                   0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                   0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                   0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                   0}, {0, 1, 0}, {0, 1, 0}}}, {{{10.5, 18.75}, {10.5, 18.}, {
                   9., 18.}, {9., 15.75}, {13.5, 15.75}, {13.5, 18.}, {12., 
                   18.}, {12., 18.75}}, {{6., 18.}, {6., 4.5}, {16.5, 4.5}, {
                   16.5, 18.}, {14.25, 18.}, {14.25, 17.25}, {15.75, 17.25}, {
                   15.75, 5.25}, {6.75, 5.25}, {6.75, 17.25}, {8.25, 17.25}, {
                   8.25, 18.}}, {{9.75, 17.25}, {12.75, 17.25}, {12.75, 
                   16.5}, {9.75, 16.5}}}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                  0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                  0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{8.25, 14.25}, {
                  14.25, 14.25}, {14.25, 13.5}, {8.25, 13.5}}, {{8.25, 12.}, {
                  14.25, 12.}, {14.25, 11.25}, {8.25, 11.25}}, {{8.25, 
                  9.75}, {14.25, 9.75}, {14.25, 9.}, {8.25, 9.}}, {{8.25, 
                  7.5}, {14.25, 7.5}, {14.25, 6.75}, {8.25, 6.75}}}]}, 
                ImageSize -> 12], ButtonFunction :> Null, 
               Appearance -> {
                "Default" -> None, "Hover" -> None, "Pressed" -> None}, 
               Evaluator -> Automatic, Method -> "Preemptive"]}}, 
            GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Center}}},
             AutoDelete -> False, 
            GridBoxItemSize -> {
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
            GridBoxSpacings -> {"Columns" -> {{0.4}}}], "Grid"], True -> 
         DynamicModuleBox[{RSNB`clickTime$$ = 0., RSNB`timeout$$ = 3.}, 
           TagBox[
            GridBox[{{#, 
               TagBox[
                ButtonBox[
                 DynamicBox[
                  ToBoxes[
                   Refresh[
                    If[AbsoluteTime[] - RSNB`clickTime$$ > RSNB`timeout$$, 
                    (RawBoxes[
                    TemplateBox[{
                    ToBoxes[#], 
                    ToBoxes[#2]}, "PrettyTooltipTemplate"]]& )[
                    Mouseover[
                    Graphics[{
                    GrayLevel[0.5], 
                    Thickness[
                    Rational[2, 45]], 
                    
                    FilledCurve[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}}, {{{10.5, 18.75}, {10.5, 
                    18.}, {9., 18.}, {9., 15.75}, {13.5, 15.75}, {13.5, 
                    18.}, {12., 18.}, {12., 18.75}}, {{6., 18.}, {6., 4.5}, {
                    16.5, 4.5}, {16.5, 18.}, {14.25, 18.}, {14.25, 17.25}, {
                    15.75, 17.25}, {15.75, 5.25}, {6.75, 5.25}, {6.75, 
                    17.25}, {8.25, 17.25}, {8.25, 18.}}, {{9.75, 17.25}, {
                    12.75, 17.25}, {12.75, 16.5}, {9.75, 16.5}}}], 
                    
                    FilledCurve[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{8.25, 
                    14.25}, {14.25, 14.25}, {14.25, 13.5}, {8.25, 13.5}}, {{
                    8.25, 12.}, {14.25, 12.}, {14.25, 11.25}, {8.25, 
                    11.25}}, {{8.25, 9.75}, {14.25, 9.75}, {14.25, 9.}, {8.25,
                     9.}}, {{8.25, 7.5}, {14.25, 7.5}, {14.25, 6.75}, {8.25, 
                    6.75}}}]}, ImageSize -> 12], 
                    Graphics[{
                    RGBColor[
                    0.9882352941176471, 0.4196078431372549, 
                    0.20392156862745098`], 
                    Thickness[
                    Rational[2, 45]], 
                    
                    FilledCurve[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}}, {{{10.5, 18.75}, {10.5, 
                    18.}, {9., 18.}, {9., 15.75}, {13.5, 15.75}, {13.5, 
                    18.}, {12., 18.}, {12., 18.75}}, {{6., 18.}, {6., 4.5}, {
                    16.5, 4.5}, {16.5, 18.}, {14.25, 18.}, {14.25, 17.25}, {
                    15.75, 17.25}, {15.75, 5.25}, {6.75, 5.25}, {6.75, 
                    17.25}, {8.25, 17.25}, {8.25, 18.}}, {{9.75, 17.25}, {
                    12.75, 17.25}, {12.75, 16.5}, {9.75, 16.5}}}], 
                    
                    FilledCurve[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{8.25, 
                    14.25}, {14.25, 14.25}, {14.25, 13.5}, {8.25, 13.5}}, {{
                    8.25, 12.}, {14.25, 12.}, {14.25, 11.25}, {8.25, 
                    11.25}}, {{8.25, 9.75}, {14.25, 9.75}, {14.25, 9.}, {8.25,
                     9.}}, {{8.25, 7.5}, {14.25, 7.5}, {14.25, 6.75}, {8.25, 
                    6.75}}}]}, ImageSize -> 12]], 
                    "Click to copy to the clipboard"], 
                    (RawBoxes[
                    TemplateBox[{
                    ToBoxes[#], 
                    ToBoxes[#2]}, "PrettyTooltipTemplate"]]& )[
                    Graphics[{
                    RGBColor[0, 
                    Rational[2, 3], 0], 
                    Thickness[
                    Rational[2, 45]], 
                    
                    FilledCurve[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}}, {{{10.5, 18.75}, {10.5, 
                    18.}, {9., 18.}, {9., 15.75}, {13.5, 15.75}, {13.5, 
                    18.}, {12., 18.}, {12., 18.75}}, {{6., 18.}, {6., 4.5}, {
                    16.5, 4.5}, {16.5, 18.}, {14.25, 18.}, {14.25, 17.25}, {
                    15.75, 17.25}, {15.75, 5.25}, {6.75, 5.25}, {6.75, 
                    17.25}, {8.25, 17.25}, {8.25, 18.}}, {{9.75, 17.25}, {
                    12.75, 17.25}, {12.75, 16.5}, {9.75, 16.5}}}], 
                    
                    FilledCurve[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{8.25, 
                    14.25}, {14.25, 14.25}, {14.25, 13.5}, {8.25, 13.5}}, {{
                    8.25, 12.}, {14.25, 12.}, {14.25, 11.25}, {8.25, 
                    11.25}}, {{8.25, 9.75}, {14.25, 9.75}, {14.25, 9.}, {8.25,
                     9.}}, {{8.25, 7.5}, {14.25, 7.5}, {14.25, 6.75}, {8.25, 
                    6.75}}}]}, ImageSize -> 12], "Copied"]], UpdateInterval -> 
                    1, TrackedSymbols :> {RSNB`clickTime$$}], StandardForm], 
                  Evaluator -> "System"], 
                 ButtonFunction :> (RSNB`clickTime$$ = AbsoluteTime[]; 
                  CopyToClipboard[
                    BinaryDeserialize[
                    BaseDecode[#2], Defer]]), 
                 Appearance -> {
                  "Default" -> None, "Hover" -> None, "Pressed" -> None}, 
                 Method -> "Queued", Evaluator -> "System"], 
                MouseAppearanceTag["LinkHand"]]}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Center}}}, AutoDelete -> 
             False, GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{0.4}}}], "Grid"], 
           DynamicModuleValues :> {}]}, 
        Dynamic[
         CurrentValue["MouseOver"]], ImageSize -> Automatic, FrameMargins -> 
        0]& )}], 
   Cell[
    StyleData["PrettyTooltipTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (TagBox[
        TooltipBox[#, 
         FrameBox[
          
          StyleBox[#2, "Text", FontColor -> 
           RGBColor[
            0.5372549019607843, 0.5372549019607843, 0.5372549019607843], 
           FontSize -> 12, FontWeight -> "Plain", FontTracking -> "Plain", 
           StripOnInput -> False], Background -> 
          RGBColor[
           0.9607843137254902, 0.9607843137254902, 0.9607843137254902], 
          FrameStyle -> 
          RGBColor[
           0.8980392156862745, 0.8980392156862745, 0.8980392156862745], 
          FrameMargins -> 8, StripOnInput -> False], TooltipDelay -> 0.1, 
         TooltipStyle -> {Background -> None, CellFrame -> 0}], 
        Annotation[#, 
         Framed[
          Style[
          RSNB`$$tooltip, "Text", FontColor -> 
           RGBColor[
            0.5372549019607843, 0.5372549019607843, 0.5372549019607843], 
           FontSize -> 12, FontWeight -> "Plain", FontTracking -> "Plain"], 
          Background -> 
          RGBColor[
           0.9607843137254902, 0.9607843137254902, 0.9607843137254902], 
          FrameStyle -> 
          RGBColor[
           0.8980392156862745, 0.8980392156862745, 0.8980392156862745], 
          FrameMargins -> 8], "Tooltip"]& ]& )}], 
   Cell[
    StyleData["ToolsGridTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (TagBox[
        GridBox[{{
           ButtonBox[
            StyleBox[
            "\"Insert Code Group\"", "Text", FontFamily -> "Source Sans Pro", 
             FontSize -> 11, StripOnInput -> False], ButtonFunction :> 
            With[{RSNB`nb$ = ButtonNotebook[], RSNB`$cp$ = $ContextPath}, 
              Quiet[
               
               Block[{$ContextPath = RSNB`$cp$, 
                 ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                 False}, 
                Internal`WithLocalSettings[
                 ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                   True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                  Needs["ResourceSystemClient`DefinitionNotebook`"], 
                  ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                  RSNB`nb$, 
                   
                   ComputationalEssayHelpers`Private`\
createComputationalParagraph[]], 
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                   False; Null]; Null]]], FrameMargins -> {{4, 4}, {0, 0}}, 
            BaseStyle -> Dynamic[
              FEPrivate`If[
               CurrentValue[Enabled], 
               FEPrivate`If[
                CurrentValue["MouseOver"], {
                FontColor -> RGBColor[0.898039, 0.898039, 0.898039]}, {
                FontColor -> 
                 RGBColor[
                  0.17960779999999998`, 0.17960779999999998`, 
                   0.17960779999999998`]}], {
               FontColor -> 
                RGBColor[
                 0.5898038999999999, 0.5898038999999999, 
                  0.5898038999999999]}], Evaluator -> "System"], 
            Appearance -> {"Default" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqt7e3HQaAbLhiuLLz588DpR48eAC3
AsgGigDF4coePnzY19cH5yI7BigOlIW4Fqilu7sbq7OB4kBZBjAgRhmRphHv
NiJ9Sny4ERMLaABrnAIA5lG1ZQ==
                "], "Byte", ColorSpace -> "RGB", 
                Interleaving -> True], "Hover" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqs7WpoaoEgoBsuGK4svPnzzfWlK+o
9dve4AJBQDZQBCgOV/bw4cOOloZVtb5wNRAEFAGKA2UhrgVqaaouRlMDQUBx
oCwDGBCjjEjTiHcbkT4lPtyIiQVi4hQArnpwkw==
                "], "Byte", 
                ColorSpace -> "RGB", Interleaving -> True], "Pressed" -> 
              Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCL2EAWRBZGVDqw5dvrz9BEZANVwxX9urVq3efvxUt
PujVuQGCgGygCFAcruzZs2fvPn3Nmb8frgaCgCJAcaAsxLVALc/efUZTA0FA
caAsAxgQo4xI04h3G5E+JT7ciIkFYuIUAHu5mtk=
                "], "Byte", 
                ColorSpace -> "RGB", Interleaving -> True]}, Background -> 
            RGBColor[0.898039, 0.898039, 0.898039], Method -> "Queued", 
            ImageSize -> {All, 20}, Enabled -> Dynamic[
              Not[
               TrueQ[
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
              TrackedSymbols :> {
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}], 
            Evaluator -> Automatic], 
           ActionMenuBox[
            ButtonBox[
             TemplateBox[{
               StyleBox[
                
                TemplateBox[{
                 "\"Insert Headings\"", 
                  "\"\[ThinSpace]\[ThinSpace]\[ThinSpace]\[FilledDownTriangle]\
\""}, "RowDefault"], "Text", FontFamily -> "Source Sans Pro", FontSize -> 11, 
                StripOnInput -> False], 
               "\"Insert a cell of the chosen type below the current \
selection\""}, "PrettyTooltipTemplate"], ButtonFunction :> 
             With[{RSNB`nb$ = ButtonNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; 
                   CurrentValue[RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
                    ProgressIndicator[Appearance -> "Necklace"]; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, Null], 
                   CurrentValue[RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
                    ""; ResourceSystemClient`DefinitionNotebook`$\
ButtonsDisabled = False; Null]; Null]]], FrameMargins -> {{4, 4}, {0, 0}}, 
             BaseStyle -> Dynamic[
               FEPrivate`If[
                CurrentValue[Enabled], 
                FEPrivate`If[
                 CurrentValue["MouseOver"], {
                 FontColor -> RGBColor[0.898039, 0.898039, 0.898039]}, {
                 FontColor -> 
                  RGBColor[
                   0.17960779999999998`, 0.17960779999999998`, 
                    0.17960779999999998`]}], {
                FontColor -> 
                 RGBColor[
                  0.5898038999999999, 0.5898038999999999, 
                   0.5898038999999999]}], Evaluator -> "System"], 
             Appearance -> {"Default" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqt7e3HQaAbLhiuLLz588DpR48eAC3
AsgGigDF4coePnzY19cH5yI7BigOlIW4Fqilu7sbq7OB4kBZBjAgRhmRphHv
NiJ9Sny4ERMLaABrnAIA5lG1ZQ==
                 "], "Byte", ColorSpace -> "RGB",
                  Interleaving -> True], "Hover" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqs7WpoaoEgoBsuGK4svPnzzfWlK+o
9dve4AJBQDZQBCgOV/bw4cOOloZVtb5wNRAEFAGKA2UhrgVqaaouRlMDQUBx
oCwDGBCjjEjTiHcbkT4lPtyIiQVi4hQArnpwkw==
                 "], "Byte", 
                 ColorSpace -> "RGB", Interleaving -> True], "Pressed" -> 
               Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCL2EAWRBZGVDqw5dvrz9BEZANVwxX9urVq3efvxUt
PujVuQGCgGygCFAcruzZs2fvPn3Nmb8frgaCgCJAcaAsxLVALc/efUZTA0FA
caAsAxgQo4xI04h3G5E+JT7ciIkFYuIUAHu5mtk=
                 "], "Byte", 
                 ColorSpace -> "RGB", Interleaving -> True]}, Background -> 
             RGBColor[0.898039, 0.898039, 0.898039], Method -> "Queued", 
             ImageSize -> {All, 20}, Enabled -> Dynamic[
               Not[
                TrueQ[
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
               TrackedSymbols :> {
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}], 
             Evaluator -> Automatic], {DynamicBox[
               ToBoxes[
                
                Grid[{{"Section", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "4") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Section"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], DynamicBox[
               ToBoxes[
                
                Grid[{{"Subsection", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "5") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Subsection"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], DynamicBox[
               ToBoxes[
                
                Grid[{{"Subsubsection", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "6") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Subsubsection"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], Delimiter, DynamicBox[
               ToBoxes[
                
                Grid[{{"Title", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "1") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Title"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], DynamicBox[
               ToBoxes[
                
                Grid[{{"Subtitle", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "2") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Subtitle"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], DynamicBox[
               ToBoxes[
                
                Grid[{{"Chapter", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "3") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Chapter"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Author\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Author"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Abstract\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Abstract"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]]}, Appearance -> None, Method -> 
            "Queued", Enabled -> Dynamic[
              Not[
               TrueQ[
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
              TrackedSymbols :> {
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}]], 
           ActionMenuBox[
            ButtonBox[
             TemplateBox[{
               StyleBox[
                
                TemplateBox[{
                 "\"Insert Text\"", 
                  "\"\[ThinSpace]\[ThinSpace]\[ThinSpace]\[FilledDownTriangle]\
\""}, "RowDefault"], "Text", FontFamily -> "Source Sans Pro", FontSize -> 11, 
                StripOnInput -> False], 
               "\"Insert a cell of the chosen type below the current \
selection\""}, "PrettyTooltipTemplate"], ButtonFunction :> 
             With[{RSNB`nb$ = ButtonNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; CurrentValue[
                    RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
                    ProgressIndicator[Appearance -> "Necklace"]; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, Null], 
                   CurrentValue[RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
                    ""; ResourceSystemClient`DefinitionNotebook`$\
ButtonsDisabled = False; Null]; Null]]], FrameMargins -> {{4, 4}, {0, 0}}, 
             BaseStyle -> Dynamic[
               FEPrivate`If[
                CurrentValue[Enabled], 
                FEPrivate`If[
                 CurrentValue["MouseOver"], {
                 FontColor -> RGBColor[0.898039, 0.898039, 0.898039]}, {
                 FontColor -> 
                  RGBColor[
                   0.17960779999999998`, 0.17960779999999998`, 
                    0.17960779999999998`]}], {
                FontColor -> 
                 RGBColor[
                  0.5898038999999999, 0.5898038999999999, 
                   0.5898038999999999]}], Evaluator -> "System"], 
             Appearance -> {"Default" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqt7e3HQaAbLhiuLLz588DpR48eAC3
AsgGigDF4coePnzY19cH5yI7BigOlIW4Fqilu7sbq7OB4kBZBjAgRhmRphHv
NiJ9Sny4ERMLaABrnAIA5lG1ZQ==
                 "], "Byte", ColorSpace -> "RGB",
                  Interleaving -> True], "Hover" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqs7WpoaoEgoBsuGK4svPnzzfWlK+o
9dve4AJBQDZQBCgOV/bw4cOOloZVtb5wNRAEFAGKA2UhrgVqaaouRlMDQUBx
oCwDGBCjjEjTiHcbkT4lPtyIiQVi4hQArnpwkw==
                 "], "Byte", 
                 ColorSpace -> "RGB", Interleaving -> True], "Pressed" -> 
               Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCL2EAWRBZGVDqw5dvrz9BEZANVwxX9urVq3efvxUt
PujVuQGCgGygCFAcruzZs2fvPn3Nmb8frgaCgCJAcaAsxLVALc/efUZTA0FA
caAsAxgQo4xI04h3G5E+JT7ciIkFYuIUAHu5mtk=
                 "], "Byte", 
                 ColorSpace -> "RGB", Interleaving -> True]}, Background -> 
             RGBColor[0.898039, 0.898039, 0.898039], Method -> "Queued", 
             ImageSize -> {All, 20}, Enabled -> Dynamic[
               Not[
                TrueQ[
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
               TrackedSymbols :> {
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}], 
             Evaluator -> Automatic], {DynamicBox[
               ToBoxes[
                Grid[{{
                  "Text", (((" (" <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "7") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Text"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], DynamicBox[
               ToBoxes[
                
                Grid[{{"CodeText", ((((" (" <> "Shift+") <> 
                    If[$OperatingSystem === "MacOSX", "Cmd", "Alt"]) <> "+") <> 
                    "7") <> ")"}}, Alignment -> {{Left, Right}, Automatic}, 
                 ItemSize -> {10, Automatic}], StandardForm]] :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "CodeText"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Item\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`createCellBelow[
                    "Item"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]]}, Appearance -> None, Method -> 
            "Queued", Enabled -> Dynamic[
              Not[
               TrueQ[
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
              TrackedSymbols :> {
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}]], 
           ActionMenuBox[
            ButtonBox[
             StyleBox[
              
              TemplateBox[{
               "\"Open/Close Groups\"", 
                "\"\[ThinSpace]\[ThinSpace]\[ThinSpace]\[FilledDownTriangle]\"\
"}, "RowDefault"], "Text", FontFamily -> "Source Sans Pro", FontSize -> 11, 
              StripOnInput -> False], ButtonFunction :> 
             With[{RSNB`nb$ = ButtonNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; CurrentValue[
                    RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
                    ProgressIndicator[Appearance -> "Necklace"]; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, Null], 
                   CurrentValue[RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
                    ""; ResourceSystemClient`DefinitionNotebook`$\
ButtonsDisabled = False; Null]; Null]]], FrameMargins -> {{4, 4}, {0, 0}}, 
             BaseStyle -> Dynamic[
               FEPrivate`If[
                CurrentValue[Enabled], 
                FEPrivate`If[
                 CurrentValue["MouseOver"], {
                 FontColor -> RGBColor[0.898039, 0.898039, 0.898039]}, {
                 FontColor -> 
                  RGBColor[
                   0.17960779999999998`, 0.17960779999999998`, 
                    0.17960779999999998`]}], {
                FontColor -> 
                 RGBColor[
                  0.5898038999999999, 0.5898038999999999, 
                   0.5898038999999999]}], Evaluator -> "System"], 
             Appearance -> {"Default" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqt7e3HQaAbLhiuLLz588DpR48eAC3
AsgGigDF4coePnzY19cH5yI7BigOlIW4Fqilu7sbq7OB4kBZBjAgRhmRphHv
NiJ9Sny4ERMLaABrnAIA5lG1ZQ==
                 "], "Byte", ColorSpace -> "RGB",
                  Interleaving -> True], "Hover" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCl2EAWRBZGVCqs7WpoaoEgoBsuGK4svPnzzfWlK+o
9dve4AJBQDZQBCgOV/bw4cOOloZVtb5wNRAEFAGKA2UhrgVqaaouRlMDQUBx
oCwDGBCjjEjTiHcbkT4lPtyIiQVi4hQArnpwkw==
                 "], "Byte", 
                 ColorSpace -> "RGB", Interleaving -> True], "Pressed" -> 
               Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQPEUCL2EAWRBZGVDqw5dvrz9BEZANVwxX9urVq3efvxUt
PujVuQGCgGygCFAcruzZs2fvPn3Nmb8frgaCgCJAcaAsxLVALc/efUZTA0FA
caAsAxgQo4xI04h3G5E+JT7ciIkFYuIUAHu5mtk=
                 "], "Byte", 
                 ColorSpace -> "RGB", Interleaving -> True]}, Background -> 
             RGBColor[0.898039, 0.898039, 0.898039], Method -> "Queued", 
             ImageSize -> {All, 20}, Enabled -> Dynamic[
               Not[
                TrueQ[
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
               TrackedSymbols :> {
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}], 
             Evaluator -> Automatic], {
            "\"Open Chapters\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`openCloseAll[
                    ButtonNotebook[], "Chapter", Open]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Close Chapters\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`openCloseAll[
                    ButtonNotebook[], "Chapter", Closed]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Open Sections\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`openCloseAll[
                    ButtonNotebook[], "Section", Open]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Close Sections\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`openCloseAll[
                    ButtonNotebook[], "Section", Closed]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]]}, Appearance -> None, Method -> 
            "Queued", Enabled -> Dynamic[
              Not[
               TrueQ[
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
              TrackedSymbols :> {
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}]]}}, 
         GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Center}}}, 
         AutoDelete -> False, 
         GridBoxBackground -> {"Columns" -> {{None}}, "Rows" -> {
             RGBColor[0.898039, 0.898039, 0.898039]}}, 
         GridBoxFrame -> {
          "Columns" -> False, 
           "RowsIndexed" -> {1 -> RGBColor[0.898039, 0.898039, 0.898039]}}, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
         GridBoxSpacings -> {
          "ColumnsIndexed" -> {1 -> 0, -1 -> 0}, "Rows" -> {{Automatic}}}, 
         FrameStyle -> GrayLevel[0.75]], "Grid"]& )}], 
   Cell[
    StyleData["MainGridTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (TagBox[
        GridBox[{{
           TagBox[
            GridBox[{{
               GraphicsBox[{{
                  Thickness[0.03333333333333333], {
                   FaceForm[{
                    RGBColor[
                    0.8200000000000001, 0.8200000000000001, 
                    0.8200000000000001], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{27.,
                     1.}, {3., 1.}, {3., 29.}, {27., 29.}}}]}, {
                   FaceForm[{
                    RGBColor[1., 1., 1.], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{4., 
                    2.}, {26., 2.}, {26., 28.}, {4., 28.}}}]}, {
                   FaceForm[{
                    RGBColor[0.749, 0.749, 0.749], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{24.,
                     2.}, {6., 2.}, {6., 10.}, {24., 10.}}}]}, {
                   FaceForm[{
                    RGBColor[0.898, 0.898, 0.898], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{23.,
                     2.}, {7., 2.}, {7., 9.}, {23., 9.}}}]}, {
                   FaceForm[{
                    RGBColor[1., 1., 1.], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{22.,
                     2.}, {12., 2.}, {12., 8.}, {22., 8.}}}]}, {
                   FaceForm[{
                    RGBColor[0.984, 0.6859999999999999, 0.365], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}}, {{{8., 7.}, {11., 7.}, {11., 8.}, {8., 8.}}, {{8., 
                    5.}, {11., 5.}, {11., 6.}, {8., 6.}}, {{8., 4.}, {11., 
                    4.}, {11., 3.}, {8., 3.}}}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1,
                     0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 
                    2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 
                    1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0,
                     1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{
                    0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {
                    0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1,
                     0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 
                    2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 
                    1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                    0}, {0, 1, 0}}}, {{{9.697299999999998, 24.522}, {
                    9.133299999999998, 24.645}, {8.861299999999998, 
                    24.259999999999998`}, {9.313299999999998, 24.092}}, {{
                    9.291299999999998, 23.933}, {8.9683, 
                    24.052999999999997`}, {9.2183, 23.726999999999997`}, {
                    9.587299999999999, 23.601}}, {{9.553299999999998, 
                    23.447}, {9.169299999999998, 23.578}, {8.760299999999999, 
                    23.456999999999997`}, {9.102299999999998, 
                    23.348999999999997`}}, {{9.083299999999998, 22.633}, {
                    9.025299999999998, 23.208}, {8.567299999999998, 23.354}, {
                    8.555299999999999, 22.865}}, {{8.521299999999998, 
                    23.551}, {9.039299999999999, 23.703}, {8.711299999999998, 
                    24.13}, {8.2183, 23.962999999999997`}}, {{8.4073, 
                    23.197}, {8.171299999999999, 22.851}, {8.171299999999999, 
                    22.445}, {8.398299999999999, 22.837}}, {{
                    8.091299999999999, 23.014}, {8.395299999999999, 23.458}, {
                    8.091299999999999, 23.871}, {7.786299999999999, 
                    23.458}}, {{8.010299999999999, 22.851}, {
                    7.773299999999999, 23.197}, {7.782299999999998, 22.837}, {
                    8.010299999999999, 22.445}}, {{7.964299999999999, 
                    23.962999999999997`}, {7.469299999999999, 24.13}, {
                    7.141299999999999, 23.703}, {7.659299999999998, 
                    23.551}}, {{7.626299999999999, 22.865}, {
                    7.614299999999998, 23.354}, {7.155299999999999, 23.209}, {
                    7.097299999999999, 22.633999999999997`}}, {{
                    7.078299999999999, 23.348999999999997`}, {
                    7.422299999999998, 23.456999999999997`}, {
                    7.0112999999999985`, 23.578}, {6.6282999999999985`, 
                    23.447}}, {{6.593299999999998, 23.601}, {
                    6.962299999999999, 23.726999999999997`}, {
                    7.214299999999998, 24.054}, {6.889299999999999, 
                    23.933999999999997`}}, {{6.867299999999998, 24.092}, {
                    7.319299999999998, 24.259999999999998`}, {
                    7.0482999999999985`, 24.645}, {6.484299999999998, 
                    24.522}}, {{7.113299999999999, 25.217}, {
                    7.158299999999999, 24.761}, {7.356299999999999, 
                    24.479999999999997`}, {7.344299999999999, 24.904}}, {{
                    7.5032999999999985`, 24.817999999999998`}, {
                    7.518299999999998, 24.279}, {8.010299999999999, 24.113}, {
                    8.010299999999999, 24.637999999999998`}}, {{
                    7.2452999999999985`, 25.302}, {7.4712999999999985`, 
                    24.997}, {7.8622999999999985`, 24.857}, {
                    7.656299999999998, 25.122}}, {{7.800299999999998, 
                    25.192}, {8.091299999999999, 24.82}, {8.381299999999998, 
                    25.192}, {8.091299999999999, 25.689}}, {{
                    8.171299999999999, 24.637999999999998`}, {
                    8.171299999999999, 24.113}, {8.663299999999998, 24.279}, {
                    8.677299999999999, 24.817999999999998`}}, {{
                    8.836299999999998, 24.904}, {8.8243, 
                    24.479999999999997`}, {9.022299999999998, 24.761}, {
                    9.068299999999999, 25.217}}, {{8.318299999999999, 
                    24.857}, {8.710299999999998, 24.997}, {8.935299999999998, 
                    25.302}, {8.525299999999998, 25.122}}, CompressedData["
1:eJxTTMoPSmViYGAQAWIQbb/k/j6+P8oOE9/W2JvOs3CQKL4u9OmhkgMDGFg4
wOTzhJoPnEo0d1i3Lan+ZqySw+2fdVl7TMwdPjqeT7vargSVN3MQkT351J5F
0eH64gJbru1mDk9BBugpQM0zcwicpthXulEOLl8ja5HuslIarp/ZRNDM5q80
3PwHEeLbLx6QgNuvt1udn/urFNx9MHmY+2H6VdgapzqftoCbD5G3hNuvGdN/
6KuHJZr7rODuh8nX3bStjOhQguuH+R9mPgC7F3w4
                    "]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{11.,
                     25.}, {24.000999999999998`, 25.}, {24.000999999999998`, 
                    26.}, {11., 26.}}}]}, {
                   FaceForm[{
                    RGBColor[0.749, 0.749, 0.749], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{24.,
                     22.}, {11., 22.}, {11., 24.}, {24., 24.}}}]}, {
                   FaceForm[{
                    RGBColor[0.984, 0.6859999999999999, 0.365], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{19., 20.}, {
                    24., 20.}, {24., 21.}, {19., 21.}}, {{6., 20.}, {18., 
                    20.}, {18., 21.}, {6., 21.}}, {{6., 18.}, {10., 18.}, {
                    10., 19.}, {6., 19.}}, {{11., 19.}, {17., 19.}, {17., 
                    18.}, {11., 18.}}}], 
                   
                   FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                    0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                    0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1,
                     0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 
                    2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 
                    1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0,
                     1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{
                    0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, CompressedData["
1:eJx10ksKgzAYReHQdtgFZFqRIEEkPioOXUOXUOjY/c90kDvogSuEn08OaoKv
7/H53UMIz2s9rnULutL+P4txh/vyiPtvY/byWudQ52Zc0MtLnRk93aOX9byE
nub3yhG9m+xlnY/2MxsP6OUJfTZmLzd16jyicUYvd/Bs3KOXE9waZ/R8v/YT
jfkfl/0EHWIqGQ==
                    "]]}, {
                   FaceForm[{
                    RGBColor[0.392, 0.392, 0.392], 
                    Opacity[1.]}], 
                   
                   FilledCurveBox[{{{1, 4, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 
                    3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}}, {{1, 4, 
                    3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 
                    3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}}, {{1, 4, 
                    3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 
                    3}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}, {1, 3, 
                    3}}, {{1, 4, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 
                    3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}}, {{1, 4, 3}, {0, 1, 
                    0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {0, 1, 
                    0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}}}, {CompressedData["
1:eJxTTMoPSmViYGAQBWIQ/YB78sqmiXoOlxjuMXF2sTisvRFf5h+n59Dj9YrF
RJDFwUzqQLQCo56D3oQFPwyfMUH4B3Ud1qs+aZ53lsHhxfb1zM959OD8lpoL
myNN9Rwq5ixS3vmH0QEo26ORC+MzOSQuvGbyfoeew7F/m6o/bWB2mLcaZKO+
g5xFuktmPqvD1mWVxzWtDBy2XwQ5iNXhjf5udf5uAwdWE0Ezm71MDs/WgSwy
cFh9/mrYG30mhw+tJ0MOrjBwiM09CjSR0QFkS/sWA7h7+koLpXkfIPgvbq75
FXPWwCHx8GXtVEkmuHoJsIXMDm0g45YYOHR8uX29sZjZgZ8b6CJ3A4fvbPEz
fETZHKqW6jjLZOs7sIO4PWwO6OEHAOM+kNE=
                    "], CompressedData["
1:eJxTTMoPSmViYGCQBmIQ3XHp3gNuYX2HJyKyJ5/aSzgsZtzDKiSi65AbVrtu
2yMxB7VDbcvDuXQcOKXnxWmeFnSo+21VcI5D24HVRNDMZi+nQ+LCaybvd2g5
RDvZvbi5hs0hXSmnoipVy0HtSfO8s6uYHaIVHD8m82g5rFcFCTA4aElMvcKZ
geBv1MsDWqnlYLM3aJpiHzPUPm0HOYt0l8x8NofU708SF4ZpO1zRTpV8FMHp
EFyiMv1/gI7D9qT6m7aVghD3hug6HN+1o5etQMzh/wSgCw30HfbxzTFetEXc
4dOGgOxZ0w0cwM4XkHQ4Xe+xv3avocO0TTyFa7pFHfi5ty6rbDdyqFu3Lak+
Ushhho9oj5eXscOcRco7/7TzOSyrPK5p9cnYQazH6xWLCbuDLNhdJnD3r/kV
k3t0HoJfKM37QNfAxEEDbDCbwxI/B+HExcYOBrvVgSI8DidDDq5Ywmfs8OaA
pbLXbwEHq0lAF8kbOXzJAQa4magDSPuEAwYOINfdfiHpgB4/AHQArZ4=
                    
                    "], CompressedData["
1:eJxTTMoPSmViYGCQAWIQvTz8lNGRjXoO67Yl1d/cK+MwK3T+6rUROg5rum9n
MLyXcujReMu7z0Db4ULa1edZq0Ucchcz7mEV0nJ4xWIiaGbD62AzG6RB08HZ
7sXNNb84HDzmNqgdctN0MNitzs+9lcXht1XBuY5LGg7rVZ80zzvL4ACxThPO
V3D8mHxmrqbDs3UgARaHAE+gAUxaDqwg4/dyOIBU5yVrOajs/NP+5TaPA5C3
mLFG22HG3cntrVEiDpNWNgV6ztVxYPsg5hFQI+UgmuX7ue+ynkNokPqCTgMZ
B3ZboANf6ztUN+v7LOOSdQC5atlMAwe+OcaLtmyXcZC1SHfJ9Dd0WA/yv620
wyJlkEVGDh4BfySKr4s7TPnGFj9jjYlDxBt9oI/4HL7cvt5YfMwU7n4w/xkm
/2PryZCDKxgdmordpnxzM3XQSZV8FCHO6QA2vtrEIXKF6VnrcwIOQdMU+0o3
Gju4/Hr7+oCliEO9x/5a2R+GDixH+w3L10o7AAMhaFqjAcT9XXIO6PEFAPga
wsg=
                    "], CompressedData["
1:eJxTTMoPSmViYGAQBWIQPSt0/uq1N/QdLjHcY+Ls4nNoUDvUtjxcz8Fq0ul6
j/28DmtvxJf5x+k6bL8IUsDusJtVSMS+TcdhveqT5nlnGRycZF4/MruF4DcF
es5tWKbroAbirmJzCC5Rmf5/gp5D1XFNoJE8Dkt1nIE69B3ico/+21TN6/BA
d8KCH4UGDo8ixIFW8DnIWqS7ZPIbOrQAdZ+/yuVgszdommKfocPq81fD3uiz
OkQ52b24ucbQwd9BOPHwZWaH5HdAkROGDln5H1pPhjA5gK25bwh3Ty3IQE4j
OB9oyG7194YO7PEzfER7mBxWgcw9b+jwmsVE0MyGBWLfOkOHApBxB1nBtmir
wtzD7QAKldpaAwdGUHDJ8zughx8AsHaP1w==
                    "], CompressedData["
1:eJxTTMoPSmViYGCQBmIQvb9W1iI9xchhwoIfhs/WcTs42b24uWaXkYNI4uHL
2qkcDlO+scXP4DF2YI+f4SPaw+JgszdomqKfscN61SfN884yOHTJJ7+LUkLw
d6vzc299ZuSQM6t8ziJlFgeQcb/mGDmogaRXsTu0nQw5uMLFyMFXtMfrFQu3
Q7EbyAZDh9yw2nXbHgk6gKzzKTVwaGI52m/4XdjB4WPymdhufQeXX29fH7AU
dugLLlGZ/l/XYc9jEdmTqUION+LL/OXEdB3W/orJPfqPx8FPTizL11jHYR/Q
mX2lrA5ty8NPGbHoOPSVFkrzPmB2MJipJTH1irbDKaAzlvgxOeyPVnD8uFgb
7n4QN9kHwc+uqFqq06wNNY/JYeetrr+p27UdykDG6bI4AGWdZV5rO/wEhp7q
E1aHRYx7WIWu6ED9y+uw4JrJe4ujug4z7k5ub70l5HBna0vNhWJ9B4ni60Kf
Dgo7GIG0HTZwOGftd3HiHRGHTxsCsmddN3TQ+TYdqEPAAT1+AE3XvCM=
                    
                    "]}]}}}, {
                ImageSize -> {Automatic, 32}, 
                 ImagePadding -> {{5, 0}, {0, 0}}, BaselinePosition -> 
                 Scaled[0.25], AspectRatio -> Automatic, 
                 ImageSize -> {30., 30.}, 
                 PlotRange -> {{0., 30.}, {0., 30.}}}], 
               StyleBox[
                TagBox[
                 GridBox[{{
                    StyleBox[
                    "\"Computational Essay\"", FontFamily -> 
                    "Source Sans Pro", FontWeight -> "SemiBold", StripOnInput -> 
                    False]}}, 
                  GridBoxAlignment -> {
                   "Columns" -> {{Left}}, "Rows" -> {{Baseline}}}, AutoDelete -> 
                  False, GridBoxDividers -> {
                   "Columns" -> {{None}}, "Rows" -> {{None}}}, 
                  GridBoxItemSize -> {
                   "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], 
                 "Grid"], FontSize -> 24, FontColor -> GrayLevel[1], 
                StripOnInput -> False]}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Baseline}}}, AutoDelete -> 
             False, GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], "Grid"], 
           "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", 
           "\[SpanFromLeft]", "\[SpanFromLeft]", "\[SpanFromLeft]", "\"\""}, {
          
           ActionMenuBox[
            TemplateBox[{
              TemplateBox[{
                TemplateBox[{"\"Sample Content\"", 
                  TemplateBox[{5}, "Spacer1"], "\"\[FilledDownTriangle]\""}, 
                 "RowDefault"], 
                "\"Clear sample content in the current notebook, or view a \
completed sample computational essay\""}, "PrettyTooltipTemplate"], Null& , 
              "\"Clear sample content in the current notebook, or view a \
completed sample computational essay\"", True}, "OrangeButtonTemplate"], {
            "\"Insert Sample Content\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    ComputationalEssayHelpers`Private`insertSampleContent[]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Clear Sample Content\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    Module[{$CellContext`cells = Flatten[
                    Map[{NotebookFind[
                    ButtonNotebook[], #, All, CellTags, AutoScroll -> False]; 
                    SelectedCells[]}& , {
                    "ExampleTopicSentence", "TopicSentenceButton", 
                    "ExampleCodeText", "ExampleInput", "ExampleOutput", 
                    "EssayStructureExample", "CitationExample", 
                    "SectionMoreInfoYourEssayTitle", 
                    "SectionMoreInfoAuthorInfoButton", 
                    "SectionMoreInfoAbstract", 
                    "SectionMoreInfoSectionHeaderInfoButton", "SectionHeader",
                     "ItemSample"}]]}, 
                    NotebookDelete[$CellContext`cells]]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]], "\"Open Sample Essay\"" :> 
             With[{RSNB`nb$ = InputNotebook[], RSNB`$cp$ = $ContextPath}, 
               Quiet[
                
                Block[{$ContextPath = RSNB`$cp$, 
                  ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
                  False}, 
                 Internal`WithLocalSettings[
                  ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    True; Once[
                    ReleaseHold[
                    CurrentValue[
                    RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                    "KernelSession"]; 
                   Needs["ResourceSystemClient`DefinitionNotebook`"], 
                   ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                   RSNB`nb$, 
                    SystemOpen["https://wolfr.am/SampleEssay"]], 
                   ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                    False; Null]; Null]]]}, Appearance -> None, Method -> 
            "Queued", Enabled -> Dynamic[
              Not[
               TrueQ[
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
              TrackedSymbols :> {
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}]], 
           
           TemplateBox[{
            "\"Open Guidelines\"", SystemOpen[
             "https://wolfr.am/ComputationalEssayGuidelines"]& , "\"\"", 
             False}, "OrangeButtonTemplate"], 
           TemplateBox[{
             TemplateBox[{
               TagBox[
                GridBox[{{"\"Tools\"", 
                   PaneSelectorBox[{False -> GraphicsBox[{
                    GrayLevel[1], 
                    AbsoluteThickness[1], 
                    LineBox[{{-1, -1}, {-1, 1}, {1, 1}, {1, -1}, {-1, -1}}], 
                    LineBox[{{0., 0.65}, {0., -0.65}}], 
                    LineBox[{{-0.65, 0.}, {0.65, 0.}}]}, ImageSize -> 8, 
                    PlotRangePadding -> None, Background -> None], True -> 
                    GraphicsBox[{
                    GrayLevel[1], 
                    AbsoluteThickness[1], 
                    LineBox[{{-1, -1}, {-1, 1}, {1, 1}, {1, -1}, {-1, -1}}], 
                    LineBox[{{-0.65, 0.}, {0.65, 0.}}]}, ImageSize -> 8, 
                    PlotRangePadding -> None, Background -> None]}, 
                    Dynamic[
                    CurrentValue[
                    EvaluationNotebook[], {TaggingRules, "ToolsOpen"}, True]],
                     BaselinePosition -> Scaled[-0.1]]}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {{Automatic}}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxItemSize -> {
                  "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                 GridBoxSpacings -> {"Columns" -> {{0.35}}}], "Grid"], 
               "\"Toggle documentation toolbar\""}, "PrettyTooltipTemplate"], 
             ResourceSystemClient`DefinitionNotebook`ToggleToolbar[
              ButtonNotebook[]]& , "\"Toggle documentation toolbar\"", False},
             "OrangeButtonTemplate"], 
           TagBox[
            
            GridBox[{{"\"\"", "\"\""}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Center}}}, AutoDelete -> 
             False, GridBoxDividers -> {
              "ColumnsIndexed" -> {2 -> True}, "Rows" -> {{False}}}, 
             GridBoxItemSize -> {"Columns" -> {{Automatic}}, "Rows" -> {{2}}},
              GridBoxSpacings -> {"Columns" -> {{0.5}}}, FrameStyle -> 
             RGBColor[0.99215, 0.843125, 0.68235]], "Grid"], 
           TemplateBox[{
             
             TemplateBox[{
              "\"Check\"", "\"Check notebook for potential errors\""}, 
              "PrettyTooltipTemplate"], 
             ResourceSystemClient`DefinitionNotebook`CheckDefinitionNotebook[
              ButtonNotebook[]]& , "\"Check notebook for potential errors\"", 
             False}, "OrangeButtonTemplate"], 
           TemplateBox[{
             
             TemplateBox[{
              "\"Publish to Cloud\"", 
               "\"Publish your computational essay to the cloud\""}, 
              "PrettyTooltipTemplate"], 
             With[{$CellContext`nb = ButtonNotebook[]}, CreateDialog[
                
                ComputationalEssayHelpers`Private`createSubmissionDialog[$\
CellContext`nb], WindowSize -> {350, 150}, Background -> White, WindowFrame -> 
                "Normal", "Evaluator" -> "System"]; Null]& , 
             "\"Publish your computational essay to the cloud\"", False}, 
            "OrangeButtonTemplate"], 
           ItemBox[
            StyleBox[
             DynamicBox[
              ToBoxes[
               CurrentValue[
                EvaluationNotebook[], {TaggingRules, "StatusMessage"}, ""], 
               StandardForm], Evaluator -> "System", 
              Initialization :> (CurrentValue[
                 EvaluationNotebook[], {TaggingRules, "StatusMessage"}] = 
               "")], "Text", 
             GrayLevel[1], StripOnInput -> False], ItemSize -> Fit, 
            StripOnInput -> False], 
           TemplateBox[{
             TemplateBox[{
              "\"Submit to Notebook Archive\"", 
               "\"Submit your essay to the Wolfram Notebook Archive\""}, 
              "PrettyTooltipTemplate"], 
             ComputationalEssayHelpers`Private`submitEssay[]& , 
             "\"Submit your essay to the Wolfram Notebook Archive\"", False}, 
            "OrangeButtonTemplate"]}}, 
         GridBoxAlignment -> {
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {-1 -> Right}, 
           "Rows" -> {{Center}}}, AutoDelete -> False, 
         GridBoxBackground -> {"Columns" -> {{None}}, "Rows" -> {
             RGBColor[0.9843, 0.68625, 0.3647], 
             RGBColor[0.7185389999999999, 0.5009625, 0.266231]}}, 
         GridBoxFrame -> {
          "Columns" -> False, 
           "RowsIndexed" -> {
            1 -> RGBColor[0.9843, 0.68625, 0.3647], 2 -> 
             RGBColor[0.7185389999999999, 0.5009625, 0.266231]}}, 
         GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
         GridBoxSpacings -> {
          "Columns" -> {5, {0.9}, 5}, 
           "RowsIndexed" -> {1 -> 1.1, 2 -> 1.3, 3 -> 1.55}}, FrameStyle -> 
         RGBColor[0.9843, 0.68625, 0.3647]], "Grid"]& )}], 
   Cell[
    StyleData["ReviewerCommentLabelTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (TagBox[
        GridBox[{{#, 
           TemplateBox[{
             GraphicsBox[{{
                Thickness[0.03333333333333333], {
                 FaceForm[{
                   RGBColor[
                   0.8200000000000001, 0.8200000000000001, 
                    0.8200000000000001], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{27., 
                  1.}, {3., 1.}, {3., 29.}, {27., 29.}}}]}, {
                 FaceForm[{
                   RGBColor[1., 1., 1.], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{4., 
                  2.}, {26., 2.}, {26., 28.}, {4., 28.}}}]}, {
                 FaceForm[{
                   RGBColor[0.749, 0.749, 0.749], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{24., 
                  2.}, {6., 2.}, {6., 10.}, {24., 10.}}}]}, {
                 FaceForm[{
                   RGBColor[0.898, 0.898, 0.898], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{23., 
                  2.}, {7., 2.}, {7., 9.}, {23., 9.}}}]}, {
                 FaceForm[{
                   RGBColor[1., 1., 1.], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{22., 
                  2.}, {12., 2.}, {12., 8.}, {22., 8.}}}]}, {
                 FaceForm[{
                   RGBColor[0.984, 0.6859999999999999, 0.365], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                  0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                  0}}}, {{{8., 7.}, {11., 7.}, {11., 8.}, {8., 8.}}, {{8., 
                  5.}, {11., 5.}, {11., 6.}, {8., 6.}}, {{8., 4.}, {11., 
                  4.}, {11., 3.}, {8., 3.}}}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                   0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                   0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                   0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                   0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                   0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                   0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                   0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                   0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                   0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                   0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                   0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                   0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                   0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                   0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                   0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 
                   0}}}, {{{9.697299999999998, 24.522}, {9.133299999999998, 
                   24.645}, {8.861299999999998, 24.259999999999998`}, {
                   9.313299999999998, 24.092}}, {{9.291299999999998, 
                   23.933}, {8.9683, 24.052999999999997`}, {9.2183, 
                   23.726999999999997`}, {9.587299999999999, 23.601}}, {{
                   9.553299999999998, 23.447}, {9.169299999999998, 23.578}, {
                   8.760299999999999, 23.456999999999997`}, {
                   9.102299999999998, 23.348999999999997`}}, {{
                   9.083299999999998, 22.633}, {9.025299999999998, 23.208}, {
                   8.567299999999998, 23.354}, {8.555299999999999, 
                   22.865}}, {{8.521299999999998, 23.551}, {9.039299999999999,
                    23.703}, {8.711299999999998, 24.13}, {8.2183, 
                   23.962999999999997`}}, {{8.4073, 23.197}, {
                   8.171299999999999, 22.851}, {8.171299999999999, 22.445}, {
                   8.398299999999999, 22.837}}, {{8.091299999999999, 
                   23.014}, {8.395299999999999, 23.458}, {8.091299999999999, 
                   23.871}, {7.786299999999999, 23.458}}, {{8.010299999999999,
                    22.851}, {7.773299999999999, 23.197}, {7.782299999999998, 
                   22.837}, {8.010299999999999, 22.445}}, {{7.964299999999999,
                    23.962999999999997`}, {7.469299999999999, 24.13}, {
                   7.141299999999999, 23.703}, {7.659299999999998, 
                   23.551}}, {{7.626299999999999, 22.865}, {7.614299999999998,
                    23.354}, {7.155299999999999, 23.209}, {7.097299999999999, 
                   22.633999999999997`}}, {{7.078299999999999, 
                   23.348999999999997`}, {7.422299999999998, 
                   23.456999999999997`}, {7.0112999999999985`, 23.578}, {
                   6.6282999999999985`, 23.447}}, {{6.593299999999998, 
                   23.601}, {6.962299999999999, 23.726999999999997`}, {
                   7.214299999999998, 24.054}, {6.889299999999999, 
                   23.933999999999997`}}, {{6.867299999999998, 24.092}, {
                   7.319299999999998, 24.259999999999998`}, {
                   7.0482999999999985`, 24.645}, {6.484299999999998, 
                   24.522}}, {{7.113299999999999, 25.217}, {7.158299999999999,
                    24.761}, {7.356299999999999, 24.479999999999997`}, {
                   7.344299999999999, 24.904}}, {{7.5032999999999985`, 
                   24.817999999999998`}, {7.518299999999998, 24.279}, {
                   8.010299999999999, 24.113}, {8.010299999999999, 
                   24.637999999999998`}}, {{7.2452999999999985`, 25.302}, {
                   7.4712999999999985`, 24.997}, {7.8622999999999985`, 
                   24.857}, {7.656299999999998, 25.122}}, {{7.800299999999998,
                    25.192}, {8.091299999999999, 24.82}, {8.381299999999998, 
                   25.192}, {8.091299999999999, 25.689}}, {{8.171299999999999,
                    24.637999999999998`}, {8.171299999999999, 24.113}, {
                   8.663299999999998, 24.279}, {8.677299999999999, 
                   24.817999999999998`}}, {{8.836299999999998, 24.904}, {
                   8.8243, 24.479999999999997`}, {9.022299999999998, 
                   24.761}, {9.068299999999999, 25.217}}, {{8.318299999999999,
                    24.857}, {8.710299999999998, 24.997}, {8.935299999999998, 
                   25.302}, {8.525299999999998, 25.122}}, CompressedData["
1:eJxTTMoPSmViYGAQAWIQbb/k/j6+P8oOE9/W2JvOs3CQKL4u9OmhkgMDGFg4
wOTzhJoPnEo0d1i3Lan+ZqySw+2fdVl7TMwdPjqeT7vargSVN3MQkT351J5F
0eH64gJbru1mDk9BBugpQM0zcwicpthXulEOLl8ja5HuslIarp/ZRNDM5q80
3PwHEeLbLx6QgNuvt1udn/urFNx9MHmY+2H6VdgapzqftoCbD5G3hNuvGdN/
6KuHJZr7rODuh8nX3bStjOhQguuH+R9mPgC7F3w4
                   "]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{11., 
                  25.}, {24.000999999999998`, 25.}, {24.000999999999998`, 
                  26.}, {11., 26.}}}]}, {
                 FaceForm[{
                   RGBColor[0.749, 0.749, 0.749], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{24., 
                  22.}, {11., 22.}, {11., 24.}, {24., 24.}}}]}, {
                 FaceForm[{
                   RGBColor[0.984, 0.6859999999999999, 0.365], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                  0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                  0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}}, {{{19., 20.}, {24.,
                   20.}, {24., 21.}, {19., 21.}}, {{6., 20.}, {18., 20.}, {
                  18., 21.}, {6., 21.}}, {{6., 18.}, {10., 18.}, {10., 19.}, {
                  6., 19.}}, {{11., 19.}, {17., 19.}, {17., 18.}, {11., 
                  18.}}}], 
                 
                 FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                  0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                  0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                  0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                  0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 
                  0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 
                  0}, {0, 1, 0}}, {{0, 2, 0}, {0, 1, 0}, {0, 1, 0}}, {{0, 2, 
                  0}, {0, 1, 0}, {0, 1, 0}}}, CompressedData["
1:eJx10ksKgzAYReHQdtgFZFqRIEEkPioOXUOXUOjY/c90kDvogSuEn08OaoKv
7/H53UMIz2s9rnULutL+P4txh/vyiPtvY/byWudQ52Zc0MtLnRk93aOX9byE
nub3yhG9m+xlnY/2MxsP6OUJfTZmLzd16jyicUYvd/Bs3KOXE9waZ/R8v/YT
jfkfl/0EHWIqGQ==
                  "]]}, {
                 FaceForm[{
                   RGBColor[0.392, 0.392, 0.392], 
                   Opacity[1.]}], 
                 
                 FilledCurveBox[{{{1, 4, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 
                   3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}}, {{1, 4, 
                   3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 
                   3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}}, {{1, 4, 
                   3}, {1, 3, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 
                   3}, {1, 3, 3}, {0, 1, 0}, {0, 1, 0}, {1, 3, 3}, {1, 3, 
                   3}}, {{1, 4, 3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 
                   3}, {0, 1, 0}, {1, 3, 3}, {1, 3, 3}}, {{1, 4, 3}, {0, 1, 
                   0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {0, 1, 
                   0}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}}}, {CompressedData["
1:eJxTTMoPSmViYGAQBWIQ/YB78sqmiXoOlxjuMXF2sTisvRFf5h+n59Dj9YrF
RJDFwUzqQLQCo56D3oQFPwyfMUH4B3Ud1qs+aZ53lsHhxfb1zM959OD8lpoL
myNN9Rwq5ixS3vmH0QEo26ORC+MzOSQuvGbyfoeew7F/m6o/bWB2mLcaZKO+
g5xFuktmPqvD1mWVxzWtDBy2XwQ5iNXhjf5udf5uAwdWE0Ezm71MDs/WgSwy
cFh9/mrYG30mhw+tJ0MOrjBwiM09CjSR0QFkS/sWA7h7+koLpXkfIPgvbq75
FXPWwCHx8GXtVEkmuHoJsIXMDm0g45YYOHR8uX29sZjZgZ8b6CJ3A4fvbPEz
fETZHKqW6jjLZOs7sIO4PWwO6OEHAOM+kNE=
                   "], CompressedData["
1:eJxTTMoPSmViYGCQBmIQ3XHp3gNuYX2HJyKyJ5/aSzgsZtzDKiSi65AbVrtu
2yMxB7VDbcvDuXQcOKXnxWmeFnSo+21VcI5D24HVRNDMZi+nQ+LCaybvd2g5
RDvZvbi5hs0hXSmnoipVy0HtSfO8s6uYHaIVHD8m82g5rFcFCTA4aElMvcKZ
geBv1MsDWqnlYLM3aJpiHzPUPm0HOYt0l8x8NofU708SF4ZpO1zRTpV8FMHp
EFyiMv1/gI7D9qT6m7aVghD3hug6HN+1o5etQMzh/wSgCw30HfbxzTFetEXc
4dOGgOxZ0w0cwM4XkHQ4Xe+xv3avocO0TTyFa7pFHfi5ty6rbDdyqFu3Lak+
Ushhho9oj5eXscOcRco7/7TzOSyrPK5p9cnYQazH6xWLCbuDLNhdJnD3r/kV
k3t0HoJfKM37QNfAxEEDbDCbwxI/B+HExcYOBrvVgSI8DidDDq5Ywmfs8OaA
pbLXbwEHq0lAF8kbOXzJAQa4magDSPuEAwYOINfdfiHpgB4/AHQArZ4=
                   
                   "], CompressedData["
1:eJxTTMoPSmViYGCQAWIQvTz8lNGRjXoO67Yl1d/cK+MwK3T+6rUROg5rum9n
MLyXcujReMu7z0Db4ULa1edZq0Ucchcz7mEV0nJ4xWIiaGbD62AzG6RB08HZ
7sXNNb84HDzmNqgdctN0MNitzs+9lcXht1XBuY5LGg7rVZ80zzvL4ACxThPO
V3D8mHxmrqbDs3UgARaHAE+gAUxaDqwg4/dyOIBU5yVrOajs/NP+5TaPA5C3
mLFG22HG3cntrVEiDpNWNgV6ztVxYPsg5hFQI+UgmuX7ue+ynkNokPqCTgMZ
B3ZboANf6ztUN+v7LOOSdQC5atlMAwe+OcaLtmyXcZC1SHfJ9Dd0WA/yv620
wyJlkEVGDh4BfySKr4s7TPnGFj9jjYlDxBt9oI/4HL7cvt5YfMwU7n4w/xkm
/2PryZCDKxgdmordpnxzM3XQSZV8FCHO6QA2vtrEIXKF6VnrcwIOQdMU+0o3
Gju4/Hr7+oCliEO9x/5a2R+GDixH+w3L10o7AAMhaFqjAcT9XXIO6PEFAPga
wsg=
                   "], CompressedData["
1:eJxTTMoPSmViYGAQBWIQPSt0/uq1N/QdLjHcY+Ls4nNoUDvUtjxcz8Fq0ul6
j/28DmtvxJf5x+k6bL8IUsDusJtVSMS+TcdhveqT5nlnGRycZF4/MruF4DcF
es5tWKbroAbirmJzCC5Rmf5/gp5D1XFNoJE8Dkt1nIE69B3ico/+21TN6/BA
d8KCH4UGDo8ixIFW8DnIWqS7ZPIbOrQAdZ+/yuVgszdommKfocPq81fD3uiz
OkQ52b24ucbQwd9BOPHwZWaH5HdAkROGDln5H1pPhjA5gK25bwh3Ty3IQE4j
OB9oyG7194YO7PEzfER7mBxWgcw9b+jwmsVE0MyGBWLfOkOHApBxB1nBtmir
wtzD7QAKldpaAwdGUHDJ8zughx8AsHaP1w==
                   "], CompressedData["
1:eJxTTMoPSmViYGCQBmIQvb9W1iI9xchhwoIfhs/WcTs42b24uWaXkYNI4uHL
2qkcDlO+scXP4DF2YI+f4SPaw+JgszdomqKfscN61SfN884yOHTJJ7+LUkLw
d6vzc299ZuSQM6t8ziJlFgeQcb/mGDmogaRXsTu0nQw5uMLFyMFXtMfrFQu3
Q7EbyAZDh9yw2nXbHgk6gKzzKTVwaGI52m/4XdjB4WPymdhufQeXX29fH7AU
dugLLlGZ/l/XYc9jEdmTqUION+LL/OXEdB3W/orJPfqPx8FPTizL11jHYR/Q
mX2lrA5ty8NPGbHoOPSVFkrzPmB2MJipJTH1irbDKaAzlvgxOeyPVnD8uFgb
7n4QN9kHwc+uqFqq06wNNY/JYeetrr+p27UdykDG6bI4AGWdZV5rO/wEhp7q
E1aHRYx7WIWu6ED9y+uw4JrJe4ujug4z7k5ub70l5HBna0vNhWJ9B4ni60Kf
Dgo7GIG0HTZwOGftd3HiHRGHTxsCsmddN3TQ+TYdqEPAAT1+AE3XvCM=
                   
                   "]}]}}}, {
              ImageSize -> 12, AspectRatio -> Automatic, 
               ImageSize -> {30., 30.}, PlotRange -> {{0., 30.}, {0., 30.}}}],
              "Wolfram Function Repository Reviewer"}, 
            "PrettyTooltipTemplate"]}}, 
         GridBoxAlignment -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Center}}}, AutoDelete -> 
         False, GridBoxItemSize -> {
          "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
         GridBoxSpacings -> {"Columns" -> {{0.25}}}], "Grid"]& )}], 
   Cell[
    StyleData["CommentCellLabelTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (StyleBox[
        TagBox[
         GridBox[{{
            StyleBox[#, FontSize -> 11]}, {
            StyleBox[
             DynamicBox[
              ToBoxes[
               DateString[
                TimeZoneConvert[
                 DateObject[#2, TimeZone -> 0]], {
                "Month", "/", "Day", "/", "Year", " ", "Hour24", ":", 
                 "Minute"}], StandardForm], SingleEvaluation -> True], 
             FontSize -> 9]}}, GridBoxAlignment -> {"Columns" -> {{Left}}}, 
          AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          GridBoxSpacings -> {"Columns" -> {{Automatic}}, "Rows" -> {{0}}}], 
         "Grid"], "CommentLabel", ShowStringCharacters -> False]& )}], 
   Cell[
    StyleData["OrangeButtonTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (ButtonBox[
        StyleBox[#, "Text", FontFamily -> "Source Sans Pro", FontWeight -> 
         "SemiBold", FontTracking -> "Condensed", FontSize -> 13, FontColor -> 
         Dynamic[
           FEPrivate`If[
            CurrentValue[Enabled], 
            GrayLevel[1], 
            RGBColor[0.8730375, 0.83578125, 0.7955875]], Evaluator -> 
           "System"], StripOnInput -> False], ButtonFunction :> 
        With[{RSNB`nb$ = ButtonNotebook[]}, 
          If[#4, CurrentValue[RSNB`nb$, {TaggingRules, "StatusMessage"}] = 
            ProgressIndicator[Appearance -> "Necklace"]]; 
          With[{RSNB`$cp$ = $ContextPath}, 
            Quiet[
             
             Block[{$ContextPath = RSNB`$cp$, 
               ResourceSystemClient`$AsyncronousResourceInformationUpdates = 
               False}, 
              Internal`WithLocalSettings[
               ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                 True; Once[
                  ReleaseHold[
                   CurrentValue[
                   RSNB`nb$, {TaggingRules, "CompatibilityTest"}]], 
                  "KernelSession"]; 
                Needs["ResourceSystemClient`DefinitionNotebook`"], 
                ResourceSystemClient`DefinitionNotebook`CheckForUpdates[
                RSNB`nb$, 
                 #2[]], 
                ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled = 
                 False; Null]; Null]]]; 
          CurrentValue[RSNB`nb$, {TaggingRules, "StatusMessage"}] = ""; Null],
         FrameMargins -> {{5, 5}, {0, 0}}, 
        Appearance -> {"Default" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQbG9wgaN1lY4QhCyIrAwo1R5n1BhlAEFANlwxXNmaCoem
KIPacD1kBBQBisOVba1zbos1QlMDQUBxoCzEtUAtjRhGQVAj2EAGMCBGGZGm
Ee82In1KfLgREwvExCkA9SYWEg==
            "], "Byte", ColorSpace -> "RGB", 
            Interleaving -> True], "Hover" -> Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQbG9wgaN1lY4QhCyIrAwo1Zds2ptsAkFANlwxXNmaCoee
JJP7c4J/r4+FICAbKAIUhyvbWufcn2L6eH4IXA0EAUWA4kBZiGuBWiammqGp
gSCgOFCWAQyIUUakacS7jUifEh9uxMQCMXEKAB4IRIc=
            "], "Byte", 
            ColorSpace -> "RGB", Interleaving -> True], "Pressed" -> 
          Image[CompressedData["
1:eJxTTMoPSmNiYGAo5gASQYnljkVFiZXBAkBOaF5xZnpeaopnXklqemqRRRIz
UJAXikHs/xgAqyAQbG9wgaNDUxIhCFkQWRlQ6vOz678/PIEgIBuuGK7s6Iz0
X++f/DnY+Ht9LAQB2UARoDhc2c4Wry/Pb/zZVw1XA1W5rxooDpSFuBZk1Nv7
aGogCCgOlGUAA2KUEWka8W4j0qfEhxsxsUBMnAIAq2R1ew==
            "], "Byte", 
            ColorSpace -> "RGB", Interleaving -> True]}, Background -> 
        RGBColor[0.7185389999999999, 0.5009625, 0.266231], Method -> "Queued",
         ImageSize -> {All, 23}, Enabled -> Dynamic[
          Not[
           TrueQ[ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled]], 
          TrackedSymbols :> {
           ResourceSystemClient`DefinitionNotebook`$ButtonsDisabled}], 
        Evaluator -> Automatic]& )}], 
   Cell[
    StyleData["SuggestionGridTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (StyleBox[
        FrameBox[
         AdjustmentBox[
          TagBox[
           GridBox[{{
              
              TemplateBox[{#2, #3, {16., 16.}, {{1., 17.}, {1., 17.}}}, 
               "SuggestionIconTemplate"], 
              
              PaneBox[#, ImageSizeAction -> "ShrinkToFit", BaselinePosition -> 
               Baseline, ImageSize -> Full], 
              AdjustmentBox[
               TemplateBox[{
                 ActionMenuBox[
                  TagBox[
                   PaneSelectorBox[{False -> GraphicsBox[{
                    EdgeForm[
                    Directive[
                    GrayLevel[1, 0], 
                    Thickness[0.025]]], 
                    FaceForm[#4], 
                    
                    RectangleBox[{-1.75, -2}, {1.75, 2}, RoundingRadius -> 
                    0.2], 
                    Thickness[0.15], #5, 
                    LineBox[{{-0.5, -1.}, {0.5, 0.}, {-0.5, 1.}}]}, 
                    ImageSize -> {Automatic, 15}, ImageMargins -> 0], True -> 
                    GraphicsBox[{
                    EdgeForm[
                    Directive[#5, 
                    Thickness[0.025]]], 
                    FaceForm[#2], 
                    
                    RectangleBox[{-1.75, -2}, {1.75, 2}, RoundingRadius -> 
                    0.2], 
                    Thickness[0.15], 
                    GrayLevel[1], 
                    LineBox[{{-0.5, -1.}, {0.5, 0.}, {-0.5, 1.}}]}, 
                    ImageSize -> {Automatic, 15}, ImageMargins -> 0]}, 
                    Dynamic[
                    CurrentValue["MouseOver"]], ImageSize -> Automatic, 
                    FrameMargins -> 0], 
                   MouseAppearanceTag["LinkHand"]], #6, Appearance -> None, 
                  Method -> "Queued"], "\"View suggestions\""}, 
                "PrettyTooltipTemplate"], BoxBaselineShift -> -0.3]}}, 
            GridBoxAlignment -> {
             "Columns" -> {{Left}}, "Rows" -> {{Baseline}}}, AutoDelete -> 
            False, GridBoxItemSize -> {
             "Columns" -> {Automatic, Automatic, Fit}, 
              "Rows" -> {{Automatic}}}, 
            GridBoxSpacings -> {"Columns" -> {{0.4}}}], "Grid"], 
          BoxMargins -> {{0.25, -1.}, {0.15, -0.15}}], 
         RoundingRadius -> {13, 75}, Background -> #4, FrameStyle -> None, 
         FrameMargins -> {{0, 8}, {0, 0}}, ImageMargins -> {{0, 0}, {5, 5}}, 
         StripOnInput -> False], "Text", FontColor -> #5, FontSize -> 14, 
        FontFamily -> "Source Sans Pro", FontWeight -> "SemiBold", 
        FontTracking -> "Plain", 
        PrivateFontOptions -> {"OperatorSubstitution" -> False}, 
        LineBreakWithin -> False]& )}], 
   Cell[
    StyleData["SuggestionIconTemplate"], 
    TemplateBoxOptions -> {DisplayFunction -> (GraphicsBox[{
         Thickness[0.05555555555555555], 
         StyleBox[{
           
           FilledCurveBox[{{{1, 4, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3,
             3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}}}, CompressedData["
1:eJxTTMoPSmVmYGBgBGJJIGZigIIGAwcIQ8kBxk94ekHp9k9Vh4qXaoYcOfoO
m+a+X37stKZDbP+hrxpzdOA0TBymDqYPl7n2pnG7PHlk4PzZRxQ2FGWIwPWD
jI3p54WbLxuVYn3fnwluD8S8H/Yo9gD5KPYA+TB7YPph9sDMh9EwcZg6FPdh
MRfdXpi7YPph7oaZD/MXzB5c4QCzBwA8nn+Z
            "]]}, 
          FaceForm[#]], 
         StyleBox[{
           
           FilledCurveBox[{{{0, 2, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1, 0}, {0, 1,
              0}, {0, 1, 0}}, {{1, 4, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}, {1,
              3, 3}, {1, 3, 3}, {1, 3, 3}, {1, 3, 3}}}, {{{8.175292500000001, 
             7.416875}, {7.796855000000001, 11.3084375}, {7.796855000000001, 
             13.38}, {10.11998, 13.38}, {10.11998, 11.3084375}, {
             9.741542500000001, 7.416875}, {8.175292500000001, 
             7.416875}}, CompressedData["
1:eJxTTMoPSmViYGCQBGIQ/cTvZcLf/4oOD6tE1rk/5HNQjDzAkqeL4FsusdsW
1KjgwAAGAg7hCSdehX2Xd5BvfR24Q07QwaZCOJPjjZyDHdf1xQW2Qg56LJYa
iWlyDv2HvmrEzBeG80GmVbmIwvkvtjT6Sb8Qg+t/BLLPUwJuPti6DEm4/WD7
2qTg7gMZJyIm7QBzP4y/zEVob88lJTi/7+dk7hV1ynD9c3LzfPxZVODmr3ro
0futUwVu/0bpbbqnzqjA3Qfjw9wP48P8B9MP8z/MfFj4wOyHhR/MfbDwRQ9/
ACBxmlc=
             "]}]}, 
          FaceForm[#2]]}, ImageSize -> #3, PlotRange -> #4, AspectRatio -> 
        Automatic, BaselinePosition -> Scaled[0.1]]& )}], 
   Cell[
    StyleData["UsageInputs", StyleDefinitions -> StyleData["Input"]], 
    CellMargins -> {{66, 10}, {0, 8}}, 
    StyleKeyMapping -> {"Tab" -> "UsageDescription"}, Evaluatable -> False, 
    CellEventActions -> {"ReturnKeyDown" :> With[{RSNB`nb = Notebooks[
           EvaluationCell[]]}, SelectionMove[
          EvaluationCell[], After, Cell]; NotebookWrite[RSNB`nb, 
          Cell["", "UsageDescription"], All]; 
        SelectionMove[RSNB`nb, Before, CellContents]], {"KeyDown", "\t"} :> 
      Replace[SelectionMove[
          SelectedNotebook[], After, Cell]; NotebookFind[
          SelectedNotebook[], "TabNext", Next, CellTags, AutoScroll -> True, 
          WrapAround -> True], Blank[NotebookSelection] :> SelectionMove[
          SelectedNotebook[], All, CellContents, AutoScroll -> True]]}, 
    ShowAutoStyles -> False, ShowCodeAssist -> False, 
    CodeAssistOptions -> {"DynamicHighlighting" -> False}, 
    LineSpacing -> {1, 3}, TabSpacings -> {2.5}, CounterIncrements -> "Text", 
    FontFamily -> "Source Sans Pro", FontSize -> 15, FontWeight -> "Plain"], 
   Cell[
    StyleData["UsageDescription", StyleDefinitions -> StyleData["Text"]], 
    CellMargins -> {{86, 10}, {7, 0}}, 
    StyleKeyMapping -> {"Backspace" -> "UsageInputs"}, CellGroupingRules -> 
    "OutputGrouping", 
    CellEventActions -> {"ReturnKeyDown" :> With[{RSNB`nb = Notebooks[
           EvaluationCell[]]}, SelectionMove[
          EvaluationCell[], After, Cell]; NotebookWrite[RSNB`nb, 
          Cell[
           BoxData[""], "UsageInputs", FontFamily -> "Source Sans Pro"], All]; 
        SelectionMove[RSNB`nb, Before, CellContents]], {"KeyDown", "\t"} :> 
      Replace[SelectionMove[
          SelectedNotebook[], After, Cell]; NotebookFind[
          SelectedNotebook[], "TabNext", Next, CellTags, AutoScroll -> True, 
          WrapAround -> True], Blank[NotebookSelection] :> SelectionMove[
          SelectedNotebook[], All, CellContents, AutoScroll -> True]]}, 
    ShowAutoSpellCheck -> False], 
   Cell[
    StyleData["Notes", StyleDefinitions -> StyleData["Item"]], CellDingbat -> 
    StyleBox["\[FilledVerySmallSquare]", FontColor -> GrayLevel[0.6]], 
    CellMargins -> {{66, 24}, {9, 7}}, StyleKeyMapping -> {}, 
    DefaultNewCellStyle -> "Notes", ShowAutoSpellCheck -> False, 
    GridBoxOptions -> {
     BaseStyle -> "TableNotes"}, $CellContext`ReturnCreatesNewCell -> False], 
   
   Cell[
    StyleData["TableNotes", StyleDefinitions -> StyleData["Notes"]], 
    CellDingbat -> None, CellFrameColor -> RGBColor[0.749, 0.694, 0.553], 
    StyleMenuListing -> None, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}, 
    GridBoxOptions -> {
     FrameStyle -> GrayLevel[0.906], 
      GridBoxAlignment -> {
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}}, 
      GridBoxDividers -> {"Columns" -> {{None}}, "Rows" -> {{True}}}, 
      GridDefaultElement -> Cell["\[Placeholder]", "TableText"]}], 
   Cell[
    StyleData["TableText"], DefaultInlineFormatType -> 
    "DefaultInputInlineFormatType", AutoQuoteCharacters -> {}, 
    StyleMenuListing -> None], 
   Cell[
    StyleData["Hyperlink"], FontColor -> 
    RGBColor[0.7185389999999999, 0.5009625, 0.266231]], 
   Cell[
    StyleData["HyperlinkActive"], FontColor -> 
    RGBColor[0.47902599999999995`, 0.333975, 0.17748733333333333`]], 
   Cell[
    StyleData["Link"], FontFamily -> "Source Sans Pro", FontColor -> Dynamic[
      If[
       CurrentValue["MouseOver"], 
       RGBColor[0.854902, 0.396078, 0.145098], 
       RGBColor[
       0.0196078431372549, 0.28627450980392155`, 0.6509803921568628]]]], 
   Cell[
    StyleData["StringTypeLink"], 
    TemplateBoxOptions -> {DisplayFunction :> (TagBox[
        DynamicBox[
         If[
          CurrentValue["MouseOver"], 
          ButtonBox[
           
           StyleBox[#, FontColor -> 
            RGBColor[0.9686274509803922, 0.4666666666666667, 0.]], 
           ButtonData -> #2], 
          ButtonBox[#, ButtonData -> #2]]], 
        MouseAppearanceTag["LinkHand"]]& )}, 
    ButtonBoxOptions -> {
     BaseStyle -> {FontColor -> GrayLevel[0.46666667], "Link"}}], 
   Cell[
    StyleData["InlineFormula"], 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    LanguageCategory -> "Formula", AutoSpacing -> True, ScriptLevel -> 1, 
    SingleLetterItalics -> False, SpanMaxSize -> 1, StyleMenuListing -> None, 
    FontFamily -> "Source Sans Pro", FontSize -> 1. Inherited, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}, 
    FractionBoxOptions -> {BaseStyle -> {SpanMaxSize -> Automatic}}, 
    GridBoxOptions -> {
     GridBoxItemSize -> {
       "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}}], 
   Cell[
    StyleData["ExampleDelimiter"], Selectable -> False, ShowCellBracket -> 
    Automatic, CellMargins -> {{66, 14}, {5, 10}}, Evaluatable -> True, 
    CellGroupingRules -> {"SectionGrouping", 58}, 
    CellEvaluationFunction -> (($Line = 0; Null)& ), ShowCellLabel -> False, 
    CellLabelAutoDelete -> True, TabFilling -> 
    "\[LongDash]\[NegativeThickSpace]", TabSpacings -> {100}, 
    StyleMenuListing -> None, FontFamily -> "Verdana", FontWeight -> Bold, 
    FontSlant -> "Plain", FontColor -> GrayLevel[0.905882]], 
   Cell[
    StyleData["ExampleText", StyleDefinitions -> StyleData["Text"]]], 
   Cell[
    StyleData["PageBreak", StyleDefinitions -> StyleData["ExampleDelimiter"]],
     Selectable -> False, CellFrame -> {{0, 0}, {1, 0}}, 
    CellMargins -> {{66, 14}, {15, -5}}, 
    CellElementSpacings -> {"CellMinHeight" -> 1}, Evaluatable -> True, 
    CellEvaluationFunction -> (($Line = 0; Null)& ), CellFrameColor -> 
    GrayLevel[
      Rational[77, 85]]], 
   Cell[
    StyleData["Subsection"], Evaluatable -> True, 
    CellEvaluationFunction -> (($Line = 0; Null)& ), ShowCellLabel -> False], 
   
   Cell[
    StyleData["Subsubsection"], Evaluatable -> True, 
    CellEvaluationFunction -> (($Line = 0; Null)& ), ShowCellLabel -> False], 
   
   Cell[
    StyleData["Excluded"], 
    CellBracketOptions -> {
     "Color" -> RGBColor[0.9, 0.4, 0.4], "Thickness" -> 2}, CellFrameMargins -> 
    4, StyleMenuListing -> None, Background -> 
    RGBColor[1, 0.95, 0.95], $CellContext`GeneratedCellStyles -> {
     "Graphics" -> {"Graphics", "Excluded"}, 
      "Message" -> {"Message", "MSG", "Excluded"}, 
      "Output" -> {"Output", "Excluded"}, "Print" -> {"Print", "Excluded"}, 
      "PrintTemporary" -> {"PrintTemporary", "Excluded"}}], 
   Cell[
    StyleData["Comment", StyleDefinitions -> StyleData["Text"]], 
    CellFrame -> {{3, 0}, {0, 0}}, CellMargins -> {{66, 0}, {1, 0}}, 
    CellElementSpacings -> {"ClosedCellHeight" -> 0}, CellFrameColor -> 
    RGBColor[0.880722, 0.611041, 0.142051], 
    CellFrameLabelMargins -> {{0, 10}, {0, 0}}, FontColor -> GrayLevel[0.25], 
    Background -> 
    RGBColor[0.9821083, 0.94165615, 
      0.87130765], $CellContext`GeneratedCellStyles -> {
     "Graphics" -> {"Graphics", "Comment"}, 
      "Message" -> {"Message", "MSG", "Comment"}, 
      "Output" -> {"Output", "Comment"}, "Print" -> {"Print", "Comment"}, 
      "PrintTemporary" -> {"PrintTemporary", "Comment"}}], 
   Cell[
    StyleData["AuthorComment", StyleDefinitions -> StyleData["Comment"]], 
    CellFrameColor -> RGBColor[0.368417, 0.506779, 0.709798], Background -> 
    RGBColor[0.90526255, 0.92601685, 
      0.9564697], $CellContext`GeneratedCellStyles -> {
     "Graphics" -> {"Graphics", "AuthorComment"}, 
      "Message" -> {"Message", "MSG", "AuthorComment"}, 
      "Output" -> {"Output", "AuthorComment"}, 
      "Print" -> {"Print", "AuthorComment"}, 
      "PrintTemporary" -> {"PrintTemporary", "AuthorComment"}}], 
   Cell[
    StyleData["ReviewerComment", StyleDefinitions -> StyleData["Comment"]], 
    CellFrameColor -> RGBColor[0.560181, 0.691569, 0.194885], Background -> 
    RGBColor[0.9340271499999999, 0.95373535, 
      0.87923275], $CellContext`GeneratedCellStyles -> {
     "Graphics" -> {"Graphics", "ReviewerComment"}, 
      "Message" -> {"Message", "MSG", "ReviewerComment"}, 
      "Output" -> {"Output", "ReviewerComment"}, 
      "Print" -> {"Print", "ReviewerComment"}, 
      "PrintTemporary" -> {"PrintTemporary", "ReviewerComment"}}], 
   Cell[
    StyleData["CommentLabel", StyleDefinitions -> StyleData["Text"]], 
    ShowStringCharacters -> False, FontSlant -> "Italic", FontColor -> 
    GrayLevel[0.5]], 
   Cell[
    StyleData["CharactersRefLink"], ShowSpecialCharacters -> False], 
   Cell[
    StyleData["DockedCell"], CellFrameColor -> GrayLevel[0.75], Background -> 
    RGBColor[0.898039, 0.898039, 0.898039]], 
   Cell[
    StyleData["ExampleImage"], PageWidth :> 650, 
    CellMargins -> {{66, 66}, {16, 5}}, Evaluatable -> False, ShowCellLabel -> 
    False, MenuSortingValue -> 10000, 
    RasterBoxOptions -> {ImageEditMode -> False}], 
   Cell[
    StyleData["Text"]]}, Visible -> False, FrontEndVersion -> 
  "13.0 for Mac OS X x86 (64-bit) (December 2, 2021)", StyleDefinitions -> 
  "PrivateStylesheetFormatting.nb"],
ExpressionUUID->"872fcd63-ec5d-4e35-8c88-9a59bc369ea2"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "b:5.0"->{
  Cell[4294, 104, 1516, 25, 152, "Text",ExpressionUUID->"5b410322-fc74-464c-8761-54bddc764685",
   CellTags->"b:5.0",
   CellID->1158596779],
  Cell[10294, 253, 916, 17, 30, "Text",ExpressionUUID->"72c04f85-fa84-45c7-a506-90fc65a88a9d",
   CellTags->"b:5.0",
   CellID->500701437],
  Cell[11213, 272, 1638, 34, 164, "Input",ExpressionUUID->"936059f3-43bf-4a5d-97c9-8572aa33f93f",
   CellTags->"b:5.0",
   CellID->1448122713],
  Cell[12876, 310, 1046, 19, 34, "Input",ExpressionUUID->"8c88cedf-4903-4600-b36e-e724e4a12286",
   CellTags->"b:5.0",
   CellID->304033761],
  Cell[13925, 331, 9068, 223, 166, "Output",ExpressionUUID->"1425609d-48c3-4418-9542-957084c5356a",
   CellTags->"b:5.0",
   CellID->1639644341],
  Cell[23008, 557, 1138, 20, 60, "Text",ExpressionUUID->"64289182-a051-4239-8c47-5ce1fc80eb14",
   CellTags->"b:5.0",
   CellID->1166265940],
  Cell[24171, 581, 900, 17, 35, "Input",ExpressionUUID->"803a85c8-a63a-4861-9475-de20a3f3bfe1",
   CellTags->"b:5.0",
   CellID->822164855],
  Cell[25074, 600, 7002, 166, 114, "Output",ExpressionUUID->"5c6b1d06-880d-4560-b19d-cb30dc826d1e",
   CellTags->"b:5.0",
   CellID->992280883],
  Cell[32113, 771, 877, 17, 35, "Input",ExpressionUUID->"6cf172ce-a5cf-4e94-bb77-fc49ffffa74e",
   CellTags->"b:5.0",
   CellID->1788765850],
  Cell[32993, 790, 3311, 84, 95, "Output",ExpressionUUID->"efcb9b81-4676-4cb9-b100-2ebdfcab90a0",
   CellTags->"b:5.0",
   CellID->2054281626],
  Cell[36341, 879, 827, 16, 35, "Input",ExpressionUUID->"cfa4bb5b-c11e-49c6-818e-ca1913a5fd20",
   CellTags->"b:5.0",
   CellID->513716746],
  Cell[37171, 897, 13251, 307, 136, "Output",ExpressionUUID->"99e833b5-c05d-4972-a18a-4f232eba6c8f",
   CellTags->"b:5.0",
   CellID->1216695363]},
 "b:5.1.0"->{
  Cell[57695, 1390, 2926, 68, 216, "Text",ExpressionUUID->"8705f09c-8d1a-48b2-8e2b-21c265ca1fad",
   CellTags->"b:5.1.0",
   CellID->766262050],
  Cell[64701, 1593, 944, 16, 56, "Text",ExpressionUUID->"791483d1-21fe-4d85-aa2b-fde78d025c98",
   CellTags->"b:5.1.0",
   CellID->721432069],
  Cell[65648, 1611, 1454, 34, 58, "Text",ExpressionUUID->"62dfc3d2-c862-43b9-8249-79d613148b49",
   CellTags->"b:5.1.0",
   CellID->1553898169],
  Cell[68241, 1681, 1204, 20, 56, "Text",ExpressionUUID->"c4048d76-ef73-45f4-9f7a-d70e3d200e85",
   CellTags->"b:5.1.0",
   CellID->797502331],
  Cell[74358, 1865, 1462, 23, 100, "Text",ExpressionUUID->"a9dfffb4-7bca-4bef-989b-ddf73abe43c3",
   CellTags->"b:5.1.0",
   CellID->1671184905],
  Cell[75845, 1892, 2629, 79, 84, "Input",ExpressionUUID->"1bab1f71-26b8-44fd-bea0-4f808dccef1d",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1176236190],
  Cell[78477, 1973, 266, 7, 34, "Output",ExpressionUUID->"f35f807e-03ec-48f0-a4e8-f2d8de1b6c7f",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1219525524],
  Cell[78746, 1982, 265, 7, 34, "Output",ExpressionUUID->"58d87256-090b-4173-b33b-b541daa036bf",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->981554900],
  Cell[82099, 2085, 855, 16, 34, "Text",ExpressionUUID->"d9456fa3-fece-4cb3-9b05-7d544938cf6a",
   CellTags->"b:5.1.0",
   CellID->1709645044],
  Cell[84782, 2162, 987, 18, 34, "Text",ExpressionUUID->"90e0606d-6b4e-449a-b633-544475cfc220",
   CellTags->"b:5.1.0",
   CellID->1862003300],
  Cell[95085, 2462, 1161, 20, 56, "Text",ExpressionUUID->"38bb8e14-24e9-499a-bdc1-1933384fd1e6",
   CellTags->"b:5.1.0",
   CellID->2954107],
  Cell[96249, 2484, 1502, 28, 78, "Text",ExpressionUUID->"061df848-1281-4ebf-89fe-eb1c11b2fa8d",
   CellTags->"b:5.1.0",
   CellID->2015713543],
  Cell[105252, 2755, 1660, 27, 122, "Text",ExpressionUUID->"40db12d6-90af-4aa6-bbd0-d25685c24a7f",
   CellTags->"b:5.1.0",
   CellID->1863427519],
  Cell[107282, 2795, 1798, 37, 123, "Text",ExpressionUUID->"512b636f-9a2a-4e6a-b7f7-c3ab4f67472e",
   CellTags->"b:5.1.0",
   CellID->1091194281],
  Cell[110267, 2872, 2766, 70, 59, "Text",ExpressionUUID->"8cc86ccb-2c28-4875-afe4-1f49df8e8af5",
   CellTags->"b:5.1.0",
   CellID->717347682],
  Cell[116022, 3020, 1499, 30, 40, "Text",ExpressionUUID->"68180cf7-ec6e-4c60-aae1-5f7267a7ac5b",
   CellTags->"b:5.1.0",
   CellID->731571697],
  Cell[119974, 3135, 2016, 46, 39, "Text",ExpressionUUID->"f07e6c48-11a2-41e5-9677-9303d6aa7dd6",
   CellTags->"b:5.1.0",
   CellID->1596268905],
  Cell[127407, 3354, 1321, 22, 34, "Text",ExpressionUUID->"10bd2765-324d-417f-9f15-04abf6b77beb",
   CellTags->"b:5.1.0",
   CellID->46958099],
  Cell[133890, 3562, 1575, 27, 84, "Text",ExpressionUUID->"b5d8fbde-d918-44d8-861c-fb4aec3e03c0",
   CellTags->"b:5.1.0",
   CellID->1165871760],
  Cell[143655, 3865, 1563, 27, 108, "Text",ExpressionUUID->"1b43ce23-81ec-42ff-aa98-8e3b6bfc0fc0",
   CellTags->"b:5.1.0",
   CellID->1111559247],
  Cell[148325, 3993, 759, 15, 36, "Text",ExpressionUUID->"0f34bf7c-5eb9-45df-9c63-bdcc36b3dab0",
   CellTags->"b:5.1.0",
   CellID->2039372162],
  Cell[152580, 4116, 2074, 40, 33, "Text",ExpressionUUID->"266b47db-0102-4aab-bcbd-2d5f7f52f5d1",
   CellTags->"b:5.1.0",
   CellID->1757173766],
  Cell[154679, 4160, 1052, 29, 85, "Input",ExpressionUUID->"d3e219f5-2e01-4e70-b9d9-d8b85396cec0",
   CellTags->"b:5.1.0",
   CellID->1388185552],
  Cell[155734, 4191, 646, 19, 49, "Output",ExpressionUUID->"16f64f7d-33f4-4bbc-80f7-fa33c5816d74",
   CellTags->"b:5.1.0",
   CellID->1053051397],
  Cell[156383, 4212, 646, 19, 49, "Output",ExpressionUUID->"fc0dda16-2515-43d1-96d0-9e97e0207348",
   CellTags->"b:5.1.0",
   CellID->1222094452],
  Cell[157032, 4233, 259, 7, 34, "Output",ExpressionUUID->"132d7cee-3cab-4aab-854c-e438748cae1b",
   CellTags->"b:5.1.0",
   CellID->1207854856],
  Cell[157306, 4243, 3772, 74, 43, "Text",ExpressionUUID->"1e0e917b-98c8-494a-82c8-612f219c9fac",
   CellTags->"b:5.1.0",
   CellID->256705393],
  Cell[172609, 4711, 2244, 42, 38, "Text",ExpressionUUID->"8370951a-626f-4804-8fe4-b67424a26141",
   CellTags->"b:5.1.0",
   CellID->1852374601],
  Cell[179250, 4884, 1943, 39, 37, "Text",ExpressionUUID->"76142ac1-212c-44d8-8824-5f1a350fa1dc",
   CellTags->"b:5.1.0",
   CellID->904569019],
  Cell[181196, 4925, 435, 13, 34, "Text",ExpressionUUID->"e3fef630-c316-443f-b674-b6195389c722",
   CellTags->"b:5.1.0",
   CellID->1054765220],
  Cell[181656, 4942, 1970, 66, 60, "Input",ExpressionUUID->"87d3bcc7-7a17-4e6d-8fd9-d088220a7b82",
   CellTags->"b:5.1.0",
   CellID->2109030202],
  Cell[183629, 5010, 257, 7, 34, "Output",ExpressionUUID->"e8a37290-a701-4057-8619-769f721640e5",
   CellTags->"b:5.1.0",
   CellID->1069939807],
  Cell[183889, 5019, 258, 7, 34, "Output",ExpressionUUID->"e0710e2b-9a81-4155-8a7d-50add735e77d",
   CellTags->"b:5.1.0",
   CellID->1597759918],
  Cell[184162, 5029, 725, 14, 34, "Text",ExpressionUUID->"de65a51e-ecbc-4710-912a-cffd1b489ae8",
   CellTags->"b:5.1.0",
   CellID->811402856],
  Cell[184912, 5047, 406, 13, 72, "Input",ExpressionUUID->"933d7000-d66e-43ad-afc6-168921c12a96",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->726642342],
  Cell[185321, 5062, 648, 19, 49, "Output",ExpressionUUID->"13b4580b-2d05-4c74-9d97-8a60cb49771e",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1415419738],
  Cell[185972, 5083, 641, 18, 34, "Output",ExpressionUUID->"26090d83-bfc7-4447-88f5-42c1cd7b6ff3",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1283178747],
  Cell[186628, 5104, 1145, 27, 39, "Text",ExpressionUUID->"ab50856d-5bb3-4032-9655-8eeb47840a9f",
   CellTags->"b:5.1.0",
   CellID->2034662426],
  Cell[197008, 5397, 674, 13, 34, "Text",ExpressionUUID->"14299fed-db8f-4335-9418-dd68c718829a",
   CellTags->"b:5.1.0",
   CellID->1616577872],
  Cell[199839, 5473, 755, 14, 34, "Text",ExpressionUUID->"67c30f82-85f7-421b-9c83-a9cbd6c65bd3",
   CellTags->"b:5.1.0",
   CellID->402056739],
  Cell[222623, 6035, 1973, 36, 162, "Text",ExpressionUUID->"65b7138e-2c0b-4209-b047-79430580396f",
   CellTags->"b:5.1.0",
   CellID->224595378],
  Cell[236235, 6367, 3570, 78, 217, "Text",ExpressionUUID->"a1c0bc78-1e48-43fc-8295-db6eabc1c8e6",
   CellTags->"b:5.1.0",
   CellID->283093329],
  Cell[239808, 6447, 4345, 112, 179, "Text",ExpressionUUID->"2004a039-55f4-425d-911d-96f92cf3571d",
   CellTags->"b:5.1.0",
   CellID->1372227264],
  Cell[244156, 6561, 1610, 38, 92, "Text",ExpressionUUID->"64a68ffc-4eb0-4b19-a155-5fa55b7ee947",
   CellTags->"b:5.1.0",
   CellID->509110344],
  Cell[245769, 6601, 2391, 70, 87, "Text",ExpressionUUID->"dc617e94-8ac5-42b4-b8fa-05b4ea0436ca",
   CellTags->"b:5.1.0",
   CellID->1067632665],
  Cell[250758, 6760, 870, 16, 34, "Text",ExpressionUUID->"e081223b-7099-4505-8d4e-be90b0429133",
   CellTags->"b:5.1.0",
   CellID->582374062],
  Cell[253914, 6854, 3227, 69, 303, "Text",ExpressionUUID->"44604929-f489-467e-b2ba-ef9aebb2b6bc",
   CellTags->"b:5.1.0",
   CellID->1134805856],
  Cell[257449, 6936, 1770, 43, 62, "Text",ExpressionUUID->"2b576ef2-4c46-401d-a02b-b0f9243b7292",
   CellTags->"b:5.1.0",
   CellID->776438691],
  Cell[264263, 7133, 779, 18, 37, "Text",ExpressionUUID->"99414cb7-98f5-4b81-a1c0-bb87486fcf32",
   CellTags->"b:5.1.0",
   CellID->92159943],
  Cell[279156, 7526, 948, 24, 78, "Text",ExpressionUUID->"b25e9833-ca2c-4a81-9bc7-c9fe257a6b66",
   CellTags->"b:5.1.0",
   CellID->1312746903],
  Cell[280129, 7554, 1294, 30, 163, "Input",ExpressionUUID->"be08d6a9-3be6-4b72-b4b0-234f8ba73d14",
   CellTags->"b:5.1.0",
   CellID->90209443],
  Cell[281426, 7586, 257, 7, 34, "Output",ExpressionUUID->"b0b420f0-f43c-4866-956c-0a4e159e6679",
   CellTags->"b:5.1.0",
   CellID->678294822],
  Cell[281686, 7595, 257, 7, 34, "Output",ExpressionUUID->"b998e457-8460-4fa1-8b26-eb171a0d7aa0",
   CellTags->"b:5.1.0",
   CellID->1257875078],
  Cell[281946, 7604, 257, 7, 34, "Output",ExpressionUUID->"345ce0fb-cd42-41d0-a1d7-476536215fc3",
   CellTags->"b:5.1.0",
   CellID->1277414878],
  Cell[308324, 8273, 670, 12, 56, "Text",ExpressionUUID->"42d52334-ebd8-42b2-8101-32588eb65786",
   CellTags->"b:5.1.0",
   CellID->1492245405],
  Cell[308997, 8287, 712, 18, 34, "Text",ExpressionUUID->"ca61ec89-fda4-4d23-bf24-1cfa5bf6831e",
   CellTags->"b:5.1.0",
   CellID->1292456657],
  Cell[309734, 8309, 522, 13, 110, "Input",ExpressionUUID->"72145571-a2e5-4c72-8ab8-a9b4f6e60298",
   CellTags->"b:5.1.0",
   CellID->521944794],
  Cell[310259, 8324, 720, 23, 69, "Output",ExpressionUUID->"1ef457bb-7798-4985-80af-d1e2651c9d90",
   CellTags->"b:5.1.0",
   CellID->1248253175],
  Cell[310982, 8349, 490, 14, 54, "Output",ExpressionUUID->"8fea3d15-22cb-4c10-b27c-0605993d8af1",
   CellTags->"b:5.1.0",
   CellID->623364682],
  Cell[311475, 8365, 746, 24, 69, "Output",ExpressionUUID->"e098b170-9bf4-4b46-b9da-bb9e1e5ebea7",
   CellTags->"b:5.1.0",
   CellID->1464980308],
  Cell[312224, 8391, 517, 15, 54, "Output",ExpressionUUID->"7eca5b89-614c-4c9c-8c80-39a9a1ae5271",
   CellTags->"b:5.1.0",
   CellID->1024023701],
  Cell[312778, 8411, 581, 18, 60, "Input",ExpressionUUID->"846f7911-b786-43b9-8805-61d75eb1c43d",
   CellTags->"b:5.1.0",
   CellID->2002938410],
  Cell[313362, 8431, 258, 7, 34, "Output",ExpressionUUID->"82ef3be0-edf6-4203-8b32-035047b1925a",
   CellTags->"b:5.1.0",
   CellID->832395649],
  Cell[313623, 8440, 259, 7, 34, "Output",ExpressionUUID->"991ce154-3a68-48a8-9759-a60280aacbad",
   CellTags->"b:5.1.0",
   CellID->1365196185],
  Cell[313897, 8450, 755, 18, 78, "Text",ExpressionUUID->"603f6fcc-c0ce-4ed3-ba9e-d265642f0371",
   CellTags->"b:5.1.0",
   CellID->1579690145],
  Cell[314677, 8472, 874, 26, 116, "Input",ExpressionUUID->"db686234-811d-466c-abc7-dd32106ff803",
   CellTags->"b:5.1.0",
   CellID->1341871396],
  Cell[315554, 8500, 629, 18, 35, "Output",ExpressionUUID->"b704a28d-c954-4f84-81f8-13099517917e",
   CellTags->"b:5.1.0",
   CellID->1136996747],
  Cell[316186, 8520, 429, 11, 34, "Output",ExpressionUUID->"b66c26b2-4f1e-4b93-9b01-54888bd75db8",
   CellTags->"b:5.1.0",
   CellID->1194835823],
  Cell[316618, 8533, 628, 18, 35, "Output",ExpressionUUID->"c5539882-b2a6-49e2-b585-d902a2fdf678",
   CellTags->"b:5.1.0",
   CellID->486094064],
  Cell[317249, 8553, 428, 11, 34, "Output",ExpressionUUID->"c2b4cd6e-3ca4-462c-bdcc-c06fc8afd577",
   CellTags->"b:5.1.0",
   CellID->755140460],
  Cell[317692, 8567, 420, 9, 34, "Text",ExpressionUUID->"0ac79b5a-fe60-404f-8003-29642a9d8a38",
   CellTags->"b:5.1.0",
   CellID->1114284212],
  Cell[318137, 8580, 385, 11, 60, "Input",ExpressionUUID->"eca7385c-53b2-4aa6-978f-5714cee4f78e",
   CellTags->"b:5.1.0",
   CellID->1717349244],
  Cell[318525, 8593, 739, 23, 43, "Output",ExpressionUUID->"54c9c7ae-00ad-4532-8ab6-59a6f0f13599",
   CellTags->"b:5.1.0",
   CellID->17357450],
  Cell[319267, 8618, 741, 23, 43, "Output",ExpressionUUID->"28ac27d1-7873-4f1f-be5c-f034f35a000e",
   CellTags->"b:5.1.0",
   CellID->1816369805],
  Cell[320536, 8656, 708, 14, 52, "Text",ExpressionUUID->"6ecce6d3-fd32-495c-aeb8-a60ec79d9c7c",
   CellTags->"b:5.1.0",
   CellID->1140252415],
  Cell[328411, 8909, 505, 10, 34, "Text",ExpressionUUID->"5bece0b1-a9db-4a26-8c9b-ed3ec189b3b0",
   CellTags->"b:5.1.0",
   CellID->1704654754],
  Cell[331428, 9000, 583, 12, 34, "Text",ExpressionUUID->"dc2ff4fd-4d6e-445b-b23d-2f6465d9e6ae",
   CellTags->"b:5.1.0",
   CellID->1200399909]},
 "i:4"->{
  Cell[75845, 1892, 2629, 79, 84, "Input",ExpressionUUID->"1bab1f71-26b8-44fd-bea0-4f808dccef1d",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1176236190],
  Cell[78477, 1973, 266, 7, 34, "Output",ExpressionUUID->"f35f807e-03ec-48f0-a4e8-f2d8de1b6c7f",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1219525524],
  Cell[78746, 1982, 265, 7, 34, "Output",ExpressionUUID->"58d87256-090b-4173-b33b-b541daa036bf",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->981554900],
  Cell[184912, 5047, 406, 13, 72, "Input",ExpressionUUID->"933d7000-d66e-43ad-afc6-168921c12a96",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->726642342],
  Cell[185321, 5062, 648, 19, 49, "Output",ExpressionUUID->"13b4580b-2d05-4c74-9d97-8a60cb49771e",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1415419738],
  Cell[185972, 5083, 641, 18, 34, "Output",ExpressionUUID->"26090d83-bfc7-4447-88f5-42c1cd7b6ff3",
   CellTags->{"i:4", "b:5.1.0"},
   CellID->1283178747]},
 "b:5.1.1"->{
  Cell[191384, 5258, 1398, 26, 39, "Text",ExpressionUUID->"5b4ae767-c154-49a6-bbc2-60e7c504a48b",
   CellTags->"b:5.1.1",
   CellID->415603317],
  Cell[338691, 9228, 1114, 18, 74, "Text",ExpressionUUID->"79d1d607-9794-4f8d-a426-8f73873942ce",
   CellTags->"b:5.1.1",
   CellID->1128508301],
  Cell[340495, 9270, 1538, 41, 83, "Text",ExpressionUUID->"1f087290-0716-406e-9674-68b89ecd0124",
   CellTags->"b:5.1.1",
   CellID->53899069],
  Cell[345804, 9428, 1222, 23, 37, "Text",ExpressionUUID->"d0b16d95-259b-4e8b-ae94-8f5abaea07e3",
   CellTags->"b:5.1.1",
   CellID->1821072405],
  Cell[363557, 9936, 4515, 126, 231, "Text",ExpressionUUID->"669fcedb-6a9f-4c29-9912-3dd3e75bc1f2",
   CellTags->"b:5.1.1",
   CellID->852049279],
  Cell[406343, 11092, 896, 29, 59, "Text",ExpressionUUID->"bb86afa1-e55f-40ec-9795-f7e7a4655594",
   CellTags->"b:5.1.1",
   CellID->230642097],
  Cell[412376, 11282, 793, 25, 37, "Text",ExpressionUUID->"072576ad-af65-44a4-b78f-f3a8bbb650b0",
   CellTags->"b:5.1.1",
   CellID->627240456],
  Cell[416137, 11394, 750, 14, 36, "Text",ExpressionUUID->"18f0fdb2-f656-4b47-8a8a-0e6ca39e7aed",
   CellTags->"b:5.1.1",
   CellID->464180710],
  Cell[423006, 11588, 467, 10, 34, "Text",ExpressionUUID->"1d49f15a-8550-4882-b664-1f3059be1691",
   CellTags->"b:5.1.1",
   CellID->1915822749],
  Cell[423498, 11602, 1338, 42, 34, "Input",ExpressionUUID->"0f089c63-fb46-4199-ae96-f6598c8f9b56",
   CellTags->"b:5.1.1",
   CellID->2010622972],
  Cell[424839, 11646, 988, 25, 151, "Output",ExpressionUUID->"4bfe338b-774f-4e05-b793-754b2b81d531",
   CellTags->"b:5.1.1",
   CellID->1243365523],
  Cell[425842, 11674, 559, 11, 56, "Text",ExpressionUUID->"41edd0f4-a913-4fe5-aa0e-06ee74a8aeaa",
   CellTags->"b:5.1.1",
   CellID->1885104859],
  Cell[426426, 11689, 950, 20, 60, "Input",ExpressionUUID->"d36ec6b6-bb31-48b7-9d11-688ad68ba4cb",
   CellTags->"b:5.1.1",
   CellID->1131121022],
  Cell[427379, 11711, 517, 15, 54, "Output",ExpressionUUID->"54879a71-f955-493a-814d-081ab1d2821d",
   CellTags->"b:5.1.1",
   CellID->80976104],
  Cell[427911, 11729, 369, 8, 34, "Text",ExpressionUUID->"66750b66-a0ea-42a0-a942-3da790a6d238",
   CellTags->"b:5.1.1",
   CellID->1225773510],
  Cell[429427, 11784, 760, 17, 35, "Input",ExpressionUUID->"6bc042b9-4294-4e99-9d85-cfe50a73743e",
   CellTags->"b:5.1.1",
   CellID->764756899],
  Cell[430190, 11803, 1133, 31, 167, "Output",ExpressionUUID->"3c7d3527-7238-4216-a2e5-e7167e15219e",
   CellTags->"b:5.1.1",
   CellID->1315572097],
  Cell[431338, 11837, 328, 11, 34, "Text",ExpressionUUID->"5140c94c-a5e4-4b80-98c5-5545d90b2af3",
   CellTags->"b:5.1.1",
   CellID->579574198],
  Cell[431691, 11852, 456, 12, 61, "Input",ExpressionUUID->"95ee239b-56e3-4ff0-8dc3-d9e7b7f0ff9f",
   CellTags->"b:5.1.1",
   CellID->2065206641],
  Cell[432150, 11866, 252, 7, 34, "Output",ExpressionUUID->"206067c9-9433-44f3-8595-f681738bab9c",
   CellTags->"b:5.1.1",
   CellID->328604767],
  Cell[432405, 11875, 254, 7, 34, "Output",ExpressionUUID->"ccef0447-e553-49a1-8819-e7eb8f971166",
   CellTags->"b:5.1.1",
   CellID->1679862532],
  Cell[432674, 11885, 413, 9, 34, "Text",ExpressionUUID->"dc027092-4b3f-4f7f-abcf-b8212448ff1e",
   CellTags->"b:5.1.1",
   CellID->149828826],
  Cell[433112, 11898, 868, 20, 60, "Input",ExpressionUUID->"e62624c5-daf6-4ba5-a42f-996bef153155",
   CellTags->"b:5.1.1",
   CellID->1322244298],
  Cell[433983, 11920, 352, 10, 34, "Output",ExpressionUUID->"efbde6d1-d46c-48b5-91cf-bccd6b7b3f8a",
   CellTags->"b:5.1.1",
   CellID->482068215],
  Cell[434598, 11941, 1126, 30, 60, "Input",ExpressionUUID->"c18fab12-935e-4306-a5d3-e86042464a76",
   CellTags->"b:5.1.1",
   CellID->799137330],
  Cell[435727, 11973, 516, 12, 34, "Output",ExpressionUUID->"c5024fca-7fe2-4b2d-abae-f58e47c05374",
   CellTags->"b:5.1.1",
   CellID->1812173021],
  Cell[436258, 11988, 503, 11, 34, "Text",ExpressionUUID->"f83f7190-02f0-451d-9153-b4bbf527b910",
   CellTags->"b:5.1.1",
   CellID->738376972],
  Cell[436786, 12003, 1352, 31, 85, "Input",ExpressionUUID->"8badbd6e-ef53-4816-8a40-46afa00fbb4c",
   CellTags->"b:5.1.1",
   CellID->1741256038],
  Cell[438141, 12036, 257, 7, 34, "Output",ExpressionUUID->"c5f09a22-6c7d-47d4-949e-7ad6acad5b85",
   CellTags->"b:5.1.1",
   CellID->1578882193],
  Cell[438401, 12045, 259, 7, 34, "Output",ExpressionUUID->"115996dd-f693-4b54-8514-abcb61d99248",
   CellTags->"b:5.1.1",
   CellID->1965075419],
  Cell[438663, 12054, 661, 21, 83, "Output",ExpressionUUID->"ed8e1f7b-ae2e-404f-a561-5fc44ae3ff68",
   CellTags->"b:5.1.1",
   CellID->871559920],
  Cell[445824, 12299, 2040, 66, 116, "Text",ExpressionUUID->"350b9b6b-8dd3-4693-a7cc-3016ffa4315e",
   CellTags->"b:5.1.1",
   CellID->1312550803],
  Cell[447867, 12367, 441, 10, 34, "Text",ExpressionUUID->"0924ebea-1d92-47ec-ae10-f63872bd02e1",
   CellTags->"b:5.1.1",
   CellID->422389411],
  Cell[466748, 12889, 422, 10, 34, "Text",ExpressionUUID->"8247f088-0ba6-48d1-bbbf-138ef69f01af",
   CellTags->"b:5.1.1",
   CellID->147551722],
  Cell[470323, 12998, 488, 11, 34, "Text",ExpressionUUID->"0a0691fe-ad9b-4811-9815-d39febd8453d",
   CellTags->"b:5.1.1",
   CellID->1203011483]},
 "b:5.1.2"->{
  Cell[474631, 13126, 447, 10, 30, "Text",ExpressionUUID->"4ceb7ec0-d33c-4955-b037-3d31a97f8fe0",
   CellTags->"b:5.1.2",
   CellID->1573400686],
  Cell[475081, 13138, 1420, 45, 154, "Text",ExpressionUUID->"b2025513-d84e-43f0-9ba2-c0a0f51efb2b",
   CellTags->"b:5.1.2",
   CellID->31700444],
  Cell[476504, 13185, 411, 9, 34, "Text",ExpressionUUID->"a20296b5-b24f-4643-a6e0-bc4518f6d0c8",
   CellTags->"b:5.1.2",
   CellID->213417852],
  Cell[515148, 14158, 530, 15, 34, "Text",ExpressionUUID->"586bbec2-4eea-4339-b199-179e89f9bd21",
   CellTags->"b:5.1.2",
   CellID->2092978741],
  Cell[519376, 14282, 551, 11, 56, "Text",ExpressionUUID->"90f4dc09-7246-4ae1-93c2-2c7a86e9d0b6",
   CellTags->"b:5.1.2",
   CellID->1912508314],
  Cell[554835, 15052, 570, 11, 56, "Text",ExpressionUUID->"c2163940-5b4a-416a-b7bd-e1d941569499",
   CellTags->"b:5.1.2",
   CellID->2020051968],
  Cell[555408, 15065, 488, 10, 36, "Text",ExpressionUUID->"0f36b87d-6a31-4caa-9dfe-915a9b6f1daa",
   CellTags->"b:5.1.2",
   CellID->1444450753],
  Cell[560420, 15219, 557, 12, 36, "Text",ExpressionUUID->"3d7e8883-1d90-4a2e-9c6d-a2f75115bdd9",
   CellTags->"b:5.1.2",
   CellID->2031050894],
  Cell[574637, 15640, 511, 11, 52, "Text",ExpressionUUID->"033c8045-1fa5-4c46-a41c-f1fdbaa0d5d0",
   CellTags->"b:5.1.2",
   CellID->902749858],
  Cell[579439, 15786, 675, 20, 61, "Input",ExpressionUUID->"6e3c251c-2ba8-4b44-a379-a3e5e8101f7b",
   CellTags->"b:5.1.2",
   CellID->1219507449],
  Cell[589038, 16058, 301, 7, 36, "Text",ExpressionUUID->"721cafa1-d38e-4a90-bf2a-75998990bd06",
   CellTags->"b:5.1.2",
   CellID->677768375],
  Cell[590737, 16110, 533, 11, 34, "Text",ExpressionUUID->"d1350ba3-91b3-4902-944c-5083a05378cf",
   CellTags->"b:5.1.2",
   CellID->1977592031],
  Cell[593478, 16186, 303, 9, 36, "Text",ExpressionUUID->"f8bab412-229a-4874-9747-da15022d46be",
   CellTags->"b:5.1.2",
   CellID->1304149312],
  Cell[593806, 16199, 1649, 48, 176, "Input",ExpressionUUID->"b07206fb-b895-4bbd-8453-214c35bab236",
   CellTags->"b:5.1.2",
   CellID->1619385502],
  Cell[595458, 16249, 794, 29, 171, "Output",ExpressionUUID->"0b0de7c8-edc7-4bce-92bb-bc026e7a75b9",
   CellTags->"b:5.1.2",
   CellID->713034620],
  Cell[596255, 16280, 496, 14, 54, "Output",ExpressionUUID->"bfb5d2f2-01ea-439b-b9da-f0714fa262ff",
   CellTags->"b:5.1.2",
   CellID->1014108080],
  Cell[596754, 16296, 258, 7, 34, "Output",ExpressionUUID->"b5284a9b-acb3-4679-bea5-5509770ca6b4",
   CellTags->"b:5.1.2",
   CellID->1684277968],
  Cell[597027, 16306, 417, 9, 36, "Text",ExpressionUUID->"3dd8387d-d3b5-4294-bb31-fca365b6fb0b",
   CellTags->"b:5.1.2",
   CellID->533165535],
  Cell[601252, 16437, 1423, 34, 64, "Text",ExpressionUUID->"20803f82-634d-4746-aced-32444db20b1b",
   CellTags->"b:5.1.2",
   CellID->776787928],
  Cell[614950, 16861, 415, 9, 36, "Text",ExpressionUUID->"7afe57c2-2263-4464-be38-77a4fbf81fab",
   CellTags->"b:5.1.2",
   CellID->1951873683]},
 "b:5.1.3"->{
  Cell[622764, 17107, 684, 15, 81, "Text",ExpressionUUID->"8da6c954-81d1-4f7c-8a2d-bb93e4a1b939",
   CellTags->"b:5.1.3",
   CellID->442078480],
  Cell[623473, 17126, 2514, 77, 215, "Input",ExpressionUUID->"d4d8d20a-1e3f-413b-b85f-aa6e7dff1774",
   CellTags->"b:5.1.3",
   CellID->1867078387],
  Cell[625990, 17205, 2117, 49, 287, "Output",ExpressionUUID->"52624d07-f318-4354-82f1-6c2ef6cdc781",
   CellTags->"b:5.1.3",
   CellID->2113844587],
  Cell[628110, 17256, 647, 19, 49, "Output",ExpressionUUID->"e6ebfea7-49a0-4f9d-90e2-fa6b1cd53162",
   CellTags->"b:5.1.3",
   CellID->1564001388],
  Cell[628760, 17277, 720, 21, 83, "Output",ExpressionUUID->"7421b217-191c-4bdb-a5f0-803cc6ee4c23",
   CellTags->"b:5.1.3",
   CellID->971488836],
  Cell[629483, 17300, 2793, 83, 171, "Output",ExpressionUUID->"e8280fcc-fc7b-45e8-a4f0-e1eea23fb5db",
   CellTags->"b:5.1.3",
   CellID->494698511],
  Cell[632279, 17385, 721, 21, 83, "Output",ExpressionUUID->"0231a761-c77f-4f65-9a03-681557d99e56",
   CellTags->"b:5.1.3",
   CellID->1488676840],
  Cell[633003, 17408, 2793, 83, 171, "Output",ExpressionUUID->"0f0e7297-493e-41fb-810d-40dba7d73559",
   CellTags->"b:5.1.3",
   CellID->2007162330],
  Cell[635811, 17494, 310, 9, 36, "Text",ExpressionUUID->"b29baa0e-fe1c-48bd-88e0-374783fa36ab",
   CellTags->"b:5.1.3",
   CellID->955400345],
  Cell[636146, 17507, 651, 17, 139, "Input",ExpressionUUID->"77c15bce-2531-4060-9197-2399d7284025",
   CellTags->"b:5.1.3",
   CellID->678369796],
  Cell[636800, 17526, 258, 7, 34, "Output",ExpressionUUID->"d709897f-1571-436d-acf3-810a4d442fbd",
   CellTags->"b:5.1.3",
   CellID->1704153234],
  Cell[637061, 17535, 256, 7, 34, "Output",ExpressionUUID->"ad6925ed-128c-47a3-8994-7714ff181a17",
   CellTags->"b:5.1.3",
   CellID->714003799],
  Cell[637320, 17544, 254, 7, 34, "Output",ExpressionUUID->"8f1b77b0-072d-47f6-b870-4c42917c1a98",
   CellTags->"b:5.1.3",
   CellID->123230357],
  Cell[637577, 17553, 254, 7, 34, "Output",ExpressionUUID->"4eda8a4a-3c8c-46f4-b912-a57caff2ab02",
   CellTags->"b:5.1.3",
   CellID->958374391],
  Cell[637834, 17562, 254, 7, 34, "Output",ExpressionUUID->"ffef8353-cebd-4bc5-931c-9efb271e5c90",
   CellTags->"b:5.1.3",
   CellID->1271037037],
  Cell[638103, 17572, 290, 9, 36, "Text",ExpressionUUID->"70d28756-e093-4be9-b8f1-b392682f057c",
   CellTags->"b:5.1.3",
   CellID->370479449],
  Cell[638418, 17585, 2159, 69, 34, "Input",ExpressionUUID->"460546ec-f266-49dc-9e00-7e552c36a036",
   CellTags->"b:5.1.3",
   CellID->1093006690],
  Cell[640580, 17656, 1064, 28, 175, "Output",ExpressionUUID->"b2f838b1-8cb1-45fd-8c02-46127f757a1a",
   CellTags->"b:5.1.3",
   CellID->1299644150],
  Cell[641681, 17689, 335, 9, 60, "Input",ExpressionUUID->"4e5ef63c-59d4-454f-af2e-982bd035eea4",
   CellTags->"b:5.1.3",
   CellID->588322392],
  Cell[642019, 17700, 255, 7, 34, "Output",ExpressionUUID->"4ebcc279-60ab-4540-974a-6d945f01c779",
   CellTags->"b:5.1.3",
   CellID->1063055413],
  Cell[642277, 17709, 276, 8, 50, "Output",ExpressionUUID->"ea572a49-e594-47a2-b8da-50234f88eb30",
   CellTags->"b:5.1.3",
   CellID->1855866898],
  Cell[642568, 17720, 497, 11, 60, "Text",ExpressionUUID->"6fc7e343-f33d-4c7a-9824-bcb0fb7d441d",
   CellTags->"b:5.1.3",
   CellID->919731556],
  Cell[643090, 17735, 343, 11, 34, "Input",ExpressionUUID->"c6d7e4cf-3956-4b95-9d40-31cdf055eb8b",
   CellTags->"b:5.1.3",
   CellID->340970586],
  Cell[643436, 17748, 697, 21, 65, "Output",ExpressionUUID->"b0e49e6d-4a47-4e54-a7f3-abc7fbc186ef",
   CellTags->"b:5.1.3",
   CellID->1502465658],
  Cell[644170, 17774, 344, 11, 34, "Input",ExpressionUUID->"ca9d29d8-68be-49a9-9834-9d318c4a5059",
   CellTags->"b:5.1.3",
   CellID->1206268706],
  Cell[644517, 17787, 696, 21, 65, "Output",ExpressionUUID->"c3acaadc-b9c5-4088-b0f1-f2785bcafc6e",
   CellTags->"b:5.1.3",
   CellID->1827592580],
  Cell[645250, 17813, 344, 11, 34, "Input",ExpressionUUID->"712643d0-bc1d-482b-a119-3420db6664e8",
   CellTags->"b:5.1.3",
   CellID->1512514062],
  Cell[645597, 17826, 646, 19, 49, "Output",ExpressionUUID->"252ba9f4-5bd9-4f1b-877b-8b07ded432a5",
   CellTags->"b:5.1.3",
   CellID->889889019]},
 "b:5.1.4"->{
  Cell[649541, 17950, 549, 14, 60, "Text",ExpressionUUID->"5c768566-a768-4ae1-a789-8924ed79d262",
   CellTags->"b:5.1.4",
   CellID->985654226],
  Cell[650115, 17968, 5969, 166, 193, "Input",ExpressionUUID->"7a100f1a-ce09-4175-a1b4-98d88c234dc6",
   CellTags->"b:5.1.4",
   CellID->201723424],
  Cell[656087, 18136, 7684, 217, 115, "Output",ExpressionUUID->"81d3bf94-4238-4b4c-a5f5-d5b8277049eb",
   CellTags->"b:5.1.4",
   CellID->541492380],
  Cell[663774, 18355, 258, 7, 34, "Output",ExpressionUUID->"b846603e-6e92-4680-8ec9-361a9bb04abf",
   CellTags->"b:5.1.4",
   CellID->1974218321],
  Cell[664035, 18364, 3696, 105, 179, "Output",ExpressionUUID->"0f3d3d2e-3118-4e2e-96f9-41fe349c1bd2",
   CellTags->"b:5.1.4",
   CellID->2064974897],
  Cell[667746, 18472, 1143, 32, 40, "Text",ExpressionUUID->"2b21b997-7386-46cc-b10f-5cd709e13a8f",
   CellTags->"b:5.1.4",
   CellID->1313411647],
  Cell[676592, 18728, 16034, 376, 191, "Text",ExpressionUUID->"696c8d91-be15-4eb7-9b07-f4e082068e79",
   CellTags->"b:5.1.4",
   CellID->205059545],
  Cell[692629, 19106, 1056, 19, 144, "Text",ExpressionUUID->"58533d77-6a16-48c6-9074-c73db65dd1c9",
   CellTags->"b:5.1.4",
   CellID->20671885],
  Cell[729226, 20033, 1137, 31, 190, "Input",ExpressionUUID->"c36cbf0e-7130-42d2-b7b5-f81a66a99bfd",
   CellTags->"b:5.1.4",
   CellID->699658063],
  Cell[730366, 20066, 944, 26, 49, "Output",ExpressionUUID->"f4cd31e4-5da3-4abb-baaa-4eb8e7636719",
   CellTags->"b:5.1.4",
   CellID->806417294],
  Cell[731313, 20094, 272, 7, 34, "Output",ExpressionUUID->"628ca715-b15b-455a-9145-42096ba4f71e",
   CellTags->"b:5.1.4",
   CellID->686164041],
  Cell[731588, 20103, 272, 7, 34, "Output",ExpressionUUID->"c564d13e-13ce-42e1-98cb-191780b12298",
   CellTags->"b:5.1.4",
   CellID->371852697],
  Cell[731863, 20112, 724, 21, 49, "Output",ExpressionUUID->"5cd90fb3-b7bc-4478-b117-6247d5e5a079",
   CellTags->"b:5.1.4",
   CellID->550865709],
  Cell[732590, 20135, 257, 7, 34, "Output",ExpressionUUID->"40bd79a2-cf69-4a76-86fd-9de6d8235c85",
   CellTags->"b:5.1.4",
   CellID->597968946],
  Cell[732850, 20144, 273, 7, 34, "Output",ExpressionUUID->"e16e7d3d-c708-429b-b56f-9229ed7d035b",
   CellTags->"b:5.1.4",
   CellID->1988091109],
  Cell[733126, 20153, 258, 7, 34, "Output",ExpressionUUID->"d0d3f249-6929-4eaa-b891-acdf731b57be",
   CellTags->"b:5.1.4",
   CellID->1149205290],
  Cell[733421, 20165, 510, 14, 87, "Input",ExpressionUUID->"bbf31f51-3ff9-4af1-bda7-8d82c4ebd0c9",
   CellTags->"b:5.1.4",
   CellID->1680097516],
  Cell[733934, 20181, 2335, 54, 83, "Output",ExpressionUUID->"7377ff9c-faf8-45db-b8c0-d79c92914f80",
   CellTags->"b:5.1.4",
   CellID->225387912],
  Cell[736272, 20237, 273, 7, 34, "Output",ExpressionUUID->"3027bfb5-3b5d-45d4-b0db-22c0e078fc5c",
   CellTags->"b:5.1.4",
   CellID->2080967323],
  Cell[736548, 20246, 272, 7, 34, "Output",ExpressionUUID->"42623be4-585c-4545-93f0-e39c9b9efff0",
   CellTags->"b:5.1.4",
   CellID->899122619],
  Cell[736835, 20256, 446, 9, 56, "Text",ExpressionUUID->"d73cbb6c-6597-4f9a-90e5-df1d14978916",
   CellTags->"b:5.1.4",
   CellID->136477009],
  Cell[737306, 20269, 731, 21, 89, "Input",ExpressionUUID->"440f67bd-3b21-4580-a6a6-f81976db1660",
   CellTags->"b:5.1.4",
   CellID->256555267],
  Cell[738040, 20292, 941, 24, 49, "Output",ExpressionUUID->"7d95a954-460e-41ed-baf9-a0226e5ac0be",
   CellTags->"b:5.1.4",
   CellID->1858917241],
  Cell[738984, 20318, 273, 7, 34, "Output",ExpressionUUID->"6b4a7cb9-a9f7-43aa-b01f-88a4c4215267",
   CellTags->"b:5.1.4",
   CellID->1229972931],
  Cell[739260, 20327, 272, 7, 34, "Output",ExpressionUUID->"f82a5d3b-df80-4e19-b534-eaa4db650605",
   CellTags->"b:5.1.4",
   CellID->477465295],
  Cell[739547, 20337, 495, 10, 36, "Text",ExpressionUUID->"0042349a-67a5-4b61-a324-847d5b09cd5a",
   CellTags->"b:5.1.4",
   CellID->1924692940],
  Cell[740067, 20351, 854, 23, 113, "Input",ExpressionUUID->"314dd437-16d4-4f41-a6d5-3867ef618bb7",
   CellTags->"b:5.1.4",
   CellID->768067819],
  Cell[740924, 20376, 258, 7, 34, "Output",ExpressionUUID->"eb8035e4-1e4a-4ade-aa21-5a3a374190f5",
   CellTags->"b:5.1.4",
   CellID->1760307873],
  Cell[741185, 20385, 275, 7, 34, "Output",ExpressionUUID->"cbf5a05e-b01d-4e48-991f-1b6483db2a24",
   CellTags->"b:5.1.4",
   CellID->1759700439],
  Cell[741463, 20394, 275, 7, 34, "Output",ExpressionUUID->"4f22ba43-33b0-476e-9c8b-25370c285aeb",
   CellTags->"b:5.1.4",
   CellID->140491789],
  Cell[741753, 20404, 426, 10, 34, "Text",ExpressionUUID->"23d544a2-af77-4240-b011-fd4d2ec93727",
   CellTags->"b:5.1.4",
   CellID->391631816],
  Cell[742204, 20418, 555, 13, 110, "Input",ExpressionUUID->"2ec33de3-e5b6-4e89-b603-d0be6efcce9e",
   CellTags->"b:5.1.4",
   CellID->118553457],
  Cell[742762, 20433, 664, 15, 34, "Output",ExpressionUUID->"d5de0961-3667-47e0-b50f-f3c3745cb9c1",
   CellTags->"b:5.1.4",
   CellID->1160969670],
  Cell[743429, 20450, 676, 17, 34, "Output",ExpressionUUID->"8acff93d-3429-4ba5-971e-61c926a6e099",
   CellTags->"b:5.1.4",
   CellID->380827048],
  Cell[1271025, 30054, 1281, 36, 76, "Text",ExpressionUUID->"400315b1-b177-4c62-b26e-0edd8c82338e",
   CellTags->"b:5.1.4",
   CellID->158527280],
  Cell[1272331, 30094, 408, 12, 35, "Input",ExpressionUUID->"914f3bf7-e6ee-42a7-aa69-a7a0c6f7e655",
   CellTags->"b:5.1.4",
   CellID->1488272680],
  Cell[1272742, 30108, 1822, 55, 75, "Output",ExpressionUUID->"fbe5180e-e382-4ea6-af96-9a78640748ba",
   CellTags->"b:5.1.4",
   CellID->515646340],
  Cell[1274601, 30168, 532, 16, 35, "Input",ExpressionUUID->"ec24f419-a90d-4ae0-b121-b916f7acde80",
   CellTags->"b:5.1.4",
   CellID->1033159208],
  Cell[1275136, 30186, 869, 25, 51, "Output",ExpressionUUID->"8c5162fb-76c0-4e44-8180-d6eca29b34a7",
   CellTags->"b:5.1.4",
   CellID->1371520735],
  Cell[1276042, 30216, 401, 12, 35, "Input",ExpressionUUID->"7de82e02-6ec9-4a49-b194-0002be469499",
   CellTags->"b:5.1.4",
   CellID->1901522861],
  Cell[1276446, 30230, 902, 26, 71, "Output",ExpressionUUID->"26c8269a-1b14-4a5a-88bb-416399b581d9",
   CellTags->"b:5.1.4",
   CellID->59526247],
  Cell[1277385, 30261, 328, 10, 34, "Input",ExpressionUUID->"4f1582d5-efa6-47c6-8f17-c3294a8b58b3",
   CellTags->"b:5.1.4",
   CellID->43090173],
  Cell[1277716, 30273, 671, 20, 49, "Output",ExpressionUUID->"99005412-e9e2-4eda-9b44-136798e44979",
   CellTags->"b:5.1.4",
   CellID->1877737474],
  Cell[1278402, 30296, 572, 19, 36, "Text",ExpressionUUID->"7030108b-77b6-43cd-b44b-e7f73753c319",
   CellTags->"b:5.1.4",
   CellID->514548572],
  Cell[1278999, 30319, 547, 17, 35, "Input",ExpressionUUID->"fbbaabab-118d-46cf-88fb-b084a3f16ffe",
   CellTags->"b:5.1.4",
   CellID->101203135],
  Cell[1279549, 30338, 1299, 35, 55, "Output",ExpressionUUID->"4a3c0989-6519-4668-9746-2c2aca44931a",
   CellTags->"b:5.1.4",
   CellID->1861532853],
  Cell[1280863, 30376, 470, 10, 34, "Text",ExpressionUUID->"cc17e065-37e8-4de6-82ed-c3e71bd47061",
   CellTags->"b:5.1.4",
   CellID->114041521]},
 "c:10"->{
  Cell[1285158, 30501, 451, 11, 67, "Subsection",ExpressionUUID->"96078100-68c0-4ba8-82c8-170ae445d16b",
   CellTags->{"c:10", "b:5.2.0"},
   CellID->1398872592]},
 "b:5.2.0"->{
  Cell[1285158, 30501, 451, 11, 67, "Subsection",ExpressionUUID->"96078100-68c0-4ba8-82c8-170ae445d16b",
   CellTags->{"c:10", "b:5.2.0"},
   CellID->1398872592],
  Cell[1285612, 30514, 1891, 55, 66, "Text",ExpressionUUID->"423adf7a-b4c9-477f-96c2-6e5353d0f829",
   CellTags->"b:5.2.0",
   CellID->200686388],
  Cell[1287528, 30573, 325, 10, 34, "Input",ExpressionUUID->"ab430eea-e3bf-4008-8e86-2e56c6f806f8",
   CellTags->"b:5.2.0",
   CellID->1386797326],
  Cell[1287856, 30585, 835, 28, 69, "Output",ExpressionUUID->"36b27891-1c9e-49d7-a45b-dc6f1732391f",
   CellTags->"b:5.2.0",
   CellID->93407228],
  Cell[1288706, 30616, 269, 7, 34, "Text",ExpressionUUID->"32d9dae5-a881-446c-bc7b-aed2b411e7fd",
   CellTags->"b:5.2.0",
   CellID->1262637191],
  Cell[1289000, 30627, 1067, 29, 86, "Input",ExpressionUUID->"85fdbe19-9955-432e-9b49-5a26c90133f4",
   CellTags->"b:5.2.0",
   CellID->1857353130],
  Cell[1290070, 30658, 835, 28, 69, "Output",ExpressionUUID->"384b85fc-54a7-4fc9-953d-4a07427de1ff",
   CellTags->"b:5.2.0",
   CellID->84735039],
  Cell[1290908, 30688, 257, 7, 34, "Output",ExpressionUUID->"e15ba8f9-3220-461d-b735-f370081330f3",
   CellTags->"b:5.2.0",
   CellID->360142512],
  Cell[1291168, 30697, 645, 19, 49, "Output",ExpressionUUID->"209d2aaf-0c6e-4dc2-9978-cfde9e8ddfe1",
   CellTags->"b:5.2.0",
   CellID->1306281938],
  Cell[1291828, 30719, 307, 8, 34, "Text",ExpressionUUID->"ec39b56f-56c0-464c-9557-3b22e44f200b",
   CellTags->"b:5.2.0",
   CellID->711763118],
  Cell[1292160, 30731, 853, 18, 34, "Input",ExpressionUUID->"e781abb5-21c3-4c79-b840-288f0f9ea0e6",
   CellTags->"b:5.2.0",
   CellID->1118810436],
  Cell[1293016, 30751, 1754, 61, 231, "Output",ExpressionUUID->"4763f15c-1741-4cd7-b27a-1dc6e4727dbb",
   CellTags->"b:5.2.0",
   CellID->955208685],
  Cell[1294785, 30815, 272, 7, 34, "Text",ExpressionUUID->"bdea370b-d82f-4b45-a5f6-91699749c01f",
   CellTags->"b:5.2.0",
   CellID->480184720],
  Cell[1295082, 30826, 880, 20, 34, "Input",ExpressionUUID->"a7d7ff51-a767-453b-b856-fed2718efbb6",
   CellTags->"b:5.2.0",
   CellID->221043614],
  Cell[1295965, 30848, 256, 7, 34, "Output",ExpressionUUID->"22fa7134-b3f8-4d27-aab2-a1444e81ca96",
   CellTags->"b:5.2.0",
   CellID->1752107470],
  Cell[1296236, 30858, 602, 11, 78, "Text",ExpressionUUID->"2a6d3308-807b-411c-a2ac-5b192fe52903",
   CellTags->"b:5.2.0",
   CellID->2080794835],
  Cell[1296863, 30873, 1155, 30, 113, "Input",ExpressionUUID->"970e4ff0-f2b9-4508-8764-82077d64850f",
   CellTags->"b:5.2.0",
   CellID->147600450],
  Cell[1298021, 30905, 989, 25, 151, "Output",ExpressionUUID->"7e21fb3e-27f2-48df-9f84-c15e48301af7",
   CellTags->"b:5.2.0",
   CellID->1374480626],
  Cell[1299013, 30932, 1086, 29, 167, "Output",ExpressionUUID->"1f590e1e-0b85-4d51-ab05-ab4683f7fa1e",
   CellTags->"b:5.2.0",
   CellID->414290403],
  Cell[1300102, 30963, 744, 23, 65, "Output",ExpressionUUID->"ca800559-5be5-4647-92fb-95f71f514d3d",
   CellTags->"b:5.2.0",
   CellID->836819647],
  Cell[1300849, 30988, 647, 19, 49, "Output",ExpressionUUID->"813d7053-d557-496e-b9ff-1da22402e551",
   CellTags->"b:5.2.0",
   CellID->557402926],
  Cell[1309242, 31311, 586, 20, 34, "Text",ExpressionUUID->"0e6c5841-2222-4e05-9291-f422ae0119a8",
   CellTags->"b:5.2.0",
   CellID->377150865],
  Cell[1309853, 31335, 386, 11, 35, "Input",ExpressionUUID->"9775c230-9ffb-415f-8280-7fe1bceda115",
   CellTags->{"i:2", "b:5.2.0"},
   CellID->1550345758],
  Cell[1310242, 31348, 2597, 89, 215, "Output",ExpressionUUID->"e4d35865-ef5d-45dc-82e4-fb4987273476",
   CellTags->{"i:2", "b:5.2.0"},
   CellID->1058031399],
  Cell[1312854, 31440, 365, 9, 34, "Text",ExpressionUUID->"6ca62813-b7ce-40d0-a9af-d31b495f3e96",
   CellTags->"b:5.2.0",
   CellID->1242065655],
  Cell[1313244, 31453, 921, 26, 160, "Input",ExpressionUUID->"34c1b253-fa12-430c-9c82-a55aaf429426",
   CellTags->"b:5.2.0",
   CellID->1856414745],
  Cell[1314168, 31481, 491, 14, 54, "Output",ExpressionUUID->"c36b3f1f-effc-4705-b6bf-f8f84f730d73",
   CellTags->"b:5.2.0",
   CellID->1169125833],
  Cell[1314662, 31497, 348, 10, 34, "Output",ExpressionUUID->"b0f93177-bf9c-4ec4-b389-75eee9a662c7",
   CellTags->"b:5.2.0",
   CellID->22505181],
  Cell[1315013, 31509, 514, 15, 54, "Output",ExpressionUUID->"126e8411-ccfc-45fd-8723-0a69fffd0f90",
   CellTags->"b:5.2.0",
   CellID->287455195],
  Cell[1315542, 31527, 441, 9, 34, "Text",ExpressionUUID->"737e77b0-1523-4e11-9f75-a3c030205c57",
   CellTags->"b:5.2.0",
   CellID->2120195599],
  Cell[1316008, 31540, 681, 18, 61, "Input",ExpressionUUID->"23cebcfc-b49e-46cc-97d0-78b06dfd9b62",
   CellTags->"b:5.2.0",
   CellID->931277722],
  Cell[1316692, 31560, 938, 28, 71, "Output",ExpressionUUID->"458e0be8-43f7-4a36-b28f-34bfb7e5c9cd",
   CellTags->"b:5.2.0",
   CellID->1568740262],
  Cell[1317633, 31590, 1088, 33, 69, "Output",ExpressionUUID->"98eca4f6-7e69-4ddf-b095-331923f10502",
   CellTags->"b:5.2.0",
   CellID->1160849215],
  Cell[1318736, 31626, 1306, 37, 66, "Text",ExpressionUUID->"9af40ab8-176b-4182-b3b5-71a266d5e7a0",
   CellTags->"b:5.2.0",
   CellID->1123854318],
  Cell[1320067, 31667, 426, 11, 61, "Input",ExpressionUUID->"a71de71b-d04b-4e4a-b136-f84d059ac3c1",
   CellTags->"b:5.2.0",
   CellID->1500847261],
  Cell[1320496, 31680, 769, 23, 71, "Output",ExpressionUUID->"e93e6498-eaf8-4e9d-9d64-19319e6a29e1",
   CellTags->"b:5.2.0",
   CellID->503823510],
  Cell[1321268, 31705, 547, 15, 48, "Output",ExpressionUUID->"66c5a31a-6bba-4278-b068-0f875ec221f2",
   CellTags->"b:5.2.0",
   CellID->233712449],
  Cell[1321830, 31723, 399, 13, 37, "Text",ExpressionUUID->"9ff36dab-3e11-41db-b1fb-16801bdd4f56",
   CellTags->"b:5.2.0",
   CellID->396997965],
  Cell[1322254, 31740, 774, 19, 135, "Input",ExpressionUUID->"85100578-7d82-42bd-8ae0-e6626e613121",
   CellTags->"b:5.2.0",
   CellID->113106526],
  Cell[1323031, 31761, 937, 28, 71, "Output",ExpressionUUID->"e59b3a1f-9485-4ecf-9cd5-8991691694df",
   CellTags->"b:5.2.0",
   CellID->257539980],
  Cell[1323971, 31791, 770, 23, 71, "Output",ExpressionUUID->"438b993d-03f4-4581-8228-95e2afb84c42",
   CellTags->"b:5.2.0",
   CellID->1294895155],
  Cell[1324744, 31816, 546, 15, 48, "Output",ExpressionUUID->"ce55b428-b606-4a92-8961-d22c8e848b6e",
   CellTags->"b:5.2.0",
   CellID->703591387],
  Cell[1325293, 31833, 720, 23, 69, "Output",ExpressionUUID->"719273fd-c55e-45db-bbe4-b43f3ba3c826",
   CellTags->"b:5.2.0",
   CellID->1215480927],
  Cell[1326016, 31858, 491, 14, 54, "Output",ExpressionUUID->"d2b6c0d6-19ed-4f71-9993-81371b97df76",
   CellTags->"b:5.2.0",
   CellID->1723489825]},
 "i:2"->{
  Cell[1309853, 31335, 386, 11, 35, "Input",ExpressionUUID->"9775c230-9ffb-415f-8280-7fe1bceda115",
   CellTags->{"i:2", "b:5.2.0"},
   CellID->1550345758],
  Cell[1310242, 31348, 2597, 89, 215, "Output",ExpressionUUID->"e4d35865-ef5d-45dc-82e4-fb4987273476",
   CellTags->{"i:2", "b:5.2.0"},
   CellID->1058031399]},
 "i:1"->{
  Cell[1326556, 31878, 452, 11, 67, "Subsection",ExpressionUUID->"77c1fd1c-ffe7-47d3-9784-83feb0f9ec2d",
   CellTags->{"i:1", "c:11", "b:5.2.1"},
   CellID->458354887],
  Cell[1333451, 32100, 371, 10, 67, "Subsection",ExpressionUUID->"f14f877b-8ff0-4d4e-b194-a27251469e91",
   CellTags->{"i:1", "c:12", "b:5.2.2"},
   CellID->723157632],
  Cell[1343816, 32435, 370, 10, 67, "Subsection",ExpressionUUID->"66fa64c3-4d88-45c6-bfae-e72b46411a05",
   CellTags->{"i:1", "c:13", "b:5.2.3"},
   CellID->1250439549]},
 "c:11"->{
  Cell[1326556, 31878, 452, 11, 67, "Subsection",ExpressionUUID->"77c1fd1c-ffe7-47d3-9784-83feb0f9ec2d",
   CellTags->{"i:1", "c:11", "b:5.2.1"},
   CellID->458354887]},
 "b:5.2.1"->{
  Cell[1326556, 31878, 452, 11, 67, "Subsection",ExpressionUUID->"77c1fd1c-ffe7-47d3-9784-83feb0f9ec2d",
   CellTags->{"i:1", "c:11", "b:5.2.1"},
   CellID->458354887],
  Cell[1327011, 31891, 904, 22, 80, "Text",ExpressionUUID->"43041a90-1cb0-4a74-87e8-e4e8602d5436",
   CellTags->"b:5.2.1",
   CellID->546744020],
  Cell[1327940, 31917, 299, 9, 34, "Input",ExpressionUUID->"bf4a536c-cffd-4294-8939-45a4120a5a0c",
   CellTags->"b:5.2.1",
   CellID->44218627],
  Cell[1328242, 31928, 719, 21, 83, "Output",ExpressionUUID->"21b94af7-8f88-4e48-ab1a-a7cba2d3d955",
   CellTags->"b:5.2.1",
   CellID->1434058039],
  Cell[1328976, 31952, 472, 15, 34, "Text",ExpressionUUID->"54c8dd22-ebd3-4b4a-823e-431c32cb6d2a",
   CellTags->"b:5.2.1",
   CellID->153122127],
  Cell[1329473, 31971, 751, 20, 35, "Input",ExpressionUUID->"5eb244d2-f3e0-459b-9565-a5e824f42ff8",
   CellTags->"b:5.2.1",
   CellID->838179383],
  Cell[1330227, 31993, 719, 21, 83, "Output",ExpressionUUID->"735da3b1-d631-4576-be3f-5a37c5a299e1",
   CellTags->"b:5.2.1",
   CellID->1004491192],
  Cell[1330961, 32017, 360, 8, 34, "Text",ExpressionUUID->"1d2a3481-0ad2-4caa-a32c-d983326035bf",
   CellTags->"b:5.2.1",
   CellID->1935649408],
  Cell[1331346, 32029, 544, 14, 34, "Input",ExpressionUUID->"f5550a7b-12fc-44a5-b276-721084790a8d",
   CellTags->"b:5.2.1",
   CellID->229831853],
  Cell[1331893, 32045, 719, 21, 83, "Output",ExpressionUUID->"da946fad-026b-4736-b336-22fcaa8a4717",
   CellTags->"b:5.2.1",
   CellID->1114514877],
  Cell[1332627, 32069, 464, 14, 34, "Text",ExpressionUUID->"13d5adce-f13f-4538-a04c-4180b12fc59e",
   CellTags->"b:5.2.1",
   CellID->1608356065],
  Cell[1333094, 32085, 320, 10, 34, "Input",ExpressionUUID->"8a662743-5658-4b86-ab58-90cfab17ff8e",
   CellTags->"b:5.2.1",
   CellID->1263719666]},
 "c:12"->{
  Cell[1333451, 32100, 371, 10, 67, "Subsection",ExpressionUUID->"f14f877b-8ff0-4d4e-b194-a27251469e91",
   CellTags->{"i:1", "c:12", "b:5.2.2"},
   CellID->723157632]},
 "b:5.2.2"->{
  Cell[1333451, 32100, 371, 10, 67, "Subsection",ExpressionUUID->"f14f877b-8ff0-4d4e-b194-a27251469e91",
   CellTags->{"i:1", "c:12", "b:5.2.2"},
   CellID->723157632],
  Cell[1333825, 32112, 482, 16, 36, "Text",ExpressionUUID->"ecb0a9ef-6c94-48e3-a22b-5f78348fb1cc",
   CellTags->"b:5.2.2",
   CellID->1500362651],
  Cell[1334310, 32130, 346, 9, 34, "Text",ExpressionUUID->"bf737c52-08be-41be-9416-b8e1837e81e7",
   CellTags->"b:5.2.2",
   CellID->842092283],
  Cell[1334659, 32141, 472, 10, 56, "Text",ExpressionUUID->"94636e20-b904-4fae-a254-03f72278173b",
   CellTags->"b:5.2.2",
   CellID->1127766651],
  Cell[1335156, 32155, 397, 11, 60, "Input",ExpressionUUID->"fdb8f1d6-a0c2-438b-a51e-2b7abf8b10ac",
   CellTags->"b:5.2.2",
   CellID->683434935],
  Cell[1335556, 32168, 741, 22, 83, "Output",ExpressionUUID->"aeaa360d-6027-4483-bbae-cb368aa39e48",
   CellTags->"b:5.2.2",
   CellID->1299168605],
  Cell[1336300, 32192, 743, 22, 83, "Output",ExpressionUUID->"a59f1c24-5588-4dac-ac18-4722fee6c1b6",
   CellTags->"b:5.2.2",
   CellID->1660505186],
  Cell[1337058, 32217, 515, 15, 34, "Text",ExpressionUUID->"49271300-b953-46df-8d14-7417c58c2c95",
   CellTags->"b:5.2.2",
   CellID->1748408389],
  Cell[1337598, 32236, 303, 9, 34, "Input",ExpressionUUID->"750862b4-8b1d-4af8-ae3f-5b521996bf51",
   CellTags->"b:5.2.2",
   CellID->1481052022],
  Cell[1337904, 32247, 1033, 27, 151, "Output",ExpressionUUID->"6e22b8ed-b7be-42b1-b4c2-b65d66d6127c",
   CellTags->"b:5.2.2",
   CellID->1560668337],
  Cell[1338952, 32277, 403, 9, 56, "Text",ExpressionUUID->"6a744824-9850-4aef-a43d-d3fed472eab3",
   CellTags->"b:5.2.2",
   CellID->558381377],
  Cell[1339358, 32288, 308, 8, 34, "Text",ExpressionUUID->"07d12111-05a7-4ced-8283-a63e3c8c0a4c",
   CellTags->"b:5.2.2",
   CellID->212265849],
  Cell[1339669, 32298, 592, 13, 56, "Text",ExpressionUUID->"dd355a86-94c0-4e60-ad12-4911bb130386",
   CellTags->"b:5.2.2",
   CellID->581786476],
  Cell[1340264, 32313, 336, 8, 34, "Text",ExpressionUUID->"47a4413c-726c-410a-bfe6-49502bcbd5db",
   CellTags->"b:5.2.2",
   CellID->592257341],
  Cell[1340625, 32325, 542, 14, 60, "Input",ExpressionUUID->"87910459-7312-4023-9b21-1fa5f5bda4c2",
   CellTags->"b:5.2.2",
   CellID->482426342],
  Cell[1341170, 32341, 735, 24, 85, "Output",ExpressionUUID->"07db9489-d66f-44da-bfc1-3139f658d400",
   CellTags->"b:5.2.2",
   CellID->787475501],
  Cell[1341920, 32368, 502, 11, 56, "Text",ExpressionUUID->"0efad72b-15e4-4a54-af92-59c418c7d0a7",
   CellTags->"b:5.2.2",
   CellID->1388762569],
  Cell[1342447, 32383, 507, 15, 61, "Input",ExpressionUUID->"ef37a271-9694-4c04-888b-2ca6dea493d2",
   CellTags->"b:5.2.2",
   CellID->2080221587],
  Cell[1342957, 32400, 810, 29, 155, "Output",ExpressionUUID->"4c7367cd-9071-4539-8e8d-2967ae26a9b5",
   CellTags->"b:5.2.2",
   CellID->159028846]},
 "c:13"->{
  Cell[1343816, 32435, 370, 10, 67, "Subsection",ExpressionUUID->"66fa64c3-4d88-45c6-bfae-e72b46411a05",
   CellTags->{"i:1", "c:13", "b:5.2.3"},
   CellID->1250439549]},
 "b:5.2.3"->{
  Cell[1343816, 32435, 370, 10, 67, "Subsection",ExpressionUUID->"66fa64c3-4d88-45c6-bfae-e72b46411a05",
   CellTags->{"i:1", "c:13", "b:5.2.3"},
   CellID->1250439549],
  Cell[1344189, 32447, 476, 10, 56, "Text",ExpressionUUID->"e849669a-46c9-4ccf-94cb-45943aeda80e",
   CellTags->"b:5.2.3",
   CellID->862530501],
  Cell[1344668, 32459, 1581, 49, 119, "Text",ExpressionUUID->"58ec87eb-ad71-4cab-90bc-edac90886601",
   CellTags->"b:5.2.3",
   CellID->1035513057],
  Cell[1346274, 32512, 321, 9, 35, "Input",ExpressionUUID->"f15862e1-8c3b-487d-a7f3-12aaf8ae5449",
   CellTags->"b:5.2.3",
   CellID->660473711],
  Cell[1346598, 32523, 987, 25, 151, "Output",ExpressionUUID->"f74de61e-9abe-4a70-8a4e-41737350f955",
   CellTags->"b:5.2.3",
   CellID->1328157854],
  Cell[1347600, 32551, 1200, 35, 81, "Text",ExpressionUUID->"244d9e5a-8a4d-4ff4-966d-09819b4774a7",
   CellTags->"b:5.2.3",
   CellID->238689434],
  Cell[1349750, 32613, 1378, 23, 140, "Text",ExpressionUUID->"bd35af58-9d9a-4d23-a15c-80c29ba3945a",
   CellTags->"b:5.2.3",
   CellID->1277398455],
  Cell[1370466, 33210, 914, 16, 34, "Text",ExpressionUUID->"2697c18c-ff70-40dc-b3a2-0ff25d39b02a",
   CellTags->"b:5.2.3",
   CellID->1803534785]},
 "Measurements"->{
  Cell[1372282, 33254, 411, 11, 51, "Subsection",ExpressionUUID->"bfef1c7b-67ac-46fa-9f73-a960decca00a",
   CellTags->{"Measurements", "c:15", "b:5.3.0"},
   CellID->432727106],
  Cell[1388564, 33765, 423, 11, 67, "Subsection",ExpressionUUID->"3dd7fa66-0334-4084-bce6-902e9f0997d0",
   CellTags->{"Measurements", "MeasureKet", "c:16", "b:5.3.1"},
   CellID->1060825169]},
 "c:15"->{
  Cell[1372282, 33254, 411, 11, 51, "Subsection",ExpressionUUID->"bfef1c7b-67ac-46fa-9f73-a960decca00a",
   CellTags->{"Measurements", "c:15", "b:5.3.0"},
   CellID->432727106]},
 "b:5.3.0"->{
  Cell[1372282, 33254, 411, 11, 51, "Subsection",ExpressionUUID->"bfef1c7b-67ac-46fa-9f73-a960decca00a",
   CellTags->{"Measurements", "c:15", "b:5.3.0"},
   CellID->432727106],
  Cell[1372696, 33267, 434, 10, 56, "Text",ExpressionUUID->"7fd703ca-332a-4150-a237-cb8d245733c6",
   CellTags->"b:5.3.0",
   CellID->1464841800],
  Cell[1373133, 33279, 487, 13, 59, "Text",ExpressionUUID->"2087b84c-12f6-436b-a18e-601bb67f5daf",
   CellTags->"b:5.3.0",
   CellID->843603392],
  Cell[1373645, 33296, 1349, 34, 61, "Input",ExpressionUUID->"df3d8886-d538-4602-8838-bcfa86ff7263",
   CellTags->"b:5.3.0",
   CellID->755171850],
  Cell[1374997, 33332, 3659, 105, 351, "Output",ExpressionUUID->"07d4dc40-be9d-4f8b-b050-b934bbb048a8",
   CellTags->"b:5.3.0",
   CellID->1805962118],
  Cell[1378671, 33440, 348, 8, 34, "Text",ExpressionUUID->"75a818a4-b3a1-46a0-b34f-3822be32edc5",
   CellTags->"b:5.3.0",
   CellID->544929180],
  Cell[1379022, 33450, 777, 19, 36, "Input",ExpressionUUID->"7567bfe0-3030-4320-9b8c-97fb10fa41a9",
   CellTags->"b:5.3.0",
   CellID->1754457452],
  Cell[1379802, 33471, 369, 8, 56, "Text",ExpressionUUID->"cbc42fde-6cd6-4047-a25f-d2fdacdb9ec1",
   CellTags->"b:5.3.0",
   CellID->68438807],
  Cell[1380174, 33481, 720, 24, 36, "Input",ExpressionUUID->"4e717616-586f-4547-8026-609143538737",
   CellTags->"b:5.3.0",
   CellID->1347278104],
  Cell[1380897, 33507, 369, 8, 34, "Text",ExpressionUUID->"211c0012-3e14-46fa-bba3-2dfdc1717141",
   CellTags->"b:5.3.0",
   CellID->635519960],
  Cell[1381291, 33519, 412, 12, 61, "Input",ExpressionUUID->"0acf171e-9753-4e36-8db2-89181325c802",
   CellTags->"b:5.3.0",
   CellID->1747791189],
  Cell[1381706, 33533, 820, 25, 99, "Output",ExpressionUUID->"e3f9ad02-a399-4fa9-b1a4-22dd98b09a81",
   CellTags->"b:5.3.0",
   CellID->271450528],
  Cell[1382529, 33560, 253, 7, 34, "Output",ExpressionUUID->"08113570-68aa-41f7-8c44-77bd9b40b8ee",
   CellTags->"b:5.3.0",
   CellID->1013757868],
  Cell[1382797, 33570, 325, 8, 34, "Text",ExpressionUUID->"7c90bc29-7053-48a1-8f31-37bd6cef62f1",
   CellTags->"b:5.3.0",
   CellID->1845189857],
  Cell[1383147, 33582, 559, 15, 87, "Input",ExpressionUUID->"9d7f315d-d755-43ce-a9d8-59bbab8942e8",
   CellTags->"b:5.3.0",
   CellID->294580272],
  Cell[1383709, 33599, 4189, 137, 119, "Output",ExpressionUUID->"0434d8fd-bbdc-4194-b14f-afe2eff599a1",
   CellTags->"b:5.3.0",
   CellID->93232178],
  Cell[1387901, 33738, 614, 21, 49, "Output",ExpressionUUID->"4b6f6c52-9904-4e4c-92e5-f74b513dad81",
   CellTags->"b:5.3.0",
   CellID->1437636983]},
 "MeasureKet"->{
  Cell[1388564, 33765, 423, 11, 67, "Subsection",ExpressionUUID->"3dd7fa66-0334-4084-bce6-902e9f0997d0",
   CellTags->{"Measurements", "MeasureKet", "c:16", "b:5.3.1"},
   CellID->1060825169]},
 "c:16"->{
  Cell[1388564, 33765, 423, 11, 67, "Subsection",ExpressionUUID->"3dd7fa66-0334-4084-bce6-902e9f0997d0",
   CellTags->{"Measurements", "MeasureKet", "c:16", "b:5.3.1"},
   CellID->1060825169]},
 "b:5.3.1"->{
  Cell[1388564, 33765, 423, 11, 67, "Subsection",ExpressionUUID->"3dd7fa66-0334-4084-bce6-902e9f0997d0",
   CellTags->{"Measurements", "MeasureKet", "c:16", "b:5.3.1"},
   CellID->1060825169],
  Cell[1388990, 33778, 310, 8, 34, "Text",ExpressionUUID->"4ac0b746-8587-448b-8344-9cc05179fceb",
   CellTags->"b:5.3.1",
   CellID->879377989],
  Cell[1389325, 33790, 1304, 33, 61, "Input",ExpressionUUID->"0f502680-4556-4ebc-8263-83eba6cae38e",
   CellTags->"b:5.3.1",
   CellID->723402469],
  Cell[1390632, 33825, 1371, 57, 367, "Output",ExpressionUUID->"c98686e0-0fb5-416d-aee4-05a122e21360",
   CellTags->"b:5.3.1",
   CellID->1026260884],
  Cell[1392018, 33885, 333, 8, 34, "Text",ExpressionUUID->"04572544-b478-4ace-9ff8-cecabda0c0bb",
   CellTags->"b:5.3.1",
   CellID->1320370816],
  Cell[1392354, 33895, 409, 9, 34, "Text",ExpressionUUID->"249ee173-4749-4add-b9ad-cac13bde2a4b",
   CellTags->"b:5.3.1",
   CellID->1523780061],
  Cell[1392766, 33906, 498, 16, 34, "Input",ExpressionUUID->"615ceb4e-3289-4eef-a6b0-946923de8f7c",
   CellTags->"b:5.3.1",
   CellID->1428994752],
  Cell[1393267, 33924, 374, 8, 56, "Text",ExpressionUUID->"98c93848-cb1a-4eca-83de-631afbe6a01a",
   CellTags->"b:5.3.1",
   CellID->1805172463],
  Cell[1393666, 33936, 292, 9, 34, "Input",ExpressionUUID->"894ac134-a2c8-4a90-b4be-d216b85943aa",
   CellTags->"b:5.3.1",
   CellID->2032104472],
  Cell[1393961, 33947, 743, 25, 103, "Output",ExpressionUUID->"8bc3aa38-af4f-4e3b-84f3-32b2ef691ac0",
   CellTags->"b:5.3.1",
   CellID->1925508331],
  Cell[1394719, 33975, 309, 8, 34, "Text",ExpressionUUID->"b18e7336-c2a6-43bf-9f19-fc03abbdd4b9",
   CellTags->"b:5.3.1",
   CellID->2147422663],
  Cell[1395053, 33987, 626, 17, 35, "Input",ExpressionUUID->"d7ae5193-39b0-4652-a806-571dc26986e9",
   CellTags->"b:5.3.1",
   CellID->1122525559],
  Cell[1395682, 34006, 819, 25, 99, "Output",ExpressionUUID->"9cca6f84-ade3-4fcf-966d-cfbca20c2e7d",
   CellTags->"b:5.3.1",
   CellID->1587442474],
  Cell[1396538, 34036, 277, 8, 35, "Input",ExpressionUUID->"3ae39e7c-dff6-4014-8b76-a10b70bad607",
   CellTags->"b:5.3.1",
   CellID->643231218],
  Cell[1396818, 34046, 255, 7, 34, "Output",ExpressionUUID->"769b8556-ed08-4951-9ed0-7d165e4be96e",
   CellTags->"b:5.3.1",
   CellID->1956313837],
  Cell[1397088, 34056, 308, 8, 34, "Text",ExpressionUUID->"bf5e9b52-7748-4582-b998-150951546de6",
   CellTags->"b:5.3.1",
   CellID->354401928]},
 "c:17"->{
  Cell[1397433, 34069, 409, 11, 67, "Subsection",ExpressionUUID->"82f38740-2373-4da1-99f9-b92faa2cf654",
   CellTags->{"c:17", "b:5.3.2"},
   CellID->1461050765],
  Cell[1417092, 34498, 447, 11, 67, "Subsection",ExpressionUUID->"42eb8e31-7dca-4919-ba24-79725a069329",
   CellTags->{"c:17", "b:5.3.2"},
   CellID->1601367499]},
 "b:5.3.2"->{
  Cell[1397433, 34069, 409, 11, 67, "Subsection",ExpressionUUID->"82f38740-2373-4da1-99f9-b92faa2cf654",
   CellTags->{"c:17", "b:5.3.2"},
   CellID->1461050765],
  Cell[1397867, 34084, 530, 15, 61, "Input",ExpressionUUID->"9e247dfe-3605-458d-b6e5-a01656ea7945",
   CellTags->"b:5.3.2",
   CellID->1552187557],
  Cell[1398400, 34101, 11048, 186, 189, "Output",ExpressionUUID->"649672a8-2b3e-4b35-88dc-28f59ef86e15",
   CellTags->"b:5.3.2",
   CellID->1792022889],
  Cell[1409463, 34290, 340, 8, 34, "Text",ExpressionUUID->"851a7271-8b08-4c25-a345-a13a7a4c6eea",
   CellTags->"b:5.3.2",
   CellID->2132410390],
  Cell[1409806, 34300, 1264, 29, 61, "Input",ExpressionUUID->"827d49ca-b51b-489b-ab92-12dc66312362",
   CellTags->"b:5.3.2",
   CellID->66839947],
  Cell[1412280, 34361, 360, 8, 56, "Text",ExpressionUUID->"2d1618ac-1449-48ae-a25c-ee91d2af4ca3",
   CellTags->"b:5.3.2",
   CellID->245041848],
  Cell[1412643, 34371, 963, 20, 36, "Input",ExpressionUUID->"288c82a6-0700-4937-8903-798913575614",
   CellTags->"b:5.3.2",
   CellID->1692188037],
  Cell[1413609, 34393, 471, 13, 34, "Text",ExpressionUUID->"99b76c70-de96-49a0-ae11-e727527f66c6",
   CellTags->"b:5.3.2",
   CellID->1478400638],
  Cell[1414083, 34408, 371, 8, 56, "Text",ExpressionUUID->"9817bba0-0d7d-4715-a61f-08d47c5883e4",
   CellTags->"b:5.3.2",
   CellID->1093727076],
  Cell[1414457, 34418, 858, 26, 36, "Input",ExpressionUUID->"e522099b-b039-4124-80dd-6da8ad4ff503",
   CellTags->"b:5.3.2",
   CellID->1958431659],
  Cell[1415318, 34446, 419, 9, 56, "Text",ExpressionUUID->"656bd96d-80a3-44bf-ad97-e78e71bbd24c",
   CellTags->"b:5.3.2",
   CellID->879035244],
  Cell[1415762, 34459, 380, 10, 35, "Input",ExpressionUUID->"b9550836-83b4-4867-a79e-efba18c94ec9",
   CellTags->"b:5.3.2",
   CellID->1405338195],
  Cell[1416145, 34471, 932, 24, 49, "Output",ExpressionUUID->"d9ada68e-e25f-4a1c-8ad2-3c3e70e005c1",
   CellTags->"b:5.3.2",
   CellID->70546248],
  Cell[1417092, 34498, 447, 11, 67, "Subsection",ExpressionUUID->"42eb8e31-7dca-4919-ba24-79725a069329",
   CellTags->{"c:17", "b:5.3.2"},
   CellID->1601367499],
  Cell[1417738, 34516, 663, 12, 100, "Text",ExpressionUUID->"5dcda514-b9c0-4011-a6f5-e80504fbbe14",
   CellTags->"b:5.3.2",
   CellID->1493893659],
  Cell[1418404, 34530, 281, 7, 34, "Text",ExpressionUUID->"08c5c505-c284-4a80-af16-2d6d06251bfa",
   CellTags->"b:5.3.2",
   CellID->1639409736],
  Cell[1418688, 34539, 797, 26, 36, "Text",ExpressionUUID->"36340450-07e6-4dab-871d-c061c637effd",
   CellTags->"b:5.3.2",
   CellID->1344241942],
  Cell[1419488, 34567, 622, 19, 34, "Input",ExpressionUUID->"4f68c5f2-e4cf-4f96-b823-8f3425dff902",
   CellTags->"b:5.3.2",
   CellID->1146352754],
  Cell[1420113, 34588, 658, 18, 35, "Input",ExpressionUUID->"ba608194-9395-4f40-9f91-dfdfb0591002",
   CellTags->"b:5.3.2",
   CellID->1674939241],
  Cell[1420796, 34610, 351, 10, 35, "Input",ExpressionUUID->"a3ecc4b2-cb74-4b65-a875-4e54249f5e29",
   CellTags->"b:5.3.2",
   CellID->1488178611],
  Cell[1421150, 34622, 935, 24, 49, "Output",ExpressionUUID->"94269919-99f7-4690-9ac0-71d25cb637a6",
   CellTags->"b:5.3.2",
   CellID->259816992],
  Cell[1422122, 34651, 450, 14, 35, "Input",ExpressionUUID->"fd89ad4c-9a40-4523-a227-4caf81d6912f",
   CellTags->"b:5.3.2",
   CellID->75878468],
  Cell[1422575, 34667, 937, 24, 49, "Output",ExpressionUUID->"707b5a4c-c20a-421b-abe6-80443ab77445",
   CellTags->"b:5.3.2",
   CellID->909930193],
  Cell[1423527, 34694, 545, 11, 56, "Text",ExpressionUUID->"ca2b8319-7a4c-4350-a2c5-7b0ff8e6adfd",
   CellTags->"b:5.3.2",
   CellID->1831609005]},
 "c:19"->{
  Cell[1424932, 34732, 333, 9, 51, "Subsection",ExpressionUUID->"2a96dd91-1063-41c4-85d3-c1e7c447c3d2",
   CellTags->{"c:19", "b:5.4.0"},
   CellID->560075586]},
 "b:5.4.0"->{
  Cell[1424932, 34732, 333, 9, 51, "Subsection",ExpressionUUID->"2a96dd91-1063-41c4-85d3-c1e7c447c3d2",
   CellTags->{"c:19", "b:5.4.0"},
   CellID->560075586],
  Cell[1425268, 34743, 517, 14, 56, "Text",ExpressionUUID->"79f7d0f2-136b-49d4-a57e-355196f6f9a2",
   CellTags->"b:5.4.0",
   CellID->769549101],
  Cell[1425788, 34759, 1003, 31, 62, "Text",ExpressionUUID->"cb329c1c-cbb4-4c25-8eb1-0da9b75b5cc9",
   CellTags->"b:5.4.0",
   CellID->1665218273],
  Cell[1426794, 34792, 466, 10, 56, "Text",ExpressionUUID->"0f5bb2f7-1e80-41ec-beff-3bbd6f960b91",
   CellTags->"b:5.4.0",
   CellID->1316626607],
  Cell[1427285, 34806, 858, 28, 87, "Input",ExpressionUUID->"25386b9f-77c4-4cf6-a233-376cdde6e458",
   CellTags->"b:5.4.0",
   CellID->871885161],
  Cell[1428146, 34836, 646, 19, 49, "Output",ExpressionUUID->"c095bab3-fecb-41e3-b107-909c4aa4cd94",
   CellTags->"b:5.4.0",
   CellID->965703464],
  Cell[1428795, 34857, 745, 23, 65, "Output",ExpressionUUID->"e001e1cf-393f-4b8e-bcdc-d25fd174e905",
   CellTags->"b:5.4.0",
   CellID->2044199069],
  Cell[1429543, 34882, 745, 23, 65, "Output",ExpressionUUID->"5fa16457-a60f-4879-b4a1-c0f667ef7558",
   CellTags->"b:5.4.0",
   CellID->1410367977],
  Cell[1430303, 34908, 276, 7, 34, "Text",ExpressionUUID->"5a62cfea-adc2-4dd8-8d29-031bfcdb1501",
   CellTags->"b:5.4.0",
   CellID->1492977446],
  Cell[1430604, 34919, 534, 12, 113, "Input",ExpressionUUID->"80baf6e5-f1ab-41a7-a52b-169f2630e3de",
   CellTags->"b:5.4.0",
   CellID->1273003374],
  Cell[1431141, 34933, 252, 7, 34, "Output",ExpressionUUID->"6231fc08-3aea-441c-be57-ba2b8f80fa80",
   CellTags->"b:5.4.0",
   CellID->130093853],
  Cell[1431396, 34942, 252, 7, 34, "Output",ExpressionUUID->"5c45bd6c-bca9-448b-82c9-618033c66c04",
   CellTags->"b:5.4.0",
   CellID->349034725],
  Cell[1431651, 34951, 984, 35, 62, "Output",ExpressionUUID->"3fe88c77-124c-49d8-93bf-b6ec35837fb3",
   CellTags->"b:5.4.0",
   CellID->1448783118],
  Cell[1432638, 34988, 271, 7, 34, "Output",ExpressionUUID->"2c33421e-361a-4c29-b258-ad62d6bc6639",
   CellTags->"b:5.4.0",
   CellID->1528274540],
  Cell[1432924, 34998, 592, 16, 37, "Text",ExpressionUUID->"42524f38-3d8c-4854-9cbe-c9f6b6de1db1",
   CellTags->"b:5.4.0",
   CellID->2135615404],
  Cell[1433541, 35018, 2323, 65, 165, "Input",ExpressionUUID->"77bb2d8c-2b3c-44a8-92db-b8d38f5c2a44",
   CellTags->"b:5.4.0",
   CellID->1216838074],
  Cell[1435867, 35085, 8314, 159, 214, "Output",ExpressionUUID->"feea8ca8-ff01-4ca8-92f3-45c13e645ae4",
   CellTags->"b:5.4.0",
   CellID->1805775660],
  Cell[1444184, 35246, 2123, 54, 208, "Output",ExpressionUUID->"79fa83cd-56dd-4800-8ef6-7e455f6e333d",
   CellTags->"b:5.4.0",
   CellID->1432618216],
  Cell[1446322, 35303, 639, 16, 37, "Text",ExpressionUUID->"df8dea70-656e-46f3-bb03-c943dd8e682e",
   CellTags->"b:5.4.0",
   CellID->910739337],
  Cell[1446986, 35323, 2440, 66, 191, "Input",ExpressionUUID->"af3a70ce-64b0-45c5-9ca9-45cbe17841d2",
   CellTags->"b:5.4.0",
   CellID->1680084790],
  Cell[1449429, 35391, 8312, 159, 214, "Output",ExpressionUUID->"9db8fd82-25d8-49fe-b485-436c4c11c959",
   CellTags->"b:5.4.0",
   CellID->427706148],
  Cell[1457744, 35552, 2122, 54, 208, "Output",ExpressionUUID->"82788c0f-78e2-4bb3-a650-24d5436a4dfe",
   CellTags->"b:5.4.0",
   CellID->829462927],
  Cell[1459881, 35609, 373, 9, 34, "Text",ExpressionUUID->"8be4c9db-7c8d-445a-9a95-308257c4b589",
   CellTags->"b:5.4.0",
   CellID->2070074774]},
 "b:5.4.1"->{
  Cell[1460269, 35621, 304, 7, 34, "Text",ExpressionUUID->"3c636c69-1843-4910-9312-31d43c7b56e6",
   CellTags->"b:5.4.1",
   CellID->746807497],
  Cell[1469213, 35819, 1481, 41, 87, "Input",ExpressionUUID->"617bd12c-6b69-4040-9ce9-f8b288633cc5",
   CellTags->"b:5.4.1",
   CellID->1699169011],
  Cell[1470697, 35862, 6700, 135, 220, "Output",ExpressionUUID->"23be53ab-3cf4-4787-b6d3-2b76ba0537c2",
   CellTags->"b:5.4.1",
   CellID->1641400277],
  Cell[1477434, 36002, 1562, 43, 88, "Input",ExpressionUUID->"9b048550-5dd9-4b4f-92e4-f87f9b2f7359",
   CellTags->"b:5.4.1",
   CellID->696030071],
  Cell[1478999, 36047, 20269, 357, 214, "Output",ExpressionUUID->"d6a6bf32-8d2c-425f-9889-ea89084cafea",
   CellTags->"b:5.4.1",
   CellID->439526177],
  Cell[1499283, 36407, 330, 9, 34, "Input",ExpressionUUID->"72fa4c96-e9bd-48e1-809a-4f6bbab4835c",
   CellTags->"b:5.4.1",
   CellID->56085277]},
 "c:4"->{
  Cell[1526113, 36909, 455, 11, 51, "Subsection",ExpressionUUID->"a5042c33-4e8a-4f68-bb87-27a3dcf5679d",
   CellTags->{"c:4", "b:5.4.2"},
   CellID->833978088],
  Cell[1605621, 39254, 524, 12, 67, "Subsection",ExpressionUUID->"414c0236-744a-4774-ac1e-3e43612d0c76",
   CellTags->{"c:4", "b:5.4.2"},
   CellID->426124609],
  Cell[1606367, 39275, 764, 20, 63, "Subsubsection",ExpressionUUID->"53c7bb61-6b78-470f-8814-bcccc172ee80",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->130368513],
  Cell[1636582, 40268, 798, 22, 63, "Subsubsection",ExpressionUUID->"6947aa9b-4ca1-4dd0-8de8-c51dbebff4f5",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->191436130],
  Cell[1655003, 40869, 728, 19, 63, "Subsubsection",ExpressionUUID->"1a8ad92b-cecd-42aa-a3c0-79223df306c8",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->570020487],
  Cell[1668012, 41305, 787, 21, 63, "Subsubsection",ExpressionUUID->"e67ab82c-2523-4967-9f99-ea6fd626de3b",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->273437630]},
 "b:5.4.2"->{
  Cell[1526113, 36909, 455, 11, 51, "Subsection",ExpressionUUID->"a5042c33-4e8a-4f68-bb87-27a3dcf5679d",
   CellTags->{"c:4", "b:5.4.2"},
   CellID->833978088],
  Cell[1526571, 36922, 324, 9, 36, "Text",ExpressionUUID->"da8a42de-319a-4623-8b82-11040cdca14f",
   CellTags->"b:5.4.2",
   CellID->43961047],
  Cell[1526898, 36933, 1076, 32, 38, "Input",ExpressionUUID->"4df0a2e5-26bf-4455-9b40-ebec7c494a13",
   CellTags->"b:5.4.2",
   CellID->118942361],
  Cell[1527977, 36967, 324, 7, 36, "Text",ExpressionUUID->"1878d201-ae56-43da-86fd-3815d5816e49",
   CellTags->"b:5.4.2",
   CellID->1904469617],
  Cell[1533028, 37102, 271, 8, 35, "Input",ExpressionUUID->"0ee2aa6d-b77e-4e12-8557-a2d4db919ff6",
   CellTags->"b:5.4.2",
   CellID->12977146],
  Cell[1533302, 37112, 584, 18, 35, "Input",ExpressionUUID->"738a328c-2255-4956-bde0-759633f7b387",
   CellTags->"b:5.4.2",
   CellID->1211044475],
  Cell[1533889, 37132, 524, 14, 37, "Text",ExpressionUUID->"2df9481b-f8a6-43f7-8749-e9d4c09686c0",
   CellTags->"b:5.4.2",
   CellID->174485059],
  Cell[1534416, 37148, 866, 30, 38, "Input",ExpressionUUID->"941befc2-a701-4368-8e45-f60d660060e6",
   CellTags->"b:5.4.2",
   CellID->1255208458],
  Cell[1535285, 37180, 239, 6, 36, "Text",ExpressionUUID->"e7fd648e-dec6-4364-a986-7e519029b0c5",
   CellTags->"b:5.4.2",
   CellID->1556689125],
  Cell[1535549, 37190, 368, 13, 38, "Input",ExpressionUUID->"81ac92b3-69fb-48f7-be76-2ce01e75a6b2",
   CellTags->"b:5.4.2",
   CellID->480852474],
  Cell[1535920, 37205, 1087, 29, 167, "Output",ExpressionUUID->"d90617c6-0bc0-449a-8d9a-f2788b560f43",
   CellTags->"b:5.4.2",
   CellID->1256785175],
  Cell[1537044, 37239, 529, 19, 67, "Input",ExpressionUUID->"c5494493-5782-4ab1-9b4f-1e25d4175d1d",
   CellTags->"b:5.4.2",
   CellID->706566857],
  Cell[1537576, 37260, 1085, 29, 167, "Output",ExpressionUUID->"1be31ca7-fe0a-4200-b422-653f17ca62f8",
   CellTags->"b:5.4.2",
   CellID->139284333],
  Cell[1538664, 37291, 1132, 31, 167, "Output",ExpressionUUID->"d5ede5ab-72c7-4535-9bd5-15cd117748e9",
   CellTags->"b:5.4.2",
   CellID->194609501],
  Cell[1539833, 37327, 531, 19, 67, "Input",ExpressionUUID->"7cef13a1-3b67-49b3-a815-d7f9fe63bf72",
   CellTags->"b:5.4.2",
   CellID->1832081336],
  Cell[1540367, 37348, 1087, 29, 167, "Output",ExpressionUUID->"31fa6b93-5c2d-4338-8283-1820a6c3b534",
   CellTags->"b:5.4.2",
   CellID->184288926],
  Cell[1541457, 37379, 1135, 31, 167, "Output",ExpressionUUID->"c401c446-2c8d-4f72-b456-8ed523cd5752",
   CellTags->"b:5.4.2",
   CellID->672560308],
  Cell[1542629, 37415, 531, 19, 67, "Input",ExpressionUUID->"4d247d11-efa0-4a43-8d56-eae24f3433b3",
   CellTags->"b:5.4.2",
   CellID->1170483466],
  Cell[1543163, 37436, 1088, 29, 167, "Output",ExpressionUUID->"ba5be03f-1f6c-49be-b532-b7c935139c05",
   CellTags->"b:5.4.2",
   CellID->1514662395],
  Cell[1544254, 37467, 1133, 31, 167, "Output",ExpressionUUID->"827f1198-eff5-4659-b607-6fb4ca53ae2a",
   CellTags->"b:5.4.2",
   CellID->659721227],
  Cell[1545402, 37501, 568, 15, 108, "Text",ExpressionUUID->"3758cfd0-0967-4f6c-8fde-b4f4ccd45ece",
   CellTags->"b:5.4.2",
   CellID->1365406542],
  Cell[1545995, 37520, 653, 21, 67, "Input",ExpressionUUID->"a9cccde9-0e06-4373-be0b-e2b544788507",
   CellTags->"b:5.4.2",
   CellID->377499552],
  Cell[1546651, 37543, 1086, 29, 167, "Output",ExpressionUUID->"95182a7c-a2a4-46e5-85ad-8cbe5f15812a",
   CellTags->"b:5.4.2",
   CellID->476592728],
  Cell[1547764, 37576, 495, 11, 43, "Text",ExpressionUUID->"bd119060-d86e-41e4-8929-b70e47e43d1e",
   CellTags->"b:5.4.2",
   CellID->244036809],
  Cell[1549106, 37616, 505, 11, 43, "Text",ExpressionUUID->"e6665051-a251-4272-b97c-76fcc830ab4b",
   CellTags->"b:5.4.2",
   CellID->463033963],
  Cell[1549614, 37629, 1541, 44, 85, "Input",ExpressionUUID->"3e6d7750-5483-4924-a97c-43464fe66bfa",
   CellTags->"b:5.4.2",
   CellID->204260416],
  Cell[1551158, 37675, 496, 11, 43, "Text",ExpressionUUID->"6ec9e7e1-20e9-4661-974d-c76ffdece25c",
   CellTags->"b:5.4.2",
   CellID->2108443174],
  Cell[1553157, 37737, 545, 12, 43, "Text",ExpressionUUID->"07e77b04-3689-4544-a342-40e40e470fa5",
   CellTags->"b:5.4.2",
   CellID->553054382],
  Cell[1561684, 37992, 564, 15, 110, "Input",ExpressionUUID->"522569ab-0bae-4c9c-ad6c-4ce4a6fef04d",
   CellTags->"b:5.4.2",
   CellID->952133089],
  Cell[1562251, 38009, 256, 7, 34, "Output",ExpressionUUID->"22b87560-7ac8-41e5-89d9-373d5c202adf",
   CellTags->"b:5.4.2",
   CellID->2003097331],
  Cell[1562510, 38018, 272, 8, 34, "Output",ExpressionUUID->"fca743f1-4e7d-4848-a5c5-a302b8e1988b",
   CellTags->"b:5.4.2",
   CellID->2103191745],
  Cell[1562785, 38028, 271, 8, 34, "Output",ExpressionUUID->"65d05e81-7873-4e78-b549-60d22c9be11f",
   CellTags->"b:5.4.2",
   CellID->762828595],
  Cell[1563059, 38038, 271, 8, 34, "Output",ExpressionUUID->"703dd229-0561-4ea1-835b-6cfcc1d2e55c",
   CellTags->"b:5.4.2",
   CellID->382823575],
  Cell[1563345, 38049, 242, 6, 36, "Text",ExpressionUUID->"977b8eb5-5c0f-4d4c-a694-41f04f4645ad",
   CellTags->"b:5.4.2",
   CellID->1600173026],
  Cell[1563612, 38059, 1341, 38, 73, "Input",ExpressionUUID->"0bc25726-f7ea-4a0a-b1e0-08e28ba068bb",
   CellTags->"b:5.4.2",
   CellID->1170336601],
  Cell[1564956, 38099, 256, 7, 34, "Output",ExpressionUUID->"8e3fa3de-c761-4e65-8643-6a5495160947",
   CellTags->"b:5.4.2",
   CellID->254818613],
  Cell[1565227, 38109, 259, 6, 36, "Text",ExpressionUUID->"930c24ab-5283-409a-b704-a75e9993ba2f",
   CellTags->"b:5.4.2",
   CellID->1044530134],
  Cell[1565511, 38119, 1692, 52, 64, "Input",ExpressionUUID->"40e3c148-b675-404d-acfb-a7388342a736",
   CellTags->"b:5.4.2",
   CellID->1886631560],
  Cell[1567206, 38173, 256, 7, 34, "Output",ExpressionUUID->"0d6d2a04-6bb1-4501-a733-dc0ad4f17055",
   CellTags->"b:5.4.2",
   CellID->654036573],
  Cell[1579510, 38475, 260, 6, 36, "Text",ExpressionUUID->"df6bbfc4-3200-4263-b932-13ddd21656fb",
   CellTags->"b:5.4.2",
   CellID->952390166],
  Cell[1579795, 38485, 453, 13, 36, "Input",ExpressionUUID->"fbd59ee9-7d97-4612-b8ab-1ae549d780fc",
   CellTags->"b:5.4.2",
   CellID->1625898871],
  Cell[1580251, 38500, 253, 7, 34, "Output",ExpressionUUID->"a2a088b2-718e-4878-a017-8c620c54a7bf",
   CellTags->"b:5.4.2",
   CellID->380440649],
  Cell[1580519, 38510, 395, 10, 60, "Text",ExpressionUUID->"c79efdd7-3e4c-47c5-a57f-94b4585fe558",
   CellTags->"b:5.4.2",
   CellID->1900400469],
  Cell[1592403, 38809, 623, 21, 38, "Input",ExpressionUUID->"95c3c0c2-992e-42e4-934e-f3b44628bf0e",
   CellTags->"b:5.4.2",
   CellID->1456718161],
  Cell[1594175, 38872, 325, 10, 35, "Input",ExpressionUUID->"a1a441ab-f990-4bd7-814e-23f73d610cef",
   CellTags->"b:5.4.2",
   CellID->1816600941],
  Cell[1594503, 38884, 770, 23, 99, "Output",ExpressionUUID->"6e24f2e0-7271-48dc-ab0f-7f7956320d28",
   CellTags->"b:5.4.2",
   CellID->1831573051],
  Cell[1595310, 38912, 501, 13, 61, "Input",ExpressionUUID->"dd1af27d-069c-4786-81fa-eb230de472ef",
   CellTags->"b:5.4.2",
   CellID->837005988],
  Cell[1595814, 38927, 770, 23, 99, "Output",ExpressionUUID->"55c9c578-e3c5-4fb9-aa92-da67b44425f9",
   CellTags->"b:5.4.2",
   CellID->1217672059],
  Cell[1596587, 38952, 2732, 81, 171, "Output",ExpressionUUID->"d79c2b96-af24-4422-8ef5-9518fc6ed155",
   CellTags->"b:5.4.2",
   CellID->2042623350],
  Cell[1599334, 39036, 247, 6, 36, "Text",ExpressionUUID->"259014a0-bd07-4247-9f1c-e84000bcf393",
   CellTags->"b:5.4.2",
   CellID->1541752466],
  Cell[1599606, 39046, 344, 9, 35, "Input",ExpressionUUID->"164485b6-37ff-40e0-b848-d50804f50944",
   CellTags->"b:5.4.2",
   CellID->696011360],
  Cell[1599953, 39057, 252, 7, 34, "Output",ExpressionUUID->"5a505f1b-9620-4af7-bae7-ea0326d3e573",
   CellTags->"b:5.4.2",
   CellID->697062508],
  Cell[1600220, 39067, 371, 10, 36, "Text",ExpressionUUID->"3de15fce-e7ca-4142-8ce7-714410eac64d",
   CellTags->"b:5.4.2",
   CellID->519502311],
  Cell[1600594, 39079, 283, 8, 36, "Text",ExpressionUUID->"da905970-b161-4ff7-bca5-43d093434d63",
   CellTags->"b:5.4.2",
   CellID->1754315922],
  Cell[1600902, 39091, 570, 17, 63, "Input",ExpressionUUID->"6636c4d3-44d9-4d77-b7f7-f0d9e2ceb1b2",
   CellTags->"b:5.4.2",
   CellID->1984711391],
  Cell[1601475, 39110, 696, 21, 65, "Output",ExpressionUUID->"810fec02-6154-44a4-b30b-580054fd8b0b",
   CellTags->"b:5.4.2",
   CellID->1006277571],
  Cell[1602174, 39133, 256, 7, 34, "Output",ExpressionUUID->"41a6dcc5-d9af-4122-b826-8180203d90ed",
   CellTags->"b:5.4.2",
   CellID->1073415672],
  Cell[1602467, 39145, 597, 18, 63, "Input",ExpressionUUID->"892fa14c-568c-4f67-8916-885ec52497db",
   CellTags->"b:5.4.2",
   CellID->180859686],
  Cell[1603067, 39165, 696, 21, 65, "Output",ExpressionUUID->"b5280a49-784e-49a2-93e0-0cb136d74e8b",
   CellTags->"b:5.4.2",
   CellID->2034564504],
  Cell[1603766, 39188, 253, 7, 34, "Output",ExpressionUUID->"0c73afa4-8417-4be9-872e-b10e0d0c5765",
   CellTags->"b:5.4.2",
   CellID->543507547],
  Cell[1604056, 39200, 570, 17, 63, "Input",ExpressionUUID->"76f24b89-53fa-43d7-98d1-b201a608ff35",
   CellTags->"b:5.4.2",
   CellID->1019382097],
  Cell[1604629, 39219, 698, 21, 65, "Output",ExpressionUUID->"760f1d12-a2f9-4a5c-982b-8050367e7585",
   CellTags->"b:5.4.2",
   CellID->1483391738],
  Cell[1605330, 39242, 254, 7, 34, "Output",ExpressionUUID->"7195585d-466a-4c9d-ac84-e2205237cf70",
   CellTags->"b:5.4.2",
   CellID->1227282543],
  Cell[1605621, 39254, 524, 12, 67, "Subsection",ExpressionUUID->"414c0236-744a-4774-ac1e-3e43612d0c76",
   CellTags->{"c:4", "b:5.4.2"},
   CellID->426124609]},
 "b:5.4.3"->{
  Cell[1606367, 39275, 764, 20, 63, "Subsubsection",ExpressionUUID->"53c7bb61-6b78-470f-8814-bcccc172ee80",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->130368513],
  Cell[1607134, 39297, 717, 16, 63, "Text",ExpressionUUID->"3e4c9e2a-34aa-4c03-9ca0-1ebf80df2c75",
   CellTags->"b:5.4.3",
   CellID->670278051],
  Cell[1607854, 39315, 1444, 46, 154, "Text",ExpressionUUID->"733da3eb-a9f9-4fcf-98e4-3f623293f0f0",
   CellTags->"b:5.4.3",
   CellID->1811474642],
  Cell[1609301, 39363, 390, 9, 34, "Text",ExpressionUUID->"7506d326-4e33-4f99-af73-1e7d97056dfd",
   CellTags->"b:5.4.3",
   CellID->578644575],
  Cell[1609694, 39374, 345, 8, 34, "Text",ExpressionUUID->"c95cba18-4335-45f4-ac0e-dd1aa47ec390",
   CellTags->"b:5.4.3",
   CellID->1473418409],
  Cell[1610064, 39386, 439, 14, 35, "Input",ExpressionUUID->"62e71298-7a55-4264-844a-b96c43876f42",
   CellTags->"b:5.4.3",
   CellID->1109266506],
  Cell[1610506, 39402, 1102, 39, 123, "Output",ExpressionUUID->"2246db1f-bd02-4529-a8d7-97a97767df17",
   CellTags->"b:5.4.3",
   CellID->357270766],
  Cell[1611623, 39444, 287, 7, 34, "Text",ExpressionUUID->"dab60708-b3ab-4253-bc31-d3f745b179cb",
   CellTags->"b:5.4.3",
   CellID->1136626735],
  Cell[1611913, 39453, 935, 30, 36, "Input",ExpressionUUID->"4dc9f31b-e280-4e97-bf80-6c70fff92811",
   CellTags->"b:5.4.3",
   CellID->1418495080],
  Cell[1612851, 39485, 733, 21, 40, "Text",ExpressionUUID->"331d6f83-cdda-4eb6-b996-73efe5159ac2",
   CellTags->"b:5.4.3",
   CellID->1430844213],
  Cell[1613609, 39510, 621, 18, 117, "Input",ExpressionUUID->"78495325-4b57-45c4-bd14-ab85a4d10b84",
   CellTags->"b:5.4.3",
   CellID->676808785],
  Cell[1614233, 39530, 818, 25, 99, "Output",ExpressionUUID->"9a0d5cba-a077-46cd-9ff1-5e5929ba9021",
   CellTags->"b:5.4.3",
   CellID->285487150],
  Cell[1615054, 39557, 818, 25, 99, "Output",ExpressionUUID->"2f77fa47-da39-4cdb-a045-8d67cd9433ac",
   CellTags->"b:5.4.3",
   CellID->704062652],
  Cell[1615875, 39584, 866, 27, 99, "Output",ExpressionUUID->"7786012e-f47f-4fb7-bc0a-b2e6ec3d921a",
   CellTags->"b:5.4.3",
   CellID->546097194],
  Cell[1616744, 39613, 865, 27, 99, "Output",ExpressionUUID->"838c3dde-cfb6-4788-bed0-30e995d60af6",
   CellTags->"b:5.4.3",
   CellID->2057915927],
  Cell[1617624, 39643, 611, 12, 56, "Text",ExpressionUUID->"0326e2b2-044f-409b-9e43-15ec86bbb1b5",
   CellTags->"b:5.4.3",
   CellID->2051854983],
  Cell[1618238, 39657, 1764, 55, 41, "Input",ExpressionUUID->"2709db0f-0240-467b-81e1-04bc8f29d496",
   CellTags->"b:5.4.3",
   CellID->1234295755],
  Cell[1620005, 39714, 394, 9, 34, "Text",ExpressionUUID->"ebe28e0d-1698-4d21-a0c1-bda102f17bb0",
   CellTags->"b:5.4.3",
   CellID->116724265],
  Cell[1620424, 39727, 502, 14, 36, "Input",ExpressionUUID->"f1b0bcf9-449a-4cdf-97c8-df7835258dea",
   CellTags->"b:5.4.3",
   CellID->1132152144],
  Cell[1620929, 39743, 3043, 101, 119, "Output",ExpressionUUID->"11a505dd-2edf-42aa-a15e-61b2bb6b696c",
   CellTags->"b:5.4.3",
   CellID->21366507],
  Cell[1624747, 39878, 342, 8, 34, "Text",ExpressionUUID->"edb1ff14-41fd-4c11-9759-a8d5538c5563",
   CellTags->"b:5.4.3",
   CellID->1375971788],
  Cell[1625114, 39890, 917, 27, 117, "Input",ExpressionUUID->"26e0210e-ea4e-43f9-aa94-1d2d0a7eae31",
   CellTags->"b:5.4.3",
   CellID->1853930020],
  Cell[1626034, 39919, 1028, 31, 115, "Output",ExpressionUUID->"6aa1cd13-b9ac-4aa2-bf7b-216ff9df1e97",
   CellTags->"b:5.4.3",
   CellID->152740802],
  Cell[1627065, 39952, 1031, 31, 115, "Output",ExpressionUUID->"8c7f8ee0-0930-41be-9acf-5f557d0e6327",
   CellTags->"b:5.4.3",
   CellID->871701049],
  Cell[1628099, 39985, 1075, 33, 115, "Output",ExpressionUUID->"89f954d2-3d7b-475d-bfe7-954d90500523",
   CellTags->"b:5.4.3",
   CellID->546090709],
  Cell[1629177, 40020, 1078, 33, 115, "Output",ExpressionUUID->"f6aa19d9-0d10-43ed-976d-95419374dd23",
   CellTags->"b:5.4.3",
   CellID->1948922532],
  Cell[1630270, 40056, 343, 8, 34, "Text",ExpressionUUID->"96e7758e-4e27-4f55-8e83-30774e3d9c88",
   CellTags->"b:5.4.3",
   CellID->1161611817],
  Cell[1630638, 40068, 1682, 51, 117, "Input",ExpressionUUID->"df10aa0a-fbde-4aec-be20-7ba91ac87bb7",
   CellTags->"b:5.4.3",
   CellID->435973442],
  Cell[1632323, 40121, 1054, 34, 115, "Output",ExpressionUUID->"9293671e-a1b6-43fe-9ef7-285013f1a3c0",
   CellTags->"b:5.4.3",
   CellID->2120411280],
  Cell[1633380, 40157, 1053, 34, 115, "Output",ExpressionUUID->"784b0996-2f19-4851-8db3-caef6ca4f321",
   CellTags->"b:5.4.3",
   CellID->261260995],
  Cell[1634436, 40193, 1053, 34, 115, "Output",ExpressionUUID->"87dd2014-db60-4d61-bd4e-dbd24ee203f3",
   CellTags->"b:5.4.3",
   CellID->1556968497],
  Cell[1635492, 40229, 1053, 34, 115, "Output",ExpressionUUID->"b09f40a2-4de6-4a67-8932-37369727080d",
   CellTags->"b:5.4.3",
   CellID->881290384],
  Cell[1636582, 40268, 798, 22, 63, "Subsubsection",ExpressionUUID->"6947aa9b-4ca1-4dd0-8de8-c51dbebff4f5",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->191436130],
  Cell[1637383, 40292, 306, 9, 60, "Text",ExpressionUUID->"25513e3a-3ad4-4cf8-aad2-984c588fa77e",
   CellTags->"b:5.4.3",
   CellID->536533704],
  Cell[1637714, 40305, 2710, 86, 227, "Input",ExpressionUUID->"84e13ff0-c022-4ed0-ac70-ece8453fc60c",
   CellTags->"b:5.4.3",
   CellID->238129375],
  Cell[1640427, 40393, 697, 22, 65, "Output",ExpressionUUID->"059d97b3-0f09-4150-956d-4506533073f0",
   CellTags->"b:5.4.3",
   CellID->652770529],
  Cell[1641127, 40417, 698, 22, 65, "Output",ExpressionUUID->"a2f15c06-716c-4cb5-be38-c86a88126d6d",
   CellTags->"b:5.4.3",
   CellID->1767812027],
  Cell[1641828, 40441, 1087, 34, 75, "Output",ExpressionUUID->"3cd2e40f-73a4-4108-84ca-b28b81b118a6",
   CellTags->"b:5.4.3",
   CellID->1180481544],
  Cell[1646615, 40591, 316, 9, 60, "Text",ExpressionUUID->"0b7b1077-c52e-46d0-8691-32716380f856",
   CellTags->"b:5.4.3",
   CellID->1478371264],
  Cell[1646934, 40602, 249, 6, 36, "Text",ExpressionUUID->"cfa49947-2cd4-44c7-81d8-d28852dc3ccd",
   CellTags->"b:5.4.3",
   CellID->600038258],
  Cell[1647186, 40610, 229, 6, 36, "Text",ExpressionUUID->"b44d39ff-9c0c-4689-b209-569ff5d5a465",
   CellTags->"b:5.4.3",
   CellID->259795894],
  Cell[1647440, 40620, 877, 29, 67, "Input",ExpressionUUID->"9431eeb6-cbe4-47c9-ab1d-86cf3444da46",
   CellTags->"b:5.4.3",
   CellID->555336107],
  Cell[1648320, 40651, 884, 29, 49, "Output",ExpressionUUID->"4992e30f-78ea-47f6-9f6e-d07070906ed7",
   CellTags->"b:5.4.3",
   CellID->518810099],
  Cell[1649207, 40682, 5462, 172, 166, "Output",ExpressionUUID->"6500a847-645a-4f95-a183-c4ced22f4aa2",
   CellTags->"b:5.4.3",
   CellID->857727073],
  Cell[1655003, 40869, 728, 19, 63, "Subsubsection",ExpressionUUID->"1a8ad92b-cecd-42aa-a3c0-79223df306c8",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->570020487],
  Cell[1655734, 40890, 342, 10, 60, "Text",ExpressionUUID->"e0391405-ccc3-4cec-951f-aaa3090bd777",
   CellTags->"b:5.4.3",
   CellID->409775742],
  Cell[1656101, 40904, 1202, 36, 104, "Input",ExpressionUUID->"c08e96fa-eb3b-4eec-a51a-da09983ba1e5",
   CellTags->"b:5.4.3",
   CellID->120839865],
  Cell[1657306, 40942, 3045, 101, 119, "Output",ExpressionUUID->"0f838e6a-e042-4b7b-856a-d7d887d6dd75",
   CellTags->"b:5.4.3",
   CellID->1908677247],
  Cell[1660354, 41045, 253, 7, 34, "Output",ExpressionUUID->"cb1e0d7a-02ec-45a7-adbc-6cf16c01189d",
   CellTags->"b:5.4.3",
   CellID->27771443],
  Cell[1660622, 41055, 1892, 64, 63, "Text",ExpressionUUID->"d3676458-3b45-4bc7-8c58-ce02f3bf2925",
   CellTags->"b:5.4.3",
   CellID->1583564640],
  Cell[1662539, 41123, 1658, 52, 62, "Input",ExpressionUUID->"98f99a8c-dba7-4eed-ac34-06853fcffa2c",
   CellTags->"b:5.4.3",
   CellID->1206067209],
  Cell[1664200, 41177, 3042, 99, 119, "Output",ExpressionUUID->"b56d98af-5fc8-430a-9bf7-157bf5e99d8b",
   CellTags->"b:5.4.3",
   CellID->750691102],
  Cell[1667245, 41278, 718, 21, 83, "Output",ExpressionUUID->"99438918-f05b-4c0c-9875-cfc7cb0ac348",
   CellTags->"b:5.4.3",
   CellID->398925189],
  Cell[1668012, 41305, 787, 21, 63, "Subsubsection",ExpressionUUID->"e67ab82c-2523-4967-9f99-ea6fd626de3b",
   CellTags->{"c:4", "b:5.4.3"},
   CellID->273437630],
  Cell[1668802, 41328, 217, 6, 36, "Text",ExpressionUUID->"a6a36d51-1715-443f-a01f-593e1213aeb8",
   CellTags->"b:5.4.3",
   CellID->51242830],
  Cell[1669044, 41338, 2057, 67, 144, "Input",ExpressionUUID->"b1a8498e-acc3-4a89-9bf4-91237a84963e",
   CellTags->"b:5.4.3",
   CellID->97301363],
  Cell[1671104, 41407, 884, 29, 49, "Output",ExpressionUUID->"fdf20322-e646-4d7f-8fa6-9fd722a73031",
   CellTags->"b:5.4.3",
   CellID->291705589],
  Cell[1671991, 41438, 409, 13, 49, "Output",ExpressionUUID->"bc2897c3-bcb6-401b-8e06-ed3010031d92",
   CellTags->"b:5.4.3",
   CellID->2138151869],
  Cell[1672403, 41453, 409, 13, 49, "Output",ExpressionUUID->"2a1d84da-f58f-4610-b135-ce15c846b56c",
   CellTags->"b:5.4.3",
   CellID->2074597032],
  Cell[1672815, 41468, 409, 13, 49, "Output",ExpressionUUID->"44cb1c0f-b233-4adb-bbae-74fee5dd4cff",
   CellTags->"b:5.4.3",
   CellID->1207824132],
  Cell[1673227, 41483, 409, 13, 49, "Output",ExpressionUUID->"77314bb1-32f5-4cfe-9785-e8073ccd8fec",
   CellTags->"b:5.4.3",
   CellID->1884755080],
  Cell[1673651, 41499, 220, 6, 36, "Text",ExpressionUUID->"cefebd86-28f5-4eab-aa1e-336911a2455c",
   CellTags->"b:5.4.3",
   CellID->1108952574],
  Cell[1673896, 41509, 1880, 60, 117, "Input",ExpressionUUID->"0dd26462-b5d4-49a5-bd3b-3efd653045a4",
   CellTags->"b:5.4.3",
   CellID->155338905],
  Cell[1675779, 41571, 1110, 36, 62, "Output",ExpressionUUID->"25c9f823-d533-4774-8dc2-b9992be0d3ca",
   CellTags->"b:5.4.3",
   CellID->1694836310],
  Cell[1676892, 41609, 1097, 36, 62, "Output",ExpressionUUID->"778386cc-ed4d-4470-89a4-d6d665655bb5",
   CellTags->"b:5.4.3",
   CellID->890768362],
  Cell[1677992, 41647, 1096, 36, 62, "Output",ExpressionUUID->"24584b6d-4627-4b7e-ab26-58b11e34eb84",
   CellTags->"b:5.4.3",
   CellID->1035356897],
  Cell[1679091, 41685, 1109, 36, 62, "Output",ExpressionUUID->"3a0df6d0-c78c-4a9b-9955-616079962c4e",
   CellTags->"b:5.4.3",
   CellID->183376238],
  Cell[1680215, 41724, 245, 6, 36, "Text",ExpressionUUID->"da296afa-186b-460c-a66e-e6358a9e2da4",
   CellTags->"b:5.4.3",
   CellID->1588345230],
  Cell[1680463, 41732, 834, 25, 36, "Input",ExpressionUUID->"f6a27bca-82b5-4d86-9714-2f3a1d287da8",
   CellTags->"b:5.4.3",
   CellID->2096548400],
  Cell[1681322, 41761, 656, 17, 35, "Input",ExpressionUUID->"37e9de03-3d22-4a96-8b16-c9c988b538b5",
   CellTags->"b:5.4.3",
   CellID->777278824],
  Cell[1681981, 41780, 5341, 110, 203, "Output",ExpressionUUID->"8741ff0e-3ed6-4189-a4f2-cc9a0f588e22",
   CellTags->"b:5.4.3",
   CellID->365398621],
  Cell[1687359, 41895, 655, 17, 35, "Input",ExpressionUUID->"c2d3bae5-6c6b-48c1-a331-7d853d4ac2b6",
   CellTags->"b:5.4.3",
   CellID->582170267],
  Cell[1688017, 41914, 5344, 110, 203, "Output",ExpressionUUID->"3e89a63c-8cb4-48da-8aa1-af26d97f52ed",
   CellTags->"b:5.4.3",
   CellID->1598876374],
  Cell[1693398, 42029, 659, 17, 35, "Input",ExpressionUUID->"ef6cb6ed-a00c-4c27-9cc5-ec26a51b165f",
   CellTags->"b:5.4.3",
   CellID->600181737],
  Cell[1694060, 42048, 5341, 110, 203, "Output",ExpressionUUID->"ab5f7a73-f80e-4460-bec2-4ffd77669feb",
   CellTags->"b:5.4.3",
   CellID->852342907],
  Cell[1699438, 42163, 655, 17, 35, "Input",ExpressionUUID->"0a0181f7-446e-4a60-8f6c-ffb24714b81e",
   CellTags->"b:5.4.3",
   CellID->523763800],
  Cell[1700096, 42182, 5342, 110, 203, "Output",ExpressionUUID->"0f75bffc-8a28-4ac9-a481-a364356055c1",
   CellTags->"b:5.4.3",
   CellID->1611312459],
  Cell[1705453, 42295, 239, 6, 36, "Text",ExpressionUUID->"9fd76b76-27ba-4378-899b-dc012d6a4464",
   CellTags->"b:5.4.3",
   CellID->362717547],
  Cell[1705695, 42303, 224, 6, 36, "Text",ExpressionUUID->"341f08e1-f1d6-47d4-9ece-4c66acef8901",
   CellTags->"b:5.4.3",
   CellID->1635222243],
  Cell[1705944, 42313, 807, 26, 62, "Input",ExpressionUUID->"30bc5f39-b9dd-4fa0-a3bd-7d1bb4589e5b",
   CellTags->"b:5.4.3",
   CellID->1832007442],
  Cell[1706754, 42341, 696, 21, 65, "Output",ExpressionUUID->"7ee00b8f-0656-42b1-b937-d3bc166554c8",
   CellTags->"b:5.4.3",
   CellID->1559709743],
  Cell[1707453, 42364, 698, 22, 65, "Output",ExpressionUUID->"1ad27cd9-e285-4e1b-b583-2577121083a1",
   CellTags->"b:5.4.3",
   CellID->1856255319],
  Cell[1708166, 42389, 234, 6, 36, "Text",ExpressionUUID->"db46e971-d383-4dd1-8ed6-843d10cac042",
   CellTags->"b:5.4.3",
   CellID->2076030655],
  Cell[1708425, 42399, 404, 10, 61, "Input",ExpressionUUID->"03277937-9bb1-4988-bc4c-4afddb197358",
   CellTags->"b:5.4.3",
   CellID->1680405776],
  Cell[1708832, 42411, 254, 7, 34, "Output",ExpressionUUID->"79d69b1b-05f0-49b0-85b6-ae2f72cb87e5",
   CellTags->"b:5.4.3",
   CellID->1588206464],
  Cell[1709089, 42420, 277, 8, 49, "Output",ExpressionUUID->"44a01e06-e0f0-428b-9690-013da4988e81",
   CellTags->"b:5.4.3",
   CellID->1911791885],
  Cell[1709381, 42431, 224, 6, 36, "Text",ExpressionUUID->"867a4f3e-0dda-4297-b8e0-1ca0f04c975e",
   CellTags->"b:5.4.3",
   CellID->1022435535],
  Cell[1709630, 42441, 875, 27, 62, "Input",ExpressionUUID->"f67fdb61-1dfb-46ae-9a07-840a31d6379e",
   CellTags->"b:5.4.3",
   CellID->2057377098],
  Cell[1710508, 42470, 695, 21, 65, "Output",ExpressionUUID->"fd2ef314-c7f2-44b7-bda8-d8b6e2bfcc88",
   CellTags->"b:5.4.3",
   CellID->835884781],
  Cell[1711206, 42493, 699, 22, 65, "Output",ExpressionUUID->"ca665d5a-7bf5-4c09-8acc-59f681ed7af9",
   CellTags->"b:5.4.3",
   CellID->2024979240],
  Cell[1711920, 42518, 234, 6, 36, "Text",ExpressionUUID->"60a190fa-0f41-492a-a4b0-d9b14ff31f86",
   CellTags->"b:5.4.3",
   CellID->1702685739],
  Cell[1712179, 42528, 404, 10, 61, "Input",ExpressionUUID->"6d69bb75-b418-4a88-807c-ef64189740d2",
   CellTags->"b:5.4.3",
   CellID->1819619098],
  Cell[1712586, 42540, 253, 7, 34, "Output",ExpressionUUID->"106d5702-30a6-4174-821d-be14be182ea7",
   CellTags->"b:5.4.3",
   CellID->505249024],
  Cell[1712842, 42549, 274, 8, 49, "Output",ExpressionUUID->"9c858dd8-9c76-49f5-a131-c4a9b34bd516",
   CellTags->"b:5.4.3",
   CellID->570006130],
  Cell[1713131, 42560, 228, 6, 36, "Text",ExpressionUUID->"837193cc-8743-4550-9e3c-3db6751fb592",
   CellTags->"b:5.4.3",
   CellID->23563159],
  Cell[1713384, 42570, 464, 14, 61, "Input",ExpressionUUID->"d94cc086-6829-439a-8eb8-cfe17ac3436c",
   CellTags->"b:5.4.3",
   CellID->889022265],
  Cell[1713851, 42586, 737, 23, 65, "Output",ExpressionUUID->"16513776-9138-4a28-8896-c11073c60b07",
   CellTags->"b:5.4.3",
   CellID->168477643],
  Cell[1714591, 42611, 738, 23, 65, "Output",ExpressionUUID->"ae96a3f4-e55c-4c22-a04f-e72aeac07731",
   CellTags->"b:5.4.3",
   CellID->1220299155],
  Cell[1715344, 42637, 251, 6, 36, "Text",ExpressionUUID->"7c7b7e20-0feb-4575-86b1-a2b18eb01871",
   CellTags->"b:5.4.3",
   CellID->1753475676],
  Cell[1715620, 42647, 650, 21, 36, "Input",ExpressionUUID->"17b1bf7f-cbc3-4e01-9628-88d401078e5f",
   CellTags->"b:5.4.3",
   CellID->747598751],
  Cell[1716273, 42670, 1054, 34, 115, "Output",ExpressionUUID->"f08a5cd2-83ab-427c-b227-0bbbc7fd8f1d",
   CellTags->"b:5.4.3",
   CellID->1099069235],
  Cell[1717364, 42709, 533, 15, 110, "Input",ExpressionUUID->"3cb795ea-91c1-4823-bd2b-d9e192062301",
   CellTags->"b:5.4.3",
   CellID->2112873107],
  Cell[1717900, 42726, 309, 9, 49, "Output",ExpressionUUID->"b53802d1-e421-495a-b1ea-95b2e4a2fdb9",
   CellTags->"b:5.4.3",
   CellID->1549784798],
  Cell[1718212, 42737, 306, 9, 49, "Output",ExpressionUUID->"1e4fd516-1ab2-4abb-81cd-047e6d4dd910",
   CellTags->"b:5.4.3",
   CellID->403945523],
  Cell[1718521, 42748, 306, 9, 49, "Output",ExpressionUUID->"2ffc1540-7700-4c76-bb68-d45039251f7e",
   CellTags->"b:5.4.3",
   CellID->916596894],
  Cell[1718830, 42759, 384, 12, 49, "Output",ExpressionUUID->"5d948752-e5e0-4e09-a783-c7a9383747aa",
   CellTags->"b:5.4.3",
   CellID->1343797527],
  Cell[1719251, 42776, 1292, 41, 35, "Input",ExpressionUUID->"c13bdabe-e2ad-453f-90b2-641b5341c13c",
   CellTags->"b:5.4.3",
   CellID->268722557],
  Cell[1720546, 42819, 773, 24, 52, "Output",ExpressionUUID->"bfad8327-6fa6-4cc2-ad29-f654e5519f14",
   CellTags->"b:5.4.3",
   CellID->119520790],
  Cell[1721356, 42848, 538, 15, 35, "Input",ExpressionUUID->"4922c4a4-3628-4490-8f52-3df4f5f87976",
   CellTags->"b:5.4.3",
   CellID->261905858],
  Cell[1721897, 42865, 5340, 110, 203, "Output",ExpressionUUID->"e4975683-94f7-451b-a862-68c0854edbde",
   CellTags->"b:5.4.3",
   CellID->888707585],
  Cell[1727274, 42980, 412, 13, 35, "Input",ExpressionUUID->"157fa8d6-6666-4815-abe4-4537b24f18d9",
   CellTags->"b:5.4.3",
   CellID->1657762703],
  Cell[1727689, 42995, 255, 7, 34, "Output",ExpressionUUID->"af262b2c-d126-4a5e-9bf4-c7d31f22ca3b",
   CellTags->"b:5.4.3",
   CellID->759616210]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"b:5.0", 1855744, 45528},
 {"b:5.1.0", 1857475, 45565},
 {"i:4", 1868749, 45797},
 {"b:5.1.1", 1869692, 45816},
 {"b:5.1.2", 1874829, 45922},
 {"b:5.1.3", 1877764, 45983},
 {"b:5.1.4", 1881897, 46068},
 {"c:10", 1888826, 46210},
 {"b:5.2.0", 1889004, 46214},
 {"i:2", 1895556, 46347},
 {"i:1", 1895884, 46354},
 {"c:11", 1896404, 46364},
 {"b:5.2.1", 1896588, 46368},
 {"c:12", 1898388, 46405},
 {"b:5.2.2", 1898572, 46409},
 {"c:13", 1901402, 46467},
 {"b:5.2.3", 1901587, 46471},
 {"Measurements", 1902811, 46496},
 {"c:15", 1903194, 46503},
 {"b:5.3.0", 1903387, 46507},
 {"MeasureKet", 1905942, 46559},
 {"c:16", 1906147, 46563},
 {"b:5.3.1", 1906355, 46567},
 {"c:17", 1908769, 46616},
 {"b:5.3.2", 1909110, 46623},
 {"c:19", 1912690, 46696},
 {"b:5.4.0", 1912866, 46700},
 {"b:5.4.1", 1916305, 46770},
 {"c:4", 1917207, 46789},
 {"b:5.4.2", 1918200, 46808},
 {"b:5.4.3", 1927693, 47001}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 266, 7, 68, "Title",ExpressionUUID->"646b22a5-a59f-4e8c-8c02-c28304c26aac",
 CellID->1427136295],
Cell[CellGroupData[{
Cell[871, 33, 411, 12, 85, "Subsubsection",ExpressionUUID->"7ea05f2a-081f-4e49-aa3a-fedd3a4205e0",
 CellID->1444391315],
Cell[1285, 47, 2689, 44, 519, "Abstract",ExpressionUUID->"0e4513ec-b784-4e44-b9be-a5d64db7d3b8",
 CellID->634856722]
}, Open  ]]
}, Open  ]],
Cell[4001, 95, 290, 7, 52, "Section",ExpressionUUID->"462e26e4-0666-4afe-bbb0-339dd94daf7f",
 CellID->2146841522],
Cell[4294, 104, 1516, 25, 152, "Text",ExpressionUUID->"5b410322-fc74-464c-8761-54bddc764685",
 CellTags->"b:5.0",
 CellID->1158596779],
Cell[5813, 131, 328, 8, 72, "Section",ExpressionUUID->"ffc28544-b529-4787-acca-cdd8ac95f2ce",
 CellID->263940212],
Cell[6144, 141, 767, 18, 55, "Abstract",ExpressionUUID->"2b461a9b-e87e-43b8-b13b-d6c82e44156f",
 CellID->422814197],
Cell[6914, 161, 1032, 24, 108, "Text",ExpressionUUID->"c79b1b97-590b-402f-b11a-b86feb2f48ff",
 CellID->953630619],
Cell[CellGroupData[{
Cell[7971, 189, 493, 13, 35, "Input",ExpressionUUID->"1b1f7252-9285-4261-9875-d8e605f93702",
 CellID->725022207],
Cell[8467, 204, 302, 7, 34, "Output",ExpressionUUID->"beebce15-ba03-4a4d-b2ee-ca9350df68d9",
 CellID->931308411]
}, Open  ]],
Cell[8784, 214, 581, 12, 60, "Text",ExpressionUUID->"a4703e14-a299-4714-89ee-29d516877b6b",
 CellID->666673628],
Cell[9368, 228, 923, 23, 64, "Text",ExpressionUUID->"e00e0296-14f8-4e20-a7ce-4512f364f2ee",
 CellID->1262785300],
Cell[10294, 253, 916, 17, 30, "Text",ExpressionUUID->"72c04f85-fa84-45c7-a506-90fc65a88a9d",
 CellTags->"b:5.0",
 CellID->500701437],
Cell[11213, 272, 1638, 34, 164, "Input",ExpressionUUID->"936059f3-43bf-4a5d-97c9-8572aa33f93f",
 CellTags->"b:5.0",
 CellID->1448122713],
Cell[CellGroupData[{
Cell[12876, 310, 1046, 19, 34, "Input",ExpressionUUID->"8c88cedf-4903-4600-b36e-e724e4a12286",
 CellTags->"b:5.0",
 CellID->304033761],
Cell[13925, 331, 9068, 223, 166, "Output",ExpressionUUID->"1425609d-48c3-4418-9542-957084c5356a",
 CellTags->"b:5.0",
 CellID->1639644341]
}, Open  ]],
Cell[23008, 557, 1138, 20, 60, "Text",ExpressionUUID->"64289182-a051-4239-8c47-5ce1fc80eb14",
 CellTags->"b:5.0",
 CellID->1166265940],
Cell[CellGroupData[{
Cell[24171, 581, 900, 17, 35, "Input",ExpressionUUID->"803a85c8-a63a-4861-9475-de20a3f3bfe1",
 CellTags->"b:5.0",
 CellID->822164855],
Cell[25074, 600, 7002, 166, 114, "Output",ExpressionUUID->"5c6b1d06-880d-4560-b19d-cb30dc826d1e",
 CellTags->"b:5.0",
 CellID->992280883]
}, Open  ]],
Cell[CellGroupData[{
Cell[32113, 771, 877, 17, 35, "Input",ExpressionUUID->"6cf172ce-a5cf-4e94-bb77-fc49ffffa74e",
 CellTags->"b:5.0",
 CellID->1788765850],
Cell[32993, 790, 3311, 84, 95, "Output",ExpressionUUID->"efcb9b81-4676-4cb9-b100-2ebdfcab90a0",
 CellTags->"b:5.0",
 CellID->2054281626]
}, Open  ]],
Cell[CellGroupData[{
Cell[36341, 879, 827, 16, 35, "Input",ExpressionUUID->"cfa4bb5b-c11e-49c6-818e-ca1913a5fd20",
 CellTags->"b:5.0",
 CellID->513716746],
Cell[37171, 897, 13251, 307, 136, "Output",ExpressionUUID->"99e833b5-c05d-4972-a18a-4f232eba6c8f",
 CellTags->"b:5.0",
 CellID->1216695363]
}, Open  ]],
Cell[CellGroupData[{
Cell[50459, 1209, 465, 10, 35, "Input",ExpressionUUID->"f705a0da-dc2c-4806-a810-ad34a1ce0f7a",
 CellID->1599448578],
Cell[50927, 1221, 5992, 144, 95, "Output",ExpressionUUID->"c695688e-fbec-4fb1-b88f-a237ed667529",
 CellID->659679207]
}, Open  ]],
Cell[56934, 1368, 359, 9, 72, "Section",ExpressionUUID->"bac64e0c-c513-4938-859f-7d8e0d5c31b0",
 CellID->194863427],
Cell[57296, 1379, 396, 9, 38, "Section",ExpressionUUID->"895a0a57-4151-47b6-b987-e39754ad7e7d",
 CellID->1923252123],
Cell[57695, 1390, 2926, 68, 216, "Text",ExpressionUUID->"8705f09c-8d1a-48b2-8e2b-21c265ca1fad",
 CellTags->"b:5.1.0",
 CellID->766262050],
Cell[CellGroupData[{
Cell[60646, 1462, 1771, 56, 174, "Input",ExpressionUUID->"8bb4b87e-9597-4f2c-8d22-b0067ef28360",
 CellID->1731517089],
Cell[62420, 1520, 614, 18, 49, "Output",ExpressionUUID->"f408f889-65a7-471f-8d71-a9682d9536cf",
 CellID->1917846235],
Cell[63037, 1540, 613, 18, 49, "Output",ExpressionUUID->"f392737c-a6a6-4317-8433-57214b2b1057",
 CellID->1659613822],
Cell[63653, 1560, 396, 10, 34, "Output",ExpressionUUID->"7fe08351-014c-466f-8f56-f60ec8916391",
 CellID->1611899118],
Cell[64052, 1572, 396, 10, 34, "Output",ExpressionUUID->"1770de92-0498-4e88-a10a-2471010586ec",
 CellID->682269321],
Cell[64451, 1584, 235, 6, 34, "Output",ExpressionUUID->"25193d62-8687-42fb-ab29-4478ba3ad567",
 CellID->1485286714]
}, Open  ]],
Cell[64701, 1593, 944, 16, 56, "Text",ExpressionUUID->"791483d1-21fe-4d85-aa2b-fde78d025c98",
 CellTags->"b:5.1.0",
 CellID->721432069],
Cell[65648, 1611, 1454, 34, 58, "Text",ExpressionUUID->"62dfc3d2-c862-43b9-8249-79d613148b49",
 CellTags->"b:5.1.0",
 CellID->1553898169],
Cell[CellGroupData[{
Cell[67127, 1649, 862, 21, 34, "Input",ExpressionUUID->"b44e2eae-c3c5-4fe8-90f2-bda784ad6fa2",
 CellID->350600306],
Cell[67992, 1672, 234, 6, 34, "Output",ExpressionUUID->"fb553eaa-4f90-4b6f-9dde-317f15ddafa5",
 CellID->863889470]
}, Open  ]],
Cell[68241, 1681, 1204, 20, 56, "Text",ExpressionUUID->"c4048d76-ef73-45f4-9f7a-d70e3d200e85",
 CellTags->"b:5.1.0",
 CellID->797502331],
Cell[CellGroupData[{
Cell[69470, 1705, 1100, 35, 92, "Input",ExpressionUUID->"421a7c60-625f-4d61-9bed-b3607455d1e5",
 CellID->2110286995],
Cell[70573, 1742, 613, 18, 49, "Output",ExpressionUUID->"fd62b351-0873-40f1-be7c-339f89bd9ee4",
 CellID->253384923],
Cell[71189, 1762, 236, 6, 34, "Output",ExpressionUUID->"be0ac67a-fe09-45ba-bfe1-d907ec0c953f",
 CellID->180328860],
Cell[71428, 1770, 396, 10, 34, "Output",ExpressionUUID->"2198bad7-f7f9-4f04-8f97-ef84011f1203",
 CellID->687724103]
}, Open  ]],
Cell[CellGroupData[{
Cell[71861, 1785, 855, 27, 92, "Input",ExpressionUUID->"6b27ead0-f23e-4eef-844f-2caca4de0718",
 CellID->698403514],
Cell[72719, 1814, 606, 17, 35, "Output",ExpressionUUID->"66c09235-f671-42a8-bc16-5ca137f84483",
 CellID->822010967],
Cell[73328, 1833, 606, 17, 35, "Output",ExpressionUUID->"fbbe3cc6-25ed-46ba-b4dd-085eaa107cbd",
 CellID->776021218],
Cell[73937, 1852, 406, 10, 34, "Output",ExpressionUUID->"2f3bb96f-d964-4398-ba4f-89fc69654a40",
 CellID->920422695]
}, Open  ]],
Cell[74358, 1865, 1462, 23, 100, "Text",ExpressionUUID->"a9dfffb4-7bca-4bef-989b-ddf73abe43c3",
 CellTags->"b:5.1.0",
 CellID->1671184905],
Cell[CellGroupData[{
Cell[75845, 1892, 2629, 79, 84, "Input",ExpressionUUID->"1bab1f71-26b8-44fd-bea0-4f808dccef1d",
 CellTags->{"i:4", "b:5.1.0"},
 CellID->1176236190],
Cell[78477, 1973, 266, 7, 34, "Output",ExpressionUUID->"f35f807e-03ec-48f0-a4e8-f2d8de1b6c7f",
 CellTags->{"i:4", "b:5.1.0"},
 CellID->1219525524],
Cell[78746, 1982, 265, 7, 34, "Output",ExpressionUUID->"58d87256-090b-4173-b33b-b541daa036bf",
 CellTags->{"i:4", "b:5.1.0"},
 CellID->981554900]
}, Open  ]],
Cell[79026, 1992, 418, 10, 36, "Text",ExpressionUUID->"e13aa294-4bfe-4442-846b-f2dda9b651d7",
 CellID->1203505892],
Cell[CellGroupData[{
Cell[79469, 2006, 706, 18, 61, "Input",ExpressionUUID->"dce7f39e-ae04-4a54-804b-9a4bfb85c746",
 CellID->1095091655],
Cell[80178, 2026, 1064, 31, 81, "Output",ExpressionUUID->"6ac1a507-8e4a-495e-afc8-7734b35f4e7b",
 CellID->23828046],
Cell[81245, 2059, 839, 23, 50, "Output",ExpressionUUID->"2c875efc-d510-4930-a005-1e1cc2a96d3c",
 CellID->1046010780]
}, Open  ]],
Cell[82099, 2085, 855, 16, 34, "Text",ExpressionUUID->"d9456fa3-fece-4cb3-9b05-7d544938cf6a",
 CellTags->"b:5.1.0",
 CellID->1709645044],
Cell[CellGroupData[{
Cell[82979, 2105, 478, 13, 61, "Input",ExpressionUUID->"72a37a5b-f28b-43dc-a165-fd1b5f82a611",
 CellID->1676876806],
Cell[83460, 2120, 1068, 31, 81, "Output",ExpressionUUID->"b378155b-aa5c-4cb5-9d19-9ac421d7ecf5",
 CellID->1002045118],
Cell[84531, 2153, 236, 6, 34, "Output",ExpressionUUID->"a7c80db6-098e-423f-a1f2-12e29bb2cb7c",
 CellID->805538452]
}, Open  ]],
Cell[84782, 2162, 987, 18, 34, "Text",ExpressionUUID->"90e0606d-6b4e-449a-b633-544475cfc220",
 CellTags->"b:5.1.0",
 CellID->1862003300],
Cell[CellGroupData[{
Cell[85794, 2184, 1876, 52, 138, "Input",ExpressionUUID->"04c02445-dc8d-44bd-96ea-df27115c85b4",
 CellID->887258699],
Cell[87673, 2238, 236, 6, 34, "Output",ExpressionUUID->"6bfd3f18-1f78-4e90-8939-fb9ce85e15b8",
 CellID->947852076],
Cell[87912, 2246, 1994, 57, 81, "Output",ExpressionUUID->"80fe2521-b1ac-4a4f-bb64-5a8da1bb12a9",
 CellID->516147886]
}, Open  ]],
Cell[CellGroupData[{
Cell[89943, 2308, 984, 26, 113, "Input",ExpressionUUID->"75d6416d-0638-443e-8264-49c3a73f2e35",
 CellID->1263581457],
Cell[90930, 2336, 1995, 57, 81, "Output",ExpressionUUID->"3d5127a9-af18-4064-8729-cee09a4b2991",
 CellID->1211069769],
Cell[92928, 2395, 1065, 31, 81, "Output",ExpressionUUID->"baba1eb5-5385-4ba5-82df-53d847f3c155",
 CellID->599601317],
Cell[93996, 2428, 235, 6, 34, "Output",ExpressionUUID->"22d9fd68-cd96-4200-926f-69dd4fce485b",
 CellID->1506063095],
Cell[94234, 2436, 836, 23, 50, "Output",ExpressionUUID->"0f571668-205d-40d0-90a7-f0acc19c805c",
 CellID->12690476]
}, Open  ]],
Cell[95085, 2462, 1161, 20, 56, "Text",ExpressionUUID->"38bb8e14-24e9-499a-bdc1-1933384fd1e6",
 CellTags->"b:5.1.0",
 CellID->2954107],
Cell[96249, 2484, 1502, 28, 78, "Text",ExpressionUUID->"061df848-1281-4ebf-89fe-eb1c11b2fa8d",
 CellTags->"b:5.1.0",
 CellID->2015713543],
Cell[CellGroupData[{
Cell[97776, 2516, 2197, 71, 242, "Input",ExpressionUUID->"dcaf459b-730f-4071-8d34-b6e176d3c55a",
 CellID->1758193617],
Cell[99976, 2589, 842, 24, 47, "Output",ExpressionUUID->"a193ae6e-ea9b-4cef-aafa-f2a565d2d8a9",
 CellID->687949079],
Cell[100821, 2615, 814, 23, 47, "Output",ExpressionUUID->"93e7be1d-da10-4d57-bdc4-94075639f657",
 CellID->308215305],
Cell[101638, 2640, 698, 20, 43, "Output",ExpressionUUID->"3ab183d8-28a5-4dd7-ae08-2f519f820ab4",
 CellID->444074571],
Cell[102339, 2662, 815, 23, 47, "Output",ExpressionUUID->"fa7beac7-eba3-4aa6-9902-0d13867ab589",
 CellID->1055641472],
Cell[103157, 2687, 841, 24, 47, "Output",ExpressionUUID->"d1aae179-8ae4-45a3-aa25-6b442284aaa1",
 CellID->1803812037],
Cell[104001, 2713, 698, 20, 43, "Output",ExpressionUUID->"fd9874a2-0910-4128-b52a-3a610cf7b16a",
 CellID->642261160],
Cell[104702, 2735, 535, 17, 48, "Output",ExpressionUUID->"908a5307-3368-4df8-b3c6-dc5d97e94882",
 CellID->1230506298]
}, Open  ]],
Cell[105252, 2755, 1660, 27, 122, "Text",ExpressionUUID->"40db12d6-90af-4aa6-bbd0-d25685c24a7f",
 CellTags->"b:5.1.0",
 CellID->1863427519],
Cell[106915, 2784, 364, 9, 72, "Section",ExpressionUUID->"07ce06ce-f55c-4908-a99a-f06074a56ead",
 CellID->1489412029],
Cell[107282, 2795, 1798, 37, 123, "Text",ExpressionUUID->"512b636f-9a2a-4e6a-b7f7-c3ab4f67472e",
 CellTags->"b:5.1.0",
 CellID->1091194281],
Cell[CellGroupData[{
Cell[109105, 2836, 908, 25, 34, "Input",ExpressionUUID->"7d20adc1-cb2c-4eb9-849d-b6d98b160d0b",
 CellID->155418018],
Cell[110016, 2863, 236, 6, 34, "Output",ExpressionUUID->"ad859d9c-8447-404d-8514-4556076dc6bb",
 CellID->851829876]
}, Open  ]],
Cell[110267, 2872, 2766, 70, 59, "Text",ExpressionUUID->"8cc86ccb-2c28-4875-afe4-1f49df8e8af5",
 CellTags->"b:5.1.0",
 CellID->717347682],
Cell[CellGroupData[{
Cell[113058, 2946, 1085, 24, 34, "Input",ExpressionUUID->"d5d524c0-f4c8-476a-bbd4-1cbae9fb0b3a",
 CellID->237211231],
Cell[114146, 2972, 1861, 45, 49, "Output",ExpressionUUID->"7f6bac79-b304-4310-8c73-2ef7f19a0264",
 CellID->1578735030]
}, Open  ]],
Cell[116022, 3020, 1499, 30, 40, "Text",ExpressionUUID->"68180cf7-ec6e-4c60-aae1-5f7267a7ac5b",
 CellTags->"b:5.1.0",
 CellID->731571697],
Cell[CellGroupData[{
Cell[117546, 3054, 1181, 38, 60, "Input",ExpressionUUID->"7828b306-78b8-4ed1-8be8-77cd668e870b",
 CellID->1736156787],
Cell[118730, 3094, 235, 6, 34, "Output",ExpressionUUID->"490b9194-7a83-463c-8fdf-991985f47de1",
 CellID->1639190525],
Cell[118968, 3102, 235, 6, 34, "Output",ExpressionUUID->"0a881632-ecd7-4116-99c6-2179839799c1",
 CellID->1954929959]
}, Open  ]],
Cell[119218, 3111, 753, 22, 34, "Input",ExpressionUUID->"b0718b17-c0f0-4edd-91ac-539b3832f0a6",
 CellID->1020640493],
Cell[119974, 3135, 2016, 46, 39, "Text",ExpressionUUID->"f07e6c48-11a2-41e5-9677-9303d6aa7dd6",
 CellTags->"b:5.1.0",
 CellID->1596268905],
Cell[CellGroupData[{
Cell[122015, 3185, 2884, 88, 164, "Input",ExpressionUUID->"573a4e68-3bbb-44c6-bcf2-fa9932aee81a",
 CellID->1526048797],
Cell[124902, 3275, 939, 28, 69, "Output",ExpressionUUID->"6ebd6cff-d005-4a1b-93cb-5498a3074938",
 CellID->8021813],
Cell[125844, 3305, 1072, 30, 55, "Output",ExpressionUUID->"ec09cae3-34c6-434e-a8d7-0a1e07fcb71d",
 CellID->1678624977],
Cell[126919, 3337, 235, 6, 34, "Output",ExpressionUUID->"b16018af-0958-45cc-97a5-81ab941173ba",
 CellID->1157317800],
Cell[127157, 3345, 235, 6, 34, "Output",ExpressionUUID->"f4c32f87-2789-4d82-8696-8987ac66758b",
 CellID->1280873721]
}, Open  ]],
Cell[127407, 3354, 1321, 22, 34, "Text",ExpressionUUID->"10bd2765-324d-417f-9f15-04abf6b77beb",
 CellTags->"b:5.1.0",
 CellID->46958099],
Cell[CellGroupData[{
Cell[128753, 3380, 4407, 155, 136, "Input",ExpressionUUID->"1bd8d563-4afa-4edf-8242-1b82aa4aa711",
 CellID->2038932978],
Cell[133163, 3537, 237, 6, 34, "Output",ExpressionUUID->"87343f3a-c53f-4a5e-9232-148e43bf03a1",
 CellID->1268551319],
Cell[133403, 3545, 234, 6, 34, "Output",ExpressionUUID->"1669dd63-ab6b-46f3-ba32-68c6a43c1986",
 CellID->324371017],
Cell[133640, 3553, 235, 6, 34, "Output",ExpressionUUID->"ac7ee7c5-7a9b-4d08-83fa-c5c67033428d",
 CellID->1390186633]
}, Open  ]],
Cell[133890, 3562, 1575, 27, 84, "Text",ExpressionUUID->"b5d8fbde-d918-44d8-861c-fb4aec3e03c0",
 CellTags->"b:5.1.0",
 CellID->1165871760],
Cell[CellGroupData[{
Cell[135490, 3593, 2817, 90, 188, "Input",ExpressionUUID->"109d637f-a49c-41e4-be33-5a9a263e7db3",
 CellID->1128177368],
Cell[138310, 3685, 623, 18, 49, "Output",ExpressionUUID->"4db31d71-50e9-4d72-8a46-24894681c555",
 CellID->244661471],
Cell[138936, 3705, 624, 18, 49, "Output",ExpressionUUID->"2e282278-aebc-4be6-b346-fe692014059c",
 CellID->1741642739],
Cell[139563, 3725, 235, 6, 34, "Output",ExpressionUUID->"ef349b95-2205-45db-940b-e89214f66a7e",
 CellID->1587405763],
Cell[139801, 3733, 237, 6, 34, "Output",ExpressionUUID->"67cff5ca-b6e6-4b96-8516-15dcea4eb53a",
 CellID->1339312060],
Cell[140041, 3741, 235, 6, 34, "Output",ExpressionUUID->"a0df63f4-ea44-4f9b-984a-48320fc68dcf",
 CellID->2041688213],
Cell[140279, 3749, 233, 6, 34, "Output",ExpressionUUID->"5e71b0cb-ed52-4546-9007-fca6145dc493",
 CellID->12600478]
}, Open  ]],
Cell[CellGroupData[{
Cell[140549, 3760, 1663, 54, 164, "Input",ExpressionUUID->"8514d453-a12e-4f3e-9abc-805b447dfcc0",
 CellID->613794198],
Cell[142215, 3816, 237, 6, 34, "Output",ExpressionUUID->"2e339475-27ff-4e34-bfcc-4dfbb1c93442",
 CellID->1322836340],
Cell[142455, 3824, 233, 6, 34, "Output",ExpressionUUID->"96160e05-1b3a-47c9-9ff9-d7009a8dff85",
 CellID->12168989],
Cell[142691, 3832, 234, 6, 34, "Output",ExpressionUUID->"cea2252a-0874-4c4a-b634-184cf5939815",
 CellID->513251658],
Cell[142928, 3840, 235, 6, 34, "Output",ExpressionUUID->"1d098063-eee3-4ae8-88d1-5bd53d09fd03",
 CellID->1926289654],
Cell[143166, 3848, 235, 6, 34, "Output",ExpressionUUID->"749e7d88-be3e-4190-b70e-5deb8cf679da",
 CellID->1834236253],
Cell[143404, 3856, 236, 6, 34, "Output",ExpressionUUID->"338d5da5-abdf-4cc4-b47c-acbdf79af97e",
 CellID->880951486]
}, Open  ]],
Cell[143655, 3865, 1563, 27, 108, "Text",ExpressionUUID->"1b43ce23-81ec-42ff-aa98-8e3b6bfc0fc0",
 CellTags->"b:5.1.0",
 CellID->1111559247],
Cell[CellGroupData[{
Cell[145243, 3896, 1246, 34, 68, "Input",ExpressionUUID->"05398b34-07d6-48bb-ad1e-084f141bfba7",
 CellID->624365819],
Cell[146492, 3932, 1042, 32, 65, "Output",ExpressionUUID->"08c3fcd6-55db-45cf-b435-f0c3db413edf",
 CellID->1399362784],
Cell[147537, 3966, 773, 24, 65, "Output",ExpressionUUID->"1153870d-32df-462e-bba0-db7147011d7f",
 CellID->1996892391]
}, Open  ]],
Cell[148325, 3993, 759, 15, 36, "Text",ExpressionUUID->"0f34bf7c-5eb9-45df-9c63-bdcc36b3dab0",
 CellTags->"b:5.1.0",
 CellID->2039372162],
Cell[CellGroupData[{
Cell[149109, 4012, 1382, 39, 112, "Input",ExpressionUUID->"f06fa297-ebe1-44dd-8567-fdac3051a8cf",
 CellID->450621595],
Cell[150494, 4053, 1071, 30, 55, "Output",ExpressionUUID->"445a95b6-23a4-4683-af28-d62173c9a048",
 CellID->895980221],
Cell[151568, 4085, 234, 6, 34, "Output",ExpressionUUID->"d3684238-27e6-4328-82c9-465d68f5e13d",
 CellID->584241583]
}, Open  ]],
Cell[151817, 4094, 406, 10, 72, "Section",ExpressionUUID->"20d42eee-0b5e-4072-98d3-d588004b8520",
 CellID->115326685],
Cell[152226, 4106, 351, 8, 38, "Section",ExpressionUUID->"b5d4e574-5fa8-4935-adfc-3d628c9911bd",
 CellID->1933253184],
Cell[152580, 4116, 2074, 40, 33, "Text",ExpressionUUID->"266b47db-0102-4aab-bcbd-2d5f7f52f5d1",
 CellTags->"b:5.1.0",
 CellID->1757173766],
Cell[CellGroupData[{
Cell[154679, 4160, 1052, 29, 85, "Input",ExpressionUUID->"d3e219f5-2e01-4e70-b9d9-d8b85396cec0",
 CellTags->"b:5.1.0",
 CellID->1388185552],
Cell[155734, 4191, 646, 19, 49, "Output",ExpressionUUID->"16f64f7d-33f4-4bbc-80f7-fa33c5816d74",
 CellTags->"b:5.1.0",
 CellID->1053051397],
Cell[156383, 4212, 646, 19, 49, "Output",ExpressionUUID->"fc0dda16-2515-43d1-96d0-9e97e0207348",
 CellTags->"b:5.1.0",
 CellID->1222094452],
Cell[157032, 4233, 259, 7, 34, "Output",ExpressionUUID->"132d7cee-3cab-4aab-854c-e438748cae1b",
 CellTags->"b:5.1.0",
 CellID->1207854856]
}, Open  ]],
Cell[157306, 4243, 3772, 74, 43, "Text",ExpressionUUID->"1e0e917b-98c8-494a-82c8-612f219c9fac",
 CellTags->"b:5.1.0",
 CellID->256705393],
Cell[CellGroupData[{
Cell[161103, 4321, 3925, 130, 191, "Input",ExpressionUUID->"6bbe0890-90bb-402f-9039-ff64459d416a",
 CellID->900017173],
Cell[165031, 4453, 395, 10, 34, "Output",ExpressionUUID->"6e44daed-8e92-46a5-bcfc-be23cf132cf7",
 CellID->253649701],
Cell[165429, 4465, 234, 6, 34, "Output",ExpressionUUID->"317aa23c-3bbe-4be3-8ae3-771b3f13fe97",
 CellID->335485412],
Cell[165666, 4473, 235, 6, 34, "Output",ExpressionUUID->"00dbe06f-4243-45b4-b53e-57b8ef9ccd28",
 CellID->1358746109],
Cell[165904, 4481, 234, 6, 34, "Output",ExpressionUUID->"b65a9f07-a339-4b31-a993-1763c66fab10",
 CellID->104751765],
Cell[166141, 4489, 235, 6, 34, "Output",ExpressionUUID->"6285c803-0e42-4ee7-b13f-baa9e5bde015",
 CellID->1773807462]
}, Open  ]],
Cell[CellGroupData[{
Cell[166413, 4500, 4389, 151, 229, "Input",ExpressionUUID->"f372be1e-667d-4cac-8f03-7d63088f44ee",
 CellID->920608321],
Cell[170805, 4653, 839, 23, 50, "Output",ExpressionUUID->"1c2ccd6c-0787-4ce0-a031-ff5e7191a290",
 CellID->1014026180],
Cell[171647, 4678, 236, 6, 34, "Output",ExpressionUUID->"7348ef03-a71a-4c35-9c20-14ac5c46d768",
 CellID->307784668],
Cell[171886, 4686, 235, 6, 34, "Output",ExpressionUUID->"b231c5ce-4c7a-41e1-afcb-3e3627ce8c92",
 CellID->1796293100],
Cell[172124, 4694, 234, 6, 34, "Output",ExpressionUUID->"06b39800-c7bc-488a-8ca6-54aa308b71f7",
 CellID->973022174],
Cell[172361, 4702, 233, 6, 34, "Output",ExpressionUUID->"957f5bc1-042b-4e0b-bf44-9bc34b4d3cef",
 CellID->495706513]
}, Open  ]],
Cell[172609, 4711, 2244, 42, 38, "Text",ExpressionUUID->"8370951a-626f-4804-8fe4-b67424a26141",
 CellTags->"b:5.1.0",
 CellID->1852374601],
Cell[CellGroupData[{
Cell[174878, 4757, 2189, 58, 183, "Input",ExpressionUUID->"776f2bf9-aa6f-4b34-9df9-68a75408e796",
 CellID->267973840],
Cell[177070, 4817, 839, 23, 50, "Output",ExpressionUUID->"98170634-9860-48ac-9420-8a0d4875dd57",
 CellID->1250297278],
Cell[177912, 4842, 848, 23, 50, "Output",ExpressionUUID->"219b2454-cef7-4e52-8fec-9de7abca5e22",
 CellID->618865451],
Cell[178763, 4867, 235, 6, 34, "Output",ExpressionUUID->"1279782f-6d15-43c7-9bc1-20a951128edb",
 CellID->1008332536],
Cell[179001, 4875, 234, 6, 34, "Output",ExpressionUUID->"2bd9f5df-c508-4627-9a67-79de9402bb43",
 CellID->1251474075]
}, Open  ]],
Cell[179250, 4884, 1943, 39, 37, "Text",ExpressionUUID->"76142ac1-212c-44d8-8824-5f1a350fa1dc",
 CellTags->"b:5.1.0",
 CellID->904569019],
Cell[181196, 4925, 435, 13, 34, "Text",ExpressionUUID->"e3fef630-c316-443f-b674-b6195389c722",
 CellTags->"b:5.1.0",
 CellID->1054765220],
Cell[CellGroupData[{
Cell[181656, 4942, 1970, 66, 60, "Input",ExpressionUUID->"87d3bcc7-7a17-4e6d-8fd9-d088220a7b82",
 CellTags->"b:5.1.0",
 CellID->2109030202],
Cell[183629, 5010, 257, 7, 34, "Output",ExpressionUUID->"e8a37290-a701-4057-8619-769f721640e5",
 CellTags->"b:5.1.0",
 CellID->1069939807],
Cell[183889, 5019, 258, 7, 34, "Output",ExpressionUUID->"e0710e2b-9a81-4155-8a7d-50add735e77d",
 CellTags->"b:5.1.0",
 CellID->1597759918]
}, Open  ]],
Cell[184162, 5029, 725, 14, 34, "Text",ExpressionUUID->"de65a51e-ecbc-4710-912a-cffd1b489ae8",
 CellTags->"b:5.1.0",
 CellID->811402856],
Cell[CellGroupData[{
Cell[184912, 5047, 406, 13, 72, "Input",ExpressionUUID->"933d7000-d66e-43ad-afc6-168921c12a96",
 CellTags->{"i:4", "b:5.1.0"},
 CellID->726642342],
Cell[185321, 5062, 648, 19, 49, "Output",ExpressionUUID->"13b4580b-2d05-4c74-9d97-8a60cb49771e",
 CellTags->{"i:4", "b:5.1.0"},
 CellID->1415419738],
Cell[185972, 5083, 641, 18, 34, "Output",ExpressionUUID->"26090d83-bfc7-4447-88f5-42c1cd7b6ff3",
 CellTags->{"i:4", "b:5.1.0"},
 CellID->1283178747]
}, Open  ]],
Cell[186628, 5104, 1145, 27, 39, "Text",ExpressionUUID->"ab50856d-5bb3-4032-9655-8eeb47840a9f",
 CellTags->"b:5.1.0",
 CellID->2034662426],
Cell[CellGroupData[{
Cell[187798, 5135, 1838, 64, 123, "Input",ExpressionUUID->"f18c5bab-64c3-43ea-b6f4-e010a573543f",
 CellID->1009141171],
Cell[189639, 5201, 625, 18, 49, "Output",ExpressionUUID->"2e6d3ee0-5e6a-48e7-afaa-0b5a752b9b59",
 CellID->1354417655],
Cell[190267, 5221, 235, 6, 34, "Output",ExpressionUUID->"d7200518-dc13-41e8-8ff6-56192120203c",
 CellID->370869385],
Cell[190505, 5229, 625, 18, 49, "Output",ExpressionUUID->"cec567fa-7401-42d2-a9c3-28c2134c495f",
 CellID->1204210101],
Cell[191133, 5249, 236, 6, 34, "Output",ExpressionUUID->"ae20154d-8a86-48a1-9aa4-5ab7514829e7",
 CellID->1273278179]
}, Open  ]],
Cell[191384, 5258, 1398, 26, 39, "Text",ExpressionUUID->"5b4ae767-c154-49a6-bbc2-60e7c504a48b",
 CellTags->"b:5.1.1",
 CellID->415603317],
Cell[CellGroupData[{
Cell[192807, 5288, 2205, 43, 87, "Input",ExpressionUUID->"12435e57-3a67-4786-8ba1-9871ec31d610",
 CellID->1903618416],
Cell[195015, 5333, 616, 18, 49, "Output",ExpressionUUID->"9651d4dd-2e16-4ebc-a23f-3eed3b60846e",
 CellID->311812098],
Cell[195634, 5353, 611, 17, 34, "Output",ExpressionUUID->"2cdf4bff-7ccd-4254-9ef7-466aac5148dc",
 CellID->765832406],
Cell[196248, 5372, 745, 22, 49, "Output",ExpressionUUID->"bbe63fcf-d744-414b-ba6c-60447147d5a6",
 CellID->1475751171]
}, Open  ]],
Cell[197008, 5397, 674, 13, 34, "Text",ExpressionUUID->"14299fed-db8f-4335-9418-dd68c718829a",
 CellTags->"b:5.1.0",
 CellID->1616577872],
Cell[CellGroupData[{
Cell[197707, 5414, 1637, 40, 61, "Input",ExpressionUUID->"70174e60-e1de-45d0-a2a3-24e56054f41e",
 CellID->41798954],
Cell[199347, 5456, 236, 6, 34, "Output",ExpressionUUID->"ed44b9ae-dc6f-4b23-ac51-09f315276114",
 CellID->1661291794],
Cell[199586, 5464, 238, 6, 34, "Output",ExpressionUUID->"706448a0-974e-4547-8dc1-522baf1a397b",
 CellID->1896287111]
}, Open  ]],
Cell[199839, 5473, 755, 14, 34, "Text",ExpressionUUID->"67c30f82-85f7-421b-9c83-a9cbd6c65bd3",
 CellTags->"b:5.1.0",
 CellID->402056739],
Cell[CellGroupData[{
Cell[200619, 5491, 399, 10, 60, "Input",ExpressionUUID->"53fc7c8e-9193-4aa1-897e-3115185ac60c",
 CellID->1110883851],
Cell[201021, 5503, 11019, 267, 190, "Output",ExpressionUUID->"f0bac21d-ab6b-4bf7-8a88-986cceb5a597",
 CellID->92669450],
Cell[212043, 5772, 10565, 260, 166, "Output",ExpressionUUID->"02b96406-fba4-4ddc-9763-515ede207d89",
 CellID->569802075]
}, Open  ]],
Cell[222623, 6035, 1973, 36, 162, "Text",ExpressionUUID->"65b7138e-2c0b-4209-b047-79430580396f",
 CellTags->"b:5.1.0",
 CellID->224595378],
Cell[CellGroupData[{
Cell[224621, 6075, 350, 9, 34, "Input",ExpressionUUID->"5b97cfcb-7e71-48aa-9e8b-ae363c55ce9f",
 CellID->535364109],
Cell[224974, 6086, 10480, 256, 189, "Output",ExpressionUUID->"b2444fd4-e723-4628-9789-24d76ee8c870",
 CellID->1033892552]
}, Open  ]],
Cell[235469, 6345, 410, 10, 72, "Section",ExpressionUUID->"11064670-1345-4b12-ab84-f3b1f527d215",
 CellID->2098327973],
Cell[235882, 6357, 350, 8, 38, "Section",ExpressionUUID->"fe051219-0d0c-4175-9359-a26ff82e8c34",
 CellID->621791177],
Cell[236235, 6367, 3570, 78, 217, "Text",ExpressionUUID->"a1c0bc78-1e48-43fc-8295-db6eabc1c8e6",
 CellTags->"b:5.1.0",
 CellID->283093329],
Cell[239808, 6447, 4345, 112, 179, "Text",ExpressionUUID->"2004a039-55f4-425d-911d-96f92cf3571d",
 CellTags->"b:5.1.0",
 CellID->1372227264],
Cell[244156, 6561, 1610, 38, 92, "Text",ExpressionUUID->"64a68ffc-4eb0-4b19-a155-5fa55b7ee947",
 CellTags->"b:5.1.0",
 CellID->509110344],
Cell[245769, 6601, 2391, 70, 87, "Text",ExpressionUUID->"dc617e94-8ac5-42b4-b8fa-05b4ea0436ca",
 CellTags->"b:5.1.0",
 CellID->1067632665],
Cell[248163, 6673, 2592, 85, 52, "Text",ExpressionUUID->"83251749-2971-4122-8fbf-b5ceb3077c5c",
 CellID->1476329970],
Cell[250758, 6760, 870, 16, 34, "Text",ExpressionUUID->"e081223b-7099-4505-8d4e-be90b0429133",
 CellTags->"b:5.1.0",
 CellID->582374062],
Cell[251631, 6778, 2280, 74, 44, "Text",ExpressionUUID->"d172148a-6af9-4aa3-9e7b-4c0ce51762e8",
 CellID->809286249],
Cell[253914, 6854, 3227, 69, 303, "Text",ExpressionUUID->"44604929-f489-467e-b2ba-ef9aebb2b6bc",
 CellTags->"b:5.1.0",
 CellID->1134805856],
Cell[CellGroupData[{
Cell[257166, 6927, 280, 7, 41, "Subsubsection",ExpressionUUID->"549a8bb5-d180-4d21-859b-f7471e232801",
 CellID->815224539],
Cell[257449, 6936, 1770, 43, 62, "Text",ExpressionUUID->"2b576ef2-4c46-401d-a02b-b0f9243b7292",
 CellTags->"b:5.1.0",
 CellID->776438691],
Cell[CellGroupData[{
Cell[259244, 6983, 1757, 47, 113, "Input",ExpressionUUID->"5fa62ff9-44bf-409a-beb3-08ad06687b12",
 CellID->194542161],
Cell[261004, 7032, 1067, 31, 81, "Output",ExpressionUUID->"41552067-1e0a-4720-8b96-c0329151cb69",
 CellID->1341933587],
Cell[262074, 7065, 1229, 35, 71, "Output",ExpressionUUID->"d70adea0-2251-4467-a77e-f35d563e7c2e",
 CellID->1004535915],
Cell[263306, 7102, 942, 28, 69, "Output",ExpressionUUID->"132c5460-1569-4eac-8320-c2c14576efbe",
 CellID->1866174338]
}, Open  ]],
Cell[264263, 7133, 779, 18, 37, "Text",ExpressionUUID->"99414cb7-98f5-4b81-a1c0-bb87486fcf32",
 CellTags->"b:5.1.0",
 CellID->92159943]
}, Open  ]],
Cell[CellGroupData[{
Cell[265079, 7156, 1183, 28, 61, "Input",ExpressionUUID->"726dcc08-65de-4a30-a169-388c2b64198a",
 CellID->166283053],
Cell[266265, 7186, 1064, 31, 81, "Output",ExpressionUUID->"370f92c5-a392-4dad-b1c0-0b75d2b8ca08",
 CellID->793434331],
Cell[267332, 7219, 1232, 35, 71, "Output",ExpressionUUID->"e0c8284a-a18c-44f5-8059-bd79ecc0c65f",
 CellID->1524836894]
}, Open  ]],
Cell[CellGroupData[{
Cell[268601, 7259, 279, 8, 35, "Input",ExpressionUUID->"1a03f5a1-d501-4596-9896-0d47caaaa945",
 CellID->596452514],
Cell[268883, 7269, 10258, 254, 142, "Output",ExpressionUUID->"0b4c55ad-70e7-4e32-88f3-2d7f4a2a483a",
 CellID->2011317807]
}, Open  ]],
Cell[279156, 7526, 948, 24, 78, "Text",ExpressionUUID->"b25e9833-ca2c-4a81-9bc7-c9fe257a6b66",
 CellTags->"b:5.1.0",
 CellID->1312746903],
Cell[CellGroupData[{
Cell[280129, 7554, 1294, 30, 163, "Input",ExpressionUUID->"be08d6a9-3be6-4b72-b4b0-234f8ba73d14",
 CellTags->"b:5.1.0",
 CellID->90209443],
Cell[281426, 7586, 257, 7, 34, "Output",ExpressionUUID->"b0b420f0-f43c-4866-956c-0a4e159e6679",
 CellTags->"b:5.1.0",
 CellID->678294822],
Cell[281686, 7595, 257, 7, 34, "Output",ExpressionUUID->"b998e457-8460-4fa1-8b26-eb171a0d7aa0",
 CellTags->"b:5.1.0",
 CellID->1257875078],
Cell[281946, 7604, 257, 7, 34, "Output",ExpressionUUID->"345ce0fb-cd42-41d0-a1d7-476536215fc3",
 CellTags->"b:5.1.0",
 CellID->1277414878]
}, Open  ]],
Cell[CellGroupData[{
Cell[282240, 7616, 460, 12, 61, "Input",ExpressionUUID->"20c9ba24-3ee0-4619-8d27-6acfdf7b8b49",
 CellID->1177281290],
Cell[282703, 7630, 1231, 35, 71, "Output",ExpressionUUID->"5b8c745b-3d64-45a7-9bc0-820050dc3f78",
 CellID->1117835487],
Cell[283937, 7667, 624, 18, 49, "Output",ExpressionUUID->"dd80ac80-7113-4b58-be51-7a6e296742a1",
 CellID->1274086053]
}, Open  ]],
Cell[284576, 7688, 393, 10, 36, "Text",ExpressionUUID->"02ad9a19-7e44-471b-8834-b73fc97c1953",
 CellID->1322053176],
Cell[CellGroupData[{
Cell[284994, 7702, 489, 12, 61, "Input",ExpressionUUID->"b6d8da8a-b89c-4faa-8ec1-2cb7950240ea",
 CellID->26653719],
Cell[285486, 7716, 10948, 267, 283, "Output",ExpressionUUID->"5638ce95-49c1-4476-a188-cdf5cd320f08",
 CellID->1004848534],
Cell[296437, 7985, 11872, 285, 212, "Output",ExpressionUUID->"1b6b8d34-8cbe-4608-8a9d-098fdda59c60",
 CellID->677910930]
}, Open  ]],
Cell[308324, 8273, 670, 12, 56, "Text",ExpressionUUID->"42d52334-ebd8-42b2-8101-32588eb65786",
 CellTags->"b:5.1.0",
 CellID->1492245405],
Cell[308997, 8287, 712, 18, 34, "Text",ExpressionUUID->"ca61ec89-fda4-4d23-bf24-1cfa5bf6831e",
 CellTags->"b:5.1.0",
 CellID->1292456657],
Cell[CellGroupData[{
Cell[309734, 8309, 522, 13, 110, "Input",ExpressionUUID->"72145571-a2e5-4c72-8ab8-a9b4f6e60298",
 CellTags->"b:5.1.0",
 CellID->521944794],
Cell[310259, 8324, 720, 23, 69, "Output",ExpressionUUID->"1ef457bb-7798-4985-80af-d1e2651c9d90",
 CellTags->"b:5.1.0",
 CellID->1248253175],
Cell[310982, 8349, 490, 14, 54, "Output",ExpressionUUID->"8fea3d15-22cb-4c10-b27c-0605993d8af1",
 CellTags->"b:5.1.0",
 CellID->623364682],
Cell[311475, 8365, 746, 24, 69, "Output",ExpressionUUID->"e098b170-9bf4-4b46-b9da-bb9e1e5ebea7",
 CellTags->"b:5.1.0",
 CellID->1464980308],
Cell[312224, 8391, 517, 15, 54, "Output",ExpressionUUID->"7eca5b89-614c-4c9c-8c80-39a9a1ae5271",
 CellTags->"b:5.1.0",
 CellID->1024023701]
}, Open  ]],
Cell[CellGroupData[{
Cell[312778, 8411, 581, 18, 60, "Input",ExpressionUUID->"846f7911-b786-43b9-8805-61d75eb1c43d",
 CellTags->"b:5.1.0",
 CellID->2002938410],
Cell[313362, 8431, 258, 7, 34, "Output",ExpressionUUID->"82ef3be0-edf6-4203-8b32-035047b1925a",
 CellTags->"b:5.1.0",
 CellID->832395649],
Cell[313623, 8440, 259, 7, 34, "Output",ExpressionUUID->"991ce154-3a68-48a8-9759-a60280aacbad",
 CellTags->"b:5.1.0",
 CellID->1365196185]
}, Open  ]],
Cell[313897, 8450, 755, 18, 78, "Text",ExpressionUUID->"603f6fcc-c0ce-4ed3-ba9e-d265642f0371",
 CellTags->"b:5.1.0",
 CellID->1579690145],
Cell[CellGroupData[{
Cell[314677, 8472, 874, 26, 116, "Input",ExpressionUUID->"db686234-811d-466c-abc7-dd32106ff803",
 CellTags->"b:5.1.0",
 CellID->1341871396],
Cell[315554, 8500, 629, 18, 35, "Output",ExpressionUUID->"b704a28d-c954-4f84-81f8-13099517917e",
 CellTags->"b:5.1.0",
 CellID->1136996747],
Cell[316186, 8520, 429, 11, 34, "Output",ExpressionUUID->"b66c26b2-4f1e-4b93-9b01-54888bd75db8",
 CellTags->"b:5.1.0",
 CellID->1194835823],
Cell[316618, 8533, 628, 18, 35, "Output",ExpressionUUID->"c5539882-b2a6-49e2-b585-d902a2fdf678",
 CellTags->"b:5.1.0",
 CellID->486094064],
Cell[317249, 8553, 428, 11, 34, "Output",ExpressionUUID->"c2b4cd6e-3ca4-462c-bdcc-c06fc8afd577",
 CellTags->"b:5.1.0",
 CellID->755140460]
}, Open  ]],
Cell[317692, 8567, 420, 9, 34, "Text",ExpressionUUID->"0ac79b5a-fe60-404f-8003-29642a9d8a38",
 CellTags->"b:5.1.0",
 CellID->1114284212],
Cell[CellGroupData[{
Cell[318137, 8580, 385, 11, 60, "Input",ExpressionUUID->"eca7385c-53b2-4aa6-978f-5714cee4f78e",
 CellTags->"b:5.1.0",
 CellID->1717349244],
Cell[318525, 8593, 739, 23, 43, "Output",ExpressionUUID->"54c9c7ae-00ad-4532-8ab6-59a6f0f13599",
 CellTags->"b:5.1.0",
 CellID->17357450],
Cell[319267, 8618, 741, 23, 43, "Output",ExpressionUUID->"28ac27d1-7873-4f1f-be5c-f034f35a000e",
 CellTags->"b:5.1.0",
 CellID->1816369805]
}, Open  ]],
Cell[320023, 8644, 510, 10, 52, "Section",ExpressionUUID->"a01cb002-95eb-40ec-b759-0ea4659b60a6",
 CellID->21438158],
Cell[320536, 8656, 708, 14, 52, "Text",ExpressionUUID->"6ecce6d3-fd32-495c-aeb8-a60ec79d9c7c",
 CellTags->"b:5.1.0",
 CellID->1140252415],
Cell[CellGroupData[{
Cell[321269, 8674, 2654, 90, 270, "Input",ExpressionUUID->"54e9d456-d14f-4116-b881-359bc92d2582",
 CellID->1681352457],
Cell[323926, 8766, 713, 20, 51, "Output",ExpressionUUID->"54d73158-7a3d-4f42-93e5-2bc80f3c9aad",
 CellID->1247270530],
Cell[324642, 8788, 636, 18, 49, "Output",ExpressionUUID->"3cb4cbf8-e600-42f2-b91f-8b2e635c8b4d",
 CellID->1287919343],
Cell[325281, 8808, 237, 6, 34, "Output",ExpressionUUID->"94001d28-43e7-4f4b-b705-8ff898381451",
 CellID->1572719688],
Cell[325521, 8816, 737, 21, 59, "Output",ExpressionUUID->"e087f93a-1081-4ee5-b824-3cf94d833c75",
 CellID->1471068940],
Cell[326261, 8839, 234, 6, 34, "Output",ExpressionUUID->"9d3d1d10-0ff4-4387-8709-9142f66c5e55",
 CellID->276446669],
Cell[326498, 8847, 660, 19, 49, "Output",ExpressionUUID->"a1bcada3-f886-4209-9a33-3d1947c48a61",
 CellID->1232037422],
Cell[327161, 8868, 761, 22, 59, "Output",ExpressionUUID->"2b9473f7-a5e2-45d3-a21e-00ea68a26bef",
 CellID->815627180],
Cell[327925, 8892, 234, 6, 34, "Output",ExpressionUUID->"018a295f-78e5-457c-9518-64fcb9fdf821",
 CellID->857895459],
Cell[328162, 8900, 234, 6, 34, "Output",ExpressionUUID->"c246a987-796c-4fd9-ac34-9b759dbcc466",
 CellID->443773455]
}, Open  ]],
Cell[328411, 8909, 505, 10, 34, "Text",ExpressionUUID->"5bece0b1-a9db-4a26-8c9b-ed3ec189b3b0",
 CellTags->"b:5.1.0",
 CellID->1704654754],
Cell[CellGroupData[{
Cell[328941, 8923, 1095, 32, 91, "Input",ExpressionUUID->"b20f5c26-3e45-4b3c-a764-e4164a48a573",
 CellID->607432992],
Cell[330039, 8957, 396, 10, 34, "Output",ExpressionUUID->"50723bbc-434f-401f-8e9e-e61df3d534bf",
 CellID->289752154],
Cell[330438, 8969, 434, 11, 34, "Output",ExpressionUUID->"2a076d91-94c9-453b-b038-d9a69db0e4af",
 CellID->1519024529],
Cell[330875, 8982, 538, 15, 42, "Output",ExpressionUUID->"44bb7560-c9fe-440b-83dc-ecf484e4011c",
 CellID->959663367]
}, Open  ]],
Cell[331428, 9000, 583, 12, 34, "Text",ExpressionUUID->"dc2ff4fd-4d6e-445b-b23d-2f6465d9e6ae",
 CellTags->"b:5.1.0",
 CellID->1200399909],
Cell[CellGroupData[{
Cell[332036, 9016, 3077, 91, 228, "Input",ExpressionUUID->"940322fb-dd84-494a-8e11-44349113ea5f",
 CellID->1965925749],
Cell[335116, 9109, 680, 21, 67, "Output",ExpressionUUID->"03c00dad-e05d-4c4c-a79b-7aa62057a8c9",
 CellID->1460020199],
Cell[335799, 9132, 703, 22, 67, "Output",ExpressionUUID->"2bf7ebd5-05e3-4e44-a13f-dff68c9fb9b9",
 CellID->1411333971],
Cell[336505, 9156, 828, 27, 87, "Output",ExpressionUUID->"4c9cc9be-c072-4a6d-8415-962501a179c8",
 CellID->1333169482],
Cell[337336, 9185, 1024, 31, 67, "Output",ExpressionUUID->"c5aa7ebe-94ba-492d-b063-5ba9beec917c",
 CellID->1882594823]
}, Open  ]],
Cell[338375, 9219, 313, 7, 52, "Section",ExpressionUUID->"d8a3e705-1ae1-4618-be39-1b045b781d93",
 CellID->2146841522],
Cell[338691, 9228, 1114, 18, 74, "Text",ExpressionUUID->"79d1d607-9794-4f8d-a426-8f73873942ce",
 CellTags->"b:5.1.1",
 CellID->1128508301],
Cell[339808, 9248, 312, 8, 72, "Section",ExpressionUUID->"b2279946-c0d8-473c-90b3-b5e1bff5f050",
 CellID->428411084],
Cell[CellGroupData[{
Cell[340145, 9260, 347, 8, 38, "Section",ExpressionUUID->"59e6e2cd-efa0-41a6-ab04-c5f8538a831d",
 CellID->1480602742],
Cell[340495, 9270, 1538, 41, 83, "Text",ExpressionUUID->"1f087290-0716-406e-9674-68b89ecd0124",
 CellTags->"b:5.1.1",
 CellID->53899069],
Cell[CellGroupData[{
Cell[342058, 9315, 913, 21, 110, "Input",ExpressionUUID->"c10252cd-53e7-4e25-8108-2ddd27e391ba",
 CellID->1709130074],
Cell[342974, 9338, 698, 20, 83, "Output",ExpressionUUID->"0b38c440-f57c-4920-91c4-d89f4ca3e4ae",
 CellID->1894618910],
Cell[343675, 9360, 700, 20, 83, "Output",ExpressionUUID->"c4a5bbb9-dc08-4d63-9fff-bca34cc9fded",
 CellID->2119986301],
Cell[344378, 9382, 698, 20, 83, "Output",ExpressionUUID->"54760f59-e900-45dd-a429-13f723aa7242",
 CellID->1708573530],
Cell[345079, 9404, 698, 20, 83, "Output",ExpressionUUID->"9cff0d88-3b78-4bc5-b2d6-b400ba4ff7c0",
 CellID->1991474673]
}, Open  ]]
}, Open  ]],
Cell[345804, 9428, 1222, 23, 37, "Text",ExpressionUUID->"d0b16d95-259b-4e8b-ae94-8f5abaea07e3",
 CellTags->"b:5.1.1",
 CellID->1821072405],
Cell[CellGroupData[{
Cell[347051, 9455, 3573, 100, 112, "Input",ExpressionUUID->"e4597cca-b1f5-481e-b4f1-48696b072f6a",
 CellID->1721137469],
Cell[350627, 9557, 862, 26, 51, "Output",ExpressionUUID->"ad6c3307-a6a9-4036-9596-b1522a1613b2",
 CellID->34706969],
Cell[351492, 9585, 866, 26, 51, "Output",ExpressionUUID->"1e86a161-460a-4ca6-b87f-1cc8b4436d75",
 CellID->1351959646],
Cell[352361, 9613, 11181, 320, 97, "Output",ExpressionUUID->"141334c4-ddfb-4ce1-9da4-458ffd4dd41a",
 CellID->2008785062]
}, Open  ]],
Cell[363557, 9936, 4515, 126, 231, "Text",ExpressionUUID->"669fcedb-6a9f-4c29-9912-3dd3e75bc1f2",
 CellTags->"b:5.1.1",
 CellID->852049279],
Cell[CellGroupData[{
Cell[368097, 10066, 282, 8, 35, "Input",ExpressionUUID->"599cc6ab-2736-4d6b-8f81-d78dc46406c0",
 CellID->1067550071],
Cell[368382, 10076, 14928, 348, 142, "Output",ExpressionUUID->"3720952f-5069-45bc-880d-338144e97559",
 CellID->1060122547]
}, Open  ]],
Cell[CellGroupData[{
Cell[383347, 10429, 2440, 102, 118, "Input",ExpressionUUID->"136dbe5b-d9d0-495d-86a6-0fcb54e654e1",
 CellID->1625879544],
Cell[385790, 10533, 2710, 80, 171, "Output",ExpressionUUID->"20b64de6-a573-4276-b26a-13306b434c68",
 CellID->1955311917],
Cell[388503, 10615, 2771, 82, 171, "Output",ExpressionUUID->"1b751396-73df-4289-96bd-580c77e65e02",
 CellID->2132622625],
Cell[391277, 10699, 2769, 82, 171, "Output",ExpressionUUID->"e22b04a5-50cd-4447-b8dc-21acba5d7e09",
 CellID->1486389945],
Cell[394049, 10783, 2770, 82, 171, "Output",ExpressionUUID->"b02eaa3d-a88d-4076-a41a-74551fe60378",
 CellID->78540064]
}, Open  ]],
Cell[CellGroupData[{
Cell[396856, 10870, 1125, 31, 63, "Input",ExpressionUUID->"86b0f2dd-2e8c-4583-a631-59c68e83dfcc",
 CellID->1791037296],
Cell[397984, 10903, 6828, 144, 213, "Output",ExpressionUUID->"0b7ceb8b-df8d-42b3-af4f-6c11a9a93ff6",
 CellID->1467896390]
}, Open  ]],
Cell[CellGroupData[{
Cell[404849, 11052, 1240, 29, 34, "Input",ExpressionUUID->"266b7de7-e43c-45c2-86d4-b607efd4d607",
 CellID->854177356],
Cell[406092, 11083, 236, 6, 34, "Output",ExpressionUUID->"c2940d72-2b35-4b27-be3c-8bfdd8b698ac",
 CellID->642489994]
}, Open  ]],
Cell[406343, 11092, 896, 29, 59, "Text",ExpressionUUID->"bb86afa1-e55f-40ec-9795-f7e7a4655594",
 CellTags->"b:5.1.1",
 CellID->230642097],
Cell[CellGroupData[{
Cell[407264, 11125, 1220, 26, 210, "Input",ExpressionUUID->"14e4918e-dd3b-40bb-a169-5ee2c053727f",
 CellID->193741444],
Cell[408487, 11153, 636, 20, 83, "Output",ExpressionUUID->"6294d5fe-2750-4942-82bc-ed4e6a3a7f11",
 CellID->781552042],
Cell[409126, 11175, 326, 8, 34, "Output",ExpressionUUID->"bfe21686-46ca-46d8-b45f-4735fe51b5eb",
 CellID->1535184842],
Cell[409455, 11185, 638, 20, 83, "Output",ExpressionUUID->"662eefe7-5a34-4337-8c10-1702604c9c10",
 CellID->1983104436],
Cell[410096, 11207, 326, 8, 34, "Output",ExpressionUUID->"0d09c2e7-011e-45e0-b8e8-7651352e9e87",
 CellID->1090054412],
Cell[410425, 11217, 639, 20, 83, "Output",ExpressionUUID->"6ceebc45-3031-47df-b0de-861d3bf3c889",
 CellID->361509927],
Cell[411067, 11239, 325, 8, 34, "Output",ExpressionUUID->"b965179b-bd66-491f-8261-1274aea2820b",
 CellID->666105726],
Cell[411395, 11249, 637, 20, 83, "Output",ExpressionUUID->"7bf5dc2b-8d18-4a60-b8be-7c994d6539d3",
 CellID->406685071],
Cell[412035, 11271, 326, 8, 34, "Output",ExpressionUUID->"d31dd1f8-97e6-43e8-b66d-3c56cddab543",
 CellID->1863023543]
}, Open  ]],
Cell[412376, 11282, 793, 25, 37, "Text",ExpressionUUID->"072576ad-af65-44a4-b78f-f3a8bbb650b0",
 CellTags->"b:5.1.1",
 CellID->627240456],
Cell[CellGroupData[{
Cell[413194, 11311, 1285, 29, 61, "Input",ExpressionUUID->"f7c116ca-3a57-44b8-86b1-105b57299532",
 CellID->33120869],
Cell[414482, 11342, 878, 28, 87, "Output",ExpressionUUID->"9933d0c7-3920-47f8-8917-d774bda21099",
 CellID->1525113941],
Cell[415363, 11372, 759, 19, 34, "Output",ExpressionUUID->"2945626f-45f8-4d82-aff2-9e5f0423baab",
 CellID->225195795]
}, Open  ]],
Cell[416137, 11394, 750, 14, 36, "Text",ExpressionUUID->"18f0fdb2-f656-4b47-8a8a-0e6ca39e7aed",
 CellTags->"b:5.1.1",
 CellID->464180710],
Cell[CellGroupData[{
Cell[416912, 11412, 1432, 32, 61, "Input",ExpressionUUID->"c9e631a1-ab16-406f-9cdd-e270b9657a67",
 CellID->1346479071],
Cell[418347, 11446, 1247, 40, 159, "Output",ExpressionUUID->"acf4e793-e3bf-4097-8a5d-79f6912b3e95",
 CellID->999540551],
Cell[419597, 11488, 1358, 32, 56, "Output",ExpressionUUID->"2545e056-7364-4a88-a144-df0bc1e5571d",
 CellID->1660953823]
}, Open  ]],
Cell[CellGroupData[{
Cell[420992, 11525, 976, 23, 66, "Input",ExpressionUUID->"bd5d3b82-cccb-4503-b2da-2d4d6082d913",
 CellID->1824587066],
Cell[421971, 11550, 685, 24, 151, "Output",ExpressionUUID->"1c62f79d-ab70-4100-8b37-48a15650f869",
 CellID->510975808],
Cell[422659, 11576, 332, 9, 34, "Output",ExpressionUUID->"7b3f4321-e561-4df2-81b7-d541ec6e0511",
 CellID->183300703]
}, Open  ]],
Cell[423006, 11588, 467, 10, 34, "Text",ExpressionUUID->"1d49f15a-8550-4882-b664-1f3059be1691",
 CellTags->"b:5.1.1",
 CellID->1915822749],
Cell[CellGroupData[{
Cell[423498, 11602, 1338, 42, 34, "Input",ExpressionUUID->"0f089c63-fb46-4199-ae96-f6598c8f9b56",
 CellTags->"b:5.1.1",
 CellID->2010622972],
Cell[424839, 11646, 988, 25, 151, "Output",ExpressionUUID->"4bfe338b-774f-4e05-b793-754b2b81d531",
 CellTags->"b:5.1.1",
 CellID->1243365523]
}, Open  ]],
Cell[425842, 11674, 559, 11, 56, "Text",ExpressionUUID->"41edd0f4-a913-4fe5-aa0e-06ee74a8aeaa",
 CellTags->"b:5.1.1",
 CellID->1885104859],
Cell[CellGroupData[{
Cell[426426, 11689, 950, 20, 60, "Input",ExpressionUUID->"d36ec6b6-bb31-48b7-9d11-688ad68ba4cb",
 CellTags->"b:5.1.1",
 CellID->1131121022],
Cell[427379, 11711, 517, 15, 54, "Output",ExpressionUUID->"54879a71-f955-493a-814d-081ab1d2821d",
 CellTags->"b:5.1.1",
 CellID->80976104]
}, Open  ]],
Cell[427911, 11729, 369, 8, 34, "Text",ExpressionUUID->"66750b66-a0ea-42a0-a942-3da790a6d238",
 CellTags->"b:5.1.1",
 CellID->1225773510],
Cell[CellGroupData[{
Cell[428305, 11741, 288, 7, 34, "Input",ExpressionUUID->"eb16a410-7bc3-4a0d-a30d-825c43c0e93e",
 CellID->2145598433],
Cell[428596, 11750, 794, 29, 171, "Output",ExpressionUUID->"28112485-ff4f-476c-a538-1558fcc3a956",
 CellID->1608231377]
}, Open  ]],
Cell[CellGroupData[{
Cell[429427, 11784, 760, 17, 35, "Input",ExpressionUUID->"6bc042b9-4294-4e99-9d85-cfe50a73743e",
 CellTags->"b:5.1.1",
 CellID->764756899],
Cell[430190, 11803, 1133, 31, 167, "Output",ExpressionUUID->"3c7d3527-7238-4216-a2e5-e7167e15219e",
 CellTags->"b:5.1.1",
 CellID->1315572097]
}, Open  ]],
Cell[431338, 11837, 328, 11, 34, "Text",ExpressionUUID->"5140c94c-a5e4-4b80-98c5-5545d90b2af3",
 CellTags->"b:5.1.1",
 CellID->579574198],
Cell[CellGroupData[{
Cell[431691, 11852, 456, 12, 61, "Input",ExpressionUUID->"95ee239b-56e3-4ff0-8dc3-d9e7b7f0ff9f",
 CellTags->"b:5.1.1",
 CellID->2065206641],
Cell[432150, 11866, 252, 7, 34, "Output",ExpressionUUID->"206067c9-9433-44f3-8595-f681738bab9c",
 CellTags->"b:5.1.1",
 CellID->328604767],
Cell[432405, 11875, 254, 7, 34, "Output",ExpressionUUID->"ccef0447-e553-49a1-8819-e7eb8f971166",
 CellTags->"b:5.1.1",
 CellID->1679862532]
}, Open  ]],
Cell[432674, 11885, 413, 9, 34, "Text",ExpressionUUID->"dc027092-4b3f-4f7f-abcf-b8212448ff1e",
 CellTags->"b:5.1.1",
 CellID->149828826],
Cell[CellGroupData[{
Cell[433112, 11898, 868, 20, 60, "Input",ExpressionUUID->"e62624c5-daf6-4ba5-a42f-996bef153155",
 CellTags->"b:5.1.1",
 CellID->1322244298],
Cell[433983, 11920, 352, 10, 34, "Output",ExpressionUUID->"efbde6d1-d46c-48b5-91cf-bccd6b7b3f8a",
 CellTags->"b:5.1.1",
 CellID->482068215]
}, Open  ]],
Cell[434350, 11933, 223, 4, 30, "Input",ExpressionUUID->"57bb4ade-4e26-4889-a880-0ab6b2156919",
 CellID->235183781],
Cell[CellGroupData[{
Cell[434598, 11941, 1126, 30, 60, "Input",ExpressionUUID->"c18fab12-935e-4306-a5d3-e86042464a76",
 CellTags->"b:5.1.1",
 CellID->799137330],
Cell[435727, 11973, 516, 12, 34, "Output",ExpressionUUID->"c5024fca-7fe2-4b2d-abae-f58e47c05374",
 CellTags->"b:5.1.1",
 CellID->1812173021]
}, Open  ]],
Cell[436258, 11988, 503, 11, 34, "Text",ExpressionUUID->"f83f7190-02f0-451d-9153-b4bbf527b910",
 CellTags->"b:5.1.1",
 CellID->738376972],
Cell[CellGroupData[{
Cell[436786, 12003, 1352, 31, 85, "Input",ExpressionUUID->"8badbd6e-ef53-4816-8a40-46afa00fbb4c",
 CellTags->"b:5.1.1",
 CellID->1741256038],
Cell[438141, 12036, 257, 7, 34, "Output",ExpressionUUID->"c5f09a22-6c7d-47d4-949e-7ad6acad5b85",
 CellTags->"b:5.1.1",
 CellID->1578882193],
Cell[438401, 12045, 259, 7, 34, "Output",ExpressionUUID->"115996dd-f693-4b54-8514-abcb61d99248",
 CellTags->"b:5.1.1",
 CellID->1965075419],
Cell[438663, 12054, 661, 21, 83, "Output",ExpressionUUID->"ed8e1f7b-ae2e-404f-a561-5fc44ae3ff68",
 CellTags->"b:5.1.1",
 CellID->871559920]
}, Open  ]],
Cell[CellGroupData[{
Cell[439361, 12080, 386, 9, 34, "Input",ExpressionUUID->"ebef3a97-96c7-4138-bde4-e07af7be0886",
 CellID->1530572997],
Cell[439750, 12091, 259, 8, 39, "Output",ExpressionUUID->"38606a2a-b950-43c6-a9b7-c7d8906ee185",
 CellID->321619253]
}, Open  ]],
Cell[CellGroupData[{
Cell[440046, 12104, 428, 11, 34, "Input",ExpressionUUID->"19bdd8fc-7fed-4c74-b6d0-d11a74ee468b",
 CellID->1781236813],
Cell[440477, 12117, 361, 12, 39, "Output",ExpressionUUID->"1d0f2f96-268d-4ba7-a8e7-2029c18f471b",
 CellID->238445672]
}, Open  ]],
Cell[CellGroupData[{
Cell[440875, 12134, 469, 16, 40, "Input",ExpressionUUID->"2ea3781d-1a83-434b-9793-181911f11833",
 CellID->1325076911],
Cell[441347, 12152, 362, 12, 39, "Output",ExpressionUUID->"07c231b5-2af7-488b-ac14-9f0526d1066a",
 CellID->351924002]
}, Open  ]],
Cell[CellGroupData[{
Cell[441746, 12169, 967, 24, 110, "Input",ExpressionUUID->"eead8c48-ee59-4d0f-92e3-9dda3a98edba",
 CellID->1162223787],
Cell[442716, 12195, 638, 20, 83, "Output",ExpressionUUID->"c770b6ff-87a3-4720-8360-e6717545c0cd",
 CellID->616737776],
Cell[443357, 12217, 326, 8, 34, "Output",ExpressionUUID->"1667494a-2dba-436e-9ecf-090e65beb109",
 CellID->1755720810],
Cell[443686, 12227, 639, 20, 83, "Output",ExpressionUUID->"00dcbd7f-a680-46f8-8d6b-3889f33f2581",
 CellID->1974343890],
Cell[444328, 12249, 327, 8, 34, "Output",ExpressionUUID->"f663377a-4b11-4e3f-8ce9-8c68b5bf35a8",
 CellID->2027929433]
}, Open  ]],
Cell[CellGroupData[{
Cell[444692, 12262, 476, 12, 34, "Input",ExpressionUUID->"daf3705f-f8bf-4d39-8e25-82cb8b17722b",
 CellID->2131418644],
Cell[445171, 12276, 638, 20, 83, "Output",ExpressionUUID->"063dacf4-f23e-4cd2-9cbe-07b26b9f12a8",
 CellID->697018894]
}, Open  ]],
Cell[445824, 12299, 2040, 66, 116, "Text",ExpressionUUID->"350b9b6b-8dd3-4693-a7cc-3016ffa4315e",
 CellTags->"b:5.1.1",
 CellID->1312550803],
Cell[447867, 12367, 441, 10, 34, "Text",ExpressionUUID->"0924ebea-1d92-47ec-ae10-f63872bd02e1",
 CellTags->"b:5.1.1",
 CellID->422389411],
Cell[CellGroupData[{
Cell[448333, 12381, 712, 21, 34, "Input",ExpressionUUID->"610f056c-36f7-4e27-9c6d-4168a136fcf8",
 CellID->1819302349],
Cell[449048, 12404, 235, 6, 34, "Output",ExpressionUUID->"4658c6c7-c97f-4a0b-9a2c-e2049348fa86",
 CellID->273257073]
}, Open  ]],
Cell[CellGroupData[{
Cell[449320, 12415, 221, 4, 30, "Input",ExpressionUUID->"41999070-83f8-4fc3-a4e2-d997de3a064b",
 CellID->858753316],
Cell[449544, 12421, 6455, 161, 159, "Output",ExpressionUUID->"87d12fa4-de21-4d9a-bbce-ade7106842bb",
 CellID->1311588625]
}, Open  ]],
Cell[CellGroupData[{
Cell[456036, 12587, 972, 29, 68, "Input",ExpressionUUID->"2557d480-d927-43ea-9f8e-60f73b425e65",
 CellID->731125707],
Cell[457011, 12618, 4488, 114, 161, "Output",ExpressionUUID->"70c26705-27fd-49dc-a0e9-77ee198ecbe5",
 CellID->2097867567]
}, Open  ]],
Cell[CellGroupData[{
Cell[461536, 12737, 2611, 67, 137, "Input",ExpressionUUID->"3336553c-4cb8-44b4-8e55-74e40473aa4f",
 CellID->83465538],
Cell[464150, 12806, 701, 20, 83, "Output",ExpressionUUID->"50fe3584-d06d-495d-9824-c31e237eeebc",
 CellID->1473682123],
Cell[464854, 12828, 701, 20, 83, "Output",ExpressionUUID->"35e232e4-9872-432d-888a-23262d9acb2f",
 CellID->1246540410],
Cell[465558, 12850, 236, 6, 34, "Output",ExpressionUUID->"daa1ba19-9a7b-49b6-9cb5-8f24c481ef3e",
 CellID->1901694385],
Cell[465797, 12858, 698, 20, 83, "Output",ExpressionUUID->"abeaba3d-da8e-456a-9bbd-81bdf90d16fe",
 CellID->778410394],
Cell[466498, 12880, 235, 6, 34, "Output",ExpressionUUID->"ffd94a7b-469f-457c-8a6b-2fa52a9bc3ae",
 CellID->273114434]
}, Open  ]],
Cell[466748, 12889, 422, 10, 34, "Text",ExpressionUUID->"8247f088-0ba6-48d1-bbbf-138ef69f01af",
 CellTags->"b:5.1.1",
 CellID->147551722],
Cell[CellGroupData[{
Cell[467195, 12903, 2158, 60, 121, "Input",ExpressionUUID->"c2f895d7-e4aa-4a58-a374-92b287633f60",
 CellID->832742574],
Cell[469356, 12965, 236, 6, 34, "Output",ExpressionUUID->"685abdc4-28d9-43a6-bd09-42a0b076c286",
 CellID->1061738599],
Cell[469595, 12973, 236, 6, 34, "Output",ExpressionUUID->"185ea40d-28f0-4d90-8647-3558fc74c95c",
 CellID->1199010470],
Cell[469834, 12981, 236, 6, 34, "Output",ExpressionUUID->"33c924bb-b264-418b-bdb0-0d5d4a1c9f22",
 CellID->1929909489],
Cell[470073, 12989, 235, 6, 34, "Output",ExpressionUUID->"040f4bfb-c1b7-443e-b70f-03648c77d96a",
 CellID->395777335]
}, Open  ]],
Cell[470323, 12998, 488, 11, 34, "Text",ExpressionUUID->"0a0691fe-ad9b-4811-9815-d39febd8453d",
 CellTags->"b:5.1.1",
 CellID->1203011483],
Cell[CellGroupData[{
Cell[470836, 13013, 2355, 65, 85, "Input",ExpressionUUID->"48c8bf72-0fdb-40c3-be68-2af8e916d230",
 CellID->1809609647],
Cell[473194, 13080, 236, 6, 34, "Output",ExpressionUUID->"f1a45c3d-4e0b-4e89-a08b-d77a56653d6e",
 CellID->1072814586],
Cell[473433, 13088, 235, 6, 34, "Output",ExpressionUUID->"72c0d098-d476-40bc-a734-f5f749d3e6da",
 CellID->522046690],
Cell[473671, 13096, 236, 6, 34, "Output",ExpressionUUID->"035b1736-a6bc-43c4-806e-431ad66a3dcc",
 CellID->1568020835]
}, Open  ]],
Cell[473922, 13105, 356, 9, 72, "Section",ExpressionUUID->"6e5e7dbf-3a1e-4e19-b04e-32513867125c",
 CellID->459418141],
Cell[474281, 13116, 347, 8, 38, "Section",ExpressionUUID->"fb7263ff-4ba6-4c08-82c9-567a1e4e5750",
 CellID->1236984822],
Cell[474631, 13126, 447, 10, 30, "Text",ExpressionUUID->"4ceb7ec0-d33c-4955-b037-3d31a97f8fe0",
 CellTags->"b:5.1.2",
 CellID->1573400686],
Cell[475081, 13138, 1420, 45, 154, "Text",ExpressionUUID->"b2025513-d84e-43f0-9ba2-c0a0f51efb2b",
 CellTags->"b:5.1.2",
 CellID->31700444],
Cell[476504, 13185, 411, 9, 34, "Text",ExpressionUUID->"a20296b5-b24f-4643-a6e0-bc4518f6d0c8",
 CellTags->"b:5.1.2",
 CellID->213417852],
Cell[CellGroupData[{
Cell[476940, 13198, 531, 12, 34, "Input",ExpressionUUID->"d01f4d51-3fff-4419-ad3b-a13929241b5a",
 CellID->616148074],
Cell[477474, 13212, 1083, 38, 123, "Output",ExpressionUUID->"8167432b-5872-454e-a76e-e27453bc8c23",
 CellID->1954341508]
}, Open  ]],
Cell[CellGroupData[{
Cell[478594, 13255, 827, 19, 86, "Input",ExpressionUUID->"98107538-1556-4605-b74e-358ca6648d9d",
 CellID->929336186],
Cell[479424, 13276, 1082, 38, 123, "Output",ExpressionUUID->"914169aa-7849-42d8-a0dd-f750da4c2973",
 CellID->855344091],
Cell[480509, 13316, 1082, 38, 123, "Output",ExpressionUUID->"7bc29fc6-dae2-4301-9b5b-463033a50ea7",
 CellID->512604419],
Cell[481594, 13356, 236, 6, 34, "Output",ExpressionUUID->"3e840b35-a2e7-4ed3-9510-01b913caaa35",
 CellID->1785562016]
}, Open  ]],
Cell[CellGroupData[{
Cell[481867, 13367, 3886, 71, 202, "Input",ExpressionUUID->"17396c8c-87f6-4568-8821-3bbe1c84a279",
 CellID->1389156325],
Cell[485756, 13440, 20936, 465, 190, "Output",ExpressionUUID->"5dd900c3-e566-4ef8-91a4-e2df6b3aff9f",
 CellID->1004319734],
Cell[506695, 13907, 2218, 62, 237, "Output",ExpressionUUID->"c29d0914-89ab-4d5d-ab1d-254032fea5f9",
 CellID->380303918],
Cell[508916, 13971, 266, 6, 34, "Output",ExpressionUUID->"1e50ea41-e01e-420f-889b-277cf00a886e",
 CellID->856616354],
Cell[509185, 13979, 2887, 75, 277, "Output",ExpressionUUID->"62a95af5-cb99-48b1-9925-28e38d202f80",
 CellID->420692190]
}, Open  ]],
Cell[512087, 14057, 309, 8, 34, "Input",ExpressionUUID->"034bec5f-0037-4e00-b809-9b0e12b05515",
 CellID->1623344346],
Cell[CellGroupData[{
Cell[512421, 14069, 601, 18, 35, "Input",ExpressionUUID->"9e8e09e3-5c6a-44b1-ad8b-24b264350687",
 CellID->434533884],
Cell[513025, 14089, 236, 6, 34, "Output",ExpressionUUID->"e055cca6-c039-4a5f-b9ba-d021b4969ec2",
 CellID->1057471406]
}, Open  ]],
Cell[CellGroupData[{
Cell[513298, 14100, 873, 21, 61, "Input",ExpressionUUID->"31d6e3ff-9f0d-48b3-b1bc-35507991ac59",
 CellID->1766588588],
Cell[514174, 14123, 721, 24, 103, "Output",ExpressionUUID->"fcd80fa0-9f02-433a-a434-de26fe33977a",
 CellID->347258070],
Cell[514898, 14149, 235, 6, 34, "Output",ExpressionUUID->"25407c94-66e7-4f85-98fb-66618acd8335",
 CellID->1653313591]
}, Open  ]],
Cell[515148, 14158, 530, 15, 34, "Text",ExpressionUUID->"586bbec2-4eea-4339-b199-179e89f9bd21",
 CellTags->"b:5.1.2",
 CellID->2092978741],
Cell[515681, 14175, 900, 21, 37, "Input",ExpressionUUID->"699ba197-e76c-4207-b34c-959c9a9e941a",
 CellID->911562127],
Cell[CellGroupData[{
Cell[516606, 14200, 824, 21, 117, "Input",ExpressionUUID->"13a1b7b8-8083-4495-8cc6-69a656a8fb6d",
 CellID->476330791],
Cell[517433, 14223, 468, 12, 54, "Output",ExpressionUUID->"9dfd7331-97f9-4c07-9196-09e6245c7827",
 CellID->950615404],
Cell[517904, 14237, 469, 12, 54, "Output",ExpressionUUID->"c3b6817c-942b-42b9-bdaa-27d4f7d2e93f",
 CellID->1862244995],
Cell[518376, 14251, 491, 13, 54, "Output",ExpressionUUID->"7414f41b-ff15-490b-b8ef-148c72229b76",
 CellID->1324959587],
Cell[518870, 14266, 491, 13, 54, "Output",ExpressionUUID->"70a69116-dfba-4a87-84c3-bb99a1e9ad78",
 CellID->1337842966]
}, Open  ]],
Cell[519376, 14282, 551, 11, 56, "Text",ExpressionUUID->"90f4dc09-7246-4ae1-93c2-2c7a86e9d0b6",
 CellTags->"b:5.1.2",
 CellID->1912508314],
Cell[CellGroupData[{
Cell[519952, 14297, 1065, 24, 75, "Input",ExpressionUUID->"77bf8c40-8dc8-4560-98c8-fc8331b16287",
 CellID->573361425],
Cell[521020, 14323, 31566, 660, 119, "Output",ExpressionUUID->"5eb5a8ac-91b7-44c8-8458-486fc09c128e",
 CellID->972945472]
}, Open  ]],
Cell[CellGroupData[{
Cell[552623, 14988, 1098, 26, 157, "Input",ExpressionUUID->"dce8d334-433e-4622-9675-a9c3784e700b",
 CellID->1252668281],
Cell[553724, 15016, 725, 24, 103, "Output",ExpressionUUID->"11056209-9f8f-4138-b8b3-1a8ab1fc81a1",
 CellID->1354059646],
Cell[554452, 15042, 368, 7, 56, "Output",ExpressionUUID->"bd20f09e-b6a0-479b-b2d0-99cce772fb06",
 CellID->796263063]
}, Open  ]],
Cell[554835, 15052, 570, 11, 56, "Text",ExpressionUUID->"c2163940-5b4a-416a-b7bd-e1d941569499",
 CellTags->"b:5.1.2",
 CellID->2020051968],
Cell[555408, 15065, 488, 10, 36, "Text",ExpressionUUID->"0f36b87d-6a31-4caa-9dfe-915a9b6f1daa",
 CellTags->"b:5.1.2",
 CellID->1444450753],
Cell[CellGroupData[{
Cell[555921, 15079, 1187, 29, 155, "Input",ExpressionUUID->"89e68763-1797-42a4-a602-e5558020733b",
 CellID->1728659983],
Cell[557111, 15110, 798, 24, 99, "Output",ExpressionUUID->"044538dd-93e4-4ce8-b9da-ee5d6bf374bd",
 CellID->1822695384],
Cell[557912, 15136, 798, 24, 99, "Output",ExpressionUUID->"715e23f8-0665-49ca-be89-5da1f539f997",
 CellID->187094433],
Cell[558713, 15162, 844, 26, 99, "Output",ExpressionUUID->"a3237d78-a465-4d81-ad59-dca7a93ccc24",
 CellID->580076223],
Cell[559560, 15190, 845, 26, 99, "Output",ExpressionUUID->"357c5321-e76e-470c-8238-00195e793eb3",
 CellID->1912806228]
}, Open  ]],
Cell[560420, 15219, 557, 12, 36, "Text",ExpressionUUID->"3d7e8883-1d90-4a2e-9c6d-a2f75115bdd9",
 CellTags->"b:5.1.2",
 CellID->2031050894],
Cell[CellGroupData[{
Cell[561002, 15235, 1040, 25, 151, "Input",ExpressionUUID->"fd7c5acb-e4e5-42db-a281-b3144d2afbc1",
 CellID->282074018],
Cell[562045, 15262, 2738, 81, 171, "Output",ExpressionUUID->"dc1ad17e-7f98-4adc-9c54-fc628bed2c14",
 CellID->704078406],
Cell[564786, 15345, 2739, 81, 171, "Output",ExpressionUUID->"28fa9ef4-4172-49d9-ad87-3d74627bc332",
 CellID->810874672],
Cell[567528, 15428, 2738, 81, 171, "Output",ExpressionUUID->"493b10bf-db0a-40e8-8040-bc415fb9ca10",
 CellID->439388442],
Cell[570269, 15511, 2798, 83, 171, "Output",ExpressionUUID->"a0df5d01-5585-4a07-a33b-9a89d208fd2b",
 CellID->1752766308]
}, Open  ]],
Cell[573082, 15597, 777, 15, 84, "Text",ExpressionUUID->"78c087b7-43b4-4de9-83af-9d870c1193ba",
 CellID->1681331124],
Cell[573862, 15614, 423, 14, 72, "Section",ExpressionUUID->"994af28d-ec69-4ba1-9f7d-a6b62f61e797",
 CellID->725256518],
Cell[574288, 15630, 346, 8, 38, "Section",ExpressionUUID->"40d98d31-4e65-47a8-9cbe-fd0149e7b59e",
 CellID->269117654],
Cell[574637, 15640, 511, 11, 52, "Text",ExpressionUUID->"033c8045-1fa5-4c46-a41c-f1fdbaa0d5d0",
 CellTags->"b:5.1.2",
 CellID->902749858],
Cell[CellGroupData[{
Cell[575173, 15655, 572, 14, 85, "Input",ExpressionUUID->"a7e1553a-8d69-4fa2-ab2a-83cb15a79cb7",
 CellID->544897351],
Cell[575748, 15671, 967, 24, 151, "Output",ExpressionUUID->"f9da1787-1aa5-4fbe-8f4f-b90598468682",
 CellID->1710152657],
Cell[576718, 15697, 968, 24, 151, "Output",ExpressionUUID->"b63fa631-731c-4c72-b50a-42db6c061c6b",
 CellID->614574751],
Cell[577689, 15723, 1735, 60, 231, "Output",ExpressionUUID->"44daae2e-6cdb-4c7f-8650-42dc03187e1b",
 CellID->1908981634]
}, Open  ]],
Cell[579439, 15786, 675, 20, 61, "Input",ExpressionUUID->"6e3c251c-2ba8-4b44-a379-a3e5e8101f7b",
 CellTags->"b:5.1.2",
 CellID->1219507449],
Cell[CellGroupData[{
Cell[580139, 15810, 832, 24, 61, "Input",ExpressionUUID->"83985a01-d559-437b-ba0f-cff5894cb5e4",
 CellID->1327906532],
Cell[580974, 15836, 3189, 87, 304, "Output",ExpressionUUID->"6b58454a-4864-4179-a452-4c555a8c01df",
 CellID->848636458]
}, Open  ]],
Cell[CellGroupData[{
Cell[584200, 15928, 783, 19, 126, "Input",ExpressionUUID->"156ed9f6-f736-4e16-9da4-a5f5b3b6f9b2",
 CellID->232906771],
Cell[584986, 15949, 269, 6, 34, "Output",ExpressionUUID->"ddc663fb-5548-4e38-a0fa-0f91150b4057",
 CellID->1594049879],
Cell[585258, 15957, 3765, 98, 304, "Output",ExpressionUUID->"ea964b46-b008-4840-8951-0bb7b119d13f",
 CellID->1337820028]
}, Open  ]],
Cell[589038, 16058, 301, 7, 36, "Text",ExpressionUUID->"721cafa1-d38e-4a90-bf2a-75998990bd06",
 CellTags->"b:5.1.2",
 CellID->677768375],
Cell[CellGroupData[{
Cell[589364, 16069, 882, 23, 61, "Input",ExpressionUUID->"4077cc5d-0b4e-4793-8ba6-379c308ce269",
 CellID->999814937],
Cell[590249, 16094, 473, 13, 54, "Output",ExpressionUUID->"69fa777b-5954-4f9f-886a-9e4a29682a7d",
 CellID->587426506]
}, Open  ]],
Cell[590737, 16110, 533, 11, 34, "Text",ExpressionUUID->"d1350ba3-91b3-4902-944c-5083a05378cf",
 CellTags->"b:5.1.2",
 CellID->1977592031],
Cell[CellGroupData[{
Cell[591295, 16125, 859, 20, 61, "Input",ExpressionUUID->"bfb58204-be27-4674-a23c-a0b24be20f62",
 CellID->784860398],
Cell[592157, 16147, 1066, 28, 167, "Output",ExpressionUUID->"39e3bd89-98b7-439e-bbdd-c9bd5fa04208",
 CellID->894961083],
Cell[593226, 16177, 237, 6, 34, "Output",ExpressionUUID->"8199e7a2-3c98-4089-a0f1-a377b8521bf1",
 CellID->635458393]
}, Open  ]],
Cell[593478, 16186, 303, 9, 36, "Text",ExpressionUUID->"f8bab412-229a-4874-9747-da15022d46be",
 CellTags->"b:5.1.2",
 CellID->1304149312],
Cell[CellGroupData[{
Cell[593806, 16199, 1649, 48, 176, "Input",ExpressionUUID->"b07206fb-b895-4bbd-8453-214c35bab236",
 CellTags->"b:5.1.2",
 CellID->1619385502],
Cell[595458, 16249, 794, 29, 171, "Output",ExpressionUUID->"0b0de7c8-edc7-4bce-92bb-bc026e7a75b9",
 CellTags->"b:5.1.2",
 CellID->713034620],
Cell[596255, 16280, 496, 14, 54, "Output",ExpressionUUID->"bfb5d2f2-01ea-439b-b9da-f0714fa262ff",
 CellTags->"b:5.1.2",
 CellID->1014108080],
Cell[596754, 16296, 258, 7, 34, "Output",ExpressionUUID->"b5284a9b-acb3-4679-bea5-5509770ca6b4",
 CellTags->"b:5.1.2",
 CellID->1684277968]
}, Open  ]],
Cell[597027, 16306, 417, 9, 36, "Text",ExpressionUUID->"3dd8387d-d3b5-4294-bb31-fca365b6fb0b",
 CellTags->"b:5.1.2",
 CellID->533165535],
Cell[CellGroupData[{
Cell[597469, 16319, 1965, 53, 141, "Input",ExpressionUUID->"108bfd4c-f0d3-431b-a8b4-71f5699e72e9",
 CellID->1406836767],
Cell[599437, 16374, 236, 6, 34, "Output",ExpressionUUID->"31450e9b-1167-43f9-b317-6d07e68cb7d9",
 CellID->1677857069],
Cell[599676, 16382, 236, 6, 34, "Output",ExpressionUUID->"c85a2181-020c-428a-9616-da3d8fc3ad53",
 CellID->1135989926],
Cell[599915, 16390, 236, 6, 34, "Output",ExpressionUUID->"3207b22a-60e0-4bd2-a7c0-b49dd55a628a",
 CellID->1453064452]
}, Open  ]],
Cell[CellGroupData[{
Cell[600188, 16401, 382, 11, 33, "Input",ExpressionUUID->"5426402a-73a9-4c61-a6a2-9cb021098c6f",
 CellID->77259091],
Cell[600573, 16414, 664, 20, 49, "Output",ExpressionUUID->"a34977fd-67aa-498d-9733-a938e39b298e",
 CellID->470211080]
}, Open  ]],
Cell[601252, 16437, 1423, 34, 64, "Text",ExpressionUUID->"20803f82-634d-4746-aced-32444db20b1b",
 CellTags->"b:5.1.2",
 CellID->776787928],
Cell[CellGroupData[{
Cell[602700, 16475, 1139, 35, 87, "Input",ExpressionUUID->"91d7c117-cef8-4e72-8e49-f77d3cf003b4",
 CellID->1719388873],
Cell[603842, 16512, 655, 19, 65, "Output",ExpressionUUID->"aa2733de-6743-4601-9868-eb06d8d6fc7b",
 CellID->97800600],
Cell[604500, 16533, 656, 19, 65, "Output",ExpressionUUID->"aac8a4cc-0168-451e-aefe-53c37bc9485f",
 CellID->909694245],
Cell[605159, 16554, 657, 19, 65, "Output",ExpressionUUID->"0f401fa5-b213-469d-a733-47f1c09338bc",
 CellID->1295092722]
}, Open  ]],
Cell[CellGroupData[{
Cell[605853, 16578, 1104, 33, 87, "Input",ExpressionUUID->"e6a2fd97-cabd-49a7-b59f-3d34a73f7a39",
 CellID->193715260],
Cell[606960, 16613, 678, 20, 65, "Output",ExpressionUUID->"a143ede5-30a1-4038-8b6f-73aa59ce4282",
 CellID->1876616309],
Cell[607641, 16635, 700, 21, 65, "Output",ExpressionUUID->"b6bd21aa-b9f8-4035-b1c4-46a9d1fc3dfa",
 CellID->197981874],
Cell[608344, 16658, 659, 19, 65, "Output",ExpressionUUID->"13da467b-1f9f-4fdd-ba39-6f79713a30f0",
 CellID->1029187115]
}, Open  ]],
Cell[CellGroupData[{
Cell[609040, 16682, 609, 17, 35, "Input",ExpressionUUID->"d85bbf77-501e-4891-9a18-7e2b94c901ba",
 CellID->901706553],
Cell[609652, 16701, 5283, 157, 171, "Output",ExpressionUUID->"b706d8ec-7a1d-4e75-bf98-40c52033c651",
 CellID->1714548867]
}, Open  ]],
Cell[614950, 16861, 415, 9, 36, "Text",ExpressionUUID->"7afe57c2-2263-4464-be38-77a4fbf81fab",
 CellTags->"b:5.1.2",
 CellID->1951873683],
Cell[615368, 16872, 963, 23, 33, "Input",ExpressionUUID->"731c3364-363a-41bc-924d-47bd8321f2bd",
 CellID->180798609],
Cell[CellGroupData[{
Cell[616356, 16899, 1689, 59, 210, "Input",ExpressionUUID->"c3d52d2e-d26e-433e-a9e6-290cf9d9f7b6",
 CellID->2140344605],
Cell[618048, 16960, 474, 13, 54, "Output",ExpressionUUID->"2014c594-055a-497c-8819-4c4e655228c5",
 CellID->1487232223],
Cell[618525, 16975, 474, 13, 54, "Output",ExpressionUUID->"a67e8792-0c32-4172-8090-5648012e9a19",
 CellID->1349804528],
Cell[619002, 16990, 473, 13, 54, "Output",ExpressionUUID->"7d1c1e2a-85a6-4139-887e-5c827f430d09",
 CellID->147455188],
Cell[619478, 17005, 472, 13, 54, "Output",ExpressionUUID->"8c7016c5-a50e-4081-bc89-1f4cc226ba1f",
 CellID->83216078],
Cell[619953, 17020, 497, 14, 54, "Output",ExpressionUUID->"aabe6975-967b-4299-8d44-f6b945280e89",
 CellID->600768749],
Cell[620453, 17036, 498, 14, 54, "Output",ExpressionUUID->"145e1ea6-863e-4b76-b7c2-6395c978b988",
 CellID->1799739896],
Cell[620954, 17052, 499, 14, 54, "Output",ExpressionUUID->"40a5361f-592f-46ea-b49b-6eee9e5d5959",
 CellID->921264077],
Cell[621456, 17068, 499, 14, 54, "Output",ExpressionUUID->"d8ce2b70-fb92-4caa-8eb0-37eea8065406",
 CellID->328247269]
}, Open  ]],
Cell[621970, 17085, 441, 10, 72, "Section",ExpressionUUID->"11c5b36d-574f-4717-a142-39d08637e107",
 CellID->459850196],
Cell[622414, 17097, 347, 8, 38, "Section",ExpressionUUID->"6c89412c-4f3c-42a9-ba55-6fbee5f2adae",
 CellID->2056082266],
Cell[622764, 17107, 684, 15, 81, "Text",ExpressionUUID->"8da6c954-81d1-4f7c-8a2d-bb93e4a1b939",
 CellTags->"b:5.1.3",
 CellID->442078480],
Cell[CellGroupData[{
Cell[623473, 17126, 2514, 77, 215, "Input",ExpressionUUID->"d4d8d20a-1e3f-413b-b85f-aa6e7dff1774",
 CellTags->"b:5.1.3",
 CellID->1867078387],
Cell[625990, 17205, 2117, 49, 287, "Output",ExpressionUUID->"52624d07-f318-4354-82f1-6c2ef6cdc781",
 CellTags->"b:5.1.3",
 CellID->2113844587],
Cell[628110, 17256, 647, 19, 49, "Output",ExpressionUUID->"e6ebfea7-49a0-4f9d-90e2-fa6b1cd53162",
 CellTags->"b:5.1.3",
 CellID->1564001388],
Cell[628760, 17277, 720, 21, 83, "Output",ExpressionUUID->"7421b217-191c-4bdb-a5f0-803cc6ee4c23",
 CellTags->"b:5.1.3",
 CellID->971488836],
Cell[629483, 17300, 2793, 83, 171, "Output",ExpressionUUID->"e8280fcc-fc7b-45e8-a4f0-e1eea23fb5db",
 CellTags->"b:5.1.3",
 CellID->494698511],
Cell[632279, 17385, 721, 21, 83, "Output",ExpressionUUID->"0231a761-c77f-4f65-9a03-681557d99e56",
 CellTags->"b:5.1.3",
 CellID->1488676840],
Cell[633003, 17408, 2793, 83, 171, "Output",ExpressionUUID->"0f0e7297-493e-41fb-810d-40dba7d73559",
 CellTags->"b:5.1.3",
 CellID->2007162330]
}, Open  ]],
Cell[635811, 17494, 310, 9, 36, "Text",ExpressionUUID->"b29baa0e-fe1c-48bd-88e0-374783fa36ab",
 CellTags->"b:5.1.3",
 CellID->955400345],
Cell[CellGroupData[{
Cell[636146, 17507, 651, 17, 139, "Input",ExpressionUUID->"77c15bce-2531-4060-9197-2399d7284025",
 CellTags->"b:5.1.3",
 CellID->678369796],
Cell[636800, 17526, 258, 7, 34, "Output",ExpressionUUID->"d709897f-1571-436d-acf3-810a4d442fbd",
 CellTags->"b:5.1.3",
 CellID->1704153234],
Cell[637061, 17535, 256, 7, 34, "Output",ExpressionUUID->"ad6925ed-128c-47a3-8994-7714ff181a17",
 CellTags->"b:5.1.3",
 CellID->714003799],
Cell[637320, 17544, 254, 7, 34, "Output",ExpressionUUID->"8f1b77b0-072d-47f6-b870-4c42917c1a98",
 CellTags->"b:5.1.3",
 CellID->123230357],
Cell[637577, 17553, 254, 7, 34, "Output",ExpressionUUID->"4eda8a4a-3c8c-46f4-b912-a57caff2ab02",
 CellTags->"b:5.1.3",
 CellID->958374391],
Cell[637834, 17562, 254, 7, 34, "Output",ExpressionUUID->"ffef8353-cebd-4bc5-931c-9efb271e5c90",
 CellTags->"b:5.1.3",
 CellID->1271037037]
}, Open  ]],
Cell[638103, 17572, 290, 9, 36, "Text",ExpressionUUID->"70d28756-e093-4be9-b8f1-b392682f057c",
 CellTags->"b:5.1.3",
 CellID->370479449],
Cell[CellGroupData[{
Cell[638418, 17585, 2159, 69, 34, "Input",ExpressionUUID->"460546ec-f266-49dc-9e00-7e552c36a036",
 CellTags->"b:5.1.3",
 CellID->1093006690],
Cell[640580, 17656, 1064, 28, 175, "Output",ExpressionUUID->"b2f838b1-8cb1-45fd-8c02-46127f757a1a",
 CellTags->"b:5.1.3",
 CellID->1299644150]
}, Open  ]],
Cell[CellGroupData[{
Cell[641681, 17689, 335, 9, 60, "Input",ExpressionUUID->"4e5ef63c-59d4-454f-af2e-982bd035eea4",
 CellTags->"b:5.1.3",
 CellID->588322392],
Cell[642019, 17700, 255, 7, 34, "Output",ExpressionUUID->"4ebcc279-60ab-4540-974a-6d945f01c779",
 CellTags->"b:5.1.3",
 CellID->1063055413],
Cell[642277, 17709, 276, 8, 50, "Output",ExpressionUUID->"ea572a49-e594-47a2-b8da-50234f88eb30",
 CellTags->"b:5.1.3",
 CellID->1855866898]
}, Open  ]],
Cell[642568, 17720, 497, 11, 60, "Text",ExpressionUUID->"6fc7e343-f33d-4c7a-9824-bcb0fb7d441d",
 CellTags->"b:5.1.3",
 CellID->919731556],
Cell[CellGroupData[{
Cell[643090, 17735, 343, 11, 34, "Input",ExpressionUUID->"c6d7e4cf-3956-4b95-9d40-31cdf055eb8b",
 CellTags->"b:5.1.3",
 CellID->340970586],
Cell[643436, 17748, 697, 21, 65, "Output",ExpressionUUID->"b0e49e6d-4a47-4e54-a7f3-abc7fbc186ef",
 CellTags->"b:5.1.3",
 CellID->1502465658]
}, Open  ]],
Cell[CellGroupData[{
Cell[644170, 17774, 344, 11, 34, "Input",ExpressionUUID->"ca9d29d8-68be-49a9-9834-9d318c4a5059",
 CellTags->"b:5.1.3",
 CellID->1206268706],
Cell[644517, 17787, 696, 21, 65, "Output",ExpressionUUID->"c3acaadc-b9c5-4088-b0f1-f2785bcafc6e",
 CellTags->"b:5.1.3",
 CellID->1827592580]
}, Open  ]],
Cell[CellGroupData[{
Cell[645250, 17813, 344, 11, 34, "Input",ExpressionUUID->"712643d0-bc1d-482b-a119-3420db6664e8",
 CellTags->"b:5.1.3",
 CellID->1512514062],
Cell[645597, 17826, 646, 19, 49, "Output",ExpressionUUID->"252ba9f4-5bd9-4f1b-877b-8b07ded432a5",
 CellTags->"b:5.1.3",
 CellID->889889019]
}, Open  ]],
Cell[CellGroupData[{
Cell[646280, 17850, 401, 11, 35, "Input",ExpressionUUID->"d4f7e0e4-6550-4199-9175-6f1f1672256d",
 CellID->1059910495],
Cell[646684, 17863, 2842, 84, 195, "Output",ExpressionUUID->"cfa9f9bd-7f4a-463b-9765-58cddf65579a",
 CellID->1288624625]
}, Open  ]],
Cell[649541, 17950, 549, 14, 60, "Text",ExpressionUUID->"5c768566-a768-4ae1-a789-8924ed79d262",
 CellTags->"b:5.1.4",
 CellID->985654226],
Cell[CellGroupData[{
Cell[650115, 17968, 5969, 166, 193, "Input",ExpressionUUID->"7a100f1a-ce09-4175-a1b4-98d88c234dc6",
 CellTags->"b:5.1.4",
 CellID->201723424],
Cell[656087, 18136, 7684, 217, 115, "Output",ExpressionUUID->"81d3bf94-4238-4b4c-a5f5-d5b8277049eb",
 CellTags->"b:5.1.4",
 CellID->541492380],
Cell[663774, 18355, 258, 7, 34, "Output",ExpressionUUID->"b846603e-6e92-4680-8ec9-361a9bb04abf",
 CellTags->"b:5.1.4",
 CellID->1974218321],
Cell[664035, 18364, 3696, 105, 179, "Output",ExpressionUUID->"0f3d3d2e-3118-4e2e-96f9-41fe349c1bd2",
 CellTags->"b:5.1.4",
 CellID->2064974897]
}, Open  ]],
Cell[667746, 18472, 1143, 32, 40, "Text",ExpressionUUID->"2b21b997-7386-46cc-b10f-5cd709e13a8f",
 CellTags->"b:5.1.4",
 CellID->1313411647],
Cell[CellGroupData[{
Cell[668914, 18508, 3397, 94, 136, "Input",ExpressionUUID->"161abee2-218d-4f07-9ad8-8423db4772e1",
 CellID->141263213],
Cell[672314, 18604, 4024, 113, 179, "Output",ExpressionUUID->"954d6cf3-7615-42fd-9e2c-f304dc803490",
 CellID->549874712],
Cell[676341, 18719, 236, 6, 34, "Output",ExpressionUUID->"820734f5-cd72-4b21-a1e4-d44536b09a48",
 CellID->1122151543]
}, Open  ]],
Cell[676592, 18728, 16034, 376, 191, "Text",ExpressionUUID->"696c8d91-be15-4eb7-9b07-f4e082068e79",
 CellTags->"b:5.1.4",
 CellID->205059545],
Cell[692629, 19106, 1056, 19, 144, "Text",ExpressionUUID->"58533d77-6a16-48c6-9074-c73db65dd1c9",
 CellTags->"b:5.1.4",
 CellID->20671885],
Cell[CellGroupData[{
Cell[693710, 19129, 252, 7, 35, "Input",ExpressionUUID->"8178b701-bb6a-4ff2-8337-abdb3305ac24",
 CellID->255844472],
Cell[693965, 19138, 2315, 53, 83, "Output",ExpressionUUID->"f5935a57-7cd7-4bb7-8d1a-539dfe47ddc9",
 CellID->799595247]
}, Open  ]],
Cell[CellGroupData[{
Cell[696317, 19196, 2163, 52, 268, "Input",ExpressionUUID->"c369f066-2a26-4ce0-9dd2-14889626582a",
 CellID->1687504028],
Cell[698483, 19250, 235, 6, 34, "Output",ExpressionUUID->"9fbc1f35-4f24-4842-b1d1-4d5e7cdd1375",
 CellID->1992137050],
Cell[698721, 19258, 235, 6, 34, "Output",ExpressionUUID->"d52e4b79-8c26-4177-a6cc-c86e418387ef",
 CellID->429858973],
Cell[698959, 19266, 3237, 89, 171, "Output",ExpressionUUID->"6254dfe1-f52e-40fc-89ab-de495d6c8f3b",
 CellID->504770703],
Cell[702199, 19357, 236, 6, 34, "Output",ExpressionUUID->"c4ef37ad-eb66-455d-8aef-81cf90a871f2",
 CellID->1120799671],
Cell[702438, 19365, 349, 7, 24, "Print",ExpressionUUID->"57226440-ad11-42f7-a21b-d71b559682e8",
 CellID->1701002660],
Cell[702790, 19374, 238, 6, 34, "Output",ExpressionUUID->"56d2e21c-61eb-4956-bdeb-f29f3bc306c9",
 CellID->1449397756],
Cell[703031, 19382, 236, 6, 34, "Output",ExpressionUUID->"27be9219-9773-4b76-a86d-0a0cdb0530a6",
 CellID->1121077171],
Cell[703270, 19390, 1553, 43, 272, "Output",ExpressionUUID->"ad74f0e7-2e15-4e82-b893-643b57f47e15",
 CellID->2069977866]
}, Open  ]],
Cell[CellGroupData[{
Cell[704860, 19438, 462, 11, 61, "Input",ExpressionUUID->"58a03225-4843-445a-a9b2-f5246e135aea",
 CellID->63672667],
Cell[705325, 19451, 10947, 267, 283, "Output",ExpressionUUID->"8f0a613f-323e-4c0b-b58b-3d7e1dd03bb0",
 CellID->882912462],
Cell[716275, 19720, 12914, 308, 189, "Output",ExpressionUUID->"c7e59ddb-8aa8-4a3a-95a8-74ef7033ef17",
 CellID->2145231711]
}, Open  ]],
Cell[CellGroupData[{
Cell[729226, 20033, 1137, 31, 190, "Input",ExpressionUUID->"c36cbf0e-7130-42d2-b7b5-f81a66a99bfd",
 CellTags->"b:5.1.4",
 CellID->699658063],
Cell[730366, 20066, 944, 26, 49, "Output",ExpressionUUID->"f4cd31e4-5da3-4abb-baaa-4eb8e7636719",
 CellTags->"b:5.1.4",
 CellID->806417294],
Cell[731313, 20094, 272, 7, 34, "Output",ExpressionUUID->"628ca715-b15b-455a-9145-42096ba4f71e",
 CellTags->"b:5.1.4",
 CellID->686164041],
Cell[731588, 20103, 272, 7, 34, "Output",ExpressionUUID->"c564d13e-13ce-42e1-98cb-191780b12298",
 CellTags->"b:5.1.4",
 CellID->371852697],
Cell[731863, 20112, 724, 21, 49, "Output",ExpressionUUID->"5cd90fb3-b7bc-4478-b117-6247d5e5a079",
 CellTags->"b:5.1.4",
 CellID->550865709],
Cell[732590, 20135, 257, 7, 34, "Output",ExpressionUUID->"40bd79a2-cf69-4a76-86fd-9de6d8235c85",
 CellTags->"b:5.1.4",
 CellID->597968946],
Cell[732850, 20144, 273, 7, 34, "Output",ExpressionUUID->"e16e7d3d-c708-429b-b56f-9229ed7d035b",
 CellTags->"b:5.1.4",
 CellID->1988091109],
Cell[733126, 20153, 258, 7, 34, "Output",ExpressionUUID->"d0d3f249-6929-4eaa-b891-acdf731b57be",
 CellTags->"b:5.1.4",
 CellID->1149205290]
}, Open  ]],
Cell[CellGroupData[{
Cell[733421, 20165, 510, 14, 87, "Input",ExpressionUUID->"bbf31f51-3ff9-4af1-bda7-8d82c4ebd0c9",
 CellTags->"b:5.1.4",
 CellID->1680097516],
Cell[733934, 20181, 2335, 54, 83, "Output",ExpressionUUID->"7377ff9c-faf8-45db-b8c0-d79c92914f80",
 CellTags->"b:5.1.4",
 CellID->225387912],
Cell[736272, 20237, 273, 7, 34, "Output",ExpressionUUID->"3027bfb5-3b5d-45d4-b0db-22c0e078fc5c",
 CellTags->"b:5.1.4",
 CellID->2080967323],
Cell[736548, 20246, 272, 7, 34, "Output",ExpressionUUID->"42623be4-585c-4545-93f0-e39c9b9efff0",
 CellTags->"b:5.1.4",
 CellID->899122619]
}, Open  ]],
Cell[736835, 20256, 446, 9, 56, "Text",ExpressionUUID->"d73cbb6c-6597-4f9a-90e5-df1d14978916",
 CellTags->"b:5.1.4",
 CellID->136477009],
Cell[CellGroupData[{
Cell[737306, 20269, 731, 21, 89, "Input",ExpressionUUID->"440f67bd-3b21-4580-a6a6-f81976db1660",
 CellTags->"b:5.1.4",
 CellID->256555267],
Cell[738040, 20292, 941, 24, 49, "Output",ExpressionUUID->"7d95a954-460e-41ed-baf9-a0226e5ac0be",
 CellTags->"b:5.1.4",
 CellID->1858917241],
Cell[738984, 20318, 273, 7, 34, "Output",ExpressionUUID->"6b4a7cb9-a9f7-43aa-b01f-88a4c4215267",
 CellTags->"b:5.1.4",
 CellID->1229972931],
Cell[739260, 20327, 272, 7, 34, "Output",ExpressionUUID->"f82a5d3b-df80-4e19-b534-eaa4db650605",
 CellTags->"b:5.1.4",
 CellID->477465295]
}, Open  ]],
Cell[739547, 20337, 495, 10, 36, "Text",ExpressionUUID->"0042349a-67a5-4b61-a324-847d5b09cd5a",
 CellTags->"b:5.1.4",
 CellID->1924692940],
Cell[CellGroupData[{
Cell[740067, 20351, 854, 23, 113, "Input",ExpressionUUID->"314dd437-16d4-4f41-a6d5-3867ef618bb7",
 CellTags->"b:5.1.4",
 CellID->768067819],
Cell[740924, 20376, 258, 7, 34, "Output",ExpressionUUID->"eb8035e4-1e4a-4ade-aa21-5a3a374190f5",
 CellTags->"b:5.1.4",
 CellID->1760307873],
Cell[741185, 20385, 275, 7, 34, "Output",ExpressionUUID->"cbf5a05e-b01d-4e48-991f-1b6483db2a24",
 CellTags->"b:5.1.4",
 CellID->1759700439],
Cell[741463, 20394, 275, 7, 34, "Output",ExpressionUUID->"4f22ba43-33b0-476e-9c8b-25370c285aeb",
 CellTags->"b:5.1.4",
 CellID->140491789]
}, Open  ]],
Cell[741753, 20404, 426, 10, 34, "Text",ExpressionUUID->"23d544a2-af77-4240-b011-fd4d2ec93727",
 CellTags->"b:5.1.4",
 CellID->391631816],
Cell[CellGroupData[{
Cell[742204, 20418, 555, 13, 110, "Input",ExpressionUUID->"2ec33de3-e5b6-4e89-b603-d0be6efcce9e",
 CellTags->"b:5.1.4",
 CellID->118553457],
Cell[742762, 20433, 664, 15, 34, "Output",ExpressionUUID->"d5de0961-3667-47e0-b50f-f3c3745cb9c1",
 CellTags->"b:5.1.4",
 CellID->1160969670],
Cell[743429, 20450, 676, 17, 34, "Output",ExpressionUUID->"8acff93d-3429-4ba5-971e-61c926a6e099",
 CellTags->"b:5.1.4",
 CellID->380827048]
}, Open  ]],
Cell[CellGroupData[{
Cell[744142, 20472, 943, 23, 110, "Input",ExpressionUUID->"b659fad7-38d3-4c25-90ac-98f17bfebf85",
 CellID->1810611030],
Cell[745088, 20497, 1797, 35, 77, "Output",ExpressionUUID->"cbb1569a-dd57-4ec9-9bd7-7f047c1caec1",
 CellID->1058927676],
Cell[746888, 20534, 1841, 34, 77, "Output",ExpressionUUID->"5c4e5926-1aac-4020-a1aa-02f3993bbd3b",
 CellID->1200467843]
}, Open  ]],
Cell[CellGroupData[{
Cell[748766, 20573, 667, 18, 35, "Input",ExpressionUUID->"513526c1-3397-44f3-afa8-03c9023c9ba6",
 CellID->1096303220],
Cell[749436, 20593, 235, 6, 34, "Output",ExpressionUUID->"2ecbdf2a-af21-484d-9935-c11820b83c95",
 CellID->654173736]
}, Open  ]],
Cell[749686, 20602, 405, 10, 72, "Section",ExpressionUUID->"763cb688-6e74-4e57-a511-cc15e8535b7e",
 CellID->1415280785],
Cell[750094, 20614, 347, 8, 38, "Section",ExpressionUUID->"3cee1896-c370-4e38-8023-9520513d906b",
 CellID->1095279323],
Cell[750444, 20624, 2354, 50, 179, "Text",ExpressionUUID->"2dea4c61-501c-4112-adb3-35484335a7de",
 CellID->897207608],
Cell[CellGroupData[{
Cell[752823, 20678, 3784, 82, 260, "Input",ExpressionUUID->"72a325a0-337e-40d0-b28f-e989726cd2a4",
 CellID->675654531],
Cell[756610, 20762, 716, 21, 65, "Output",ExpressionUUID->"67a133be-b9d5-495b-af3c-ffe8e81c421f",
 CellID->1729191959],
Cell[757329, 20785, 1409, 41, 65, "Output",ExpressionUUID->"390b93ee-edf9-45e1-b3ad-145a1f2f9b41",
 CellID->633060062],
Cell[758741, 20828, 1408, 41, 65, "Output",ExpressionUUID->"5a937b39-3266-4981-98fc-148ce7a0bd80",
 CellID->1206474796],
Cell[760152, 20871, 648, 19, 49, "Output",ExpressionUUID->"859cefd0-b306-4015-a837-1dfcee8683db",
 CellID->681301398],
Cell[760803, 20892, 648, 19, 49, "Output",ExpressionUUID->"9684a667-975a-4a4c-bb65-48aad74c2b59",
 CellID->249790382]
}, Open  ]],
Cell[761466, 20914, 1347, 22, 108, "Text",ExpressionUUID->"985101a8-7c32-4fdc-9112-529d67b1e756",
 CellID->582757263],
Cell[CellGroupData[{
Cell[762838, 20940, 1732, 47, 165, "Input",ExpressionUUID->"cba99342-a767-476f-87dc-8e46b3772d5f",
 CellID->1024088374],
Cell[764573, 20989, 1375, 42, 65, "Output",ExpressionUUID->"3a081f3b-c2ad-45c3-bda8-6758acbcd75d",
 CellID->2043904036],
Cell[765951, 21033, 1374, 42, 65, "Output",ExpressionUUID->"4baa5ed8-77e0-47e9-a646-27b9c6c53ca3",
 CellID->746715640]
}, Open  ]],
Cell[767340, 21078, 2131, 43, 98, "Text",ExpressionUUID->"d2783001-50ca-46d7-8e26-ff9b455d91bb",
 CellID->1885079621],
Cell[769474, 21123, 1639, 27, 60, "Text",ExpressionUUID->"fcbcad92-bfae-40ef-b6bd-f1e62c52729d",
 CellID->1330108578],
Cell[CellGroupData[{
Cell[771138, 21154, 1322, 35, 61, "Input",ExpressionUUID->"19224dfb-e542-413e-82d3-fcf1bd964a40",
 CellID->583441831],
Cell[772463, 21191, 782, 24, 69, "Output",ExpressionUUID->"3211d361-cf25-48ee-88d0-4f561decb56d",
 CellID->155328412]
}, Open  ]],
Cell[CellGroupData[{
Cell[773282, 21220, 3358, 104, 96, "Input",ExpressionUUID->"7fa7607f-559e-409c-bd34-45a9812e3a85",
 CellID->1273091116],
Cell[776643, 21326, 591, 17, 34, "Output",ExpressionUUID->"5df4b52e-af2c-4ed4-bd10-f1196f0b10d1",
 CellID->1411989379],
Cell[777237, 21345, 592, 17, 34, "Output",ExpressionUUID->"5ac56d9f-8473-41eb-922b-f77f5497dcb5",
 CellID->1611193503],
Cell[777832, 21364, 680, 19, 34, "Output",ExpressionUUID->"4e26d33b-36f2-4270-b046-adb14c2b05d1",
 CellID->1707899898]
}, Open  ]],
Cell[778527, 21386, 2656, 47, 169, "Text",ExpressionUUID->"aba21136-242b-4724-b036-6686a8afdf58",
 CellID->1257041351],
Cell[CellGroupData[{
Cell[781208, 21437, 740, 22, 85, "Input",ExpressionUUID->"8823fb43-9bc2-4171-940d-cfa7804f178c",
 CellID->1736955736],
Cell[781951, 21461, 785, 24, 69, "Output",ExpressionUUID->"2b7161ea-949f-4cb7-bed4-07cf42fb0a1c",
 CellID->1407159884],
Cell[782739, 21487, 439, 13, 34, "Output",ExpressionUUID->"d5f2d2b9-b1bd-4b12-8e6b-390bdf33b926",
 CellID->2046249624]
}, Open  ]],
Cell[783193, 21503, 1876, 30, 60, "Text",ExpressionUUID->"868d063b-5624-4b57-aefe-0eec9d1664fe",
 CellID->437104054],
Cell[CellGroupData[{
Cell[785094, 21537, 1405, 28, 61, "Input",ExpressionUUID->"6e5fafac-6efb-4600-adbc-8a1185655e53",
 CellID->488748264],
Cell[786502, 21567, 31251, 584, 609, "Output",ExpressionUUID->"33defc2a-8adb-4235-993f-6365d00d312e",
 CellID->1514307510]
}, Open  ]],
Cell[817768, 22154, 1725, 28, 36, "Text",ExpressionUUID->"451c8a0e-04b0-4b06-97ff-552dc0a725e6",
 CellID->2014739072],
Cell[CellGroupData[{
Cell[819518, 22186, 859, 21, 86, "Input",ExpressionUUID->"efe8f920-845c-429f-be8d-3c4e3aab5ec9",
 CellID->1104787296],
Cell[820380, 22209, 38663, 705, 609, "Output",ExpressionUUID->"e876c999-d475-4445-aee5-7e01be9c0f76",
 CellID->1137619973]
}, Open  ]],
Cell[859058, 22917, 1807, 27, 43, "Text",ExpressionUUID->"77a7d123-074a-4420-8673-7ee271b3434e",
 CellID->1748979971],
Cell[CellGroupData[{
Cell[860890, 22948, 1307, 28, 87, "Input",ExpressionUUID->"4d0e5bb3-c316-4d7b-be16-5edeb148bdfb",
 CellID->1126512153],
Cell[862200, 22978, 85877, 1479, 609, "Output",ExpressionUUID->"9bfd5b18-428b-4f05-8812-486b5af138e9",
 CellID->931976970]
}, Open  ]],
Cell[948092, 24460, 1854, 28, 43, "Text",ExpressionUUID->"85eedebf-dc13-4950-bfb5-d7829f038189",
 CellID->805007800],
Cell[CellGroupData[{
Cell[949971, 24492, 512, 13, 73, "Input",ExpressionUUID->"fc315e2e-60c5-42eb-84df-6a8eca98444c",
 CellID->1028383359],
Cell[950486, 24507, 114501, 1949, 609, "Output",ExpressionUUID->"1eb7b1c9-ef4a-414f-be58-b16c6988e68c",
 CellID->2138697219]
}, Open  ]],
Cell[1065002, 26459, 1830, 28, 43, "Text",ExpressionUUID->"42db371b-9291-4877-8ba7-0d7892c3676d",
 CellID->1791024081],
Cell[1066835, 26489, 1798, 27, 43, "Text",ExpressionUUID->"13a71bd0-39b1-4e90-91b6-1af41b3f51a4",
 CellID->358333546],
Cell[CellGroupData[{
Cell[1068658, 26520, 717, 20, 87, "Input",ExpressionUUID->"9fe08296-0e67-401e-b3ef-bfe41b15c356",
 CellID->190738675],
Cell[1069378, 26542, 102518, 1752, 609, "Output",ExpressionUUID->"4e528103-629a-4cbb-a669-55897b39c48d",
 CellID->502876247]
}, Open  ]],
Cell[1171911, 28297, 1987, 30, 74, "Text",ExpressionUUID->"f553844c-2ad1-43f5-a262-3a33e45e3e20",
 CellID->558168206],
Cell[1173901, 28329, 1920, 29, 74, "Text",ExpressionUUID->"b27eac72-7104-414d-88e7-db7267c77709",
 CellID->831021091],
Cell[1175824, 28360, 3234, 67, 198, "Text",ExpressionUUID->"2a53a9ac-81e4-4a5c-823d-0059c9bf74fc",
 CellID->837984814],
Cell[1179061, 28429, 4646, 93, 413, "Text",ExpressionUUID->"43958927-e025-4603-8f2f-e995a2cfa831",
 CellID->1388692359],
Cell[CellGroupData[{
Cell[1183732, 28526, 762, 20, 87, "Input",ExpressionUUID->"c1147c5c-9cc5-4384-81e8-2a37b89a2d90",
 CellID->1537566662],
Cell[1184497, 28548, 85434, 1471, 227, "Output",ExpressionUUID->"ffb3e82a-dee0-47b0-8337-8e54dcb54054",
 CellID->1492932384]
}, Open  ]],
Cell[1269946, 30022, 421, 10, 72, "Section",ExpressionUUID->"94035194-a202-48d5-afa6-b9ab9f372e60",
 CellID->129759577],
Cell[1270370, 30034, 347, 8, 38, "Section",ExpressionUUID->"d9be0725-e062-4d88-8b75-4faef815e1e6",
 CellID->1173308934],
Cell[1270720, 30044, 302, 8, 38, "Section",ExpressionUUID->"8adb5c2f-de23-4e66-bbe2-ae15e9058ee0",
 CellID->93467406],
Cell[1271025, 30054, 1281, 36, 76, "Text",ExpressionUUID->"400315b1-b177-4c62-b26e-0edd8c82338e",
 CellTags->"b:5.1.4",
 CellID->158527280],
Cell[CellGroupData[{
Cell[1272331, 30094, 408, 12, 35, "Input",ExpressionUUID->"914f3bf7-e6ee-42a7-aa69-a7a0c6f7e655",
 CellTags->"b:5.1.4",
 CellID->1488272680],
Cell[1272742, 30108, 1822, 55, 75, "Output",ExpressionUUID->"fbe5180e-e382-4ea6-af96-9a78640748ba",
 CellTags->"b:5.1.4",
 CellID->515646340]
}, Open  ]],
Cell[CellGroupData[{
Cell[1274601, 30168, 532, 16, 35, "Input",ExpressionUUID->"ec24f419-a90d-4ae0-b121-b916f7acde80",
 CellTags->"b:5.1.4",
 CellID->1033159208],
Cell[1275136, 30186, 869, 25, 51, "Output",ExpressionUUID->"8c5162fb-76c0-4e44-8180-d6eca29b34a7",
 CellTags->"b:5.1.4",
 CellID->1371520735]
}, Open  ]],
Cell[CellGroupData[{
Cell[1276042, 30216, 401, 12, 35, "Input",ExpressionUUID->"7de82e02-6ec9-4a49-b194-0002be469499",
 CellTags->"b:5.1.4",
 CellID->1901522861],
Cell[1276446, 30230, 902, 26, 71, "Output",ExpressionUUID->"26c8269a-1b14-4a5a-88bb-416399b581d9",
 CellTags->"b:5.1.4",
 CellID->59526247]
}, Open  ]],
Cell[CellGroupData[{
Cell[1277385, 30261, 328, 10, 34, "Input",ExpressionUUID->"4f1582d5-efa6-47c6-8f17-c3294a8b58b3",
 CellTags->"b:5.1.4",
 CellID->43090173],
Cell[1277716, 30273, 671, 20, 49, "Output",ExpressionUUID->"99005412-e9e2-4eda-9b44-136798e44979",
 CellTags->"b:5.1.4",
 CellID->1877737474]
}, Open  ]],
Cell[1278402, 30296, 572, 19, 36, "Text",ExpressionUUID->"7030108b-77b6-43cd-b44b-e7f73753c319",
 CellTags->"b:5.1.4",
 CellID->514548572],
Cell[CellGroupData[{
Cell[1278999, 30319, 547, 17, 35, "Input",ExpressionUUID->"fbbaabab-118d-46cf-88fb-b084a3f16ffe",
 CellTags->"b:5.1.4",
 CellID->101203135],
Cell[1279549, 30338, 1299, 35, 55, "Output",ExpressionUUID->"4a3c0989-6519-4668-9746-2c2aca44931a",
 CellTags->"b:5.1.4",
 CellID->1861532853]
}, Open  ]],
Cell[1280863, 30376, 470, 10, 34, "Text",ExpressionUUID->"cc17e065-37e8-4de6-82ed-c3e71bd47061",
 CellTags->"b:5.1.4",
 CellID->114041521],
Cell[1281336, 30388, 477, 11, 72, "Section",ExpressionUUID->"86a3909e-d805-4a69-a62e-d5f45d4f7317",
 CellID->1073997505],
Cell[1281816, 30401, 347, 8, 38, "Section",ExpressionUUID->"dbd7505c-250b-4f3b-8f5d-f9905c5c8089",
 CellID->1076013500],
Cell[1282166, 30411, 538, 12, 52, "Text",ExpressionUUID->"2386e5b5-d9f0-41cd-b978-b135e70278a2",
 CellID->920889186],
Cell[1282707, 30425, 848, 27, 37, "Text",ExpressionUUID->"652ba143-01be-4936-be81-ec43574aaa12",
 CellID->469905173],
Cell[1283558, 30454, 1575, 43, 84, "Text",ExpressionUUID->"31c1a603-724d-4381-969e-e6e0d9c3c9c6",
 CellID->866060983],
Cell[CellGroupData[{
Cell[1285158, 30501, 451, 11, 67, "Subsection",ExpressionUUID->"96078100-68c0-4ba8-82c8-170ae445d16b",
 CellTags->{"c:10", "b:5.2.0"},
 CellID->1398872592],
Cell[1285612, 30514, 1891, 55, 66, "Text",ExpressionUUID->"423adf7a-b4c9-477f-96c2-6e5353d0f829",
 CellTags->"b:5.2.0",
 CellID->200686388],
Cell[CellGroupData[{
Cell[1287528, 30573, 325, 10, 34, "Input",ExpressionUUID->"ab430eea-e3bf-4008-8e86-2e56c6f806f8",
 CellTags->"b:5.2.0",
 CellID->1386797326],
Cell[1287856, 30585, 835, 28, 69, "Output",ExpressionUUID->"36b27891-1c9e-49d7-a45b-dc6f1732391f",
 CellTags->"b:5.2.0",
 CellID->93407228]
}, Open  ]],
Cell[1288706, 30616, 269, 7, 34, "Text",ExpressionUUID->"32d9dae5-a881-446c-bc7b-aed2b411e7fd",
 CellTags->"b:5.2.0",
 CellID->1262637191],
Cell[CellGroupData[{
Cell[1289000, 30627, 1067, 29, 86, "Input",ExpressionUUID->"85fdbe19-9955-432e-9b49-5a26c90133f4",
 CellTags->"b:5.2.0",
 CellID->1857353130],
Cell[1290070, 30658, 835, 28, 69, "Output",ExpressionUUID->"384b85fc-54a7-4fc9-953d-4a07427de1ff",
 CellTags->"b:5.2.0",
 CellID->84735039],
Cell[1290908, 30688, 257, 7, 34, "Output",ExpressionUUID->"e15ba8f9-3220-461d-b735-f370081330f3",
 CellTags->"b:5.2.0",
 CellID->360142512],
Cell[1291168, 30697, 645, 19, 49, "Output",ExpressionUUID->"209d2aaf-0c6e-4dc2-9978-cfde9e8ddfe1",
 CellTags->"b:5.2.0",
 CellID->1306281938]
}, Open  ]],
Cell[1291828, 30719, 307, 8, 34, "Text",ExpressionUUID->"ec39b56f-56c0-464c-9557-3b22e44f200b",
 CellTags->"b:5.2.0",
 CellID->711763118],
Cell[CellGroupData[{
Cell[1292160, 30731, 853, 18, 34, "Input",ExpressionUUID->"e781abb5-21c3-4c79-b840-288f0f9ea0e6",
 CellTags->"b:5.2.0",
 CellID->1118810436],
Cell[1293016, 30751, 1754, 61, 231, "Output",ExpressionUUID->"4763f15c-1741-4cd7-b27a-1dc6e4727dbb",
 CellTags->"b:5.2.0",
 CellID->955208685]
}, Open  ]],
Cell[1294785, 30815, 272, 7, 34, "Text",ExpressionUUID->"bdea370b-d82f-4b45-a5f6-91699749c01f",
 CellTags->"b:5.2.0",
 CellID->480184720],
Cell[CellGroupData[{
Cell[1295082, 30826, 880, 20, 34, "Input",ExpressionUUID->"a7d7ff51-a767-453b-b856-fed2718efbb6",
 CellTags->"b:5.2.0",
 CellID->221043614],
Cell[1295965, 30848, 256, 7, 34, "Output",ExpressionUUID->"22fa7134-b3f8-4d27-aab2-a1444e81ca96",
 CellTags->"b:5.2.0",
 CellID->1752107470]
}, Open  ]],
Cell[1296236, 30858, 602, 11, 78, "Text",ExpressionUUID->"2a6d3308-807b-411c-a2ac-5b192fe52903",
 CellTags->"b:5.2.0",
 CellID->2080794835],
Cell[CellGroupData[{
Cell[1296863, 30873, 1155, 30, 113, "Input",ExpressionUUID->"970e4ff0-f2b9-4508-8764-82077d64850f",
 CellTags->"b:5.2.0",
 CellID->147600450],
Cell[1298021, 30905, 989, 25, 151, "Output",ExpressionUUID->"7e21fb3e-27f2-48df-9f84-c15e48301af7",
 CellTags->"b:5.2.0",
 CellID->1374480626],
Cell[1299013, 30932, 1086, 29, 167, "Output",ExpressionUUID->"1f590e1e-0b85-4d51-ab05-ab4683f7fa1e",
 CellTags->"b:5.2.0",
 CellID->414290403],
Cell[1300102, 30963, 744, 23, 65, "Output",ExpressionUUID->"ca800559-5be5-4647-92fb-95f71f514d3d",
 CellTags->"b:5.2.0",
 CellID->836819647],
Cell[1300849, 30988, 647, 19, 49, "Output",ExpressionUUID->"813d7053-d557-496e-b9ff-1da22402e551",
 CellTags->"b:5.2.0",
 CellID->557402926]
}, Open  ]],
Cell[CellGroupData[{
Cell[1301533, 31012, 395, 10, 60, "Input",ExpressionUUID->"e80eb3c5-804f-402f-a192-e34de8b00ef9",
 CellID->1761374433],
Cell[1301931, 31024, 1081, 38, 123, "Output",ExpressionUUID->"3af14d91-d854-4fb2-ad4f-d5a9f60f9b4d",
 CellID->947309068],
Cell[1303015, 31064, 6212, 244, 231, "Output",ExpressionUUID->"a25f421d-2e79-4ed0-b05d-f3a6477880f0",
 CellID->2127230665]
}, Open  ]],
Cell[1309242, 31311, 586, 20, 34, "Text",ExpressionUUID->"0e6c5841-2222-4e05-9291-f422ae0119a8",
 CellTags->"b:5.2.0",
 CellID->377150865],
Cell[CellGroupData[{
Cell[1309853, 31335, 386, 11, 35, "Input",ExpressionUUID->"9775c230-9ffb-415f-8280-7fe1bceda115",
 CellTags->{"i:2", "b:5.2.0"},
 CellID->1550345758],
Cell[1310242, 31348, 2597, 89, 215, "Output",ExpressionUUID->"e4d35865-ef5d-45dc-82e4-fb4987273476",
 CellTags->{"i:2", "b:5.2.0"},
 CellID->1058031399]
}, Open  ]],
Cell[1312854, 31440, 365, 9, 34, "Text",ExpressionUUID->"6ca62813-b7ce-40d0-a9af-d31b495f3e96",
 CellTags->"b:5.2.0",
 CellID->1242065655],
Cell[CellGroupData[{
Cell[1313244, 31453, 921, 26, 160, "Input",ExpressionUUID->"34c1b253-fa12-430c-9c82-a55aaf429426",
 CellTags->"b:5.2.0",
 CellID->1856414745],
Cell[1314168, 31481, 491, 14, 54, "Output",ExpressionUUID->"c36b3f1f-effc-4705-b6bf-f8f84f730d73",
 CellTags->"b:5.2.0",
 CellID->1169125833],
Cell[1314662, 31497, 348, 10, 34, "Output",ExpressionUUID->"b0f93177-bf9c-4ec4-b389-75eee9a662c7",
 CellTags->"b:5.2.0",
 CellID->22505181],
Cell[1315013, 31509, 514, 15, 54, "Output",ExpressionUUID->"126e8411-ccfc-45fd-8723-0a69fffd0f90",
 CellTags->"b:5.2.0",
 CellID->287455195]
}, Open  ]],
Cell[1315542, 31527, 441, 9, 34, "Text",ExpressionUUID->"737e77b0-1523-4e11-9f75-a3c030205c57",
 CellTags->"b:5.2.0",
 CellID->2120195599],
Cell[CellGroupData[{
Cell[1316008, 31540, 681, 18, 61, "Input",ExpressionUUID->"23cebcfc-b49e-46cc-97d0-78b06dfd9b62",
 CellTags->"b:5.2.0",
 CellID->931277722],
Cell[1316692, 31560, 938, 28, 71, "Output",ExpressionUUID->"458e0be8-43f7-4a36-b28f-34bfb7e5c9cd",
 CellTags->"b:5.2.0",
 CellID->1568740262],
Cell[1317633, 31590, 1088, 33, 69, "Output",ExpressionUUID->"98eca4f6-7e69-4ddf-b095-331923f10502",
 CellTags->"b:5.2.0",
 CellID->1160849215]
}, Open  ]],
Cell[1318736, 31626, 1306, 37, 66, "Text",ExpressionUUID->"9af40ab8-176b-4182-b3b5-71a266d5e7a0",
 CellTags->"b:5.2.0",
 CellID->1123854318],
Cell[CellGroupData[{
Cell[1320067, 31667, 426, 11, 61, "Input",ExpressionUUID->"a71de71b-d04b-4e4a-b136-f84d059ac3c1",
 CellTags->"b:5.2.0",
 CellID->1500847261],
Cell[1320496, 31680, 769, 23, 71, "Output",ExpressionUUID->"e93e6498-eaf8-4e9d-9d64-19319e6a29e1",
 CellTags->"b:5.2.0",
 CellID->503823510],
Cell[1321268, 31705, 547, 15, 48, "Output",ExpressionUUID->"66c5a31a-6bba-4278-b068-0f875ec221f2",
 CellTags->"b:5.2.0",
 CellID->233712449]
}, Open  ]],
Cell[1321830, 31723, 399, 13, 37, "Text",ExpressionUUID->"9ff36dab-3e11-41db-b1fb-16801bdd4f56",
 CellTags->"b:5.2.0",
 CellID->396997965],
Cell[CellGroupData[{
Cell[1322254, 31740, 774, 19, 135, "Input",ExpressionUUID->"85100578-7d82-42bd-8ae0-e6626e613121",
 CellTags->"b:5.2.0",
 CellID->113106526],
Cell[1323031, 31761, 937, 28, 71, "Output",ExpressionUUID->"e59b3a1f-9485-4ecf-9cd5-8991691694df",
 CellTags->"b:5.2.0",
 CellID->257539980],
Cell[1323971, 31791, 770, 23, 71, "Output",ExpressionUUID->"438b993d-03f4-4581-8228-95e2afb84c42",
 CellTags->"b:5.2.0",
 CellID->1294895155],
Cell[1324744, 31816, 546, 15, 48, "Output",ExpressionUUID->"ce55b428-b606-4a92-8961-d22c8e848b6e",
 CellTags->"b:5.2.0",
 CellID->703591387],
Cell[1325293, 31833, 720, 23, 69, "Output",ExpressionUUID->"719273fd-c55e-45db-bbe4-b43f3ba3c826",
 CellTags->"b:5.2.0",
 CellID->1215480927],
Cell[1326016, 31858, 491, 14, 54, "Output",ExpressionUUID->"d2b6c0d6-19ed-4f71-9993-81371b97df76",
 CellTags->"b:5.2.0",
 CellID->1723489825]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1326556, 31878, 452, 11, 67, "Subsection",ExpressionUUID->"77c1fd1c-ffe7-47d3-9784-83feb0f9ec2d",
 CellTags->{"i:1", "c:11", "b:5.2.1"},
 CellID->458354887],
Cell[1327011, 31891, 904, 22, 80, "Text",ExpressionUUID->"43041a90-1cb0-4a74-87e8-e4e8602d5436",
 CellTags->"b:5.2.1",
 CellID->546744020],
Cell[CellGroupData[{
Cell[1327940, 31917, 299, 9, 34, "Input",ExpressionUUID->"bf4a536c-cffd-4294-8939-45a4120a5a0c",
 CellTags->"b:5.2.1",
 CellID->44218627],
Cell[1328242, 31928, 719, 21, 83, "Output",ExpressionUUID->"21b94af7-8f88-4e48-ab1a-a7cba2d3d955",
 CellTags->"b:5.2.1",
 CellID->1434058039]
}, Open  ]],
Cell[1328976, 31952, 472, 15, 34, "Text",ExpressionUUID->"54c8dd22-ebd3-4b4a-823e-431c32cb6d2a",
 CellTags->"b:5.2.1",
 CellID->153122127],
Cell[CellGroupData[{
Cell[1329473, 31971, 751, 20, 35, "Input",ExpressionUUID->"5eb244d2-f3e0-459b-9565-a5e824f42ff8",
 CellTags->"b:5.2.1",
 CellID->838179383],
Cell[1330227, 31993, 719, 21, 83, "Output",ExpressionUUID->"735da3b1-d631-4576-be3f-5a37c5a299e1",
 CellTags->"b:5.2.1",
 CellID->1004491192]
}, Open  ]],
Cell[1330961, 32017, 360, 8, 34, "Text",ExpressionUUID->"1d2a3481-0ad2-4caa-a32c-d983326035bf",
 CellTags->"b:5.2.1",
 CellID->1935649408],
Cell[CellGroupData[{
Cell[1331346, 32029, 544, 14, 34, "Input",ExpressionUUID->"f5550a7b-12fc-44a5-b276-721084790a8d",
 CellTags->"b:5.2.1",
 CellID->229831853],
Cell[1331893, 32045, 719, 21, 83, "Output",ExpressionUUID->"da946fad-026b-4736-b336-22fcaa8a4717",
 CellTags->"b:5.2.1",
 CellID->1114514877]
}, Open  ]],
Cell[1332627, 32069, 464, 14, 34, "Text",ExpressionUUID->"13d5adce-f13f-4538-a04c-4180b12fc59e",
 CellTags->"b:5.2.1",
 CellID->1608356065],
Cell[1333094, 32085, 320, 10, 34, "Input",ExpressionUUID->"8a662743-5658-4b86-ab58-90cfab17ff8e",
 CellTags->"b:5.2.1",
 CellID->1263719666]
}, Open  ]],
Cell[CellGroupData[{
Cell[1333451, 32100, 371, 10, 67, "Subsection",ExpressionUUID->"f14f877b-8ff0-4d4e-b194-a27251469e91",
 CellTags->{"i:1", "c:12", "b:5.2.2"},
 CellID->723157632],
Cell[1333825, 32112, 482, 16, 36, "Text",ExpressionUUID->"ecb0a9ef-6c94-48e3-a22b-5f78348fb1cc",
 CellTags->"b:5.2.2",
 CellID->1500362651],
Cell[1334310, 32130, 346, 9, 34, "Text",ExpressionUUID->"bf737c52-08be-41be-9416-b8e1837e81e7",
 CellTags->"b:5.2.2",
 CellID->842092283],
Cell[1334659, 32141, 472, 10, 56, "Text",ExpressionUUID->"94636e20-b904-4fae-a254-03f72278173b",
 CellTags->"b:5.2.2",
 CellID->1127766651],
Cell[CellGroupData[{
Cell[1335156, 32155, 397, 11, 60, "Input",ExpressionUUID->"fdb8f1d6-a0c2-438b-a51e-2b7abf8b10ac",
 CellTags->"b:5.2.2",
 CellID->683434935],
Cell[1335556, 32168, 741, 22, 83, "Output",ExpressionUUID->"aeaa360d-6027-4483-bbae-cb368aa39e48",
 CellTags->"b:5.2.2",
 CellID->1299168605],
Cell[1336300, 32192, 743, 22, 83, "Output",ExpressionUUID->"a59f1c24-5588-4dac-ac18-4722fee6c1b6",
 CellTags->"b:5.2.2",
 CellID->1660505186]
}, Open  ]],
Cell[1337058, 32217, 515, 15, 34, "Text",ExpressionUUID->"49271300-b953-46df-8d14-7417c58c2c95",
 CellTags->"b:5.2.2",
 CellID->1748408389],
Cell[CellGroupData[{
Cell[1337598, 32236, 303, 9, 34, "Input",ExpressionUUID->"750862b4-8b1d-4af8-ae3f-5b521996bf51",
 CellTags->"b:5.2.2",
 CellID->1481052022],
Cell[1337904, 32247, 1033, 27, 151, "Output",ExpressionUUID->"6e22b8ed-b7be-42b1-b4c2-b65d66d6127c",
 CellTags->"b:5.2.2",
 CellID->1560668337]
}, Open  ]],
Cell[1338952, 32277, 403, 9, 56, "Text",ExpressionUUID->"6a744824-9850-4aef-a43d-d3fed472eab3",
 CellTags->"b:5.2.2",
 CellID->558381377],
Cell[1339358, 32288, 308, 8, 34, "Text",ExpressionUUID->"07d12111-05a7-4ced-8283-a63e3c8c0a4c",
 CellTags->"b:5.2.2",
 CellID->212265849],
Cell[1339669, 32298, 592, 13, 56, "Text",ExpressionUUID->"dd355a86-94c0-4e60-ad12-4911bb130386",
 CellTags->"b:5.2.2",
 CellID->581786476],
Cell[1340264, 32313, 336, 8, 34, "Text",ExpressionUUID->"47a4413c-726c-410a-bfe6-49502bcbd5db",
 CellTags->"b:5.2.2",
 CellID->592257341],
Cell[CellGroupData[{
Cell[1340625, 32325, 542, 14, 60, "Input",ExpressionUUID->"87910459-7312-4023-9b21-1fa5f5bda4c2",
 CellTags->"b:5.2.2",
 CellID->482426342],
Cell[1341170, 32341, 735, 24, 85, "Output",ExpressionUUID->"07db9489-d66f-44da-bfc1-3139f658d400",
 CellTags->"b:5.2.2",
 CellID->787475501]
}, Open  ]],
Cell[1341920, 32368, 502, 11, 56, "Text",ExpressionUUID->"0efad72b-15e4-4a54-af92-59c418c7d0a7",
 CellTags->"b:5.2.2",
 CellID->1388762569],
Cell[CellGroupData[{
Cell[1342447, 32383, 507, 15, 61, "Input",ExpressionUUID->"ef37a271-9694-4c04-888b-2ca6dea493d2",
 CellTags->"b:5.2.2",
 CellID->2080221587],
Cell[1342957, 32400, 810, 29, 155, "Output",ExpressionUUID->"4c7367cd-9071-4539-8e8d-2967ae26a9b5",
 CellTags->"b:5.2.2",
 CellID->159028846]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1343816, 32435, 370, 10, 67, "Subsection",ExpressionUUID->"66fa64c3-4d88-45c6-bfae-e72b46411a05",
 CellTags->{"i:1", "c:13", "b:5.2.3"},
 CellID->1250439549],
Cell[1344189, 32447, 476, 10, 56, "Text",ExpressionUUID->"e849669a-46c9-4ccf-94cb-45943aeda80e",
 CellTags->"b:5.2.3",
 CellID->862530501],
Cell[1344668, 32459, 1581, 49, 119, "Text",ExpressionUUID->"58ec87eb-ad71-4cab-90bc-edac90886601",
 CellTags->"b:5.2.3",
 CellID->1035513057],
Cell[CellGroupData[{
Cell[1346274, 32512, 321, 9, 35, "Input",ExpressionUUID->"f15862e1-8c3b-487d-a7f3-12aaf8ae5449",
 CellTags->"b:5.2.3",
 CellID->660473711],
Cell[1346598, 32523, 987, 25, 151, "Output",ExpressionUUID->"f74de61e-9abe-4a70-8a4e-41737350f955",
 CellTags->"b:5.2.3",
 CellID->1328157854]
}, Open  ]],
Cell[1347600, 32551, 1200, 35, 81, "Text",ExpressionUUID->"244d9e5a-8a4d-4ff4-966d-09819b4774a7",
 CellTags->"b:5.2.3",
 CellID->238689434]
}, Open  ]],
Cell[1348815, 32589, 583, 12, 72, "Section",ExpressionUUID->"edf09c97-aeae-4cfe-b9a0-31bfa6089694",
 CellID->1928629920],
Cell[1349401, 32603, 346, 8, 38, "Section",ExpressionUUID->"6cd292e0-fe28-4e0f-8250-8d3c7d3ecbf2",
 CellID->364897622],
Cell[1349750, 32613, 1378, 23, 140, "Text",ExpressionUUID->"bd35af58-9d9a-4d23-a15c-80c29ba3945a",
 CellTags->"b:5.2.3",
 CellID->1277398455],
Cell[1351131, 32638, 1567, 43, 94, "Input",ExpressionUUID->"9f67f090-9d75-47d1-9641-442ca08b02dd",
 CellID->907264004],
Cell[CellGroupData[{
Cell[1352723, 32685, 995, 21, 94, "Input",ExpressionUUID->"82bc2856-328c-4c4c-8119-447e30ff13e5",
 CellID->1964003762],
Cell[1353721, 32708, 2217, 62, 237, "Output",ExpressionUUID->"8f18a51a-ed2f-4f81-a6f5-83748bd89551",
 CellID->1404025260],
Cell[1355941, 32772, 1122, 34, 31, "Print",ExpressionUUID->"c30a653f-cb8f-4ef2-ac5d-566d0cd96c5f",
 CellID->902231584],
Cell[1357066, 32808, 1122, 34, 31, "Print",ExpressionUUID->"721fe36d-4f0c-4bca-bcdb-4c36b65bd47f",
 CellID->424200821],
Cell[1358191, 32844, 1125, 34, 31, "Print",ExpressionUUID->"46201f85-7647-43ff-b8d2-b96ad69fb173",
 CellID->2044974154],
Cell[1359319, 32880, 1081, 38, 123, "Output",ExpressionUUID->"b4788a50-7dcb-478c-915b-8681c802d5fa",
 CellID->1552319690],
Cell[1360403, 32920, 234, 6, 34, "Output",ExpressionUUID->"f3dabc06-d625-45b4-9264-8cf2f9b58f9e",
 CellID->58202427]
}, Open  ]],
Cell[1360652, 32929, 949, 22, 94, "Input",ExpressionUUID->"90cc0339-b80d-4271-909f-094afb388905",
 CellID->1726970606],
Cell[CellGroupData[{
Cell[1361626, 32955, 961, 21, 94, "Input",ExpressionUUID->"6948eaa6-dc37-4c3d-8e99-4f82c1bdeda8",
 CellID->79360055],
Cell[1362590, 32978, 3238, 87, 336, "Output",ExpressionUUID->"875b2402-d134-435f-9875-a6a0168f75fe",
 CellID->1803152923],
Cell[1365831, 33067, 1122, 34, 31, "Print",ExpressionUUID->"2cf9d4aa-47fc-47d5-9a80-7a2d9ec90096",
 CellID->301950397],
Cell[1366956, 33103, 1122, 34, 31, "Print",ExpressionUUID->"ec77b679-095c-45cf-800a-dad04fe42700",
 CellID->376464518],
Cell[1368081, 33139, 1122, 34, 31, "Print",ExpressionUUID->"b8e33a9d-bfa6-494e-bdb6-f14baf7342e1",
 CellID->752329964],
Cell[1369206, 33175, 985, 24, 151, "Output",ExpressionUUID->"b0d57a29-2881-45cc-9a26-2473b5e3ea9d",
 CellID->25991412],
Cell[1370194, 33201, 257, 6, 82, "Output",ExpressionUUID->"8eb7d353-b029-4425-a15b-f255aa597360",
 CellID->898481143]
}, Open  ]],
Cell[1370466, 33210, 914, 16, 34, "Text",ExpressionUUID->"2697c18c-ff70-40dc-b3a2-0ff25d39b02a",
 CellTags->"b:5.2.3",
 CellID->1803534785],
Cell[1371383, 33228, 525, 12, 72, "Section",ExpressionUUID->"34995d9a-e47a-4781-95f1-09334f1e7618",
 CellID->805520140],
Cell[1371911, 33242, 346, 8, 38, "Section",ExpressionUUID->"14f0b4a7-f3eb-4e80-95a6-85c3a89a765a",
 CellID->640082292],
Cell[CellGroupData[{
Cell[1372282, 33254, 411, 11, 51, "Subsection",ExpressionUUID->"bfef1c7b-67ac-46fa-9f73-a960decca00a",
 CellTags->{"Measurements", "c:15", "b:5.3.0"},
 CellID->432727106],
Cell[1372696, 33267, 434, 10, 56, "Text",ExpressionUUID->"7fd703ca-332a-4150-a237-cb8d245733c6",
 CellTags->"b:5.3.0",
 CellID->1464841800],
Cell[1373133, 33279, 487, 13, 59, "Text",ExpressionUUID->"2087b84c-12f6-436b-a18e-601bb67f5daf",
 CellTags->"b:5.3.0",
 CellID->843603392],
Cell[CellGroupData[{
Cell[1373645, 33296, 1349, 34, 61, "Input",ExpressionUUID->"df3d8886-d538-4602-8838-bcfa86ff7263",
 CellTags->"b:5.3.0",
 CellID->755171850],
Cell[1374997, 33332, 3659, 105, 351, "Output",ExpressionUUID->"07d4dc40-be9d-4f8b-b050-b934bbb048a8",
 CellTags->"b:5.3.0",
 CellID->1805962118]
}, Open  ]],
Cell[1378671, 33440, 348, 8, 34, "Text",ExpressionUUID->"75a818a4-b3a1-46a0-b34f-3822be32edc5",
 CellTags->"b:5.3.0",
 CellID->544929180],
Cell[1379022, 33450, 777, 19, 36, "Input",ExpressionUUID->"7567bfe0-3030-4320-9b8c-97fb10fa41a9",
 CellTags->"b:5.3.0",
 CellID->1754457452],
Cell[1379802, 33471, 369, 8, 56, "Text",ExpressionUUID->"cbc42fde-6cd6-4047-a25f-d2fdacdb9ec1",
 CellTags->"b:5.3.0",
 CellID->68438807],
Cell[1380174, 33481, 720, 24, 36, "Input",ExpressionUUID->"4e717616-586f-4547-8026-609143538737",
 CellTags->"b:5.3.0",
 CellID->1347278104],
Cell[1380897, 33507, 369, 8, 34, "Text",ExpressionUUID->"211c0012-3e14-46fa-bba3-2dfdc1717141",
 CellTags->"b:5.3.0",
 CellID->635519960],
Cell[CellGroupData[{
Cell[1381291, 33519, 412, 12, 61, "Input",ExpressionUUID->"0acf171e-9753-4e36-8db2-89181325c802",
 CellTags->"b:5.3.0",
 CellID->1747791189],
Cell[1381706, 33533, 820, 25, 99, "Output",ExpressionUUID->"e3f9ad02-a399-4fa9-b1a4-22dd98b09a81",
 CellTags->"b:5.3.0",
 CellID->271450528],
Cell[1382529, 33560, 253, 7, 34, "Output",ExpressionUUID->"08113570-68aa-41f7-8c44-77bd9b40b8ee",
 CellTags->"b:5.3.0",
 CellID->1013757868]
}, Open  ]],
Cell[1382797, 33570, 325, 8, 34, "Text",ExpressionUUID->"7c90bc29-7053-48a1-8f31-37bd6cef62f1",
 CellTags->"b:5.3.0",
 CellID->1845189857],
Cell[CellGroupData[{
Cell[1383147, 33582, 559, 15, 87, "Input",ExpressionUUID->"9d7f315d-d755-43ce-a9d8-59bbab8942e8",
 CellTags->"b:5.3.0",
 CellID->294580272],
Cell[1383709, 33599, 4189, 137, 119, "Output",ExpressionUUID->"0434d8fd-bbdc-4194-b14f-afe2eff599a1",
 CellTags->"b:5.3.0",
 CellID->93232178],
Cell[1387901, 33738, 614, 21, 49, "Output",ExpressionUUID->"4b6f6c52-9904-4e4c-92e5-f74b513dad81",
 CellTags->"b:5.3.0",
 CellID->1437636983]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1388564, 33765, 423, 11, 67, "Subsection",ExpressionUUID->"3dd7fa66-0334-4084-bce6-902e9f0997d0",
 CellTags->{"Measurements", "MeasureKet", "c:16", "b:5.3.1"},
 CellID->1060825169],
Cell[1388990, 33778, 310, 8, 34, "Text",ExpressionUUID->"4ac0b746-8587-448b-8344-9cc05179fceb",
 CellTags->"b:5.3.1",
 CellID->879377989],
Cell[CellGroupData[{
Cell[1389325, 33790, 1304, 33, 61, "Input",ExpressionUUID->"0f502680-4556-4ebc-8263-83eba6cae38e",
 CellTags->"b:5.3.1",
 CellID->723402469],
Cell[1390632, 33825, 1371, 57, 367, "Output",ExpressionUUID->"c98686e0-0fb5-416d-aee4-05a122e21360",
 CellTags->"b:5.3.1",
 CellID->1026260884]
}, Open  ]],
Cell[1392018, 33885, 333, 8, 34, "Text",ExpressionUUID->"04572544-b478-4ace-9ff8-cecabda0c0bb",
 CellTags->"b:5.3.1",
 CellID->1320370816],
Cell[1392354, 33895, 409, 9, 34, "Text",ExpressionUUID->"249ee173-4749-4add-b9ad-cac13bde2a4b",
 CellTags->"b:5.3.1",
 CellID->1523780061],
Cell[1392766, 33906, 498, 16, 34, "Input",ExpressionUUID->"615ceb4e-3289-4eef-a6b0-946923de8f7c",
 CellTags->"b:5.3.1",
 CellID->1428994752],
Cell[1393267, 33924, 374, 8, 56, "Text",ExpressionUUID->"98c93848-cb1a-4eca-83de-631afbe6a01a",
 CellTags->"b:5.3.1",
 CellID->1805172463],
Cell[CellGroupData[{
Cell[1393666, 33936, 292, 9, 34, "Input",ExpressionUUID->"894ac134-a2c8-4a90-b4be-d216b85943aa",
 CellTags->"b:5.3.1",
 CellID->2032104472],
Cell[1393961, 33947, 743, 25, 103, "Output",ExpressionUUID->"8bc3aa38-af4f-4e3b-84f3-32b2ef691ac0",
 CellTags->"b:5.3.1",
 CellID->1925508331]
}, Open  ]],
Cell[1394719, 33975, 309, 8, 34, "Text",ExpressionUUID->"b18e7336-c2a6-43bf-9f19-fc03abbdd4b9",
 CellTags->"b:5.3.1",
 CellID->2147422663],
Cell[CellGroupData[{
Cell[1395053, 33987, 626, 17, 35, "Input",ExpressionUUID->"d7ae5193-39b0-4652-a806-571dc26986e9",
 CellTags->"b:5.3.1",
 CellID->1122525559],
Cell[1395682, 34006, 819, 25, 99, "Output",ExpressionUUID->"9cca6f84-ade3-4fcf-966d-cfbca20c2e7d",
 CellTags->"b:5.3.1",
 CellID->1587442474]
}, Open  ]],
Cell[CellGroupData[{
Cell[1396538, 34036, 277, 8, 35, "Input",ExpressionUUID->"3ae39e7c-dff6-4014-8b76-a10b70bad607",
 CellTags->"b:5.3.1",
 CellID->643231218],
Cell[1396818, 34046, 255, 7, 34, "Output",ExpressionUUID->"769b8556-ed08-4951-9ed0-7d165e4be96e",
 CellTags->"b:5.3.1",
 CellID->1956313837]
}, Open  ]],
Cell[1397088, 34056, 308, 8, 34, "Text",ExpressionUUID->"bf5e9b52-7748-4582-b998-150951546de6",
 CellTags->"b:5.3.1",
 CellID->354401928]
}, Open  ]],
Cell[CellGroupData[{
Cell[1397433, 34069, 409, 11, 67, "Subsection",ExpressionUUID->"82f38740-2373-4da1-99f9-b92faa2cf654",
 CellTags->{"c:17", "b:5.3.2"},
 CellID->1461050765],
Cell[CellGroupData[{
Cell[1397867, 34084, 530, 15, 61, "Input",ExpressionUUID->"9e247dfe-3605-458d-b6e5-a01656ea7945",
 CellTags->"b:5.3.2",
 CellID->1552187557],
Cell[1398400, 34101, 11048, 186, 189, "Output",ExpressionUUID->"649672a8-2b3e-4b35-88dc-28f59ef86e15",
 CellTags->"b:5.3.2",
 CellID->1792022889]
}, Open  ]],
Cell[1409463, 34290, 340, 8, 34, "Text",ExpressionUUID->"851a7271-8b08-4c25-a345-a13a7a4c6eea",
 CellTags->"b:5.3.2",
 CellID->2132410390],
Cell[1409806, 34300, 1264, 29, 61, "Input",ExpressionUUID->"827d49ca-b51b-489b-ab92-12dc66312362",
 CellTags->"b:5.3.2",
 CellID->66839947],
Cell[1411073, 34331, 1192, 27, 61, "Input",ExpressionUUID->"65957d83-be46-4e61-a011-a749a75c3f80",
 CellID->1282197004]
}, Open  ]],
Cell[1412280, 34361, 360, 8, 56, "Text",ExpressionUUID->"2d1618ac-1449-48ae-a25c-ee91d2af4ca3",
 CellTags->"b:5.3.2",
 CellID->245041848],
Cell[1412643, 34371, 963, 20, 36, "Input",ExpressionUUID->"288c82a6-0700-4937-8903-798913575614",
 CellTags->"b:5.3.2",
 CellID->1692188037],
Cell[1413609, 34393, 471, 13, 34, "Text",ExpressionUUID->"99b76c70-de96-49a0-ae11-e727527f66c6",
 CellTags->"b:5.3.2",
 CellID->1478400638],
Cell[1414083, 34408, 371, 8, 56, "Text",ExpressionUUID->"9817bba0-0d7d-4715-a61f-08d47c5883e4",
 CellTags->"b:5.3.2",
 CellID->1093727076],
Cell[1414457, 34418, 858, 26, 36, "Input",ExpressionUUID->"e522099b-b039-4124-80dd-6da8ad4ff503",
 CellTags->"b:5.3.2",
 CellID->1958431659],
Cell[1415318, 34446, 419, 9, 56, "Text",ExpressionUUID->"656bd96d-80a3-44bf-ad97-e78e71bbd24c",
 CellTags->"b:5.3.2",
 CellID->879035244],
Cell[CellGroupData[{
Cell[1415762, 34459, 380, 10, 35, "Input",ExpressionUUID->"b9550836-83b4-4867-a79e-efba18c94ec9",
 CellTags->"b:5.3.2",
 CellID->1405338195],
Cell[1416145, 34471, 932, 24, 49, "Output",ExpressionUUID->"d9ada68e-e25f-4a1c-8ad2-3c3e70e005c1",
 CellTags->"b:5.3.2",
 CellID->70546248]
}, Open  ]],
Cell[1417092, 34498, 447, 11, 67, "Subsection",ExpressionUUID->"42eb8e31-7dca-4919-ba24-79725a069329",
 CellTags->{"c:17", "b:5.3.2"},
 CellID->1601367499],
Cell[1417542, 34511, 193, 3, 26, "Input",ExpressionUUID->"36621e38-6ac0-4314-9866-510056697f52",
 CellID->1762967566],
Cell[1417738, 34516, 663, 12, 100, "Text",ExpressionUUID->"5dcda514-b9c0-4011-a6f5-e80504fbbe14",
 CellTags->"b:5.3.2",
 CellID->1493893659],
Cell[1418404, 34530, 281, 7, 34, "Text",ExpressionUUID->"08c5c505-c284-4a80-af16-2d6d06251bfa",
 CellTags->"b:5.3.2",
 CellID->1639409736],
Cell[1418688, 34539, 797, 26, 36, "Text",ExpressionUUID->"36340450-07e6-4dab-871d-c061c637effd",
 CellTags->"b:5.3.2",
 CellID->1344241942],
Cell[1419488, 34567, 622, 19, 34, "Input",ExpressionUUID->"4f68c5f2-e4cf-4f96-b823-8f3425dff902",
 CellTags->"b:5.3.2",
 CellID->1146352754],
Cell[1420113, 34588, 658, 18, 35, "Input",ExpressionUUID->"ba608194-9395-4f40-9f91-dfdfb0591002",
 CellTags->"b:5.3.2",
 CellID->1674939241],
Cell[CellGroupData[{
Cell[1420796, 34610, 351, 10, 35, "Input",ExpressionUUID->"a3ecc4b2-cb74-4b65-a875-4e54249f5e29",
 CellTags->"b:5.3.2",
 CellID->1488178611],
Cell[1421150, 34622, 935, 24, 49, "Output",ExpressionUUID->"94269919-99f7-4690-9ac0-71d25cb637a6",
 CellTags->"b:5.3.2",
 CellID->259816992]
}, Open  ]],
Cell[CellGroupData[{
Cell[1422122, 34651, 450, 14, 35, "Input",ExpressionUUID->"fd89ad4c-9a40-4523-a227-4caf81d6912f",
 CellTags->"b:5.3.2",
 CellID->75878468],
Cell[1422575, 34667, 937, 24, 49, "Output",ExpressionUUID->"707b5a4c-c20a-421b-abe6-80443ab77445",
 CellTags->"b:5.3.2",
 CellID->909930193]
}, Open  ]],
Cell[1423527, 34694, 545, 11, 56, "Text",ExpressionUUID->"ca2b8319-7a4c-4350-a2c5-7b0ff8e6adfd",
 CellTags->"b:5.3.2",
 CellID->1831609005],
Cell[1424075, 34707, 482, 11, 72, "Section",ExpressionUUID->"1ec40ec8-2849-466e-8d16-e8ebd0b016ad",
 CellID->1560841593],
Cell[1424560, 34720, 347, 8, 38, "Section",ExpressionUUID->"4e723664-6ab4-4d23-a380-d63a27e48b6f",
 CellID->1551905446],
Cell[CellGroupData[{
Cell[1424932, 34732, 333, 9, 51, "Subsection",ExpressionUUID->"2a96dd91-1063-41c4-85d3-c1e7c447c3d2",
 CellTags->{"c:19", "b:5.4.0"},
 CellID->560075586],
Cell[1425268, 34743, 517, 14, 56, "Text",ExpressionUUID->"79f7d0f2-136b-49d4-a57e-355196f6f9a2",
 CellTags->"b:5.4.0",
 CellID->769549101],
Cell[1425788, 34759, 1003, 31, 62, "Text",ExpressionUUID->"cb329c1c-cbb4-4c25-8eb1-0da9b75b5cc9",
 CellTags->"b:5.4.0",
 CellID->1665218273],
Cell[1426794, 34792, 466, 10, 56, "Text",ExpressionUUID->"0f5bb2f7-1e80-41ec-beff-3bbd6f960b91",
 CellTags->"b:5.4.0",
 CellID->1316626607],
Cell[CellGroupData[{
Cell[1427285, 34806, 858, 28, 87, "Input",ExpressionUUID->"25386b9f-77c4-4cf6-a233-376cdde6e458",
 CellTags->"b:5.4.0",
 CellID->871885161],
Cell[1428146, 34836, 646, 19, 49, "Output",ExpressionUUID->"c095bab3-fecb-41e3-b107-909c4aa4cd94",
 CellTags->"b:5.4.0",
 CellID->965703464],
Cell[1428795, 34857, 745, 23, 65, "Output",ExpressionUUID->"e001e1cf-393f-4b8e-bcdc-d25fd174e905",
 CellTags->"b:5.4.0",
 CellID->2044199069],
Cell[1429543, 34882, 745, 23, 65, "Output",ExpressionUUID->"5fa16457-a60f-4879-b4a1-c0f667ef7558",
 CellTags->"b:5.4.0",
 CellID->1410367977]
}, Open  ]],
Cell[1430303, 34908, 276, 7, 34, "Text",ExpressionUUID->"5a62cfea-adc2-4dd8-8d29-031bfcdb1501",
 CellTags->"b:5.4.0",
 CellID->1492977446],
Cell[CellGroupData[{
Cell[1430604, 34919, 534, 12, 113, "Input",ExpressionUUID->"80baf6e5-f1ab-41a7-a52b-169f2630e3de",
 CellTags->"b:5.4.0",
 CellID->1273003374],
Cell[1431141, 34933, 252, 7, 34, "Output",ExpressionUUID->"6231fc08-3aea-441c-be57-ba2b8f80fa80",
 CellTags->"b:5.4.0",
 CellID->130093853],
Cell[1431396, 34942, 252, 7, 34, "Output",ExpressionUUID->"5c45bd6c-bca9-448b-82c9-618033c66c04",
 CellTags->"b:5.4.0",
 CellID->349034725],
Cell[1431651, 34951, 984, 35, 62, "Output",ExpressionUUID->"3fe88c77-124c-49d8-93bf-b6ec35837fb3",
 CellTags->"b:5.4.0",
 CellID->1448783118],
Cell[1432638, 34988, 271, 7, 34, "Output",ExpressionUUID->"2c33421e-361a-4c29-b258-ad62d6bc6639",
 CellTags->"b:5.4.0",
 CellID->1528274540]
}, Open  ]],
Cell[1432924, 34998, 592, 16, 37, "Text",ExpressionUUID->"42524f38-3d8c-4854-9cbe-c9f6b6de1db1",
 CellTags->"b:5.4.0",
 CellID->2135615404],
Cell[CellGroupData[{
Cell[1433541, 35018, 2323, 65, 165, "Input",ExpressionUUID->"77bb2d8c-2b3c-44a8-92db-b8d38f5c2a44",
 CellTags->"b:5.4.0",
 CellID->1216838074],
Cell[1435867, 35085, 8314, 159, 214, "Output",ExpressionUUID->"feea8ca8-ff01-4ca8-92f3-45c13e645ae4",
 CellTags->"b:5.4.0",
 CellID->1805775660],
Cell[1444184, 35246, 2123, 54, 208, "Output",ExpressionUUID->"79fa83cd-56dd-4800-8ef6-7e455f6e333d",
 CellTags->"b:5.4.0",
 CellID->1432618216]
}, Open  ]],
Cell[1446322, 35303, 639, 16, 37, "Text",ExpressionUUID->"df8dea70-656e-46f3-bb03-c943dd8e682e",
 CellTags->"b:5.4.0",
 CellID->910739337],
Cell[CellGroupData[{
Cell[1446986, 35323, 2440, 66, 191, "Input",ExpressionUUID->"af3a70ce-64b0-45c5-9ca9-45cbe17841d2",
 CellTags->"b:5.4.0",
 CellID->1680084790],
Cell[1449429, 35391, 8312, 159, 214, "Output",ExpressionUUID->"9db8fd82-25d8-49fe-b485-436c4c11c959",
 CellTags->"b:5.4.0",
 CellID->427706148],
Cell[1457744, 35552, 2122, 54, 208, "Output",ExpressionUUID->"82788c0f-78e2-4bb3-a650-24d5436a4dfe",
 CellTags->"b:5.4.0",
 CellID->829462927]
}, Open  ]],
Cell[1459881, 35609, 373, 9, 34, "Text",ExpressionUUID->"8be4c9db-7c8d-445a-9a95-308257c4b589",
 CellTags->"b:5.4.0",
 CellID->2070074774]
}, Open  ]],
Cell[1460269, 35621, 304, 7, 34, "Text",ExpressionUUID->"3c636c69-1843-4910-9312-31d43c7b56e6",
 CellTags->"b:5.4.1",
 CellID->746807497],
Cell[CellGroupData[{
Cell[1460598, 35632, 1795, 44, 94, "Input",ExpressionUUID->"bd048c81-aff1-4d2b-9040-00ebc4707bec",
 CellID->1841029403],
Cell[1462396, 35678, 6780, 136, 220, "Output",ExpressionUUID->"c6bcfd58-6c23-4908-9dbf-fd68a3cbe7f0",
 CellID->1467061412]
}, Open  ]],
Cell[CellGroupData[{
Cell[1469213, 35819, 1481, 41, 87, "Input",ExpressionUUID->"617bd12c-6b69-4040-9ce9-f8b288633cc5",
 CellTags->"b:5.4.1",
 CellID->1699169011],
Cell[1470697, 35862, 6700, 135, 220, "Output",ExpressionUUID->"23be53ab-3cf4-4787-b6d3-2b76ba0537c2",
 CellTags->"b:5.4.1",
 CellID->1641400277]
}, Open  ]],
Cell[CellGroupData[{
Cell[1477434, 36002, 1562, 43, 88, "Input",ExpressionUUID->"9b048550-5dd9-4b4f-92e4-f87f9b2f7359",
 CellTags->"b:5.4.1",
 CellID->696030071],
Cell[1478999, 36047, 20269, 357, 214, "Output",ExpressionUUID->"d6a6bf32-8d2c-425f-9889-ea89084cafea",
 CellTags->"b:5.4.1",
 CellID->439526177]
}, Open  ]],
Cell[1499283, 36407, 330, 9, 34, "Input",ExpressionUUID->"72fa4c96-e9bd-48e1-809a-4f6bbab4835c",
 CellTags->"b:5.4.1",
 CellID->56085277],
Cell[CellGroupData[{
Cell[1499638, 36420, 1253, 35, 73, "Input",ExpressionUUID->"44243b68-cfba-40a1-80e4-93a333b7a50f",
 CellID->619177287],
Cell[1500894, 36457, 24302, 423, 211, "Output",ExpressionUUID->"568eed8f-28ba-4cbb-af38-f2d4633322c1",
 CellID->1920194806]
}, Open  ]],
Cell[1525211, 36883, 527, 12, 72, "Section",ExpressionUUID->"1af3606f-3951-45b8-b9fc-ed7733087af3",
 CellID->836847359],
Cell[1525741, 36897, 347, 8, 38, "Section",ExpressionUUID->"d3e96145-fa72-4bdd-b7a0-e7c7be37d967",
 CellID->1023158510],
Cell[CellGroupData[{
Cell[1526113, 36909, 455, 11, 51, "Subsection",ExpressionUUID->"a5042c33-4e8a-4f68-bb87-27a3dcf5679d",
 CellTags->{"c:4", "b:5.4.2"},
 CellID->833978088],
Cell[1526571, 36922, 324, 9, 36, "Text",ExpressionUUID->"da8a42de-319a-4623-8b82-11040cdca14f",
 CellTags->"b:5.4.2",
 CellID->43961047],
Cell[1526898, 36933, 1076, 32, 38, "Input",ExpressionUUID->"4df0a2e5-26bf-4455-9b40-ebec7c494a13",
 CellTags->"b:5.4.2",
 CellID->118942361],
Cell[1527977, 36967, 324, 7, 36, "Text",ExpressionUUID->"1878d201-ae56-43da-86fd-3815d5816e49",
 CellTags->"b:5.4.2",
 CellID->1904469617],
Cell[CellGroupData[{
Cell[1528326, 36978, 1497, 32, 61, "Input",ExpressionUUID->"81e22272-8376-433f-99e9-9657bb175343",
 CellID->177094384],
Cell[1529826, 37012, 3187, 87, 304, "Output",ExpressionUUID->"192e8347-a04b-4068-b0bb-d94d3cf0a9f0",
 CellID->329857326]
}, Open  ]],
Cell[1533028, 37102, 271, 8, 35, "Input",ExpressionUUID->"0ee2aa6d-b77e-4e12-8557-a2d4db919ff6",
 CellTags->"b:5.4.2",
 CellID->12977146],
Cell[1533302, 37112, 584, 18, 35, "Input",ExpressionUUID->"738a328c-2255-4956-bde0-759633f7b387",
 CellTags->"b:5.4.2",
 CellID->1211044475],
Cell[1533889, 37132, 524, 14, 37, "Text",ExpressionUUID->"2df9481b-f8a6-43f7-8749-e9d4c09686c0",
 CellTags->"b:5.4.2",
 CellID->174485059],
Cell[1534416, 37148, 866, 30, 38, "Input",ExpressionUUID->"941befc2-a701-4368-8e45-f60d660060e6",
 CellTags->"b:5.4.2",
 CellID->1255208458],
Cell[1535285, 37180, 239, 6, 36, "Text",ExpressionUUID->"e7fd648e-dec6-4364-a986-7e519029b0c5",
 CellTags->"b:5.4.2",
 CellID->1556689125],
Cell[CellGroupData[{
Cell[1535549, 37190, 368, 13, 38, "Input",ExpressionUUID->"81ac92b3-69fb-48f7-be76-2ce01e75a6b2",
 CellTags->"b:5.4.2",
 CellID->480852474],
Cell[1535920, 37205, 1087, 29, 167, "Output",ExpressionUUID->"d90617c6-0bc0-449a-8d9a-f2788b560f43",
 CellTags->"b:5.4.2",
 CellID->1256785175]
}, Open  ]],
Cell[CellGroupData[{
Cell[1537044, 37239, 529, 19, 67, "Input",ExpressionUUID->"c5494493-5782-4ab1-9b4f-1e25d4175d1d",
 CellTags->"b:5.4.2",
 CellID->706566857],
Cell[1537576, 37260, 1085, 29, 167, "Output",ExpressionUUID->"1be31ca7-fe0a-4200-b422-653f17ca62f8",
 CellTags->"b:5.4.2",
 CellID->139284333],
Cell[1538664, 37291, 1132, 31, 167, "Output",ExpressionUUID->"d5ede5ab-72c7-4535-9bd5-15cd117748e9",
 CellTags->"b:5.4.2",
 CellID->194609501]
}, Open  ]],
Cell[CellGroupData[{
Cell[1539833, 37327, 531, 19, 67, "Input",ExpressionUUID->"7cef13a1-3b67-49b3-a815-d7f9fe63bf72",
 CellTags->"b:5.4.2",
 CellID->1832081336],
Cell[1540367, 37348, 1087, 29, 167, "Output",ExpressionUUID->"31fa6b93-5c2d-4338-8283-1820a6c3b534",
 CellTags->"b:5.4.2",
 CellID->184288926],
Cell[1541457, 37379, 1135, 31, 167, "Output",ExpressionUUID->"c401c446-2c8d-4f72-b456-8ed523cd5752",
 CellTags->"b:5.4.2",
 CellID->672560308]
}, Open  ]],
Cell[CellGroupData[{
Cell[1542629, 37415, 531, 19, 67, "Input",ExpressionUUID->"4d247d11-efa0-4a43-8d56-eae24f3433b3",
 CellTags->"b:5.4.2",
 CellID->1170483466],
Cell[1543163, 37436, 1088, 29, 167, "Output",ExpressionUUID->"ba5be03f-1f6c-49be-b532-b7c935139c05",
 CellTags->"b:5.4.2",
 CellID->1514662395],
Cell[1544254, 37467, 1133, 31, 167, "Output",ExpressionUUID->"827f1198-eff5-4659-b607-6fb4ca53ae2a",
 CellTags->"b:5.4.2",
 CellID->659721227]
}, Open  ]],
Cell[1545402, 37501, 568, 15, 108, "Text",ExpressionUUID->"3758cfd0-0967-4f6c-8fde-b4f4ccd45ece",
 CellTags->"b:5.4.2",
 CellID->1365406542],
Cell[CellGroupData[{
Cell[1545995, 37520, 653, 21, 67, "Input",ExpressionUUID->"a9cccde9-0e06-4373-be0b-e2b544788507",
 CellTags->"b:5.4.2",
 CellID->377499552],
Cell[1546651, 37543, 1086, 29, 167, "Output",ExpressionUUID->"95182a7c-a2a4-46e5-85ad-8cbe5f15812a",
 CellTags->"b:5.4.2",
 CellID->476592728]
}, Open  ]]
}, Open  ]],
Cell[1547764, 37576, 495, 11, 43, "Text",ExpressionUUID->"bd119060-d86e-41e4-8929-b70e47e43d1e",
 CellTags->"b:5.4.2",
 CellID->244036809],
Cell[1548262, 37589, 841, 25, 38, "Input",ExpressionUUID->"e51cde0d-b15b-4ad1-b38f-a60bd7dd5d9a",
 CellID->968277226],
Cell[1549106, 37616, 505, 11, 43, "Text",ExpressionUUID->"e6665051-a251-4272-b97c-76fcc830ab4b",
 CellTags->"b:5.4.2",
 CellID->463033963],
Cell[1549614, 37629, 1541, 44, 85, "Input",ExpressionUUID->"3e6d7750-5483-4924-a97c-43464fe66bfa",
 CellTags->"b:5.4.2",
 CellID->204260416],
Cell[1551158, 37675, 496, 11, 43, "Text",ExpressionUUID->"6ec9e7e1-20e9-4661-974d-c76ffdece25c",
 CellTags->"b:5.4.2",
 CellID->2108443174],
Cell[1551657, 37688, 1497, 47, 87, "Input",ExpressionUUID->"4f2eb1d4-5739-4340-8724-7b83fa681bb8",
 CellID->1325943806],
Cell[1553157, 37737, 545, 12, 43, "Text",ExpressionUUID->"07e77b04-3689-4544-a342-40e40e470fa5",
 CellTags->"b:5.4.2",
 CellID->553054382],
Cell[1553705, 37751, 830, 25, 35, "Input",ExpressionUUID->"14ece725-9666-4ca9-a292-607efa8bc7b9",
 CellID->875774058],
Cell[CellGroupData[{
Cell[1554560, 37780, 863, 19, 138, "Input",ExpressionUUID->"ff67f2ba-2d53-4156-84db-13d09ea73ee8",
 CellID->699742523],
Cell[1555426, 37801, 656, 19, 65, "Output",ExpressionUUID->"667d2665-80ff-41ef-b847-099e109d6a97",
 CellID->2127459833],
Cell[1556085, 37822, 2126, 62, 67, "Output",ExpressionUUID->"709e551f-ed4f-4176-90fa-093adc8219c1",
 CellID->614690681],
Cell[1558214, 37886, 656, 19, 65, "Output",ExpressionUUID->"6b1ee898-776c-4cbc-b8ca-80ff36dcf312",
 CellID->1709933497],
Cell[1558873, 37907, 655, 19, 65, "Output",ExpressionUUID->"485bcaca-eb30-4c24-bed2-4af39d8038ff",
 CellID->1226119925],
Cell[1559531, 37928, 2116, 59, 67, "Output",ExpressionUUID->"094d1cba-978c-4302-a7a2-01e5d0cc5215",
 CellID->144502863]
}, Open  ]],
Cell[CellGroupData[{
Cell[1561684, 37992, 564, 15, 110, "Input",ExpressionUUID->"522569ab-0bae-4c9c-ad6c-4ce4a6fef04d",
 CellTags->"b:5.4.2",
 CellID->952133089],
Cell[1562251, 38009, 256, 7, 34, "Output",ExpressionUUID->"22b87560-7ac8-41e5-89d9-373d5c202adf",
 CellTags->"b:5.4.2",
 CellID->2003097331],
Cell[1562510, 38018, 272, 8, 34, "Output",ExpressionUUID->"fca743f1-4e7d-4848-a5c5-a302b8e1988b",
 CellTags->"b:5.4.2",
 CellID->2103191745],
Cell[1562785, 38028, 271, 8, 34, "Output",ExpressionUUID->"65d05e81-7873-4e78-b549-60d22c9be11f",
 CellTags->"b:5.4.2",
 CellID->762828595],
Cell[1563059, 38038, 271, 8, 34, "Output",ExpressionUUID->"703dd229-0561-4ea1-835b-6cfcc1d2e55c",
 CellTags->"b:5.4.2",
 CellID->382823575]
}, Open  ]],
Cell[1563345, 38049, 242, 6, 36, "Text",ExpressionUUID->"977b8eb5-5c0f-4d4c-a694-41f04f4645ad",
 CellTags->"b:5.4.2",
 CellID->1600173026],
Cell[CellGroupData[{
Cell[1563612, 38059, 1341, 38, 73, "Input",ExpressionUUID->"0bc25726-f7ea-4a0a-b1e0-08e28ba068bb",
 CellTags->"b:5.4.2",
 CellID->1170336601],
Cell[1564956, 38099, 256, 7, 34, "Output",ExpressionUUID->"8e3fa3de-c761-4e65-8643-6a5495160947",
 CellTags->"b:5.4.2",
 CellID->254818613]
}, Open  ]],
Cell[1565227, 38109, 259, 6, 36, "Text",ExpressionUUID->"930c24ab-5283-409a-b704-a75e9993ba2f",
 CellTags->"b:5.4.2",
 CellID->1044530134],
Cell[CellGroupData[{
Cell[1565511, 38119, 1692, 52, 64, "Input",ExpressionUUID->"40e3c148-b675-404d-acfb-a7388342a736",
 CellTags->"b:5.4.2",
 CellID->1886631560],
Cell[1567206, 38173, 256, 7, 34, "Output",ExpressionUUID->"0d6d2a04-6bb1-4501-a733-dc0ad4f17055",
 CellTags->"b:5.4.2",
 CellID->654036573]
}, Open  ]],
Cell[CellGroupData[{
Cell[1567499, 38185, 300, 8, 34, "Input",ExpressionUUID->"eaf6241f-c904-4e82-b649-df4b8d5760f5",
 CellID->1020580965],
Cell[1567802, 38195, 11693, 277, 516, "Output",ExpressionUUID->"61967dd7-8761-4fa9-873c-ce431109323c",
 CellID->1571377065]
}, Open  ]],
Cell[1579510, 38475, 260, 6, 36, "Text",ExpressionUUID->"df6bbfc4-3200-4263-b932-13ddd21656fb",
 CellTags->"b:5.4.2",
 CellID->952390166],
Cell[CellGroupData[{
Cell[1579795, 38485, 453, 13, 36, "Input",ExpressionUUID->"fbd59ee9-7d97-4612-b8ab-1ae549d780fc",
 CellTags->"b:5.4.2",
 CellID->1625898871],
Cell[1580251, 38500, 253, 7, 34, "Output",ExpressionUUID->"a2a088b2-718e-4878-a017-8c620c54a7bf",
 CellTags->"b:5.4.2",
 CellID->380440649]
}, Open  ]],
Cell[1580519, 38510, 395, 10, 60, "Text",ExpressionUUID->"c79efdd7-3e4c-47c5-a57f-94b4585fe558",
 CellTags->"b:5.4.2",
 CellID->1900400469],
Cell[CellGroupData[{
Cell[1580939, 38524, 451, 13, 35, "Input",ExpressionUUID->"61faaa55-732b-4c85-8dc5-120448f439ca",
 CellID->506400652],
Cell[1581393, 38539, 651, 19, 34, "Output",ExpressionUUID->"e3b22ae9-8cff-4f03-9b37-6ca9bf3c97d3",
 CellID->1007170624]
}, Open  ]],
Cell[CellGroupData[{
Cell[1582081, 38563, 314, 8, 34, "Input",ExpressionUUID->"198f2f50-5945-463a-a847-2b0244ce2b23",
 CellID->598065103],
Cell[1582398, 38573, 9990, 233, 609, "Output",ExpressionUUID->"eee7ff0f-0aa1-4030-99ba-95818a06ee69",
 CellID->1050571914]
}, Open  ]],
Cell[1592403, 38809, 623, 21, 38, "Input",ExpressionUUID->"95c3c0c2-992e-42e4-934e-f3b44628bf0e",
 CellTags->"b:5.4.2",
 CellID->1456718161],
Cell[CellGroupData[{
Cell[1593051, 38834, 337, 9, 35, "Input",ExpressionUUID->"5f3f5934-304b-419b-9693-c2ca00cd8a30",
 CellID->1748556127],
Cell[1593391, 38845, 747, 22, 99, "Output",ExpressionUUID->"cc859ba3-d4ca-4bba-87c5-27d749054ff9",
 CellID->351612964]
}, Open  ]],
Cell[CellGroupData[{
Cell[1594175, 38872, 325, 10, 35, "Input",ExpressionUUID->"a1a441ab-f990-4bd7-814e-23f73d610cef",
 CellTags->"b:5.4.2",
 CellID->1816600941],
Cell[1594503, 38884, 770, 23, 99, "Output",ExpressionUUID->"6e24f2e0-7271-48dc-ab0f-7f7956320d28",
 CellTags->"b:5.4.2",
 CellID->1831573051]
}, Open  ]],
Cell[CellGroupData[{
Cell[1595310, 38912, 501, 13, 61, "Input",ExpressionUUID->"dd1af27d-069c-4786-81fa-eb230de472ef",
 CellTags->"b:5.4.2",
 CellID->837005988],
Cell[1595814, 38927, 770, 23, 99, "Output",ExpressionUUID->"55c9c578-e3c5-4fb9-aa92-da67b44425f9",
 CellTags->"b:5.4.2",
 CellID->1217672059],
Cell[1596587, 38952, 2732, 81, 171, "Output",ExpressionUUID->"d79c2b96-af24-4422-8ef5-9518fc6ed155",
 CellTags->"b:5.4.2",
 CellID->2042623350]
}, Open  ]],
Cell[1599334, 39036, 247, 6, 36, "Text",ExpressionUUID->"259014a0-bd07-4247-9f1c-e84000bcf393",
 CellTags->"b:5.4.2",
 CellID->1541752466],
Cell[CellGroupData[{
Cell[1599606, 39046, 344, 9, 35, "Input",ExpressionUUID->"164485b6-37ff-40e0-b848-d50804f50944",
 CellTags->"b:5.4.2",
 CellID->696011360],
Cell[1599953, 39057, 252, 7, 34, "Output",ExpressionUUID->"5a505f1b-9620-4af7-bae7-ea0326d3e573",
 CellTags->"b:5.4.2",
 CellID->697062508]
}, Open  ]],
Cell[1600220, 39067, 371, 10, 36, "Text",ExpressionUUID->"3de15fce-e7ca-4142-8ce7-714410eac64d",
 CellTags->"b:5.4.2",
 CellID->519502311],
Cell[1600594, 39079, 283, 8, 36, "Text",ExpressionUUID->"da905970-b161-4ff7-bca5-43d093434d63",
 CellTags->"b:5.4.2",
 CellID->1754315922],
Cell[CellGroupData[{
Cell[1600902, 39091, 570, 17, 63, "Input",ExpressionUUID->"6636c4d3-44d9-4d77-b7f7-f0d9e2ceb1b2",
 CellTags->"b:5.4.2",
 CellID->1984711391],
Cell[1601475, 39110, 696, 21, 65, "Output",ExpressionUUID->"810fec02-6154-44a4-b30b-580054fd8b0b",
 CellTags->"b:5.4.2",
 CellID->1006277571],
Cell[1602174, 39133, 256, 7, 34, "Output",ExpressionUUID->"41a6dcc5-d9af-4122-b826-8180203d90ed",
 CellTags->"b:5.4.2",
 CellID->1073415672]
}, Open  ]],
Cell[CellGroupData[{
Cell[1602467, 39145, 597, 18, 63, "Input",ExpressionUUID->"892fa14c-568c-4f67-8916-885ec52497db",
 CellTags->"b:5.4.2",
 CellID->180859686],
Cell[1603067, 39165, 696, 21, 65, "Output",ExpressionUUID->"b5280a49-784e-49a2-93e0-0cb136d74e8b",
 CellTags->"b:5.4.2",
 CellID->2034564504],
Cell[1603766, 39188, 253, 7, 34, "Output",ExpressionUUID->"0c73afa4-8417-4be9-872e-b10e0d0c5765",
 CellTags->"b:5.4.2",
 CellID->543507547]
}, Open  ]],
Cell[CellGroupData[{
Cell[1604056, 39200, 570, 17, 63, "Input",ExpressionUUID->"76f24b89-53fa-43d7-98d1-b201a608ff35",
 CellTags->"b:5.4.2",
 CellID->1019382097],
Cell[1604629, 39219, 698, 21, 65, "Output",ExpressionUUID->"760f1d12-a2f9-4a5c-982b-8050367e7585",
 CellTags->"b:5.4.2",
 CellID->1483391738],
Cell[1605330, 39242, 254, 7, 34, "Output",ExpressionUUID->"7195585d-466a-4c9d-ac84-e2205237cf70",
 CellTags->"b:5.4.2",
 CellID->1227282543]
}, Open  ]],
Cell[CellGroupData[{
Cell[1605621, 39254, 524, 12, 67, "Subsection",ExpressionUUID->"414c0236-744a-4774-ac1e-3e43612d0c76",
 CellTags->{"c:4", "b:5.4.2"},
 CellID->426124609],
Cell[1606148, 39268, 194, 3, 30, "Input",ExpressionUUID->"2575b7e7-4e66-46e1-83b4-fdccffb26159",
 CellID->1258694513],
Cell[CellGroupData[{
Cell[1606367, 39275, 764, 20, 63, "Subsubsection",ExpressionUUID->"53c7bb61-6b78-470f-8814-bcccc172ee80",
 CellTags->{"c:4", "b:5.4.3"},
 CellID->130368513],
Cell[1607134, 39297, 717, 16, 63, "Text",ExpressionUUID->"3e4c9e2a-34aa-4c03-9ca0-1ebf80df2c75",
 CellTags->"b:5.4.3",
 CellID->670278051],
Cell[1607854, 39315, 1444, 46, 154, "Text",ExpressionUUID->"733da3eb-a9f9-4fcf-98e4-3f623293f0f0",
 CellTags->"b:5.4.3",
 CellID->1811474642],
Cell[1609301, 39363, 390, 9, 34, "Text",ExpressionUUID->"7506d326-4e33-4f99-af73-1e7d97056dfd",
 CellTags->"b:5.4.3",
 CellID->578644575],
Cell[1609694, 39374, 345, 8, 34, "Text",ExpressionUUID->"c95cba18-4335-45f4-ac0e-dd1aa47ec390",
 CellTags->"b:5.4.3",
 CellID->1473418409],
Cell[CellGroupData[{
Cell[1610064, 39386, 439, 14, 35, "Input",ExpressionUUID->"62e71298-7a55-4264-844a-b96c43876f42",
 CellTags->"b:5.4.3",
 CellID->1109266506],
Cell[1610506, 39402, 1102, 39, 123, "Output",ExpressionUUID->"2246db1f-bd02-4529-a8d7-97a97767df17",
 CellTags->"b:5.4.3",
 CellID->357270766]
}, Open  ]],
Cell[1611623, 39444, 287, 7, 34, "Text",ExpressionUUID->"dab60708-b3ab-4253-bc31-d3f745b179cb",
 CellTags->"b:5.4.3",
 CellID->1136626735],
Cell[1611913, 39453, 935, 30, 36, "Input",ExpressionUUID->"4dc9f31b-e280-4e97-bf80-6c70fff92811",
 CellTags->"b:5.4.3",
 CellID->1418495080],
Cell[1612851, 39485, 733, 21, 40, "Text",ExpressionUUID->"331d6f83-cdda-4eb6-b996-73efe5159ac2",
 CellTags->"b:5.4.3",
 CellID->1430844213],
Cell[CellGroupData[{
Cell[1613609, 39510, 621, 18, 117, "Input",ExpressionUUID->"78495325-4b57-45c4-bd14-ab85a4d10b84",
 CellTags->"b:5.4.3",
 CellID->676808785],
Cell[1614233, 39530, 818, 25, 99, "Output",ExpressionUUID->"9a0d5cba-a077-46cd-9ff1-5e5929ba9021",
 CellTags->"b:5.4.3",
 CellID->285487150],
Cell[1615054, 39557, 818, 25, 99, "Output",ExpressionUUID->"2f77fa47-da39-4cdb-a045-8d67cd9433ac",
 CellTags->"b:5.4.3",
 CellID->704062652],
Cell[1615875, 39584, 866, 27, 99, "Output",ExpressionUUID->"7786012e-f47f-4fb7-bc0a-b2e6ec3d921a",
 CellTags->"b:5.4.3",
 CellID->546097194],
Cell[1616744, 39613, 865, 27, 99, "Output",ExpressionUUID->"838c3dde-cfb6-4788-bed0-30e995d60af6",
 CellTags->"b:5.4.3",
 CellID->2057915927]
}, Open  ]],
Cell[1617624, 39643, 611, 12, 56, "Text",ExpressionUUID->"0326e2b2-044f-409b-9e43-15ec86bbb1b5",
 CellTags->"b:5.4.3",
 CellID->2051854983],
Cell[1618238, 39657, 1764, 55, 41, "Input",ExpressionUUID->"2709db0f-0240-467b-81e1-04bc8f29d496",
 CellTags->"b:5.4.3",
 CellID->1234295755],
Cell[1620005, 39714, 394, 9, 34, "Text",ExpressionUUID->"ebe28e0d-1698-4d21-a0c1-bda102f17bb0",
 CellTags->"b:5.4.3",
 CellID->116724265],
Cell[CellGroupData[{
Cell[1620424, 39727, 502, 14, 36, "Input",ExpressionUUID->"f1b0bcf9-449a-4cdf-97c8-df7835258dea",
 CellTags->"b:5.4.3",
 CellID->1132152144],
Cell[1620929, 39743, 3043, 101, 119, "Output",ExpressionUUID->"11a505dd-2edf-42aa-a15e-61b2bb6b696c",
 CellTags->"b:5.4.3",
 CellID->21366507]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1624033, 39851, 463, 16, 33, "Input",ExpressionUUID->"5d3f3d60-dfeb-4429-9c51-d6988fccbe30",
 CellID->1632876762],
Cell[1624499, 39869, 233, 6, 34, "Output",ExpressionUUID->"e062c410-b2b8-4b47-adeb-d9a2f8437340",
 CellID->477114100]
}, Open  ]],
Cell[1624747, 39878, 342, 8, 34, "Text",ExpressionUUID->"edb1ff14-41fd-4c11-9759-a8d5538c5563",
 CellTags->"b:5.4.3",
 CellID->1375971788],
Cell[CellGroupData[{
Cell[1625114, 39890, 917, 27, 117, "Input",ExpressionUUID->"26e0210e-ea4e-43f9-aa94-1d2d0a7eae31",
 CellTags->"b:5.4.3",
 CellID->1853930020],
Cell[1626034, 39919, 1028, 31, 115, "Output",ExpressionUUID->"6aa1cd13-b9ac-4aa2-bf7b-216ff9df1e97",
 CellTags->"b:5.4.3",
 CellID->152740802],
Cell[1627065, 39952, 1031, 31, 115, "Output",ExpressionUUID->"8c7f8ee0-0930-41be-9acf-5f557d0e6327",
 CellTags->"b:5.4.3",
 CellID->871701049],
Cell[1628099, 39985, 1075, 33, 115, "Output",ExpressionUUID->"89f954d2-3d7b-475d-bfe7-954d90500523",
 CellTags->"b:5.4.3",
 CellID->546090709],
Cell[1629177, 40020, 1078, 33, 115, "Output",ExpressionUUID->"f6aa19d9-0d10-43ed-976d-95419374dd23",
 CellTags->"b:5.4.3",
 CellID->1948922532]
}, Open  ]],
Cell[1630270, 40056, 343, 8, 34, "Text",ExpressionUUID->"96e7758e-4e27-4f55-8e83-30774e3d9c88",
 CellTags->"b:5.4.3",
 CellID->1161611817],
Cell[CellGroupData[{
Cell[1630638, 40068, 1682, 51, 117, "Input",ExpressionUUID->"df10aa0a-fbde-4aec-be20-7ba91ac87bb7",
 CellTags->"b:5.4.3",
 CellID->435973442],
Cell[1632323, 40121, 1054, 34, 115, "Output",ExpressionUUID->"9293671e-a1b6-43fe-9ef7-285013f1a3c0",
 CellTags->"b:5.4.3",
 CellID->2120411280],
Cell[1633380, 40157, 1053, 34, 115, "Output",ExpressionUUID->"784b0996-2f19-4851-8db3-caef6ca4f321",
 CellTags->"b:5.4.3",
 CellID->261260995],
Cell[1634436, 40193, 1053, 34, 115, "Output",ExpressionUUID->"87dd2014-db60-4d61-bd4e-dbd24ee203f3",
 CellTags->"b:5.4.3",
 CellID->1556968497],
Cell[1635492, 40229, 1053, 34, 115, "Output",ExpressionUUID->"b09f40a2-4de6-4a67-8932-37369727080d",
 CellTags->"b:5.4.3",
 CellID->881290384]
}, Open  ]],
Cell[CellGroupData[{
Cell[1636582, 40268, 798, 22, 63, "Subsubsection",ExpressionUUID->"6947aa9b-4ca1-4dd0-8de8-c51dbebff4f5",
 CellTags->{"c:4", "b:5.4.3"},
 CellID->191436130],
Cell[1637383, 40292, 306, 9, 60, "Text",ExpressionUUID->"25513e3a-3ad4-4cf8-aad2-984c588fa77e",
 CellTags->"b:5.4.3",
 CellID->536533704],
Cell[CellGroupData[{
Cell[1637714, 40305, 2710, 86, 227, "Input",ExpressionUUID->"84e13ff0-c022-4ed0-ac70-ece8453fc60c",
 CellTags->"b:5.4.3",
 CellID->238129375],
Cell[1640427, 40393, 697, 22, 65, "Output",ExpressionUUID->"059d97b3-0f09-4150-956d-4506533073f0",
 CellTags->"b:5.4.3",
 CellID->652770529],
Cell[1641127, 40417, 698, 22, 65, "Output",ExpressionUUID->"a2f15c06-716c-4cb5-be38-c86a88126d6d",
 CellTags->"b:5.4.3",
 CellID->1767812027],
Cell[1641828, 40441, 1087, 34, 75, "Output",ExpressionUUID->"3cd2e40f-73a4-4108-84ca-b28b81b118a6",
 CellTags->"b:5.4.3",
 CellID->1180481544]
}, Open  ]],
Cell[CellGroupData[{
Cell[1642952, 40480, 426, 12, 36, "Input",ExpressionUUID->"b4c29c3f-9d36-4715-bc0c-4b035c283778",
 CellID->1248054017],
Cell[1643381, 40494, 3219, 94, 181, "Output",ExpressionUUID->"c8a45c80-bbfb-4b2c-b17a-6390f8434bcd",
 CellID->1899379022]
}, Open  ]],
Cell[1646615, 40591, 316, 9, 60, "Text",ExpressionUUID->"0b7b1077-c52e-46d0-8691-32716380f856",
 CellTags->"b:5.4.3",
 CellID->1478371264],
Cell[1646934, 40602, 249, 6, 36, "Text",ExpressionUUID->"cfa49947-2cd4-44c7-81d8-d28852dc3ccd",
 CellTags->"b:5.4.3",
 CellID->600038258],
Cell[1647186, 40610, 229, 6, 36, "Text",ExpressionUUID->"b44d39ff-9c0c-4689-b209-569ff5d5a465",
 CellTags->"b:5.4.3",
 CellID->259795894],
Cell[CellGroupData[{
Cell[1647440, 40620, 877, 29, 67, "Input",ExpressionUUID->"9431eeb6-cbe4-47c9-ab1d-86cf3444da46",
 CellTags->"b:5.4.3",
 CellID->555336107],
Cell[1648320, 40651, 884, 29, 49, "Output",ExpressionUUID->"4992e30f-78ea-47f6-9f6e-d07070906ed7",
 CellTags->"b:5.4.3",
 CellID->518810099],
Cell[1649207, 40682, 5462, 172, 166, "Output",ExpressionUUID->"6500a847-645a-4f95-a183-c4ced22f4aa2",
 CellTags->"b:5.4.3",
 CellID->857727073]
}, Open  ]],
Cell[1654684, 40857, 282, 7, 34, "Input",ExpressionUUID->"f028965b-cf28-4640-b3c2-65b81fdb7753",
 CellID->1839307272]
}, Open  ]],
Cell[CellGroupData[{
Cell[1655003, 40869, 728, 19, 63, "Subsubsection",ExpressionUUID->"1a8ad92b-cecd-42aa-a3c0-79223df306c8",
 CellTags->{"c:4", "b:5.4.3"},
 CellID->570020487],
Cell[1655734, 40890, 342, 10, 60, "Text",ExpressionUUID->"e0391405-ccc3-4cec-951f-aaa3090bd777",
 CellTags->"b:5.4.3",
 CellID->409775742],
Cell[CellGroupData[{
Cell[1656101, 40904, 1202, 36, 104, "Input",ExpressionUUID->"c08e96fa-eb3b-4eec-a51a-da09983ba1e5",
 CellTags->"b:5.4.3",
 CellID->120839865],
Cell[1657306, 40942, 3045, 101, 119, "Output",ExpressionUUID->"0f838e6a-e042-4b7b-856a-d7d887d6dd75",
 CellTags->"b:5.4.3",
 CellID->1908677247],
Cell[1660354, 41045, 253, 7, 34, "Output",ExpressionUUID->"cb1e0d7a-02ec-45a7-adbc-6cf16c01189d",
 CellTags->"b:5.4.3",
 CellID->27771443]
}, Open  ]],
Cell[1660622, 41055, 1892, 64, 63, "Text",ExpressionUUID->"d3676458-3b45-4bc7-8c58-ce02f3bf2925",
 CellTags->"b:5.4.3",
 CellID->1583564640],
Cell[CellGroupData[{
Cell[1662539, 41123, 1658, 52, 62, "Input",ExpressionUUID->"98f99a8c-dba7-4eed-ac34-06853fcffa2c",
 CellTags->"b:5.4.3",
 CellID->1206067209],
Cell[1664200, 41177, 3042, 99, 119, "Output",ExpressionUUID->"b56d98af-5fc8-430a-9bf7-157bf5e99d8b",
 CellTags->"b:5.4.3",
 CellID->750691102],
Cell[1667245, 41278, 718, 21, 83, "Output",ExpressionUUID->"99438918-f05b-4c0c-9875-cfc7cb0ac348",
 CellTags->"b:5.4.3",
 CellID->398925189]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1668012, 41305, 787, 21, 63, "Subsubsection",ExpressionUUID->"e67ab82c-2523-4967-9f99-ea6fd626de3b",
 CellTags->{"c:4", "b:5.4.3"},
 CellID->273437630],
Cell[1668802, 41328, 217, 6, 36, "Text",ExpressionUUID->"a6a36d51-1715-443f-a01f-593e1213aeb8",
 CellTags->"b:5.4.3",
 CellID->51242830],
Cell[CellGroupData[{
Cell[1669044, 41338, 2057, 67, 144, "Input",ExpressionUUID->"b1a8498e-acc3-4a89-9bf4-91237a84963e",
 CellTags->"b:5.4.3",
 CellID->97301363],
Cell[1671104, 41407, 884, 29, 49, "Output",ExpressionUUID->"fdf20322-e646-4d7f-8fa6-9fd722a73031",
 CellTags->"b:5.4.3",
 CellID->291705589],
Cell[1671991, 41438, 409, 13, 49, "Output",ExpressionUUID->"bc2897c3-bcb6-401b-8e06-ed3010031d92",
 CellTags->"b:5.4.3",
 CellID->2138151869],
Cell[1672403, 41453, 409, 13, 49, "Output",ExpressionUUID->"2a1d84da-f58f-4610-b135-ce15c846b56c",
 CellTags->"b:5.4.3",
 CellID->2074597032],
Cell[1672815, 41468, 409, 13, 49, "Output",ExpressionUUID->"44cb1c0f-b233-4adb-bbae-74fee5dd4cff",
 CellTags->"b:5.4.3",
 CellID->1207824132],
Cell[1673227, 41483, 409, 13, 49, "Output",ExpressionUUID->"77314bb1-32f5-4cfe-9785-e8073ccd8fec",
 CellTags->"b:5.4.3",
 CellID->1884755080]
}, Open  ]],
Cell[1673651, 41499, 220, 6, 36, "Text",ExpressionUUID->"cefebd86-28f5-4eab-aa1e-336911a2455c",
 CellTags->"b:5.4.3",
 CellID->1108952574],
Cell[CellGroupData[{
Cell[1673896, 41509, 1880, 60, 117, "Input",ExpressionUUID->"0dd26462-b5d4-49a5-bd3b-3efd653045a4",
 CellTags->"b:5.4.3",
 CellID->155338905],
Cell[1675779, 41571, 1110, 36, 62, "Output",ExpressionUUID->"25c9f823-d533-4774-8dc2-b9992be0d3ca",
 CellTags->"b:5.4.3",
 CellID->1694836310],
Cell[1676892, 41609, 1097, 36, 62, "Output",ExpressionUUID->"778386cc-ed4d-4470-89a4-d6d665655bb5",
 CellTags->"b:5.4.3",
 CellID->890768362],
Cell[1677992, 41647, 1096, 36, 62, "Output",ExpressionUUID->"24584b6d-4627-4b7e-ab26-58b11e34eb84",
 CellTags->"b:5.4.3",
 CellID->1035356897],
Cell[1679091, 41685, 1109, 36, 62, "Output",ExpressionUUID->"3a0df6d0-c78c-4a9b-9955-616079962c4e",
 CellTags->"b:5.4.3",
 CellID->183376238]
}, Open  ]],
Cell[1680215, 41724, 245, 6, 36, "Text",ExpressionUUID->"da296afa-186b-460c-a66e-e6358a9e2da4",
 CellTags->"b:5.4.3",
 CellID->1588345230],
Cell[1680463, 41732, 834, 25, 36, "Input",ExpressionUUID->"f6a27bca-82b5-4d86-9714-2f3a1d287da8",
 CellTags->"b:5.4.3",
 CellID->2096548400],
Cell[CellGroupData[{
Cell[1681322, 41761, 656, 17, 35, "Input",ExpressionUUID->"37e9de03-3d22-4a96-8b16-c9c988b538b5",
 CellTags->"b:5.4.3",
 CellID->777278824],
Cell[1681981, 41780, 5341, 110, 203, "Output",ExpressionUUID->"8741ff0e-3ed6-4189-a4f2-cc9a0f588e22",
 CellTags->"b:5.4.3",
 CellID->365398621]
}, Open  ]],
Cell[CellGroupData[{
Cell[1687359, 41895, 655, 17, 35, "Input",ExpressionUUID->"c2d3bae5-6c6b-48c1-a331-7d853d4ac2b6",
 CellTags->"b:5.4.3",
 CellID->582170267],
Cell[1688017, 41914, 5344, 110, 203, "Output",ExpressionUUID->"3e89a63c-8cb4-48da-8aa1-af26d97f52ed",
 CellTags->"b:5.4.3",
 CellID->1598876374]
}, Open  ]],
Cell[CellGroupData[{
Cell[1693398, 42029, 659, 17, 35, "Input",ExpressionUUID->"ef6cb6ed-a00c-4c27-9cc5-ec26a51b165f",
 CellTags->"b:5.4.3",
 CellID->600181737],
Cell[1694060, 42048, 5341, 110, 203, "Output",ExpressionUUID->"ab5f7a73-f80e-4460-bec2-4ffd77669feb",
 CellTags->"b:5.4.3",
 CellID->852342907]
}, Open  ]],
Cell[CellGroupData[{
Cell[1699438, 42163, 655, 17, 35, "Input",ExpressionUUID->"0a0181f7-446e-4a60-8f6c-ffb24714b81e",
 CellTags->"b:5.4.3",
 CellID->523763800],
Cell[1700096, 42182, 5342, 110, 203, "Output",ExpressionUUID->"0f75bffc-8a28-4ac9-a481-a364356055c1",
 CellTags->"b:5.4.3",
 CellID->1611312459]
}, Open  ]],
Cell[1705453, 42295, 239, 6, 36, "Text",ExpressionUUID->"9fd76b76-27ba-4378-899b-dc012d6a4464",
 CellTags->"b:5.4.3",
 CellID->362717547],
Cell[1705695, 42303, 224, 6, 36, "Text",ExpressionUUID->"341f08e1-f1d6-47d4-9ece-4c66acef8901",
 CellTags->"b:5.4.3",
 CellID->1635222243],
Cell[CellGroupData[{
Cell[1705944, 42313, 807, 26, 62, "Input",ExpressionUUID->"30bc5f39-b9dd-4fa0-a3bd-7d1bb4589e5b",
 CellTags->"b:5.4.3",
 CellID->1832007442],
Cell[1706754, 42341, 696, 21, 65, "Output",ExpressionUUID->"7ee00b8f-0656-42b1-b937-d3bc166554c8",
 CellTags->"b:5.4.3",
 CellID->1559709743],
Cell[1707453, 42364, 698, 22, 65, "Output",ExpressionUUID->"1ad27cd9-e285-4e1b-b583-2577121083a1",
 CellTags->"b:5.4.3",
 CellID->1856255319]
}, Open  ]],
Cell[1708166, 42389, 234, 6, 36, "Text",ExpressionUUID->"db46e971-d383-4dd1-8ed6-843d10cac042",
 CellTags->"b:5.4.3",
 CellID->2076030655],
Cell[CellGroupData[{
Cell[1708425, 42399, 404, 10, 61, "Input",ExpressionUUID->"03277937-9bb1-4988-bc4c-4afddb197358",
 CellTags->"b:5.4.3",
 CellID->1680405776],
Cell[1708832, 42411, 254, 7, 34, "Output",ExpressionUUID->"79d69b1b-05f0-49b0-85b6-ae2f72cb87e5",
 CellTags->"b:5.4.3",
 CellID->1588206464],
Cell[1709089, 42420, 277, 8, 49, "Output",ExpressionUUID->"44a01e06-e0f0-428b-9690-013da4988e81",
 CellTags->"b:5.4.3",
 CellID->1911791885]
}, Open  ]],
Cell[1709381, 42431, 224, 6, 36, "Text",ExpressionUUID->"867a4f3e-0dda-4297-b8e0-1ca0f04c975e",
 CellTags->"b:5.4.3",
 CellID->1022435535],
Cell[CellGroupData[{
Cell[1709630, 42441, 875, 27, 62, "Input",ExpressionUUID->"f67fdb61-1dfb-46ae-9a07-840a31d6379e",
 CellTags->"b:5.4.3",
 CellID->2057377098],
Cell[1710508, 42470, 695, 21, 65, "Output",ExpressionUUID->"fd2ef314-c7f2-44b7-bda8-d8b6e2bfcc88",
 CellTags->"b:5.4.3",
 CellID->835884781],
Cell[1711206, 42493, 699, 22, 65, "Output",ExpressionUUID->"ca665d5a-7bf5-4c09-8acc-59f681ed7af9",
 CellTags->"b:5.4.3",
 CellID->2024979240]
}, Open  ]],
Cell[1711920, 42518, 234, 6, 36, "Text",ExpressionUUID->"60a190fa-0f41-492a-a4b0-d9b14ff31f86",
 CellTags->"b:5.4.3",
 CellID->1702685739],
Cell[CellGroupData[{
Cell[1712179, 42528, 404, 10, 61, "Input",ExpressionUUID->"6d69bb75-b418-4a88-807c-ef64189740d2",
 CellTags->"b:5.4.3",
 CellID->1819619098],
Cell[1712586, 42540, 253, 7, 34, "Output",ExpressionUUID->"106d5702-30a6-4174-821d-be14be182ea7",
 CellTags->"b:5.4.3",
 CellID->505249024],
Cell[1712842, 42549, 274, 8, 49, "Output",ExpressionUUID->"9c858dd8-9c76-49f5-a131-c4a9b34bd516",
 CellTags->"b:5.4.3",
 CellID->570006130]
}, Open  ]],
Cell[1713131, 42560, 228, 6, 36, "Text",ExpressionUUID->"837193cc-8743-4550-9e3c-3db6751fb592",
 CellTags->"b:5.4.3",
 CellID->23563159],
Cell[CellGroupData[{
Cell[1713384, 42570, 464, 14, 61, "Input",ExpressionUUID->"d94cc086-6829-439a-8eb8-cfe17ac3436c",
 CellTags->"b:5.4.3",
 CellID->889022265],
Cell[1713851, 42586, 737, 23, 65, "Output",ExpressionUUID->"16513776-9138-4a28-8896-c11073c60b07",
 CellTags->"b:5.4.3",
 CellID->168477643],
Cell[1714591, 42611, 738, 23, 65, "Output",ExpressionUUID->"ae96a3f4-e55c-4c22-a04f-e72aeac07731",
 CellTags->"b:5.4.3",
 CellID->1220299155]
}, Open  ]],
Cell[1715344, 42637, 251, 6, 36, "Text",ExpressionUUID->"7c7b7e20-0feb-4575-86b1-a2b18eb01871",
 CellTags->"b:5.4.3",
 CellID->1753475676],
Cell[CellGroupData[{
Cell[1715620, 42647, 650, 21, 36, "Input",ExpressionUUID->"17b1bf7f-cbc3-4e01-9628-88d401078e5f",
 CellTags->"b:5.4.3",
 CellID->747598751],
Cell[1716273, 42670, 1054, 34, 115, "Output",ExpressionUUID->"f08a5cd2-83ab-427c-b227-0bbbc7fd8f1d",
 CellTags->"b:5.4.3",
 CellID->1099069235]
}, Open  ]],
Cell[CellGroupData[{
Cell[1717364, 42709, 533, 15, 110, "Input",ExpressionUUID->"3cb795ea-91c1-4823-bd2b-d9e192062301",
 CellTags->"b:5.4.3",
 CellID->2112873107],
Cell[1717900, 42726, 309, 9, 49, "Output",ExpressionUUID->"b53802d1-e421-495a-b1ea-95b2e4a2fdb9",
 CellTags->"b:5.4.3",
 CellID->1549784798],
Cell[1718212, 42737, 306, 9, 49, "Output",ExpressionUUID->"1e4fd516-1ab2-4abb-81cd-047e6d4dd910",
 CellTags->"b:5.4.3",
 CellID->403945523],
Cell[1718521, 42748, 306, 9, 49, "Output",ExpressionUUID->"2ffc1540-7700-4c76-bb68-d45039251f7e",
 CellTags->"b:5.4.3",
 CellID->916596894],
Cell[1718830, 42759, 384, 12, 49, "Output",ExpressionUUID->"5d948752-e5e0-4e09-a783-c7a9383747aa",
 CellTags->"b:5.4.3",
 CellID->1343797527]
}, Open  ]],
Cell[CellGroupData[{
Cell[1719251, 42776, 1292, 41, 35, "Input",ExpressionUUID->"c13bdabe-e2ad-453f-90b2-641b5341c13c",
 CellTags->"b:5.4.3",
 CellID->268722557],
Cell[1720546, 42819, 773, 24, 52, "Output",ExpressionUUID->"bfad8327-6fa6-4cc2-ad29-f654e5519f14",
 CellTags->"b:5.4.3",
 CellID->119520790]
}, Open  ]],
Cell[CellGroupData[{
Cell[1721356, 42848, 538, 15, 35, "Input",ExpressionUUID->"4922c4a4-3628-4490-8f52-3df4f5f87976",
 CellTags->"b:5.4.3",
 CellID->261905858],
Cell[1721897, 42865, 5340, 110, 203, "Output",ExpressionUUID->"e4975683-94f7-451b-a862-68c0854edbde",
 CellTags->"b:5.4.3",
 CellID->888707585]
}, Open  ]],
Cell[CellGroupData[{
Cell[1727274, 42980, 412, 13, 35, "Input",ExpressionUUID->"157fa8d6-6666-4815-abe4-4537b24f18d9",
 CellTags->"b:5.4.3",
 CellID->1657762703],
Cell[1727689, 42995, 255, 7, 34, "Output",ExpressionUUID->"af262b2c-d126-4a5e-9bf4-c7d31f22ca3b",
 CellTags->"b:5.4.3",
 CellID->759616210]
}, Open  ]]
}, Open  ]],
Cell[1727971, 43006, 301, 6, 30, "Input",ExpressionUUID->"f6773634-32c5-4b23-a0ec-22e6bce09cee",
 CellID->528483340],
Cell[1728275, 43014, 998, 19, 87, "Abstract",ExpressionUUID->"353d10cc-4a82-43f8-b397-b7e93b5cf08e",
 CellID->1524764736],
Cell[1729276, 43035, 572, 12, 72, "Section",ExpressionUUID->"b1943aa5-2ef2-4e61-86db-560a54a28f50",
 CellID->1275267708],
Cell[CellGroupData[{
Cell[1729873, 43051, 346, 8, 38, "Section",ExpressionUUID->"00c78d83-2b0c-4e94-8651-abe28f170866",
 CellID->802582215],
Cell[1730222, 43061, 2514, 39, 303, "Abstract",ExpressionUUID->"f515fd47-13e4-4c5b-a3bc-ebf91bb5d762",
 CellID->1694717084]
}, Open  ]],
Cell[1732751, 43103, 721, 14, 72, "Section",ExpressionUUID->"a6fcca51-d1a0-41b5-ac8e-03dcd2c9b47d",
 CellID->224028014],
Cell[1733475, 43119, 174, 2, 53, "Subtitle",ExpressionUUID->"9a660cc0-d636-4f83-b3cf-9c42710a9fd8",
 CellID->1670161643],
Cell[1733652, 43123, 4604, 117, 420, "Text",ExpressionUUID->"b7b99083-bfb3-4041-89c9-cf75413513ee",
 CellID->320828263],
Cell[1738259, 43242, 785, 21, 53, "Subtitle",ExpressionUUID->"35700604-5db4-4071-8960-66417a5ddc7e",
 CellID->809174117],
Cell[1739047, 43265, 1448, 31, 211, "Text",ExpressionUUID->"82238a1a-a31e-4fd1-b73d-46139cc45cf7",
 CellID->863742561],
Cell[1740498, 43298, 549, 11, 53, "Subtitle",ExpressionUUID->"d8da1847-7d4e-4dd5-9c31-606156ecb3f8",
 CellID->548490404],
Cell[1741050, 43311, 623, 13, 72, "Section",ExpressionUUID->"e695390e-046f-49f1-871a-005a68ccb899",
 CellID->952766083],
Cell[1741676, 43326, 468, 10, 38, "Section",ExpressionUUID->"3b433320-8a06-4fab-b769-2d5df6f394a8",
 CellID->1501484949],
Cell[1742147, 43338, 2643, 58, 679, "Abstract",ExpressionUUID->"f90e3e77-6240-4375-bf66-1c55eb753dc8",
 CellID->257579753],
Cell[1744793, 43398, 725, 14, 72, "Section",ExpressionUUID->"2b2a631f-3117-4a6c-82be-10fb33ebf0a5",
 CellID->2026474339],
Cell[1745521, 43414, 398, 9, 38, "Section",ExpressionUUID->"27fdc772-b883-4e32-bd3e-f8c7e567b5c5",
 CellID->2011057800],
Cell[1745922, 43425, 2264, 40, 151, "Abstract",ExpressionUUID->"846406fa-1bc6-46b3-8c6e-62ec672045e1",
 CellID->1757444899]
}
]
*)

