(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    872797,      19148]
NotebookOptionsPosition[    772843,      17272]
NotebookOutlinePosition[    826368,      18237]
CellTagsIndexPosition[    824471,      18165]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell["Shor's factoring algorithms", "Title",
 CellTags->"c:1",ExpressionUUID->"5cbfd1a9-b24b-490f-b89a-0154cd6bb247"],

Cell["Four examples of Shor's factoring algorithm", "Subtitle",ExpressionUUID->"45086d7e-0eb3-4dad-8435-5a2a00a2c3ac"],

Cell[TextData[{
 "Version: 4.0-July-2014--For ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 9.0,10."
}], "Abstract",
 CellChangeTimes->{{3.395090887981901*^9, 3.395090925377634*^9}, {
   3.512204933064251*^9, 3.5122049705968847`*^9}, 3.5122120029978447`*^9, {
   3.614175844992116*^9, 
   3.614175864474527*^9}},ExpressionUUID->"0c762211-3ba8-4afc-aaf2-\
2ab77f473a68"],

Cell[CellGroupData[{

Cell["\<\
Four examples of Shor's factoring algorithm                        To open \
double click at down arrow---------------------------->>\
\>", "Section",
 CellChangeTimes->{{3.512205126506694*^9, 3.51220512963229*^9}},
 FontFamily->"Times",
 CellTags->{
  "c:2", "b:3.0"},ExpressionUUID->"907f866b-f1f1-4a7e-9031-4b4cff6a52be"],

Cell[TextData[{
 "Last Version: 18-Aug-2005\n",
 Cell[BoxData[
  FormBox[GridBox[{
     {
      RowBox[{"References", ":"}], 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"[", "1", "]"}], " ", "Peter", " ", 
         RowBox[{"W", ".", " ", "Shor"}]}], ",", " ", 
        RowBox[{"SIAM", " ", 
         RowBox[{"J", ".", " ", "COMPUT"}], " ", "26", " ", 
         RowBox[{"(", "5", ")"}], " ", "1484", " ", 
         RowBox[{"(", "1997", ")"}]}]}], "\n", 
       RowBox[{
        RowBox[{"[", "2", "]"}], " ", "Edward", " ", "Gerjuoy", " ", 
        RowBox[{"Am", ".", " ", "J", ".", " ", "Phys", ".", " ", "73"}], " ", 
        RowBox[{"(", "6", ")"}], " ", "521", " ", 
        RowBox[{"(", "2005", ")"}]}], "\n", 
       RowBox[{
        RowBox[{
         RowBox[{"[", "3", "]"}], " ", "QDENSITY", " ", "Manual"}], ",", " ", 
        RowBox[{"Sect", ".", " ", "9"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
      "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}], TraditionalForm]], 
  "DefinitionBox",
  GridBoxOptions->{
  GridBoxDividers->{
   "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
    "RowsIndexed" -> {}},
  GridBoxSpacings->{"Columns" -> {
      Offset[0.27999999999999997`], {
       Offset[0.5599999999999999]}, 
      Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
      Offset[0.2], {
       Offset[0.16]}, 
      Offset[0.2]}, "RowsIndexed" -> {}}},
  CellTags->"S5.1.1",ExpressionUUID->"aed5570f-2ed5-4998-a532-b2c9e039e344"],
 "\n\nWe present four different implementations of the factoring algorithm. \
The differences between them arise from the method used to perform the \
Quantum Fourier Transform. All of them follow essentially the same steps and \
mostly share the same comments and explanations.\n\n(1) The first case is \
closer to an experiment implementation, using a density matrix, where the QFT \
is explicitly constructed quantum mechanically using Hadamards and Controlled \
gates. \n\n(2) In the second case, a state vector formulation is used , \
instead of a density matrix, to compute the QFT.  This approach saves space, \
as explained in ",
 ButtonBox["QFT.nb",
  BaseStyle->"Hyperlink",
  ButtonData:>{"QFT.nb", None}],
 ". \n\n(3) The third case, is based on the second approach but also \
explictly constructs the state vectors for both registers, and uses the state \
vector formula for QFT.\n\n(4) Finally, if one is interested in analyzing the \
probabilities for factoring larger numbers, and not in a voracious, closer to \
reality implementation of the QFT, then use this version which replaces QFT \
by  Fourier and InverseFourier, the built-in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for Discreet Fourier Transforms (DFT), check ",
 ButtonBox["QFT.nb",
  BaseStyle->"Hyperlink",
  ButtonData:>{"QFT.nb", None}],
 "."
}], "Abstract",
 CellTags->"b:3.0",ExpressionUUID->"900ba9e6-ee5f-439e-82e3-dc68971e5335"],

Cell[TextData[ButtonBox["Click here to get the 2014 palette: QDENSpalette.nb",
 BaseStyle->"Hyperlink",
 ButtonData:>{"QDENSpalette14.nb", None}]], "Text",
 CellTags->"b:3.0",ExpressionUUID->"06a8d43b-f285-4c75-ae6f-593ebe1e7985"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shor's Factoring algorithm using density matrix", "Section",
 FontFamily->"Times",
 CellTags->"c:2",ExpressionUUID->"04b471a7-d63e-4fd6-bc85-119c0edac590"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and NumberTheory\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:3", "b:3.1.0"},ExpressionUUID->"88f3631c-804c-4474-babf-cd4d489e78da"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"General", "::", "shdw"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]}], "Input",
 CellChangeTimes->{{3.860156788886376*^9, 3.8601567902998323`*^9}},
 FontFamily->"Courier New",
 CellTags->"b:3.1.0",ExpressionUUID->"b4eb02b7-74b4-442d-936a-abbd145f01f5"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{
  3.857208199005001*^9, {3.857208265533367*^9, 3.857208293069716*^9}, 
   3.857208332341462*^9, 3.8593722817759457`*^9, 3.860156796114237*^9},
 CellTags->"b:3.1.0",ExpressionUUID->"65d05b40-19dc-4dd1-8ed4-49b41aeb2f2c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.1.0",ExpressionUUID->"649e5966-1fc3-4e8b-b424-4b2d1b82b726"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["about", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "about", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["FidelityB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "FidelityB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["OneOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "OneOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "res2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["U", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "U", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Adj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Adj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Oracle", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Oracle", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "res2$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "v", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["AF", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "AF", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTrace", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PartialTrace", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Rotqbit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Rotqbit", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Vs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Vs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Bra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Bra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Hadamard", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Hadamard", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTraceGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PartialTraceGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RotX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "v$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "BraV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["HALL", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "HALL", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTranspose", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PartialTranspose", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RotY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ws", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ws", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "BraX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["i8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "i8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ProjG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ProjG", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RotZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "BraY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["intro", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "intro", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CNOT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "CNOT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["j8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "j8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTrGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PTrGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Schmidt", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Schmidt", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Coeff", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Coeff", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Ket1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Ket1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Purity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Purity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma0", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Sigma0", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Comm", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Comm", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "KetV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Sigma1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroMatrix", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ZeroMatrix", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompMeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "CompMeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "KetX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "PY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Sigma2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ZeroT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompPTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "CompPTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "KetY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomBra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma3", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Sigma3", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalH]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ControlledX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "mc", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomBra2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Snos", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Snos", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalP]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ControlledY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "mcl", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBraN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomBraN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SP", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "SP", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalP]X", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[ScriptCapitalP]X", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ControlledZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "mcl$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomKet1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SpinObs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "SpinObs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalP]Y", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[ScriptCapitalP]Y", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASE", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "CPHASE", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "mc$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomKet2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stA", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "stA", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[DoubleStruckCapitalI]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASEV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "CPHASEV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "MeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKetN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomKetN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "stB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Rho]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["cRQN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "cRQN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiProjCBasis", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "MultiProjCBasis", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomQubit1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Swap", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Swap", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Rho]$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Definef", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Definef", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiQProj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "MultiQProj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomQubit2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "s$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Sigma]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Sigma]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["DForm2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "DForm2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Mutual", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Mutual", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubitN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "RandomQubitN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ThreeOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ThreeOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Phi]A1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Discord", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Discord", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "nb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsDate", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "rcsDate", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Toffoli", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Toffoli", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Phi]A2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "EnTropy", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["neg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "neg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsRevision", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "rcsRevision", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ts", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Phi]B1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "EnTropy2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nneg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "nneg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "res1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "ts$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[Phi]B2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Fidelity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "Fidelity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nq", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "nq", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "res1$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["TwoOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "TwoOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[CapitalOmega]s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-aec6ddd8-6efe-409e-8f8b-a09ec2187375", {
                    "\[CapitalOmega]s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.19]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.85720819965206*^9, {3.857208266415018*^9, 3.857208293688032*^9}, 
   3.8572083331066523`*^9, 3.8593722826201477`*^9, 3.860156797060329*^9},
 CellTags->"b:3.1.0",ExpressionUUID->"13adc82c-aea0-47cb-8afb-8300adf6340b"]
}, Open  ]],

Cell["\<\
To improve performance we precompile the quantum Fourier transform.  We \
prepare some utilities first, for alter application:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.0",ExpressionUUID->"2f36eb48-6974-4bac-b28c-d45b8fef80e2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ControlP", "[", 
    RowBox[{"L_", ",", "q1_", ",", "q2_", ",", "k_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "\[Phi]", "}"}], ",", 
     RowBox[{
      RowBox[{"\[Phi]", "=", 
       RowBox[{"Exp", "[", 
        RowBox[{"2", "*", " ", "\[Pi]", " ", "*", " ", 
         RowBox[{"\[ImaginaryI]", " ", "/", 
          SuperscriptBox["2", "k"]}]}], "]"}]}], ";", 
      RowBox[{
       RowBox[{"1", "/", "4"}], "*", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"3", "+", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "0", ",", "0"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "3", ",", "0"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", "\[Phi]"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "0", ",", "3"}], "]"}]}], 
         "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"\[Phi]", "-", "1"}], ")"}], "*", 
          RowBox[{"TwoOp", "[", 
           RowBox[{"L", ",", "q1", ",", "q2", ",", "3", ",", "3"}], "]"}]}]}],
         ")"}]}]}]}], "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"dc00caca-95c7-42c7-9d5d-bd9e64252067"],

Cell[BoxData[
 RowBox[{
  RowBox[{"InvertQubits", "[", "L_", "]"}], ":=", 
  RowBox[{
   RowBox[{"InvertQubits", "[", "L", "]"}], "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Reverse", "[", 
         RowBox[{"IntegerDigits", "[", 
          RowBox[{"i", ",", "2", ",", "L"}], "]"}], "]"}], "\[Equal]", " ", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"j", ",", "2", ",", "L"}], "]"}]}], ",", "1", ",", "0"}], 
      "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "L"}], "-", "1"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", 
       RowBox[{
        RowBox[{"2", "^", "L"}], "-", "1"}]}], "}"}]}], "]"}]}]}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"8647f407-8468-4a92-8945-e6c6a324eeda"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompControlP", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q1", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"q2", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Lv", "=", "L"}], ",", 
         RowBox[{"qv1", "=", "q1"}], ",", 
         RowBox[{"qv2", "=", "q2"}], ",", 
         RowBox[{"kv", "=", "k"}]}], "  ", "}"}], ",", " ", 
       RowBox[{"ControlP", "[", 
        RowBox[{"Lv", ",", "qv1", ",", "qv2", ",", "kv"}], "]"}]}], "]"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"ControlP", "[", 
         RowBox[{"Lv", ",", "qv1", ",", "qv2", ",", "kv"}], "]"}], ",", 
        "_Complex", ",", "2"}], "}"}], "}"}]}], "]"}]}], " ", ";"}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"c4a5dd86-cdea-4873-8dc0-d922706225c0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompHad", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Lv", "=", "L"}], ",", 
         RowBox[{"iv", "=", "i"}]}], "}"}], ",", " ", 
       RowBox[{"had", "[", 
        RowBox[{"Lv", ",", "iv"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"had", "[", 
         RowBox[{"Lv", ",", "iv"}], "]"}], ",", "_Real", ",", "2"}], "}"}], 
      "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"ce7152e7-628f-4c48-b960-d5298267cd7e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompInvertQubits", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"L", ",", "_Integer"}], "}"}], "}"}], ",", 
     RowBox[{"Block", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Lv", "=", "L"}], " ", "}"}], ",", 
       RowBox[{"InvertQubits", "[", "Lv", "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "  ", 
       RowBox[{
        RowBox[{"InvertQubits", "[", "Lv", "]"}], ",", "_Real", ",", "2"}], 
       "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"759956d6-6ee7-44fe-bcc6-3e20155405fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumFourierTransform", "=", 
    RowBox[{"Compile", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"L", ",", "_Integer"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\[CapitalOmega]s", ",", "_Complex", ",", "2"}], "}"}]}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"Module", "[", 
       RowBox[{
        RowBox[{"{", "\[CapitalOmega]", "}"}], ",", 
        RowBox[{
         RowBox[{"\[CapitalOmega]", "=", "\[CapitalOmega]s"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"i", "=", "1"}], ",", 
           RowBox[{"i", "\[LessEqual]", "L"}], ",", 
           RowBox[{"i", "++"}], ",", 
           RowBox[{
            RowBox[{"\[CapitalOmega]", "=", 
             RowBox[{"N", "[", 
              RowBox[{
               RowBox[{"CompHad", "[", 
                RowBox[{"L", ",", "i"}], "]"}], ".", "\[CapitalOmega]"}], 
              "]"}]}], ";", 
            RowBox[{"For", "[", " ", 
             RowBox[{
              RowBox[{"j", "=", 
               RowBox[{"i", "+", "1"}]}], ",", 
              RowBox[{"j", "\[LessEqual]", "L"}], ",", 
              RowBox[{"j", "++"}], ",", 
              RowBox[{"\[CapitalOmega]", "=", 
               RowBox[{"N", "[", 
                RowBox[{
                 RowBox[{"CompControlP", "[", 
                  RowBox[{"L", ",", "j", ",", "i", ",", 
                   RowBox[{"j", "-", "i", "+", "1"}]}], "]"}], ".", 
                 "\[CapitalOmega]"}], "]"}]}]}], "]"}]}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"\[CapitalOmega]", "=", 
          RowBox[{
           RowBox[{"CompInvertQubits", "[", "L", "]"}], ".", 
           "\[CapitalOmega]"}]}], ";", "\[CapitalOmega]"}]}], "]"}], ",", " ",
       "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompHad", "[", 
           RowBox[{"L", ",", "i"}], "]"}], ",", "_Real", ",", "2"}], "}"}], 
        ",", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompControlP", "[", 
           RowBox[{"L", ",", "j", ",", "i", ",", 
            RowBox[{"j", "-", "i", "+", "1"}]}], "]"}], ",", "_Complex", ",", 
          "2"}], "}"}], ",", 
        RowBox[{"{", " ", 
         RowBox[{
          RowBox[{"CompInvertQubits", "[", "L", "]"}], ",", "_Real", ",", 
          "2"}], "}"}]}], "}"}]}], " ", "]"}]}], ";"}], "  "}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"a2860cee-cde7-4626-9571-dde99c327565"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.0",ExpressionUUID->"587831e8-2e6d-4f5b-b757-c8ee1cc11f0f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"7b386c71-97b5-4506-bbfb-5f46553f44f6"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.0",ExpressionUUID->"1a8cc137-fbd0-4251-af22-8c20742ba62b"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.1.0",ExpressionUUID->"25d86afd-2547-4bc4-b195-117c8216f2d6"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers needed\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:4", "b:3.1.1"},ExpressionUUID->"c8df16bb-759e-4dae-8fd6-5a5adba1a925"],

Cell["M is the number we are trying to factor:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1",ExpressionUUID->"ac5ac293-60fa-4e41-a6a7-2a2754b4b4f4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"M", "=", "21"}], ";"}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"e8d6f5b9-86f7-4959-aec4-1db9f7599b91"],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1",ExpressionUUID->"46b63a6c-9ce5-4e06-ba8b-9f7d176a0445"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"7f83aa48-6ba0-4ff1-b9cc-498239cdba80"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime, in which case we quit the process:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1",ExpressionUUID->"b1a5a53a-6d02-49f9-9012-fbbc29a58a70"],

Cell[BoxData[
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
     RowBox[{"Even", "=", "True"}], ";", 
     RowBox[{"MP", "=", 
      RowBox[{"MP", "/", "2"}]}]}]}], "]"}], ";"}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"49d10476-5555-4aa0-8ed9-45e1130ff675"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"MP", "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"M", ",", " ", "\"\< is a power of 2\>\""}], "]"}], ";", 
    RowBox[{"Quit", "[", "]"}]}], ","}], "]"}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"79e53433-c843-4cbd-b058-d683850990fc"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"cd8301cc-cbbd-475f-9136-1efe9f1fd9f7"],

Cell[TextData[{
 " We need the ",
 StyleBox["minimun number",
  FontWeight->"Bold"],
 " of qubits for both registers. \nHere we should be careful, a high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "8c439326-c8d0-49b7-b5ea-bb91f28c5fd3"],
 " number will lengthen the computation.  \nBy default we give estimates for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "c7e2c213-7f01-4430-a9c0-f6be7d216e8c"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "443bda0a-29a8-46d7-a3ce-0c80813d5be1"],
 ",based on the fact that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "c4b54b2e-ec7e-42fe-84af-d2023a7147b9"],
 " should be able to \nrepresent numbers up to M^2, while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "ac1332b3-76b4-47d5-8373-4053507cbaa9"],
 "should allow to write up to M/2. However, in \nmany cases it could be more \
illustrative to lower the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "49039dcc-09bc-4f88-b656-d9997588e491"],
 " \n[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "85bc3927-4c47-4133-b13c-6dde16dbafe8"],
 "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1",ExpressionUUID->"44059999-2373-4ce3-b6f4-56f37ed4449c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"c07d56b5-fa69-45a1-992e-30de76f52e72"],

Cell[BoxData[
 TagBox["512",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.8572081997727213`*^9, {3.857208266497488*^9, 3.857208293762598*^9}, 
   3.857208333183464*^9, 3.859372282712646*^9, 3.860156797155157*^9},
 CellTags->"b:3.1.1",ExpressionUUID->"3ea2b04e-4659-4e7a-9860-b83ea910a039"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["n", "1"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["n", "2"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["n", "1"], "=", "6"}], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"5244f1fd-21cc-4fd8-a168-14c9d5454f04"],

Cell[BoxData[
 TagBox["6",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.8572081997891893`*^9, {3.857208266514208*^9, 3.857208293820278*^9}, 
   3.857208333200811*^9, 3.8593722827289352`*^9, 3.8601567971722097`*^9},
 CellTags->"b:3.1.1",ExpressionUUID->"102c44dd-e4f6-4291-af6f-37cf1941e5f8"],

Cell[BoxData[
 TagBox["5",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.8572081997891893`*^9, {3.857208266514208*^9, 3.857208293820278*^9}, 
   3.857208333200811*^9, 3.8593722827289352`*^9, 3.860156797175284*^9},
 CellTags->"b:3.1.1",ExpressionUUID->"ab6390af-9238-4326-b067-94c297a31aea"]
}, Open  ]],

Cell["\<\
Gerjuoy's test: a failure to pass this test  means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.1",ExpressionUUID->"56644221-7526-4d00-85ee-243218b9ca87"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.1.1",ExpressionUUID->"a0971beb-0a6d-481f-852b-b7812373b1f2"],

Cell[BoxData["\<\"CHECK YOUR \\!\\(\\*SubscriptBox[\\(n\\), \\(1\\)]\\), YOU \
MAY HAVE PROBLEMS GETTING THE PERIOD\"\>"], "Print",
 CellChangeTimes->{
  3.857208199799564*^9, {3.857208266525701*^9, 3.857208293831293*^9}, 
   3.857208333211504*^9, 3.859372282739317*^9, 3.8601567971859093`*^9},
 CellTags->"b:3.1.1",ExpressionUUID->"8041f7cd-1c19-4263-a9db-e2c5b5e52bae"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]],ExpressionUUID->
  "6dc7afc6-8c40-4857-9d6b-725136b9cc1f"],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:5", "b:3.1.2"},ExpressionUUID->"9f64068a-1716-41c1-9380-6ebcdb05a054"],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]],ExpressionUUID->
  "3fe1ff74-9a68-4693-850a-1586ea3f63bb"],
 " ; which is a \nstate where all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "8fdb8469-979c-48d4-a036-a08b88c36550"],
 " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.2",ExpressionUUID->"cb14f4d9-dac5-471b-9abf-cf40e32855ae"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]],ExpressionUUID->
  "010cc559-d3cb-4d2e-b5a8-c02ebdc7a07d"]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:6", "b:3.1.3"},ExpressionUUID->"686a6ce0-9161-4ca6-9cb2-35da78e8e4e4"],

Cell[" This is the random Coprime to MP: ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.3",ExpressionUUID->"4ed792cc-4499-4bdd-9595-a2f9c931b454"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.1.3",ExpressionUUID->"76cc33e8-bcd3-4ff3-ad27-e07e7a8a1618"],

Cell[BoxData[
 TagBox["17",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208199815468*^9, {3.857208266543095*^9, 3.857208293847069*^9}, 
   3.857208333228609*^9, 3.8593722827543983`*^9, 3.8601567972051277`*^9},
 CellTags->"b:3.1.3",ExpressionUUID->"b007c3ab-8b4a-420b-8861-0bcb7e80ed76"]
}, Open  ]],

Cell[TextData[{
 " The full wave function after we build the second register with  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]],ExpressionUUID->
  "2a51283f-3c83-49ae-9482-2db9d64fdf9c"],
 "reads:\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"|", 
         SubscriptBox["\[Psi]", "12"]}], "\[RightAngleBracket]"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{
           RowBox[{"-", 
            SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           SuperscriptBox["2", 
            SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
      "\[RightAngleBracket]"}], "|", 
     SubscriptBox["f", "j"]}], "  ", "\[RightAngleBracket]"}], 
   TraditionalForm]],ExpressionUUID->"90115d84-6e6e-4ca5-ac00-8ebdfc97e239"],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["xguess", "j"], 
     RowBox[{"(", 
      RowBox[{"mod", " ", "M"}], ")"}]}]}], TraditionalForm]],ExpressionUUID->
  "2837e763-7dc3-47a6-add2-400c6d0e7615"]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.3",ExpressionUUID->"a65dbdc9-e5b3-406d-b575-4a8c6c6ac5e1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:7", "b:3.1.4"},ExpressionUUID->"f1b39d7d-e749-4586-9c88-b559e8a6831e"],

Cell[TextData[{
 "We take an hypothetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one fo the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]],ExpressionUUID->
  "e6b0dd0b-16b9-4f30-9f13-6b1a8138212b"],
 "]"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4",ExpressionUUID->"2829ad68-1cbc-4d82-9aa0-9f58698439d8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"c22e121b-2b66-4028-bae9-0bdf25ef1fe3"],

Cell[BoxData[
 TagBox["5",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208199822935*^9, {3.857208266551256*^9, 3.857208293855008*^9}, 
   3.857208333236937*^9, 3.859372282762418*^9, 3.8601567972147503`*^9},
 CellTags->"b:3.1.4",ExpressionUUID->"71e0158b-0328-4cbc-a541-18b72debbd67"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SuperscriptBox["2", 
  SubscriptBox["n", "1"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  SubscriptBox["n", "2"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], ")"}]]}], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"15079ef9-8449-4a35-97ea-10a9a1703247"],

Cell[BoxData[
 TagBox["64",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208199839114*^9, {3.857208266568111*^9, 3.857208293871922*^9}, 
   3.857208333254527*^9, 3.859372282777516*^9, 3.860156797233396*^9},
 CellTags->"b:3.1.4",ExpressionUUID->"c453e302-e944-492e-95ea-b82213eb45bd"],

Cell[BoxData[
 TagBox["32",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208199839114*^9, {3.857208266568111*^9, 3.857208293871922*^9}, 
   3.857208333254527*^9, 3.859372282777516*^9, 3.8601567972366*^9},
 CellTags->"b:3.1.4",ExpressionUUID->"8b4f24f5-6186-4b24-aa87-9d3850622244"],

Cell[BoxData[
 TagBox["2048",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208199839114*^9, {3.857208266568111*^9, 3.857208293871922*^9}, 
   3.857208333254527*^9, 3.859372282777516*^9, 3.8601567972397137`*^9},
 CellTags->"b:3.1.4",ExpressionUUID->"71b521e3-4c00-4f4b-91fc-29a58ad27f60"]
}, Open  ]],

Cell[TextData[{
 "The next function gives the number in the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "a8601568-2ca8-433b-be64-f575e5f05e34"],
 "register in decimal basis from a given number i in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["n", "1"],
     TraditionalForm], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]],ExpressionUUID->
  "3be3aebf-5a62-43fe-b79d-7bb98ac3f848"],
 " bases."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4",ExpressionUUID->"060b2127-7e7c-4426-9aa7-9d061258bb9f"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xv", "[", "i_", "]"}], ":=", 
   RowBox[{"FromDigits", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}]}], "]"}], ",", 
       SubscriptBox["n", "1"]}], "]"}], ",", "2"}], "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"f3f296c0-1e48-474b-a107-66c839177611"],

Cell[TextData[{
 "We select the positions in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]],ExpressionUUID->
  "97dc92cd-32e5-4154-a8d0-f68965573115"],
 "register which remain when reg2 has been measured:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4",ExpressionUUID->"abd55984-1aed-4308-8dee-dc9fe9e708b9"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dens1", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Mod", "[", 
         RowBox[{
          RowBox[{"xguess", "^", 
           RowBox[{"xv", "[", "i", "]"}]}], ",", "MP"}], "]"}], "\[Equal]", 
        "reg2Measurement"}], ",", " ", 
       RowBox[{"dens1", "=", 
        RowBox[{"Join", "[", " ", 
         RowBox[{"dens1", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"xv", "[", "i", "]"}], "+", "1"}], " ", "}"}]}], "]"}]}], 
       ",", "\"\<\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        SuperscriptBox["2", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "1"], "+", 
           SubscriptBox["n", "2"]}], ")"}]], "-", "1"}], ",", 
       SuperscriptBox["2", 
        SubscriptBox["n", "2"]]}], " ", "}"}]}], "]"}], ";"}], 
  " "}]}], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"5601443d-d566-43af-ad75-45e13b6090fb"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4",ExpressionUUID->"4f931af5-cc57-41d9-9964-9a93cc1c44df"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"reg1", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"reg1", ",", "1", ",", 
      RowBox[{"dens1", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "dens1", "]"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"reg1", "/", 
    RowBox[{"Norm", "[", "reg1", "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"75aaafbf-5124-4626-bdfc-6089b808f357"],

Cell[BoxData[
 RowBox[{
  RowBox[{"densx", "=", 
   RowBox[{"Distribute", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"dens1", ",", "dens1"}], "}"}], ",", "List"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"4eeef5dc-3a2b-45c0-842b-bf2a695be73c"],

Cell["Share some memory.:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4",ExpressionUUID->"290722f7-0475-4a29-84d9-5ad1c7ebdec0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", " ", "]"}], ";"}]], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"aa14095f-36c7-4f3c-afc5-bd352551d47e"],

Cell["Prepere desnity matrix for reg1", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.4",ExpressionUUID->"aa262bf7-c9d3-430f-9d48-add5e451f579"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rho1", " ", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"m", ",", "_Integer", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "_Integer", ",", "2"}], "}"}]}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", "t", "}"}], ",", 
       RowBox[{
        RowBox[{"t", "=", "n"}], ";", " ", 
        RowBox[{"ReplacePart", "[", 
         RowBox[{"m", ",", "1", ",", "densx"}], "]"}]}]}], "]"}]}], "   ", 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{3.395094265479423*^9},
 CellTags->"b:3.1.4",ExpressionUUID->"827d4594-f283-42f4-a122-1f0f6f5c3d93"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]1", " ", "=", 
   RowBox[{
    FractionBox["1", 
     SqrtBox[
      RowBox[{"Length", "[", " ", "densx", "]"}]]], 
    RowBox[{"rho1", "[", 
     RowBox[{
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]], ",", 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]}], "}"}]}], "]"}], ",", "densx"}], 
     "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.395094076233127*^9, 3.3950940974912663`*^9}},
 CellTags->"b:3.1.4",ExpressionUUID->"16c2cfe1-4ab7-4500-a0b5-963e0addaaf7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", " ", "]"}], ";"}]], "Input",
 CellTags->"b:3.1.4",ExpressionUUID->"e3556bdb-cf40-4ad9-9be6-0a4e749eeb90"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:8", "b:3.1.5"},ExpressionUUID->"b52dae01-44a5-4675-9e09-dd23c9a6099a"],

Cell["We apply the inverse of the quantum fourier transform to \[Rho]1:", \
"Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.5",ExpressionUUID->"a3937fed-1b25-41f5-a01a-6dda38b9b91d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rho2", " ", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "_Complex", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"y", ",", "_Complex", ",", "2"}], "}"}]}], "}"}], ",", " ", 
     RowBox[{"N", "[", 
      RowBox[{
       RowBox[{"Conjugate", "[", "x", "]"}], ".", "y", ".", "x"}], "]"}]}], 
    "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.5",ExpressionUUID->"c46732cf-92e0-4ba9-9fcf-eefcc10bcace"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.5",ExpressionUUID->"b3e06932-15c6-494a-89e3-a0203cb336ac"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Rho]2", "=", " ", 
   RowBox[{"rho2", " ", "[", 
    RowBox[{
     RowBox[{"QuantumFourierTransform", "[", 
      RowBox[{
       SubscriptBox["n", "1"], ",", " ", 
       RowBox[{"N", "[", 
        RowBox[{"IdentityMatrix", "[", 
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "]"}], "]"}]}], "]"}], ",", " ", 
     "\[Rho]1"}], "  ", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.1.5",ExpressionUUID->"bbf072e9-dbcf-4f7b-9828-19ad0cbe65c8"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< seconds\>\""}], 
  "]"}]}], "Input",
 CellTags->"b:3.1.5",ExpressionUUID->"a4a9565e-a784-45be-b020-258073baaebf"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The QFT took \"\>", "\[InvisibleSpace]", 
   "1.563284`6.344552880795966", "\[InvisibleSpace]", "\<\" seconds\"\>"}],
  SequenceForm["The QFT took ", 1.563284`6.344552880795966, " seconds"],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.8572082006704082`*^9, {3.8572082682761993`*^9, 3.85720829467735*^9}, 
   3.857208336919264*^9, 3.859372285619142*^9, 3.860156799259974*^9},
 CellTags->"b:3.1.5",ExpressionUUID->"316b04d2-209e-4891-ae27-e4195f7eebd7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chop", "[", 
  RowBox[{"Tr", "[", "\[Rho]2", "]"}], "]"}]], "Input",
 CellTags->"b:3.1.5",ExpressionUUID->"212087b0-9807-4723-ba08-85686d31b9d7"],

Cell[BoxData[
 TagBox["0.9999999999999992`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.8572082006782017`*^9, {3.857208268301515*^9, 3.857208294684655*^9}, 
   3.857208337092476*^9, 3.859372285630561*^9, 3.860156799272553*^9},
 CellTags->"b:3.1.5",ExpressionUUID->"ad13fe6c-d084-4a9a-9a43-d8713161e7fe"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:9", "b:3.1.6"},ExpressionUUID->"289e9253-934d-4688-944c-597dffdb479f"],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.6",ExpressionUUID->"9157ff98-06d6-4397-9451-fefd9911320a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.1.6",ExpressionUUID->"9dd58b55-04a6-4b7a-9081-e0cd144b2197"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{"\[Rho]2", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"num", "+", "1"}], ",", 
         RowBox[{"num", "+", "1"}]}], "]"}], "]"}], "]"}]}], ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.1.6",ExpressionUUID->"a9453a43-185a-476c-a22a-0af6ebe086fb"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.6",ExpressionUUID->"f536a44b-7cde-488d-8153-a60b2a606cf8"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.3`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.1.6",ExpressionUUID->"b3d96486-217f-4f3c-9c15-ddca88a729b6"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], 
      LineBox[{{0.9026302117342296, 0.015851049491079044`}, {1., 
       0.0007057375947216827}, {2., 0.0007413467876771409}, {3., 
       0.0008070997807950861}, {4., 0.0009152913087920383}, {5., 
       0.001090700180602437}, {6., 0.0013864456170611754`}, {7., 
       0.0019324500467675127`}, {8., 0.003124999999999997}, {9., 
       0.006763862353264627}, {9.321359088567922, 0.015851049491079044`}}], 
      LineBox[{{11.901810187485902`, 0.015851049491079044`}, {12., 
       0.005334708691207957}, {13., 0.0011983420790088613`}, {14., 
       0.00033098368265917065`}, {15., 0.00006494141625510418}, {16., 0.}, {
       17., 0.0000649414162551048}, {18., 0.0003309836826591701}, {19., 
       0.0011983420790088626`}, {20., 0.005334708691207955}, {
       20.098189812514097`, 0.015851049491079044`}}], 
      LineBox[{{22.678640911432076`, 0.015851049491079044`}, {23., 
       0.0067638623532646274`}, {24., 0.003124999999999997}, {25., 
       0.0019324500467675148`}, {26., 0.0013864456170611765`}, {27., 
       0.0010907001806024358`}, {28., 0.000915291308792038}, {29., 
       0.0008070997807950858}, {30., 0.0007413467876771401}, {31., 
       0.0007057375947216823}, {31.09736978826577, 0.015851049491079044`}}], 
      LineBox[{{32.90263021173423, 0.015851049491079044`}, {33., 
       0.0007057375947216827}, {34., 0.0007413467876771409}, {35., 
       0.0008070997807950861}, {36., 0.0009152913087920383}, {37., 
       0.001090700180602437}, {38., 0.0013864456170611754`}, {39., 
       0.0019324500467675127`}, {40., 0.003124999999999997}, {41., 
       0.006763862353264627}, {41.321359088567924`, 0.015851049491079044`}}], 
      LineBox[{{43.9018101874859, 0.015851049491079044`}, {44., 
       0.005334708691207957}, {45., 0.0011983420790088613`}, {46., 
       0.00033098368265917065`}, {47., 0.00006494141625510418}, {48., 0.}, {
       49., 0.0000649414162551048}, {50., 0.0003309836826591701}, {51., 
       0.0011983420790088626`}, {52., 0.005334708691207955}, {
       52.0981898125141, 0.015851049491079044`}}], 
      LineBox[{{54.678640911432076`, 0.015851049491079044`}, {55., 
       0.0067638623532646274`}, {56., 0.003124999999999997}, {57., 
       0.0019324500467675148`}, {58., 0.0013864456170611765`}, {59., 
       0.0010907001806024358`}, {60., 0.000915291308792038}, {61., 
       0.0008070997807950858}, {62., 0.0007413467876771401}, {63., 
       0.0007057375947216823}}]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 63.}, {0, 0.015851049491079044`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208203723407*^9, {3.85720826854624*^9, 3.857208295669837*^9}, 
   3.857208337276299*^9, 3.859372285912191*^9, 3.860156799557703*^9},
 CellTags->"b:3.1.6",ExpressionUUID->"84d70b42-f76e-423a-9896-5db6403c5ba9"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd0d0rQ3Ecx/GTIq48pMxa2uQpLc+bYeyDPW9ny0PiYilFlDxGtNLvXxAX
XE65WG7OhVISFwoXu3FnykqKGxdzoVaWh+VHfb+/Op3enc45v9/3ZZpaHJ7O
UxQFP1fu/reyX7l16fittCNSfP1Wb/TIVqBlYvqU3yu7EBv2x93lFb/sEtju
Z6vyP1XZ5XCPX8ycpoZk63BkCx+aT8ZkG1BWaVryxidlG7Gzn1tzsqvxvBrb
nD+Oyq7B6O1C5DUdk12HjwdtL35zJrsB6lbi/S65JrsRV/ponvVgRLYZTzqt
IqtZZDdhO9lcVFtQKrsFClmtOCfP25Ak77cjQb7fgQD5vwUZsj8r238nXsj5
bOz8XTCQ+XRDI/PrAch87bCQ+fdinfj0IU78HJggvgDxFwD176f+op/5D1B/
McD8B6m/GGT+Tnp+4WT+Ljo/4WL+buov3MzfQ/2Fh/l7QfiFl/n7qL/wMX8/
9Rd+5h9g+w8w/yA7f5D5q9RfqMw/RP1FiPmHqb8I//t/A+FRFbo=
       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 63.}, {0., 0.3}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208203723407*^9, {3.85720826854624*^9, 3.857208295669837*^9}, 
   3.857208337276299*^9, 3.859372285912191*^9, 3.860156799599683*^9},
 CellTags->"b:3.1.6",ExpressionUUID->"e1b1e22f-7e81-4840-9e09-2d08b84e06c4"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:10", "b:3.1.7"},ExpressionUUID->"a28b1b85-2a1c-4bf8-a47f-2f7e35433a20"],

Cell[TextData[{
 "Finally we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["xguess", 
    RowBox[{"r", "/", "2"}]], TraditionalForm]],ExpressionUUID->
  "a88bc629-3006-4f22-838a-b3312e304cb9"],
 "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.7",ExpressionUUID->"f7c184aa-833a-4e13-941d-b0b1c1dbe8e1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.1.7",ExpressionUUID->"41a74ad2-09df-4a4b-bda4-53bc22a0aaba"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.1.7",ExpressionUUID->"09968d3f-b28f-4049-810d-5da4c38e3db0"],

Cell[CellGroupData[{

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<xguess=\>\"", ",", "xguess"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{"TotalProbSucc", "\[Equal]", "0"}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<Your total probability was zero, you xguess was not very lucky!\>\"",
     "]"}]}], "]"}]}], "Input",
 CellTags->"b:3.1.7",ExpressionUUID->"ee37e1b2-be7a-46db-ab35-9931b666d5a6"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"Period", "Number", "\<\"f1\"\>", "\<\"f2\"\>", "Prob"},
     {"\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208204048841*^9, {3.857208268624764*^9, 3.857208295890811*^9}, 
   3.857208337341692*^9, 3.859372285989891*^9, 3.860156799673843*^9},
 CellTags->"b:3.1.7",ExpressionUUID->"7ed0d11e-2edb-4fd6-b4a2-31a1d8b854de"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"xguess=\"\>", "\[InvisibleSpace]", "17"}],
  SequenceForm["xguess=", 17],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.857208204052853*^9, {3.857208268629245*^9, 3.8572082958951473`*^9}, 
   3.857208337345478*^9, 3.859372285993863*^9, 3.8601567996779413`*^9},
 CellTags->"b:3.1.7",ExpressionUUID->"e9c6c664-481c-44aa-a3b0-1349371fa0f6"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The TOTAL PROBABILITY OF SUCCESS is thus:\"\>", 
   "\[InvisibleSpace]", "0", "\[InvisibleSpace]", "\<\"  with xguess=\"\>", 
   "\[InvisibleSpace]", "17", 
   "\[InvisibleSpace]", "\<\" and measured register=\"\>", 
   "\[InvisibleSpace]", "5"}],
  SequenceForm[
  "The TOTAL PROBABILITY OF SUCCESS is thus:", 0, "  with xguess=", 17, 
   " and measured register=", 5],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.857208204052853*^9, {3.857208268629245*^9, 3.8572082958951473`*^9}, 
   3.857208337345478*^9, 3.859372285993863*^9, 3.8601567996816893`*^9},
 CellTags->"b:3.1.7",ExpressionUUID->"16ff7102-3cb5-45d0-9e1d-61136997fb88"],

Cell[BoxData["\<\"Your total probability was zero, you xguess was not very \
lucky!\"\>"], "Print",
 CellChangeTimes->{
  3.857208204052853*^9, {3.857208268629245*^9, 3.8572082958951473`*^9}, 
   3.857208337345478*^9, 3.859372285993863*^9, 3.860156799684299*^9},
 CellTags->"b:3.1.7",ExpressionUUID->"6c7d3dcf-ab09-45d7-ae46-52dbb7301474"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.1.7",ExpressionUUID->"6423670b-1dcd-4cd3-9d9d-4446fb731ced"],

Cell[BoxData[
 TagBox["5.975267`7.227902309649355",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.8572082040740957`*^9, {3.857208268654711*^9, 3.8572082959171247`*^9}, 
   3.857208337364182*^9, 3.859372286016108*^9, 3.860156799702476*^9},
 CellTags->"b:3.1.7",ExpressionUUID->"66aa6b4f-a1e5-4b71-a53d-4495d48bc65f"]
}, Open  ]],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.1.7",ExpressionUUID->"9c2ff836-91fa-4545-8eaa-5ebfacc6f941"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       SuperscriptBox["xguess", "i"], ",", "M"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "60"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"a", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:3.1.7",ExpressionUUID->"c4bf62c2-3d0b-4462-b554-73d07a5767ce"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdzEtqwmAYRuHQkUMHDjpwUEVEpEi1Wm9pE1sv9dZBlyA4dgtZskvQkv8U
/hMIL+cjeRqny+/5IUmS9P7+bflcs3L7eTiEfQlbCTsMW9XWtI957Nbz2H+S
35Tfkt+W35Hflf8svyefOz7/4Q/kv+o7OyPd3+SP5U/kT+XP5M/lp/Lf5X/I
z+TnsV/Q+IvYL2j8z9gvaPwv+TT+Uj6Nv5JP46/l0/gb+TT+t3wafyufxt/J
p/H38mn8g3wa/yifxv/592/S70V9
       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 60}, {0, 20.}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.8572082041085777`*^9, {3.85720826869492*^9, 3.857208295948264*^9}, 
   3.85720833739732*^9, 3.859372286061411*^9, 3.8601567997456284`*^9},
 CellTags->"b:3.1.7",ExpressionUUID->"25fcd2c7-95c8-4f6f-967f-004e3b149d70"]
}, Open  ]],

Cell["Quit", "Input",
 CellTags->"b:3.1.7",ExpressionUUID->"2cfdf17c-21c1-452f-b9e1-fc9ee72b3423"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Shor's Factoring algorithm using state vector formula for QFT\
\>", "Section",
 FontFamily->"Times",
 CellTags->"c:11",ExpressionUUID->"24d13847-d1ae-4f2b-bce4-15298cc8b739"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and NumberTheory\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:12", "b:3.2.0"},ExpressionUUID->"aac51844-15b4-485e-86df-82617cf00316"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.2.0",ExpressionUUID->"7ed690ea-76a8-4af3-a937-3d3353901106"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{3.857208355890478*^9},
 CellTags->"b:3.2.0",ExpressionUUID->"d8f96f63-fb5e-4053-a48d-87e2a44f804e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.2.0",ExpressionUUID->"809f814a-6ec4-42c1-9af2-c6571e4692f0"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["about", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "about", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["FidelityB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "FidelityB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["OneOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "OneOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "res2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["U", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "U", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Adj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Adj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Oracle", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Oracle", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "res2$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "v", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["AF", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "AF", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTrace", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PartialTrace", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Rotqbit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Rotqbit", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Vs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Vs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Bra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Bra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Hadamard", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Hadamard", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTraceGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PartialTraceGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RotX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "v$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "BraV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["HALL", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "HALL", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTranspose", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PartialTranspose", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RotY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ws", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ws", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "BraX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["i8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "i8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ProjG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ProjG", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RotZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "BraY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["intro", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "intro", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CNOT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "CNOT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["j8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "j8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTrGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PTrGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Schmidt", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Schmidt", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Coeff", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Coeff", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Ket1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Ket1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Purity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Purity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma0", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Sigma0", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Comm", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Comm", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "KetV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Sigma1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroMatrix", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ZeroMatrix", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompMeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "CompMeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "KetX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "PY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Sigma2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ZeroT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompPTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "CompPTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "KetY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomBra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma3", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Sigma3", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalH]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ControlledX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "mc", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomBra2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Snos", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Snos", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalP]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ControlledY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "mcl", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBraN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomBraN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SP", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "SP", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[DoubleStruckCapitalI]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ControlledZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "mcl$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomKet1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SpinObs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "SpinObs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Rho]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASE", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "CPHASE", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "mc$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomKet2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stA", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "stA", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Rho]$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASEV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "CPHASEV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "MeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKetN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomKetN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "stB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Sigma]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Sigma]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["cRQN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "cRQN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiProjCBasis", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "MultiProjCBasis", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomQubit1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Swap", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Swap", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Phi]A1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Definef", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Definef", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiQProj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "MultiQProj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomQubit2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "s$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Phi]A2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["DForm2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "DForm2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Mutual", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Mutual", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubitN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "RandomQubitN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ThreeOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ThreeOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Phi]B1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Discord", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Discord", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "nb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsDate", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "rcsDate", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Toffoli", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Toffoli", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[Phi]B2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "EnTropy", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["neg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "neg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsRevision", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "rcsRevision", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ts", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[CapitalOmega]s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "\[CapitalOmega]s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "EnTropy2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nneg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "nneg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "res1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "ts$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], ""},
                {
                 ButtonBox[
                  StyleBox["Fidelity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "Fidelity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nq", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "nq", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "res1$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["TwoOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-a1f3db48-92c8-4d4c-8c0c-25599282ded1", {
                    "TwoOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], ""}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.19]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356513267*^9},
 CellTags->"b:3.2.0",ExpressionUUID->"aa0e8521-6ff2-4dd6-9990-d558bb3b574b"]
}, Open  ]],

Cell["\<\
To improve performance we precompile the quantum Fourier transform, now using \
state vectors:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.0",ExpressionUUID->"43ec1d98-66a3-43cf-8fe2-cc272e5d61f6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Zeroff", "[", 
   RowBox[{"vac_", ",", "R_"}], "]"}], ":=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vac", "[", 
      RowBox[{"[", 
       RowBox[{"R", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "/", 
     RowBox[{"2", "^", "i"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "R", "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CompZeroff", " ", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"vac", ",", "_Integer", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"R", ",", "_Integer", ",", "1"}], "}"}]}], "}"}], ",", 
     RowBox[{"Zeroff", "[", 
      RowBox[{"vac", ",", "R"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Zeroff", "[", 
         RowBox[{"vac", ",", "R"}], "]"}], ",", "_Real"}], "}"}], "}"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellTags->"b:3.2.0",ExpressionUUID->"f0036fac-fccb-4bcb-85ee-d4ad0d046f54"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"QuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"CompZeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[CircleTimes]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.614176064807555*^9, 3.614176074581315*^9}, {
   3.625187402849154*^9, 3.625187407890999*^9}, {3.6251874843020487`*^9, 
   3.625187490287272*^9}, 3.8572081527791758`*^9},
 CellTags->"b:3.2.0",ExpressionUUID->"1e23594c-fde4-464e-ab54-8fd19efab08d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex", 
        ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.2.0",ExpressionUUID->"e8755a09-1536-4b1d-b029-ea5c5e6325e2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"IQuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"CompZeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{
            RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[CircleTimes]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.614176083784274*^9, 3.614176094504168*^9}, 
   3.8572081527829103`*^9},
 CellTags->"b:3.2.0",ExpressionUUID->"e9c23b01-eff7-4f7c-84fd-b9205f03704b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompIQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex", 
        ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.2.0",ExpressionUUID->"d0cdb5e1-88f8-4b06-a0b3-46073347374d"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.0",ExpressionUUID->"42ca929b-57a0-444b-bfc3-60fbee7b1ad3"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.2.0",ExpressionUUID->"e0c55fff-f275-4acb-8fbc-b33125148bf7"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.0",ExpressionUUID->"796b1850-1712-4faa-acf4-f31ca425d71e"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.2.0",ExpressionUUID->"188c8c18-4b35-4253-8788-146aa2bd355f"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:13", "b:3.2.1"},ExpressionUUID->"ee450305-874f-4775-a17d-d467161ffbb5"],

Cell["M is the number we are trying to factor:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1",ExpressionUUID->"1277f93f-4321-4bda-8ce6-e066ab62c620"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{"13", "*", "17"}]}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"0064d9bc-9c92-4916-a8f3-97442721d0a3"],

Cell[BoxData[
 TagBox["221",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572083565667467`*^9},
 CellTags->"b:3.2.1",ExpressionUUID->"b3e68a6c-f29f-4843-b05f-c7208a726d0b"]
}, Open  ]],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1",ExpressionUUID->"70df8bf4-3406-4418-8c1f-c07ada681540"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"39a392c8-853f-4f12-96c8-5d643445308d"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime and quit if it is:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1",ExpressionUUID->"cbb2b275-bd49-4ea6-9453-af1d54658d39"],

Cell[BoxData[
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
     RowBox[{"Even", "=", "True"}], ";", 
     RowBox[{"MP", "=", 
      RowBox[{"MP", "/", "2"}]}]}]}], "]"}], ";"}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"cf1f548b-177a-4cd7-9f1b-a78601360609"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"MP", "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"M", ",", " ", "\"\< is a power of 2\>\""}], "]"}], ";", 
    RowBox[{"Quit", "[", "]"}]}], ","}], "]"}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"5b676943-2e71-488f-8155-e16155f0ad7f"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"f18367b0-4334-422b-bb5d-d5fa993ea94b"],

Cell[TextData[{
 " We need the ",
 StyleBox["minimum",
  FontWeight->"Bold"],
 " ",
 StyleBox[" number",
  FontWeight->"Bold"],
 " of qubits for both registers. \nHere we should be careful, a high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "c4000fc4-d40b-4dce-b5bb-ad3114fb3a67"],
 " number will lengthen the computation.  \nBy default we give estimates for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "76cf0de9-d555-486a-9281-f32d23fe4793"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "da931194-c4e4-4d55-a07e-4effaeb066ff"],
 ",based on the fact that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "9859c915-1bbd-4dc4-a4f6-97bfcea7f222"],
 " should be able to \nrepresent numbers up to M^2, while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "94e527c4-d1e5-4f0b-807f-57a7c30bf5e5"],
 "should allow to write up to M/2. However,  in \nmany cases it could be more \
illustrative to lower the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "c566b227-a183-4818-a3be-cb0b8e3df3fb"],
 " \n[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "4c767054-da94-44b9-a426-3c2e6a2d14c0"],
 "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
 CellChangeTimes->{3.857208406292485*^9},
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1",ExpressionUUID->"05789577-c6ec-4b56-ade0-c770b29a92df"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"d6fb0edd-4b20-4244-a877-3dbf510cd72c"],

Cell[BoxData[
 TagBox["65536",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356629265*^9},
 CellTags->"b:3.2.1",ExpressionUUID->"daaa0c1e-6d1e-4492-bcf5-e20df6da9bcd"]
}, Open  ]],

Cell[TextData[{
 "Note the user should experiment with the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", 
    RowBox[{"1", "   "}]], TraditionalForm]],ExpressionUUID->
  "db9e349f-2ac1-4b98-878e-080201cab66f"],
 "to reduce the computation time, especially for large M."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1",ExpressionUUID->"335e8ea3-2873-44d9-b1a4-7a5b8c264732"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["n", "1"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["n", "2"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{" ", 
  RowBox[{
   SubscriptBox["n", "1"], "=", "9"}], " "}], "\[IndentingNewLine]", 
 SubscriptBox["n", "1"], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellChangeTimes->{{3.625187579649707*^9, 3.625187583890175*^9}, {
  3.625187644452126*^9, 3.625187649929905*^9}},
 CellTags->"b:3.2.1",ExpressionUUID->"f1cc2945-4e2e-4e68-9bf6-eb7aaa7e8756"],

Cell[BoxData[
 TagBox["9",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356646736*^9},
 CellTags->"b:3.2.1",ExpressionUUID->"33425887-852b-44b0-95cd-4f7a4942c583"],

Cell[BoxData[
 TagBox["9",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572083566505136`*^9},
 CellTags->"b:3.2.1",ExpressionUUID->"53aa7c81-2544-49c3-8218-82071356b912"],

Cell[BoxData[
 TagBox["8",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356652927*^9},
 CellTags->"b:3.2.1",ExpressionUUID->"10011bf6-248a-4a3c-a25d-36a6fa332b23"]
}, Open  ]],

Cell["\<\
Gerjuoy's test: a failure to pass this test  means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.1",ExpressionUUID->"eb6b992e-8347-460c-b83d-6b917f3d28f6"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.2.1",ExpressionUUID->"edab14a3-a909-4841-bbfb-7663f5a5a735"],

Cell[BoxData["\<\"CHECK YOUR \\!\\(\\*SubscriptBox[\\(n\\), \\(1\\)]\\), YOU \
MAY HAVE PROBLEMS GETTING THE PERIOD\"\>"], "Print",
 CellChangeTimes->{3.8572083566608953`*^9},
 CellTags->"b:3.2.1",ExpressionUUID->"42bc22b7-3e0a-46ed-b274-a20c434bec81"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]],ExpressionUUID->
  "633f6268-6071-4755-945b-e4c5a03a0c21"],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:14", "b:3.2.2"},ExpressionUUID->"b21b802a-e344-4045-9440-8754a9b40edf"],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]],ExpressionUUID->
  "ca32231e-e213-4cfc-a8a5-9e7105e00e7c"],
 " ; which is a \nstate where all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "0e8fe41e-b0ad-44c9-8345-eae3aed247bd"],
 " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicty construct it.  "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.2",ExpressionUUID->"2f7d1a0f-0728-490d-a43d-fdb912c228a9"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]],ExpressionUUID->
  "f849ef37-94f2-45a8-a4c5-854e67d57bba"]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:15", "b:3.2.3"},ExpressionUUID->"7eaa4595-dff0-4dc2-be10-dab949968968"],

Cell[" This is the random Coprime to MP: ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.3",ExpressionUUID->"553878d0-20c0-4d41-a2f1-b451599dbdfb"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.2.3",ExpressionUUID->"9f0e4179-0a52-4597-99dc-bf4b814f9208"],

Cell[BoxData[
 TagBox["121",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572083566778193`*^9},
 CellTags->"b:3.2.3",ExpressionUUID->"5921de34-eda2-4ae7-b705-17e1fc854e6f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", "16"}]], "Input",
 CellTags->"b:3.2.3",ExpressionUUID->"717d66cb-f527-488b-8d26-7ba1be80c1a1"],

Cell[BoxData[
 TagBox["16",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356685658*^9},
 CellTags->"b:3.2.3",ExpressionUUID->"483f3aac-fd15-48b2-8547-d32b192396fe"]
}, Open  ]],

Cell[TextData[{
 " The full wave function after we build the second register with  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]],ExpressionUUID->
  "fb5bd35c-7233-4840-9e3a-7f1ccda6a60b"],
 "reads:\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"|", 
         SubscriptBox["\[Psi]", "12"]}], "\[RightAngleBracket]"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{
           RowBox[{"-", 
            SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           SuperscriptBox["2", 
            SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
      "\[RightAngleBracket]"}], "|", 
     SubscriptBox["f", "j"]}], "  ", "\[RightAngleBracket]"}], 
   TraditionalForm]],ExpressionUUID->"8d2b8590-f65e-466e-b39c-c9ba7d0ec304"],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["xguess", "j"], 
     RowBox[{"(", 
      RowBox[{"mod", " ", "M"}], ")"}]}]}], TraditionalForm]],ExpressionUUID->
  "b3aeca97-49ff-449f-a1c5-845a2e289580"]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.3",ExpressionUUID->"1b942e64-f0ef-4a8a-b57e-c8ff4e8265fc"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:16", "b:3.2.4"},ExpressionUUID->"12cde5ef-8089-4818-ad4d-98f516b5bfa3"],

Cell[TextData[{
 "We take an hypothetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one of the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]],ExpressionUUID->
  "4be46d5b-14e1-44f7-ab52-d754eae0c457"],
 "]"
}], "Text",
 CellChangeTimes->{{3.857208432877664*^9, 3.8572084443299627`*^9}},
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4",ExpressionUUID->"2974a720-8c21-403c-9df2-f379891ace8a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.2.4",ExpressionUUID->"38b54cd7-ffbe-4061-b025-2315bca1f3f5"],

Cell[BoxData[
 TagBox["16",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572083567031384`*^9},
 CellTags->"b:3.2.4",ExpressionUUID->"7fa3010c-5136-438c-8ee2-6b289c0268b7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SuperscriptBox["2", 
  SubscriptBox["n", "1"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  SubscriptBox["n", "2"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], ")"}]]}], "Input",
 CellTags->"b:3.2.4",ExpressionUUID->"b384365a-429e-4db7-825b-83d26a561649"],

Cell[BoxData[
 TagBox["512",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356711553*^9},
 CellTags->"b:3.2.4",ExpressionUUID->"e5b32156-a616-4788-b07a-3c5df7baf38b"],

Cell[BoxData[
 TagBox["256",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356715461*^9},
 CellTags->"b:3.2.4",ExpressionUUID->"934d2906-c1b7-4072-b879-4745c9fbd08e"],

Cell[BoxData[
 TagBox["131072",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208356717881*^9},
 CellTags->"b:3.2.4",ExpressionUUID->"1fa36f6c-3f15-4d8b-b9a1-4f94a3205cc0"]
}, Open  ]],

Cell[TextData[{
 "The next function gives the number in the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "8ac627e5-c1f2-493a-90d6-894695d82e51"],
 "register in decimal basis from a given number i in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["n", "1"],
     TraditionalForm], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]],ExpressionUUID->
  "04375f2c-ccb0-42c5-aa62-0913d91b0ac7"],
 " bases."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4",ExpressionUUID->"0d71df29-876e-416b-b38a-362fc8bc158d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"xv", "[", "i_", "]"}], ":=", 
   RowBox[{"FromDigits", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}]}], "]"}], ",", 
       SubscriptBox["n", "1"]}], "]"}], ",", "2"}], "]"}]}], ";"}]], "Input",
 CellTags->"b:3.2.4",ExpressionUUID->"25559b2e-086c-4304-a916-0ce52db6f5a9"],

Cell[TextData[{
 "We select the positions in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]],ExpressionUUID->
  "5134bf2c-951a-4671-982b-b7ff31ba918d"],
 "register which remain when reg2 has been measured:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4",ExpressionUUID->"be63e8d6-0bcf-4559-b4c6-3e40e92a167d"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dens1", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Mod", "[", 
         RowBox[{
          RowBox[{"xguess", "^", 
           RowBox[{"xv", "[", "i", "]"}]}], ",", "MP"}], "]"}], "\[Equal]", 
        "reg2Measurement"}], ",", " ", 
       RowBox[{"dens1", "=", 
        RowBox[{"Join", "[", " ", 
         RowBox[{"dens1", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"xv", "[", "i", "]"}], "+", "1"}], " ", "}"}]}], "]"}]}], 
       ",", "\"\<\>\""}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        SuperscriptBox["2", 
         RowBox[{"(", 
          RowBox[{
           SubscriptBox["n", "1"], "+", 
           SubscriptBox["n", "2"]}], ")"}]], "-", "1"}], ",", 
       SuperscriptBox["2", 
        SubscriptBox["n", "2"]]}], " ", "}"}]}], "]"}], ";"}], 
  " "}]}], "Input",
 CellTags->"b:3.2.4",ExpressionUUID->"ae252cbc-516d-45a3-b589-0385ca4ed2f9"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4",ExpressionUUID->"a2ce1843-6c3d-49ba-abc2-e32f635779a9"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"reg1", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"reg1", ",", "1", ",", 
      RowBox[{"dens1", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "dens1", "]"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"reg1", "/", 
    RowBox[{"Norm", "[", "reg1", "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.2.4",ExpressionUUID->"d1dc27a2-c27a-43eb-bd8f-e8db44fe18ca"],

Cell["Share some memory", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.4",ExpressionUUID->"a169e8d9-6724-4f5d-8a01-041fb60846a9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Share", "[", " ", "]"}], ";"}]], "Input",
 CellTags->"b:3.2.4",ExpressionUUID->"79576aff-ff3a-4eba-986f-d0a506d1b9f0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:17", "b:3.2.5"},ExpressionUUID->"e2b44cc5-4a59-474f-99a5-766e9b3e0cce"],

Cell["\<\
We apply the inverse of the quantum Fourier transform to reg1 and check \
normalization:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.5",ExpressionUUID->"26d2857a-d9db-4e3d-b0bf-ad46b54832fb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.2.5",ExpressionUUID->"0ab15e2f-cff2-41c1-81b4-edb1378861e1"],

Cell[CellGroupData[{

Cell[BoxData[{
 SubscriptBox["n", "1"], "\[IndentingNewLine]", 
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"0", ",", "2", ",", 
   SubscriptBox["n", "1"]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"CompIQuantumFourierTransformV", "[", 
  RowBox[{"IntegerDigits", "[", 
   RowBox[{"0", ",", "2", ",", 
    SubscriptBox["n", "1"]}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.625186436403755*^9, 3.625186443029406*^9}, 
   3.6251864875596313`*^9, {3.62518687628403*^9, 
   3.625186876678273*^9}},ExpressionUUID->"7277ef25-d376-49bc-9ceb-\
cfd5aa629a2f"],

Cell[BoxData[
 TagBox["9",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208356938293*^9},ExpressionUUID->"8e4fa450-6297-4a76-be09-\
b860eb83e49e"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208356947483*^9},ExpressionUUID->"25794dba-9fff-4d97-a479-\
274f1610a704"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]},
     {
      RowBox[{"0.044194173824159216`", "\[VeryThinSpace]", "+", 
       RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.857208356972454*^9},ExpressionUUID->"5dd70b23-0876-411c-9d1e-\
556a3268fcfd"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"reg1", "[", 
        RowBox[{"[", 
         RowBox[{"i", "+", "1"}], "]"}], "]"}], "*", 
       RowBox[{"CompIQuantumFourierTransformV", "[", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"i", ",", "2", ",", 
          SubscriptBox["n", "1"]}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         RowBox[{"2", "^", 
          SubscriptBox["n", "1"]}], "-", "1"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"QFTVec", "=", "QFTVec"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", " ", "[", 
  RowBox[{
  "\"\<The norm of the inverse fourier transformed vector is: \>\"", ",", 
   RowBox[{"Norm", "[", "QFTVec", "]"}]}], "]"}]}], "Input",
 CellTags->"b:3.2.5",ExpressionUUID->"71a9281e-1a08-42a4-bf13-5cd629a8f8d3"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The norm of the inverse fourier transformed vector is: \"\>", 
   "\[InvisibleSpace]", "0.9999999999999993`"}],
  SequenceForm[
  "The norm of the inverse fourier transformed vector is: ", 
   0.9999999999999993],
  Editable->False]], "Print",
 CellChangeTimes->{3.8572083572153473`*^9},
 CellTags->"b:3.2.5",ExpressionUUID->"1ffdbfb5-8eb0-4056-9e08-907f495b602e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< seconds\>\""}], 
  "]"}]}], "Input",
 CellTags->"b:3.2.5",ExpressionUUID->"c36e697b-51e7-4fdf-93f7-98e1bdc0b2b5"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The QFT took \"\>", "\[InvisibleSpace]", 
   "0.290168`5.613164514170429", "\[InvisibleSpace]", "\<\" seconds\"\>"}],
  SequenceForm["The QFT took ", 0.290168`5.613164514170429, " seconds"],
  Editable->False]], "Print",
 CellChangeTimes->{3.85720835722302*^9},
 CellTags->"b:3.2.5",ExpressionUUID->"3f97b5cc-cc36-4018-a63f-c610d5f1532b"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:18", "b:3.2.6"},ExpressionUUID->"ddd77bd5-ee92-4f00-bc30-a05161b1d4cb"],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.6",ExpressionUUID->"80d9d151-4ffe-418a-ab5a-48ace74cac97"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.2.6",ExpressionUUID->"691b62c1-45d8-4356-94ca-fcca56f24d50"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"QFTVec", "[", 
          RowBox[{"[", 
           RowBox[{"num", "+", "1"}], "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "^", "2"}], "]"}]}], ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.2.6",ExpressionUUID->"b23662eb-3dc3-4197-b052-917149eb8671"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.6",ExpressionUUID->"d8c92dbc-2b52-4dfb-a9f8-044e215b0c4e"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.3`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.2.6",ExpressionUUID->"13e27a7f-7b84-4a33-8e0b-09cd7ccfd29c"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.0055000000000000005`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJw11HlQjHEcx/FHIqEVQ2K2UStJdCmV3dr99FTa3a7drUbC5Oqfkhyt1hE7
OTIko9Kw7hkSyhiEzBiiaLZUOpwzcrUlpUjkytX398w888xr3s/nz9/PYVmq
JtGM47iov++/r8ORJ3xWb7esxD1/WEKyp4z7//TImkceMiyc1SodNIeyqtOz
13gwj8ABr73beV9ma9ysrM5tBPN4tDQNMXOJYLZF0/d8+4hFzEIYuo/1hKxi
tkfWZN57yjZmEXTmmZtbDcyOqNldIT5TyuwE9+LpzWmNzM7YU9xyc8lnZhdI
Hz7du9XWRJ6FCznvRO0yZjdsLKovv5zE7IHs51v8Bw4ye2Kdc6/QZGSeDd94
v+Zsro3shfedFn02YmZvTDl8KbE4nXkOCnvMS3RlzD5487zf9/xvZl98ejX1
QrqyneyHGGlfpI2BeS4kOi6+4gOzGGs9yrdVyN+RJTAKM3QJRcz+yLWemfrW
qoMcAPdNebVnNzBLYfNibptVJ7MMtVUjK9MT35OB8DKFMM5E1gMrFjfEuqR2
Ug9ETsD6W1qui3ogzk2cZKo1kDkel+Fl/1T6gTqPjjFHVPIuMhcEN8cfHa8L
u6kHoaVrp4j36aEeDAXGntx3nKwPhrR01ZWXYz5SD0G9YL6kOousD8FdgTj4
i8Un6vMgctNuFO4n6+fBo+7tL4FjL/VQ5IcG37heTtaHItbYFCdK/kxdjmnD
7LvWO/RRlyMupWqHpYnMKXCy4U7TwhtfqCtg23bf3enEV+pKPGtWSpYe6qeu
ROEC17Pqom/Uw1C/sji3oO479TAInOV2knE/qYdj16gtqpp1v6iHgz+VFpbW
P0A9Akga8MnRcIPnUx8BrZ9OUSMbQuc1EgsMrXapoWbUI6FsfVB3OmUo9Sh0
uolHZ14zpx4FV+mDBu+Zw6mr4D88IzCh3mLQUKEwaPmyrRct6X8VHgnbS7c/
HjXo2yrw53aNPhYjoL0altlz7pUctaa9GgUfTySd0YyjvRp5q6+6+sSPp70a
YlHb5MhqG9proOStMgyVk2ivQW/MzrJGrR3tNfA2Ls1MFjjQXoNHnnk/n1k5
0j4aAUZhbIG5M+2jcalO69S/0lWWMqHWMCMlGux++wPQBG1M
       "]], LineBox[CompressedData["
1:eJxV1GtIFFEUB/DJViKLXmQPy1qjsgILjQjKx59CFDQXFNr37p2ZdUGiLLHI
cmnLqFBKQ9NCqJXIMNSVCsk+xKREapqPwiWy2so00xhsiZBAsvYSZy8Mw2H4
3bmPc06MlJ+dGyYIQsbs8/ed6WQZfV8PomlHVbj9UHyK8G/oEZ/V/PiZdnsw
hh7pmuiCM8LmYOzWo2ROcdRIrTYYK3oc90zA2baaewNqXwaSqnKWc29A+e8t
Ux3ZS7g34FTP6O28igjuDah2H7WY34Rzb0R9h7r1bEwY90Y0Jek3VepmkoPe
iErXUKGveDoYK0aUqql96qWfwVgwIV1NLCvb/yMYw4SSweFdiUtV7k0oX3Ti
RkvRN+5NyKmcHjhXPcq9GfMqruxelfaZezPep7gjB475uTdjKP7EZJNmmHsz
Osc3Jjzt93Fvwfri3p1X415zb8GY7I59Ej7AvQVdnTE3a8p7uLdAdC6caOl+
zr0VgeQCk+ZOB/dW9E5N5r6LULi34lr1p22tvjburYgt/NDwqPAh9zZItxas
jGv0cm9De/ODX2GDd7m3oXWNxjmxt457G9TU0ugVdTXc2zE31fVq8vtl7u34
6ItSxu+7ubdDHcuKDJQc5t6Oi/eivnTlHeCeQaBDy3DBS76DQTdCPGPoD5mf
oXsf+b+HoTaNrE9hKFpL1u9nGPGS/QkivHVk/1oR+fR8ICKzhpwfE1Efcr4i
5qeQ8/eIaKf3o4h4Qe/PL6Kb3q8gYZmL3L9Wwh6aH5AgJJD8YRICIfklYTHN
P4+E0zQ/FQknaf76JbhofgsyNtD818pgtD4g4zytHyZDSKb1JaOR1p9HRgOt
T0WGntavf3Y+Wt+CAzO0/rUOHKH9AQ44af9gDuhC+osD63j/edujM10vc/zv
T38AqOZxvQ==
       "]], LineBox[CompressedData["
1:eJw11HtMUwcUx/ErggSliMThWEq8oEHEoaJIEVz5CToBowLR6NCFwkZve58z
Fqk6pEHER3wNjY+6sS7OiQrGzeEGiVlZcCOg4kBicCFWYwuoVRFRfLFp7uEk
Jyefv77/nYgCJafQh2GYFe/2/c0cPDpHyDCidtYhvzwhLoV5PwYjGtqKoobE
WNU2I+JatKsO+0ardhjhiTv45pZmqmqnETEt+WVCUIRqlxHDKyvqO4rCVTMc
xqZqSuyXw1SzHKZH9ny0vDVUNTgc+epibELuROpzqOp38KdyQqjPYdKeeX/V
fhdMfQ7ZZ3YGVq0Moj6Hn7W9deU3x1GfQ3naFwWl5wOob0K1X8nCvOv+1Ddh
ov5qe/yMMdQ3oX1mUmDZb77UNyHLfbXtpDSa+iYU2N3hyhIf6pvwaaI140rK
KOqbwPHDCftyGOqbEPejZallaFiv9s3YOW5r1pUNb1WzZkyPTg9PDnmjGmb0
iTWVh9teqTaYoXwWezq7+qVqmxlnOzOT848NqXaY0eX5e1aU44Vqpxkd7X/e
WNvwXLXLjANS8/YAzyD1efj7sd6NEWSWh6PlxppI4Rn1edQtWdTwe+MA9XnE
t917GzSVbOORPbNos/abp9TnkTY+adFzf7KTR2nQ6uTWHf3U55FQJ//qGk9m
BKRhwg/7v39CfQEvvBWRqQlkCBiY8vr+3Z8eU1/AB8HfZqV7H1FfQD7msl16
skPA5Elhnmt2L/UFVH6y8Y8ihuwSoPu8fVWM8pD6IjT1Gdo1ngfUF+FsHnu5
uJAMERe75/doHt6nvoiQLQevnd5EtolYFzxDuachO0RYtSXWvOo+6ovont24
rSmd7BIx2srkNj3qpb6EKP3g8lA7mZXw9M6Uc8WZZEi48O+Q7ux/PdSXoOv3
rbXWk20SKo7/UlhTTHZIyPT6D4YmkZ0SLLmJnXsYsktCXvSA1tPiob6M/ltb
FwwfJbMyLNXXGy/wZMjYta8vsjeFbJBx85+uvaUfkm0yKmtuXzI8c1NfRm7N
tE5LB9kpw7W7KelUHdklw8ev7Gu3ncwo6AtLjZ+8jcwqSHxc9WSxTIaCha8O
scvWkQ0KdJ2jfGKWkW0K+KbWyg6M9BV0z91bnqob6SvIbj45Z/3skb6CEwHH
7Gs/duu/nHbJGvjOI//tf5rRjDs=
       "]], LineBox[CompressedData["
1:eJw91GlMlEccx/FHCm48WFbT4JElLpQiBVFUuiBY+JViRBpxRU3p0oZVgy8A
3XmeZxBaixK8mtYDRYyu15J4IIV2U0XFpikoKILAyhFjSSpVWVBZWQ8UPGhX
n398ksnkk5lM5vtiHv9V5pQMD0EQlrjH2zk5u/TysA9D5ax9XulZs+OEt5+G
wWvsQUvajJ7Yd9YxOBtOzBHDyeEMJXN3bomPJIPBVt+0tx1kA8PtjlEeIYvJ
Jobil/t0i78hM4Y1A0ddC9aRCxi2T42PmLaZXOQ+37Pwhx4L2crQ/lNd9Kkq
so3BWDG9k7eTaxiOVdz+0/SMbHeff+PWzk2THYq7GS7suh/QF0d2ufeX2WvP
ZJIFEeVdG+ePHCBrRKQGP9U6Gsk6EcuMUZ07hF7qFzHYrxr0jSZDxLRDv2dU
5JINIs67PCvzqskmEXe7hiJ/+Y/MRAz/+9GvuUl91C8iJHYw2ddCLhIxOU8w
1j0iW0X8HF67uS7xPvWLaNHm56WXkWvc65pQ8z3vB9QvInBDccvp78jdIlT/
zOv17ie7RJxtGFufm/GQ+iVI1Yu0qQ6yRoL527YVIeZ+6pdQ8tn6v3IEJ/VL
ODRpiqPFQoaE3zBXdyv2EfVLGPE5bEh0kk0S9IGvHtw5OUD9Ejqc2wLi9S7q
lzAPE0p3HyMXSYiqWne22+cx9bv3q7+KadpOtkm4po5OeK56Qv0SgmbmfK/d
Q7ZL+Lj13ht14FPql3B4YcLFC7Vkl4TMxo7UgKxn1C8j1EvnXO8/SP0ylq1t
2DrGQdbJKG+71JF28Tn1y/DtvToryPqC+mU0dybFrDw4RP0y/vg67PTSsmHq
l2HPrti7v/Ul9cuYEJzoFzPxNfXLKBi30XBdfkP9MhKO8y/50Aj1y0DmiH5X
iqC8T5sMHpW36HrcKMU1MoyWHj/zQg/Fdvf9e5pbT6z9QHG3jIGZ0eMLz3sq
dsmIiG1uiwgdrVjg+GR0/ufpdhW9f47KL1av2mQbo1jHcUnbV7Xl5jjF4Rxz
yn8cf3S5WjE4VDs+vVJ5RKPYwHHgsTXzVMpExSaOEnYuTG/8UDHjmB/QOzW5
yVdxAYc+3jvfUj9FcRHHq+Xbqttz/BRbOUIbVxZmqf2pn6NldvHrv70DqZ8j
slG7Yr9nMPVzlLbmBA1lh8WlbT2zJqmLv/+//Q/pfZKr
       "]], LineBox[CompressedData["
1:eJxVlHtIVEEUxm+iBBa1RVpuCSuUj8BakwjJx1ehBpGJRUEFmqGgJZm5M3c2
zdX+KJLSUrSIaqWnle5SIikhGxY9sFotfITSUmaZW7smEhJIa3OJ48Dl8mO4
d+Y75/tOWPaRjBw/RVG2+Z6Zd01If09suwlNa2sDMg/FJCkzy2FCVFpz+zPD
GslOE7b6hxaVKeGSXSaUzynRD182SPaakGkdQ25biGSF4cObiYTanUsk6xhO
/Ikc78zQSTYwlHWNXM+rDpRsZLBaCvfvGwiQDIZbnZ6o8jA/yekMVxL2rKrZ
MZ34j7MYTpX2FveVTEku9O17kt96Tk9KtjCkeOIrK7f8klzNcLdncH38Io9k
K0PVAnbJLr5LtjME10x1V9SNSHYwfKs6t2FZ6mfJTobeJEtQ91GXZBdDawxz
N/kPSvYytIyuXPfE2SdZ4bhz/HXs+ej3knUcQwctER0B3ZINHI0vwq7WV3VJ
NnLE5c4fs796Lhkcg4lFe/1vdkpO57g47s4ZCnRo+jk66j6tbu1r0/RzuI99
bHxU3KLp51h4bd7S6Ps2TT9Hb/PD3349tzX9HBeW++eObWzQ9HOUpJwJDW6o
1/RzDCSXvnP/OKvp59jUr3eMPrBo+jmefk0LmjhZoOnnqL2n//Iyb7umX4VC
l06Fn43sG1TwYfK9UcV0H/k/fPubyfnpKvJTyf2yVOxeQe5fqOKxjeizqMhu
oPpVFBST+lhVhNeT+tlV3KD1daiITCL1d6r4SfvjUmGn/fOqqKH9VQQWl9L+
CyRQfxgEJql/jAJzQfwFAX018V+6QD71Z5ZAHvVvoUAm9bdFIG6W/wUO03xY
xez82AV0iSRfDgEbzZ/TxzSfLoFcml+vQAXNt2JG16z8m3GAzgeDGTl0fhjN
2EXnC8wwaPOnYCQiuivZ/H8+/QV/AXz5
       "]], LineBox[CompressedData["
1:eJwtz39M1HUcx/GvCDJU8FKicMc6iBkTD880zqCOV0ADLNnJdCJmoI4/4oec
CR75/X4/H0Kz1vxB4FycSucWiQnGwiu1X9DOYpzIyY85ba2ruAPN0yOFkJTC
fd6f7b3PHn+938/YbRV5xUGSJOXOzOO/ec/rjY54GW3Lj4QUlq5Ikx4/g4zv
+6qWTJbphSHD2KPdcDQ4Qdgsw7ei4eGN8HjhIhn6nq21pRGxwhYZE+v3Xxio
ihGukfFkerhquxQtXCcjKW5kca4rStguo8HylT65IFK4XUbTmL3kVN5C4U4Z
iw688FPbCY2wW0bO5x/Mb1ofIeyZuV876th3bZ5wQEZ9xvZtvD1MWFLwTYj6
SqE7VFijINLU278qcY6wTkF/Usr82q+DqV9Blre3r7l8NvUr2GnzxlRkBVG/
gozV1TmX02ZRv4I3S6aTD+VJ1K/A+Gnla5WT0ybRr6BmHjNf3vVIuE7BsoTs
mNSFD4XtCh6UtdYf7ZsSbldQtkl/el3LA+FOBSeH1qRubZwUdisY9P28fIn9
H2GPgmv9Pw5uvjghHFDwRXn3e2G+cWFJRWSIzr87lqxRcaxnMD+u9L6wTkVH
VubF8133hA0qUvuGH0XEk6EiP6lqj/ajv4XNKjIWpGROhJKLVLwbsTHV9f6Y
sEWF3rHjnGcBuUaFEU+cPPxJgPpVjPj3x6Unk+0q5sT/e+uPz+5Sv4pYzXFz
tv8O9at4Cyt1101kt4rEp6J9V2x+6lfBX979Q5VEDszs29K/YWnFbepnmDqf
o833/UX9DG3dcy9Zi8k6ho5fXxwJv32L+hm0csOV0++QwbBOk1gxHE42M2zU
qtWFLTepn8Fp6NrrzCZbGMatUoHzzij1M8w1jedG2ch1DBO/P3vWuoZsZ3D9
Mmk8898I9TPMHgtuq75A7mTYdOzL4lYr2c2Q7A8dj0ohexi2FKweOiCRAwyZ
Cfe0vh4f9XM4brCXpj8mazjKW9xdHSVkHYft0M240TSygWP46vWD/GkyOKyt
v31XdN9L/RzbW58bqhwgF3H4PnSmnHKQLRy9wbWK10au4fgzOn3VM3vJdRy7
7jYFXt1BtnO8PXVEt/YNcjuHa3BW0NK15E6OFqerfgBkN8e3Kw/uSzeSPRyL
u5uf32kgBziuhjXaNi/zmv4HAXaQeQ==
       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 511.}, {0, 0.0002657992350385507}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208358521874*^9},
 CellTags->"b:3.2.6",ExpressionUUID->"1dbd491e-690a-4984-87a5-8ede684b0365"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.0055000000000000005`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdmHl0VOUZxkcWKSgwUkSkoV4QkcXgsCUQIDyEPQpMSICQALnZ92T2fROs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       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 511.}, {0., 0.3}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208358589079*^9},
 CellTags->"b:3.2.6",ExpressionUUID->"a042b52f-4d3f-4228-b66a-b43061007647"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:19", "b:3.2.7"},ExpressionUUID->"e624a1be-c636-4105-bfe5-d1a2feea774f"],

Cell[TextData[{
 "Finally we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["xguess", 
    RowBox[{"r", "/", "2"}]], TraditionalForm]],ExpressionUUID->
  "8a333696-1c04-4643-9f75-bb8ea79f2371"],
 "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.7",ExpressionUUID->"53f4d29d-2f8a-425c-a2e4-60f1e617e6b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.2.7",ExpressionUUID->"a58fd27c-b350-4aef-9026-c515f1fe64ec"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.2.7",ExpressionUUID->"694abdfc-0b09-4f1b-b2af-ba0ccca3454a"],

Cell[CellGroupData[{

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<xguess=\>\"", ",", "xguess"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}]}], "Input",
 CellTags->"b:3.2.7",ExpressionUUID->"9ea8d0bf-0b54-4f09-a85b-4ca0f0de669a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"Period", "Number", "\<\"f1\"\>", "\<\"f2\"\>", "Prob"},
     {"\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>"},
     {"6", "85", "17", "13", "0.11417182031964805`"},
     {"6", "86", "17", "13", "0.027742064787512132`"},
     {"6", "426", "17", "13", "0.027742064787512177`"},
     {"6", "427", "17", "13", "0.11417182031964809`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208358933132*^9},
 CellTags->"b:3.2.7",ExpressionUUID->"1673fbea-2472-4523-98d9-f2d97d3677df"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"xguess=\"\>", "\[InvisibleSpace]", "16"}],
  SequenceForm["xguess=", 16],
  Editable->False]], "Print",
 CellChangeTimes->{3.8572083589381332`*^9},
 CellTags->"b:3.2.7",ExpressionUUID->"ae84e901-6925-46ed-877b-a7bcfd16ce21"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The TOTAL PROBABILITY OF SUCCESS is thus:\"\>", 
   "\[InvisibleSpace]", "0.2838277702143205`", 
   "\[InvisibleSpace]", "\<\"  with xguess=\"\>", "\[InvisibleSpace]", "16", 
   "\[InvisibleSpace]", "\<\" and measured register=\"\>", 
   "\[InvisibleSpace]", "16"}],
  SequenceForm[
  "The TOTAL PROBABILITY OF SUCCESS is thus:", 0.2838277702143205, 
   "  with xguess=", 16, " and measured register=", 16],
  Editable->False]], "Print",
 CellChangeTimes->{3.857208358940922*^9},
 CellTags->"b:3.2.7",ExpressionUUID->"bf7df94e-6b48-4877-8581-5469ee4f047b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.2.7",ExpressionUUID->"ded7f919-5894-4236-a13d-f453620e6bf4"],

Cell[BoxData[
 TagBox["5.18339`7.166158880027163",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572083589609756`*^9},
 CellTags->"b:3.2.7",ExpressionUUID->"90a47588-f8ca-4a79-8719-d5a3de8b4d32"]
}, Open  ]],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.2.7",ExpressionUUID->"7111c831-3093-4a08-84dc-432a63ded561"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       SuperscriptBox["xguess", "i"], ",", "M"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "60"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"a", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellTags->"b:3.2.7",ExpressionUUID->"b2ecdeee-b04b-4f5c-9c9c-2a84fd795290"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdzjkKwlAYReFgZWlhYWGhIiIi4jxG85xnC3tTKNZuIUt2CSr5r5ATCJcT
ku+leH9dnynP8/zv/dv4egfxNp09iDeaW6etb9YZ29A2a/uwzbmkm3dJvwC/
BL8MvwK/Cr8Gvw6/AV/vyW/Bb8Pv4Lsu/B78PvwB/CH8Efwx/Al8H/4U/gx+
AN8l/UgtX+eG+A/5i6QfqeUv4avlr+Cr5a/hq+Vv4Kvlb+Gr5e/gq+Xv4avl
H+Cr5R/hq+Wf4Kvln+Gr5V/+/gc2FFfZ
       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 60}, {0, 152.}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208359009454*^9},
 CellTags->"b:3.2.7",ExpressionUUID->"d3e7fe35-788f-4b5c-bc2c-d420e1e22685"]
}, Open  ]],

Cell["Quit", "Input",
 CellTags->"b:3.2.7",ExpressionUUID->"af3eb7f0-c3c0-4018-8dc7-edf49c603f86"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Shor's Factoring algorithm with explicit construction of both registers \
(using state vectors)\
\>", "Section",
 CellChangeTimes->{3.857208508737338*^9},
 FontFamily->"Times",
 CellTags->"c:20",ExpressionUUID->"342b6cff-8d04-4829-b39a-30888a1b6349"],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 0: Define functions for QFT and Number Theory[check ",
 ButtonBox["QFT2014.nb ",
  BaseStyle->"Hyperlink",
  ButtonData:>{"QFT2014.nb", None}],
 "for details]"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:21", "b:3.3.0"},ExpressionUUID->"4fe22e9c-e4f8-49a9-9be9-52d1f03d8739"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.3.0",ExpressionUUID->"c2fd9143-1605-4192-a843-f52d21f90686"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{3.857208518643111*^9},
 CellTags->"b:3.3.0",ExpressionUUID->"a817e1e6-cf02-46c6-b909-79652efa790c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.3.0",ExpressionUUID->"c4e2ef04-0253-4a51-890d-7a0b90f5cc16"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["about", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "about", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["FidelityB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "FidelityB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["OneOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "OneOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "res2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["U", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "U", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Adj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Adj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Oracle", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Oracle", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "res2$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "v", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["AF", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "AF", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTrace", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PartialTrace", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Rotqbit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Rotqbit", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Vs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Vs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Bra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Bra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Hadamard", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Hadamard", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTraceGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PartialTraceGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RotX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "v$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "BraV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["HALL", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "HALL", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTranspose", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PartialTranspose", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RotY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ws", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ws", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "BraX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["i8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "i8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ProjG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ProjG", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RotZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "BraY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["intro", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "intro", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CNOT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "CNOT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["j8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "j8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTrGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PTrGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Schmidt", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Schmidt", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Coeff", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Coeff", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Ket1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Ket1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Purity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Purity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma0", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Sigma0", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Comm", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Comm", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "KetV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Sigma1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroMatrix", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ZeroMatrix", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompMeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "CompMeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "KetX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "PY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Sigma2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ZeroT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompPTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "CompPTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "KetY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomBra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma3", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Sigma3", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalH]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ControlledX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "mc", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomBra2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Snos", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Snos", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalP]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ControlledY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "mcl", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBraN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomBraN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SP", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "SP", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[DoubleStruckCapitalI]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ControlledZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "mcl$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomKet1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SpinObs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "SpinObs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Rho]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASE", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "CPHASE", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "mc$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomKet2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stA", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "stA", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Rho]$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASEV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "CPHASEV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "MeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKetN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomKetN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "stB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Sigma]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Sigma]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["cRQN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "cRQN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiProjCBasis", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "MultiProjCBasis", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomQubit1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Swap", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Swap", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Phi]A1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Definef", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Definef", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiQProj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "MultiQProj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomQubit2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "s$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Phi]A2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["DForm2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "DForm2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Mutual", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Mutual", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubitN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "RandomQubitN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ThreeOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ThreeOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Phi]B1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Discord", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Discord", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "nb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsDate", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "rcsDate", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Toffoli", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Toffoli", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[Phi]B2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "EnTropy", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["neg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "neg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsRevision", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "rcsRevision", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ts", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[CapitalOmega]s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "\[CapitalOmega]s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "EnTropy2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nneg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "nneg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "res1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "ts$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], ""},
                {
                 ButtonBox[
                  StyleBox["Fidelity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "Fidelity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nq", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "nq", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "res1$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["TwoOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-fa20fe14-bdac-4f18-93a9-a249816d5be5", {
                    "TwoOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], ""}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.19]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572085194209538`*^9},
 CellTags->"b:3.3.0",ExpressionUUID->"0cd6f6d9-8b9d-4c1f-b5ff-edc1b03ea79c"]
}, Open  ]],

Cell["\<\
To improve performance we precompile the quantum Fourier transform:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.0",ExpressionUUID->"bc0cd033-3aaa-4057-8d66-7d53c8450df9"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Zeroff", "[", 
   RowBox[{"vac_", ",", "R_"}], "]"}], ":=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"vac", "[", 
      RowBox[{"[", 
       RowBox[{"R", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "/", 
     RowBox[{"2", "^", "i"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "R", "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CompZeroff", " ", "=", " ", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"vac", ",", "_Integer", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"R", ",", "_Integer", ",", "1"}], "}"}]}], "}"}], ",", 
     RowBox[{"Zeroff", "[", 
      RowBox[{"vac", ",", "R"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Zeroff", "[", 
         RowBox[{"vac", ",", "R"}], "]"}], ",", "_Real"}], "}"}], "}"}]}], 
    "]"}]}], ";"}]}], "Input",
 CellTags->"b:3.3.0",ExpressionUUID->"7e6edb2f-271f-44d6-8557-6e8ef8678a3d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"QuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"Zeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{"2", "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[CircleTimes]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  ", "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.625186598713747*^9, 3.625186623970701*^9}, {
   3.6251867728252163`*^9, 3.6251867832093287`*^9}, {3.625332015413991*^9, 
   3.625332016413701*^9}, 3.857208152823502*^9},
 CellTags->"b:3.3.0",ExpressionUUID->"171c8b0d-1e2c-45d2-b4af-3d9680453c2a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"QuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex", 
        ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellTags->"b:3.3.0",ExpressionUUID->"198be410-ca96-4b49-9f66-44022697997d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"IQuantumFourierTransformV", "[", "vac_", "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\[CapitalOmega]", ",", "L"}], "}"}], ",", 
      RowBox[{
       RowBox[{"L", "=", 
        RowBox[{"Length", "[", "vac", "]"}]}], ";", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Ket", "[", "0", "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
             RowBox[{"Zeroff", "[", 
              RowBox[{"vac", ",", 
               RowBox[{"Range", "[", 
                RowBox[{"L", ",", "L"}], "]"}]}], "]"}]}], "]"}], "*", 
           RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "2"}], ",", 
         RowBox[{"i", "\[LessEqual]", "L"}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"phass", "=", 
           RowBox[{
            RowBox[{"-", "2"}], "*", "\[Pi]", "*", " ", "\[ImaginaryI]", "*", 
            RowBox[{"Zeroff", "[", 
             RowBox[{"vac", ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{"L", "-", "i", "+", "1"}], ",", "L"}], "]"}]}], 
             "]"}]}]}], ";", 
          RowBox[{"\[CapitalOmega]", "=", 
           RowBox[{"AF", "[", 
            RowBox[{"\[CapitalOmega]", "\[CircleTimes]", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ket", "[", "0", "]"}], "+", 
               RowBox[{
                RowBox[{"Exp", "[", "phass", "]"}], "*", 
                RowBox[{"Ket", "[", "1", "]"}]}]}], ")"}]}], "]"}]}]}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"\[CapitalOmega]", "=", 
        RowBox[{"\[CapitalOmega]", "/", 
         RowBox[{"2", "^", 
          RowBox[{"(", 
           RowBox[{"L", "/", "2"}], ")"}]}]}]}], ";", "\[CapitalOmega]"}]}], 
     "]"}]}], ";"}], "  "}]], "Input",
 CellChangeTimes->{{3.625332022556472*^9, 3.625332024086441*^9}, {
   3.625332110055339*^9, 3.62533211335984*^9}, 3.857208152827344*^9},
 CellTags->"b:3.3.0",ExpressionUUID->"e76c7614-569c-4eb2-a92a-d20b6496be74"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CompIQuantumFourierTransformV", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"v", ",", "_Integer", ",", "1"}], "}"}], "}"}], ",", " ", 
     RowBox[{"IQuantumFourierTransformV", "[", "v", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", "   ", 
       RowBox[{
        RowBox[{"IQuantumFourierTransformV", "[", "v", "]"}], ",", "_Complex",
         ",", "2"}], "}"}], "}"}]}], "]"}]}], "  ", ";"}]], "Input",
 CellChangeTimes->{{3.6253315317114964`*^9, 3.6253315362777853`*^9}},
 CellTags->"b:3.3.0",ExpressionUUID->"077d92a3-f006-465d-880e-8cdd676fecc6"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the continued fraction expansion. 
i is the ith cycle of continued fraction and j is the number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.0",ExpressionUUID->"7bb36b78-356c-40cc-9073-18c80b475843"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.3.0",ExpressionUUID->"c6f014f8-9677-4c13-a45c-3600fba54ab1"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.0",ExpressionUUID->"377ac93b-27c8-4407-acee-7c8901cf8efc"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.3.0",ExpressionUUID->"92d20556-4222-4574-9912-38a3d8b0565d"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39,55,77 and \
determine the computer size required \
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:22", "b:3.3.1"},ExpressionUUID->"75a1a666-9231-472e-a60a-556d78499aec"],

Cell["M is the number we are trying to factorize:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1",ExpressionUUID->"bc6c8017-a8d6-4071-9b96-76a0845145b6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"M", "=", "35"}], ";"}]], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"3eaa8960-b60b-43d3-86bd-7dec3d67f698"],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1",ExpressionUUID->"c51200c3-cc31-49ff-ad87-d83cd98b455f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"0f0b1380-8ed9-403f-acdb-c98014cbcfc5"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime and quit if it is:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1",ExpressionUUID->"5addb373-3c99-403a-8fd4-0e6acf1858ed"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
    "Print", "[", "\"\<Your number is a Even, please revise it\>\"", "]"}]}], 
   "]"}], ";"}]], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"a3de0cd8-1787-477a-9d4b-65eeba4e97da"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"e91a2e55-c340-401c-8bf3-a601d91bf15a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"dfd7d6f3-7933-4af1-89eb-d12597718341"],

Cell[BoxData[
 TagBox["2048",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.85720851949876*^9},
 CellTags->"b:3.3.1",ExpressionUUID->"68bbec5e-2cb3-403b-a17a-9052f8e96bde"]
}, Open  ]],

Cell[TextData[{
 " We need the ",
 StyleBox["minimum",
  FontWeight->"Bold"],
 " ",
 StyleBox[" number",
  FontWeight->"Bold"],
 " of qubits for both registers. Here we should be careful, \na high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "c50a1d78-5078-4614-bb9f-9da6ba1b62ae"],
 " number will lengthen the computation, while providing high probability of \
success, \nhowever in many cases it could be more illustrative to lower the \
value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "7b2eaf06-360d-47fb-b7ad-e05cc4d60173"],
 " and check the \nresulting probabilities."
}], "Text",
 CellChangeTimes->{3.857208529127781*^9},
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1",ExpressionUUID->"ac05b555-7280-4b0d-9072-708fb8846e53"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["n", "1"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["n", "2"], "=", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["n", "1"], "=", "9"}], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"3938e008-044f-49f9-830e-0bb45b68835d"],

Cell[BoxData[
 TagBox["9",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208519517487*^9},
 CellTags->"b:3.3.1",ExpressionUUID->"67804c97-742d-49b3-82f1-c162318ab594"],

Cell[BoxData[
 TagBox["6",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208519521377*^9},
 CellTags->"b:3.3.1",ExpressionUUID->"292b2530-9e5b-40df-b3d6-ca8506917915"]
}, Open  ]],

Cell["\<\
Gerjuoy's test: a failure to pass this test means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.1",ExpressionUUID->"9ed2b209-50f8-433a-84ff-2ee340dac26a"],

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.3.1",ExpressionUUID->"428e9b2f-f116-4abc-ac21-5cb707575033"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]],ExpressionUUID->
  "f34253bc-ef3f-4e11-9a0b-e7b5261effe4"],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:23", "b:3.3.2"},ExpressionUUID->"0e43a654-d0e3-4044-b2f0-dbd29c60d5a1"],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]],ExpressionUUID->
  "68c80e65-f79e-4d8b-9245-b58ad572415b"],
 " ; which is a \nstate where all qubits have been Hadamarded"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.2",ExpressionUUID->"ce708b63-423b-4bde-9355-ca1ae75e8830"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "1"], "=", 
   RowBox[{
    RowBox[{"2", "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SubscriptBox["n", "1"]}], "/", "2"}], ")"}]}], "*", 
    RowBox[{"Table", "[", 
     RowBox[{"1", ",", 
      RowBox[{"{", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}], "}"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Norm", "[", 
  SubscriptBox["\[Psi]", "1"], "]"}]}], "Input",
 CellTags->"b:3.3.2",ExpressionUUID->"542ec1d5-c543-4d75-b997-1d0742a06253"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208519543931*^9},
 CellTags->"b:3.3.2",ExpressionUUID->"5f4d811b-0190-41e1-9487-f58a720d4c51"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]],ExpressionUUID->
  "cbfb23b3-ad1e-4cb0-8d11-710dc576d780"]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:24", "b:3.3.3"},ExpressionUUID->"5a1b48ea-7940-4a0c-93ea-25e918710549"],

Cell[" This is the random Coprime to MP used in Shor's Algo ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.3",ExpressionUUID->"a55f7a7e-6341-49e3-aae9-c51f64ade20d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.3.3",ExpressionUUID->"74329215-36c5-41dc-8ef3-50537c53856b"],

Cell[BoxData[
 TagBox["26",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208519552638*^9},
 CellTags->"b:3.3.3",ExpressionUUID->"60087992-bd7e-4127-b6d3-5f7df6d3ebe2"]
}, Open  ]],

Cell["\<\
The full state composed of register 1 and register 2 is build in the 
following way:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.3",ExpressionUUID->"b2a0d65d-7b23-4347-994d-df493e99c8b6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "12"], "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}], ")"}]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    SubscriptBox["\[Psi]", "12"], "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{
      SubscriptBox["\[Psi]", "12"], ",", "1", ",", 
      RowBox[{
       RowBox[{"Mod", "[", 
        RowBox[{
         RowBox[{"xguess", "^", "i"}], ",", "MP"}], "]"}], " ", "+", 
       RowBox[{"i", "*", 
        SuperscriptBox["2", 
         SubscriptBox["n", "2"]]}], "+", "1"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", 
     RowBox[{
      SuperscriptBox["2", 
       SubscriptBox["n", "1"]], "-", "1"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "12"], "=", 
   RowBox[{
    SubscriptBox["\[Psi]", "12"], "/", 
    RowBox[{"Norm", "[", 
     SubscriptBox["\[Psi]", "12"], "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.3.3",ExpressionUUID->"f7f641d1-bc4e-41d1-8802-482b03510c28"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"ListPlot", "[", 
  SubscriptBox["\[Psi]", "12"], "]"}], "\n", 
 RowBox[{
  RowBox[{"Norm", "[", 
   SubscriptBox["\[Psi]", "12"], "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.614177547228889*^9, 3.614177582910481*^9}},
 CellTags->"b:3.3.3",ExpressionUUID->"63221bfe-763f-4efa-98bd-303157724000"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      NCache[
       Rational[1, 360], 0.002777777777777778]], AbsoluteThickness[1.6], 
     PointBox[CompressedData["
1:eJxc3HF01fMf+PEryaRvJpPJZJJMJpNkMrkymUwmk8lkMplMJpNJciWZTCaT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      "]]}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 32768}, {0, 0.04419417382415922}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208520062244*^9},
 CellTags->"b:3.3.3",ExpressionUUID->"091c98e8-8d48-49e3-b34d-96d553d9e399"]
}, Open  ]],

Cell["Therefore the state of the full system is,", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.3",ExpressionUUID->"57544e27-7ff3-4d46-8a94-65fe0263ea05"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Measure the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:25", "b:3.3.4"},ExpressionUUID->"01673a7b-f377-4d2a-933e-90abcb15638f"],

Cell["\<\
We perform a measurement on register 2, that can be done in the following \
way. First a random 
possible outcome for register two is generated:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.4",ExpressionUUID->"826f2ec6-3675-46dd-aff0-c5f0b2feae57"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.3.4",ExpressionUUID->"a805bfb9-0273-4bdc-af45-80e6825e7c2f"],

Cell[BoxData[
 TagBox["16",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208520087426*^9},
 CellTags->"b:3.3.4",ExpressionUUID->"61815ec8-745e-44f3-ab96-2cde0786d2ec"]
}, Open  ]],

Cell[TextData[{
 "Then register 1 will be in the following state, making use of the function ",
 ButtonBox["MeasureKet",
  BaseStyle->"Hyperlink",
  ButtonData:>{"Tutorial.nb", "MeasureKet"}],
 ": "
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.4",ExpressionUUID->"aff5676d-c965-4ea6-9c5e-a55804bc80a1"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"reg2list", "=", 
  RowBox[{"Table", "[", 
   RowBox[{"i", ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{
       SubscriptBox["n", "1"], "+", "1"}], ",", 
      RowBox[{
       SubscriptBox["n", "1"], "+", 
       SubscriptBox["n", "2"]}]}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"reg2mes", "=", 
  RowBox[{"IntegerDigits", "[", 
   RowBox[{"reg2Measurement", ",", "2", ",", 
    SubscriptBox["n", "2"]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", 
    RowBox[{"1", "P"}]], "=", 
   RowBox[{"MeasureKet", "[", 
    RowBox[{
     SubscriptBox["\[Psi]", "12"], ",", "reg2list", ",", "reg2mes"}], "]"}]}],
   ";"}]}], "Input",
 CellTags->"b:3.3.4",ExpressionUUID->"be6ec06f-42ab-4307-9ed7-a45365c9cb53"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"10"},
      {"11"},
      {"12"},
      {"13"},
      {"14"},
      {"15"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208520096307*^9},
 CellTags->"b:3.3.4",ExpressionUUID->"243cc6b6-cbe9-4ed1-a27c-7e8fcda02a6a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"1"},
      {"0"},
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208520100943*^9},
 CellTags->"b:3.3.4",ExpressionUUID->"c8adf71f-3895-44fb-a1d2-42cf6292f19f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Norm", "[", 
  SubscriptBox["\[Psi]", 
   RowBox[{"1", " ", "P"}]], "]"}], "\n", 
 RowBox[{"ListPlot", "[", 
  SubscriptBox["\[Psi]", 
   RowBox[{"1", " ", "P"}]], "]"}]}], "Input",
 CellTags->"b:3.3.4",ExpressionUUID->"343c5077-25a9-4897-896c-e8b5e414b5df"],

Cell[BoxData[
 TagBox["1",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208520121896*^9},
 CellTags->"b:3.3.4",ExpressionUUID->"a50b030c-d242-4a4e-b547-f79633715e74"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], PointBox[CompressedData["
1:eJxd0jnO3kQAx+FRKkoXFCkoXhBCCCHksO9xIBAIBJyFhISETL7ky744+57M
ESxOMEdwTzMFB/ARpqfxEUAoQvLz6m0e/TT/sSy/unXj0PaO8O9vx3//F79l
d1j/ujVfwg1+udu56+8/4l9/vtjZSX8Fb/Br+HX8Bvtv0t/Cb+N3cIt3sf8u
/T38Pv4Af4g/Yv9j+if4U/wZ/hx/wf6X9N24WzvhsIe+h/2v6Dh8TcdhL30v
+9/QcfiWjsM++j72v6Pj8D0dh/30/ez/QMfhRzoOB+gH2P+JjsPPdBz6tbt+
vZ/oBYeDnMcJl4M8/yHO44QLDoc5f5jnpxccjnAeJ1yO8Py/cB4nXHA4yvmj
PD+94HCM8zjhcozn/5XzOOGCw3HOH+f56QWHE5zHCZcTPP9vnMcJFxxOcv4k
z08vOJziPE64nOL5f+c8TrjgcJrzp3l+esEhrr3BHY6RfXrGBVccznD/mfV+
R4844YwLruyHLe7HHY444by13i/0isNZ7scdjmd5//SMC644nOP+c7x/esQJ
Z1xwZT9scz/ucMQJ523eP73icJ77cYfjed4/PeOCKw4XuP8C758eccIZF1zZ
Dxe5H3c44oTzRd4/veJwiftxh+Ml3j8944IrDpe5/zLvnx5xwhkXXNkPV7gf
dzjihPMV3j+94nCV+3GH41XePz3jgisO17j/Gu+fHnHCGRdc2Q/XuR93OOKE
83XeP73icIP7cYfjDd4/PeOCKw7D2s2w3t/QW9zhHkc8sJ/oI854wgXP7Ff6
gsPNtRu8we1Nvk96jyMecMIj+5k+4YJnXPHCfri17g3e4BZ3uL+13o/0ASc8
4own9gt9xhUvONxeu7nN909vcYd7HPHAfqKPOOMJFzyzX+kLDnfWbvAGt3f4
/uk9jnjACY/sZ/qEC55xxQv74e66N3iDW9zh/i7fP33ACY8444n9Qp9xxQsO
99Zu7vH901vc4R5HPLCf6CPOeMIFz+xX+oLD/bUbvMHtfb5/eo8jHnDCI/uZ
PuGCZ1zxwn54sO4N3uAWd7h/wPdPH3DCI854Yr/QZ1zxgsPDtZuHfP/0Fne4
xxEP7Cf6iDOecMEz+5W+4PBo7QZvcPuI75/e44gHnPDIfqZPuOAZV7ywHx6v
e4M3uMUd7h/z/dMHnPCIM57YL/QZV7zg8GTt5gnfP73FHe5xxAP7iT7ijCdc
8Mx+pS84PF27wRvcPuX7p/c44gEnPLKf6RMueMYVL+yHZ+ve4A1ucYf7Z3z/
9AEnPOKMJ/YLfcYVLzg8X7t5zvdPb3GHexzxwH6ijzjjCRc8s1/pCw7pf/8D
vpptIA==
      "]]}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 512}, {0, 0.10846522890932808`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208520151383*^9},
 CellTags->"b:3.3.4",ExpressionUUID->"a4cc93c1-779b-4ee2-97ee-11c9b00d1036"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Quantum Fourier Transform register 1", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:26", "b:3.3.5"},ExpressionUUID->"7c24ac23-2009-4b56-aa2e-336e4ba30209"],

Cell[TextData[{
 "We apply the inverse of the quantum Fourier transform to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", 
    RowBox[{"1", "P"}]], TraditionalForm]],ExpressionUUID->
  "f0fe8204-61e1-48f7-9719-6dc8c11e31ab"],
 ":"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.5",ExpressionUUID->"e3990e54-8488-46f9-b5c9-f8b7644c9bb0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"240a7c30-dd95-41ad-85da-b1f8816e1a43"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["\[Psi]", 
         RowBox[{"1", "P"}]], "[", 
        RowBox[{"[", 
         RowBox[{"i", "+", "1"}], "]"}], "]"}], "*", 
       RowBox[{"CompIQuantumFourierTransformV", "[", 
        RowBox[{"IntegerDigits", "[", 
         RowBox[{"i", ",", "2", ",", 
          SubscriptBox["n", "1"]}], "]"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"QFTVec", "/", 
    RowBox[{"Norm", "[", "QFTVec", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Norm", "[", "QFTVec", "]"}]}], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"c086152a-9f6f-4ad4-af89-02d73a44d001"],

Cell[BoxData[
 TagBox["1.`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208525619025*^9},
 CellTags->"b:3.3.5",ExpressionUUID->"6b514fbd-8031-473d-a711-93a0cec748c8"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"e21b57c6-a08b-4017-9c75-7c81905be74a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< Seconds\>\""}], "]"}]], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"0563c3d8-b7e5-4ea6-a6f6-9886fa08c85d"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The QFT took \"\>", "\[InvisibleSpace]", 
   "5.585231`6.897556137674074", "\[InvisibleSpace]", "\<\" Seconds\"\>"}],
  SequenceForm["The QFT took ", 5.585231`6.897556137674074, " Seconds"],
  Editable->False]], "Print",
 CellChangeTimes->{3.8572085257623034`*^9},
 CellTags->"b:3.3.5",ExpressionUUID->"e3ecae34-1fec-48bf-aefc-3ca643621745"]
}, Open  ]],

Cell["\<\
We store the possible measurements and their probabilities in MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.5",ExpressionUUID->"bd1bf6e7-69a2-408c-82f3-7553b09337c7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"4bdce0bf-05bb-49be-83ab-788348d94d1d"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{"QFTVec", "[", 
          RowBox[{"[", 
           RowBox[{"num", "+", "1"}], "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "^", "2"}], "]"}]}], ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"a21e999c-5d12-4918-b2f3-dc36e0c8e8d8"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Normali", "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{"Normali", "=", 
     RowBox[{"Normali", "+", 
      RowBox[{"MeasProb", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", 
      RowBox[{"Length", "[", "MeasProb", "]"}]}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", "Normali"}], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"f75c87f8-51f9-4542-a676-d1ba0a480785"],

Cell[BoxData[
 TagBox["1.0000000000000004`",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.85720852579006*^9},
 CellTags->"b:3.3.5",ExpressionUUID->"9db99c1f-e3d2-482d-9d0d-faee1a4246f2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Meassured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Meassured", " ", "number"}], ",", "Probability"}], "}"}]}], 
   ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.12`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.3.5",ExpressionUUID->"b18139f7-9804-4806-b169-4e1114f219bb"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.0055000000000000005`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJw11HlM02ccx/GfVUNTjANBV5Uh55gOBMtAnEc/pcWjFughIyxudFOGogwF
MzECktEuGgQ1ElszQiPHppMIgjIUDTCmIJcr6pCFORTkWrWIisiRePT7PMmT
X155/z7/Pu7fJqnjeBzHKd/ed1+p36Nm2ahVfKg/wCEhXiTm3p8RsdvCrsfP
d99YZzMHQ2mw4EQKMx/JeTsK8jOYHbC0Ich/SS6zMxKMbY1ehcxC5BUOzbp+
ldkFp0qu+012MruhuH9cPfaa2QOCId99t9xukr3gM5MrPalg/hinZb9sS01n
/gRF0StyTZXMy5AWrbUEWZl9kTrvUkxiQCN5OWKe/uaZvp85AHbbl+0u/JN5
BZaelRrdhE1kEUwuF1olycyB4B7qD0TfYf4MNSuFr66uvkUOgma6Jaj9PHMw
nCuysqY8m8krMaQX7awpZg5B4nRTUfbyFvIqTOYtWjNRz/w54p1H6s/EtpJX
I67bu+xTfht5DczbDo2rrjGvRZWhsaMnrZ28Dsb/rZXV8ttkMWR6g2KH019k
4Ocj0REuR8iZAJ/LT5lrZ6YuwSwn8U99OeRMCQT/XX4W5dpBPRQJ59Je+V0h
Z4aiZ6cgJCL2DnUpFhx9UnDR8S51KcT++Q1JZjIng2HQ8W6d6R51GbbXVnnw
Mv6mHgbHy/t+qEnspB4Gba8xtGTPferrkfVN+ouyw13U12N4YP+xOdX/UN+A
scX13cEzuqlvQELyDf2M+H+pb0R9wRldseUB9Y2YffhJyIC8h/omTPq0lm4N
fEh9E74ye427ix5RlyN2r/9HfHkvdTlcDUbdlow+6pvxe2C1/f2Ox9Q349eR
qBx7xQB1Bb4vr0wMtA5SV8Bi2iM6+scw9XAscs178GObhXo4tOd8aqeEVuoR
SPJuv9fVOUI9ArXNko4vIkepR2KX9Mtrlonn1COxJUc3J6nvJXUlPhS+O+M2
Qwnfiq8RVjJB/yuhOxGTf8A0bXOdElO8s3t7j3H0Xqig1Tpf+q6IZzNUqIgL
6b29ZLbNmSpk507W8Dz4NtepcNPPazDFbE97NTJ4x4ePixxor4ZkdOHWuvlO
tFejLKqyqaFlAe3VEGWnX9GdX0x7DYzNB1PLXd1pr4Gd4KVUX+5New1Euf1d
vHBfcckHY3aeVRqw9+4Nyz13eA==
       "]], LineBox[CompressedData["
1:eJxVlGtIFGEUhkcrLW+MFVaUuEItlKJLWSBRvVgUi5Bg2l5mL9/MrJhRWiAE
0mWzQsGULAv6ka5WENJFVAoUcln7YRQWaVmkskFJkj+kKLXSUvYUZweG4eWb
Z+b7znnfk6qV5RdFSpJknr8XnuU7Ep/UGCw4M2aSDxdv3iUtXLAgmJhX1Bdr
CmmvBQeflcuzDRtD2m9BVGd7ZeH39SEtWfGqsCnzaW0q8VacvXbvpfFrMvFW
vL384caKyTXEWzF+50K3qTeJeBvk7KPeTVkribdhsPNQW9O65cTb8PzuUAdU
mXgb0mqOJLiG44m3ozh+2KGUxBJvR19cd9H2A8uItyNGiZv8Vh1NvB3b1BO6
eXYJ8Qrq4/eNDd5eTLyChtI/en7FIuIVtDx6EEgpjyR+/v03gYLR2gjiHTie
3VqZ3isR70BbnX3omEza60CkUlWsf5rbGeIdSClta53onw1pyYlTFSXNzne/
QxpObHhfeDUngrTXicCtYfFw9y/inWi050bnNv4k3oX9V8aTphJIw4WEnkB/
VMMM8S7In805cjppvwsDGQMfx19PE+/GHuNozM1LpOHG0vu+mNMKaa8b3pmq
ri9bSfvdmJ7LSq5O+ccLrFq9cJE2CIzwdQgonBcCaWHfFzDx//sEevj+/AJb
+P6DAmv5+SQVJ/n5DSrqeH2gopbXT6jIC6vv/Dqvv09FHO+PX0UG719QRTPv
r6ShnvffoKGF+wMarnP/CA3nwvyl4Tz3n0+DkfvTryGR+zeo4TH3t6TDzP1v
0JHJ8wEdAZ4foeNFWL50RPD8+XSM8Hz69fD8BnVc5PmWPGjn+Td48KODzQd4
UMDnh/BggubLXnUquavM83/+/AU4sFrf
       "]], LineBox[CompressedData["
1:eJw11GtMU3cYx/Fj0diAGh3T4RxQFLJlCmixSMaiP0WJI4xtZIxhiNSpPZee
0wNCvERAVDAqcosuZQmxVcCMiYqiC4rGIiJYLgqoiGGuDkQwVZjXilfIeXyS
f55883/xeff4/SbHrlcxDPPj6BvbQa7x6Y/9DdjWP3+qwGqXMGMDA77L7+9W
fT9Pab0Bbu7PI3KqApTOMqDAvnVzlY+f0lYDdLkZZ7OPzlLaZkBNXHVTffMM
pR0GrHgyM9E23VNphkWZqvBhoXaq0hoW9kD/gdR2D/JZlOa/rlXNVpPPonJ9
WO813wnkszDrPz1tKFWRz8LT7c+U3gKGfBblRQklWyxvFys+C/Wp1VhRPqI0
w+Ezr7FxKa3hkJyXPUnue640OEgRq847R54qredgty/t+OWHJ0pncUgJaLvZ
3TWstJWDpeLLi2+8hpS2cUjyOXB3R6uTfA5B1mTtvksPyedRX1UthQwNkM/D
OhyX5xH9gHwer0JqPG533CefxwJzcfbPmX3k89iaEuytjuoln8f2dn+Xn/Y/
8nmEf9VSmRhyj3wesbsfhT2IcpAvINVyKLvMeZd8AbkbGnLGsf+QL+DWrLqe
0HE95I/+D2wqmFRzh3wB9jUZz07s7iZfQFpv8bLy5NvkC7hyJm1jrdRFvoCm
i3/PVmXeIt+IyIFpN2yWm+QbER5cUi+33yDfiG/2PTp4chq13oi5gntYTFIn
+UbUV6S/DDzbQb4RK/8983+cD7XNCMlzya6+vHbyjQBTkjplIjUjInJvfMwX
e66TLyI9xxzNeVJDRKhzqLom6hr5IlYVN3Y40tvIF1G0bpvrp/Ot5IvI6wk4
MVdNbROB6cN1h5JayBdh/v3zb0fqmsmXYH7XVJobRK2RMHmXlq8ts5MvIfHU
zp1v5lDrJfS/bda1Hb1KvoTIRV4vz4VTWyXo7uVsie9sIl9CjPfxlqUbqB0S
FlZEFGu8qBkTLOu+Fg9fbiTfhITHf83J2EQNE7o+OZ0gzafWmzAYr3fqhq6Q
b8LJXxfkW6qprSYcWX5k7eYMapsJghtTuT+a2mFCxeC8tKsaakbGhH5X7ItX
DeTLeF9+IfB1FzVk+JYNjr9wjlovQ/1Ha6P/YeosGc8u6YJ986mtMl4c4A6W
ZFLbZDDHQt2LUqkdMjQzu+8/FRsWd3rraieP9sd79wFKW5p+
       "]], LineBox[CompressedData["
1:eJw91H9M1HUcx/Gvlw52uDqCFH+EgDBmQVzHMLZavAj7h5GRSYxmcaWEksTn
8/18l7gAWdw1HIK4mHctxiE/SpPFBWooOrhIQX7JoYY0sjOQXx1Bmor82IK+
7/XePvvsuX3++Dz+eQd+mLUjTSNJUuLSWb7X8rSgx08xHBrV6zLSDTHS8ugY
AtYN3r2///Kr/3UAg6Vuq/aYoNYzyGV7K8rzqMGwpS0qYlMJdSJDhrWnPbiK
2shQVjWx8tIFasZwvPZS+PwAdT5DzejsjoePqUsZtBNhytWAK2pXMoQ+IdV9
mUBtZ/hq2ze7s3OpWxmqk18ssTVS9zHkJBvdUdPULobsp8+kZOrb1Z5hSPnr
u825B6glDo89z+2v+plax7HlZJw1wK+D/By2jd93x8rUeg7pjvlg8nVqcDS/
5PfowstXyc/x9mJXVO9paiOHb0NBwcLmTvJzTJgN+5prqPM5Mhc7qote6CI/
x3zZ+lfmHNSVHOm+M44Tqd3k50gbCql/3rOH/BzO3Ydm37pI3cdxztLe78rp
JT+H9c/pxqb4a+Tn2Ga2JOz16SO/jK8PJ2/feJhaJ8NTKhdPejjJL2OlT8wX
I8XUehna38/+neTfT34ZGadyHoWfp06U4dqnjd6eep38MtYcmar4wfsG+WXE
RJS3ZTmp82VYxr1vtNpukl/GnpZzQZq8X8gvw/us8mlz5gD5ZRiHra/Vslvk
l1HwQe4/9YWD5JcxOXbg6OqmX8kv4+EGx9DWFUPkX/qvfNm8Iv038gs4Kk6Y
aty3yS+wqnAqeizeRX6B+dDuul2Rd8gv8J4zeDbQ8Af5BVJ5xLOe8cPkF/C3
WE0780bIL/BjZJPXrf675Bf4diap2CthjPwCn9gbMyOnx8kv4LYxw5GfJskv
sN6/7PbnPW7yCxhPhbYs+E2TXyArpPfm4MAM+QVaOmP733nzHvkFPo5796J7
7j75BXYWm1ZnjTwgv4K1fsszS34FYQ3v4/XaOfIrMB1LKT9oWyS/ggXNST58
VFL3BRQYjb5nPqrWqJ2ooCEtevjaplVqGxUUlcw3a4I81WYKroQHjwunl9r5
CvI0pZOlBp3apQpi763b1fqMj9qVCuqTGjvautaobVdgKMo9bzq9Qe1WBdbO
z7Lt/oFq9ynw0D6IM9tD1HYtvS8ZHdS8ERYzUbDoKJxS/t93/wIPs55N
       "]], LineBox[CompressedData["
1:eJxVlG1IFEEcxlezLF+OtcKKEs8ooRRdygoJ6sGCOIQ8TPvgZZyJmFF2gbcz
s/RyWaFQipYFfUhPLQjpZVEpSMjj/GIUdqRlkcoFJUl+uIpKMy1lhpgbWJYf
sy/z/P/P8085cjK/LFJRFNv8tXAvzov6ur7fjXPjmnqsfMtuZWEF3Agm5JX1
x2qcg24cfF6lzjZt4hxyY0l3Z3Xhjw2cFR2vClsyn9WlcFZ1nL9xP5D6LYmz
Vcfbqx9urQit4azpmLh7qUfrS+QMHWr2Cc/mrJWc7TqGuo+aLeuWc3bqeHFv
uAslKmeXjrTLxy2HR+I5e3SUx48cclTEcm7Q0R/XU7bzwDLOXh0xjrjQ99po
zqaO7SWk1Da7mLNPR2P8vvGhO1FCv46myr+l+cYioV9H2+OH/uSqSKF//vk3
/oKxugihn+BUdkd1ep8i9BOY9UXDLlWwlSDSUVNe+mluF9dPkFxpdkwOzHIG
wRmjorX43R/OdoKN7wuv50QIdhL4b484H+2Z4ewiaC7Kjc5t/s3ZQ7D/2kTi
L4vgBgJLr39gSdM0Zy+B+tmWo6YLNgkGMwY/Trye4uwj2Js6FtPeIDhAsPSB
N+asQ3CQwDNd8+TLNsEhgqm5rKTaZMEKxarVC0uwSjEq71spHPL7GkWa/H1Q
aPL/7RS98vmcFFvl87so1sr6PBSnw/RT1Mv18VLUyfUzKfLk+vrm9+X6Byji
5P4EKTLk/oUoWuX+KgyNYf1naJP9YWW4KftHY7gg+wsMF2X/2RlSZX86GRJk
/7oYnsr+9jDYwvzPkCnnw8vgl/NjMryU8+VjiJDzF2AYlfMZZOH5DTFckfOt
GOgMy7+Bn13SfLAaKJDnh2ZgUsyXdnPGMrrD+D9//gEOa2er
       "]], LineBox[CompressedData["
1:eJwtz39M1HUcx/GvR64bWEPRxEw4FJZLkfPoiEXDl5GuGCGxkHAsrtI05o1D
WOi+n8/3SwnOQuBYNmhj3ingIklP0Yai85AQPH544A+kkZ2BCO6My5+IWkGf
93v77L3HPv+8n2Gf5qRu0kiStG7qTe/5TBNfMEeGOqIPzN5sWCVNj07Ge2Uj
A5r3lwvrZfj5P0godkQIQ0a5S97uCAkTTpFhLOEniw4tFDbJaEpr7GjtfEnY
ImPN3QWZznlBwoUyajXW21ZDoLBVhisyfDSvN0DYLqOm7EmzZrFW2CGjYVPs
0MXQmcJOGZWmucc/r9EIu2UE+f2YO1QuCXtk1FVkVO+wPYv/3z4Z2mMfY03d
pLDEMD94eiaEAxkspUWzcoYfCOsYzAkbTnsn7wnrGVyu1X3r190VBkNuRM+V
gX6fcAqDrf7Vs0+Dx4VNDFkhe69/3e0VtjCssFsMe87dFi5kaHU0mqPHR4Wt
DHZfWmlA0i1hO8Pj6KaAa303hR0MKyurij5UhoWdDHJu1CJt4pCwm+Gr3vCJ
MMOfwh6GuKVdDZnRN6ifIXX3ndhbiR7q58iz7S+q9V6nfo6SbW3FMzb/Tv0c
Vxe2DMbMGKT+qf/RgvJZTb9RP4frE37/yO4B6ufIH6p6u85yjfo5zp/I/7LZ
3E/9HB1nf1msUa5SP8fa0dmXnbYr1M8RF1XdmtN7mfo53txzZ9/R2WQHx7Js
/9jkrEvUz9Fazx5Fnuyjfo53/zjxd1oI2cNhDlq1a7i0l/o5IFXnvfg8WVKw
9tv05Fe+cVO/AlZcmbQliKxTEOMdb2xKvEj9CjZUtfd5WA/1K6jYqE58cLqb
+hWUDkYcWaYlmxRgnq9lf1YX9Suo/P7ltyZbOql/yv901JSsIFsVvLDL8EVz
rYv6FWQe27nz6RKyQ8HIs05jz6EL1D91/xvBj07Fkd0KjDeKd6Rf6qB+BcmL
Dnet3kb2KXi9PqFKF0yWVNg2vrb1wK/t1K8i46+flvACsk5F/5zjGWY9Wa9i
LN3kNY6fp34VRz9aWWZrJKeoOPjOwc+2c7JJRbaf1PBdEtmion5sef4FHblQ
xcyRidSHj9uoX8W/dWcin/ST7SpCa8eeO3OK7FCh/aG7PfwA2ani/jljVGgZ
2a3i4d4t+6oVskeF9HOMf0Ue2adCt2Dg5r2tbfH/ARDzmwE=
       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Meassured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 511.}, {0, 0.0002808221763107368}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208526061545*^9},
 CellTags->"b:3.3.5",ExpressionUUID->"98548c40-7051-4cb4-a49d-63860f5ff0aa"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.0055000000000000005`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdV3lUU3cafYAWBtED1TruROsuorLVreUquOCCbAHZHwkhC4S4ndFatKmC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       "]], LineBox[CompressedData["
1:eJxdl3lUU3cexQNoYRA9oVrHndi61Iqaoji2OuN1aWe0itFKHdFq3MalIi95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       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Meassured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 511.}, {0., 0.12}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208526113029*^9},
 CellTags->"b:3.3.5",ExpressionUUID->"68048384-4d28-4dc3-be8d-b911c3697ab9"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead, we study all posible measurements and \
their probabilities\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:27", "b:3.3.6"},ExpressionUUID->"da7869f9-7d7c-4587-91ba-cc7e24024f8e"],

Cell["\<\
Finally,  we go through all the measurements, compute the periods and check \
the 
factors we would find and their probabilities:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.3.6",ExpressionUUID->"57c0bbfd-1392-4b3a-a5b9-9bda4650de71"],

Cell[CellGroupData[{

Cell[BoxData["xguess"], "Input",
 CellTags->"b:3.3.6",ExpressionUUID->"37c3e03e-ea93-43c1-a115-c7ef670c6446"],

Cell[BoxData[
 TagBox["26",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572085261397057`*^9},
 CellTags->"b:3.3.6",ExpressionUUID->"2b2f7218-96e7-48df-8621-7c6f86f890ed"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.3.6",ExpressionUUID->"d77df3a4-0dda-4c2f-bbe4-15048556c0af"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.3.6",ExpressionUUID->"e133b724-ace1-4387-ad6e-30b4598866d1"],

Cell[CellGroupData[{

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}]}], "Input",
 CellTags->"b:3.3.6",ExpressionUUID->"6e2edc63-0cff-491b-b526-62b7711b89de"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"Period", "Number", "\<\"f1\"\>", "\<\"f2\"\>", "Prob"},
     {"\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>"},
     {"6", "82", "7", "5", "0.0010945495043870226`"},
     {"6", "83", "7", "5", "0.0022624830031657797`"},
     {"6", "84", "7", "5", "0.00702133926039535`"},
     {"6", "85", "7", "5", "0.1138972652391974`"},
     {"6", "86", "7", "5", "0.02888310407691217`"},
     {"6", "87", "7", "5", "0.004690013600719175`"},
     {"6", "88", "7", "5", "0.0018602301808350334`"},
     {"6", "424", "7", "5", "0.001860230180835031`"},
     {"6", "425", "7", "5", "0.0046900136007191635`"},
     {"6", "426", "7", "5", "0.02888310407691218`"},
     {"6", "427", "7", "5", "0.1138972652391974`"},
     {"6", "428", "7", "5", "0.00702133926039535`"},
     {"6", "429", "7", "5", "0.002262483003165784`"},
     {"6", "430", "7", "5", "0.0010945495043870222`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572085262307377`*^9},
 CellTags->"b:3.3.6",ExpressionUUID->"a170edfd-4f1a-464f-8a56-4b93dd632905"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The TOTAL PROBABILITY OF SUCCESS is thus:\"\>", 
   "\[InvisibleSpace]", "0.3194179697312238`", 
   "\[InvisibleSpace]", "\<\"  with xguess=\"\>", "\[InvisibleSpace]", "26", 
   "\[InvisibleSpace]", "\<\" and measured register=\"\>", 
   "\[InvisibleSpace]", "16"}],
  SequenceForm[
  "The TOTAL PROBABILITY OF SUCCESS is thus:", 0.3194179697312238, 
   "  with xguess=", 26, " and measured register=", 16],
  Editable->False]], "Print",
 CellChangeTimes->{3.8572085262355547`*^9},
 CellTags->"b:3.3.6",ExpressionUUID->"ca13c08e-1ebf-4394-b021-10a70b8a224a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.3.6",ExpressionUUID->"0c26bbdb-24d2-4a49-ace0-1549096fe8cd"],

Cell[BoxData[
 TagBox["9.758527`7.440929261567344",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208526242318*^9},
 CellTags->"b:3.3.6",ExpressionUUID->"632b187a-5b42-4522-8afe-3e88fb680420"]
}, Open  ]],

Cell[BoxData["Quit"], "Input",
 CellTags->"b:3.3.6",ExpressionUUID->"4e8a88c3-d0c8-4560-b680-35bb9930e6ea"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Shor's Factoring algorithm using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " DFT formula"
}], "Section",
 FontFamily->"Times",
 CellTags->"c:28",ExpressionUUID->"13fd45fe-183f-48a7-b25c-9ed750d8b40d"],

Cell[CellGroupData[{

Cell["\<\
Step 0: Preliminars: Define functions for QFT [check QFT.nb ] and NumberTheory\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:29", "b:3.4.0"},ExpressionUUID->"fa752dc4-6539-4dda-9b31-ef8167f375d1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<QDENSITY`Qdensity`\>\"", "]"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.4.0",ExpressionUUID->"db8fee6a-8830-4d13-a934-10f86c1f3716"],

Cell[BoxData["\<\"Type \\\"?intro\\\", \\\"?QDENSITY`Qdensity`*\\\" or \
\\\"?about for help.\"\>"], "Print",
 CellChangeTimes->{3.85720858859521*^9},
 CellTags->"b:3.4.0",ExpressionUUID->"17059a87-af8c-4524-8811-404757c4c88c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "QDENSITY`Qdensity`*"}]], "Input",
 FontFamily->"Courier New",
 CellTags->"b:3.4.0",ExpressionUUID->"6be6dcfe-df44-4c6f-b2ba-643ae1a1844a"],

Cell[BoxData[
 TagBox[
  StyleBox[
   FrameBox[GridBox[{
      {
       DynamicModuleBox[{Typeset`open$$ = True}, 
        PaneSelectorBox[{False->
         ButtonBox[
          RowBox[{
           
           DynamicBox[FEPrivate`FrontEndResource[
            "FEBitmaps", "RightPointerOpener"]], " ", 
           StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
          Appearance->None,
          BaseStyle->"InformationGridLink",
          ButtonFunction:>FEPrivate`Set[Typeset`open$$, True],
          Evaluator->Automatic,
          Method->"Preemptive"], True->
         PaneBox[GridBox[{
            {
             ButtonBox[
              RowBox[{
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "DownPointerOpener"],
                ImageSizeCache->{10., {3., 7.}}], " ", 
               StyleBox["QDENSITY`Qdensity`", "InformationGridGroupHeader"]}],
              Appearance->None,
              BaseStyle->"InformationGridLink",
              ButtonFunction:>FEPrivate`Set[Typeset`open$$, False],
              Evaluator->Automatic,
              Method->"Preemptive"]},
            {
             PaneBox[GridBox[{
                {
                 ButtonBox[
                  StyleBox["about", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "about", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["FidelityB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "FidelityB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["OneOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "OneOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "res2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["U", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "U", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Adj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Adj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Oracle", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Oracle", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res2$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "res2$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "v", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["AF", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "AF", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Had", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Had", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTrace", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PartialTrace", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Rotqbit", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Rotqbit", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Vs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Vs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Bra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Bra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Hadamard", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Hadamard", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTraceGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PartialTraceGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RotX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["v$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "v$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "BraV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["HALL", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "HALL", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PartialTranspose", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PartialTranspose", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RotY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ws", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ws", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "BraX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["i8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "i8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ProjG", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ProjG", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RotZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RotZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["BraY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "BraY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["intro", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "intro", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Xb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CNOT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "CNOT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["j8", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "j8", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PTrGen", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PTrGen", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Schmidt", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Schmidt", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Coeff", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Coeff", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Ket1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Ket1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Purity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Purity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma0", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Sigma0", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Xd", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Xd", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Comm", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Comm", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "KetV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Sigma1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroMatrix", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ZeroMatrix", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompMeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "CompMeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "KetX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["PY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "PY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Sigma2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ZeroT", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ZeroT", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CompPTr", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "CompPTr", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["KetY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "KetY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomBra1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Sigma3", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Sigma3", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalH]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[ScriptCapitalH]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledX", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ControlledX", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "mc", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBra2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomBra2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Snos", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Snos", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[ScriptCapitalP]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[ScriptCapitalP]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledY", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ControlledY", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "mcl", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomBraN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomBraN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SP", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "SP", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[DoubleStruckCapitalI]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[DoubleStruckCapitalI]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["ControlledZ", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ControlledZ", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mcl$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "mcl$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomKet1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["SpinObs", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "SpinObs", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Rho]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASE", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "CPHASE", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["mc$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "mc$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKet2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomKet2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stA", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "stA", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Rho]$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Rho]$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["CPHASEV", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "CPHASEV", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MeasureKet", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "MeasureKet", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomKetN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomKetN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["stB", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "stB", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Sigma]", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Sigma]", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["cRQN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "cRQN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiProjCBasis", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "MultiProjCBasis", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomQubit1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Swap", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Swap", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Phi]A1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Definef", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Definef", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["MultiQProj", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "MultiQProj", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubit2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomQubit2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["s$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "s$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]A2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Phi]A2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["DForm2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "DForm2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Mutual", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Mutual", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["RandomQubitN", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "RandomQubitN", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ThreeOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ThreeOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Phi]B1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["Discord", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Discord", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nb", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "nb", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsDate", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "rcsDate", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["Toffoli", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Toffoli", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[Phi]B2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[Phi]B2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "EnTropy", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["neg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "neg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["rcsRevision", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "rcsRevision", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ts", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["\[CapitalOmega]s", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "\[CapitalOmega]s", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic]},
                {
                 ButtonBox[
                  StyleBox["EnTropy2", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "EnTropy2", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nneg", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "nneg", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "res1", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["ts$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "ts$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], ""},
                {
                 ButtonBox[
                  StyleBox["Fidelity", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "Fidelity", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["nq", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "nq", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["res1$", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "res1$", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], 
                 ButtonBox[
                  StyleBox["TwoOp", "InformationGridButton"],
                  Appearance->None,
                  BaseStyle->"InformationGridLink",
                  
                  ButtonData:>{
                   "Info-8e1e43d2-6134-4a3d-ba16-2426a4977b7c", {
                    "TwoOp", "QDENSITY`Qdensity`"}, False},
                  ButtonNote->"QDENSITY`Qdensity`",
                  Evaluator->Automatic], ""}
               },
               DefaultBaseStyle->"Text",
               
               GridBoxAlignment->{
                "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
               GridBoxItemSize->{"Columns" -> {{
                    Scaled[0.19]}}}],
              ImageMargins->{{10, 0}, {0, 2}}]}
           },
           GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}}],
          FrameMargins->{{0, 0}, {8, 0}}]}, Dynamic[Typeset`open$$],
         ImageSize->Automatic]]}
     },
     GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
     GridBoxDividers->{"ColumnsIndexed" -> {{False}}, "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "Rows" -> {
         Offset[0.2], {
          Offset[0.8]}, 
         Offset[0.2]}}],
    BaseStyle->"InformationTitleFrame"], "InformationGridPlain"],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589351935*^9},
 CellTags->"b:3.4.0",ExpressionUUID->"752c25da-ce10-4c47-a188-36bbf1a10539"]
}, Open  ]],

Cell["\<\
In this notebook we make use of the Discrete Fourier Transform already \
present in Mathematica.\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.0",ExpressionUUID->"690a4ac2-1826-42c2-bc24-d918af016df9"],

Cell["\<\
To get the period r which will appear in a quotient of the form s/r we will \
use the 
continued fraction. i is the ith cycle of continued fraction and j is the \
number measured. \
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.0",ExpressionUUID->"00f81e3c-9822-4a10-8d8a-c2aa293af7c2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"denomi", "[", 
    RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
   RowBox[{"Denominator", "[", 
    RowBox[{"FromContinuedFraction", "[", " ", 
     RowBox[{"ContinuedFraction", "[", 
      RowBox[{
       FractionBox["j", 
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]]], ",", "i"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"b:3.4.0",ExpressionUUID->"ce6b1d7f-de2d-4661-aa19-7de144df4975"],

Cell["Flag for the case when factoring an even number ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.0",ExpressionUUID->"d4e65207-d148-4f77-8af6-7d37c0e23612"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"Even", "=", "False"}], ";"}]}]], "Input",
 CellTags->"b:3.4.0",ExpressionUUID->"6b908e59-b062-41bf-9f5c-a4c3954841d9"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 1: Choose the number to be factorized: 15, 21, 33, 35, 39, 55, 77 and \
determine size of registers\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:30", "b:3.4.1"},ExpressionUUID->"40b0c817-b042-450e-8a68-59611963dd0a"],

Cell["M is the number we are trying to factor:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1",ExpressionUUID->"65476cea-fc27-466e-b37d-56df137ec07f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"M", "=", 
  RowBox[{"11", "*", "7"}]}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"8b33a7d1-c0a1-4dce-a927-f7a260b48888"],

Cell[BoxData[
 TagBox["77",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589387001*^9},
 CellTags->"b:3.4.1",ExpressionUUID->"2ef3ed32-9aff-4e56-bf06-4cbc7c3bc8ed"]
}, Open  ]],

Cell["MP is the number we will work with and possibly modify:", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1",ExpressionUUID->"c9562f80-d03b-4483-8912-d1e88dfa9b0d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MP", "=", "M"}], ";"}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"dcf258a9-23ac-4c26-83dd-d4cab8ebc184"],

Cell["\<\
 We perform a couple of tests to check whether the number is a power of 2 or \
prime:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1",ExpressionUUID->"895b6e93-24d7-491d-a4d5-3027e52c3ffb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"While", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{"MP", ",", "2"}], "]"}], "\[Equal]", "0"}], ",", 
    RowBox[{
     RowBox[{"Even", "=", "True"}], ";", 
     RowBox[{"MP", "=", 
      RowBox[{"MP", "/", "2"}]}]}]}], "]"}], ";"}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"999cc834-bf63-4ae8-9c00-e52f7e5daa6f"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{"MP", "\[Equal]", "1"}], ",", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{"M", ",", " ", "\"\< is a power of 2\>\""}], "]"}], ";", 
    RowBox[{"Quit", "[", "]"}]}], ","}], "]"}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"060ec69a-c967-4e67-9d74-5dcd22e5376d"],

Cell[BoxData[
 RowBox[{"If", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PrimeQ", "[", "MP", "]"}], "\[Equal]", "True"}], ",", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"Even", "\[Equal]", "False"}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"M", ",", "\"\< is prime, therefore has no factors\>\""}], 
       "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}], ",", 
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<The prime factors of \>\"", ",", "M", ",", "\"\< are \>\"", ",", 
        "2", ",", "\"\< and \>\"", ",", "MP"}], "]"}], ";", 
      RowBox[{"Quit", "[", "]"}]}]}], "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"b8c78657-830f-4660-95c7-aa419f101d3b"],

Cell[TextData[{
 " We need the",
 StyleBox[" minimum ",
  FontWeight->"Bold"],
 StyleBox[" number",
  FontWeight->"Bold"],
 " of qubits for both registers. \nHere we should be careful, a high ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "e2f82ced-6d8d-4236-8a4e-43c997697adf"],
 " number will lengthen the computation.  \nBy default we give estimates for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "ad7af503-0e77-4820-846d-875ea8ecf7dd"],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "3ec6f53a-89a7-4c37-bfca-d2b18fed97e9"],
 ",based on the fact that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "ed5162db-4052-4054-901a-8a3a22af5079"],
 " should be able to \nrepresent numbers up to M^2, while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "ccbb9ebe-3fe3-4a0c-a85c-99abb2c21162"],
 "should allow to write up to M/2. However in \nmany cases it could be more \
illustrative to lower the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "e787b257-5c61-493b-aaca-c27843f55b0d"],
 " \n[ ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "2"], TraditionalForm]],ExpressionUUID->
  "3a1304c5-cd41-4b93-9184-bb0d7e0def9a"],
 "is not relevant for the code's performance] and check the resulting \
probabilities."
}], "Text",
 CellChangeTimes->{3.857208600769747*^9},
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1",ExpressionUUID->"de9b52f6-1820-44d5-b767-072adf4daf93"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Q", "=", 
  RowBox[{"2", "^", 
   RowBox[{"Ceiling", "[", 
    RowBox[{"Log", "[", 
     RowBox[{"2", ",", 
      RowBox[{"MP", "^", "2"}]}], "]"}], "]"}]}]}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"f4a53142-a969-4623-9d81-eb839c5f7651"],

Cell[BoxData[
 TagBox["8192",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589423904*^9},
 CellTags->"b:3.4.1",ExpressionUUID->"56cadb2b-bcf1-401b-94b4-69638b0b3295"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  SubscriptBox["n", "1"], "=", 
  RowBox[{"Ceiling", "[", 
   RowBox[{"Log", "[", 
    RowBox[{"2", ",", "Q"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  SubscriptBox["n", "2"], "=", 
  RowBox[{"Ceiling", "[", 
   RowBox[{"Log", "[", 
    RowBox[{"2", ",", "MP"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 SubscriptBox["n", 
  RowBox[{"1", " "}]], "\[IndentingNewLine]", 
 SubscriptBox["n", "2"]}], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"e7a9ae3e-8bbe-43a9-96da-071d42450322"],

Cell[BoxData[
 TagBox["13",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.85720858943259*^9},
 CellTags->"b:3.4.1",ExpressionUUID->"c575adfe-2eed-4b60-9e2c-b5953d44c001"],

Cell[BoxData[
 TagBox["7",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572085894366207`*^9},
 CellTags->"b:3.4.1",ExpressionUUID->"ebee835a-17ab-4d42-8d82-5e6664d0a36d"],

Cell[BoxData[
 TagBox["13",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589439679*^9},
 CellTags->"b:3.4.1",ExpressionUUID->"168c94dd-a4e1-4f21-8de5-bc60d0c0539d"],

Cell[BoxData[
 TagBox["7",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589443089*^9},
 CellTags->"b:3.4.1",ExpressionUUID->"1db8c5d5-f630-4e04-bfb3-b2c2a88455ff"]
}, Open  ]],

Cell["\<\
Gerjuoy's test: a failure to pass this test means we cannot ensure that we \
will 
in fact find the period using the continued fraction expansion [ After \
Eq.(54) of Ref.[2] ]\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.1",ExpressionUUID->"456b650f-390c-4a4f-b37b-46bce1bfd17c"],

Cell[BoxData[
 RowBox[{"If", " ", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", "^", 
     SubscriptBox["n", "1"]}], "<", 
    RowBox[{
     RowBox[{"MP", "^", "2"}], "/", "4"}]}], ",", " ", 
   RowBox[{
   "Print", "[", 
    "\"\<CHECK YOUR \!\(\*SubscriptBox[\(n\), \(1\)]\), YOU MAY HAVE PROBLEMS \
GETTING THE PERIOD\>\"", "]"}], ","}], "]"}]], "Input",
 CellTags->"b:3.4.1",ExpressionUUID->"ec5f04cd-9377-4837-a62d-9edf9333f31c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 2: Load the first register with the integers less than or equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]], "-", "1"}], TraditionalForm]],ExpressionUUID->
  "b85a26c7-03e4-4958-8c8d-085fc07518ba"],
 "(a.k.a. Hadamard all qubits in register 1)"
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:31", "b:3.4.2"},ExpressionUUID->"d119ab91-7c89-498d-b356-84531733b861"],

Cell[TextData[{
 " Register one needs to be set to the following: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"|", 
       SubscriptBox["\[Psi]", "1"]}], "\[RightAngleBracket]"}], "=", " ", 
     RowBox[{
      RowBox[{
       SuperscriptBox["2", 
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"j", "=", "0"}], 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
    "\[RightAngleBracket]"}], TraditionalForm]],ExpressionUUID->
  "1d1b560a-5e02-4b04-b838-88afb518490b"],
 " ; which is a \nstate where all ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "fa5e5dff-8aed-459a-ab07-5063546e2ff9"],
 " qubits have been Hadamarded.\nIn this computer simulation we will not \
explicity construct it.  "
}], "Text",
 CellChangeTimes->{3.857208615175002*^9},
 FontFamily->"Helvetica",
 CellTags->"b:3.4.2",ExpressionUUID->"289a1164-3337-4f7b-831d-4dfe47842b31"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Step 3: Select an integer (xguess) coprime to M and load ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["xguess", "j"], 
      RowBox[{"(", 
       RowBox[{"mod", " ", "M"}], ")"}]}], " ", "into", " ", "the", " ", 
     "second", " ", "register"}]}], TraditionalForm]],ExpressionUUID->
  "98544007-b871-40ff-9d25-2d01fc660cfc"]
}], "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:32", "b:3.4.3"},ExpressionUUID->"8278dbd4-1671-4f09-9f3d-4542bf1a7152"],

Cell[" This is the random Coprime to MP: ", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.3",ExpressionUUID->"e6fb4063-dc12-4577-9f5e-9afb37053f46"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xguess", "=", 
  RowBox[{
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Range", "[", "MP", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"GCD", "[", 
        RowBox[{"#1", ",", "MP"}], "]"}], "\[Equal]", "1"}], "&"}]}], "]"}], 
   "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", 
      RowBox[{"EulerPhi", "[", "MP", "]"}]}], "}"}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellTags->"b:3.4.3",ExpressionUUID->"af481f0f-ff83-4e3e-a33b-4dc67303bd13"],

Cell[BoxData[
 TagBox["64",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589464863*^9},
 CellTags->"b:3.4.3",ExpressionUUID->"742e003d-8937-4305-be67-a6b89d51c545"]
}, Open  ]],

Cell[TextData[{
 " The full wave function after we build the second register with  ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]],ExpressionUUID->
  "e345a3a1-9101-4f71-9f83-6ff3f9c17d50"],
 "reads:\n ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"|", 
         SubscriptBox["\[Psi]", "12"]}], "\[RightAngleBracket]"}], "=", " ", 
       RowBox[{
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{
           RowBox[{"-", 
            SubscriptBox["n", "1"]}], "/", "2"}]], " ", 
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           SuperscriptBox["2", 
            SubscriptBox["n", "1"]], "-", "1"}]]}], " ", "|", "j"}]}], "   ", 
      "\[RightAngleBracket]"}], "|", 
     SubscriptBox["f", "j"]}], "  ", "\[RightAngleBracket]"}], 
   TraditionalForm]],ExpressionUUID->"a6ea8f48-f2bd-47ef-8639-2e2b30ab9a64"],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "j"], "\[Congruent]", " ", 
    RowBox[{
     SuperscriptBox["xguess", "j"], 
     RowBox[{"(", 
      RowBox[{"mod", " ", "M"}], ")"}]}]}], TraditionalForm]],ExpressionUUID->
  "c128ff7f-ae9c-49a3-bf98-dfbedf000091"]
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.3",ExpressionUUID->"a0412f62-d1dd-474d-833f-b77f8e163b4d"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 4: Now we measure the state of the second register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:33", "b:3.4.4"},ExpressionUUID->"0a840919-cec3-4179-8e83-b7031f518721"],

Cell[TextData[{
 "We take an hypothetical measurement of the second register: (NOTE that by \
construction the \nmeasured value must be one of the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["f", "j"], TraditionalForm]],ExpressionUUID->
  "fe81009d-6d37-4b0c-bfcf-e4b0f5aa01c0"],
 "]"
}], "Text",
 CellChangeTimes->{{3.8572086382043467`*^9, 3.857208644189144*^9}},
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4",ExpressionUUID->"98551af2-721f-4145-add0-0c1e10724682"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"reg2Measurement", "=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    SuperscriptBox["xguess", 
     RowBox[{"RandomInteger", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          SubscriptBox["n", "1"]], "-", "1"}]}], "}"}], "]"}]], ",", "MP"}], 
   "]"}]}]], "Input",
 CellTags->"b:3.4.4",ExpressionUUID->"e4371f2e-8aef-4731-8b71-ddc9261ac155"],

Cell[BoxData[
 TagBox["71",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589482943*^9},
 CellTags->"b:3.4.4",ExpressionUUID->"ed933a35-e02c-4298-a655-9909c549e2a4"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 SuperscriptBox["2", 
  SubscriptBox["n", "1"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  SubscriptBox["n", "2"]], "\[IndentingNewLine]", 
 SuperscriptBox["2", 
  RowBox[{"(", 
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], ")"}]]}], "Input",
 CellTags->"b:3.4.4",ExpressionUUID->"e49c3b07-13d7-4dc7-9099-c33c3774ee08"],

Cell[BoxData[
 TagBox["8192",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589492296*^9},
 CellTags->"b:3.4.4",ExpressionUUID->"a29bf67b-5961-4fe2-988b-6a787f8115e3"],

Cell[BoxData[
 TagBox["128",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589496439*^9},
 CellTags->"b:3.4.4",ExpressionUUID->"f43a7e3e-dfc2-4e1f-988f-2ef3656b0da4"],

Cell[BoxData[
 TagBox["1048576",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589499119*^9},
 CellTags->"b:3.4.4",ExpressionUUID->"5afc2192-6b29-4b3f-b5c0-9d05e19b4bb2"]
}, Open  ]],

Cell[TextData[{
 "The next function gives the number in the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]],ExpressionUUID->
  "431ecc03-f504-44c3-9aa7-50a97dc0ccb3"],
 "register in decimal basis from a given number i in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     SubscriptBox["n", "1"],
     TraditionalForm], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]],ExpressionUUID->
  "2879c2f7-cad2-45f1-8d97-a01ec576bfe8"],
 " bases."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4",ExpressionUUID->"1c4a527d-8219-49e3-8ce9-83158e473dc7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"xv", "[", "i_", "]"}], ":=", 
   RowBox[{"FromDigits", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{
       RowBox[{"IntegerDigits", "[", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{
          SubscriptBox["n", "1"], "+", 
          SubscriptBox["n", "2"]}]}], "]"}], ",", 
       SubscriptBox["n", "1"]}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Compxv", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"i", ",", "_Integer", ",", "0"}], "}"}], "}"}], ",", 
     RowBox[{"xv", "[", "i", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"xv", "[", "i", "]"}], ",", "_Integer", ",", "0"}], "}"}], 
      "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{3.395094492673421*^9},
 CellTags->"b:3.4.4",ExpressionUUID->"1fef000b-fc26-4d0d-a42a-6d92b8acbb95"],

Cell[TextData[{
 "We select the positions in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"]}], TraditionalForm]],ExpressionUUID->
  "f8711f1c-3215-4544-ac4e-b4ec42ecc6dd"],
 "register which remain when reg2 has been measured:"
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4",ExpressionUUID->"33971208-b4af-4f5e-acd3-850ceea0f6af"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dens1", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Do", "[", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"xguess", "^", 
            RowBox[{"xv", "[", "i", "]"}]}], ",", "MP"}], "]"}], "\[Equal]", 
         "reg2Measurement"}], ",", " ", 
        RowBox[{"dens1", "=", 
         RowBox[{"Join", "[", " ", 
          RowBox[{"dens1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"xv", "[", "i", "]"}], "+", "1"}], " ", "}"}]}], "]"}]}],
         ",", "\"\<\>\""}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", 
        RowBox[{
         SuperscriptBox["2", 
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["n", "1"], "+", 
            SubscriptBox["n", "2"]}], ")"}]], "-", "1"}], ",", 
        SuperscriptBox["2", 
         SubscriptBox["n", "2"]]}], " ", "}"}]}], "]"}], ";"}], " ", "*)"}], 
  RowBox[{
   RowBox[{"Do", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"xx", "=", 
       RowBox[{"Compxv", "[", 
        RowBox[{"i", "*", 
         SuperscriptBox["2", 
          SubscriptBox["n", "2"]]}], "]"}]}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"xguess", "^", "xx"}], ",", "MP"}], "]"}], "\[Equal]", 
         "reg2Measurement"}], ",", " ", 
        RowBox[{"dens1", "=", 
         RowBox[{"Join", "[", " ", 
          RowBox[{"dens1", ",", 
           RowBox[{"{", 
            RowBox[{"xx", "+", "1"}], " ", "}"}]}], "]"}]}], ",", 
        "\"\<\>\""}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{
        SuperscriptBox["2", 
         SubscriptBox["n", "1"]], "-", "1"}]}], " ", "}"}]}], "]"}], 
   ";"}]}]}], "Input",
 CellTags->"b:3.4.4",ExpressionUUID->"448b68fe-3d99-4f92-b723-79dc663a692e"],

Cell["\<\
And then after the measurement reg1 is in the following state:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.4",ExpressionUUID->"c3dd20c5-e84b-4681-8264-5b7e1e242a51"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{"0", ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"2", "^", 
        SubscriptBox["n", "1"]}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"reg1", "=", 
    RowBox[{"ReplacePart", "[", 
     RowBox[{"reg1", ",", "1", ",", 
      RowBox[{"dens1", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", 
     RowBox[{"Length", "[", "dens1", "]"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"reg1", "=", 
   RowBox[{"reg1", "/", 
    RowBox[{"Norm", "[", "reg1", "]"}]}]}], ";"}]}], "Input",
 CellTags->"b:3.4.4",ExpressionUUID->"a46d6d01-6126-4bc0-a824-9be50b8f05b2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Share", "[", " ", "]"}]], "Input",
 CellTags->"b:3.4.4",ExpressionUUID->"b0bb4c62-ccb1-4992-9e41-a9e48f7a2082"],

Cell[BoxData[
 TagBox["779880",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208589835268*^9},
 CellTags->"b:3.4.4",ExpressionUUID->"a817ccd8-a8b0-4722-8b26-c322a00ad647"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Step 5: Perform the inverse QFT on first register", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:34", "b:3.4.5"},ExpressionUUID->"64dcee58-a1d1-4934-8ecb-0881b5c947d8"],

Cell["\<\
We apply the inverse of the quantum Fourier transform to reg1 and check \
normalization:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.5",ExpressionUUID->"ebf67078-deeb-4d02-9435-9900f8c0280d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"time1", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.4.5",ExpressionUUID->"0416feec-c6d7-4a5a-ae7e-e21f7cf45018"],

Cell[BoxData[
 RowBox[{
  RowBox[{"QFTVec", "=", 
   RowBox[{"Fourier", "[", "reg1", "]"}]}], ";"}]], "Input",
 CellTags->"b:3.4.5",ExpressionUUID->"1330a74d-ae4a-4cd0-8e63-995c764cb490"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"time2", "=", 
   RowBox[{"SessionTime", "[", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<The QFT took \>\"", ",", 
   RowBox[{"time2", "-", "time1"}], ",", "\"\< seconds\>\""}], 
  "]"}]}], "Input",
 CellTags->"b:3.4.5",ExpressionUUID->"c82cb01c-7267-45d4-b7de-f54e6f34bd71"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The QFT took \"\>", "\[InvisibleSpace]", 
   "0.016999`4.380938371724635", "\[InvisibleSpace]", "\<\" seconds\"\>"}],
  SequenceForm["The QFT took ", 0.016999`4.380938371724635, " seconds"],
  Editable->False]], "Print",
 CellChangeTimes->{3.8572085898605833`*^9},
 CellTags->"b:3.4.5",ExpressionUUID->"6cc5eb13-0120-41ea-9b29-545c996602d8"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 6: Measure register 1, instead: study all possible results and their \
probabilites\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:35", "b:3.4.6"},ExpressionUUID->"b0f85306-8e1b-4b5d-9f95-71b193ff39db"],

Cell["\<\
We store the possible measurements and their probabilities in variable \
MeasProb:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.6",ExpressionUUID->"d3d9fc44-7014-4763-816b-a6fe354e2989"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MeasProb", "=", 
   RowBox[{"{", " ", "}"}]}], ";"}]], "Input",
 CellTags->"b:3.4.6",ExpressionUUID->"4beb6907-d3ca-4541-8990-a9a35d1fbd0c"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"num", "=", "0"}], ",", 
   RowBox[{"num", "<", 
    SuperscriptBox["2", 
     SubscriptBox["n", "1"]]}], ",", 
   RowBox[{
    RowBox[{"\[CapitalOmega]", "=", 
     RowBox[{"Chop", "[", 
      RowBox[{
       RowBox[{"Abs", "[", 
        RowBox[{"QFTVec", "[", 
         RowBox[{"[", 
          RowBox[{"num", "+", "1"}], "]"}], "]"}], "]"}], "^", "2"}], "]"}]}],
     ";", 
    RowBox[{"MeasProb", "=", 
     RowBox[{"Append", "[", 
      RowBox[{"MeasProb", ",", 
       RowBox[{"{", 
        RowBox[{"num", ",", "\[CapitalOmega]"}], "}"}]}], "]"}]}], ";", 
    RowBox[{"num", "++"}]}]}], "]"}]], "Input",
 CellTags->"b:3.4.6",ExpressionUUID->"59092ff9-c10f-45fe-af99-7a433c7a95d9"],

Cell["\<\
Plots of the possible measurements and their probabilities are quite \
illustrative:\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.6",ExpressionUUID->"81a1cf8d-fa5b-4026-92f8-ac50e66e6e2d"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"Plot1", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"MeasProb", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], 
    ",", 
    RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "\n", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"MeasProb", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Measured", " ", "number"}], ",", "Probability"}], "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0.3`"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "400"}]}], "]"}]}], "Input",
 CellTags->"b:3.4.6",ExpressionUUID->"2a6ccb02-d902-4138-b163-dab440cbb2bb"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.003666666666666667],
       AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJw93HlYzPv7P/CyZg9FSEKUPXtKPBGFUBRttGilMu17Tfu0TbMXQkglSmQp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       "]], LineBox[CompressedData["
1:eJw93HtYDt3XB/Ak55yTJAnJKUklSfKVJEmKhIQQT5KQJFFIEpJUkrrPZxWF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       "]], LineBox[CompressedData["
1:eJxdnHdcjt8f/7Mqm4SshGSTvcIL2dkkJHtVykhWskl2sncI9zXu67qkjJCZ
rY/RJytUyPwYWVm/8/3d79/j93Z//vk8no9yd93nOue8znmPV5VRwf3G5rWx
sblawMbmf/+/EH0438tmJswckOIZbae1tfnffy1M+Pklctml9WYLtzJhy/kl
n22Pqxb2MGGXb0aXBdmKhduaMLXT4qIpzsQwocOj2yWifGULdzBhR5zj84A9
koU9TcjjWGyR3VuThTubMD+t5uNS7Yi7mrA30mOb56ZDFu5uwpejg7W+nw9a
2MuEhUs/uH71Ie5lQnyHK8k5Zw5YuI8Jnyq2e1/UnbifCZGbBnYsuDfWwgNM
WHCy7VytPLG3CYG1s9qmbNxvYR8TJtnVfty8LPEQE6o03dTu2LZ9FvYVn1e9
4r6arsR+JqwJTljlq+218AgTwuxPbqoH4lEmjB4cOSj0doyFx5gQUy+0bSl/
4nEmXLp4c25GPuIJJgS1P//m3O49FvY3YZ5dgfebQRxowlq3oqM9M3dbOMiE
y77PF5oiiCeL99Gi733NnXiqCdurBvqFxe+ycIgJvdauyggpQRxqwvNhof0r
Be+08EwTCqyrkdbt1g4Lzzah/cDda1JaEoeZ4LajeOjy/dstHG5CweemHr1K
E88X7z+zyNzciG0WXijGs1SeogE2xItNWD5s77zwOVstvNSE7gcTLzrmbrFw
hAmH6ndpUDyMOFKM50PbPn3yEK8UPLva9xPLN1t4tQmhlw67Ni9NvNaEnY9q
ZR/Yu8nCUSYU84oa97kJcbSY3ys2n7O7vNHCG00oU8+z7KXhxJtNuFVn9CiX
7xssvFXM/60bF/2IJt5uwopVX+t2aky804Snn7vff3k72sK7Tfi480Ryaihx
jAlpHR1efq9AvM+EQuXGPG1/br2FY03QTrn/2BFIfNCEA3+yXX6XJTaZcLdJ
2t1+F6IsLJuQHuHfYFUIsWrC/uNXbuysTqyZkDng1v1ZaessbJgQsr9LbolV
xHEmfLUf22tQR+J4E+Im7p+EH2stfNSEWMdbfklxxMdNOOPRK/pqMHGi4Gvb
X3jXJT4lfn97q9rmO2ssnGRCqdTvwUv7E581Yc84fWXlu6stfN6E6kUqVRg/
mPiiCSktQoZ1f7LKwpdMWHw2oVnCROIrJiw518t1++eVFr5mgkfEu+bZi4hv
mHB/x+WSKxyJU0yo4Jf3QciBFRa+ZUKb0GHY6EF8x4Rj9RtOenI70sKpJlzx
Xm3baRJxmpgPObvc4u2J75vg/7Gtj1vscgs/NOFEu4SoiE7E6SYoT040TXke
YeEnJrzVlxX4sJw4w4QjMTVrPK5PnCXm99BPFdfcXWbh5ybUGlbp3Oe5xNkm
HD4xx8u+JvErE4yTY9ol3l5q4Tcm+KnGVLsFxO9M6Dv4eaEnDYjfm7BMGz/D
48kSC38U/75JZnCpKOIcsV9dPfthaCfiLyYknkpqUzR3sYW/meB6eP1dZ404
14QRU41ZEeOJf5rwYFr9Gh1ciH+L8XLeV9rz/iIL20j4N7ndkaUbiPNKiFmR
KuftR5xfQlQRhzStOLGthIWD1E6RNxZa2F5Cb8fLTSNWExeSsKZze6d9vYmL
SKj1zG3FvyWJi0mI3dpycMW7CyxcQsIV29CZgVuIHSSsyNTsz/oRO0r4Pr1d
v7LVictIWH9CPj/uzXwLO0k4fGX2Z1MccXkJczP/25k+l7iihPi4dzt/diF2
Fj8ffu/zHwdiFwn9I35FJp+cZ+GqEjaOrra78yBiVwn3K/Z/N+1juIXdJGyq
uXl65zXENSVMXv9n8pF6xLXF56vXBxy9PtfCdSWMvXbuWudg4voSJk2dftzH
gdhdwq1BcuPn8WEWbiSh+N7781/4EjeRUPnu1goD8xM3k7Ak00Opocwh/ZdQ
4unwoj6DiFtJ8M/ahMd5iT0kbIisPCPePJv0X0LQ6ejC/w4jhoSrfZMWtCpK
3EH8/pvxS+6fnEX6L2FLidovtWDizhKe6KP3J1Ql7iphc+nK/2anziT9l5C4
q/DptiuJvcT4NA9POtyBuJeEH29njeiQO4P0X0K5Q0mezwzifhLuBb5utCmQ
eICEpk6RxfvXIPaWsNaj+6NSmaGk/xIa3XNNvLOLeIiEPnFR2WuHEftK0Csk
H+9YkdhPQvDV9/uzH0wn/ZfQ48r85nO2E4+SkPQntMzXYcRjJBQbEJfXx4V4
nISJbv3ubMkMIf2X0PPt79lHDxD7Swi7MvyXOZA4UEKFnNCZYY2Ig8T4Jhar
XP77NNJ/CY9K12q5JIl4qpjfE6dVOx5BHCLhw45TvY70JQ6VsNd1TrtpFYhn
Sjj1ocWq98+mkv6L+dasTkYtnThMQpWz5twKc4nDxft8eW/fuW7E8yXUuGuT
VKks8UIJ7Sr19ajxbArpv4Rl60aWf3SYeKmEBhWu+nssIo6Q4LTxXM8O/Ykj
JRRZfeLj62rEK8V+cPH3pmafJ5P+SygdOiiycjLxWgkhznGu+7YQR0k4//bm
zTOBxNESHq57lCcUxBslNAls0vaUI/FmCa5VbWq73w4m/Zew6Hdmp7zhxNsl
9Cv+usq7WsQ7JbQIuVrjv9Qg0n8JQ+5FptksIY6R4On98apzE+J9Yv+raqt5
Zk0i/ZfQOHd+16ANxAclvCzRP2hTF2KTBCXctXhibiDpv4QF+R/UTVOJVQnZ
jtUevRhNrEnoludn1xfliA0JU1t1kW+lBJD+SzizdMjA2AjieAnbymzY59ee
+KiE0VO+Fv6W60/6L+ZL/lPFgo8QJ0roO/43kiYTn5JwrlKXLtl1iZMktMlo
n5r5ciLpv4SavX0M8wHi8xLkp0PMXccRX5Tg1762JlUnviTh7IlRsXefTSD9
F/M7s2DUuVjiaxK8O7ZdMHUC8Q0J00u/XPO4NnGKhE45U17YvRtP+i+hS1DI
qWyd+I6EQqEzfOaFEqdKaLYrtsDl1sRpYv+ruKrquTzE9yUY/rfKBlwaR/ov
YZXybfCpNcTpEgK3VZx4fBDxE7Hf5k1I8XUhzpAQfnV4jb0vx5L+S6gXHlVs
xWHi5xIG1gtp5BBOnC306sbWbs26Eb+SkJN91/GzI/EbCQ7bvvl5PR1D+i/B
6/m+ny1V4vcSqns4XTsxh/ijhEMdvsfd7EacI77fZbeD05yIv4j99umyXftf
jCb9l+Bb6dGO0QnEuRLmPIpSpWXEPyV8PBL9bI4P8W8J39KqDL9Vi9hGRtii
mHbKj1Gk/zIcAsIPF7pBnF9GSsmcd9m7iW1l2Ff8UK9zCLG9jHvzXh926Upc
SEbv7SeuhlYkLiKjUN0ZyZ0/jCT9F/++eE+bNReJS8iIyRqZ2nM7sYOMJlMV
bcFUYkcZF26m/azZjbiMjBvelRp0cCF2kvHZy/Xs1a8jSP9ljO/yotn5m8QV
ZTxut86x9kFiZxnLho7I/jGf2EXGN5+ITi2GEFeVsXR/6QuPGhO7ylhV+p8C
r4sSu8lYOKWrz8mU4aT/MgpH/zwXtJS4toxGAdOUFx7EdWW0HpE/vHqOH+m/
jM6nQg9Vl4ndZZQtnaI8G0PcSPx9tVqp0c7ETWS0cRgdsC1tGOm/4PAG/Vau
J24h43qwHNi4N3ErGScc+k9cXZjYQ4zPoYt/tl7yJf2X4fvk0HvvpcSQUcBm
eoEkT+IO4vsU7FniTj5iTxl2ex+8XntuKOm/jMZZNebkLCLuKgPZwYd+ehJ3
lzGmcT7/A7bEXjJ2Pap09OOlIaT/MtaFnd/4cAVxHxnbru8uMbo3cT8Zvco9
br64FPEAGV7diri1TRtM+i/DVc/5sXoHsY94n4sXPQ0ZTTxERsmPMQXe1SL2
lXH+RcWd3977kP7LuD8t7EXUUeIRMlyS/3M9MZ94lIxZN2vvDu5GPEZG/3Hd
D6kOxOPEfC11dGPow0Gk/zKCJxS7cCGW2F/G3pu/Nm6dQhwoIzJvo+mfPIiD
xOd/an7jhj3xZPH9244pU+euN+m/mD8VjifbxhCHiPd77dbIEcHEoTKyll+Y
2KwN8UwZ9Z8U919WmHi2GL/l4R/73h9I+i8jMa7M5uhDxOEyPib9yek5i3i+
jIBRkw6GdSNeKGNm80jvyuWJF8v4eXrUwSavB5D+y7hzNqXHiUTiCBmrHcZ/
0lcTR8pYGz/Mu+xI4pUyNkwLd3/VmHi1jIKHS3aoZUe8VsbgC1d7377fn/Rf
RrP5x6u+UImjxd9r1Dhi9CLijTKOfkzq28WHeLOMHa1KDVxXj3irjIfpbwKQ
j3i7jOTepUL63+tH+i+jTv22vS+biXfLGFvHLnP3UuIY8T6a3Xe6N4x4n4xy
n549mtyUOFbGxVlFqo0vSnxQhuR35+HxZ31J/2VUshmSL+AUsSzDp2z7qNBN
xKqMnqtiIx5OJtZkVF3r8HJbd2JDxqE3/scOuxLHifm/51nean/6kP7LWKP9
+Cf7HvFRGYOa+DbLf4T4uIzo2McdQtYSJ8qYPHZO0RaBxKfE/JrW1NyrK3GS
jBb+Tp4JrsRnZbw4VOP15DzE52WUDjqw8nNib9J/GbUPPMo5MIP4ktg/lnf0
+9qI+IrYn+UzlTLe9SL9F/PnzZlV4yTiGzKaTy1fbNEE4hTxPO2/5WvmRnxL
Rv7eV+/NzepJ+i++/+uGf3z2EqcKPSj0LPvCKOI0MV+v7StxrirxfaFXPap/
7pXpRfovY9PNH+Um7CNOl3Gk60q7ouOIn4j5WfVrd4+axBkyXo2yC8151YP0
X8y30lUqN1GJn8v4Zef+9PsU4myxH93eONSzGfErGZ5lnRs5/OhO+i9DrplV
ZFQS8Tux3tK9NjdaSvxeRuDxD4sXehF/lBHVI2pZn1LEOWI93M8/eNv9bqT/
Mob3rhE/Mob4mwynUSnDY/yJcwXbnCnv25j4p5gfORvMq352Jf2XkXD12Kum
F4ltFBT54r29/1rivArele+++dkQ4vwKyiQ4xr+qTmyr4MejmykjP3Qh/VeQ
2b5ycreTxIUUtAl0nbNnOXERBUfrTL013pu4mIIGzW3it1UjLqGgeb7KDh4f
OpP+K0i4EpDa5TSxo4Kc5POfT64iLqNg/a4243b4EjspCP5eqsqzOsTlFdzs
sa9q9I9OpP/ieS77+R+6SuwsODoxt9p2YhcFi383uZd3EnFVBWFFGxfr1JbY
VUFj58L7XxYndlOwI+699N9TT9J/BX2nv2nsHUdcW4Ft11t9Ki4jrqtg8K7m
5doPIa6voGT2xNhz9YjdFfSoPtZxbx7iRgrG77+yIP1uR9J/BS+y/i0+3UTc
TEHtmGEZY+cRt1Dw74v4ytoA4lYKFs1Ket+vNrGHgjj3lPk9bYjbKliyTv61
M7UD6b8C/7mLN3VWiDsomFrgxfyOi4k9Ffw39MS76KHEnRVszm7h1KoxcVfx
/mY+btaoMHF3BYdWBS8Ny2xP+q8IPc3wdEok7qVgwJoxRv4NxH3E78/sXrJ9
EHE/BWq/orHXuhAPUFAvvX1CbBVibwVdb69bdeMHSP8V+HyPm9YxlXiI4EH+
8fl1Yl8Fb+dP2VdyJbGfggrfF62eOJ54hIJXRq/kwh2JRykYeTBg71dn4jHi
eR0TPvp8aUf6ryCk4J8O/c3EExTM8HJ783ACsb+CRpu8YnOqEgcqCJpUv96W
R21J/xVM9C6Hy5uJJyvY5fNkz6IBxFMV+G1IcUouQRyiCH2NDYq63ob0X7xP
ef7krEjimQq2NL337kRX4tkKTh4NPVnWjjhMQZ8Xow9/ueBB+i/el5JlGrSE
eL6CMy8DFrTsRLxQwbYh68puK0C8WIH7aru+cy62Jv1XsLFKlOO9ZcQRCqrZ
/exxuBtxpIJ934tm2RchXqmgXakl19JutCL9V5C4uOC3KlHEaxWcCK7umzmQ
OEqBTUbrDxXKE0crGPVxnnElvSXpv4Ivua93vNtLvFmBw/cWcUsnEm9VcOH7
3W/rGhBvV5D08vGU4l9akP4rePnCqebvROLdCmYvfuI2bDFxjILdM+qE1fEi
3ieeN3lOiwmOxLEKPr1PC3J41Jz0X0H3Opea1IolNon1e9czRgomlhWse3f0
n80tiVUF2Yc/P/4vH7Emnqdv3TfmG81I/xU0O/q8bNoW4jgxvjNslo0fSxyv
YOuVXcOHNSQ+qojzcP0rR381Jf1X8NFpwIeZV4gTFXSp9zbPjk3EpxR4HrGt
4zqWOEmspzWpO4o2Jj6r4IbHkVU+eYjPK1i4vKx7/pQmpP8KArtGJBbfRXxJ
wWmXmMDZQcRXxP4wq2BYx7bE1xSMnTaupn8x4hsKjJ9++tv0xqT/YnzervW5
bia+Jfb33itH2i8gvqNgys7rJXf2I05VEOB+VFvlSpwmxvPA2mWpXxqR/itw
DpvxdMZl4ocKJkSNLjxlO3G6WP8vKw44E0z8RMHdwc2LTOhInCHGc3zRpWPK
EmcpeFz5ROn41w1J/xXYOZYr6ZtEnC3Gp3/CzYEbiF8pMK9KM8X4E78R41t/
0q/2IH4n5u/mYq6NyxC/F78flBs2/Y076b/Qp8yiE+zPEecoyO92pvPrLcRf
xPPfqRpWaQrxN7G+598O3NmVOFdBZKc88ya5EP8U8+v2xSaOHxqQ/itIqxjY
58UeYhsVR9ID3Pr3J86rYllQizXeBYjzq1i9tcm49wn1Sf9VRG2SllcOILZX
cfrppfsZzsSFVNzosrZNy9v1SP9V/CicurlKBHExFZ7VmqdubUNcQsXI5SUz
9n6qS/qvYq4WJrU2ETuquFCwg+vIkcRlVIw7cBelyhE7qXD/1Otbn3/qkP6r
cJs90rNcJHFFFX3fDqwS0JHYWcW0nLyrO/2qTfqvIm3SxRUHEoirquixLNx5
7VRiVxU5eRd5/a5H7Kbiw7qdzh9e1iL9V1EF9rvHxxLXVtH8h2/6iNHEdcX3
ddvy8oELcX0VqT0c7vybXpP0X0XLB5ON/juIG6mY/uvQtl6+xE1ULF3+/WBy
BeJmKt48f/D55IMapP9i/D3td7tvJ26lYuEW/2TnYcQe4u+9rLR6mTNxWxXr
o8Y5+T9xI/1XcXOrvPJCDHEHFaVrhZbaMpbYU4xH7sh/M2sSd1aR8q1Kgf1v
qpP+q6hzf/rVBxpxdxXx/1Qfumw6sZeKsf1+PY9tRdxLRfE3F/a2tCHuo6Jt
hQ6X2l50Jf1XsWdQzVVHVhIPUFE1v3O9Hf2JvVXkrjCycsoT+4j3s2TIxxMZ
1Uj/VXhn+m/8YCL2VeHQyrHUhmnEfmL+FPKT93sQjxDPX73qwaq2xKNUpFe1
71U4pSrpv4otN+a/8NtKPE5F6wNnTlccSzxBhf2ppGqe7sT+Kvr8aDDqzo8q
pP8qDuWZdfVaMnGQitl2x/S60cSTxfyblTjh0wjiqSpW5uR2cqlPHCLGo8L6
3cYPF9J/FWu2dP7n0GXimeJ9VgxtVGAz8WwV2yZUcjk/jjhMxdOgm1+fNSUO
V+E103DyL0A8X8W7mG8Xd5kqk/6r+NbqUUi9HsSLxfr7sj9v2Btn0n8VU369
axW5mjhCxdm83q+9GxJHqnCtOi3ryZ1KpP8qhtz657vbLOLVKn5mvcxT05l4
rQo1xik141xF0n8V1X9v6eftTxytYtM/C7wXlCTeqCIBR86NOlaB9F+8vypZ
4d9HEm9V4bNoYbBnYeLtKlwanZ3X4Uh50n8Vjxas3P5xOPFuFfgy50j/wsQx
Yj8snpo4JqEc6b+K/u2WHHAdQxwrPu+Ow6SoksQHVZzJu8dOOe1E+q9ilt2z
adOCiGUVE/pX0bMqEasqxl+sfObP9bKk/yra5a978Fw4sSH236dFJzVwJ45T
MfNMvnIdMsqQ/qsw1zuZkBtNfFRFvykDB/l0JT6uol6tXvZDfpYm/Rffr2Tk
o98a8SkVO093edNpHHGSeP+LKnrVr0h8VsWg7v+VSbjlSPqvYvPGTxMeLye+
qKJB5L4Rse2JL4n59u20c+HcUqT/Yr2ubX+xyGHiayrKFvQLMwUS3xDzOTx8
QqYbcYqKGdGt4+KfOpD+i/3HdvCKajuI76jIynOmQq3BxKlCf3bN23CxNHGa
4Jmnav+4VZL0X0W5lknFL64lfqjCzuvhbLfexOkq9KWz91UoRvxE7H+fiyXv
u16C9F9FjX2lK59ZRZwl1sOMYq+DexE/V5HHZVaAVpw4W+z/v/dcyLe3OOm/
ilZzHeok1yB+oyJza8noQ0ox0n8Vxwr0XHeoCfF7sV+s6J5wIbEo6b+KUrNH
7PzZiThHxcEaaoZXShHSfxXR4Y6uR4YSfxP7UVx92ybZhUn/VQQU7OF+aTrx
TxWfD47uOTk/8W8VlzeVLVInuhDpvxk9kvY6f3UlzmtGssPDVrfjC5L+mxGd
4Fr+VDdiWzNa+gbMin9kT/pvxohol3InphIXMiOsd8OTV+yJi5hxaJRNvaxd
dqT/ZpSqM6KxbQviEmakt125s8E/tqT/ZrxpWQ3DA4gdzRi20eXtelviMmbM
3Zs691pMAdJ/M3wXZV+3A3F5Mx54eyZ6pucn/Tfjc8NKdRbOJXY242z8sz+n
KhG7mHHMsXu13FP5SP/NyImuvaDRSGJXMzbabbQfn5/YzYyJA89LGw/mJf03
I6J4yRFJPYlrm7G51pdKmZ/ykP6bMfa47dNfW4nrm+Fw5uWB4h2J3c04XG5g
SNk3NqT/ZpQdZN/DcSNxEzNul73UuEB74mZmFM6vT71170+b/8stzDjY/uOt
8MbErcyo0/30oaJrflvYw4zj8eOc5775ZeG24vO/lPC72oMYZkypn7Dti/zT
wh3MqNm8fIlfxYk9zTi95dj3hyE/LNzZjLdPrmyPfpBr4a5mzHnk3rGqJ3F3
MxLKTvNYrH63sJcZXWYdeXS0PHEvM1Y+NS84FfHNwn3MyKgUG7H+21cL9zOj
08HhPs38iQeYce5uZd/9j75Y2NuM0rk78j/uR+xjRqj2K/np5c8WHiLmW6Pk
KnIHYl8xf78+jvI4mWNhPzPS3BftWdeSeIQZIXta3TuU8MnCo8R8G/HvlUXN
iceYsS/dv4jzsY8WHmdGo/ftWs7wIJ5gxof7tz+sOvPBwv5i/WTdfDCiK3Gg
Gc1H5o15l/LewkFmHBnpk9N0KPFkMZ7havGAG/9ZeKoZNTbWrZf28p2FQ8yQ
Wk45rdkTh5rx48X1Xh9rv7XwTDNeLlw/aWufNxaebcavqbZHY2e9tnCYGaOd
+t9xjH1l4XAxXn4Lp2bcfWnh+WL9RAd5FS1IvNCMVp2PVl/fLtvCi8Xznjhx
dcqsFxZeKv59waWlD8Q/t3CEGT1vXMl0//rMwpFivr+/51K0NfFKMzat3ZDU
cmGWhVebUazhhKS4a5kWXmvGpCKta4SVJ44S63tTYIHVARkWjjZjdZPkkOzT
Ty280QzbKjM3Ly9LvNmMC6G2O4KnPrHwVjOc8myY3HDPYwtvN6OZ85V+tmPS
LbzTjEEzgqYMqffIwrvNuF8tLrj2rwcWjjHDhv+3z4pjrfigFZusWLZi1Yo1
KzasOM6K4634qBUft+JEKz5lxUlWfNaKz1vxRSu+ZMVXrPiaFd+w4hQrvmXF
d6w41YrTrPi+FT+04nQrfmLFGVacZcXPrTjbil9Z8RsrfmfF7634oxXnWPEX
K/5mxblW/NOKf1uxjfY357HivFacz4rzW3EBK7a1YjsrtrfiglZcyIoLW3ER
Ky5qxcWsuLgVl7DiklbsYMWlrNjRiktbcRkrLmvFTlZczorLW3EFK65oxZWs
2NmKK1uxixVXseKqVlzNil2tuLoVu1lxDSuuacW1rLi2Fdex4rpWXM+K61tx
Ayt211Ddje3/DTWkzmT60EiDmwvTj8YaTtkwfWmiodxMpj9NNYx0ZvrUTMO/
TZl+Ndcgb2T61kJDHq5/LcXzNWL62EpDmTVMP1trSP6P6auHhgJcf9to6ML1
ua34Ply/22kYw/UdGiZFMf1vrwH8fNBBw/uy7PzQUUPQNHa+8NSQdxE7f3TS
0IufTzprONCCnV+6aCi2iZ1vumqoOI+df7ppqD2KnY+6a6g7nJ2femgYnsnO
V14a7txj56+e4vN/svNZLw2eD9n5rbeGS37sfNdHw4Bd7PzXV8Ohhux82E9D
7jd2fuyvoXdjdr4cIMZHZefPgWJ8frDzqbeGBXfY+XWQhp8H2PnWR8NnZ3b+
HawhOJOdj4doKDSDnZ+HapjsxM7Xvhqq3WPn72Ea3mSw87mfhvtb2fl9uIZ8
/Hw/QowXP/+P1JD1md0PRmlYzu8PozXk5/eLMRrWd2L3j7Eadtiy+8k4DSuL
s/vLeA37h7P7zQQNN8qz+89EDQq/H/mL+XSM3Z8CxPPVYferQPH3irH71yQN
J/n9LEjDxILs/hYs1vMGdr+brGFJaXb/myLWL78fTtUg8fvjNA22Xux+GSLm
92J2/5yuYQ2/n4ZqKM7vrzM0tOf325li/YHdf2dpGMLvx7M1LOL35zkaxvL7
dZj4Ob9/zxXrfSi7n4drCOD393ni80ux+/18DYH8/r9AQyaPDyzU8IXHDxaJ
+V+IxRcWa/Dn8YclYj7y+MRS8XMev1imQZ3F4hsRGmIiWfxjufj3PD4SqeEP
j5+s0LCJx1dWavjN4y+rNAx1YvGZ1RrKz2TxmzVivvP4zloNbb6w+M86Da2X
sfhQlAajB4sfrdfQiMeXosV85vGnDRqa8/jURjGfefxqk4Zmdiy+tVmD63oW
/9oi5tM8Fh/bKva3Qix+tk2D9xoWX9suPo/H33YIPeLxuZ3iffL43S4Nu3l8
b7eYHzz+t0fsfzw+GKNhG48f7tVwjMcX92kozOOP+zXsqc/ik7EaLp9i8csD
GgZlsPjmQQ2mAiz+eUjDjAssPmrS4Mzjp5LYv+x5fFVDJR5/Vf6337L4rKqh
Jo/fmjXsLMbiu5qGxBwW/9XF/s3jw4aGUHcWPz6s4dV8Fl+OE9+Xx5+PaIhr
z+LT8ULvefw6QcPMXyy+fVTDCR7/Pqah1TMWHz8u9lcePz8hfs7j64ka5vP4
+0kxPjw+f0rDBh6/P63hGY/vJ4nzw34W/z+jYRrPD5wV+1cwyx+cE3rK8wvn
hX5XYvmHC+Lvb2X5iYsaplZk+YtkDQU/s/zGJQ2+s1n+47IY3wIsP3JFw3ae
P7kq9vuvLL9yTewHPP9yXcMHleVnbmioeonlb25qOF6N5XdSNLSrwfI//2g4
WJDlh25pGNWS5Y9ui/XB80t3xP7F8093xfzh+alUsX/kY/mrfzV892b5rTSx
vsuz/Nc98X1fsfzYfQ1H+7P82QMNXjy/9lDDinss//ZIfJ/vLD+XLvbv7yx/
91iDQ02W33uiofM2lv97KtYDzw9maOjxiuUPM8Xzbmb5xSyxX/H84zPx97JZ
fvK5hrQVLH/5QkORnyy/mS3OVzz/+VKcz3h+9JXYz3j+9LWGT7ksv/pGw/l2
LP/6VsNbnp99J75vPpa//U+c15ay/O57sX8FsfzvBw1LeX74o9A7nj/+pEGf
y/LLOeL3ef75s1jvPD/9RcwXnr/+KuYvz29/09CE57+/a1jF8+O5GtJ5/vyH
2L94fv2nmC88//5LwzCen/+t4QjP3/8R74vn9210JPD8fx4dt3h9QF4dO3j9
QD4dx3h9QX4d7rz+oICOi7dZfYKtjpvVWf2CnY7WvL7BXsdKXv9QUEeVLaw+
opCOhrx+orCOnry+ooiO/Lz+oqiOsqVYfUYxHY4urH6juI7mvL6jhI7nvP6j
pI6SvD7EQcd0Xj9SSkcdXl/iqMPzIKs/Ka0jhNenlNHRkdevlNWxk9e3OOlI
5fUv5XTY/mL1MeV1nA1h9TMVdCzl9TUVdfTi9TeVdOTl9TnOOh7x+p3KOnbz
+h4XHX15/U8VHVt5fVBVHR14/VA1Mf7lWH2Rq46jvP6ouo6qvD7JTccIXr9U
Q7yfI6y+qaaYD/1Y/VMtHVG8Pqq2jrm8fqqOjmm8vqquDj9ef1VPxzten1Vf
zD9ev9VARxte3+WuY9siVv/VUMxfXh/WSIcTrx9rrGPDT1Zf1kTHh2+s/qyp
jqe8Pq2Zjim8fq25jgtPWX1bCzHevP6tpXhfvD6ulXgeB1Y/11rHBF5f5yHm
K6+/a6PjLq/Pa6tjIK/fayfWN6/vgw47Xv/XXjCvD+ygowevH+wo3jevL/TU
sZrXH3bSsYjXJ3YWz8vrF7voWMPrG7vquMTrH7vpaMTrI7vr8OD1kz10jOf1
lV46hvVk9Zc9dVQvxOoze4nxKMXqN3vrWMDrO/uI73uY1X/21WHD60P76cjh
9aP9xXjw+tIBOvx5/elAHU15faq3YF6/OkiHW39W3+qjw8zrXweL9bKa1ccO
0aHdYPWzQ3Wc4PW1vjpa8frbYTpu8PpcPx3/8vrd4WK98/reETrW8/rfkToi
eH3wKB27eP3waB3xvL54jA6XbFZ/PFa8H16fPE48L69fHi/WE69vnqAjnNc/
TxTrgddH+4v5zeunA3QM4PXVgTpG8frrSTqO8/rsIB1FeP12sA4fXt89WegP
r/+eIvYjXh8+VcdEXj8+TUfXbqy+PERHA15/Pl3/uz49VOzvvH59ho4tvL59
po5qvP59lo66vD5+tpivvH5+jo6vAay+PkyHyuvv54r9P53V54frqMzr9+fp
yFeB1ffPF+PN6/8X6NjO+wMWiv2e9w8s0lGI9xcsFu+H9x8s0fG9F+tPWKoj
4x7rX1imYyHvb4gQ+wfvf1iuYy3vj4jUMZn3T6zQoZdh/RUrdWTeYv0Xq3S8
smf9Gat1JPH+jTVCr3l/x1oxflVY/8c68fy8PyRKzG/eP7JeRxfeXxIt3j/v
P9mgI5n3p2zUcfkl61/ZpOO/nqy/ZbPY/9qw/pctOuaPZ/0xW3WkfWH9M9t0
NNZYf812sT+sYP03O3RE8/6cnWJ/5P07u4T+mlh/z24dnXn/zx6hp7w/KEac
D3n/0F5xPjnA+ov2CT3m/Uf7dcTy/qRYsT/x/qUD4nnfsv6mg+L5eP/TITG/
eH+USYc375+SdEz6q79K6Dvvv1LEfOP9WaqO2rx/yyzWJ+/v0sT84/1fuo5O
vD/M0BHH+8cO60h5zPrL4nR85v1nR8R5gPenxQs9acz61xL0v/vbjuoI5f1v
x8T74f1xx3WE8f65EzoO8P66RB1LeP/dSXGe5f15p8R5Oon1753WsY739yXp
aML7/86I5z3D+gPPivMq7x88p//dX3he7Fe8//CCjiu8P/GiWK+8fzFZCCfv
b7ykYwbvf7wszs+8P/KK0FfeP3lVRzfeX3lNhxfvv7wu9MGR9Wfe0LGP92/e
1DGH93em6BjH+z//EfOX94fe0nGE94/e1pH7gfWX3hHjw/tP7wp95f2pqTrK
8f7Vf3WU4f2taToM3v96T0eLuaw/9r6OgHTWP/tAvF/eX/tQx8cXrP/2kRhf
3p+bLvYn3r/7WMdve9bf+0Scj/6E/P/+36fi+Xl/cIaOAibWP5wp9kfeX5wl
xn8y6z9+pqPiPNaf/FzoD+9ffiHWl8b6m7PFfdKR9T+/FOdH3h/9SpzneP/0
azE+vL/6jY4/61n/9Vsdh3h/9jtx3uT92//piOT93e91POP93x90BPH+8I9C
f3n/+Cexf3Rm/eU54jzTk/Wff9bxnvenf9Fxh/evfxXvh/e3fxP3a97//l3/
uz8+V8frOqx//of497y//qeO27z//pfYn3l//m9xf+T9+390FOP9/TYGZvL+
/zwG8t9j/gB5DRSMZ/4B+QyEcX+B/AYGc/+BAgZ2c38CWwOFuX+BnYHT3N/A
3kAZ7n9Q0EAr7o9QyEBp7p9Q2IDtS+avUMRAIvdfKGpA4f4MxQxU5/4NxQ1E
cH+HEgbqcf+HkgaGc38IBwNLuX9EKQN7uL+Eo4Fi3H+itIEe3J+ijIEv05l/
RVkxftzfwsmAiftflDNwwJH5Y5Q38LYD88+oYMCF+2tUNHCV+29UEu+D+3M4
GzjP/TsqG3/7e7gYaMn9P6oYf/uDVDXQlfuHVBPjVYD5i7gaGMv9R6obOMH9
SdwMLFzC/EtqGEjg/iY1DXTm/ie1DDzm/ii1Dfhy/5Q6BpZzf5W6Bu5z/5V6
Bhpyf5b6BtZx/5YGBppzfxd3A97c/6Whga/TmD9MIwN1uX9MYwMPub9ME/F+
qzD/maYGanN/mmYGrnH/muYGTnF/mxYGZnP/m5bG3/44rQzM5f45rcX65P46
HmL9cP+dNgYac3+etgbmc/+edgbacH8fiPnL/X/aG9jA/YE6GKjB/YM6GhjP
/YU8xfvk/kOdDOzn/kSdDTzn/kVdxHzn/kZdDbzm/kfdDKzn/kjdxXzn/kk9
xHhxfyUvAxO5/1JPA21rMH+mXgbsuH9Tb/E+uL9THzFfuP9TXwNzuD9UP+Nv
/6j+BgZyf6kBBn6mMf+pgQZWcH8qbwOu3L9qkIFb3N/KR6zvg8z/arCBB9wf
a4iBz82Yf9ZQwdxfy9eAyv23hhn4zf25/Ayc5P5dww3EcH+vEQZecv+vkQai
uT/YKAMXuX/YaANZ3F9sjNjPuf/YWAOR3J9snIEGacy/bLzYf2KYv9kEA8u4
/9lEAxL3R/MX64/7pwWIv68wf7VA8b64/9ok429/tiChF5WYf1uwgS3c322y
gVrc/22KgVncH26q+H7cP26agfcZzF8uRIwv95+bbmA796cLNfCM+9fNMBDO
/e1miv2S+9/NMrCX++PNNlCZ++fNEfsf99cLM9CF++/NNTCP+/OFi9/n/n3z
DOzk/n7zDcRz/78FBhy4P+BC42//wEUGJnF/wcUG3Lj/4BIDMvcnXCr2L+5f
uMzAGe5vGCH0gfsfLjfgxf0RI8X+zP0TVwi95/6KK8X+w/0XVxnoyf0ZVxtY
y/0b1wi95P6Oa8X75P6P6wzs4v6QUeL7cP/I9WL9c3/JaANFuf/kBgNx3J9y
o4Ej3L9yk4HW3N9ys4Gnw5j/5Raxfrk/5lbx82XMP3ObgQzur7ldfD/uv7nD
QDr359wp9m/u37lLzC/u77nbQCXu/7lH7M/cHzRGjBf3D91r4C73F91nwOD+
o/sNXOD+pLEGbnL/0gMG1nB/04Ni/+f+p4fE/sP9UU0G/uH+qZKB23/5qwq9
4f6rihh/7s+qCn3j/q1m429/V83AL+7/qoufc39Yw4CZ+8ceFnrE/WXjxHmN
+88eMVCS+9PGG3jH/WsTDOTl/rZHxXmZ+98eMxDE/XGPGwjg/rknDEzl/rqJ
xt/+uycN9Ob+vKfE+Yf79542MIP7+yYZ6M79f88YyMP9gc8aqMP9g88ZGMD9
hc+L8z33H74g9gvuT3xRzHfuX5ws1gP3N74kvi/3P74s9Ib7I18Res/9k68a
COT+ytfE/sH9l68b8OT+zDfE/sz9m28aGML9nVPE53P/53+EHnF/6FuCuX/0
bQMluL/0nf89D/OfvmvAn/tTp4r3xf2r/zWwhPtbpxn4wP2v7xm4xP2x7xtw
5/7ZDww0Pcb8tR8KveH+24+EHnJ/7nQD+bh/92MDK7m/9xMDHtz/+6mBcdwf
PEOcB7l/eKZY39xfPEucR79a/Md3NnULXyH4//mT/x/K+rgj
       "]], LineBox[CompressedData["
1:eJw93HlYTV0XAHANhGSslEYUkkQpmWopkSZFikhIRBkqKVKGSGRIUQqhSIOE
e6YGlVDRoDTRoCSRSkmkgb79Pa3z+sfze+7t3HP2sPZa956zp27fv9ZZeNiw
YUeGDxv2//9jlXxVYi4LwMemZMUVsUf6w/7/L1QAHh2rFjlUpAw5TAB/4w+J
HFuKviKAsiTPv9rxD4d8VQDycg+WWcigwwWQ0HZ8fvT55CFHCOD5peTDA6Lo
awK4v3PqK4MTD4YcKYCNenK5y4XQUQKYMsy+ve5U0pCvC6DBYbH3D3H0DQGk
WV3X3xSeOOSbAlih4pLQPx0dLYD1y+NDKSphyLcEkD7vj85uE/RtAXQfsZb/
9yF+yHcEoKY1avRGH3SMANiv39u3SqJjBVB63bv075P7Q75Lri/14ysZG/Q9
AdQ1w7frPXFDjhOAwtFtuh430fcFUN9zMPyMMTpeACfSDhfkd9wbcoIANA9a
75h+A50oAKewI4LTpugkAdzr2PbyQ9/dIT8QgEuSoZ1iMjpZANMlgosXbEM/
FIAJ83TepMnoFAGoLs+TSyqOHfIjAez4kJ35/Qz6sQDUJ8mWlRiinwjgqrZ8
+IrBmCELBNB+vzJn2VM0JQDZQ2bnHx1F0wLQ/dwbcXkZmhGAtlfyn4rBO0Nm
BTCQEKbm+RzNCaAxIGyUYxA6VQCXnjZWXrJEpwlgp0bF7AEpdLoA6M+Nv6I+
3B5yhgCMPn/0dYpHPxWAxfYDVhae6EwBjP1yfp2VATpLAKH3Nz7cMQadLYAv
MOH2cPrWkJ8JYNze2oqV69A5ZDzsOfDJ/2f0kJ8LgLK4qHb7KvqFABbZWB27
uQj9UgAzu+019tTfHHKuAFpGm7T0BqLzBNA1fHUpzEPnk/6V+btLu+bGkF8J
YN1Tn5DcM+jXAjjf7nuuQwddIICga+XfYz9fH3KhAB5Ltia9D0cXCcB08yHh
M6vRxWS8SdiejPkbNeQ3AtAyDBGbLkCXkP7Y484O7kaXCuDi8cK/mtPQb8l4
adp4LqkmcshlAkjZr225JxxdTua7kdvhHWvRFQIY05XjEzoOXUn6/7Xbz7ai
a0OuEoBG/598twvod2T8+ZiOE7dEvxeA5fH745+NQ1cLICO7bnXQ24gh1whg
ueIRF/twdK0AxHUmrZ6/CV1HXk+592vEVPQH0r+3he5WfAkfcr0A/BVMqIgU
dANpX1OflFU+6I8C2KAnvrJhObqRjL+qm5abx6A/CSDHOf4FU3V1yE0CSE0z
zPsWi/4sgNy67oifB9DNpL8dwiIL9dFfyPU1zFjjLoH+SsaH+wf7utorQ24h
/TtljN+YZPQ3AYxerh7W54duFcDsI53hsWvQbQKws9l+U2Qaup3Ep+y+Culf
YUP+LoDOjJKz7/PRHQLIbHg1xeQmupO077EyYUcP9A8BfBo1vWyKCbpLAB2O
fto+iuifAkgqPfXR81fokLvJ68ckHo0uQv8i51PvHbjoHvo3Ob7chK1//dA9
ArBVPmy2zg79h4z3UWP36M1H95L5lbe5I0Ec3SeA8pxw5XvNl4fcLwDPFWNs
ZuagB8j5+id1akSj/wrAbzBgL3ME/U8Aq7ZaWmfaoQfJ+Y77M99IBz2MgjPD
tSnDSWghCuSWjQpO+xEyZGEKTEs8veNL0CIU3An7tWr0I7QoBQ9GL3pXdwk9
nAKfyMfjpx5Aj6Bg5VuD5lIrtBgF7LN3G7/PR4+kIEp+ns/BSehRFLjvef3k
xbdLQx5NwQU95+VuKWhxChST4E/NQfQYCiLHuZuMX4KWoKBG2Y8dLoweS0Fp
QckH7tXFIY+jIM5pgeuMUPR4CkYom8ibbUJPoOBshsiL6aroiRQ8mrR4ZXLH
hSFPIp9/WflaUxpakoIY++qs3EC0FDn+aLFyq3VoaQqCGmz6jymjJ1PwZIW0
m/X380OWoUDsdLVrTgZaloLXSaPWlASjp1BwzTbAx2cTWo60hxds5NTR8hQY
vunZf2EgeMgKFKiteGnyqwitSMHmu2VH22+hlSjYMnHUE08PtDJpvzI66rQx
eioFB/2MfynLoqdRcHxPV7BR+7khT6cgcIfvgtZnaBUK7u+veSMVgVal4Hac
iOkzN/QMCrZLR0S2GqJnkvF1Oj4rVBY9i4K98kfyqM6zQ1ajYF29fu6afPRs
cr31ddXbbqPVKXCcyWq0+KDnUPDCeE/nJ2u0BgWn3aMdrNTRcylI1y3k5gxH
a1LgdnukpXd90JDnUeBVUeo+Nw09n4z3KXXHra6itcj41miurzmA1qag+fex
/mJz9AIKbL2aV81SQ+tQYNNdCs3D0boUNO0y2y326cyQF1Jg1nPC7Fw2Wo+C
9ffv+OyNRi+ioEg2KSjlKHoxBc4O5lPWbEIvoUD8edpb08XopRREPLngcEcW
vYz0z6EoN+vewCHrUzCup4Td+B5tQMGPLbJ/uVQ0UNB7W2vYvij0cgq2vr98
ztsXbUjBoVOGruWb0UYUvD+ZdyhAH72CAqsvA4EBymhjCqTO9B0pE0avpMBu
VY+ux+fTQ15F/n6axt0t+WgTChI/Pn0UkYReTUGH/7U18pfQphS803X0afVA
m1Ggr2c5U9QObU7B3N/THJyXoC3IeJkkqjRWGW1J5v+n3Yf+iqLXkPl8Usxd
69upIVtR0OhBKT94g7Ym/VFke303jV5Lwbeuor7d19HrKJjwvG7zgxNoGzLe
rKZ80nRBryfza0L5k5+WaFsK4oNEf/3WQdtR8NxpSaWeAnoDBedHnw/kRNEb
KXC96bvIqy1gyPYU7N7aobqvHL2J/P0FucCYDPRmCsaEeERMuId2IOdz60pU
2gX0FgrCMg3KbhxCO1LQJQ1BrCN6KwUij7eKjF6N3kbB6NObL4dpobeT8dP0
5sQ6ebQTBbHas2euHoHeQQH171alZ+fJITtTsGm9wte31eidFJQf1mOcX6J3
kfXgg0q0+iO0C2mfXXGD06+jd1OwoF90mWUgeg+ZrzbD0hPc0a7k+hVeNOg4
oN1IvDKvndxlgt5LAZOpVli3AL2PnJ9slesfZfR+Cn42WexaLoE+QMGbRvX5
XO+JIbtTcOmunv7mZrQHBRp2m4ZrlKE9Sbx+HTKolo0+SOJt/Mf7a5PRXhQk
dWzwjYtCHyKfF9r3RyUI7U3G97q0O0VeaB8K2os9/txyQh+mwIKuPXHdGn2E
xB+9wBdZBmhfCjxiX5uMmYs+Sua/4utrx+XRfhQI390G8uJof3L9U6YXfeg9
PuRjpL+jLYrzvqKPU/Cx7WV5ZRX6BIk3wuM8xfPQJ8n69dZwmTODDqBAaL74
q0/30KcoePs+KjnwKvo0BXq34wIsTqMDKfiUteXzQi/0GTKe1rrsM3ZGB1Ew
0FiZ7rUefZa0h9oMt3xj9DnS/w3aSst00cEU5KcuOlQ2A32eAsnDeZOCJ6Mv
UNCj8vS600j0RQqU6HM7U7uODfkSBTMP3hJqK0eHkPneLL1hC4u+TPqz/Hn6
uCh0KAXVPxqPDx5Fh1GQc3X8vzlb0Vco6I7RPx9hhL5KHJsYqD8THU76R7bc
dKo4OoICusdDz6DDf8jXKBhrve9xRBk6koKRWmWyszl0FJlv8QVVv26gr1Pg
/evqur4T6Bvk+lp9e3R3oW9SMM/7tWaSOTqaglfWr/bYaqFvkXhT9URGTwZ9
m4ID4g+S1vzzG/IdMh8zMiJuNKFjKFhTnrdWsQAdS8H0/ACb8sfouyQ+zvWU
zryGvkfG65G6STXH0HFk/FTkvZq9C32fgpvy35kHluh4sh4Ny9i0VRedQM7X
ZOViY0V0IgXaaisq7Uegkyj4mj+Svv396JAfULCw+qiDdBU6mYKptjIb0rPQ
DymYYlM791I8OoW8/td87+XL6EcUTLvhevvZEfRjCj6/emSrtAP9hKy3T0Yo
J1qgBRToHHx+YdtCNEXme3SslvFUNE2BavaLGBtxNEPWT6nbnpd++Q6ZJeuH
xfClv+rRHAVaTlJXTr9Gp5Lxc2WZFNDoNAoW7dGyV7mNTif9Xfnt3/xgdAYZ
727Xb7scQj8l+WXK47q8behMCq66vVm/xgKdRUGGdm7CPz10NgV1H0N9K1TQ
z8j7F23cVzYenUOuL9Jxxp+BI0N+TuJP2ujVRi3oF6T/At8fpSrQL8l8Ft22
2ywHnUviv4UJK5aCzqPA5PqK8S3X0fkUzHGSUuoIQr+ioCDw+CXZQ+jXpP0u
Wo51cUIXUODfv3J1tRW6kIJtMXM69uuji0h8VkxLmTUHXUxB3qroJaJT0G/I
+vzcZ4XwSHQJBQkxlUen/T485FIKqkQ0r+1oQr8lx3tTb5r/Fl1G1qejs00t
nqHLKVj2aZ75zxR0BclfFbvE0qLRlWR98V6lc+cCuorEsx0fz8cfRb8j8+Hs
ivw3ruj3FASU5t6R3oSupsAy+Hz5UVN0DVn/Ik7LCy9G15L1/sYe3Tg1dB2p
f8zT6l1k0R8oOCdgKlePQteTfP3C9mqTXp8hN1Agk+Kf6NyC/kjBlwNnxG6/
RzeS8TX7yZOeV+hPJJ9jv+11S0M3UTBjWGn/30T0Z5I/XmOUHlxHN1MguunG
zUPn0V/I+iw7b7GDH/orBU/NXqc57kO3kPhR6vrZzxH9jQJzLv04a4VuJdcz
arjNGEN0G1kP7i6YeVQb3U5eV6rKGaGK/k7ByRTpziRpdAcFTnKpPq4j0Z0k
v6IVZhr3eQ/5B4nHDkty9drQXSQ/DYqeuvID+icZr3a7hu8tQXeT9fz+Kv3k
HPQvChSWnYgcQaN/UyDb+KnJ5z66h4LkqILvIlHoP+R47xUD759H95Lxmm5+
xOkYuo/MhzVm1xd6oPspiK69mjrdGT1Awf4F6nEzN6D/kvpozHktIzP0PzJe
NqbOO6SPHiTxg9XyzJ6PHkaDVnF34VRVtBANs/O8J0TJoIXJ6znJkmpj0CI0
SFsJJRUPHhqyKA2Xfq9LOPcTPZyGRGagafMX9Aga7kTlahvXoMVocDHpcDV6
gx5Jw6RI1R12z9GjaGjXDfp5nEWPpsHW4O6PrCS0OA0SrZ66krfRY2gSrxyi
/K6gJWhojJ33uz8IPZYm+cvM6Zf80ONoKLNr6tP1QI+nIdx9n93PnegJNHwu
0J70YhN6Ig0iZiNGJVihJ9Fw+N0UyRhjtCRpH52R41MWo6VoqH/z8t0bTbQ0
DS+iE01FVdGTaVg36GpoPgUtQ8Osa/I3749Dy9LgbqCuJz0cPYVc/7dtPyL6
vIYsR8OfkKa4OZ1oeRq6NeYtLP+MVqBB7IHsiYs1aEUaDn0qdNpUilaiITCi
IHNxHlqZtI93m/ucp+ipNIxQsXeYJ0BPo8F5Yei2lQno6TTcsN7huO8WWoWG
mJk/FsZfRavScDE8ofBnMHoGDVmcxaDVSfRMGv7+a32c5YOeRcNXy4Zcg/1o
NRpuKZUplzqjZ5PjRXMxnpvR6jSkl73RnLkOPYeG2II7Sa2r0Ro0XHg2rC8b
0HNp2L5GbDBuIVqThuqbk2/dnIueR84vsjrzrip6Pg3FagZGGfJoLTJedx+e
0DQJrU3Dxi9C4+TF0QtoeF95ZeoOYbQODUJu8zUzeg8OWZdc7whZRZUf6IU0
NJwuLr/+Fa1HxmOw1OLpDehFNBQW/ViQVoVeTMNmx7gHW9+gl9AQJ0IHSeeh
l9KgerzuUU0mehn5/C55iUcMWp8GViX5dNhDtAENuW0iIoFxaKDh94m3e89E
o5fTcLNBNCUiHG1Ig7bKsxTmItqIhh/fd25sCkSvoOF4wcWrysfQxmT+hY2y
dPVGr6Thzft6txf70atomGJtXavugjYh7bOkPPDOVvRqGja9l7RT3Yg2paGr
vHsha402I+2dek7SzhRtTsN456nvRI3QFjRUHl24L3sJ2pKM36/+OWcXoNfQ
EGlcm71VA21Fw8TdO9cbz0Bb0/C4ytltoRJ6LekvY5suXRn0OhpCJ9eWGU1A
2xA3Vv11GI1eT86309rhtAjalvRvbkB96oDnkO1o8I+vcO//hd5A2mfaByHT
DvRG8vdh2/zufUXb0/Ap1fqtRCN6E4k3ekXfT9agN5P5qKH0TLQC7UDaJytz
YVgxegsN2Q8sl8/NRzuS+FF7o7DiGXorDdv+3Mk/m47eRsPavuMqpjR6Ow1P
8wPfSKegnWgoKVBL7YhH7yCfFziqtDwG7UxDqvb3Ebk30DvJ9W/ON30ejt5F
Q83RR+cLQ9AuNChfa874eA69m4bMT6UFwqfRe2gYvWVYouYxtCsNLV4zTFwO
o91I+yQpRjzwRO8l81235czAXvQ+Gu7fTxLf4ILeT+Zz2KNJWdvRB2iwWbjr
8nwHtDsNptOdDqXYoT1omPxpE6W3Fu1JQ138nGVF5uiD5PMGyoT3rkJ70fB9
q2fPZEP0IbIejl4jUbwU7U3DlrNFupcWon3IeD/ivXOTFvow+Ty9notaGugj
NDhdkImVnIX2pWHfrvyrQtPRR8n6cE7SvlcB7UfDQ2P1T/0yaH8aNGZvUhkl
iT5GA509ccLUcejjNEgtb7hhNBp9goZmc/f0A8PRJ2k4ezd0a/wwdACJJ581
j3/r9xjyKRqWrVg3dmEP+jQN+StVBy90oQPJ9QS0rexoR5+hQWezRLV9CzqI
nF/2qOSSJvRZ0p9fjlNWDehzJB5Ja36urkEH02C9qV17fxX6PA3R1o+vSpSh
L9Aw2HR+kC5GXyTHG/9wp8tr9CUSH5O2ZKvkokPI+FUWFW59hr5M+g+UpqY/
RYeSeDBoNulqKjqMxGP95cU+NPoKidehJat2PkZfpSFsbLOXQzI6nIYzo4Is
tySgI2g40lKZ53IPfY2GGW1j3h29g44k8fO3n1fUTXQUDXdtI8KfRaKvk/aK
rJ/fdRV9g8w329/L5oSib9Ig1xP2eP9FdDQNfmfOBmScQ98i1xeUnTjxDPo2
ab+0xYoeAeg7NBh0TKioPoaOoSHpvXeu+VF0LA3/6umWPB/0XRrOt87RNfdC
3yNeuj72vTs6jvT3ksxZB/ah75P12nJs5jhXdDwNCQnCm7hd6ATSXzP29+ze
gU6kYZysfrDKNnQSDUVTD4z96oB+QMM5MfnDlD06mcS34H3Pz9qhH5L2NUr8
vMsGnUJe3zPznaU1+hENVTnrwvQt0Y/JeHV+Mm6hGfoJDRNu1RjrmaAFJH69
PjrX0BhN0XBUfUOOjSGapqHp2Yq+/QZohsQLjblvwpaiWRoOBEouzl6E5miY
qjtmwS9ddCpZT6qWMVoL0GlkfZ/YnOozH51O5pv3er28uegMGj46ds5WnIN+
SvLVgrnBfmroTBJPl9xY+3kGOouGl9eqjqxXQWfTcE887G/RVPQz0v/1YeWW
Sugcku87Pemtkkc/p6E2//3O3VPQL8j6KzZSWkQG/ZKGRWkWQvek0Lk0eI4o
nrZmEjqPhlFHMg8KTUDn01DasfNH+lj0Kxp2eEwN8x+Dfk2Dh5K9zerR6AKS
b4pf0pQfiS6kwXiGlVLPcHQRibeKBorVIuhiGjJMxs18KYR+Q/rrxOkF3KD7
kEtoeCDls1zwF11Kg+79YhOmH/2W5D+SJ42f9aLLyPzafEOrvAddTvLHID3x
jl/oChJfrpx8PaEbXUmOb8W6Lu1CV5H3W6t829eJfkfD+rKZyxO+o9+TeBsz
z621DV1NQ+fIzJ06regaGlY9dpxzpgVdS8O0cJpr+IKuo8GkZb+YYTP6Aw3H
/llKJjWh60n+pjO9Ru4TuoG0/5dCuysf0R9pkF2zNFCyAd1I8p+D7ttvfEB/
okFd4ehX9Tp0Ew3PHvtKPq9Bfyb5s+atxm3V6GYaFCVV14x8j/5C6p9Vm2yZ
KvRXkq+NvdXtWoluoeFk9+YZahXob2R9Sx3/tb0M3UrqP0Nj3bS36DYakk9f
kbhYim4n9aPv/T27S9DfSX9I6Jqav0F3kPV9mFiCbjG6k8T/1o6AWUXoHyQf
VamomlqI7iL9k5JwZ1oB+icNe+fu/zD7NbqbBosKneDFr9C/SL5lJ3/POh/9
m4aUmjVqB/LQPSQefxOXCs9F/yH5w/Yo5+cv0b00lLusk/nzAt1H8pPNN2cv
4N1PQ1tuW6T3c/QAyZ+sXzjl5KD/kvyWok5L8v5Hg1HK5r97n6EHSTzpcckv
zkYPY6BZ40qTLm8hBvTK4tbEZaGFGXBZ6i2mwFuEAd3TLaOuZ6JFGbAN7LBR
5j2cgcpXQc0PnqJHMKB2PCcDeIsxkOzFVtRkoEcy8EIsRNOf9ygGFsqdzJ/J
ezQDVyaURr9LR4szIJpd9CiE9xgGdJwrB9bwlmDA5rbtCWneYxmgXfKWNqWh
xzEQL5yikcp7PAMjem+vvcp7AgMbJfUTfXhPZOC79QJtJ96TGFhSq99mw1uS
vF9kRrkFbykGXi7IavnP0gzkdLepr+c9mYHqyujI/44nw8DpQ6XzD/OWZUDM
4+LP/85nCrn+rW11/52vHAMnSsZ2/nc98gxQy2aoTeavV4EBE4kDQVa8FRlQ
P6MrcZm3EgOOpSXMf+2pzMBAS8yp/9p7KgPvYxQO+fGexkDv9cxz1bynM3DB
YHyOPt+fKgw8z0iQS+StygCIakbI8+NhBgNTX1gtCOc9kwHV7YVd0vx4msWA
1oDr22jeagxobh1RpsGPx9kMdIsf637OW52Br7Nydbfx43kOGX9n30UO58e7
BgPz4rIUHvOey4D0jcs5Tvx80WRgvZp7kAI/n+YxcDnpwv4PvOczcFRxuk8c
Px+1GLgneyTGi5+v2qT/ir98N+Pn8wIG+kYWO6jx812HgUlzrndI8PFAlwFr
rdd3e3kvJK8nN/q18fFDjwGr5z6+zXx8WcRA+3yN6C98/FlMxrNZcnMHH5+W
MHC258n6QT5+LWUgnBJtk+Lj3TIGnoR7JWrz8VCfAZnVny9v4OOlAQMTx0Hs
KT6eAgMHX/jXs+Xo5eR49XGruvj4a0jGs/vrWm0+XhsxkKY28vbRd+gV5Pjr
g0ML+fhuzMCKxmB6Or8erGSgdLvl6JO16FVkvkuohH7h1w8TBibk7LG0qUev
ZsBh1AXI59cfUwbmywnvMWpEm5HxWG9R+JJfv8wZ2N49bPeaz2gLBqKSuOUN
/PpnyYD/ZKUNPl/RaxhYNbw1efI3tBWJdyMbVmTy66s1mU9HihXd2tFrGRAK
CFs2tQO9jsQLWdnYOn79tmFgymhb6zv8+r6egYf/LEzd+PXflsS/YxNC9H+j
7cj86n2qIvMHvYGBgHy/0b18frGRgX6dIKOPfP5hz0DGqalvS/j8ZBMD135G
cXl8/rKZgdDctb3/5TcODKw2p8IL+PxnC5l/TQvD3vH5kSM5/ginH21i6K2k
/83zBSP5/GobA64HxerU+fxrOwNj9xbtt+XzMydy/ZvL9wWNR+9gIOLup9pn
E9HODJzvepMhzOd/O4kNfCeYTUbvYuBYQ3ldlCzahbRv0HuVLjn0bgZk14e1
WCmi9zAg915Wk1VGuzKQMi30j8p0tBsDf/Pmro5SRe9lQPBdU2XyLPQ+BiQ3
fDp/fTZ6PwM/dt8NnKmBPkD6Z3GtZLom2p2BzdeElthqoT0YGD8lcuAPn497
MnBSVcchdiH6IIkPA15b1y9GezFQkD9q9Lhl6EMM1MY9cCjh6wFvBiIFsxwj
+XrBhwHxybYT9/D1xGEG8ifI+Rry9cYRBoJHud2cxtcjvgzsuWrmN5qvV46S
+XH+tWKfFdqPAYUJIwN/rEP7k/67M5/ttEUfYyBsagDTsxF9nIHU9pVBI/h6
6gS5vs8V8xS2ok+S+OQbQS1xQgcwsOm5kMz2nXz9T8bzyUjHkN18/c9Aln7X
pVw3vv4n7zd7mCx6gK//Sbyt9X9q6snX/wyISKm8uHaIr/8ZeCO9N/f7Yb7+
J/F+kt4rcz++/mfg8xaP0ifH+fqftLevTJPyKb7+Z+DDPN0REXx9e5GBfdea
l0oF8/U/A77aNheu8/VxCANmfdyAGl8/X2Ygad+WkGy+vg5lwDjlpoUjX3+H
MXD3bP2iEXx9foX054Ub9vRtvv4n642yA+12l6//GbC7lLpmTjxf/zOQoGqv
2Z3E1/9k/dLv2fgiha//GfDcuLHihoCv/8l8nb/xvh/L1/8MHCiur9iZztf/
DIRcb965IYuv/xlQ8vfeuvY5X/+T9brlQb5NHl//k/7+9fieYwFf/zPw1ivn
r8cbvv4n+d5Y9aqL/PcpMSQ+mq/WE/Dft8Qy4JUePLOB/z7mLgPOg+5JUvz3
NfdIvG05/3od/31OHANOof7BUV/5+p+Bwwf6ulva+Pqf5JOeLyUMf/D1P1n/
ZJnamF98/c/AmAuUy5g+vv4n62dqGuf/j6//yXqxq7ysVxi/n0pm4PgGpWd+
YuiHJF45FpwZPQadwsDII1M0b49HPyLxq3Nrpr4U+jHpX7fhy5pl0U/+v36c
eBKuiBaQ9Zd2nGHFf/9GkfXxucHtifz3czQDZZffzvwwB80wsH977Ysn89Es
WS/PLDgWoovmSLzxf7nFewk6leRHEy+47gJ0GgNFXjeStxqj0xmIcZbRcDJF
Z5D2Ml7UtW8N+ikDVw3Nhp+yQWeSfHzfw30xG9FZ5PqaK3Vfb0Fnk/j73Xxn
vxP6GQMbJiUP192NziH5mFW8wpF96OcM3FAXZfP471tfkPytNqpakf8+9iUD
8q6bLh/zR+eSeF+5uqklAJ1H8jmdY7UOZ9H5DBjUqQXUXES/YiD6xIWP26+g
X5P4tkVEqDsSXUDae0Pft4u30IUM3J7ZE699D13EgKnvlRWNiehiBjwmBeRF
PUK/YaB4xLklDgy6hJx/QWCiWga6lIFZrnsVh+Wg35L5+sDkTkMeuoyBdYsW
LysoQpcz8Dr3yEBWGbqCAeXl9i2Z79GVDBzpNhibX4+uIvGUTfSq+Yx+R+a7
1PzZva3o9wwEHj2hO7ULXc1A/Y7t99b9QdcwMCrgyYmQf+haBmq+u1RWieLv
DXUMTFt/9aGaOPoDA4+mb5UK5H+vqCf1BTdaun0yuoHMx4sd6VsU0R/J+buf
nFijgm5k4Hq8lep2dfQnUn/NERrWPR/dROJtqc3DED305//334ylegboZgaE
d7nS34zRX0j+PV17Rrw5+itZH4bFRx1Yh275f3wcp7TCHv2N5JuF+VnTtqFb
SX570OrsGP73ozaSX8osCxLif19qZ6Bq5/JXQofQ3xlo7VbfKOGH7mCg8PCX
FSqn0J0k/ov6Rq4MRv8g+bz26B0HQ9FdDOh7VDxOjkT/JPF4zuKQrtvobgY+
jRWMMYpH/2JARf/dglsp6N8MBF1xlBZj0T1kfH2QzzrC/173h1yPlPCyvpfo
XgYexChFny5C95H1IeJup3wFup+sJ50fDLNr0QMMbC1fEru/Cf2XgZLyVcrq
beh/DGTOTcnv+okeJPmxhH5Sbj96GAvjihoq40Tw90shFkT179qH8b9vCrPg
8JU2Ps///inCwva1O+5flkOLsmBmrHQpdjp6OAuLJH3G5KijR7Aw7dNUzXZt
tBgLxzt+jFBdih7JQtS7+tjdK9CjWHjZMlw63Rw9moX7O+7slV2PFmehvXBi
+ikH9BgW+rU/jP3L/14swUJIYNLhk/vQY1m4XL1yjJQ3ehw5/6e3q6hj6PEs
PD1V3LYtCD2Bhec3RzvIX0ZPZOFrPbWoKRI9iYUdH9ZeSY1BS7LwZ0mI/40k
tBQLmoP6oy9QaGkWEr/YrQ7mf1+fzELO8z6LiFy0DAuf8vfNfvwGLcvClZZp
P969Q09hIWH+Q2psI//7PwuHFErOrG3lf/9nIe33Ab+73fzv/yx0acbEi/7j
f/9nYXJOtLynGN5PoMRC+LmCto7xaGUW3JLYOb78/QhTyfu/yXdKqqCnsUBN
bDB9qoGezsKHjROsDy5Eq7Dgnd0mvXg5WpWFbp/qR2PN0DNYCDLfr91lg57J
wnrRvYKmLehZLNQV5xg3u6DVSHs/uDD4m78/YzYLU97L/Jbm799QZ0Fn4Qf9
lWfQc1j4mRnYceoyWoOFU63fFMquo+eS61Of1DAvDq3JAm2tZx79CD2PhTyx
Fz5y6ej5LLQttz4a/xKtxcJC6sFe4xK0Ngv7TCqcuqrRC1iYEDrglfIZrUPG
u92Z9KOdaF0WIrskzO370QtZ0GvabGAyAu+n0WNBxNgreeUE9CIWNEwuPFwv
j15M+jN9/CavmeglLMSdu1NwTwu9lPS/vozsl2XoZWR+nDm1edFqtD4LQs80
Um7YoA1YUDgcNnvCVjSQ/hnY+zXcFb2cBZuGh8JzvdGGZD5cOhFWeRJtxMKW
ajM65CJ6BenvUw9PbubvZzJmYVPNaenFceiVLPww+has9gS9igWjynn/Zmei
Tcj1/mgN0n+NXk3mR3nrqh2VaFMWnJcZ2N1oRJuxIDlsQVnzd7Q5C04Jdu+M
+tEWLFxU8gkQiOH9XZYs/DW4+0NHEr2GBbXzc6BAGW3FQkl+7HlPDbQ1C3sT
6rvnLkavZWG4b2/435XodeT1N5aX69ehbcj41zgpXLEVvZ6Fh9m+oz7sRduy
oCyx52XfEbQdiWexOevVg9AbWFhSbVi9/yp6Iwuxpzo98mPQ9qT9H08BnUfo
TSw0OdtvZ56iN5Pz2x7WZ1aAdiDt1dCt9vsdegsLNZv7JtDNaEcSfze5lQd1
o7eSeKO9LcxdGO8P3MZCes0y9/3j0dvJ+EllQk4qop1YWOBeIpk0B72DBZfM
BuUW/n5EZxYMVl16t3Q1eicLE70rHePs0LtY2GovWqO6E+3CwpeT6fvSD6J3
s/Dmw0+DXQHoPWQ8JfR7qoWiXVkoMPWZJ3wH7Ubiwcpd13/w91/uZUFshlJN
TyZ6HwsNblbSUsXo/SzMPTvi4Oo69AEWfAvi5a62od1J/2ztX/CnH+3BQqil
8lcPcbxf1JOFV17XHIbLoQ+SeHhxRWbKbLQX6a+lO7U9FqMPseAofqfZ3BTt
zUJYQL8Q2KN9yHxe1JBitgd9mAXLT4pyB46gj7BwaV/7vuRzaF8SPxRbKkX4
+2ePslAbP/mYZxLaj7y++WvYQDran8RbIx3DW4XoYywcXruC3VSHPk7i+5V8
Za3v6BNkfm759GDaIPokCxtOqZ7T4O8HDiDzLTzky9qp6FMsHFG41h2qhT7N
glb9tI/tRuhAFs6qnK9zWo8+Q9b34fem/NqJDmLhYyr96o4P+iwLKq3+8nvO
oc+xkDJuibHlDXQwC4/zbh2yfIg+z4KMfv2nPdnoCyy0XkrIjn2LvkisN3d5
bxP6EgtKJxWiXHvQIeT4fxKG9Y/C+7Uvs6D+5WzifXl0KAvNFX+K3DXRYSz0
jk29s9EQfYX8/SwpB8f16KssZIV9XBjggg4nfz/Pf89LX3QEOd/WKo0Zl9DX
SH/b7km/F4OOZMHYZrnFCgYdReLtiz4F4dfo66Q/NPJd6uvQN1io9j6wu/YH
+iYLfUnWm3uH4/3y0Syspbb7605B3yLrn6qOxJW56NssTBrfs3KiEX//P8nf
yg7ueWzH3//PwvmBKay7G3//P7HaH7d1J/j7/8n868ostA3n7/9nIfBd3uSj
Sfz9/yzUL58U8jybv/+fhReeZj4alfz9/yyUX8+dlNbK3//PwviaoLjdQvh8
QiILtptWei2djE5iYbmr80stDfQDMr5jhtWZG6GTyXov3iIcvBH9kAXxjfH+
7fvRKSyMUQ64djAQ/Yicn5xvpPJN9GMWUg9bF/wQoJ+Q+TXnrM/X12gBCxLm
u9tEG9EUC8O8Bj1W9aJpFkbrT9v4aDw+78GQeJDo2WYwC82ykPTAyfe3AZoj
7ZGstb7cDp3KgvX07lfv96PTWLgw+/JssSB0OgsdcwvyN99GZ5D8KUq/p5pD
PyXrW6jzqJOl6EwWpJiodWu/obNYaOy7IBUnhM/HZJP2Foz4uXAs+hlpH6nM
9g9T0Dmkf5Ifzb8+E/2cBbvAta1uC9AvyPvvhayyWY5+yQIT/tbT2hKdy4L2
3/6bOzah80j7qOzrvuCCzmfhbdSouCL++aBX5PorNRunB6Bfs6Ba9eJdSAi6
gIX8qnlpktHoQhaKB7exKUnoIlKPKBULbU9FF5PP23ewQJ1/nukNyaft6ldK
VKBLWDBdOeP+iE/oUhJ/fDuUZH6g37KwsdX98/JBdBkZXyrec05J4PNV5WQ+
6jnMrpdDV7AQ/VhGat1sdCUL+hd1pzfooatY2Gz0IThwFfodCzPzywJX2qLf
k/GScNhc2RldzUKRapmU5EF0DcnXrALlpwWga1kINvgQZRqKriPxDYoKLtxB
fyD5gFTHt9ZH6HoW5CF4gTP//FoDid+WEZ9736A/svBNN3VJYj26kcSvtNVe
hzrQn0h7l9uWbhlEN7Gkvoy75DQOn6f7TPKRu2znKSV0MwuzivJ0nmuiv5B8
JPTtbUVAf2XhXNnVgxFW6BYWTo650q2xDf2NXP/Yyc6N/PN/rWT9fq4lTp9E
t7HA/s6bExeGbmchRvvMeOYu+jsLEPfzbxON7iD5k8TOZdp56E5SX6SNk7nz
Dv2DhdNX/lbN/YbuIvncQrn82n70T9LeE3UXJEng84/dpN6MHLY7Sgn9i4yP
01JU4nz0b5JvK/U71Rqhe8j5Rs3I17RF/2FB+PjgvHsu6F4WOqd+6ljqi+4j
9aHKLLse/nnOfhZMJKelld9GD7DgJ6a6t0KA/kvWI9uzL3tz0f9I/HdMkTWs
Rg+S+W4v8uxhO3oYB836QpIghM+fCnGwJkfP7bckWpiDGmtrlbez0CIcDC4a
e61sKVqUg22Fa1X6rdDDObh8a8bM1c7oERy4NKQOPj2MFuPgGrN7rP1F9EgO
bme8oBRi0aM4KAm+tH4Uhx7NQcTmVzYKRWhxDl6nP5ps34gew4GpTb5odg9a
goN1b2oC1kjg875jOZjwWvTfyOnocRyc+ZhS16aHHs/B/so7F/st0RM4EMsb
8NNxRk/koOiE8ZTrvuhJHIzrdCmbE4qW5OB65LMprfFoKQ5OLVdwqsxCS3Pw
+0aOckclejIHgSrer3S+o2XI64OnyhOH4/PSshxEjn2WZqmAnsLBhpGXf0/V
QctxYPnih4iqBVqegzsX/bZucEYrcOCf4BGc4YdW5MD6c+rwteFoJQ66NrZb
y6aglTloN3DsnpiPnsqB8DdlT4OP6GkcFNePsr3Vh57OwfIpWTO0JfF5cxXy
99vTDvRroFU5iK20fvZnFXoGB43BWffVt6NnciDjqEZf4Z9vn8XBkcb4owsi
0GocpBxaFyb+BD2bA4tzhiFTitDqHCwddmPM9q/oOaT/sqNiG0XweXsN8rpH
RNcNJfRcDnZvmxB5eQlakwP3S6zeMzv++X8OjH64e805yD//z8HL6j2/ykL4
5/85SIiU2cYl88//c7BKf5VT9Wv++X8OTtyYlrD4K//8Pwfn90Hje36/AV0O
tLISs7np6IVkfhz59qdqOVqPAx29iYZ6W9GLyPUsfbW+2h+9mMyvx7admTfR
SzjIkEkuac5AL+XA9Y8Xt7YWvYxc/3MFN5EBtD4HFX/rn/TK4X4LBhxI3S2z
1V2KBg5aCnpU0jajl3Nw9iK0n/ZDG5L5r373ZFQ02oiMB79Z6X+y0Cs4sE/o
3H/nI9qYAw85M7+rIrh/xEoOSouycipU0avI9b9TnO5kgjYh8WDNpVP6rujV
HPzZH13ichFtyoGtwpruxsdoMw5GCNQakivQ5hzEzzIPet2LtuDgy8UjTYsU
cX8MSw66GzU7BwzRaziSj6YkSrqgrTgoc5eafPIC2poD49I9S1cK0Gs5iLlL
q25/j15H4surMY2Vg2gbDvqKE0/dm4H7e6wnf39ESPWNBdqWjF9B0QcbL7Qd
B3sb1d8uvInewIFZ5oPph1+iN3Kw4OKeAdnvaHvyepPiObnJuB/JJg52FOgN
+gN6MweKx3+GGu9BO5D2vrb3xIEr6C0c1GVkiwtloR05mPZB1udPC3orOd7f
0gkbpHC/lG0chB09ozZ1OXo7Bw1CelLr96KdSH912s/7FYnewUHH4YEa4Ty0
Mzme2Xv/Y93onWR87H9qqDsB93fZRY6nO/ux8yy0C3l9VWdusQF6NweTnjka
edih93DwcXLrV6v9aFcOvrbr+rmeQbuR+WY++1XOLfReDlYuG7xmx6H3cSCU
uy5jdil6P+kfsyUjln1DH+Ag397K4pLI5SG7k/HSkOs6TQHtwYFhkb9Bjy7a
k4MPMyYnjrVGH+TgUITQ5d2uaC8Ovu0OrxsZiD7EQa2Yml/7bbQ3Bzld8ttk
MtA+HBTo9/ufq0If5qAtLKhkxU/0EQ4KmZp1JuNCh+zLwfvog+Lh6uijHPyT
t/k31wTtR/pfWlJP0hntT9Yb+2GvVp5EH+Pgxfh5uXm30Mc5kH1za/W1p+gT
HExca+6RWoM+ycHIBosdc/rQARy82tJt0iWD+yGdIuNPvslyjB76NIk/1tqJ
vnboQPJ+56Ary73RZzgw/6Nn4RiBDiLz9dFu2UoWfZasJ5/nrrj/Dn2OA1r6
oWJ5LzqYA88VMGyLHO73dJ6Dg3Ovrly5DH2BjM/DVesvOqIvkvH5qvTawpPo
SxzklkmeMLqHDiHH2694RpCPvszBc8GJ2Wfb0KFkPE0d3/VyPO5XFUbe3yzi
5qqDvkLW1xXqc33t0Vc5mKMoVfn7GDqcOF1/VMM9dATpn5eGXnqF6Gtkfl4e
VSL0Ex1Jxv+TvqplU3C/rSgO3rRo2nzn9+O6zoHe6QRJqT3oGxxYTT/VJQhF
3+TgRp9mSWE6Opr0b2JC5LbP6FscCKKpFQfH4f5gtzm4KDe5dMRi9B2yvnRV
mcg5o2NIPBjwyxaEoGNJ/26ytnubgb7Lwcao6IWHW9D3OJib1h+eKI37l8Vx
EK5ziXFZgb7PwWPJjZ9oD3Q8OT9F/0NX7qATOFBfcLViRCk6keSjvb89xgnh
fmpJHMzLHVf8aD76AQfJ4pRn+3Z0Mjmelve/vCvohxwMSAyfbJSHTiHj7Z/f
wW196EccBJVU1KrOxf3eHnPAHNJRDduOfsLBowxHrQcRaAHJb4Rlez2L0BTJ
z1RD/dtFcL85moPQ+5Xv5JagGQ4cV+hrCnuiWQ6U/31Mi01CcyTfLJXME+H3
s0vl4O/MhseqSrjfXRoHrTqn2sbYo9NJ/rJhoD/jKjqD5AvxZw4veot+ykFw
vc6kwHG4v14mB1FpPufjLNBZHGSKpmVGn0dnk/nj+TvSoxD9jOTHZxxUZ0jg
fn45JL+I+H3sqSW//x8HcivMPi69zO//R/IJv8lR8eX8/n8cxK0dlBwui/sH
5pL489q0ynYLOo/Mn5/3T8TcRedz8M7jZ0lbK/oVqUeaA9YoTMH9CV+T9aHq
lXu+PrqAg7Gsr32fE7qQg10Tr96lz6KLOOg3Xdc/kIIuJvND4dGUskr0Gw6y
djW8WfwPXULim1VVr+4M3F+xlNQrwsVnX/P7L74l+Y9WScAfb3QZB/ohFoPP
76DLyfVtfTRzXiG6ggOFw71LoQddSfKhwm1XeqbhfpBVHIzOF4rcuAb9jvRf
ln/hbn7/yPek/rmXmaeeiK7mYNaGmCWx79A1JN/Iyv1RJob7U9aS+SVjUfRs
IbqO5BsxiQUHXNAfyOfJHB/+ORJdz4HG4sMp0wrRDWT+5C+QmzuI/kiuz1SH
E9fG/TIbyfkI10hm70J/4mDJ8V31a26im0h+qib+JasM/ZkDacO/8lLiuF9n
MweX3kdftTFCfyHzYVrC2YCj6K8c9HC77O4z6BZyvE2DAS9/oL+R/P3xs4xP
GrhfaCsHTx5dfCfmim4j9d3KFe8XJqDbSXz8trT98Ff0dxKfK+oOlM/C/Uk7
OKg6r9NpsQfdSRySNKv9AfoHOb++Z6NTO9FdJH/43JLO6eD+pz85+B7seu2n
L7qbxKOxTjruz9G/yPnWvN2+aAzur/qbfP67Qg0bW3QPyW/tBY2v7qD/cNA0
O2xc7Hd0L8mf8iICvy7F/Vv7ONA2ur419jy6nwNv+bLB2jr0AMl/g65PvDUP
94P9S+rpF9vDf/L7xf4j9aIEU91UjR7kgBL2OOWnhfvLDkuF+aYyl/OD0UKp
IOpibl/ajBZOhdjGw5GxRrhfrUgqdNXfczWLQYumwrmwfesrhHG/2+GpoP9r
8SpLZ/SIVDBxvHnn+Su0WCqcqmtdZDQP988dmQpByhMca66hR6VC0U3z/UVC
j/Tzaj6ryBHz+/H+D3TCshI=
       "]], LineBox[CompressedData["
1:eJw113lYT10XBmCZM7yIzDIUIZkzlFgpIlKh0kAylAxJhRQyhCSFlKTQoBRF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       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 8191.}, {0, 9.554389806033786*^-7}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.8572085915266733`*^9},
 CellTags->"b:3.4.6",ExpressionUUID->"8f8885ea-371f-465e-83d9-bb9094c7d91b"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.003666666666666667],
       AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJw83XlYjG0bBvDs2bNESELInj0lTkQhhKKNFq1Upk17Tfu0zzRLMwnZs5RS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       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{
     FormBox[
      RowBox[{"Measured", " ", "number"}], TraditionalForm], 
     FormBox["Probability", TraditionalForm]},
   AxesOrigin->{0, 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->400,
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0, 8191.}, {0., 0.3}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {0, 0}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.85720859180297*^9},
 CellTags->"b:3.4.6",ExpressionUUID->"01606e01-3e64-4a7b-8cba-6588bc9eb555"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Step 7: Attempt to infer the period r for all possible measurements\
\>", "Subsection",
 CellFrame->True,
 FontFamily->"Times",
 CellTags->{
  "c:36", "b:3.4.7"},ExpressionUUID->"e091aa0c-714d-4e06-8a2f-d32343528a13"],

Cell[TextData[{
 "Finally,  we go through all the measurements, compute the periods and check \
the \nfactors we would find and their probabilities. \nSchematically: we \
search for the period using ContinuedFraction, once a possible \nperiod is \
found it checks whether ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["xguess", 
    RowBox[{"r", "/", "2"}]], TraditionalForm]],ExpressionUUID->
  "9647b225-5e73-459c-aba7-551849f2b568"],
 "is an integer, if so, it computes the two \npossible factors f1 and f2."
}], "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.7",ExpressionUUID->"d5f16e23-ba79-4739-8e7c-179a1b65c4ed"],

Cell[BoxData[
 RowBox[{
  RowBox[{"r", "=", "0"}], ";"}]], "Input",
 CellTags->"b:3.4.7",ExpressionUUID->"a07dce25-960f-45b5-970a-f73f2465d8e5"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Alldata", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Period", ",", "Number", ",", "\"\<f1\>\"", ",", "\"\<f2\>\"", ",", 
       "Prob"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"j", "=", "1"}], ",", 
    RowBox[{"j", "\[LessEqual]", 
     RowBox[{"Length", "[", "MeasProb", "]"}]}], ",", 
    RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"FromContinuedFraction", "[", " ", 
          RowBox[{"ContinuedFraction", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"MeasProb", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}], 
             SuperscriptBox["2", 
              SubscriptBox["n", "1"]]], ",", 
            RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "\[NotEqual]", 
         FractionBox[
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}], 
          SuperscriptBox["2", 
           SubscriptBox["n", "1"]]]}], " ", "&&", " ", 
        RowBox[{
         RowBox[{"denomi", "[", 
          RowBox[{"i", ",", 
           RowBox[{"MeasProb", "[", 
            RowBox[{"[", 
             RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}], "<=", 
         RowBox[{"MP", "/", "2"}]}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"r", "=", 
        RowBox[{"denomi", "[", 
         RowBox[{"i", ",", 
          RowBox[{"MeasProb", "[", 
           RowBox[{"[", 
            RowBox[{"j", ",", "1"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"IntegerQ", "[", 
        SuperscriptBox["xguess", 
         RowBox[{"r", "/", "2"}]], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"f1", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], " ", "+", "1"}], ",", "MP"}], "]"}]}], 
        " ", ";", "\[IndentingNewLine]", 
        RowBox[{"f2", "=", 
         RowBox[{"GCD", "[", 
          RowBox[{
           RowBox[{
            SuperscriptBox["xguess", 
             RowBox[{"r", "/", "2"}]], "-", "1"}], ",", "MP"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"MeasProb", "[", 
             RowBox[{"[", 
              RowBox[{"j", ",", "2"}], "]"}], "]"}], ">", "0.001"}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", " ", 
            RowBox[{"{", 
             RowBox[{"M", ",", "1"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"f1", ",", "f2"}], "}"}], "\[NotEqual]", 
            RowBox[{"{", 
             RowBox[{"1", ",", "M"}], "}"}]}], "&&", 
           RowBox[{
            RowBox[{"f1", "*", "f2"}], "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f1", "\[NotEqual]", "1"}], "&&", 
           RowBox[{"f2", "\[NotEqual]", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Alldata", "=", 
            RowBox[{"Join", "[", 
             RowBox[{"Alldata", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"r", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "1"}], "]"}], "]"}], ",", "f1", ",", 
                 "f2", ",", 
                 RowBox[{"MeasProb", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "2"}], "]"}], "]"}]}], "}"}], "}"}]}], 
             "]"}]}], ";"}], "\[IndentingNewLine]", ",", 
          "\"\<you were quite unlucky in your xguess!\>\""}], "]"}]}]}], 
      "]"}]}]}], "]"}], ";"}]}], "Input",
 CellTags->"b:3.4.7",ExpressionUUID->"df519045-8997-4196-a8ba-7bcb9e8f5e80"],

Cell[CellGroupData[{

Cell[BoxData[{"Alldata", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TotalProbSucc", "=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Alldata", "[", 
      RowBox[{"[", 
       RowBox[{"i", ",", "5"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3", ",", 
       RowBox[{"Length", "[", "Alldata", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{"\"\<xguess=\>\"", ",", "xguess"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"Print", "[", 
  RowBox[{
  "\"\<The TOTAL PROBABILITY OF SUCCESS is thus:\>\"", ",", "TotalProbSucc", 
   ",", "\"\<  with xguess=\>\"", ",", "xguess", ",", " ", 
   "\"\< and measured register=\>\"", ",", "reg2Measurement"}], 
  "]"}]}], "Input",
 CellTags->"b:3.4.7",ExpressionUUID->"660471de-1a16-41d9-8c6e-ddb17a01c4c4"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"Period", "Number", "\<\"f1\"\>", "\<\"f2\"\>", "Prob"},
     {"\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>"},
     {"5", "1635", "11", "7", "0.0015832810966176943`"},
     {"5", "1636", "11", "7", "0.0031791307582666874`"},
     {"5", "1637", "11", "7", "0.009347388976710917`"},
     {"5", "1638", "11", "7", "0.11456248251565725`"},
     {"5", "1639", "11", "7", "0.050942048993393454`"},
     {"5", "1640", "11", "7", "0.007167306150551231`"},
     {"5", "1641", "11", "7", "0.0027156058538077518`"},
     {"5", "1642", "11", "7", "0.0014171846576107403`"},
     {"5", "3275", "11", "7", "0.002169594068188375`"},
     {"5", "3276", "11", "7", "0.010960417873372395`"},
     {"5", "3277", "11", "7", "0.17499686726268446`"},
     {"5", "3278", "11", "7", "0.004850768376092203`"},
     {"5", "3279", "11", "7", "0.0014401575781801989`"},
     {"5", "4913", "11", "7", "0.0014401575781801982`"},
     {"5", "4914", "11", "7", "0.004850768376092203`"},
     {"5", "4915", "11", "7", "0.17499686726268443`"},
     {"5", "4916", "11", "7", "0.01096041787337239`"},
     {"5", "4917", "11", "7", "0.002169594068188374`"},
     {"5", "6550", "11", "7", "0.0014171846576107403`"},
     {"5", "6551", "11", "7", "0.002715605853807751`"},
     {"5", "6552", "11", "7", "0.007167306150551229`"},
     {"5", "6553", "11", "7", "0.05094204899339349`"},
     {"5", "6554", "11", "7", "0.11456248251565725`"},
     {"5", "6555", "11", "7", "0.009347388976710917`"},
     {"5", "6556", "11", "7", "0.0031791307582666874`"},
     {"5", "6557", "11", "7", "0.0015832810966176943`"}
    },
    GridBoxAlignment->{"Columns" -> {{Center}}, "Rows" -> {{Baseline}}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208593482675*^9},
 CellTags->"b:3.4.7",ExpressionUUID->"f4adddef-0c7f-4d98-afc5-805531ff7432"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"xguess=\"\>", "\[InvisibleSpace]", "64"}],
  SequenceForm["xguess=", 64],
  Editable->False]], "Print",
 CellChangeTimes->{3.857208593487649*^9},
 CellTags->"b:3.4.7",ExpressionUUID->"a55a1b3f-ac77-41a6-9665-c22e0b0842b1"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The TOTAL PROBABILITY OF SUCCESS is thus:\"\>", 
   "\[InvisibleSpace]", "0.7706644683222669`", 
   "\[InvisibleSpace]", "\<\"  with xguess=\"\>", "\[InvisibleSpace]", "64", 
   "\[InvisibleSpace]", "\<\" and measured register=\"\>", 
   "\[InvisibleSpace]", "71"}],
  SequenceForm[
  "The TOTAL PROBABILITY OF SUCCESS is thus:", 0.7706644683222669, 
   "  with xguess=", 64, " and measured register=", 71],
  Editable->False]], "Print",
 CellChangeTimes->{3.857208593489663*^9},
 CellTags->"b:3.4.7",ExpressionUUID->"fe58f465-af30-4318-8add-2197fc98fc5b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SessionTime", "[", "]"}]], "Input",
 CellTags->"b:3.4.7",ExpressionUUID->"66632646-9f1c-477c-bb1a-2220bd57b970"],

Cell[BoxData[
 TagBox["7.034958`7.298806502456124",
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208593496271*^9},
 CellTags->"b:3.4.7",ExpressionUUID->"ececa644-1a9b-4896-849a-c240279dbc09"]
}, Open  ]],

Cell["\<\
In these examples, where we consider small numbers, 21, 35, 15, 33, 55 etc, 
it is possible to plot the function xguess^j and check whether the correct \
period
was found with Shor's Algorithm\
\>", "Text",
 FontFamily->"Helvetica",
 CellTags->"b:3.4.7",ExpressionUUID->"226ff202-9888-49a2-9320-8d3a2c9f85c3"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"Table", "[", " ", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       RowBox[{"xguess", "^", "i"}], ",", "M"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "60"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"a", ",", 
   RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.625323644663308*^9, 3.625323645505684*^9}},
 CellTags->"b:3.4.7",ExpressionUUID->"58762dcb-c072-4bc8-a28e-a1e30cce3aaf"],

Cell[BoxData[
 TagBox[
  GraphicsBox[{{}, {{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.011000000000000001`], AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxdzssKQVEYhuGdkaGBgYEBkiTJ+cxeTtt2DJegjN3CujSX5BLQXp9ar9Lf
p9XjLd6ft0cqCILp9/u7yecdJvdi3A/uNtxNu7t0N5Pc19XtrPGdnPG9PLwC
vBK8MrwKvCq8Grw6PL2T14TXwrs2vA68LrwevD68AbwhvBG8MbwJvCm8Gbw5
vBCe8T2rLW/he3YBT++cZ7XVt/I9u0Lf2vestvo26NPW/0Toi9C3RZ+2+mL0
xejboU9bfXv0aavvgL4D+o7o01bfCX0n9J3/3ge3s1Xt
       "]]}}, {{}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0., 0},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   Method->{
    "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
     "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Identity[
          Part[#, 1]], 
         Identity[
          Part[#, 2]]}& )}},
   PlotRange->{{0., 60}, {0, 71.}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.02], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.857208593535762*^9},
 CellTags->"b:3.4.7",ExpressionUUID->"8ede2d58-e834-4d41-9986-56da39c73074"]
}, Open  ]],

Cell["Quit", "Input",
 CellTags->"b:3.4.7",ExpressionUUID->"ef71a855-21d0-4afc-85fc-6e29744051bc"]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{890, 606},
WindowMargins->{{231, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingStartingPageNumber->53,
PrintingPageRange->{1, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{612, 792},
"PostScriptOutputFile":>FrontEnd`FileName[{
  "export", "home", "brunojd", "PITT", "QC", "QdensityPK"}, "35_22.ps", 
  CharacterEncoding -> "WindowsANSI"],
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintingMargins"->{{54, 54}, {72, 72}}},
ShowSelection->True,
TaggingRules:>{
 "PaginationCache" -> {{2005, 8, 9, 3, 36, 22.593709`8.106562514035977}, {53, 
    53, 53, 53, 53, 53, 53, 53, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 
    55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 55, 56, 56, 56, 56, 56, 
    56, 56, 56, 56, 56, 56, 56, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 57, 
    57, 57, 57, 57, 57, 57, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 58, 
    59, 59, 59, 59, 59, 59, 59, 59, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 
    60, 60, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 61, 62, 62, 
    62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 63, 63, 63, 63, 63, 63, 63, 63, 
    63, 63, 63, 63, 63, 63, 64, 64, 64, 64, 64, 64, 64, 64, 64, 65, 65, 65, 
    65, 65, 65, 65, 65, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 66, 67, 
    67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 67, 68, 68, 68, 68, 68, 68, 
    68, 68, 68, 68, 68, 68, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 
    69, 69, 70, 70, 70, 70, 70, 70, 70, 70, 71, 71, 71, 71, 71, 71, 71, 71, 
    71, 71, 71, 71, 71, 71, 71, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 72, 
    72, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 73, 74, 74, 74, 74, 
    74, 74, 74, 74, 74, 74, 74, 74, 74, 75, 75, 75, 75, 75, 75, 75, 76, 76}, 
    Automatic}},
CellLabelAutoDelete->True,
Magnification->1,
FrontEndVersion->"13.0 for Mac OS X x86 (64-bit) (December 2, 2021)",
StyleDefinitions->"Classroom.nb",
ExpressionUUID->"4b383f03-b9b3-4c21-acb1-ec20aefffbcb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "c:1"->{
  Cell[545, 20, 117, 1, 51, "Title",ExpressionUUID->"5cbfd1a9-b24b-490f-b89a-0154cd6bb247",
   CellTags->"c:1"]},
 "c:2"->{
  Cell[1189, 39, 334, 7, 51, "Section",ExpressionUUID->"907f866b-f1f1-4a7e-9031-4b4cff6a52be",
   CellTags->{"c:2", "b:3.0"}],
  Cell[4800, 125, 161, 2, 51, "Section",ExpressionUUID->"04b471a7-d63e-4fd6-bc85-119c0edac590",
   CellTags->"c:2"]},
 "b:3.0"->{
  Cell[1189, 39, 334, 7, 51, "Section",ExpressionUUID->"907f866b-f1f1-4a7e-9031-4b4cff6a52be",
   CellTags->{"c:2", "b:3.0"}],
  Cell[1526, 48, 3004, 67, 523, "Abstract",ExpressionUUID->"900ba9e6-ee5f-439e-82e3-dc68971e5335",
   CellTags->"b:3.0"],
  Cell[4533, 117, 230, 3, 22, "Text",ExpressionUUID->"06a8d43b-f285-4c75-ae6f-593ebe1e7985",
   CellTags->"b:3.0"]},
 "c:3"->{
  Cell[4986, 131, 237, 6, 60, "Subsection",ExpressionUUID->"88f3631c-804c-4474-babf-cd4d489e78da",
   CellTags->{"c:3", "b:3.1.0"}]},
 "b:3.1.0"->{
  Cell[4986, 131, 237, 6, 60, "Subsection",ExpressionUUID->"88f3631c-804c-4474-babf-cd4d489e78da",
   CellTags->{"c:3", "b:3.1.0"}],
  Cell[5248, 141, 367, 7, 64, "Input",ExpressionUUID->"b4eb02b7-74b4-442d-936a-abbd145f01f5",
   CellTags->"b:3.1.0"],
  Cell[5618, 150, 349, 5, 24, "Print",ExpressionUUID->"65d05b40-19dc-4dd1-8ed4-49b41aeb2f2c",
   CellTags->"b:3.1.0"],
  Cell[6004, 160, 167, 3, 46, "Input",ExpressionUUID->"649e5966-1fc3-4e8b-b424-4b2d1b82b726",
   CellTags->"b:3.1.0"],
  Cell[6174, 165, 57032, 1281, 493, "Output",ExpressionUUID->"13adc82c-aea0-47cb-8afb-8300adf6340b",
   CellTags->"b:3.1.0"],
  Cell[63221, 1449, 253, 5, 23, "Text",ExpressionUUID->"2f36eb48-6974-4bac-b28c-d45b8fef80e2",
   CellTags->"b:3.1.0"],
  Cell[63477, 1456, 1531, 42, 101, "Input",ExpressionUUID->"dc00caca-95c7-42c7-9d5d-bd9e64252067",
   CellTags->"b:3.1.0"],
  Cell[65011, 1500, 868, 24, 87, "Input",ExpressionUUID->"8647f407-8468-4a92-8945-e6c6a324eeda",
   CellTags->"b:3.1.0"],
  Cell[65882, 1526, 1156, 32, 87, "Input",ExpressionUUID->"c4a5dd86-cdea-4873-8dc0-d922706225c0",
   CellTags->"b:3.1.0"],
  Cell[67041, 1560, 841, 25, 68, "Input",ExpressionUUID->"ce7152e7-628f-4c48-b960-d5298267cd7e",
   CellTags->"b:3.1.0"],
  Cell[67885, 1587, 653, 18, 68, "Input",ExpressionUUID->"759956d6-6ee7-44fe-bcc6-3e20155405fb",
   CellTags->"b:3.1.0"],
  Cell[68541, 1607, 2616, 67, 163, "Input",ExpressionUUID->"a2860cee-cde7-4626-9571-dde99c327565",
   CellTags->"b:3.1.0"],
  Cell[71160, 1676, 307, 7, 51, "Text",ExpressionUUID->"587831e8-2e6d-4f5b-b757-c8ee1cc11f0f",
   CellTags->"b:3.1.0"],
  Cell[71470, 1685, 462, 13, 65, "Input",ExpressionUUID->"7b386c71-97b5-4506-bbfb-5f46553f44f6",
   CellTags->"b:3.1.0"],
  Cell[71935, 1700, 167, 2, 23, "Text",ExpressionUUID->"1a8cc137-fbd0-4251-af22-8c20742ba62b",
   CellTags->"b:3.1.0"],
  Cell[72105, 1704, 170, 4, 48, "Input",ExpressionUUID->"25d86afd-2547-4bc4-b195-117c8216f2d6",
   CellTags->"b:3.1.0"]},
 "c:4"->{
  Cell[72312, 1713, 270, 7, 60, "Subsection",ExpressionUUID->"c8df16bb-759e-4dae-8fd6-5a5adba1a925",
   CellTags->{"c:4", "b:3.1.1"}]},
 "b:3.1.1"->{
  Cell[72312, 1713, 270, 7, 60, "Subsection",ExpressionUUID->"c8df16bb-759e-4dae-8fd6-5a5adba1a925",
   CellTags->{"c:4", "b:3.1.1"}],
  Cell[72585, 1722, 159, 2, 23, "Text",ExpressionUUID->"ac5ac293-60fa-4e41-a6a7-2a2754b4b4f4",
   CellTags->"b:3.1.1"],
  Cell[72747, 1726, 145, 3, 48, "Input",ExpressionUUID->"e8d6f5b9-86f7-4959-aec4-1db9f7599b91",
   CellTags->"b:3.1.1"],
  Cell[72895, 1731, 174, 2, 23, "Text",ExpressionUUID->"46b63a6c-9ce5-4e06-ba8b-9f7d176a0445",
   CellTags->"b:3.1.1"],
  Cell[73072, 1735, 145, 3, 48, "Input",ExpressionUUID->"7f83aa48-6ba0-4ff1-b9cc-498239cdba80",
   CellTags->"b:3.1.1"],
  Cell[73220, 1740, 247, 5, 23, "Text",ExpressionUUID->"b1a5a53a-6d02-49f9-9012-fbbc29a58a70",
   CellTags->"b:3.1.1"],
  Cell[73470, 1747, 382, 11, 48, "Input",ExpressionUUID->"49d10476-5555-4aa0-8ed9-45e1130ff675",
   CellTags->"b:3.1.1"],
  Cell[73855, 1760, 332, 8, 48, "Input",ExpressionUUID->"79e53433-c843-4cbd-b058-d683850990fc",
   CellTags->"b:3.1.1"],
  Cell[74190, 1770, 735, 19, 87, "Input",ExpressionUUID->"cd8301cc-cbbd-475f-9136-1efe9f1fd9f7",
   CellTags->"b:3.1.1"],
  Cell[74928, 1791, 1583, 44, 173, "Text",ExpressionUUID->"44059999-2373-4ce3-b6f4-56f37ed4449c",
   CellTags->"b:3.1.1"],
  Cell[76536, 1839, 270, 7, 48, "Input",ExpressionUUID->"c07d56b5-fa69-45a1-992e-30de76f52e72",
   CellTags->"b:3.1.1"],
  Cell[76809, 1848, 332, 7, 48, "Output",ExpressionUUID->"3ea2b04e-4659-4e7a-9860-b83ea910a039",
   CellTags->"b:3.1.1"],
  Cell[77178, 1860, 568, 16, 106, "Input",ExpressionUUID->"5244f1fd-21cc-4fd8-a168-14c9d5454f04",
   CellTags->"b:3.1.1"],
  Cell[77749, 1878, 334, 7, 48, "Output",ExpressionUUID->"102c44dd-e4f6-4291-af6f-37cf1941e5f8",
   CellTags->"b:3.1.1"],
  Cell[78086, 1887, 332, 7, 48, "Output",ExpressionUUID->"ab6390af-9238-4326-b067-94c297a31aea",
   CellTags->"b:3.1.1"],
  Cell[78433, 1897, 304, 7, 51, "Text",ExpressionUUID->"56644221-7526-4d00-85ee-243218b9ca87",
   CellTags->"b:3.1.1"],
  Cell[78762, 1908, 434, 12, 48, "Input",ExpressionUUID->"a0971beb-0a6d-481f-852b-b7812373b1f2",
   CellTags->"b:3.1.1"],
  Cell[79199, 1922, 371, 5, 24, "Print",ExpressionUUID->"8041f7cd-1c19-4263-a9db-e2c5b5e52bae",
   CellTags->"b:3.1.1"]},
 "c:5"->{
  Cell[79619, 1933, 468, 13, 62, "Subsection",ExpressionUUID->"9f64068a-1716-41c1-9380-6ebcdb05a054",
   CellTags->{"c:5", "b:3.1.2"}]},
 "b:3.1.2"->{
  Cell[79619, 1933, 468, 13, 62, "Subsection",ExpressionUUID->"9f64068a-1716-41c1-9380-6ebcdb05a054",
   CellTags->{"c:5", "b:3.1.2"}],
  Cell[80090, 1948, 1067, 31, 97, "Text",ExpressionUUID->"cb14f4d9-dac5-471b-9abf-cf40e32855ae",
   CellTags->"b:3.1.2"]},
 "c:6"->{
  Cell[81194, 1984, 584, 17, 65, "Subsection",ExpressionUUID->"686a6ce0-9161-4ca6-9cb2-35da78e8e4e4",
   CellTags->{"c:6", "b:3.1.3"}]},
 "b:3.1.3"->{
  Cell[81194, 1984, 584, 17, 65, "Subsection",ExpressionUUID->"686a6ce0-9161-4ca6-9cb2-35da78e8e4e4",
   CellTags->{"c:6", "b:3.1.3"}],
  Cell[81781, 2003, 154, 2, 23, "Text",ExpressionUUID->"4ed792cc-4499-4bdd-9595-a2f9c931b454",
   CellTags->"b:3.1.3"],
  Cell[81960, 2009, 554, 16, 48, "Input",ExpressionUUID->"76cc33e8-bcd3-4ff3-ad27-e07e7a8a1618",
   CellTags->"b:3.1.3"],
  Cell[82517, 2027, 333, 7, 48, "Output",ExpressionUUID->"b007c3ab-8b4a-420b-8861-0bcb7e80ed76",
   CellTags->"b:3.1.3"],
  Cell[82865, 2037, 1380, 42, 63, "Text",ExpressionUUID->"a65dbdc9-e5b3-406d-b575-4a8c6c6ac5e1",
   CellTags->"b:3.1.3"]},
 "c:7"->{
  Cell[84282, 2084, 206, 4, 60, "Subsection",ExpressionUUID->"f1b39d7d-e749-4586-9c88-b559e8a6831e",
   CellTags->{"c:7", "b:3.1.4"}]},
 "b:3.1.4"->{
  Cell[84282, 2084, 206, 4, 60, "Subsection",ExpressionUUID->"f1b39d7d-e749-4586-9c88-b559e8a6831e",
   CellTags->{"c:7", "b:3.1.4"}],
  Cell[84491, 2090, 400, 10, 55, "Text",ExpressionUUID->"2829ad68-1cbc-4d82-9aa0-9f58698439d8",
   CellTags->"b:3.1.4"],
  Cell[84916, 2104, 418, 12, 60, "Input",ExpressionUUID->"c22e121b-2b66-4028-bae9-0bdf25ef1fe3",
   CellTags->"b:3.1.4"],
  Cell[85337, 2118, 330, 7, 48, "Output",ExpressionUUID->"71e0158b-0328-4cbc-a541-18b72debbd67",
   CellTags->"b:3.1.4"],
  Cell[85704, 2130, 370, 10, 87, "Input",ExpressionUUID->"15079ef9-8449-4a35-97ea-10a9a1703247",
   CellTags->"b:3.1.4"],
  Cell[86077, 2142, 329, 7, 48, "Output",ExpressionUUID->"c453e302-e944-492e-95ea-b82213eb45bd",
   CellTags->"b:3.1.4"],
  Cell[86409, 2151, 327, 7, 48, "Output",ExpressionUUID->"8b4f24f5-6186-4b24-aa87-9d3850622244",
   CellTags->"b:3.1.4"],
  Cell[86739, 2160, 333, 7, 48, "Output",ExpressionUUID->"71b521e3-4c00-4f4b-91fc-29a58ad27f60",
   CellTags->"b:3.1.4"],
  Cell[87087, 2170, 596, 18, 25, "Text",ExpressionUUID->"060b2127-7e7c-4426-9aa7-9d061258bb9f",
   CellTags->"b:3.1.4"],
  Cell[87686, 2190, 501, 14, 48, "Input",ExpressionUUID->"f3f296c0-1e48-474b-a107-66c839177611",
   CellTags->"b:3.1.4"],
  Cell[88190, 2206, 400, 11, 25, "Text",ExpressionUUID->"abd55984-1aed-4308-8dee-dc9fe9e708b9",
   CellTags->"b:3.1.4"],
  Cell[88593, 2219, 1109, 34, 93, "Input",ExpressionUUID->"5601443d-d566-43af-ad75-45e13b6090fb",
   CellTags->"b:3.1.4"],
  Cell[89705, 2255, 189, 4, 23, "Text",ExpressionUUID->"4f931af5-cc57-41d9-9964-9a93cc1c44df",
   CellTags->"b:3.1.4"],
  Cell[89897, 2261, 819, 25, 87, "Input",ExpressionUUID->"75aaafbf-5124-4626-bdfc-6089b808f357",
   CellTags->"b:3.1.4"],
  Cell[90719, 2288, 278, 8, 48, "Input",ExpressionUUID->"4eeef5dc-3a2b-45c0-842b-bf2a695be73c",
   CellTags->"b:3.1.4"],
  Cell[91000, 2298, 138, 2, 23, "Text",ExpressionUUID->"290722f7-0475-4a29-84d9-5ad1c7ebdec0",
   CellTags->"b:3.1.4"],
  Cell[91141, 2302, 153, 3, 48, "Input",ExpressionUUID->"aa14095f-36c7-4f3c-afc5-bd352551d47e",
   CellTags->"b:3.1.4"],
  Cell[91297, 2307, 150, 2, 23, "Text",ExpressionUUID->"aa262bf7-c9d3-430f-9d48-add5e451f579",
   CellTags->"b:3.1.4"],
  Cell[91450, 2311, 708, 20, 48, "Input",ExpressionUUID->"827d4594-f283-42f4-a122-1f0f6f5c3d93",
   CellTags->"b:3.1.4"],
  Cell[92161, 2333, 652, 19, 71, "Input",ExpressionUUID->"16c2cfe1-4ab7-4500-a0b5-963e0addaaf7",
   CellTags->"b:3.1.4"],
  Cell[92816, 2354, 153, 3, 48, "Input",ExpressionUUID->"e3556bdb-cf40-4ad9-9be6-0a4e749eeb90",
   CellTags->"b:3.1.4"]},
 "c:8"->{
  Cell[93006, 2362, 200, 4, 60, "Subsection",ExpressionUUID->"b52dae01-44a5-4675-9e09-dd23c9a6099a",
   CellTags->{"c:8", "b:3.1.5"}]},
 "b:3.1.5"->{
  Cell[93006, 2362, 200, 4, 60, "Subsection",ExpressionUUID->"b52dae01-44a5-4675-9e09-dd23c9a6099a",
   CellTags->{"c:8", "b:3.1.5"}],
  Cell[93209, 2368, 186, 3, 23, "Text",ExpressionUUID->"a3937fed-1b25-41f5-a01a-6dda38b9b91d",
   CellTags->"b:3.1.5"],
  Cell[93398, 2373, 532, 15, 48, "Input",ExpressionUUID->"c46732cf-92e0-4ba9-9fcf-eefcc10bcace",
   CellTags->"b:3.1.5"],
  Cell[93933, 2390, 182, 4, 48, "Input",ExpressionUUID->"b3e06932-15c6-494a-89e3-a0203cb336ac",
   CellTags->"b:3.1.5"],
  Cell[94118, 2396, 493, 13, 51, "Input",ExpressionUUID->"bbf072e9-dbcf-4f7b-9828-19ad0cbe65c8",
   CellTags->"b:3.1.5"],
  Cell[94636, 2413, 346, 8, 68, "Input",ExpressionUUID->"a4a9565e-a784-45be-b020-258073baaebf",
   CellTags->"b:3.1.5"],
  Cell[94985, 2423, 510, 9, 24, "Print",ExpressionUUID->"316b04d2-209e-4891-ae27-e4195f7eebd7",
   CellTags->"b:3.1.5"],
  Cell[95532, 2437, 169, 3, 48, "Input",ExpressionUUID->"212087b0-9807-4723-ba08-85686d31b9d7",
   CellTags->"b:3.1.5"],
  Cell[95704, 2442, 348, 7, 48, "Output",ExpressionUUID->"ad13fe6c-d084-4a9a-9a43-d8713161e7fe",
   CellTags->"b:3.1.5"]},
 "c:9"->{
  Cell[96101, 2455, 247, 7, 60, "Subsection",ExpressionUUID->"289e9253-934d-4688-944c-597dffdb479f",
   CellTags->{"c:9", "b:3.1.6"}]},
 "b:3.1.6"->{
  Cell[96101, 2455, 247, 7, 60, "Subsection",ExpressionUUID->"289e9253-934d-4688-944c-597dffdb479f",
   CellTags->{"c:9", "b:3.1.6"}],
  Cell[96351, 2464, 209, 5, 23, "Text",ExpressionUUID->"9157ff98-06d6-4397-9451-fefd9911320a",
   CellTags->"b:3.1.6"],
  Cell[96563, 2471, 175, 4, 48, "Input",ExpressionUUID->"9dd58b55-04a6-4b7a-9081-e0cd144b2197",
   CellTags->"b:3.1.6"],
  Cell[96741, 2477, 733, 21, 51, "Input",ExpressionUUID->"a9453a43-185a-476c-a22a-0af6ebe086fb",
   CellTags->"b:3.1.6"],
  Cell[97477, 2500, 211, 5, 23, "Text",ExpressionUUID->"f536a44b-7cde-488d-8153-a60b2a606cf8",
   CellTags->"b:3.1.6"],
  Cell[97713, 2509, 858, 22, 87, "Input",ExpressionUUID->"b3d96486-217f-4f3c-9c15-ddca88a729b6",
   CellTags->"b:3.1.6"],
  Cell[98574, 2533, 4007, 80, 227, "Output",ExpressionUUID->"84d70b42-f76e-423a-9896-5db6403c5ba9",
   CellTags->"b:3.1.6"],
  Cell[102584, 2615, 2112, 52, 230, "Output",ExpressionUUID->"e1b1e22f-7e81-4840-9e09-2d08b84e06c4",
   CellTags->"b:3.1.6"]},
 "c:10"->{
  Cell[104745, 2673, 227, 6, 60, "Subsection",ExpressionUUID->"a28b1b85-2a1c-4bf8-a47f-2f7e35433a20",
   CellTags->{"c:10", "b:3.1.7"}]},
 "b:3.1.7"->{
  Cell[104745, 2673, 227, 6, 60, "Subsection",ExpressionUUID->"a28b1b85-2a1c-4bf8-a47f-2f7e35433a20",
   CellTags->{"c:10", "b:3.1.7"}],
  Cell[104975, 2681, 625, 13, 136, "Text",ExpressionUUID->"f7c184aa-833a-4e13-941d-b0b1c1dbe8e1",
   CellTags->"b:3.1.7"],
  Cell[105603, 2696, 144, 3, 48, "Input",ExpressionUUID->"41a74ad2-09df-4a4b-bda4-53bc22a0aaba",
   CellTags->"b:3.1.7"],
  Cell[105750, 2701, 4274, 115, 288, "Input",ExpressionUUID->"09968d3f-b28f-4049-810d-5da4c38e3db0",
   CellTags->"b:3.1.7"],
  Cell[110049, 2820, 1071, 27, 144, "Input",ExpressionUUID->"ee37e1b2-be7a-46db-ab35-9931b666d5a6",
   CellTags->"b:3.1.7"],
  Cell[111123, 2849, 816, 19, 64, "Output",ExpressionUUID->"7ed0d11e-2edb-4fd6-b4a2-31a1d8b854de",
   CellTags->"b:3.1.7"],
  Cell[111942, 2870, 394, 8, 24, "Print",ExpressionUUID->"e9c6c664-481c-44aa-a3b0-1349371fa0f6",
   CellTags->"b:3.1.7"],
  Cell[112339, 2880, 691, 14, 24, "Print",ExpressionUUID->"16ff7102-3cb5-45d0-9e1d-61136997fb88",
   CellTags->"b:3.1.7"],
  Cell[113033, 2896, 339, 5, 24, "Print",ExpressionUUID->"6c7d3dcf-ab09-45d7-ae46-52dbb7301474",
   CellTags->"b:3.1.7"],
  Cell[113409, 2906, 136, 2, 48, "Input",ExpressionUUID->"6423670b-1dcd-4cd3-9d9d-4446fb731ced",
   CellTags->"b:3.1.7"],
  Cell[113548, 2910, 357, 7, 48, "Output",ExpressionUUID->"66aa6b4f-a1e5-4b71-a53d-4495d48bc65f",
   CellTags->"b:3.1.7"],
  Cell[113920, 2920, 319, 7, 79, "Text",ExpressionUUID->"9c2ff836-91fa-4545-8eaa-5ebfacc6f941",
   CellTags->"b:3.1.7"],
  Cell[114264, 2931, 468, 13, 75, "Input",ExpressionUUID->"c4bf62c2-3d0b-4462-b554-73d07a5767ce",
   CellTags->"b:3.1.7"],
  Cell[114735, 2946, 1753, 46, 264, "Output",ExpressionUUID->"25fcd2c7-95c8-4f6f-967f-004e3b149d70",
   CellTags->"b:3.1.7"],
  Cell[116503, 2995, 98, 1, 48, "Input",ExpressionUUID->"2cfdf17c-21c1-452f-b9e1-fc9ee72b3423",
   CellTags->"b:3.1.7"]},
 "c:11"->{
  Cell[116650, 3002, 184, 4, 51, "Section",ExpressionUUID->"24d13847-d1ae-4f2b-bce4-15298cc8b739",
   CellTags->"c:11"]},
 "c:12"->{
  Cell[116859, 3010, 238, 6, 60, "Subsection",ExpressionUUID->"aac51844-15b4-485e-86df-82617cf00316",
   CellTags->{"c:12", "b:3.2.0"}]},
 "b:3.2.0"->{
  Cell[116859, 3010, 238, 6, 60, "Subsection",ExpressionUUID->"aac51844-15b4-485e-86df-82617cf00316",
   CellTags->{"c:12", "b:3.2.0"}],
  Cell[117122, 3020, 188, 3, 46, "Input",ExpressionUUID->"7ed690ea-76a8-4af3-a937-3d3353901106",
   CellTags->"b:3.2.0"],
  Cell[117313, 3025, 228, 3, 24, "Print",ExpressionUUID->"d8f96f63-fb5e-4053-a48d-87e2a44f804e",
   CellTags->"b:3.2.0"],
  Cell[117578, 3033, 167, 3, 46, "Input",ExpressionUUID->"809f814a-6ec4-42c1-9af2-c6571e4692f0",
   CellTags->"b:3.2.0"],
  Cell[117748, 3038, 55968, 1259, 493, "Output",ExpressionUUID->"aa0e8521-6ff2-4dd6-9990-d558bb3b574b",
   CellTags->"b:3.2.0"],
  Cell[173731, 4300, 221, 5, 23, "Text",ExpressionUUID->"43ec1d98-66a3-43cf-8fe2-cc272e5d61f6",
   CellTags->"b:3.2.0"],
  Cell[173955, 4307, 1105, 34, 68, "Input",ExpressionUUID->"f0036fac-fccb-4bcb-85ee-d4ad0d046f54",
   CellTags->"b:3.2.0"],
  Cell[175063, 4343, 2369, 59, 163, "Input",ExpressionUUID->"1e23594c-fde4-464e-ab54-8fd19efab08d",
   CellTags->"b:3.2.0"],
  Cell[177435, 4404, 575, 14, 68, "Input",ExpressionUUID->"e8755a09-1536-4b1d-b029-ea5c5e6325e2",
   CellTags->"b:3.2.0"],
  Cell[178013, 4420, 2329, 60, 163, "Input",ExpressionUUID->"e9c23b01-eff7-4f7c-84fd-b9205f03704b",
   CellTags->"b:3.2.0"],
  Cell[180345, 4482, 576, 14, 68, "Input",ExpressionUUID->"d0cdb5e1-88f8-4b06-a0b3-46073347374d",
   CellTags->"b:3.2.0"],
  Cell[180924, 4498, 307, 7, 51, "Text",ExpressionUUID->"42ca929b-57a0-444b-bfc3-60fbee7b1ad3",
   CellTags->"b:3.2.0"],
  Cell[181234, 4507, 462, 13, 65, "Input",ExpressionUUID->"e0c55fff-f275-4acb-8fbc-b33125148bf7",
   CellTags->"b:3.2.0"],
  Cell[181699, 4522, 167, 2, 23, "Text",ExpressionUUID->"796b1850-1712-4faa-acf4-f31ca425d71e",
   CellTags->"b:3.2.0"],
  Cell[181869, 4526, 170, 4, 48, "Input",ExpressionUUID->"188c8c18-4b35-4253-8788-146aa2bd355f",
   CellTags->"b:3.2.0"]},
 "c:13"->{
  Cell[182076, 4535, 264, 7, 60, "Subsection",ExpressionUUID->"ee450305-874f-4775-a17d-d467161ffbb5",
   CellTags->{"c:13", "b:3.2.1"}]},
 "b:3.2.1"->{
  Cell[182076, 4535, 264, 7, 60, "Subsection",ExpressionUUID->"ee450305-874f-4775-a17d-d467161ffbb5",
   CellTags->{"c:13", "b:3.2.1"}],
  Cell[182343, 4544, 159, 2, 23, "Text",ExpressionUUID->"1277f93f-4321-4bda-8ce6-e066ab62c620",
   CellTags->"b:3.2.1"],
  Cell[182527, 4550, 151, 3, 48, "Input",ExpressionUUID->"0064d9bc-9c92-4916-a8f3-97442721d0a3",
   CellTags->"b:3.2.1"],
  Cell[182681, 4555, 213, 5, 48, "Output",ExpressionUUID->"b3e68a6c-f29f-4843-b05f-c7208a726d0b",
   CellTags->"b:3.2.1"],
  Cell[182909, 4563, 174, 2, 23, "Text",ExpressionUUID->"70df8bf4-3406-4418-8c1f-c07ada681540",
   CellTags->"b:3.2.1"],
  Cell[183086, 4567, 145, 3, 48, "Input",ExpressionUUID->"39a392c8-853f-4f12-96c8-5d643445308d",
   CellTags->"b:3.2.1"],
  Cell[183234, 4572, 230, 5, 23, "Text",ExpressionUUID->"cbb2b275-bd49-4ea6-9453-af1d54658d39",
   CellTags->"b:3.2.1"],
  Cell[183467, 4579, 382, 11, 48, "Input",ExpressionUUID->"cf1f548b-177a-4cd7-9f1b-a78601360609",
   CellTags->"b:3.2.1"],
  Cell[183852, 4592, 332, 8, 48, "Input",ExpressionUUID->"5b676943-2e71-488f-8155-e16155f0ad7f",
   CellTags->"b:3.2.1"],
  Cell[184187, 4602, 735, 19, 87, "Input",ExpressionUUID->"f18367b0-4334-422b-bb5d-d5fa993ea94b",
   CellTags->"b:3.2.1"],
  Cell[184925, 4623, 1669, 48, 173, "Text",ExpressionUUID->"05789577-c6ec-4b56-ade0-c770b29a92df",
   CellTags->"b:3.2.1"],
  Cell[186619, 4675, 270, 7, 48, "Input",ExpressionUUID->"d6fb0edd-4b20-4244-a877-3dbf510cd72c",
   CellTags->"b:3.2.1"],
  Cell[186892, 4684, 213, 5, 48, "Output",ExpressionUUID->"daaa0c1e-6d1e-4492-bcf5-e20df6da9bcd",
   CellTags->"b:3.2.1"],
  Cell[187120, 4692, 397, 10, 25, "Text",ExpressionUUID->"335e8ea3-2873-44d9-b1a4-7a5b8c264732",
   CellTags->"b:3.2.1"],
  Cell[187542, 4706, 756, 20, 125, "Input",ExpressionUUID->"f1cc2945-4e2e-4e68-9bf6-eb7aaa7e8756",
   CellTags->"b:3.2.1"],
  Cell[188301, 4728, 209, 5, 48, "Output",ExpressionUUID->"33425887-852b-44b0-95cd-4f7a4942c583",
   CellTags->"b:3.2.1"],
  Cell[188513, 4735, 211, 5, 48, "Output",ExpressionUUID->"53aa7c81-2544-49c3-8218-82071356b912",
   CellTags->"b:3.2.1"],
  Cell[188727, 4742, 209, 5, 48, "Output",ExpressionUUID->"10011bf6-248a-4a3c-a25d-36a6fa332b23",
   CellTags->"b:3.2.1"],
  Cell[188951, 4750, 304, 7, 51, "Text",ExpressionUUID->"eb6b992e-8347-460c-b83d-6b917f3d28f6",
   CellTags->"b:3.2.1"],
  Cell[189280, 4761, 434, 12, 48, "Input",ExpressionUUID->"edab14a3-a909-4841-bbfb-7663f5a5a735",
   CellTags->"b:3.2.1"],
  Cell[189717, 4775, 252, 3, 24, "Print",ExpressionUUID->"42bc22b7-3e0a-46ed-b274-a20c434bec81",
   CellTags->"b:3.2.1"]},
 "c:14"->{
  Cell[190018, 4784, 469, 13, 62, "Subsection",ExpressionUUID->"b21b802a-e344-4045-9440-8754a9b40edf",
   CellTags->{"c:14", "b:3.2.2"}]},
 "b:3.2.2"->{
  Cell[190018, 4784, 469, 13, 62, "Subsection",ExpressionUUID->"b21b802a-e344-4045-9440-8754a9b40edf",
   CellTags->{"c:14", "b:3.2.2"}],
  Cell[190490, 4799, 1067, 31, 97, "Text",ExpressionUUID->"2f7d1a0f-0728-490d-a43d-fdb912c228a9",
   CellTags->"b:3.2.2"]},
 "c:15"->{
  Cell[191594, 4835, 585, 17, 65, "Subsection",ExpressionUUID->"7eaa4595-dff0-4dc2-be10-dab949968968",
   CellTags->{"c:15", "b:3.2.3"}]},
 "b:3.2.3"->{
  Cell[191594, 4835, 585, 17, 65, "Subsection",ExpressionUUID->"7eaa4595-dff0-4dc2-be10-dab949968968",
   CellTags->{"c:15", "b:3.2.3"}],
  Cell[192182, 4854, 154, 2, 23, "Text",ExpressionUUID->"553878d0-20c0-4d41-a2f1-b451599dbdfb",
   CellTags->"b:3.2.3"],
  Cell[192361, 4860, 554, 16, 48, "Input",ExpressionUUID->"9f0e4179-0a52-4597-99dc-bf4b814f9208",
   CellTags->"b:3.2.3"],
  Cell[192918, 4878, 213, 5, 48, "Output",ExpressionUUID->"5921de34-eda2-4ae7-b705-17e1fc854e6f",
   CellTags->"b:3.2.3"],
  Cell[193168, 4888, 132, 2, 48, "Input",ExpressionUUID->"717d66cb-f527-488b-8d26-7ba1be80c1a1",
   CellTags->"b:3.2.3"],
  Cell[193303, 4892, 210, 5, 48, "Output",ExpressionUUID->"483f3aac-fd15-48b2-8547-d32b192396fe",
   CellTags->"b:3.2.3"],
  Cell[193528, 4900, 1380, 42, 63, "Text",ExpressionUUID->"1b942e64-f0ef-4a8a-b57e-c8ff4e8265fc",
   CellTags->"b:3.2.3"]},
 "c:16"->{
  Cell[194945, 4947, 207, 4, 60, "Subsection",ExpressionUUID->"12cde5ef-8089-4818-ad4d-98f516b5bfa3",
   CellTags->{"c:16", "b:3.2.4"}]},
 "b:3.2.4"->{
  Cell[194945, 4947, 207, 4, 60, "Subsection",ExpressionUUID->"12cde5ef-8089-4818-ad4d-98f516b5bfa3",
   CellTags->{"c:16", "b:3.2.4"}],
  Cell[195155, 4953, 468, 11, 55, "Text",ExpressionUUID->"2974a720-8c21-403c-9df2-f379891ace8a",
   CellTags->"b:3.2.4"],
  Cell[195648, 4968, 418, 12, 60, "Input",ExpressionUUID->"38b54cd7-ffbe-4061-b025-2315bca1f3f5",
   CellTags->"b:3.2.4"],
  Cell[196069, 4982, 212, 5, 48, "Output",ExpressionUUID->"7fa3010c-5136-438c-8ee2-6b289c0268b7",
   CellTags->"b:3.2.4"],
  Cell[196318, 4992, 370, 10, 87, "Input",ExpressionUUID->"b384365a-429e-4db7-825b-83d26a561649",
   CellTags->"b:3.2.4"],
  Cell[196691, 5004, 211, 5, 48, "Output",ExpressionUUID->"e5b32156-a616-4788-b07a-3c5df7baf38b",
   CellTags->"b:3.2.4"],
  Cell[196905, 5011, 211, 5, 48, "Output",ExpressionUUID->"934d2906-c1b7-4072-b879-4745c9fbd08e",
   CellTags->"b:3.2.4"],
  Cell[197119, 5018, 214, 5, 48, "Output",ExpressionUUID->"1fa36f6c-3f15-4d8b-b9a1-4f94a3205cc0",
   CellTags->"b:3.2.4"],
  Cell[197348, 5026, 596, 18, 25, "Text",ExpressionUUID->"0d71df29-876e-416b-b38a-362fc8bc158d",
   CellTags->"b:3.2.4"],
  Cell[197947, 5046, 501, 14, 48, "Input",ExpressionUUID->"25559b2e-086c-4304-a916-0ce52db6f5a9",
   CellTags->"b:3.2.4"],
  Cell[198451, 5062, 400, 11, 25, "Text",ExpressionUUID->"be63e8d6-0bcf-4559-b4c6-3e40e92a167d",
   CellTags->"b:3.2.4"],
  Cell[198854, 5075, 1109, 34, 93, "Input",ExpressionUUID->"ae252cbc-516d-45a3-b589-0385ca4ed2f9",
   CellTags->"b:3.2.4"],
  Cell[199966, 5111, 189, 4, 23, "Text",ExpressionUUID->"a2ce1843-6c3d-49ba-abc2-e32f635779a9",
   CellTags->"b:3.2.4"],
  Cell[200158, 5117, 819, 25, 87, "Input",ExpressionUUID->"d1dc27a2-c27a-43eb-bd8f-e8db44fe18ca",
   CellTags->"b:3.2.4"],
  Cell[200980, 5144, 136, 2, 23, "Text",ExpressionUUID->"a169e8d9-6724-4f5d-8a01-041fb60846a9",
   CellTags->"b:3.2.4"],
  Cell[201119, 5148, 153, 3, 48, "Input",ExpressionUUID->"79576aff-ff3a-4eba-986f-d0a506d1b9f0",
   CellTags->"b:3.2.4"]},
 "c:17"->{
  Cell[201309, 5156, 201, 4, 60, "Subsection",ExpressionUUID->"e2b44cc5-4a59-474f-99a5-766e9b3e0cce",
   CellTags->{"c:17", "b:3.2.5"}]},
 "b:3.2.5"->{
  Cell[201309, 5156, 201, 4, 60, "Subsection",ExpressionUUID->"e2b44cc5-4a59-474f-99a5-766e9b3e0cce",
   CellTags->{"c:17", "b:3.2.5"}],
  Cell[201513, 5162, 215, 5, 23, "Text",ExpressionUUID->"26d2857a-d9db-4e3d-b0bf-ad46b54832fb",
   CellTags->"b:3.2.5"],
  Cell[201731, 5169, 182, 4, 48, "Input",ExpressionUUID->"0ab15e2f-cff2-41c1-81b4-edb1378861e1",
   CellTags->"b:3.2.5"],
  Cell[265947, 6785, 968, 26, 87, "Input",ExpressionUUID->"71a9281e-1a08-42a4-bf13-5cd629a8f8d3",
   CellTags->"b:3.2.5"],
  Cell[266918, 6813, 413, 9, 24, "Print",ExpressionUUID->"1ffdbfb5-8eb0-4056-9e08-907f495b602e",
   CellTags->"b:3.2.5"],
  Cell[267368, 6827, 346, 8, 68, "Input",ExpressionUUID->"c36e697b-51e7-4fdf-93f7-98e1bdc0b2b5",
   CellTags->"b:3.2.5"],
  Cell[267717, 6837, 387, 7, 24, "Print",ExpressionUUID->"3f97b5cc-cc36-4018-a63f-c610d5f1532b",
   CellTags->"b:3.2.5"]},
 "c:18"->{
  Cell[268153, 6850, 248, 7, 60, "Subsection",ExpressionUUID->"ddd77bd5-ee92-4f00-bc30-a05161b1d4cb",
   CellTags->{"c:18", "b:3.2.6"}]},
 "b:3.2.6"->{
  Cell[268153, 6850, 248, 7, 60, "Subsection",ExpressionUUID->"ddd77bd5-ee92-4f00-bc30-a05161b1d4cb",
   CellTags->{"c:18", "b:3.2.6"}],
  Cell[268404, 6859, 209, 5, 23, "Text",ExpressionUUID->"80d9d151-4ffe-418a-ab5a-48ace74cac97",
   CellTags->"b:3.2.6"],
  Cell[268616, 6866, 175, 4, 48, "Input",ExpressionUUID->"691b62c1-45d8-4356-94ca-fcca56f24d50",
   CellTags->"b:3.2.6"],
  Cell[268794, 6872, 805, 23, 51, "Input",ExpressionUUID->"b23662eb-3dc3-4197-b052-917149eb8671",
   CellTags->"b:3.2.6"],
  Cell[269602, 6897, 211, 5, 23, "Text",ExpressionUUID->"d8c92dbc-2b52-4dfb-a9f8-044e215b0c4e",
   CellTags->"b:3.2.6"],
  Cell[269838, 6906, 858, 22, 87, "Input",ExpressionUUID->"13e27a7f-7b84-4a33-8e0b-09cd7ccfd29c",
   CellTags->"b:3.2.6"],
  Cell[270699, 6930, 8310, 159, 219, "Output",ExpressionUUID->"1dbd491e-690a-4984-87a5-8ede684b0365",
   CellTags->"b:3.2.6"],
  Cell[279012, 7091, 6112, 118, 230, "Output",ExpressionUUID->"a042b52f-4d3f-4228-b66a-b43061007647",
   CellTags->"b:3.2.6"]},
 "c:19"->{
  Cell[285173, 7215, 227, 6, 60, "Subsection",ExpressionUUID->"e624a1be-c636-4105-bfe5-d1a2feea774f",
   CellTags->{"c:19", "b:3.2.7"}]},
 "b:3.2.7"->{
  Cell[285173, 7215, 227, 6, 60, "Subsection",ExpressionUUID->"e624a1be-c636-4105-bfe5-d1a2feea774f",
   CellTags->{"c:19", "b:3.2.7"}],
  Cell[285403, 7223, 625, 13, 136, "Text",ExpressionUUID->"53f4d29d-2f8a-425c-a2e4-60f1e617e6b1",
   CellTags->"b:3.2.7"],
  Cell[286031, 7238, 144, 3, 48, "Input",ExpressionUUID->"a58fd27c-b350-4aef-9026-c515f1fe64ec",
   CellTags->"b:3.2.7"],
  Cell[286178, 7243, 4274, 115, 288, "Input",ExpressionUUID->"694abdfc-0b09-4f1b-b2af-ba0ccca3454a",
   CellTags->"b:3.2.7"],
  Cell[290477, 7362, 822, 20, 125, "Input",ExpressionUUID->"9ea8d0bf-0b54-4f09-a85b-4ca0f0de669a",
   CellTags->"b:3.2.7"],
  Cell[291302, 7384, 917, 21, 132, "Output",ExpressionUUID->"1673fbea-2472-4523-98d9-f2d97d3677df",
   CellTags->"b:3.2.7"],
  Cell[292222, 7407, 273, 6, 24, "Print",ExpressionUUID->"ae84e901-6925-46ed-877b-a7bcfd16ce21",
   CellTags->"b:3.2.7"],
  Cell[292498, 7415, 605, 12, 24, "Print",ExpressionUUID->"bf7df94e-6b48-4877-8581-5469ee4f047b",
   CellTags->"b:3.2.7"],
  Cell[293140, 7432, 136, 2, 48, "Input",ExpressionUUID->"ded7f919-5894-4236-a13d-f453620e6bf4",
   CellTags->"b:3.2.7"],
  Cell[293279, 7436, 235, 5, 48, "Output",ExpressionUUID->"90a47588-f8ca-4a79-8719-d5a3de8b4d32",
   CellTags->"b:3.2.7"],
  Cell[293529, 7444, 319, 7, 79, "Text",ExpressionUUID->"7111c831-3093-4a08-84dc-432a63ded561",
   CellTags->"b:3.2.7"],
  Cell[293873, 7455, 468, 13, 75, "Input",ExpressionUUID->"b2ecdeee-b04b-4f5c-9c9c-2a84fd795290",
   CellTags->"b:3.2.7"],
  Cell[294344, 7470, 1637, 44, 259, "Output",ExpressionUUID->"d3e7fe35-788f-4b5c-bc2c-d420e1e22685",
   CellTags->"b:3.2.7"],
  Cell[295996, 7517, 98, 1, 48, "Input",ExpressionUUID->"af3eb7f0-c3c0-4018-8dc7-edf49c603f86",
   CellTags->"b:3.2.7"]},
 "c:20"->{
  Cell[296143, 7524, 260, 6, 31, "Section",ExpressionUUID->"342b6cff-8d04-4829-b39a-30888a1b6349",
   CellTags->"c:20"]},
 "c:21"->{
  Cell[296428, 7534, 330, 10, 60, "Subsection",ExpressionUUID->"4fe22e9c-e4f8-49a9-9be9-52d1f03d8739",
   CellTags->{"c:21", "b:3.3.0"}]},
 "b:3.3.0"->{
  Cell[296428, 7534, 330, 10, 60, "Subsection",ExpressionUUID->"4fe22e9c-e4f8-49a9-9be9-52d1f03d8739",
   CellTags->{"c:21", "b:3.3.0"}],
  Cell[296783, 7548, 188, 3, 46, "Input",ExpressionUUID->"c2fd9143-1605-4192-a843-f52d21f90686",
   CellTags->"b:3.3.0"],
  Cell[296974, 7553, 228, 3, 24, "Print",ExpressionUUID->"a817e1e6-cf02-46c6-b909-79652efa790c",
   CellTags->"b:3.3.0"],
  Cell[297239, 7561, 167, 3, 46, "Input",ExpressionUUID->"c4e2ef04-0253-4a51-890d-7a0b90f5cc16",
   CellTags->"b:3.3.0"],
  Cell[297409, 7566, 55970, 1259, 493, "Output",ExpressionUUID->"0cd6f6d9-8b9d-4c1f-b5ff-edc1b03ea79c",
   CellTags->"b:3.3.0"],
  Cell[353394, 8828, 194, 4, 23, "Text",ExpressionUUID->"bc0cd033-3aaa-4057-8d66-7d53c8450df9",
   CellTags->"b:3.3.0"],
  Cell[353591, 8834, 1105, 34, 68, "Input",ExpressionUUID->"7e6edb2f-271f-44d6-8557-6e8ef8678a3d",
   CellTags->"b:3.3.0"],
  Cell[354699, 8870, 2388, 59, 163, "Input",ExpressionUUID->"171c8b0d-1e2c-45d2-b4af-3d9680453c2a",
   CellTags->"b:3.3.0"],
  Cell[357090, 8931, 575, 14, 68, "Input",ExpressionUUID->"198be410-ca96-4b49-9f66-44022697997d",
   CellTags->"b:3.3.0"],
  Cell[357668, 8947, 2368, 60, 163, "Input",ExpressionUUID->"e76c7614-569c-4eb2-a92a-d20b6496be74",
   CellTags->"b:3.3.0"],
  Cell[360039, 9009, 648, 15, 68, "Input",ExpressionUUID->"077d92a3-f006-465d-880e-8cdd676fecc6",
   CellTags->"b:3.3.0"],
  Cell[360690, 9026, 315, 6, 51, "Text",ExpressionUUID->"7bb36b78-356c-40cc-9073-18c80b475843",
   CellTags->"b:3.3.0"],
  Cell[361008, 9034, 462, 13, 65, "Input",ExpressionUUID->"c6f014f8-9677-4c13-a45c-3600fba54ab1",
   CellTags->"b:3.3.0"],
  Cell[361473, 9049, 167, 2, 23, "Text",ExpressionUUID->"377ac93b-27c8-4407-acee-7c8901cf8efc",
   CellTags->"b:3.3.0"],
  Cell[361643, 9053, 170, 4, 48, "Input",ExpressionUUID->"92d20556-4222-4574-9912-38a3d8b0565d",
   CellTags->"b:3.3.0"]},
 "c:22"->{
  Cell[361850, 9062, 272, 7, 60, "Subsection",ExpressionUUID->"75a1a666-9231-472e-a60a-556d78499aec",
   CellTags->{"c:22", "b:3.3.1"}]},
 "b:3.3.1"->{
  Cell[361850, 9062, 272, 7, 60, "Subsection",ExpressionUUID->"75a1a666-9231-472e-a60a-556d78499aec",
   CellTags->{"c:22", "b:3.3.1"}],
  Cell[362125, 9071, 162, 2, 23, "Text",ExpressionUUID->"bc6c8017-a8d6-4071-9b96-76a0845145b6",
   CellTags->"b:3.3.1"],
  Cell[362290, 9075, 145, 3, 48, "Input",ExpressionUUID->"3eaa8960-b60b-43d3-86bd-7dec3d67f698",
   CellTags->"b:3.3.1"],
  Cell[362438, 9080, 174, 2, 23, "Text",ExpressionUUID->"c51200c3-cc31-49ff-ad87-d83cd98b455f",
   CellTags->"b:3.3.1"],
  Cell[362615, 9084, 145, 3, 48, "Input",ExpressionUUID->"0f0b1380-8ed9-403f-acdb-c98014cbcfc5",
   CellTags->"b:3.3.1"],
  Cell[362763, 9089, 230, 5, 23, "Text",ExpressionUUID->"5addb373-3c99-403a-8fd4-0e6acf1858ed",
   CellTags->"b:3.3.1"],
  Cell[362996, 9096, 356, 10, 48, "Input",ExpressionUUID->"a3de0cd8-1787-477a-9d4b-65eeba4e97da",
   CellTags->"b:3.3.1"],
  Cell[363355, 9108, 735, 19, 87, "Input",ExpressionUUID->"e91a2e55-c340-401c-8bf3-a601d91bf15a",
   CellTags->"b:3.3.1"],
  Cell[364115, 9131, 270, 7, 48, "Input",ExpressionUUID->"dfd7d6f3-7933-4af1-89eb-d12597718341",
   CellTags->"b:3.3.1"],
  Cell[364388, 9140, 211, 5, 48, "Output",ExpressionUUID->"68bbec5e-2cb3-403b-a17a-9052f8e96bde",
   CellTags->"b:3.3.1"],
  Cell[364614, 9148, 832, 23, 111, "Text",ExpressionUUID->"ac05b555-7280-4b0d-9072-708fb8846e53",
   CellTags->"b:3.3.1"],
  Cell[365471, 9175, 568, 16, 106, "Input",ExpressionUUID->"3938e008-044f-49f9-830e-0bb45b68835d",
   CellTags->"b:3.3.1"],
  Cell[366042, 9193, 209, 5, 48, "Output",ExpressionUUID->"67804c97-742d-49b3-82f1-c162318ab594",
   CellTags->"b:3.3.1"],
  Cell[366254, 9200, 209, 5, 48, "Output",ExpressionUUID->"292b2530-9e5b-40df-b3d6-ca8506917915",
   CellTags->"b:3.3.1"],
  Cell[366478, 9208, 303, 7, 51, "Text",ExpressionUUID->"9ed2b209-50f8-433a-84ff-2ee340dac26a",
   CellTags->"b:3.3.1"],
  Cell[366784, 9217, 434, 12, 48, "Input",ExpressionUUID->"428e9b2f-f116-4abc-ac21-5cb707575033",
   CellTags->"b:3.3.1"]},
 "c:23"->{
  Cell[367255, 9234, 469, 13, 62, "Subsection",ExpressionUUID->"0e43a654-d0e3-4044-b2f0-dbd29c60d5a1",
   CellTags->{"c:23", "b:3.3.2"}]},
 "b:3.3.2"->{
  Cell[367255, 9234, 469, 13, 62, "Subsection",ExpressionUUID->"0e43a654-d0e3-4044-b2f0-dbd29c60d5a1",
   CellTags->{"c:23", "b:3.3.2"}],
  Cell[367727, 9249, 861, 25, 67, "Text",ExpressionUUID->"ce708b63-423b-4bde-9355-ca1ae75e8830",
   CellTags->"b:3.3.2"],
  Cell[368613, 9278, 558, 18, 68, "Input",ExpressionUUID->"542ec1d5-c543-4d75-b997-1d0742a06253",
   CellTags->"b:3.3.2"],
  Cell[369174, 9298, 209, 5, 48, "Output",ExpressionUUID->"5f4d811b-0190-41e1-9487-f58a720d4c51",
   CellTags->"b:3.3.2"]},
 "c:24"->{
  Cell[369432, 9309, 585, 17, 65, "Subsection",ExpressionUUID->"5a1b48ea-7940-4a0c-93ea-25e918710549",
   CellTags->{"c:24", "b:3.3.3"}]},
 "b:3.3.3"->{
  Cell[369432, 9309, 585, 17, 65, "Subsection",ExpressionUUID->"5a1b48ea-7940-4a0c-93ea-25e918710549",
   CellTags->{"c:24", "b:3.3.3"}],
  Cell[370020, 9328, 173, 2, 23, "Text",ExpressionUUID->"a55f7a7e-6341-49e3-aae9-c51f64ade20d",
   CellTags->"b:3.3.3"],
  Cell[370218, 9334, 554, 16, 48, "Input",ExpressionUUID->"74329215-36c5-41dc-8ef3-50537c53856b",
   CellTags->"b:3.3.3"],
  Cell[370775, 9352, 210, 5, 48, "Output",ExpressionUUID->"60087992-bd7e-4127-b6d3-5f7df6d3ebe2",
   CellTags->"b:3.3.3"],
  Cell[371000, 9360, 211, 5, 51, "Text",ExpressionUUID->"b2a0d65d-7b23-4347-994d-df493e99c8b6",
   CellTags->"b:3.3.3"],
  Cell[371214, 9367, 1279, 41, 90, "Input",ExpressionUUID->"f7f641d1-bc4e-41d1-8802-482b03510c28",
   CellTags->"b:3.3.3"],
  Cell[372518, 9412, 322, 7, 68, "Input",ExpressionUUID->"63221bfe-763f-4efa-98bd-303157724000",
   CellTags->"b:3.3.3"],
  Cell[372843, 9421, 69747, 1162, 254, "Output",ExpressionUUID->"091c98e8-8d48-49e3-b34d-96d553d9e399",
   CellTags->"b:3.3.3"],
  Cell[442605, 10586, 161, 2, 23, "Text",ExpressionUUID->"57544e27-7ff3-4d46-8a94-65fe0263ea05",
   CellTags->"b:3.3.3"]},
 "c:25"->{
  Cell[442803, 10593, 187, 4, 60, "Subsection",ExpressionUUID->"01673a7b-f377-4d2a-933e-90abcb15638f",
   CellTags->{"c:25", "b:3.3.4"}]},
 "b:3.3.4"->{
  Cell[442803, 10593, 187, 4, 60, "Subsection",ExpressionUUID->"01673a7b-f377-4d2a-933e-90abcb15638f",
   CellTags->{"c:25", "b:3.3.4"}],
  Cell[442993, 10599, 271, 6, 51, "Text",ExpressionUUID->"826f2ec6-3675-46dd-aff0-c5f0b2feae57",
   CellTags->"b:3.3.4"],
  Cell[443289, 10609, 418, 12, 60, "Input",ExpressionUUID->"a805bfb9-0273-4bdc-af45-80e6825e7c2f",
   CellTags->"b:3.3.4"],
  Cell[443710, 10623, 210, 5, 48, "Output",ExpressionUUID->"61815ec8-745e-44f3-ab96-2cde0786d2ec",
   CellTags->"b:3.3.4"],
  Cell[443935, 10631, 313, 8, 23, "Text",ExpressionUUID->"aff5676d-c965-4ea6-9c5e-a55804bc80a1",
   CellTags->"b:3.3.4"],
  Cell[444273, 10643, 791, 23, 87, "Input",ExpressionUUID->"be6ec06f-42ab-4307-9ed7-a45365c9cb53",
   CellTags->"b:3.3.4"],
  Cell[445067, 10668, 699, 23, 132, "Output",ExpressionUUID->"243cc6b6-cbe9-4ed1-a27c-7e8fcda02a6a",
   CellTags->"b:3.3.4"],
  Cell[445769, 10693, 693, 23, 132, "Output",ExpressionUUID->"c8adf71f-3895-44fb-a1d2-42cf6292f19f",
   CellTags->"b:3.3.4"],
  Cell[446499, 10721, 283, 7, 68, "Input",ExpressionUUID->"343c5077-25a9-4897-896c-e8b5e414b5df",
   CellTags->"b:3.3.4"],
  Cell[446785, 10730, 209, 5, 48, "Output",ExpressionUUID->"a50b030c-d242-4a4e-b547-f79633715e74",
   CellTags->"b:3.3.4"],
  Cell[446997, 10737, 2715, 62, 254, "Output",ExpressionUUID->"a4cc93c1-779b-4ee2-97ee-11c9b00d1036",
   CellTags->"b:3.3.4"]},
 "c:26"->{
  Cell[449761, 10805, 196, 4, 60, "Subsection",ExpressionUUID->"7c24ac23-2009-4b56-aa2e-336e4ba30209",
   CellTags->{"c:26", "b:3.3.5"}]},
 "b:3.3.5"->{
  Cell[449761, 10805, 196, 4, 60, "Subsection",ExpressionUUID->"7c24ac23-2009-4b56-aa2e-336e4ba30209",
   CellTags->{"c:26", "b:3.3.5"}],
  Cell[449960, 10811, 353, 10, 25, "Text",ExpressionUUID->"e3990e54-8488-46f9-b5c9-f8b7644c9bb0",
   CellTags->"b:3.3.5"],
  Cell[450316, 10823, 182, 4, 48, "Input",ExpressionUUID->"240a7c30-dd95-41ad-85da-b1f8816e1a43",
   CellTags->"b:3.3.5"],
  Cell[450523, 10831, 960, 27, 89, "Input",ExpressionUUID->"c086152a-9f6f-4ad4-af89-02d73a44d001",
   CellTags->"b:3.3.5"],
  Cell[451486, 10860, 211, 5, 48, "Output",ExpressionUUID->"6b514fbd-8031-473d-a711-93a0cec748c8",
   CellTags->"b:3.3.5"],
  Cell[451712, 10868, 182, 4, 48, "Input",ExpressionUUID->"e21b57c6-a08b-4017-9c75-7c81905be74a",
   CellTags->"b:3.3.5"],
  Cell[451919, 10876, 235, 4, 48, "Input",ExpressionUUID->"0563c3d8-b7e5-4ea6-a6f6-9886fa08c85d",
   CellTags->"b:3.3.5"],
  Cell[452157, 10882, 390, 7, 24, "Print",ExpressionUUID->"e3ecae34-1fec-48bf-aefc-3ca643621745",
   CellTags->"b:3.3.5"],
  Cell[452562, 10892, 198, 4, 23, "Text",ExpressionUUID->"bd1bf6e7-69a2-408c-82f3-7553b09337c7",
   CellTags->"b:3.3.5"],
  Cell[452763, 10898, 175, 4, 48, "Input",ExpressionUUID->"4bdce0bf-05bb-49be-83ab-788348d94d1d",
   CellTags->"b:3.3.5"],
  Cell[452941, 10904, 805, 23, 51, "Input",ExpressionUUID->"a21e999c-5d12-4918-b2f3-dc36e0c8e8d8",
   CellTags->"b:3.3.5"],
  Cell[453771, 10931, 551, 15, 87, "Input",ExpressionUUID->"f75c87f8-51f9-4542-a676-d1ba0a480785",
   CellTags->"b:3.3.5"],
  Cell[454325, 10948, 226, 5, 48, "Output",ExpressionUUID->"9db99c1f-e3d2-482d-9d0d-faee1a4246f2",
   CellTags->"b:3.3.5"],
  Cell[454588, 10958, 865, 23, 87, "Input",ExpressionUUID->"b18139f7-9804-4806-b169-4e1114f219bb",
   CellTags->"b:3.3.5"],
  Cell[455456, 10983, 8314, 158, 214, "Output",ExpressionUUID->"98548c40-7051-4cb4-a49d-63860f5ff0aa",
   CellTags->"b:3.3.5"],
  Cell[463773, 11143, 7235, 137, 226, "Output",ExpressionUUID->"68048384-4d28-4dc3-be8d-b911c3697ab9",
   CellTags->"b:3.3.5"]},
 "c:27"->{
  Cell[471057, 11286, 256, 7, 60, "Subsection",ExpressionUUID->"da7869f9-7d7c-4587-91ba-cc7e24024f8e",
   CellTags->{"c:27", "b:3.3.6"}]},
 "b:3.3.6"->{
  Cell[471057, 11286, 256, 7, 60, "Subsection",ExpressionUUID->"da7869f9-7d7c-4587-91ba-cc7e24024f8e",
   CellTags->{"c:27", "b:3.3.6"}],
  Cell[471316, 11295, 256, 6, 51, "Text",ExpressionUUID->"57c0bbfd-1392-4b3a-a5b9-9bda4650de71",
   CellTags->"b:3.3.6"],
  Cell[471597, 11305, 109, 1, 48, "Input",ExpressionUUID->"37c3e03e-ea93-43c1-a115-c7ef670c6446",
   CellTags->"b:3.3.6"],
  Cell[471709, 11308, 212, 5, 48, "Output",ExpressionUUID->"2b2f7218-96e7-48df-8621-7c6f86f890ed",
   CellTags->"b:3.3.6"],
  Cell[471936, 11316, 144, 3, 48, "Input",ExpressionUUID->"d77df3a4-0dda-4c2f-bbe4-15048556c0af",
   CellTags->"b:3.3.6"],
  Cell[472083, 11321, 4274, 115, 288, "Input",ExpressionUUID->"e133b724-ace1-4387-ad6e-30b4598866d1",
   CellTags->"b:3.3.6"],
  Cell[476382, 11440, 721, 18, 106, "Input",ExpressionUUID->"6e2edc63-0cff-491b-b526-62b7711b89de",
   CellTags->"b:3.3.6"],
  Cell[477106, 11460, 1445, 31, 302, "Output",ExpressionUUID->"a170edfd-4f1a-464f-8a56-4b93dd632905",
   CellTags->"b:3.3.6"],
  Cell[478554, 11493, 607, 12, 24, "Print",ExpressionUUID->"ca13c08e-1ebf-4394-b021-10a70b8a224a",
   CellTags->"b:3.3.6"],
  Cell[479198, 11510, 136, 2, 48, "Input",ExpressionUUID->"0c26bbdb-24d2-4a49-ace0-1549096fe8cd",
   CellTags->"b:3.3.6"],
  Cell[479337, 11514, 234, 5, 48, "Output",ExpressionUUID->"632b187a-5b42-4522-8afe-3e88fb680420",
   CellTags->"b:3.3.6"],
  Cell[479586, 11522, 107, 1, 48, "Input",ExpressionUUID->"4e8a88c3-d0c8-4560-b680-35bb9930e6ea",
   CellTags->"b:3.3.6"]},
 "c:28"->{
  Cell[479742, 11529, 229, 7, 31, "Section",ExpressionUUID->"13fd45fe-183f-48a7-b25c-9ed750d8b40d",
   CellTags->"c:28"]},
 "c:29"->{
  Cell[479996, 11540, 238, 6, 60, "Subsection",ExpressionUUID->"fa752dc4-6539-4dda-9b31-ef8167f375d1",
   CellTags->{"c:29", "b:3.4.0"}]},
 "b:3.4.0"->{
  Cell[479996, 11540, 238, 6, 60, "Subsection",ExpressionUUID->"fa752dc4-6539-4dda-9b31-ef8167f375d1",
   CellTags->{"c:29", "b:3.4.0"}],
  Cell[480259, 11550, 188, 3, 46, "Input",ExpressionUUID->"db8fee6a-8830-4d13-a934-10f86c1f3716",
   CellTags->"b:3.4.0"],
  Cell[480450, 11555, 227, 3, 24, "Print",ExpressionUUID->"17059a87-af8c-4524-8811-404757c4c88c",
   CellTags->"b:3.4.0"],
  Cell[480714, 11563, 167, 3, 46, "Input",ExpressionUUID->"6be6dcfe-df44-4c6f-b2ba-643ae1a1844a",
   CellTags->"b:3.4.0"],
  Cell[480884, 11568, 55968, 1259, 493, "Output",ExpressionUUID->"752c25da-ce10-4c47-a188-36bbf1a10539",
   CellTags->"b:3.4.0"],
  Cell[536867, 12830, 223, 5, 23, "Text",ExpressionUUID->"690a4ac2-1826-42c2-bc24-d918af016df9",
   CellTags->"b:3.4.0"],
  Cell[537093, 12837, 307, 7, 51, "Text",ExpressionUUID->"00f81e3c-9822-4a10-8d8a-c2aa293af7c2",
   CellTags->"b:3.4.0"],
  Cell[537403, 12846, 462, 13, 65, "Input",ExpressionUUID->"ce6b1d7f-de2d-4661-aa19-7de144df4975",
   CellTags->"b:3.4.0"],
  Cell[537868, 12861, 167, 2, 23, "Text",ExpressionUUID->"d4e65207-d148-4f77-8af6-7d37c0e23612",
   CellTags->"b:3.4.0"],
  Cell[538038, 12865, 170, 4, 48, "Input",ExpressionUUID->"6b908e59-b062-41bf-9f5c-a4c3954841d9",
   CellTags->"b:3.4.0"]},
 "c:30"->{
  Cell[538245, 12874, 264, 7, 60, "Subsection",ExpressionUUID->"40b0c817-b042-450e-8a68-59611963dd0a",
   CellTags->{"c:30", "b:3.4.1"}]},
 "b:3.4.1"->{
  Cell[538245, 12874, 264, 7, 60, "Subsection",ExpressionUUID->"40b0c817-b042-450e-8a68-59611963dd0a",
   CellTags->{"c:30", "b:3.4.1"}],
  Cell[538512, 12883, 159, 2, 23, "Text",ExpressionUUID->"65476cea-fc27-466e-b37d-56df137ec07f",
   CellTags->"b:3.4.1"],
  Cell[538696, 12889, 150, 3, 48, "Input",ExpressionUUID->"8b33a7d1-c0a1-4dce-a927-f7a260b48888",
   CellTags->"b:3.4.1"],
  Cell[538849, 12894, 210, 5, 48, "Output",ExpressionUUID->"2ef3ed32-9aff-4e56-bf06-4cbc7c3bc8ed",
   CellTags->"b:3.4.1"],
  Cell[539074, 12902, 174, 2, 23, "Text",ExpressionUUID->"c9562f80-d03b-4483-8912-d1e88dfa9b0d",
   CellTags->"b:3.4.1"],
  Cell[539251, 12906, 145, 3, 48, "Input",ExpressionUUID->"dcf258a9-23ac-4c26-83dd-d4cab8ebc184",
   CellTags->"b:3.4.1"],
  Cell[539399, 12911, 212, 5, 23, "Text",ExpressionUUID->"895b6e93-24d7-491d-a4d5-3027e52c3ffb",
   CellTags->"b:3.4.1"],
  Cell[539614, 12918, 382, 11, 48, "Input",ExpressionUUID->"999cc834-bf63-4ae8-9c00-e52f7e5daa6f",
   CellTags->"b:3.4.1"],
  Cell[539999, 12931, 332, 8, 48, "Input",ExpressionUUID->"060ec69a-c967-4e67-9d74-5dcd22e5376d",
   CellTags->"b:3.4.1"],
  Cell[540334, 12941, 735, 19, 87, "Input",ExpressionUUID->"b8c78657-830f-4660-95c7-aa419f101d3b",
   CellTags->"b:3.4.1"],
  Cell[541072, 12962, 1662, 47, 173, "Text",ExpressionUUID->"de9b52f6-1820-44d5-b767-072adf4daf93",
   CellTags->"b:3.4.1"],
  Cell[542759, 13013, 270, 7, 48, "Input",ExpressionUUID->"f4a53142-a969-4623-9d81-eb839c5f7651",
   CellTags->"b:3.4.1"],
  Cell[543032, 13022, 212, 5, 48, "Output",ExpressionUUID->"56cadb2b-bcf1-401b-94b4-69638b0b3295",
   CellTags->"b:3.4.1"],
  Cell[543281, 13032, 519, 14, 106, "Input",ExpressionUUID->"e7a9ae3e-8bbe-43a9-96da-071d42450322",
   CellTags->"b:3.4.1"],
  Cell[543803, 13048, 209, 5, 48, "Output",ExpressionUUID->"c575adfe-2eed-4b60-9e2c-b5953d44c001",
   CellTags->"b:3.4.1"],
  Cell[544015, 13055, 211, 5, 48, "Output",ExpressionUUID->"ebee835a-17ab-4d42-8d82-5e6664d0a36d",
   CellTags->"b:3.4.1"],
  Cell[544229, 13062, 210, 5, 48, "Output",ExpressionUUID->"168c94dd-a4e1-4f21-8de5-bc60d0c0539d",
   CellTags->"b:3.4.1"],
  Cell[544442, 13069, 209, 5, 48, "Output",ExpressionUUID->"1db8c5d5-f630-4e04-bfb3-b2c2a88455ff",
   CellTags->"b:3.4.1"],
  Cell[544666, 13077, 303, 7, 51, "Text",ExpressionUUID->"456b650f-390c-4a4f-b37b-46bce1bfd17c",
   CellTags->"b:3.4.1"],
  Cell[544972, 13086, 434, 12, 48, "Input",ExpressionUUID->"ec5f04cd-9377-4837-a62d-9edf9333f31c",
   CellTags->"b:3.4.1"]},
 "c:31"->{
  Cell[545443, 13103, 469, 13, 62, "Subsection",ExpressionUUID->"d119ab91-7c89-498d-b356-84531733b861",
   CellTags->{"c:31", "b:3.4.2"}]},
 "b:3.4.2"->{
  Cell[545443, 13103, 469, 13, 62, "Subsection",ExpressionUUID->"d119ab91-7c89-498d-b356-84531733b861",
   CellTags->{"c:31", "b:3.4.2"}],
  Cell[545915, 13118, 1110, 32, 97, "Text",ExpressionUUID->"289a1164-3337-4f7b-831d-4dfe47842b31",
   CellTags->"b:3.4.2"]},
 "c:32"->{
  Cell[547062, 13155, 585, 17, 65, "Subsection",ExpressionUUID->"8278dbd4-1671-4f09-9f3d-4542bf1a7152",
   CellTags->{"c:32", "b:3.4.3"}]},
 "b:3.4.3"->{
  Cell[547062, 13155, 585, 17, 65, "Subsection",ExpressionUUID->"8278dbd4-1671-4f09-9f3d-4542bf1a7152",
   CellTags->{"c:32", "b:3.4.3"}],
  Cell[547650, 13174, 154, 2, 23, "Text",ExpressionUUID->"e6fb4063-dc12-4577-9f5e-9afb37053f46",
   CellTags->"b:3.4.3"],
  Cell[547829, 13180, 554, 16, 48, "Input",ExpressionUUID->"af481f0f-ff83-4e3e-a33b-4dc67303bd13",
   CellTags->"b:3.4.3"],
  Cell[548386, 13198, 210, 5, 48, "Output",ExpressionUUID->"742e003d-8937-4305-be67-a6b89d51c545",
   CellTags->"b:3.4.3"],
  Cell[548611, 13206, 1380, 42, 63, "Text",ExpressionUUID->"a0412f62-d1dd-474d-833f-b77f8e163b4d",
   CellTags->"b:3.4.3"]},
 "c:33"->{
  Cell[550028, 13253, 207, 4, 60, "Subsection",ExpressionUUID->"0a840919-cec3-4179-8e83-b7031f518721",
   CellTags->{"c:33", "b:3.4.4"}]},
 "b:3.4.4"->{
  Cell[550028, 13253, 207, 4, 60, "Subsection",ExpressionUUID->"0a840919-cec3-4179-8e83-b7031f518721",
   CellTags->{"c:33", "b:3.4.4"}],
  Cell[550238, 13259, 468, 11, 55, "Text",ExpressionUUID->"98551af2-721f-4145-add0-0c1e10724682",
   CellTags->"b:3.4.4"],
  Cell[550731, 13274, 418, 12, 60, "Input",ExpressionUUID->"e4371f2e-8aef-4731-8b71-ddc9261ac155",
   CellTags->"b:3.4.4"],
  Cell[551152, 13288, 210, 5, 48, "Output",ExpressionUUID->"ed933a35-e02c-4298-a655-9909c549e2a4",
   CellTags->"b:3.4.4"],
  Cell[551399, 13298, 370, 10, 87, "Input",ExpressionUUID->"e49c3b07-13d7-4dc7-9099-c33c3774ee08",
   CellTags->"b:3.4.4"],
  Cell[551772, 13310, 212, 5, 48, "Output",ExpressionUUID->"a29bf67b-5961-4fe2-988b-6a787f8115e3",
   CellTags->"b:3.4.4"],
  Cell[551987, 13317, 211, 5, 48, "Output",ExpressionUUID->"f43a7e3e-dfc2-4e1f-988f-2ef3656b0da4",
   CellTags->"b:3.4.4"],
  Cell[552201, 13324, 215, 5, 48, "Output",ExpressionUUID->"5afc2192-6b29-4b3f-b5c0-9d05e19b4bb2",
   CellTags->"b:3.4.4"],
  Cell[552431, 13332, 596, 18, 25, "Text",ExpressionUUID->"1c4a527d-8219-49e3-8ce9-83158e473dc7",
   CellTags->"b:3.4.4"],
  Cell[553030, 13352, 961, 29, 68, "Input",ExpressionUUID->"1fef000b-fc26-4d0d-a42a-6d92b8acbb95",
   CellTags->"b:3.4.4"],
  Cell[553994, 13383, 400, 11, 25, "Text",ExpressionUUID->"33971208-b4af-4f5e-acd3-850ceea0f6af",
   CellTags->"b:3.4.4"],
  Cell[554397, 13396, 2019, 62, 115, "Input",ExpressionUUID->"448b68fe-3d99-4f92-b723-79dc663a692e",
   CellTags->"b:3.4.4"],
  Cell[556419, 13460, 189, 4, 23, "Text",ExpressionUUID->"c3dd20c5-e84b-4681-8264-5b7e1e242a51",
   CellTags->"b:3.4.4"],
  Cell[556611, 13466, 819, 25, 87, "Input",ExpressionUUID->"a46d6d01-6126-4bc0-a824-9be50b8f05b2",
   CellTags->"b:3.4.4"],
  Cell[557455, 13495, 135, 2, 48, "Input",ExpressionUUID->"b0bb4c62-ccb1-4992-9e41-a9e48f7a2082",
   CellTags->"b:3.4.4"],
  Cell[557593, 13499, 214, 5, 48, "Output",ExpressionUUID->"a817ccd8-a8b0-4722-8b26-c322a00ad647",
   CellTags->"b:3.4.4"]},
 "c:34"->{
  Cell[557856, 13510, 201, 4, 60, "Subsection",ExpressionUUID->"64dcee58-a1d1-4934-8ecb-0881b5c947d8",
   CellTags->{"c:34", "b:3.4.5"}]},
 "b:3.4.5"->{
  Cell[557856, 13510, 201, 4, 60, "Subsection",ExpressionUUID->"64dcee58-a1d1-4934-8ecb-0881b5c947d8",
   CellTags->{"c:34", "b:3.4.5"}],
  Cell[558060, 13516, 215, 5, 23, "Text",ExpressionUUID->"ebf67078-deeb-4d02-9435-9900f8c0280d",
   CellTags->"b:3.4.5"],
  Cell[558278, 13523, 182, 4, 48, "Input",ExpressionUUID->"0416feec-c6d7-4a5a-ae7e-e21f7cf45018",
   CellTags->"b:3.4.5"],
  Cell[558463, 13529, 187, 4, 48, "Input",ExpressionUUID->"1330a74d-ae4a-4cd0-8e63-995c764cb490",
   CellTags->"b:3.4.5"],
  Cell[558675, 13537, 346, 8, 68, "Input",ExpressionUUID->"c82cb01c-7267-45d4-b7de-f54e6f34bd71",
   CellTags->"b:3.4.5"],
  Cell[559024, 13547, 390, 7, 24, "Print",ExpressionUUID->"6cc5eb13-0120-41ea-9b29-545c996602d8",
   CellTags->"b:3.4.5"]},
 "c:35"->{
  Cell[559463, 13560, 248, 7, 60, "Subsection",ExpressionUUID->"b0f85306-8e1b-4b5d-9f95-71b193ff39db",
   CellTags->{"c:35", "b:3.4.6"}]},
 "b:3.4.6"->{
  Cell[559463, 13560, 248, 7, 60, "Subsection",ExpressionUUID->"b0f85306-8e1b-4b5d-9f95-71b193ff39db",
   CellTags->{"c:35", "b:3.4.6"}],
  Cell[559714, 13569, 209, 5, 23, "Text",ExpressionUUID->"d3d9fc44-7014-4763-816b-a6fe354e2989",
   CellTags->"b:3.4.6"],
  Cell[559926, 13576, 175, 4, 48, "Input",ExpressionUUID->"4beb6907-d3ca-4541-8990-a9a35d1fbd0c",
   CellTags->"b:3.4.6"],
  Cell[560104, 13582, 743, 22, 51, "Input",ExpressionUUID->"59092ff9-c10f-45fe-af99-7a433c7a95d9",
   CellTags->"b:3.4.6"],
  Cell[560850, 13606, 211, 5, 23, "Text",ExpressionUUID->"81a1cf8d-fa5b-4026-92f8-ac50e66e6e2d",
   CellTags->"b:3.4.6"],
  Cell[561086, 13615, 858, 22, 87, "Input",ExpressionUUID->"2a6ccb02-d902-4138-b163-dab440cbb2bb",
   CellTags->"b:3.4.6"],
  Cell[561947, 13639, 93480, 1554, 219, "Output",ExpressionUUID->"8f8885ea-371f-465e-83d9-bb9094c7d91b",
   CellTags->"b:3.4.6"],
  Cell[655430, 15195, 105076, 1740, 230, "Output",ExpressionUUID->"01606e01-3e64-4a7b-8cba-6588bc9eb555",
   CellTags->"b:3.4.6"]},
 "c:36"->{
  Cell[760555, 16941, 227, 6, 60, "Subsection",ExpressionUUID->"e091aa0c-714d-4e06-8a2f-d32343528a13",
   CellTags->{"c:36", "b:3.4.7"}]},
 "b:3.4.7"->{
  Cell[760555, 16941, 227, 6, 60, "Subsection",ExpressionUUID->"e091aa0c-714d-4e06-8a2f-d32343528a13",
   CellTags->{"c:36", "b:3.4.7"}],
  Cell[760785, 16949, 627, 13, 136, "Text",ExpressionUUID->"d5f16e23-ba79-4739-8e7c-179a1b65c4ed",
   CellTags->"b:3.4.7"],
  Cell[761415, 16964, 144, 3, 48, "Input",ExpressionUUID->"a07dce25-960f-45b5-970a-f73f2465d8e5",
   CellTags->"b:3.4.7"],
  Cell[761562, 16969, 4274, 115, 288, "Input",ExpressionUUID->"df519045-8997-4196-a8ba-7bcb9e8f5e80",
   CellTags->"b:3.4.7"],
  Cell[765861, 17088, 822, 20, 125, "Input",ExpressionUUID->"660471de-1a16-41d9-8c6e-ddb17a01c4c4",
   CellTags->"b:3.4.7"],
  Cell[766686, 17110, 2156, 43, 506, "Output",ExpressionUUID->"f4adddef-0c7f-4d98-afc5-805531ff7432",
   CellTags->"b:3.4.7"],
  Cell[768845, 17155, 271, 6, 24, "Print",ExpressionUUID->"a55a1b3f-ac77-41a6-9665-c22e0b0842b1",
   CellTags->"b:3.4.7"],
  Cell[769119, 17163, 605, 12, 24, "Print",ExpressionUUID->"fe58f465-af30-4318-8add-2197fc98fc5b",
   CellTags->"b:3.4.7"],
  Cell[769761, 17180, 136, 2, 48, "Input",ExpressionUUID->"66632646-9f1c-477c-bb1a-2220bd57b970",
   CellTags->"b:3.4.7"],
  Cell[769900, 17184, 234, 5, 48, "Output",ExpressionUUID->"ececa644-1a9b-4896-849a-c240279dbc09",
   CellTags->"b:3.4.7"],
  Cell[770149, 17192, 319, 7, 79, "Text",ExpressionUUID->"226ff202-9888-49a2-9320-8d3a2c9f85c3",
   CellTags->"b:3.4.7"],
  Cell[770493, 17203, 558, 15, 68, "Input",ExpressionUUID->"58762dcb-c072-4bc8-a28e-a1e30cce3aaf",
   CellTags->"b:3.4.7"],
  Cell[771054, 17220, 1648, 44, 264, "Output",ExpressionUUID->"8ede2d58-e834-4d41-9986-56da39c73074",
   CellTags->"b:3.4.7"],
  Cell[772717, 17267, 98, 1, 48, "Input",ExpressionUUID->"ef71a855-21d0-4afc-85fc-6e29744051bc",
   CellTags->"b:3.4.7"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"c:1", 775018, 17318},
 {"c:2", 775142, 17321},
 {"b:3.0", 775399, 17326},
 {"c:3", 775775, 17333},
 {"b:3.1.0", 775923, 17336},
 {"c:4", 777890, 17369},
 {"b:3.1.1", 778040, 17372},
 {"c:5", 780249, 17409},
 {"b:3.1.2", 780400, 17412},
 {"c:6", 780669, 17417},
 {"b:3.1.3", 780820, 17420},
 {"c:7", 781453, 17431},
 {"b:3.1.4", 781603, 17434},
 {"c:8", 784178, 17477},
 {"b:3.1.5", 784328, 17480},
 {"c:9", 785444, 17499},
 {"b:3.1.6", 785594, 17502},
 {"c:10", 786597, 17519},
 {"b:3.1.7", 786749, 17522},
 {"c:11", 788621, 17553},
 {"c:12", 788754, 17556},
 {"b:3.2.0", 788906, 17559},
 {"c:13", 790778, 17590},
 {"b:3.2.1", 790930, 17593},
 {"c:14", 793527, 17636},
 {"b:3.2.2", 793680, 17639},
 {"c:15", 793953, 17644},
 {"b:3.2.3", 794106, 17647},
 {"c:16", 794991, 17662},
 {"b:3.2.4", 795143, 17665},
 {"c:17", 797130, 17698},
 {"b:3.2.5", 797282, 17701},
 {"c:18", 798163, 17716},
 {"b:3.2.6", 798315, 17719},
 {"c:19", 799328, 17736},
 {"b:3.2.7", 799480, 17739},
 {"c:20", 801230, 17768},
 {"c:21", 801363, 17771},
 {"b:3.3.0", 801516, 17774},
 {"c:22", 803389, 17805},
 {"b:3.3.1", 803541, 17808},
 {"c:23", 805525, 17841},
 {"b:3.3.2", 805678, 17844},
 {"c:24", 806196, 17853},
 {"b:3.3.3", 806349, 17856},
 {"c:25", 807484, 17875},
 {"b:3.3.4", 807637, 17878},
 {"c:26", 809029, 17901},
 {"b:3.3.5", 809182, 17904},
 {"c:27", 811192, 17937},
 {"b:3.3.6", 811345, 17940},
 {"c:28", 812860, 17965},
 {"c:29", 812994, 17968},
 {"b:3.4.0", 813147, 17971},
 {"c:30", 814409, 17992},
 {"b:3.4.1", 814562, 17995},
 {"c:31", 817058, 18036},
 {"b:3.4.2", 817212, 18039},
 {"c:32", 817487, 18044},
 {"b:3.4.3", 817641, 18047},
 {"c:33", 818286, 18058},
 {"b:3.4.4", 818439, 18061},
 {"c:34", 820445, 18094},
 {"b:3.4.5", 820598, 18097},
 {"c:35", 821362, 18110},
 {"b:3.4.6", 821515, 18113},
 {"c:36", 822541, 18130},
 {"b:3.4.7", 822694, 18133}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 117, 1, 51, "Title",ExpressionUUID->"5cbfd1a9-b24b-490f-b89a-0154cd6bb247",
 CellTags->"c:1"],
Cell[665, 23, 118, 0, 35, "Subtitle",ExpressionUUID->"45086d7e-0eb3-4dad-8435-5a2a00a2c3ac"],
Cell[786, 25, 378, 10, 62, "Abstract",ExpressionUUID->"0c762211-3ba8-4afc-aaf2-2ab77f473a68"],
Cell[CellGroupData[{
Cell[1189, 39, 334, 7, 51, "Section",ExpressionUUID->"907f866b-f1f1-4a7e-9031-4b4cff6a52be",
 CellTags->{"c:2", "b:3.0"}],
Cell[1526, 48, 3004, 67, 523, "Abstract",ExpressionUUID->"900ba9e6-ee5f-439e-82e3-dc68971e5335",
 CellTags->"b:3.0"],
Cell[4533, 117, 230, 3, 22, "Text",ExpressionUUID->"06a8d43b-f285-4c75-ae6f-593ebe1e7985",
 CellTags->"b:3.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4800, 125, 161, 2, 51, "Section",ExpressionUUID->"04b471a7-d63e-4fd6-bc85-119c0edac590",
 CellTags->"c:2"],
Cell[CellGroupData[{
Cell[4986, 131, 237, 6, 60, "Subsection",ExpressionUUID->"88f3631c-804c-4474-babf-cd4d489e78da",
 CellTags->{"c:3", "b:3.1.0"}],
Cell[CellGroupData[{
Cell[5248, 141, 367, 7, 64, "Input",ExpressionUUID->"b4eb02b7-74b4-442d-936a-abbd145f01f5",
 CellTags->"b:3.1.0"],
Cell[5618, 150, 349, 5, 24, "Print",ExpressionUUID->"65d05b40-19dc-4dd1-8ed4-49b41aeb2f2c",
 CellTags->"b:3.1.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6004, 160, 167, 3, 46, "Input",ExpressionUUID->"649e5966-1fc3-4e8b-b424-4b2d1b82b726",
 CellTags->"b:3.1.0"],
Cell[6174, 165, 57032, 1281, 493, "Output",ExpressionUUID->"13adc82c-aea0-47cb-8afb-8300adf6340b",
 CellTags->"b:3.1.0"]
}, Open  ]],
Cell[63221, 1449, 253, 5, 23, "Text",ExpressionUUID->"2f36eb48-6974-4bac-b28c-d45b8fef80e2",
 CellTags->"b:3.1.0"],
Cell[63477, 1456, 1531, 42, 101, "Input",ExpressionUUID->"dc00caca-95c7-42c7-9d5d-bd9e64252067",
 CellTags->"b:3.1.0"],
Cell[65011, 1500, 868, 24, 87, "Input",ExpressionUUID->"8647f407-8468-4a92-8945-e6c6a324eeda",
 CellTags->"b:3.1.0"],
Cell[65882, 1526, 1156, 32, 87, "Input",ExpressionUUID->"c4a5dd86-cdea-4873-8dc0-d922706225c0",
 CellTags->"b:3.1.0"],
Cell[67041, 1560, 841, 25, 68, "Input",ExpressionUUID->"ce7152e7-628f-4c48-b960-d5298267cd7e",
 CellTags->"b:3.1.0"],
Cell[67885, 1587, 653, 18, 68, "Input",ExpressionUUID->"759956d6-6ee7-44fe-bcc6-3e20155405fb",
 CellTags->"b:3.1.0"],
Cell[68541, 1607, 2616, 67, 163, "Input",ExpressionUUID->"a2860cee-cde7-4626-9571-dde99c327565",
 CellTags->"b:3.1.0"],
Cell[71160, 1676, 307, 7, 51, "Text",ExpressionUUID->"587831e8-2e6d-4f5b-b757-c8ee1cc11f0f",
 CellTags->"b:3.1.0"],
Cell[71470, 1685, 462, 13, 65, "Input",ExpressionUUID->"7b386c71-97b5-4506-bbfb-5f46553f44f6",
 CellTags->"b:3.1.0"],
Cell[71935, 1700, 167, 2, 23, "Text",ExpressionUUID->"1a8cc137-fbd0-4251-af22-8c20742ba62b",
 CellTags->"b:3.1.0"],
Cell[72105, 1704, 170, 4, 48, "Input",ExpressionUUID->"25d86afd-2547-4bc4-b195-117c8216f2d6",
 CellTags->"b:3.1.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72312, 1713, 270, 7, 60, "Subsection",ExpressionUUID->"c8df16bb-759e-4dae-8fd6-5a5adba1a925",
 CellTags->{"c:4", "b:3.1.1"}],
Cell[72585, 1722, 159, 2, 23, "Text",ExpressionUUID->"ac5ac293-60fa-4e41-a6a7-2a2754b4b4f4",
 CellTags->"b:3.1.1"],
Cell[72747, 1726, 145, 3, 48, "Input",ExpressionUUID->"e8d6f5b9-86f7-4959-aec4-1db9f7599b91",
 CellTags->"b:3.1.1"],
Cell[72895, 1731, 174, 2, 23, "Text",ExpressionUUID->"46b63a6c-9ce5-4e06-ba8b-9f7d176a0445",
 CellTags->"b:3.1.1"],
Cell[73072, 1735, 145, 3, 48, "Input",ExpressionUUID->"7f83aa48-6ba0-4ff1-b9cc-498239cdba80",
 CellTags->"b:3.1.1"],
Cell[73220, 1740, 247, 5, 23, "Text",ExpressionUUID->"b1a5a53a-6d02-49f9-9012-fbbc29a58a70",
 CellTags->"b:3.1.1"],
Cell[73470, 1747, 382, 11, 48, "Input",ExpressionUUID->"49d10476-5555-4aa0-8ed9-45e1130ff675",
 CellTags->"b:3.1.1"],
Cell[73855, 1760, 332, 8, 48, "Input",ExpressionUUID->"79e53433-c843-4cbd-b058-d683850990fc",
 CellTags->"b:3.1.1"],
Cell[74190, 1770, 735, 19, 87, "Input",ExpressionUUID->"cd8301cc-cbbd-475f-9136-1efe9f1fd9f7",
 CellTags->"b:3.1.1"],
Cell[74928, 1791, 1583, 44, 173, "Text",ExpressionUUID->"44059999-2373-4ce3-b6f4-56f37ed4449c",
 CellTags->"b:3.1.1"],
Cell[CellGroupData[{
Cell[76536, 1839, 270, 7, 48, "Input",ExpressionUUID->"c07d56b5-fa69-45a1-992e-30de76f52e72",
 CellTags->"b:3.1.1"],
Cell[76809, 1848, 332, 7, 48, "Output",ExpressionUUID->"3ea2b04e-4659-4e7a-9860-b83ea910a039",
 CellTags->"b:3.1.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77178, 1860, 568, 16, 106, "Input",ExpressionUUID->"5244f1fd-21cc-4fd8-a168-14c9d5454f04",
 CellTags->"b:3.1.1"],
Cell[77749, 1878, 334, 7, 48, "Output",ExpressionUUID->"102c44dd-e4f6-4291-af6f-37cf1941e5f8",
 CellTags->"b:3.1.1"],
Cell[78086, 1887, 332, 7, 48, "Output",ExpressionUUID->"ab6390af-9238-4326-b067-94c297a31aea",
 CellTags->"b:3.1.1"]
}, Open  ]],
Cell[78433, 1897, 304, 7, 51, "Text",ExpressionUUID->"56644221-7526-4d00-85ee-243218b9ca87",
 CellTags->"b:3.1.1"],
Cell[CellGroupData[{
Cell[78762, 1908, 434, 12, 48, "Input",ExpressionUUID->"a0971beb-0a6d-481f-852b-b7812373b1f2",
 CellTags->"b:3.1.1"],
Cell[79199, 1922, 371, 5, 24, "Print",ExpressionUUID->"8041f7cd-1c19-4263-a9db-e2c5b5e52bae",
 CellTags->"b:3.1.1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[79619, 1933, 468, 13, 62, "Subsection",ExpressionUUID->"9f64068a-1716-41c1-9380-6ebcdb05a054",
 CellTags->{"c:5", "b:3.1.2"}],
Cell[80090, 1948, 1067, 31, 97, "Text",ExpressionUUID->"cb14f4d9-dac5-471b-9abf-cf40e32855ae",
 CellTags->"b:3.1.2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81194, 1984, 584, 17, 65, "Subsection",ExpressionUUID->"686a6ce0-9161-4ca6-9cb2-35da78e8e4e4",
 CellTags->{"c:6", "b:3.1.3"}],
Cell[81781, 2003, 154, 2, 23, "Text",ExpressionUUID->"4ed792cc-4499-4bdd-9595-a2f9c931b454",
 CellTags->"b:3.1.3"],
Cell[CellGroupData[{
Cell[81960, 2009, 554, 16, 48, "Input",ExpressionUUID->"76cc33e8-bcd3-4ff3-ad27-e07e7a8a1618",
 CellTags->"b:3.1.3"],
Cell[82517, 2027, 333, 7, 48, "Output",ExpressionUUID->"b007c3ab-8b4a-420b-8861-0bcb7e80ed76",
 CellTags->"b:3.1.3"]
}, Open  ]],
Cell[82865, 2037, 1380, 42, 63, "Text",ExpressionUUID->"a65dbdc9-e5b3-406d-b575-4a8c6c6ac5e1",
 CellTags->"b:3.1.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84282, 2084, 206, 4, 60, "Subsection",ExpressionUUID->"f1b39d7d-e749-4586-9c88-b559e8a6831e",
 CellTags->{"c:7", "b:3.1.4"}],
Cell[84491, 2090, 400, 10, 55, "Text",ExpressionUUID->"2829ad68-1cbc-4d82-9aa0-9f58698439d8",
 CellTags->"b:3.1.4"],
Cell[CellGroupData[{
Cell[84916, 2104, 418, 12, 60, "Input",ExpressionUUID->"c22e121b-2b66-4028-bae9-0bdf25ef1fe3",
 CellTags->"b:3.1.4"],
Cell[85337, 2118, 330, 7, 48, "Output",ExpressionUUID->"71e0158b-0328-4cbc-a541-18b72debbd67",
 CellTags->"b:3.1.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85704, 2130, 370, 10, 87, "Input",ExpressionUUID->"15079ef9-8449-4a35-97ea-10a9a1703247",
 CellTags->"b:3.1.4"],
Cell[86077, 2142, 329, 7, 48, "Output",ExpressionUUID->"c453e302-e944-492e-95ea-b82213eb45bd",
 CellTags->"b:3.1.4"],
Cell[86409, 2151, 327, 7, 48, "Output",ExpressionUUID->"8b4f24f5-6186-4b24-aa87-9d3850622244",
 CellTags->"b:3.1.4"],
Cell[86739, 2160, 333, 7, 48, "Output",ExpressionUUID->"71b521e3-4c00-4f4b-91fc-29a58ad27f60",
 CellTags->"b:3.1.4"]
}, Open  ]],
Cell[87087, 2170, 596, 18, 25, "Text",ExpressionUUID->"060b2127-7e7c-4426-9aa7-9d061258bb9f",
 CellTags->"b:3.1.4"],
Cell[87686, 2190, 501, 14, 48, "Input",ExpressionUUID->"f3f296c0-1e48-474b-a107-66c839177611",
 CellTags->"b:3.1.4"],
Cell[88190, 2206, 400, 11, 25, "Text",ExpressionUUID->"abd55984-1aed-4308-8dee-dc9fe9e708b9",
 CellTags->"b:3.1.4"],
Cell[88593, 2219, 1109, 34, 93, "Input",ExpressionUUID->"5601443d-d566-43af-ad75-45e13b6090fb",
 CellTags->"b:3.1.4"],
Cell[89705, 2255, 189, 4, 23, "Text",ExpressionUUID->"4f931af5-cc57-41d9-9964-9a93cc1c44df",
 CellTags->"b:3.1.4"],
Cell[89897, 2261, 819, 25, 87, "Input",ExpressionUUID->"75aaafbf-5124-4626-bdfc-6089b808f357",
 CellTags->"b:3.1.4"],
Cell[90719, 2288, 278, 8, 48, "Input",ExpressionUUID->"4eeef5dc-3a2b-45c0-842b-bf2a695be73c",
 CellTags->"b:3.1.4"],
Cell[91000, 2298, 138, 2, 23, "Text",ExpressionUUID->"290722f7-0475-4a29-84d9-5ad1c7ebdec0",
 CellTags->"b:3.1.4"],
Cell[91141, 2302, 153, 3, 48, "Input",ExpressionUUID->"aa14095f-36c7-4f3c-afc5-bd352551d47e",
 CellTags->"b:3.1.4"],
Cell[91297, 2307, 150, 2, 23, "Text",ExpressionUUID->"aa262bf7-c9d3-430f-9d48-add5e451f579",
 CellTags->"b:3.1.4"],
Cell[91450, 2311, 708, 20, 48, "Input",ExpressionUUID->"827d4594-f283-42f4-a122-1f0f6f5c3d93",
 CellTags->"b:3.1.4"],
Cell[92161, 2333, 652, 19, 71, "Input",ExpressionUUID->"16c2cfe1-4ab7-4500-a0b5-963e0addaaf7",
 CellTags->"b:3.1.4"],
Cell[92816, 2354, 153, 3, 48, "Input",ExpressionUUID->"e3556bdb-cf40-4ad9-9be6-0a4e749eeb90",
 CellTags->"b:3.1.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93006, 2362, 200, 4, 60, "Subsection",ExpressionUUID->"b52dae01-44a5-4675-9e09-dd23c9a6099a",
 CellTags->{"c:8", "b:3.1.5"}],
Cell[93209, 2368, 186, 3, 23, "Text",ExpressionUUID->"a3937fed-1b25-41f5-a01a-6dda38b9b91d",
 CellTags->"b:3.1.5"],
Cell[93398, 2373, 532, 15, 48, "Input",ExpressionUUID->"c46732cf-92e0-4ba9-9fcf-eefcc10bcace",
 CellTags->"b:3.1.5"],
Cell[93933, 2390, 182, 4, 48, "Input",ExpressionUUID->"b3e06932-15c6-494a-89e3-a0203cb336ac",
 CellTags->"b:3.1.5"],
Cell[94118, 2396, 493, 13, 51, "Input",ExpressionUUID->"bbf072e9-dbcf-4f7b-9828-19ad0cbe65c8",
 CellTags->"b:3.1.5"],
Cell[CellGroupData[{
Cell[94636, 2413, 346, 8, 68, "Input",ExpressionUUID->"a4a9565e-a784-45be-b020-258073baaebf",
 CellTags->"b:3.1.5"],
Cell[94985, 2423, 510, 9, 24, "Print",ExpressionUUID->"316b04d2-209e-4891-ae27-e4195f7eebd7",
 CellTags->"b:3.1.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[95532, 2437, 169, 3, 48, "Input",ExpressionUUID->"212087b0-9807-4723-ba08-85686d31b9d7",
 CellTags->"b:3.1.5"],
Cell[95704, 2442, 348, 7, 48, "Output",ExpressionUUID->"ad13fe6c-d084-4a9a-9a43-d8713161e7fe",
 CellTags->"b:3.1.5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[96101, 2455, 247, 7, 60, "Subsection",ExpressionUUID->"289e9253-934d-4688-944c-597dffdb479f",
 CellTags->{"c:9", "b:3.1.6"}],
Cell[96351, 2464, 209, 5, 23, "Text",ExpressionUUID->"9157ff98-06d6-4397-9451-fefd9911320a",
 CellTags->"b:3.1.6"],
Cell[96563, 2471, 175, 4, 48, "Input",ExpressionUUID->"9dd58b55-04a6-4b7a-9081-e0cd144b2197",
 CellTags->"b:3.1.6"],
Cell[96741, 2477, 733, 21, 51, "Input",ExpressionUUID->"a9453a43-185a-476c-a22a-0af6ebe086fb",
 CellTags->"b:3.1.6"],
Cell[97477, 2500, 211, 5, 23, "Text",ExpressionUUID->"f536a44b-7cde-488d-8153-a60b2a606cf8",
 CellTags->"b:3.1.6"],
Cell[CellGroupData[{
Cell[97713, 2509, 858, 22, 87, "Input",ExpressionUUID->"b3d96486-217f-4f3c-9c15-ddca88a729b6",
 CellTags->"b:3.1.6"],
Cell[98574, 2533, 4007, 80, 227, "Output",ExpressionUUID->"84d70b42-f76e-423a-9896-5db6403c5ba9",
 CellTags->"b:3.1.6"],
Cell[102584, 2615, 2112, 52, 230, "Output",ExpressionUUID->"e1b1e22f-7e81-4840-9e09-2d08b84e06c4",
 CellTags->"b:3.1.6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[104745, 2673, 227, 6, 60, "Subsection",ExpressionUUID->"a28b1b85-2a1c-4bf8-a47f-2f7e35433a20",
 CellTags->{"c:10", "b:3.1.7"}],
Cell[104975, 2681, 625, 13, 136, "Text",ExpressionUUID->"f7c184aa-833a-4e13-941d-b0b1c1dbe8e1",
 CellTags->"b:3.1.7"],
Cell[105603, 2696, 144, 3, 48, "Input",ExpressionUUID->"41a74ad2-09df-4a4b-bda4-53bc22a0aaba",
 CellTags->"b:3.1.7"],
Cell[105750, 2701, 4274, 115, 288, "Input",ExpressionUUID->"09968d3f-b28f-4049-810d-5da4c38e3db0",
 CellTags->"b:3.1.7"],
Cell[CellGroupData[{
Cell[110049, 2820, 1071, 27, 144, "Input",ExpressionUUID->"ee37e1b2-be7a-46db-ab35-9931b666d5a6",
 CellTags->"b:3.1.7"],
Cell[111123, 2849, 816, 19, 64, "Output",ExpressionUUID->"7ed0d11e-2edb-4fd6-b4a2-31a1d8b854de",
 CellTags->"b:3.1.7"],
Cell[111942, 2870, 394, 8, 24, "Print",ExpressionUUID->"e9c6c664-481c-44aa-a3b0-1349371fa0f6",
 CellTags->"b:3.1.7"],
Cell[112339, 2880, 691, 14, 24, "Print",ExpressionUUID->"16ff7102-3cb5-45d0-9e1d-61136997fb88",
 CellTags->"b:3.1.7"],
Cell[113033, 2896, 339, 5, 24, "Print",ExpressionUUID->"6c7d3dcf-ab09-45d7-ae46-52dbb7301474",
 CellTags->"b:3.1.7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113409, 2906, 136, 2, 48, "Input",ExpressionUUID->"6423670b-1dcd-4cd3-9d9d-4446fb731ced",
 CellTags->"b:3.1.7"],
Cell[113548, 2910, 357, 7, 48, "Output",ExpressionUUID->"66aa6b4f-a1e5-4b71-a53d-4495d48bc65f",
 CellTags->"b:3.1.7"]
}, Open  ]],
Cell[113920, 2920, 319, 7, 79, "Text",ExpressionUUID->"9c2ff836-91fa-4545-8eaa-5ebfacc6f941",
 CellTags->"b:3.1.7"],
Cell[CellGroupData[{
Cell[114264, 2931, 468, 13, 75, "Input",ExpressionUUID->"c4bf62c2-3d0b-4462-b554-73d07a5767ce",
 CellTags->"b:3.1.7"],
Cell[114735, 2946, 1753, 46, 264, "Output",ExpressionUUID->"25fcd2c7-95c8-4f6f-967f-004e3b149d70",
 CellTags->"b:3.1.7"]
}, Open  ]],
Cell[116503, 2995, 98, 1, 48, "Input",ExpressionUUID->"2cfdf17c-21c1-452f-b9e1-fc9ee72b3423",
 CellTags->"b:3.1.7"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[116650, 3002, 184, 4, 51, "Section",ExpressionUUID->"24d13847-d1ae-4f2b-bce4-15298cc8b739",
 CellTags->"c:11"],
Cell[CellGroupData[{
Cell[116859, 3010, 238, 6, 60, "Subsection",ExpressionUUID->"aac51844-15b4-485e-86df-82617cf00316",
 CellTags->{"c:12", "b:3.2.0"}],
Cell[CellGroupData[{
Cell[117122, 3020, 188, 3, 46, "Input",ExpressionUUID->"7ed690ea-76a8-4af3-a937-3d3353901106",
 CellTags->"b:3.2.0"],
Cell[117313, 3025, 228, 3, 24, "Print",ExpressionUUID->"d8f96f63-fb5e-4053-a48d-87e2a44f804e",
 CellTags->"b:3.2.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[117578, 3033, 167, 3, 46, "Input",ExpressionUUID->"809f814a-6ec4-42c1-9af2-c6571e4692f0",
 CellTags->"b:3.2.0"],
Cell[117748, 3038, 55968, 1259, 493, "Output",ExpressionUUID->"aa0e8521-6ff2-4dd6-9990-d558bb3b574b",
 CellTags->"b:3.2.0"]
}, Open  ]],
Cell[173731, 4300, 221, 5, 23, "Text",ExpressionUUID->"43ec1d98-66a3-43cf-8fe2-cc272e5d61f6",
 CellTags->"b:3.2.0"],
Cell[173955, 4307, 1105, 34, 68, "Input",ExpressionUUID->"f0036fac-fccb-4bcb-85ee-d4ad0d046f54",
 CellTags->"b:3.2.0"],
Cell[175063, 4343, 2369, 59, 163, "Input",ExpressionUUID->"1e23594c-fde4-464e-ab54-8fd19efab08d",
 CellTags->"b:3.2.0"],
Cell[177435, 4404, 575, 14, 68, "Input",ExpressionUUID->"e8755a09-1536-4b1d-b029-ea5c5e6325e2",
 CellTags->"b:3.2.0"],
Cell[178013, 4420, 2329, 60, 163, "Input",ExpressionUUID->"e9c23b01-eff7-4f7c-84fd-b9205f03704b",
 CellTags->"b:3.2.0"],
Cell[180345, 4482, 576, 14, 68, "Input",ExpressionUUID->"d0cdb5e1-88f8-4b06-a0b3-46073347374d",
 CellTags->"b:3.2.0"],
Cell[180924, 4498, 307, 7, 51, "Text",ExpressionUUID->"42ca929b-57a0-444b-bfc3-60fbee7b1ad3",
 CellTags->"b:3.2.0"],
Cell[181234, 4507, 462, 13, 65, "Input",ExpressionUUID->"e0c55fff-f275-4acb-8fbc-b33125148bf7",
 CellTags->"b:3.2.0"],
Cell[181699, 4522, 167, 2, 23, "Text",ExpressionUUID->"796b1850-1712-4faa-acf4-f31ca425d71e",
 CellTags->"b:3.2.0"],
Cell[181869, 4526, 170, 4, 48, "Input",ExpressionUUID->"188c8c18-4b35-4253-8788-146aa2bd355f",
 CellTags->"b:3.2.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[182076, 4535, 264, 7, 60, "Subsection",ExpressionUUID->"ee450305-874f-4775-a17d-d467161ffbb5",
 CellTags->{"c:13", "b:3.2.1"}],
Cell[182343, 4544, 159, 2, 23, "Text",ExpressionUUID->"1277f93f-4321-4bda-8ce6-e066ab62c620",
 CellTags->"b:3.2.1"],
Cell[CellGroupData[{
Cell[182527, 4550, 151, 3, 48, "Input",ExpressionUUID->"0064d9bc-9c92-4916-a8f3-97442721d0a3",
 CellTags->"b:3.2.1"],
Cell[182681, 4555, 213, 5, 48, "Output",ExpressionUUID->"b3e68a6c-f29f-4843-b05f-c7208a726d0b",
 CellTags->"b:3.2.1"]
}, Open  ]],
Cell[182909, 4563, 174, 2, 23, "Text",ExpressionUUID->"70df8bf4-3406-4418-8c1f-c07ada681540",
 CellTags->"b:3.2.1"],
Cell[183086, 4567, 145, 3, 48, "Input",ExpressionUUID->"39a392c8-853f-4f12-96c8-5d643445308d",
 CellTags->"b:3.2.1"],
Cell[183234, 4572, 230, 5, 23, "Text",ExpressionUUID->"cbb2b275-bd49-4ea6-9453-af1d54658d39",
 CellTags->"b:3.2.1"],
Cell[183467, 4579, 382, 11, 48, "Input",ExpressionUUID->"cf1f548b-177a-4cd7-9f1b-a78601360609",
 CellTags->"b:3.2.1"],
Cell[183852, 4592, 332, 8, 48, "Input",ExpressionUUID->"5b676943-2e71-488f-8155-e16155f0ad7f",
 CellTags->"b:3.2.1"],
Cell[184187, 4602, 735, 19, 87, "Input",ExpressionUUID->"f18367b0-4334-422b-bb5d-d5fa993ea94b",
 CellTags->"b:3.2.1"],
Cell[184925, 4623, 1669, 48, 173, "Text",ExpressionUUID->"05789577-c6ec-4b56-ade0-c770b29a92df",
 CellTags->"b:3.2.1"],
Cell[CellGroupData[{
Cell[186619, 4675, 270, 7, 48, "Input",ExpressionUUID->"d6fb0edd-4b20-4244-a877-3dbf510cd72c",
 CellTags->"b:3.2.1"],
Cell[186892, 4684, 213, 5, 48, "Output",ExpressionUUID->"daaa0c1e-6d1e-4492-bcf5-e20df6da9bcd",
 CellTags->"b:3.2.1"]
}, Open  ]],
Cell[187120, 4692, 397, 10, 25, "Text",ExpressionUUID->"335e8ea3-2873-44d9-b1a4-7a5b8c264732",
 CellTags->"b:3.2.1"],
Cell[CellGroupData[{
Cell[187542, 4706, 756, 20, 125, "Input",ExpressionUUID->"f1cc2945-4e2e-4e68-9bf6-eb7aaa7e8756",
 CellTags->"b:3.2.1"],
Cell[188301, 4728, 209, 5, 48, "Output",ExpressionUUID->"33425887-852b-44b0-95cd-4f7a4942c583",
 CellTags->"b:3.2.1"],
Cell[188513, 4735, 211, 5, 48, "Output",ExpressionUUID->"53aa7c81-2544-49c3-8218-82071356b912",
 CellTags->"b:3.2.1"],
Cell[188727, 4742, 209, 5, 48, "Output",ExpressionUUID->"10011bf6-248a-4a3c-a25d-36a6fa332b23",
 CellTags->"b:3.2.1"]
}, Open  ]],
Cell[188951, 4750, 304, 7, 51, "Text",ExpressionUUID->"eb6b992e-8347-460c-b83d-6b917f3d28f6",
 CellTags->"b:3.2.1"],
Cell[CellGroupData[{
Cell[189280, 4761, 434, 12, 48, "Input",ExpressionUUID->"edab14a3-a909-4841-bbfb-7663f5a5a735",
 CellTags->"b:3.2.1"],
Cell[189717, 4775, 252, 3, 24, "Print",ExpressionUUID->"42bc22b7-3e0a-46ed-b274-a20c434bec81",
 CellTags->"b:3.2.1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[190018, 4784, 469, 13, 62, "Subsection",ExpressionUUID->"b21b802a-e344-4045-9440-8754a9b40edf",
 CellTags->{"c:14", "b:3.2.2"}],
Cell[190490, 4799, 1067, 31, 97, "Text",ExpressionUUID->"2f7d1a0f-0728-490d-a43d-fdb912c228a9",
 CellTags->"b:3.2.2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[191594, 4835, 585, 17, 65, "Subsection",ExpressionUUID->"7eaa4595-dff0-4dc2-be10-dab949968968",
 CellTags->{"c:15", "b:3.2.3"}],
Cell[192182, 4854, 154, 2, 23, "Text",ExpressionUUID->"553878d0-20c0-4d41-a2f1-b451599dbdfb",
 CellTags->"b:3.2.3"],
Cell[CellGroupData[{
Cell[192361, 4860, 554, 16, 48, "Input",ExpressionUUID->"9f0e4179-0a52-4597-99dc-bf4b814f9208",
 CellTags->"b:3.2.3"],
Cell[192918, 4878, 213, 5, 48, "Output",ExpressionUUID->"5921de34-eda2-4ae7-b705-17e1fc854e6f",
 CellTags->"b:3.2.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193168, 4888, 132, 2, 48, "Input",ExpressionUUID->"717d66cb-f527-488b-8d26-7ba1be80c1a1",
 CellTags->"b:3.2.3"],
Cell[193303, 4892, 210, 5, 48, "Output",ExpressionUUID->"483f3aac-fd15-48b2-8547-d32b192396fe",
 CellTags->"b:3.2.3"]
}, Open  ]],
Cell[193528, 4900, 1380, 42, 63, "Text",ExpressionUUID->"1b942e64-f0ef-4a8a-b57e-c8ff4e8265fc",
 CellTags->"b:3.2.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[194945, 4947, 207, 4, 60, "Subsection",ExpressionUUID->"12cde5ef-8089-4818-ad4d-98f516b5bfa3",
 CellTags->{"c:16", "b:3.2.4"}],
Cell[195155, 4953, 468, 11, 55, "Text",ExpressionUUID->"2974a720-8c21-403c-9df2-f379891ace8a",
 CellTags->"b:3.2.4"],
Cell[CellGroupData[{
Cell[195648, 4968, 418, 12, 60, "Input",ExpressionUUID->"38b54cd7-ffbe-4061-b025-2315bca1f3f5",
 CellTags->"b:3.2.4"],
Cell[196069, 4982, 212, 5, 48, "Output",ExpressionUUID->"7fa3010c-5136-438c-8ee2-6b289c0268b7",
 CellTags->"b:3.2.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196318, 4992, 370, 10, 87, "Input",ExpressionUUID->"b384365a-429e-4db7-825b-83d26a561649",
 CellTags->"b:3.2.4"],
Cell[196691, 5004, 211, 5, 48, "Output",ExpressionUUID->"e5b32156-a616-4788-b07a-3c5df7baf38b",
 CellTags->"b:3.2.4"],
Cell[196905, 5011, 211, 5, 48, "Output",ExpressionUUID->"934d2906-c1b7-4072-b879-4745c9fbd08e",
 CellTags->"b:3.2.4"],
Cell[197119, 5018, 214, 5, 48, "Output",ExpressionUUID->"1fa36f6c-3f15-4d8b-b9a1-4f94a3205cc0",
 CellTags->"b:3.2.4"]
}, Open  ]],
Cell[197348, 5026, 596, 18, 25, "Text",ExpressionUUID->"0d71df29-876e-416b-b38a-362fc8bc158d",
 CellTags->"b:3.2.4"],
Cell[197947, 5046, 501, 14, 48, "Input",ExpressionUUID->"25559b2e-086c-4304-a916-0ce52db6f5a9",
 CellTags->"b:3.2.4"],
Cell[198451, 5062, 400, 11, 25, "Text",ExpressionUUID->"be63e8d6-0bcf-4559-b4c6-3e40e92a167d",
 CellTags->"b:3.2.4"],
Cell[198854, 5075, 1109, 34, 93, "Input",ExpressionUUID->"ae252cbc-516d-45a3-b589-0385ca4ed2f9",
 CellTags->"b:3.2.4"],
Cell[199966, 5111, 189, 4, 23, "Text",ExpressionUUID->"a2ce1843-6c3d-49ba-abc2-e32f635779a9",
 CellTags->"b:3.2.4"],
Cell[200158, 5117, 819, 25, 87, "Input",ExpressionUUID->"d1dc27a2-c27a-43eb-bd8f-e8db44fe18ca",
 CellTags->"b:3.2.4"],
Cell[200980, 5144, 136, 2, 23, "Text",ExpressionUUID->"a169e8d9-6724-4f5d-8a01-041fb60846a9",
 CellTags->"b:3.2.4"],
Cell[201119, 5148, 153, 3, 48, "Input",ExpressionUUID->"79576aff-ff3a-4eba-986f-d0a506d1b9f0",
 CellTags->"b:3.2.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201309, 5156, 201, 4, 60, "Subsection",ExpressionUUID->"e2b44cc5-4a59-474f-99a5-766e9b3e0cce",
 CellTags->{"c:17", "b:3.2.5"}],
Cell[201513, 5162, 215, 5, 23, "Text",ExpressionUUID->"26d2857a-d9db-4e3d-b0bf-ad46b54832fb",
 CellTags->"b:3.2.5"],
Cell[201731, 5169, 182, 4, 48, "Input",ExpressionUUID->"0ab15e2f-cff2-41c1-81b4-edb1378861e1",
 CellTags->"b:3.2.5"],
Cell[CellGroupData[{
Cell[201938, 5177, 554, 12, 87, "Input",ExpressionUUID->"7277ef25-d376-49bc-9ceb-cfd5aa629a2f"],
Cell[202495, 5191, 192, 6, 48, "Output",ExpressionUUID->"8e4fa450-6297-4a76-be09-b860eb83e49e"],
Cell[202690, 5199, 715, 27, 182, "Output",ExpressionUUID->"25794dba-9fff-4d97-a479-274f1610a704"],
Cell[203408, 5228, 62502, 1552, 8734, "Output",ExpressionUUID->"5dd70b23-0876-411c-9d1e-556a3268fcfd"]
}, Open  ]],
Cell[CellGroupData[{
Cell[265947, 6785, 968, 26, 87, "Input",ExpressionUUID->"71a9281e-1a08-42a4-bf13-5cd629a8f8d3",
 CellTags->"b:3.2.5"],
Cell[266918, 6813, 413, 9, 24, "Print",ExpressionUUID->"1ffdbfb5-8eb0-4056-9e08-907f495b602e",
 CellTags->"b:3.2.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[267368, 6827, 346, 8, 68, "Input",ExpressionUUID->"c36e697b-51e7-4fdf-93f7-98e1bdc0b2b5",
 CellTags->"b:3.2.5"],
Cell[267717, 6837, 387, 7, 24, "Print",ExpressionUUID->"3f97b5cc-cc36-4018-a63f-c610d5f1532b",
 CellTags->"b:3.2.5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[268153, 6850, 248, 7, 60, "Subsection",ExpressionUUID->"ddd77bd5-ee92-4f00-bc30-a05161b1d4cb",
 CellTags->{"c:18", "b:3.2.6"}],
Cell[268404, 6859, 209, 5, 23, "Text",ExpressionUUID->"80d9d151-4ffe-418a-ab5a-48ace74cac97",
 CellTags->"b:3.2.6"],
Cell[268616, 6866, 175, 4, 48, "Input",ExpressionUUID->"691b62c1-45d8-4356-94ca-fcca56f24d50",
 CellTags->"b:3.2.6"],
Cell[268794, 6872, 805, 23, 51, "Input",ExpressionUUID->"b23662eb-3dc3-4197-b052-917149eb8671",
 CellTags->"b:3.2.6"],
Cell[269602, 6897, 211, 5, 23, "Text",ExpressionUUID->"d8c92dbc-2b52-4dfb-a9f8-044e215b0c4e",
 CellTags->"b:3.2.6"],
Cell[CellGroupData[{
Cell[269838, 6906, 858, 22, 87, "Input",ExpressionUUID->"13e27a7f-7b84-4a33-8e0b-09cd7ccfd29c",
 CellTags->"b:3.2.6"],
Cell[270699, 6930, 8310, 159, 219, "Output",ExpressionUUID->"1dbd491e-690a-4984-87a5-8ede684b0365",
 CellTags->"b:3.2.6"],
Cell[279012, 7091, 6112, 118, 230, "Output",ExpressionUUID->"a042b52f-4d3f-4228-b66a-b43061007647",
 CellTags->"b:3.2.6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[285173, 7215, 227, 6, 60, "Subsection",ExpressionUUID->"e624a1be-c636-4105-bfe5-d1a2feea774f",
 CellTags->{"c:19", "b:3.2.7"}],
Cell[285403, 7223, 625, 13, 136, "Text",ExpressionUUID->"53f4d29d-2f8a-425c-a2e4-60f1e617e6b1",
 CellTags->"b:3.2.7"],
Cell[286031, 7238, 144, 3, 48, "Input",ExpressionUUID->"a58fd27c-b350-4aef-9026-c515f1fe64ec",
 CellTags->"b:3.2.7"],
Cell[286178, 7243, 4274, 115, 288, "Input",ExpressionUUID->"694abdfc-0b09-4f1b-b2af-ba0ccca3454a",
 CellTags->"b:3.2.7"],
Cell[CellGroupData[{
Cell[290477, 7362, 822, 20, 125, "Input",ExpressionUUID->"9ea8d0bf-0b54-4f09-a85b-4ca0f0de669a",
 CellTags->"b:3.2.7"],
Cell[291302, 7384, 917, 21, 132, "Output",ExpressionUUID->"1673fbea-2472-4523-98d9-f2d97d3677df",
 CellTags->"b:3.2.7"],
Cell[292222, 7407, 273, 6, 24, "Print",ExpressionUUID->"ae84e901-6925-46ed-877b-a7bcfd16ce21",
 CellTags->"b:3.2.7"],
Cell[292498, 7415, 605, 12, 24, "Print",ExpressionUUID->"bf7df94e-6b48-4877-8581-5469ee4f047b",
 CellTags->"b:3.2.7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[293140, 7432, 136, 2, 48, "Input",ExpressionUUID->"ded7f919-5894-4236-a13d-f453620e6bf4",
 CellTags->"b:3.2.7"],
Cell[293279, 7436, 235, 5, 48, "Output",ExpressionUUID->"90a47588-f8ca-4a79-8719-d5a3de8b4d32",
 CellTags->"b:3.2.7"]
}, Open  ]],
Cell[293529, 7444, 319, 7, 79, "Text",ExpressionUUID->"7111c831-3093-4a08-84dc-432a63ded561",
 CellTags->"b:3.2.7"],
Cell[CellGroupData[{
Cell[293873, 7455, 468, 13, 75, "Input",ExpressionUUID->"b2ecdeee-b04b-4f5c-9c9c-2a84fd795290",
 CellTags->"b:3.2.7"],
Cell[294344, 7470, 1637, 44, 259, "Output",ExpressionUUID->"d3e7fe35-788f-4b5c-bc2c-d420e1e22685",
 CellTags->"b:3.2.7"]
}, Open  ]],
Cell[295996, 7517, 98, 1, 48, "Input",ExpressionUUID->"af3eb7f0-c3c0-4018-8dc7-edf49c603f86",
 CellTags->"b:3.2.7"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[296143, 7524, 260, 6, 31, "Section",ExpressionUUID->"342b6cff-8d04-4829-b39a-30888a1b6349",
 CellTags->"c:20"],
Cell[CellGroupData[{
Cell[296428, 7534, 330, 10, 60, "Subsection",ExpressionUUID->"4fe22e9c-e4f8-49a9-9be9-52d1f03d8739",
 CellTags->{"c:21", "b:3.3.0"}],
Cell[CellGroupData[{
Cell[296783, 7548, 188, 3, 46, "Input",ExpressionUUID->"c2fd9143-1605-4192-a843-f52d21f90686",
 CellTags->"b:3.3.0"],
Cell[296974, 7553, 228, 3, 24, "Print",ExpressionUUID->"a817e1e6-cf02-46c6-b909-79652efa790c",
 CellTags->"b:3.3.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[297239, 7561, 167, 3, 46, "Input",ExpressionUUID->"c4e2ef04-0253-4a51-890d-7a0b90f5cc16",
 CellTags->"b:3.3.0"],
Cell[297409, 7566, 55970, 1259, 493, "Output",ExpressionUUID->"0cd6f6d9-8b9d-4c1f-b5ff-edc1b03ea79c",
 CellTags->"b:3.3.0"]
}, Open  ]],
Cell[353394, 8828, 194, 4, 23, "Text",ExpressionUUID->"bc0cd033-3aaa-4057-8d66-7d53c8450df9",
 CellTags->"b:3.3.0"],
Cell[353591, 8834, 1105, 34, 68, "Input",ExpressionUUID->"7e6edb2f-271f-44d6-8557-6e8ef8678a3d",
 CellTags->"b:3.3.0"],
Cell[354699, 8870, 2388, 59, 163, "Input",ExpressionUUID->"171c8b0d-1e2c-45d2-b4af-3d9680453c2a",
 CellTags->"b:3.3.0"],
Cell[357090, 8931, 575, 14, 68, "Input",ExpressionUUID->"198be410-ca96-4b49-9f66-44022697997d",
 CellTags->"b:3.3.0"],
Cell[357668, 8947, 2368, 60, 163, "Input",ExpressionUUID->"e76c7614-569c-4eb2-a92a-d20b6496be74",
 CellTags->"b:3.3.0"],
Cell[360039, 9009, 648, 15, 68, "Input",ExpressionUUID->"077d92a3-f006-465d-880e-8cdd676fecc6",
 CellTags->"b:3.3.0"],
Cell[360690, 9026, 315, 6, 51, "Text",ExpressionUUID->"7bb36b78-356c-40cc-9073-18c80b475843",
 CellTags->"b:3.3.0"],
Cell[361008, 9034, 462, 13, 65, "Input",ExpressionUUID->"c6f014f8-9677-4c13-a45c-3600fba54ab1",
 CellTags->"b:3.3.0"],
Cell[361473, 9049, 167, 2, 23, "Text",ExpressionUUID->"377ac93b-27c8-4407-acee-7c8901cf8efc",
 CellTags->"b:3.3.0"],
Cell[361643, 9053, 170, 4, 48, "Input",ExpressionUUID->"92d20556-4222-4574-9912-38a3d8b0565d",
 CellTags->"b:3.3.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361850, 9062, 272, 7, 60, "Subsection",ExpressionUUID->"75a1a666-9231-472e-a60a-556d78499aec",
 CellTags->{"c:22", "b:3.3.1"}],
Cell[362125, 9071, 162, 2, 23, "Text",ExpressionUUID->"bc6c8017-a8d6-4071-9b96-76a0845145b6",
 CellTags->"b:3.3.1"],
Cell[362290, 9075, 145, 3, 48, "Input",ExpressionUUID->"3eaa8960-b60b-43d3-86bd-7dec3d67f698",
 CellTags->"b:3.3.1"],
Cell[362438, 9080, 174, 2, 23, "Text",ExpressionUUID->"c51200c3-cc31-49ff-ad87-d83cd98b455f",
 CellTags->"b:3.3.1"],
Cell[362615, 9084, 145, 3, 48, "Input",ExpressionUUID->"0f0b1380-8ed9-403f-acdb-c98014cbcfc5",
 CellTags->"b:3.3.1"],
Cell[362763, 9089, 230, 5, 23, "Text",ExpressionUUID->"5addb373-3c99-403a-8fd4-0e6acf1858ed",
 CellTags->"b:3.3.1"],
Cell[362996, 9096, 356, 10, 48, "Input",ExpressionUUID->"a3de0cd8-1787-477a-9d4b-65eeba4e97da",
 CellTags->"b:3.3.1"],
Cell[363355, 9108, 735, 19, 87, "Input",ExpressionUUID->"e91a2e55-c340-401c-8bf3-a601d91bf15a",
 CellTags->"b:3.3.1"],
Cell[CellGroupData[{
Cell[364115, 9131, 270, 7, 48, "Input",ExpressionUUID->"dfd7d6f3-7933-4af1-89eb-d12597718341",
 CellTags->"b:3.3.1"],
Cell[364388, 9140, 211, 5, 48, "Output",ExpressionUUID->"68bbec5e-2cb3-403b-a17a-9052f8e96bde",
 CellTags->"b:3.3.1"]
}, Open  ]],
Cell[364614, 9148, 832, 23, 111, "Text",ExpressionUUID->"ac05b555-7280-4b0d-9072-708fb8846e53",
 CellTags->"b:3.3.1"],
Cell[CellGroupData[{
Cell[365471, 9175, 568, 16, 106, "Input",ExpressionUUID->"3938e008-044f-49f9-830e-0bb45b68835d",
 CellTags->"b:3.3.1"],
Cell[366042, 9193, 209, 5, 48, "Output",ExpressionUUID->"67804c97-742d-49b3-82f1-c162318ab594",
 CellTags->"b:3.3.1"],
Cell[366254, 9200, 209, 5, 48, "Output",ExpressionUUID->"292b2530-9e5b-40df-b3d6-ca8506917915",
 CellTags->"b:3.3.1"]
}, Open  ]],
Cell[366478, 9208, 303, 7, 51, "Text",ExpressionUUID->"9ed2b209-50f8-433a-84ff-2ee340dac26a",
 CellTags->"b:3.3.1"],
Cell[366784, 9217, 434, 12, 48, "Input",ExpressionUUID->"428e9b2f-f116-4abc-ac21-5cb707575033",
 CellTags->"b:3.3.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[367255, 9234, 469, 13, 62, "Subsection",ExpressionUUID->"0e43a654-d0e3-4044-b2f0-dbd29c60d5a1",
 CellTags->{"c:23", "b:3.3.2"}],
Cell[367727, 9249, 861, 25, 67, "Text",ExpressionUUID->"ce708b63-423b-4bde-9355-ca1ae75e8830",
 CellTags->"b:3.3.2"],
Cell[CellGroupData[{
Cell[368613, 9278, 558, 18, 68, "Input",ExpressionUUID->"542ec1d5-c543-4d75-b997-1d0742a06253",
 CellTags->"b:3.3.2"],
Cell[369174, 9298, 209, 5, 48, "Output",ExpressionUUID->"5f4d811b-0190-41e1-9487-f58a720d4c51",
 CellTags->"b:3.3.2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[369432, 9309, 585, 17, 65, "Subsection",ExpressionUUID->"5a1b48ea-7940-4a0c-93ea-25e918710549",
 CellTags->{"c:24", "b:3.3.3"}],
Cell[370020, 9328, 173, 2, 23, "Text",ExpressionUUID->"a55f7a7e-6341-49e3-aae9-c51f64ade20d",
 CellTags->"b:3.3.3"],
Cell[CellGroupData[{
Cell[370218, 9334, 554, 16, 48, "Input",ExpressionUUID->"74329215-36c5-41dc-8ef3-50537c53856b",
 CellTags->"b:3.3.3"],
Cell[370775, 9352, 210, 5, 48, "Output",ExpressionUUID->"60087992-bd7e-4127-b6d3-5f7df6d3ebe2",
 CellTags->"b:3.3.3"]
}, Open  ]],
Cell[371000, 9360, 211, 5, 51, "Text",ExpressionUUID->"b2a0d65d-7b23-4347-994d-df493e99c8b6",
 CellTags->"b:3.3.3"],
Cell[371214, 9367, 1279, 41, 90, "Input",ExpressionUUID->"f7f641d1-bc4e-41d1-8802-482b03510c28",
 CellTags->"b:3.3.3"],
Cell[CellGroupData[{
Cell[372518, 9412, 322, 7, 68, "Input",ExpressionUUID->"63221bfe-763f-4efa-98bd-303157724000",
 CellTags->"b:3.3.3"],
Cell[372843, 9421, 69747, 1162, 254, "Output",ExpressionUUID->"091c98e8-8d48-49e3-b34d-96d553d9e399",
 CellTags->"b:3.3.3"]
}, Open  ]],
Cell[442605, 10586, 161, 2, 23, "Text",ExpressionUUID->"57544e27-7ff3-4d46-8a94-65fe0263ea05",
 CellTags->"b:3.3.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[442803, 10593, 187, 4, 60, "Subsection",ExpressionUUID->"01673a7b-f377-4d2a-933e-90abcb15638f",
 CellTags->{"c:25", "b:3.3.4"}],
Cell[442993, 10599, 271, 6, 51, "Text",ExpressionUUID->"826f2ec6-3675-46dd-aff0-c5f0b2feae57",
 CellTags->"b:3.3.4"],
Cell[CellGroupData[{
Cell[443289, 10609, 418, 12, 60, "Input",ExpressionUUID->"a805bfb9-0273-4bdc-af45-80e6825e7c2f",
 CellTags->"b:3.3.4"],
Cell[443710, 10623, 210, 5, 48, "Output",ExpressionUUID->"61815ec8-745e-44f3-ab96-2cde0786d2ec",
 CellTags->"b:3.3.4"]
}, Open  ]],
Cell[443935, 10631, 313, 8, 23, "Text",ExpressionUUID->"aff5676d-c965-4ea6-9c5e-a55804bc80a1",
 CellTags->"b:3.3.4"],
Cell[CellGroupData[{
Cell[444273, 10643, 791, 23, 87, "Input",ExpressionUUID->"be6ec06f-42ab-4307-9ed7-a45365c9cb53",
 CellTags->"b:3.3.4"],
Cell[445067, 10668, 699, 23, 132, "Output",ExpressionUUID->"243cc6b6-cbe9-4ed1-a27c-7e8fcda02a6a",
 CellTags->"b:3.3.4"],
Cell[445769, 10693, 693, 23, 132, "Output",ExpressionUUID->"c8adf71f-3895-44fb-a1d2-42cf6292f19f",
 CellTags->"b:3.3.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446499, 10721, 283, 7, 68, "Input",ExpressionUUID->"343c5077-25a9-4897-896c-e8b5e414b5df",
 CellTags->"b:3.3.4"],
Cell[446785, 10730, 209, 5, 48, "Output",ExpressionUUID->"a50b030c-d242-4a4e-b547-f79633715e74",
 CellTags->"b:3.3.4"],
Cell[446997, 10737, 2715, 62, 254, "Output",ExpressionUUID->"a4cc93c1-779b-4ee2-97ee-11c9b00d1036",
 CellTags->"b:3.3.4"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[449761, 10805, 196, 4, 60, "Subsection",ExpressionUUID->"7c24ac23-2009-4b56-aa2e-336e4ba30209",
 CellTags->{"c:26", "b:3.3.5"}],
Cell[449960, 10811, 353, 10, 25, "Text",ExpressionUUID->"e3990e54-8488-46f9-b5c9-f8b7644c9bb0",
 CellTags->"b:3.3.5"],
Cell[450316, 10823, 182, 4, 48, "Input",ExpressionUUID->"240a7c30-dd95-41ad-85da-b1f8816e1a43",
 CellTags->"b:3.3.5"],
Cell[CellGroupData[{
Cell[450523, 10831, 960, 27, 89, "Input",ExpressionUUID->"c086152a-9f6f-4ad4-af89-02d73a44d001",
 CellTags->"b:3.3.5"],
Cell[451486, 10860, 211, 5, 48, "Output",ExpressionUUID->"6b514fbd-8031-473d-a711-93a0cec748c8",
 CellTags->"b:3.3.5"]
}, Open  ]],
Cell[451712, 10868, 182, 4, 48, "Input",ExpressionUUID->"e21b57c6-a08b-4017-9c75-7c81905be74a",
 CellTags->"b:3.3.5"],
Cell[CellGroupData[{
Cell[451919, 10876, 235, 4, 48, "Input",ExpressionUUID->"0563c3d8-b7e5-4ea6-a6f6-9886fa08c85d",
 CellTags->"b:3.3.5"],
Cell[452157, 10882, 390, 7, 24, "Print",ExpressionUUID->"e3ecae34-1fec-48bf-aefc-3ca643621745",
 CellTags->"b:3.3.5"]
}, Open  ]],
Cell[452562, 10892, 198, 4, 23, "Text",ExpressionUUID->"bd1bf6e7-69a2-408c-82f3-7553b09337c7",
 CellTags->"b:3.3.5"],
Cell[452763, 10898, 175, 4, 48, "Input",ExpressionUUID->"4bdce0bf-05bb-49be-83ab-788348d94d1d",
 CellTags->"b:3.3.5"],
Cell[452941, 10904, 805, 23, 51, "Input",ExpressionUUID->"a21e999c-5d12-4918-b2f3-dc36e0c8e8d8",
 CellTags->"b:3.3.5"],
Cell[CellGroupData[{
Cell[453771, 10931, 551, 15, 87, "Input",ExpressionUUID->"f75c87f8-51f9-4542-a676-d1ba0a480785",
 CellTags->"b:3.3.5"],
Cell[454325, 10948, 226, 5, 48, "Output",ExpressionUUID->"9db99c1f-e3d2-482d-9d0d-faee1a4246f2",
 CellTags->"b:3.3.5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[454588, 10958, 865, 23, 87, "Input",ExpressionUUID->"b18139f7-9804-4806-b169-4e1114f219bb",
 CellTags->"b:3.3.5"],
Cell[455456, 10983, 8314, 158, 214, "Output",ExpressionUUID->"98548c40-7051-4cb4-a49d-63860f5ff0aa",
 CellTags->"b:3.3.5"],
Cell[463773, 11143, 7235, 137, 226, "Output",ExpressionUUID->"68048384-4d28-4dc3-be8d-b911c3697ab9",
 CellTags->"b:3.3.5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[471057, 11286, 256, 7, 60, "Subsection",ExpressionUUID->"da7869f9-7d7c-4587-91ba-cc7e24024f8e",
 CellTags->{"c:27", "b:3.3.6"}],
Cell[471316, 11295, 256, 6, 51, "Text",ExpressionUUID->"57c0bbfd-1392-4b3a-a5b9-9bda4650de71",
 CellTags->"b:3.3.6"],
Cell[CellGroupData[{
Cell[471597, 11305, 109, 1, 48, "Input",ExpressionUUID->"37c3e03e-ea93-43c1-a115-c7ef670c6446",
 CellTags->"b:3.3.6"],
Cell[471709, 11308, 212, 5, 48, "Output",ExpressionUUID->"2b2f7218-96e7-48df-8621-7c6f86f890ed",
 CellTags->"b:3.3.6"]
}, Open  ]],
Cell[471936, 11316, 144, 3, 48, "Input",ExpressionUUID->"d77df3a4-0dda-4c2f-bbe4-15048556c0af",
 CellTags->"b:3.3.6"],
Cell[472083, 11321, 4274, 115, 288, "Input",ExpressionUUID->"e133b724-ace1-4387-ad6e-30b4598866d1",
 CellTags->"b:3.3.6"],
Cell[CellGroupData[{
Cell[476382, 11440, 721, 18, 106, "Input",ExpressionUUID->"6e2edc63-0cff-491b-b526-62b7711b89de",
 CellTags->"b:3.3.6"],
Cell[477106, 11460, 1445, 31, 302, "Output",ExpressionUUID->"a170edfd-4f1a-464f-8a56-4b93dd632905",
 CellTags->"b:3.3.6"],
Cell[478554, 11493, 607, 12, 24, "Print",ExpressionUUID->"ca13c08e-1ebf-4394-b021-10a70b8a224a",
 CellTags->"b:3.3.6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[479198, 11510, 136, 2, 48, "Input",ExpressionUUID->"0c26bbdb-24d2-4a49-ace0-1549096fe8cd",
 CellTags->"b:3.3.6"],
Cell[479337, 11514, 234, 5, 48, "Output",ExpressionUUID->"632b187a-5b42-4522-8afe-3e88fb680420",
 CellTags->"b:3.3.6"]
}, Open  ]],
Cell[479586, 11522, 107, 1, 48, "Input",ExpressionUUID->"4e8a88c3-d0c8-4560-b680-35bb9930e6ea",
 CellTags->"b:3.3.6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[479742, 11529, 229, 7, 31, "Section",ExpressionUUID->"13fd45fe-183f-48a7-b25c-9ed750d8b40d",
 CellTags->"c:28"],
Cell[CellGroupData[{
Cell[479996, 11540, 238, 6, 60, "Subsection",ExpressionUUID->"fa752dc4-6539-4dda-9b31-ef8167f375d1",
 CellTags->{"c:29", "b:3.4.0"}],
Cell[CellGroupData[{
Cell[480259, 11550, 188, 3, 46, "Input",ExpressionUUID->"db8fee6a-8830-4d13-a934-10f86c1f3716",
 CellTags->"b:3.4.0"],
Cell[480450, 11555, 227, 3, 24, "Print",ExpressionUUID->"17059a87-af8c-4524-8811-404757c4c88c",
 CellTags->"b:3.4.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[480714, 11563, 167, 3, 46, "Input",ExpressionUUID->"6be6dcfe-df44-4c6f-b2ba-643ae1a1844a",
 CellTags->"b:3.4.0"],
Cell[480884, 11568, 55968, 1259, 493, "Output",ExpressionUUID->"752c25da-ce10-4c47-a188-36bbf1a10539",
 CellTags->"b:3.4.0"]
}, Open  ]],
Cell[536867, 12830, 223, 5, 23, "Text",ExpressionUUID->"690a4ac2-1826-42c2-bc24-d918af016df9",
 CellTags->"b:3.4.0"],
Cell[537093, 12837, 307, 7, 51, "Text",ExpressionUUID->"00f81e3c-9822-4a10-8d8a-c2aa293af7c2",
 CellTags->"b:3.4.0"],
Cell[537403, 12846, 462, 13, 65, "Input",ExpressionUUID->"ce6b1d7f-de2d-4661-aa19-7de144df4975",
 CellTags->"b:3.4.0"],
Cell[537868, 12861, 167, 2, 23, "Text",ExpressionUUID->"d4e65207-d148-4f77-8af6-7d37c0e23612",
 CellTags->"b:3.4.0"],
Cell[538038, 12865, 170, 4, 48, "Input",ExpressionUUID->"6b908e59-b062-41bf-9f5c-a4c3954841d9",
 CellTags->"b:3.4.0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[538245, 12874, 264, 7, 60, "Subsection",ExpressionUUID->"40b0c817-b042-450e-8a68-59611963dd0a",
 CellTags->{"c:30", "b:3.4.1"}],
Cell[538512, 12883, 159, 2, 23, "Text",ExpressionUUID->"65476cea-fc27-466e-b37d-56df137ec07f",
 CellTags->"b:3.4.1"],
Cell[CellGroupData[{
Cell[538696, 12889, 150, 3, 48, "Input",ExpressionUUID->"8b33a7d1-c0a1-4dce-a927-f7a260b48888",
 CellTags->"b:3.4.1"],
Cell[538849, 12894, 210, 5, 48, "Output",ExpressionUUID->"2ef3ed32-9aff-4e56-bf06-4cbc7c3bc8ed",
 CellTags->"b:3.4.1"]
}, Open  ]],
Cell[539074, 12902, 174, 2, 23, "Text",ExpressionUUID->"c9562f80-d03b-4483-8912-d1e88dfa9b0d",
 CellTags->"b:3.4.1"],
Cell[539251, 12906, 145, 3, 48, "Input",ExpressionUUID->"dcf258a9-23ac-4c26-83dd-d4cab8ebc184",
 CellTags->"b:3.4.1"],
Cell[539399, 12911, 212, 5, 23, "Text",ExpressionUUID->"895b6e93-24d7-491d-a4d5-3027e52c3ffb",
 CellTags->"b:3.4.1"],
Cell[539614, 12918, 382, 11, 48, "Input",ExpressionUUID->"999cc834-bf63-4ae8-9c00-e52f7e5daa6f",
 CellTags->"b:3.4.1"],
Cell[539999, 12931, 332, 8, 48, "Input",ExpressionUUID->"060ec69a-c967-4e67-9d74-5dcd22e5376d",
 CellTags->"b:3.4.1"],
Cell[540334, 12941, 735, 19, 87, "Input",ExpressionUUID->"b8c78657-830f-4660-95c7-aa419f101d3b",
 CellTags->"b:3.4.1"],
Cell[541072, 12962, 1662, 47, 173, "Text",ExpressionUUID->"de9b52f6-1820-44d5-b767-072adf4daf93",
 CellTags->"b:3.4.1"],
Cell[CellGroupData[{
Cell[542759, 13013, 270, 7, 48, "Input",ExpressionUUID->"f4a53142-a969-4623-9d81-eb839c5f7651",
 CellTags->"b:3.4.1"],
Cell[543032, 13022, 212, 5, 48, "Output",ExpressionUUID->"56cadb2b-bcf1-401b-94b4-69638b0b3295",
 CellTags->"b:3.4.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[543281, 13032, 519, 14, 106, "Input",ExpressionUUID->"e7a9ae3e-8bbe-43a9-96da-071d42450322",
 CellTags->"b:3.4.1"],
Cell[543803, 13048, 209, 5, 48, "Output",ExpressionUUID->"c575adfe-2eed-4b60-9e2c-b5953d44c001",
 CellTags->"b:3.4.1"],
Cell[544015, 13055, 211, 5, 48, "Output",ExpressionUUID->"ebee835a-17ab-4d42-8d82-5e6664d0a36d",
 CellTags->"b:3.4.1"],
Cell[544229, 13062, 210, 5, 48, "Output",ExpressionUUID->"168c94dd-a4e1-4f21-8de5-bc60d0c0539d",
 CellTags->"b:3.4.1"],
Cell[544442, 13069, 209, 5, 48, "Output",ExpressionUUID->"1db8c5d5-f630-4e04-bfb3-b2c2a88455ff",
 CellTags->"b:3.4.1"]
}, Open  ]],
Cell[544666, 13077, 303, 7, 51, "Text",ExpressionUUID->"456b650f-390c-4a4f-b37b-46bce1bfd17c",
 CellTags->"b:3.4.1"],
Cell[544972, 13086, 434, 12, 48, "Input",ExpressionUUID->"ec5f04cd-9377-4837-a62d-9edf9333f31c",
 CellTags->"b:3.4.1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[545443, 13103, 469, 13, 62, "Subsection",ExpressionUUID->"d119ab91-7c89-498d-b356-84531733b861",
 CellTags->{"c:31", "b:3.4.2"}],
Cell[545915, 13118, 1110, 32, 97, "Text",ExpressionUUID->"289a1164-3337-4f7b-831d-4dfe47842b31",
 CellTags->"b:3.4.2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[547062, 13155, 585, 17, 65, "Subsection",ExpressionUUID->"8278dbd4-1671-4f09-9f3d-4542bf1a7152",
 CellTags->{"c:32", "b:3.4.3"}],
Cell[547650, 13174, 154, 2, 23, "Text",ExpressionUUID->"e6fb4063-dc12-4577-9f5e-9afb37053f46",
 CellTags->"b:3.4.3"],
Cell[CellGroupData[{
Cell[547829, 13180, 554, 16, 48, "Input",ExpressionUUID->"af481f0f-ff83-4e3e-a33b-4dc67303bd13",
 CellTags->"b:3.4.3"],
Cell[548386, 13198, 210, 5, 48, "Output",ExpressionUUID->"742e003d-8937-4305-be67-a6b89d51c545",
 CellTags->"b:3.4.3"]
}, Open  ]],
Cell[548611, 13206, 1380, 42, 63, "Text",ExpressionUUID->"a0412f62-d1dd-474d-833f-b77f8e163b4d",
 CellTags->"b:3.4.3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[550028, 13253, 207, 4, 60, "Subsection",ExpressionUUID->"0a840919-cec3-4179-8e83-b7031f518721",
 CellTags->{"c:33", "b:3.4.4"}],
Cell[550238, 13259, 468, 11, 55, "Text",ExpressionUUID->"98551af2-721f-4145-add0-0c1e10724682",
 CellTags->"b:3.4.4"],
Cell[CellGroupData[{
Cell[550731, 13274, 418, 12, 60, "Input",ExpressionUUID->"e4371f2e-8aef-4731-8b71-ddc9261ac155",
 CellTags->"b:3.4.4"],
Cell[551152, 13288, 210, 5, 48, "Output",ExpressionUUID->"ed933a35-e02c-4298-a655-9909c549e2a4",
 CellTags->"b:3.4.4"]
}, Open  ]],
Cell[CellGroupData[{
Cell[551399, 13298, 370, 10, 87, "Input",ExpressionUUID->"e49c3b07-13d7-4dc7-9099-c33c3774ee08",
 CellTags->"b:3.4.4"],
Cell[551772, 13310, 212, 5, 48, "Output",ExpressionUUID->"a29bf67b-5961-4fe2-988b-6a787f8115e3",
 CellTags->"b:3.4.4"],
Cell[551987, 13317, 211, 5, 48, "Output",ExpressionUUID->"f43a7e3e-dfc2-4e1f-988f-2ef3656b0da4",
 CellTags->"b:3.4.4"],
Cell[552201, 13324, 215, 5, 48, "Output",ExpressionUUID->"5afc2192-6b29-4b3f-b5c0-9d05e19b4bb2",
 CellTags->"b:3.4.4"]
}, Open  ]],
Cell[552431, 13332, 596, 18, 25, "Text",ExpressionUUID->"1c4a527d-8219-49e3-8ce9-83158e473dc7",
 CellTags->"b:3.4.4"],
Cell[553030, 13352, 961, 29, 68, "Input",ExpressionUUID->"1fef000b-fc26-4d0d-a42a-6d92b8acbb95",
 CellTags->"b:3.4.4"],
Cell[553994, 13383, 400, 11, 25, "Text",ExpressionUUID->"33971208-b4af-4f5e-acd3-850ceea0f6af",
 CellTags->"b:3.4.4"],
Cell[554397, 13396, 2019, 62, 115, "Input",ExpressionUUID->"448b68fe-3d99-4f92-b723-79dc663a692e",
 CellTags->"b:3.4.4"],
Cell[556419, 13460, 189, 4, 23, "Text",ExpressionUUID->"c3dd20c5-e84b-4681-8264-5b7e1e242a51",
 CellTags->"b:3.4.4"],
Cell[556611, 13466, 819, 25, 87, "Input",ExpressionUUID->"a46d6d01-6126-4bc0-a824-9be50b8f05b2",
 CellTags->"b:3.4.4"],
Cell[CellGroupData[{
Cell[557455, 13495, 135, 2, 48, "Input",ExpressionUUID->"b0bb4c62-ccb1-4992-9e41-a9e48f7a2082",
 CellTags->"b:3.4.4"],
Cell[557593, 13499, 214, 5, 48, "Output",ExpressionUUID->"a817ccd8-a8b0-4722-8b26-c322a00ad647",
 CellTags->"b:3.4.4"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[557856, 13510, 201, 4, 60, "Subsection",ExpressionUUID->"64dcee58-a1d1-4934-8ecb-0881b5c947d8",
 CellTags->{"c:34", "b:3.4.5"}],
Cell[558060, 13516, 215, 5, 23, "Text",ExpressionUUID->"ebf67078-deeb-4d02-9435-9900f8c0280d",
 CellTags->"b:3.4.5"],
Cell[558278, 13523, 182, 4, 48, "Input",ExpressionUUID->"0416feec-c6d7-4a5a-ae7e-e21f7cf45018",
 CellTags->"b:3.4.5"],
Cell[558463, 13529, 187, 4, 48, "Input",ExpressionUUID->"1330a74d-ae4a-4cd0-8e63-995c764cb490",
 CellTags->"b:3.4.5"],
Cell[CellGroupData[{
Cell[558675, 13537, 346, 8, 68, "Input",ExpressionUUID->"c82cb01c-7267-45d4-b7de-f54e6f34bd71",
 CellTags->"b:3.4.5"],
Cell[559024, 13547, 390, 7, 24, "Print",ExpressionUUID->"6cc5eb13-0120-41ea-9b29-545c996602d8",
 CellTags->"b:3.4.5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[559463, 13560, 248, 7, 60, "Subsection",ExpressionUUID->"b0f85306-8e1b-4b5d-9f95-71b193ff39db",
 CellTags->{"c:35", "b:3.4.6"}],
Cell[559714, 13569, 209, 5, 23, "Text",ExpressionUUID->"d3d9fc44-7014-4763-816b-a6fe354e2989",
 CellTags->"b:3.4.6"],
Cell[559926, 13576, 175, 4, 48, "Input",ExpressionUUID->"4beb6907-d3ca-4541-8990-a9a35d1fbd0c",
 CellTags->"b:3.4.6"],
Cell[560104, 13582, 743, 22, 51, "Input",ExpressionUUID->"59092ff9-c10f-45fe-af99-7a433c7a95d9",
 CellTags->"b:3.4.6"],
Cell[560850, 13606, 211, 5, 23, "Text",ExpressionUUID->"81a1cf8d-fa5b-4026-92f8-ac50e66e6e2d",
 CellTags->"b:3.4.6"],
Cell[CellGroupData[{
Cell[561086, 13615, 858, 22, 87, "Input",ExpressionUUID->"2a6ccb02-d902-4138-b163-dab440cbb2bb",
 CellTags->"b:3.4.6"],
Cell[561947, 13639, 93480, 1554, 219, "Output",ExpressionUUID->"8f8885ea-371f-465e-83d9-bb9094c7d91b",
 CellTags->"b:3.4.6"],
Cell[655430, 15195, 105076, 1740, 230, "Output",ExpressionUUID->"01606e01-3e64-4a7b-8cba-6588bc9eb555",
 CellTags->"b:3.4.6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[760555, 16941, 227, 6, 60, "Subsection",ExpressionUUID->"e091aa0c-714d-4e06-8a2f-d32343528a13",
 CellTags->{"c:36", "b:3.4.7"}],
Cell[760785, 16949, 627, 13, 136, "Text",ExpressionUUID->"d5f16e23-ba79-4739-8e7c-179a1b65c4ed",
 CellTags->"b:3.4.7"],
Cell[761415, 16964, 144, 3, 48, "Input",ExpressionUUID->"a07dce25-960f-45b5-970a-f73f2465d8e5",
 CellTags->"b:3.4.7"],
Cell[761562, 16969, 4274, 115, 288, "Input",ExpressionUUID->"df519045-8997-4196-a8ba-7bcb9e8f5e80",
 CellTags->"b:3.4.7"],
Cell[CellGroupData[{
Cell[765861, 17088, 822, 20, 125, "Input",ExpressionUUID->"660471de-1a16-41d9-8c6e-ddb17a01c4c4",
 CellTags->"b:3.4.7"],
Cell[766686, 17110, 2156, 43, 506, "Output",ExpressionUUID->"f4adddef-0c7f-4d98-afc5-805531ff7432",
 CellTags->"b:3.4.7"],
Cell[768845, 17155, 271, 6, 24, "Print",ExpressionUUID->"a55a1b3f-ac77-41a6-9665-c22e0b0842b1",
 CellTags->"b:3.4.7"],
Cell[769119, 17163, 605, 12, 24, "Print",ExpressionUUID->"fe58f465-af30-4318-8add-2197fc98fc5b",
 CellTags->"b:3.4.7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[769761, 17180, 136, 2, 48, "Input",ExpressionUUID->"66632646-9f1c-477c-bb1a-2220bd57b970",
 CellTags->"b:3.4.7"],
Cell[769900, 17184, 234, 5, 48, "Output",ExpressionUUID->"ececa644-1a9b-4896-849a-c240279dbc09",
 CellTags->"b:3.4.7"]
}, Open  ]],
Cell[770149, 17192, 319, 7, 79, "Text",ExpressionUUID->"226ff202-9888-49a2-9320-8d3a2c9f85c3",
 CellTags->"b:3.4.7"],
Cell[CellGroupData[{
Cell[770493, 17203, 558, 15, 68, "Input",ExpressionUUID->"58762dcb-c072-4bc8-a28e-a1e30cce3aaf",
 CellTags->"b:3.4.7"],
Cell[771054, 17220, 1648, 44, 264, "Output",ExpressionUUID->"8ede2d58-e834-4d41-9986-56da39c73074",
 CellTags->"b:3.4.7"]
}, Open  ]],
Cell[772717, 17267, 98, 1, 48, "Input",ExpressionUUID->"ef71a855-21d0-4afc-85fc-6e29744051bc",
 CellTags->"b:3.4.7"]
}, Open  ]]
}, Open  ]]
}
]
*)

